/********************************************************************************
** Form generated from reading UI file 'locationDialogGui.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LOCATIONDIALOGGUI_H
#define UI_LOCATIONDIALOGGUI_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListView>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "AngleSpinBox.hpp"
#include "Dialog.hpp"
#include "MapWidget.hpp"

QT_BEGIN_NAMESPACE

class Ui_locationDialogForm
{
public:
    QVBoxLayout *vboxLayout;
    TitleBar *titleBar;
    QFrame *locationContent;
    QGridLayout *gridLayout_2;
    QFrame *frame;
    QGridLayout *gridLayout_3;
    MapWidget *mapWidget;
    QFrame *frame_citylist;
    QVBoxLayout *verticalLayout;
    QListView *citiesListView;
    QLineEdit *citySearchLineEdit;
    QPushButton *resetListPushButton;
    QGroupBox *groupBox;
    QGridLayout *gridLayout;
    QFrame *frame_buttons;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *horizontalSpacer;
    QPushButton *addLocationToListPushButton;
    QPushButton *deleteLocationFromListPushButton;
    QPushButton *pushButtonReturnToDefault;
    QFrame *frame_coordinates;
    QGridLayout *gridLayout_4;
    QLabel *labelLongitude;
    AngleSpinBox *latitudeSpinBox;
    QCheckBox *useAsDefaultLocationCheckBox;
    QToolButton *gpsToolButton;
    AngleSpinBox *longitudeSpinBox;
    QLabel *labelLatitude;
    QCheckBox *useIpQueryCheckBox;
    QSpinBox *altitudeSpinBox;
    QLabel *labelElevation;
    QCheckBox *autoChangeLandscapesCheckBox;
    QSpacerItem *horizontalSpacer_2;
    QFrame *frame_names;
    QGridLayout *gridLayout_5;
    QLineEdit *cityNameLineEdit;
    QLabel *labelPlanet;
    QLabel *labelTimeZone;
    QLabel *labelRegion;
    QComboBox *regionNameComboBox;
    QLabel *labelName;
    QCheckBox *useCustomTimeZoneCheckBox;
    QCheckBox *dstCheckBox;
    QCheckBox *autoEnableEnvironmentCheckBox;
    QComboBox *timeZoneNameComboBox;
    QComboBox *planetNameComboBox;

    void setupUi(QWidget *locationDialogForm)
    {
        if (locationDialogForm->objectName().isEmpty())
            locationDialogForm->setObjectName("locationDialogForm");
        locationDialogForm->resize(786, 300);
        locationDialogForm->setStyleSheet(QString::fromUtf8(""));
        vboxLayout = new QVBoxLayout(locationDialogForm);
        vboxLayout->setSpacing(0);
        vboxLayout->setObjectName("vboxLayout");
        vboxLayout->setContentsMargins(0, 0, 0, 0);
        titleBar = new TitleBar(locationDialogForm);
        titleBar->setObjectName("titleBar");

        vboxLayout->addWidget(titleBar);

        locationContent = new QFrame(locationDialogForm);
        locationContent->setObjectName("locationContent");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Minimum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(locationContent->sizePolicy().hasHeightForWidth());
        locationContent->setSizePolicy(sizePolicy);
        locationContent->setFrameShape(QFrame::StyledPanel);
        gridLayout_2 = new QGridLayout(locationContent);
        gridLayout_2->setSpacing(0);
        gridLayout_2->setObjectName("gridLayout_2");
        gridLayout_2->setContentsMargins(0, 0, 0, 0);
        frame = new QFrame(locationContent);
        frame->setObjectName("frame");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::MinimumExpanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(frame->sizePolicy().hasHeightForWidth());
        frame->setSizePolicy(sizePolicy1);
        frame->setFrameShape(QFrame::StyledPanel);
        frame->setFrameShadow(QFrame::Raised);
        gridLayout_3 = new QGridLayout(frame);
        gridLayout_3->setSpacing(0);
        gridLayout_3->setObjectName("gridLayout_3");
        gridLayout_3->setContentsMargins(0, 0, 0, 0);
        mapWidget = new MapWidget(frame);
        mapWidget->setObjectName("mapWidget");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy2.setHorizontalStretch(1);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(mapWidget->sizePolicy().hasHeightForWidth());
        mapWidget->setSizePolicy(sizePolicy2);

        gridLayout_3->addWidget(mapWidget, 1, 0, 1, 1);

        frame_citylist = new QFrame(frame);
        frame_citylist->setObjectName("frame_citylist");
        sizePolicy1.setHeightForWidth(frame_citylist->sizePolicy().hasHeightForWidth());
        frame_citylist->setSizePolicy(sizePolicy1);
        frame_citylist->setFrameShape(QFrame::StyledPanel);
        frame_citylist->setFrameShadow(QFrame::Raised);
        verticalLayout = new QVBoxLayout(frame_citylist);
        verticalLayout->setSpacing(0);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        citiesListView = new QListView(frame_citylist);
        citiesListView->setObjectName("citiesListView");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Expanding);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(citiesListView->sizePolicy().hasHeightForWidth());
        citiesListView->setSizePolicy(sizePolicy3);
        citiesListView->setFocusPolicy(Qt::NoFocus);
        citiesListView->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        citiesListView->setEditTriggers(QAbstractItemView::NoEditTriggers);
        citiesListView->setUniformItemSizes(true);

        verticalLayout->addWidget(citiesListView);

        citySearchLineEdit = new QLineEdit(frame_citylist);
        citySearchLineEdit->setObjectName("citySearchLineEdit");
        citySearchLineEdit->setAlignment(Qt::AlignBottom|Qt::AlignLeading|Qt::AlignLeft);

        verticalLayout->addWidget(citySearchLineEdit);

        resetListPushButton = new QPushButton(frame_citylist);
        resetListPushButton->setObjectName("resetListPushButton");

        verticalLayout->addWidget(resetListPushButton);


        gridLayout_3->addWidget(frame_citylist, 1, 1, 1, 1);


        gridLayout_2->addWidget(frame, 0, 0, 1, 2);

        groupBox = new QGroupBox(locationContent);
        groupBox->setObjectName("groupBox");
        gridLayout = new QGridLayout(groupBox);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setContentsMargins(0, 0, 0, 0);
        frame_buttons = new QFrame(groupBox);
        frame_buttons->setObjectName("frame_buttons");
        frame_buttons->setFrameShape(QFrame::StyledPanel);
        frame_buttons->setFrameShadow(QFrame::Raised);
        horizontalLayout = new QHBoxLayout(frame_buttons);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        addLocationToListPushButton = new QPushButton(frame_buttons);
        addLocationToListPushButton->setObjectName("addLocationToListPushButton");
        addLocationToListPushButton->setEnabled(false);
        addLocationToListPushButton->setFocusPolicy(Qt::NoFocus);

        horizontalLayout->addWidget(addLocationToListPushButton);

        deleteLocationFromListPushButton = new QPushButton(frame_buttons);
        deleteLocationFromListPushButton->setObjectName("deleteLocationFromListPushButton");
        deleteLocationFromListPushButton->setEnabled(false);
        deleteLocationFromListPushButton->setFocusPolicy(Qt::NoFocus);

        horizontalLayout->addWidget(deleteLocationFromListPushButton);

        pushButtonReturnToDefault = new QPushButton(frame_buttons);
        pushButtonReturnToDefault->setObjectName("pushButtonReturnToDefault");
        pushButtonReturnToDefault->setEnabled(false);
        pushButtonReturnToDefault->setFocusPolicy(Qt::NoFocus);

        horizontalLayout->addWidget(pushButtonReturnToDefault);


        gridLayout->addWidget(frame_buttons, 2, 1, 1, 3);

        frame_coordinates = new QFrame(groupBox);
        frame_coordinates->setObjectName("frame_coordinates");
        frame_coordinates->setFrameShape(QFrame::StyledPanel);
        frame_coordinates->setFrameShadow(QFrame::Raised);
        gridLayout_4 = new QGridLayout(frame_coordinates);
        gridLayout_4->setObjectName("gridLayout_4");
        gridLayout_4->setHorizontalSpacing(6);
        gridLayout_4->setContentsMargins(0, 0, 0, 0);
        labelLongitude = new QLabel(frame_coordinates);
        labelLongitude->setObjectName("labelLongitude");

        gridLayout_4->addWidget(labelLongitude, 1, 0, 1, 1);

        latitudeSpinBox = new AngleSpinBox(frame_coordinates);
        latitudeSpinBox->setObjectName("latitudeSpinBox");
        QSizePolicy sizePolicy4(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Fixed);
        sizePolicy4.setHorizontalStretch(0);
        sizePolicy4.setVerticalStretch(0);
        sizePolicy4.setHeightForWidth(latitudeSpinBox->sizePolicy().hasHeightForWidth());
        latitudeSpinBox->setSizePolicy(sizePolicy4);
        latitudeSpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_4->addWidget(latitudeSpinBox, 0, 1, 1, 1);

        useAsDefaultLocationCheckBox = new QCheckBox(frame_coordinates);
        useAsDefaultLocationCheckBox->setObjectName("useAsDefaultLocationCheckBox");

        gridLayout_4->addWidget(useAsDefaultLocationCheckBox, 5, 0, 1, 2);

        gpsToolButton = new QToolButton(frame_coordinates);
        gpsToolButton->setObjectName("gpsToolButton");
        QSizePolicy sizePolicy5(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);
        sizePolicy5.setHorizontalStretch(0);
        sizePolicy5.setVerticalStretch(0);
        sizePolicy5.setHeightForWidth(gpsToolButton->sizePolicy().hasHeightForWidth());
        gpsToolButton->setSizePolicy(sizePolicy5);
        gpsToolButton->setCheckable(true);
        gpsToolButton->setAutoRepeat(false);
        gpsToolButton->setAutoRepeatDelay(3500);
        gpsToolButton->setAutoRepeatInterval(3500);
        gpsToolButton->setToolButtonStyle(Qt::ToolButtonTextOnly);
        gpsToolButton->setAutoRaise(false);

        gridLayout_4->addWidget(gpsToolButton, 3, 0, 1, 2);

        longitudeSpinBox = new AngleSpinBox(frame_coordinates);
        longitudeSpinBox->setObjectName("longitudeSpinBox");
        longitudeSpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_4->addWidget(longitudeSpinBox, 1, 1, 1, 1);

        labelLatitude = new QLabel(frame_coordinates);
        labelLatitude->setObjectName("labelLatitude");

        gridLayout_4->addWidget(labelLatitude, 0, 0, 1, 1);

        useIpQueryCheckBox = new QCheckBox(frame_coordinates);
        useIpQueryCheckBox->setObjectName("useIpQueryCheckBox");

        gridLayout_4->addWidget(useIpQueryCheckBox, 4, 0, 1, 2);

        altitudeSpinBox = new QSpinBox(frame_coordinates);
        altitudeSpinBox->setObjectName("altitudeSpinBox");
        altitudeSpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        altitudeSpinBox->setMinimum(-2000);
        altitudeSpinBox->setMaximum(200000);
        altitudeSpinBox->setSingleStep(100);

        gridLayout_4->addWidget(altitudeSpinBox, 2, 1, 1, 1);

        labelElevation = new QLabel(frame_coordinates);
        labelElevation->setObjectName("labelElevation");

        gridLayout_4->addWidget(labelElevation, 2, 0, 1, 1);

        autoChangeLandscapesCheckBox = new QCheckBox(frame_coordinates);
        autoChangeLandscapesCheckBox->setObjectName("autoChangeLandscapesCheckBox");
        QSizePolicy sizePolicy6(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy6.setHorizontalStretch(0);
        sizePolicy6.setVerticalStretch(0);
        sizePolicy6.setHeightForWidth(autoChangeLandscapesCheckBox->sizePolicy().hasHeightForWidth());
        autoChangeLandscapesCheckBox->setSizePolicy(sizePolicy6);

        gridLayout_4->addWidget(autoChangeLandscapesCheckBox, 6, 0, 1, 2);


        gridLayout->addWidget(frame_coordinates, 0, 1, 1, 1);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(horizontalSpacer_2, 0, 2, 1, 1);

        frame_names = new QFrame(groupBox);
        frame_names->setObjectName("frame_names");
        QSizePolicy sizePolicy7(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Maximum);
        sizePolicy7.setHorizontalStretch(0);
        sizePolicy7.setVerticalStretch(0);
        sizePolicy7.setHeightForWidth(frame_names->sizePolicy().hasHeightForWidth());
        frame_names->setSizePolicy(sizePolicy7);
        frame_names->setFrameShape(QFrame::StyledPanel);
        frame_names->setFrameShadow(QFrame::Raised);
        gridLayout_5 = new QGridLayout(frame_names);
        gridLayout_5->setObjectName("gridLayout_5");
        gridLayout_5->setContentsMargins(0, 0, 0, 0);
        cityNameLineEdit = new QLineEdit(frame_names);
        cityNameLineEdit->setObjectName("cityNameLineEdit");

        gridLayout_5->addWidget(cityNameLineEdit, 0, 1, 1, 1);

        labelPlanet = new QLabel(frame_names);
        labelPlanet->setObjectName("labelPlanet");

        gridLayout_5->addWidget(labelPlanet, 2, 0, 1, 1);

        labelTimeZone = new QLabel(frame_names);
        labelTimeZone->setObjectName("labelTimeZone");

        gridLayout_5->addWidget(labelTimeZone, 3, 0, 1, 1);

        labelRegion = new QLabel(frame_names);
        labelRegion->setObjectName("labelRegion");

        gridLayout_5->addWidget(labelRegion, 1, 0, 1, 1);

        regionNameComboBox = new QComboBox(frame_names);
        regionNameComboBox->setObjectName("regionNameComboBox");
        regionNameComboBox->setEditable(true);
        regionNameComboBox->setInsertPolicy(QComboBox::NoInsert);

        gridLayout_5->addWidget(regionNameComboBox, 1, 1, 1, 1);

        labelName = new QLabel(frame_names);
        labelName->setObjectName("labelName");

        gridLayout_5->addWidget(labelName, 0, 0, 1, 1);

        useCustomTimeZoneCheckBox = new QCheckBox(frame_names);
        useCustomTimeZoneCheckBox->setObjectName("useCustomTimeZoneCheckBox");

        gridLayout_5->addWidget(useCustomTimeZoneCheckBox, 4, 0, 1, 2);

        dstCheckBox = new QCheckBox(frame_names);
        dstCheckBox->setObjectName("dstCheckBox");

        gridLayout_5->addWidget(dstCheckBox, 5, 0, 1, 2);

        autoEnableEnvironmentCheckBox = new QCheckBox(frame_names);
        autoEnableEnvironmentCheckBox->setObjectName("autoEnableEnvironmentCheckBox");

        gridLayout_5->addWidget(autoEnableEnvironmentCheckBox, 6, 0, 1, 2);

        timeZoneNameComboBox = new QComboBox(frame_names);
        timeZoneNameComboBox->setObjectName("timeZoneNameComboBox");
        timeZoneNameComboBox->setEnabled(true);
        timeZoneNameComboBox->setEditable(true);
        timeZoneNameComboBox->setInsertPolicy(QComboBox::NoInsert);

        gridLayout_5->addWidget(timeZoneNameComboBox, 3, 1, 1, 1);

        planetNameComboBox = new QComboBox(frame_names);
        planetNameComboBox->setObjectName("planetNameComboBox");
        planetNameComboBox->setEditable(true);
        planetNameComboBox->setInsertPolicy(QComboBox::NoInsert);

        gridLayout_5->addWidget(planetNameComboBox, 2, 1, 1, 1);


        gridLayout->addWidget(frame_names, 0, 3, 1, 1);


        gridLayout_2->addWidget(groupBox, 2, 1, 1, 1);


        vboxLayout->addWidget(locationContent);

        QWidget::setTabOrder(citySearchLineEdit, resetListPushButton);
        QWidget::setTabOrder(resetListPushButton, latitudeSpinBox);
        QWidget::setTabOrder(latitudeSpinBox, longitudeSpinBox);
        QWidget::setTabOrder(longitudeSpinBox, altitudeSpinBox);
        QWidget::setTabOrder(altitudeSpinBox, cityNameLineEdit);
        QWidget::setTabOrder(cityNameLineEdit, regionNameComboBox);
        QWidget::setTabOrder(regionNameComboBox, planetNameComboBox);
        QWidget::setTabOrder(planetNameComboBox, timeZoneNameComboBox);
        QWidget::setTabOrder(timeZoneNameComboBox, useCustomTimeZoneCheckBox);
        QWidget::setTabOrder(useCustomTimeZoneCheckBox, dstCheckBox);
        QWidget::setTabOrder(dstCheckBox, useIpQueryCheckBox);
        QWidget::setTabOrder(useIpQueryCheckBox, useAsDefaultLocationCheckBox);
        QWidget::setTabOrder(useAsDefaultLocationCheckBox, addLocationToListPushButton);
        QWidget::setTabOrder(addLocationToListPushButton, deleteLocationFromListPushButton);
        QWidget::setTabOrder(deleteLocationFromListPushButton, pushButtonReturnToDefault);

        retranslateUi(locationDialogForm);

        QMetaObject::connectSlotsByName(locationDialogForm);
    } // setupUi

    void retranslateUi(QWidget *locationDialogForm)
    {
        titleBar->setProperty("title", QVariant(QCoreApplication::translate("locationDialogForm", "Location", nullptr)));
#if QT_CONFIG(tooltip)
        citiesListView->setToolTip(QCoreApplication::translate("locationDialogForm", "Standard list of locations contains only English names (without diacritics!)", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        citySearchLineEdit->setToolTip(QCoreApplication::translate("locationDialogForm", "Standard list of locations contains only English names (without diacritics!)", nullptr));
#endif // QT_CONFIG(tooltip)
        citySearchLineEdit->setText(QString());
        resetListPushButton->setText(QCoreApplication::translate("locationDialogForm", "Reset Location List", nullptr));
        groupBox->setTitle(QCoreApplication::translate("locationDialogForm", "Current location information", nullptr));
        addLocationToListPushButton->setText(QCoreApplication::translate("locationDialogForm", "Add to list", nullptr));
        deleteLocationFromListPushButton->setText(QCoreApplication::translate("locationDialogForm", "Delete from list", nullptr));
        pushButtonReturnToDefault->setText(QCoreApplication::translate("locationDialogForm", "Return to default location", nullptr));
        labelLongitude->setText(QCoreApplication::translate("locationDialogForm", "Longitude:", nullptr));
#if QT_CONFIG(tooltip)
        latitudeSpinBox->setToolTip(QCoreApplication::translate("locationDialogForm", "You can enter values in decimal degrees, or using dms format, for example: +1d 12m 8s", nullptr));
#endif // QT_CONFIG(tooltip)
        useAsDefaultLocationCheckBox->setText(QCoreApplication::translate("locationDialogForm", "Use current location as default", nullptr));
        gpsToolButton->setText(QCoreApplication::translate("locationDialogForm", "Get location from GPS", nullptr));
#if QT_CONFIG(tooltip)
        longitudeSpinBox->setToolTip(QCoreApplication::translate("locationDialogForm", "You can enter values in decimal degrees, or using dms format, for example: +1d 12m 8s", nullptr));
#endif // QT_CONFIG(tooltip)
        labelLatitude->setText(QCoreApplication::translate("locationDialogForm", "Latitude:", nullptr));
        useIpQueryCheckBox->setText(QCoreApplication::translate("locationDialogForm", "Get location from Network", nullptr));
#if QT_CONFIG(tooltip)
        altitudeSpinBox->setToolTip(QCoreApplication::translate("locationDialogForm", "Enter the elevation in meter", nullptr));
#endif // QT_CONFIG(tooltip)
        altitudeSpinBox->setSuffix(QCoreApplication::translate("locationDialogForm", " m", nullptr));
        labelElevation->setText(QCoreApplication::translate("locationDialogForm", "Elevation:", nullptr));
#if QT_CONFIG(tooltip)
        autoChangeLandscapesCheckBox->setToolTip(QCoreApplication::translate("locationDialogForm", "Automatic change of landscape when planet or location is changed", nullptr));
#endif // QT_CONFIG(tooltip)
        autoChangeLandscapesCheckBox->setText(QCoreApplication::translate("locationDialogForm", "Auto-select landscapes", nullptr));
        cityNameLineEdit->setPlaceholderText(QCoreApplication::translate("locationDialogForm", "New Location", nullptr));
        labelPlanet->setText(QCoreApplication::translate("locationDialogForm", "Planet:", nullptr));
        labelTimeZone->setText(QCoreApplication::translate("locationDialogForm", "Time zone:", nullptr));
        labelRegion->setText(QCoreApplication::translate("locationDialogForm", "Region:", nullptr));
        labelName->setText(QCoreApplication::translate("locationDialogForm", "Name/City:", nullptr));
        useCustomTimeZoneCheckBox->setText(QCoreApplication::translate("locationDialogForm", "Use custom time zone", nullptr));
#if QT_CONFIG(tooltip)
        dstCheckBox->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
        dstCheckBox->setText(QCoreApplication::translate("locationDialogForm", "Enable daylight saving time", nullptr));
#if QT_CONFIG(tooltip)
        autoEnableEnvironmentCheckBox->setToolTip(QCoreApplication::translate("locationDialogForm", "Auto-enable atmosphere when planet is changed", nullptr));
#endif // QT_CONFIG(tooltip)
        autoEnableEnvironmentCheckBox->setText(QCoreApplication::translate("locationDialogForm", "Auto-enable atmosphere", nullptr));
        (void)locationDialogForm;
    } // retranslateUi

};

namespace Ui {
    class locationDialogForm: public Ui_locationDialogForm {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_LOCATIONDIALOGGUI_H
