/****************************************************************************
** Meta object code from reading C++ file 'LensDistortionEstimator.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../stellarium/plugins/LensDistortionEstimator/src/LensDistortionEstimator.hpp"
#include <QtCore/qmetatype.h>
#include <QtCore/qplugin.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'LensDistortionEstimator.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN23LensDistortionEstimatorE_t {};
} // unnamed namespace

template <> constexpr inline auto LensDistortionEstimator::qt_create_metaobjectdata<qt_meta_tag_ZN23LensDistortionEstimatorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "LensDistortionEstimator",
        "imageToggled",
        "",
        "enabled",
        "dialogToggled",
        "imageAxesToggled",
        "pointMarkersToggled",
        "imgPosResetOnLoadingToggled",
        "projectionCenterMarkerToggled",
        "projectionCenterMarkerColorChanged",
        "Vec3f",
        "color",
        "selectedPointMarkerColorChanged",
        "pointMarkerColorChanged",
        "imageAxesColorChanged",
        "setImageEnabled",
        "enable",
        "setDialogVisible",
        "setImageAxesEnabled",
        "setPointMarkersEnabled",
        "setProjectionCenterMarkerEnabled",
        "setImgPosResetOnLoadingEnabled",
        "setProjectionCenterMarkerColor",
        "setSelectedPointMarkerColor",
        "setPointMarkerColor",
        "setImageAxesColor",
        "dialogVisible",
        "imageEnabled",
        "imageAxesEnabled",
        "pointMarkersEnabled",
        "projectionCenterMarkerEnabled",
        "imgPosResetOnLoadingEnabled",
        "selectedPointMarkerColor",
        "pointMarkerColor",
        "projectionCenterMarkerColor",
        "imageAxesColor"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'imageToggled'
        QtMocHelpers::SignalData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'dialogToggled'
        QtMocHelpers::SignalData<void(bool)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'imageAxesToggled'
        QtMocHelpers::SignalData<void(bool)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'pointMarkersToggled'
        QtMocHelpers::SignalData<void(bool)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'imgPosResetOnLoadingToggled'
        QtMocHelpers::SignalData<void(bool)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'projectionCenterMarkerToggled'
        QtMocHelpers::SignalData<void(bool)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'projectionCenterMarkerColorChanged'
        QtMocHelpers::SignalData<void(const Vec3f &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Signal 'selectedPointMarkerColorChanged'
        QtMocHelpers::SignalData<void(const Vec3f &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Signal 'pointMarkerColorChanged'
        QtMocHelpers::SignalData<void(const Vec3f &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Signal 'imageAxesColorChanged'
        QtMocHelpers::SignalData<void(const Vec3f &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Slot 'setImageEnabled'
        QtMocHelpers::SlotData<void(bool)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 16 },
        }}),
        // Slot 'setDialogVisible'
        QtMocHelpers::SlotData<void(bool)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 16 },
        }}),
        // Slot 'setImageAxesEnabled'
        QtMocHelpers::SlotData<void(bool)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'setPointMarkersEnabled'
        QtMocHelpers::SlotData<void(bool)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'setProjectionCenterMarkerEnabled'
        QtMocHelpers::SlotData<void(bool)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'setImgPosResetOnLoadingEnabled'
        QtMocHelpers::SlotData<void(bool)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'setProjectionCenterMarkerColor'
        QtMocHelpers::SlotData<void(const Vec3f &)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Slot 'setSelectedPointMarkerColor'
        QtMocHelpers::SlotData<void(const Vec3f &)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Slot 'setPointMarkerColor'
        QtMocHelpers::SlotData<void(const Vec3f &)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Slot 'setImageAxesColor'
        QtMocHelpers::SlotData<void(const Vec3f &)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'dialogVisible'
        QtMocHelpers::PropertyData<bool>(26, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'imageEnabled'
        QtMocHelpers::PropertyData<bool>(27, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'imageAxesEnabled'
        QtMocHelpers::PropertyData<bool>(28, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'pointMarkersEnabled'
        QtMocHelpers::PropertyData<bool>(29, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'projectionCenterMarkerEnabled'
        QtMocHelpers::PropertyData<bool>(30, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
        // property 'imgPosResetOnLoadingEnabled'
        QtMocHelpers::PropertyData<bool>(31, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'selectedPointMarkerColor'
        QtMocHelpers::PropertyData<Vec3f>(32, 0x80000000 | 10, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 7),
        // property 'pointMarkerColor'
        QtMocHelpers::PropertyData<Vec3f>(33, 0x80000000 | 10, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 8),
        // property 'projectionCenterMarkerColor'
        QtMocHelpers::PropertyData<Vec3f>(34, 0x80000000 | 10, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 6),
        // property 'imageAxesColor'
        QtMocHelpers::PropertyData<Vec3f>(35, 0x80000000 | 10, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 9),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<LensDistortionEstimator, qt_meta_tag_ZN23LensDistortionEstimatorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject LensDistortionEstimator::staticMetaObject = { {
    QMetaObject::SuperData::link<StelModule::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23LensDistortionEstimatorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23LensDistortionEstimatorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN23LensDistortionEstimatorE_t>.metaTypes,
    nullptr
} };

void LensDistortionEstimator::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<LensDistortionEstimator *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->imageToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->dialogToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->imageAxesToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->pointMarkersToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->imgPosResetOnLoadingToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->projectionCenterMarkerToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->projectionCenterMarkerColorChanged((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1]))); break;
        case 7: _t->selectedPointMarkerColorChanged((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1]))); break;
        case 8: _t->pointMarkerColorChanged((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1]))); break;
        case 9: _t->imageAxesColorChanged((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1]))); break;
        case 10: _t->setImageEnabled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 11: _t->setDialogVisible((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 12: _t->setImageAxesEnabled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 13: _t->setPointMarkersEnabled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 14: _t->setProjectionCenterMarkerEnabled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 15: _t->setImgPosResetOnLoadingEnabled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 16: _t->setProjectionCenterMarkerColor((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1]))); break;
        case 17: _t->setSelectedPointMarkerColor((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1]))); break;
        case 18: _t->setPointMarkerColor((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1]))); break;
        case 19: _t->setImageAxesColor((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 16:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 17:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 18:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 19:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (LensDistortionEstimator::*)(bool )>(_a, &LensDistortionEstimator::imageToggled, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (LensDistortionEstimator::*)(bool )>(_a, &LensDistortionEstimator::dialogToggled, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (LensDistortionEstimator::*)(bool )>(_a, &LensDistortionEstimator::imageAxesToggled, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (LensDistortionEstimator::*)(bool )>(_a, &LensDistortionEstimator::pointMarkersToggled, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (LensDistortionEstimator::*)(bool )>(_a, &LensDistortionEstimator::imgPosResetOnLoadingToggled, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (LensDistortionEstimator::*)(bool )>(_a, &LensDistortionEstimator::projectionCenterMarkerToggled, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (LensDistortionEstimator::*)(const Vec3f & )>(_a, &LensDistortionEstimator::projectionCenterMarkerColorChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (LensDistortionEstimator::*)(const Vec3f & )>(_a, &LensDistortionEstimator::selectedPointMarkerColorChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (LensDistortionEstimator::*)(const Vec3f & )>(_a, &LensDistortionEstimator::pointMarkerColorChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (LensDistortionEstimator::*)(const Vec3f & )>(_a, &LensDistortionEstimator::imageAxesColorChanged, 9))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 9:
        case 8:
        case 7:
        case 6:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Vec3f >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->isDialogVisible(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->isImageEnabled(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->areImageAxesEnabled(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->arePointMarkersEnabled(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->isProjectionCenterMarkerEnabled(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->isImgPosResetOnLoadingEnabled(); break;
        case 6: *reinterpret_cast<Vec3f*>(_v) = _t->getSelectedPointMarkerColor(); break;
        case 7: *reinterpret_cast<Vec3f*>(_v) = _t->getPointMarkerColor(); break;
        case 8: *reinterpret_cast<Vec3f*>(_v) = _t->getProjectionCenterMarkerColor(); break;
        case 9: *reinterpret_cast<Vec3f*>(_v) = _t->getImageAxesColor(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setDialogVisible(*reinterpret_cast<bool*>(_v)); break;
        case 1: _t->setImageEnabled(*reinterpret_cast<bool*>(_v)); break;
        case 2: _t->setImageAxesEnabled(*reinterpret_cast<bool*>(_v)); break;
        case 3: _t->setPointMarkersEnabled(*reinterpret_cast<bool*>(_v)); break;
        case 4: _t->setProjectionCenterMarkerEnabled(*reinterpret_cast<bool*>(_v)); break;
        case 5: _t->setImgPosResetOnLoadingEnabled(*reinterpret_cast<bool*>(_v)); break;
        case 6: _t->setSelectedPointMarkerColor(*reinterpret_cast<Vec3f*>(_v)); break;
        case 7: _t->setPointMarkerColor(*reinterpret_cast<Vec3f*>(_v)); break;
        case 8: _t->setProjectionCenterMarkerColor(*reinterpret_cast<Vec3f*>(_v)); break;
        case 9: _t->setImageAxesColor(*reinterpret_cast<Vec3f*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *LensDistortionEstimator::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *LensDistortionEstimator::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23LensDistortionEstimatorE_t>.strings))
        return static_cast<void*>(this);
    return StelModule::qt_metacast(_clname);
}

int LensDistortionEstimator::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StelModule::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 20)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 20;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 20)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 20;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void LensDistortionEstimator::imageToggled(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void LensDistortionEstimator::dialogToggled(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void LensDistortionEstimator::imageAxesToggled(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void LensDistortionEstimator::pointMarkersToggled(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void LensDistortionEstimator::imgPosResetOnLoadingToggled(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void LensDistortionEstimator::projectionCenterMarkerToggled(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void LensDistortionEstimator::projectionCenterMarkerColorChanged(const Vec3f & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void LensDistortionEstimator::selectedPointMarkerColorChanged(const Vec3f & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void LensDistortionEstimator::pointMarkerColorChanged(const Vec3f & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void LensDistortionEstimator::imageAxesColorChanged(const Vec3f & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN42LensDistortionEstimatorStelPluginInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto LensDistortionEstimatorStelPluginInterface::qt_create_metaobjectdata<qt_meta_tag_ZN42LensDistortionEstimatorStelPluginInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "LensDistortionEstimatorStelPluginInterface"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<LensDistortionEstimatorStelPluginInterface, qt_meta_tag_ZN42LensDistortionEstimatorStelPluginInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject LensDistortionEstimatorStelPluginInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN42LensDistortionEstimatorStelPluginInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN42LensDistortionEstimatorStelPluginInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN42LensDistortionEstimatorStelPluginInterfaceE_t>.metaTypes,
    nullptr
} };

void LensDistortionEstimatorStelPluginInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<LensDistortionEstimatorStelPluginInterface *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *LensDistortionEstimatorStelPluginInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *LensDistortionEstimatorStelPluginInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN42LensDistortionEstimatorStelPluginInterfaceE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "StelPluginInterface"))
        return static_cast< StelPluginInterface*>(this);
    if (!strcmp(_clname, "org.stellarium.StelPluginInterface/2.0"))
        return static_cast< StelPluginInterface*>(this);
    return QObject::qt_metacast(_clname);
}

int LensDistortionEstimatorStelPluginInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}

#ifdef QT_MOC_EXPORT_PLUGIN_V2
static constexpr unsigned char qt_pluginMetaDataV2_LensDistortionEstimatorStelPluginInterface[] = {
    0xbf, 
    // "IID"
    0x02,  0x78,  0x26,  'o',  'r',  'g',  '.',  's', 
    't',  'e',  'l',  'l',  'a',  'r',  'i',  'u', 
    'm',  '.',  'S',  't',  'e',  'l',  'P',  'l', 
    'u',  'g',  'i',  'n',  'I',  'n',  't',  'e', 
    'r',  'f',  'a',  'c',  'e',  '/',  '2',  '.', 
    '0', 
    // "className"
    0x03,  0x78,  0x2a,  'L',  'e',  'n',  's',  'D', 
    'i',  's',  't',  'o',  'r',  't',  'i',  'o', 
    'n',  'E',  's',  't',  'i',  'm',  'a',  't', 
    'o',  'r',  'S',  't',  'e',  'l',  'P',  'l', 
    'u',  'g',  'i',  'n',  'I',  'n',  't',  'e', 
    'r',  'f',  'a',  'c',  'e', 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN_V2(LensDistortionEstimatorStelPluginInterface, LensDistortionEstimatorStelPluginInterface, qt_pluginMetaDataV2_LensDistortionEstimatorStelPluginInterface)
#else
QT_PLUGIN_METADATA_SECTION
Q_CONSTINIT static constexpr unsigned char qt_pluginMetaData_LensDistortionEstimatorStelPluginInterface[] = {
    'Q', 'T', 'M', 'E', 'T', 'A', 'D', 'A', 'T', 'A', ' ', '!',
    // metadata version, Qt version, architectural requirements
    0, QT_VERSION_MAJOR, QT_VERSION_MINOR, qPluginArchRequirements(),
    0xbf, 
    // "IID"
    0x02,  0x78,  0x26,  'o',  'r',  'g',  '.',  's', 
    't',  'e',  'l',  'l',  'a',  'r',  'i',  'u', 
    'm',  '.',  'S',  't',  'e',  'l',  'P',  'l', 
    'u',  'g',  'i',  'n',  'I',  'n',  't',  'e', 
    'r',  'f',  'a',  'c',  'e',  '/',  '2',  '.', 
    '0', 
    // "className"
    0x03,  0x78,  0x2a,  'L',  'e',  'n',  's',  'D', 
    'i',  's',  't',  'o',  'r',  't',  'i',  'o', 
    'n',  'E',  's',  't',  'i',  'm',  'a',  't', 
    'o',  'r',  'S',  't',  'e',  'l',  'P',  'l', 
    'u',  'g',  'i',  'n',  'I',  'n',  't',  'e', 
    'r',  'f',  'a',  'c',  'e', 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN(LensDistortionEstimatorStelPluginInterface, LensDistortionEstimatorStelPluginInterface)
#endif  // QT_MOC_EXPORT_PLUGIN_V2

QT_WARNING_POP
