/********************************************************************************
** Form generated from reading UI file 'calendarsDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CALENDARSDIALOG_H
#define UI_CALENDARSDIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTextBrowser>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "Dialog.hpp"
#include "gui/ColorButton.hpp"

QT_BEGIN_NAMESPACE

class Ui_calendarsDialog
{
public:
    QVBoxLayout *verticalLayout_2;
    TitleBar *titleBar;
    QTabWidget *tabs;
    QWidget *displayTab;
    QVBoxLayout *verticalLayout_3;
    QGridLayout *gridLayout;
    QCheckBox *islamicCheckBox;
    QCheckBox *julianCheckBox;
    QCheckBox *egyptianCheckBox;
    QCheckBox *ethiopicCheckBox;
    QCheckBox *copticCheckBox;
    QCheckBox *armenianCheckBox;
    QCheckBox *oldHinduSolarCheckBox;
    QCheckBox *japaneseCheckBox;
    QCheckBox *frenchArithmeticCheckBox;
    QCheckBox *newHinduSolarCheckBox;
    QCheckBox *hebrewCheckBox;
    QCheckBox *mayaTzolkinCheckBox;
    QCheckBox *persianAstronomicalCheckBox;
    QCheckBox *olympicCheckBox;
    QCheckBox *balineseCheckBox;
    QCheckBox *persianArithmeticCheckBox;
    QCheckBox *astroHinduSolarCheckBox;
    QCheckBox *isoCheckBox;
    QCheckBox *bahaiArithmeticCheckBox;
    QCheckBox *zoroastrianCheckBox;
    QCheckBox *aztecTonalpohualliCheckBox;
    QCheckBox *mayaLCCheckBox;
    QCheckBox *frenchAstronomicalCheckBox;
    QCheckBox *astroHinduLunarCheckBox;
    QCheckBox *oldHinduLunarCheckBox;
    QCheckBox *tibetanCheckBox;
    QCheckBox *romanCheckBox;
    QCheckBox *revisedJulianCheckBox;
    QCheckBox *newHinduLunarCheckBox;
    QCheckBox *gregorianCheckBox;
    QCheckBox *chineseCheckBox;
    QCheckBox *bahaiAstronomicalCheckBox;
    QCheckBox *aztecXihuitlCheckBox;
    QCheckBox *icelandicCheckBox;
    QCheckBox *koreanCheckBox;
    QCheckBox *mayaHaabCheckBox;
    QCheckBox *vietnameseCheckBox;
    QHBoxLayout *horizontalLayout_Settings;
    ColorButton *textcolorToolButton;
    QCheckBox *overrideTextColorCheckBox;
    QSpacerItem *verticalSpacer_3;
    QHBoxLayout *horizontalLayout_7;
    QPushButton *restoreDefaultsButton;
    QWidget *lunisolarTab;
    QVBoxLayout *lunisolarVerticalLayout;
    QGridLayout *lunisolarGridLayout;
    QSpinBox *isoYearSpinBox;
    QLabel *labelDOW;
    QLineEdit *gregorianWeekdayLineEdit;
    QLabel *labelMonth;
    QSpinBox *julianMonthSpinBox;
    QSpinBox *gregorianDaySpinBox;
    QLabel *labelRD;
    QSpinBox *isoWeekSpinBox;
    QSpinBox *gregorianMonthSpinBox;
    QSpinBox *julianDaySpinBox;
    QSpinBox *gregorianYearSpinBox;
    QLabel *labelRDvalue;
    QLabel *labelISOWeekCalendar;
    QLineEdit *julianWeekdayLineEdit;
    QLabel *labelYear;
    QSpinBox *isoDaySpinBox;
    QLabel *labelDay;
    QSpinBox *julianYearSpinBox;
    QLabel *labelGregorianCalendar;
    QLabel *labelJulianCalendar;
    QLabel *labelRevisedJulianCalendar;
    QSpinBox *revisedJulianYearSpinBox;
    QSpinBox *revisedJulianMonthSpinBox;
    QSpinBox *revisedJulianDaySpinBox;
    QLineEdit *revisedJulianWeekdayLineEdit;
    QSpacerItem *verticalSpacer;
    QWidget *chineseTab;
    QVBoxLayout *chineseVerticalLayout;
    QWidget *mesoamericaTab;
    QVBoxLayout *mesoamericaVerticalLayout;
    QGridLayout *mesoamericaGridLayout;
    QLabel *labelKatun;
    QLabel *labelXihuitl;
    QSpinBox *tunSpinBox;
    QSpinBox *katunSpinBox;
    QLabel *labelMaya;
    QSpinBox *baktunSpinBox;
    QSpinBox *haabDaySpinBox;
    QLabel *labelHaab;
    QLabel *labelTun;
    QLabel *labelKin;
    QLineEdit *tzolkinLineEdit;
    QSpinBox *haabMonthSpinBox;
    QLabel *labelUinal;
    QLabel *labelBaktun;
    QLabel *labelLongCount;
    QSpinBox *tzolkinNameSpinBox;
    QPushButton *nextTzolkinButton;
    QSpinBox *tzolkinNumberSpinBox;
    QSpinBox *uinalSpinBox;
    QSpinBox *kinSpinBox;
    QPushButton *nextHaabButton;
    QPushButton *previousTzolkinButton;
    QLineEdit *haabLineEdit;
    QPushButton *previousHaabButton;
    QLabel *labelAztec;
    QLabel *labelTzolkin;
    QLabel *labelTonalpohualli;
    QSpinBox *xihuitlMonthSpinBox;
    QSpinBox *tonalpohualliNumberSpinBox;
    QSpinBox *xihuitlDaySpinBox;
    QSpinBox *tonalpohualliNameSpinBox;
    QLineEdit *xihuitlLineEdit;
    QLineEdit *tonalpohualliLineEdit;
    QPushButton *previousXihuitlButton;
    QPushButton *nextXihuitlButton;
    QPushButton *previousTonalpohualliButton;
    QPushButton *nextTonalpohualliButton;
    QSpacerItem *verticalSpacerMesoamerica;
    QWidget *aboutTab;
    QVBoxLayout *verticalLayout_11;
    QTextBrowser *aboutTextBrowser;

    void setupUi(QWidget *calendarsDialog)
    {
        if (calendarsDialog->objectName().isEmpty())
            calendarsDialog->setObjectName("calendarsDialog");
        calendarsDialog->resize(635, 490);
        verticalLayout_2 = new QVBoxLayout(calendarsDialog);
        verticalLayout_2->setSpacing(0);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        titleBar = new TitleBar(calendarsDialog);
        titleBar->setObjectName("titleBar");

        verticalLayout_2->addWidget(titleBar);

        tabs = new QTabWidget(calendarsDialog);
        tabs->setObjectName("tabs");
        tabs->setDocumentMode(false);
        displayTab = new QWidget();
        displayTab->setObjectName("displayTab");
        verticalLayout_3 = new QVBoxLayout(displayTab);
        verticalLayout_3->setObjectName("verticalLayout_3");
        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        islamicCheckBox = new QCheckBox(displayTab);
        islamicCheckBox->setObjectName("islamicCheckBox");

        gridLayout->addWidget(islamicCheckBox, 5, 0, 1, 1);

        julianCheckBox = new QCheckBox(displayTab);
        julianCheckBox->setObjectName("julianCheckBox");

        gridLayout->addWidget(julianCheckBox, 0, 0, 1, 1);

        egyptianCheckBox = new QCheckBox(displayTab);
        egyptianCheckBox->setObjectName("egyptianCheckBox");

        gridLayout->addWidget(egyptianCheckBox, 2, 0, 1, 1);

        ethiopicCheckBox = new QCheckBox(displayTab);
        ethiopicCheckBox->setObjectName("ethiopicCheckBox");

        gridLayout->addWidget(ethiopicCheckBox, 3, 1, 1, 1);

        copticCheckBox = new QCheckBox(displayTab);
        copticCheckBox->setObjectName("copticCheckBox");

        gridLayout->addWidget(copticCheckBox, 3, 0, 1, 1);

        armenianCheckBox = new QCheckBox(displayTab);
        armenianCheckBox->setObjectName("armenianCheckBox");

        gridLayout->addWidget(armenianCheckBox, 2, 1, 1, 1);

        oldHinduSolarCheckBox = new QCheckBox(displayTab);
        oldHinduSolarCheckBox->setObjectName("oldHinduSolarCheckBox");

        gridLayout->addWidget(oldHinduSolarCheckBox, 9, 0, 1, 1);

        japaneseCheckBox = new QCheckBox(displayTab);
        japaneseCheckBox->setObjectName("japaneseCheckBox");

        gridLayout->addWidget(japaneseCheckBox, 12, 2, 1, 1);

        frenchArithmeticCheckBox = new QCheckBox(displayTab);
        frenchArithmeticCheckBox->setObjectName("frenchArithmeticCheckBox");

        gridLayout->addWidget(frenchArithmeticCheckBox, 4, 1, 1, 1);

        newHinduSolarCheckBox = new QCheckBox(displayTab);
        newHinduSolarCheckBox->setObjectName("newHinduSolarCheckBox");

        gridLayout->addWidget(newHinduSolarCheckBox, 9, 1, 1, 1);

        hebrewCheckBox = new QCheckBox(displayTab);
        hebrewCheckBox->setObjectName("hebrewCheckBox");

        gridLayout->addWidget(hebrewCheckBox, 4, 2, 1, 1);

        mayaTzolkinCheckBox = new QCheckBox(displayTab);
        mayaTzolkinCheckBox->setObjectName("mayaTzolkinCheckBox");

        gridLayout->addWidget(mayaTzolkinCheckBox, 7, 2, 1, 1);

        persianAstronomicalCheckBox = new QCheckBox(displayTab);
        persianAstronomicalCheckBox->setObjectName("persianAstronomicalCheckBox");

        gridLayout->addWidget(persianAstronomicalCheckBox, 5, 2, 1, 1);

        olympicCheckBox = new QCheckBox(displayTab);
        olympicCheckBox->setObjectName("olympicCheckBox");

        gridLayout->addWidget(olympicCheckBox, 1, 1, 1, 1);

        balineseCheckBox = new QCheckBox(displayTab);
        balineseCheckBox->setObjectName("balineseCheckBox");

        gridLayout->addWidget(balineseCheckBox, 11, 0, 1, 1);

        persianArithmeticCheckBox = new QCheckBox(displayTab);
        persianArithmeticCheckBox->setObjectName("persianArithmeticCheckBox");

        gridLayout->addWidget(persianArithmeticCheckBox, 5, 1, 1, 1);

        astroHinduSolarCheckBox = new QCheckBox(displayTab);
        astroHinduSolarCheckBox->setObjectName("astroHinduSolarCheckBox");

        gridLayout->addWidget(astroHinduSolarCheckBox, 9, 2, 1, 1);

        isoCheckBox = new QCheckBox(displayTab);
        isoCheckBox->setObjectName("isoCheckBox");

        gridLayout->addWidget(isoCheckBox, 1, 2, 1, 1);

        bahaiArithmeticCheckBox = new QCheckBox(displayTab);
        bahaiArithmeticCheckBox->setObjectName("bahaiArithmeticCheckBox");

        gridLayout->addWidget(bahaiArithmeticCheckBox, 6, 1, 1, 1);

        zoroastrianCheckBox = new QCheckBox(displayTab);
        zoroastrianCheckBox->setObjectName("zoroastrianCheckBox");

        gridLayout->addWidget(zoroastrianCheckBox, 3, 2, 1, 1);

        aztecTonalpohualliCheckBox = new QCheckBox(displayTab);
        aztecTonalpohualliCheckBox->setObjectName("aztecTonalpohualliCheckBox");

        gridLayout->addWidget(aztecTonalpohualliCheckBox, 8, 2, 1, 1);

        mayaLCCheckBox = new QCheckBox(displayTab);
        mayaLCCheckBox->setObjectName("mayaLCCheckBox");

        gridLayout->addWidget(mayaLCCheckBox, 7, 0, 1, 1);

        frenchAstronomicalCheckBox = new QCheckBox(displayTab);
        frenchAstronomicalCheckBox->setObjectName("frenchAstronomicalCheckBox");

        gridLayout->addWidget(frenchAstronomicalCheckBox, 4, 0, 1, 1);

        astroHinduLunarCheckBox = new QCheckBox(displayTab);
        astroHinduLunarCheckBox->setObjectName("astroHinduLunarCheckBox");

        gridLayout->addWidget(astroHinduLunarCheckBox, 10, 2, 1, 1);

        oldHinduLunarCheckBox = new QCheckBox(displayTab);
        oldHinduLunarCheckBox->setObjectName("oldHinduLunarCheckBox");

        gridLayout->addWidget(oldHinduLunarCheckBox, 10, 0, 1, 1);

        tibetanCheckBox = new QCheckBox(displayTab);
        tibetanCheckBox->setObjectName("tibetanCheckBox");

        gridLayout->addWidget(tibetanCheckBox, 11, 2, 1, 1);

        romanCheckBox = new QCheckBox(displayTab);
        romanCheckBox->setObjectName("romanCheckBox");

        gridLayout->addWidget(romanCheckBox, 0, 2, 1, 1);

        revisedJulianCheckBox = new QCheckBox(displayTab);
        revisedJulianCheckBox->setObjectName("revisedJulianCheckBox");

        gridLayout->addWidget(revisedJulianCheckBox, 1, 0, 1, 1);

        newHinduLunarCheckBox = new QCheckBox(displayTab);
        newHinduLunarCheckBox->setObjectName("newHinduLunarCheckBox");

        gridLayout->addWidget(newHinduLunarCheckBox, 10, 1, 1, 1);

        gregorianCheckBox = new QCheckBox(displayTab);
        gregorianCheckBox->setObjectName("gregorianCheckBox");

        gridLayout->addWidget(gregorianCheckBox, 0, 1, 1, 1);

        chineseCheckBox = new QCheckBox(displayTab);
        chineseCheckBox->setObjectName("chineseCheckBox");

        gridLayout->addWidget(chineseCheckBox, 12, 0, 1, 1);

        bahaiAstronomicalCheckBox = new QCheckBox(displayTab);
        bahaiAstronomicalCheckBox->setObjectName("bahaiAstronomicalCheckBox");

        gridLayout->addWidget(bahaiAstronomicalCheckBox, 6, 2, 1, 1);

        aztecXihuitlCheckBox = new QCheckBox(displayTab);
        aztecXihuitlCheckBox->setObjectName("aztecXihuitlCheckBox");

        gridLayout->addWidget(aztecXihuitlCheckBox, 8, 1, 1, 1);

        icelandicCheckBox = new QCheckBox(displayTab);
        icelandicCheckBox->setObjectName("icelandicCheckBox");

        gridLayout->addWidget(icelandicCheckBox, 2, 2, 1, 1);

        koreanCheckBox = new QCheckBox(displayTab);
        koreanCheckBox->setObjectName("koreanCheckBox");

        gridLayout->addWidget(koreanCheckBox, 12, 1, 1, 1);

        mayaHaabCheckBox = new QCheckBox(displayTab);
        mayaHaabCheckBox->setObjectName("mayaHaabCheckBox");

        gridLayout->addWidget(mayaHaabCheckBox, 7, 1, 1, 1);

        vietnameseCheckBox = new QCheckBox(displayTab);
        vietnameseCheckBox->setObjectName("vietnameseCheckBox");

        gridLayout->addWidget(vietnameseCheckBox, 13, 1, 1, 1);


        verticalLayout_3->addLayout(gridLayout);

        horizontalLayout_Settings = new QHBoxLayout();
        horizontalLayout_Settings->setObjectName("horizontalLayout_Settings");
        textcolorToolButton = new ColorButton(displayTab);
        textcolorToolButton->setObjectName("textcolorToolButton");

        horizontalLayout_Settings->addWidget(textcolorToolButton);

        overrideTextColorCheckBox = new QCheckBox(displayTab);
        overrideTextColorCheckBox->setObjectName("overrideTextColorCheckBox");

        horizontalLayout_Settings->addWidget(overrideTextColorCheckBox);


        verticalLayout_3->addLayout(horizontalLayout_Settings);

        verticalSpacer_3 = new QSpacerItem(20, 120, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_3->addItem(verticalSpacer_3);

        horizontalLayout_7 = new QHBoxLayout();
        horizontalLayout_7->setObjectName("horizontalLayout_7");
        restoreDefaultsButton = new QPushButton(displayTab);
        restoreDefaultsButton->setObjectName("restoreDefaultsButton");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(restoreDefaultsButton->sizePolicy().hasHeightForWidth());
        restoreDefaultsButton->setSizePolicy(sizePolicy);

        horizontalLayout_7->addWidget(restoreDefaultsButton);


        verticalLayout_3->addLayout(horizontalLayout_7);

        tabs->addTab(displayTab, QString());
        lunisolarTab = new QWidget();
        lunisolarTab->setObjectName("lunisolarTab");
        lunisolarVerticalLayout = new QVBoxLayout(lunisolarTab);
        lunisolarVerticalLayout->setObjectName("lunisolarVerticalLayout");
        lunisolarGridLayout = new QGridLayout();
        lunisolarGridLayout->setObjectName("lunisolarGridLayout");
        isoYearSpinBox = new QSpinBox(lunisolarTab);
        isoYearSpinBox->setObjectName("isoYearSpinBox");
        isoYearSpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        isoYearSpinBox->setMinimum(-1000000);
        isoYearSpinBox->setMaximum(100000);

        lunisolarGridLayout->addWidget(isoYearSpinBox, 4, 1, 1, 1);

        labelDOW = new QLabel(lunisolarTab);
        labelDOW->setObjectName("labelDOW");

        lunisolarGridLayout->addWidget(labelDOW, 0, 4, 1, 1);

        gregorianWeekdayLineEdit = new QLineEdit(lunisolarTab);
        gregorianWeekdayLineEdit->setObjectName("gregorianWeekdayLineEdit");
        gregorianWeekdayLineEdit->setEnabled(false);
        gregorianWeekdayLineEdit->setAcceptDrops(false);
        gregorianWeekdayLineEdit->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        lunisolarGridLayout->addWidget(gregorianWeekdayLineEdit, 3, 4, 1, 1);

        labelMonth = new QLabel(lunisolarTab);
        labelMonth->setObjectName("labelMonth");

        lunisolarGridLayout->addWidget(labelMonth, 0, 2, 1, 1);

        julianMonthSpinBox = new QSpinBox(lunisolarTab);
        julianMonthSpinBox->setObjectName("julianMonthSpinBox");
        julianMonthSpinBox->setWrapping(true);
        julianMonthSpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        julianMonthSpinBox->setMinimum(0);
        julianMonthSpinBox->setMaximum(13);

        lunisolarGridLayout->addWidget(julianMonthSpinBox, 1, 2, 1, 1);

        gregorianDaySpinBox = new QSpinBox(lunisolarTab);
        gregorianDaySpinBox->setObjectName("gregorianDaySpinBox");
        gregorianDaySpinBox->setWrapping(true);
        gregorianDaySpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        gregorianDaySpinBox->setMinimum(0);
        gregorianDaySpinBox->setMaximum(32);

        lunisolarGridLayout->addWidget(gregorianDaySpinBox, 3, 3, 1, 1);

        labelRD = new QLabel(lunisolarTab);
        labelRD->setObjectName("labelRD");
        labelRD->setText(QString::fromUtf8("Rata Die"));

        lunisolarGridLayout->addWidget(labelRD, 5, 0, 1, 1);

        isoWeekSpinBox = new QSpinBox(lunisolarTab);
        isoWeekSpinBox->setObjectName("isoWeekSpinBox");
        isoWeekSpinBox->setWrapping(true);
        isoWeekSpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        isoWeekSpinBox->setMinimum(0);
        isoWeekSpinBox->setMaximum(54);

        lunisolarGridLayout->addWidget(isoWeekSpinBox, 4, 2, 1, 1);

        gregorianMonthSpinBox = new QSpinBox(lunisolarTab);
        gregorianMonthSpinBox->setObjectName("gregorianMonthSpinBox");
        gregorianMonthSpinBox->setWrapping(true);
        gregorianMonthSpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        gregorianMonthSpinBox->setMinimum(0);
        gregorianMonthSpinBox->setMaximum(13);

        lunisolarGridLayout->addWidget(gregorianMonthSpinBox, 3, 2, 1, 1);

        julianDaySpinBox = new QSpinBox(lunisolarTab);
        julianDaySpinBox->setObjectName("julianDaySpinBox");
        julianDaySpinBox->setWrapping(true);
        julianDaySpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        julianDaySpinBox->setMinimum(0);
        julianDaySpinBox->setMaximum(32);

        lunisolarGridLayout->addWidget(julianDaySpinBox, 1, 3, 1, 1);

        gregorianYearSpinBox = new QSpinBox(lunisolarTab);
        gregorianYearSpinBox->setObjectName("gregorianYearSpinBox");
        gregorianYearSpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        gregorianYearSpinBox->setMinimum(-100000);
        gregorianYearSpinBox->setMaximum(100000);
        gregorianYearSpinBox->setValue(2000);

        lunisolarGridLayout->addWidget(gregorianYearSpinBox, 3, 1, 1, 1);

        labelRDvalue = new QLabel(lunisolarTab);
        labelRDvalue->setObjectName("labelRDvalue");
        labelRDvalue->setText(QString::fromUtf8("RD number"));

        lunisolarGridLayout->addWidget(labelRDvalue, 5, 4, 1, 1);

        labelISOWeekCalendar = new QLabel(lunisolarTab);
        labelISOWeekCalendar->setObjectName("labelISOWeekCalendar");

        lunisolarGridLayout->addWidget(labelISOWeekCalendar, 4, 0, 1, 1);

        julianWeekdayLineEdit = new QLineEdit(lunisolarTab);
        julianWeekdayLineEdit->setObjectName("julianWeekdayLineEdit");
        julianWeekdayLineEdit->setEnabled(false);
        julianWeekdayLineEdit->setAcceptDrops(false);
        julianWeekdayLineEdit->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        lunisolarGridLayout->addWidget(julianWeekdayLineEdit, 1, 4, 1, 1);

        labelYear = new QLabel(lunisolarTab);
        labelYear->setObjectName("labelYear");

        lunisolarGridLayout->addWidget(labelYear, 0, 1, 1, 1);

        isoDaySpinBox = new QSpinBox(lunisolarTab);
        isoDaySpinBox->setObjectName("isoDaySpinBox");
        isoDaySpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        isoDaySpinBox->setMinimum(0);
        isoDaySpinBox->setMaximum(8);
        isoDaySpinBox->setValue(1);

        lunisolarGridLayout->addWidget(isoDaySpinBox, 4, 3, 1, 1);

        labelDay = new QLabel(lunisolarTab);
        labelDay->setObjectName("labelDay");

        lunisolarGridLayout->addWidget(labelDay, 0, 3, 1, 1);

        julianYearSpinBox = new QSpinBox(lunisolarTab);
        julianYearSpinBox->setObjectName("julianYearSpinBox");
        julianYearSpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        julianYearSpinBox->setMinimum(-100000);
        julianYearSpinBox->setMaximum(100000);
        julianYearSpinBox->setValue(2000);

        lunisolarGridLayout->addWidget(julianYearSpinBox, 1, 1, 1, 1);

        labelGregorianCalendar = new QLabel(lunisolarTab);
        labelGregorianCalendar->setObjectName("labelGregorianCalendar");

        lunisolarGridLayout->addWidget(labelGregorianCalendar, 3, 0, 1, 1);

        labelJulianCalendar = new QLabel(lunisolarTab);
        labelJulianCalendar->setObjectName("labelJulianCalendar");

        lunisolarGridLayout->addWidget(labelJulianCalendar, 1, 0, 1, 1);

        labelRevisedJulianCalendar = new QLabel(lunisolarTab);
        labelRevisedJulianCalendar->setObjectName("labelRevisedJulianCalendar");

        lunisolarGridLayout->addWidget(labelRevisedJulianCalendar, 2, 0, 1, 1);

        revisedJulianYearSpinBox = new QSpinBox(lunisolarTab);
        revisedJulianYearSpinBox->setObjectName("revisedJulianYearSpinBox");
        revisedJulianYearSpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        revisedJulianYearSpinBox->setMinimum(-100000);
        revisedJulianYearSpinBox->setMaximum(100000);
        revisedJulianYearSpinBox->setValue(2000);

        lunisolarGridLayout->addWidget(revisedJulianYearSpinBox, 2, 1, 1, 1);

        revisedJulianMonthSpinBox = new QSpinBox(lunisolarTab);
        revisedJulianMonthSpinBox->setObjectName("revisedJulianMonthSpinBox");
        revisedJulianMonthSpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        revisedJulianMonthSpinBox->setMaximum(13);

        lunisolarGridLayout->addWidget(revisedJulianMonthSpinBox, 2, 2, 1, 1);

        revisedJulianDaySpinBox = new QSpinBox(lunisolarTab);
        revisedJulianDaySpinBox->setObjectName("revisedJulianDaySpinBox");
        revisedJulianDaySpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        revisedJulianDaySpinBox->setMaximum(32);

        lunisolarGridLayout->addWidget(revisedJulianDaySpinBox, 2, 3, 1, 1);

        revisedJulianWeekdayLineEdit = new QLineEdit(lunisolarTab);
        revisedJulianWeekdayLineEdit->setObjectName("revisedJulianWeekdayLineEdit");
        revisedJulianWeekdayLineEdit->setEnabled(false);
        revisedJulianWeekdayLineEdit->setAcceptDrops(false);
        revisedJulianWeekdayLineEdit->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        lunisolarGridLayout->addWidget(revisedJulianWeekdayLineEdit, 2, 4, 1, 1);


        lunisolarVerticalLayout->addLayout(lunisolarGridLayout);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        lunisolarVerticalLayout->addItem(verticalSpacer);

        tabs->addTab(lunisolarTab, QString());
        chineseTab = new QWidget();
        chineseTab->setObjectName("chineseTab");
        chineseVerticalLayout = new QVBoxLayout(chineseTab);
        chineseVerticalLayout->setObjectName("chineseVerticalLayout");
        tabs->addTab(chineseTab, QString());
        mesoamericaTab = new QWidget();
        mesoamericaTab->setObjectName("mesoamericaTab");
        mesoamericaVerticalLayout = new QVBoxLayout(mesoamericaTab);
        mesoamericaVerticalLayout->setObjectName("mesoamericaVerticalLayout");
        mesoamericaGridLayout = new QGridLayout();
        mesoamericaGridLayout->setObjectName("mesoamericaGridLayout");
        labelKatun = new QLabel(mesoamericaTab);
        labelKatun->setObjectName("labelKatun");
        labelKatun->setAlignment(Qt::AlignCenter);

        mesoamericaGridLayout->addWidget(labelKatun, 0, 2, 1, 1);

        labelXihuitl = new QLabel(mesoamericaTab);
        labelXihuitl->setObjectName("labelXihuitl");

        mesoamericaGridLayout->addWidget(labelXihuitl, 5, 0, 1, 1);

        tunSpinBox = new QSpinBox(mesoamericaTab);
        tunSpinBox->setObjectName("tunSpinBox");
        tunSpinBox->setWrapping(true);
        tunSpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        tunSpinBox->setMinimum(-1);
        tunSpinBox->setMaximum(20);

        mesoamericaGridLayout->addWidget(tunSpinBox, 1, 3, 1, 1);

        katunSpinBox = new QSpinBox(mesoamericaTab);
        katunSpinBox->setObjectName("katunSpinBox");
        katunSpinBox->setWrapping(true);
        katunSpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        katunSpinBox->setMinimum(-1);
        katunSpinBox->setMaximum(20);

        mesoamericaGridLayout->addWidget(katunSpinBox, 1, 2, 1, 1);

        labelMaya = new QLabel(mesoamericaTab);
        labelMaya->setObjectName("labelMaya");

        mesoamericaGridLayout->addWidget(labelMaya, 0, 0, 1, 1);

        baktunSpinBox = new QSpinBox(mesoamericaTab);
        baktunSpinBox->setObjectName("baktunSpinBox");
        baktunSpinBox->setWrapping(true);
        baktunSpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        baktunSpinBox->setMinimum(-1000);
        baktunSpinBox->setMaximum(1000);

        mesoamericaGridLayout->addWidget(baktunSpinBox, 1, 1, 1, 1);

        haabDaySpinBox = new QSpinBox(mesoamericaTab);
        haabDaySpinBox->setObjectName("haabDaySpinBox");
        haabDaySpinBox->setWrapping(true);
        haabDaySpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        haabDaySpinBox->setMaximum(19);

        mesoamericaGridLayout->addWidget(haabDaySpinBox, 2, 2, 1, 1);

        labelHaab = new QLabel(mesoamericaTab);
        labelHaab->setObjectName("labelHaab");

        mesoamericaGridLayout->addWidget(labelHaab, 2, 0, 1, 1);

        labelTun = new QLabel(mesoamericaTab);
        labelTun->setObjectName("labelTun");
        labelTun->setAlignment(Qt::AlignCenter);

        mesoamericaGridLayout->addWidget(labelTun, 0, 3, 1, 1);

        labelKin = new QLabel(mesoamericaTab);
        labelKin->setObjectName("labelKin");
        labelKin->setAlignment(Qt::AlignCenter);

        mesoamericaGridLayout->addWidget(labelKin, 0, 5, 1, 1);

        tzolkinLineEdit = new QLineEdit(mesoamericaTab);
        tzolkinLineEdit->setObjectName("tzolkinLineEdit");
        tzolkinLineEdit->setEnabled(false);
        tzolkinLineEdit->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        mesoamericaGridLayout->addWidget(tzolkinLineEdit, 3, 3, 1, 1);

        haabMonthSpinBox = new QSpinBox(mesoamericaTab);
        haabMonthSpinBox->setObjectName("haabMonthSpinBox");
        haabMonthSpinBox->setWrapping(true);
        haabMonthSpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        haabMonthSpinBox->setMinimum(1);
        haabMonthSpinBox->setMaximum(19);

        mesoamericaGridLayout->addWidget(haabMonthSpinBox, 2, 1, 1, 1);

        labelUinal = new QLabel(mesoamericaTab);
        labelUinal->setObjectName("labelUinal");
        labelUinal->setAlignment(Qt::AlignCenter);

        mesoamericaGridLayout->addWidget(labelUinal, 0, 4, 1, 1);

        labelBaktun = new QLabel(mesoamericaTab);
        labelBaktun->setObjectName("labelBaktun");
        labelBaktun->setAlignment(Qt::AlignCenter);

        mesoamericaGridLayout->addWidget(labelBaktun, 0, 1, 1, 1);

        labelLongCount = new QLabel(mesoamericaTab);
        labelLongCount->setObjectName("labelLongCount");

        mesoamericaGridLayout->addWidget(labelLongCount, 1, 0, 1, 1);

        tzolkinNameSpinBox = new QSpinBox(mesoamericaTab);
        tzolkinNameSpinBox->setObjectName("tzolkinNameSpinBox");
        tzolkinNameSpinBox->setWrapping(true);
        tzolkinNameSpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        tzolkinNameSpinBox->setMinimum(1);
        tzolkinNameSpinBox->setMaximum(20);

        mesoamericaGridLayout->addWidget(tzolkinNameSpinBox, 3, 2, 1, 1);

        nextTzolkinButton = new QPushButton(mesoamericaTab);
        nextTzolkinButton->setObjectName("nextTzolkinButton");
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/graphicGui/btTimeForward-off.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        nextTzolkinButton->setIcon(icon);

        mesoamericaGridLayout->addWidget(nextTzolkinButton, 3, 5, 1, 1);

        tzolkinNumberSpinBox = new QSpinBox(mesoamericaTab);
        tzolkinNumberSpinBox->setObjectName("tzolkinNumberSpinBox");
        tzolkinNumberSpinBox->setWrapping(true);
        tzolkinNumberSpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        tzolkinNumberSpinBox->setMinimum(1);
        tzolkinNumberSpinBox->setMaximum(13);

        mesoamericaGridLayout->addWidget(tzolkinNumberSpinBox, 3, 1, 1, 1);

        uinalSpinBox = new QSpinBox(mesoamericaTab);
        uinalSpinBox->setObjectName("uinalSpinBox");
        uinalSpinBox->setWrapping(true);
        uinalSpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        uinalSpinBox->setMinimum(-1);
        uinalSpinBox->setMaximum(18);

        mesoamericaGridLayout->addWidget(uinalSpinBox, 1, 4, 1, 1);

        kinSpinBox = new QSpinBox(mesoamericaTab);
        kinSpinBox->setObjectName("kinSpinBox");
        kinSpinBox->setWrapping(true);
        kinSpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        kinSpinBox->setMinimum(-1);
        kinSpinBox->setMaximum(20);

        mesoamericaGridLayout->addWidget(kinSpinBox, 1, 5, 1, 1);

        nextHaabButton = new QPushButton(mesoamericaTab);
        nextHaabButton->setObjectName("nextHaabButton");
        nextHaabButton->setIcon(icon);

        mesoamericaGridLayout->addWidget(nextHaabButton, 2, 5, 1, 1);

        previousTzolkinButton = new QPushButton(mesoamericaTab);
        previousTzolkinButton->setObjectName("previousTzolkinButton");
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/graphicGui/btTimeRewind-off.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        previousTzolkinButton->setIcon(icon1);

        mesoamericaGridLayout->addWidget(previousTzolkinButton, 3, 4, 1, 1);

        haabLineEdit = new QLineEdit(mesoamericaTab);
        haabLineEdit->setObjectName("haabLineEdit");
        haabLineEdit->setEnabled(false);
        haabLineEdit->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        mesoamericaGridLayout->addWidget(haabLineEdit, 2, 3, 1, 1);

        previousHaabButton = new QPushButton(mesoamericaTab);
        previousHaabButton->setObjectName("previousHaabButton");
        previousHaabButton->setIcon(icon1);

        mesoamericaGridLayout->addWidget(previousHaabButton, 2, 4, 1, 1);

        labelAztec = new QLabel(mesoamericaTab);
        labelAztec->setObjectName("labelAztec");

        mesoamericaGridLayout->addWidget(labelAztec, 4, 0, 1, 1);

        labelTzolkin = new QLabel(mesoamericaTab);
        labelTzolkin->setObjectName("labelTzolkin");

        mesoamericaGridLayout->addWidget(labelTzolkin, 3, 0, 1, 1);

        labelTonalpohualli = new QLabel(mesoamericaTab);
        labelTonalpohualli->setObjectName("labelTonalpohualli");

        mesoamericaGridLayout->addWidget(labelTonalpohualli, 6, 0, 1, 1);

        xihuitlMonthSpinBox = new QSpinBox(mesoamericaTab);
        xihuitlMonthSpinBox->setObjectName("xihuitlMonthSpinBox");
        xihuitlMonthSpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        xihuitlMonthSpinBox->setMinimum(1);
        xihuitlMonthSpinBox->setMaximum(19);

        mesoamericaGridLayout->addWidget(xihuitlMonthSpinBox, 5, 1, 1, 1);

        tonalpohualliNumberSpinBox = new QSpinBox(mesoamericaTab);
        tonalpohualliNumberSpinBox->setObjectName("tonalpohualliNumberSpinBox");
        tonalpohualliNumberSpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        tonalpohualliNumberSpinBox->setMinimum(1);
        tonalpohualliNumberSpinBox->setMaximum(13);

        mesoamericaGridLayout->addWidget(tonalpohualliNumberSpinBox, 6, 1, 1, 1);

        xihuitlDaySpinBox = new QSpinBox(mesoamericaTab);
        xihuitlDaySpinBox->setObjectName("xihuitlDaySpinBox");
        xihuitlDaySpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        xihuitlDaySpinBox->setMinimum(1);
        xihuitlDaySpinBox->setMaximum(20);

        mesoamericaGridLayout->addWidget(xihuitlDaySpinBox, 5, 2, 1, 1);

        tonalpohualliNameSpinBox = new QSpinBox(mesoamericaTab);
        tonalpohualliNameSpinBox->setObjectName("tonalpohualliNameSpinBox");
        tonalpohualliNameSpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        tonalpohualliNameSpinBox->setMinimum(1);
        tonalpohualliNameSpinBox->setMaximum(20);

        mesoamericaGridLayout->addWidget(tonalpohualliNameSpinBox, 6, 2, 1, 1);

        xihuitlLineEdit = new QLineEdit(mesoamericaTab);
        xihuitlLineEdit->setObjectName("xihuitlLineEdit");
        xihuitlLineEdit->setEnabled(false);
        xihuitlLineEdit->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        mesoamericaGridLayout->addWidget(xihuitlLineEdit, 5, 3, 1, 1);

        tonalpohualliLineEdit = new QLineEdit(mesoamericaTab);
        tonalpohualliLineEdit->setObjectName("tonalpohualliLineEdit");
        tonalpohualliLineEdit->setEnabled(false);
        tonalpohualliLineEdit->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        mesoamericaGridLayout->addWidget(tonalpohualliLineEdit, 6, 3, 1, 1);

        previousXihuitlButton = new QPushButton(mesoamericaTab);
        previousXihuitlButton->setObjectName("previousXihuitlButton");
        previousXihuitlButton->setIcon(icon1);

        mesoamericaGridLayout->addWidget(previousXihuitlButton, 5, 4, 1, 1);

        nextXihuitlButton = new QPushButton(mesoamericaTab);
        nextXihuitlButton->setObjectName("nextXihuitlButton");
        nextXihuitlButton->setIcon(icon);

        mesoamericaGridLayout->addWidget(nextXihuitlButton, 5, 5, 1, 1);

        previousTonalpohualliButton = new QPushButton(mesoamericaTab);
        previousTonalpohualliButton->setObjectName("previousTonalpohualliButton");
        previousTonalpohualliButton->setIcon(icon1);

        mesoamericaGridLayout->addWidget(previousTonalpohualliButton, 6, 4, 1, 1);

        nextTonalpohualliButton = new QPushButton(mesoamericaTab);
        nextTonalpohualliButton->setObjectName("nextTonalpohualliButton");
        nextTonalpohualliButton->setIcon(icon);

        mesoamericaGridLayout->addWidget(nextTonalpohualliButton, 6, 5, 1, 1);


        mesoamericaVerticalLayout->addLayout(mesoamericaGridLayout);

        verticalSpacerMesoamerica = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        mesoamericaVerticalLayout->addItem(verticalSpacerMesoamerica);

        tabs->addTab(mesoamericaTab, QString());
        aboutTab = new QWidget();
        aboutTab->setObjectName("aboutTab");
        verticalLayout_11 = new QVBoxLayout(aboutTab);
        verticalLayout_11->setSpacing(9);
        verticalLayout_11->setObjectName("verticalLayout_11");
        aboutTextBrowser = new QTextBrowser(aboutTab);
        aboutTextBrowser->setObjectName("aboutTextBrowser");
        aboutTextBrowser->setOpenExternalLinks(true);
        aboutTextBrowser->setOpenLinks(true);

        verticalLayout_11->addWidget(aboutTextBrowser);

        tabs->addTab(aboutTab, QString());

        verticalLayout_2->addWidget(tabs);


        retranslateUi(calendarsDialog);

        tabs->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(calendarsDialog);
    } // setupUi

    void retranslateUi(QWidget *calendarsDialog)
    {
        calendarsDialog->setWindowTitle(QCoreApplication::translate("calendarsDialog", "Calendars Configuration", nullptr));
        titleBar->setTitle(QCoreApplication::translate("calendarsDialog", "Calendars Plug-in Configuration", nullptr));
        islamicCheckBox->setText(QCoreApplication::translate("calendarsDialog", "Islamic", "calendar"));
        julianCheckBox->setText(QCoreApplication::translate("calendarsDialog", "Julian", "calendar"));
        egyptianCheckBox->setText(QCoreApplication::translate("calendarsDialog", "Egyptian", "calendar"));
        ethiopicCheckBox->setText(QCoreApplication::translate("calendarsDialog", "Ethiopic", "calendar"));
        copticCheckBox->setText(QCoreApplication::translate("calendarsDialog", "Coptic", "calendar"));
        armenianCheckBox->setText(QCoreApplication::translate("calendarsDialog", "Armenian", "calendar"));
        oldHinduSolarCheckBox->setText(QCoreApplication::translate("calendarsDialog", "Old Hindu Solar", "calendar"));
        japaneseCheckBox->setText(QCoreApplication::translate("calendarsDialog", "Japanese", "calendar"));
        frenchArithmeticCheckBox->setText(QCoreApplication::translate("calendarsDialog", "French Revolution (arithmetic)", "calendar"));
        newHinduSolarCheckBox->setText(QCoreApplication::translate("calendarsDialog", "New Hindu Solar", "calendar"));
        hebrewCheckBox->setText(QCoreApplication::translate("calendarsDialog", "Hebrew", "calendar"));
        mayaTzolkinCheckBox->setText(QCoreApplication::translate("calendarsDialog", "Maya Tzolkin", "calendar"));
        persianAstronomicalCheckBox->setText(QCoreApplication::translate("calendarsDialog", "Persian (astronomical)", "calendar"));
        olympicCheckBox->setText(QCoreApplication::translate("calendarsDialog", "Olympic", "calendar"));
        balineseCheckBox->setText(QCoreApplication::translate("calendarsDialog", "Balinese Pawukon", "calendar"));
        persianArithmeticCheckBox->setText(QCoreApplication::translate("calendarsDialog", "Persian (arithmetic)", "calendar"));
        astroHinduSolarCheckBox->setText(QCoreApplication::translate("calendarsDialog", "Hindu Astro Solar", "calendar"));
        isoCheckBox->setText(QCoreApplication::translate("calendarsDialog", "ISO Week", "calendar"));
        bahaiArithmeticCheckBox->setText(QCoreApplication::translate("calendarsDialog", "Bah\303\241\342\200\231\303\255 (arithmetic)", "calendar"));
        zoroastrianCheckBox->setText(QCoreApplication::translate("calendarsDialog", "Zoroastrian", "calendar"));
        aztecTonalpohualliCheckBox->setText(QCoreApplication::translate("calendarsDialog", "Aztec Tonalpohualli", "calendar"));
        mayaLCCheckBox->setText(QCoreApplication::translate("calendarsDialog", "Maya Long Count", "calendar"));
        frenchAstronomicalCheckBox->setText(QCoreApplication::translate("calendarsDialog", "French Revolution (astronomical)", "calendar"));
        astroHinduLunarCheckBox->setText(QCoreApplication::translate("calendarsDialog", "Hindu Astro Lunisolar", "calendar"));
        oldHinduLunarCheckBox->setText(QCoreApplication::translate("calendarsDialog", "Old Hindu Lunar", "calendar"));
        tibetanCheckBox->setText(QCoreApplication::translate("calendarsDialog", "Tibetan", "calendar"));
        romanCheckBox->setText(QCoreApplication::translate("calendarsDialog", "Roman", "calendar"));
        revisedJulianCheckBox->setText(QCoreApplication::translate("calendarsDialog", "Revised Julian", "calendar"));
        newHinduLunarCheckBox->setText(QCoreApplication::translate("calendarsDialog", "New Hindu Lunar", "calendar"));
        gregorianCheckBox->setText(QCoreApplication::translate("calendarsDialog", "Gregorian", "calendar"));
        chineseCheckBox->setText(QCoreApplication::translate("calendarsDialog", "Chinese", "calendar"));
        bahaiAstronomicalCheckBox->setText(QCoreApplication::translate("calendarsDialog", "Bah\303\241\342\200\231\303\255 (astronomical)", "calendar"));
        aztecXihuitlCheckBox->setText(QCoreApplication::translate("calendarsDialog", "Aztec Xihuitl", "calendar"));
        icelandicCheckBox->setText(QCoreApplication::translate("calendarsDialog", "Icelandic", "calendar"));
        koreanCheckBox->setText(QCoreApplication::translate("calendarsDialog", "Korean", "calendar"));
        mayaHaabCheckBox->setText(QCoreApplication::translate("calendarsDialog", "Maya Haab", "calendar"));
        vietnameseCheckBox->setText(QCoreApplication::translate("calendarsDialog", "Vietnamese", "calendar"));
        textcolorToolButton->setText(QString());
        overrideTextColorCheckBox->setText(QCoreApplication::translate("calendarsDialog", "Override automatic text color", nullptr));
        restoreDefaultsButton->setText(QCoreApplication::translate("calendarsDialog", "Restore defaults", nullptr));
        tabs->setTabText(tabs->indexOf(displayTab), QCoreApplication::translate("calendarsDialog", "Display", nullptr));
        labelDOW->setText(QCoreApplication::translate("calendarsDialog", "Weekday", nullptr));
        labelMonth->setText(QCoreApplication::translate("calendarsDialog", "Month", nullptr));
        labelISOWeekCalendar->setText(QCoreApplication::translate("calendarsDialog", "ISO Week", "calendar"));
        labelYear->setText(QCoreApplication::translate("calendarsDialog", "Year", nullptr));
        labelDay->setText(QCoreApplication::translate("calendarsDialog", "Day", nullptr));
#if QT_CONFIG(tooltip)
        julianYearSpinBox->setToolTip(QCoreApplication::translate("calendarsDialog", "Negative years here mark years B.C.", "calendar"));
#endif // QT_CONFIG(tooltip)
        labelGregorianCalendar->setText(QCoreApplication::translate("calendarsDialog", "Gregorian", "calendar"));
        labelJulianCalendar->setText(QCoreApplication::translate("calendarsDialog", "Julian", "calendar"));
        labelRevisedJulianCalendar->setText(QCoreApplication::translate("calendarsDialog", "Revised Julian", "calendar"));
        tabs->setTabText(tabs->indexOf(lunisolarTab), QCoreApplication::translate("calendarsDialog", "Lunisolar", nullptr));
        tabs->setTabText(tabs->indexOf(chineseTab), QCoreApplication::translate("calendarsDialog", "Chinese", "calendar"));
        labelKatun->setText(QCoreApplication::translate("calendarsDialog", "Katun", "calendar"));
        labelXihuitl->setText(QCoreApplication::translate("calendarsDialog", "Xihuitl", "calendar"));
        labelMaya->setText(QCoreApplication::translate("calendarsDialog", "Maya", "calendar"));
        labelHaab->setText(QCoreApplication::translate("calendarsDialog", "Haab", "calendar"));
        labelTun->setText(QCoreApplication::translate("calendarsDialog", "Tun", "calendar"));
        labelKin->setText(QCoreApplication::translate("calendarsDialog", "Kin", "calendar"));
        labelUinal->setText(QCoreApplication::translate("calendarsDialog", "Uinal", "calendar"));
        labelBaktun->setText(QCoreApplication::translate("calendarsDialog", "Baktun", "calendar"));
        labelLongCount->setText(QCoreApplication::translate("calendarsDialog", "Long Count", "calendar"));
#if QT_CONFIG(tooltip)
        nextTzolkinButton->setToolTip(QCoreApplication::translate("calendarsDialog", "Step forward", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        nextHaabButton->setToolTip(QCoreApplication::translate("calendarsDialog", "Step forward", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        previousTzolkinButton->setToolTip(QCoreApplication::translate("calendarsDialog", "Step backward", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        previousHaabButton->setToolTip(QCoreApplication::translate("calendarsDialog", "Step backward", nullptr));
#endif // QT_CONFIG(tooltip)
        labelAztec->setText(QCoreApplication::translate("calendarsDialog", "Aztec", "calendar"));
        labelTzolkin->setText(QCoreApplication::translate("calendarsDialog", "Tzolkin", "calendar"));
        labelTonalpohualli->setText(QCoreApplication::translate("calendarsDialog", "Tonalpohualli", "calendar"));
#if QT_CONFIG(tooltip)
        previousXihuitlButton->setToolTip(QCoreApplication::translate("calendarsDialog", "Step backward", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        nextXihuitlButton->setToolTip(QCoreApplication::translate("calendarsDialog", "Step forward", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        previousTonalpohualliButton->setToolTip(QCoreApplication::translate("calendarsDialog", "Step backward", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        nextTonalpohualliButton->setToolTip(QCoreApplication::translate("calendarsDialog", "Step forward", nullptr));
#endif // QT_CONFIG(tooltip)
        tabs->setTabText(tabs->indexOf(mesoamericaTab), QCoreApplication::translate("calendarsDialog", "Mesoamerica", "calendar"));
        tabs->setTabText(tabs->indexOf(aboutTab), QCoreApplication::translate("calendarsDialog", "About", "tab in plugin windows"));
    } // retranslateUi

};

namespace Ui {
    class calendarsDialog: public Ui_calendarsDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_CALENDARSDIALOG_H
