/****************************************************************************
** Meta object code from reading C++ file 'NomenclatureMgr.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../stellarium/src/core/modules/NomenclatureMgr.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'NomenclatureMgr.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15NomenclatureMgrE_t {};
} // unnamed namespace

template <> constexpr inline auto NomenclatureMgr::qt_create_metaobjectdata<qt_meta_tag_ZN15NomenclatureMgrE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "NomenclatureMgr",
        "flagShowNomenclatureChanged",
        "",
        "b",
        "flagShowTerminatorZoneOnlyChanged",
        "terminatorMinAltitudeChanged",
        "deg",
        "terminatorMaxAltitudeChanged",
        "flagOutlineCratersChanged",
        "localNomenclatureHidingChanged",
        "specialNomenclatureOnlyDisplayingChanged",
        "nomenclatureColorChanged",
        "Vec3f",
        "color",
        "setColor",
        "c",
        "getColor",
        "setFlagShowNomenclature",
        "getFlagShowNomenclature",
        "setFlagShowTerminatorZoneOnly",
        "getFlagShowTerminatorZoneOnly",
        "setTerminatorMinAltitude",
        "getTerminatorMinAltitude",
        "setTerminatorMaxAltitude",
        "getTerminatorMaxAltitude",
        "setFlagOutlineCraters",
        "getFlagOutlineCraters",
        "setFlagHideLocalNomenclature",
        "getFlagHideLocalNomenclature",
        "setFlagShowSpecialNomenclatureOnly",
        "getFlagShowSpecialNomenclatureOnly",
        "updateI18n",
        "updateNomenclatureData",
        "setForceItems",
        "setFontSize",
        "size",
        "flagShowNomenclature",
        "flagShowTerminatorZoneOnly",
        "terminatorMinAltitude",
        "terminatorMaxAltitude",
        "flagOutlineCraters",
        "flagHideLocalNomenclature",
        "specialNomenclatureOnlyDisplayed",
        "nomenclatureColor"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'flagShowNomenclatureChanged'
        QtMocHelpers::SignalData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagShowTerminatorZoneOnlyChanged'
        QtMocHelpers::SignalData<void(bool)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'terminatorMinAltitudeChanged'
        QtMocHelpers::SignalData<void(int)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 6 },
        }}),
        // Signal 'terminatorMaxAltitudeChanged'
        QtMocHelpers::SignalData<void(int)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 6 },
        }}),
        // Signal 'flagOutlineCratersChanged'
        QtMocHelpers::SignalData<void(bool)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'localNomenclatureHidingChanged'
        QtMocHelpers::SignalData<void(bool)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'specialNomenclatureOnlyDisplayingChanged'
        QtMocHelpers::SignalData<void(bool)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'nomenclatureColorChanged'
        QtMocHelpers::SignalData<void(const Vec3f &) const>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Slot 'setColor'
        QtMocHelpers::SlotData<void(const Vec3f &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 15 },
        }}),
        // Slot 'getColor'
        QtMocHelpers::SlotData<void() const>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setFlagShowNomenclature'
        QtMocHelpers::SlotData<void(bool)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'getFlagShowNomenclature'
        QtMocHelpers::SlotData<bool() const>(18, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFlagShowTerminatorZoneOnly'
        QtMocHelpers::SlotData<void(bool)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'getFlagShowTerminatorZoneOnly'
        QtMocHelpers::SlotData<bool() const>(20, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setTerminatorMinAltitude'
        QtMocHelpers::SlotData<void(int)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 6 },
        }}),
        // Slot 'getTerminatorMinAltitude'
        QtMocHelpers::SlotData<int() const>(22, 2, QMC::AccessPublic, QMetaType::Int),
        // Slot 'setTerminatorMaxAltitude'
        QtMocHelpers::SlotData<void(int)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 6 },
        }}),
        // Slot 'getTerminatorMaxAltitude'
        QtMocHelpers::SlotData<int() const>(24, 2, QMC::AccessPublic, QMetaType::Int),
        // Slot 'setFlagOutlineCraters'
        QtMocHelpers::SlotData<void(bool)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'getFlagOutlineCraters'
        QtMocHelpers::SlotData<bool() const>(26, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFlagHideLocalNomenclature'
        QtMocHelpers::SlotData<void(bool)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'getFlagHideLocalNomenclature'
        QtMocHelpers::SlotData<bool() const>(28, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFlagShowSpecialNomenclatureOnly'
        QtMocHelpers::SlotData<void(bool)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'getFlagShowSpecialNomenclatureOnly'
        QtMocHelpers::SlotData<bool() const>(30, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'updateI18n'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'updateNomenclatureData'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setForceItems'
        QtMocHelpers::SlotData<void(bool)>(33, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'setFontSize'
        QtMocHelpers::SlotData<void(int)>(34, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 35 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'flagShowNomenclature'
        QtMocHelpers::PropertyData<bool>(36, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'flagShowTerminatorZoneOnly'
        QtMocHelpers::PropertyData<bool>(37, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'terminatorMinAltitude'
        QtMocHelpers::PropertyData<int>(38, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'terminatorMaxAltitude'
        QtMocHelpers::PropertyData<int>(39, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'flagOutlineCraters'
        QtMocHelpers::PropertyData<bool>(40, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'flagHideLocalNomenclature'
        QtMocHelpers::PropertyData<bool>(41, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
        // property 'specialNomenclatureOnlyDisplayed'
        QtMocHelpers::PropertyData<bool>(42, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 6),
        // property 'nomenclatureColor'
        QtMocHelpers::PropertyData<Vec3f>(43, 0x80000000 | 12, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag, 7),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<NomenclatureMgr, qt_meta_tag_ZN15NomenclatureMgrE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject NomenclatureMgr::staticMetaObject = { {
    QMetaObject::SuperData::link<StelObjectModule::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15NomenclatureMgrE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15NomenclatureMgrE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15NomenclatureMgrE_t>.metaTypes,
    nullptr
} };

void NomenclatureMgr::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<NomenclatureMgr *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->flagShowNomenclatureChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->flagShowTerminatorZoneOnlyChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->terminatorMinAltitudeChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 3: _t->terminatorMaxAltitudeChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->flagOutlineCratersChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->localNomenclatureHidingChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->specialNomenclatureOnlyDisplayingChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->nomenclatureColorChanged((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1]))); break;
        case 8: _t->setColor((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1]))); break;
        case 9: _t->getColor(); break;
        case 10: _t->setFlagShowNomenclature((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 11: { bool _r = _t->getFlagShowNomenclature();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 12: _t->setFlagShowTerminatorZoneOnly((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 13: { bool _r = _t->getFlagShowTerminatorZoneOnly();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 14: _t->setTerminatorMinAltitude((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 15: { int _r = _t->getTerminatorMinAltitude();
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 16: _t->setTerminatorMaxAltitude((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 17: { int _r = _t->getTerminatorMaxAltitude();
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 18: _t->setFlagOutlineCraters((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 19: { bool _r = _t->getFlagOutlineCraters();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 20: _t->setFlagHideLocalNomenclature((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 21: { bool _r = _t->getFlagHideLocalNomenclature();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 22: _t->setFlagShowSpecialNomenclatureOnly((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 23: { bool _r = _t->getFlagShowSpecialNomenclatureOnly();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 24: _t->updateI18n(); break;
        case 25: _t->updateNomenclatureData(); break;
        case 26: _t->setForceItems((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 27: _t->setFontSize((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (NomenclatureMgr::*)(bool )>(_a, &NomenclatureMgr::flagShowNomenclatureChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (NomenclatureMgr::*)(bool )>(_a, &NomenclatureMgr::flagShowTerminatorZoneOnlyChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (NomenclatureMgr::*)(int )>(_a, &NomenclatureMgr::terminatorMinAltitudeChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (NomenclatureMgr::*)(int )>(_a, &NomenclatureMgr::terminatorMaxAltitudeChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (NomenclatureMgr::*)(bool )>(_a, &NomenclatureMgr::flagOutlineCratersChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (NomenclatureMgr::*)(bool )>(_a, &NomenclatureMgr::localNomenclatureHidingChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (NomenclatureMgr::*)(bool )>(_a, &NomenclatureMgr::specialNomenclatureOnlyDisplayingChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (NomenclatureMgr::*)(const Vec3f & ) const>(_a, &NomenclatureMgr::nomenclatureColorChanged, 7))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 7:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Vec3f >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->getFlagShowNomenclature(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->getFlagShowTerminatorZoneOnly(); break;
        case 2: *reinterpret_cast<int*>(_v) = _t->getTerminatorMinAltitude(); break;
        case 3: *reinterpret_cast<int*>(_v) = _t->getTerminatorMaxAltitude(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->getFlagOutlineCraters(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->getFlagHideLocalNomenclature(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->getFlagShowSpecialNomenclatureOnly(); break;
        case 7: *reinterpret_cast<Vec3f*>(_v) = _t->getColor(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setFlagShowNomenclature(*reinterpret_cast<bool*>(_v)); break;
        case 1: _t->setFlagShowTerminatorZoneOnly(*reinterpret_cast<bool*>(_v)); break;
        case 2: _t->setTerminatorMinAltitude(*reinterpret_cast<int*>(_v)); break;
        case 3: _t->setTerminatorMaxAltitude(*reinterpret_cast<int*>(_v)); break;
        case 4: _t->setFlagOutlineCraters(*reinterpret_cast<bool*>(_v)); break;
        case 5: _t->setFlagHideLocalNomenclature(*reinterpret_cast<bool*>(_v)); break;
        case 6: _t->setFlagShowSpecialNomenclatureOnly(*reinterpret_cast<bool*>(_v)); break;
        case 7: _t->setColor(*reinterpret_cast<Vec3f*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *NomenclatureMgr::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *NomenclatureMgr::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15NomenclatureMgrE_t>.strings))
        return static_cast<void*>(this);
    return StelObjectModule::qt_metacast(_clname);
}

int NomenclatureMgr::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StelObjectModule::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 28)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 28;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 28)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 28;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void NomenclatureMgr::flagShowNomenclatureChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void NomenclatureMgr::flagShowTerminatorZoneOnlyChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void NomenclatureMgr::terminatorMinAltitudeChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void NomenclatureMgr::terminatorMaxAltitudeChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void NomenclatureMgr::flagOutlineCratersChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void NomenclatureMgr::localNomenclatureHidingChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void NomenclatureMgr::specialNomenclatureOnlyDisplayingChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void NomenclatureMgr::nomenclatureColorChanged(const Vec3f & _t1)const
{
    QMetaObject::activate<void>(const_cast< NomenclatureMgr *>(this), &staticMetaObject, 7, nullptr, _t1);
}
QT_WARNING_POP
