/****************************************************************************
** Meta object code from reading C++ file 'SyncServerEventSenders.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../stellarium/plugins/RemoteSync/src/SyncServerEventSenders.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'SyncServerEventSenders.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN21SyncServerEventSenderE_t {};
} // unnamed namespace

template <> constexpr inline auto SyncServerEventSender::qt_create_metaobjectdata<qt_meta_tag_ZN21SyncServerEventSenderE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "SyncServerEventSender",
        "reactToStellariumEvent",
        "",
        "newClientConnected",
        "SyncRemotePeer&",
        "client"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'reactToStellariumEvent'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'newClientConnected'
        QtMocHelpers::SlotData<void(SyncRemotePeer &)>(3, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<SyncServerEventSender, qt_meta_tag_ZN21SyncServerEventSenderE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject SyncServerEventSender::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21SyncServerEventSenderE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21SyncServerEventSenderE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN21SyncServerEventSenderE_t>.metaTypes,
    nullptr
} };

void SyncServerEventSender::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SyncServerEventSender *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->reactToStellariumEvent(); break;
        case 1: _t->newClientConnected((*reinterpret_cast<std::add_pointer_t<SyncRemotePeer&>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *SyncServerEventSender::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SyncServerEventSender::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21SyncServerEventSenderE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int SyncServerEventSender::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN15TimeEventSenderE_t {};
} // unnamed namespace

template <> constexpr inline auto TimeEventSender::qt_create_metaobjectdata<qt_meta_tag_ZN15TimeEventSenderE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TimeEventSender"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TimeEventSender, qt_meta_tag_ZN15TimeEventSenderE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject TimeEventSender::staticMetaObject = { {
    QMetaObject::SuperData::link<TypedSyncServerEventSender<SyncProtocol::Time>::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15TimeEventSenderE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15TimeEventSenderE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15TimeEventSenderE_t>.metaTypes,
    nullptr
} };

void TimeEventSender::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TimeEventSender *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *TimeEventSender::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TimeEventSender::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15TimeEventSenderE_t>.strings))
        return static_cast<void*>(this);
    return TypedSyncServerEventSender<SyncProtocol::Time>::qt_metacast(_clname);
}

int TimeEventSender::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = TypedSyncServerEventSender<SyncProtocol::Time>::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN19LocationEventSenderE_t {};
} // unnamed namespace

template <> constexpr inline auto LocationEventSender::qt_create_metaobjectdata<qt_meta_tag_ZN19LocationEventSenderE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "LocationEventSender"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<LocationEventSender, qt_meta_tag_ZN19LocationEventSenderE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject LocationEventSender::staticMetaObject = { {
    QMetaObject::SuperData::link<TypedSyncServerEventSender<SyncProtocol::Location>::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19LocationEventSenderE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19LocationEventSenderE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN19LocationEventSenderE_t>.metaTypes,
    nullptr
} };

void LocationEventSender::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<LocationEventSender *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *LocationEventSender::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *LocationEventSender::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19LocationEventSenderE_t>.strings))
        return static_cast<void*>(this);
    return TypedSyncServerEventSender<SyncProtocol::Location>::qt_metacast(_clname);
}

int LocationEventSender::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = TypedSyncServerEventSender<SyncProtocol::Location>::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN20SelectionEventSenderE_t {};
} // unnamed namespace

template <> constexpr inline auto SelectionEventSender::qt_create_metaobjectdata<qt_meta_tag_ZN20SelectionEventSenderE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "SelectionEventSender"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<SelectionEventSender, qt_meta_tag_ZN20SelectionEventSenderE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject SelectionEventSender::staticMetaObject = { {
    QMetaObject::SuperData::link<TypedSyncServerEventSender<SyncProtocol::Selection>::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20SelectionEventSenderE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20SelectionEventSenderE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN20SelectionEventSenderE_t>.metaTypes,
    nullptr
} };

void SelectionEventSender::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SelectionEventSender *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *SelectionEventSender::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SelectionEventSender::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20SelectionEventSenderE_t>.strings))
        return static_cast<void*>(this);
    return TypedSyncServerEventSender<SyncProtocol::Selection>::qt_metacast(_clname);
}

int SelectionEventSender::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = TypedSyncServerEventSender<SyncProtocol::Selection>::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN23StelPropertyEventSenderE_t {};
} // unnamed namespace

template <> constexpr inline auto StelPropertyEventSender::qt_create_metaobjectdata<qt_meta_tag_ZN23StelPropertyEventSenderE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "StelPropertyEventSender",
        "newClientConnected",
        "",
        "SyncRemotePeer&",
        "client",
        "sendStelPropChange",
        "StelProperty*",
        "prop",
        "QVariant",
        "val"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'newClientConnected'
        QtMocHelpers::SlotData<void(SyncRemotePeer &)>(1, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'sendStelPropChange'
        QtMocHelpers::SlotData<void(StelProperty *, const QVariant &)>(5, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 6, 7 }, { 0x80000000 | 8, 9 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<StelPropertyEventSender, qt_meta_tag_ZN23StelPropertyEventSenderE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject StelPropertyEventSender::staticMetaObject = { {
    QMetaObject::SuperData::link<SyncServerEventSender::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23StelPropertyEventSenderE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23StelPropertyEventSenderE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN23StelPropertyEventSenderE_t>.metaTypes,
    nullptr
} };

void StelPropertyEventSender::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<StelPropertyEventSender *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->newClientConnected((*reinterpret_cast<std::add_pointer_t<SyncRemotePeer&>>(_a[1]))); break;
        case 1: _t->sendStelPropChange((*reinterpret_cast<std::add_pointer_t<StelProperty*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[2]))); break;
        default: ;
        }
    }
}

const QMetaObject *StelPropertyEventSender::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *StelPropertyEventSender::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23StelPropertyEventSenderE_t>.strings))
        return static_cast<void*>(this);
    return SyncServerEventSender::qt_metacast(_clname);
}

int StelPropertyEventSender::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = SyncServerEventSender::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN15ViewEventSenderE_t {};
} // unnamed namespace

template <> constexpr inline auto ViewEventSender::qt_create_metaobjectdata<qt_meta_tag_ZN15ViewEventSenderE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ViewEventSender"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ViewEventSender, qt_meta_tag_ZN15ViewEventSenderE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ViewEventSender::staticMetaObject = { {
    QMetaObject::SuperData::link<TypedSyncServerEventSender<SyncProtocol::View>::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15ViewEventSenderE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15ViewEventSenderE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15ViewEventSenderE_t>.metaTypes,
    nullptr
} };

void ViewEventSender::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ViewEventSender *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *ViewEventSender::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ViewEventSender::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15ViewEventSenderE_t>.strings))
        return static_cast<void*>(this);
    return TypedSyncServerEventSender<SyncProtocol::View>::qt_metacast(_clname);
}

int ViewEventSender::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = TypedSyncServerEventSender<SyncProtocol::View>::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
