/****************************************************************************
** Meta object code from reading C++ file 'SpecificTimeMgr.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../stellarium/src/core/modules/SpecificTimeMgr.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'SpecificTimeMgr.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15SpecificTimeMgrE_t {};
} // unnamed namespace

template <> constexpr inline auto SpecificTimeMgr::qt_create_metaobjectdata<qt_meta_tag_ZN15SpecificTimeMgrE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "SpecificTimeMgr",
        "twilightAltitudeChanged",
        "",
        "alt",
        "eventYearChanged",
        "setTwilightAltitude",
        "nextTransit",
        "previousTransit",
        "todayTransit",
        "nextRising",
        "previousRising",
        "todayRising",
        "nextSetting",
        "previousSetting",
        "todaySetting",
        "todayMorningTwilight",
        "todayEveningTwilight",
        "previousMorningTwilight",
        "previousEveningTwilight",
        "nextMorningTwilight",
        "nextEveningTwilight",
        "todayMorningAtAltitude",
        "nextMorningAtAltitude",
        "previousMorningAtAltitude",
        "todayEveningAtAltitude",
        "nextEveningAtAltitude",
        "previousEveningAtAltitude",
        "currentMarchEquinox",
        "nextMarchEquinox",
        "previousMarchEquinox",
        "currentSeptemberEquinox",
        "nextSeptemberEquinox",
        "previousSeptemberEquinox",
        "currentJuneSolstice",
        "nextJuneSolstice",
        "previousJuneSolstice",
        "currentDecemberSolstice",
        "nextDecemberSolstice",
        "previousDecemberSolstice",
        "twilightAltitude",
        "Equinox",
        "March",
        "September",
        "Solstice",
        "June",
        "December"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'twilightAltitudeChanged'
        QtMocHelpers::SignalData<void(double)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Signal 'eventYearChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setTwilightAltitude'
        QtMocHelpers::SlotData<void(double)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'nextTransit'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'previousTransit'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'todayTransit'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'nextRising'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'previousRising'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'todayRising'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'nextSetting'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'previousSetting'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'todaySetting'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'todayMorningTwilight'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'todayEveningTwilight'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'previousMorningTwilight'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'previousEveningTwilight'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'nextMorningTwilight'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'nextEveningTwilight'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'todayMorningAtAltitude'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'nextMorningAtAltitude'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'previousMorningAtAltitude'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'todayEveningAtAltitude'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'nextEveningAtAltitude'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'previousEveningAtAltitude'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'currentMarchEquinox'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'nextMarchEquinox'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'previousMarchEquinox'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'currentSeptemberEquinox'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'nextSeptemberEquinox'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'previousSeptemberEquinox'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'currentJuneSolstice'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'nextJuneSolstice'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'previousJuneSolstice'
        QtMocHelpers::SlotData<void()>(35, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'currentDecemberSolstice'
        QtMocHelpers::SlotData<void()>(36, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'nextDecemberSolstice'
        QtMocHelpers::SlotData<void()>(37, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'previousDecemberSolstice'
        QtMocHelpers::SlotData<void()>(38, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'twilightAltitude'
        QtMocHelpers::PropertyData<double>(39, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Equinox'
        QtMocHelpers::EnumData<enum Equinox>(40, 40, QMC::EnumFlags{}).add({
            {   41, Equinox::March },
            {   42, Equinox::September },
        }),
        // enum 'Solstice'
        QtMocHelpers::EnumData<enum Solstice>(43, 43, QMC::EnumFlags{}).add({
            {   44, Solstice::June },
            {   45, Solstice::December },
        }),
    };
    return QtMocHelpers::metaObjectData<SpecificTimeMgr, qt_meta_tag_ZN15SpecificTimeMgrE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject SpecificTimeMgr::staticMetaObject = { {
    QMetaObject::SuperData::link<StelModule::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15SpecificTimeMgrE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15SpecificTimeMgrE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15SpecificTimeMgrE_t>.metaTypes,
    nullptr
} };

void SpecificTimeMgr::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SpecificTimeMgr *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->twilightAltitudeChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 1: _t->eventYearChanged(); break;
        case 2: _t->setTwilightAltitude((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 3: _t->nextTransit(); break;
        case 4: _t->previousTransit(); break;
        case 5: _t->todayTransit(); break;
        case 6: _t->nextRising(); break;
        case 7: _t->previousRising(); break;
        case 8: _t->todayRising(); break;
        case 9: _t->nextSetting(); break;
        case 10: _t->previousSetting(); break;
        case 11: _t->todaySetting(); break;
        case 12: _t->todayMorningTwilight(); break;
        case 13: _t->todayEveningTwilight(); break;
        case 14: _t->previousMorningTwilight(); break;
        case 15: _t->previousEveningTwilight(); break;
        case 16: _t->nextMorningTwilight(); break;
        case 17: _t->nextEveningTwilight(); break;
        case 18: _t->todayMorningAtAltitude(); break;
        case 19: _t->nextMorningAtAltitude(); break;
        case 20: _t->previousMorningAtAltitude(); break;
        case 21: _t->todayEveningAtAltitude(); break;
        case 22: _t->nextEveningAtAltitude(); break;
        case 23: _t->previousEveningAtAltitude(); break;
        case 24: _t->currentMarchEquinox(); break;
        case 25: _t->nextMarchEquinox(); break;
        case 26: _t->previousMarchEquinox(); break;
        case 27: _t->currentSeptemberEquinox(); break;
        case 28: _t->nextSeptemberEquinox(); break;
        case 29: _t->previousSeptemberEquinox(); break;
        case 30: _t->currentJuneSolstice(); break;
        case 31: _t->nextJuneSolstice(); break;
        case 32: _t->previousJuneSolstice(); break;
        case 33: _t->currentDecemberSolstice(); break;
        case 34: _t->nextDecemberSolstice(); break;
        case 35: _t->previousDecemberSolstice(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (SpecificTimeMgr::*)(double )>(_a, &SpecificTimeMgr::twilightAltitudeChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (SpecificTimeMgr::*)()>(_a, &SpecificTimeMgr::eventYearChanged, 1))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<double*>(_v) = _t->getTwilightAltitude(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setTwilightAltitude(*reinterpret_cast<double*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *SpecificTimeMgr::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SpecificTimeMgr::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15SpecificTimeMgrE_t>.strings))
        return static_cast<void*>(this);
    return StelModule::qt_metacast(_clname);
}

int SpecificTimeMgr::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StelModule::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 36)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 36;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 36)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 36;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void SpecificTimeMgr::twilightAltitudeChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void SpecificTimeMgr::eventYearChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
