/****************************************************************************
** Meta object code from reading C++ file 'StarMgr.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../stellarium/src/core/modules/StarMgr.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'StarMgr.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN7StarMgrE_t {};
} // unnamed namespace

template <> constexpr inline auto StarMgr::qt_create_metaobjectdata<qt_meta_tag_ZN7StarMgrE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "StarMgr",
        "starLabelsDisplayedChanged",
        "",
        "displayed",
        "starsDisplayedChanged",
        "designationUsageChanged",
        "flag",
        "flagDblStarsDesignationChanged",
        "flagVarStarsDesignationChanged",
        "flagHIPDesignationChanged",
        "flagAdditionalNamesDisplayedChanged",
        "labelsAmountChanged",
        "a",
        "setFlagStars",
        "b",
        "getFlagStars",
        "setFlagLabels",
        "getFlagLabels",
        "setLabelsAmount",
        "getLabelsAmount",
        "setFontSize",
        "newFontSize",
        "setFlagSciNames",
        "f",
        "getFlagSciNames",
        "setDesignationUsage",
        "getDesignationUsage",
        "setFlagDblStarsDesignation",
        "getFlagDblStarsDesignation",
        "setFlagVarStarsDesignation",
        "getFlagVarStarsDesignation",
        "setFlagHIPDesignation",
        "getFlagHIPDesignation",
        "setFlagAdditionalNames",
        "getFlagAdditionalNames",
        "updateI18n",
        "updateSkyCulture",
        "StelSkyCulture",
        "skyCulture",
        "increaseStarsMagnitudeLimit",
        "reduceStarsMagnitudeLimit",
        "flagStarsDisplayed",
        "flagLabelsDisplayed",
        "labelsAmount",
        "flagAdditionalNamesDisplayed",
        "flagDesignationLabels",
        "flagDblStarsDesignation",
        "flagVarStarsDesignation",
        "flagHIPDesignation"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'starLabelsDisplayedChanged'
        QtMocHelpers::SignalData<void(const bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'starsDisplayedChanged'
        QtMocHelpers::SignalData<void(const bool)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'designationUsageChanged'
        QtMocHelpers::SignalData<void(const bool)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 6 },
        }}),
        // Signal 'flagDblStarsDesignationChanged'
        QtMocHelpers::SignalData<void(const bool)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 6 },
        }}),
        // Signal 'flagVarStarsDesignationChanged'
        QtMocHelpers::SignalData<void(const bool)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 6 },
        }}),
        // Signal 'flagHIPDesignationChanged'
        QtMocHelpers::SignalData<void(const bool)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 6 },
        }}),
        // Signal 'flagAdditionalNamesDisplayedChanged'
        QtMocHelpers::SignalData<void(const bool)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'labelsAmountChanged'
        QtMocHelpers::SignalData<void(double)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 12 },
        }}),
        // Slot 'setFlagStars'
        QtMocHelpers::SlotData<void(bool)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 14 },
        }}),
        // Slot 'getFlagStars'
        QtMocHelpers::SlotData<bool() const>(15, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFlagLabels'
        QtMocHelpers::SlotData<void(bool)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 14 },
        }}),
        // Slot 'getFlagLabels'
        QtMocHelpers::SlotData<bool() const>(17, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setLabelsAmount'
        QtMocHelpers::SlotData<void(double)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 12 },
        }}),
        // Slot 'getLabelsAmount'
        QtMocHelpers::SlotData<double() const>(19, 2, QMC::AccessPublic, QMetaType::Double),
        // Slot 'setFontSize'
        QtMocHelpers::SlotData<void(int)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 21 },
        }}),
        // Slot 'setFlagSciNames'
        QtMocHelpers::SlotData<void(bool)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 23 },
        }}),
        // Slot 'getFlagSciNames'
        QtMocHelpers::SlotData<bool()>(24, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setDesignationUsage'
        QtMocHelpers::SlotData<void(const bool)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 6 },
        }}),
        // Slot 'getDesignationUsage'
        QtMocHelpers::SlotData<bool()>(26, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFlagDblStarsDesignation'
        QtMocHelpers::SlotData<void(const bool)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 6 },
        }}),
        // Slot 'getFlagDblStarsDesignation'
        QtMocHelpers::SlotData<bool()>(28, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFlagVarStarsDesignation'
        QtMocHelpers::SlotData<void(const bool)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 6 },
        }}),
        // Slot 'getFlagVarStarsDesignation'
        QtMocHelpers::SlotData<bool()>(30, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFlagHIPDesignation'
        QtMocHelpers::SlotData<void(const bool)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 6 },
        }}),
        // Slot 'getFlagHIPDesignation'
        QtMocHelpers::SlotData<bool()>(32, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFlagAdditionalNames'
        QtMocHelpers::SlotData<void(bool)>(33, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 6 },
        }}),
        // Slot 'getFlagAdditionalNames'
        QtMocHelpers::SlotData<bool()>(34, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'updateI18n'
        QtMocHelpers::SlotData<void()>(35, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateSkyCulture'
        QtMocHelpers::SlotData<void(const StelSkyCulture &)>(36, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 37, 38 },
        }}),
        // Slot 'increaseStarsMagnitudeLimit'
        QtMocHelpers::SlotData<void()>(39, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'reduceStarsMagnitudeLimit'
        QtMocHelpers::SlotData<void()>(40, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'flagStarsDisplayed'
        QtMocHelpers::PropertyData<bool>(41, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 1),
        // property 'flagLabelsDisplayed'
        QtMocHelpers::PropertyData<bool>(42, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 0),
        // property 'labelsAmount'
        QtMocHelpers::PropertyData<double>(43, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 7),
        // property 'flagAdditionalNamesDisplayed'
        QtMocHelpers::PropertyData<bool>(44, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 6),
        // property 'flagDesignationLabels'
        QtMocHelpers::PropertyData<bool>(45, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 2),
        // property 'flagDblStarsDesignation'
        QtMocHelpers::PropertyData<bool>(46, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'flagVarStarsDesignation'
        QtMocHelpers::PropertyData<bool>(47, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'flagHIPDesignation'
        QtMocHelpers::PropertyData<bool>(48, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<StarMgr, qt_meta_tag_ZN7StarMgrE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject StarMgr::staticMetaObject = { {
    QMetaObject::SuperData::link<StelObjectModule::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7StarMgrE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7StarMgrE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN7StarMgrE_t>.metaTypes,
    nullptr
} };

void StarMgr::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<StarMgr *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->starLabelsDisplayedChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->starsDisplayedChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->designationUsageChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->flagDblStarsDesignationChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->flagVarStarsDesignationChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->flagHIPDesignationChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->flagAdditionalNamesDisplayedChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->labelsAmountChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 8: _t->setFlagStars((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 9: { bool _r = _t->getFlagStars();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 10: _t->setFlagLabels((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 11: { bool _r = _t->getFlagLabels();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 12: _t->setLabelsAmount((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 13: { double _r = _t->getLabelsAmount();
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 14: _t->setFontSize((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 15: _t->setFlagSciNames((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 16: { bool _r = _t->getFlagSciNames();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 17: _t->setDesignationUsage((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 18: { bool _r = _t->getDesignationUsage();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 19: _t->setFlagDblStarsDesignation((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 20: { bool _r = _t->getFlagDblStarsDesignation();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 21: _t->setFlagVarStarsDesignation((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 22: { bool _r = _t->getFlagVarStarsDesignation();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 23: _t->setFlagHIPDesignation((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 24: { bool _r = _t->getFlagHIPDesignation();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 25: _t->setFlagAdditionalNames((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 26: { bool _r = _t->getFlagAdditionalNames();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 27: _t->updateI18n(); break;
        case 28: _t->updateSkyCulture((*reinterpret_cast<std::add_pointer_t<StelSkyCulture>>(_a[1]))); break;
        case 29: _t->increaseStarsMagnitudeLimit(); break;
        case 30: _t->reduceStarsMagnitudeLimit(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (StarMgr::*)(const bool )>(_a, &StarMgr::starLabelsDisplayedChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (StarMgr::*)(const bool )>(_a, &StarMgr::starsDisplayedChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (StarMgr::*)(const bool )>(_a, &StarMgr::designationUsageChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (StarMgr::*)(const bool )>(_a, &StarMgr::flagDblStarsDesignationChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (StarMgr::*)(const bool )>(_a, &StarMgr::flagVarStarsDesignationChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (StarMgr::*)(const bool )>(_a, &StarMgr::flagHIPDesignationChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (StarMgr::*)(const bool )>(_a, &StarMgr::flagAdditionalNamesDisplayedChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (StarMgr::*)(double )>(_a, &StarMgr::labelsAmountChanged, 7))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->getFlagStars(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->getFlagLabels(); break;
        case 2: *reinterpret_cast<double*>(_v) = _t->getLabelsAmount(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->getFlagAdditionalNames(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->getDesignationUsage(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->getFlagDblStarsDesignation(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->getFlagVarStarsDesignation(); break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->getFlagHIPDesignation(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setFlagStars(*reinterpret_cast<bool*>(_v)); break;
        case 1: _t->setFlagLabels(*reinterpret_cast<bool*>(_v)); break;
        case 2: _t->setLabelsAmount(*reinterpret_cast<double*>(_v)); break;
        case 3: _t->setFlagAdditionalNames(*reinterpret_cast<bool*>(_v)); break;
        case 4: _t->setDesignationUsage(*reinterpret_cast<bool*>(_v)); break;
        case 5: _t->setFlagDblStarsDesignation(*reinterpret_cast<bool*>(_v)); break;
        case 6: _t->setFlagVarStarsDesignation(*reinterpret_cast<bool*>(_v)); break;
        case 7: _t->setFlagHIPDesignation(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *StarMgr::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *StarMgr::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7StarMgrE_t>.strings))
        return static_cast<void*>(this);
    return StelObjectModule::qt_metacast(_clname);
}

int StarMgr::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StelObjectModule::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 31)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 31;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 31)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 31;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void StarMgr::starLabelsDisplayedChanged(const bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void StarMgr::starsDisplayedChanged(const bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void StarMgr::designationUsageChanged(const bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void StarMgr::flagDblStarsDesignationChanged(const bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void StarMgr::flagVarStarsDesignationChanged(const bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void StarMgr::flagHIPDesignationChanged(const bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void StarMgr::flagAdditionalNamesDisplayedChanged(const bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void StarMgr::labelsAmountChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}
QT_WARNING_POP
