/********************************************************************************
** Form generated from reading UI file 'astroCalcCustomStepsDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_ASTROCALCCUSTOMSTEPSDIALOG_H
#define UI_ASTROCALCCUSTOMSTEPSDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "Dialog.hpp"
#include "gui/StelCloseButton.hpp"

QT_BEGIN_NAMESPACE

class Ui_astroCalcCustomStepsDialogForm
{
public:
    QVBoxLayout *verticalLayout;
    TitleBar *titleBar;
    QGroupBox *customStepsGroupBox;
    QVBoxLayout *verticalLayout_3;
    QHBoxLayout *horizontalLayout;
    QLabel *timeStepLabel;
    QDoubleSpinBox *timeStepDoubleSpinBox;
    QHBoxLayout *horizontalLayout_2;
    QLabel *unitMeasurementLabel;
    QComboBox *unitMeasurementComboBox;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *astroCalcCustomStepsDialogForm)
    {
        if (astroCalcCustomStepsDialogForm->objectName().isEmpty())
            astroCalcCustomStepsDialogForm->setObjectName("astroCalcCustomStepsDialogForm");
        astroCalcCustomStepsDialogForm->resize(335, 155);
        verticalLayout = new QVBoxLayout(astroCalcCustomStepsDialogForm);
        verticalLayout->setSpacing(0);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        titleBar = new TitleBar(astroCalcCustomStepsDialogForm);
        titleBar->setObjectName("titleBar");

        verticalLayout->addWidget(titleBar);

        customStepsGroupBox = new QGroupBox(astroCalcCustomStepsDialogForm);
        customStepsGroupBox->setObjectName("customStepsGroupBox");
        customStepsGroupBox->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);
        verticalLayout_3 = new QVBoxLayout(customStepsGroupBox);
        verticalLayout_3->setObjectName("verticalLayout_3");
        verticalLayout_3->setContentsMargins(0, 0, 0, 0);
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        timeStepLabel = new QLabel(customStepsGroupBox);
        timeStepLabel->setObjectName("timeStepLabel");

        horizontalLayout->addWidget(timeStepLabel);

        timeStepDoubleSpinBox = new QDoubleSpinBox(customStepsGroupBox);
        timeStepDoubleSpinBox->setObjectName("timeStepDoubleSpinBox");
        timeStepDoubleSpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        timeStepDoubleSpinBox->setDecimals(6);
        timeStepDoubleSpinBox->setMinimum(0.000001000000000);
        timeStepDoubleSpinBox->setMaximum(999999.000000000000000);
        timeStepDoubleSpinBox->setValue(3.000000000000000);

        horizontalLayout->addWidget(timeStepDoubleSpinBox);


        verticalLayout_3->addLayout(horizontalLayout);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        unitMeasurementLabel = new QLabel(customStepsGroupBox);
        unitMeasurementLabel->setObjectName("unitMeasurementLabel");

        horizontalLayout_2->addWidget(unitMeasurementLabel);

        unitMeasurementComboBox = new QComboBox(customStepsGroupBox);
        unitMeasurementComboBox->setObjectName("unitMeasurementComboBox");
        unitMeasurementComboBox->setEditable(true);
        unitMeasurementComboBox->setInsertPolicy(QComboBox::NoInsert);

        horizontalLayout_2->addWidget(unitMeasurementComboBox);


        verticalLayout_3->addLayout(horizontalLayout_2);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_3->addItem(verticalSpacer);


        verticalLayout->addWidget(customStepsGroupBox);


        retranslateUi(astroCalcCustomStepsDialogForm);

        QMetaObject::connectSlotsByName(astroCalcCustomStepsDialogForm);
    } // setupUi

    void retranslateUi(QWidget *astroCalcCustomStepsDialogForm)
    {
        titleBar->setTitle(QCoreApplication::translate("astroCalcCustomStepsDialogForm", "Custom interval", nullptr));
        customStepsGroupBox->setTitle(QCoreApplication::translate("astroCalcCustomStepsDialogForm", "Custom time step for ephemeris visualization", nullptr));
        timeStepLabel->setText(QCoreApplication::translate("astroCalcCustomStepsDialogForm", "Time step:", nullptr));
        unitMeasurementLabel->setText(QCoreApplication::translate("astroCalcCustomStepsDialogForm", "Unit of measurement:", nullptr));
        (void)astroCalcCustomStepsDialogForm;
    } // retranslateUi

};

namespace Ui {
    class astroCalcCustomStepsDialogForm: public Ui_astroCalcCustomStepsDialogForm {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_ASTROCALCCUSTOMSTEPSDIALOG_H
