/****************************************************************************
** Meta object code from reading C++ file 'ZodiacalLight.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../stellarium/src/core/modules/ZodiacalLight.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ZodiacalLight.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13ZodiacalLightE_t {};
} // unnamed namespace

template <> constexpr inline auto ZodiacalLight::qt_create_metaobjectdata<qt_meta_tag_ZN13ZodiacalLightE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ZodiacalLight",
        "zodiacalLightDisplayedChanged",
        "",
        "displayed",
        "intensityChanged",
        "intensity",
        "colorChanged",
        "Vec3f",
        "color",
        "getIntensity",
        "setIntensity",
        "aintensity",
        "getColor",
        "setColor",
        "c",
        "setFlagShow",
        "b",
        "getFlagShow",
        "handleLocationChanged",
        "StelLocation",
        "loc",
        "flagZodiacalLightDisplayed"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'zodiacalLightDisplayedChanged'
        QtMocHelpers::SignalData<void(const bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'intensityChanged'
        QtMocHelpers::SignalData<void(double)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 5 },
        }}),
        // Signal 'colorChanged'
        QtMocHelpers::SignalData<void(Vec3f)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Slot 'getIntensity'
        QtMocHelpers::SlotData<double() const>(9, 2, QMC::AccessPublic, QMetaType::Double),
        // Slot 'setIntensity'
        QtMocHelpers::SlotData<void(double)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 11 },
        }}),
        // Slot 'getColor'
        QtMocHelpers::SlotData<Vec3f() const>(12, 2, QMC::AccessPublic, 0x80000000 | 7),
        // Slot 'setColor'
        QtMocHelpers::SlotData<void(const Vec3f &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 14 },
        }}),
        // Slot 'setFlagShow'
        QtMocHelpers::SlotData<void(bool)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 16 },
        }}),
        // Slot 'getFlagShow'
        QtMocHelpers::SlotData<bool() const>(17, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'handleLocationChanged'
        QtMocHelpers::SlotData<void(const StelLocation &)>(18, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 19, 20 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'flagZodiacalLightDisplayed'
        QtMocHelpers::PropertyData<bool>(21, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 0),
        // property 'intensity'
        QtMocHelpers::PropertyData<double>(5, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'color'
        QtMocHelpers::PropertyData<Vec3f>(8, 0x80000000 | 7, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 2),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ZodiacalLight, qt_meta_tag_ZN13ZodiacalLightE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ZodiacalLight::staticMetaObject = { {
    QMetaObject::SuperData::link<StelModule::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13ZodiacalLightE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13ZodiacalLightE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13ZodiacalLightE_t>.metaTypes,
    nullptr
} };

void ZodiacalLight::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ZodiacalLight *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->zodiacalLightDisplayedChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->intensityChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 2: _t->colorChanged((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1]))); break;
        case 3: { double _r = _t->getIntensity();
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 4: _t->setIntensity((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 5: { Vec3f _r = _t->getColor();
            if (_a[0]) *reinterpret_cast<Vec3f*>(_a[0]) = std::move(_r); }  break;
        case 6: _t->setColor((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1]))); break;
        case 7: _t->setFlagShow((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: { bool _r = _t->getFlagShow();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 9: _t->handleLocationChanged((*reinterpret_cast<std::add_pointer_t<StelLocation>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< StelLocation >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ZodiacalLight::*)(const bool )>(_a, &ZodiacalLight::zodiacalLightDisplayedChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ZodiacalLight::*)(double )>(_a, &ZodiacalLight::intensityChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ZodiacalLight::*)(Vec3f )>(_a, &ZodiacalLight::colorChanged, 2))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 2:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Vec3f >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->getFlagShow(); break;
        case 1: *reinterpret_cast<double*>(_v) = _t->getIntensity(); break;
        case 2: *reinterpret_cast<Vec3f*>(_v) = _t->getColor(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setFlagShow(*reinterpret_cast<bool*>(_v)); break;
        case 1: _t->setIntensity(*reinterpret_cast<double*>(_v)); break;
        case 2: _t->setColor(*reinterpret_cast<Vec3f*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *ZodiacalLight::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ZodiacalLight::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13ZodiacalLightE_t>.strings))
        return static_cast<void*>(this);
    return StelModule::qt_metacast(_clname);
}

int ZodiacalLight::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StelModule::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void ZodiacalLight::zodiacalLightDisplayedChanged(const bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void ZodiacalLight::intensityChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void ZodiacalLight::colorChanged(Vec3f _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}
QT_WARNING_POP
