/********************************************************************************
** Form generated from reading UI file 'atmosphereDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_ATMOSPHEREDIALOG_H
#define UI_ATMOSPHEREDIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "Dialog.hpp"

QT_BEGIN_NAMESPACE

class Ui_atmosphereDialogForm
{
public:
    QVBoxLayout *verticalLayout;
    TitleBar *titleBar;
    QGroupBox *visualModelConfigGroup;
    QVBoxLayout *verticalLayout_5;
    QHBoxLayout *horizontalLayout;
    QLabel *label_2;
    QComboBox *atmosphereModel;
    QSpacerItem *horizontalSpacer;
    QFrame *showMySky_optionsGroup;
    QVBoxLayout *verticalLayout_2;
    QLabel *showMySky_statusLabel;
    QHBoxLayout *showMySky_pathToDataLayout;
    QLabel *showMySky_pathToModelLabel;
    QLineEdit *showMySky_pathToModelEdit;
    QPushButton *showMySky_pathToModelBrowseBtn;
    QHBoxLayout *showMySky_eclipseSimQualityLayout;
    QLabel *label;
    QSpinBox *showMySky_eclipseSimulationQualitySpinBox;
    QSpacerItem *horizontalSpacer_2;
    QCheckBox *showMySky_debugOptionsEnabled;
    QFrame *showMySky_debugOptionsGroup;
    QVBoxLayout *verticalLayout_4;
    QCheckBox *showMySky_zeroOrderEnabled;
    QCheckBox *showMySky_singleScatteringEnabled;
    QCheckBox *showMySky_multipleScatteringEnabled;
    QCheckBox *showMySky_pseudoMirrorEnabled;
    QGroupBox *atmosphereGroupBox;
    QVBoxLayout *verticalLayout_3;
    QFormLayout *formLayout_atm;
    QLabel *pressureLabel;
    QDoubleSpinBox *pressureDoubleSpinBox;
    QLabel *temperatureLabel;
    QDoubleSpinBox *temperatureDoubleSpinBox;
    QLabel *extinctionLabel;
    QDoubleSpinBox *extinctionDoubleSpinBox;
    QPushButton *standardAtmosphereButton;
    QGroupBox *groupBox_experimental;
    QVBoxLayout *verticalLayout_experimental;
    QCheckBox *checkBox_noScatter;
    QGridLayout *gridLayout_2;
    QCheckBox *checkBox_TfromK;
    QDoubleSpinBox *doubleSpinBox_T;
    QLabel *label_22;

    void setupUi(QWidget *atmosphereDialogForm)
    {
        if (atmosphereDialogForm->objectName().isEmpty())
            atmosphereDialogForm->setObjectName("atmosphereDialogForm");
        atmosphereDialogForm->resize(317, 549);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Minimum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(atmosphereDialogForm->sizePolicy().hasHeightForWidth());
        atmosphereDialogForm->setSizePolicy(sizePolicy);
        verticalLayout = new QVBoxLayout(atmosphereDialogForm);
        verticalLayout->setSpacing(0);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        titleBar = new TitleBar(atmosphereDialogForm);
        titleBar->setObjectName("titleBar");

        verticalLayout->addWidget(titleBar);

        visualModelConfigGroup = new QGroupBox(atmosphereDialogForm);
        visualModelConfigGroup->setObjectName("visualModelConfigGroup");
        sizePolicy.setHeightForWidth(visualModelConfigGroup->sizePolicy().hasHeightForWidth());
        visualModelConfigGroup->setSizePolicy(sizePolicy);
        verticalLayout_5 = new QVBoxLayout(visualModelConfigGroup);
        verticalLayout_5->setObjectName("verticalLayout_5");
        verticalLayout_5->setContentsMargins(0, 0, 0, 0);
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        label_2 = new QLabel(visualModelConfigGroup);
        label_2->setObjectName("label_2");

        horizontalLayout->addWidget(label_2);

        atmosphereModel = new QComboBox(visualModelConfigGroup);
        atmosphereModel->addItem(QString::fromUtf8("Preetham"));
        atmosphereModel->addItem(QString::fromUtf8("ShowMySky"));
        atmosphereModel->setObjectName("atmosphereModel");
        atmosphereModel->setCurrentText(QString::fromUtf8("Preetham"));

        horizontalLayout->addWidget(atmosphereModel);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);


        verticalLayout_5->addLayout(horizontalLayout);

        showMySky_optionsGroup = new QFrame(visualModelConfigGroup);
        showMySky_optionsGroup->setObjectName("showMySky_optionsGroup");
        verticalLayout_2 = new QVBoxLayout(showMySky_optionsGroup);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        showMySky_statusLabel = new QLabel(showMySky_optionsGroup);
        showMySky_statusLabel->setObjectName("showMySky_statusLabel");
        showMySky_statusLabel->setWordWrap(true);
        showMySky_statusLabel->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextSelectableByKeyboard|Qt::TextSelectableByMouse);

        verticalLayout_2->addWidget(showMySky_statusLabel);

        showMySky_pathToDataLayout = new QHBoxLayout();
        showMySky_pathToDataLayout->setObjectName("showMySky_pathToDataLayout");
        showMySky_pathToModelLabel = new QLabel(showMySky_optionsGroup);
        showMySky_pathToModelLabel->setObjectName("showMySky_pathToModelLabel");

        showMySky_pathToDataLayout->addWidget(showMySky_pathToModelLabel);

        showMySky_pathToModelEdit = new QLineEdit(showMySky_optionsGroup);
        showMySky_pathToModelEdit->setObjectName("showMySky_pathToModelEdit");

        showMySky_pathToDataLayout->addWidget(showMySky_pathToModelEdit);

        showMySky_pathToModelBrowseBtn = new QPushButton(showMySky_optionsGroup);
        showMySky_pathToModelBrowseBtn->setObjectName("showMySky_pathToModelBrowseBtn");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(showMySky_pathToModelBrowseBtn->sizePolicy().hasHeightForWidth());
        showMySky_pathToModelBrowseBtn->setSizePolicy(sizePolicy1);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/graphicGui/uibtFolder.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        icon.addFile(QString::fromUtf8(":/graphicGui/uibtFolder-disabled.png"), QSize(), QIcon::Mode::Disabled, QIcon::State::Off);
        showMySky_pathToModelBrowseBtn->setIcon(icon);

        showMySky_pathToDataLayout->addWidget(showMySky_pathToModelBrowseBtn);


        verticalLayout_2->addLayout(showMySky_pathToDataLayout);

        showMySky_eclipseSimQualityLayout = new QHBoxLayout();
        showMySky_eclipseSimQualityLayout->setObjectName("showMySky_eclipseSimQualityLayout");
        label = new QLabel(showMySky_optionsGroup);
        label->setObjectName("label");

        showMySky_eclipseSimQualityLayout->addWidget(label);

        showMySky_eclipseSimulationQualitySpinBox = new QSpinBox(showMySky_optionsGroup);
        showMySky_eclipseSimulationQualitySpinBox->setObjectName("showMySky_eclipseSimulationQualitySpinBox");
        showMySky_eclipseSimulationQualitySpinBox->setMaximum(3);

        showMySky_eclipseSimQualityLayout->addWidget(showMySky_eclipseSimulationQualitySpinBox);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        showMySky_eclipseSimQualityLayout->addItem(horizontalSpacer_2);


        verticalLayout_2->addLayout(showMySky_eclipseSimQualityLayout);

        showMySky_debugOptionsEnabled = new QCheckBox(showMySky_optionsGroup);
        showMySky_debugOptionsEnabled->setObjectName("showMySky_debugOptionsEnabled");
        showMySky_debugOptionsEnabled->setChecked(true);

        verticalLayout_2->addWidget(showMySky_debugOptionsEnabled);

        showMySky_debugOptionsGroup = new QFrame(showMySky_optionsGroup);
        showMySky_debugOptionsGroup->setObjectName("showMySky_debugOptionsGroup");
        verticalLayout_4 = new QVBoxLayout(showMySky_debugOptionsGroup);
        verticalLayout_4->setObjectName("verticalLayout_4");
        showMySky_zeroOrderEnabled = new QCheckBox(showMySky_debugOptionsGroup);
        showMySky_zeroOrderEnabled->setObjectName("showMySky_zeroOrderEnabled");

        verticalLayout_4->addWidget(showMySky_zeroOrderEnabled);

        showMySky_singleScatteringEnabled = new QCheckBox(showMySky_debugOptionsGroup);
        showMySky_singleScatteringEnabled->setObjectName("showMySky_singleScatteringEnabled");

        verticalLayout_4->addWidget(showMySky_singleScatteringEnabled);

        showMySky_multipleScatteringEnabled = new QCheckBox(showMySky_debugOptionsGroup);
        showMySky_multipleScatteringEnabled->setObjectName("showMySky_multipleScatteringEnabled");

        verticalLayout_4->addWidget(showMySky_multipleScatteringEnabled);

        showMySky_pseudoMirrorEnabled = new QCheckBox(showMySky_debugOptionsGroup);
        showMySky_pseudoMirrorEnabled->setObjectName("showMySky_pseudoMirrorEnabled");

        verticalLayout_4->addWidget(showMySky_pseudoMirrorEnabled);


        verticalLayout_2->addWidget(showMySky_debugOptionsGroup);


        verticalLayout_5->addWidget(showMySky_optionsGroup);


        verticalLayout->addWidget(visualModelConfigGroup);

        atmosphereGroupBox = new QGroupBox(atmosphereDialogForm);
        atmosphereGroupBox->setObjectName("atmosphereGroupBox");
        verticalLayout_3 = new QVBoxLayout(atmosphereGroupBox);
        verticalLayout_3->setObjectName("verticalLayout_3");
        verticalLayout_3->setContentsMargins(0, 0, 0, 0);
        formLayout_atm = new QFormLayout();
        formLayout_atm->setObjectName("formLayout_atm");
        pressureLabel = new QLabel(atmosphereGroupBox);
        pressureLabel->setObjectName("pressureLabel");

        formLayout_atm->setWidget(0, QFormLayout::ItemRole::LabelRole, pressureLabel);

        pressureDoubleSpinBox = new QDoubleSpinBox(atmosphereGroupBox);
        pressureDoubleSpinBox->setObjectName("pressureDoubleSpinBox");
        pressureDoubleSpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        pressureDoubleSpinBox->setMaximum(1500.000000000000000);
        pressureDoubleSpinBox->setValue(1013.250000000000000);

        formLayout_atm->setWidget(0, QFormLayout::ItemRole::FieldRole, pressureDoubleSpinBox);

        temperatureLabel = new QLabel(atmosphereGroupBox);
        temperatureLabel->setObjectName("temperatureLabel");
        temperatureLabel->setInputMethodHints(Qt::ImhNone);
        temperatureLabel->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);

        formLayout_atm->setWidget(1, QFormLayout::ItemRole::LabelRole, temperatureLabel);

        temperatureDoubleSpinBox = new QDoubleSpinBox(atmosphereGroupBox);
        temperatureDoubleSpinBox->setObjectName("temperatureDoubleSpinBox");
        temperatureDoubleSpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        temperatureDoubleSpinBox->setDecimals(1);
        temperatureDoubleSpinBox->setMinimum(-100.000000000000000);
        temperatureDoubleSpinBox->setValue(10.000000000000000);

        formLayout_atm->setWidget(1, QFormLayout::ItemRole::FieldRole, temperatureDoubleSpinBox);

        extinctionLabel = new QLabel(atmosphereGroupBox);
        extinctionLabel->setObjectName("extinctionLabel");

        formLayout_atm->setWidget(2, QFormLayout::ItemRole::LabelRole, extinctionLabel);

        extinctionDoubleSpinBox = new QDoubleSpinBox(atmosphereGroupBox);
        extinctionDoubleSpinBox->setObjectName("extinctionDoubleSpinBox");
        extinctionDoubleSpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        extinctionDoubleSpinBox->setMaximum(3.000000000000000);
        extinctionDoubleSpinBox->setSingleStep(0.010000000000000);
        extinctionDoubleSpinBox->setValue(0.220000000000000);

        formLayout_atm->setWidget(2, QFormLayout::ItemRole::FieldRole, extinctionDoubleSpinBox);


        verticalLayout_3->addLayout(formLayout_atm);

        standardAtmosphereButton = new QPushButton(atmosphereGroupBox);
        standardAtmosphereButton->setObjectName("standardAtmosphereButton");

        verticalLayout_3->addWidget(standardAtmosphereButton);


        verticalLayout->addWidget(atmosphereGroupBox);

        groupBox_experimental = new QGroupBox(atmosphereDialogForm);
        groupBox_experimental->setObjectName("groupBox_experimental");
        verticalLayout_experimental = new QVBoxLayout(groupBox_experimental);
        verticalLayout_experimental->setObjectName("verticalLayout_experimental");
        verticalLayout_experimental->setContentsMargins(0, 0, 0, 0);
        checkBox_noScatter = new QCheckBox(groupBox_experimental);
        checkBox_noScatter->setObjectName("checkBox_noScatter");

        verticalLayout_experimental->addWidget(checkBox_noScatter);

        gridLayout_2 = new QGridLayout();
        gridLayout_2->setObjectName("gridLayout_2");
        checkBox_TfromK = new QCheckBox(groupBox_experimental);
        checkBox_TfromK->setObjectName("checkBox_TfromK");
#if QT_CONFIG(tooltip)
        checkBox_TfromK->setToolTip(QString::fromUtf8(""));
#endif // QT_CONFIG(tooltip)

        gridLayout_2->addWidget(checkBox_TfromK, 1, 0, 1, 1);

        doubleSpinBox_T = new QDoubleSpinBox(groupBox_experimental);
        doubleSpinBox_T->setObjectName("doubleSpinBox_T");
        doubleSpinBox_T->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        doubleSpinBox_T->setMinimum(1.000000000000000);
        doubleSpinBox_T->setSingleStep(0.100000000000000);

        gridLayout_2->addWidget(doubleSpinBox_T, 1, 1, 1, 1);


        verticalLayout_experimental->addLayout(gridLayout_2);

        label_22 = new QLabel(groupBox_experimental);
        label_22->setObjectName("label_22");

        verticalLayout_experimental->addWidget(label_22);


        verticalLayout->addWidget(groupBox_experimental);

#if QT_CONFIG(shortcut)
        label_2->setBuddy(atmosphereModel);
        showMySky_pathToModelLabel->setBuddy(showMySky_pathToModelEdit);
        label->setBuddy(showMySky_eclipseSimulationQualitySpinBox);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(atmosphereModel, showMySky_pathToModelEdit);
        QWidget::setTabOrder(showMySky_pathToModelEdit, showMySky_pathToModelBrowseBtn);
        QWidget::setTabOrder(showMySky_pathToModelBrowseBtn, showMySky_debugOptionsEnabled);
        QWidget::setTabOrder(showMySky_debugOptionsEnabled, showMySky_zeroOrderEnabled);
        QWidget::setTabOrder(showMySky_zeroOrderEnabled, showMySky_singleScatteringEnabled);
        QWidget::setTabOrder(showMySky_singleScatteringEnabled, showMySky_multipleScatteringEnabled);
        QWidget::setTabOrder(showMySky_multipleScatteringEnabled, pressureDoubleSpinBox);
        QWidget::setTabOrder(pressureDoubleSpinBox, temperatureDoubleSpinBox);
        QWidget::setTabOrder(temperatureDoubleSpinBox, extinctionDoubleSpinBox);
        QWidget::setTabOrder(extinctionDoubleSpinBox, standardAtmosphereButton);
        QWidget::setTabOrder(standardAtmosphereButton, checkBox_noScatter);
        QWidget::setTabOrder(checkBox_noScatter, checkBox_TfromK);
        QWidget::setTabOrder(checkBox_TfromK, doubleSpinBox_T);

        retranslateUi(atmosphereDialogForm);

        QMetaObject::connectSlotsByName(atmosphereDialogForm);
    } // setupUi

    void retranslateUi(QWidget *atmosphereDialogForm)
    {
        titleBar->setTitle(QCoreApplication::translate("atmosphereDialogForm", "Atmosphere Details", nullptr));
        visualModelConfigGroup->setTitle(QCoreApplication::translate("atmosphereDialogForm", "Visual Model", nullptr));
        label_2->setText(QCoreApplication::translate("atmosphereDialogForm", "Choose &model:", nullptr));

        showMySky_statusLabel->setText(QString());
        showMySky_pathToModelLabel->setText(QCoreApplication::translate("atmosphereDialogForm", "Path to &data:", nullptr));
        showMySky_pathToModelBrowseBtn->setText(QString());
        label->setText(QCoreApplication::translate("atmosphereDialogForm", "Eclipse simulation &quality:", nullptr));
        showMySky_debugOptionsEnabled->setText(QCoreApplication::translate("atmosphereDialogForm", "Show debugging options", nullptr));
        showMySky_zeroOrderEnabled->setText(QCoreApplication::translate("atmosphereDialogForm", "Render zero-order scattering", nullptr));
        showMySky_singleScatteringEnabled->setText(QCoreApplication::translate("atmosphereDialogForm", "Render single scattering", nullptr));
        showMySky_multipleScatteringEnabled->setText(QCoreApplication::translate("atmosphereDialogForm", "Render multiple scattering", nullptr));
        showMySky_pseudoMirrorEnabled->setText(QCoreApplication::translate("atmosphereDialogForm", "Mirror the sky under the horizon", nullptr));
        atmosphereGroupBox->setTitle(QCoreApplication::translate("atmosphereDialogForm", "Refraction Settings", nullptr));
        pressureLabel->setText(QCoreApplication::translate("atmosphereDialogForm", "Pressure (mbar):", nullptr));
        temperatureLabel->setText(QCoreApplication::translate("atmosphereDialogForm", "Temperature (C):", nullptr));
#if QT_CONFIG(tooltip)
        extinctionLabel->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        extinctionLabel->setWhatsThis(QCoreApplication::translate("atmosphereDialogForm", "Extinction is the loss of star brightness due to Earth's atmosphere. It is given in mag/airmass, where airmass is number of atmospheres light has to pass. (zenith: 1; horizon: about 40)", nullptr));
#endif // QT_CONFIG(whatsthis)
        extinctionLabel->setText(QCoreApplication::translate("atmosphereDialogForm", "Extinction Coefficient (k):", nullptr));
#if QT_CONFIG(tooltip)
        extinctionDoubleSpinBox->setToolTip(QCoreApplication::translate("atmosphereDialogForm", "Use about 0.12 for superb mountaintops, 0.2 for good rural landscape, 0.35 for murky conditions.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        standardAtmosphereButton->setToolTip(QCoreApplication::translate("atmosphereDialogForm", "Set the International Standard Atmosphere at the height 0 km MSL", nullptr));
#endif // QT_CONFIG(tooltip)
        standardAtmosphereButton->setText(QCoreApplication::translate("atmosphereDialogForm", "Standard Atmosphere", nullptr));
        groupBox_experimental->setTitle(QCoreApplication::translate("atmosphereDialogForm", "Experimental", nullptr));
        checkBox_noScatter->setText(QCoreApplication::translate("atmosphereDialogForm", "Suppress scattering (only refraction/extinction)", nullptr));
        checkBox_TfromK->setText(QCoreApplication::translate("atmosphereDialogForm", "T=25(k-0.16)+1 -- or T=", nullptr));
        label_22->setText(QCoreApplication::translate("atmosphereDialogForm", "T=2/5(default)/6 at k=0.16/0.32/0.36. ", nullptr));
        (void)atmosphereDialogForm;
    } // retranslateUi

};

namespace Ui {
    class atmosphereDialogForm: public Ui_atmosphereDialogForm {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_ATMOSPHEREDIALOG_H
