/********************************************************************************
** Form generated from reading UI file 'searchDialogGui.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SEARCHDIALOGGUI_H
#define UI_SEARCHDIALOGGUI_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListView>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTextBrowser>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "AngleSpinBox.hpp"
#include "Dialog.hpp"
#include "gui/StelCloseButton.hpp"

QT_BEGIN_NAMESPACE

class Ui_searchDialogForm
{
public:
    QVBoxLayout *verticalLayout;
    TitleBar *titleBar;
    QTabWidget *tabWidget;
    QWidget *objectTab;
    QVBoxLayout *verticalLayout_2;
    QFrame *horizontalSearchLayout;
    QHBoxLayout *horizontalLayout;
    QLineEdit *lineEditSearchSkyObject;
    QToolButton *pushButtonGotoSearchSkyObject;
    QListView *searchListView;
    QLabel *simbadStatusLabel;
    QFrame *greekLettersFrame;
    QGridLayout *gridLayout;
    QPushButton *iotaPushButton;
    QPushButton *alphaPushButton;
    QPushButton *betaPushButton;
    QPushButton *gammaPushButton;
    QPushButton *deltaPushButton;
    QPushButton *epsilonPushButton;
    QPushButton *zetaPushButton;
    QPushButton *etaPushButton;
    QPushButton *thetaPushButton;
    QPushButton *kappaPushButton;
    QPushButton *lambdaPushButton;
    QPushButton *muPushButton;
    QPushButton *nuPushButton;
    QPushButton *xiPushButton;
    QPushButton *omicronPushButton;
    QPushButton *piPushButton;
    QPushButton *rhoPushButton;
    QPushButton *sigmaPushButton;
    QPushButton *tauPushButton;
    QPushButton *upsilonPushButton;
    QPushButton *phiPushButton;
    QPushButton *chiPushButton;
    QPushButton *psiPushButton;
    QPushButton *omegaPushButton;
    QLabel *labelGreekLetterTitle;
    QWidget *simbadTab;
    QGridLayout *gridLayout_4;
    QPushButton *simbadCooQueryButton;
    QTextBrowser *simbadCooResultsTextBrowser;
    QHBoxLayout *horizontalLayout_3;
    QCheckBox *allIDsCheckBox;
    QCheckBox *typesCheckBox;
    QCheckBox *spectralClassCheckBox;
    QCheckBox *morphoCheckBox;
    QCheckBox *dimsCheckBox;
    QHBoxLayout *horizontalLayout_4;
    QLabel *searchRadiusLabel;
    QSpinBox *searchRadiusSpinBox;
    QSpacerItem *horizontalSpacer;
    QLabel *maxResultsLabel;
    QSpinBox *resultsSpinBox;
    QLabel *simbadCooStatusLabel;
    QWidget *tabPosition;
    QVBoxLayout *verticalLayout_3;
    QFrame *j2000Frame;
    QGridLayout *gridLayout_2;
    QSpacerItem *verticalSpacer_2;
    QHBoxLayout *coordinateSystemLayout;
    QLabel *coordinateSystemLabel;
    QComboBox *coordinateSystemComboBox;
    QGridLayout *coordinates;
    QLabel *AxisYLabel;
    AngleSpinBox *AxisXSpinBox;
    AngleSpinBox *AxisYSpinBox;
    QLabel *AxisXLabel;
    QToolButton *goPushButton;
    QLabel *label;
    QWidget *tabLists;
    QGridLayout *gridLayout_3;
    QComboBox *objectTypeComboBox;
    QCheckBox *searchInEnglishCheckBox;
    QLineEdit *searchInListLineEdit;
    QListView *objectsListView;
    QWidget *tabOptions;
    QGridLayout *gridLayout_6;
    QGroupBox *searchOptions;
    QVBoxLayout *verticalLayout_8;
    QCheckBox *checkBoxUseStartOfWords;
    QCheckBox *checkBoxUseSortingByLength;
    QCheckBox *checkBoxLockPosition;
    QCheckBox *checkBoxAutoClosing;
    QCheckBox *checkBoxFOVCenterMarker;
    QSpacerItem *verticalSpacer;
    QGroupBox *simbadGroupBox;
    QVBoxLayout *verticalLayout_6;
    QHBoxLayout *horizontalLayout_2;
    QLabel *labelServer;
    QComboBox *serverListComboBox;
    QGroupBox *recentSearches;
    QGridLayout *gridLayout_5;
    QHBoxLayout *horizontalLayout_5;
    QLabel *recentSearchSizeLabel;
    QSpinBox *recentSearchSizeSpinBox;
    QSpacerItem *horizontalSpacer_2;
    QToolButton *recentSearchClearDataPushButton;

    void setupUi(QWidget *searchDialogForm)
    {
        if (searchDialogForm->objectName().isEmpty())
            searchDialogForm->setObjectName("searchDialogForm");
        searchDialogForm->resize(300, 300);
        QSizePolicy sizePolicy(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::MinimumExpanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(searchDialogForm->sizePolicy().hasHeightForWidth());
        searchDialogForm->setSizePolicy(sizePolicy);
        searchDialogForm->setFocusPolicy(Qt::ClickFocus);
        searchDialogForm->setStyleSheet(QString::fromUtf8(""));
        verticalLayout = new QVBoxLayout(searchDialogForm);
        verticalLayout->setSpacing(0);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        titleBar = new TitleBar(searchDialogForm);
        titleBar->setObjectName("titleBar");

        verticalLayout->addWidget(titleBar);

        tabWidget = new QTabWidget(searchDialogForm);
        tabWidget->setObjectName("tabWidget");
        objectTab = new QWidget();
        objectTab->setObjectName("objectTab");
        verticalLayout_2 = new QVBoxLayout(objectTab);
        verticalLayout_2->setObjectName("verticalLayout_2");
        horizontalSearchLayout = new QFrame(objectTab);
        horizontalSearchLayout->setObjectName("horizontalSearchLayout");
        horizontalSearchLayout->setFrameShape(QFrame::StyledPanel);
        horizontalLayout = new QHBoxLayout(horizontalSearchLayout);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        lineEditSearchSkyObject = new QLineEdit(horizontalSearchLayout);
        lineEditSearchSkyObject->setObjectName("lineEditSearchSkyObject");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(lineEditSearchSkyObject->sizePolicy().hasHeightForWidth());
        lineEditSearchSkyObject->setSizePolicy(sizePolicy1);
        QFont font;
        font.setPointSize(15);
        lineEditSearchSkyObject->setFont(font);
        lineEditSearchSkyObject->setFocusPolicy(Qt::StrongFocus);
        lineEditSearchSkyObject->setContextMenuPolicy(Qt::CustomContextMenu);
        lineEditSearchSkyObject->setAutoFillBackground(false);

        horizontalLayout->addWidget(lineEditSearchSkyObject);

        pushButtonGotoSearchSkyObject = new QToolButton(horizontalSearchLayout);
        pushButtonGotoSearchSkyObject->setObjectName("pushButtonGotoSearchSkyObject");
        pushButtonGotoSearchSkyObject->setFocusPolicy(Qt::NoFocus);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/graphicGui/uibtSearch.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        icon.addFile(QString::fromUtf8(":/graphicGui/uibtSearch.png"), QSize(), QIcon::Mode::Disabled, QIcon::State::Off);
        pushButtonGotoSearchSkyObject->setIcon(icon);
        pushButtonGotoSearchSkyObject->setCheckable(false);

        horizontalLayout->addWidget(pushButtonGotoSearchSkyObject);


        verticalLayout_2->addWidget(horizontalSearchLayout);

        searchListView = new QListView(objectTab);
        searchListView->setObjectName("searchListView");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(searchListView->sizePolicy().hasHeightForWidth());
        searchListView->setSizePolicy(sizePolicy2);
        QFont font1;
        font1.setFamilies({QString::fromUtf8("Ubuntu")});
        searchListView->setFont(font1);
        searchListView->setFocusPolicy(Qt::NoFocus);
        searchListView->setEditTriggers(QAbstractItemView::NoEditTriggers);
        searchListView->setProperty("isWrapping", QVariant(false));

        verticalLayout_2->addWidget(searchListView);

        simbadStatusLabel = new QLabel(objectTab);
        simbadStatusLabel->setObjectName("simbadStatusLabel");
        simbadStatusLabel->setAlignment(Qt::AlignCenter);

        verticalLayout_2->addWidget(simbadStatusLabel);

        greekLettersFrame = new QFrame(objectTab);
        greekLettersFrame->setObjectName("greekLettersFrame");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(greekLettersFrame->sizePolicy().hasHeightForWidth());
        greekLettersFrame->setSizePolicy(sizePolicy3);
        gridLayout = new QGridLayout(greekLettersFrame);
        gridLayout->setSpacing(0);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setContentsMargins(0, 0, 0, 0);
        iotaPushButton = new QPushButton(greekLettersFrame);
        iotaPushButton->setObjectName("iotaPushButton");
        iotaPushButton->setFocusPolicy(Qt::NoFocus);
        iotaPushButton->setText(QString::fromUtf8("\316\271"));

        gridLayout->addWidget(iotaPushButton, 1, 8, 1, 1);

        alphaPushButton = new QPushButton(greekLettersFrame);
        alphaPushButton->setObjectName("alphaPushButton");
        alphaPushButton->setFocusPolicy(Qt::NoFocus);
        alphaPushButton->setText(QString::fromUtf8("\316\261"));

        gridLayout->addWidget(alphaPushButton, 1, 0, 1, 1);

        betaPushButton = new QPushButton(greekLettersFrame);
        betaPushButton->setObjectName("betaPushButton");
        betaPushButton->setFocusPolicy(Qt::NoFocus);
        betaPushButton->setText(QString::fromUtf8("\316\262"));

        gridLayout->addWidget(betaPushButton, 1, 1, 1, 1);

        gammaPushButton = new QPushButton(greekLettersFrame);
        gammaPushButton->setObjectName("gammaPushButton");
        gammaPushButton->setFocusPolicy(Qt::NoFocus);
        gammaPushButton->setText(QString::fromUtf8("\316\263"));

        gridLayout->addWidget(gammaPushButton, 1, 2, 1, 1);

        deltaPushButton = new QPushButton(greekLettersFrame);
        deltaPushButton->setObjectName("deltaPushButton");
        deltaPushButton->setFocusPolicy(Qt::NoFocus);
        deltaPushButton->setText(QString::fromUtf8("\316\264"));

        gridLayout->addWidget(deltaPushButton, 1, 3, 1, 1);

        epsilonPushButton = new QPushButton(greekLettersFrame);
        epsilonPushButton->setObjectName("epsilonPushButton");
        epsilonPushButton->setFocusPolicy(Qt::NoFocus);
        epsilonPushButton->setText(QString::fromUtf8("\316\265"));

        gridLayout->addWidget(epsilonPushButton, 1, 4, 1, 1);

        zetaPushButton = new QPushButton(greekLettersFrame);
        zetaPushButton->setObjectName("zetaPushButton");
        zetaPushButton->setFocusPolicy(Qt::NoFocus);
        zetaPushButton->setText(QString::fromUtf8("\316\266"));

        gridLayout->addWidget(zetaPushButton, 1, 5, 1, 1);

        etaPushButton = new QPushButton(greekLettersFrame);
        etaPushButton->setObjectName("etaPushButton");
        etaPushButton->setFocusPolicy(Qt::NoFocus);
        etaPushButton->setText(QString::fromUtf8("\316\267"));

        gridLayout->addWidget(etaPushButton, 1, 6, 1, 1);

        thetaPushButton = new QPushButton(greekLettersFrame);
        thetaPushButton->setObjectName("thetaPushButton");
        thetaPushButton->setFocusPolicy(Qt::NoFocus);
        thetaPushButton->setText(QString::fromUtf8("\316\270"));

        gridLayout->addWidget(thetaPushButton, 1, 7, 1, 1);

        kappaPushButton = new QPushButton(greekLettersFrame);
        kappaPushButton->setObjectName("kappaPushButton");
        kappaPushButton->setFocusPolicy(Qt::NoFocus);
        kappaPushButton->setText(QString::fromUtf8("\316\272"));

        gridLayout->addWidget(kappaPushButton, 1, 9, 1, 1);

        lambdaPushButton = new QPushButton(greekLettersFrame);
        lambdaPushButton->setObjectName("lambdaPushButton");
        lambdaPushButton->setFocusPolicy(Qt::NoFocus);
        lambdaPushButton->setText(QString::fromUtf8("\316\273"));

        gridLayout->addWidget(lambdaPushButton, 1, 10, 1, 1);

        muPushButton = new QPushButton(greekLettersFrame);
        muPushButton->setObjectName("muPushButton");
        muPushButton->setFocusPolicy(Qt::NoFocus);
        muPushButton->setText(QString::fromUtf8("\316\274"));

        gridLayout->addWidget(muPushButton, 1, 11, 1, 1);

        nuPushButton = new QPushButton(greekLettersFrame);
        nuPushButton->setObjectName("nuPushButton");
        nuPushButton->setFocusPolicy(Qt::NoFocus);
        nuPushButton->setText(QString::fromUtf8("\316\275"));

        gridLayout->addWidget(nuPushButton, 2, 0, 1, 1);

        xiPushButton = new QPushButton(greekLettersFrame);
        xiPushButton->setObjectName("xiPushButton");
        xiPushButton->setFocusPolicy(Qt::NoFocus);
        xiPushButton->setText(QString::fromUtf8("\316\276"));

        gridLayout->addWidget(xiPushButton, 2, 1, 1, 1);

        omicronPushButton = new QPushButton(greekLettersFrame);
        omicronPushButton->setObjectName("omicronPushButton");
        omicronPushButton->setFocusPolicy(Qt::NoFocus);
        omicronPushButton->setText(QString::fromUtf8("\316\277"));

        gridLayout->addWidget(omicronPushButton, 2, 2, 1, 1);

        piPushButton = new QPushButton(greekLettersFrame);
        piPushButton->setObjectName("piPushButton");
        piPushButton->setFocusPolicy(Qt::NoFocus);
        piPushButton->setText(QString::fromUtf8("\317\200"));

        gridLayout->addWidget(piPushButton, 2, 3, 1, 1);

        rhoPushButton = new QPushButton(greekLettersFrame);
        rhoPushButton->setObjectName("rhoPushButton");
        rhoPushButton->setFocusPolicy(Qt::NoFocus);
        rhoPushButton->setText(QString::fromUtf8("\317\201"));

        gridLayout->addWidget(rhoPushButton, 2, 4, 1, 1);

        sigmaPushButton = new QPushButton(greekLettersFrame);
        sigmaPushButton->setObjectName("sigmaPushButton");
        sigmaPushButton->setFocusPolicy(Qt::NoFocus);
        sigmaPushButton->setText(QString::fromUtf8("\317\203"));

        gridLayout->addWidget(sigmaPushButton, 2, 5, 1, 1);

        tauPushButton = new QPushButton(greekLettersFrame);
        tauPushButton->setObjectName("tauPushButton");
        tauPushButton->setFocusPolicy(Qt::NoFocus);
        tauPushButton->setText(QString::fromUtf8("\317\204"));

        gridLayout->addWidget(tauPushButton, 2, 6, 1, 1);

        upsilonPushButton = new QPushButton(greekLettersFrame);
        upsilonPushButton->setObjectName("upsilonPushButton");
        upsilonPushButton->setFocusPolicy(Qt::NoFocus);
        upsilonPushButton->setText(QString::fromUtf8("\317\205"));

        gridLayout->addWidget(upsilonPushButton, 2, 7, 1, 1);

        phiPushButton = new QPushButton(greekLettersFrame);
        phiPushButton->setObjectName("phiPushButton");
        phiPushButton->setFocusPolicy(Qt::NoFocus);
        phiPushButton->setText(QString::fromUtf8("\317\206"));

        gridLayout->addWidget(phiPushButton, 2, 8, 1, 1);

        chiPushButton = new QPushButton(greekLettersFrame);
        chiPushButton->setObjectName("chiPushButton");
        chiPushButton->setFocusPolicy(Qt::NoFocus);
        chiPushButton->setText(QString::fromUtf8("\317\207"));

        gridLayout->addWidget(chiPushButton, 2, 9, 1, 1);

        psiPushButton = new QPushButton(greekLettersFrame);
        psiPushButton->setObjectName("psiPushButton");
        psiPushButton->setFocusPolicy(Qt::NoFocus);
        psiPushButton->setText(QString::fromUtf8("\317\210"));

        gridLayout->addWidget(psiPushButton, 2, 10, 1, 1);

        omegaPushButton = new QPushButton(greekLettersFrame);
        omegaPushButton->setObjectName("omegaPushButton");
        omegaPushButton->setFocusPolicy(Qt::NoFocus);
        omegaPushButton->setText(QString::fromUtf8("\317\211"));

        gridLayout->addWidget(omegaPushButton, 2, 11, 1, 1);

        labelGreekLetterTitle = new QLabel(greekLettersFrame);
        labelGreekLetterTitle->setObjectName("labelGreekLetterTitle");
        labelGreekLetterTitle->setAlignment(Qt::AlignCenter);

        gridLayout->addWidget(labelGreekLetterTitle, 0, 0, 1, 12);


        verticalLayout_2->addWidget(greekLettersFrame);

        tabWidget->addTab(objectTab, QString());
        simbadTab = new QWidget();
        simbadTab->setObjectName("simbadTab");
        sizePolicy3.setHeightForWidth(simbadTab->sizePolicy().hasHeightForWidth());
        simbadTab->setSizePolicy(sizePolicy3);
        gridLayout_4 = new QGridLayout(simbadTab);
        gridLayout_4->setObjectName("gridLayout_4");
        simbadCooQueryButton = new QPushButton(simbadTab);
        simbadCooQueryButton->setObjectName("simbadCooQueryButton");

        gridLayout_4->addWidget(simbadCooQueryButton, 2, 0, 1, 1);

        simbadCooResultsTextBrowser = new QTextBrowser(simbadTab);
        simbadCooResultsTextBrowser->setObjectName("simbadCooResultsTextBrowser");
        QSizePolicy sizePolicy4(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy4.setHorizontalStretch(35);
        sizePolicy4.setVerticalStretch(0);
        sizePolicy4.setHeightForWidth(simbadCooResultsTextBrowser->sizePolicy().hasHeightForWidth());
        simbadCooResultsTextBrowser->setSizePolicy(sizePolicy4);
        QFont font2;
        font2.setFamilies({QString::fromUtf8("Courier")});
        simbadCooResultsTextBrowser->setFont(font2);
        simbadCooResultsTextBrowser->setAcceptRichText(false);

        gridLayout_4->addWidget(simbadCooResultsTextBrowser, 3, 0, 1, 1);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        allIDsCheckBox = new QCheckBox(simbadTab);
        allIDsCheckBox->setObjectName("allIDsCheckBox");

        horizontalLayout_3->addWidget(allIDsCheckBox);

        typesCheckBox = new QCheckBox(simbadTab);
        typesCheckBox->setObjectName("typesCheckBox");

        horizontalLayout_3->addWidget(typesCheckBox);

        spectralClassCheckBox = new QCheckBox(simbadTab);
        spectralClassCheckBox->setObjectName("spectralClassCheckBox");

        horizontalLayout_3->addWidget(spectralClassCheckBox);

        morphoCheckBox = new QCheckBox(simbadTab);
        morphoCheckBox->setObjectName("morphoCheckBox");

        horizontalLayout_3->addWidget(morphoCheckBox);

        dimsCheckBox = new QCheckBox(simbadTab);
        dimsCheckBox->setObjectName("dimsCheckBox");

        horizontalLayout_3->addWidget(dimsCheckBox);


        gridLayout_4->addLayout(horizontalLayout_3, 1, 0, 1, 1);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        searchRadiusLabel = new QLabel(simbadTab);
        searchRadiusLabel->setObjectName("searchRadiusLabel");

        horizontalLayout_4->addWidget(searchRadiusLabel);

        searchRadiusSpinBox = new QSpinBox(simbadTab);
        searchRadiusSpinBox->setObjectName("searchRadiusSpinBox");
        searchRadiusSpinBox->setMinimum(1);
        searchRadiusSpinBox->setMaximum(150);
        searchRadiusSpinBox->setValue(30);

        horizontalLayout_4->addWidget(searchRadiusSpinBox);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_4->addItem(horizontalSpacer);

        maxResultsLabel = new QLabel(simbadTab);
        maxResultsLabel->setObjectName("maxResultsLabel");
        maxResultsLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        horizontalLayout_4->addWidget(maxResultsLabel);

        resultsSpinBox = new QSpinBox(simbadTab);
        resultsSpinBox->setObjectName("resultsSpinBox");
        resultsSpinBox->setMinimum(1);
        resultsSpinBox->setMaximum(10);
        resultsSpinBox->setValue(3);

        horizontalLayout_4->addWidget(resultsSpinBox);


        gridLayout_4->addLayout(horizontalLayout_4, 0, 0, 1, 1);

        simbadCooStatusLabel = new QLabel(simbadTab);
        simbadCooStatusLabel->setObjectName("simbadCooStatusLabel");
        simbadCooStatusLabel->setAlignment(Qt::AlignCenter);

        gridLayout_4->addWidget(simbadCooStatusLabel, 4, 0, 1, 1);

        tabWidget->addTab(simbadTab, QString());
        tabWidget->setTabText(tabWidget->indexOf(simbadTab), QString::fromUtf8("SIMBAD"));
        tabPosition = new QWidget();
        tabPosition->setObjectName("tabPosition");
        verticalLayout_3 = new QVBoxLayout(tabPosition);
        verticalLayout_3->setObjectName("verticalLayout_3");
        verticalLayout_3->setContentsMargins(0, 0, 0, 0);
        j2000Frame = new QFrame(tabPosition);
        j2000Frame->setObjectName("j2000Frame");
        gridLayout_2 = new QGridLayout(j2000Frame);
        gridLayout_2->setObjectName("gridLayout_2");
        gridLayout_2->setContentsMargins(0, 0, 0, 0);
        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_2->addItem(verticalSpacer_2, 3, 0, 1, 2);

        coordinateSystemLayout = new QHBoxLayout();
        coordinateSystemLayout->setObjectName("coordinateSystemLayout");
        coordinateSystemLabel = new QLabel(j2000Frame);
        coordinateSystemLabel->setObjectName("coordinateSystemLabel");

        coordinateSystemLayout->addWidget(coordinateSystemLabel);

        coordinateSystemComboBox = new QComboBox(j2000Frame);
        coordinateSystemComboBox->setObjectName("coordinateSystemComboBox");

        coordinateSystemLayout->addWidget(coordinateSystemComboBox);


        gridLayout_2->addLayout(coordinateSystemLayout, 0, 0, 1, 2);

        coordinates = new QGridLayout();
        coordinates->setObjectName("coordinates");
        coordinates->setContentsMargins(-1, 5, -1, -1);
        AxisYLabel = new QLabel(j2000Frame);
        AxisYLabel->setObjectName("AxisYLabel");

        coordinates->addWidget(AxisYLabel, 0, 1, 1, 1);

        AxisXSpinBox = new AngleSpinBox(j2000Frame);
        AxisXSpinBox->setObjectName("AxisXSpinBox");
        QSizePolicy sizePolicy5(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy5.setHorizontalStretch(0);
        sizePolicy5.setVerticalStretch(0);
        sizePolicy5.setHeightForWidth(AxisXSpinBox->sizePolicy().hasHeightForWidth());
        AxisXSpinBox->setSizePolicy(sizePolicy5);

        coordinates->addWidget(AxisXSpinBox, 1, 0, 1, 1);

        AxisYSpinBox = new AngleSpinBox(j2000Frame);
        AxisYSpinBox->setObjectName("AxisYSpinBox");
        sizePolicy5.setHeightForWidth(AxisYSpinBox->sizePolicy().hasHeightForWidth());
        AxisYSpinBox->setSizePolicy(sizePolicy5);

        coordinates->addWidget(AxisYSpinBox, 1, 1, 1, 1);

        AxisXLabel = new QLabel(j2000Frame);
        AxisXLabel->setObjectName("AxisXLabel");

        coordinates->addWidget(AxisXLabel, 0, 0, 1, 1);

        goPushButton = new QToolButton(j2000Frame);
        goPushButton->setObjectName("goPushButton");
        goPushButton->setText(QString::fromUtf8(""));
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/graphicGui/uibtProcess.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        icon1.addFile(QString::fromUtf8(":/graphicGui/uibtProcess.png"), QSize(), QIcon::Mode::Disabled, QIcon::State::Off);
        goPushButton->setIcon(icon1);

        coordinates->addWidget(goPushButton, 1, 2, 1, 1);


        gridLayout_2->addLayout(coordinates, 1, 0, 1, 2);

        label = new QLabel(j2000Frame);
        label->setObjectName("label");
        label->setAlignment(Qt::AlignCenter);
        label->setWordWrap(true);

        gridLayout_2->addWidget(label, 2, 0, 1, 2);


        verticalLayout_3->addWidget(j2000Frame);

        tabWidget->addTab(tabPosition, QString());
        tabLists = new QWidget();
        tabLists->setObjectName("tabLists");
        tabLists->setFont(font1);
        gridLayout_3 = new QGridLayout(tabLists);
        gridLayout_3->setObjectName("gridLayout_3");
        gridLayout_3->setSizeConstraint(QLayout::SetDefaultConstraint);
        objectTypeComboBox = new QComboBox(tabLists);
        objectTypeComboBox->setObjectName("objectTypeComboBox");
        objectTypeComboBox->setEditable(true);
        objectTypeComboBox->setInsertPolicy(QComboBox::NoInsert);

        gridLayout_3->addWidget(objectTypeComboBox, 0, 0, 1, 2);

        searchInEnglishCheckBox = new QCheckBox(tabLists);
        searchInEnglishCheckBox->setObjectName("searchInEnglishCheckBox");

        gridLayout_3->addWidget(searchInEnglishCheckBox, 0, 2, 1, 1);

        searchInListLineEdit = new QLineEdit(tabLists);
        searchInListLineEdit->setObjectName("searchInListLineEdit");

        gridLayout_3->addWidget(searchInListLineEdit, 1, 0, 1, 3);

        objectsListView = new QListView(tabLists);
        objectsListView->setObjectName("objectsListView");

        gridLayout_3->addWidget(objectsListView, 2, 0, 1, 3);

        tabWidget->addTab(tabLists, QString());
        tabOptions = new QWidget();
        tabOptions->setObjectName("tabOptions");
        gridLayout_6 = new QGridLayout(tabOptions);
        gridLayout_6->setObjectName("gridLayout_6");
        gridLayout_6->setContentsMargins(0, 0, 0, 0);
        searchOptions = new QGroupBox(tabOptions);
        searchOptions->setObjectName("searchOptions");
        verticalLayout_8 = new QVBoxLayout(searchOptions);
        verticalLayout_8->setObjectName("verticalLayout_8");
        verticalLayout_8->setContentsMargins(0, 0, 0, 0);
        checkBoxUseStartOfWords = new QCheckBox(searchOptions);
        checkBoxUseStartOfWords->setObjectName("checkBoxUseStartOfWords");
        checkBoxUseStartOfWords->setChecked(false);

        verticalLayout_8->addWidget(checkBoxUseStartOfWords);

        checkBoxUseSortingByLength = new QCheckBox(searchOptions);
        checkBoxUseSortingByLength->setObjectName("checkBoxUseSortingByLength");

        verticalLayout_8->addWidget(checkBoxUseSortingByLength);

        checkBoxLockPosition = new QCheckBox(searchOptions);
        checkBoxLockPosition->setObjectName("checkBoxLockPosition");

        verticalLayout_8->addWidget(checkBoxLockPosition);

        checkBoxAutoClosing = new QCheckBox(searchOptions);
        checkBoxAutoClosing->setObjectName("checkBoxAutoClosing");

        verticalLayout_8->addWidget(checkBoxAutoClosing);

        checkBoxFOVCenterMarker = new QCheckBox(searchOptions);
        checkBoxFOVCenterMarker->setObjectName("checkBoxFOVCenterMarker");

        verticalLayout_8->addWidget(checkBoxFOVCenterMarker);


        gridLayout_6->addWidget(searchOptions, 1, 0, 1, 2);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_6->addItem(verticalSpacer, 3, 0, 1, 2);

        simbadGroupBox = new QGroupBox(tabOptions);
        simbadGroupBox->setObjectName("simbadGroupBox");
        simbadGroupBox->setFlat(true);
        simbadGroupBox->setCheckable(true);
        simbadGroupBox->setChecked(false);
        verticalLayout_6 = new QVBoxLayout(simbadGroupBox);
        verticalLayout_6->setObjectName("verticalLayout_6");
        verticalLayout_6->setContentsMargins(0, 0, 0, 0);
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        labelServer = new QLabel(simbadGroupBox);
        labelServer->setObjectName("labelServer");

        horizontalLayout_2->addWidget(labelServer);

        serverListComboBox = new QComboBox(simbadGroupBox);
        serverListComboBox->setObjectName("serverListComboBox");
        serverListComboBox->setEditable(false);

        horizontalLayout_2->addWidget(serverListComboBox);


        verticalLayout_6->addLayout(horizontalLayout_2);


        gridLayout_6->addWidget(simbadGroupBox, 0, 0, 1, 2);

        recentSearches = new QGroupBox(tabOptions);
        recentSearches->setObjectName("recentSearches");
        gridLayout_5 = new QGridLayout(recentSearches);
        gridLayout_5->setObjectName("gridLayout_5");
        gridLayout_5->setContentsMargins(0, 0, 0, 0);
        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        recentSearchSizeLabel = new QLabel(recentSearches);
        recentSearchSizeLabel->setObjectName("recentSearchSizeLabel");
        recentSearchSizeLabel->setEnabled(true);
        recentSearchSizeLabel->setToolTipDuration(-1);

        horizontalLayout_5->addWidget(recentSearchSizeLabel);

        recentSearchSizeSpinBox = new QSpinBox(recentSearches);
        recentSearchSizeSpinBox->setObjectName("recentSearchSizeSpinBox");
        recentSearchSizeSpinBox->setEnabled(true);
        QFont font3;
        font3.setBold(true);
        recentSearchSizeSpinBox->setFont(font3);
        recentSearchSizeSpinBox->setToolTipDuration(-1);
        recentSearchSizeSpinBox->setAutoFillBackground(true);
        recentSearchSizeSpinBox->setStyleSheet(QString::fromUtf8(""));
        recentSearchSizeSpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        recentSearchSizeSpinBox->setMinimum(0);
        recentSearchSizeSpinBox->setMaximum(99);
        recentSearchSizeSpinBox->setValue(20);

        horizontalLayout_5->addWidget(recentSearchSizeSpinBox);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_5->addItem(horizontalSpacer_2);

        recentSearchClearDataPushButton = new QToolButton(recentSearches);
        recentSearchClearDataPushButton->setObjectName("recentSearchClearDataPushButton");
        QIcon icon2;
        icon2.addFile(QString::fromUtf8(":/graphicGui/uibtDelete.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        icon2.addFile(QString::fromUtf8(":/graphicGui/uibtDelete-disabled.png"), QSize(), QIcon::Mode::Disabled, QIcon::State::Off);
        recentSearchClearDataPushButton->setIcon(icon2);

        horizontalLayout_5->addWidget(recentSearchClearDataPushButton);


        gridLayout_5->addLayout(horizontalLayout_5, 0, 0, 1, 1);


        gridLayout_6->addWidget(recentSearches, 2, 0, 1, 2);

        tabWidget->addTab(tabOptions, QString());

        verticalLayout->addWidget(tabWidget);

        QWidget::setTabOrder(tabWidget, lineEditSearchSkyObject);

        retranslateUi(searchDialogForm);

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(searchDialogForm);
    } // setupUi

    void retranslateUi(QWidget *searchDialogForm)
    {
        searchDialogForm->setWindowTitle(QCoreApplication::translate("searchDialogForm", "Find Object", nullptr));
#if QT_CONFIG(tooltip)
        searchDialogForm->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
        titleBar->setTitle(QCoreApplication::translate("searchDialogForm", "Search window", nullptr));
#if QT_CONFIG(tooltip)
        lineEditSearchSkyObject->setToolTip(QCoreApplication::translate("searchDialogForm", "Use tab key for select of found items", nullptr));
#endif // QT_CONFIG(tooltip)
        lineEditSearchSkyObject->setText(QString());
        simbadStatusLabel->setText(QString());
#if QT_CONFIG(tooltip)
        iotaPushButton->setToolTip(QCoreApplication::translate("searchDialogForm", "iota", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        alphaPushButton->setToolTip(QCoreApplication::translate("searchDialogForm", "alpha", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        betaPushButton->setToolTip(QCoreApplication::translate("searchDialogForm", "beta", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        gammaPushButton->setToolTip(QCoreApplication::translate("searchDialogForm", "gamma", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        deltaPushButton->setToolTip(QCoreApplication::translate("searchDialogForm", "delta", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        epsilonPushButton->setToolTip(QCoreApplication::translate("searchDialogForm", "epsilon", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        zetaPushButton->setToolTip(QCoreApplication::translate("searchDialogForm", "zeta", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        etaPushButton->setToolTip(QCoreApplication::translate("searchDialogForm", "eta", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        thetaPushButton->setToolTip(QCoreApplication::translate("searchDialogForm", "theta", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        kappaPushButton->setToolTip(QCoreApplication::translate("searchDialogForm", "kappa", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        lambdaPushButton->setToolTip(QCoreApplication::translate("searchDialogForm", "lambda", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        muPushButton->setToolTip(QCoreApplication::translate("searchDialogForm", "mu", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        nuPushButton->setToolTip(QCoreApplication::translate("searchDialogForm", "nu", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        xiPushButton->setToolTip(QCoreApplication::translate("searchDialogForm", "xi", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        omicronPushButton->setToolTip(QCoreApplication::translate("searchDialogForm", "omicron", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        piPushButton->setToolTip(QCoreApplication::translate("searchDialogForm", "pi", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        rhoPushButton->setToolTip(QCoreApplication::translate("searchDialogForm", "rho", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        sigmaPushButton->setToolTip(QCoreApplication::translate("searchDialogForm", "sigma", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        tauPushButton->setToolTip(QCoreApplication::translate("searchDialogForm", "tau", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        upsilonPushButton->setToolTip(QCoreApplication::translate("searchDialogForm", "upsilon", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        phiPushButton->setToolTip(QCoreApplication::translate("searchDialogForm", "phi", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        chiPushButton->setToolTip(QCoreApplication::translate("searchDialogForm", "chi", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        psiPushButton->setToolTip(QCoreApplication::translate("searchDialogForm", "psi", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        omegaPushButton->setToolTip(QCoreApplication::translate("searchDialogForm", "omega", nullptr));
#endif // QT_CONFIG(tooltip)
        labelGreekLetterTitle->setText(QCoreApplication::translate("searchDialogForm", "Greek letters for Bayer designations", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(objectTab), QCoreApplication::translate("searchDialogForm", "Object", nullptr));
        simbadCooQueryButton->setText(QCoreApplication::translate("searchDialogForm", "Query SIMBAD about selected object", nullptr));
        allIDsCheckBox->setText(QCoreApplication::translate("searchDialogForm", "All IDs", nullptr));
        typesCheckBox->setText(QCoreApplication::translate("searchDialogForm", "Types", nullptr));
        spectralClassCheckBox->setText(QCoreApplication::translate("searchDialogForm", "Spectral Class", nullptr));
#if QT_CONFIG(tooltip)
        morphoCheckBox->setToolTip(QCoreApplication::translate("searchDialogForm", "Morphological description", nullptr));
#endif // QT_CONFIG(tooltip)
        morphoCheckBox->setText(QCoreApplication::translate("searchDialogForm", "Morph. Descr.", nullptr));
        dimsCheckBox->setText(QCoreApplication::translate("searchDialogForm", "Dimensions", nullptr));
        searchRadiusLabel->setText(QCoreApplication::translate("searchDialogForm", "Max. search radius", nullptr));
        searchRadiusSpinBox->setSuffix(QCoreApplication::translate("searchDialogForm", " arcsec", nullptr));
        maxResultsLabel->setText(QCoreApplication::translate("searchDialogForm", "Max. results ", nullptr));
        simbadCooStatusLabel->setText(QString());
        coordinateSystemLabel->setText(QCoreApplication::translate("searchDialogForm", "Coordinate system:", nullptr));
        AxisYLabel->setText(QString());
        AxisXLabel->setText(QString());
#if QT_CONFIG(tooltip)
        goPushButton->setToolTip(QCoreApplication::translate("searchDialogForm", "Go to position", nullptr));
#endif // QT_CONFIG(tooltip)
        label->setText(QCoreApplication::translate("searchDialogForm", "Note: this tool doesn't apply the refraction correction for coordinates.", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabPosition), QCoreApplication::translate("searchDialogForm", "Position", nullptr));
#if QT_CONFIG(tooltip)
        objectTypeComboBox->setToolTip(QCoreApplication::translate("searchDialogForm", "Some objects may be found after activation respective plug-ins", nullptr));
#endif // QT_CONFIG(tooltip)
        searchInEnglishCheckBox->setText(QCoreApplication::translate("searchDialogForm", "names in English", nullptr));
        searchInListLineEdit->setPlaceholderText(QCoreApplication::translate("searchDialogForm", "Search in list...", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabLists), QCoreApplication::translate("searchDialogForm", "Lists", nullptr));
        searchOptions->setTitle(QCoreApplication::translate("searchDialogForm", "Search options", nullptr));
        checkBoxUseStartOfWords->setText(QCoreApplication::translate("searchDialogForm", "Use autofill only from the beginning of words", nullptr));
        checkBoxUseSortingByLength->setText(QCoreApplication::translate("searchDialogForm", "Use sorting by string length", nullptr));
        checkBoxLockPosition->setText(QCoreApplication::translate("searchDialogForm", "Lock position when coordinates are used", nullptr));
        checkBoxAutoClosing->setText(QCoreApplication::translate("searchDialogForm", "Automatic closing dialog", nullptr));
        checkBoxFOVCenterMarker->setText(QCoreApplication::translate("searchDialogForm", "Show FOV center marker when position is search", nullptr));
#if QT_CONFIG(tooltip)
        simbadGroupBox->setToolTip(QCoreApplication::translate("searchDialogForm", "Use on-line astronomical database SIMBAD", nullptr));
#endif // QT_CONFIG(tooltip)
        simbadGroupBox->setTitle(QCoreApplication::translate("searchDialogForm", "Use SIMBAD", nullptr));
        labelServer->setText(QCoreApplication::translate("searchDialogForm", "Server:", nullptr));
        recentSearches->setTitle(QCoreApplication::translate("searchDialogForm", "Recent Searches", nullptr));
#if QT_CONFIG(tooltip)
        recentSearchSizeLabel->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
        recentSearchSizeLabel->setText(QCoreApplication::translate("searchDialogForm", "Max items to display:", nullptr));
#if QT_CONFIG(tooltip)
        recentSearchClearDataPushButton->setToolTip(QCoreApplication::translate("searchDialogForm", "Deletes all recent search object data", nullptr));
#endif // QT_CONFIG(tooltip)
        tabWidget->setTabText(tabWidget->indexOf(tabOptions), QCoreApplication::translate("searchDialogForm", "Options", nullptr));
    } // retranslateUi

};

namespace Ui {
    class searchDialogForm: public Ui_searchDialogForm {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SEARCHDIALOGGUI_H
