/********************************************************************************
** Form generated from reading UI file 'ObservabilityDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_OBSERVABILITYDIALOG_H
#define UI_OBSERVABILITYDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSlider>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTextBrowser>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "Dialog.hpp"
#include "gui/StelCloseButton.hpp"

QT_BEGIN_NAMESPACE

class Ui_ObservabilityDialog
{
public:
    QVBoxLayout *verticalLayout_2;
    TitleBar *titleBar;
    QTabWidget *tabs;
    QWidget *settingsTab;
    QVBoxLayout *verticalLayout;
    QGroupBox *fontSettingsBox;
    QGridLayout *gridLayout;
    QLabel *redLabel;
    QSlider *redSlider;
    QLabel *greenLabel;
    QSlider *greenSlider;
    QLabel *blueLabel;
    QSlider *blueSlider;
    QLabel *fontSizeLabel;
    QSlider *fontSize;
    QGroupBox *observingConditionsBox;
    QGridLayout *gridLayout_2;
    QSlider *sunAltitudeSlider;
    QLabel *horizonAltitudeLabel;
    QSlider *horizonAltitudeSlider;
    QLabel *sunAltitudeLabel;
    QGroupBox *displayedFieldsBox;
    QVBoxLayout *verticalLayout_4;
    QCheckBox *todayCheckBox;
    QCheckBox *acroCosCheckBox;
    QCheckBox *oppositionCheckBox;
    QCheckBox *goodNightsCheckBox;
    QCheckBox *fullMoonCheckBox;
    QSpacerItem *verticalSpacer;
    QHBoxLayout *horizontalLayout;
    QPushButton *restoreDefaultsButton;
    QSpacerItem *horizontalSpacer;
    QPushButton *saveSettingsButton;
    QWidget *aboutTab;
    QVBoxLayout *verticalLayout_11;
    QTextBrowser *aboutTextBrowser;

    void setupUi(QWidget *ObservabilityDialog)
    {
        if (ObservabilityDialog->objectName().isEmpty())
            ObservabilityDialog->setObjectName("ObservabilityDialog");
        ObservabilityDialog->resize(530, 554);
        verticalLayout_2 = new QVBoxLayout(ObservabilityDialog);
        verticalLayout_2->setSpacing(0);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        titleBar = new TitleBar(ObservabilityDialog);
        titleBar->setObjectName("titleBar");

        verticalLayout_2->addWidget(titleBar);

        tabs = new QTabWidget(ObservabilityDialog);
        tabs->setObjectName("tabs");
        tabs->setDocumentMode(false);
        settingsTab = new QWidget();
        settingsTab->setObjectName("settingsTab");
        verticalLayout = new QVBoxLayout(settingsTab);
        verticalLayout->setObjectName("verticalLayout");
        fontSettingsBox = new QGroupBox(settingsTab);
        fontSettingsBox->setObjectName("fontSettingsBox");
        gridLayout = new QGridLayout(fontSettingsBox);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setContentsMargins(0, 0, 0, 0);
        redLabel = new QLabel(fontSettingsBox);
        redLabel->setObjectName("redLabel");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(redLabel->sizePolicy().hasHeightForWidth());
        redLabel->setSizePolicy(sizePolicy);
        redLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(redLabel, 0, 0, 1, 1);

        redSlider = new QSlider(fontSettingsBox);
        redSlider->setObjectName("redSlider");
        redSlider->setMaximumSize(QSize(508, 19));
        redSlider->setOrientation(Qt::Horizontal);

        gridLayout->addWidget(redSlider, 0, 1, 1, 1);

        greenLabel = new QLabel(fontSettingsBox);
        greenLabel->setObjectName("greenLabel");
        greenLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(greenLabel, 1, 0, 1, 1);

        greenSlider = new QSlider(fontSettingsBox);
        greenSlider->setObjectName("greenSlider");
        greenSlider->setOrientation(Qt::Horizontal);

        gridLayout->addWidget(greenSlider, 1, 1, 1, 1);

        blueLabel = new QLabel(fontSettingsBox);
        blueLabel->setObjectName("blueLabel");
        blueLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(blueLabel, 2, 0, 1, 1);

        blueSlider = new QSlider(fontSettingsBox);
        blueSlider->setObjectName("blueSlider");
        blueSlider->setMaximum(99);
        blueSlider->setOrientation(Qt::Horizontal);

        gridLayout->addWidget(blueSlider, 2, 1, 1, 1);

        fontSizeLabel = new QLabel(fontSettingsBox);
        fontSizeLabel->setObjectName("fontSizeLabel");
        fontSizeLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(fontSizeLabel, 3, 0, 1, 1);

        fontSize = new QSlider(fontSettingsBox);
        fontSize->setObjectName("fontSize");
        fontSize->setMinimum(5);
        fontSize->setMaximum(30);
        fontSize->setOrientation(Qt::Horizontal);

        gridLayout->addWidget(fontSize, 3, 1, 1, 1);


        verticalLayout->addWidget(fontSettingsBox);

        observingConditionsBox = new QGroupBox(settingsTab);
        observingConditionsBox->setObjectName("observingConditionsBox");
        gridLayout_2 = new QGridLayout(observingConditionsBox);
        gridLayout_2->setObjectName("gridLayout_2");
        sunAltitudeSlider = new QSlider(observingConditionsBox);
        sunAltitudeSlider->setObjectName("sunAltitudeSlider");
        sunAltitudeSlider->setMinimum(-20);
        sunAltitudeSlider->setMaximum(0);
        sunAltitudeSlider->setOrientation(Qt::Horizontal);

        gridLayout_2->addWidget(sunAltitudeSlider, 1, 1, 1, 1);

        horizonAltitudeLabel = new QLabel(observingConditionsBox);
        horizonAltitudeLabel->setObjectName("horizonAltitudeLabel");
        horizonAltitudeLabel->setText(QString::fromUtf8("Horizon altitude: 0 deg."));
        horizonAltitudeLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_2->addWidget(horizonAltitudeLabel, 0, 0, 1, 1);

        horizonAltitudeSlider = new QSlider(observingConditionsBox);
        horizonAltitudeSlider->setObjectName("horizonAltitudeSlider");
        horizonAltitudeSlider->setMaximum(45);
        horizonAltitudeSlider->setOrientation(Qt::Horizontal);

        gridLayout_2->addWidget(horizonAltitudeSlider, 0, 1, 1, 1);

        sunAltitudeLabel = new QLabel(observingConditionsBox);
        sunAltitudeLabel->setObjectName("sunAltitudeLabel");
        sunAltitudeLabel->setText(QString::fromUtf8("Sun altitude at twilight: 0 deg."));
        sunAltitudeLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_2->addWidget(sunAltitudeLabel, 1, 0, 1, 1);


        verticalLayout->addWidget(observingConditionsBox);

        displayedFieldsBox = new QGroupBox(settingsTab);
        displayedFieldsBox->setObjectName("displayedFieldsBox");
        verticalLayout_4 = new QVBoxLayout(displayedFieldsBox);
        verticalLayout_4->setObjectName("verticalLayout_4");
        todayCheckBox = new QCheckBox(displayedFieldsBox);
        todayCheckBox->setObjectName("todayCheckBox");

        verticalLayout_4->addWidget(todayCheckBox);

        acroCosCheckBox = new QCheckBox(displayedFieldsBox);
        acroCosCheckBox->setObjectName("acroCosCheckBox");

        verticalLayout_4->addWidget(acroCosCheckBox);

        oppositionCheckBox = new QCheckBox(displayedFieldsBox);
        oppositionCheckBox->setObjectName("oppositionCheckBox");

        verticalLayout_4->addWidget(oppositionCheckBox);

        goodNightsCheckBox = new QCheckBox(displayedFieldsBox);
        goodNightsCheckBox->setObjectName("goodNightsCheckBox");

        verticalLayout_4->addWidget(goodNightsCheckBox);

        fullMoonCheckBox = new QCheckBox(displayedFieldsBox);
        fullMoonCheckBox->setObjectName("fullMoonCheckBox");

        verticalLayout_4->addWidget(fullMoonCheckBox);


        verticalLayout->addWidget(displayedFieldsBox);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        restoreDefaultsButton = new QPushButton(settingsTab);
        restoreDefaultsButton->setObjectName("restoreDefaultsButton");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(restoreDefaultsButton->sizePolicy().hasHeightForWidth());
        restoreDefaultsButton->setSizePolicy(sizePolicy1);

        horizontalLayout->addWidget(restoreDefaultsButton);

        horizontalSpacer = new QSpacerItem(20, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        saveSettingsButton = new QPushButton(settingsTab);
        saveSettingsButton->setObjectName("saveSettingsButton");
        sizePolicy1.setHeightForWidth(saveSettingsButton->sizePolicy().hasHeightForWidth());
        saveSettingsButton->setSizePolicy(sizePolicy1);

        horizontalLayout->addWidget(saveSettingsButton);


        verticalLayout->addLayout(horizontalLayout);

        tabs->addTab(settingsTab, QString());
        aboutTab = new QWidget();
        aboutTab->setObjectName("aboutTab");
        verticalLayout_11 = new QVBoxLayout(aboutTab);
        verticalLayout_11->setSpacing(9);
        verticalLayout_11->setObjectName("verticalLayout_11");
        aboutTextBrowser = new QTextBrowser(aboutTab);
        aboutTextBrowser->setObjectName("aboutTextBrowser");
        aboutTextBrowser->setOpenExternalLinks(true);
        aboutTextBrowser->setOpenLinks(true);

        verticalLayout_11->addWidget(aboutTextBrowser);

        tabs->addTab(aboutTab, QString());

        verticalLayout_2->addWidget(tabs);

        QWidget::setTabOrder(tabs, redSlider);
        QWidget::setTabOrder(redSlider, greenSlider);
        QWidget::setTabOrder(greenSlider, blueSlider);
        QWidget::setTabOrder(blueSlider, fontSize);
        QWidget::setTabOrder(fontSize, todayCheckBox);
        QWidget::setTabOrder(todayCheckBox, acroCosCheckBox);
        QWidget::setTabOrder(acroCosCheckBox, oppositionCheckBox);
        QWidget::setTabOrder(oppositionCheckBox, goodNightsCheckBox);
        QWidget::setTabOrder(goodNightsCheckBox, fullMoonCheckBox);
        QWidget::setTabOrder(fullMoonCheckBox, aboutTextBrowser);

        retranslateUi(ObservabilityDialog);

        tabs->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(ObservabilityDialog);
    } // setupUi

    void retranslateUi(QWidget *ObservabilityDialog)
    {
        ObservabilityDialog->setWindowTitle(QCoreApplication::translate("ObservabilityDialog", "Observability Configuration", nullptr));
        titleBar->setTitle(QCoreApplication::translate("ObservabilityDialog", "Observability Plug-in Configuration", nullptr));
        fontSettingsBox->setTitle(QCoreApplication::translate("ObservabilityDialog", "Font color and size", nullptr));
        redLabel->setText(QCoreApplication::translate("ObservabilityDialog", "Red", nullptr));
        greenLabel->setText(QCoreApplication::translate("ObservabilityDialog", "Green", nullptr));
        blueLabel->setText(QCoreApplication::translate("ObservabilityDialog", "Blue", nullptr));
        fontSizeLabel->setText(QCoreApplication::translate("ObservabilityDialog", "Font Size", nullptr));
        observingConditionsBox->setTitle(QCoreApplication::translate("ObservabilityDialog", "Observing conditions", nullptr));
        displayedFieldsBox->setTitle(QCoreApplication::translate("ObservabilityDialog", "Showing options", nullptr));
        todayCheckBox->setText(QCoreApplication::translate("ObservabilityDialog", "Today's ephemeris (rise, set, and transit times)", nullptr));
        acroCosCheckBox->setText(QCoreApplication::translate("ObservabilityDialog", "Dates of Acronychal/Cosmical/Heliacal rise/set", nullptr));
        oppositionCheckBox->setText(QCoreApplication::translate("ObservabilityDialog", "Date of largest separation to the Sun", nullptr));
        goodNightsCheckBox->setText(QCoreApplication::translate("ObservabilityDialog", "Nights with the source above horizon", nullptr));
        fullMoonCheckBox->setText(QCoreApplication::translate("ObservabilityDialog", "Dates of previous and next Full Moon", nullptr));
        restoreDefaultsButton->setText(QCoreApplication::translate("ObservabilityDialog", "Restore defaults", nullptr));
        saveSettingsButton->setText(QCoreApplication::translate("ObservabilityDialog", "Save settings", nullptr));
        tabs->setTabText(tabs->indexOf(settingsTab), QCoreApplication::translate("ObservabilityDialog", "Settings", nullptr));
        tabs->setTabText(tabs->indexOf(aboutTab), QCoreApplication::translate("ObservabilityDialog", "About", "tab in plugin windows"));
    } // retranslateUi

};

namespace Ui {
    class ObservabilityDialog: public Ui_ObservabilityDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_OBSERVABILITYDIALOG_H
