/********************************************************************************
** Form generated from reading UI file 'remoteSyncDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_REMOTESYNCDIALOG_H
#define UI_REMOTESYNCDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QButtonGroup>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTextBrowser>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "Dialog.hpp"
#include "gui/StelCloseButton.hpp"

QT_BEGIN_NAMESPACE

class Ui_remoteSyncDialog
{
public:
    QVBoxLayout *verticalLayout_2;
    TitleBar *titleBar;
    QTabWidget *tabs;
    QWidget *settingsTab;
    QFormLayout *formLayout;
    QLabel *label_2;
    QLabel *statusLabel;
    QGroupBox *serverGroupBox;
    QFormLayout *formLayout_2;
    QPushButton *serverButton;
    QWidget *serverControls;
    QFormLayout *formLayout_4;
    QLabel *label;
    QSpinBox *serverPortSpinBox;
    QLabel *label_serverName;
    QGroupBox *clientGroupBox;
    QFormLayout *formLayout_3;
    QPushButton *clientButton;
    QWidget *clientControls;
    QFormLayout *formLayout_5;
    QLabel *label_3;
    QLineEdit *clientServerHostEdit;
    QLabel *label_4;
    QSpinBox *clientServerPortSpinBox;
    QLabel *label_7;
    QComboBox *comboBoxClientConnectionLost;
    QLabel *label_8;
    QComboBox *comboBoxClientServerQuits;
    QSpacerItem *verticalSpacer_3;
    QWidget *clientSettingsTab;
    QVBoxLayout *verticalLayout;
    QLabel *label_6;
    QGroupBox *groupBox;
    QGridLayout *gridLayout_2;
    QCheckBox *checkBoxOptionSelection;
    QCheckBox *checkBoxOptionTime;
    QCheckBox *checkBoxOptionLocation;
    QCheckBox *checkBoxOptionStelProperty;
    QCheckBox *checkBoxOptionView;
    QGroupBox *groupBox_2;
    QGridLayout *gridLayout;
    QCheckBox *checkBoxExcludeGUIProps;
    QGridLayout *gridLayoutMoveToGroupbox2;
    QListWidget *listWidgetAllProperties;
    QVBoxLayout *verticalLayoutExclusionButtons;
    QPushButton *pushButtonSelectProperties;
    QPushButton *pushButtonDeselectProperties;
    QListWidget *listWidgetSelectedProperties;
    QLabel *labelExcluded;
    QLabel *labelSynced;
    QLabel *label_5;
    QWidget *aboutTab;
    QVBoxLayout *verticalLayout_11;
    QTextBrowser *aboutTextBrowser;
    QHBoxLayout *horizontalLayout_7;
    QPushButton *restoreDefaultsButton;
    QSpacerItem *horizontalSpacer;
    QPushButton *saveSettingsButton;
    QButtonGroup *buttonGroupSyncOptions;

    void setupUi(QWidget *remoteSyncDialog)
    {
        if (remoteSyncDialog->objectName().isEmpty())
            remoteSyncDialog->setObjectName("remoteSyncDialog");
        remoteSyncDialog->resize(574, 697);
        verticalLayout_2 = new QVBoxLayout(remoteSyncDialog);
        verticalLayout_2->setSpacing(0);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        titleBar = new TitleBar(remoteSyncDialog);
        titleBar->setObjectName("titleBar");

        verticalLayout_2->addWidget(titleBar);

        tabs = new QTabWidget(remoteSyncDialog);
        tabs->setObjectName("tabs");
        tabs->setDocumentMode(false);
        settingsTab = new QWidget();
        settingsTab->setObjectName("settingsTab");
        formLayout = new QFormLayout(settingsTab);
        formLayout->setObjectName("formLayout");
        label_2 = new QLabel(settingsTab);
        label_2->setObjectName("label_2");

        formLayout->setWidget(3, QFormLayout::ItemRole::LabelRole, label_2);

        statusLabel = new QLabel(settingsTab);
        statusLabel->setObjectName("statusLabel");

        formLayout->setWidget(3, QFormLayout::ItemRole::FieldRole, statusLabel);

        serverGroupBox = new QGroupBox(settingsTab);
        serverGroupBox->setObjectName("serverGroupBox");
        formLayout_2 = new QFormLayout(serverGroupBox);
        formLayout_2->setObjectName("formLayout_2");
        serverButton = new QPushButton(serverGroupBox);
        serverButton->setObjectName("serverButton");

        formLayout_2->setWidget(3, QFormLayout::ItemRole::SpanningRole, serverButton);

        serverControls = new QWidget(serverGroupBox);
        serverControls->setObjectName("serverControls");
        formLayout_4 = new QFormLayout(serverControls);
        formLayout_4->setObjectName("formLayout_4");
        formLayout_4->setContentsMargins(0, 0, 0, 0);
        label = new QLabel(serverControls);
        label->setObjectName("label");

        formLayout_4->setWidget(0, QFormLayout::ItemRole::LabelRole, label);

        serverPortSpinBox = new QSpinBox(serverControls);
        serverPortSpinBox->setObjectName("serverPortSpinBox");
        serverPortSpinBox->setMinimum(1);
        serverPortSpinBox->setMaximum(65535);
        serverPortSpinBox->setValue(20180);

        formLayout_4->setWidget(0, QFormLayout::ItemRole::FieldRole, serverPortSpinBox);


        formLayout_2->setWidget(1, QFormLayout::ItemRole::SpanningRole, serverControls);

        label_serverName = new QLabel(serverGroupBox);
        label_serverName->setObjectName("label_serverName");
        label_serverName->setText(QString::fromUtf8("Server Name, IP: "));

        formLayout_2->setWidget(0, QFormLayout::ItemRole::LabelRole, label_serverName);


        formLayout->setWidget(5, QFormLayout::ItemRole::SpanningRole, serverGroupBox);

        clientGroupBox = new QGroupBox(settingsTab);
        clientGroupBox->setObjectName("clientGroupBox");
        formLayout_3 = new QFormLayout(clientGroupBox);
        formLayout_3->setObjectName("formLayout_3");
        clientButton = new QPushButton(clientGroupBox);
        clientButton->setObjectName("clientButton");

        formLayout_3->setWidget(4, QFormLayout::ItemRole::SpanningRole, clientButton);

        clientControls = new QWidget(clientGroupBox);
        clientControls->setObjectName("clientControls");
        formLayout_5 = new QFormLayout(clientControls);
        formLayout_5->setObjectName("formLayout_5");
        formLayout_5->setContentsMargins(0, 0, 0, 0);
        label_3 = new QLabel(clientControls);
        label_3->setObjectName("label_3");

        formLayout_5->setWidget(0, QFormLayout::ItemRole::LabelRole, label_3);

        clientServerHostEdit = new QLineEdit(clientControls);
        clientServerHostEdit->setObjectName("clientServerHostEdit");

        formLayout_5->setWidget(0, QFormLayout::ItemRole::FieldRole, clientServerHostEdit);

        label_4 = new QLabel(clientControls);
        label_4->setObjectName("label_4");

        formLayout_5->setWidget(1, QFormLayout::ItemRole::LabelRole, label_4);

        clientServerPortSpinBox = new QSpinBox(clientControls);
        clientServerPortSpinBox->setObjectName("clientServerPortSpinBox");
        clientServerPortSpinBox->setMinimum(1);
        clientServerPortSpinBox->setMaximum(65535);
        clientServerPortSpinBox->setValue(20180);

        formLayout_5->setWidget(1, QFormLayout::ItemRole::FieldRole, clientServerPortSpinBox);

        label_7 = new QLabel(clientControls);
        label_7->setObjectName("label_7");

        formLayout_5->setWidget(2, QFormLayout::ItemRole::LabelRole, label_7);

        comboBoxClientConnectionLost = new QComboBox(clientControls);
        comboBoxClientConnectionLost->addItem(QString());
        comboBoxClientConnectionLost->addItem(QString());
        comboBoxClientConnectionLost->addItem(QString());
        comboBoxClientConnectionLost->setObjectName("comboBoxClientConnectionLost");

        formLayout_5->setWidget(2, QFormLayout::ItemRole::FieldRole, comboBoxClientConnectionLost);

        label_8 = new QLabel(clientControls);
        label_8->setObjectName("label_8");

        formLayout_5->setWidget(3, QFormLayout::ItemRole::LabelRole, label_8);

        comboBoxClientServerQuits = new QComboBox(clientControls);
        comboBoxClientServerQuits->setObjectName("comboBoxClientServerQuits");

        formLayout_5->setWidget(3, QFormLayout::ItemRole::FieldRole, comboBoxClientServerQuits);


        formLayout_3->setWidget(1, QFormLayout::ItemRole::SpanningRole, clientControls);


        formLayout->setWidget(7, QFormLayout::ItemRole::SpanningRole, clientGroupBox);

        verticalSpacer_3 = new QSpacerItem(20, 120, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        formLayout->setItem(8, QFormLayout::ItemRole::LabelRole, verticalSpacer_3);

        tabs->addTab(settingsTab, QString());
        clientSettingsTab = new QWidget();
        clientSettingsTab->setObjectName("clientSettingsTab");
        verticalLayout = new QVBoxLayout(clientSettingsTab);
        verticalLayout->setObjectName("verticalLayout");
        label_6 = new QLabel(clientSettingsTab);
        label_6->setObjectName("label_6");
        QFont font;
        font.setPointSize(10);
        font.setBold(true);
        font.setItalic(false);
        font.setStrikeOut(false);
        label_6->setFont(font);

        verticalLayout->addWidget(label_6);

        groupBox = new QGroupBox(clientSettingsTab);
        groupBox->setObjectName("groupBox");
        gridLayout_2 = new QGridLayout(groupBox);
        gridLayout_2->setObjectName("gridLayout_2");
        checkBoxOptionSelection = new QCheckBox(groupBox);
        buttonGroupSyncOptions = new QButtonGroup(remoteSyncDialog);
        buttonGroupSyncOptions->setObjectName("buttonGroupSyncOptions");
        buttonGroupSyncOptions->setExclusive(false);
        buttonGroupSyncOptions->addButton(checkBoxOptionSelection);
        checkBoxOptionSelection->setObjectName("checkBoxOptionSelection");

        gridLayout_2->addWidget(checkBoxOptionSelection, 0, 2, 1, 1);

        checkBoxOptionTime = new QCheckBox(groupBox);
        buttonGroupSyncOptions->addButton(checkBoxOptionTime);
        checkBoxOptionTime->setObjectName("checkBoxOptionTime");

        gridLayout_2->addWidget(checkBoxOptionTime, 0, 0, 1, 1);

        checkBoxOptionLocation = new QCheckBox(groupBox);
        buttonGroupSyncOptions->addButton(checkBoxOptionLocation);
        checkBoxOptionLocation->setObjectName("checkBoxOptionLocation");

        gridLayout_2->addWidget(checkBoxOptionLocation, 0, 1, 1, 1);

        checkBoxOptionStelProperty = new QCheckBox(groupBox);
        buttonGroupSyncOptions->addButton(checkBoxOptionStelProperty);
        checkBoxOptionStelProperty->setObjectName("checkBoxOptionStelProperty");
        checkBoxOptionStelProperty->setText(QString::fromUtf8("StelProperty"));

        gridLayout_2->addWidget(checkBoxOptionStelProperty, 1, 0, 1, 1);

        checkBoxOptionView = new QCheckBox(groupBox);
        buttonGroupSyncOptions->addButton(checkBoxOptionView);
        checkBoxOptionView->setObjectName("checkBoxOptionView");

        gridLayout_2->addWidget(checkBoxOptionView, 1, 1, 1, 1);


        verticalLayout->addWidget(groupBox);

        groupBox_2 = new QGroupBox(clientSettingsTab);
        groupBox_2->setObjectName("groupBox_2");
        gridLayout = new QGridLayout(groupBox_2);
        gridLayout->setObjectName("gridLayout");
        checkBoxExcludeGUIProps = new QCheckBox(groupBox_2);
        buttonGroupSyncOptions->addButton(checkBoxExcludeGUIProps);
        checkBoxExcludeGUIProps->setObjectName("checkBoxExcludeGUIProps");

        gridLayout->addWidget(checkBoxExcludeGUIProps, 0, 0, 1, 1);

        gridLayoutMoveToGroupbox2 = new QGridLayout();
        gridLayoutMoveToGroupbox2->setObjectName("gridLayoutMoveToGroupbox2");
        listWidgetAllProperties = new QListWidget(groupBox_2);
        listWidgetAllProperties->setObjectName("listWidgetAllProperties");
        listWidgetAllProperties->setSelectionMode(QAbstractItemView::ExtendedSelection);

        gridLayoutMoveToGroupbox2->addWidget(listWidgetAllProperties, 1, 0, 1, 1);

        verticalLayoutExclusionButtons = new QVBoxLayout();
        verticalLayoutExclusionButtons->setObjectName("verticalLayoutExclusionButtons");
        verticalLayoutExclusionButtons->setContentsMargins(-1, 120, -1, 120);
        pushButtonSelectProperties = new QPushButton(groupBox_2);
        pushButtonSelectProperties->setObjectName("pushButtonSelectProperties");
        pushButtonSelectProperties->setText(QString::fromUtf8("-->"));

        verticalLayoutExclusionButtons->addWidget(pushButtonSelectProperties);

        pushButtonDeselectProperties = new QPushButton(groupBox_2);
        pushButtonDeselectProperties->setObjectName("pushButtonDeselectProperties");
        pushButtonDeselectProperties->setText(QString::fromUtf8("<--"));

        verticalLayoutExclusionButtons->addWidget(pushButtonDeselectProperties);


        gridLayoutMoveToGroupbox2->addLayout(verticalLayoutExclusionButtons, 1, 1, 1, 1);

        listWidgetSelectedProperties = new QListWidget(groupBox_2);
        listWidgetSelectedProperties->setObjectName("listWidgetSelectedProperties");
        listWidgetSelectedProperties->setSelectionMode(QAbstractItemView::ExtendedSelection);

        gridLayoutMoveToGroupbox2->addWidget(listWidgetSelectedProperties, 1, 2, 1, 1);

        labelExcluded = new QLabel(groupBox_2);
        labelExcluded->setObjectName("labelExcluded");

        gridLayoutMoveToGroupbox2->addWidget(labelExcluded, 0, 2, 1, 1);

        labelSynced = new QLabel(groupBox_2);
        labelSynced->setObjectName("labelSynced");

        gridLayoutMoveToGroupbox2->addWidget(labelSynced, 0, 0, 1, 1);


        gridLayout->addLayout(gridLayoutMoveToGroupbox2, 2, 0, 1, 1);

        label_5 = new QLabel(groupBox_2);
        label_5->setObjectName("label_5");

        gridLayout->addWidget(label_5, 1, 0, 1, 1);


        verticalLayout->addWidget(groupBox_2);

        tabs->addTab(clientSettingsTab, QString());
        aboutTab = new QWidget();
        aboutTab->setObjectName("aboutTab");
        verticalLayout_11 = new QVBoxLayout(aboutTab);
        verticalLayout_11->setSpacing(9);
        verticalLayout_11->setObjectName("verticalLayout_11");
        aboutTextBrowser = new QTextBrowser(aboutTab);
        aboutTextBrowser->setObjectName("aboutTextBrowser");
        aboutTextBrowser->setOpenExternalLinks(true);
        aboutTextBrowser->setOpenLinks(true);

        verticalLayout_11->addWidget(aboutTextBrowser);

        tabs->addTab(aboutTab, QString());

        verticalLayout_2->addWidget(tabs);

        horizontalLayout_7 = new QHBoxLayout();
        horizontalLayout_7->setObjectName("horizontalLayout_7");
        horizontalLayout_7->setContentsMargins(9, 9, 9, 9);
        restoreDefaultsButton = new QPushButton(remoteSyncDialog);
        restoreDefaultsButton->setObjectName("restoreDefaultsButton");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(restoreDefaultsButton->sizePolicy().hasHeightForWidth());
        restoreDefaultsButton->setSizePolicy(sizePolicy);

        horizontalLayout_7->addWidget(restoreDefaultsButton);

        horizontalSpacer = new QSpacerItem(20, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout_7->addItem(horizontalSpacer);

        saveSettingsButton = new QPushButton(remoteSyncDialog);
        saveSettingsButton->setObjectName("saveSettingsButton");
        sizePolicy.setHeightForWidth(saveSettingsButton->sizePolicy().hasHeightForWidth());
        saveSettingsButton->setSizePolicy(sizePolicy);

        horizontalLayout_7->addWidget(saveSettingsButton);


        verticalLayout_2->addLayout(horizontalLayout_7);

#if QT_CONFIG(shortcut)
        label->setBuddy(serverPortSpinBox);
        label_3->setBuddy(clientServerHostEdit);
        label_4->setBuddy(clientServerPortSpinBox);
        label_7->setBuddy(comboBoxClientConnectionLost);
        label_8->setBuddy(comboBoxClientServerQuits);
#endif // QT_CONFIG(shortcut)

        retranslateUi(remoteSyncDialog);

        tabs->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(remoteSyncDialog);
    } // setupUi

    void retranslateUi(QWidget *remoteSyncDialog)
    {
        remoteSyncDialog->setWindowTitle(QCoreApplication::translate("remoteSyncDialog", "Remote Control Configuration", nullptr));
        titleBar->setTitle(QCoreApplication::translate("remoteSyncDialog", "Remote Sync Plug-in Configuration", nullptr));
        label_2->setText(QCoreApplication::translate("remoteSyncDialog", "Status:", nullptr));
        statusLabel->setText(QString());
        serverGroupBox->setTitle(QCoreApplication::translate("remoteSyncDialog", "Server mode", nullptr));
        serverButton->setText(QCoreApplication::translate("remoteSyncDialog", "Start server", nullptr));
        label->setText(QCoreApplication::translate("remoteSyncDialog", "Server port", nullptr));
        clientGroupBox->setTitle(QCoreApplication::translate("remoteSyncDialog", "Client mode", nullptr));
        clientButton->setText(QCoreApplication::translate("remoteSyncDialog", "Connect to server", nullptr));
        label_3->setText(QCoreApplication::translate("remoteSyncDialog", "Server host address/name", nullptr));
        clientServerHostEdit->setText(QCoreApplication::translate("remoteSyncDialog", "127.0.0.1", nullptr));
        label_4->setText(QCoreApplication::translate("remoteSyncDialog", "Server port", nullptr));
        label_7->setText(QCoreApplication::translate("remoteSyncDialog", "When connection is lost", nullptr));
        comboBoxClientConnectionLost->setItemText(0, QCoreApplication::translate("remoteSyncDialog", "Do nothing", nullptr));
        comboBoxClientConnectionLost->setItemText(1, QCoreApplication::translate("remoteSyncDialog", "Try reconnecting", nullptr));
        comboBoxClientConnectionLost->setItemText(2, QCoreApplication::translate("remoteSyncDialog", "Quit", nullptr));

        label_8->setText(QCoreApplication::translate("remoteSyncDialog", "When server quits", nullptr));
        tabs->setTabText(tabs->indexOf(settingsTab), QCoreApplication::translate("remoteSyncDialog", "Remote Sync", nullptr));
        label_6->setText(QCoreApplication::translate("remoteSyncDialog", "Changes on this page require a re-connection to the server", nullptr));
        groupBox->setTitle(QCoreApplication::translate("remoteSyncDialog", "Settings applied on this client", nullptr));
        checkBoxOptionSelection->setText(QCoreApplication::translate("remoteSyncDialog", "Selection", nullptr));
        checkBoxOptionTime->setText(QCoreApplication::translate("remoteSyncDialog", "Time", nullptr));
        checkBoxOptionLocation->setText(QCoreApplication::translate("remoteSyncDialog", "Location", nullptr));
        checkBoxOptionView->setText(QCoreApplication::translate("remoteSyncDialog", "View direction", nullptr));
        groupBox_2->setTitle(QCoreApplication::translate("remoteSyncDialog", "Property filters", nullptr));
        checkBoxExcludeGUIProps->setText(QCoreApplication::translate("remoteSyncDialog", "Exclude GUI-related properties", nullptr));
#if QT_CONFIG(tooltip)
        listWidgetAllProperties->setToolTip(QCoreApplication::translate("remoteSyncDialog", "These are all available properties.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        listWidgetSelectedProperties->setToolTip(QCoreApplication::translate("remoteSyncDialog", "These are the properties currently excluded from synchronisation.", nullptr));
#endif // QT_CONFIG(tooltip)
        labelExcluded->setText(QCoreApplication::translate("remoteSyncDialog", "Excluded Properties", nullptr));
        labelSynced->setText(QCoreApplication::translate("remoteSyncDialog", "Synchronized Properties", nullptr));
        label_5->setText(QCoreApplication::translate("remoteSyncDialog", "Additional excluded properties:", nullptr));
        tabs->setTabText(tabs->indexOf(clientSettingsTab), QCoreApplication::translate("remoteSyncDialog", "Client settings", nullptr));
        tabs->setTabText(tabs->indexOf(aboutTab), QCoreApplication::translate("remoteSyncDialog", "About", "tab in plugin windows"));
        restoreDefaultsButton->setText(QCoreApplication::translate("remoteSyncDialog", "Restore defaults", nullptr));
        saveSettingsButton->setText(QCoreApplication::translate("remoteSyncDialog", "Save settings", nullptr));
    } // retranslateUi

};

namespace Ui {
    class remoteSyncDialog: public Ui_remoteSyncDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_REMOTESYNCDIALOG_H
