/****************************************************************************
** Meta object code from reading C++ file 'V3d.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../stellarium/src/scripting/V3d.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'V3d.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3V3dE_t {};
} // unnamed namespace

template <> constexpr inline auto V3d::qt_create_metaobjectdata<qt_meta_tag_ZN3V3dE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "V3d",
        "toVec3d",
        "Vec3d",
        "",
        "toVec3f",
        "Vec3f",
        "x",
        "y",
        "z",
        "set",
        "setX",
        "setY",
        "setZ",
        "toString",
        "toHex",
        "other",
        "vec"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'toVec3d'
        QtMocHelpers::SlotData<Vec3d() const>(1, 3, QMC::AccessPublic, 0x80000000 | 2),
        // Slot 'toVec3f'
        QtMocHelpers::SlotData<Vec3f() const>(4, 3, QMC::AccessPublic, 0x80000000 | 5),
        // Slot 'x'
        QtMocHelpers::SlotData<double() const>(6, 3, QMC::AccessPublic, QMetaType::Double),
        // Slot 'y'
        QtMocHelpers::SlotData<double() const>(7, 3, QMC::AccessPublic, QMetaType::Double),
        // Slot 'z'
        QtMocHelpers::SlotData<double() const>(8, 3, QMC::AccessPublic, QMetaType::Double),
        // Slot 'set'
        QtMocHelpers::SlotData<void(const double, const double, const double)>(9, 3, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 6 }, { QMetaType::Double, 7 }, { QMetaType::Double, 8 },
        }}),
        // Slot 'setX'
        QtMocHelpers::SlotData<void(double)>(10, 3, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 6 },
        }}),
        // Slot 'setY'
        QtMocHelpers::SlotData<void(double)>(11, 3, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 7 },
        }}),
        // Slot 'setZ'
        QtMocHelpers::SlotData<void(double)>(12, 3, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 8 },
        }}),
        // Slot 'toString'
        QtMocHelpers::SlotData<QString() const>(13, 3, QMC::AccessPublic, QMetaType::QString),
        // Slot 'toHex'
        QtMocHelpers::SlotData<QString() const>(14, 3, QMC::AccessPublic, QMetaType::QString),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    using Constructor = QtMocHelpers::NoType;
    QtMocHelpers::UintData qt_constructors {
        QtMocHelpers::ConstructorData<Constructor()>(3, QMC::AccessPublic),
        QtMocHelpers::ConstructorData<Constructor(const V3d &)>(3, QMC::AccessPublic, {{
            { 0x80000000 | 0, 15 },
        }}),
        QtMocHelpers::ConstructorData<Constructor(const double, const double, const double)>(3, QMC::AccessPublic, {{
            { QMetaType::Double, 6 }, { QMetaType::Double, 7 }, { QMetaType::Double, 8 },
        }}),
        QtMocHelpers::ConstructorData<Constructor(const Vec3d &)>(3, QMC::AccessPublic, {{
            { 0x80000000 | 2, 16 },
        }}),
    };
    return QtMocHelpers::metaObjectData<V3d, qt_meta_tag_ZN3V3dE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors);
}
Q_CONSTINIT const QMetaObject V3d::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3V3dE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3V3dE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3V3dE_t>.metaTypes,
    nullptr
} };

void V3d::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<V3d *>(_o);
    if (_c == QMetaObject::CreateInstance) {
        switch (_id) {
        case 0: { V3d *_r = new V3d();
            if (_a[0]) *reinterpret_cast<QObject**>(_a[0]) = _r; } break;
        case 1: { V3d *_r = new V3d((*reinterpret_cast<std::add_pointer_t<V3d>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QObject**>(_a[0]) = _r; } break;
        case 2: { V3d *_r = new V3d((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QObject**>(_a[0]) = _r; } break;
        case 3: { V3d *_r = new V3d((*reinterpret_cast<std::add_pointer_t<Vec3d>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QObject**>(_a[0]) = _r; } break;
        default: break;
        }
    }
    if (_c == QMetaObject::ConstructInPlace) {
        switch (_id) {
        case 0: { new (_a[0]) V3d(); } break;
        case 1: { new (_a[0]) V3d((*reinterpret_cast<std::add_pointer_t<V3d>>(_a[1]))); } break;
        case 2: { new (_a[0]) V3d((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3]))); } break;
        case 3: { new (_a[0]) V3d((*reinterpret_cast<std::add_pointer_t<Vec3d>>(_a[1]))); } break;
        default: break;
        }
    }
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { Vec3d _r = _t->toVec3d();
            if (_a[0]) *reinterpret_cast<Vec3d*>(_a[0]) = std::move(_r); }  break;
        case 1: { Vec3f _r = _t->toVec3f();
            if (_a[0]) *reinterpret_cast<Vec3f*>(_a[0]) = std::move(_r); }  break;
        case 2: { double _r = _t->x();
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 3: { double _r = _t->y();
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 4: { double _r = _t->z();
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 5: _t->set((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3]))); break;
        case 6: _t->setX((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 7: _t->setY((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 8: _t->setZ((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 9: { QString _r = _t->toString();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 10: { QString _r = _t->toHex();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
}

const QMetaObject *V3d::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *V3d::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3V3dE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int V3d::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 11;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN3V3fE_t {};
} // unnamed namespace

template <> constexpr inline auto V3f::qt_create_metaobjectdata<qt_meta_tag_ZN3V3fE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "V3f",
        "toVec3d",
        "Vec3d",
        "",
        "toVec3f",
        "Vec3f",
        "x",
        "y",
        "z",
        "set",
        "setX",
        "setY",
        "setZ",
        "toString",
        "toHex",
        "other",
        "vec"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'toVec3d'
        QtMocHelpers::SlotData<Vec3d() const>(1, 3, QMC::AccessPublic, 0x80000000 | 2),
        // Slot 'toVec3f'
        QtMocHelpers::SlotData<Vec3f() const>(4, 3, QMC::AccessPublic, 0x80000000 | 5),
        // Slot 'x'
        QtMocHelpers::SlotData<float() const>(6, 3, QMC::AccessPublic, QMetaType::Float),
        // Slot 'y'
        QtMocHelpers::SlotData<float() const>(7, 3, QMC::AccessPublic, QMetaType::Float),
        // Slot 'z'
        QtMocHelpers::SlotData<float() const>(8, 3, QMC::AccessPublic, QMetaType::Float),
        // Slot 'set'
        QtMocHelpers::SlotData<void(const float, const float, const float)>(9, 3, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 6 }, { QMetaType::Float, 7 }, { QMetaType::Float, 8 },
        }}),
        // Slot 'setX'
        QtMocHelpers::SlotData<void(float)>(10, 3, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 6 },
        }}),
        // Slot 'setY'
        QtMocHelpers::SlotData<void(float)>(11, 3, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 7 },
        }}),
        // Slot 'setZ'
        QtMocHelpers::SlotData<void(float)>(12, 3, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 8 },
        }}),
        // Slot 'toString'
        QtMocHelpers::SlotData<QString() const>(13, 3, QMC::AccessPublic, QMetaType::QString),
        // Slot 'toHex'
        QtMocHelpers::SlotData<QString() const>(14, 3, QMC::AccessPublic, QMetaType::QString),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    using Constructor = QtMocHelpers::NoType;
    QtMocHelpers::UintData qt_constructors {
        QtMocHelpers::ConstructorData<Constructor()>(3, QMC::AccessPublic),
        QtMocHelpers::ConstructorData<Constructor(const V3f &)>(3, QMC::AccessPublic, {{
            { 0x80000000 | 0, 15 },
        }}),
        QtMocHelpers::ConstructorData<Constructor(const float, const float, const float)>(3, QMC::AccessPublic, {{
            { QMetaType::Float, 6 }, { QMetaType::Float, 7 }, { QMetaType::Float, 8 },
        }}),
        QtMocHelpers::ConstructorData<Constructor(const Vec3f &)>(3, QMC::AccessPublic, {{
            { 0x80000000 | 5, 16 },
        }}),
    };
    return QtMocHelpers::metaObjectData<V3f, qt_meta_tag_ZN3V3fE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors);
}
Q_CONSTINIT const QMetaObject V3f::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3V3fE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3V3fE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3V3fE_t>.metaTypes,
    nullptr
} };

void V3f::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<V3f *>(_o);
    if (_c == QMetaObject::CreateInstance) {
        switch (_id) {
        case 0: { V3f *_r = new V3f();
            if (_a[0]) *reinterpret_cast<QObject**>(_a[0]) = _r; } break;
        case 1: { V3f *_r = new V3f((*reinterpret_cast<std::add_pointer_t<V3f>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QObject**>(_a[0]) = _r; } break;
        case 2: { V3f *_r = new V3f((*reinterpret_cast<std::add_pointer_t<float>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QObject**>(_a[0]) = _r; } break;
        case 3: { V3f *_r = new V3f((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QObject**>(_a[0]) = _r; } break;
        default: break;
        }
    }
    if (_c == QMetaObject::ConstructInPlace) {
        switch (_id) {
        case 0: { new (_a[0]) V3f(); } break;
        case 1: { new (_a[0]) V3f((*reinterpret_cast<std::add_pointer_t<V3f>>(_a[1]))); } break;
        case 2: { new (_a[0]) V3f((*reinterpret_cast<std::add_pointer_t<float>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[3]))); } break;
        case 3: { new (_a[0]) V3f((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1]))); } break;
        default: break;
        }
    }
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { Vec3d _r = _t->toVec3d();
            if (_a[0]) *reinterpret_cast<Vec3d*>(_a[0]) = std::move(_r); }  break;
        case 1: { Vec3f _r = _t->toVec3f();
            if (_a[0]) *reinterpret_cast<Vec3f*>(_a[0]) = std::move(_r); }  break;
        case 2: { float _r = _t->x();
            if (_a[0]) *reinterpret_cast<float*>(_a[0]) = std::move(_r); }  break;
        case 3: { float _r = _t->y();
            if (_a[0]) *reinterpret_cast<float*>(_a[0]) = std::move(_r); }  break;
        case 4: { float _r = _t->z();
            if (_a[0]) *reinterpret_cast<float*>(_a[0]) = std::move(_r); }  break;
        case 5: _t->set((*reinterpret_cast<std::add_pointer_t<float>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[3]))); break;
        case 6: _t->setX((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 7: _t->setY((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 8: _t->setZ((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 9: { QString _r = _t->toString();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 10: { QString _r = _t->toHex();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
}

const QMetaObject *V3f::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *V3f::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3V3fE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int V3f::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 11;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN5ColorE_t {};
} // unnamed namespace

template <> constexpr inline auto Color::qt_create_metaobjectdata<qt_meta_tag_ZN5ColorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Color",
        "toVec3d",
        "Vec3d",
        "",
        "toVec3f",
        "Vec3f",
        "getR",
        "r",
        "getG",
        "g",
        "getB",
        "b",
        "set",
        "setR",
        "setG",
        "setB",
        "toString",
        "toRGBString",
        "toHex",
        "other",
        "x",
        "y",
        "z",
        "hexColor",
        "vec"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'toVec3d'
        QtMocHelpers::SlotData<Vec3d() const>(1, 3, QMC::AccessPublic, 0x80000000 | 2),
        // Slot 'toVec3f'
        QtMocHelpers::SlotData<Vec3f() const>(4, 3, QMC::AccessPublic, 0x80000000 | 5),
        // Slot 'getR'
        QtMocHelpers::SlotData<double() const>(6, 3, QMC::AccessPublic, QMetaType::Double),
        // Slot 'r'
        QtMocHelpers::SlotData<double() const>(7, 3, QMC::AccessPublic, QMetaType::Double),
        // Slot 'getG'
        QtMocHelpers::SlotData<double() const>(8, 3, QMC::AccessPublic, QMetaType::Double),
        // Slot 'g'
        QtMocHelpers::SlotData<double() const>(9, 3, QMC::AccessPublic, QMetaType::Double),
        // Slot 'getB'
        QtMocHelpers::SlotData<double() const>(10, 3, QMC::AccessPublic, QMetaType::Double),
        // Slot 'b'
        QtMocHelpers::SlotData<double() const>(11, 3, QMC::AccessPublic, QMetaType::Double),
        // Slot 'set'
        QtMocHelpers::SlotData<void(const double, const double, const double)>(12, 3, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 7 }, { QMetaType::Double, 9 }, { QMetaType::Double, 11 },
        }}),
        // Slot 'setR'
        QtMocHelpers::SlotData<void(double)>(13, 3, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 7 },
        }}),
        // Slot 'setG'
        QtMocHelpers::SlotData<void(double)>(14, 3, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 9 },
        }}),
        // Slot 'setB'
        QtMocHelpers::SlotData<void(double)>(15, 3, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 11 },
        }}),
        // Slot 'toString'
        QtMocHelpers::SlotData<QString() const>(16, 3, QMC::AccessPublic, QMetaType::QString),
        // Slot 'toRGBString'
        QtMocHelpers::SlotData<QString() const>(17, 3, QMC::AccessPublic, QMetaType::QString),
        // Slot 'toHex'
        QtMocHelpers::SlotData<QString() const>(18, 3, QMC::AccessPublic, QMetaType::QString),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    using Constructor = QtMocHelpers::NoType;
    QtMocHelpers::UintData qt_constructors {
        QtMocHelpers::ConstructorData<Constructor()>(3, QMC::AccessPublic),
        QtMocHelpers::ConstructorData<Constructor(const Color &)>(3, QMC::AccessPublic, {{
            { 0x80000000 | 0, 19 },
        }}),
        QtMocHelpers::ConstructorData<Constructor(const double, const double, const double)>(3, QMC::AccessPublic, {{
            { QMetaType::Double, 20 }, { QMetaType::Double, 21 }, { QMetaType::Double, 22 },
        }}),
        QtMocHelpers::ConstructorData<Constructor(QString)>(3, QMC::AccessPublic, {{
            { QMetaType::QString, 23 },
        }}),
        QtMocHelpers::ConstructorData<Constructor(const Vec3f &)>(3, QMC::AccessPublic, {{
            { 0x80000000 | 5, 24 },
        }}),
    };
    return QtMocHelpers::metaObjectData<Color, qt_meta_tag_ZN5ColorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors);
}
Q_CONSTINIT const QMetaObject Color::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5ColorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5ColorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN5ColorE_t>.metaTypes,
    nullptr
} };

void Color::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Color *>(_o);
    if (_c == QMetaObject::CreateInstance) {
        switch (_id) {
        case 0: { Color *_r = new Color();
            if (_a[0]) *reinterpret_cast<QObject**>(_a[0]) = _r; } break;
        case 1: { Color *_r = new Color((*reinterpret_cast<std::add_pointer_t<Color>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QObject**>(_a[0]) = _r; } break;
        case 2: { Color *_r = new Color((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QObject**>(_a[0]) = _r; } break;
        case 3: { Color *_r = new Color((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QObject**>(_a[0]) = _r; } break;
        case 4: { Color *_r = new Color((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QObject**>(_a[0]) = _r; } break;
        default: break;
        }
    }
    if (_c == QMetaObject::ConstructInPlace) {
        switch (_id) {
        case 0: { new (_a[0]) Color(); } break;
        case 1: { new (_a[0]) Color((*reinterpret_cast<std::add_pointer_t<Color>>(_a[1]))); } break;
        case 2: { new (_a[0]) Color((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3]))); } break;
        case 3: { new (_a[0]) Color((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); } break;
        case 4: { new (_a[0]) Color((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1]))); } break;
        default: break;
        }
    }
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { Vec3d _r = _t->toVec3d();
            if (_a[0]) *reinterpret_cast<Vec3d*>(_a[0]) = std::move(_r); }  break;
        case 1: { Vec3f _r = _t->toVec3f();
            if (_a[0]) *reinterpret_cast<Vec3f*>(_a[0]) = std::move(_r); }  break;
        case 2: { double _r = _t->getR();
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 3: { double _r = _t->r();
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 4: { double _r = _t->getG();
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 5: { double _r = _t->g();
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 6: { double _r = _t->getB();
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 7: { double _r = _t->b();
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 8: _t->set((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3]))); break;
        case 9: _t->setR((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 10: _t->setG((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 11: _t->setB((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 12: { QString _r = _t->toString();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 13: { QString _r = _t->toRGBString();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 14: { QString _r = _t->toHex();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
}

const QMetaObject *Color::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Color::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5ColorE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Color::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 15;
    }
    return _id;
}
QT_WARNING_POP
