/********************************************************************************
** Form generated from reading UI file 'satellitesImportDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SATELLITESIMPORTDIALOG_H
#define UI_SATELLITESIMPORTDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListView>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "Dialog.hpp"
#include "gui/StelCloseButton.hpp"

QT_BEGIN_NAMESPACE

class Ui_satellitesImportDialog
{
public:
    QVBoxLayout *verticalLayoutMain;
    TitleBar *titleBar;
    QStackedWidget *stackedWidget;
    QWidget *pageStart;
    QVBoxLayout *verticalLayout;
    QGroupBox *groupBoxWorking;
    QVBoxLayout *verticalLayout_3;
    QLabel *labelMessage;
    QPushButton *pushButtonGetData;
    QPushButton *pushButtonAbort;
    QWidget *pageResults;
    QVBoxLayout *verticalLayout_2;
    QGroupBox *groupBoxResults;
    QGridLayout *gridLayout;
    QPushButton *pushButtonMarkAll;
    QPushButton *pushButtonMarkNone;
    QPushButton *pushButtonDiscard;
    QPushButton *pushButtonAdd;
    QListView *listView;
    QLineEdit *lineEditSearch;

    void setupUi(QWidget *satellitesImportDialog)
    {
        if (satellitesImportDialog->objectName().isEmpty())
            satellitesImportDialog->setObjectName("satellitesImportDialog");
        satellitesImportDialog->resize(398, 298);
        verticalLayoutMain = new QVBoxLayout(satellitesImportDialog);
        verticalLayoutMain->setSpacing(0);
        verticalLayoutMain->setContentsMargins(0, 0, 0, 0);
        verticalLayoutMain->setObjectName("verticalLayoutMain");
        titleBar = new TitleBar(satellitesImportDialog);
        titleBar->setObjectName("titleBar");

        verticalLayoutMain->addWidget(titleBar);

        stackedWidget = new QStackedWidget(satellitesImportDialog);
        stackedWidget->setObjectName("stackedWidget");
        pageStart = new QWidget();
        pageStart->setObjectName("pageStart");
        verticalLayout = new QVBoxLayout(pageStart);
        verticalLayout->setSpacing(0);
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        verticalLayout->setObjectName("verticalLayout");
        groupBoxWorking = new QGroupBox(pageStart);
        groupBoxWorking->setObjectName("groupBoxWorking");
        verticalLayout_3 = new QVBoxLayout(groupBoxWorking);
        verticalLayout_3->setContentsMargins(0, 0, 0, 0);
        verticalLayout_3->setObjectName("verticalLayout_3");
        labelMessage = new QLabel(groupBoxWorking);
        labelMessage->setObjectName("labelMessage");
        labelMessage->setAlignment(Qt::AlignCenter);
        labelMessage->setWordWrap(true);

        verticalLayout_3->addWidget(labelMessage);

        pushButtonGetData = new QPushButton(groupBoxWorking);
        pushButtonGetData->setObjectName("pushButtonGetData");

        verticalLayout_3->addWidget(pushButtonGetData);

        pushButtonAbort = new QPushButton(groupBoxWorking);
        pushButtonAbort->setObjectName("pushButtonAbort");

        verticalLayout_3->addWidget(pushButtonAbort);


        verticalLayout->addWidget(groupBoxWorking);

        stackedWidget->addWidget(pageStart);
        pageResults = new QWidget();
        pageResults->setObjectName("pageResults");
        verticalLayout_2 = new QVBoxLayout(pageResults);
        verticalLayout_2->setSpacing(0);
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        verticalLayout_2->setObjectName("verticalLayout_2");
        groupBoxResults = new QGroupBox(pageResults);
        groupBoxResults->setObjectName("groupBoxResults");
        gridLayout = new QGridLayout(groupBoxResults);
        gridLayout->setContentsMargins(0, 0, 0, 0);
        gridLayout->setObjectName("gridLayout");
        pushButtonMarkAll = new QPushButton(groupBoxResults);
        pushButtonMarkAll->setObjectName("pushButtonMarkAll");

        gridLayout->addWidget(pushButtonMarkAll, 2, 0, 1, 1);

        pushButtonMarkNone = new QPushButton(groupBoxResults);
        pushButtonMarkNone->setObjectName("pushButtonMarkNone");

        gridLayout->addWidget(pushButtonMarkNone, 2, 1, 1, 1);

        pushButtonDiscard = new QPushButton(groupBoxResults);
        pushButtonDiscard->setObjectName("pushButtonDiscard");

        gridLayout->addWidget(pushButtonDiscard, 2, 2, 1, 1);

        pushButtonAdd = new QPushButton(groupBoxResults);
        pushButtonAdd->setObjectName("pushButtonAdd");

        gridLayout->addWidget(pushButtonAdd, 3, 0, 1, 3);

        listView = new QListView(groupBoxResults);
        listView->setObjectName("listView");
        listView->setUniformItemSizes(true);

        gridLayout->addWidget(listView, 1, 0, 1, 3);

        lineEditSearch = new QLineEdit(groupBoxResults);
        lineEditSearch->setObjectName("lineEditSearch");

        gridLayout->addWidget(lineEditSearch, 0, 0, 1, 3);


        verticalLayout_2->addWidget(groupBoxResults);

        stackedWidget->addWidget(pageResults);

        verticalLayoutMain->addWidget(stackedWidget);

        QWidget::setTabOrder(pushButtonGetData, pushButtonAbort);
        QWidget::setTabOrder(pushButtonAbort, lineEditSearch);
        QWidget::setTabOrder(lineEditSearch, listView);
        QWidget::setTabOrder(listView, pushButtonMarkAll);
        QWidget::setTabOrder(pushButtonMarkAll, pushButtonMarkNone);
        QWidget::setTabOrder(pushButtonMarkNone, pushButtonDiscard);
        QWidget::setTabOrder(pushButtonDiscard, pushButtonAdd);

        retranslateUi(satellitesImportDialog);

        stackedWidget->setCurrentIndex(1);


        QMetaObject::connectSlotsByName(satellitesImportDialog);
    } // setupUi

    void retranslateUi(QWidget *satellitesImportDialog)
    {
        titleBar->setTitle(QCoreApplication::translate("satellitesImportDialog", "More Satellites", nullptr));
        pushButtonGetData->setText(QCoreApplication::translate("satellitesImportDialog", "Get data from update sources", nullptr));
        pushButtonAbort->setText(QCoreApplication::translate("satellitesImportDialog", "Abort download", nullptr));
        groupBoxResults->setTitle(QCoreApplication::translate("satellitesImportDialog", "New satellites", nullptr));
        pushButtonMarkAll->setText(QCoreApplication::translate("satellitesImportDialog", "Mark all", nullptr));
        pushButtonMarkNone->setText(QCoreApplication::translate("satellitesImportDialog", "Mark none", nullptr));
        pushButtonDiscard->setText(QCoreApplication::translate("satellitesImportDialog", "Discard", nullptr));
        pushButtonAdd->setText(QCoreApplication::translate("satellitesImportDialog", "Add", nullptr));
        (void)satellitesImportDialog;
    } // retranslateUi

};

namespace Ui {
    class satellitesImportDialog: public Ui_satellitesImportDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SATELLITESIMPORTDIALOG_H
