/****************************************************************************
** Meta object code from reading C++ file 'ConfigurationDialog.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../stellarium/src/gui/ConfigurationDialog.hpp"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ConfigurationDialog.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN19ConfigurationDialogE_t {};
} // unnamed namespace

template <> constexpr inline auto ConfigurationDialog::qt_create_metaobjectdata<qt_meta_tag_ZN19ConfigurationDialogE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ConfigurationDialog",
        "retranslate",
        "",
        "updateSelectedInfoGui",
        "setNoSelectedInfo",
        "setAllSelectedInfo",
        "setBriefSelectedInfo",
        "setDefaultSelectedInfo",
        "setCustomSelectedInfo",
        "setSelectedInfoFromCheckBoxes",
        "saveCustomSelectedInfo",
        "setCustomSelectedNarration",
        "setSelectedNarrationFromCheckBoxes",
        "saveCustomSelectedNarration",
        "updateCurrentLanguage",
        "selectLanguage",
        "id",
        "storeLanguageSettings",
        "storeFontSettings",
        "setStartupTimeMode",
        "showShortcutsWindow",
        "setDiskViewport",
        "setSphericMirror",
        "newStarCatalogData",
        "downloadStars",
        "cancelDownload",
        "downloadFinished",
        "downloadError",
        "QNetworkReply::NetworkError",
        "resetEphemControls",
        "updateConfigLabels",
        "browseForScreenshotDir",
        "selectScreenshotDir",
        "updateDpiTooltip",
        "saveAllSettings",
        "saveCurrentViewDirSettings",
        "setDefaultViewOptions",
        "populatePluginsList",
        "pluginsSelectionChanged",
        "QListWidgetItem*",
        "item",
        "previousItem",
        "pluginConfigureCurrentSelection",
        "loadAtStartupChanged",
        "Qt::CheckState",
        "state",
        "populateDeltaTAlgorithmsList",
        "setDeltaTAlgorithm",
        "algorithmID",
        "setDeltaTAlgorithmDescription",
        "showCustomDeltaTEquationDialog",
        "showConfigureScreenshotsDialog",
        "updateDateTimeDisplayFormat",
        "populateDateFormatsList",
        "setDateFormat",
        "populateTimeFormatsList",
        "setTimeFormat",
        "setButtonBarDTFormat",
        "populateDitherList",
        "setDitherFormat",
        "scriptSelectionChanged",
        "s",
        "runScriptClicked",
        "stopScriptClicked",
        "aScriptIsRunning",
        "aScriptHasStopped",
        "populateScriptsList",
        "setFixedDateTimeToCurrent",
        "de430ButtonClicked",
        "de431ButtonClicked",
        "de440ButtonClicked",
        "de441ButtonClicked",
        "populateFontWritingSystemCombo",
        "handleFontBoxWritingSystem",
        "index",
        "populateScreenshotFileformatsCombo",
        "setKeyNavigationState"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'retranslate'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'updateSelectedInfoGui'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setNoSelectedInfo'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setAllSelectedInfo'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setBriefSelectedInfo'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setDefaultSelectedInfo'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setCustomSelectedInfo'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setSelectedInfoFromCheckBoxes'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'saveCustomSelectedInfo'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setCustomSelectedNarration'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setSelectedNarrationFromCheckBoxes'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'saveCustomSelectedNarration'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateCurrentLanguage'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'selectLanguage'
        QtMocHelpers::SlotData<void(const int)>(15, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 16 },
        }}),
        // Slot 'storeLanguageSettings'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'storeFontSettings'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setStartupTimeMode'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'showShortcutsWindow'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setDiskViewport'
        QtMocHelpers::SlotData<void(bool)>(21, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'setSphericMirror'
        QtMocHelpers::SlotData<void(bool)>(22, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'newStarCatalogData'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'downloadStars'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'cancelDownload'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'downloadFinished'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'downloadError'
        QtMocHelpers::SlotData<void(QNetworkReply::NetworkError)>(27, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 28, 2 },
        }}),
        // Slot 'resetEphemControls'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateConfigLabels'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'browseForScreenshotDir'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'selectScreenshotDir'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateDpiTooltip'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'saveAllSettings'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'saveCurrentViewDirSettings'
        QtMocHelpers::SlotData<void()>(35, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setDefaultViewOptions'
        QtMocHelpers::SlotData<void()>(36, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'populatePluginsList'
        QtMocHelpers::SlotData<void()>(37, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'pluginsSelectionChanged'
        QtMocHelpers::SlotData<void(QListWidgetItem *, QListWidgetItem *)>(38, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 39, 40 }, { 0x80000000 | 39, 41 },
        }}),
        // Slot 'pluginConfigureCurrentSelection'
        QtMocHelpers::SlotData<void()>(42, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'loadAtStartupChanged'
        QtMocHelpers::SlotData<void(Qt::CheckState)>(43, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 44, 45 },
        }}),
        // Slot 'populateDeltaTAlgorithmsList'
        QtMocHelpers::SlotData<void()>(46, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setDeltaTAlgorithm'
        QtMocHelpers::SlotData<void(int)>(47, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 48 },
        }}),
        // Slot 'setDeltaTAlgorithmDescription'
        QtMocHelpers::SlotData<void()>(49, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'showCustomDeltaTEquationDialog'
        QtMocHelpers::SlotData<void()>(50, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'showConfigureScreenshotsDialog'
        QtMocHelpers::SlotData<void()>(51, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateDateTimeDisplayFormat'
        QtMocHelpers::SlotData<void()>(52, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'populateDateFormatsList'
        QtMocHelpers::SlotData<void()>(53, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setDateFormat'
        QtMocHelpers::SlotData<void()>(54, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'populateTimeFormatsList'
        QtMocHelpers::SlotData<void()>(55, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setTimeFormat'
        QtMocHelpers::SlotData<void()>(56, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setButtonBarDTFormat'
        QtMocHelpers::SlotData<void()>(57, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'populateDitherList'
        QtMocHelpers::SlotData<void()>(58, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setDitherFormat'
        QtMocHelpers::SlotData<void()>(59, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'scriptSelectionChanged'
        QtMocHelpers::SlotData<void(const QString &)>(60, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 61 },
        }}),
        // Slot 'runScriptClicked'
        QtMocHelpers::SlotData<void()>(62, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'stopScriptClicked'
        QtMocHelpers::SlotData<void()>(63, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'aScriptIsRunning'
        QtMocHelpers::SlotData<void()>(64, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'aScriptHasStopped'
        QtMocHelpers::SlotData<void()>(65, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'populateScriptsList'
        QtMocHelpers::SlotData<void()>(66, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setFixedDateTimeToCurrent'
        QtMocHelpers::SlotData<void()>(67, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'de430ButtonClicked'
        QtMocHelpers::SlotData<void()>(68, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'de431ButtonClicked'
        QtMocHelpers::SlotData<void()>(69, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'de440ButtonClicked'
        QtMocHelpers::SlotData<void()>(70, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'de441ButtonClicked'
        QtMocHelpers::SlotData<void()>(71, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'populateFontWritingSystemCombo'
        QtMocHelpers::SlotData<void()>(72, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'handleFontBoxWritingSystem'
        QtMocHelpers::SlotData<void(int)>(73, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 74 },
        }}),
        // Slot 'populateScreenshotFileformatsCombo'
        QtMocHelpers::SlotData<void()>(75, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setKeyNavigationState'
        QtMocHelpers::SlotData<void(bool)>(76, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 45 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ConfigurationDialog, qt_meta_tag_ZN19ConfigurationDialogE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ConfigurationDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<StelDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19ConfigurationDialogE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19ConfigurationDialogE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN19ConfigurationDialogE_t>.metaTypes,
    nullptr
} };

void ConfigurationDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ConfigurationDialog *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->retranslate(); break;
        case 1: _t->updateSelectedInfoGui(); break;
        case 2: _t->setNoSelectedInfo(); break;
        case 3: _t->setAllSelectedInfo(); break;
        case 4: _t->setBriefSelectedInfo(); break;
        case 5: _t->setDefaultSelectedInfo(); break;
        case 6: _t->setCustomSelectedInfo(); break;
        case 7: _t->setSelectedInfoFromCheckBoxes(); break;
        case 8: _t->saveCustomSelectedInfo(); break;
        case 9: _t->setCustomSelectedNarration(); break;
        case 10: _t->setSelectedNarrationFromCheckBoxes(); break;
        case 11: _t->saveCustomSelectedNarration(); break;
        case 12: _t->updateCurrentLanguage(); break;
        case 13: _t->selectLanguage((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 14: _t->storeLanguageSettings(); break;
        case 15: _t->storeFontSettings(); break;
        case 16: _t->setStartupTimeMode(); break;
        case 17: _t->showShortcutsWindow(); break;
        case 18: _t->setDiskViewport((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 19: _t->setSphericMirror((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 20: _t->newStarCatalogData(); break;
        case 21: _t->downloadStars(); break;
        case 22: _t->cancelDownload(); break;
        case 23: _t->downloadFinished(); break;
        case 24: _t->downloadError((*reinterpret_cast<std::add_pointer_t<QNetworkReply::NetworkError>>(_a[1]))); break;
        case 25: _t->resetEphemControls(); break;
        case 26: _t->updateConfigLabels(); break;
        case 27: _t->browseForScreenshotDir(); break;
        case 28: _t->selectScreenshotDir(); break;
        case 29: _t->updateDpiTooltip(); break;
        case 30: _t->saveAllSettings(); break;
        case 31: _t->saveCurrentViewDirSettings(); break;
        case 32: _t->setDefaultViewOptions(); break;
        case 33: _t->populatePluginsList(); break;
        case 34: _t->pluginsSelectionChanged((*reinterpret_cast<std::add_pointer_t<QListWidgetItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QListWidgetItem*>>(_a[2]))); break;
        case 35: _t->pluginConfigureCurrentSelection(); break;
        case 36: _t->loadAtStartupChanged((*reinterpret_cast<std::add_pointer_t<Qt::CheckState>>(_a[1]))); break;
        case 37: _t->populateDeltaTAlgorithmsList(); break;
        case 38: _t->setDeltaTAlgorithm((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 39: _t->setDeltaTAlgorithmDescription(); break;
        case 40: _t->showCustomDeltaTEquationDialog(); break;
        case 41: _t->showConfigureScreenshotsDialog(); break;
        case 42: _t->updateDateTimeDisplayFormat(); break;
        case 43: _t->populateDateFormatsList(); break;
        case 44: _t->setDateFormat(); break;
        case 45: _t->populateTimeFormatsList(); break;
        case 46: _t->setTimeFormat(); break;
        case 47: _t->setButtonBarDTFormat(); break;
        case 48: _t->populateDitherList(); break;
        case 49: _t->setDitherFormat(); break;
        case 50: _t->scriptSelectionChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 51: _t->runScriptClicked(); break;
        case 52: _t->stopScriptClicked(); break;
        case 53: _t->aScriptIsRunning(); break;
        case 54: _t->aScriptHasStopped(); break;
        case 55: _t->populateScriptsList(); break;
        case 56: _t->setFixedDateTimeToCurrent(); break;
        case 57: _t->de430ButtonClicked(); break;
        case 58: _t->de431ButtonClicked(); break;
        case 59: _t->de440ButtonClicked(); break;
        case 60: _t->de441ButtonClicked(); break;
        case 61: _t->populateFontWritingSystemCombo(); break;
        case 62: _t->handleFontBoxWritingSystem((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 63: _t->populateScreenshotFileformatsCombo(); break;
        case 64: _t->setKeyNavigationState((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 24:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QNetworkReply::NetworkError >(); break;
            }
            break;
        }
    }
}

const QMetaObject *ConfigurationDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ConfigurationDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19ConfigurationDialogE_t>.strings))
        return static_cast<void*>(this);
    return StelDialog::qt_metacast(_clname);
}

int ConfigurationDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StelDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 65)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 65;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 65)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 65;
    }
    return _id;
}
QT_WARNING_POP
