/********************************************************************************
** Form generated from reading UI file 'navStarsWindow.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_NAVSTARSWINDOW_H
#define UI_NAVSTARSWINDOW_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTextBrowser>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "Dialog.hpp"
#include "gui/StelCloseButton.hpp"

QT_BEGIN_NAMESPACE

class Ui_navStarsWindowForm
{
public:
    QVBoxLayout *verticalLayout;
    TitleBar *titleBar;
    QTabWidget *tabWidget;
    QWidget *tabNavStars;
    QVBoxLayout *verticalLayoutTimeZone;
    QGroupBox *groupBoxNavStars;
    QVBoxLayout *verticalLayout_2;
    QGridLayout *gridLayout;
    QLabel *labelNote;
    QLabel *nsSetLabel;
    QComboBox *nsSetComboBox;
    QLabel *nsSetDescription;
    QGroupBox *groupBoxSettings;
    QVBoxLayout *verticalLayout_4;
    QCheckBox *displayAtStartupCheckBox;
    QCheckBox *highlightWhenVisible;
    QCheckBox *limitInfoToNavStars;
    QCheckBox *tabulatedDisplay;
    QCheckBox *showExtraDecimals;
    QCheckBox *upperLimb;
    QCheckBox *useUTCCheckBox;
    QSpacerItem *verticalSpacer;
    QHBoxLayout *horizontalLayoutNsSettings;
    QPushButton *pushButtonReset;
    QSpacerItem *horizontalSpacer;
    QPushButton *pushButtonSave;
    QWidget *tabTwilight;
    QGridLayout *gridLayout_2;
    QGroupBox *groupBoxTwilight;
    QGridLayout *gridLayout_3;
    QHBoxLayout *horizontalLayout_2;
    QLabel *labelDate;
    QLabel *labelToday;
    QGridLayout *gridLayout_4;
    QLabel *labelNauticalTwilightDuration;
    QLabel *labelDayEnd;
    QLabel *labelNauticalTwilight;
    QLabel *labelAstronomicalTwilightDuration;
    QLabel *labelCivilTwilightDuration;
    QLabel *labelMoonRise;
    QLabel *labelNauticalTwilightBegin;
    QLabel *labelMoonSet;
    QLabel *labelCivilTwilightBegin;
    QLabel *labelMR;
    QLabel *labelCivilTwilightEnd;
    QFrame *line_2;
    QLabel *labelEnd;
    QFrame *line;
    QLabel *labelNauticalTwilightEnd;
    QLabel *labelAstronomicalTwilightBegin;
    QLabel *labelDay;
    QLabel *labelAstronomicalTwilight;
    QLabel *labelMS;
    QLabel *labelDuration;
    QLabel *labelDayDuration;
    QLabel *labelAstronomicalTwilightEnd;
    QLabel *labelMoon;
    QLabel *labelDayBegin;
    QLabel *labelCivilTwilight;
    QLabel *labelBegin;
    QHBoxLayout *horizontalLayout;
    QPushButton *refreshData;
    QSpacerItem *verticalSpacer_2;
    QWidget *tabAbout;
    QVBoxLayout *verticalLayout_3;
    QTextBrowser *aboutTextBrowser;

    void setupUi(QWidget *navStarsWindowForm)
    {
        if (navStarsWindowForm->objectName().isEmpty())
            navStarsWindowForm->setObjectName("navStarsWindowForm");
        navStarsWindowForm->resize(374, 463);
        verticalLayout = new QVBoxLayout(navStarsWindowForm);
        verticalLayout->setSpacing(0);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        titleBar = new TitleBar(navStarsWindowForm);
        titleBar->setObjectName("titleBar");

        verticalLayout->addWidget(titleBar);

        tabWidget = new QTabWidget(navStarsWindowForm);
        tabWidget->setObjectName("tabWidget");
        tabNavStars = new QWidget();
        tabNavStars->setObjectName("tabNavStars");
        verticalLayoutTimeZone = new QVBoxLayout(tabNavStars);
        verticalLayoutTimeZone->setSpacing(0);
        verticalLayoutTimeZone->setObjectName("verticalLayoutTimeZone");
        verticalLayoutTimeZone->setContentsMargins(0, 0, 0, 0);
        groupBoxNavStars = new QGroupBox(tabNavStars);
        groupBoxNavStars->setObjectName("groupBoxNavStars");
        groupBoxNavStars->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);
        verticalLayout_2 = new QVBoxLayout(groupBoxNavStars);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        labelNote = new QLabel(groupBoxNavStars);
        labelNote->setObjectName("labelNote");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::MinimumExpanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(labelNote->sizePolicy().hasHeightForWidth());
        labelNote->setSizePolicy(sizePolicy);
        labelNote->setAlignment(Qt::AlignRight|Qt::AlignTop|Qt::AlignTrailing);

        gridLayout->addWidget(labelNote, 1, 0, 1, 1);

        nsSetLabel = new QLabel(groupBoxNavStars);
        nsSetLabel->setObjectName("nsSetLabel");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(nsSetLabel->sizePolicy().hasHeightForWidth());
        nsSetLabel->setSizePolicy(sizePolicy1);
        nsSetLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(nsSetLabel, 0, 0, 1, 1);

        nsSetComboBox = new QComboBox(groupBoxNavStars);
        nsSetComboBox->setObjectName("nsSetComboBox");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(nsSetComboBox->sizePolicy().hasHeightForWidth());
        nsSetComboBox->setSizePolicy(sizePolicy2);
        nsSetComboBox->setEditable(false);
        nsSetComboBox->setInsertPolicy(QComboBox::NoInsert);
        nsSetComboBox->setSizeAdjustPolicy(QComboBox::AdjustToContents);

        gridLayout->addWidget(nsSetComboBox, 0, 1, 1, 1);

        nsSetDescription = new QLabel(groupBoxNavStars);
        nsSetDescription->setObjectName("nsSetDescription");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::MinimumExpanding);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(nsSetDescription->sizePolicy().hasHeightForWidth());
        nsSetDescription->setSizePolicy(sizePolicy3);
        nsSetDescription->setMinimumSize(QSize(0, 80));
        nsSetDescription->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
        nsSetDescription->setWordWrap(true);

        gridLayout->addWidget(nsSetDescription, 1, 1, 1, 1);


        verticalLayout_2->addLayout(gridLayout);


        verticalLayoutTimeZone->addWidget(groupBoxNavStars);

        groupBoxSettings = new QGroupBox(tabNavStars);
        groupBoxSettings->setObjectName("groupBoxSettings");
        verticalLayout_4 = new QVBoxLayout(groupBoxSettings);
        verticalLayout_4->setObjectName("verticalLayout_4");
        verticalLayout_4->setContentsMargins(0, 0, 0, 0);
        displayAtStartupCheckBox = new QCheckBox(groupBoxSettings);
        displayAtStartupCheckBox->setObjectName("displayAtStartupCheckBox");

        verticalLayout_4->addWidget(displayAtStartupCheckBox);

        highlightWhenVisible = new QCheckBox(groupBoxSettings);
        highlightWhenVisible->setObjectName("highlightWhenVisible");

        verticalLayout_4->addWidget(highlightWhenVisible);

        limitInfoToNavStars = new QCheckBox(groupBoxSettings);
        limitInfoToNavStars->setObjectName("limitInfoToNavStars");

        verticalLayout_4->addWidget(limitInfoToNavStars);

        tabulatedDisplay = new QCheckBox(groupBoxSettings);
        tabulatedDisplay->setObjectName("tabulatedDisplay");

        verticalLayout_4->addWidget(tabulatedDisplay);

        showExtraDecimals = new QCheckBox(groupBoxSettings);
        showExtraDecimals->setObjectName("showExtraDecimals");

        verticalLayout_4->addWidget(showExtraDecimals);

        upperLimb = new QCheckBox(groupBoxSettings);
        upperLimb->setObjectName("upperLimb");

        verticalLayout_4->addWidget(upperLimb);

        useUTCCheckBox = new QCheckBox(groupBoxSettings);
        useUTCCheckBox->setObjectName("useUTCCheckBox");

        verticalLayout_4->addWidget(useUTCCheckBox);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_4->addItem(verticalSpacer);

        horizontalLayoutNsSettings = new QHBoxLayout();
        horizontalLayoutNsSettings->setObjectName("horizontalLayoutNsSettings");
        pushButtonReset = new QPushButton(groupBoxSettings);
        pushButtonReset->setObjectName("pushButtonReset");

        horizontalLayoutNsSettings->addWidget(pushButtonReset);

        horizontalSpacer = new QSpacerItem(20, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayoutNsSettings->addItem(horizontalSpacer);

        pushButtonSave = new QPushButton(groupBoxSettings);
        pushButtonSave->setObjectName("pushButtonSave");

        horizontalLayoutNsSettings->addWidget(pushButtonSave);


        verticalLayout_4->addLayout(horizontalLayoutNsSettings);


        verticalLayoutTimeZone->addWidget(groupBoxSettings);

        tabWidget->addTab(tabNavStars, QString());
        tabTwilight = new QWidget();
        tabTwilight->setObjectName("tabTwilight");
        gridLayout_2 = new QGridLayout(tabTwilight);
        gridLayout_2->setObjectName("gridLayout_2");
        gridLayout_2->setContentsMargins(0, 0, 0, -1);
        groupBoxTwilight = new QGroupBox(tabTwilight);
        groupBoxTwilight->setObjectName("groupBoxTwilight");
        gridLayout_3 = new QGridLayout(groupBoxTwilight);
        gridLayout_3->setObjectName("gridLayout_3");
        gridLayout_3->setContentsMargins(0, 0, 0, 0);
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        labelDate = new QLabel(groupBoxTwilight);
        labelDate->setObjectName("labelDate");
        labelDate->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        horizontalLayout_2->addWidget(labelDate);

        labelToday = new QLabel(groupBoxTwilight);
        labelToday->setObjectName("labelToday");

        horizontalLayout_2->addWidget(labelToday);


        gridLayout_3->addLayout(horizontalLayout_2, 0, 0, 1, 1);

        gridLayout_4 = new QGridLayout();
        gridLayout_4->setObjectName("gridLayout_4");
        labelNauticalTwilightDuration = new QLabel(groupBoxTwilight);
        labelNauticalTwilightDuration->setObjectName("labelNauticalTwilightDuration");
        labelNauticalTwilightDuration->setAlignment(Qt::AlignCenter);

        gridLayout_4->addWidget(labelNauticalTwilightDuration, 5, 3, 1, 1);

        labelDayEnd = new QLabel(groupBoxTwilight);
        labelDayEnd->setObjectName("labelDayEnd");
        labelDayEnd->setAlignment(Qt::AlignCenter);

        gridLayout_4->addWidget(labelDayEnd, 3, 2, 1, 1);

        labelNauticalTwilight = new QLabel(groupBoxTwilight);
        labelNauticalTwilight->setObjectName("labelNauticalTwilight");
        labelNauticalTwilight->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_4->addWidget(labelNauticalTwilight, 5, 0, 1, 1);

        labelAstronomicalTwilightDuration = new QLabel(groupBoxTwilight);
        labelAstronomicalTwilightDuration->setObjectName("labelAstronomicalTwilightDuration");
        labelAstronomicalTwilightDuration->setAlignment(Qt::AlignCenter);

        gridLayout_4->addWidget(labelAstronomicalTwilightDuration, 6, 3, 1, 1);

        labelCivilTwilightDuration = new QLabel(groupBoxTwilight);
        labelCivilTwilightDuration->setObjectName("labelCivilTwilightDuration");
        labelCivilTwilightDuration->setAlignment(Qt::AlignCenter);

        gridLayout_4->addWidget(labelCivilTwilightDuration, 4, 3, 1, 1);

        labelMoonRise = new QLabel(groupBoxTwilight);
        labelMoonRise->setObjectName("labelMoonRise");
        labelMoonRise->setAlignment(Qt::AlignCenter);

        gridLayout_4->addWidget(labelMoonRise, 9, 1, 1, 1);

        labelNauticalTwilightBegin = new QLabel(groupBoxTwilight);
        labelNauticalTwilightBegin->setObjectName("labelNauticalTwilightBegin");
        labelNauticalTwilightBegin->setAlignment(Qt::AlignCenter);

        gridLayout_4->addWidget(labelNauticalTwilightBegin, 5, 1, 1, 1);

        labelMoonSet = new QLabel(groupBoxTwilight);
        labelMoonSet->setObjectName("labelMoonSet");
        labelMoonSet->setAlignment(Qt::AlignCenter);

        gridLayout_4->addWidget(labelMoonSet, 9, 2, 1, 1);

        labelCivilTwilightBegin = new QLabel(groupBoxTwilight);
        labelCivilTwilightBegin->setObjectName("labelCivilTwilightBegin");
        labelCivilTwilightBegin->setAlignment(Qt::AlignCenter);

        gridLayout_4->addWidget(labelCivilTwilightBegin, 4, 1, 1, 1);

        labelMR = new QLabel(groupBoxTwilight);
        labelMR->setObjectName("labelMR");
        labelMR->setAlignment(Qt::AlignCenter);

        gridLayout_4->addWidget(labelMR, 8, 1, 1, 1);

        labelCivilTwilightEnd = new QLabel(groupBoxTwilight);
        labelCivilTwilightEnd->setObjectName("labelCivilTwilightEnd");
        labelCivilTwilightEnd->setAlignment(Qt::AlignCenter);

        gridLayout_4->addWidget(labelCivilTwilightEnd, 4, 2, 1, 1);

        line_2 = new QFrame(groupBoxTwilight);
        line_2->setObjectName("line_2");
        line_2->setMinimumSize(QSize(0, 1));
        line_2->setMaximumSize(QSize(16777215, 1));
        line_2->setFrameShape(QFrame::Shape::HLine);
        line_2->setFrameShadow(QFrame::Shadow::Sunken);

        gridLayout_4->addWidget(line_2, 7, 0, 1, 4);

        labelEnd = new QLabel(groupBoxTwilight);
        labelEnd->setObjectName("labelEnd");
        labelEnd->setAlignment(Qt::AlignCenter);

        gridLayout_4->addWidget(labelEnd, 2, 2, 1, 1);

        line = new QFrame(groupBoxTwilight);
        line->setObjectName("line");
        line->setMinimumSize(QSize(0, 1));
        line->setMaximumSize(QSize(16777215, 1));
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        gridLayout_4->addWidget(line, 1, 0, 1, 4);

        labelNauticalTwilightEnd = new QLabel(groupBoxTwilight);
        labelNauticalTwilightEnd->setObjectName("labelNauticalTwilightEnd");
        labelNauticalTwilightEnd->setAlignment(Qt::AlignCenter);

        gridLayout_4->addWidget(labelNauticalTwilightEnd, 5, 2, 1, 1);

        labelAstronomicalTwilightBegin = new QLabel(groupBoxTwilight);
        labelAstronomicalTwilightBegin->setObjectName("labelAstronomicalTwilightBegin");
        labelAstronomicalTwilightBegin->setAlignment(Qt::AlignCenter);

        gridLayout_4->addWidget(labelAstronomicalTwilightBegin, 6, 1, 1, 1);

        labelDay = new QLabel(groupBoxTwilight);
        labelDay->setObjectName("labelDay");
        labelDay->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_4->addWidget(labelDay, 3, 0, 1, 1);

        labelAstronomicalTwilight = new QLabel(groupBoxTwilight);
        labelAstronomicalTwilight->setObjectName("labelAstronomicalTwilight");
        labelAstronomicalTwilight->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_4->addWidget(labelAstronomicalTwilight, 6, 0, 1, 1);

        labelMS = new QLabel(groupBoxTwilight);
        labelMS->setObjectName("labelMS");
        labelMS->setAlignment(Qt::AlignCenter);

        gridLayout_4->addWidget(labelMS, 8, 2, 1, 1);

        labelDuration = new QLabel(groupBoxTwilight);
        labelDuration->setObjectName("labelDuration");
        labelDuration->setAlignment(Qt::AlignCenter);

        gridLayout_4->addWidget(labelDuration, 2, 3, 1, 1);

        labelDayDuration = new QLabel(groupBoxTwilight);
        labelDayDuration->setObjectName("labelDayDuration");
        labelDayDuration->setAlignment(Qt::AlignCenter);

        gridLayout_4->addWidget(labelDayDuration, 3, 3, 1, 1);

        labelAstronomicalTwilightEnd = new QLabel(groupBoxTwilight);
        labelAstronomicalTwilightEnd->setObjectName("labelAstronomicalTwilightEnd");
        labelAstronomicalTwilightEnd->setAlignment(Qt::AlignCenter);

        gridLayout_4->addWidget(labelAstronomicalTwilightEnd, 6, 2, 1, 1);

        labelMoon = new QLabel(groupBoxTwilight);
        labelMoon->setObjectName("labelMoon");
        labelMoon->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_4->addWidget(labelMoon, 9, 0, 1, 1);

        labelDayBegin = new QLabel(groupBoxTwilight);
        labelDayBegin->setObjectName("labelDayBegin");
        labelDayBegin->setAlignment(Qt::AlignCenter);

        gridLayout_4->addWidget(labelDayBegin, 3, 1, 1, 1);

        labelCivilTwilight = new QLabel(groupBoxTwilight);
        labelCivilTwilight->setObjectName("labelCivilTwilight");
        labelCivilTwilight->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_4->addWidget(labelCivilTwilight, 4, 0, 1, 1);

        labelBegin = new QLabel(groupBoxTwilight);
        labelBegin->setObjectName("labelBegin");
        labelBegin->setAlignment(Qt::AlignCenter);

        gridLayout_4->addWidget(labelBegin, 2, 1, 1, 1);


        gridLayout_3->addLayout(gridLayout_4, 1, 0, 1, 1);


        gridLayout_2->addWidget(groupBoxTwilight, 0, 0, 1, 1);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(12, -1, 12, -1);
        refreshData = new QPushButton(tabTwilight);
        refreshData->setObjectName("refreshData");

        horizontalLayout->addWidget(refreshData);


        gridLayout_2->addLayout(horizontalLayout, 4, 0, 1, 1);

        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_2->addItem(verticalSpacer_2, 1, 0, 1, 1);

        tabWidget->addTab(tabTwilight, QString());
        tabAbout = new QWidget();
        tabAbout->setObjectName("tabAbout");
        verticalLayout_3 = new QVBoxLayout(tabAbout);
        verticalLayout_3->setObjectName("verticalLayout_3");
        aboutTextBrowser = new QTextBrowser(tabAbout);
        aboutTextBrowser->setObjectName("aboutTextBrowser");
        aboutTextBrowser->setOpenExternalLinks(true);

        verticalLayout_3->addWidget(aboutTextBrowser);

        tabWidget->addTab(tabAbout, QString());

        verticalLayout->addWidget(tabWidget);


        retranslateUi(navStarsWindowForm);

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(navStarsWindowForm);
    } // setupUi

    void retranslateUi(QWidget *navStarsWindowForm)
    {
        titleBar->setTitle(QCoreApplication::translate("navStarsWindowForm", "Navigational stars", nullptr));
        groupBoxNavStars->setTitle(QCoreApplication::translate("navStarsWindowForm", "Set of navigational stars", nullptr));
        labelNote->setText(QCoreApplication::translate("navStarsWindowForm", "Note:", nullptr));
        nsSetLabel->setText(QCoreApplication::translate("navStarsWindowForm", "Current set:", nullptr));
        nsSetDescription->setText(QString());
        groupBoxSettings->setTitle(QCoreApplication::translate("navStarsWindowForm", "Settings", nullptr));
        displayAtStartupCheckBox->setText(QCoreApplication::translate("navStarsWindowForm", "Enable display at startup", nullptr));
        highlightWhenVisible->setText(QCoreApplication::translate("navStarsWindowForm", "Only highlight when visible", nullptr));
        limitInfoToNavStars->setText(QCoreApplication::translate("navStarsWindowForm", "Only show extra information for marked Navigation Stars", nullptr));
        tabulatedDisplay->setText(QCoreApplication::translate("navStarsWindowForm", "Show information as a tabulated list", nullptr));
        showExtraDecimals->setText(QCoreApplication::translate("navStarsWindowForm", "Show extra decimals in info", nullptr));
        upperLimb->setText(QCoreApplication::translate("navStarsWindowForm", "Use upper limb for Sun and Moon", nullptr));
        useUTCCheckBox->setText(QCoreApplication::translate("navStarsWindowForm", "Use UTC time when navigational stars are displayed", nullptr));
        pushButtonReset->setText(QCoreApplication::translate("navStarsWindowForm", "Restore defaults", nullptr));
        pushButtonSave->setText(QCoreApplication::translate("navStarsWindowForm", "Save settings", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabNavStars), QCoreApplication::translate("navStarsWindowForm", "Navigational stars", nullptr));
        groupBoxTwilight->setTitle(QCoreApplication::translate("navStarsWindowForm", "Today in the current location", nullptr));
        labelDate->setText(QCoreApplication::translate("navStarsWindowForm", "Today:", nullptr));
        labelToday->setText(QString());
#if QT_CONFIG(tooltip)
        labelNauticalTwilightDuration->setToolTip(QCoreApplication::translate("navStarsWindowForm", "Duration of nautical twilight (including daytime)", nullptr));
#endif // QT_CONFIG(tooltip)
        labelNauticalTwilightDuration->setText(QString());
        labelDayEnd->setText(QString());
        labelNauticalTwilight->setText(QCoreApplication::translate("navStarsWindowForm", "Nautical Twilight", nullptr));
#if QT_CONFIG(tooltip)
        labelAstronomicalTwilightDuration->setToolTip(QCoreApplication::translate("navStarsWindowForm", "Duration of astronomical twilight (including daytime)", nullptr));
#endif // QT_CONFIG(tooltip)
        labelAstronomicalTwilightDuration->setText(QString());
#if QT_CONFIG(tooltip)
        labelCivilTwilightDuration->setToolTip(QCoreApplication::translate("navStarsWindowForm", "Duration of civil twilight (including daytime)", nullptr));
#endif // QT_CONFIG(tooltip)
        labelCivilTwilightDuration->setText(QString());
        labelMoonRise->setText(QString());
        labelNauticalTwilightBegin->setText(QString());
        labelMoonSet->setText(QString());
        labelCivilTwilightBegin->setText(QString());
        labelMR->setText(QCoreApplication::translate("navStarsWindowForm", "Moonrise", nullptr));
        labelCivilTwilightEnd->setText(QString());
        labelEnd->setText(QCoreApplication::translate("navStarsWindowForm", "End", nullptr));
        labelNauticalTwilightEnd->setText(QString());
        labelAstronomicalTwilightBegin->setText(QString());
        labelDay->setText(QCoreApplication::translate("navStarsWindowForm", "Daytime", nullptr));
        labelAstronomicalTwilight->setText(QCoreApplication::translate("navStarsWindowForm", "Astronomical Twilight", nullptr));
        labelMS->setText(QCoreApplication::translate("navStarsWindowForm", "Moonset", nullptr));
        labelDuration->setText(QCoreApplication::translate("navStarsWindowForm", "Duration", nullptr));
#if QT_CONFIG(tooltip)
        labelDayDuration->setToolTip(QCoreApplication::translate("navStarsWindowForm", "Duration of daytime", nullptr));
#endif // QT_CONFIG(tooltip)
        labelDayDuration->setText(QString());
        labelAstronomicalTwilightEnd->setText(QString());
        labelMoon->setText(QCoreApplication::translate("navStarsWindowForm", "Moon", nullptr));
        labelDayBegin->setText(QString());
        labelCivilTwilight->setText(QCoreApplication::translate("navStarsWindowForm", "Civil Twilight", nullptr));
        labelBegin->setText(QCoreApplication::translate("navStarsWindowForm", "Begin", nullptr));
        refreshData->setText(QCoreApplication::translate("navStarsWindowForm", "Refresh", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabTwilight), QCoreApplication::translate("navStarsWindowForm", "Today", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabAbout), QCoreApplication::translate("navStarsWindowForm", "About", "tab in plugin windows"));
        (void)navStarsWindowForm;
    } // retranslateUi

};

namespace Ui {
    class navStarsWindowForm: public Ui_navStarsWindowForm {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_NAVSTARSWINDOW_H
