/********************************************************************************
** Form generated from reading UI file 'supernovaeDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SUPERNOVAEDIALOG_H
#define UI_SUPERNOVAEDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDateTimeEdit>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTextBrowser>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "Dialog.hpp"
#include "gui/StelCloseButton.hpp"

QT_BEGIN_NAMESPACE

class Ui_supernovaeDialog
{
public:
    QVBoxLayout *verticalLayout_2;
    TitleBar *titleBar;
    QTabWidget *tabs;
    QWidget *settingsTab;
    QVBoxLayout *verticalLayout_3;
    QGroupBox *updatesGroup;
    QVBoxLayout *verticalLayout_12;
    QCheckBox *internetUpdatesCheckbox;
    QGridLayout *gridLayout;
    QLabel *lastUpdateLabel;
    QDateTimeEdit *lastUpdateDateTimeEdit;
    QLabel *updateFrequencyLabel;
    QSpinBox *updateFrequencySpinBox;
    QLabel *nextUpdateLabel;
    QSpacerItem *horizontalSpacer;
    QPushButton *updateButton;
    QSpacerItem *verticalSpacer_3;
    QHBoxLayout *horizontalLayout_7;
    QPushButton *restoreDefaultsButton;
    QSpacerItem *horizontalSpacer_2;
    QPushButton *saveSettingsButton;
    QWidget *aboutTab;
    QVBoxLayout *verticalLayout_11;
    QTextBrowser *aboutTextBrowser;

    void setupUi(QWidget *supernovaeDialog)
    {
        if (supernovaeDialog->objectName().isEmpty())
            supernovaeDialog->setObjectName("supernovaeDialog");
        supernovaeDialog->resize(530, 334);
        verticalLayout_2 = new QVBoxLayout(supernovaeDialog);
        verticalLayout_2->setSpacing(0);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        titleBar = new TitleBar(supernovaeDialog);
        titleBar->setObjectName("titleBar");

        verticalLayout_2->addWidget(titleBar);

        tabs = new QTabWidget(supernovaeDialog);
        tabs->setObjectName("tabs");
        tabs->setDocumentMode(false);
        settingsTab = new QWidget();
        settingsTab->setObjectName("settingsTab");
        verticalLayout_3 = new QVBoxLayout(settingsTab);
        verticalLayout_3->setObjectName("verticalLayout_3");
        updatesGroup = new QGroupBox(settingsTab);
        updatesGroup->setObjectName("updatesGroup");
        updatesGroup->setFlat(true);
        updatesGroup->setCheckable(false);
        verticalLayout_12 = new QVBoxLayout(updatesGroup);
        verticalLayout_12->setObjectName("verticalLayout_12");
        internetUpdatesCheckbox = new QCheckBox(updatesGroup);
        internetUpdatesCheckbox->setObjectName("internetUpdatesCheckbox");

        verticalLayout_12->addWidget(internetUpdatesCheckbox);

        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        lastUpdateLabel = new QLabel(updatesGroup);
        lastUpdateLabel->setObjectName("lastUpdateLabel");

        gridLayout->addWidget(lastUpdateLabel, 0, 0, 1, 1);

        lastUpdateDateTimeEdit = new QDateTimeEdit(updatesGroup);
        lastUpdateDateTimeEdit->setObjectName("lastUpdateDateTimeEdit");
        lastUpdateDateTimeEdit->setEnabled(false);
        lastUpdateDateTimeEdit->setFrame(false);
        lastUpdateDateTimeEdit->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        lastUpdateDateTimeEdit->setButtonSymbols(QAbstractSpinBox::NoButtons);

        gridLayout->addWidget(lastUpdateDateTimeEdit, 0, 2, 1, 1);

        updateFrequencyLabel = new QLabel(updatesGroup);
        updateFrequencyLabel->setObjectName("updateFrequencyLabel");

        gridLayout->addWidget(updateFrequencyLabel, 1, 0, 1, 1);

        updateFrequencySpinBox = new QSpinBox(updatesGroup);
        updateFrequencySpinBox->setObjectName("updateFrequencySpinBox");
        updateFrequencySpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        updateFrequencySpinBox->setMinimum(1);
        updateFrequencySpinBox->setMaximum(9999);
        updateFrequencySpinBox->setValue(1);

        gridLayout->addWidget(updateFrequencySpinBox, 1, 2, 1, 1);

        nextUpdateLabel = new QLabel(updatesGroup);
        nextUpdateLabel->setObjectName("nextUpdateLabel");

        gridLayout->addWidget(nextUpdateLabel, 2, 0, 1, 1);

        horizontalSpacer = new QSpacerItem(17, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(horizontalSpacer, 2, 1, 1, 1);

        updateButton = new QPushButton(updatesGroup);
        updateButton->setObjectName("updateButton");

        gridLayout->addWidget(updateButton, 2, 2, 1, 1);


        verticalLayout_12->addLayout(gridLayout);


        verticalLayout_3->addWidget(updatesGroup);

        verticalSpacer_3 = new QSpacerItem(20, 120, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_3->addItem(verticalSpacer_3);

        horizontalLayout_7 = new QHBoxLayout();
        horizontalLayout_7->setObjectName("horizontalLayout_7");
        restoreDefaultsButton = new QPushButton(settingsTab);
        restoreDefaultsButton->setObjectName("restoreDefaultsButton");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(restoreDefaultsButton->sizePolicy().hasHeightForWidth());
        restoreDefaultsButton->setSizePolicy(sizePolicy);

        horizontalLayout_7->addWidget(restoreDefaultsButton);

        horizontalSpacer_2 = new QSpacerItem(20, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout_7->addItem(horizontalSpacer_2);

        saveSettingsButton = new QPushButton(settingsTab);
        saveSettingsButton->setObjectName("saveSettingsButton");
        sizePolicy.setHeightForWidth(saveSettingsButton->sizePolicy().hasHeightForWidth());
        saveSettingsButton->setSizePolicy(sizePolicy);

        horizontalLayout_7->addWidget(saveSettingsButton);


        verticalLayout_3->addLayout(horizontalLayout_7);

        tabs->addTab(settingsTab, QString());
        aboutTab = new QWidget();
        aboutTab->setObjectName("aboutTab");
        verticalLayout_11 = new QVBoxLayout(aboutTab);
        verticalLayout_11->setSpacing(9);
        verticalLayout_11->setObjectName("verticalLayout_11");
        aboutTextBrowser = new QTextBrowser(aboutTab);
        aboutTextBrowser->setObjectName("aboutTextBrowser");
        aboutTextBrowser->setOpenExternalLinks(true);
        aboutTextBrowser->setOpenLinks(true);

        verticalLayout_11->addWidget(aboutTextBrowser);

        tabs->addTab(aboutTab, QString());

        verticalLayout_2->addWidget(tabs);


        retranslateUi(supernovaeDialog);

        QMetaObject::connectSlotsByName(supernovaeDialog);
    } // setupUi

    void retranslateUi(QWidget *supernovaeDialog)
    {
        supernovaeDialog->setWindowTitle(QCoreApplication::translate("supernovaeDialog", "Historical Supernovae Configuration", nullptr));
        titleBar->setTitle(QCoreApplication::translate("supernovaeDialog", "Historical Supernovae Plug-in Configuration", nullptr));
        updatesGroup->setTitle(QCoreApplication::translate("supernovaeDialog", "Update catalog from Internet", nullptr));
        internetUpdatesCheckbox->setText(QCoreApplication::translate("supernovaeDialog", "Update from Internet sources", nullptr));
        lastUpdateLabel->setText(QCoreApplication::translate("supernovaeDialog", "Last update:", nullptr));
        updateFrequencyLabel->setText(QCoreApplication::translate("supernovaeDialog", "Update frequency (days):", nullptr));
        nextUpdateLabel->setText(QCoreApplication::translate("supernovaeDialog", "[next update info]", nullptr));
        updateButton->setText(QCoreApplication::translate("supernovaeDialog", "Update now", nullptr));
        restoreDefaultsButton->setText(QCoreApplication::translate("supernovaeDialog", "Restore defaults", nullptr));
        saveSettingsButton->setText(QCoreApplication::translate("supernovaeDialog", "Save settings", nullptr));
        tabs->setTabText(tabs->indexOf(settingsTab), QCoreApplication::translate("supernovaeDialog", "Settings", nullptr));
        tabs->setTabText(tabs->indexOf(aboutTab), QCoreApplication::translate("supernovaeDialog", "About", "tab in plugin windows"));
    } // retranslateUi

};

namespace Ui {
    class supernovaeDialog: public Ui_supernovaeDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SUPERNOVAEDIALOG_H
