/****************************************************************************
** Meta object code from reading C++ file 'Nebula.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../stellarium/src/core/modules/Nebula.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'Nebula.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6NebulaE_t {};
} // unnamed namespace

template <> constexpr inline auto Nebula::qt_create_metaobjectdata<qt_meta_tag_ZN6NebulaE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Nebula",
        "CatalogGroup",
        "CatalogGroupFlags",
        "CatNone",
        "CatNGC",
        "CatIC",
        "CatM",
        "CatC",
        "CatB",
        "CatSh2",
        "CatLBN",
        "CatLDN",
        "CatRCW",
        "CatVdB",
        "CatCr",
        "CatMel",
        "CatPGC",
        "CatUGC",
        "CatCed",
        "CatArp",
        "CatVV",
        "CatPK",
        "CatPNG",
        "CatSNRG",
        "CatACO",
        "CatHCG",
        "CatESO",
        "CatVdBH",
        "CatDWB",
        "CatTr",
        "CatSt",
        "CatRu",
        "CatVdBHa",
        "CatOther",
        "CatAll",
        "TypeGroup",
        "TypeGroupFlags",
        "TypeNone",
        "TypeGalaxies",
        "TypeActiveGalaxies",
        "TypeInteractingGalaxies",
        "TypeOpenStarClusters",
        "TypeGlobularStarClusters",
        "TypeHydrogenRegions",
        "TypeBrightNebulae",
        "TypeDarkNebulae",
        "TypePlanetaryNebulae",
        "TypeSupernovaRemnants",
        "TypeGalaxyClusters",
        "TypeOther",
        "TypeAll",
        "NebulaType",
        "NebGx",
        "NebAGx",
        "NebRGx",
        "NebIGx",
        "NebQSO",
        "NebCl",
        "NebOc",
        "NebGc",
        "NebSA",
        "NebSC",
        "NebN",
        "NebPn",
        "NebDn",
        "NebRn",
        "NebBn",
        "NebEn",
        "NebCn",
        "NebHII",
        "NebSNR",
        "NebISM",
        "NebEMO",
        "NebBLL",
        "NebBLA",
        "NebMolCld",
        "NebYSO",
        "NebPossQSO",
        "NebPossPN",
        "NebPPN",
        "NebStar",
        "NebSymbioticStar",
        "NebEmissionLineStar",
        "NebSNC",
        "NebSNRC",
        "NebGxCl",
        "NebPartOfGx",
        "NebRegion",
        "NebUnknown"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // flag 'CatalogGroup'
        QtMocHelpers::EnumData<CatalogGroup>(1, 2, QMC::EnumIsFlag).add({
            {    3, CatalogGroupFlags::CatNone },
            {    4, CatalogGroupFlags::CatNGC },
            {    5, CatalogGroupFlags::CatIC },
            {    6, CatalogGroupFlags::CatM },
            {    7, CatalogGroupFlags::CatC },
            {    8, CatalogGroupFlags::CatB },
            {    9, CatalogGroupFlags::CatSh2 },
            {   10, CatalogGroupFlags::CatLBN },
            {   11, CatalogGroupFlags::CatLDN },
            {   12, CatalogGroupFlags::CatRCW },
            {   13, CatalogGroupFlags::CatVdB },
            {   14, CatalogGroupFlags::CatCr },
            {   15, CatalogGroupFlags::CatMel },
            {   16, CatalogGroupFlags::CatPGC },
            {   17, CatalogGroupFlags::CatUGC },
            {   18, CatalogGroupFlags::CatCed },
            {   19, CatalogGroupFlags::CatArp },
            {   20, CatalogGroupFlags::CatVV },
            {   21, CatalogGroupFlags::CatPK },
            {   22, CatalogGroupFlags::CatPNG },
            {   23, CatalogGroupFlags::CatSNRG },
            {   24, CatalogGroupFlags::CatACO },
            {   25, CatalogGroupFlags::CatHCG },
            {   26, CatalogGroupFlags::CatESO },
            {   27, CatalogGroupFlags::CatVdBH },
            {   28, CatalogGroupFlags::CatDWB },
            {   29, CatalogGroupFlags::CatTr },
            {   30, CatalogGroupFlags::CatSt },
            {   31, CatalogGroupFlags::CatRu },
            {   32, CatalogGroupFlags::CatVdBHa },
            {   33, CatalogGroupFlags::CatOther },
            {   34, CatalogGroupFlags::CatAll },
        }),
        // flag 'TypeGroup'
        QtMocHelpers::EnumData<TypeGroup>(35, 36, QMC::EnumIsFlag).add({
            {   37, TypeGroupFlags::TypeNone },
            {   38, TypeGroupFlags::TypeGalaxies },
            {   39, TypeGroupFlags::TypeActiveGalaxies },
            {   40, TypeGroupFlags::TypeInteractingGalaxies },
            {   41, TypeGroupFlags::TypeOpenStarClusters },
            {   42, TypeGroupFlags::TypeGlobularStarClusters },
            {   43, TypeGroupFlags::TypeHydrogenRegions },
            {   44, TypeGroupFlags::TypeBrightNebulae },
            {   45, TypeGroupFlags::TypeDarkNebulae },
            {   46, TypeGroupFlags::TypePlanetaryNebulae },
            {   47, TypeGroupFlags::TypeSupernovaRemnants },
            {   48, TypeGroupFlags::TypeGalaxyClusters },
            {   49, TypeGroupFlags::TypeOther },
            {   50, TypeGroupFlags::TypeAll },
        }),
        // enum 'NebulaType'
        QtMocHelpers::EnumData<enum NebulaType>(51, 51, QMC::EnumFlags{}).add({
            {   52, NebulaType::NebGx },
            {   53, NebulaType::NebAGx },
            {   54, NebulaType::NebRGx },
            {   55, NebulaType::NebIGx },
            {   56, NebulaType::NebQSO },
            {   57, NebulaType::NebCl },
            {   58, NebulaType::NebOc },
            {   59, NebulaType::NebGc },
            {   60, NebulaType::NebSA },
            {   61, NebulaType::NebSC },
            {   62, NebulaType::NebN },
            {   63, NebulaType::NebPn },
            {   64, NebulaType::NebDn },
            {   65, NebulaType::NebRn },
            {   66, NebulaType::NebBn },
            {   67, NebulaType::NebEn },
            {   68, NebulaType::NebCn },
            {   69, NebulaType::NebHII },
            {   70, NebulaType::NebSNR },
            {   71, NebulaType::NebISM },
            {   72, NebulaType::NebEMO },
            {   73, NebulaType::NebBLL },
            {   74, NebulaType::NebBLA },
            {   75, NebulaType::NebMolCld },
            {   76, NebulaType::NebYSO },
            {   77, NebulaType::NebPossQSO },
            {   78, NebulaType::NebPossPN },
            {   79, NebulaType::NebPPN },
            {   80, NebulaType::NebStar },
            {   81, NebulaType::NebSymbioticStar },
            {   82, NebulaType::NebEmissionLineStar },
            {   83, NebulaType::NebSNC },
            {   84, NebulaType::NebSNRC },
            {   85, NebulaType::NebGxCl },
            {   86, NebulaType::NebPartOfGx },
            {   87, NebulaType::NebRegion },
            {   88, NebulaType::NebUnknown },
        }),
    };
    return QtMocHelpers::metaObjectData<Nebula, qt_meta_tag_ZN6NebulaE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Nebula::staticMetaObject = { {
    QtPrivate::MetaObjectForType<StelObject>::value,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6NebulaE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6NebulaE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6NebulaE_t>.metaTypes,
    nullptr
} };

QT_WARNING_POP
