/********************************************************************************
** Form generated from reading UI file 'mosaicCameraDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MOSAICCAMERADIALOG_H
#define UI_MOSAICCAMERADIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTextBrowser>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "AngleSpinBox.hpp"
#include "Dialog.hpp"

QT_BEGIN_NAMESPACE

class Ui_mosaicCameraDialog
{
public:
    QVBoxLayout *verticalLayout_2;
    TitleBar *titleBar;
    QTabWidget *tabs;
    QWidget *generalTab;
    QVBoxLayout *verticalLayout;
    QGroupBox *groupBoxMosaicCamera;
    QVBoxLayout *verticalLayout_4;
    QCheckBox *checkBoxShowButton;
    QSpacerItem *verticalSpacer;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *horizontalSpacer;
    QPushButton *pushButtonRestoreDefaults;
    QPushButton *pushButtonSaveSettings;
    QWidget *CameraTab;
    QHBoxLayout *horizontalLayout1;
    QListWidget *cameraListWidget;
    QVBoxLayout *verticalLayout_3;
    QFormLayout *formLayout;
    QLabel *RALabel;
    AngleSpinBox *RASpinBox;
    QLabel *DecLabel;
    AngleSpinBox *DecSpinBox;
    QLabel *RotationLabel;
    AngleSpinBox *RotationSpinBox;
    QLabel *visibleLabel;
    QCheckBox *visibleCheckBox;
    QTextBrowser *cameraTextBrowser;
    QPushButton *setViewToCameraButton;
    QPushButton *setMosaicToViewButton;
    QPushButton *setMosaicToObjectButton;
    QWidget *aboutTab;
    QVBoxLayout *verticalLayout_11;
    QTextBrowser *aboutTextBrowser;

    void setupUi(QWidget *mosaicCameraDialog)
    {
        if (mosaicCameraDialog->objectName().isEmpty())
            mosaicCameraDialog->setObjectName("mosaicCameraDialog");
        mosaicCameraDialog->resize(530, 400);
        verticalLayout_2 = new QVBoxLayout(mosaicCameraDialog);
        verticalLayout_2->setSpacing(0);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        titleBar = new TitleBar(mosaicCameraDialog);
        titleBar->setObjectName("titleBar");

        verticalLayout_2->addWidget(titleBar);

        tabs = new QTabWidget(mosaicCameraDialog);
        tabs->setObjectName("tabs");
        tabs->setDocumentMode(false);
        generalTab = new QWidget();
        generalTab->setObjectName("generalTab");
        verticalLayout = new QVBoxLayout(generalTab);
        verticalLayout->setObjectName("verticalLayout");
        groupBoxMosaicCamera = new QGroupBox(generalTab);
        groupBoxMosaicCamera->setObjectName("groupBoxMosaicCamera");
        verticalLayout_4 = new QVBoxLayout(groupBoxMosaicCamera);
        verticalLayout_4->setObjectName("verticalLayout_4");
        checkBoxShowButton = new QCheckBox(groupBoxMosaicCamera);
        checkBoxShowButton->setObjectName("checkBoxShowButton");

        verticalLayout_4->addWidget(checkBoxShowButton);


        verticalLayout->addWidget(groupBoxMosaicCamera);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        pushButtonRestoreDefaults = new QPushButton(generalTab);
        pushButtonRestoreDefaults->setObjectName("pushButtonRestoreDefaults");

        horizontalLayout->addWidget(pushButtonRestoreDefaults);

        pushButtonSaveSettings = new QPushButton(generalTab);
        pushButtonSaveSettings->setObjectName("pushButtonSaveSettings");

        horizontalLayout->addWidget(pushButtonSaveSettings);


        verticalLayout->addLayout(horizontalLayout);

        tabs->addTab(generalTab, QString());
        CameraTab = new QWidget();
        CameraTab->setObjectName("CameraTab");
        horizontalLayout1 = new QHBoxLayout(CameraTab);
        horizontalLayout1->setObjectName("horizontalLayout1");
        cameraListWidget = new QListWidget(CameraTab);
        cameraListWidget->setObjectName("cameraListWidget");

        horizontalLayout1->addWidget(cameraListWidget);

        verticalLayout_3 = new QVBoxLayout();
        verticalLayout_3->setObjectName("verticalLayout_3");
        formLayout = new QFormLayout();
        formLayout->setObjectName("formLayout");
        RALabel = new QLabel(CameraTab);
        RALabel->setObjectName("RALabel");

        formLayout->setWidget(0, QFormLayout::ItemRole::LabelRole, RALabel);

        RASpinBox = new AngleSpinBox(CameraTab);
        RASpinBox->setObjectName("RASpinBox");
        RASpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        RASpinBox->setMinimum(-360);
        RASpinBox->setMaximum(360);

        formLayout->setWidget(0, QFormLayout::ItemRole::FieldRole, RASpinBox);

        DecLabel = new QLabel(CameraTab);
        DecLabel->setObjectName("DecLabel");

        formLayout->setWidget(1, QFormLayout::ItemRole::LabelRole, DecLabel);

        DecSpinBox = new AngleSpinBox(CameraTab);
        DecSpinBox->setObjectName("DecSpinBox");
        DecSpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        DecSpinBox->setMinimum(-90);
        DecSpinBox->setMaximum(90);

        formLayout->setWidget(1, QFormLayout::ItemRole::FieldRole, DecSpinBox);

        RotationLabel = new QLabel(CameraTab);
        RotationLabel->setObjectName("RotationLabel");

        formLayout->setWidget(2, QFormLayout::ItemRole::LabelRole, RotationLabel);

        RotationSpinBox = new AngleSpinBox(CameraTab);
        RotationSpinBox->setObjectName("RotationSpinBox");
        RotationSpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        RotationSpinBox->setMinimum(-90);
        RotationSpinBox->setMaximum(90);

        formLayout->setWidget(2, QFormLayout::ItemRole::FieldRole, RotationSpinBox);

        visibleLabel = new QLabel(CameraTab);
        visibleLabel->setObjectName("visibleLabel");

        formLayout->setWidget(3, QFormLayout::ItemRole::LabelRole, visibleLabel);

        visibleCheckBox = new QCheckBox(CameraTab);
        visibleCheckBox->setObjectName("visibleCheckBox");

        formLayout->setWidget(3, QFormLayout::ItemRole::FieldRole, visibleCheckBox);

        cameraTextBrowser = new QTextBrowser(CameraTab);
        cameraTextBrowser->setObjectName("cameraTextBrowser");
        cameraTextBrowser->setOpenExternalLinks(true);

        formLayout->setWidget(4, QFormLayout::ItemRole::SpanningRole, cameraTextBrowser);


        verticalLayout_3->addLayout(formLayout);

        setViewToCameraButton = new QPushButton(CameraTab);
        setViewToCameraButton->setObjectName("setViewToCameraButton");

        verticalLayout_3->addWidget(setViewToCameraButton);

        setMosaicToViewButton = new QPushButton(CameraTab);
        setMosaicToViewButton->setObjectName("setMosaicToViewButton");

        verticalLayout_3->addWidget(setMosaicToViewButton);

        setMosaicToObjectButton = new QPushButton(CameraTab);
        setMosaicToObjectButton->setObjectName("setMosaicToObjectButton");

        verticalLayout_3->addWidget(setMosaicToObjectButton);

        verticalLayout_3->setStretch(0, 1);
        verticalLayout_3->setStretch(1, 1);

        horizontalLayout1->addLayout(verticalLayout_3);

        tabs->addTab(CameraTab, QString());
        aboutTab = new QWidget();
        aboutTab->setObjectName("aboutTab");
        verticalLayout_11 = new QVBoxLayout(aboutTab);
        verticalLayout_11->setSpacing(9);
        verticalLayout_11->setObjectName("verticalLayout_11");
        aboutTextBrowser = new QTextBrowser(aboutTab);
        aboutTextBrowser->setObjectName("aboutTextBrowser");
        aboutTextBrowser->setOpenExternalLinks(true);
        aboutTextBrowser->setOpenLinks(true);

        verticalLayout_11->addWidget(aboutTextBrowser);

        tabs->addTab(aboutTab, QString());

        verticalLayout_2->addWidget(tabs);


        retranslateUi(mosaicCameraDialog);

        tabs->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(mosaicCameraDialog);
    } // setupUi

    void retranslateUi(QWidget *mosaicCameraDialog)
    {
        mosaicCameraDialog->setWindowTitle(QCoreApplication::translate("mosaicCameraDialog", "Mosaic Camera Configuration", nullptr));
        titleBar->setProperty("title", QVariant(QCoreApplication::translate("mosaicCameraDialog", "Mosaic Camera", nullptr)));
        groupBoxMosaicCamera->setTitle(QCoreApplication::translate("mosaicCameraDialog", "Mosaic Camera", nullptr));
        checkBoxShowButton->setText(QCoreApplication::translate("mosaicCameraDialog", "Show plug-in button on toolbar", nullptr));
        pushButtonRestoreDefaults->setText(QCoreApplication::translate("mosaicCameraDialog", "Restore Defaults", nullptr));
        pushButtonSaveSettings->setText(QCoreApplication::translate("mosaicCameraDialog", "Save settings", nullptr));
        tabs->setTabText(tabs->indexOf(generalTab), QCoreApplication::translate("mosaicCameraDialog", "Mosaic Camera", nullptr));
        RALabel->setText(QCoreApplication::translate("mosaicCameraDialog", "Right Ascension:", nullptr));
        DecLabel->setText(QCoreApplication::translate("mosaicCameraDialog", "Declination:", nullptr));
        RotationLabel->setText(QCoreApplication::translate("mosaicCameraDialog", "Rotation:", nullptr));
        visibleLabel->setText(QCoreApplication::translate("mosaicCameraDialog", "Visible:", nullptr));
        setViewToCameraButton->setText(QCoreApplication::translate("mosaicCameraDialog", "Center view on camera", nullptr));
        setMosaicToViewButton->setText(QCoreApplication::translate("mosaicCameraDialog", "Center camera on current view", nullptr));
        setMosaicToObjectButton->setText(QCoreApplication::translate("mosaicCameraDialog", "Center camera on current object", nullptr));
        tabs->setTabText(tabs->indexOf(CameraTab), QCoreApplication::translate("mosaicCameraDialog", "Cameras", nullptr));
        tabs->setTabText(tabs->indexOf(aboutTab), QCoreApplication::translate("mosaicCameraDialog", "About", "tab in plugin windows"));
    } // retranslateUi

};

namespace Ui {
    class mosaicCameraDialog: public Ui_mosaicCameraDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_MOSAICCAMERADIALOG_H
