/********************************************************************************
** Form generated from reading UI file 'telescopeConfigurationDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TELESCOPECONFIGURATIONDIALOG_H
#define UI_TELESCOPECONFIGURATIONDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QScrollArea>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "Dialog.hpp"
#include "gui/StelCloseButton.hpp"

QT_BEGIN_NAMESPACE

class Ui_telescopeConfigurationDialog
{
public:
    QVBoxLayout *_2;
    TitleBar *titleBar;
    QGroupBox *groupBoxType;
    QVBoxLayout *verticalLayout;
    QRadioButton *radioButtonTelescopeLocal;
    QRadioButton *radioButtonTelescopeConnection;
    QRadioButton *radioButtonTelescopeRTS2;
    QRadioButton *radioButtonTelescopeINDI;
    QRadioButton *radioButtonTelescopeASCOM;
    QRadioButton *radioButtonTelescopeVirtual;
    QScrollArea *scrollArea;
    QWidget *scrollAreaWidgetContents;
    QVBoxLayout *verticalLayoutScrollArea;
    QGroupBox *groupBoxTelescopeProperties;
    QVBoxLayout *verticalLayoutTelescope;
    QLabel *labelTelescopeName;
    QLineEdit *lineEditTelescopeName;
    QHBoxLayout *horizontalLayoutDelay;
    QLabel *labelConnectionDelay;
    QDoubleSpinBox *doubleSpinBoxTelescopeDelay;
    QLabel *labelEquinox;
    QFrame *frameEquinox;
    QHBoxLayout *horizontalLayoutEpoch;
    QRadioButton *radioButtonJ2000;
    QRadioButton *radioButtonJNow;
    QCheckBox *checkBoxConnectAtStartup;
    QVBoxLayout *INDILayout;
    QVBoxLayout *ASCOMLayout;
    QGroupBox *groupBoxDeviceSettings;
    QVBoxLayout *verticalLayoutDevice;
    QHBoxLayout *horizontalLayoutSerialPort;
    QLabel *labelSerialPort;
    QComboBox *comboSerialPort;
    QLabel *labelDeviceModel;
    QComboBox *comboBoxDeviceModel;
    QLabel *labelDeviceModelDescription;
    QGroupBox *groupBoxConnectionSettings;
    QGridLayout *gridLayoutConnectionSettings;
    QLabel *labelHost;
    QLineEdit *lineEditHostName;
    QLabel *labelColon;
    QSpinBox *spinBoxTCPPort;
    QLabel *labelTCPPort;
    QGroupBox *groupBoxRTS2Settings;
    QGridLayout *gridLayoutRTS2Settings;
    QLabel *labelRTS2Host;
    QLabel *labelRTS2Username;
    QLabel *labelRTS2Password;
    QLineEdit *lineEditRTS2Username;
    QLineEdit *lineEditRTS2Password;
    QLineEdit *lineEditRTS2Url;
    QHBoxLayout *horizontalLayoutRTSRefresh;
    QLabel *labelRTS2Refresh;
    QDoubleSpinBox *doubleSpinBoxRTS2Refresh;
    QGroupBox *groupBoxUserInterface;
    QVBoxLayout *verticalLayoutUserInterface;
    QCheckBox *checkBoxCircles;
    QLabel *labelCircles;
    QLineEdit *lineEditCircleList;
    QFrame *frameOkCancelButtons;
    QHBoxLayout *horizontalLayout;
    QPushButton *pushButtonSave;
    QPushButton *pushButtonDiscard;

    void setupUi(QWidget *telescopeConfigurationDialog)
    {
        if (telescopeConfigurationDialog->objectName().isEmpty())
            telescopeConfigurationDialog->setObjectName("telescopeConfigurationDialog");
        telescopeConfigurationDialog->setWindowModality(Qt::WindowModal);
        telescopeConfigurationDialog->resize(535, 489);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(telescopeConfigurationDialog->sizePolicy().hasHeightForWidth());
        telescopeConfigurationDialog->setSizePolicy(sizePolicy);
        telescopeConfigurationDialog->setMinimumSize(QSize(400, 480));
        _2 = new QVBoxLayout(telescopeConfigurationDialog);
        _2->setSpacing(0);
        _2->setObjectName("_2");
        _2->setContentsMargins(0, 0, 0, 0);
        titleBar = new TitleBar(telescopeConfigurationDialog);
        titleBar->setObjectName("titleBar");

        _2->addWidget(titleBar);

        groupBoxType = new QGroupBox(telescopeConfigurationDialog);
        groupBoxType->setObjectName("groupBoxType");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(groupBoxType->sizePolicy().hasHeightForWidth());
        groupBoxType->setSizePolicy(sizePolicy1);
        verticalLayout = new QVBoxLayout(groupBoxType);
        verticalLayout->setSpacing(6);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        radioButtonTelescopeLocal = new QRadioButton(groupBoxType);
        radioButtonTelescopeLocal->setObjectName("radioButtonTelescopeLocal");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(radioButtonTelescopeLocal->sizePolicy().hasHeightForWidth());
        radioButtonTelescopeLocal->setSizePolicy(sizePolicy2);
        radioButtonTelescopeLocal->setMinimumSize(QSize(0, 20));
        radioButtonTelescopeLocal->setChecked(true);

        verticalLayout->addWidget(radioButtonTelescopeLocal);

        radioButtonTelescopeConnection = new QRadioButton(groupBoxType);
        radioButtonTelescopeConnection->setObjectName("radioButtonTelescopeConnection");
        sizePolicy2.setHeightForWidth(radioButtonTelescopeConnection->sizePolicy().hasHeightForWidth());
        radioButtonTelescopeConnection->setSizePolicy(sizePolicy2);
        radioButtonTelescopeConnection->setMinimumSize(QSize(0, 20));

        verticalLayout->addWidget(radioButtonTelescopeConnection);

        radioButtonTelescopeRTS2 = new QRadioButton(groupBoxType);
        radioButtonTelescopeRTS2->setObjectName("radioButtonTelescopeRTS2");
        sizePolicy2.setHeightForWidth(radioButtonTelescopeRTS2->sizePolicy().hasHeightForWidth());
        radioButtonTelescopeRTS2->setSizePolicy(sizePolicy2);
        radioButtonTelescopeRTS2->setMinimumSize(QSize(0, 20));

        verticalLayout->addWidget(radioButtonTelescopeRTS2);

        radioButtonTelescopeINDI = new QRadioButton(groupBoxType);
        radioButtonTelescopeINDI->setObjectName("radioButtonTelescopeINDI");
        radioButtonTelescopeINDI->setText(QString::fromUtf8("INDI/INDIGO"));

        verticalLayout->addWidget(radioButtonTelescopeINDI);

        radioButtonTelescopeASCOM = new QRadioButton(groupBoxType);
        radioButtonTelescopeASCOM->setObjectName("radioButtonTelescopeASCOM");
        radioButtonTelescopeASCOM->setText(QString::fromUtf8("ASCOM"));

        verticalLayout->addWidget(radioButtonTelescopeASCOM);

        radioButtonTelescopeVirtual = new QRadioButton(groupBoxType);
        radioButtonTelescopeVirtual->setObjectName("radioButtonTelescopeVirtual");
        sizePolicy2.setHeightForWidth(radioButtonTelescopeVirtual->sizePolicy().hasHeightForWidth());
        radioButtonTelescopeVirtual->setSizePolicy(sizePolicy2);
        radioButtonTelescopeVirtual->setMinimumSize(QSize(0, 20));

        verticalLayout->addWidget(radioButtonTelescopeVirtual);


        _2->addWidget(groupBoxType);

        scrollArea = new QScrollArea(telescopeConfigurationDialog);
        scrollArea->setObjectName("scrollArea");
        scrollArea->setWidgetResizable(true);
        scrollAreaWidgetContents = new QWidget();
        scrollAreaWidgetContents->setObjectName("scrollAreaWidgetContents");
        scrollAreaWidgetContents->setGeometry(QRect(0, 0, 519, 733));
        verticalLayoutScrollArea = new QVBoxLayout(scrollAreaWidgetContents);
        verticalLayoutScrollArea->setSpacing(0);
        verticalLayoutScrollArea->setObjectName("verticalLayoutScrollArea");
        verticalLayoutScrollArea->setContentsMargins(0, 0, 0, 0);
        groupBoxTelescopeProperties = new QGroupBox(scrollAreaWidgetContents);
        groupBoxTelescopeProperties->setObjectName("groupBoxTelescopeProperties");
        verticalLayoutTelescope = new QVBoxLayout(groupBoxTelescopeProperties);
        verticalLayoutTelescope->setObjectName("verticalLayoutTelescope");
        verticalLayoutTelescope->setContentsMargins(0, 0, 0, 0);
        labelTelescopeName = new QLabel(groupBoxTelescopeProperties);
        labelTelescopeName->setObjectName("labelTelescopeName");
        sizePolicy2.setHeightForWidth(labelTelescopeName->sizePolicy().hasHeightForWidth());
        labelTelescopeName->setSizePolicy(sizePolicy2);
        labelTelescopeName->setMinimumSize(QSize(0, 24));
        labelTelescopeName->setBaseSize(QSize(110, 24));

        verticalLayoutTelescope->addWidget(labelTelescopeName);

        lineEditTelescopeName = new QLineEdit(groupBoxTelescopeProperties);
        lineEditTelescopeName->setObjectName("lineEditTelescopeName");
        lineEditTelescopeName->setMinimumSize(QSize(0, 24));
        lineEditTelescopeName->setMaxLength(50);

        verticalLayoutTelescope->addWidget(lineEditTelescopeName);

        horizontalLayoutDelay = new QHBoxLayout();
        horizontalLayoutDelay->setObjectName("horizontalLayoutDelay");
        labelConnectionDelay = new QLabel(groupBoxTelescopeProperties);
        labelConnectionDelay->setObjectName("labelConnectionDelay");
        sizePolicy2.setHeightForWidth(labelConnectionDelay->sizePolicy().hasHeightForWidth());
        labelConnectionDelay->setSizePolicy(sizePolicy2);
        labelConnectionDelay->setMinimumSize(QSize(0, 24));

        horizontalLayoutDelay->addWidget(labelConnectionDelay);

        doubleSpinBoxTelescopeDelay = new QDoubleSpinBox(groupBoxTelescopeProperties);
        doubleSpinBoxTelescopeDelay->setObjectName("doubleSpinBoxTelescopeDelay");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(doubleSpinBoxTelescopeDelay->sizePolicy().hasHeightForWidth());
        doubleSpinBoxTelescopeDelay->setSizePolicy(sizePolicy3);
        doubleSpinBoxTelescopeDelay->setMinimumSize(QSize(50, 24));
        doubleSpinBoxTelescopeDelay->setSuffix(QString::fromUtf8("s"));
        doubleSpinBoxTelescopeDelay->setDecimals(2);
        doubleSpinBoxTelescopeDelay->setMinimum(0.100000000000000);
        doubleSpinBoxTelescopeDelay->setMaximum(10.000000000000000);
        doubleSpinBoxTelescopeDelay->setSingleStep(0.100000000000000);
        doubleSpinBoxTelescopeDelay->setValue(0.500000000000000);

        horizontalLayoutDelay->addWidget(doubleSpinBoxTelescopeDelay);


        verticalLayoutTelescope->addLayout(horizontalLayoutDelay);

        labelEquinox = new QLabel(groupBoxTelescopeProperties);
        labelEquinox->setObjectName("labelEquinox");

        verticalLayoutTelescope->addWidget(labelEquinox);

        frameEquinox = new QFrame(groupBoxTelescopeProperties);
        frameEquinox->setObjectName("frameEquinox");
        horizontalLayoutEpoch = new QHBoxLayout(frameEquinox);
        horizontalLayoutEpoch->setObjectName("horizontalLayoutEpoch");
        horizontalLayoutEpoch->setContentsMargins(0, 0, 0, 0);
        radioButtonJ2000 = new QRadioButton(frameEquinox);
        radioButtonJ2000->setObjectName("radioButtonJ2000");

        horizontalLayoutEpoch->addWidget(radioButtonJ2000);

        radioButtonJNow = new QRadioButton(frameEquinox);
        radioButtonJNow->setObjectName("radioButtonJNow");

        horizontalLayoutEpoch->addWidget(radioButtonJNow);


        verticalLayoutTelescope->addWidget(frameEquinox);

        checkBoxConnectAtStartup = new QCheckBox(groupBoxTelescopeProperties);
        checkBoxConnectAtStartup->setObjectName("checkBoxConnectAtStartup");
        checkBoxConnectAtStartup->setMinimumSize(QSize(0, 24));

        verticalLayoutTelescope->addWidget(checkBoxConnectAtStartup);


        verticalLayoutScrollArea->addWidget(groupBoxTelescopeProperties);

        INDILayout = new QVBoxLayout();
        INDILayout->setObjectName("INDILayout");
        INDILayout->setContentsMargins(0, 0, 0, 0);

        verticalLayoutScrollArea->addLayout(INDILayout);

        ASCOMLayout = new QVBoxLayout();
        ASCOMLayout->setObjectName("ASCOMLayout");
        ASCOMLayout->setContentsMargins(0, 0, 0, 0);

        verticalLayoutScrollArea->addLayout(ASCOMLayout);

        groupBoxDeviceSettings = new QGroupBox(scrollAreaWidgetContents);
        groupBoxDeviceSettings->setObjectName("groupBoxDeviceSettings");
        verticalLayoutDevice = new QVBoxLayout(groupBoxDeviceSettings);
        verticalLayoutDevice->setObjectName("verticalLayoutDevice");
        verticalLayoutDevice->setContentsMargins(0, 0, 0, 0);
        horizontalLayoutSerialPort = new QHBoxLayout();
        horizontalLayoutSerialPort->setObjectName("horizontalLayoutSerialPort");
        labelSerialPort = new QLabel(groupBoxDeviceSettings);
        labelSerialPort->setObjectName("labelSerialPort");
        QSizePolicy sizePolicy4(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy4.setHorizontalStretch(0);
        sizePolicy4.setVerticalStretch(0);
        sizePolicy4.setHeightForWidth(labelSerialPort->sizePolicy().hasHeightForWidth());
        labelSerialPort->setSizePolicy(sizePolicy4);
        labelSerialPort->setMinimumSize(QSize(0, 24));

        horizontalLayoutSerialPort->addWidget(labelSerialPort);

        comboSerialPort = new QComboBox(groupBoxDeviceSettings);
        comboSerialPort->setObjectName("comboSerialPort");
        comboSerialPort->setEditable(true);

        horizontalLayoutSerialPort->addWidget(comboSerialPort);


        verticalLayoutDevice->addLayout(horizontalLayoutSerialPort);

        labelDeviceModel = new QLabel(groupBoxDeviceSettings);
        labelDeviceModel->setObjectName("labelDeviceModel");
        sizePolicy2.setHeightForWidth(labelDeviceModel->sizePolicy().hasHeightForWidth());
        labelDeviceModel->setSizePolicy(sizePolicy2);
        labelDeviceModel->setMinimumSize(QSize(0, 24));

        verticalLayoutDevice->addWidget(labelDeviceModel);

        comboBoxDeviceModel = new QComboBox(groupBoxDeviceSettings);
        comboBoxDeviceModel->setObjectName("comboBoxDeviceModel");
        sizePolicy2.setHeightForWidth(comboBoxDeviceModel->sizePolicy().hasHeightForWidth());
        comboBoxDeviceModel->setSizePolicy(sizePolicy2);
        comboBoxDeviceModel->setMinimumSize(QSize(0, 24));

        verticalLayoutDevice->addWidget(comboBoxDeviceModel);

        labelDeviceModelDescription = new QLabel(groupBoxDeviceSettings);
        labelDeviceModelDescription->setObjectName("labelDeviceModelDescription");
        QSizePolicy sizePolicy5(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy5.setHorizontalStretch(0);
        sizePolicy5.setVerticalStretch(0);
        sizePolicy5.setHeightForWidth(labelDeviceModelDescription->sizePolicy().hasHeightForWidth());
        labelDeviceModelDescription->setSizePolicy(sizePolicy5);
        labelDeviceModelDescription->setMinimumSize(QSize(0, 0));
        labelDeviceModelDescription->setContextMenuPolicy(Qt::DefaultContextMenu);
        labelDeviceModelDescription->setFrameShape(QFrame::NoFrame);
        labelDeviceModelDescription->setTextFormat(Qt::PlainText);
        labelDeviceModelDescription->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
        labelDeviceModelDescription->setWordWrap(true);
        labelDeviceModelDescription->setTextInteractionFlags(Qt::TextSelectableByKeyboard|Qt::TextSelectableByMouse);

        verticalLayoutDevice->addWidget(labelDeviceModelDescription);


        verticalLayoutScrollArea->addWidget(groupBoxDeviceSettings);

        groupBoxConnectionSettings = new QGroupBox(scrollAreaWidgetContents);
        groupBoxConnectionSettings->setObjectName("groupBoxConnectionSettings");
        gridLayoutConnectionSettings = new QGridLayout(groupBoxConnectionSettings);
        gridLayoutConnectionSettings->setObjectName("gridLayoutConnectionSettings");
        gridLayoutConnectionSettings->setContentsMargins(0, 0, 0, 0);
        labelHost = new QLabel(groupBoxConnectionSettings);
        labelHost->setObjectName("labelHost");
        labelHost->setEnabled(true);

        gridLayoutConnectionSettings->addWidget(labelHost, 0, 0, 1, 1);

        lineEditHostName = new QLineEdit(groupBoxConnectionSettings);
        lineEditHostName->setObjectName("lineEditHostName");
        lineEditHostName->setEnabled(true);
        lineEditHostName->setText(QString::fromUtf8("localhost"));
        lineEditHostName->setMaxLength(255);

        gridLayoutConnectionSettings->addWidget(lineEditHostName, 1, 0, 1, 1);

        labelColon = new QLabel(groupBoxConnectionSettings);
        labelColon->setObjectName("labelColon");
        labelColon->setText(QString::fromUtf8(":"));

        gridLayoutConnectionSettings->addWidget(labelColon, 1, 1, 1, 1);

        spinBoxTCPPort = new QSpinBox(groupBoxConnectionSettings);
        spinBoxTCPPort->setObjectName("spinBoxTCPPort");
        sizePolicy4.setHeightForWidth(spinBoxTCPPort->sizePolicy().hasHeightForWidth());
        spinBoxTCPPort->setSizePolicy(sizePolicy4);
        spinBoxTCPPort->setMinimum(1024);
        spinBoxTCPPort->setMaximum(65535);

        gridLayoutConnectionSettings->addWidget(spinBoxTCPPort, 1, 2, 1, 1);

        labelTCPPort = new QLabel(groupBoxConnectionSettings);
        labelTCPPort->setObjectName("labelTCPPort");
        labelTCPPort->setEnabled(true);

        gridLayoutConnectionSettings->addWidget(labelTCPPort, 0, 2, 1, 1);


        verticalLayoutScrollArea->addWidget(groupBoxConnectionSettings);

        groupBoxRTS2Settings = new QGroupBox(scrollAreaWidgetContents);
        groupBoxRTS2Settings->setObjectName("groupBoxRTS2Settings");
        gridLayoutRTS2Settings = new QGridLayout(groupBoxRTS2Settings);
        gridLayoutRTS2Settings->setObjectName("gridLayoutRTS2Settings");
        gridLayoutRTS2Settings->setContentsMargins(0, 0, 0, 0);
        labelRTS2Host = new QLabel(groupBoxRTS2Settings);
        labelRTS2Host->setObjectName("labelRTS2Host");

        gridLayoutRTS2Settings->addWidget(labelRTS2Host, 0, 0, 1, 1);

        labelRTS2Username = new QLabel(groupBoxRTS2Settings);
        labelRTS2Username->setObjectName("labelRTS2Username");

        gridLayoutRTS2Settings->addWidget(labelRTS2Username, 2, 0, 1, 1);

        labelRTS2Password = new QLabel(groupBoxRTS2Settings);
        labelRTS2Password->setObjectName("labelRTS2Password");

        gridLayoutRTS2Settings->addWidget(labelRTS2Password, 2, 1, 1, 1);

        lineEditRTS2Username = new QLineEdit(groupBoxRTS2Settings);
        lineEditRTS2Username->setObjectName("lineEditRTS2Username");
        lineEditRTS2Username->setText(QString::fromUtf8(""));
        lineEditRTS2Username->setMaxLength(50);

        gridLayoutRTS2Settings->addWidget(lineEditRTS2Username, 3, 0, 1, 1);

        lineEditRTS2Password = new QLineEdit(groupBoxRTS2Settings);
        lineEditRTS2Password->setObjectName("lineEditRTS2Password");
        lineEditRTS2Password->setText(QString::fromUtf8(""));
        lineEditRTS2Password->setMaxLength(50);
        lineEditRTS2Password->setEchoMode(QLineEdit::Password);

        gridLayoutRTS2Settings->addWidget(lineEditRTS2Password, 3, 1, 1, 1);

        lineEditRTS2Url = new QLineEdit(groupBoxRTS2Settings);
        lineEditRTS2Url->setObjectName("lineEditRTS2Url");
        lineEditRTS2Url->setText(QString::fromUtf8("localhost"));
        lineEditRTS2Url->setMaxLength(255);

        gridLayoutRTS2Settings->addWidget(lineEditRTS2Url, 1, 0, 1, 2);

        horizontalLayoutRTSRefresh = new QHBoxLayout();
        horizontalLayoutRTSRefresh->setObjectName("horizontalLayoutRTSRefresh");
        labelRTS2Refresh = new QLabel(groupBoxRTS2Settings);
        labelRTS2Refresh->setObjectName("labelRTS2Refresh");
        sizePolicy2.setHeightForWidth(labelRTS2Refresh->sizePolicy().hasHeightForWidth());
        labelRTS2Refresh->setSizePolicy(sizePolicy2);
        labelRTS2Refresh->setMinimumSize(QSize(0, 24));

        horizontalLayoutRTSRefresh->addWidget(labelRTS2Refresh);

        doubleSpinBoxRTS2Refresh = new QDoubleSpinBox(groupBoxRTS2Settings);
        doubleSpinBoxRTS2Refresh->setObjectName("doubleSpinBoxRTS2Refresh");
        sizePolicy3.setHeightForWidth(doubleSpinBoxRTS2Refresh->sizePolicy().hasHeightForWidth());
        doubleSpinBoxRTS2Refresh->setSizePolicy(sizePolicy3);
        doubleSpinBoxRTS2Refresh->setMinimumSize(QSize(50, 24));
        doubleSpinBoxRTS2Refresh->setSuffix(QString::fromUtf8("s"));
        doubleSpinBoxRTS2Refresh->setDecimals(2);
        doubleSpinBoxRTS2Refresh->setMinimum(0.100000000000000);
        doubleSpinBoxRTS2Refresh->setMaximum(10.000000000000000);
        doubleSpinBoxRTS2Refresh->setSingleStep(0.100000000000000);
        doubleSpinBoxRTS2Refresh->setValue(0.500000000000000);

        horizontalLayoutRTSRefresh->addWidget(doubleSpinBoxRTS2Refresh);


        gridLayoutRTS2Settings->addLayout(horizontalLayoutRTSRefresh, 4, 0, 1, 2);


        verticalLayoutScrollArea->addWidget(groupBoxRTS2Settings);

        groupBoxUserInterface = new QGroupBox(scrollAreaWidgetContents);
        groupBoxUserInterface->setObjectName("groupBoxUserInterface");
        verticalLayoutUserInterface = new QVBoxLayout(groupBoxUserInterface);
        verticalLayoutUserInterface->setObjectName("verticalLayoutUserInterface");
        verticalLayoutUserInterface->setContentsMargins(0, 0, 0, 0);
        checkBoxCircles = new QCheckBox(groupBoxUserInterface);
        checkBoxCircles->setObjectName("checkBoxCircles");

        verticalLayoutUserInterface->addWidget(checkBoxCircles);

        labelCircles = new QLabel(groupBoxUserInterface);
        labelCircles->setObjectName("labelCircles");
        labelCircles->setEnabled(false);

        verticalLayoutUserInterface->addWidget(labelCircles);

        lineEditCircleList = new QLineEdit(groupBoxUserInterface);
        lineEditCircleList->setObjectName("lineEditCircleList");
        lineEditCircleList->setEnabled(false);

        verticalLayoutUserInterface->addWidget(lineEditCircleList);


        verticalLayoutScrollArea->addWidget(groupBoxUserInterface);

        scrollArea->setWidget(scrollAreaWidgetContents);

        _2->addWidget(scrollArea);

        frameOkCancelButtons = new QFrame(telescopeConfigurationDialog);
        frameOkCancelButtons->setObjectName("frameOkCancelButtons");
        horizontalLayout = new QHBoxLayout(frameOkCancelButtons);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(0, 5, 0, 5);
        pushButtonSave = new QPushButton(frameOkCancelButtons);
        pushButtonSave->setObjectName("pushButtonSave");
        sizePolicy.setHeightForWidth(pushButtonSave->sizePolicy().hasHeightForWidth());
        pushButtonSave->setSizePolicy(sizePolicy);
        pushButtonSave->setMinimumSize(QSize(140, 28));
        pushButtonSave->setLayoutDirection(Qt::RightToLeft);

        horizontalLayout->addWidget(pushButtonSave);

        pushButtonDiscard = new QPushButton(frameOkCancelButtons);
        pushButtonDiscard->setObjectName("pushButtonDiscard");
        sizePolicy.setHeightForWidth(pushButtonDiscard->sizePolicy().hasHeightForWidth());
        pushButtonDiscard->setSizePolicy(sizePolicy);
        pushButtonDiscard->setMinimumSize(QSize(140, 28));

        horizontalLayout->addWidget(pushButtonDiscard);


        _2->addWidget(frameOkCancelButtons);

        QWidget::setTabOrder(radioButtonTelescopeLocal, radioButtonTelescopeConnection);
        QWidget::setTabOrder(radioButtonTelescopeConnection, radioButtonTelescopeVirtual);
        QWidget::setTabOrder(radioButtonTelescopeVirtual, scrollArea);
        QWidget::setTabOrder(scrollArea, lineEditTelescopeName);
        QWidget::setTabOrder(lineEditTelescopeName, doubleSpinBoxTelescopeDelay);
        QWidget::setTabOrder(doubleSpinBoxTelescopeDelay, checkBoxConnectAtStartup);
        QWidget::setTabOrder(checkBoxConnectAtStartup, comboBoxDeviceModel);
        QWidget::setTabOrder(comboBoxDeviceModel, lineEditHostName);
        QWidget::setTabOrder(lineEditHostName, spinBoxTCPPort);
        QWidget::setTabOrder(spinBoxTCPPort, checkBoxCircles);
        QWidget::setTabOrder(checkBoxCircles, lineEditCircleList);
        QWidget::setTabOrder(lineEditCircleList, pushButtonDiscard);

        retranslateUi(telescopeConfigurationDialog);
        QObject::connect(checkBoxCircles, &QCheckBox::toggled, labelCircles, &QLabel::setEnabled);
        QObject::connect(checkBoxCircles, &QCheckBox::toggled, lineEditCircleList, &QLineEdit::setEnabled);

        QMetaObject::connectSlotsByName(telescopeConfigurationDialog);
    } // setupUi

    void retranslateUi(QWidget *telescopeConfigurationDialog)
    {
        titleBar->setTitle(QCoreApplication::translate("telescopeConfigurationDialog", "Telescope Configuration", nullptr));
        groupBoxType->setTitle(QCoreApplication::translate("telescopeConfigurationDialog", "Telescope controlled by:", nullptr));
#if QT_CONFIG(tooltip)
        radioButtonTelescopeLocal->setToolTip(QCoreApplication::translate("telescopeConfigurationDialog", "A telescope connected to this computer via a serial port and controlled directly by Stellarium.", nullptr));
#endif // QT_CONFIG(tooltip)
        radioButtonTelescopeLocal->setText(QCoreApplication::translate("telescopeConfigurationDialog", "Stellarium, directly through a serial port", nullptr));
#if QT_CONFIG(tooltip)
        radioButtonTelescopeConnection->setToolTip(QCoreApplication::translate("telescopeConfigurationDialog", "A telescope controlled by an external application, either at this computer or at a remote machine.", nullptr));
#endif // QT_CONFIG(tooltip)
        radioButtonTelescopeConnection->setText(QCoreApplication::translate("telescopeConfigurationDialog", "External software or a remote computer", nullptr));
#if QT_CONFIG(tooltip)
        radioButtonTelescopeRTS2->setToolTip(QCoreApplication::translate("telescopeConfigurationDialog", "Telescope which is accessible through RTS2(.org) JSON API", nullptr));
#endif // QT_CONFIG(tooltip)
        radioButtonTelescopeRTS2->setText(QCoreApplication::translate("telescopeConfigurationDialog", "RTS2 telescope", nullptr));
        radioButtonTelescopeVirtual->setText(QCoreApplication::translate("telescopeConfigurationDialog", "Nothing, just simulate one (a moving reticle)", nullptr));
        groupBoxTelescopeProperties->setTitle(QCoreApplication::translate("telescopeConfigurationDialog", "Telescope properties", nullptr));
        labelTelescopeName->setText(QCoreApplication::translate("telescopeConfigurationDialog", "Name:", nullptr));
        labelConnectionDelay->setText(QCoreApplication::translate("telescopeConfigurationDialog", "Connection delay:", nullptr));
#if QT_CONFIG(tooltip)
        doubleSpinBoxTelescopeDelay->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
        labelEquinox->setText(QCoreApplication::translate("telescopeConfigurationDialog", "Coordinate system:", nullptr));
        radioButtonJ2000->setText(QCoreApplication::translate("telescopeConfigurationDialog", "J2000 (default)", nullptr));
        radioButtonJNow->setText(QCoreApplication::translate("telescopeConfigurationDialog", "Equinox of the date (JNow)", nullptr));
#if QT_CONFIG(tooltip)
        checkBoxConnectAtStartup->setToolTip(QCoreApplication::translate("telescopeConfigurationDialog", "Automatically attempt to start the telescope or establish connection when Stellarium starts", nullptr));
#endif // QT_CONFIG(tooltip)
        checkBoxConnectAtStartup->setText(QCoreApplication::translate("telescopeConfigurationDialog", "Start/connect at startup", nullptr));
        groupBoxDeviceSettings->setTitle(QCoreApplication::translate("telescopeConfigurationDialog", "Device settings", nullptr));
        labelSerialPort->setText(QCoreApplication::translate("telescopeConfigurationDialog", "Serial port:", nullptr));
        labelDeviceModel->setText(QCoreApplication::translate("telescopeConfigurationDialog", "Device model:", nullptr));
        groupBoxConnectionSettings->setTitle(QCoreApplication::translate("telescopeConfigurationDialog", "Connection settings", nullptr));
        labelHost->setText(QCoreApplication::translate("telescopeConfigurationDialog", "Host:", nullptr));
#if QT_CONFIG(tooltip)
        lineEditHostName->setToolTip(QCoreApplication::translate("telescopeConfigurationDialog", "Host name or IPv4 address of the machine that hosts the telescope server", nullptr));
#endif // QT_CONFIG(tooltip)
        labelTCPPort->setText(QCoreApplication::translate("telescopeConfigurationDialog", "TCP port:", nullptr));
        groupBoxRTS2Settings->setTitle(QCoreApplication::translate("telescopeConfigurationDialog", "RTS2 telescope settings", nullptr));
        labelRTS2Host->setText(QCoreApplication::translate("telescopeConfigurationDialog", "URL:", nullptr));
        labelRTS2Username->setText(QCoreApplication::translate("telescopeConfigurationDialog", "Username:", nullptr));
        labelRTS2Password->setText(QCoreApplication::translate("telescopeConfigurationDialog", "Password:", nullptr));
#if QT_CONFIG(tooltip)
        lineEditRTS2Username->setToolTip(QCoreApplication::translate("telescopeConfigurationDialog", "RTS2 username", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        lineEditRTS2Password->setToolTip(QCoreApplication::translate("telescopeConfigurationDialog", "RTS2 password", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        lineEditRTS2Url->setToolTip(QCoreApplication::translate("telescopeConfigurationDialog", "URL of the machine that hosts the RTS2 JSON (httpd) server", nullptr));
#endif // QT_CONFIG(tooltip)
        labelRTS2Refresh->setText(QCoreApplication::translate("telescopeConfigurationDialog", "Refresh every:", nullptr));
#if QT_CONFIG(tooltip)
        doubleSpinBoxRTS2Refresh->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
        groupBoxUserInterface->setTitle(QCoreApplication::translate("telescopeConfigurationDialog", "User interface settings", nullptr));
#if QT_CONFIG(tooltip)
        checkBoxCircles->setToolTip(QCoreApplication::translate("telescopeConfigurationDialog", "Show circles with fixed angular size around the telescope reticle", nullptr));
#endif // QT_CONFIG(tooltip)
        checkBoxCircles->setText(QCoreApplication::translate("telescopeConfigurationDialog", "Use field of view indicators", nullptr));
        labelCircles->setText(QCoreApplication::translate("telescopeConfigurationDialog", "Circle size(s):", nullptr));
#if QT_CONFIG(tooltip)
        lineEditCircleList->setToolTip(QCoreApplication::translate("telescopeConfigurationDialog", "Up to ten decimal values in degrees of arc, separated with commas", nullptr));
#endif // QT_CONFIG(tooltip)
        lineEditCircleList->setText(QString());
        pushButtonSave->setText(QCoreApplication::translate("telescopeConfigurationDialog", "OK", nullptr));
        pushButtonDiscard->setText(QCoreApplication::translate("telescopeConfigurationDialog", "Cancel", nullptr));
        (void)telescopeConfigurationDialog;
    } // retranslateUi

};

namespace Ui {
    class telescopeConfigurationDialog: public Ui_telescopeConfigurationDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_TELESCOPECONFIGURATIONDIALOG_H
