/****************************************************************************
** Meta object code from reading C++ file 'NebulaTexturesDialog.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../stellarium/plugins/NebulaTextures/src/gui/NebulaTexturesDialog.hpp"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'NebulaTexturesDialog.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN20NebulaTexturesDialogE_t {};
} // unnamed namespace

template <> constexpr inline auto NebulaTexturesDialog::qt_create_metaobjectdata<qt_meta_tag_ZN20NebulaTexturesDialogE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "NebulaTexturesDialog",
        "retranslate",
        "",
        "initializeRefreshIfNeeded",
        "reloadData",
        "refreshTextures",
        "getShowCustomTextures",
        "setShowCustomTextures",
        "b",
        "getAvoidAreaConflict",
        "setAvoidAreaConflict",
        "restoreDefaults",
        "openImageFile",
        "solveImage",
        "cancelSolve",
        "recoverSolvedCorners",
        "moveToCenterCoord",
        "toggleTempTexturePreview",
        "toggleDefaultTextureVisibility",
        "updateBrightnessLevel",
        "index",
        "addCustomTexture",
        "refreshTempTexturePreview",
        "showRecoverCoordsButton",
        "gotoSelectedItem",
        "QListWidgetItem*",
        "item",
        "removeTexture"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'retranslate'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'initializeRefreshIfNeeded'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'reloadData'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'refreshTextures'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'getShowCustomTextures'
        QtMocHelpers::SlotData<bool()>(6, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setShowCustomTextures'
        QtMocHelpers::SlotData<void(bool)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Slot 'getAvoidAreaConflict'
        QtMocHelpers::SlotData<bool()>(9, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setAvoidAreaConflict'
        QtMocHelpers::SlotData<void(bool)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Slot 'restoreDefaults'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'openImageFile'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'solveImage'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'cancelSolve'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'recoverSolvedCorners'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'moveToCenterCoord'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'toggleTempTexturePreview'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'toggleDefaultTextureVisibility'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateBrightnessLevel'
        QtMocHelpers::SlotData<void(int)>(19, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 20 },
        }}),
        // Slot 'addCustomTexture'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'refreshTempTexturePreview'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'showRecoverCoordsButton'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'gotoSelectedItem'
        QtMocHelpers::SlotData<void(QListWidgetItem *)>(24, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 25, 26 },
        }}),
        // Slot 'removeTexture'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<NebulaTexturesDialog, qt_meta_tag_ZN20NebulaTexturesDialogE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject NebulaTexturesDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<StelDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20NebulaTexturesDialogE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20NebulaTexturesDialogE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN20NebulaTexturesDialogE_t>.metaTypes,
    nullptr
} };

void NebulaTexturesDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<NebulaTexturesDialog *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->retranslate(); break;
        case 1: _t->initializeRefreshIfNeeded(); break;
        case 2: _t->reloadData(); break;
        case 3: _t->refreshTextures(); break;
        case 4: { bool _r = _t->getShowCustomTextures();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 5: _t->setShowCustomTextures((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: { bool _r = _t->getAvoidAreaConflict();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 7: _t->setAvoidAreaConflict((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->restoreDefaults(); break;
        case 9: _t->openImageFile(); break;
        case 10: _t->solveImage(); break;
        case 11: _t->cancelSolve(); break;
        case 12: _t->recoverSolvedCorners(); break;
        case 13: _t->moveToCenterCoord(); break;
        case 14: _t->toggleTempTexturePreview(); break;
        case 15: _t->toggleDefaultTextureVisibility(); break;
        case 16: _t->updateBrightnessLevel((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 17: _t->addCustomTexture(); break;
        case 18: _t->refreshTempTexturePreview(); break;
        case 19: _t->showRecoverCoordsButton(); break;
        case 20: _t->gotoSelectedItem((*reinterpret_cast<std::add_pointer_t<QListWidgetItem*>>(_a[1]))); break;
        case 21: _t->removeTexture(); break;
        default: ;
        }
    }
}

const QMetaObject *NebulaTexturesDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *NebulaTexturesDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20NebulaTexturesDialogE_t>.strings))
        return static_cast<void*>(this);
    return StelDialog::qt_metacast(_clname);
}

int NebulaTexturesDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StelDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 22)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 22)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 22;
    }
    return _id;
}
QT_WARNING_POP
