/****************************************************************************
** Meta object code from reading C++ file 'RemoteSync.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../stellarium/plugins/RemoteSync/src/RemoteSync.hpp"
#include <QtCore/qmetatype.h>
#include <QtCore/qplugin.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'RemoteSync.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10RemoteSyncE_t {};
} // unnamed namespace

template <> constexpr inline auto RemoteSync::qt_create_metaobjectdata<qt_meta_tag_ZN10RemoteSyncE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "RemoteSync",
        "errorOccurred",
        "",
        "errorString",
        "clientServerHostChanged",
        "clientServerHost",
        "clientServerPortChanged",
        "port",
        "serverPortChanged",
        "clientSyncOptionsChanged",
        "SyncClient::SyncOptions",
        "options",
        "stelPropFilterChanged",
        "stelPropFilter",
        "connectionLostBehaviorChanged",
        "RemoteSync::ClientBehavior",
        "bh",
        "quitBehaviorChanged",
        "stateChanged",
        "RemoteSync::SyncState",
        "state",
        "setClientServerHost",
        "setClientServerPort",
        "setServerPort",
        "setClientSyncOptions",
        "setStelPropFilter",
        "setConnectionLostBehavior",
        "setQuitBehavior",
        "startServer",
        "stopServer",
        "connectToServer",
        "disconnectFromServer",
        "loadSettings",
        "saveSettings",
        "restoreDefaultSettings",
        "clientDisconnected",
        "clean",
        "clientConnected",
        "SyncState",
        "IDLE",
        "SERVER",
        "CLIENT",
        "CLIENT_CONNECTING",
        "CLIENT_CLOSING",
        "CLIENT_WAIT_RECONNECT",
        "ClientBehavior",
        "NONE",
        "RECONNECT",
        "QUIT"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'errorOccurred'
        QtMocHelpers::SignalData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'clientServerHostChanged'
        QtMocHelpers::SignalData<void(const QString &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 5 },
        }}),
        // Signal 'clientServerPortChanged'
        QtMocHelpers::SignalData<void(const int)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 7 },
        }}),
        // Signal 'serverPortChanged'
        QtMocHelpers::SignalData<void(const int)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 7 },
        }}),
        // Signal 'clientSyncOptionsChanged'
        QtMocHelpers::SignalData<void(const SyncClient::SyncOptions)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Signal 'stelPropFilterChanged'
        QtMocHelpers::SignalData<void(const QStringList &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 13 },
        }}),
        // Signal 'connectionLostBehaviorChanged'
        QtMocHelpers::SignalData<void(const RemoteSync::ClientBehavior)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Signal 'quitBehaviorChanged'
        QtMocHelpers::SignalData<void(const RemoteSync::ClientBehavior)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Signal 'stateChanged'
        QtMocHelpers::SignalData<void(RemoteSync::SyncState)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 19, 20 },
        }}),
        // Slot 'setClientServerHost'
        QtMocHelpers::SlotData<void(const QString &)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 5 },
        }}),
        // Slot 'setClientServerPort'
        QtMocHelpers::SlotData<void(const int)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 7 },
        }}),
        // Slot 'setServerPort'
        QtMocHelpers::SlotData<void(const int)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 7 },
        }}),
        // Slot 'setClientSyncOptions'
        QtMocHelpers::SlotData<void(SyncClient::SyncOptions)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Slot 'setStelPropFilter'
        QtMocHelpers::SlotData<void(const QStringList &)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 13 },
        }}),
        // Slot 'setConnectionLostBehavior'
        QtMocHelpers::SlotData<void(const RemoteSync::ClientBehavior)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Slot 'setQuitBehavior'
        QtMocHelpers::SlotData<void(const RemoteSync::ClientBehavior)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Slot 'startServer'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'stopServer'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'connectToServer'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'disconnectFromServer'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'loadSettings'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'saveSettings'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'restoreDefaultSettings'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'clientDisconnected'
        QtMocHelpers::SlotData<void(bool)>(35, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 36 },
        }}),
        // Slot 'clientConnected'
        QtMocHelpers::SlotData<void()>(37, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'SyncState'
        QtMocHelpers::EnumData<enum SyncState>(38, 38, QMC::EnumFlags{}).add({
            {   39, SyncState::IDLE },
            {   40, SyncState::SERVER },
            {   41, SyncState::CLIENT },
            {   42, SyncState::CLIENT_CONNECTING },
            {   43, SyncState::CLIENT_CLOSING },
            {   44, SyncState::CLIENT_WAIT_RECONNECT },
        }),
        // enum 'ClientBehavior'
        QtMocHelpers::EnumData<enum ClientBehavior>(45, 45, QMC::EnumFlags{}).add({
            {   46, ClientBehavior::NONE },
            {   47, ClientBehavior::RECONNECT },
            {   48, ClientBehavior::QUIT },
        }),
    };
    return QtMocHelpers::metaObjectData<RemoteSync, qt_meta_tag_ZN10RemoteSyncE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject RemoteSync::staticMetaObject = { {
    QMetaObject::SuperData::link<StelModule::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10RemoteSyncE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10RemoteSyncE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10RemoteSyncE_t>.metaTypes,
    nullptr
} };

void RemoteSync::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<RemoteSync *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->errorOccurred((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->clientServerHostChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->clientServerPortChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 3: _t->serverPortChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->clientSyncOptionsChanged((*reinterpret_cast<std::add_pointer_t<SyncClient::SyncOptions>>(_a[1]))); break;
        case 5: _t->stelPropFilterChanged((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 6: _t->connectionLostBehaviorChanged((*reinterpret_cast<std::add_pointer_t<RemoteSync::ClientBehavior>>(_a[1]))); break;
        case 7: _t->quitBehaviorChanged((*reinterpret_cast<std::add_pointer_t<RemoteSync::ClientBehavior>>(_a[1]))); break;
        case 8: _t->stateChanged((*reinterpret_cast<std::add_pointer_t<RemoteSync::SyncState>>(_a[1]))); break;
        case 9: _t->setClientServerHost((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 10: _t->setClientServerPort((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 11: _t->setServerPort((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 12: _t->setClientSyncOptions((*reinterpret_cast<std::add_pointer_t<SyncClient::SyncOptions>>(_a[1]))); break;
        case 13: _t->setStelPropFilter((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 14: _t->setConnectionLostBehavior((*reinterpret_cast<std::add_pointer_t<RemoteSync::ClientBehavior>>(_a[1]))); break;
        case 15: _t->setQuitBehavior((*reinterpret_cast<std::add_pointer_t<RemoteSync::ClientBehavior>>(_a[1]))); break;
        case 16: _t->startServer(); break;
        case 17: _t->stopServer(); break;
        case 18: _t->connectToServer(); break;
        case 19: _t->disconnectFromServer(); break;
        case 20: _t->loadSettings(); break;
        case 21: _t->saveSettings(); break;
        case 22: _t->restoreDefaultSettings(); break;
        case 23: _t->clientDisconnected((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 24: _t->clientConnected(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< RemoteSync::SyncState >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (RemoteSync::*)(const QString & )>(_a, &RemoteSync::errorOccurred, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (RemoteSync::*)(const QString & )>(_a, &RemoteSync::clientServerHostChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (RemoteSync::*)(const int )>(_a, &RemoteSync::clientServerPortChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (RemoteSync::*)(const int )>(_a, &RemoteSync::serverPortChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (RemoteSync::*)(const SyncClient::SyncOptions )>(_a, &RemoteSync::clientSyncOptionsChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (RemoteSync::*)(const QStringList & )>(_a, &RemoteSync::stelPropFilterChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (RemoteSync::*)(const RemoteSync::ClientBehavior )>(_a, &RemoteSync::connectionLostBehaviorChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (RemoteSync::*)(const RemoteSync::ClientBehavior )>(_a, &RemoteSync::quitBehaviorChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (RemoteSync::*)(RemoteSync::SyncState )>(_a, &RemoteSync::stateChanged, 8))
            return;
    }
}

const QMetaObject *RemoteSync::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *RemoteSync::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10RemoteSyncE_t>.strings))
        return static_cast<void*>(this);
    return StelModule::qt_metacast(_clname);
}

int RemoteSync::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StelModule::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 25)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 25;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 25)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 25;
    }
    return _id;
}

// SIGNAL 0
void RemoteSync::errorOccurred(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void RemoteSync::clientServerHostChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void RemoteSync::clientServerPortChanged(const int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void RemoteSync::serverPortChanged(const int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void RemoteSync::clientSyncOptionsChanged(const SyncClient::SyncOptions _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void RemoteSync::stelPropFilterChanged(const QStringList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void RemoteSync::connectionLostBehaviorChanged(const RemoteSync::ClientBehavior _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void RemoteSync::quitBehaviorChanged(const RemoteSync::ClientBehavior _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void RemoteSync::stateChanged(RemoteSync::SyncState _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN29RemoteSyncStelPluginInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto RemoteSyncStelPluginInterface::qt_create_metaobjectdata<qt_meta_tag_ZN29RemoteSyncStelPluginInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "RemoteSyncStelPluginInterface"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<RemoteSyncStelPluginInterface, qt_meta_tag_ZN29RemoteSyncStelPluginInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject RemoteSyncStelPluginInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN29RemoteSyncStelPluginInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN29RemoteSyncStelPluginInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN29RemoteSyncStelPluginInterfaceE_t>.metaTypes,
    nullptr
} };

void RemoteSyncStelPluginInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<RemoteSyncStelPluginInterface *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *RemoteSyncStelPluginInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *RemoteSyncStelPluginInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN29RemoteSyncStelPluginInterfaceE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "StelPluginInterface"))
        return static_cast< StelPluginInterface*>(this);
    if (!strcmp(_clname, "org.stellarium.StelPluginInterface/2.0"))
        return static_cast< StelPluginInterface*>(this);
    return QObject::qt_metacast(_clname);
}

int RemoteSyncStelPluginInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}

#ifdef QT_MOC_EXPORT_PLUGIN_V2
static constexpr unsigned char qt_pluginMetaDataV2_RemoteSyncStelPluginInterface[] = {
    0xbf, 
    // "IID"
    0x02,  0x78,  0x26,  'o',  'r',  'g',  '.',  's', 
    't',  'e',  'l',  'l',  'a',  'r',  'i',  'u', 
    'm',  '.',  'S',  't',  'e',  'l',  'P',  'l', 
    'u',  'g',  'i',  'n',  'I',  'n',  't',  'e', 
    'r',  'f',  'a',  'c',  'e',  '/',  '2',  '.', 
    '0', 
    // "className"
    0x03,  0x78,  0x1d,  'R',  'e',  'm',  'o',  't', 
    'e',  'S',  'y',  'n',  'c',  'S',  't',  'e', 
    'l',  'P',  'l',  'u',  'g',  'i',  'n',  'I', 
    'n',  't',  'e',  'r',  'f',  'a',  'c',  'e', 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN_V2(RemoteSyncStelPluginInterface, RemoteSyncStelPluginInterface, qt_pluginMetaDataV2_RemoteSyncStelPluginInterface)
#else
QT_PLUGIN_METADATA_SECTION
Q_CONSTINIT static constexpr unsigned char qt_pluginMetaData_RemoteSyncStelPluginInterface[] = {
    'Q', 'T', 'M', 'E', 'T', 'A', 'D', 'A', 'T', 'A', ' ', '!',
    // metadata version, Qt version, architectural requirements
    0, QT_VERSION_MAJOR, QT_VERSION_MINOR, qPluginArchRequirements(),
    0xbf, 
    // "IID"
    0x02,  0x78,  0x26,  'o',  'r',  'g',  '.',  's', 
    't',  'e',  'l',  'l',  'a',  'r',  'i',  'u', 
    'm',  '.',  'S',  't',  'e',  'l',  'P',  'l', 
    'u',  'g',  'i',  'n',  'I',  'n',  't',  'e', 
    'r',  'f',  'a',  'c',  'e',  '/',  '2',  '.', 
    '0', 
    // "className"
    0x03,  0x78,  0x1d,  'R',  'e',  'm',  'o',  't', 
    'e',  'S',  'y',  'n',  'c',  'S',  't',  'e', 
    'l',  'P',  'l',  'u',  'g',  'i',  'n',  'I', 
    'n',  't',  'e',  'r',  'f',  'a',  'c',  'e', 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN(RemoteSyncStelPluginInterface, RemoteSyncStelPluginInterface)
#endif  // QT_MOC_EXPORT_PLUGIN_V2

QT_WARNING_POP
