/********************************************************************************
** Form generated from reading UI file 'ocularDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_OCULARDIALOG_H
#define UI_OCULARDIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListView>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTextBrowser>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "Dialog.hpp"
#include "gui/ColorButton.hpp"

QT_BEGIN_NAMESPACE

class Ui_ocularDialogForm
{
public:
    QVBoxLayout *verticalLayout_2;
    TitleBar *titleBar;
    QFrame *Contents;
    QVBoxLayout *verticalLayout_4;
    QTabWidget *Tabs;
    QWidget *generalTab;
    QGridLayout *gridLayout_10;
    QGroupBox *groupBoxInterface;
    QGridLayout *gridLayout_7;
    QHBoxLayout *horizontalLayout_8;
    QLabel *arrowButtonScaleLabel;
    QSpacerItem *horizontalSpacer_7;
    QSpinBox *arrowButtonScaleSpinBox;
    QCheckBox *checkBoxInitialFOV;
    QCheckBox *checkBoxResolutionCriterion;
    QHBoxLayout *guiPanelHorizontalLayout;
    QCheckBox *checkBoxControlPanel;
    QSpacerItem *horizontalSpacer;
    QLabel *guiFontSizeLabel;
    QSpinBox *guiFontSizeSpinBox;
    QCheckBox *checkBoxInitialDirection;
    QHBoxLayout *colorHorizontalLayout;
    ColorButton *lineColorToolButton;
    QLabel *lineColorLabel;
    ColorButton *textColorToolButton;
    QLabel *textColorLabel;
    QSpacerItem *colorHorizontalSpacer;
    QCheckBox *checkBoxToolbarButton;
    QSpacerItem *verticalSpacer_7;
    QGroupBox *groupBoxOcularOptions;
    QGridLayout *gridLayout_6;
    QCheckBox *scaleImageCircleCheckBox;
    QCheckBox *hideGridsLinesCheckBox;
    QCheckBox *requireSelectionCheckBox;
    QCheckBox *contourCheckBox;
    QCheckBox *limitStellarMagnitudeCheckBox;
    QSpacerItem *verticalSpacer_6;
    QCheckBox *cardinalsCheckBox;
    QCheckBox *alignCrosshairCheckBox;
    QHBoxLayout *horizontalLayout_15;
    QCheckBox *semiTransparencyCheckBox;
    QSpacerItem *horizontalSpacer_4;
    QSpinBox *transparencySpinBox;
    QGroupBox *groupBoxSensorOptions;
    QGridLayout *gridLayout_8;
    QHBoxLayout *horizontalLayout_14;
    QSpacerItem *horizontalSpacer_5;
    QLabel *labelFocuserOverlay;
    QCheckBox *checkBoxUseSmallFocuser;
    QCheckBox *checkBoxUseMediumFocuser;
    QCheckBox *checkBoxUseLargeFocuser;
    QSpacerItem *horizontalSpacer_6;
    QCheckBox *checkBoxShowCcdCropOverlay;
    QHBoxLayout *guiSensorPanelHorizontalLayout;
    QSpacerItem *horizontalSpacer_2;
    QLabel *guiCcdCropOverlaySizeLabel;
    QSpinBox *guiCcdCropOverlayHSizeSpinBox;
    QSpinBox *guiCcdCropOverlayVSizeSpinBox;
    QCheckBox *checkBoxShowCcdCropOverlayPixelGrid;
    QSpacerItem *horizontalSpacer_3;
    QCheckBox *checkBoxCCDFOVScaling;
    QCheckBox *checkBoxCCDMaxExposureTime;
    QCheckBox *checkBoxDMSDegrees;
    QCheckBox *checkBoxTypeOfMount;
    QHBoxLayout *horizontalLayout_4;
    ColorButton *focuserColorToolButton;
    QCheckBox *checkBoxShowFocuserOverlay;
    QCheckBox *checkBoxHorizontalCoordinates;
    QCheckBox *checkBoxShowOAGLimits;
    QGroupBox *groupBoxTelradOptions;
    QGridLayout *gridLayout_9;
    QHBoxLayout *horizontalLayout;
    QLabel *labelTelradFOV;
    QDoubleSpinBox *doubleSpinBoxTelradFOV1;
    QDoubleSpinBox *doubleSpinBoxTelradFOV2;
    QDoubleSpinBox *doubleSpinBoxTelradFOV3;
    QDoubleSpinBox *doubleSpinBoxTelradFOV4;
    QPushButton *pushButtonRestoreTelradFOV;
    QCheckBox *checkBoxTelradFOVScaling;
    QSpacerItem *verticalSpacer_5;
    QSpacerItem *verticalSpacer_3;
    QWidget *ocularsTab;
    QGridLayout *gridLayout_2;
    QHBoxLayout *horizontalLayout_9;
    QVBoxLayout *verticalLayout_7;
    QListView *ocularListView;
    QHBoxLayout *horizontalLayout_10;
    QPushButton *addOcular;
    QPushButton *pushButtonMoveOcularUp;
    QPushButton *pushButtonMoveOcularDown;
    QPushButton *deleteOcular;
    QGridLayout *gridLayout_11;
    QLabel *labelFS;
    QLineEdit *ocularName;
    QDoubleSpinBox *ocularFL;
    QCheckBox *permanentCrosshairCheckBox;
    QCheckBox *binocularsCheckBox;
    QDoubleSpinBox *ocularAFov;
    QLabel *labelFOV;
    QLabel *labelFL;
    QDoubleSpinBox *ocularFieldStop;
    QLabel *labelName;
    QSpacerItem *verticalSpacer;
    QWidget *lensesTab;
    QGridLayout *gridLayout_3;
    QHBoxLayout *horizontalLayout_2;
    QVBoxLayout *verticalLayout;
    QListView *lensListView;
    QHBoxLayout *horizontalLayout_5;
    QPushButton *addLens;
    QPushButton *pushButtonMoveLensUp;
    QPushButton *pushButtonMoveLensDown;
    QPushButton *deleteLens;
    QGridLayout *gridLayout_12;
    QLabel *label_7;
    QDoubleSpinBox *lensMultiplier;
    QLabel *lensNote;
    QLineEdit *lensName;
    QLabel *label_13;
    QSpacerItem *verticalSpacer_4;
    QWidget *sensorsTab;
    QGridLayout *gridLayout_4;
    QHBoxLayout *horizontalLayout_3;
    QVBoxLayout *verticalLayout_5;
    QListView *ccdListView;
    QHBoxLayout *horizontalLayout_11;
    QPushButton *addCCD;
    QPushButton *pushButtonMoveSensorUp;
    QPushButton *pushButtonMoveSensorDown;
    QPushButton *deleteCCD;
    QGridLayout *gridLayout_13;
    QDoubleSpinBox *OAGPrismW;
    QDoubleSpinBox *OAGPrismH;
    QCheckBox *OAG_checkBox;
    QDoubleSpinBox *ccdChipX;
    QSpinBox *ccdBinningY;
    QSpinBox *ccdResY;
    QDoubleSpinBox *OAGPrismPA;
    QDoubleSpinBox *ccdRotAngle;
    QLabel *CCDResY_lbl;
    QLabel *ccdBinningX_lbl;
    QLabel *CCDResX_lbl;
    QLabel *CCDName_lbl;
    QLabel *OAGPrismH_lbl;
    QSpinBox *ccdBinningX;
    QLabel *OAGDist_lbl;
    QLineEdit *ccdName;
    QSpacerItem *CCDverticalSpacer;
    QDoubleSpinBox *ccdChipY;
    QDoubleSpinBox *OAGDist;
    QSpinBox *ccdResX;
    QLabel *ccdRotAngle_lbl;
    QLabel *ccdBinningY_lbl;
    QLabel *OAGPrismW_lbl;
    QLabel *CCDChipY_lbl;
    QLabel *CCDChipX_lbl;
    QLabel *OAGPrismPA_lbl;
    QWidget *telescopesTab;
    QHBoxLayout *horizontalLayout_6;
    QHBoxLayout *horizontalLayout_12;
    QVBoxLayout *verticalLayout_8;
    QListView *telescopeListView;
    QHBoxLayout *horizontalLayout_13;
    QPushButton *addTelescope;
    QPushButton *pushButtonMoveTelescopeUp;
    QPushButton *pushButtonMoveTelescopeDown;
    QPushButton *deleteTelescope;
    QGridLayout *gridLayout_14;
    QLabel *label_8;
    QDoubleSpinBox *telescopeFL;
    QDoubleSpinBox *telescopeDiameter;
    QCheckBox *telescopeHFlip;
    QCheckBox *telescopeVFlip;
    QCheckBox *telescopeEQ;
    QLineEdit *telescopeName;
    QLabel *label_12;
    QLabel *label_9;
    QSpacerItem *verticalSpacer_2;
    QWidget *aboutTab;
    QGridLayout *gridLayout;
    QTextBrowser *textBrowser;

    void setupUi(QWidget *ocularDialogForm)
    {
        if (ocularDialogForm->objectName().isEmpty())
            ocularDialogForm->setObjectName("ocularDialogForm");
        ocularDialogForm->resize(750, 597);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(ocularDialogForm->sizePolicy().hasHeightForWidth());
        ocularDialogForm->setSizePolicy(sizePolicy);
        verticalLayout_2 = new QVBoxLayout(ocularDialogForm);
        verticalLayout_2->setSpacing(0);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        titleBar = new TitleBar(ocularDialogForm);
        titleBar->setObjectName("titleBar");

        verticalLayout_2->addWidget(titleBar);

        Contents = new QFrame(ocularDialogForm);
        Contents->setObjectName("Contents");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(Contents->sizePolicy().hasHeightForWidth());
        Contents->setSizePolicy(sizePolicy1);
        verticalLayout_4 = new QVBoxLayout(Contents);
        verticalLayout_4->setSpacing(0);
        verticalLayout_4->setObjectName("verticalLayout_4");
        verticalLayout_4->setContentsMargins(0, 0, 0, 0);
        Tabs = new QTabWidget(Contents);
        Tabs->setObjectName("Tabs");
        Tabs->setFocusPolicy(Qt::NoFocus);
        Tabs->setUsesScrollButtons(false);
        generalTab = new QWidget();
        generalTab->setObjectName("generalTab");
        gridLayout_10 = new QGridLayout(generalTab);
        gridLayout_10->setSpacing(0);
        gridLayout_10->setObjectName("gridLayout_10");
        gridLayout_10->setContentsMargins(0, 0, 0, 0);
        groupBoxInterface = new QGroupBox(generalTab);
        groupBoxInterface->setObjectName("groupBoxInterface");
        gridLayout_7 = new QGridLayout(groupBoxInterface);
        gridLayout_7->setObjectName("gridLayout_7");
        gridLayout_7->setContentsMargins(0, 0, 0, 0);
        horizontalLayout_8 = new QHBoxLayout();
        horizontalLayout_8->setObjectName("horizontalLayout_8");
        arrowButtonScaleLabel = new QLabel(groupBoxInterface);
        arrowButtonScaleLabel->setObjectName("arrowButtonScaleLabel");

        horizontalLayout_8->addWidget(arrowButtonScaleLabel);

        horizontalSpacer_7 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_8->addItem(horizontalSpacer_7);

        arrowButtonScaleSpinBox = new QSpinBox(groupBoxInterface);
        arrowButtonScaleSpinBox->setObjectName("arrowButtonScaleSpinBox");
        arrowButtonScaleSpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        arrowButtonScaleSpinBox->setSuffix(QString::fromUtf8("%"));
        arrowButtonScaleSpinBox->setMinimum(100);
        arrowButtonScaleSpinBox->setMaximum(250);

        horizontalLayout_8->addWidget(arrowButtonScaleSpinBox);


        gridLayout_7->addLayout(horizontalLayout_8, 11, 0, 1, 1);

        checkBoxInitialFOV = new QCheckBox(groupBoxInterface);
        checkBoxInitialFOV->setObjectName("checkBoxInitialFOV");

        gridLayout_7->addWidget(checkBoxInitialFOV, 6, 0, 1, 1);

        checkBoxResolutionCriterion = new QCheckBox(groupBoxInterface);
        checkBoxResolutionCriterion->setObjectName("checkBoxResolutionCriterion");

        gridLayout_7->addWidget(checkBoxResolutionCriterion, 8, 0, 1, 1);

        guiPanelHorizontalLayout = new QHBoxLayout();
        guiPanelHorizontalLayout->setObjectName("guiPanelHorizontalLayout");
        checkBoxControlPanel = new QCheckBox(groupBoxInterface);
        checkBoxControlPanel->setObjectName("checkBoxControlPanel");

        guiPanelHorizontalLayout->addWidget(checkBoxControlPanel);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        guiPanelHorizontalLayout->addItem(horizontalSpacer);

        guiFontSizeLabel = new QLabel(groupBoxInterface);
        guiFontSizeLabel->setObjectName("guiFontSizeLabel");
        guiFontSizeLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        guiPanelHorizontalLayout->addWidget(guiFontSizeLabel);

        guiFontSizeSpinBox = new QSpinBox(groupBoxInterface);
        guiFontSizeSpinBox->setObjectName("guiFontSizeSpinBox");
        guiFontSizeSpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        guiFontSizeSpinBox->setMinimum(10);
        guiFontSizeSpinBox->setMaximum(30);
        guiFontSizeSpinBox->setValue(12);

        guiPanelHorizontalLayout->addWidget(guiFontSizeSpinBox);


        gridLayout_7->addLayout(guiPanelHorizontalLayout, 3, 0, 1, 1);

        checkBoxInitialDirection = new QCheckBox(groupBoxInterface);
        checkBoxInitialDirection->setObjectName("checkBoxInitialDirection");

        gridLayout_7->addWidget(checkBoxInitialDirection, 7, 0, 1, 1);

        colorHorizontalLayout = new QHBoxLayout();
        colorHorizontalLayout->setObjectName("colorHorizontalLayout");
        lineColorToolButton = new ColorButton(groupBoxInterface);
        lineColorToolButton->setObjectName("lineColorToolButton");

        colorHorizontalLayout->addWidget(lineColorToolButton);

        lineColorLabel = new QLabel(groupBoxInterface);
        lineColorLabel->setObjectName("lineColorLabel");

        colorHorizontalLayout->addWidget(lineColorLabel);

        textColorToolButton = new ColorButton(groupBoxInterface);
        textColorToolButton->setObjectName("textColorToolButton");

        colorHorizontalLayout->addWidget(textColorToolButton);

        textColorLabel = new QLabel(groupBoxInterface);
        textColorLabel->setObjectName("textColorLabel");

        colorHorizontalLayout->addWidget(textColorLabel);

        colorHorizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        colorHorizontalLayout->addItem(colorHorizontalSpacer);


        gridLayout_7->addLayout(colorHorizontalLayout, 14, 0, 1, 1);

        checkBoxToolbarButton = new QCheckBox(groupBoxInterface);
        checkBoxToolbarButton->setObjectName("checkBoxToolbarButton");

        gridLayout_7->addWidget(checkBoxToolbarButton, 9, 0, 1, 1);

        verticalSpacer_7 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_7->addItem(verticalSpacer_7, 16, 0, 1, 1);


        gridLayout_10->addWidget(groupBoxInterface, 0, 0, 1, 1);

        groupBoxOcularOptions = new QGroupBox(generalTab);
        groupBoxOcularOptions->setObjectName("groupBoxOcularOptions");
        gridLayout_6 = new QGridLayout(groupBoxOcularOptions);
        gridLayout_6->setObjectName("gridLayout_6");
        gridLayout_6->setContentsMargins(0, 0, 0, 0);
        scaleImageCircleCheckBox = new QCheckBox(groupBoxOcularOptions);
        scaleImageCircleCheckBox->setObjectName("scaleImageCircleCheckBox");

        gridLayout_6->addWidget(scaleImageCircleCheckBox, 2, 0, 1, 1);

        hideGridsLinesCheckBox = new QCheckBox(groupBoxOcularOptions);
        hideGridsLinesCheckBox->setObjectName("hideGridsLinesCheckBox");

        gridLayout_6->addWidget(hideGridsLinesCheckBox, 7, 0, 1, 1);

        requireSelectionCheckBox = new QCheckBox(groupBoxOcularOptions);
        requireSelectionCheckBox->setObjectName("requireSelectionCheckBox");

        gridLayout_6->addWidget(requireSelectionCheckBox, 0, 0, 1, 1);

        contourCheckBox = new QCheckBox(groupBoxOcularOptions);
        contourCheckBox->setObjectName("contourCheckBox");

        gridLayout_6->addWidget(contourCheckBox, 8, 0, 1, 1);

        limitStellarMagnitudeCheckBox = new QCheckBox(groupBoxOcularOptions);
        limitStellarMagnitudeCheckBox->setObjectName("limitStellarMagnitudeCheckBox");

        gridLayout_6->addWidget(limitStellarMagnitudeCheckBox, 3, 0, 1, 1);

        verticalSpacer_6 = new QSpacerItem(0, 5, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_6->addItem(verticalSpacer_6, 11, 0, 1, 1);

        cardinalsCheckBox = new QCheckBox(groupBoxOcularOptions);
        cardinalsCheckBox->setObjectName("cardinalsCheckBox");

        gridLayout_6->addWidget(cardinalsCheckBox, 9, 0, 1, 1);

        alignCrosshairCheckBox = new QCheckBox(groupBoxOcularOptions);
        alignCrosshairCheckBox->setObjectName("alignCrosshairCheckBox");

        gridLayout_6->addWidget(alignCrosshairCheckBox, 10, 0, 1, 1);

        horizontalLayout_15 = new QHBoxLayout();
        horizontalLayout_15->setObjectName("horizontalLayout_15");
        semiTransparencyCheckBox = new QCheckBox(groupBoxOcularOptions);
        semiTransparencyCheckBox->setObjectName("semiTransparencyCheckBox");

        horizontalLayout_15->addWidget(semiTransparencyCheckBox);

        horizontalSpacer_4 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_15->addItem(horizontalSpacer_4);

        transparencySpinBox = new QSpinBox(groupBoxOcularOptions);
        transparencySpinBox->setObjectName("transparencySpinBox");
        transparencySpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        transparencySpinBox->setSuffix(QString::fromUtf8("%"));
        transparencySpinBox->setMinimum(10);
        transparencySpinBox->setMaximum(90);
        transparencySpinBox->setValue(85);

        horizontalLayout_15->addWidget(transparencySpinBox);


        gridLayout_6->addLayout(horizontalLayout_15, 4, 0, 1, 1);


        gridLayout_10->addWidget(groupBoxOcularOptions, 0, 1, 1, 1);

        groupBoxSensorOptions = new QGroupBox(generalTab);
        groupBoxSensorOptions->setObjectName("groupBoxSensorOptions");
        gridLayout_8 = new QGridLayout(groupBoxSensorOptions);
        gridLayout_8->setObjectName("gridLayout_8");
        gridLayout_8->setContentsMargins(0, 0, 0, 0);
        horizontalLayout_14 = new QHBoxLayout();
        horizontalLayout_14->setObjectName("horizontalLayout_14");
        horizontalSpacer_5 = new QSpacerItem(22, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout_14->addItem(horizontalSpacer_5);

        labelFocuserOverlay = new QLabel(groupBoxSensorOptions);
        labelFocuserOverlay->setObjectName("labelFocuserOverlay");

        horizontalLayout_14->addWidget(labelFocuserOverlay);

        checkBoxUseSmallFocuser = new QCheckBox(groupBoxSensorOptions);
        checkBoxUseSmallFocuser->setObjectName("checkBoxUseSmallFocuser");
        checkBoxUseSmallFocuser->setText(QString::fromUtf8("1.25\""));

        horizontalLayout_14->addWidget(checkBoxUseSmallFocuser);

        checkBoxUseMediumFocuser = new QCheckBox(groupBoxSensorOptions);
        checkBoxUseMediumFocuser->setObjectName("checkBoxUseMediumFocuser");
        checkBoxUseMediumFocuser->setText(QString::fromUtf8("2.0\""));

        horizontalLayout_14->addWidget(checkBoxUseMediumFocuser);

        checkBoxUseLargeFocuser = new QCheckBox(groupBoxSensorOptions);
        checkBoxUseLargeFocuser->setObjectName("checkBoxUseLargeFocuser");
        checkBoxUseLargeFocuser->setText(QString::fromUtf8("3.3\""));

        horizontalLayout_14->addWidget(checkBoxUseLargeFocuser);

        horizontalSpacer_6 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_14->addItem(horizontalSpacer_6);


        gridLayout_8->addLayout(horizontalLayout_14, 10, 0, 1, 1);

        checkBoxShowCcdCropOverlay = new QCheckBox(groupBoxSensorOptions);
        checkBoxShowCcdCropOverlay->setObjectName("checkBoxShowCcdCropOverlay");

        gridLayout_8->addWidget(checkBoxShowCcdCropOverlay, 6, 0, 1, 1);

        guiSensorPanelHorizontalLayout = new QHBoxLayout();
        guiSensorPanelHorizontalLayout->setObjectName("guiSensorPanelHorizontalLayout");
        horizontalSpacer_2 = new QSpacerItem(22, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        guiSensorPanelHorizontalLayout->addItem(horizontalSpacer_2);

        guiCcdCropOverlaySizeLabel = new QLabel(groupBoxSensorOptions);
        guiCcdCropOverlaySizeLabel->setObjectName("guiCcdCropOverlaySizeLabel");

        guiSensorPanelHorizontalLayout->addWidget(guiCcdCropOverlaySizeLabel);

        guiCcdCropOverlayHSizeSpinBox = new QSpinBox(groupBoxSensorOptions);
        guiCcdCropOverlayHSizeSpinBox->setObjectName("guiCcdCropOverlayHSizeSpinBox");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Maximum, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(guiCcdCropOverlayHSizeSpinBox->sizePolicy().hasHeightForWidth());
        guiCcdCropOverlayHSizeSpinBox->setSizePolicy(sizePolicy2);
        guiCcdCropOverlayHSizeSpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        guiCcdCropOverlayHSizeSpinBox->setMinimum(1);
        guiCcdCropOverlayHSizeSpinBox->setMaximum(6000);
        guiCcdCropOverlayHSizeSpinBox->setValue(250);

        guiSensorPanelHorizontalLayout->addWidget(guiCcdCropOverlayHSizeSpinBox);

        guiCcdCropOverlayVSizeSpinBox = new QSpinBox(groupBoxSensorOptions);
        guiCcdCropOverlayVSizeSpinBox->setObjectName("guiCcdCropOverlayVSizeSpinBox");
        sizePolicy2.setHeightForWidth(guiCcdCropOverlayVSizeSpinBox->sizePolicy().hasHeightForWidth());
        guiCcdCropOverlayVSizeSpinBox->setSizePolicy(sizePolicy2);
        guiCcdCropOverlayVSizeSpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        guiCcdCropOverlayVSizeSpinBox->setMinimum(1);
        guiCcdCropOverlayVSizeSpinBox->setMaximum(6000);
        guiCcdCropOverlayVSizeSpinBox->setValue(250);

        guiSensorPanelHorizontalLayout->addWidget(guiCcdCropOverlayVSizeSpinBox);

        checkBoxShowCcdCropOverlayPixelGrid = new QCheckBox(groupBoxSensorOptions);
        checkBoxShowCcdCropOverlayPixelGrid->setObjectName("checkBoxShowCcdCropOverlayPixelGrid");

        guiSensorPanelHorizontalLayout->addWidget(checkBoxShowCcdCropOverlayPixelGrid);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        guiSensorPanelHorizontalLayout->addItem(horizontalSpacer_3);


        gridLayout_8->addLayout(guiSensorPanelHorizontalLayout, 7, 0, 1, 1);

        checkBoxCCDFOVScaling = new QCheckBox(groupBoxSensorOptions);
        checkBoxCCDFOVScaling->setObjectName("checkBoxCCDFOVScaling");

        gridLayout_8->addWidget(checkBoxCCDFOVScaling, 0, 0, 1, 1);

        checkBoxCCDMaxExposureTime = new QCheckBox(groupBoxSensorOptions);
        checkBoxCCDMaxExposureTime->setObjectName("checkBoxCCDMaxExposureTime");

        gridLayout_8->addWidget(checkBoxCCDMaxExposureTime, 3, 0, 1, 1);

        checkBoxDMSDegrees = new QCheckBox(groupBoxSensorOptions);
        checkBoxDMSDegrees->setObjectName("checkBoxDMSDegrees");

        gridLayout_8->addWidget(checkBoxDMSDegrees, 1, 0, 1, 1);

        checkBoxTypeOfMount = new QCheckBox(groupBoxSensorOptions);
        checkBoxTypeOfMount->setObjectName("checkBoxTypeOfMount");

        gridLayout_8->addWidget(checkBoxTypeOfMount, 4, 0, 1, 1);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        focuserColorToolButton = new ColorButton(groupBoxSensorOptions);
        focuserColorToolButton->setObjectName("focuserColorToolButton");

        horizontalLayout_4->addWidget(focuserColorToolButton);

        checkBoxShowFocuserOverlay = new QCheckBox(groupBoxSensorOptions);
        checkBoxShowFocuserOverlay->setObjectName("checkBoxShowFocuserOverlay");

        horizontalLayout_4->addWidget(checkBoxShowFocuserOverlay);


        gridLayout_8->addLayout(horizontalLayout_4, 9, 0, 1, 1);

        checkBoxHorizontalCoordinates = new QCheckBox(groupBoxSensorOptions);
        checkBoxHorizontalCoordinates->setObjectName("checkBoxHorizontalCoordinates");

        gridLayout_8->addWidget(checkBoxHorizontalCoordinates, 2, 0, 1, 1);

        checkBoxShowOAGLimits = new QCheckBox(groupBoxSensorOptions);
        checkBoxShowOAGLimits->setObjectName("checkBoxShowOAGLimits");

        gridLayout_8->addWidget(checkBoxShowOAGLimits, 5, 0, 1, 1);


        gridLayout_10->addWidget(groupBoxSensorOptions, 1, 0, 1, 1);

        groupBoxTelradOptions = new QGroupBox(generalTab);
        groupBoxTelradOptions->setObjectName("groupBoxTelradOptions");
        groupBoxTelradOptions->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignmentFlag::AlignTop);
        gridLayout_9 = new QGridLayout(groupBoxTelradOptions);
        gridLayout_9->setObjectName("gridLayout_9");
        gridLayout_9->setContentsMargins(0, 0, 0, 0);
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        labelTelradFOV = new QLabel(groupBoxTelradOptions);
        labelTelradFOV->setObjectName("labelTelradFOV");
        labelTelradFOV->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);

        horizontalLayout->addWidget(labelTelradFOV);

        doubleSpinBoxTelradFOV1 = new QDoubleSpinBox(groupBoxTelradOptions);
        doubleSpinBoxTelradFOV1->setObjectName("doubleSpinBoxTelradFOV1");
        doubleSpinBoxTelradFOV1->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        doubleSpinBoxTelradFOV1->setSuffix(QString::fromUtf8("\302\260"));
        doubleSpinBoxTelradFOV1->setMinimum(0.000000000000000);
        doubleSpinBoxTelradFOV1->setMaximum(28.000000000000000);
        doubleSpinBoxTelradFOV1->setSingleStep(0.100000000000000);
        doubleSpinBoxTelradFOV1->setValue(0.500000000000000);

        horizontalLayout->addWidget(doubleSpinBoxTelradFOV1);

        doubleSpinBoxTelradFOV2 = new QDoubleSpinBox(groupBoxTelradOptions);
        doubleSpinBoxTelradFOV2->setObjectName("doubleSpinBoxTelradFOV2");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(doubleSpinBoxTelradFOV2->sizePolicy().hasHeightForWidth());
        doubleSpinBoxTelradFOV2->setSizePolicy(sizePolicy3);
        doubleSpinBoxTelradFOV2->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        doubleSpinBoxTelradFOV2->setSuffix(QString::fromUtf8("\302\260"));
        doubleSpinBoxTelradFOV2->setMinimum(0.000000000000000);
        doubleSpinBoxTelradFOV2->setMaximum(28.000000000000000);
        doubleSpinBoxTelradFOV2->setSingleStep(0.100000000000000);
        doubleSpinBoxTelradFOV2->setValue(2.000000000000000);

        horizontalLayout->addWidget(doubleSpinBoxTelradFOV2);

        doubleSpinBoxTelradFOV3 = new QDoubleSpinBox(groupBoxTelradOptions);
        doubleSpinBoxTelradFOV3->setObjectName("doubleSpinBoxTelradFOV3");
        sizePolicy3.setHeightForWidth(doubleSpinBoxTelradFOV3->sizePolicy().hasHeightForWidth());
        doubleSpinBoxTelradFOV3->setSizePolicy(sizePolicy3);
        doubleSpinBoxTelradFOV3->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        doubleSpinBoxTelradFOV3->setSuffix(QString::fromUtf8("\302\260"));
        doubleSpinBoxTelradFOV3->setMinimum(0.000000000000000);
        doubleSpinBoxTelradFOV3->setMaximum(28.000000000000000);
        doubleSpinBoxTelradFOV3->setSingleStep(0.100000000000000);
        doubleSpinBoxTelradFOV3->setValue(4.000000000000000);

        horizontalLayout->addWidget(doubleSpinBoxTelradFOV3);

        doubleSpinBoxTelradFOV4 = new QDoubleSpinBox(groupBoxTelradOptions);
        doubleSpinBoxTelradFOV4->setObjectName("doubleSpinBoxTelradFOV4");
        sizePolicy3.setHeightForWidth(doubleSpinBoxTelradFOV4->sizePolicy().hasHeightForWidth());
        doubleSpinBoxTelradFOV4->setSizePolicy(sizePolicy3);
        doubleSpinBoxTelradFOV4->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        doubleSpinBoxTelradFOV4->setSuffix(QString::fromUtf8("\302\260"));
        doubleSpinBoxTelradFOV4->setMaximum(30.000000000000000);
        doubleSpinBoxTelradFOV4->setSingleStep(0.100000000000000);

        horizontalLayout->addWidget(doubleSpinBoxTelradFOV4);


        gridLayout_9->addLayout(horizontalLayout, 1, 0, 1, 1);

        pushButtonRestoreTelradFOV = new QPushButton(groupBoxTelradOptions);
        pushButtonRestoreTelradFOV->setObjectName("pushButtonRestoreTelradFOV");

        gridLayout_9->addWidget(pushButtonRestoreTelradFOV, 4, 0, 1, 1);

        checkBoxTelradFOVScaling = new QCheckBox(groupBoxTelradOptions);
        checkBoxTelradFOVScaling->setObjectName("checkBoxTelradFOVScaling");

        gridLayout_9->addWidget(checkBoxTelradFOVScaling, 0, 0, 1, 1);

        verticalSpacer_5 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_9->addItem(verticalSpacer_5, 5, 0, 1, 1);


        gridLayout_10->addWidget(groupBoxTelradOptions, 1, 1, 1, 1);

        verticalSpacer_3 = new QSpacerItem(0, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_10->addItem(verticalSpacer_3, 2, 0, 1, 2);

        Tabs->addTab(generalTab, QString());
        ocularsTab = new QWidget();
        ocularsTab->setObjectName("ocularsTab");
        gridLayout_2 = new QGridLayout(ocularsTab);
        gridLayout_2->setObjectName("gridLayout_2");
        horizontalLayout_9 = new QHBoxLayout();
        horizontalLayout_9->setObjectName("horizontalLayout_9");
        verticalLayout_7 = new QVBoxLayout();
        verticalLayout_7->setObjectName("verticalLayout_7");
        ocularListView = new QListView(ocularsTab);
        ocularListView->setObjectName("ocularListView");
        ocularListView->setEditTriggers(QAbstractItemView::EditTrigger::NoEditTriggers);
        ocularListView->setProperty("isWrapping", QVariant(false));
        ocularListView->setResizeMode(QListView::ResizeMode::Fixed);
        ocularListView->setViewMode(QListView::ViewMode::ListMode);
        ocularListView->setModelColumn(0);

        verticalLayout_7->addWidget(ocularListView);

        horizontalLayout_10 = new QHBoxLayout();
        horizontalLayout_10->setObjectName("horizontalLayout_10");
        addOcular = new QPushButton(ocularsTab);
        addOcular->setObjectName("addOcular");
        addOcular->setMinimumSize(QSize(0, 24));
        addOcular->setFocusPolicy(Qt::NoFocus);

        horizontalLayout_10->addWidget(addOcular);

        pushButtonMoveOcularUp = new QPushButton(ocularsTab);
        pushButtonMoveOcularUp->setObjectName("pushButtonMoveOcularUp");
        sizePolicy3.setHeightForWidth(pushButtonMoveOcularUp->sizePolicy().hasHeightForWidth());
        pushButtonMoveOcularUp->setSizePolicy(sizePolicy3);
        pushButtonMoveOcularUp->setMinimumSize(QSize(24, 24));
        pushButtonMoveOcularUp->setFocusPolicy(Qt::NoFocus);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/graphicGui/uibtUp.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        icon.addFile(QString::fromUtf8(":/graphicGui/uibtUp-disabled.png"), QSize(), QIcon::Mode::Disabled, QIcon::State::Off);
        pushButtonMoveOcularUp->setIcon(icon);

        horizontalLayout_10->addWidget(pushButtonMoveOcularUp);

        pushButtonMoveOcularDown = new QPushButton(ocularsTab);
        pushButtonMoveOcularDown->setObjectName("pushButtonMoveOcularDown");
        sizePolicy3.setHeightForWidth(pushButtonMoveOcularDown->sizePolicy().hasHeightForWidth());
        pushButtonMoveOcularDown->setSizePolicy(sizePolicy3);
        pushButtonMoveOcularDown->setMinimumSize(QSize(24, 24));
        pushButtonMoveOcularDown->setFocusPolicy(Qt::NoFocus);
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/graphicGui/uibtDown.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        icon1.addFile(QString::fromUtf8(":/graphicGui/uibtDown-disabled.png"), QSize(), QIcon::Mode::Disabled, QIcon::State::Off);
        pushButtonMoveOcularDown->setIcon(icon1);

        horizontalLayout_10->addWidget(pushButtonMoveOcularDown);

        deleteOcular = new QPushButton(ocularsTab);
        deleteOcular->setObjectName("deleteOcular");
        deleteOcular->setMinimumSize(QSize(0, 24));
        deleteOcular->setFocusPolicy(Qt::NoFocus);

        horizontalLayout_10->addWidget(deleteOcular);


        verticalLayout_7->addLayout(horizontalLayout_10);


        horizontalLayout_9->addLayout(verticalLayout_7);

        gridLayout_11 = new QGridLayout();
        gridLayout_11->setObjectName("gridLayout_11");
        labelFS = new QLabel(ocularsTab);
        labelFS->setObjectName("labelFS");

        gridLayout_11->addWidget(labelFS, 3, 0, 1, 1);

        ocularName = new QLineEdit(ocularsTab);
        ocularName->setObjectName("ocularName");
        QSizePolicy sizePolicy4(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy4.setHorizontalStretch(0);
        sizePolicy4.setVerticalStretch(0);
        sizePolicy4.setHeightForWidth(ocularName->sizePolicy().hasHeightForWidth());
        ocularName->setSizePolicy(sizePolicy4);
        ocularName->setBaseSize(QSize(100, 22));
        ocularName->setMaxLength(50);

        gridLayout_11->addWidget(ocularName, 0, 1, 1, 1);

        ocularFL = new QDoubleSpinBox(ocularsTab);
        ocularFL->setObjectName("ocularFL");
        ocularFL->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        ocularFL->setDecimals(2);
        ocularFL->setMinimum(1.000000000000000);
        ocularFL->setMaximum(500.000000000000000);
        ocularFL->setSingleStep(0.100000000000000);

        gridLayout_11->addWidget(ocularFL, 2, 1, 1, 1);

        permanentCrosshairCheckBox = new QCheckBox(ocularsTab);
        permanentCrosshairCheckBox->setObjectName("permanentCrosshairCheckBox");

        gridLayout_11->addWidget(permanentCrosshairCheckBox, 4, 1, 1, 1);

        binocularsCheckBox = new QCheckBox(ocularsTab);
        binocularsCheckBox->setObjectName("binocularsCheckBox");

        gridLayout_11->addWidget(binocularsCheckBox, 4, 0, 1, 1);

        ocularAFov = new QDoubleSpinBox(ocularsTab);
        ocularAFov->setObjectName("ocularAFov");
        ocularAFov->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        ocularAFov->setDecimals(2);
        ocularAFov->setMinimum(1.000000000000000);
        ocularAFov->setMaximum(120.000000000000000);
        ocularAFov->setSingleStep(0.100000000000000);

        gridLayout_11->addWidget(ocularAFov, 1, 1, 1, 1);

        labelFOV = new QLabel(ocularsTab);
        labelFOV->setObjectName("labelFOV");

        gridLayout_11->addWidget(labelFOV, 1, 0, 1, 1);

        labelFL = new QLabel(ocularsTab);
        labelFL->setObjectName("labelFL");

        gridLayout_11->addWidget(labelFL, 2, 0, 1, 1);

        ocularFieldStop = new QDoubleSpinBox(ocularsTab);
        ocularFieldStop->setObjectName("ocularFieldStop");
        ocularFieldStop->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        ocularFieldStop->setDecimals(3);
        ocularFieldStop->setMinimum(0.000000000000000);
        ocularFieldStop->setMaximum(200.000000000000000);
        ocularFieldStop->setSingleStep(0.100000000000000);
        ocularFieldStop->setValue(0.000000000000000);

        gridLayout_11->addWidget(ocularFieldStop, 3, 1, 1, 1);

        labelName = new QLabel(ocularsTab);
        labelName->setObjectName("labelName");

        gridLayout_11->addWidget(labelName, 0, 0, 1, 1);

        verticalSpacer = new QSpacerItem(0, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_11->addItem(verticalSpacer, 5, 0, 1, 1);


        horizontalLayout_9->addLayout(gridLayout_11);


        gridLayout_2->addLayout(horizontalLayout_9, 0, 0, 1, 1);

        Tabs->addTab(ocularsTab, QString());
        lensesTab = new QWidget();
        lensesTab->setObjectName("lensesTab");
        gridLayout_3 = new QGridLayout(lensesTab);
        gridLayout_3->setObjectName("gridLayout_3");
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName("verticalLayout");
        lensListView = new QListView(lensesTab);
        lensListView->setObjectName("lensListView");
        lensListView->setEditTriggers(QAbstractItemView::EditTrigger::NoEditTriggers);
        lensListView->setProperty("isWrapping", QVariant(false));
        lensListView->setResizeMode(QListView::ResizeMode::Fixed);
        lensListView->setViewMode(QListView::ViewMode::ListMode);

        verticalLayout->addWidget(lensListView);

        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        addLens = new QPushButton(lensesTab);
        addLens->setObjectName("addLens");
        addLens->setMinimumSize(QSize(0, 24));
        addLens->setFocusPolicy(Qt::NoFocus);

        horizontalLayout_5->addWidget(addLens);

        pushButtonMoveLensUp = new QPushButton(lensesTab);
        pushButtonMoveLensUp->setObjectName("pushButtonMoveLensUp");
        sizePolicy3.setHeightForWidth(pushButtonMoveLensUp->sizePolicy().hasHeightForWidth());
        pushButtonMoveLensUp->setSizePolicy(sizePolicy3);
        pushButtonMoveLensUp->setMinimumSize(QSize(24, 24));
        pushButtonMoveLensUp->setFocusPolicy(Qt::NoFocus);
        pushButtonMoveLensUp->setIcon(icon);

        horizontalLayout_5->addWidget(pushButtonMoveLensUp);

        pushButtonMoveLensDown = new QPushButton(lensesTab);
        pushButtonMoveLensDown->setObjectName("pushButtonMoveLensDown");
        sizePolicy3.setHeightForWidth(pushButtonMoveLensDown->sizePolicy().hasHeightForWidth());
        pushButtonMoveLensDown->setSizePolicy(sizePolicy3);
        pushButtonMoveLensDown->setMinimumSize(QSize(24, 24));
        pushButtonMoveLensDown->setFocusPolicy(Qt::NoFocus);
        pushButtonMoveLensDown->setIcon(icon1);

        horizontalLayout_5->addWidget(pushButtonMoveLensDown);

        deleteLens = new QPushButton(lensesTab);
        deleteLens->setObjectName("deleteLens");
        deleteLens->setMinimumSize(QSize(0, 24));
        deleteLens->setFocusPolicy(Qt::NoFocus);

        horizontalLayout_5->addWidget(deleteLens);


        verticalLayout->addLayout(horizontalLayout_5);


        horizontalLayout_2->addLayout(verticalLayout);

        gridLayout_12 = new QGridLayout();
        gridLayout_12->setObjectName("gridLayout_12");
        label_7 = new QLabel(lensesTab);
        label_7->setObjectName("label_7");

        gridLayout_12->addWidget(label_7, 0, 0, 1, 1);

        lensMultiplier = new QDoubleSpinBox(lensesTab);
        lensMultiplier->setObjectName("lensMultiplier");
        lensMultiplier->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        lensMultiplier->setDecimals(3);
        lensMultiplier->setMinimum(0.010000000000000);
        lensMultiplier->setMaximum(9.000000000000000);
        lensMultiplier->setSingleStep(0.010000000000000);

        gridLayout_12->addWidget(lensMultiplier, 1, 1, 1, 1);

        lensNote = new QLabel(lensesTab);
        lensNote->setObjectName("lensNote");
#if QT_CONFIG(tooltip)
        lensNote->setToolTip(QString::fromUtf8(""));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        lensNote->setStatusTip(QString::fromUtf8(""));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(accessibility)
        lensNote->setAccessibleName(QString::fromUtf8(""));
#endif // QT_CONFIG(accessibility)
#if QT_CONFIG(accessibility)
        lensNote->setAccessibleDescription(QString::fromUtf8(""));
#endif // QT_CONFIG(accessibility)
        lensNote->setTextFormat(Qt::TextFormat::AutoText);
        lensNote->setScaledContents(false);
        lensNote->setWordWrap(true);
        lensNote->setIndent(9);

        gridLayout_12->addWidget(lensNote, 2, 0, 1, 2);

        lensName = new QLineEdit(lensesTab);
        lensName->setObjectName("lensName");
        lensName->setBaseSize(QSize(100, 22));
        lensName->setMaxLength(50);

        gridLayout_12->addWidget(lensName, 0, 1, 1, 1);

        label_13 = new QLabel(lensesTab);
        label_13->setObjectName("label_13");

        gridLayout_12->addWidget(label_13, 1, 0, 1, 1);

        verticalSpacer_4 = new QSpacerItem(0, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_12->addItem(verticalSpacer_4, 3, 0, 1, 1);


        horizontalLayout_2->addLayout(gridLayout_12);


        gridLayout_3->addLayout(horizontalLayout_2, 0, 0, 1, 1);

        Tabs->addTab(lensesTab, QString());
        sensorsTab = new QWidget();
        sensorsTab->setObjectName("sensorsTab");
        gridLayout_4 = new QGridLayout(sensorsTab);
        gridLayout_4->setObjectName("gridLayout_4");
        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        verticalLayout_5 = new QVBoxLayout();
        verticalLayout_5->setObjectName("verticalLayout_5");
        ccdListView = new QListView(sensorsTab);
        ccdListView->setObjectName("ccdListView");
        ccdListView->setEditTriggers(QAbstractItemView::EditTrigger::NoEditTriggers);
        ccdListView->setProperty("isWrapping", QVariant(false));
        ccdListView->setViewMode(QListView::ViewMode::ListMode);
        ccdListView->setModelColumn(0);

        verticalLayout_5->addWidget(ccdListView);

        horizontalLayout_11 = new QHBoxLayout();
        horizontalLayout_11->setObjectName("horizontalLayout_11");
        addCCD = new QPushButton(sensorsTab);
        addCCD->setObjectName("addCCD");
        addCCD->setMinimumSize(QSize(0, 24));
        addCCD->setFocusPolicy(Qt::NoFocus);

        horizontalLayout_11->addWidget(addCCD);

        pushButtonMoveSensorUp = new QPushButton(sensorsTab);
        pushButtonMoveSensorUp->setObjectName("pushButtonMoveSensorUp");
        sizePolicy3.setHeightForWidth(pushButtonMoveSensorUp->sizePolicy().hasHeightForWidth());
        pushButtonMoveSensorUp->setSizePolicy(sizePolicy3);
        pushButtonMoveSensorUp->setMinimumSize(QSize(24, 24));
        pushButtonMoveSensorUp->setFocusPolicy(Qt::NoFocus);
        pushButtonMoveSensorUp->setIcon(icon);

        horizontalLayout_11->addWidget(pushButtonMoveSensorUp);

        pushButtonMoveSensorDown = new QPushButton(sensorsTab);
        pushButtonMoveSensorDown->setObjectName("pushButtonMoveSensorDown");
        sizePolicy3.setHeightForWidth(pushButtonMoveSensorDown->sizePolicy().hasHeightForWidth());
        pushButtonMoveSensorDown->setSizePolicy(sizePolicy3);
        pushButtonMoveSensorDown->setMinimumSize(QSize(24, 24));
        pushButtonMoveSensorDown->setFocusPolicy(Qt::NoFocus);
        pushButtonMoveSensorDown->setIcon(icon1);

        horizontalLayout_11->addWidget(pushButtonMoveSensorDown);

        deleteCCD = new QPushButton(sensorsTab);
        deleteCCD->setObjectName("deleteCCD");
        deleteCCD->setMinimumSize(QSize(0, 24));
        deleteCCD->setFocusPolicy(Qt::NoFocus);

        horizontalLayout_11->addWidget(deleteCCD);


        verticalLayout_5->addLayout(horizontalLayout_11);


        horizontalLayout_3->addLayout(verticalLayout_5);

        gridLayout_13 = new QGridLayout();
        gridLayout_13->setObjectName("gridLayout_13");
        OAGPrismW = new QDoubleSpinBox(sensorsTab);
        OAGPrismW->setObjectName("OAGPrismW");
        OAGPrismW->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        OAGPrismW->setDecimals(2);
        OAGPrismW->setSingleStep(0.100000000000000);

        gridLayout_13->addWidget(OAGPrismW, 11, 1, 1, 1);

        OAGPrismH = new QDoubleSpinBox(sensorsTab);
        OAGPrismH->setObjectName("OAGPrismH");
        OAGPrismH->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        OAGPrismH->setDecimals(2);
        OAGPrismH->setSingleStep(0.100000000000000);

        gridLayout_13->addWidget(OAGPrismH, 10, 1, 1, 1);

        OAG_checkBox = new QCheckBox(sensorsTab);
        OAG_checkBox->setObjectName("OAG_checkBox");

        gridLayout_13->addWidget(OAG_checkBox, 8, 0, 1, 2);

        ccdChipX = new QDoubleSpinBox(sensorsTab);
        ccdChipX->setObjectName("ccdChipX");
        ccdChipX->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        ccdChipX->setDecimals(2);
        ccdChipX->setMinimum(0.100000000000000);
        ccdChipX->setMaximum(9999.000000000000000);
        ccdChipX->setSingleStep(0.100000000000000);

        gridLayout_13->addWidget(ccdChipX, 3, 1, 1, 1);

        ccdBinningY = new QSpinBox(sensorsTab);
        ccdBinningY->setObjectName("ccdBinningY");
        ccdBinningY->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        ccdBinningY->setMinimum(1);
        ccdBinningY->setMaximum(4);

        gridLayout_13->addWidget(ccdBinningY, 7, 1, 1, 1);

        ccdResY = new QSpinBox(sensorsTab);
        ccdResY->setObjectName("ccdResY");
        ccdResY->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        ccdResY->setMinimum(1);
        ccdResY->setMaximum(100000);

        gridLayout_13->addWidget(ccdResY, 2, 1, 1, 1);

        OAGPrismPA = new QDoubleSpinBox(sensorsTab);
        OAGPrismPA->setObjectName("OAGPrismPA");
        OAGPrismPA->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        OAGPrismPA->setDecimals(0);
        OAGPrismPA->setMinimum(0.000000000000000);
        OAGPrismPA->setMaximum(360.000000000000000);
        OAGPrismPA->setSingleStep(1.000000000000000);

        gridLayout_13->addWidget(OAGPrismPA, 12, 1, 1, 1);

        ccdRotAngle = new QDoubleSpinBox(sensorsTab);
        ccdRotAngle->setObjectName("ccdRotAngle");
        ccdRotAngle->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        ccdRotAngle->setDecimals(0);
        ccdRotAngle->setMinimum(-360.000000000000000);
        ccdRotAngle->setMaximum(360.000000000000000);

        gridLayout_13->addWidget(ccdRotAngle, 5, 1, 1, 1);

        CCDResY_lbl = new QLabel(sensorsTab);
        CCDResY_lbl->setObjectName("CCDResY_lbl");

        gridLayout_13->addWidget(CCDResY_lbl, 2, 0, 1, 1);

        ccdBinningX_lbl = new QLabel(sensorsTab);
        ccdBinningX_lbl->setObjectName("ccdBinningX_lbl");

        gridLayout_13->addWidget(ccdBinningX_lbl, 6, 0, 1, 1);

        CCDResX_lbl = new QLabel(sensorsTab);
        CCDResX_lbl->setObjectName("CCDResX_lbl");

        gridLayout_13->addWidget(CCDResX_lbl, 1, 0, 1, 1);

        CCDName_lbl = new QLabel(sensorsTab);
        CCDName_lbl->setObjectName("CCDName_lbl");

        gridLayout_13->addWidget(CCDName_lbl, 0, 0, 1, 1);

        OAGPrismH_lbl = new QLabel(sensorsTab);
        OAGPrismH_lbl->setObjectName("OAGPrismH_lbl");

        gridLayout_13->addWidget(OAGPrismH_lbl, 10, 0, 1, 1);

        ccdBinningX = new QSpinBox(sensorsTab);
        ccdBinningX->setObjectName("ccdBinningX");
        ccdBinningX->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        ccdBinningX->setMinimum(1);
        ccdBinningX->setMaximum(4);
        ccdBinningX->setValue(1);

        gridLayout_13->addWidget(ccdBinningX, 6, 1, 1, 1);

        OAGDist_lbl = new QLabel(sensorsTab);
        OAGDist_lbl->setObjectName("OAGDist_lbl");

        gridLayout_13->addWidget(OAGDist_lbl, 9, 0, 1, 1);

        ccdName = new QLineEdit(sensorsTab);
        ccdName->setObjectName("ccdName");
        sizePolicy4.setHeightForWidth(ccdName->sizePolicy().hasHeightForWidth());
        ccdName->setSizePolicy(sizePolicy4);
        ccdName->setBaseSize(QSize(100, 22));
        ccdName->setMaxLength(50);

        gridLayout_13->addWidget(ccdName, 0, 1, 1, 1);

        CCDverticalSpacer = new QSpacerItem(0, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_13->addItem(CCDverticalSpacer, 13, 0, 1, 1);

        ccdChipY = new QDoubleSpinBox(sensorsTab);
        ccdChipY->setObjectName("ccdChipY");
        ccdChipY->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        ccdChipY->setDecimals(2);
        ccdChipY->setMinimum(0.100000000000000);
        ccdChipY->setMaximum(9999.000000000000000);
        ccdChipY->setSingleStep(0.100000000000000);

        gridLayout_13->addWidget(ccdChipY, 4, 1, 1, 1);

        OAGDist = new QDoubleSpinBox(sensorsTab);
        OAGDist->setObjectName("OAGDist");
        OAGDist->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        OAGDist->setDecimals(2);
        OAGDist->setSingleStep(0.100000000000000);

        gridLayout_13->addWidget(OAGDist, 9, 1, 1, 1);

        ccdResX = new QSpinBox(sensorsTab);
        ccdResX->setObjectName("ccdResX");
        ccdResX->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        ccdResX->setMinimum(1);
        ccdResX->setMaximum(100000);

        gridLayout_13->addWidget(ccdResX, 1, 1, 1, 1);

        ccdRotAngle_lbl = new QLabel(sensorsTab);
        ccdRotAngle_lbl->setObjectName("ccdRotAngle_lbl");

        gridLayout_13->addWidget(ccdRotAngle_lbl, 5, 0, 1, 1);

        ccdBinningY_lbl = new QLabel(sensorsTab);
        ccdBinningY_lbl->setObjectName("ccdBinningY_lbl");

        gridLayout_13->addWidget(ccdBinningY_lbl, 7, 0, 1, 1);

        OAGPrismW_lbl = new QLabel(sensorsTab);
        OAGPrismW_lbl->setObjectName("OAGPrismW_lbl");

        gridLayout_13->addWidget(OAGPrismW_lbl, 11, 0, 1, 1);

        CCDChipY_lbl = new QLabel(sensorsTab);
        CCDChipY_lbl->setObjectName("CCDChipY_lbl");

        gridLayout_13->addWidget(CCDChipY_lbl, 4, 0, 1, 1);

        CCDChipX_lbl = new QLabel(sensorsTab);
        CCDChipX_lbl->setObjectName("CCDChipX_lbl");

        gridLayout_13->addWidget(CCDChipX_lbl, 3, 0, 1, 1);

        OAGPrismPA_lbl = new QLabel(sensorsTab);
        OAGPrismPA_lbl->setObjectName("OAGPrismPA_lbl");

        gridLayout_13->addWidget(OAGPrismPA_lbl, 12, 0, 1, 1);


        horizontalLayout_3->addLayout(gridLayout_13);


        gridLayout_4->addLayout(horizontalLayout_3, 0, 0, 1, 1);

        Tabs->addTab(sensorsTab, QString());
        telescopesTab = new QWidget();
        telescopesTab->setObjectName("telescopesTab");
        horizontalLayout_6 = new QHBoxLayout(telescopesTab);
        horizontalLayout_6->setObjectName("horizontalLayout_6");
        horizontalLayout_12 = new QHBoxLayout();
        horizontalLayout_12->setObjectName("horizontalLayout_12");
        verticalLayout_8 = new QVBoxLayout();
        verticalLayout_8->setObjectName("verticalLayout_8");
        telescopeListView = new QListView(telescopesTab);
        telescopeListView->setObjectName("telescopeListView");
        telescopeListView->setEditTriggers(QAbstractItemView::EditTrigger::NoEditTriggers);
        telescopeListView->setModelColumn(0);

        verticalLayout_8->addWidget(telescopeListView);

        horizontalLayout_13 = new QHBoxLayout();
        horizontalLayout_13->setObjectName("horizontalLayout_13");
        addTelescope = new QPushButton(telescopesTab);
        addTelescope->setObjectName("addTelescope");
        addTelescope->setMinimumSize(QSize(0, 24));
        addTelescope->setFocusPolicy(Qt::NoFocus);

        horizontalLayout_13->addWidget(addTelescope);

        pushButtonMoveTelescopeUp = new QPushButton(telescopesTab);
        pushButtonMoveTelescopeUp->setObjectName("pushButtonMoveTelescopeUp");
        sizePolicy3.setHeightForWidth(pushButtonMoveTelescopeUp->sizePolicy().hasHeightForWidth());
        pushButtonMoveTelescopeUp->setSizePolicy(sizePolicy3);
        pushButtonMoveTelescopeUp->setMinimumSize(QSize(24, 24));
        pushButtonMoveTelescopeUp->setFocusPolicy(Qt::NoFocus);
        pushButtonMoveTelescopeUp->setIcon(icon);

        horizontalLayout_13->addWidget(pushButtonMoveTelescopeUp);

        pushButtonMoveTelescopeDown = new QPushButton(telescopesTab);
        pushButtonMoveTelescopeDown->setObjectName("pushButtonMoveTelescopeDown");
        sizePolicy3.setHeightForWidth(pushButtonMoveTelescopeDown->sizePolicy().hasHeightForWidth());
        pushButtonMoveTelescopeDown->setSizePolicy(sizePolicy3);
        pushButtonMoveTelescopeDown->setMinimumSize(QSize(24, 24));
        pushButtonMoveTelescopeDown->setFocusPolicy(Qt::NoFocus);
        pushButtonMoveTelescopeDown->setIcon(icon1);

        horizontalLayout_13->addWidget(pushButtonMoveTelescopeDown);

        deleteTelescope = new QPushButton(telescopesTab);
        deleteTelescope->setObjectName("deleteTelescope");
        deleteTelescope->setMinimumSize(QSize(0, 24));
        deleteTelescope->setFocusPolicy(Qt::NoFocus);

        horizontalLayout_13->addWidget(deleteTelescope);


        verticalLayout_8->addLayout(horizontalLayout_13);


        horizontalLayout_12->addLayout(verticalLayout_8);

        gridLayout_14 = new QGridLayout();
        gridLayout_14->setObjectName("gridLayout_14");
        label_8 = new QLabel(telescopesTab);
        label_8->setObjectName("label_8");

        gridLayout_14->addWidget(label_8, 0, 0, 1, 1);

        telescopeFL = new QDoubleSpinBox(telescopesTab);
        telescopeFL->setObjectName("telescopeFL");
        telescopeFL->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        telescopeFL->setDecimals(2);
        telescopeFL->setMinimum(1.000000000000000);
        telescopeFL->setMaximum(9999999.000000000000000);
        telescopeFL->setSingleStep(0.100000000000000);

        gridLayout_14->addWidget(telescopeFL, 1, 1, 1, 1);

        telescopeDiameter = new QDoubleSpinBox(telescopesTab);
        telescopeDiameter->setObjectName("telescopeDiameter");
        telescopeDiameter->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        telescopeDiameter->setDecimals(2);
        telescopeDiameter->setMinimum(1.000000000000000);
        telescopeDiameter->setMaximum(999999.000000000000000);
        telescopeDiameter->setSingleStep(0.100000000000000);

        gridLayout_14->addWidget(telescopeDiameter, 2, 1, 1, 1);

        telescopeHFlip = new QCheckBox(telescopesTab);
        telescopeHFlip->setObjectName("telescopeHFlip");

        gridLayout_14->addWidget(telescopeHFlip, 3, 0, 1, 2);

        telescopeVFlip = new QCheckBox(telescopesTab);
        telescopeVFlip->setObjectName("telescopeVFlip");

        gridLayout_14->addWidget(telescopeVFlip, 4, 0, 1, 2);

        telescopeEQ = new QCheckBox(telescopesTab);
        telescopeEQ->setObjectName("telescopeEQ");

        gridLayout_14->addWidget(telescopeEQ, 5, 0, 1, 2);

        telescopeName = new QLineEdit(telescopesTab);
        telescopeName->setObjectName("telescopeName");
        telescopeName->setBaseSize(QSize(100, 22));
        telescopeName->setMaxLength(50);

        gridLayout_14->addWidget(telescopeName, 0, 1, 1, 1);

        label_12 = new QLabel(telescopesTab);
        label_12->setObjectName("label_12");

        gridLayout_14->addWidget(label_12, 2, 0, 1, 1);

        label_9 = new QLabel(telescopesTab);
        label_9->setObjectName("label_9");

        gridLayout_14->addWidget(label_9, 1, 0, 1, 1);

        verticalSpacer_2 = new QSpacerItem(0, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_14->addItem(verticalSpacer_2, 6, 0, 1, 1);


        horizontalLayout_12->addLayout(gridLayout_14);


        horizontalLayout_6->addLayout(horizontalLayout_12);

        Tabs->addTab(telescopesTab, QString());
        aboutTab = new QWidget();
        aboutTab->setObjectName("aboutTab");
        gridLayout = new QGridLayout(aboutTab);
        gridLayout->setObjectName("gridLayout");
        textBrowser = new QTextBrowser(aboutTab);
        textBrowser->setObjectName("textBrowser");
        textBrowser->setOpenExternalLinks(true);

        gridLayout->addWidget(textBrowser, 0, 0, 1, 1);

        Tabs->addTab(aboutTab, QString());

        verticalLayout_4->addWidget(Tabs);


        verticalLayout_2->addWidget(Contents);

        QWidget::setTabOrder(checkBoxControlPanel, guiFontSizeSpinBox);
        QWidget::setTabOrder(guiFontSizeSpinBox, checkBoxInitialFOV);
        QWidget::setTabOrder(checkBoxInitialFOV, checkBoxInitialDirection);
        QWidget::setTabOrder(checkBoxInitialDirection, checkBoxResolutionCriterion);
        QWidget::setTabOrder(checkBoxResolutionCriterion, checkBoxToolbarButton);
        QWidget::setTabOrder(checkBoxToolbarButton, arrowButtonScaleSpinBox);
        QWidget::setTabOrder(arrowButtonScaleSpinBox, lineColorToolButton);
        QWidget::setTabOrder(lineColorToolButton, textColorToolButton);
        QWidget::setTabOrder(textColorToolButton, checkBoxCCDFOVScaling);
        QWidget::setTabOrder(checkBoxCCDFOVScaling, checkBoxDMSDegrees);
        QWidget::setTabOrder(checkBoxDMSDegrees, checkBoxTypeOfMount);
        QWidget::setTabOrder(checkBoxTypeOfMount, guiCcdCropOverlayHSizeSpinBox);
        QWidget::setTabOrder(guiCcdCropOverlayHSizeSpinBox, guiCcdCropOverlayVSizeSpinBox);
        QWidget::setTabOrder(guiCcdCropOverlayVSizeSpinBox, checkBoxShowFocuserOverlay);
        QWidget::setTabOrder(checkBoxShowFocuserOverlay, checkBoxUseSmallFocuser);
        QWidget::setTabOrder(checkBoxUseSmallFocuser, checkBoxUseMediumFocuser);
        QWidget::setTabOrder(checkBoxUseMediumFocuser, checkBoxUseLargeFocuser);
        QWidget::setTabOrder(checkBoxUseLargeFocuser, requireSelectionCheckBox);
        QWidget::setTabOrder(requireSelectionCheckBox, scaleImageCircleCheckBox);
        QWidget::setTabOrder(scaleImageCircleCheckBox, limitStellarMagnitudeCheckBox);
        QWidget::setTabOrder(limitStellarMagnitudeCheckBox, hideGridsLinesCheckBox);
        QWidget::setTabOrder(hideGridsLinesCheckBox, contourCheckBox);
        QWidget::setTabOrder(contourCheckBox, cardinalsCheckBox);
        QWidget::setTabOrder(cardinalsCheckBox, alignCrosshairCheckBox);
        QWidget::setTabOrder(alignCrosshairCheckBox, checkBoxTelradFOVScaling);
        QWidget::setTabOrder(checkBoxTelradFOVScaling, ocularName);
        QWidget::setTabOrder(ocularName, ocularAFov);
        QWidget::setTabOrder(ocularAFov, ocularFL);
        QWidget::setTabOrder(ocularFL, ocularFieldStop);
        QWidget::setTabOrder(ocularFieldStop, binocularsCheckBox);
        QWidget::setTabOrder(binocularsCheckBox, permanentCrosshairCheckBox);
        QWidget::setTabOrder(permanentCrosshairCheckBox, ocularListView);
        QWidget::setTabOrder(ocularListView, addOcular);
        QWidget::setTabOrder(addOcular, pushButtonMoveOcularUp);
        QWidget::setTabOrder(pushButtonMoveOcularUp, pushButtonMoveOcularDown);
        QWidget::setTabOrder(pushButtonMoveOcularDown, deleteOcular);
        QWidget::setTabOrder(deleteOcular, lensName);
        QWidget::setTabOrder(lensName, lensMultiplier);
        QWidget::setTabOrder(lensMultiplier, lensListView);
        QWidget::setTabOrder(lensListView, addLens);
        QWidget::setTabOrder(addLens, pushButtonMoveLensUp);
        QWidget::setTabOrder(pushButtonMoveLensUp, pushButtonMoveLensDown);
        QWidget::setTabOrder(pushButtonMoveLensDown, deleteLens);
        QWidget::setTabOrder(deleteLens, ccdName);
        QWidget::setTabOrder(ccdName, ccdResX);
        QWidget::setTabOrder(ccdResX, ccdResY);
        QWidget::setTabOrder(ccdResY, ccdChipX);
        QWidget::setTabOrder(ccdChipX, ccdChipY);
        QWidget::setTabOrder(ccdChipY, ccdRotAngle);
        QWidget::setTabOrder(ccdRotAngle, ccdBinningX);
        QWidget::setTabOrder(ccdBinningX, ccdBinningY);
        QWidget::setTabOrder(ccdBinningY, OAG_checkBox);
        QWidget::setTabOrder(OAG_checkBox, OAGDist);
        QWidget::setTabOrder(OAGDist, OAGPrismH);
        QWidget::setTabOrder(OAGPrismH, OAGPrismW);
        QWidget::setTabOrder(OAGPrismW, OAGPrismPA);
        QWidget::setTabOrder(OAGPrismPA, ccdListView);
        QWidget::setTabOrder(ccdListView, addCCD);
        QWidget::setTabOrder(addCCD, pushButtonMoveSensorUp);
        QWidget::setTabOrder(pushButtonMoveSensorUp, pushButtonMoveSensorDown);
        QWidget::setTabOrder(pushButtonMoveSensorDown, deleteCCD);
        QWidget::setTabOrder(deleteCCD, telescopeName);
        QWidget::setTabOrder(telescopeName, telescopeFL);
        QWidget::setTabOrder(telescopeFL, telescopeDiameter);
        QWidget::setTabOrder(telescopeDiameter, telescopeHFlip);
        QWidget::setTabOrder(telescopeHFlip, telescopeVFlip);
        QWidget::setTabOrder(telescopeVFlip, telescopeEQ);
        QWidget::setTabOrder(telescopeEQ, telescopeListView);
        QWidget::setTabOrder(telescopeListView, addTelescope);
        QWidget::setTabOrder(addTelescope, pushButtonMoveTelescopeUp);
        QWidget::setTabOrder(pushButtonMoveTelescopeUp, pushButtonMoveTelescopeDown);
        QWidget::setTabOrder(pushButtonMoveTelescopeDown, deleteTelescope);
        QWidget::setTabOrder(deleteTelescope, textBrowser);

        retranslateUi(ocularDialogForm);

        Tabs->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(ocularDialogForm);
    } // setupUi

    void retranslateUi(QWidget *ocularDialogForm)
    {
        titleBar->setProperty("title", QVariant(QCoreApplication::translate("ocularDialogForm", "Oculars", nullptr)));
        groupBoxInterface->setTitle(QCoreApplication::translate("ocularDialogForm", "Interface", nullptr));
#if QT_CONFIG(tooltip)
        arrowButtonScaleLabel->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
        arrowButtonScaleLabel->setText(QCoreApplication::translate("ocularDialogForm", "Arrow button scale (requires restart)", nullptr));
#if QT_CONFIG(tooltip)
        checkBoxInitialFOV->setToolTip(QCoreApplication::translate("ocularDialogForm", "Mark it if you want the FOV value the same like at the startup of Stellarium.", nullptr));
#endif // QT_CONFIG(tooltip)
        checkBoxInitialFOV->setText(QCoreApplication::translate("ocularDialogForm", "Restore FOV to initial values", nullptr));
        checkBoxResolutionCriterion->setText(QCoreApplication::translate("ocularDialogForm", "Show resolution criteria", nullptr));
        checkBoxControlPanel->setText(QCoreApplication::translate("ocularDialogForm", "On-screen control panel", nullptr));
#if QT_CONFIG(tooltip)
        guiFontSizeLabel->setToolTip(QCoreApplication::translate("ocularDialogForm", "Font size of text on on-screen control panel", nullptr));
#endif // QT_CONFIG(tooltip)
        guiFontSizeLabel->setText(QCoreApplication::translate("ocularDialogForm", "...font size", nullptr));
#if QT_CONFIG(tooltip)
        guiFontSizeSpinBox->setToolTip(QCoreApplication::translate("ocularDialogForm", "Font size of text on on-screen control panel", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        checkBoxInitialDirection->setToolTip(QCoreApplication::translate("ocularDialogForm", "Mark it if you want the direction value the same like at the startup of Stellarium.", nullptr));
#endif // QT_CONFIG(tooltip)
        checkBoxInitialDirection->setText(QCoreApplication::translate("ocularDialogForm", "Restore direction to initial values", nullptr));
#if QT_CONFIG(tooltip)
        lineColorToolButton->setToolTip(QCoreApplication::translate("ocularDialogForm", "Color for ocular circles and sensor frames", nullptr));
#endif // QT_CONFIG(tooltip)
        lineColorToolButton->setText(QString());
        lineColorLabel->setText(QCoreApplication::translate("ocularDialogForm", "Line Color", nullptr));
#if QT_CONFIG(tooltip)
        textColorToolButton->setToolTip(QCoreApplication::translate("ocularDialogForm", "Color for text which is shown only when GUI is switched off.", nullptr));
#endif // QT_CONFIG(tooltip)
        textColorToolButton->setText(QString());
        textColorLabel->setText(QCoreApplication::translate("ocularDialogForm", "Text Color", nullptr));
        checkBoxToolbarButton->setText(QCoreApplication::translate("ocularDialogForm", "Show oculars button on toolbar", nullptr));
        groupBoxOcularOptions->setTitle(QCoreApplication::translate("ocularDialogForm", "Ocular view", nullptr));
#if QT_CONFIG(tooltip)
        scaleImageCircleCheckBox->setToolTip(QCoreApplication::translate("ocularDialogForm", "Show FOV circles relative to the apparent FOV of the widest-angle ocular", nullptr));
#endif // QT_CONFIG(tooltip)
        scaleImageCircleCheckBox->setText(QCoreApplication::translate("ocularDialogForm", "Scale image circle", nullptr));
        hideGridsLinesCheckBox->setText(QCoreApplication::translate("ocularDialogForm", "Hide grids and lines when enabled", nullptr));
        requireSelectionCheckBox->setText(QCoreApplication::translate("ocularDialogForm", "Enable only if an object is selected", nullptr));
#if QT_CONFIG(tooltip)
        contourCheckBox->setToolTip(QCoreApplication::translate("ocularDialogForm", "Show border circle", nullptr));
#endif // QT_CONFIG(tooltip)
        contourCheckBox->setText(QCoreApplication::translate("ocularDialogForm", "Show FOV outline", nullptr));
#if QT_CONFIG(tooltip)
        limitStellarMagnitudeCheckBox->setToolTip(QCoreApplication::translate("ocularDialogForm", "Apply limit for stellar magnitude based on telescope aperture", nullptr));
#endif // QT_CONFIG(tooltip)
        limitStellarMagnitudeCheckBox->setText(QCoreApplication::translate("ocularDialogForm", "Auto-limit stellar magnitude", nullptr));
#if QT_CONFIG(tooltip)
        cardinalsCheckBox->setToolTip(QCoreApplication::translate("ocularDialogForm", "Show cardinal directions in equatorial coordinate system", nullptr));
#endif // QT_CONFIG(tooltip)
        cardinalsCheckBox->setText(QCoreApplication::translate("ocularDialogForm", "Show compass rose", nullptr));
#if QT_CONFIG(tooltip)
        alignCrosshairCheckBox->setToolTip(QCoreApplication::translate("ocularDialogForm", "Align the crosshair in equatorial coordinate system", nullptr));
#endif // QT_CONFIG(tooltip)
        alignCrosshairCheckBox->setText(QCoreApplication::translate("ocularDialogForm", "Align crosshair", nullptr));
#if QT_CONFIG(tooltip)
        semiTransparencyCheckBox->setToolTip(QCoreApplication::translate("ocularDialogForm", "In ocular mode, allow some visibility outside ocular circle.", nullptr));
#endif // QT_CONFIG(tooltip)
        semiTransparencyCheckBox->setText(QCoreApplication::translate("ocularDialogForm", "Use semi-transparent mask", nullptr));
#if QT_CONFIG(tooltip)
        transparencySpinBox->setToolTip(QCoreApplication::translate("ocularDialogForm", "Opacity of semi-transparent mask", nullptr));
#endif // QT_CONFIG(tooltip)
        groupBoxSensorOptions->setTitle(QCoreApplication::translate("ocularDialogForm", "Sensor view", nullptr));
        labelFocuserOverlay->setText(QCoreApplication::translate("ocularDialogForm", "Use size of focuser:", nullptr));
#if QT_CONFIG(tooltip)
        checkBoxShowCcdCropOverlay->setToolTip(QCoreApplication::translate("ocularDialogForm", "Enable a rectangular overlay showing the effective size of a number of pixels.", nullptr));
#endif // QT_CONFIG(tooltip)
        checkBoxShowCcdCropOverlay->setText(QCoreApplication::translate("ocularDialogForm", "Show sensor crop overlay", nullptr));
        guiCcdCropOverlaySizeLabel->setText(QCoreApplication::translate("ocularDialogForm", "Overlay size (px)", nullptr));
#if QT_CONFIG(tooltip)
        guiCcdCropOverlayHSizeSpinBox->setToolTip(QCoreApplication::translate("ocularDialogForm", "Horizontal size (width)", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        guiCcdCropOverlayVSizeSpinBox->setToolTip(QCoreApplication::translate("ocularDialogForm", "Vertical size (height)", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        checkBoxShowCcdCropOverlayPixelGrid->setToolTip(QCoreApplication::translate("ocularDialogForm", "Enable pixel grid of sensor binned pixels within sensor crop overlay.", nullptr));
#endif // QT_CONFIG(tooltip)
        checkBoxShowCcdCropOverlayPixelGrid->setText(QCoreApplication::translate("ocularDialogForm", "Show pixel grid", nullptr));
        checkBoxCCDFOVScaling->setText(QCoreApplication::translate("ocularDialogForm", "Enable autozoom when switching CCD", nullptr));
#if QT_CONFIG(tooltip)
        checkBoxCCDMaxExposureTime->setToolTip(QCoreApplication::translate("ocularDialogForm", "Show maximum exposure time for Solar system objects to avoid movement streak", nullptr));
#endif // QT_CONFIG(tooltip)
        checkBoxCCDMaxExposureTime->setText(QCoreApplication::translate("ocularDialogForm", "Show max exposure time for moving objects", nullptr));
        checkBoxDMSDegrees->setText(QCoreApplication::translate("ocularDialogForm", "Use degrees and minutes for FOV of CCD", nullptr));
#if QT_CONFIG(tooltip)
        checkBoxTypeOfMount->setToolTip(QCoreApplication::translate("ocularDialogForm", "Enable automatic switch of mount type from the telescope settings for horizontal orientation of CCD frame.", nullptr));
#endif // QT_CONFIG(tooltip)
        checkBoxTypeOfMount->setText(QCoreApplication::translate("ocularDialogForm", "Enable automatic switch of mount type", nullptr));
#if QT_CONFIG(tooltip)
        focuserColorToolButton->setToolTip(QCoreApplication::translate("ocularDialogForm", "Color for focuser overlay line", nullptr));
#endif // QT_CONFIG(tooltip)
        checkBoxShowFocuserOverlay->setText(QCoreApplication::translate("ocularDialogForm", "Show focuser overlay", nullptr));
        checkBoxHorizontalCoordinates->setText(QCoreApplication::translate("ocularDialogForm", "Use horizontal coordinates instead equatorial coordinates", nullptr));
        checkBoxShowOAGLimits->setText(QCoreApplication::translate("ocularDialogForm", "Show angular limits of the off-axis guide", nullptr));
        groupBoxTelradOptions->setTitle(QCoreApplication::translate("ocularDialogForm", "Telrad view", nullptr));
#if QT_CONFIG(tooltip)
        labelTelradFOV->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
        labelTelradFOV->setText(QCoreApplication::translate("ocularDialogForm", "FOV:", "abbreviation"));
#if QT_CONFIG(tooltip)
        doubleSpinBoxTelradFOV1->setToolTip(QCoreApplication::translate("ocularDialogForm", "FOV for circle 1", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        doubleSpinBoxTelradFOV2->setToolTip(QCoreApplication::translate("ocularDialogForm", "FOV for circle 2", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        doubleSpinBoxTelradFOV3->setToolTip(QCoreApplication::translate("ocularDialogForm", "FOV for circle 3", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        doubleSpinBoxTelradFOV4->setToolTip(QCoreApplication::translate("ocularDialogForm", "FOV for circle 4", nullptr));
#endif // QT_CONFIG(tooltip)
        pushButtonRestoreTelradFOV->setText(QCoreApplication::translate("ocularDialogForm", "Restore original Telrad FOV", nullptr));
        checkBoxTelradFOVScaling->setText(QCoreApplication::translate("ocularDialogForm", "Enable autozoom when switching Telrad", nullptr));
        Tabs->setTabText(Tabs->indexOf(generalTab), QCoreApplication::translate("ocularDialogForm", "General", nullptr));
        addOcular->setText(QCoreApplication::translate("ocularDialogForm", "Add", nullptr));
        deleteOcular->setText(QCoreApplication::translate("ocularDialogForm", "Delete", nullptr));
        labelFS->setText(QCoreApplication::translate("ocularDialogForm", "Field stop:", nullptr));
        permanentCrosshairCheckBox->setText(QCoreApplication::translate("ocularDialogForm", "Has permanent cross-hairs", nullptr));
        binocularsCheckBox->setText(QCoreApplication::translate("ocularDialogForm", "Binoculars", nullptr));
        labelFOV->setText(QCoreApplication::translate("ocularDialogForm", "aFOV:", nullptr));
        labelFL->setText(QCoreApplication::translate("ocularDialogForm", "Focal length:", nullptr));
        labelName->setText(QCoreApplication::translate("ocularDialogForm", "Name:", nullptr));
        Tabs->setTabText(Tabs->indexOf(ocularsTab), QCoreApplication::translate("ocularDialogForm", "Eyepieces", nullptr));
        addLens->setText(QCoreApplication::translate("ocularDialogForm", "Add", nullptr));
        deleteLens->setText(QCoreApplication::translate("ocularDialogForm", "Delete", nullptr));
        label_7->setText(QCoreApplication::translate("ocularDialogForm", "Name:", nullptr));
        lensNote->setText(QCoreApplication::translate("ocularDialogForm", "Values of multiplicity >1 expand the focal length (Barlow lens). Values of multiplicity <1 reduce the focal length (Shapley lens).", nullptr));
        label_13->setText(QCoreApplication::translate("ocularDialogForm", "Multiplier:", nullptr));
        Tabs->setTabText(Tabs->indexOf(lensesTab), QCoreApplication::translate("ocularDialogForm", "Lenses", nullptr));
        addCCD->setText(QCoreApplication::translate("ocularDialogForm", "Add", nullptr));
        deleteCCD->setText(QCoreApplication::translate("ocularDialogForm", "Delete", nullptr));
        OAG_checkBox->setText(QCoreApplication::translate("ocularDialogForm", "Off-Axis guider", nullptr));
        CCDResY_lbl->setText(QCoreApplication::translate("ocularDialogForm", "Resolution y:", nullptr));
        ccdBinningX_lbl->setText(QCoreApplication::translate("ocularDialogForm", "Binning x:", nullptr));
        CCDResX_lbl->setText(QCoreApplication::translate("ocularDialogForm", "Resolution x:", nullptr));
        CCDName_lbl->setText(QCoreApplication::translate("ocularDialogForm", "Name:", nullptr));
        OAGPrismH_lbl->setText(QCoreApplication::translate("ocularDialogForm", "Prism/CCD height:", nullptr));
#if QT_CONFIG(tooltip)
        OAGDist_lbl->setToolTip(QCoreApplication::translate("ocularDialogForm", "This is the distance from the center of the Imaging Sensor to the center of the Prism", nullptr));
#endif // QT_CONFIG(tooltip)
        OAGDist_lbl->setText(QCoreApplication::translate("ocularDialogForm", "Prism/CCD distance:", nullptr));
        ccdRotAngle_lbl->setText(QCoreApplication::translate("ocularDialogForm", "Rotation Angle:", nullptr));
        ccdBinningY_lbl->setText(QCoreApplication::translate("ocularDialogForm", "Binning y:", nullptr));
        OAGPrismW_lbl->setText(QCoreApplication::translate("ocularDialogForm", "Prism/CCD width:", nullptr));
        CCDChipY_lbl->setText(QCoreApplication::translate("ocularDialogForm", "Chip height:", nullptr));
        CCDChipX_lbl->setText(QCoreApplication::translate("ocularDialogForm", "Chip width:", nullptr));
        OAGPrismPA_lbl->setText(QCoreApplication::translate("ocularDialogForm", "Position angle:", nullptr));
        Tabs->setTabText(Tabs->indexOf(sensorsTab), QCoreApplication::translate("ocularDialogForm", "Sensors", nullptr));
        addTelescope->setText(QCoreApplication::translate("ocularDialogForm", "Add", nullptr));
        deleteTelescope->setText(QCoreApplication::translate("ocularDialogForm", "Delete", nullptr));
        label_8->setText(QCoreApplication::translate("ocularDialogForm", "Name:", nullptr));
        telescopeHFlip->setText(QCoreApplication::translate("ocularDialogForm", "Horizontal flip", nullptr));
        telescopeVFlip->setText(QCoreApplication::translate("ocularDialogForm", "Vertical flip", nullptr));
        telescopeEQ->setText(QCoreApplication::translate("ocularDialogForm", "Equatorial Mount", nullptr));
        label_12->setText(QCoreApplication::translate("ocularDialogForm", "Diameter:", nullptr));
        label_9->setText(QCoreApplication::translate("ocularDialogForm", "Focal length:", nullptr));
        Tabs->setTabText(Tabs->indexOf(telescopesTab), QCoreApplication::translate("ocularDialogForm", "Telescopes", nullptr));
        Tabs->setTabText(Tabs->indexOf(aboutTab), QCoreApplication::translate("ocularDialogForm", "About", "tab in plugin windows"));
        (void)ocularDialogForm;
    } // retranslateUi

};

namespace Ui {
    class ocularDialogForm: public Ui_ocularDialogForm {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_OCULARDIALOG_H
