/****************************************************************************
** Meta object code from reading C++ file 'Quasars.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../stellarium/plugins/Quasars/src/Quasars.hpp"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>
#include <QtCore/qplugin.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'Quasars.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN7QuasarsE_t {};
} // unnamed namespace

template <> constexpr inline auto Quasars::qt_create_metaobjectdata<qt_meta_tag_ZN7QuasarsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Quasars",
        "updateStateChanged",
        "",
        "Quasars::UpdateState",
        "state",
        "jsonUpdateComplete",
        "flagQuasarsVisibilityChanged",
        "b",
        "displayModeChanged",
        "markersModeChanged",
        "quasarsColorChanged",
        "Vec3f",
        "updateJSON",
        "setFlagShowQuasars",
        "getFlagShowQuasars",
        "setFlagUseQuasarMarkers",
        "getFlagUseQuasarMarkers",
        "setFlagShowQuasarsButton",
        "getFlagShowQuasarsButton",
        "getCountQuasars",
        "getDisplayMode",
        "setDisplayMode",
        "getMarkerColor",
        "setMarkerColor",
        "c",
        "setFontSize",
        "s",
        "checkForUpdate",
        "updateDownloadProgress",
        "bytesReceived",
        "bytesTotal",
        "downloadComplete",
        "QNetworkReply*",
        "reply",
        "displayMessage",
        "message",
        "hexColor",
        "reloadCatalog",
        "saveSettings",
        "quasarsVisible",
        "quasarsColor",
        "flagDisplayMode",
        "flagMarkersMode"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'updateStateChanged'
        QtMocHelpers::SignalData<void(Quasars::UpdateState)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'jsonUpdateComplete'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'flagQuasarsVisibilityChanged'
        QtMocHelpers::SignalData<void(bool)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 7 },
        }}),
        // Signal 'displayModeChanged'
        QtMocHelpers::SignalData<void(bool)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 7 },
        }}),
        // Signal 'markersModeChanged'
        QtMocHelpers::SignalData<void(bool)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 7 },
        }}),
        // Signal 'quasarsColorChanged'
        QtMocHelpers::SignalData<void(Vec3f)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 2 },
        }}),
        // Slot 'updateJSON'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setFlagShowQuasars'
        QtMocHelpers::SlotData<void(bool)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 7 },
        }}),
        // Slot 'getFlagShowQuasars'
        QtMocHelpers::SlotData<bool()>(14, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFlagUseQuasarMarkers'
        QtMocHelpers::SlotData<void(bool)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 7 },
        }}),
        // Slot 'getFlagUseQuasarMarkers'
        QtMocHelpers::SlotData<bool()>(16, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFlagShowQuasarsButton'
        QtMocHelpers::SlotData<void(bool)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 7 },
        }}),
        // Slot 'getFlagShowQuasarsButton'
        QtMocHelpers::SlotData<bool()>(18, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'getCountQuasars'
        QtMocHelpers::SlotData<int()>(19, 2, QMC::AccessPublic, QMetaType::Int),
        // Slot 'getDisplayMode'
        QtMocHelpers::SlotData<bool()>(20, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setDisplayMode'
        QtMocHelpers::SlotData<void(bool)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 7 },
        }}),
        // Slot 'getMarkerColor'
        QtMocHelpers::SlotData<Vec3f()>(22, 2, QMC::AccessPublic, 0x80000000 | 11),
        // Slot 'setMarkerColor'
        QtMocHelpers::SlotData<void(const Vec3f &)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 24 },
        }}),
        // Slot 'setFontSize'
        QtMocHelpers::SlotData<void(int)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 26 },
        }}),
        // Slot 'checkForUpdate'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateDownloadProgress'
        QtMocHelpers::SlotData<void(qint64, qint64)>(28, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::LongLong, 29 }, { QMetaType::LongLong, 30 },
        }}),
        // Slot 'downloadComplete'
        QtMocHelpers::SlotData<void(QNetworkReply *)>(31, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 32, 33 },
        }}),
        // Slot 'displayMessage'
        QtMocHelpers::SlotData<void(const QString &, const QString &)>(34, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 35 }, { QMetaType::QString, 36 },
        }}),
        // Slot 'displayMessage'
        QtMocHelpers::SlotData<void(const QString &)>(34, 2, QMC::AccessPrivate | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 35 },
        }}),
        // Slot 'reloadCatalog'
        QtMocHelpers::SlotData<void()>(37, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'saveSettings'
        QtMocHelpers::SlotData<void()>(38, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'quasarsVisible'
        QtMocHelpers::PropertyData<bool>(39, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 2),
        // property 'quasarsColor'
        QtMocHelpers::PropertyData<Vec3f>(40, 0x80000000 | 11, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag, 5),
        // property 'flagDisplayMode'
        QtMocHelpers::PropertyData<bool>(41, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 3),
        // property 'flagMarkersMode'
        QtMocHelpers::PropertyData<bool>(42, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 4),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Quasars, qt_meta_tag_ZN7QuasarsE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Quasars::staticMetaObject = { {
    QMetaObject::SuperData::link<StelObjectModule::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7QuasarsE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7QuasarsE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN7QuasarsE_t>.metaTypes,
    nullptr
} };

void Quasars::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Quasars *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->updateStateChanged((*reinterpret_cast<std::add_pointer_t<Quasars::UpdateState>>(_a[1]))); break;
        case 1: _t->jsonUpdateComplete(); break;
        case 2: _t->flagQuasarsVisibilityChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->displayModeChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->markersModeChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->quasarsColorChanged((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1]))); break;
        case 6: _t->updateJSON(); break;
        case 7: _t->setFlagShowQuasars((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: { bool _r = _t->getFlagShowQuasars();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 9: _t->setFlagUseQuasarMarkers((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 10: { bool _r = _t->getFlagUseQuasarMarkers();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 11: _t->setFlagShowQuasarsButton((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 12: { bool _r = _t->getFlagShowQuasarsButton();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 13: { int _r = _t->getCountQuasars();
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 14: { bool _r = _t->getDisplayMode();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 15: _t->setDisplayMode((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 16: { Vec3f _r = _t->getMarkerColor();
            if (_a[0]) *reinterpret_cast<Vec3f*>(_a[0]) = std::move(_r); }  break;
        case 17: _t->setMarkerColor((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1]))); break;
        case 18: _t->setFontSize((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 19: _t->checkForUpdate(); break;
        case 20: _t->updateDownloadProgress((*reinterpret_cast<std::add_pointer_t<qint64>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qint64>>(_a[2]))); break;
        case 21: _t->downloadComplete((*reinterpret_cast<std::add_pointer_t<QNetworkReply*>>(_a[1]))); break;
        case 22: _t->displayMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 23: _t->displayMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 24: _t->reloadCatalog(); break;
        case 25: _t->saveSettings(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 17:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 21:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QNetworkReply* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Quasars::*)(Quasars::UpdateState )>(_a, &Quasars::updateStateChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Quasars::*)()>(_a, &Quasars::jsonUpdateComplete, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Quasars::*)(bool )>(_a, &Quasars::flagQuasarsVisibilityChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Quasars::*)(bool )>(_a, &Quasars::displayModeChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Quasars::*)(bool )>(_a, &Quasars::markersModeChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (Quasars::*)(Vec3f )>(_a, &Quasars::quasarsColorChanged, 5))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Vec3f >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->getFlagShowQuasars(); break;
        case 1: *reinterpret_cast<Vec3f*>(_v) = _t->getMarkerColor(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->getDisplayMode(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->getFlagUseQuasarMarkers(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setFlagShowQuasars(*reinterpret_cast<bool*>(_v)); break;
        case 1: _t->setMarkerColor(*reinterpret_cast<Vec3f*>(_v)); break;
        case 2: _t->setDisplayMode(*reinterpret_cast<bool*>(_v)); break;
        case 3: _t->setFlagUseQuasarMarkers(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *Quasars::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Quasars::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7QuasarsE_t>.strings))
        return static_cast<void*>(this);
    return StelObjectModule::qt_metacast(_clname);
}

int Quasars::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StelObjectModule::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 26)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 26;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 26)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 26;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void Quasars::updateStateChanged(Quasars::UpdateState _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void Quasars::jsonUpdateComplete()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void Quasars::flagQuasarsVisibilityChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void Quasars::displayModeChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void Quasars::markersModeChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void Quasars::quasarsColorChanged(Vec3f _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN26QuasarsStelPluginInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto QuasarsStelPluginInterface::qt_create_metaobjectdata<qt_meta_tag_ZN26QuasarsStelPluginInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QuasarsStelPluginInterface"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QuasarsStelPluginInterface, qt_meta_tag_ZN26QuasarsStelPluginInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QuasarsStelPluginInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26QuasarsStelPluginInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26QuasarsStelPluginInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN26QuasarsStelPluginInterfaceE_t>.metaTypes,
    nullptr
} };

void QuasarsStelPluginInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QuasarsStelPluginInterface *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *QuasarsStelPluginInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QuasarsStelPluginInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26QuasarsStelPluginInterfaceE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "StelPluginInterface"))
        return static_cast< StelPluginInterface*>(this);
    if (!strcmp(_clname, "org.stellarium.StelPluginInterface/2.0"))
        return static_cast< StelPluginInterface*>(this);
    return QObject::qt_metacast(_clname);
}

int QuasarsStelPluginInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}

#ifdef QT_MOC_EXPORT_PLUGIN_V2
static constexpr unsigned char qt_pluginMetaDataV2_QuasarsStelPluginInterface[] = {
    0xbf, 
    // "IID"
    0x02,  0x78,  0x26,  'o',  'r',  'g',  '.',  's', 
    't',  'e',  'l',  'l',  'a',  'r',  'i',  'u', 
    'm',  '.',  'S',  't',  'e',  'l',  'P',  'l', 
    'u',  'g',  'i',  'n',  'I',  'n',  't',  'e', 
    'r',  'f',  'a',  'c',  'e',  '/',  '2',  '.', 
    '0', 
    // "className"
    0x03,  0x78,  0x1a,  'Q',  'u',  'a',  's',  'a', 
    'r',  's',  'S',  't',  'e',  'l',  'P',  'l', 
    'u',  'g',  'i',  'n',  'I',  'n',  't',  'e', 
    'r',  'f',  'a',  'c',  'e', 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN_V2(QuasarsStelPluginInterface, QuasarsStelPluginInterface, qt_pluginMetaDataV2_QuasarsStelPluginInterface)
#else
QT_PLUGIN_METADATA_SECTION
Q_CONSTINIT static constexpr unsigned char qt_pluginMetaData_QuasarsStelPluginInterface[] = {
    'Q', 'T', 'M', 'E', 'T', 'A', 'D', 'A', 'T', 'A', ' ', '!',
    // metadata version, Qt version, architectural requirements
    0, QT_VERSION_MAJOR, QT_VERSION_MINOR, qPluginArchRequirements(),
    0xbf, 
    // "IID"
    0x02,  0x78,  0x26,  'o',  'r',  'g',  '.',  's', 
    't',  'e',  'l',  'l',  'a',  'r',  'i',  'u', 
    'm',  '.',  'S',  't',  'e',  'l',  'P',  'l', 
    'u',  'g',  'i',  'n',  'I',  'n',  't',  'e', 
    'r',  'f',  'a',  'c',  'e',  '/',  '2',  '.', 
    '0', 
    // "className"
    0x03,  0x78,  0x1a,  'Q',  'u',  'a',  's',  'a', 
    'r',  's',  'S',  't',  'e',  'l',  'P',  'l', 
    'u',  'g',  'i',  'n',  'I',  'n',  't',  'e', 
    'r',  'f',  'a',  'c',  'e', 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN(QuasarsStelPluginInterface, QuasarsStelPluginInterface)
#endif  // QT_MOC_EXPORT_PLUGIN_V2

QT_WARNING_POP
