/****************************************************************************
** Meta object code from reading C++ file 'CustomObjectMgr.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../stellarium/src/core/modules/CustomObjectMgr.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'CustomObjectMgr.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15CustomObjectMgrE_t {};
} // unnamed namespace

template <> constexpr inline auto CustomObjectMgr::qt_create_metaobjectdata<qt_meta_tag_ZN15CustomObjectMgrE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "CustomObjectMgr",
        "searchByEnglishName",
        "CustomObjectP",
        "",
        "customObjectEnglishName",
        "addCustomObject",
        "designation",
        "Vec3d",
        "coordinates",
        "isVisible",
        "ra",
        "dec",
        "addCustomObjectRaDec",
        "addCustomObjectAltAzi",
        "alt",
        "azi",
        "removeCustomObjects",
        "removeCustomObject",
        "englishName",
        "setMarkersColor",
        "Vec3f",
        "c",
        "getMarkersColor",
        "setMarkersSize",
        "size",
        "getMarkersSize",
        "setSelectPriority",
        "priority",
        "getSelectPriority",
        "selectedObjectChange",
        "StelModule::StelModuleSelectAction",
        "action",
        "setFontSize",
        "s"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'searchByEnglishName'
        QtMocHelpers::SlotData<CustomObjectP(QString) const>(1, 3, QMC::AccessPublic, 0x80000000 | 2, {{
            { QMetaType::QString, 4 },
        }}),
        // Slot 'addCustomObject'
        QtMocHelpers::SlotData<void(const QString &, Vec3d, bool)>(5, 3, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 }, { 0x80000000 | 7, 8 }, { QMetaType::Bool, 9 },
        }}),
        // Slot 'addCustomObject'
        QtMocHelpers::SlotData<void(const QString &, Vec3d)>(5, 3, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 6 }, { 0x80000000 | 7, 8 },
        }}),
        // Slot 'addCustomObject'
        QtMocHelpers::SlotData<void(const QString &, const QString &, const QString &, bool)>(5, 3, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 }, { QMetaType::QString, 10 }, { QMetaType::QString, 11 }, { QMetaType::Bool, 9 },
        }}),
        // Slot 'addCustomObject'
        QtMocHelpers::SlotData<void(const QString &, const QString &, const QString &)>(5, 3, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 6 }, { QMetaType::QString, 10 }, { QMetaType::QString, 11 },
        }}),
        // Slot 'addCustomObjectRaDec'
        QtMocHelpers::SlotData<void(const QString &, const QString &, const QString &, bool)>(12, 3, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 }, { QMetaType::QString, 10 }, { QMetaType::QString, 11 }, { QMetaType::Bool, 9 },
        }}),
        // Slot 'addCustomObjectRaDec'
        QtMocHelpers::SlotData<void(const QString &, const QString &, const QString &)>(12, 3, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 6 }, { QMetaType::QString, 10 }, { QMetaType::QString, 11 },
        }}),
        // Slot 'addCustomObjectAltAzi'
        QtMocHelpers::SlotData<void(const QString &, const QString &, const QString &, bool)>(13, 3, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 }, { QMetaType::QString, 14 }, { QMetaType::QString, 15 }, { QMetaType::Bool, 9 },
        }}),
        // Slot 'addCustomObjectAltAzi'
        QtMocHelpers::SlotData<void(const QString &, const QString &, const QString &)>(13, 3, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 6 }, { QMetaType::QString, 14 }, { QMetaType::QString, 15 },
        }}),
        // Slot 'removeCustomObjects'
        QtMocHelpers::SlotData<void()>(16, 3, QMC::AccessPublic, QMetaType::Void),
        // Slot 'removeCustomObject'
        QtMocHelpers::SlotData<void(QString)>(17, 3, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 18 },
        }}),
        // Slot 'setMarkersColor'
        QtMocHelpers::SlotData<void(const Vec3f &)>(19, 3, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 20, 21 },
        }}),
        // Slot 'getMarkersColor'
        QtMocHelpers::SlotData<Vec3f() const>(22, 3, QMC::AccessPublic, 0x80000000 | 20),
        // Slot 'setMarkersSize'
        QtMocHelpers::SlotData<void(const float)>(23, 3, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 24 },
        }}),
        // Slot 'getMarkersSize'
        QtMocHelpers::SlotData<float() const>(25, 3, QMC::AccessPublic, QMetaType::Float),
        // Slot 'setSelectPriority'
        QtMocHelpers::SlotData<void(float)>(26, 3, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 27 },
        }}),
        // Slot 'getSelectPriority'
        QtMocHelpers::SlotData<float() const>(28, 3, QMC::AccessPublic, QMetaType::Float),
        // Slot 'selectedObjectChange'
        QtMocHelpers::SlotData<void(StelModule::StelModuleSelectAction)>(29, 3, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 30, 31 },
        }}),
        // Slot 'removeCustomObject'
        QtMocHelpers::SlotData<void(CustomObjectP)>(17, 3, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 2, 3 },
        }}),
        // Slot 'setFontSize'
        QtMocHelpers::SlotData<void(int)>(32, 3, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 33 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CustomObjectMgr, qt_meta_tag_ZN15CustomObjectMgrE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject CustomObjectMgr::staticMetaObject = { {
    QMetaObject::SuperData::link<StelObjectModule::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15CustomObjectMgrE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15CustomObjectMgrE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15CustomObjectMgrE_t>.metaTypes,
    nullptr
} };

void CustomObjectMgr::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CustomObjectMgr *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { CustomObjectP _r = _t->searchByEnglishName((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<CustomObjectP*>(_a[0]) = std::move(_r); }  break;
        case 1: _t->addCustomObject((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Vec3d>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 2: _t->addCustomObject((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Vec3d>>(_a[2]))); break;
        case 3: _t->addCustomObject((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4]))); break;
        case 4: _t->addCustomObject((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 5: _t->addCustomObjectRaDec((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4]))); break;
        case 6: _t->addCustomObjectRaDec((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 7: _t->addCustomObjectAltAzi((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4]))); break;
        case 8: _t->addCustomObjectAltAzi((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 9: _t->removeCustomObjects(); break;
        case 10: _t->removeCustomObject((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 11: _t->setMarkersColor((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1]))); break;
        case 12: { Vec3f _r = _t->getMarkersColor();
            if (_a[0]) *reinterpret_cast<Vec3f*>(_a[0]) = std::move(_r); }  break;
        case 13: _t->setMarkersSize((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 14: { float _r = _t->getMarkersSize();
            if (_a[0]) *reinterpret_cast<float*>(_a[0]) = std::move(_r); }  break;
        case 15: _t->setSelectPriority((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 16: { float _r = _t->getSelectPriority();
            if (_a[0]) *reinterpret_cast<float*>(_a[0]) = std::move(_r); }  break;
        case 17: _t->selectedObjectChange((*reinterpret_cast<std::add_pointer_t<StelModule::StelModuleSelectAction>>(_a[1]))); break;
        case 18: _t->removeCustomObject((*reinterpret_cast<std::add_pointer_t<CustomObjectP>>(_a[1]))); break;
        case 19: _t->setFontSize((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3d >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3d >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 17:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< StelModule::StelModuleSelectAction >(); break;
            }
            break;
        }
    }
}

const QMetaObject *CustomObjectMgr::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CustomObjectMgr::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15CustomObjectMgrE_t>.strings))
        return static_cast<void*>(this);
    return StelObjectModule::qt_metacast(_clname);
}

int CustomObjectMgr::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StelObjectModule::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 20)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 20;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 20)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 20;
    }
    return _id;
}
QT_WARNING_POP
