/********************************************************************************
** Form generated from reading UI file 'skylightDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SKYLIGHTDIALOG_H
#define UI_SKYLIGHTDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "Dialog.hpp"
#include "gui/StelCloseButton.hpp"

QT_BEGIN_NAMESPACE

class Ui_skylightDialogForm
{
public:
    QVBoxLayout *verticalLayout;
    TitleBar *titleBar;
    QGroupBox *atmosphereGroupBox;
    QVBoxLayout *verticalLayout_3;
    QFrame *line;
    QGridLayout *gridLayout_2;
    QComboBox *comboBox_skylightDecimals;
    QLabel *label_skylightDecimals;
    QCheckBox *checkBox_drawSunAfterAtmosphere;
    QCheckBox *checkBox_earlySunHalo;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *horizontalSpacer_3;
    QPushButton *resetStellPushButton;
    QPushButton *resetPreetPushButton;
    QCheckBox *checkBox_Schaefer;
    QHBoxLayout *horizontalLayout_2;
    QLabel *label_35;
    QLabel *label_22;
    QGridLayout *gridLayout_zenithXY;
    QLabel *label_24;
    QLabel *label_th3;
    QLabel *label_th2;
    QLabel *label_th1;
    QLabel *label_th0;
    QDoubleSpinBox *doubleSpinBox_zX32;
    QDoubleSpinBox *doubleSpinBox_zX33;
    QDoubleSpinBox *doubleSpinBox_zX34;
    QLabel *label_26;
    QLabel *label_25;
    QLabel *label_27;
    QLabel *label_zCY;
    QDoubleSpinBox *doubleSpinBox_zX11;
    QDoubleSpinBox *doubleSpinBox_zY31;
    QDoubleSpinBox *doubleSpinBox_zY32;
    QDoubleSpinBox *doubleSpinBox_zY33;
    QDoubleSpinBox *doubleSpinBox_zY34;
    QLabel *label_26a;
    QLabel *label_25a;
    QLabel *label_27a;
    QDoubleSpinBox *doubleSpinBox_zY11;
    QDoubleSpinBox *doubleSpinBox_zY12;
    QDoubleSpinBox *doubleSpinBox_zY13;
    QDoubleSpinBox *doubleSpinBox_zY21;
    QDoubleSpinBox *doubleSpinBox_zY22;
    QDoubleSpinBox *doubleSpinBox_zY23;
    QDoubleSpinBox *doubleSpinBox_zY24;
    QDoubleSpinBox *doubleSpinBox_zX12;
    QDoubleSpinBox *doubleSpinBox_zX13;
    QDoubleSpinBox *doubleSpinBox_zX21;
    QDoubleSpinBox *doubleSpinBox_zX22;
    QDoubleSpinBox *doubleSpinBox_zX23;
    QDoubleSpinBox *doubleSpinBox_zX24;
    QDoubleSpinBox *doubleSpinBox_zX31;
    QPushButton *pushButton_ResetZYstel;
    QPushButton *pushButton_ResetZYpreetham;
    QPushButton *pushButton_ResetZXstel;
    QPushButton *pushButton_ResetZXpreetham;
    QHBoxLayout *horizontalLayout_3;
    QLabel *label_34;
    QLabel *label_23;
    QGridLayout *gridLayout;
    QDoubleSpinBox *doubleSpinBox_DYT;
    QLabel *label;
    QLabel *label_2;
    QLabel *label_5;
    QLabel *label_6;
    QLabel *label_7;
    QLabel *label_8;
    QLabel *label_9;
    QLabel *label_10;
    QLabel *label_11;
    QDoubleSpinBox *doubleSpinBox_Ax;
    QDoubleSpinBox *doubleSpinBox_DY;
    QDoubleSpinBox *doubleSpinBox_CY;
    QDoubleSpinBox *doubleSpinBox_EY;
    QDoubleSpinBox *doubleSpinBox_Dy;
    QDoubleSpinBox *doubleSpinBox_EyT;
    QDoubleSpinBox *doubleSpinBox_Ey;
    QDoubleSpinBox *doubleSpinBox_Bx;
    QDoubleSpinBox *doubleSpinBox_CxT;
    QDoubleSpinBox *doubleSpinBox_Cx;
    QDoubleSpinBox *doubleSpinBox_DxT;
    QDoubleSpinBox *doubleSpinBox_Dx;
    QDoubleSpinBox *doubleSpinBox_ExT;
    QDoubleSpinBox *doubleSpinBox_Ex;
    QLabel *label_12;
    QDoubleSpinBox *doubleSpinBox_BxT;
    QDoubleSpinBox *doubleSpinBox_AYT;
    QDoubleSpinBox *doubleSpinBox_EYT;
    QDoubleSpinBox *doubleSpinBox_BYT;
    QDoubleSpinBox *doubleSpinBox_AY;
    QDoubleSpinBox *doubleSpinBox_AyT;
    QDoubleSpinBox *doubleSpinBox_Ay;
    QDoubleSpinBox *doubleSpinBox_ByT;
    QDoubleSpinBox *doubleSpinBox_By;
    QDoubleSpinBox *doubleSpinBox_CyT;
    QDoubleSpinBox *doubleSpinBox_Cy;
    QDoubleSpinBox *doubleSpinBox_DyT;
    QLabel *label_3;
    QLabel *label_4;
    QDoubleSpinBox *doubleSpinBox_AxT;
    QDoubleSpinBox *doubleSpinBox_BY;
    QDoubleSpinBox *doubleSpinBox_CYT;
    QLabel *label_13;
    QLabel *label_14;
    QLabel *label_15;
    QLabel *label_16;
    QLabel *label_17;
    QLabel *label_18;
    QLabel *label_19;
    QLabel *label_20;
    QLabel *label_21;
    QLabel *label_28;
    QLabel *label_29;
    QLabel *label_30;
    QLabel *label_31;
    QLabel *label_32;
    QPushButton *pushButton_ResetDistYPreetham;
    QPushButton *pushButton_ResetDistYStel;
    QPushButton *pushButton_ResetDistxPreetham;
    QPushButton *pushButton_ResetDistxStel;
    QPushButton *pushButton_ResetDistyPreetham;
    QPushButton *pushButton_ResetDistyStel;
    QSpacerItem *verticalSpacer_2;
    QLabel *label_36;

    void setupUi(QWidget *skylightDialogForm)
    {
        if (skylightDialogForm->objectName().isEmpty())
            skylightDialogForm->setObjectName("skylightDialogForm");
        skylightDialogForm->resize(1094, 746);
        verticalLayout = new QVBoxLayout(skylightDialogForm);
        verticalLayout->setSpacing(0);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        titleBar = new TitleBar(skylightDialogForm);
        titleBar->setObjectName("titleBar");

        verticalLayout->addWidget(titleBar);

        atmosphereGroupBox = new QGroupBox(skylightDialogForm);
        atmosphereGroupBox->setObjectName("atmosphereGroupBox");
        verticalLayout_3 = new QVBoxLayout(atmosphereGroupBox);
        verticalLayout_3->setObjectName("verticalLayout_3");
        verticalLayout_3->setContentsMargins(0, 0, 0, 0);
        line = new QFrame(atmosphereGroupBox);
        line->setObjectName("line");
        line->setMinimumSize(QSize(0, 1));
        line->setMaximumSize(QSize(16777215, 1));
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout_3->addWidget(line);

        gridLayout_2 = new QGridLayout();
        gridLayout_2->setObjectName("gridLayout_2");
        comboBox_skylightDecimals = new QComboBox(atmosphereGroupBox);
        comboBox_skylightDecimals->setObjectName("comboBox_skylightDecimals");

        gridLayout_2->addWidget(comboBox_skylightDecimals, 3, 1, 1, 1);

        label_skylightDecimals = new QLabel(atmosphereGroupBox);
        label_skylightDecimals->setObjectName("label_skylightDecimals");

        gridLayout_2->addWidget(label_skylightDecimals, 3, 0, 1, 1);

        checkBox_drawSunAfterAtmosphere = new QCheckBox(atmosphereGroupBox);
        checkBox_drawSunAfterAtmosphere->setObjectName("checkBox_drawSunAfterAtmosphere");

        gridLayout_2->addWidget(checkBox_drawSunAfterAtmosphere, 2, 2, 1, 1);

        checkBox_earlySunHalo = new QCheckBox(atmosphereGroupBox);
        checkBox_earlySunHalo->setObjectName("checkBox_earlySunHalo");

        gridLayout_2->addWidget(checkBox_earlySunHalo, 1, 2, 1, 1);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_3);

        resetStellPushButton = new QPushButton(atmosphereGroupBox);
        resetStellPushButton->setObjectName("resetStellPushButton");

        horizontalLayout->addWidget(resetStellPushButton);

        resetPreetPushButton = new QPushButton(atmosphereGroupBox);
        resetPreetPushButton->setObjectName("resetPreetPushButton");

        horizontalLayout->addWidget(resetPreetPushButton);


        gridLayout_2->addLayout(horizontalLayout, 3, 2, 1, 1);

        checkBox_Schaefer = new QCheckBox(atmosphereGroupBox);
        checkBox_Schaefer->setObjectName("checkBox_Schaefer");

        gridLayout_2->addWidget(checkBox_Schaefer, 1, 0, 1, 1);


        verticalLayout_3->addLayout(gridLayout_2);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        label_35 = new QLabel(atmosphereGroupBox);
        label_35->setObjectName("label_35");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(label_35->sizePolicy().hasHeightForWidth());
        label_35->setSizePolicy(sizePolicy);
        label_35->setText(QString::fromUtf8("zenithLum=zenithY = (4.0453T  - 4.9710) tan &chi;  -  0.2155T + 2.4192 kcd/m\302\262 ; &chi;=(4/9-T/120)(&pi;-2&theta;<sub>s</sub>) "));
        label_35->setTextFormat(Qt::AutoText);

        horizontalLayout_2->addWidget(label_35);

        label_22 = new QLabel(atmosphereGroupBox);
        label_22->setObjectName("label_22");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(label_22->sizePolicy().hasHeightForWidth());
        label_22->setSizePolicy(sizePolicy1);

        horizontalLayout_2->addWidget(label_22);


        verticalLayout_3->addLayout(horizontalLayout_2);

        gridLayout_zenithXY = new QGridLayout();
        gridLayout_zenithXY->setObjectName("gridLayout_zenithXY");
        label_24 = new QLabel(atmosphereGroupBox);
        label_24->setObjectName("label_24");
        label_24->setText(QString::fromUtf8("zenithColorX="));

        gridLayout_zenithXY->addWidget(label_24, 1, 0, 1, 1);

        label_th3 = new QLabel(atmosphereGroupBox);
        label_th3->setObjectName("label_th3");
        label_th3->setText(QString::fromUtf8(" theta\302\263"));

        gridLayout_zenithXY->addWidget(label_th3, 10, 0, 1, 1);

        label_th2 = new QLabel(atmosphereGroupBox);
        label_th2->setObjectName("label_th2");
        label_th2->setText(QString::fromUtf8("theta\302\262"));

        gridLayout_zenithXY->addWidget(label_th2, 10, 1, 1, 1);

        label_th1 = new QLabel(atmosphereGroupBox);
        label_th1->setObjectName("label_th1");
        label_th1->setText(QString::fromUtf8("theta"));

        gridLayout_zenithXY->addWidget(label_th1, 10, 2, 1, 1);

        label_th0 = new QLabel(atmosphereGroupBox);
        label_th0->setObjectName("label_th0");
        label_th0->setText(QString::fromUtf8("1"));

        gridLayout_zenithXY->addWidget(label_th0, 10, 3, 1, 1);

        doubleSpinBox_zX32 = new QDoubleSpinBox(atmosphereGroupBox);
        doubleSpinBox_zX32->setObjectName("doubleSpinBox_zX32");
        doubleSpinBox_zX32->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        doubleSpinBox_zX32->setDecimals(5);
        doubleSpinBox_zX32->setMinimum(-0.500000000000000);
        doubleSpinBox_zX32->setMaximum(0.500000000000000);
        doubleSpinBox_zX32->setSingleStep(0.000100000000000);

        gridLayout_zenithXY->addWidget(doubleSpinBox_zX32, 5, 1, 1, 1);

        doubleSpinBox_zX33 = new QDoubleSpinBox(atmosphereGroupBox);
        doubleSpinBox_zX33->setObjectName("doubleSpinBox_zX33");
        doubleSpinBox_zX33->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        doubleSpinBox_zX33->setDecimals(5);
        doubleSpinBox_zX33->setMinimum(-0.100000000000000);
        doubleSpinBox_zX33->setMaximum(0.100000000000000);
        doubleSpinBox_zX33->setSingleStep(0.000100000000000);

        gridLayout_zenithXY->addWidget(doubleSpinBox_zX33, 5, 2, 1, 1);

        doubleSpinBox_zX34 = new QDoubleSpinBox(atmosphereGroupBox);
        doubleSpinBox_zX34->setObjectName("doubleSpinBox_zX34");
        doubleSpinBox_zX34->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        doubleSpinBox_zX34->setDecimals(5);
        doubleSpinBox_zX34->setMinimum(-0.500000000000000);
        doubleSpinBox_zX34->setMaximum(0.500000000000000);
        doubleSpinBox_zX34->setSingleStep(0.000100000000000);

        gridLayout_zenithXY->addWidget(doubleSpinBox_zX34, 5, 3, 1, 1);

        label_26 = new QLabel(atmosphereGroupBox);
        label_26->setObjectName("label_26");
        label_26->setText(QString::fromUtf8("T\302\262"));

        gridLayout_zenithXY->addWidget(label_26, 3, 4, 1, 1);

        label_25 = new QLabel(atmosphereGroupBox);
        label_25->setObjectName("label_25");
        label_25->setText(QString::fromUtf8("T"));

        gridLayout_zenithXY->addWidget(label_25, 4, 4, 1, 1);

        label_27 = new QLabel(atmosphereGroupBox);
        label_27->setObjectName("label_27");
        label_27->setText(QString::fromUtf8("1"));

        gridLayout_zenithXY->addWidget(label_27, 5, 4, 1, 1);

        label_zCY = new QLabel(atmosphereGroupBox);
        label_zCY->setObjectName("label_zCY");
        label_zCY->setText(QString::fromUtf8("zenithColorY="));

        gridLayout_zenithXY->addWidget(label_zCY, 6, 0, 1, 1);

        doubleSpinBox_zX11 = new QDoubleSpinBox(atmosphereGroupBox);
        doubleSpinBox_zX11->setObjectName("doubleSpinBox_zX11");
        doubleSpinBox_zX11->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        doubleSpinBox_zX11->setDecimals(5);
        doubleSpinBox_zX11->setMinimum(-0.010000000000000);
        doubleSpinBox_zX11->setMaximum(0.010000000000000);
        doubleSpinBox_zX11->setSingleStep(0.000100000000000);

        gridLayout_zenithXY->addWidget(doubleSpinBox_zX11, 3, 0, 1, 1);

        doubleSpinBox_zY31 = new QDoubleSpinBox(atmosphereGroupBox);
        doubleSpinBox_zY31->setObjectName("doubleSpinBox_zY31");
        doubleSpinBox_zY31->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        doubleSpinBox_zY31->setDecimals(5);
        doubleSpinBox_zY31->setMinimum(-0.500000000000000);
        doubleSpinBox_zY31->setMaximum(0.500000000000000);
        doubleSpinBox_zY31->setSingleStep(0.000100000000000);

        gridLayout_zenithXY->addWidget(doubleSpinBox_zY31, 9, 0, 1, 1);

        doubleSpinBox_zY32 = new QDoubleSpinBox(atmosphereGroupBox);
        doubleSpinBox_zY32->setObjectName("doubleSpinBox_zY32");
        doubleSpinBox_zY32->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        doubleSpinBox_zY32->setDecimals(5);
        doubleSpinBox_zY32->setMinimum(-0.500000000000000);
        doubleSpinBox_zY32->setMaximum(0.500000000000000);
        doubleSpinBox_zY32->setSingleStep(0.000100000000000);

        gridLayout_zenithXY->addWidget(doubleSpinBox_zY32, 9, 1, 1, 1);

        doubleSpinBox_zY33 = new QDoubleSpinBox(atmosphereGroupBox);
        doubleSpinBox_zY33->setObjectName("doubleSpinBox_zY33");
        doubleSpinBox_zY33->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        doubleSpinBox_zY33->setDecimals(5);
        doubleSpinBox_zY33->setMinimum(-0.100000000000000);
        doubleSpinBox_zY33->setMaximum(0.100000000000000);
        doubleSpinBox_zY33->setSingleStep(0.000100000000000);

        gridLayout_zenithXY->addWidget(doubleSpinBox_zY33, 9, 2, 1, 1);

        doubleSpinBox_zY34 = new QDoubleSpinBox(atmosphereGroupBox);
        doubleSpinBox_zY34->setObjectName("doubleSpinBox_zY34");
        doubleSpinBox_zY34->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        doubleSpinBox_zY34->setDecimals(5);
        doubleSpinBox_zY34->setMinimum(-0.500000000000000);
        doubleSpinBox_zY34->setMaximum(0.500000000000000);
        doubleSpinBox_zY34->setSingleStep(0.000100000000000);

        gridLayout_zenithXY->addWidget(doubleSpinBox_zY34, 9, 3, 1, 1);

        label_26a = new QLabel(atmosphereGroupBox);
        label_26a->setObjectName("label_26a");
        label_26a->setText(QString::fromUtf8("T\302\262"));

        gridLayout_zenithXY->addWidget(label_26a, 7, 4, 1, 1);

        label_25a = new QLabel(atmosphereGroupBox);
        label_25a->setObjectName("label_25a");
        label_25a->setText(QString::fromUtf8("T"));

        gridLayout_zenithXY->addWidget(label_25a, 8, 4, 1, 1);

        label_27a = new QLabel(atmosphereGroupBox);
        label_27a->setObjectName("label_27a");
        label_27a->setText(QString::fromUtf8("1"));

        gridLayout_zenithXY->addWidget(label_27a, 9, 4, 1, 1);

        doubleSpinBox_zY11 = new QDoubleSpinBox(atmosphereGroupBox);
        doubleSpinBox_zY11->setObjectName("doubleSpinBox_zY11");
        doubleSpinBox_zY11->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        doubleSpinBox_zY11->setDecimals(5);
        doubleSpinBox_zY11->setMinimum(-0.010000000000000);
        doubleSpinBox_zY11->setMaximum(0.010000000000000);
        doubleSpinBox_zY11->setSingleStep(0.000100000000000);

        gridLayout_zenithXY->addWidget(doubleSpinBox_zY11, 7, 0, 1, 1);

        doubleSpinBox_zY12 = new QDoubleSpinBox(atmosphereGroupBox);
        doubleSpinBox_zY12->setObjectName("doubleSpinBox_zY12");
        doubleSpinBox_zY12->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        doubleSpinBox_zY12->setDecimals(5);
        doubleSpinBox_zY12->setMinimum(-0.010000000000000);
        doubleSpinBox_zY12->setMaximum(0.010000000000000);
        doubleSpinBox_zY12->setSingleStep(0.000100000000000);

        gridLayout_zenithXY->addWidget(doubleSpinBox_zY12, 7, 1, 1, 1);

        doubleSpinBox_zY13 = new QDoubleSpinBox(atmosphereGroupBox);
        doubleSpinBox_zY13->setObjectName("doubleSpinBox_zY13");
        doubleSpinBox_zY13->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        doubleSpinBox_zY13->setDecimals(5);
        doubleSpinBox_zY13->setMinimum(-0.010000000000000);
        doubleSpinBox_zY13->setMaximum(0.010000000000000);
        doubleSpinBox_zY13->setSingleStep(0.000100000000000);

        gridLayout_zenithXY->addWidget(doubleSpinBox_zY13, 7, 2, 1, 1);

        doubleSpinBox_zY21 = new QDoubleSpinBox(atmosphereGroupBox);
        doubleSpinBox_zY21->setObjectName("doubleSpinBox_zY21");
        doubleSpinBox_zY21->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        doubleSpinBox_zY21->setDecimals(5);
        doubleSpinBox_zY21->setMinimum(-0.100000000000000);
        doubleSpinBox_zY21->setMaximum(0.100000000000000);
        doubleSpinBox_zY21->setSingleStep(0.000100000000000);

        gridLayout_zenithXY->addWidget(doubleSpinBox_zY21, 8, 0, 1, 1);

        doubleSpinBox_zY22 = new QDoubleSpinBox(atmosphereGroupBox);
        doubleSpinBox_zY22->setObjectName("doubleSpinBox_zY22");
        doubleSpinBox_zY22->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        doubleSpinBox_zY22->setDecimals(5);
        doubleSpinBox_zY22->setMinimum(-0.100000000000000);
        doubleSpinBox_zY22->setMaximum(0.100000000000000);
        doubleSpinBox_zY22->setSingleStep(0.000100000000000);

        gridLayout_zenithXY->addWidget(doubleSpinBox_zY22, 8, 1, 1, 1);

        doubleSpinBox_zY23 = new QDoubleSpinBox(atmosphereGroupBox);
        doubleSpinBox_zY23->setObjectName("doubleSpinBox_zY23");
        doubleSpinBox_zY23->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        doubleSpinBox_zY23->setDecimals(5);
        doubleSpinBox_zY23->setMinimum(-0.100000000000000);
        doubleSpinBox_zY23->setMaximum(0.100000000000000);
        doubleSpinBox_zY23->setSingleStep(0.000100000000000);

        gridLayout_zenithXY->addWidget(doubleSpinBox_zY23, 8, 2, 1, 1);

        doubleSpinBox_zY24 = new QDoubleSpinBox(atmosphereGroupBox);
        doubleSpinBox_zY24->setObjectName("doubleSpinBox_zY24");
        doubleSpinBox_zY24->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        doubleSpinBox_zY24->setDecimals(5);
        doubleSpinBox_zY24->setMinimum(-0.010000000000000);
        doubleSpinBox_zY24->setMaximum(0.010000000000000);
        doubleSpinBox_zY24->setSingleStep(0.000100000000000);

        gridLayout_zenithXY->addWidget(doubleSpinBox_zY24, 8, 3, 1, 1);

        doubleSpinBox_zX12 = new QDoubleSpinBox(atmosphereGroupBox);
        doubleSpinBox_zX12->setObjectName("doubleSpinBox_zX12");
        doubleSpinBox_zX12->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        doubleSpinBox_zX12->setDecimals(5);
        doubleSpinBox_zX12->setMinimum(-0.010000000000000);
        doubleSpinBox_zX12->setMaximum(0.010000000000000);
        doubleSpinBox_zX12->setSingleStep(0.000100000000000);

        gridLayout_zenithXY->addWidget(doubleSpinBox_zX12, 3, 1, 1, 1);

        doubleSpinBox_zX13 = new QDoubleSpinBox(atmosphereGroupBox);
        doubleSpinBox_zX13->setObjectName("doubleSpinBox_zX13");
        doubleSpinBox_zX13->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        doubleSpinBox_zX13->setDecimals(5);
        doubleSpinBox_zX13->setMinimum(-0.010000000000000);
        doubleSpinBox_zX13->setMaximum(0.010000000000000);
        doubleSpinBox_zX13->setSingleStep(0.000100000000000);

        gridLayout_zenithXY->addWidget(doubleSpinBox_zX13, 3, 2, 1, 1);

        doubleSpinBox_zX21 = new QDoubleSpinBox(atmosphereGroupBox);
        doubleSpinBox_zX21->setObjectName("doubleSpinBox_zX21");
        doubleSpinBox_zX21->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        doubleSpinBox_zX21->setDecimals(5);
        doubleSpinBox_zX21->setMinimum(-0.100000000000000);
        doubleSpinBox_zX21->setMaximum(0.100000000000000);
        doubleSpinBox_zX21->setSingleStep(0.000100000000000);

        gridLayout_zenithXY->addWidget(doubleSpinBox_zX21, 4, 0, 1, 1);

        doubleSpinBox_zX22 = new QDoubleSpinBox(atmosphereGroupBox);
        doubleSpinBox_zX22->setObjectName("doubleSpinBox_zX22");
        doubleSpinBox_zX22->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        doubleSpinBox_zX22->setDecimals(5);
        doubleSpinBox_zX22->setMinimum(-0.100000000000000);
        doubleSpinBox_zX22->setMaximum(0.100000000000000);
        doubleSpinBox_zX22->setSingleStep(0.000100000000000);

        gridLayout_zenithXY->addWidget(doubleSpinBox_zX22, 4, 1, 1, 1);

        doubleSpinBox_zX23 = new QDoubleSpinBox(atmosphereGroupBox);
        doubleSpinBox_zX23->setObjectName("doubleSpinBox_zX23");
        doubleSpinBox_zX23->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        doubleSpinBox_zX23->setDecimals(5);
        doubleSpinBox_zX23->setMinimum(-0.100000000000000);
        doubleSpinBox_zX23->setMaximum(0.100000000000000);
        doubleSpinBox_zX23->setSingleStep(0.000100000000000);

        gridLayout_zenithXY->addWidget(doubleSpinBox_zX23, 4, 2, 1, 1);

        doubleSpinBox_zX24 = new QDoubleSpinBox(atmosphereGroupBox);
        doubleSpinBox_zX24->setObjectName("doubleSpinBox_zX24");
        doubleSpinBox_zX24->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        doubleSpinBox_zX24->setDecimals(5);
        doubleSpinBox_zX24->setMinimum(-0.010000000000000);
        doubleSpinBox_zX24->setMaximum(0.010000000000000);
        doubleSpinBox_zX24->setSingleStep(0.000100000000000);

        gridLayout_zenithXY->addWidget(doubleSpinBox_zX24, 4, 3, 1, 1);

        doubleSpinBox_zX31 = new QDoubleSpinBox(atmosphereGroupBox);
        doubleSpinBox_zX31->setObjectName("doubleSpinBox_zX31");
        doubleSpinBox_zX31->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        doubleSpinBox_zX31->setDecimals(5);
        doubleSpinBox_zX31->setMinimum(-0.500000000000000);
        doubleSpinBox_zX31->setMaximum(0.500000000000000);
        doubleSpinBox_zX31->setSingleStep(0.000100000000000);

        gridLayout_zenithXY->addWidget(doubleSpinBox_zX31, 5, 0, 1, 1);

        pushButton_ResetZYstel = new QPushButton(atmosphereGroupBox);
        pushButton_ResetZYstel->setObjectName("pushButton_ResetZYstel");

        gridLayout_zenithXY->addWidget(pushButton_ResetZYstel, 6, 2, 1, 1);

        pushButton_ResetZYpreetham = new QPushButton(atmosphereGroupBox);
        pushButton_ResetZYpreetham->setObjectName("pushButton_ResetZYpreetham");

        gridLayout_zenithXY->addWidget(pushButton_ResetZYpreetham, 6, 3, 1, 1);

        pushButton_ResetZXstel = new QPushButton(atmosphereGroupBox);
        pushButton_ResetZXstel->setObjectName("pushButton_ResetZXstel");

        gridLayout_zenithXY->addWidget(pushButton_ResetZXstel, 1, 2, 1, 1);

        pushButton_ResetZXpreetham = new QPushButton(atmosphereGroupBox);
        pushButton_ResetZXpreetham->setObjectName("pushButton_ResetZXpreetham");

        gridLayout_zenithXY->addWidget(pushButton_ResetZXpreetham, 1, 3, 1, 1);


        verticalLayout_3->addLayout(gridLayout_zenithXY);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        label_34 = new QLabel(atmosphereGroupBox);
        label_34->setObjectName("label_34");
        sizePolicy.setHeightForWidth(label_34->sizePolicy().hasHeightForWidth());
        label_34->setSizePolicy(sizePolicy);
        label_34->setText(QString::fromUtf8("F(&theta;; &gamma;) = (1+Ae<sup>B/cos &theta;</sup>)(1 + Ce<sup>Dg</sup> + E cos\302\262 &gamma; ) --- YP = Yz   F(&theta;; &gamma;) /  F(0; &theta;<sub>s</sub>) "));
        label_34->setTextFormat(Qt::AutoText);

        horizontalLayout_3->addWidget(label_34);

        label_23 = new QLabel(atmosphereGroupBox);
        label_23->setObjectName("label_23");
        sizePolicy1.setHeightForWidth(label_23->sizePolicy().hasHeightForWidth());
        label_23->setSizePolicy(sizePolicy1);

        horizontalLayout_3->addWidget(label_23);


        verticalLayout_3->addLayout(horizontalLayout_3);

        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        doubleSpinBox_DYT = new QDoubleSpinBox(atmosphereGroupBox);
        doubleSpinBox_DYT->setObjectName("doubleSpinBox_DYT");
        doubleSpinBox_DYT->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        doubleSpinBox_DYT->setDecimals(4);
        doubleSpinBox_DYT->setMinimum(-9.000000000000000);
        doubleSpinBox_DYT->setMaximum(9.000000000000000);
        doubleSpinBox_DYT->setSingleStep(0.001000000000000);
        doubleSpinBox_DYT->setValue(0.180000000000000);

        gridLayout->addWidget(doubleSpinBox_DYT, 4, 1, 1, 1);

        label = new QLabel(atmosphereGroupBox);
        label->setObjectName("label");
        label->setText(QString::fromUtf8("* T"));

        gridLayout->addWidget(label, 0, 1, 1, 1);

        label_2 = new QLabel(atmosphereGroupBox);
        label_2->setObjectName("label_2");
        label_2->setText(QString::fromUtf8("AY="));
        label_2->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label_2, 1, 0, 1, 1);

        label_5 = new QLabel(atmosphereGroupBox);
        label_5->setObjectName("label_5");
        label_5->setText(QString::fromUtf8("CY="));
        label_5->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label_5, 3, 0, 1, 1);

        label_6 = new QLabel(atmosphereGroupBox);
        label_6->setObjectName("label_6");
        label_6->setText(QString::fromUtf8("DY="));
        label_6->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label_6, 4, 0, 1, 1);

        label_7 = new QLabel(atmosphereGroupBox);
        label_7->setObjectName("label_7");
        label_7->setText(QString::fromUtf8("EY="));
        label_7->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label_7, 5, 0, 1, 1);

        label_8 = new QLabel(atmosphereGroupBox);
        label_8->setObjectName("label_8");
        label_8->setText(QString::fromUtf8("Ax="));
        label_8->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label_8, 1, 3, 1, 1);

        label_9 = new QLabel(atmosphereGroupBox);
        label_9->setObjectName("label_9");
        label_9->setText(QString::fromUtf8("Bx="));
        label_9->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label_9, 2, 3, 1, 1);

        label_10 = new QLabel(atmosphereGroupBox);
        label_10->setObjectName("label_10");
        label_10->setText(QString::fromUtf8("Cx="));
        label_10->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label_10, 3, 3, 1, 1);

        label_11 = new QLabel(atmosphereGroupBox);
        label_11->setObjectName("label_11");
        label_11->setText(QString::fromUtf8("Dx="));
        label_11->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label_11, 4, 3, 1, 1);

        doubleSpinBox_Ax = new QDoubleSpinBox(atmosphereGroupBox);
        doubleSpinBox_Ax->setObjectName("doubleSpinBox_Ax");
        doubleSpinBox_Ax->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        doubleSpinBox_Ax->setDecimals(4);
        doubleSpinBox_Ax->setMinimum(-2.000000000000000);
        doubleSpinBox_Ax->setMaximum(1.000000000000000);
        doubleSpinBox_Ax->setSingleStep(0.001000000000000);
        doubleSpinBox_Ax->setValue(0.180000000000000);

        gridLayout->addWidget(doubleSpinBox_Ax, 1, 5, 1, 1);

        doubleSpinBox_DY = new QDoubleSpinBox(atmosphereGroupBox);
        doubleSpinBox_DY->setObjectName("doubleSpinBox_DY");
        doubleSpinBox_DY->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        doubleSpinBox_DY->setDecimals(4);
        doubleSpinBox_DY->setMinimum(-9.900000000000000);
        doubleSpinBox_DY->setMaximum(9.900000000000000);
        doubleSpinBox_DY->setSingleStep(0.001000000000000);
        doubleSpinBox_DY->setValue(0.180000000000000);

        gridLayout->addWidget(doubleSpinBox_DY, 4, 2, 1, 1);

        doubleSpinBox_CY = new QDoubleSpinBox(atmosphereGroupBox);
        doubleSpinBox_CY->setObjectName("doubleSpinBox_CY");
        doubleSpinBox_CY->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        doubleSpinBox_CY->setDecimals(4);
        doubleSpinBox_CY->setMinimum(-9.900000000000000);
        doubleSpinBox_CY->setMaximum(9.900000000000000);
        doubleSpinBox_CY->setSingleStep(0.001000000000000);
        doubleSpinBox_CY->setValue(0.180000000000000);

        gridLayout->addWidget(doubleSpinBox_CY, 3, 2, 1, 1);

        doubleSpinBox_EY = new QDoubleSpinBox(atmosphereGroupBox);
        doubleSpinBox_EY->setObjectName("doubleSpinBox_EY");
        doubleSpinBox_EY->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        doubleSpinBox_EY->setDecimals(4);
        doubleSpinBox_EY->setMinimum(-9.000000000000000);
        doubleSpinBox_EY->setMaximum(9.000000000000000);
        doubleSpinBox_EY->setSingleStep(0.001000000000000);
        doubleSpinBox_EY->setValue(0.180000000000000);

        gridLayout->addWidget(doubleSpinBox_EY, 5, 2, 1, 1);

        doubleSpinBox_Dy = new QDoubleSpinBox(atmosphereGroupBox);
        doubleSpinBox_Dy->setObjectName("doubleSpinBox_Dy");
        doubleSpinBox_Dy->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        doubleSpinBox_Dy->setDecimals(4);
        doubleSpinBox_Dy->setMinimum(-2.000000000000000);
        doubleSpinBox_Dy->setMaximum(1.000000000000000);
        doubleSpinBox_Dy->setSingleStep(0.001000000000000);
        doubleSpinBox_Dy->setValue(0.180000000000000);

        gridLayout->addWidget(doubleSpinBox_Dy, 4, 8, 1, 1);

        doubleSpinBox_EyT = new QDoubleSpinBox(atmosphereGroupBox);
        doubleSpinBox_EyT->setObjectName("doubleSpinBox_EyT");
        doubleSpinBox_EyT->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        doubleSpinBox_EyT->setDecimals(4);
        doubleSpinBox_EyT->setMinimum(-2.000000000000000);
        doubleSpinBox_EyT->setMaximum(1.000000000000000);
        doubleSpinBox_EyT->setSingleStep(0.001000000000000);
        doubleSpinBox_EyT->setValue(0.180000000000000);

        gridLayout->addWidget(doubleSpinBox_EyT, 5, 7, 1, 1);

        doubleSpinBox_Ey = new QDoubleSpinBox(atmosphereGroupBox);
        doubleSpinBox_Ey->setObjectName("doubleSpinBox_Ey");
        doubleSpinBox_Ey->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        doubleSpinBox_Ey->setDecimals(4);
        doubleSpinBox_Ey->setMinimum(-2.000000000000000);
        doubleSpinBox_Ey->setMaximum(1.000000000000000);
        doubleSpinBox_Ey->setSingleStep(0.001000000000000);
        doubleSpinBox_Ey->setValue(0.180000000000000);

        gridLayout->addWidget(doubleSpinBox_Ey, 5, 8, 1, 1);

        doubleSpinBox_Bx = new QDoubleSpinBox(atmosphereGroupBox);
        doubleSpinBox_Bx->setObjectName("doubleSpinBox_Bx");
        doubleSpinBox_Bx->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        doubleSpinBox_Bx->setDecimals(4);
        doubleSpinBox_Bx->setMinimum(-2.000000000000000);
        doubleSpinBox_Bx->setMaximum(1.000000000000000);
        doubleSpinBox_Bx->setSingleStep(0.001000000000000);
        doubleSpinBox_Bx->setValue(0.180000000000000);

        gridLayout->addWidget(doubleSpinBox_Bx, 2, 5, 1, 1);

        doubleSpinBox_CxT = new QDoubleSpinBox(atmosphereGroupBox);
        doubleSpinBox_CxT->setObjectName("doubleSpinBox_CxT");
        doubleSpinBox_CxT->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        doubleSpinBox_CxT->setDecimals(4);
        doubleSpinBox_CxT->setMinimum(-2.000000000000000);
        doubleSpinBox_CxT->setMaximum(1.000000000000000);
        doubleSpinBox_CxT->setSingleStep(0.001000000000000);
        doubleSpinBox_CxT->setValue(0.180000000000000);

        gridLayout->addWidget(doubleSpinBox_CxT, 3, 4, 1, 1);

        doubleSpinBox_Cx = new QDoubleSpinBox(atmosphereGroupBox);
        doubleSpinBox_Cx->setObjectName("doubleSpinBox_Cx");
        doubleSpinBox_Cx->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        doubleSpinBox_Cx->setDecimals(4);
        doubleSpinBox_Cx->setMinimum(-2.000000000000000);
        doubleSpinBox_Cx->setMaximum(1.000000000000000);
        doubleSpinBox_Cx->setSingleStep(0.001000000000000);
        doubleSpinBox_Cx->setValue(0.180000000000000);

        gridLayout->addWidget(doubleSpinBox_Cx, 3, 5, 1, 1);

        doubleSpinBox_DxT = new QDoubleSpinBox(atmosphereGroupBox);
        doubleSpinBox_DxT->setObjectName("doubleSpinBox_DxT");
        doubleSpinBox_DxT->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        doubleSpinBox_DxT->setDecimals(4);
        doubleSpinBox_DxT->setMinimum(-2.000000000000000);
        doubleSpinBox_DxT->setMaximum(1.000000000000000);
        doubleSpinBox_DxT->setSingleStep(0.001000000000000);
        doubleSpinBox_DxT->setValue(0.180000000000000);

        gridLayout->addWidget(doubleSpinBox_DxT, 4, 4, 1, 1);

        doubleSpinBox_Dx = new QDoubleSpinBox(atmosphereGroupBox);
        doubleSpinBox_Dx->setObjectName("doubleSpinBox_Dx");
        doubleSpinBox_Dx->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        doubleSpinBox_Dx->setDecimals(4);
        doubleSpinBox_Dx->setMinimum(-2.000000000000000);
        doubleSpinBox_Dx->setMaximum(1.000000000000000);
        doubleSpinBox_Dx->setSingleStep(0.001000000000000);
        doubleSpinBox_Dx->setValue(0.180000000000000);

        gridLayout->addWidget(doubleSpinBox_Dx, 4, 5, 1, 1);

        doubleSpinBox_ExT = new QDoubleSpinBox(atmosphereGroupBox);
        doubleSpinBox_ExT->setObjectName("doubleSpinBox_ExT");
        doubleSpinBox_ExT->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        doubleSpinBox_ExT->setDecimals(4);
        doubleSpinBox_ExT->setMinimum(-2.000000000000000);
        doubleSpinBox_ExT->setMaximum(1.000000000000000);
        doubleSpinBox_ExT->setSingleStep(0.001000000000000);
        doubleSpinBox_ExT->setValue(0.180000000000000);

        gridLayout->addWidget(doubleSpinBox_ExT, 5, 4, 1, 1);

        doubleSpinBox_Ex = new QDoubleSpinBox(atmosphereGroupBox);
        doubleSpinBox_Ex->setObjectName("doubleSpinBox_Ex");
        doubleSpinBox_Ex->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        doubleSpinBox_Ex->setDecimals(4);
        doubleSpinBox_Ex->setMinimum(-2.000000000000000);
        doubleSpinBox_Ex->setMaximum(1.000000000000000);
        doubleSpinBox_Ex->setSingleStep(0.001000000000000);
        doubleSpinBox_Ex->setValue(0.180000000000000);

        gridLayout->addWidget(doubleSpinBox_Ex, 5, 5, 1, 1);

        label_12 = new QLabel(atmosphereGroupBox);
        label_12->setObjectName("label_12");
        label_12->setText(QString::fromUtf8("Ex="));
        label_12->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label_12, 5, 3, 1, 1);

        doubleSpinBox_BxT = new QDoubleSpinBox(atmosphereGroupBox);
        doubleSpinBox_BxT->setObjectName("doubleSpinBox_BxT");
        doubleSpinBox_BxT->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        doubleSpinBox_BxT->setDecimals(4);
        doubleSpinBox_BxT->setMinimum(-2.000000000000000);
        doubleSpinBox_BxT->setMaximum(1.000000000000000);
        doubleSpinBox_BxT->setSingleStep(0.001000000000000);
        doubleSpinBox_BxT->setValue(0.180000000000000);

        gridLayout->addWidget(doubleSpinBox_BxT, 2, 4, 1, 1);

        doubleSpinBox_AYT = new QDoubleSpinBox(atmosphereGroupBox);
        doubleSpinBox_AYT->setObjectName("doubleSpinBox_AYT");
        doubleSpinBox_AYT->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        doubleSpinBox_AYT->setDecimals(4);
        doubleSpinBox_AYT->setMinimum(-9.000000000000000);
        doubleSpinBox_AYT->setMaximum(9.000000000000000);
        doubleSpinBox_AYT->setSingleStep(0.001000000000000);
        doubleSpinBox_AYT->setValue(0.180000000000000);

        gridLayout->addWidget(doubleSpinBox_AYT, 1, 1, 1, 1);

        doubleSpinBox_EYT = new QDoubleSpinBox(atmosphereGroupBox);
        doubleSpinBox_EYT->setObjectName("doubleSpinBox_EYT");
        doubleSpinBox_EYT->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        doubleSpinBox_EYT->setDecimals(4);
        doubleSpinBox_EYT->setMinimum(-9.000000000000000);
        doubleSpinBox_EYT->setMaximum(9.000000000000000);
        doubleSpinBox_EYT->setSingleStep(0.001000000000000);
        doubleSpinBox_EYT->setValue(0.180000000000000);

        gridLayout->addWidget(doubleSpinBox_EYT, 5, 1, 1, 1);

        doubleSpinBox_BYT = new QDoubleSpinBox(atmosphereGroupBox);
        doubleSpinBox_BYT->setObjectName("doubleSpinBox_BYT");
        doubleSpinBox_BYT->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        doubleSpinBox_BYT->setDecimals(4);
        doubleSpinBox_BYT->setMinimum(-9.000000000000000);
        doubleSpinBox_BYT->setMaximum(9.000000000000000);
        doubleSpinBox_BYT->setSingleStep(0.001000000000000);
        doubleSpinBox_BYT->setValue(0.180000000000000);

        gridLayout->addWidget(doubleSpinBox_BYT, 2, 1, 1, 1);

        doubleSpinBox_AY = new QDoubleSpinBox(atmosphereGroupBox);
        doubleSpinBox_AY->setObjectName("doubleSpinBox_AY");
        doubleSpinBox_AY->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        doubleSpinBox_AY->setDecimals(4);
        doubleSpinBox_AY->setMinimum(-9.000000000000000);
        doubleSpinBox_AY->setMaximum(9.000000000000000);
        doubleSpinBox_AY->setSingleStep(0.001000000000000);
        doubleSpinBox_AY->setValue(0.180000000000000);

        gridLayout->addWidget(doubleSpinBox_AY, 1, 2, 1, 1);

        doubleSpinBox_AyT = new QDoubleSpinBox(atmosphereGroupBox);
        doubleSpinBox_AyT->setObjectName("doubleSpinBox_AyT");
        doubleSpinBox_AyT->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        doubleSpinBox_AyT->setDecimals(4);
        doubleSpinBox_AyT->setMinimum(-2.000000000000000);
        doubleSpinBox_AyT->setMaximum(1.000000000000000);
        doubleSpinBox_AyT->setSingleStep(0.001000000000000);
        doubleSpinBox_AyT->setValue(0.180000000000000);

        gridLayout->addWidget(doubleSpinBox_AyT, 1, 7, 1, 1);

        doubleSpinBox_Ay = new QDoubleSpinBox(atmosphereGroupBox);
        doubleSpinBox_Ay->setObjectName("doubleSpinBox_Ay");
        doubleSpinBox_Ay->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        doubleSpinBox_Ay->setDecimals(4);
        doubleSpinBox_Ay->setMinimum(-2.000000000000000);
        doubleSpinBox_Ay->setMaximum(1.000000000000000);
        doubleSpinBox_Ay->setSingleStep(0.001000000000000);
        doubleSpinBox_Ay->setValue(0.180000000000000);

        gridLayout->addWidget(doubleSpinBox_Ay, 1, 8, 1, 1);

        doubleSpinBox_ByT = new QDoubleSpinBox(atmosphereGroupBox);
        doubleSpinBox_ByT->setObjectName("doubleSpinBox_ByT");
        doubleSpinBox_ByT->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        doubleSpinBox_ByT->setDecimals(4);
        doubleSpinBox_ByT->setMinimum(-2.000000000000000);
        doubleSpinBox_ByT->setMaximum(1.000000000000000);
        doubleSpinBox_ByT->setSingleStep(0.001000000000000);
        doubleSpinBox_ByT->setValue(0.180000000000000);

        gridLayout->addWidget(doubleSpinBox_ByT, 2, 7, 1, 1);

        doubleSpinBox_By = new QDoubleSpinBox(atmosphereGroupBox);
        doubleSpinBox_By->setObjectName("doubleSpinBox_By");
        doubleSpinBox_By->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        doubleSpinBox_By->setDecimals(4);
        doubleSpinBox_By->setMinimum(-2.000000000000000);
        doubleSpinBox_By->setMaximum(1.000000000000000);
        doubleSpinBox_By->setSingleStep(0.001000000000000);
        doubleSpinBox_By->setValue(0.180000000000000);

        gridLayout->addWidget(doubleSpinBox_By, 2, 8, 1, 1);

        doubleSpinBox_CyT = new QDoubleSpinBox(atmosphereGroupBox);
        doubleSpinBox_CyT->setObjectName("doubleSpinBox_CyT");
        doubleSpinBox_CyT->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        doubleSpinBox_CyT->setDecimals(4);
        doubleSpinBox_CyT->setMinimum(-2.000000000000000);
        doubleSpinBox_CyT->setMaximum(1.000000000000000);
        doubleSpinBox_CyT->setSingleStep(0.001000000000000);
        doubleSpinBox_CyT->setValue(0.180000000000000);

        gridLayout->addWidget(doubleSpinBox_CyT, 3, 7, 1, 1);

        doubleSpinBox_Cy = new QDoubleSpinBox(atmosphereGroupBox);
        doubleSpinBox_Cy->setObjectName("doubleSpinBox_Cy");
        doubleSpinBox_Cy->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        doubleSpinBox_Cy->setDecimals(4);
        doubleSpinBox_Cy->setMinimum(-2.000000000000000);
        doubleSpinBox_Cy->setMaximum(1.000000000000000);
        doubleSpinBox_Cy->setSingleStep(0.001000000000000);
        doubleSpinBox_Cy->setValue(0.180000000000000);

        gridLayout->addWidget(doubleSpinBox_Cy, 3, 8, 1, 1);

        doubleSpinBox_DyT = new QDoubleSpinBox(atmosphereGroupBox);
        doubleSpinBox_DyT->setObjectName("doubleSpinBox_DyT");
        doubleSpinBox_DyT->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        doubleSpinBox_DyT->setDecimals(4);
        doubleSpinBox_DyT->setMinimum(-2.000000000000000);
        doubleSpinBox_DyT->setMaximum(1.000000000000000);
        doubleSpinBox_DyT->setSingleStep(0.001000000000000);
        doubleSpinBox_DyT->setValue(0.180000000000000);

        gridLayout->addWidget(doubleSpinBox_DyT, 4, 7, 1, 1);

        label_3 = new QLabel(atmosphereGroupBox);
        label_3->setObjectName("label_3");
        label_3->setText(QString::fromUtf8("+ const"));

        gridLayout->addWidget(label_3, 0, 2, 1, 1);

        label_4 = new QLabel(atmosphereGroupBox);
        label_4->setObjectName("label_4");
        label_4->setText(QString::fromUtf8("BY="));
        label_4->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label_4, 2, 0, 1, 1);

        doubleSpinBox_AxT = new QDoubleSpinBox(atmosphereGroupBox);
        doubleSpinBox_AxT->setObjectName("doubleSpinBox_AxT");
        doubleSpinBox_AxT->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        doubleSpinBox_AxT->setDecimals(4);
        doubleSpinBox_AxT->setMinimum(-2.000000000000000);
        doubleSpinBox_AxT->setMaximum(1.000000000000000);
        doubleSpinBox_AxT->setSingleStep(0.001000000000000);
        doubleSpinBox_AxT->setValue(0.180000000000000);

        gridLayout->addWidget(doubleSpinBox_AxT, 1, 4, 1, 1);

        doubleSpinBox_BY = new QDoubleSpinBox(atmosphereGroupBox);
        doubleSpinBox_BY->setObjectName("doubleSpinBox_BY");
        doubleSpinBox_BY->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        doubleSpinBox_BY->setDecimals(4);
        doubleSpinBox_BY->setMinimum(-9.000000000000000);
        doubleSpinBox_BY->setMaximum(9.000000000000000);
        doubleSpinBox_BY->setSingleStep(0.001000000000000);
        doubleSpinBox_BY->setValue(0.180000000000000);

        gridLayout->addWidget(doubleSpinBox_BY, 2, 2, 1, 1);

        doubleSpinBox_CYT = new QDoubleSpinBox(atmosphereGroupBox);
        doubleSpinBox_CYT->setObjectName("doubleSpinBox_CYT");
        doubleSpinBox_CYT->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        doubleSpinBox_CYT->setDecimals(4);
        doubleSpinBox_CYT->setMinimum(-9.000000000000000);
        doubleSpinBox_CYT->setMaximum(9.000000000000000);
        doubleSpinBox_CYT->setSingleStep(0.001000000000000);
        doubleSpinBox_CYT->setValue(0.180000000000000);

        gridLayout->addWidget(doubleSpinBox_CYT, 3, 1, 1, 1);

        label_13 = new QLabel(atmosphereGroupBox);
        label_13->setObjectName("label_13");
        label_13->setText(QString::fromUtf8("Ay="));
        label_13->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label_13, 1, 6, 1, 1);

        label_14 = new QLabel(atmosphereGroupBox);
        label_14->setObjectName("label_14");
        label_14->setText(QString::fromUtf8("By="));
        label_14->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label_14, 2, 6, 1, 1);

        label_15 = new QLabel(atmosphereGroupBox);
        label_15->setObjectName("label_15");
        label_15->setText(QString::fromUtf8("Cy="));
        label_15->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label_15, 3, 6, 1, 1);

        label_16 = new QLabel(atmosphereGroupBox);
        label_16->setObjectName("label_16");
        label_16->setText(QString::fromUtf8("Dy="));
        label_16->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label_16, 4, 6, 1, 1);

        label_17 = new QLabel(atmosphereGroupBox);
        label_17->setObjectName("label_17");
        label_17->setText(QString::fromUtf8("Ey="));
        label_17->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label_17, 5, 6, 1, 1);

        label_18 = new QLabel(atmosphereGroupBox);
        label_18->setObjectName("label_18");
        label_18->setText(QString::fromUtf8("* T"));

        gridLayout->addWidget(label_18, 0, 4, 1, 1);

        label_19 = new QLabel(atmosphereGroupBox);
        label_19->setObjectName("label_19");
        label_19->setText(QString::fromUtf8("+ const"));

        gridLayout->addWidget(label_19, 0, 5, 1, 1);

        label_20 = new QLabel(atmosphereGroupBox);
        label_20->setObjectName("label_20");
        label_20->setText(QString::fromUtf8("* T"));

        gridLayout->addWidget(label_20, 0, 7, 1, 1);

        label_21 = new QLabel(atmosphereGroupBox);
        label_21->setObjectName("label_21");
        label_21->setText(QString::fromUtf8("+ const"));

        gridLayout->addWidget(label_21, 0, 8, 1, 1);

        label_28 = new QLabel(atmosphereGroupBox);
        label_28->setObjectName("label_28");

        gridLayout->addWidget(label_28, 1, 9, 1, 1);

        label_29 = new QLabel(atmosphereGroupBox);
        label_29->setObjectName("label_29");

        gridLayout->addWidget(label_29, 2, 9, 1, 1);

        label_30 = new QLabel(atmosphereGroupBox);
        label_30->setObjectName("label_30");

        gridLayout->addWidget(label_30, 3, 9, 1, 1);

        label_31 = new QLabel(atmosphereGroupBox);
        label_31->setObjectName("label_31");

        gridLayout->addWidget(label_31, 4, 9, 1, 1);

        label_32 = new QLabel(atmosphereGroupBox);
        label_32->setObjectName("label_32");

        gridLayout->addWidget(label_32, 5, 9, 1, 1);

        pushButton_ResetDistYPreetham = new QPushButton(atmosphereGroupBox);
        pushButton_ResetDistYPreetham->setObjectName("pushButton_ResetDistYPreetham");

        gridLayout->addWidget(pushButton_ResetDistYPreetham, 6, 2, 1, 1);

        pushButton_ResetDistYStel = new QPushButton(atmosphereGroupBox);
        pushButton_ResetDistYStel->setObjectName("pushButton_ResetDistYStel");

        gridLayout->addWidget(pushButton_ResetDistYStel, 6, 1, 1, 1);

        pushButton_ResetDistxPreetham = new QPushButton(atmosphereGroupBox);
        pushButton_ResetDistxPreetham->setObjectName("pushButton_ResetDistxPreetham");

        gridLayout->addWidget(pushButton_ResetDistxPreetham, 6, 5, 1, 1);

        pushButton_ResetDistxStel = new QPushButton(atmosphereGroupBox);
        pushButton_ResetDistxStel->setObjectName("pushButton_ResetDistxStel");

        gridLayout->addWidget(pushButton_ResetDistxStel, 6, 4, 1, 1);

        pushButton_ResetDistyPreetham = new QPushButton(atmosphereGroupBox);
        pushButton_ResetDistyPreetham->setObjectName("pushButton_ResetDistyPreetham");

        gridLayout->addWidget(pushButton_ResetDistyPreetham, 6, 8, 1, 1);

        pushButton_ResetDistyStel = new QPushButton(atmosphereGroupBox);
        pushButton_ResetDistyStel->setObjectName("pushButton_ResetDistyStel");

        gridLayout->addWidget(pushButton_ResetDistyStel, 6, 7, 1, 1);


        verticalLayout_3->addLayout(gridLayout);

        verticalSpacer_2 = new QSpacerItem(0, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_3->addItem(verticalSpacer_2);

        label_36 = new QLabel(atmosphereGroupBox);
        label_36->setObjectName("label_36");

        verticalLayout_3->addWidget(label_36);


        verticalLayout->addWidget(atmosphereGroupBox);


        retranslateUi(skylightDialogForm);

        QMetaObject::connectSlotsByName(skylightDialogForm);
    } // setupUi

    void retranslateUi(QWidget *skylightDialogForm)
    {
        titleBar->setTitle(QCoreApplication::translate("skylightDialogForm", "Skylight Details", nullptr));
        atmosphereGroupBox->setTitle(QCoreApplication::translate("skylightDialogForm", "Skylight Settings", nullptr));
        comboBox_skylightDecimals->setCurrentText(QString());
        label_skylightDecimals->setText(QCoreApplication::translate("skylightDialogForm", "switch parameters below by", nullptr));
        checkBox_drawSunAfterAtmosphere->setText(QCoreApplication::translate("skylightDialogForm", "Draw Sun after atmosphere (attempt better red ball effect)", nullptr));
        checkBox_earlySunHalo->setText(QCoreApplication::translate("skylightDialogForm", "Draw solar glare before Sun", nullptr));
#if QT_CONFIG(tooltip)
        resetStellPushButton->setToolTip(QCoreApplication::translate("skylightDialogForm", "Reset all ABCDE values for zenith and distribution to those in Stellarium's original implementation paper", nullptr));
#endif // QT_CONFIG(tooltip)
        resetStellPushButton->setText(QCoreApplication::translate("skylightDialogForm", "Reset all xy (Stellarium)", nullptr));
#if QT_CONFIG(tooltip)
        resetPreetPushButton->setToolTip(QCoreApplication::translate("skylightDialogForm", "Reset all ABCDE values for zenith and distribution to those in Preetham's 1999 paper", nullptr));
#endif // QT_CONFIG(tooltip)
        resetPreetPushButton->setText(QCoreApplication::translate("skylightDialogForm", "Reset all xy (Preetham)", nullptr));
#if QT_CONFIG(tooltip)
        checkBox_Schaefer->setToolTip(QCoreApplication::translate("skylightDialogForm", "Schaefer's skylight model is better than the result from the CIE skylight distribution.", nullptr));
#endif // QT_CONFIG(tooltip)
        checkBox_Schaefer->setText(QCoreApplication::translate("skylightDialogForm", "Y from Schaefer, not Preetham sky", nullptr));
        label_22->setText(QCoreApplication::translate("skylightDialogForm", "[Preetham]", nullptr));
        pushButton_ResetZYstel->setText(QCoreApplication::translate("skylightDialogForm", "Reset: Stellarium", nullptr));
        pushButton_ResetZYpreetham->setText(QCoreApplication::translate("skylightDialogForm", "Reset: Preetham", nullptr));
        pushButton_ResetZXstel->setText(QCoreApplication::translate("skylightDialogForm", "Reset: Stellarium", nullptr));
        pushButton_ResetZXpreetham->setText(QCoreApplication::translate("skylightDialogForm", "Reset: Preetham", nullptr));
        label_23->setText(QCoreApplication::translate("skylightDialogForm", "with zenith angle &theta;, solar zenith angle &theta;<sub>s</sub>, solar angle &gamma;", nullptr));
        label_28->setText(QCoreApplication::translate("skylightDialogForm", "Horizon bright/dark", nullptr));
        label_29->setText(QCoreApplication::translate("skylightDialogForm", "Hor.Gradient", nullptr));
        label_30->setText(QCoreApplication::translate("skylightDialogForm", "circumsolar Intensity", nullptr));
        label_31->setText(QCoreApplication::translate("skylightDialogForm", "highlightSize", nullptr));
        label_32->setText(QCoreApplication::translate("skylightDialogForm", "rel. backscatter", nullptr));
        pushButton_ResetDistYPreetham->setText(QCoreApplication::translate("skylightDialogForm", "Reset: Preetham", nullptr));
        pushButton_ResetDistYStel->setText(QCoreApplication::translate("skylightDialogForm", "Reset: Stellarium", nullptr));
        pushButton_ResetDistxPreetham->setText(QCoreApplication::translate("skylightDialogForm", "Reset: Preetham", nullptr));
        pushButton_ResetDistxStel->setText(QCoreApplication::translate("skylightDialogForm", "Reset: Stellarium", nullptr));
        pushButton_ResetDistyPreetham->setText(QCoreApplication::translate("skylightDialogForm", "Reset: Preetham", nullptr));
        pushButton_ResetDistyStel->setText(QCoreApplication::translate("skylightDialogForm", "Reset: Stellarium", nullptr));
        label_36->setText(QCoreApplication::translate("skylightDialogForm", "Note: when the sky brightness (CIE Y) is set by Schaefer's model, the bottom-left settings will not visibly change anything.", nullptr));
        (void)skylightDialogForm;
    } // retranslateUi

};

namespace Ui {
    class skylightDialogForm: public Ui_skylightDialogForm {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SKYLIGHTDIALOG_H
