/********************************************************************************
** Form generated from reading UI file 'satellitesFilterDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SATELLITESFILTERDIALOG_H
#define UI_SATELLITESFILTERDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "Dialog.hpp"
#include "gui/StelCloseButton.hpp"

QT_BEGIN_NAMESPACE

class Ui_satellitesFilterDialog
{
public:
    QVBoxLayout *verticalLayout;
    TitleBar *titleBar;
    QGroupBox *groupBox;
    QGridLayout *gridLayout_2;
    QDoubleSpinBox *maxPerigee;
    QDoubleSpinBox *maxApogee;
    QCheckBox *inclinationCheckBox;
    QLabel *labelMax;
    QCheckBox *stdMagnitudeCheckBox;
    QDoubleSpinBox *maxEccentricity;
    QDoubleSpinBox *minApogee;
    QDoubleSpinBox *minPeriod;
    QDoubleSpinBox *minEccentricity;
    QDoubleSpinBox *maxRCS;
    QDoubleSpinBox *maxPeriod;
    QDoubleSpinBox *minRCS;
    QLabel *labelMin;
    QSpacerItem *verticalSpacer;
    QCheckBox *eccentricityCheckBox;
    QCheckBox *periodCheckBox;
    QDoubleSpinBox *minPerigee;
    QDoubleSpinBox *minInclination;
    QDoubleSpinBox *maxInclination;
    QCheckBox *rcsCheckBox;
    QCheckBox *perigeeCheckBox;
    QCheckBox *apogeeCheckBox;

    void setupUi(QWidget *satellitesFilterDialog)
    {
        if (satellitesFilterDialog->objectName().isEmpty())
            satellitesFilterDialog->setObjectName("satellitesFilterDialog");
        satellitesFilterDialog->resize(312, 316);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(satellitesFilterDialog->sizePolicy().hasHeightForWidth());
        satellitesFilterDialog->setSizePolicy(sizePolicy);
        verticalLayout = new QVBoxLayout(satellitesFilterDialog);
        verticalLayout->setSpacing(0);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        titleBar = new TitleBar(satellitesFilterDialog);
        titleBar->setObjectName("titleBar");

        verticalLayout->addWidget(titleBar);

        groupBox = new QGroupBox(satellitesFilterDialog);
        groupBox->setObjectName("groupBox");
        gridLayout_2 = new QGridLayout(groupBox);
        gridLayout_2->setObjectName("gridLayout_2");
        gridLayout_2->setContentsMargins(0, 0, 0, 0);
        maxPerigee = new QDoubleSpinBox(groupBox);
        maxPerigee->setObjectName("maxPerigee");
        maxPerigee->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        maxPerigee->setDecimals(0);
        maxPerigee->setMinimum(70.000000000000000);
        maxPerigee->setMaximum(1000000.000000000000000);
        maxPerigee->setValue(1500.000000000000000);

        gridLayout_2->addWidget(maxPerigee, 5, 2, 1, 1);

        maxApogee = new QDoubleSpinBox(groupBox);
        maxApogee->setObjectName("maxApogee");
        maxApogee->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        maxApogee->setDecimals(0);
        maxApogee->setMinimum(70.000000000000000);
        maxApogee->setMaximum(1000000.000000000000000);
        maxApogee->setValue(55000.000000000000000);

        gridLayout_2->addWidget(maxApogee, 4, 2, 1, 1);

        inclinationCheckBox = new QCheckBox(groupBox);
        inclinationCheckBox->setObjectName("inclinationCheckBox");

        gridLayout_2->addWidget(inclinationCheckBox, 1, 0, 1, 1);

        labelMax = new QLabel(groupBox);
        labelMax->setObjectName("labelMax");
        labelMax->setAlignment(Qt::AlignCenter);

        gridLayout_2->addWidget(labelMax, 0, 2, 1, 1);

        stdMagnitudeCheckBox = new QCheckBox(groupBox);
        stdMagnitudeCheckBox->setObjectName("stdMagnitudeCheckBox");
        stdMagnitudeCheckBox->setEnabled(true);

        gridLayout_2->addWidget(stdMagnitudeCheckBox, 7, 0, 1, 3);

        maxEccentricity = new QDoubleSpinBox(groupBox);
        maxEccentricity->setObjectName("maxEccentricity");
        maxEccentricity->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        maxEccentricity->setDecimals(1);
        maxEccentricity->setMaximum(1.000000000000000);
        maxEccentricity->setSingleStep(0.100000000000000);
        maxEccentricity->setValue(0.900000000000000);

        gridLayout_2->addWidget(maxEccentricity, 3, 2, 1, 1);

        minApogee = new QDoubleSpinBox(groupBox);
        minApogee->setObjectName("minApogee");
        minApogee->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        minApogee->setDecimals(0);
        minApogee->setMinimum(1.000000000000000);
        minApogee->setMaximum(1000000.000000000000000);
        minApogee->setValue(20000.000000000000000);

        gridLayout_2->addWidget(minApogee, 4, 1, 1, 1);

        minPeriod = new QDoubleSpinBox(groupBox);
        minPeriod->setObjectName("minPeriod");
        minPeriod->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        minPeriod->setDecimals(0);
        minPeriod->setMaximum(100000.000000000000000);

        gridLayout_2->addWidget(minPeriod, 2, 1, 1, 1);

        minEccentricity = new QDoubleSpinBox(groupBox);
        minEccentricity->setObjectName("minEccentricity");
        minEccentricity->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        minEccentricity->setDecimals(1);
        minEccentricity->setMaximum(1.000000000000000);
        minEccentricity->setSingleStep(0.100000000000000);
        minEccentricity->setValue(0.300000000000000);

        gridLayout_2->addWidget(minEccentricity, 3, 1, 1, 1);

        maxRCS = new QDoubleSpinBox(groupBox);
        maxRCS->setObjectName("maxRCS");
        maxRCS->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        maxRCS->setMinimum(0.100000000000000);
        maxRCS->setMaximum(10000.000000000000000);

        gridLayout_2->addWidget(maxRCS, 6, 2, 1, 1);

        maxPeriod = new QDoubleSpinBox(groupBox);
        maxPeriod->setObjectName("maxPeriod");
        maxPeriod->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        maxPeriod->setDecimals(0);
        maxPeriod->setMaximum(100000.000000000000000);
        maxPeriod->setValue(150.000000000000000);

        gridLayout_2->addWidget(maxPeriod, 2, 2, 1, 1);

        minRCS = new QDoubleSpinBox(groupBox);
        minRCS->setObjectName("minRCS");
        minRCS->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        minRCS->setMinimum(0.100000000000000);
        minRCS->setMaximum(10000.000000000000000);

        gridLayout_2->addWidget(minRCS, 6, 1, 1, 1);

        labelMin = new QLabel(groupBox);
        labelMin->setObjectName("labelMin");
        labelMin->setAlignment(Qt::AlignCenter);

        gridLayout_2->addWidget(labelMin, 0, 1, 1, 1);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_2->addItem(verticalSpacer, 8, 0, 1, 1);

        eccentricityCheckBox = new QCheckBox(groupBox);
        eccentricityCheckBox->setObjectName("eccentricityCheckBox");

        gridLayout_2->addWidget(eccentricityCheckBox, 3, 0, 1, 1);

        periodCheckBox = new QCheckBox(groupBox);
        periodCheckBox->setObjectName("periodCheckBox");

        gridLayout_2->addWidget(periodCheckBox, 2, 0, 1, 1);

        minPerigee = new QDoubleSpinBox(groupBox);
        minPerigee->setObjectName("minPerigee");
        minPerigee->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        minPerigee->setDecimals(0);
        minPerigee->setMinimum(1.000000000000000);
        minPerigee->setMaximum(1000000.000000000000000);
        minPerigee->setValue(200.000000000000000);

        gridLayout_2->addWidget(minPerigee, 5, 1, 1, 1);

        minInclination = new QDoubleSpinBox(groupBox);
        minInclination->setObjectName("minInclination");
        minInclination->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        minInclination->setDecimals(0);
        minInclination->setMaximum(360.000000000000000);
        minInclination->setValue(120.000000000000000);

        gridLayout_2->addWidget(minInclination, 1, 1, 1, 1);

        maxInclination = new QDoubleSpinBox(groupBox);
        maxInclination->setObjectName("maxInclination");
        maxInclination->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        maxInclination->setDecimals(0);
        maxInclination->setMaximum(360.000000000000000);
        maxInclination->setValue(180.000000000000000);

        gridLayout_2->addWidget(maxInclination, 1, 2, 1, 1);

        rcsCheckBox = new QCheckBox(groupBox);
        rcsCheckBox->setObjectName("rcsCheckBox");

        gridLayout_2->addWidget(rcsCheckBox, 6, 0, 1, 1);

        perigeeCheckBox = new QCheckBox(groupBox);
        perigeeCheckBox->setObjectName("perigeeCheckBox");

        gridLayout_2->addWidget(perigeeCheckBox, 5, 0, 1, 1);

        apogeeCheckBox = new QCheckBox(groupBox);
        apogeeCheckBox->setObjectName("apogeeCheckBox");

        gridLayout_2->addWidget(apogeeCheckBox, 4, 0, 1, 1);


        verticalLayout->addWidget(groupBox);


        retranslateUi(satellitesFilterDialog);

        QMetaObject::connectSlotsByName(satellitesFilterDialog);
    } // setupUi

    void retranslateUi(QWidget *satellitesFilterDialog)
    {
        titleBar->setTitle(QCoreApplication::translate("satellitesFilterDialog", "Satellites filter", nullptr));
        groupBox->setTitle(QCoreApplication::translate("satellitesFilterDialog", "Settings of custom filter", nullptr));
        inclinationCheckBox->setText(QCoreApplication::translate("satellitesFilterDialog", "Inclination", nullptr));
        labelMax->setText(QCoreApplication::translate("satellitesFilterDialog", "Max.", nullptr));
        stdMagnitudeCheckBox->setText(QCoreApplication::translate("satellitesFilterDialog", "Known standard magnitude", nullptr));
        labelMin->setText(QCoreApplication::translate("satellitesFilterDialog", "Min.", nullptr));
        eccentricityCheckBox->setText(QCoreApplication::translate("satellitesFilterDialog", "Eccentricity", nullptr));
        periodCheckBox->setText(QCoreApplication::translate("satellitesFilterDialog", "Period", nullptr));
        rcsCheckBox->setText(QCoreApplication::translate("satellitesFilterDialog", "RCS", nullptr));
        perigeeCheckBox->setText(QCoreApplication::translate("satellitesFilterDialog", "Perigee", nullptr));
        apogeeCheckBox->setText(QCoreApplication::translate("satellitesFilterDialog", "Apogee", nullptr));
        (void)satellitesFilterDialog;
    } // retranslateUi

};

namespace Ui {
    class satellitesFilterDialog: public Ui_satellitesFilterDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SATELLITESFILTERDIALOG_H
