/********************************************************************************
** Form generated from reading UI file 'scmStartDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SCMSTARTDIALOG_H
#define UI_SCMSTARTDIALOG_H

#include <Dialog.hpp>
#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_scmStartDialog
{
public:
    QVBoxLayout *vboxLayout;
    TitleBar *titleBar;
    QFrame *Content;
    QGridLayout *gridLayout_2;
    QFrame *frame;
    QGridLayout *gridLayout_3;
    QVBoxLayout *verticalLayout;
    QLabel *welcomeLabel;
    QHBoxLayout *horizontalLayout;
    QPushButton *scmStartCreatepushButton;
    QPushButton *scmStartEditpushButton;
    QPushButton *scmStartConvertpushButton;
    QPushButton *scmStartCancelpushButton;

    void setupUi(QWidget *scmStartDialog)
    {
        if (scmStartDialog->objectName().isEmpty())
            scmStartDialog->setObjectName("scmStartDialog");
        scmStartDialog->resize(304, 93);
        scmStartDialog->setStyleSheet(QString::fromUtf8(""));
        vboxLayout = new QVBoxLayout(scmStartDialog);
        vboxLayout->setSpacing(0);
        vboxLayout->setObjectName("vboxLayout");
        vboxLayout->setContentsMargins(0, 0, 0, 0);
        titleBar = new TitleBar(scmStartDialog);
        titleBar->setObjectName("titleBar");

        vboxLayout->addWidget(titleBar);

        Content = new QFrame(scmStartDialog);
        Content->setObjectName("Content");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Minimum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(Content->sizePolicy().hasHeightForWidth());
        Content->setSizePolicy(sizePolicy);
        Content->setFrameShape(QFrame::StyledPanel);
        gridLayout_2 = new QGridLayout(Content);
        gridLayout_2->setSpacing(0);
        gridLayout_2->setObjectName("gridLayout_2");
        gridLayout_2->setContentsMargins(0, 0, 0, 0);
        frame = new QFrame(Content);
        frame->setObjectName("frame");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::MinimumExpanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(frame->sizePolicy().hasHeightForWidth());
        frame->setSizePolicy(sizePolicy1);
        frame->setFrameShape(QFrame::StyledPanel);
        frame->setFrameShadow(QFrame::Raised);
        gridLayout_3 = new QGridLayout(frame);
        gridLayout_3->setSpacing(0);
        gridLayout_3->setObjectName("gridLayout_3");
        gridLayout_3->setContentsMargins(0, 0, 0, 0);
        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName("verticalLayout");
        welcomeLabel = new QLabel(frame);
        welcomeLabel->setObjectName("welcomeLabel");
        welcomeLabel->setAlignment(Qt::AlignCenter);
        welcomeLabel->setWordWrap(true);
        welcomeLabel->setMargin(20);

        verticalLayout->addWidget(welcomeLabel);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(5);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(5, 5, 5, 5);
        scmStartCreatepushButton = new QPushButton(frame);
        scmStartCreatepushButton->setObjectName("scmStartCreatepushButton");

        horizontalLayout->addWidget(scmStartCreatepushButton);

        scmStartEditpushButton = new QPushButton(frame);
        scmStartEditpushButton->setObjectName("scmStartEditpushButton");
        scmStartEditpushButton->setEnabled(false);

        horizontalLayout->addWidget(scmStartEditpushButton);

        scmStartConvertpushButton = new QPushButton(frame);
        scmStartConvertpushButton->setObjectName("scmStartConvertpushButton");

        horizontalLayout->addWidget(scmStartConvertpushButton);

        scmStartCancelpushButton = new QPushButton(frame);
        scmStartCancelpushButton->setObjectName("scmStartCancelpushButton");

        horizontalLayout->addWidget(scmStartCancelpushButton);


        verticalLayout->addLayout(horizontalLayout);


        gridLayout_3->addLayout(verticalLayout, 0, 0, 1, 1);


        gridLayout_2->addWidget(frame, 0, 0, 1, 2);


        vboxLayout->addWidget(Content);


        retranslateUi(scmStartDialog);

        QMetaObject::connectSlotsByName(scmStartDialog);
    } // setupUi

    void retranslateUi(QWidget *scmStartDialog)
    {
        titleBar->setProperty("title", QVariant(QCoreApplication::translate("scmStartDialog", "Sky Culture Maker", nullptr)));
        welcomeLabel->setText(QCoreApplication::translate("scmStartDialog", "Welcome to the Sky Culture Maker!", nullptr));
#if QT_CONFIG(tooltip)
        scmStartCreatepushButton->setToolTip(QCoreApplication::translate("scmStartDialog", "Create a new sky culture", nullptr));
#endif // QT_CONFIG(tooltip)
        scmStartCreatepushButton->setText(QCoreApplication::translate("scmStartDialog", "Create", nullptr));
#if QT_CONFIG(tooltip)
        scmStartEditpushButton->setToolTip(QCoreApplication::translate("scmStartDialog", "Not implemented yet.", nullptr));
#endif // QT_CONFIG(tooltip)
        scmStartEditpushButton->setText(QCoreApplication::translate("scmStartDialog", "Edit", nullptr));
        scmStartConvertpushButton->setText(QCoreApplication::translate("scmStartDialog", "Convert", nullptr));
        scmStartCancelpushButton->setText(QCoreApplication::translate("scmStartDialog", "Cancel", nullptr));
        (void)scmStartDialog;
    } // retranslateUi

};

namespace Ui {
    class scmStartDialog: public Ui_scmStartDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SCMSTARTDIALOG_H
