/****************************************************************************
** Meta object code from reading C++ file 'AngleMeasure.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../stellarium/plugins/AngleMeasure/src/AngleMeasure.hpp"
#include <QtCore/qmetatype.h>
#include <QtCore/qplugin.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'AngleMeasure.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12AngleMeasureE_t {};
} // unnamed namespace

template <> constexpr inline auto AngleMeasure::qt_create_metaobjectdata<qt_meta_tag_ZN12AngleMeasureE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "AngleMeasure",
        "flagAngleMeasureChanged",
        "",
        "b",
        "flagPanAndSelectChanged",
        "flagFollowCursorChanged",
        "dmsFormatChanged",
        "flagShowEquatorialChanged",
        "flagShowHorizontalChanged",
        "flagShowEquatorialPAChanged",
        "flagShowHorizontalPAChanged",
        "flagShowHorizontalStartSkylinkedChanged",
        "flagShowHorizontalEndSkylinkedChanged",
        "equatorialTextColorChanged",
        "Vec3f",
        "c",
        "equatorialLineColorChanged",
        "horizontalTextColorChanged",
        "horizontalLineColorChanged",
        "isEnabled",
        "isPanAndSelect",
        "isFollowCursor",
        "isDmsFormat",
        "isEquatorial",
        "isHorizontal",
        "isEquatorialPADisplayed",
        "isHorizontalPADisplayed",
        "isHorizontalStartSkylinked",
        "isHorizontalEndSkylinked",
        "getEquatorialTextColor",
        "getEquatorialLineColor",
        "getHorizontalTextColor",
        "getHorizontalLineColor",
        "enableAngleMeasure",
        "followCursor",
        "useDmsFormat",
        "showEquatorialPA",
        "showHorizontalPA",
        "showEquatorial",
        "showHorizontal",
        "showHorizontalStartSkylinked",
        "showHorizontalEndSkylinked",
        "panAndSelect",
        "setEquatorialTextColor",
        "color",
        "setEquatorialLineColor",
        "setHorizontalTextColor",
        "setHorizontalLineColor",
        "updateMessageText",
        "clearMessage",
        "enabled",
        "flagPanAndSelect",
        "flagFollowCursor",
        "dmsFormat",
        "flagShowEquatorial",
        "flagShowHorizontal",
        "flagShowHorizontalStartSkylinked",
        "flagShowHorizontalEndSkylinked",
        "flagShowEquatorialPA",
        "flagShowHorizontalPA",
        "equatorialTextColor",
        "equatorialLineColor",
        "horizontalTextColor",
        "horizontalLineColor"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'flagAngleMeasureChanged'
        QtMocHelpers::SignalData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagPanAndSelectChanged'
        QtMocHelpers::SignalData<void(bool)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagFollowCursorChanged'
        QtMocHelpers::SignalData<void(bool)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'dmsFormatChanged'
        QtMocHelpers::SignalData<void(bool)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagShowEquatorialChanged'
        QtMocHelpers::SignalData<void(bool)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagShowHorizontalChanged'
        QtMocHelpers::SignalData<void(bool)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagShowEquatorialPAChanged'
        QtMocHelpers::SignalData<void(bool)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagShowHorizontalPAChanged'
        QtMocHelpers::SignalData<void(bool)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagShowHorizontalStartSkylinkedChanged'
        QtMocHelpers::SignalData<void(bool)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagShowHorizontalEndSkylinkedChanged'
        QtMocHelpers::SignalData<void(bool)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'equatorialTextColorChanged'
        QtMocHelpers::SignalData<void(const Vec3f &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Signal 'equatorialLineColorChanged'
        QtMocHelpers::SignalData<void(const Vec3f &)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Signal 'horizontalTextColorChanged'
        QtMocHelpers::SignalData<void(const Vec3f &)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Signal 'horizontalLineColorChanged'
        QtMocHelpers::SignalData<void(const Vec3f &)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Slot 'isEnabled'
        QtMocHelpers::SlotData<bool() const>(19, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'isPanAndSelect'
        QtMocHelpers::SlotData<bool() const>(20, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'isFollowCursor'
        QtMocHelpers::SlotData<bool() const>(21, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'isDmsFormat'
        QtMocHelpers::SlotData<bool() const>(22, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'isEquatorial'
        QtMocHelpers::SlotData<bool() const>(23, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'isHorizontal'
        QtMocHelpers::SlotData<bool() const>(24, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'isEquatorialPADisplayed'
        QtMocHelpers::SlotData<bool() const>(25, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'isHorizontalPADisplayed'
        QtMocHelpers::SlotData<bool() const>(26, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'isHorizontalStartSkylinked'
        QtMocHelpers::SlotData<bool() const>(27, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'isHorizontalEndSkylinked'
        QtMocHelpers::SlotData<bool() const>(28, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'getEquatorialTextColor'
        QtMocHelpers::SlotData<Vec3f() const>(29, 2, QMC::AccessPublic, 0x80000000 | 14),
        // Slot 'getEquatorialLineColor'
        QtMocHelpers::SlotData<Vec3f() const>(30, 2, QMC::AccessPublic, 0x80000000 | 14),
        // Slot 'getHorizontalTextColor'
        QtMocHelpers::SlotData<Vec3f() const>(31, 2, QMC::AccessPublic, 0x80000000 | 14),
        // Slot 'getHorizontalLineColor'
        QtMocHelpers::SlotData<Vec3f() const>(32, 2, QMC::AccessPublic, 0x80000000 | 14),
        // Slot 'enableAngleMeasure'
        QtMocHelpers::SlotData<void(bool)>(33, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'followCursor'
        QtMocHelpers::SlotData<void(bool)>(34, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'useDmsFormat'
        QtMocHelpers::SlotData<void(bool)>(35, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'showEquatorialPA'
        QtMocHelpers::SlotData<void(bool)>(36, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'showHorizontalPA'
        QtMocHelpers::SlotData<void(bool)>(37, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'showEquatorial'
        QtMocHelpers::SlotData<void(bool)>(38, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'showHorizontal'
        QtMocHelpers::SlotData<void(bool)>(39, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'showHorizontalStartSkylinked'
        QtMocHelpers::SlotData<void(bool)>(40, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'showHorizontalEndSkylinked'
        QtMocHelpers::SlotData<void(bool)>(41, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'panAndSelect'
        QtMocHelpers::SlotData<void(bool)>(42, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'setEquatorialTextColor'
        QtMocHelpers::SlotData<void(const Vec3f &)>(43, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 44 },
        }}),
        // Slot 'setEquatorialLineColor'
        QtMocHelpers::SlotData<void(const Vec3f &)>(45, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 44 },
        }}),
        // Slot 'setHorizontalTextColor'
        QtMocHelpers::SlotData<void(const Vec3f &)>(46, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 44 },
        }}),
        // Slot 'setHorizontalLineColor'
        QtMocHelpers::SlotData<void(const Vec3f &)>(47, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 44 },
        }}),
        // Slot 'updateMessageText'
        QtMocHelpers::SlotData<void()>(48, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'clearMessage'
        QtMocHelpers::SlotData<void()>(49, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'enabled'
        QtMocHelpers::PropertyData<bool>(50, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 0),
        // property 'flagPanAndSelect'
        QtMocHelpers::PropertyData<bool>(51, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 1),
        // property 'flagFollowCursor'
        QtMocHelpers::PropertyData<bool>(52, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 2),
        // property 'dmsFormat'
        QtMocHelpers::PropertyData<bool>(53, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 3),
        // property 'flagShowEquatorial'
        QtMocHelpers::PropertyData<bool>(54, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 4),
        // property 'flagShowHorizontal'
        QtMocHelpers::PropertyData<bool>(55, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 5),
        // property 'flagShowHorizontalStartSkylinked'
        QtMocHelpers::PropertyData<bool>(56, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 8),
        // property 'flagShowHorizontalEndSkylinked'
        QtMocHelpers::PropertyData<bool>(57, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 9),
        // property 'flagShowEquatorialPA'
        QtMocHelpers::PropertyData<bool>(58, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 6),
        // property 'flagShowHorizontalPA'
        QtMocHelpers::PropertyData<bool>(59, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 7),
        // property 'equatorialTextColor'
        QtMocHelpers::PropertyData<Vec3f>(60, 0x80000000 | 14, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 10),
        // property 'equatorialLineColor'
        QtMocHelpers::PropertyData<Vec3f>(61, 0x80000000 | 14, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 11),
        // property 'horizontalTextColor'
        QtMocHelpers::PropertyData<Vec3f>(62, 0x80000000 | 14, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 12),
        // property 'horizontalLineColor'
        QtMocHelpers::PropertyData<Vec3f>(63, 0x80000000 | 14, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 13),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<AngleMeasure, qt_meta_tag_ZN12AngleMeasureE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject AngleMeasure::staticMetaObject = { {
    QMetaObject::SuperData::link<StelModule::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12AngleMeasureE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12AngleMeasureE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12AngleMeasureE_t>.metaTypes,
    nullptr
} };

void AngleMeasure::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AngleMeasure *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->flagAngleMeasureChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->flagPanAndSelectChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->flagFollowCursorChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->dmsFormatChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->flagShowEquatorialChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->flagShowHorizontalChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->flagShowEquatorialPAChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->flagShowHorizontalPAChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->flagShowHorizontalStartSkylinkedChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 9: _t->flagShowHorizontalEndSkylinkedChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 10: _t->equatorialTextColorChanged((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1]))); break;
        case 11: _t->equatorialLineColorChanged((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1]))); break;
        case 12: _t->horizontalTextColorChanged((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1]))); break;
        case 13: _t->horizontalLineColorChanged((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1]))); break;
        case 14: { bool _r = _t->isEnabled();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 15: { bool _r = _t->isPanAndSelect();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 16: { bool _r = _t->isFollowCursor();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 17: { bool _r = _t->isDmsFormat();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 18: { bool _r = _t->isEquatorial();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 19: { bool _r = _t->isHorizontal();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 20: { bool _r = _t->isEquatorialPADisplayed();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 21: { bool _r = _t->isHorizontalPADisplayed();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 22: { bool _r = _t->isHorizontalStartSkylinked();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 23: { bool _r = _t->isHorizontalEndSkylinked();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 24: { Vec3f _r = _t->getEquatorialTextColor();
            if (_a[0]) *reinterpret_cast<Vec3f*>(_a[0]) = std::move(_r); }  break;
        case 25: { Vec3f _r = _t->getEquatorialLineColor();
            if (_a[0]) *reinterpret_cast<Vec3f*>(_a[0]) = std::move(_r); }  break;
        case 26: { Vec3f _r = _t->getHorizontalTextColor();
            if (_a[0]) *reinterpret_cast<Vec3f*>(_a[0]) = std::move(_r); }  break;
        case 27: { Vec3f _r = _t->getHorizontalLineColor();
            if (_a[0]) *reinterpret_cast<Vec3f*>(_a[0]) = std::move(_r); }  break;
        case 28: _t->enableAngleMeasure((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 29: _t->followCursor((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 30: _t->useDmsFormat((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 31: _t->showEquatorialPA((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 32: _t->showHorizontalPA((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 33: _t->showEquatorial((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 34: _t->showHorizontal((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 35: _t->showHorizontalStartSkylinked((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 36: _t->showHorizontalEndSkylinked((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 37: _t->panAndSelect((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 38: _t->setEquatorialTextColor((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1]))); break;
        case 39: _t->setEquatorialLineColor((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1]))); break;
        case 40: _t->setHorizontalTextColor((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1]))); break;
        case 41: _t->setHorizontalLineColor((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1]))); break;
        case 42: _t->updateMessageText(); break;
        case 43: _t->clearMessage(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 13:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 38:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 39:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 40:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 41:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (AngleMeasure::*)(bool )>(_a, &AngleMeasure::flagAngleMeasureChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (AngleMeasure::*)(bool )>(_a, &AngleMeasure::flagPanAndSelectChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (AngleMeasure::*)(bool )>(_a, &AngleMeasure::flagFollowCursorChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (AngleMeasure::*)(bool )>(_a, &AngleMeasure::dmsFormatChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (AngleMeasure::*)(bool )>(_a, &AngleMeasure::flagShowEquatorialChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (AngleMeasure::*)(bool )>(_a, &AngleMeasure::flagShowHorizontalChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (AngleMeasure::*)(bool )>(_a, &AngleMeasure::flagShowEquatorialPAChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (AngleMeasure::*)(bool )>(_a, &AngleMeasure::flagShowHorizontalPAChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (AngleMeasure::*)(bool )>(_a, &AngleMeasure::flagShowHorizontalStartSkylinkedChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (AngleMeasure::*)(bool )>(_a, &AngleMeasure::flagShowHorizontalEndSkylinkedChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (AngleMeasure::*)(const Vec3f & )>(_a, &AngleMeasure::equatorialTextColorChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (AngleMeasure::*)(const Vec3f & )>(_a, &AngleMeasure::equatorialLineColorChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (AngleMeasure::*)(const Vec3f & )>(_a, &AngleMeasure::horizontalTextColorChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (AngleMeasure::*)(const Vec3f & )>(_a, &AngleMeasure::horizontalLineColorChanged, 13))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 13:
        case 12:
        case 11:
        case 10:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Vec3f >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->isEnabled(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->isPanAndSelect(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->isFollowCursor(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->isDmsFormat(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->isEquatorial(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->isHorizontal(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->isHorizontalStartSkylinked(); break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->isHorizontalEndSkylinked(); break;
        case 8: *reinterpret_cast<bool*>(_v) = _t->isEquatorialPADisplayed(); break;
        case 9: *reinterpret_cast<bool*>(_v) = _t->isHorizontalPADisplayed(); break;
        case 10: *reinterpret_cast<Vec3f*>(_v) = _t->getEquatorialTextColor(); break;
        case 11: *reinterpret_cast<Vec3f*>(_v) = _t->getEquatorialLineColor(); break;
        case 12: *reinterpret_cast<Vec3f*>(_v) = _t->getHorizontalTextColor(); break;
        case 13: *reinterpret_cast<Vec3f*>(_v) = _t->getHorizontalLineColor(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->enableAngleMeasure(*reinterpret_cast<bool*>(_v)); break;
        case 1: _t->panAndSelect(*reinterpret_cast<bool*>(_v)); break;
        case 2: _t->followCursor(*reinterpret_cast<bool*>(_v)); break;
        case 3: _t->useDmsFormat(*reinterpret_cast<bool*>(_v)); break;
        case 4: _t->showEquatorial(*reinterpret_cast<bool*>(_v)); break;
        case 5: _t->showHorizontal(*reinterpret_cast<bool*>(_v)); break;
        case 6: _t->showHorizontalStartSkylinked(*reinterpret_cast<bool*>(_v)); break;
        case 7: _t->showHorizontalEndSkylinked(*reinterpret_cast<bool*>(_v)); break;
        case 8: _t->showEquatorialPA(*reinterpret_cast<bool*>(_v)); break;
        case 9: _t->showHorizontalPA(*reinterpret_cast<bool*>(_v)); break;
        case 10: _t->setEquatorialTextColor(*reinterpret_cast<Vec3f*>(_v)); break;
        case 11: _t->setEquatorialLineColor(*reinterpret_cast<Vec3f*>(_v)); break;
        case 12: _t->setHorizontalTextColor(*reinterpret_cast<Vec3f*>(_v)); break;
        case 13: _t->setHorizontalLineColor(*reinterpret_cast<Vec3f*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *AngleMeasure::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *AngleMeasure::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12AngleMeasureE_t>.strings))
        return static_cast<void*>(this);
    return StelModule::qt_metacast(_clname);
}

int AngleMeasure::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StelModule::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 44)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 44;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 44)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 44;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    return _id;
}

// SIGNAL 0
void AngleMeasure::flagAngleMeasureChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void AngleMeasure::flagPanAndSelectChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void AngleMeasure::flagFollowCursorChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void AngleMeasure::dmsFormatChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void AngleMeasure::flagShowEquatorialChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void AngleMeasure::flagShowHorizontalChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void AngleMeasure::flagShowEquatorialPAChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void AngleMeasure::flagShowHorizontalPAChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void AngleMeasure::flagShowHorizontalStartSkylinkedChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void AngleMeasure::flagShowHorizontalEndSkylinkedChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void AngleMeasure::equatorialTextColorChanged(const Vec3f & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void AngleMeasure::equatorialLineColorChanged(const Vec3f & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void AngleMeasure::horizontalTextColorChanged(const Vec3f & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void AngleMeasure::horizontalLineColorChanged(const Vec3f & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN31AngleMeasureStelPluginInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto AngleMeasureStelPluginInterface::qt_create_metaobjectdata<qt_meta_tag_ZN31AngleMeasureStelPluginInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "AngleMeasureStelPluginInterface"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<AngleMeasureStelPluginInterface, qt_meta_tag_ZN31AngleMeasureStelPluginInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject AngleMeasureStelPluginInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN31AngleMeasureStelPluginInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN31AngleMeasureStelPluginInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN31AngleMeasureStelPluginInterfaceE_t>.metaTypes,
    nullptr
} };

void AngleMeasureStelPluginInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AngleMeasureStelPluginInterface *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *AngleMeasureStelPluginInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *AngleMeasureStelPluginInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN31AngleMeasureStelPluginInterfaceE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "StelPluginInterface"))
        return static_cast< StelPluginInterface*>(this);
    if (!strcmp(_clname, "org.stellarium.StelPluginInterface/2.0"))
        return static_cast< StelPluginInterface*>(this);
    return QObject::qt_metacast(_clname);
}

int AngleMeasureStelPluginInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}

#ifdef QT_MOC_EXPORT_PLUGIN_V2
static constexpr unsigned char qt_pluginMetaDataV2_AngleMeasureStelPluginInterface[] = {
    0xbf, 
    // "IID"
    0x02,  0x78,  0x26,  'o',  'r',  'g',  '.',  's', 
    't',  'e',  'l',  'l',  'a',  'r',  'i',  'u', 
    'm',  '.',  'S',  't',  'e',  'l',  'P',  'l', 
    'u',  'g',  'i',  'n',  'I',  'n',  't',  'e', 
    'r',  'f',  'a',  'c',  'e',  '/',  '2',  '.', 
    '0', 
    // "className"
    0x03,  0x78,  0x1f,  'A',  'n',  'g',  'l',  'e', 
    'M',  'e',  'a',  's',  'u',  'r',  'e',  'S', 
    't',  'e',  'l',  'P',  'l',  'u',  'g',  'i', 
    'n',  'I',  'n',  't',  'e',  'r',  'f',  'a', 
    'c',  'e', 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN_V2(AngleMeasureStelPluginInterface, AngleMeasureStelPluginInterface, qt_pluginMetaDataV2_AngleMeasureStelPluginInterface)
#else
QT_PLUGIN_METADATA_SECTION
Q_CONSTINIT static constexpr unsigned char qt_pluginMetaData_AngleMeasureStelPluginInterface[] = {
    'Q', 'T', 'M', 'E', 'T', 'A', 'D', 'A', 'T', 'A', ' ', '!',
    // metadata version, Qt version, architectural requirements
    0, QT_VERSION_MAJOR, QT_VERSION_MINOR, qPluginArchRequirements(),
    0xbf, 
    // "IID"
    0x02,  0x78,  0x26,  'o',  'r',  'g',  '.',  's', 
    't',  'e',  'l',  'l',  'a',  'r',  'i',  'u', 
    'm',  '.',  'S',  't',  'e',  'l',  'P',  'l', 
    'u',  'g',  'i',  'n',  'I',  'n',  't',  'e', 
    'r',  'f',  'a',  'c',  'e',  '/',  '2',  '.', 
    '0', 
    // "className"
    0x03,  0x78,  0x1f,  'A',  'n',  'g',  'l',  'e', 
    'M',  'e',  'a',  's',  'u',  'r',  'e',  'S', 
    't',  'e',  'l',  'P',  'l',  'u',  'g',  'i', 
    'n',  'I',  'n',  't',  'e',  'r',  'f',  'a', 
    'c',  'e', 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN(AngleMeasureStelPluginInterface, AngleMeasureStelPluginInterface)
#endif  // QT_MOC_EXPORT_PLUGIN_V2

QT_WARNING_POP
