/****************************************************************************
** Meta object code from reading C++ file 'LocationDialog.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../stellarium/src/gui/LocationDialog.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'LocationDialog.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14LocationDialogE_t {};
} // unnamed namespace

template <> constexpr inline auto LocationDialog::qt_create_metaobjectdata<qt_meta_tag_ZN14LocationDialogE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "LocationDialog",
        "styleChanged",
        "",
        "style",
        "retranslate",
        "reloadLocations",
        "reportEdit",
        "saveTimeZone",
        "setTimezone",
        "tz",
        "updateFromProgram",
        "StelLocation",
        "newLocation",
        "setLocationFromMap",
        "longitude",
        "latitude",
        "QColor",
        "color",
        "setLocationFromList",
        "QModelIndex",
        "index",
        "moveToAnotherPlanet",
        "setLocationFromCoords",
        "i",
        "addCurrentLocationToList",
        "deleteCurrentLocationFromList",
        "filterSitesByRegion",
        "resetLocationList",
        "ipQueryLocation",
        "state",
        "setDisplayFormatForSpins",
        "flagDecimalDegrees",
        "gpsEnableQueryLocation",
        "enable",
        "gpsReturn",
        "success",
        "resetGPSbuttonLabel",
        "setDefaultLocation",
        "updateTimeZoneControls",
        "useCustomTimeZone"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'styleChanged'
        QtMocHelpers::SlotData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'retranslate'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'reloadLocations'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'reportEdit'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'saveTimeZone'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setTimezone'
        QtMocHelpers::SlotData<void(QString)>(8, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 9 },
        }}),
        // Slot 'updateFromProgram'
        QtMocHelpers::SlotData<void(const StelLocation &)>(10, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'setLocationFromMap'
        QtMocHelpers::SlotData<void(double, double, const QColor &)>(13, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 14 }, { QMetaType::Double, 15 }, { 0x80000000 | 16, 17 },
        }}),
        // Slot 'setLocationFromList'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(18, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 19, 20 },
        }}),
        // Slot 'moveToAnotherPlanet'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setLocationFromCoords'
        QtMocHelpers::SlotData<void(int)>(22, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 23 },
        }}),
        // Slot 'setLocationFromCoords'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPrivate | QMC::MethodCloned, QMetaType::Void),
        // Slot 'addCurrentLocationToList'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'deleteCurrentLocationFromList'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'filterSitesByRegion'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'resetLocationList'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'ipQueryLocation'
        QtMocHelpers::SlotData<void(bool)>(28, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 29 },
        }}),
        // Slot 'setDisplayFormatForSpins'
        QtMocHelpers::SlotData<void(bool)>(30, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 31 },
        }}),
        // Slot 'gpsEnableQueryLocation'
        QtMocHelpers::SlotData<void(bool)>(32, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 33 },
        }}),
        // Slot 'gpsReturn'
        QtMocHelpers::SlotData<void(bool)>(34, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 35 },
        }}),
        // Slot 'resetGPSbuttonLabel'
        QtMocHelpers::SlotData<void()>(36, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setDefaultLocation'
        QtMocHelpers::SlotData<void(bool)>(37, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 29 },
        }}),
        // Slot 'updateTimeZoneControls'
        QtMocHelpers::SlotData<void(bool)>(38, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 39 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<LocationDialog, qt_meta_tag_ZN14LocationDialogE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject LocationDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<StelDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14LocationDialogE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14LocationDialogE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14LocationDialogE_t>.metaTypes,
    nullptr
} };

void LocationDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<LocationDialog *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->styleChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->retranslate(); break;
        case 2: _t->reloadLocations(); break;
        case 3: _t->reportEdit(); break;
        case 4: _t->saveTimeZone(); break;
        case 5: _t->setTimezone((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->updateFromProgram((*reinterpret_cast<std::add_pointer_t<StelLocation>>(_a[1]))); break;
        case 7: _t->setLocationFromMap((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QColor>>(_a[3]))); break;
        case 8: _t->setLocationFromList((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 9: _t->moveToAnotherPlanet(); break;
        case 10: _t->setLocationFromCoords((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 11: _t->setLocationFromCoords(); break;
        case 12: _t->addCurrentLocationToList(); break;
        case 13: _t->deleteCurrentLocationFromList(); break;
        case 14: _t->filterSitesByRegion(); break;
        case 15: _t->resetLocationList(); break;
        case 16: _t->ipQueryLocation((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 17: _t->setDisplayFormatForSpins((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 18: _t->gpsEnableQueryLocation((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 19: _t->gpsReturn((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 20: _t->resetGPSbuttonLabel(); break;
        case 21: _t->setDefaultLocation((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 22: _t->updateTimeZoneControls((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *LocationDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *LocationDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14LocationDialogE_t>.strings))
        return static_cast<void*>(this);
    return StelDialog::qt_metacast(_clname);
}

int LocationDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StelDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 23)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 23;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 23)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 23;
    }
    return _id;
}
QT_WARNING_POP
