/********************************************************************************
** Form generated from reading UI file 'satellitesDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SATELLITESDIALOG_H
#define UI_SATELLITESDIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDateTimeEdit>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListView>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QSplitter>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTextBrowser>
#include <QtWidgets/QTextEdit>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QTreeWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "Dialog.hpp"
#include "gui/ColorButton.hpp"

QT_BEGIN_NAMESPACE

class Ui_satellitesDialog
{
public:
    QVBoxLayout *verticalLayout_2;
    TitleBar *titleBar;
    QTabWidget *tabs;
    QWidget *settingsTab;
    QVBoxLayout *verticalLayoutSettungsTab;
    QGroupBox *updatesGroup;
    QGridLayout *updatesGridLayout;
    QHBoxLayout *horizontalLayoutAutoAdd;
    QCheckBox *checkBoxAutoAdd;
    QPushButton *jumpToSourcesButton;
    QCheckBox *internetUpdatesCheckbox;
    QGridLayout *gridLayout_7;
    QHBoxLayout *horizontalLayout_11;
    QLabel *updateFrequencyLabel;
    QSpacerItem *horizontalSpacer_6;
    QSpinBox *updateFrequencySpinBox;
    QHBoxLayout *horizontalLayout_10;
    QLabel *label_7;
    QSpacerItem *horizontalSpacer;
    QDateTimeEdit *lastUpdateDateTimeEdit;
    QLabel *nextUpdateLabel;
    QPushButton *updateButton;
    QCheckBox *checkBoxAutoRemove;
    QCheckBox *checkBoxAutoDisplay;
    QGroupBox *visualisationBox;
    QGridLayout *gridLayout_9;
    QHBoxLayout *horizontalLayout_16;
    QCheckBox *orbitLinesCheckBox;
    QSpacerItem *horizontalSpacer_13;
    QSpinBox *orbitSegmentsSpin;
    QSpinBox *orbitDurationSpin;
    QSpinBox *orbitFadeSpin;
    QSpinBox *orbitThicknessSpin;
    QHBoxLayout *horizontalLayout_9;
    QCheckBox *umbraCheckBox;
    QSpacerItem *horizontalSpacer_3;
    ColorButton *umbraColor;
    QGridLayout *gridLayout_2;
    QDoubleSpinBox *minAltitude;
    QDoubleSpinBox *maxAltitude;
    QCheckBox *magnitudeCheckBox;
    QCheckBox *altitudeCheckBox;
    QDoubleSpinBox *minMagnitude;
    QDoubleSpinBox *maxMagnitude;
    QSpacerItem *horizontalSpacer_12;
    QGridLayout *gridLayout_3;
    QSpacerItem *horizontalSpacer_14;
    QCheckBox *penumbraCheckBox;
    ColorButton *penumbraColor;
    QCheckBox *umbraAtAltitude;
    QDoubleSpinBox *umbraAltitude;
    QHBoxLayout *horizontalLayout_15;
    QCheckBox *iconicCheckBox;
    QSpacerItem *horizontalSpacer_4;
    ColorButton *transitColorButton;
    QCheckBox *hideInvisibleSatellites;
    QHBoxLayout *horizontalLayout_13;
    QCheckBox *labelsCheckBox;
    QSpacerItem *horizontalSpacer_11;
    QSpinBox *fontSizeSpinBox;
    QHBoxLayout *horizontalLayout_18;
    QCheckBox *coloredInvisibleSatellites;
    QSpacerItem *horizontalSpacer_5;
    ColorButton *invisibleColorButton;
    QSpacerItem *verticalSpacer_3;
    QFrame *frame;
    QHBoxLayout *horizontalLayout_7;
    QPushButton *restoreDefaultsButton;
    QSpacerItem *horizontalSpacer_7;
    QPushButton *saveSettingsButton;
    QWidget *satellitesTab;
    QVBoxLayout *verticalLayout_3;
    QSplitter *splitter;
    QWidget *layoutWidgetSatList;
    QVBoxLayout *satlist_layout;
    QHBoxLayout *horizontalLayout_19;
    QComboBox *groupFilterCombo;
    QToolButton *customFilterButton;
    QLineEdit *lineEditSearch;
    QListView *satellitesList;
    QHBoxLayout *horizontalLayout_2;
    QToolButton *addSatellitesButton;
    QToolButton *removeSatellitesButton;
    QToolButton *commSatelliteButton;
    QToolButton *selectAllButton;
    QSpacerItem *horizontalSpacer_2;
    QToolButton *satMarkerColorPickerButton;
    QToolButton *satOrbitColorPickerButton;
    QToolButton *satInfoColorPickerButton;
    QWidget *layoutWidget;
    QVBoxLayout *sat_details;
    QHBoxLayout *horizontalLayout_4;
    QLabel *label;
    QLineEdit *nameEdit;
    QHBoxLayout *horizontalLayoutCatalogNumber;
    QLabel *labelCatalogNumber;
    QLineEdit *noradNumberEdit;
    QLabel *labelCosparNumber;
    QLineEdit *cosparNumberEdit;
    QHBoxLayout *horizontalLayout_8;
    QLabel *labelStdMagnitude;
    QLineEdit *stdMagnitudeLineEdit;
    QLabel *labelRCS;
    QLineEdit *rcsLineEdit;
    QHBoxLayout *horizontalLayout_5;
    QLabel *labelPerigee;
    QLineEdit *perigeeLineEdit;
    QLabel *labelApogee;
    QLineEdit *apogeeLineEdit;
    QLabel *labelPeriod;
    QLineEdit *periodLineEdit;
    QGridLayout *gridLayout_5;
    QCheckBox *orbitCheckbox;
    QCheckBox *displayedCheckbox;
    QCheckBox *userCheckBox;
    QVBoxLayout *verticalLayout_6;
    QLabel *label_4;
    QTextEdit *descriptionTextEdit;
    QVBoxLayout *verticalLayout_4;
    QLabel *label_3;
    QListWidget *groupsListWidget;
    QVBoxLayout *verticalLayout_5;
    QLabel *labelTle;
    QLineEdit *tleFirstLineEdit;
    QLineEdit *tleSecondLineEdit;
    QHBoxLayout *horizontalLayout_12;
    QLabel *labelTleEpoch;
    QLabel *labelTleEpochData;
    QSpacerItem *horizontalSpacer_8;
    QLabel *validAgeLabel;
    QSpinBox *validAgeSpinBox;
    QWidget *sourcesTab;
    QVBoxLayout *verticalLayout_10;
    QListWidget *sourceList;
    QLabel *labelAutoAdd;
    QHBoxLayout *horizontalLayout;
    QLineEdit *sourceEdit;
    QToolButton *addSourceButton;
    QToolButton *editSourceButton;
    QToolButton *saveSourceButton;
    QToolButton *deleteSourceButton;
    QSpacerItem *horizontalSpacer_10;
    QToolButton *resetSourcesButton;
    QWidget *iridiumTab;
    QGridLayout *gridLayout;
    QTreeWidget *iridiumFlaresTreeWidget;
    QHBoxLayout *horizontalLayout_3;
    QHBoxLayout *horizontalLayout_6;
    QLabel *labelFlaresPredictionDepth;
    QSpinBox *flaresPredictionDepthSpinBox;
    QPushButton *predictIridiumFlaresPushButton;
    QPushButton *predictedIridiumFlaresSaveButton;
    QWidget *aboutTab;
    QVBoxLayout *verticalLayout_11;
    QTextBrowser *aboutTextBrowser;

    void setupUi(QWidget *satellitesDialog)
    {
        if (satellitesDialog->objectName().isEmpty())
            satellitesDialog->setObjectName("satellitesDialog");
        satellitesDialog->resize(664, 485);
        verticalLayout_2 = new QVBoxLayout(satellitesDialog);
        verticalLayout_2->setSpacing(0);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        titleBar = new TitleBar(satellitesDialog);
        titleBar->setObjectName("titleBar");

        verticalLayout_2->addWidget(titleBar);

        tabs = new QTabWidget(satellitesDialog);
        tabs->setObjectName("tabs");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(tabs->sizePolicy().hasHeightForWidth());
        tabs->setSizePolicy(sizePolicy);
        tabs->setDocumentMode(false);
        settingsTab = new QWidget();
        settingsTab->setObjectName("settingsTab");
        verticalLayoutSettungsTab = new QVBoxLayout(settingsTab);
        verticalLayoutSettungsTab->setSpacing(0);
        verticalLayoutSettungsTab->setObjectName("verticalLayoutSettungsTab");
        verticalLayoutSettungsTab->setContentsMargins(0, 0, 0, 0);
        updatesGroup = new QGroupBox(settingsTab);
        updatesGroup->setObjectName("updatesGroup");
        updatesGridLayout = new QGridLayout(updatesGroup);
        updatesGridLayout->setObjectName("updatesGridLayout");
        updatesGridLayout->setContentsMargins(0, 0, 0, 0);
        horizontalLayoutAutoAdd = new QHBoxLayout();
        horizontalLayoutAutoAdd->setObjectName("horizontalLayoutAutoAdd");
        checkBoxAutoAdd = new QCheckBox(updatesGroup);
        checkBoxAutoAdd->setObjectName("checkBoxAutoAdd");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(checkBoxAutoAdd->sizePolicy().hasHeightForWidth());
        checkBoxAutoAdd->setSizePolicy(sizePolicy1);

        horizontalLayoutAutoAdd->addWidget(checkBoxAutoAdd);

        jumpToSourcesButton = new QPushButton(updatesGroup);
        jumpToSourcesButton->setObjectName("jumpToSourcesButton");
        jumpToSourcesButton->setEnabled(false);
        jumpToSourcesButton->setText(QString::fromUtf8("..."));

        horizontalLayoutAutoAdd->addWidget(jumpToSourcesButton);


        updatesGridLayout->addLayout(horizontalLayoutAutoAdd, 1, 0, 1, 1);

        internetUpdatesCheckbox = new QCheckBox(updatesGroup);
        internetUpdatesCheckbox->setObjectName("internetUpdatesCheckbox");

        updatesGridLayout->addWidget(internetUpdatesCheckbox, 0, 0, 1, 1);

        gridLayout_7 = new QGridLayout();
        gridLayout_7->setObjectName("gridLayout_7");
        horizontalLayout_11 = new QHBoxLayout();
        horizontalLayout_11->setObjectName("horizontalLayout_11");
        updateFrequencyLabel = new QLabel(updatesGroup);
        updateFrequencyLabel->setObjectName("updateFrequencyLabel");

        horizontalLayout_11->addWidget(updateFrequencyLabel);

        horizontalSpacer_6 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_11->addItem(horizontalSpacer_6);

        updateFrequencySpinBox = new QSpinBox(updatesGroup);
        updateFrequencySpinBox->setObjectName("updateFrequencySpinBox");
        updateFrequencySpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        updateFrequencySpinBox->setMinimum(1);
        updateFrequencySpinBox->setMaximum(9999);
        updateFrequencySpinBox->setValue(1);

        horizontalLayout_11->addWidget(updateFrequencySpinBox);


        gridLayout_7->addLayout(horizontalLayout_11, 0, 1, 1, 1);

        horizontalLayout_10 = new QHBoxLayout();
        horizontalLayout_10->setObjectName("horizontalLayout_10");
        label_7 = new QLabel(updatesGroup);
        label_7->setObjectName("label_7");

        horizontalLayout_10->addWidget(label_7);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_10->addItem(horizontalSpacer);

        lastUpdateDateTimeEdit = new QDateTimeEdit(updatesGroup);
        lastUpdateDateTimeEdit->setObjectName("lastUpdateDateTimeEdit");
        lastUpdateDateTimeEdit->setEnabled(false);
        lastUpdateDateTimeEdit->setFocusPolicy(Qt::NoFocus);
        lastUpdateDateTimeEdit->setFrame(false);
        lastUpdateDateTimeEdit->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        lastUpdateDateTimeEdit->setButtonSymbols(QAbstractSpinBox::NoButtons);

        horizontalLayout_10->addWidget(lastUpdateDateTimeEdit);


        gridLayout_7->addLayout(horizontalLayout_10, 0, 0, 1, 1);

        nextUpdateLabel = new QLabel(updatesGroup);
        nextUpdateLabel->setObjectName("nextUpdateLabel");
        nextUpdateLabel->setText(QString::fromUtf8("[next update info]"));
        nextUpdateLabel->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);

        gridLayout_7->addWidget(nextUpdateLabel, 1, 0, 1, 1);

        updateButton = new QPushButton(updatesGroup);
        updateButton->setObjectName("updateButton");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Preferred);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(updateButton->sizePolicy().hasHeightForWidth());
        updateButton->setSizePolicy(sizePolicy2);

        gridLayout_7->addWidget(updateButton, 1, 1, 1, 1);


        updatesGridLayout->addLayout(gridLayout_7, 4, 0, 1, 1);

        checkBoxAutoRemove = new QCheckBox(updatesGroup);
        checkBoxAutoRemove->setObjectName("checkBoxAutoRemove");

        updatesGridLayout->addWidget(checkBoxAutoRemove, 2, 0, 1, 1);

        checkBoxAutoDisplay = new QCheckBox(updatesGroup);
        checkBoxAutoDisplay->setObjectName("checkBoxAutoDisplay");

        updatesGridLayout->addWidget(checkBoxAutoDisplay, 3, 0, 1, 1);


        verticalLayoutSettungsTab->addWidget(updatesGroup);

        visualisationBox = new QGroupBox(settingsTab);
        visualisationBox->setObjectName("visualisationBox");
        gridLayout_9 = new QGridLayout(visualisationBox);
        gridLayout_9->setObjectName("gridLayout_9");
        gridLayout_9->setContentsMargins(0, 0, 0, 0);
        horizontalLayout_16 = new QHBoxLayout();
        horizontalLayout_16->setObjectName("horizontalLayout_16");
        orbitLinesCheckBox = new QCheckBox(visualisationBox);
        orbitLinesCheckBox->setObjectName("orbitLinesCheckBox");

        horizontalLayout_16->addWidget(orbitLinesCheckBox);

        horizontalSpacer_13 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_16->addItem(horizontalSpacer_13);

        orbitSegmentsSpin = new QSpinBox(visualisationBox);
        orbitSegmentsSpin->setObjectName("orbitSegmentsSpin");
        orbitSegmentsSpin->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        orbitSegmentsSpin->setMinimum(3);
        orbitSegmentsSpin->setMaximum(9000);
        orbitSegmentsSpin->setValue(180);

        horizontalLayout_16->addWidget(orbitSegmentsSpin);

        orbitDurationSpin = new QSpinBox(visualisationBox);
        orbitDurationSpin->setObjectName("orbitDurationSpin");
        orbitDurationSpin->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        orbitDurationSpin->setMinimum(1);
        orbitDurationSpin->setMaximum(120);
        orbitDurationSpin->setValue(5);

        horizontalLayout_16->addWidget(orbitDurationSpin);

        orbitFadeSpin = new QSpinBox(visualisationBox);
        orbitFadeSpin->setObjectName("orbitFadeSpin");
        orbitFadeSpin->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        orbitFadeSpin->setMaximum(50);
        orbitFadeSpin->setValue(5);

        horizontalLayout_16->addWidget(orbitFadeSpin);

        orbitThicknessSpin = new QSpinBox(visualisationBox);
        orbitThicknessSpin->setObjectName("orbitThicknessSpin");
        orbitThicknessSpin->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        orbitThicknessSpin->setMinimum(1);
        orbitThicknessSpin->setMaximum(5);
        orbitThicknessSpin->setValue(5);

        horizontalLayout_16->addWidget(orbitThicknessSpin);


        gridLayout_9->addLayout(horizontalLayout_16, 1, 0, 1, 1);

        horizontalLayout_9 = new QHBoxLayout();
        horizontalLayout_9->setObjectName("horizontalLayout_9");
        umbraCheckBox = new QCheckBox(visualisationBox);
        umbraCheckBox->setObjectName("umbraCheckBox");

        horizontalLayout_9->addWidget(umbraCheckBox);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_9->addItem(horizontalSpacer_3);

        umbraColor = new ColorButton(visualisationBox);
        umbraColor->setObjectName("umbraColor");

        horizontalLayout_9->addWidget(umbraColor);


        gridLayout_9->addLayout(horizontalLayout_9, 2, 0, 1, 1);

        gridLayout_2 = new QGridLayout();
        gridLayout_2->setObjectName("gridLayout_2");
        gridLayout_2->setVerticalSpacing(6);
        minAltitude = new QDoubleSpinBox(visualisationBox);
        minAltitude->setObjectName("minAltitude");
        minAltitude->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        minAltitude->setDecimals(0);
        minAltitude->setMinimum(1.000000000000000);
        minAltitude->setMaximum(1000000.000000000000000);
        minAltitude->setValue(200.000000000000000);

        gridLayout_2->addWidget(minAltitude, 0, 1, 1, 1);

        maxAltitude = new QDoubleSpinBox(visualisationBox);
        maxAltitude->setObjectName("maxAltitude");
        maxAltitude->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        maxAltitude->setDecimals(0);
        maxAltitude->setMinimum(1.000000000000000);
        maxAltitude->setMaximum(1000000.000000000000000);
        maxAltitude->setValue(500.000000000000000);

        gridLayout_2->addWidget(maxAltitude, 0, 2, 1, 1);

        magnitudeCheckBox = new QCheckBox(visualisationBox);
        magnitudeCheckBox->setObjectName("magnitudeCheckBox");

        gridLayout_2->addWidget(magnitudeCheckBox, 1, 0, 1, 1);

        altitudeCheckBox = new QCheckBox(visualisationBox);
        altitudeCheckBox->setObjectName("altitudeCheckBox");

        gridLayout_2->addWidget(altitudeCheckBox, 0, 0, 1, 1);

        minMagnitude = new QDoubleSpinBox(visualisationBox);
        minMagnitude->setObjectName("minMagnitude");
        minMagnitude->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        minMagnitude->setMinimum(-12.000000000000000);
        minMagnitude->setMaximum(20.000000000000000);

        gridLayout_2->addWidget(minMagnitude, 1, 2, 1, 1);

        maxMagnitude = new QDoubleSpinBox(visualisationBox);
        maxMagnitude->setObjectName("maxMagnitude");
        maxMagnitude->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        maxMagnitude->setMinimum(-12.000000000000000);
        maxMagnitude->setMaximum(20.000000000000000);

        gridLayout_2->addWidget(maxMagnitude, 1, 1, 1, 1);


        gridLayout_9->addLayout(gridLayout_2, 3, 2, 1, 1);

        horizontalSpacer_12 = new QSpacerItem(5, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        gridLayout_9->addItem(horizontalSpacer_12, 0, 1, 1, 1);

        gridLayout_3 = new QGridLayout();
        gridLayout_3->setObjectName("gridLayout_3");
        horizontalSpacer_14 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_3->addItem(horizontalSpacer_14, 0, 1, 1, 1);

        penumbraCheckBox = new QCheckBox(visualisationBox);
        penumbraCheckBox->setObjectName("penumbraCheckBox");

        gridLayout_3->addWidget(penumbraCheckBox, 0, 0, 1, 1);

        penumbraColor = new ColorButton(visualisationBox);
        penumbraColor->setObjectName("penumbraColor");

        gridLayout_3->addWidget(penumbraColor, 0, 2, 1, 1);

        umbraAtAltitude = new QCheckBox(visualisationBox);
        umbraAtAltitude->setObjectName("umbraAtAltitude");

        gridLayout_3->addWidget(umbraAtAltitude, 1, 0, 1, 1);

        umbraAltitude = new QDoubleSpinBox(visualisationBox);
        umbraAltitude->setObjectName("umbraAltitude");
        umbraAltitude->setMinimumSize(QSize(120, 0));
        umbraAltitude->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        umbraAltitude->setDecimals(1);
        umbraAltitude->setMinimum(50.000000000000000);
        umbraAltitude->setMaximum(400000.000000000000000);
        umbraAltitude->setValue(300.000000000000000);

        gridLayout_3->addWidget(umbraAltitude, 1, 1, 1, 2);


        gridLayout_9->addLayout(gridLayout_3, 3, 0, 1, 1);

        horizontalLayout_15 = new QHBoxLayout();
        horizontalLayout_15->setObjectName("horizontalLayout_15");
        iconicCheckBox = new QCheckBox(visualisationBox);
        iconicCheckBox->setObjectName("iconicCheckBox");

        horizontalLayout_15->addWidget(iconicCheckBox);

        horizontalSpacer_4 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_15->addItem(horizontalSpacer_4);

        transitColorButton = new ColorButton(visualisationBox);
        transitColorButton->setObjectName("transitColorButton");
        transitColorButton->setText(QString::fromUtf8(""));

        horizontalLayout_15->addWidget(transitColorButton);


        gridLayout_9->addLayout(horizontalLayout_15, 0, 2, 1, 1);

        hideInvisibleSatellites = new QCheckBox(visualisationBox);
        hideInvisibleSatellites->setObjectName("hideInvisibleSatellites");

        gridLayout_9->addWidget(hideInvisibleSatellites, 2, 2, 1, 1);

        horizontalLayout_13 = new QHBoxLayout();
        horizontalLayout_13->setObjectName("horizontalLayout_13");
        labelsCheckBox = new QCheckBox(visualisationBox);
        labelsCheckBox->setObjectName("labelsCheckBox");

        horizontalLayout_13->addWidget(labelsCheckBox);

        horizontalSpacer_11 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_13->addItem(horizontalSpacer_11);

        fontSizeSpinBox = new QSpinBox(visualisationBox);
        fontSizeSpinBox->setObjectName("fontSizeSpinBox");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(fontSizeSpinBox->sizePolicy().hasHeightForWidth());
        fontSizeSpinBox->setSizePolicy(sizePolicy3);
        fontSizeSpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        fontSizeSpinBox->setMinimum(-100);
        fontSizeSpinBox->setMaximum(200);
        fontSizeSpinBox->setValue(10);

        horizontalLayout_13->addWidget(fontSizeSpinBox);


        gridLayout_9->addLayout(horizontalLayout_13, 0, 0, 1, 1);

        horizontalLayout_18 = new QHBoxLayout();
        horizontalLayout_18->setObjectName("horizontalLayout_18");
        coloredInvisibleSatellites = new QCheckBox(visualisationBox);
        coloredInvisibleSatellites->setObjectName("coloredInvisibleSatellites");

        horizontalLayout_18->addWidget(coloredInvisibleSatellites);

        horizontalSpacer_5 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_18->addItem(horizontalSpacer_5);

        invisibleColorButton = new ColorButton(visualisationBox);
        invisibleColorButton->setObjectName("invisibleColorButton");

        horizontalLayout_18->addWidget(invisibleColorButton);


        gridLayout_9->addLayout(horizontalLayout_18, 1, 2, 1, 1);


        verticalLayoutSettungsTab->addWidget(visualisationBox);

        verticalSpacer_3 = new QSpacerItem(0, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayoutSettungsTab->addItem(verticalSpacer_3);

        frame = new QFrame(settingsTab);
        frame->setObjectName("frame");
        horizontalLayout_7 = new QHBoxLayout(frame);
        horizontalLayout_7->setObjectName("horizontalLayout_7");
        horizontalLayout_7->setContentsMargins(9, 0, 9, 9);
        restoreDefaultsButton = new QPushButton(frame);
        restoreDefaultsButton->setObjectName("restoreDefaultsButton");
        sizePolicy1.setHeightForWidth(restoreDefaultsButton->sizePolicy().hasHeightForWidth());
        restoreDefaultsButton->setSizePolicy(sizePolicy1);

        horizontalLayout_7->addWidget(restoreDefaultsButton);

        horizontalSpacer_7 = new QSpacerItem(20, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout_7->addItem(horizontalSpacer_7);

        saveSettingsButton = new QPushButton(frame);
        saveSettingsButton->setObjectName("saveSettingsButton");
        sizePolicy1.setHeightForWidth(saveSettingsButton->sizePolicy().hasHeightForWidth());
        saveSettingsButton->setSizePolicy(sizePolicy1);

        horizontalLayout_7->addWidget(saveSettingsButton);


        verticalLayoutSettungsTab->addWidget(frame);

        tabs->addTab(settingsTab, QString());
        satellitesTab = new QWidget();
        satellitesTab->setObjectName("satellitesTab");
        sizePolicy.setHeightForWidth(satellitesTab->sizePolicy().hasHeightForWidth());
        satellitesTab->setSizePolicy(sizePolicy);
        verticalLayout_3 = new QVBoxLayout(satellitesTab);
        verticalLayout_3->setObjectName("verticalLayout_3");
        splitter = new QSplitter(satellitesTab);
        splitter->setObjectName("splitter");
        splitter->setEnabled(true);
        sizePolicy.setHeightForWidth(splitter->sizePolicy().hasHeightForWidth());
        splitter->setSizePolicy(sizePolicy);
        splitter->setOrientation(Qt::Horizontal);
        splitter->setHandleWidth(7);
        splitter->setChildrenCollapsible(false);
        layoutWidgetSatList = new QWidget(splitter);
        layoutWidgetSatList->setObjectName("layoutWidgetSatList");
        satlist_layout = new QVBoxLayout(layoutWidgetSatList);
        satlist_layout->setObjectName("satlist_layout");
        satlist_layout->setContentsMargins(0, 0, 0, 0);
        horizontalLayout_19 = new QHBoxLayout();
        horizontalLayout_19->setObjectName("horizontalLayout_19");
        groupFilterCombo = new QComboBox(layoutWidgetSatList);
        groupFilterCombo->setObjectName("groupFilterCombo");
        sizePolicy1.setHeightForWidth(groupFilterCombo->sizePolicy().hasHeightForWidth());
        groupFilterCombo->setSizePolicy(sizePolicy1);
        groupFilterCombo->setEditable(true);
        groupFilterCombo->setInsertPolicy(QComboBox::NoInsert);

        horizontalLayout_19->addWidget(groupFilterCombo);

        customFilterButton = new QToolButton(layoutWidgetSatList);
        customFilterButton->setObjectName("customFilterButton");
        customFilterButton->setEnabled(false);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/graphicGui/uibtSettings.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        icon.addFile(QString::fromUtf8(":/graphicGui/uibtSettings-disabled.png"), QSize(), QIcon::Mode::Disabled, QIcon::State::Off);
        customFilterButton->setIcon(icon);

        horizontalLayout_19->addWidget(customFilterButton);


        satlist_layout->addLayout(horizontalLayout_19);

        lineEditSearch = new QLineEdit(layoutWidgetSatList);
        lineEditSearch->setObjectName("lineEditSearch");

        satlist_layout->addWidget(lineEditSearch);

        satellitesList = new QListView(layoutWidgetSatList);
        satellitesList->setObjectName("satellitesList");
        satellitesList->setSelectionMode(QAbstractItemView::ExtendedSelection);

        satlist_layout->addWidget(satellitesList);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        addSatellitesButton = new QToolButton(layoutWidgetSatList);
        addSatellitesButton->setObjectName("addSatellitesButton");
        QSizePolicy sizePolicy4(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy4.setHorizontalStretch(0);
        sizePolicy4.setVerticalStretch(0);
        sizePolicy4.setHeightForWidth(addSatellitesButton->sizePolicy().hasHeightForWidth());
        addSatellitesButton->setSizePolicy(sizePolicy4);
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/graphicGui/uibtAdd.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        icon1.addFile(QString::fromUtf8(":/graphicGui/uibtAdd-disabled.png"), QSize(), QIcon::Mode::Disabled, QIcon::State::Off);
        addSatellitesButton->setIcon(icon1);

        horizontalLayout_2->addWidget(addSatellitesButton);

        removeSatellitesButton = new QToolButton(layoutWidgetSatList);
        removeSatellitesButton->setObjectName("removeSatellitesButton");
        sizePolicy4.setHeightForWidth(removeSatellitesButton->sizePolicy().hasHeightForWidth());
        removeSatellitesButton->setSizePolicy(sizePolicy4);
        QIcon icon2;
        icon2.addFile(QString::fromUtf8(":/graphicGui/uibtRemove.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        icon2.addFile(QString::fromUtf8(":/graphicGui/uibtRemove-disabled.png"), QSize(), QIcon::Mode::Disabled, QIcon::State::Off);
        removeSatellitesButton->setIcon(icon2);

        horizontalLayout_2->addWidget(removeSatellitesButton);

        commSatelliteButton = new QToolButton(layoutWidgetSatList);
        commSatelliteButton->setObjectName("commSatelliteButton");
        QIcon icon3;
        icon3.addFile(QString::fromUtf8(":/satellites/bt_comms_sat.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        icon3.addFile(QString::fromUtf8(":/satellites/bt_comms_sat.png"), QSize(), QIcon::Mode::Disabled, QIcon::State::Off);
        commSatelliteButton->setIcon(icon3);

        horizontalLayout_2->addWidget(commSatelliteButton);

        selectAllButton = new QToolButton(layoutWidgetSatList);
        selectAllButton->setObjectName("selectAllButton");
        QIcon icon4;
        icon4.addFile(QString::fromUtf8(":/graphicGui/uieCheckbox-checked-disabled.png"), QSize(), QIcon::Mode::Normal, QIcon::State::On);
        icon4.addFile(QString::fromUtf8(":/graphicGui/uieCheckbox-checked-disabled.png"), QSize(), QIcon::Mode::Disabled, QIcon::State::Off);
        selectAllButton->setIcon(icon4);

        horizontalLayout_2->addWidget(selectAllButton);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_2);

        satMarkerColorPickerButton = new QToolButton(layoutWidgetSatList);
        satMarkerColorPickerButton->setObjectName("satMarkerColorPickerButton");
        sizePolicy4.setHeightForWidth(satMarkerColorPickerButton->sizePolicy().hasHeightForWidth());
        satMarkerColorPickerButton->setSizePolicy(sizePolicy4);
        satMarkerColorPickerButton->setText(QString::fromUtf8("Marker Color"));

        horizontalLayout_2->addWidget(satMarkerColorPickerButton);

        satOrbitColorPickerButton = new QToolButton(layoutWidgetSatList);
        satOrbitColorPickerButton->setObjectName("satOrbitColorPickerButton");
        sizePolicy4.setHeightForWidth(satOrbitColorPickerButton->sizePolicy().hasHeightForWidth());
        satOrbitColorPickerButton->setSizePolicy(sizePolicy4);
        satOrbitColorPickerButton->setText(QString::fromUtf8("Orbit Color"));

        horizontalLayout_2->addWidget(satOrbitColorPickerButton);

        satInfoColorPickerButton = new QToolButton(layoutWidgetSatList);
        satInfoColorPickerButton->setObjectName("satInfoColorPickerButton");
        sizePolicy4.setHeightForWidth(satInfoColorPickerButton->sizePolicy().hasHeightForWidth());
        satInfoColorPickerButton->setSizePolicy(sizePolicy4);
        satInfoColorPickerButton->setText(QString::fromUtf8("Info Color"));

        horizontalLayout_2->addWidget(satInfoColorPickerButton);


        satlist_layout->addLayout(horizontalLayout_2);

        splitter->addWidget(layoutWidgetSatList);
        layoutWidget = new QWidget(splitter);
        layoutWidget->setObjectName("layoutWidget");
        sat_details = new QVBoxLayout(layoutWidget);
        sat_details->setObjectName("sat_details");
        sat_details->setContentsMargins(0, 0, 0, 0);
        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        label = new QLabel(layoutWidget);
        label->setObjectName("label");

        horizontalLayout_4->addWidget(label);

        nameEdit = new QLineEdit(layoutWidget);
        nameEdit->setObjectName("nameEdit");
        nameEdit->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        nameEdit->setReadOnly(true);

        horizontalLayout_4->addWidget(nameEdit);


        sat_details->addLayout(horizontalLayout_4);

        horizontalLayoutCatalogNumber = new QHBoxLayout();
        horizontalLayoutCatalogNumber->setObjectName("horizontalLayoutCatalogNumber");
        labelCatalogNumber = new QLabel(layoutWidget);
        labelCatalogNumber->setObjectName("labelCatalogNumber");

        horizontalLayoutCatalogNumber->addWidget(labelCatalogNumber);

        noradNumberEdit = new QLineEdit(layoutWidget);
        noradNumberEdit->setObjectName("noradNumberEdit");
        noradNumberEdit->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        noradNumberEdit->setReadOnly(true);

        horizontalLayoutCatalogNumber->addWidget(noradNumberEdit);

        labelCosparNumber = new QLabel(layoutWidget);
        labelCosparNumber->setObjectName("labelCosparNumber");

        horizontalLayoutCatalogNumber->addWidget(labelCosparNumber);

        cosparNumberEdit = new QLineEdit(layoutWidget);
        cosparNumberEdit->setObjectName("cosparNumberEdit");
        cosparNumberEdit->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        cosparNumberEdit->setReadOnly(true);

        horizontalLayoutCatalogNumber->addWidget(cosparNumberEdit);


        sat_details->addLayout(horizontalLayoutCatalogNumber);

        horizontalLayout_8 = new QHBoxLayout();
        horizontalLayout_8->setObjectName("horizontalLayout_8");
        labelStdMagnitude = new QLabel(layoutWidget);
        labelStdMagnitude->setObjectName("labelStdMagnitude");

        horizontalLayout_8->addWidget(labelStdMagnitude);

        stdMagnitudeLineEdit = new QLineEdit(layoutWidget);
        stdMagnitudeLineEdit->setObjectName("stdMagnitudeLineEdit");
        stdMagnitudeLineEdit->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        stdMagnitudeLineEdit->setReadOnly(true);

        horizontalLayout_8->addWidget(stdMagnitudeLineEdit);

        labelRCS = new QLabel(layoutWidget);
        labelRCS->setObjectName("labelRCS");
        labelRCS->setText(QString::fromUtf8("RCS:"));

        horizontalLayout_8->addWidget(labelRCS);

        rcsLineEdit = new QLineEdit(layoutWidget);
        rcsLineEdit->setObjectName("rcsLineEdit");
        rcsLineEdit->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        rcsLineEdit->setReadOnly(true);

        horizontalLayout_8->addWidget(rcsLineEdit);


        sat_details->addLayout(horizontalLayout_8);

        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        labelPerigee = new QLabel(layoutWidget);
        labelPerigee->setObjectName("labelPerigee");

        horizontalLayout_5->addWidget(labelPerigee);

        perigeeLineEdit = new QLineEdit(layoutWidget);
        perigeeLineEdit->setObjectName("perigeeLineEdit");
        perigeeLineEdit->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        perigeeLineEdit->setReadOnly(true);

        horizontalLayout_5->addWidget(perigeeLineEdit);

        labelApogee = new QLabel(layoutWidget);
        labelApogee->setObjectName("labelApogee");

        horizontalLayout_5->addWidget(labelApogee);

        apogeeLineEdit = new QLineEdit(layoutWidget);
        apogeeLineEdit->setObjectName("apogeeLineEdit");
        apogeeLineEdit->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        apogeeLineEdit->setReadOnly(true);

        horizontalLayout_5->addWidget(apogeeLineEdit);

        labelPeriod = new QLabel(layoutWidget);
        labelPeriod->setObjectName("labelPeriod");

        horizontalLayout_5->addWidget(labelPeriod);

        periodLineEdit = new QLineEdit(layoutWidget);
        periodLineEdit->setObjectName("periodLineEdit");
        periodLineEdit->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        periodLineEdit->setReadOnly(true);

        horizontalLayout_5->addWidget(periodLineEdit);


        sat_details->addLayout(horizontalLayout_5);

        gridLayout_5 = new QGridLayout();
        gridLayout_5->setObjectName("gridLayout_5");
        gridLayout_5->setHorizontalSpacing(0);
        orbitCheckbox = new QCheckBox(layoutWidget);
        orbitCheckbox->setObjectName("orbitCheckbox");
        orbitCheckbox->setTristate(false);

        gridLayout_5->addWidget(orbitCheckbox, 0, 1, 1, 1);

        displayedCheckbox = new QCheckBox(layoutWidget);
        displayedCheckbox->setObjectName("displayedCheckbox");
        displayedCheckbox->setTristate(false);

        gridLayout_5->addWidget(displayedCheckbox, 0, 0, 1, 1);

        userCheckBox = new QCheckBox(layoutWidget);
        userCheckBox->setObjectName("userCheckBox");

        gridLayout_5->addWidget(userCheckBox, 0, 2, 1, 1);


        sat_details->addLayout(gridLayout_5);

        verticalLayout_6 = new QVBoxLayout();
        verticalLayout_6->setSpacing(0);
        verticalLayout_6->setObjectName("verticalLayout_6");
        label_4 = new QLabel(layoutWidget);
        label_4->setObjectName("label_4");

        verticalLayout_6->addWidget(label_4);

        descriptionTextEdit = new QTextEdit(layoutWidget);
        descriptionTextEdit->setObjectName("descriptionTextEdit");
        QSizePolicy sizePolicy5(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);
        sizePolicy5.setHorizontalStretch(0);
        sizePolicy5.setVerticalStretch(0);
        sizePolicy5.setHeightForWidth(descriptionTextEdit->sizePolicy().hasHeightForWidth());
        descriptionTextEdit->setSizePolicy(sizePolicy5);
        descriptionTextEdit->setMaximumSize(QSize(16777215, 60));
        descriptionTextEdit->setReadOnly(false);
        descriptionTextEdit->setAcceptRichText(false);

        verticalLayout_6->addWidget(descriptionTextEdit);


        sat_details->addLayout(verticalLayout_6);

        verticalLayout_4 = new QVBoxLayout();
        verticalLayout_4->setSpacing(0);
        verticalLayout_4->setObjectName("verticalLayout_4");
        label_3 = new QLabel(layoutWidget);
        label_3->setObjectName("label_3");

        verticalLayout_4->addWidget(label_3);

        groupsListWidget = new QListWidget(layoutWidget);
        groupsListWidget->setObjectName("groupsListWidget");
        groupsListWidget->setEditTriggers(QAbstractItemView::CurrentChanged|QAbstractItemView::SelectedClicked);
        groupsListWidget->setProperty("isWrapping", QVariant(true));

        verticalLayout_4->addWidget(groupsListWidget);


        sat_details->addLayout(verticalLayout_4);

        verticalLayout_5 = new QVBoxLayout();
        verticalLayout_5->setSpacing(0);
        verticalLayout_5->setObjectName("verticalLayout_5");
        labelTle = new QLabel(layoutWidget);
        labelTle->setObjectName("labelTle");

        verticalLayout_5->addWidget(labelTle);

        tleFirstLineEdit = new QLineEdit(layoutWidget);
        tleFirstLineEdit->setObjectName("tleFirstLineEdit");
        tleFirstLineEdit->setFrame(false);
        tleFirstLineEdit->setReadOnly(true);

        verticalLayout_5->addWidget(tleFirstLineEdit);

        tleSecondLineEdit = new QLineEdit(layoutWidget);
        tleSecondLineEdit->setObjectName("tleSecondLineEdit");
        tleSecondLineEdit->setFrame(false);
        tleSecondLineEdit->setReadOnly(true);

        verticalLayout_5->addWidget(tleSecondLineEdit);


        sat_details->addLayout(verticalLayout_5);

        horizontalLayout_12 = new QHBoxLayout();
        horizontalLayout_12->setObjectName("horizontalLayout_12");
        labelTleEpoch = new QLabel(layoutWidget);
        labelTleEpoch->setObjectName("labelTleEpoch");

        horizontalLayout_12->addWidget(labelTleEpoch);

        labelTleEpochData = new QLabel(layoutWidget);
        labelTleEpochData->setObjectName("labelTleEpochData");
        QSizePolicy sizePolicy6(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy6.setHorizontalStretch(0);
        sizePolicy6.setVerticalStretch(0);
        sizePolicy6.setHeightForWidth(labelTleEpochData->sizePolicy().hasHeightForWidth());
        labelTleEpochData->setSizePolicy(sizePolicy6);
        labelTleEpochData->setText(QString::fromUtf8(""));

        horizontalLayout_12->addWidget(labelTleEpochData);

        horizontalSpacer_8 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_12->addItem(horizontalSpacer_8);

        validAgeLabel = new QLabel(layoutWidget);
        validAgeLabel->setObjectName("validAgeLabel");

        horizontalLayout_12->addWidget(validAgeLabel);

        validAgeSpinBox = new QSpinBox(layoutWidget);
        validAgeSpinBox->setObjectName("validAgeSpinBox");
        validAgeSpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        validAgeSpinBox->setMinimum(1);
        validAgeSpinBox->setMaximum(720);
        validAgeSpinBox->setValue(30);

        horizontalLayout_12->addWidget(validAgeSpinBox);


        sat_details->addLayout(horizontalLayout_12);

        splitter->addWidget(layoutWidget);

        verticalLayout_3->addWidget(splitter);

        tabs->addTab(satellitesTab, QString());
        sourcesTab = new QWidget();
        sourcesTab->setObjectName("sourcesTab");
        verticalLayout_10 = new QVBoxLayout(sourcesTab);
        verticalLayout_10->setObjectName("verticalLayout_10");
        sourceList = new QListWidget(sourcesTab);
        sourceList->setObjectName("sourceList");
        sourceList->setEditTriggers(QAbstractItemView::DoubleClicked);
        sourceList->setSortingEnabled(false);

        verticalLayout_10->addWidget(sourceList);

        labelAutoAdd = new QLabel(sourcesTab);
        labelAutoAdd->setObjectName("labelAutoAdd");
        labelAutoAdd->setWordWrap(true);

        verticalLayout_10->addWidget(labelAutoAdd);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        sourceEdit = new QLineEdit(sourcesTab);
        sourceEdit->setObjectName("sourceEdit");
        sourceEdit->setInputMethodHints(Qt::ImhUrlCharactersOnly);

        horizontalLayout->addWidget(sourceEdit);

        addSourceButton = new QToolButton(sourcesTab);
        addSourceButton->setObjectName("addSourceButton");
        QIcon icon5;
        icon5.addFile(QString::fromUtf8(":/graphicGui/uibtCreate.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        icon5.addFile(QString::fromUtf8(":/graphicGui/uibtCreate-disabled.png"), QSize(), QIcon::Mode::Disabled, QIcon::State::Off);
        addSourceButton->setIcon(icon5);

        horizontalLayout->addWidget(addSourceButton);

        editSourceButton = new QToolButton(sourcesTab);
        editSourceButton->setObjectName("editSourceButton");
        QIcon icon6;
        icon6.addFile(QString::fromUtf8(":/graphicGui/uibtEdit.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        icon6.addFile(QString::fromUtf8(":/graphicGui/uibtEdit-disabled.png"), QSize(), QIcon::Mode::Disabled, QIcon::State::Off);
        editSourceButton->setIcon(icon6);

        horizontalLayout->addWidget(editSourceButton);

        saveSourceButton = new QToolButton(sourcesTab);
        saveSourceButton->setObjectName("saveSourceButton");
        QIcon icon7;
        icon7.addFile(QString::fromUtf8(":/graphicGui/uibtSave.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        icon7.addFile(QString::fromUtf8(":/graphicGui/uibtSave-disabled.png"), QSize(), QIcon::Mode::Disabled, QIcon::State::Off);
        saveSourceButton->setIcon(icon7);

        horizontalLayout->addWidget(saveSourceButton);

        deleteSourceButton = new QToolButton(sourcesTab);
        deleteSourceButton->setObjectName("deleteSourceButton");
        QIcon icon8;
        icon8.addFile(QString::fromUtf8(":/graphicGui/uibtDelete.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        icon8.addFile(QString::fromUtf8(":/graphicGui/uibtDelete-disabled.png"), QSize(), QIcon::Mode::Disabled, QIcon::State::Off);
        deleteSourceButton->setIcon(icon8);

        horizontalLayout->addWidget(deleteSourceButton);

        horizontalSpacer_10 = new QSpacerItem(20, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_10);

        resetSourcesButton = new QToolButton(sourcesTab);
        resetSourcesButton->setObjectName("resetSourcesButton");
        QIcon icon9;
        icon9.addFile(QString::fromUtf8(":/graphicGui/uibtDownload.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        icon9.addFile(QString::fromUtf8(":/graphicGui/uibtDownload-disabled.png"), QSize(), QIcon::Mode::Disabled, QIcon::State::Off);
        resetSourcesButton->setIcon(icon9);

        horizontalLayout->addWidget(resetSourcesButton);


        verticalLayout_10->addLayout(horizontalLayout);

        tabs->addTab(sourcesTab, QString());
        iridiumTab = new QWidget();
        iridiumTab->setObjectName("iridiumTab");
        gridLayout = new QGridLayout(iridiumTab);
        gridLayout->setObjectName("gridLayout");
        iridiumFlaresTreeWidget = new QTreeWidget(iridiumTab);
        iridiumFlaresTreeWidget->setObjectName("iridiumFlaresTreeWidget");
        iridiumFlaresTreeWidget->setEditTriggers(QAbstractItemView::NoEditTriggers);
        iridiumFlaresTreeWidget->setSortingEnabled(true);
        iridiumFlaresTreeWidget->setExpandsOnDoubleClick(false);
        iridiumFlaresTreeWidget->setColumnCount(0);

        gridLayout->addWidget(iridiumFlaresTreeWidget, 0, 0, 1, 1);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        horizontalLayout_6 = new QHBoxLayout();
        horizontalLayout_6->setObjectName("horizontalLayout_6");
        labelFlaresPredictionDepth = new QLabel(iridiumTab);
        labelFlaresPredictionDepth->setObjectName("labelFlaresPredictionDepth");

        horizontalLayout_6->addWidget(labelFlaresPredictionDepth);

        flaresPredictionDepthSpinBox = new QSpinBox(iridiumTab);
        flaresPredictionDepthSpinBox->setObjectName("flaresPredictionDepthSpinBox");
        flaresPredictionDepthSpinBox->setMinimum(1);
        flaresPredictionDepthSpinBox->setMaximum(14);
        flaresPredictionDepthSpinBox->setValue(1);

        horizontalLayout_6->addWidget(flaresPredictionDepthSpinBox);


        horizontalLayout_3->addLayout(horizontalLayout_6);

        predictIridiumFlaresPushButton = new QPushButton(iridiumTab);
        predictIridiumFlaresPushButton->setObjectName("predictIridiumFlaresPushButton");

        horizontalLayout_3->addWidget(predictIridiumFlaresPushButton);

        predictedIridiumFlaresSaveButton = new QPushButton(iridiumTab);
        predictedIridiumFlaresSaveButton->setObjectName("predictedIridiumFlaresSaveButton");

        horizontalLayout_3->addWidget(predictedIridiumFlaresSaveButton);


        gridLayout->addLayout(horizontalLayout_3, 1, 0, 1, 1);

        tabs->addTab(iridiumTab, QString());
        aboutTab = new QWidget();
        aboutTab->setObjectName("aboutTab");
        verticalLayout_11 = new QVBoxLayout(aboutTab);
        verticalLayout_11->setSpacing(9);
        verticalLayout_11->setObjectName("verticalLayout_11");
        verticalLayout_11->setContentsMargins(9, 9, 9, 9);
        aboutTextBrowser = new QTextBrowser(aboutTab);
        aboutTextBrowser->setObjectName("aboutTextBrowser");
        aboutTextBrowser->setOpenExternalLinks(true);
        aboutTextBrowser->setOpenLinks(true);

        verticalLayout_11->addWidget(aboutTextBrowser);

        tabs->addTab(aboutTab, QString());

        verticalLayout_2->addWidget(tabs);

#if QT_CONFIG(shortcut)
        updateFrequencyLabel->setBuddy(updateFrequencySpinBox);
        nextUpdateLabel->setBuddy(updateButton);
        label->setBuddy(nameEdit);
        labelCatalogNumber->setBuddy(noradNumberEdit);
        labelCosparNumber->setBuddy(cosparNumberEdit);
        labelStdMagnitude->setBuddy(stdMagnitudeLineEdit);
        labelRCS->setBuddy(rcsLineEdit);
        label_4->setBuddy(descriptionTextEdit);
        label_3->setBuddy(groupsListWidget);
        labelTle->setBuddy(tleFirstLineEdit);
        labelFlaresPredictionDepth->setBuddy(flaresPredictionDepthSpinBox);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(deleteSourceButton, internetUpdatesCheckbox);
        QWidget::setTabOrder(internetUpdatesCheckbox, checkBoxAutoAdd);
        QWidget::setTabOrder(checkBoxAutoAdd, jumpToSourcesButton);
        QWidget::setTabOrder(jumpToSourcesButton, checkBoxAutoRemove);
        QWidget::setTabOrder(checkBoxAutoRemove, updateFrequencySpinBox);
        QWidget::setTabOrder(updateFrequencySpinBox, updateButton);
        QWidget::setTabOrder(updateButton, restoreDefaultsButton);
        QWidget::setTabOrder(restoreDefaultsButton, saveSettingsButton);
        QWidget::setTabOrder(saveSettingsButton, tabs);
        QWidget::setTabOrder(tabs, lineEditSearch);
        QWidget::setTabOrder(lineEditSearch, satellitesList);
        QWidget::setTabOrder(satellitesList, satMarkerColorPickerButton);
        QWidget::setTabOrder(satMarkerColorPickerButton, satOrbitColorPickerButton);
        QWidget::setTabOrder(satOrbitColorPickerButton, satInfoColorPickerButton);
        QWidget::setTabOrder(satInfoColorPickerButton, nameEdit);
        QWidget::setTabOrder(nameEdit, noradNumberEdit);
        QWidget::setTabOrder(noradNumberEdit, cosparNumberEdit);
        QWidget::setTabOrder(cosparNumberEdit, stdMagnitudeLineEdit);
        QWidget::setTabOrder(stdMagnitudeLineEdit, rcsLineEdit);
        QWidget::setTabOrder(rcsLineEdit, displayedCheckbox);
        QWidget::setTabOrder(displayedCheckbox, orbitCheckbox);
        QWidget::setTabOrder(orbitCheckbox, userCheckBox);
        QWidget::setTabOrder(userCheckBox, descriptionTextEdit);
        QWidget::setTabOrder(descriptionTextEdit, groupsListWidget);
        QWidget::setTabOrder(groupsListWidget, tleFirstLineEdit);
        QWidget::setTabOrder(tleFirstLineEdit, tleSecondLineEdit);
        QWidget::setTabOrder(tleSecondLineEdit, removeSatellitesButton);
        QWidget::setTabOrder(removeSatellitesButton, addSatellitesButton);
        QWidget::setTabOrder(addSatellitesButton, saveSourceButton);
        QWidget::setTabOrder(saveSourceButton, iridiumFlaresTreeWidget);
        QWidget::setTabOrder(iridiumFlaresTreeWidget, flaresPredictionDepthSpinBox);
        QWidget::setTabOrder(flaresPredictionDepthSpinBox, predictIridiumFlaresPushButton);
        QWidget::setTabOrder(predictIridiumFlaresPushButton, predictedIridiumFlaresSaveButton);
        QWidget::setTabOrder(predictedIridiumFlaresSaveButton, editSourceButton);
        QWidget::setTabOrder(editSourceButton, sourceList);
        QWidget::setTabOrder(sourceList, sourceEdit);
        QWidget::setTabOrder(sourceEdit, aboutTextBrowser);
        QWidget::setTabOrder(aboutTextBrowser, addSourceButton);

        retranslateUi(satellitesDialog);
        QObject::connect(checkBoxAutoAdd, &QCheckBox::toggled, jumpToSourcesButton, &QPushButton::setEnabled);
        QObject::connect(checkBoxAutoAdd, &QCheckBox::toggled, labelAutoAdd, &QLabel::setVisible);

        tabs->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(satellitesDialog);
    } // setupUi

    void retranslateUi(QWidget *satellitesDialog)
    {
        satellitesDialog->setWindowTitle(QCoreApplication::translate("satellitesDialog", "Satellites Configuration", nullptr));
        titleBar->setProperty("title", QVariant(QCoreApplication::translate("satellitesDialog", "Satellites Configuration", nullptr)));
        updatesGroup->setTitle(QCoreApplication::translate("satellitesDialog", "Updates", nullptr));
        checkBoxAutoAdd->setText(QCoreApplication::translate("satellitesDialog", "On update, add all new satellites from the selected source(s)", nullptr));
        internetUpdatesCheckbox->setText(QCoreApplication::translate("satellitesDialog", "Update satellite data from Internet sources", nullptr));
        updateFrequencyLabel->setText(QCoreApplication::translate("satellitesDialog", "Update frequency:", nullptr));
        label_7->setText(QCoreApplication::translate("satellitesDialog", "Last update:", nullptr));
        updateButton->setText(QCoreApplication::translate("satellitesDialog", "Update now", nullptr));
        checkBoxAutoRemove->setText(QCoreApplication::translate("satellitesDialog", "On update, remove the satellites that are no longer listed in the update sources", nullptr));
        checkBoxAutoDisplay->setText(QCoreApplication::translate("satellitesDialog", "On update, mark all newly added satellites as displayed", nullptr));
        visualisationBox->setTitle(QCoreApplication::translate("satellitesDialog", "Visualisation settings", nullptr));
        orbitLinesCheckBox->setText(QCoreApplication::translate("satellitesDialog", "Orbit lines", nullptr));
#if QT_CONFIG(tooltip)
        orbitSegmentsSpin->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        orbitDurationSpin->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        orbitFadeSpin->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
        umbraCheckBox->setText(QCoreApplication::translate("satellitesDialog", "Show Earth's umbra", nullptr));
#if QT_CONFIG(tooltip)
        umbraColor->setToolTip(QCoreApplication::translate("satellitesDialog", "The color of Earth's umbra circle", nullptr));
#endif // QT_CONFIG(tooltip)
        magnitudeCheckBox->setText(QCoreApplication::translate("satellitesDialog", "Magnitude range", nullptr));
        altitudeCheckBox->setText(QCoreApplication::translate("satellitesDialog", "Altitude range", nullptr));
        penumbraCheckBox->setText(QCoreApplication::translate("satellitesDialog", "Show Earth's penumbra", nullptr));
#if QT_CONFIG(tooltip)
        penumbraColor->setToolTip(QCoreApplication::translate("satellitesDialog", "The color of Earth's penumbra circle", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        umbraAtAltitude->setToolTip(QCoreApplication::translate("satellitesDialog", "Distance to the center of umbra from Earth's surface (height of imagined satellite above a spherical Earth)", nullptr));
#endif // QT_CONFIG(tooltip)
        umbraAtAltitude->setText(QCoreApplication::translate("satellitesDialog", "Show umbra/penumbra at altitude:", nullptr));
#if QT_CONFIG(tooltip)
        umbraAltitude->setToolTip(QCoreApplication::translate("satellitesDialog", "Distance to the center of umbra from Earth's surface (height of imagined satellite above a spherical Earth)", nullptr));
#endif // QT_CONFIG(tooltip)
        iconicCheckBox->setText(QCoreApplication::translate("satellitesDialog", "Show satellites as markers", nullptr));
#if QT_CONFIG(tooltip)
        transitColorButton->setToolTip(QCoreApplication::translate("satellitesDialog", "Color of satellite markers in transit through the Sun or the Moon", nullptr));
#endif // QT_CONFIG(tooltip)
        hideInvisibleSatellites->setText(QCoreApplication::translate("satellitesDialog", "Hide invisible satellites", nullptr));
        labelsCheckBox->setText(QCoreApplication::translate("satellitesDialog", "Labels", nullptr));
#if QT_CONFIG(tooltip)
        fontSizeSpinBox->setToolTip(QCoreApplication::translate("satellitesDialog", "Font size for labels", nullptr));
#endif // QT_CONFIG(tooltip)
        fontSizeSpinBox->setSuffix(QCoreApplication::translate("satellitesDialog", "px", "pixels"));
#if QT_CONFIG(tooltip)
        coloredInvisibleSatellites->setToolTip(QCoreApplication::translate("satellitesDialog", "Use special color for all invisible satellites", nullptr));
#endif // QT_CONFIG(tooltip)
        coloredInvisibleSatellites->setText(QCoreApplication::translate("satellitesDialog", "Set color for invisible satellites", nullptr));
#if QT_CONFIG(tooltip)
        invisibleColorButton->setToolTip(QCoreApplication::translate("satellitesDialog", "Color of markers, labels and orbits for invisible satellites", nullptr));
#endif // QT_CONFIG(tooltip)
        invisibleColorButton->setText(QString());
        restoreDefaultsButton->setText(QCoreApplication::translate("satellitesDialog", "Restore defaults", nullptr));
        saveSettingsButton->setText(QCoreApplication::translate("satellitesDialog", "Save settings", nullptr));
        tabs->setTabText(tabs->indexOf(settingsTab), QCoreApplication::translate("satellitesDialog", "Settings", nullptr));
#if QT_CONFIG(tooltip)
        customFilterButton->setToolTip(QCoreApplication::translate("satellitesDialog", "Settings of custom filter", nullptr));
#endif // QT_CONFIG(tooltip)
        lineEditSearch->setPlaceholderText(QCoreApplication::translate("satellitesDialog", "Search ...", nullptr));
#if QT_CONFIG(tooltip)
        satellitesList->setToolTip(QCoreApplication::translate("satellitesDialog", "Double-click a satellite to start tracking it.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        addSatellitesButton->setToolTip(QCoreApplication::translate("satellitesDialog", "Add more satellites", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        removeSatellitesButton->setToolTip(QCoreApplication::translate("satellitesDialog", "Remove the selected satellites", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        commSatelliteButton->setToolTip(QCoreApplication::translate("satellitesDialog", "Edit communication data for the selected satellite", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        selectAllButton->setToolTip(QCoreApplication::translate("satellitesDialog", "Select all satellites in list", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        satMarkerColorPickerButton->setToolTip(QCoreApplication::translate("satellitesDialog", "Change markers color for selected satellites", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        satOrbitColorPickerButton->setToolTip(QCoreApplication::translate("satellitesDialog", "Change orbits color for selected satellites", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        satInfoColorPickerButton->setToolTip(QCoreApplication::translate("satellitesDialog", "Change info color for selected satellites", nullptr));
#endif // QT_CONFIG(tooltip)
        label->setText(QCoreApplication::translate("satellitesDialog", "Name:", nullptr));
#if QT_CONFIG(tooltip)
        labelCatalogNumber->setToolTip(QCoreApplication::translate("satellitesDialog", "The Satellite Catalog Number (also known as NORAD (North American Aerospace Defense) Catalog Number, NORAD ID)", nullptr));
#endif // QT_CONFIG(tooltip)
        labelCatalogNumber->setText(QCoreApplication::translate("satellitesDialog", "Catalog number:", nullptr));
#if QT_CONFIG(tooltip)
        noradNumberEdit->setToolTip(QCoreApplication::translate("satellitesDialog", "The Satellite Catalog Number (also known as NORAD (North American Aerospace Defense) Catalog Number, NORAD ID)", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        labelCosparNumber->setToolTip(QCoreApplication::translate("satellitesDialog", "The International Designator, also known as COSPAR ID", nullptr));
#endif // QT_CONFIG(tooltip)
        labelCosparNumber->setText(QCoreApplication::translate("satellitesDialog", "I.D.:", nullptr));
#if QT_CONFIG(tooltip)
        cosparNumberEdit->setToolTip(QCoreApplication::translate("satellitesDialog", "The International Designator, also known as COSPAR ID", nullptr));
#endif // QT_CONFIG(tooltip)
        labelStdMagnitude->setText(QCoreApplication::translate("satellitesDialog", "Standard magnitude:", nullptr));
#if QT_CONFIG(tooltip)
        labelPerigee->setToolTip(QCoreApplication::translate("satellitesDialog", "Perigee altitude in kilometers", nullptr));
#endif // QT_CONFIG(tooltip)
        labelPerigee->setText(QCoreApplication::translate("satellitesDialog", "Perigee:", nullptr));
#if QT_CONFIG(tooltip)
        perigeeLineEdit->setToolTip(QCoreApplication::translate("satellitesDialog", "Perigee altitude in kilometers", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        labelApogee->setToolTip(QCoreApplication::translate("satellitesDialog", "Apogee altitude in kilometers", nullptr));
#endif // QT_CONFIG(tooltip)
        labelApogee->setText(QCoreApplication::translate("satellitesDialog", "Apogee:", nullptr));
#if QT_CONFIG(tooltip)
        apogeeLineEdit->setToolTip(QCoreApplication::translate("satellitesDialog", "Apogee altitude in kilometers", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        labelPeriod->setToolTip(QCoreApplication::translate("satellitesDialog", "Orbital period in minutes", nullptr));
#endif // QT_CONFIG(tooltip)
        labelPeriod->setText(QCoreApplication::translate("satellitesDialog", "Period:", nullptr));
#if QT_CONFIG(tooltip)
        periodLineEdit->setToolTip(QCoreApplication::translate("satellitesDialog", "Orbital period in minutes", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        orbitCheckbox->setToolTip(QCoreApplication::translate("satellitesDialog", "Display orbit line(s) for the selected satellite(s)", nullptr));
#endif // QT_CONFIG(tooltip)
        orbitCheckbox->setText(QCoreApplication::translate("satellitesDialog", "Orbit", nullptr));
#if QT_CONFIG(tooltip)
        displayedCheckbox->setToolTip(QCoreApplication::translate("satellitesDialog", "Display the selected satellite(s)", nullptr));
#endif // QT_CONFIG(tooltip)
        displayedCheckbox->setText(QCoreApplication::translate("satellitesDialog", "Displayed", nullptr));
#if QT_CONFIG(tooltip)
        userCheckBox->setToolTip(QCoreApplication::translate("satellitesDialog", "Do not update TLE of selected satellite(s)", nullptr));
#endif // QT_CONFIG(tooltip)
        userCheckBox->setText(QCoreApplication::translate("satellitesDialog", "Do not update", nullptr));
        label_4->setText(QCoreApplication::translate("satellitesDialog", "Description:", nullptr));
        descriptionTextEdit->setPlaceholderText(QCoreApplication::translate("satellitesDialog", "(add personal notes here)", nullptr));
        label_3->setText(QCoreApplication::translate("satellitesDialog", "Groups:", nullptr));
        labelTle->setText(QCoreApplication::translate("satellitesDialog", "TLE set:", nullptr));
        labelTleEpoch->setText(QCoreApplication::translate("satellitesDialog", "Epoch of the TLE:", nullptr));
#if QT_CONFIG(tooltip)
        validAgeLabel->setToolTip(QCoreApplication::translate("satellitesDialog", "The valid time range for epoch of TLE in days, used in filtering of satellites", nullptr));
#endif // QT_CONFIG(tooltip)
        validAgeLabel->setText(QCoreApplication::translate("satellitesDialog", "Age:", nullptr));
#if QT_CONFIG(tooltip)
        validAgeSpinBox->setToolTip(QCoreApplication::translate("satellitesDialog", "The valid time range for epoch of TLE in days, used in filtering of satellites", nullptr));
#endif // QT_CONFIG(tooltip)
        tabs->setTabText(tabs->indexOf(satellitesTab), QCoreApplication::translate("satellitesDialog", "Satellites", nullptr));
        labelAutoAdd->setText(QCoreApplication::translate("satellitesDialog", "Satellites in the marked source lists will be automatically added on the next update if they are not already in the collection. Unmarked entries will also be fetched to update only previously included satellites.", nullptr));
#if QT_CONFIG(tooltip)
        sourceEdit->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        addSourceButton->setToolTip(QCoreApplication::translate("satellitesDialog", "Add new source", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        editSourceButton->setToolTip(QCoreApplication::translate("satellitesDialog", "Edit selected source", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        saveSourceButton->setToolTip(QCoreApplication::translate("satellitesDialog", "Save source", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        deleteSourceButton->setToolTip(QCoreApplication::translate("satellitesDialog", "Remove selected source", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        resetSourcesButton->setToolTip(QCoreApplication::translate("satellitesDialog", "Reset sources list to it defaults", nullptr));
#endif // QT_CONFIG(tooltip)
        tabs->setTabText(tabs->indexOf(sourcesTab), QCoreApplication::translate("satellitesDialog", "Sources", nullptr));
        labelFlaresPredictionDepth->setText(QCoreApplication::translate("satellitesDialog", "Flare prediction (days):", nullptr));
#if QT_CONFIG(tooltip)
        predictIridiumFlaresPushButton->setToolTip(QCoreApplication::translate("satellitesDialog", "Calculations require time, please be patient", nullptr));
#endif // QT_CONFIG(tooltip)
        predictIridiumFlaresPushButton->setText(QCoreApplication::translate("satellitesDialog", "Predict Iridium flares", nullptr));
        predictedIridiumFlaresSaveButton->setText(QCoreApplication::translate("satellitesDialog", "Save predictions...", nullptr));
        tabs->setTabText(tabs->indexOf(iridiumTab), QCoreApplication::translate("satellitesDialog", "Iridium flares", nullptr));
        tabs->setTabText(tabs->indexOf(aboutTab), QCoreApplication::translate("satellitesDialog", "About", "tab in plugin windows"));
    } // retranslateUi

};

namespace Ui {
    class satellitesDialog: public Ui_satellitesDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SATELLITESDIALOG_H
