/****************************************************************************
** Meta object code from reading C++ file 'SatellitesDialog.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../stellarium/plugins/Satellites/src/gui/SatellitesDialog.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'SatellitesDialog.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16SatellitesDialogE_t {};
} // unnamed namespace

template <> constexpr inline auto SatellitesDialog::qt_create_metaobjectdata<qt_meta_tag_ZN16SatellitesDialogE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "SatellitesDialog",
        "retranslate",
        "",
        "jumpToSourcesTab",
        "updateCountdown",
        "filterListByGroup",
        "index",
        "updateSatelliteData",
        "updateSatelliteAndSaveData",
        "updateFilteredSatellitesList",
        "selectFilteredSatellitesList",
        "saveSatellites",
        "showUpdateState",
        "Satellites::UpdateState",
        "state",
        "showUpdateCompleted",
        "updated",
        "total",
        "added",
        "missing",
        "handleOrbitLinesGroup",
        "handleUmbraGroup",
        "saveEditedSource",
        "updateButtonsProperties",
        "saveSourceList",
        "deleteSourceRow",
        "addSourceRow",
        "editSourceRow",
        "toggleCheckableSources",
        "restoreDefaults",
        "restoreTleSources",
        "saveSettings",
        "addSatellites",
        "TleDataList",
        "newSatellites",
        "removeSatellites",
        "setFlags",
        "handleGroupChanges",
        "QListWidgetItem*",
        "item",
        "trackSatellite",
        "QModelIndex",
        "updateTLEs",
        "searchSatellitesClear",
        "askSatMarkerColor",
        "askSatOrbitColor",
        "askSatInfoColor",
        "descriptionTextChanged",
        "setRightSideToROMode",
        "setRightSideToRWMode",
        "enableMinMaxAltitude",
        "enableMinMaxMagnitude"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'retranslate'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'jumpToSourcesTab'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateCountdown'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'filterListByGroup'
        QtMocHelpers::SlotData<void(int)>(5, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 6 },
        }}),
        // Slot 'updateSatelliteData'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateSatelliteAndSaveData'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateFilteredSatellitesList'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'selectFilteredSatellitesList'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'saveSatellites'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'showUpdateState'
        QtMocHelpers::SlotData<void(Satellites::UpdateState)>(12, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
        // Slot 'showUpdateCompleted'
        QtMocHelpers::SlotData<void(int, int, int, int)>(15, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 16 }, { QMetaType::Int, 17 }, { QMetaType::Int, 18 }, { QMetaType::Int, 19 },
        }}),
        // Slot 'handleOrbitLinesGroup'
        QtMocHelpers::SlotData<void(bool)>(20, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 14 },
        }}),
        // Slot 'handleUmbraGroup'
        QtMocHelpers::SlotData<void(bool)>(21, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 14 },
        }}),
        // Slot 'saveEditedSource'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateButtonsProperties'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'saveSourceList'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'deleteSourceRow'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'addSourceRow'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'editSourceRow'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'toggleCheckableSources'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'restoreDefaults'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'restoreTleSources'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'saveSettings'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'addSatellites'
        QtMocHelpers::SlotData<void(const TleDataList &)>(32, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 33, 34 },
        }}),
        // Slot 'removeSatellites'
        QtMocHelpers::SlotData<void()>(35, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setFlags'
        QtMocHelpers::SlotData<void()>(36, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'handleGroupChanges'
        QtMocHelpers::SlotData<void(QListWidgetItem *)>(37, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 38, 39 },
        }}),
        // Slot 'trackSatellite'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(40, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 41, 6 },
        }}),
        // Slot 'updateTLEs'
        QtMocHelpers::SlotData<void()>(42, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'searchSatellitesClear'
        QtMocHelpers::SlotData<void()>(43, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'askSatMarkerColor'
        QtMocHelpers::SlotData<void()>(44, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'askSatOrbitColor'
        QtMocHelpers::SlotData<void()>(45, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'askSatInfoColor'
        QtMocHelpers::SlotData<void()>(46, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'descriptionTextChanged'
        QtMocHelpers::SlotData<void()>(47, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setRightSideToROMode'
        QtMocHelpers::SlotData<void()>(48, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setRightSideToRWMode'
        QtMocHelpers::SlotData<void()>(49, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'enableMinMaxAltitude'
        QtMocHelpers::SlotData<void(bool)>(50, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 14 },
        }}),
        // Slot 'enableMinMaxMagnitude'
        QtMocHelpers::SlotData<void(bool)>(51, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 14 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<SatellitesDialog, qt_meta_tag_ZN16SatellitesDialogE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject SatellitesDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<StelDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16SatellitesDialogE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16SatellitesDialogE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16SatellitesDialogE_t>.metaTypes,
    nullptr
} };

void SatellitesDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SatellitesDialog *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->retranslate(); break;
        case 1: _t->jumpToSourcesTab(); break;
        case 2: _t->updateCountdown(); break;
        case 3: _t->filterListByGroup((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->updateSatelliteData(); break;
        case 5: _t->updateSatelliteAndSaveData(); break;
        case 6: _t->updateFilteredSatellitesList(); break;
        case 7: _t->selectFilteredSatellitesList(); break;
        case 8: _t->saveSatellites(); break;
        case 9: _t->showUpdateState((*reinterpret_cast<std::add_pointer_t<Satellites::UpdateState>>(_a[1]))); break;
        case 10: _t->showUpdateCompleted((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4]))); break;
        case 11: _t->handleOrbitLinesGroup((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 12: _t->handleUmbraGroup((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 13: _t->saveEditedSource(); break;
        case 14: _t->updateButtonsProperties(); break;
        case 15: _t->saveSourceList(); break;
        case 16: _t->deleteSourceRow(); break;
        case 17: _t->addSourceRow(); break;
        case 18: _t->editSourceRow(); break;
        case 19: _t->toggleCheckableSources(); break;
        case 20: _t->restoreDefaults(); break;
        case 21: _t->restoreTleSources(); break;
        case 22: _t->saveSettings(); break;
        case 23: _t->addSatellites((*reinterpret_cast<std::add_pointer_t<TleDataList>>(_a[1]))); break;
        case 24: _t->removeSatellites(); break;
        case 25: _t->setFlags(); break;
        case 26: _t->handleGroupChanges((*reinterpret_cast<std::add_pointer_t<QListWidgetItem*>>(_a[1]))); break;
        case 27: _t->trackSatellite((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 28: _t->updateTLEs(); break;
        case 29: _t->searchSatellitesClear(); break;
        case 30: _t->askSatMarkerColor(); break;
        case 31: _t->askSatOrbitColor(); break;
        case 32: _t->askSatInfoColor(); break;
        case 33: _t->descriptionTextChanged(); break;
        case 34: _t->setRightSideToROMode(); break;
        case 35: _t->setRightSideToRWMode(); break;
        case 36: _t->enableMinMaxAltitude((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 37: _t->enableMinMaxMagnitude((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *SatellitesDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SatellitesDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16SatellitesDialogE_t>.strings))
        return static_cast<void*>(this);
    return StelDialog::qt_metacast(_clname);
}

int SatellitesDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StelDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 38)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 38;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 38)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 38;
    }
    return _id;
}
QT_WARNING_POP
