/********************************************************************************
** Form generated from reading UI file 'addRemoveLandscapesDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_ADDREMOVELANDSCAPESDIALOG_H
#define UI_ADDREMOVELANDSCAPESDIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "Dialog.hpp"
#include "gui/StelCloseButton.hpp"

QT_BEGIN_NAMESPACE

class Ui_addRemoveLandscapesDialogForm
{
public:
    QVBoxLayout *verticalLayout;
    TitleBar *titleBar;
    QGroupBox *groupBoxAdd;
    QGridLayout *gridLayout;
    QPushButton *pushButtonBrowseForArchive;
    QCheckBox *checkBoxUseLandscape;
    QGroupBox *groupBoxMessage;
    QGridLayout *gridLayout_4;
    QLabel *labelMessage;
    QPushButton *pushButtonMessageOK;
    QSpacerItem *horizontalSpacer;
    QGroupBox *groupBoxRemove;
    QHBoxLayout *horizontalLayout;
    QListWidget *listWidgetUserLandscapes;
    QFrame *frameSidePane;
    QVBoxLayout *verticalLayoutSidePane;
    QLabel *labelLandscapeName;
    QLabel *labelLandscapeSize;
    QPushButton *pushButtonRemove;
    QLabel *labelWarning;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *addRemoveLandscapesDialogForm)
    {
        if (addRemoveLandscapesDialogForm->objectName().isEmpty())
            addRemoveLandscapesDialogForm->setObjectName("addRemoveLandscapesDialogForm");
        addRemoveLandscapesDialogForm->resize(300, 300);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Maximum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(addRemoveLandscapesDialogForm->sizePolicy().hasHeightForWidth());
        addRemoveLandscapesDialogForm->setSizePolicy(sizePolicy);
        verticalLayout = new QVBoxLayout(addRemoveLandscapesDialogForm);
        verticalLayout->setSpacing(0);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        titleBar = new TitleBar(addRemoveLandscapesDialogForm);
        titleBar->setObjectName("titleBar");

        verticalLayout->addWidget(titleBar);

        groupBoxAdd = new QGroupBox(addRemoveLandscapesDialogForm);
        groupBoxAdd->setObjectName("groupBoxAdd");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(groupBoxAdd->sizePolicy().hasHeightForWidth());
        groupBoxAdd->setSizePolicy(sizePolicy1);
        gridLayout = new QGridLayout(groupBoxAdd);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setContentsMargins(0, 0, 0, 0);
        pushButtonBrowseForArchive = new QPushButton(groupBoxAdd);
        pushButtonBrowseForArchive->setObjectName("pushButtonBrowseForArchive");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(pushButtonBrowseForArchive->sizePolicy().hasHeightForWidth());
        pushButtonBrowseForArchive->setSizePolicy(sizePolicy2);

        gridLayout->addWidget(pushButtonBrowseForArchive, 0, 0, 1, 2);

        checkBoxUseLandscape = new QCheckBox(groupBoxAdd);
        checkBoxUseLandscape->setObjectName("checkBoxUseLandscape");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(checkBoxUseLandscape->sizePolicy().hasHeightForWidth());
        checkBoxUseLandscape->setSizePolicy(sizePolicy3);
        checkBoxUseLandscape->setChecked(true);

        gridLayout->addWidget(checkBoxUseLandscape, 1, 0, 1, 2);


        verticalLayout->addWidget(groupBoxAdd);

        groupBoxMessage = new QGroupBox(addRemoveLandscapesDialogForm);
        groupBoxMessage->setObjectName("groupBoxMessage");
        groupBoxMessage->setTitle(QString::fromUtf8("Message"));
        gridLayout_4 = new QGridLayout(groupBoxMessage);
        gridLayout_4->setObjectName("gridLayout_4");
        gridLayout_4->setContentsMargins(0, 0, 0, 0);
        labelMessage = new QLabel(groupBoxMessage);
        labelMessage->setObjectName("labelMessage");
        QSizePolicy sizePolicy4(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy4.setHorizontalStretch(0);
        sizePolicy4.setVerticalStretch(0);
        sizePolicy4.setHeightForWidth(labelMessage->sizePolicy().hasHeightForWidth());
        labelMessage->setSizePolicy(sizePolicy4);
        QFont font;
        font.setFamilies({QString::fromUtf8("DejaVu Sans")});
        font.setPointSize(12);
        labelMessage->setFont(font);
        labelMessage->setWordWrap(true);
        labelMessage->setTextInteractionFlags(Qt::TextSelectableByKeyboard|Qt::TextSelectableByMouse);

        gridLayout_4->addWidget(labelMessage, 0, 0, 1, 2);

        pushButtonMessageOK = new QPushButton(groupBoxMessage);
        pushButtonMessageOK->setObjectName("pushButtonMessageOK");
        QSizePolicy sizePolicy5(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy5.setHorizontalStretch(0);
        sizePolicy5.setVerticalStretch(0);
        sizePolicy5.setHeightForWidth(pushButtonMessageOK->sizePolicy().hasHeightForWidth());
        pushButtonMessageOK->setSizePolicy(sizePolicy5);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/graphicGui/uibtClose.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        pushButtonMessageOK->setIcon(icon);

        gridLayout_4->addWidget(pushButtonMessageOK, 1, 1, 1, 1);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_4->addItem(horizontalSpacer, 1, 0, 1, 1);


        verticalLayout->addWidget(groupBoxMessage);

        groupBoxRemove = new QGroupBox(addRemoveLandscapesDialogForm);
        groupBoxRemove->setObjectName("groupBoxRemove");
        sizePolicy1.setHeightForWidth(groupBoxRemove->sizePolicy().hasHeightForWidth());
        groupBoxRemove->setSizePolicy(sizePolicy1);
        horizontalLayout = new QHBoxLayout(groupBoxRemove);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        listWidgetUserLandscapes = new QListWidget(groupBoxRemove);
        listWidgetUserLandscapes->setObjectName("listWidgetUserLandscapes");
        sizePolicy4.setHeightForWidth(listWidgetUserLandscapes->sizePolicy().hasHeightForWidth());
        listWidgetUserLandscapes->setSizePolicy(sizePolicy4);

        horizontalLayout->addWidget(listWidgetUserLandscapes);

        frameSidePane = new QFrame(groupBoxRemove);
        frameSidePane->setObjectName("frameSidePane");
        sizePolicy2.setHeightForWidth(frameSidePane->sizePolicy().hasHeightForWidth());
        frameSidePane->setSizePolicy(sizePolicy2);
        verticalLayoutSidePane = new QVBoxLayout(frameSidePane);
        verticalLayoutSidePane->setObjectName("verticalLayoutSidePane");
        verticalLayoutSidePane->setContentsMargins(0, 0, 0, 0);
        labelLandscapeName = new QLabel(frameSidePane);
        labelLandscapeName->setObjectName("labelLandscapeName");
        labelLandscapeName->setText(QString::fromUtf8(""));

        verticalLayoutSidePane->addWidget(labelLandscapeName);

        labelLandscapeSize = new QLabel(frameSidePane);
        labelLandscapeSize->setObjectName("labelLandscapeSize");
        sizePolicy1.setHeightForWidth(labelLandscapeSize->sizePolicy().hasHeightForWidth());
        labelLandscapeSize->setSizePolicy(sizePolicy1);

        verticalLayoutSidePane->addWidget(labelLandscapeSize);

        pushButtonRemove = new QPushButton(frameSidePane);
        pushButtonRemove->setObjectName("pushButtonRemove");
        sizePolicy3.setHeightForWidth(pushButtonRemove->sizePolicy().hasHeightForWidth());
        pushButtonRemove->setSizePolicy(sizePolicy3);

        verticalLayoutSidePane->addWidget(pushButtonRemove);

        labelWarning = new QLabel(frameSidePane);
        labelWarning->setObjectName("labelWarning");
        QSizePolicy sizePolicy6(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy6.setHorizontalStretch(0);
        sizePolicy6.setVerticalStretch(0);
        sizePolicy6.setHeightForWidth(labelWarning->sizePolicy().hasHeightForWidth());
        labelWarning->setSizePolicy(sizePolicy6);
        labelWarning->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
        labelWarning->setWordWrap(true);

        verticalLayoutSidePane->addWidget(labelWarning);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayoutSidePane->addItem(verticalSpacer);


        horizontalLayout->addWidget(frameSidePane);


        verticalLayout->addWidget(groupBoxRemove);

        QWidget::setTabOrder(checkBoxUseLandscape, pushButtonBrowseForArchive);
        QWidget::setTabOrder(pushButtonBrowseForArchive, pushButtonMessageOK);
        QWidget::setTabOrder(pushButtonMessageOK, listWidgetUserLandscapes);
        QWidget::setTabOrder(listWidgetUserLandscapes, pushButtonRemove);

        retranslateUi(addRemoveLandscapesDialogForm);

        QMetaObject::connectSlotsByName(addRemoveLandscapesDialogForm);
    } // setupUi

    void retranslateUi(QWidget *addRemoveLandscapesDialogForm)
    {
        titleBar->setTitle(QCoreApplication::translate("addRemoveLandscapesDialogForm", "Add/Remove Landscapes", nullptr));
        groupBoxAdd->setTitle(QCoreApplication::translate("addRemoveLandscapesDialogForm", "Add a new landscape", nullptr));
        pushButtonBrowseForArchive->setText(QCoreApplication::translate("addRemoveLandscapesDialogForm", "Install a new landscape from a ZIP archive...", nullptr));
        checkBoxUseLandscape->setText(QCoreApplication::translate("addRemoveLandscapesDialogForm", "Switch to the new landscape after installation", nullptr));
        labelMessage->setText(QString());
        pushButtonMessageOK->setText(QString());
        groupBoxRemove->setTitle(QCoreApplication::translate("addRemoveLandscapesDialogForm", "Remove an installed landscape", nullptr));
        labelLandscapeSize->setText(QString());
        pushButtonRemove->setText(QCoreApplication::translate("addRemoveLandscapesDialogForm", "Remove", nullptr));
        labelWarning->setText(QCoreApplication::translate("addRemoveLandscapesDialogForm", "WARNING: Removing the selected landscape means deleting its files. This operation is irreversible.", nullptr));
        (void)addRemoveLandscapesDialogForm;
    } // retranslateUi

};

namespace Ui {
    class addRemoveLandscapesDialogForm: public Ui_addRemoveLandscapesDialogForm {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_ADDREMOVELANDSCAPESDIALOG_H
