/****************************************************************************
** Meta object code from reading C++ file 'NavStars.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../stellarium/plugins/NavStars/src/NavStars.hpp"
#include <QtCore/qmetatype.h>
#include <QtCore/qplugin.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'NavStars.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8NavStarsE_t {};
} // unnamed namespace

template <> constexpr inline auto NavStars::qt_create_metaobjectdata<qt_meta_tag_ZN8NavStarsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "NavStars",
        "navStarsMarksChanged",
        "",
        "b",
        "enableAtStartupChanged",
        "highlightWhenVisibleChanged",
        "limitInfoToNavStarsChanged",
        "upperLimbChanged",
        "tabulatedDisplayChanged",
        "showExtraDecimalsChanged",
        "flagUseUTCTimeChanged",
        "setNavStarsMarks",
        "getNavStarsMarks",
        "setEnableAtStartup",
        "getEnableAtStartup",
        "setHighlightWhenVisible",
        "getHighlightWhenVisible",
        "setLimitInfoToNavStars",
        "getLimitInfoToNavStars",
        "setUpperLimb",
        "getUpperLimb",
        "setTabulatedDisplay",
        "getTabulatedDisplay",
        "setShowExtraDecimals",
        "getShowExtraDecimals",
        "setFlagUseUTCTime",
        "getFlagUseUTCTime",
        "setCurrentNavigationalStarsSet",
        "NavigationalStarsSet",
        "nsset",
        "getCurrentNavigationalStarsSet",
        "NavStars::NavigationalStarsSet",
        "getCurrentNavigationalStarsSetKey",
        "getCurrentNavigationalStarsSetDescription",
        "setCurrentNavigationalStarsSetKey",
        "key",
        "addExtraInfo",
        "StelCore*",
        "core",
        "extraInfo",
        "StelObjectP",
        "selectedObject",
        "displayStandardInfo",
        "NavStarsCalculator&",
        "calc",
        "extraText",
        "displayTabulatedInfo",
        "oneRowTwoCells",
        "a",
        "extra",
        "tabulatedView",
        "isPermittedObject",
        "s",
        "saveSettings",
        "setUseDecimalDegrees",
        "flag",
        "navStarsVisible",
        "displayAtStartup",
        "highlightWhenVisible",
        "limitInfoToNavStars",
        "upperLimb",
        "tabulatedDisplay",
        "showExtraDecimals",
        "useUTCTime",
        "AngloAmerican",
        "French",
        "British",
        "Wrinkles",
        "Russian",
        "USSRAvia",
        "USSRSpace",
        "German",
        "GeminiAPS",
        "MeadeLX200",
        "MeadeETX",
        "MeadeAS494",
        "MeadeAS497",
        "CelestronNS",
        "Apollo",
        "SkywatcherSS",
        "VixenSB",
        "ArgoNavis",
        "OrionIS",
        "SkyCommander"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'navStarsMarksChanged'
        QtMocHelpers::SignalData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'enableAtStartupChanged'
        QtMocHelpers::SignalData<void(bool)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'highlightWhenVisibleChanged'
        QtMocHelpers::SignalData<void(bool)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'limitInfoToNavStarsChanged'
        QtMocHelpers::SignalData<void(bool)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'upperLimbChanged'
        QtMocHelpers::SignalData<void(bool)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'tabulatedDisplayChanged'
        QtMocHelpers::SignalData<void(bool)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'showExtraDecimalsChanged'
        QtMocHelpers::SignalData<void(bool)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagUseUTCTimeChanged'
        QtMocHelpers::SignalData<void(bool)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'setNavStarsMarks'
        QtMocHelpers::SlotData<void(const bool)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'getNavStarsMarks'
        QtMocHelpers::SlotData<bool() const>(12, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setEnableAtStartup'
        QtMocHelpers::SlotData<void(bool)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'getEnableAtStartup'
        QtMocHelpers::SlotData<bool() const>(14, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setHighlightWhenVisible'
        QtMocHelpers::SlotData<void(bool)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'getHighlightWhenVisible'
        QtMocHelpers::SlotData<bool() const>(16, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setLimitInfoToNavStars'
        QtMocHelpers::SlotData<void(bool)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'getLimitInfoToNavStars'
        QtMocHelpers::SlotData<bool() const>(18, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setUpperLimb'
        QtMocHelpers::SlotData<void(bool)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'getUpperLimb'
        QtMocHelpers::SlotData<bool() const>(20, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setTabulatedDisplay'
        QtMocHelpers::SlotData<void(bool)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'getTabulatedDisplay'
        QtMocHelpers::SlotData<bool() const>(22, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setShowExtraDecimals'
        QtMocHelpers::SlotData<void(bool)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'getShowExtraDecimals'
        QtMocHelpers::SlotData<bool() const>(24, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFlagUseUTCTime'
        QtMocHelpers::SlotData<void(bool)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'getFlagUseUTCTime'
        QtMocHelpers::SlotData<bool() const>(26, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setCurrentNavigationalStarsSet'
        QtMocHelpers::SlotData<void(enum NavigationalStarsSet)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 28, 29 },
        }}),
        // Slot 'getCurrentNavigationalStarsSet'
        QtMocHelpers::SlotData<NavStars::NavigationalStarsSet() const>(30, 2, QMC::AccessPublic, 0x80000000 | 31),
        // Slot 'getCurrentNavigationalStarsSetKey'
        QtMocHelpers::SlotData<QString() const>(32, 2, QMC::AccessPublic, QMetaType::QString),
        // Slot 'getCurrentNavigationalStarsSetDescription'
        QtMocHelpers::SlotData<QString() const>(33, 2, QMC::AccessPublic, QMetaType::QString),
        // Slot 'setCurrentNavigationalStarsSetKey'
        QtMocHelpers::SlotData<void(const QString &)>(34, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 35 },
        }}),
        // Slot 'addExtraInfo'
        QtMocHelpers::SlotData<void(StelCore *)>(36, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 37, 38 },
        }}),
        // Slot 'extraInfo'
        QtMocHelpers::SlotData<void(StelCore *, const StelObjectP &)>(39, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 37, 38 }, { 0x80000000 | 40, 41 },
        }}),
        // Slot 'displayStandardInfo'
        QtMocHelpers::SlotData<void(const StelObjectP &, NavStarsCalculator &, const QString &)>(42, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 40, 41 }, { 0x80000000 | 43, 44 }, { QMetaType::QString, 45 },
        }}),
        // Slot 'displayTabulatedInfo'
        QtMocHelpers::SlotData<void(const StelObjectP &, NavStarsCalculator &, const QString &)>(46, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 40, 41 }, { 0x80000000 | 43, 44 }, { QMetaType::QString, 45 },
        }}),
        // Slot 'oneRowTwoCells'
        QtMocHelpers::SlotData<QString(const QString &, const QString &, const QString &, bool)>(47, 2, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::QString, 48 }, { QMetaType::QString, 3 }, { QMetaType::QString, 49 }, { QMetaType::Bool, 50 },
        }}),
        // Slot 'isPermittedObject'
        QtMocHelpers::SlotData<bool(const QString &)>(51, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 52 },
        }}),
        // Slot 'saveSettings'
        QtMocHelpers::SlotData<void()>(53, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setUseDecimalDegrees'
        QtMocHelpers::SlotData<void(bool)>(54, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 55 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'navStarsVisible'
        QtMocHelpers::PropertyData<bool>(56, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 0),
        // property 'displayAtStartup'
        QtMocHelpers::PropertyData<bool>(57, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 1),
        // property 'highlightWhenVisible'
        QtMocHelpers::PropertyData<bool>(58, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'limitInfoToNavStars'
        QtMocHelpers::PropertyData<bool>(59, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'upperLimb'
        QtMocHelpers::PropertyData<bool>(60, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'tabulatedDisplay'
        QtMocHelpers::PropertyData<bool>(61, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
        // property 'showExtraDecimals'
        QtMocHelpers::PropertyData<bool>(62, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 6),
        // property 'useUTCTime'
        QtMocHelpers::PropertyData<bool>(63, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 7),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'NavigationalStarsSet'
        QtMocHelpers::EnumData<enum NavigationalStarsSet>(28, 28, QMC::EnumFlags{}).add({
            {   64, NavigationalStarsSet::AngloAmerican },
            {   65, NavigationalStarsSet::French },
            {   66, NavigationalStarsSet::British },
            {   67, NavigationalStarsSet::Wrinkles },
            {   68, NavigationalStarsSet::Russian },
            {   69, NavigationalStarsSet::USSRAvia },
            {   70, NavigationalStarsSet::USSRSpace },
            {   71, NavigationalStarsSet::German },
            {   72, NavigationalStarsSet::GeminiAPS },
            {   73, NavigationalStarsSet::MeadeLX200 },
            {   74, NavigationalStarsSet::MeadeETX },
            {   75, NavigationalStarsSet::MeadeAS494 },
            {   76, NavigationalStarsSet::MeadeAS497 },
            {   77, NavigationalStarsSet::CelestronNS },
            {   78, NavigationalStarsSet::Apollo },
            {   79, NavigationalStarsSet::SkywatcherSS },
            {   80, NavigationalStarsSet::VixenSB },
            {   81, NavigationalStarsSet::ArgoNavis },
            {   82, NavigationalStarsSet::OrionIS },
            {   83, NavigationalStarsSet::SkyCommander },
        }),
    };
    return QtMocHelpers::metaObjectData<NavStars, qt_meta_tag_ZN8NavStarsE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject NavStars::staticMetaObject = { {
    QMetaObject::SuperData::link<StelModule::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8NavStarsE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8NavStarsE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8NavStarsE_t>.metaTypes,
    nullptr
} };

void NavStars::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<NavStars *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->navStarsMarksChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->enableAtStartupChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->highlightWhenVisibleChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->limitInfoToNavStarsChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->upperLimbChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->tabulatedDisplayChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->showExtraDecimalsChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->flagUseUTCTimeChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->setNavStarsMarks((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 9: { bool _r = _t->getNavStarsMarks();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 10: _t->setEnableAtStartup((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 11: { bool _r = _t->getEnableAtStartup();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 12: _t->setHighlightWhenVisible((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 13: { bool _r = _t->getHighlightWhenVisible();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 14: _t->setLimitInfoToNavStars((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 15: { bool _r = _t->getLimitInfoToNavStars();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 16: _t->setUpperLimb((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 17: { bool _r = _t->getUpperLimb();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 18: _t->setTabulatedDisplay((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 19: { bool _r = _t->getTabulatedDisplay();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 20: _t->setShowExtraDecimals((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 21: { bool _r = _t->getShowExtraDecimals();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 22: _t->setFlagUseUTCTime((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 23: { bool _r = _t->getFlagUseUTCTime();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 24: _t->setCurrentNavigationalStarsSet((*reinterpret_cast<std::add_pointer_t<enum NavigationalStarsSet>>(_a[1]))); break;
        case 25: { NavStars::NavigationalStarsSet _r = _t->getCurrentNavigationalStarsSet();
            if (_a[0]) *reinterpret_cast<NavStars::NavigationalStarsSet*>(_a[0]) = std::move(_r); }  break;
        case 26: { QString _r = _t->getCurrentNavigationalStarsSetKey();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 27: { QString _r = _t->getCurrentNavigationalStarsSetDescription();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 28: _t->setCurrentNavigationalStarsSetKey((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 29: _t->addExtraInfo((*reinterpret_cast<std::add_pointer_t<StelCore*>>(_a[1]))); break;
        case 30: _t->extraInfo((*reinterpret_cast<std::add_pointer_t<StelCore*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<StelObjectP>>(_a[2]))); break;
        case 31: _t->displayStandardInfo((*reinterpret_cast<std::add_pointer_t<StelObjectP>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<NavStarsCalculator&>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 32: _t->displayTabulatedInfo((*reinterpret_cast<std::add_pointer_t<StelObjectP>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<NavStarsCalculator&>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 33: { QString _r = _t->oneRowTwoCells((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 34: { bool _r = _t->isPermittedObject((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 35: _t->saveSettings(); break;
        case 36: _t->setUseDecimalDegrees((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 30:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< StelObjectP >(); break;
            }
            break;
        case 31:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< StelObjectP >(); break;
            }
            break;
        case 32:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< StelObjectP >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (NavStars::*)(bool )>(_a, &NavStars::navStarsMarksChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (NavStars::*)(bool )>(_a, &NavStars::enableAtStartupChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (NavStars::*)(bool )>(_a, &NavStars::highlightWhenVisibleChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (NavStars::*)(bool )>(_a, &NavStars::limitInfoToNavStarsChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (NavStars::*)(bool )>(_a, &NavStars::upperLimbChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (NavStars::*)(bool )>(_a, &NavStars::tabulatedDisplayChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (NavStars::*)(bool )>(_a, &NavStars::showExtraDecimalsChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (NavStars::*)(bool )>(_a, &NavStars::flagUseUTCTimeChanged, 7))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->getNavStarsMarks(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->getEnableAtStartup(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->getHighlightWhenVisible(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->getLimitInfoToNavStars(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->getUpperLimb(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->getTabulatedDisplay(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->getShowExtraDecimals(); break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->getFlagUseUTCTime(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setNavStarsMarks(*reinterpret_cast<bool*>(_v)); break;
        case 1: _t->setEnableAtStartup(*reinterpret_cast<bool*>(_v)); break;
        case 2: _t->setHighlightWhenVisible(*reinterpret_cast<bool*>(_v)); break;
        case 3: _t->setLimitInfoToNavStars(*reinterpret_cast<bool*>(_v)); break;
        case 4: _t->setUpperLimb(*reinterpret_cast<bool*>(_v)); break;
        case 5: _t->setTabulatedDisplay(*reinterpret_cast<bool*>(_v)); break;
        case 6: _t->setShowExtraDecimals(*reinterpret_cast<bool*>(_v)); break;
        case 7: _t->setFlagUseUTCTime(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *NavStars::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *NavStars::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8NavStarsE_t>.strings))
        return static_cast<void*>(this);
    return StelModule::qt_metacast(_clname);
}

int NavStars::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StelModule::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 37)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 37;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 37)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 37;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void NavStars::navStarsMarksChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void NavStars::enableAtStartupChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void NavStars::highlightWhenVisibleChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void NavStars::limitInfoToNavStarsChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void NavStars::upperLimbChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void NavStars::tabulatedDisplayChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void NavStars::showExtraDecimalsChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void NavStars::flagUseUTCTimeChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN27NavStarsStelPluginInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto NavStarsStelPluginInterface::qt_create_metaobjectdata<qt_meta_tag_ZN27NavStarsStelPluginInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "NavStarsStelPluginInterface"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<NavStarsStelPluginInterface, qt_meta_tag_ZN27NavStarsStelPluginInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject NavStarsStelPluginInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27NavStarsStelPluginInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27NavStarsStelPluginInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN27NavStarsStelPluginInterfaceE_t>.metaTypes,
    nullptr
} };

void NavStarsStelPluginInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<NavStarsStelPluginInterface *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *NavStarsStelPluginInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *NavStarsStelPluginInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27NavStarsStelPluginInterfaceE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "StelPluginInterface"))
        return static_cast< StelPluginInterface*>(this);
    if (!strcmp(_clname, "org.stellarium.StelPluginInterface/2.0"))
        return static_cast< StelPluginInterface*>(this);
    return QObject::qt_metacast(_clname);
}

int NavStarsStelPluginInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}

#ifdef QT_MOC_EXPORT_PLUGIN_V2
static constexpr unsigned char qt_pluginMetaDataV2_NavStarsStelPluginInterface[] = {
    0xbf, 
    // "IID"
    0x02,  0x78,  0x26,  'o',  'r',  'g',  '.',  's', 
    't',  'e',  'l',  'l',  'a',  'r',  'i',  'u', 
    'm',  '.',  'S',  't',  'e',  'l',  'P',  'l', 
    'u',  'g',  'i',  'n',  'I',  'n',  't',  'e', 
    'r',  'f',  'a',  'c',  'e',  '/',  '2',  '.', 
    '0', 
    // "className"
    0x03,  0x78,  0x1b,  'N',  'a',  'v',  'S',  't', 
    'a',  'r',  's',  'S',  't',  'e',  'l',  'P', 
    'l',  'u',  'g',  'i',  'n',  'I',  'n',  't', 
    'e',  'r',  'f',  'a',  'c',  'e', 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN_V2(NavStarsStelPluginInterface, NavStarsStelPluginInterface, qt_pluginMetaDataV2_NavStarsStelPluginInterface)
#else
QT_PLUGIN_METADATA_SECTION
Q_CONSTINIT static constexpr unsigned char qt_pluginMetaData_NavStarsStelPluginInterface[] = {
    'Q', 'T', 'M', 'E', 'T', 'A', 'D', 'A', 'T', 'A', ' ', '!',
    // metadata version, Qt version, architectural requirements
    0, QT_VERSION_MAJOR, QT_VERSION_MINOR, qPluginArchRequirements(),
    0xbf, 
    // "IID"
    0x02,  0x78,  0x26,  'o',  'r',  'g',  '.',  's', 
    't',  'e',  'l',  'l',  'a',  'r',  'i',  'u', 
    'm',  '.',  'S',  't',  'e',  'l',  'P',  'l', 
    'u',  'g',  'i',  'n',  'I',  'n',  't',  'e', 
    'r',  'f',  'a',  'c',  'e',  '/',  '2',  '.', 
    '0', 
    // "className"
    0x03,  0x78,  0x1b,  'N',  'a',  'v',  'S',  't', 
    'a',  'r',  's',  'S',  't',  'e',  'l',  'P', 
    'l',  'u',  'g',  'i',  'n',  'I',  'n',  't', 
    'e',  'r',  'f',  'a',  'c',  'e', 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN(NavStarsStelPluginInterface, NavStarsStelPluginInterface)
#endif  // QT_MOC_EXPORT_PLUGIN_V2

QT_WARNING_POP
