/****************************************************************************
** Meta object code from reading C++ file 'AsterismMgr.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../stellarium/src/core/modules/AsterismMgr.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'AsterismMgr.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11AsterismMgrE_t {};
} // unnamed namespace

template <> constexpr inline auto AsterismMgr::qt_create_metaobjectdata<qt_meta_tag_ZN11AsterismMgrE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "AsterismMgr",
        "fontSizeChanged",
        "",
        "newSize",
        "linesColorChanged",
        "Vec3f",
        "color",
        "linesDisplayedChanged",
        "displayed",
        "namesColorChanged",
        "namesDisplayedChanged",
        "asterismLineThicknessChanged",
        "thickness",
        "rayHelpersColorChanged",
        "rayHelpersDisplayedChanged",
        "rayHelperThicknessChanged",
        "isolateAsterismSelectedChanged",
        "isolate",
        "namesFadeDurationChanged",
        "duration",
        "linesFadeDurationChanged",
        "rayHelpersFadeDurationChanged",
        "setLinesColor",
        "getLinesColor",
        "setFlagLines",
        "getFlagLines",
        "setRayHelpersColor",
        "getRayHelpersColor",
        "setFlagRayHelpers",
        "getFlagRayHelpers",
        "setLabelsColor",
        "getLabelsColor",
        "setFlagLabels",
        "getFlagLabels",
        "setFontSize",
        "newFontSize",
        "getFontSize",
        "setAsterismLineThickness",
        "getAsterismLineThickness",
        "setRayHelperThickness",
        "getRayHelperThickness",
        "isLinesDefined",
        "setFlagIsolateAsterismSelected",
        "getFlagIsolateAsterismSelected",
        "selectAsterism",
        "englishName",
        "deselectAsterism",
        "deselectAsterisms",
        "selectAllAsterisms",
        "setLinesFadeDuration",
        "getLinesFadeDuration",
        "setLabelsFadeDuration",
        "getLabelsFadeDuration",
        "setRayHelpersFadeDuration",
        "getRayHelpersFadeDuration",
        "updateSkyCulture",
        "StelSkyCulture",
        "skyCulture",
        "updateI18n",
        "selectedObjectChange",
        "StelModule::StelModuleSelectAction",
        "action",
        "switchSelectionMode",
        "fontSize",
        "linesColor",
        "linesDisplayed",
        "rayHelpersColor",
        "rayHelpersDisplayed",
        "namesColor",
        "namesDisplayed",
        "asterismLineThickness",
        "rayHelperThickness",
        "isolateAsterismSelected",
        "namesFadeDuration",
        "linesFadeDuration",
        "rayHelpersFadeDuration"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'fontSizeChanged'
        QtMocHelpers::SignalData<void(const int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'linesColorChanged'
        QtMocHelpers::SignalData<void(const Vec3f &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Signal 'linesDisplayedChanged'
        QtMocHelpers::SignalData<void(const bool)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Signal 'namesColorChanged'
        QtMocHelpers::SignalData<void(const Vec3f &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Signal 'namesDisplayedChanged'
        QtMocHelpers::SignalData<void(const bool)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Signal 'asterismLineThicknessChanged'
        QtMocHelpers::SignalData<void(int)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 12 },
        }}),
        // Signal 'rayHelpersColorChanged'
        QtMocHelpers::SignalData<void(const Vec3f &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Signal 'rayHelpersDisplayedChanged'
        QtMocHelpers::SignalData<void(const bool)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Signal 'rayHelperThicknessChanged'
        QtMocHelpers::SignalData<void(int)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 12 },
        }}),
        // Signal 'isolateAsterismSelectedChanged'
        QtMocHelpers::SignalData<void(const bool)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 17 },
        }}),
        // Signal 'namesFadeDurationChanged'
        QtMocHelpers::SignalData<void(const float)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 19 },
        }}),
        // Signal 'linesFadeDurationChanged'
        QtMocHelpers::SignalData<void(const float)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 19 },
        }}),
        // Signal 'rayHelpersFadeDurationChanged'
        QtMocHelpers::SignalData<void(const float)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 19 },
        }}),
        // Slot 'setLinesColor'
        QtMocHelpers::SlotData<void(const Vec3f &)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Slot 'getLinesColor'
        QtMocHelpers::SlotData<Vec3f() const>(23, 2, QMC::AccessPublic, 0x80000000 | 5),
        // Slot 'setFlagLines'
        QtMocHelpers::SlotData<void(const bool)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Slot 'getFlagLines'
        QtMocHelpers::SlotData<bool() const>(25, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setRayHelpersColor'
        QtMocHelpers::SlotData<void(const Vec3f &)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Slot 'getRayHelpersColor'
        QtMocHelpers::SlotData<Vec3f() const>(27, 2, QMC::AccessPublic, 0x80000000 | 5),
        // Slot 'setFlagRayHelpers'
        QtMocHelpers::SlotData<void(const bool)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Slot 'getFlagRayHelpers'
        QtMocHelpers::SlotData<bool() const>(29, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setLabelsColor'
        QtMocHelpers::SlotData<void(const Vec3f &)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Slot 'getLabelsColor'
        QtMocHelpers::SlotData<Vec3f() const>(31, 2, QMC::AccessPublic, 0x80000000 | 5),
        // Slot 'setFlagLabels'
        QtMocHelpers::SlotData<void(const bool)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Slot 'getFlagLabels'
        QtMocHelpers::SlotData<bool() const>(33, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFontSize'
        QtMocHelpers::SlotData<void(const int)>(34, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 35 },
        }}),
        // Slot 'getFontSize'
        QtMocHelpers::SlotData<int() const>(36, 2, QMC::AccessPublic, QMetaType::Int),
        // Slot 'setAsterismLineThickness'
        QtMocHelpers::SlotData<void(const int)>(37, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 12 },
        }}),
        // Slot 'getAsterismLineThickness'
        QtMocHelpers::SlotData<int() const>(38, 2, QMC::AccessPublic, QMetaType::Int),
        // Slot 'setRayHelperThickness'
        QtMocHelpers::SlotData<void(const int)>(39, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 12 },
        }}),
        // Slot 'getRayHelperThickness'
        QtMocHelpers::SlotData<int() const>(40, 2, QMC::AccessPublic, QMetaType::Int),
        // Slot 'isLinesDefined'
        QtMocHelpers::SlotData<bool()>(41, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFlagIsolateAsterismSelected'
        QtMocHelpers::SlotData<void(const bool)>(42, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 17 },
        }}),
        // Slot 'getFlagIsolateAsterismSelected'
        QtMocHelpers::SlotData<bool() const>(43, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'selectAsterism'
        QtMocHelpers::SlotData<void(const QString &)>(44, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 45 },
        }}),
        // Slot 'deselectAsterism'
        QtMocHelpers::SlotData<void(const QString &)>(46, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 45 },
        }}),
        // Slot 'deselectAsterisms'
        QtMocHelpers::SlotData<void()>(47, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'selectAllAsterisms'
        QtMocHelpers::SlotData<void()>(48, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setLinesFadeDuration'
        QtMocHelpers::SlotData<void(const float)>(49, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 19 },
        }}),
        // Slot 'getLinesFadeDuration'
        QtMocHelpers::SlotData<float() const>(50, 2, QMC::AccessPublic, QMetaType::Float),
        // Slot 'setLabelsFadeDuration'
        QtMocHelpers::SlotData<void(const float)>(51, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 19 },
        }}),
        // Slot 'getLabelsFadeDuration'
        QtMocHelpers::SlotData<float() const>(52, 2, QMC::AccessPublic, QMetaType::Float),
        // Slot 'setRayHelpersFadeDuration'
        QtMocHelpers::SlotData<void(const float)>(53, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 19 },
        }}),
        // Slot 'getRayHelpersFadeDuration'
        QtMocHelpers::SlotData<float() const>(54, 2, QMC::AccessPublic, QMetaType::Float),
        // Slot 'updateSkyCulture'
        QtMocHelpers::SlotData<void(const StelSkyCulture &)>(55, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 56, 57 },
        }}),
        // Slot 'updateI18n'
        QtMocHelpers::SlotData<void()>(58, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'selectedObjectChange'
        QtMocHelpers::SlotData<void(StelModule::StelModuleSelectAction)>(59, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 60, 61 },
        }}),
        // Slot 'switchSelectionMode'
        QtMocHelpers::SlotData<void()>(62, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'fontSize'
        QtMocHelpers::PropertyData<int>(63, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'linesColor'
        QtMocHelpers::PropertyData<Vec3f>(64, 0x80000000 | 5, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 1),
        // property 'linesDisplayed'
        QtMocHelpers::PropertyData<bool>(65, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 2),
        // property 'rayHelpersColor'
        QtMocHelpers::PropertyData<Vec3f>(66, 0x80000000 | 5, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 6),
        // property 'rayHelpersDisplayed'
        QtMocHelpers::PropertyData<bool>(67, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 7),
        // property 'namesColor'
        QtMocHelpers::PropertyData<Vec3f>(68, 0x80000000 | 5, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag, 3),
        // property 'namesDisplayed'
        QtMocHelpers::PropertyData<bool>(69, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 4),
        // property 'asterismLineThickness'
        QtMocHelpers::PropertyData<int>(70, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
        // property 'rayHelperThickness'
        QtMocHelpers::PropertyData<int>(71, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 8),
        // property 'isolateAsterismSelected'
        QtMocHelpers::PropertyData<bool>(72, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 9),
        // property 'namesFadeDuration'
        QtMocHelpers::PropertyData<float>(73, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable, 10),
        // property 'linesFadeDuration'
        QtMocHelpers::PropertyData<float>(74, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 11),
        // property 'rayHelpersFadeDuration'
        QtMocHelpers::PropertyData<float>(75, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 12),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<AsterismMgr, qt_meta_tag_ZN11AsterismMgrE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject AsterismMgr::staticMetaObject = { {
    QMetaObject::SuperData::link<StelObjectModule::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11AsterismMgrE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11AsterismMgrE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11AsterismMgrE_t>.metaTypes,
    nullptr
} };

void AsterismMgr::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AsterismMgr *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->fontSizeChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->linesColorChanged((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1]))); break;
        case 2: _t->linesDisplayedChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->namesColorChanged((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1]))); break;
        case 4: _t->namesDisplayedChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->asterismLineThicknessChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 6: _t->rayHelpersColorChanged((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1]))); break;
        case 7: _t->rayHelpersDisplayedChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->rayHelperThicknessChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 9: _t->isolateAsterismSelectedChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 10: _t->namesFadeDurationChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 11: _t->linesFadeDurationChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 12: _t->rayHelpersFadeDurationChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 13: _t->setLinesColor((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1]))); break;
        case 14: { Vec3f _r = _t->getLinesColor();
            if (_a[0]) *reinterpret_cast<Vec3f*>(_a[0]) = std::move(_r); }  break;
        case 15: _t->setFlagLines((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 16: { bool _r = _t->getFlagLines();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 17: _t->setRayHelpersColor((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1]))); break;
        case 18: { Vec3f _r = _t->getRayHelpersColor();
            if (_a[0]) *reinterpret_cast<Vec3f*>(_a[0]) = std::move(_r); }  break;
        case 19: _t->setFlagRayHelpers((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 20: { bool _r = _t->getFlagRayHelpers();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 21: _t->setLabelsColor((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1]))); break;
        case 22: { Vec3f _r = _t->getLabelsColor();
            if (_a[0]) *reinterpret_cast<Vec3f*>(_a[0]) = std::move(_r); }  break;
        case 23: _t->setFlagLabels((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 24: { bool _r = _t->getFlagLabels();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 25: _t->setFontSize((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 26: { int _r = _t->getFontSize();
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 27: _t->setAsterismLineThickness((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 28: { int _r = _t->getAsterismLineThickness();
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 29: _t->setRayHelperThickness((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 30: { int _r = _t->getRayHelperThickness();
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 31: { bool _r = _t->isLinesDefined();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 32: _t->setFlagIsolateAsterismSelected((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 33: { bool _r = _t->getFlagIsolateAsterismSelected();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 34: _t->selectAsterism((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 35: _t->deselectAsterism((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 36: _t->deselectAsterisms(); break;
        case 37: _t->selectAllAsterisms(); break;
        case 38: _t->setLinesFadeDuration((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 39: { float _r = _t->getLinesFadeDuration();
            if (_a[0]) *reinterpret_cast<float*>(_a[0]) = std::move(_r); }  break;
        case 40: _t->setLabelsFadeDuration((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 41: { float _r = _t->getLabelsFadeDuration();
            if (_a[0]) *reinterpret_cast<float*>(_a[0]) = std::move(_r); }  break;
        case 42: _t->setRayHelpersFadeDuration((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 43: { float _r = _t->getRayHelpersFadeDuration();
            if (_a[0]) *reinterpret_cast<float*>(_a[0]) = std::move(_r); }  break;
        case 44: _t->updateSkyCulture((*reinterpret_cast<std::add_pointer_t<StelSkyCulture>>(_a[1]))); break;
        case 45: _t->updateI18n(); break;
        case 46: _t->selectedObjectChange((*reinterpret_cast<std::add_pointer_t<StelModule::StelModuleSelectAction>>(_a[1]))); break;
        case 47: _t->switchSelectionMode(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 13:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 17:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 21:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 46:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< StelModule::StelModuleSelectAction >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (AsterismMgr::*)(const int )>(_a, &AsterismMgr::fontSizeChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (AsterismMgr::*)(const Vec3f & )>(_a, &AsterismMgr::linesColorChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (AsterismMgr::*)(const bool )>(_a, &AsterismMgr::linesDisplayedChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (AsterismMgr::*)(const Vec3f & )>(_a, &AsterismMgr::namesColorChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (AsterismMgr::*)(const bool )>(_a, &AsterismMgr::namesDisplayedChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (AsterismMgr::*)(int )>(_a, &AsterismMgr::asterismLineThicknessChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (AsterismMgr::*)(const Vec3f & )>(_a, &AsterismMgr::rayHelpersColorChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (AsterismMgr::*)(const bool )>(_a, &AsterismMgr::rayHelpersDisplayedChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (AsterismMgr::*)(int )>(_a, &AsterismMgr::rayHelperThicknessChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (AsterismMgr::*)(const bool )>(_a, &AsterismMgr::isolateAsterismSelectedChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (AsterismMgr::*)(const float )>(_a, &AsterismMgr::namesFadeDurationChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (AsterismMgr::*)(const float )>(_a, &AsterismMgr::linesFadeDurationChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (AsterismMgr::*)(const float )>(_a, &AsterismMgr::rayHelpersFadeDurationChanged, 12))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 5:
        case 3:
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Vec3f >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->getFontSize(); break;
        case 1: *reinterpret_cast<Vec3f*>(_v) = _t->getLinesColor(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->getFlagLines(); break;
        case 3: *reinterpret_cast<Vec3f*>(_v) = _t->getRayHelpersColor(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->getFlagRayHelpers(); break;
        case 5: *reinterpret_cast<Vec3f*>(_v) = _t->getLabelsColor(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->getFlagLabels(); break;
        case 7: *reinterpret_cast<int*>(_v) = _t->getAsterismLineThickness(); break;
        case 8: *reinterpret_cast<int*>(_v) = _t->getRayHelperThickness(); break;
        case 9: *reinterpret_cast<bool*>(_v) = _t->getFlagIsolateAsterismSelected(); break;
        case 10: *reinterpret_cast<float*>(_v) = _t->getLabelsFadeDuration(); break;
        case 11: *reinterpret_cast<float*>(_v) = _t->getLinesFadeDuration(); break;
        case 12: *reinterpret_cast<float*>(_v) = _t->getRayHelpersFadeDuration(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setFontSize(*reinterpret_cast<int*>(_v)); break;
        case 1: _t->setLinesColor(*reinterpret_cast<Vec3f*>(_v)); break;
        case 2: _t->setFlagLines(*reinterpret_cast<bool*>(_v)); break;
        case 3: _t->setRayHelpersColor(*reinterpret_cast<Vec3f*>(_v)); break;
        case 4: _t->setFlagRayHelpers(*reinterpret_cast<bool*>(_v)); break;
        case 5: _t->setLabelsColor(*reinterpret_cast<Vec3f*>(_v)); break;
        case 6: _t->setFlagLabels(*reinterpret_cast<bool*>(_v)); break;
        case 7: _t->setAsterismLineThickness(*reinterpret_cast<int*>(_v)); break;
        case 8: _t->setRayHelperThickness(*reinterpret_cast<int*>(_v)); break;
        case 9: _t->setFlagIsolateAsterismSelected(*reinterpret_cast<bool*>(_v)); break;
        case 10: _t->setLabelsFadeDuration(*reinterpret_cast<float*>(_v)); break;
        case 11: _t->setLinesFadeDuration(*reinterpret_cast<float*>(_v)); break;
        case 12: _t->setRayHelpersFadeDuration(*reinterpret_cast<float*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *AsterismMgr::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *AsterismMgr::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11AsterismMgrE_t>.strings))
        return static_cast<void*>(this);
    return StelObjectModule::qt_metacast(_clname);
}

int AsterismMgr::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StelObjectModule::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 48)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 48;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 48)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 48;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    return _id;
}

// SIGNAL 0
void AsterismMgr::fontSizeChanged(const int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void AsterismMgr::linesColorChanged(const Vec3f & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void AsterismMgr::linesDisplayedChanged(const bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void AsterismMgr::namesColorChanged(const Vec3f & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void AsterismMgr::namesDisplayedChanged(const bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void AsterismMgr::asterismLineThicknessChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void AsterismMgr::rayHelpersColorChanged(const Vec3f & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void AsterismMgr::rayHelpersDisplayedChanged(const bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void AsterismMgr::rayHelperThicknessChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void AsterismMgr::isolateAsterismSelectedChanged(const bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void AsterismMgr::namesFadeDurationChanged(const float _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void AsterismMgr::linesFadeDurationChanged(const float _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void AsterismMgr::rayHelpersFadeDurationChanged(const float _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}
QT_WARNING_POP
