/********************************************************************************
** Form generated from reading UI file 'onlineQueriesDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_ONLINEQUERIESDIALOG_H
#define UI_ONLINEQUERIESDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTextBrowser>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "Dialog.hpp"
#include "gui/StelCloseButton.hpp"

QT_BEGIN_NAMESPACE

class Ui_onlineQueriesDialogForm
{
public:
    QVBoxLayout *vboxLayout;
    TitleBar *titleBar;
    QFrame *viewContent;
    QVBoxLayout *verticalLayout_5;
    QWidget *widget;
    QGridLayout *gridLayout_3;
    QTabWidget *tabWidget;
    QWidget *tab_queries;
    QVBoxLayout *verticalLayout;
    QGridLayout *gridLayout_queryButtons;
    QPushButton *backPushButton;
    QSpacerItem *horizontalSpacer;
    QPushButton *wikipediaPushButton;
    QPushButton *custom1PushButton;
    QPushButton *aavsoPushButton;
    QPushButton *ancientSkiesPushButton;
    QPushButton *gcvsPushButton;
    QPushButton *forwardPushButton;
    QPushButton *custom3PushButton;
    QPushButton *custom2PushButton;
    QWidget *tab_about;
    QVBoxLayout *verticalLayout_2;
    QTextBrowser *aboutTextBrowser;

    void setupUi(QWidget *onlineQueriesDialogForm)
    {
        if (onlineQueriesDialogForm->objectName().isEmpty())
            onlineQueriesDialogForm->setObjectName("onlineQueriesDialogForm");
        onlineQueriesDialogForm->resize(804, 450);
        QFont font;
        onlineQueriesDialogForm->setFont(font);
        onlineQueriesDialogForm->setAutoFillBackground(false);
        onlineQueriesDialogForm->setStyleSheet(QString::fromUtf8(""));
        vboxLayout = new QVBoxLayout(onlineQueriesDialogForm);
        vboxLayout->setSpacing(0);
        vboxLayout->setObjectName("vboxLayout");
        vboxLayout->setContentsMargins(0, 0, 0, 0);
        titleBar = new TitleBar(onlineQueriesDialogForm);
        titleBar->setObjectName("titleBar");

        vboxLayout->addWidget(titleBar);

        viewContent = new QFrame(onlineQueriesDialogForm);
        viewContent->setObjectName("viewContent");
        viewContent->setFrameShape(QFrame::StyledPanel);
        verticalLayout_5 = new QVBoxLayout(viewContent);
        verticalLayout_5->setSpacing(0);
        verticalLayout_5->setObjectName("verticalLayout_5");
        verticalLayout_5->setContentsMargins(0, 0, 0, 0);
        widget = new QWidget(viewContent);
        widget->setObjectName("widget");
        gridLayout_3 = new QGridLayout(widget);
        gridLayout_3->setSpacing(0);
        gridLayout_3->setObjectName("gridLayout_3");
        gridLayout_3->setContentsMargins(0, 0, 0, 0);
        tabWidget = new QTabWidget(widget);
        tabWidget->setObjectName("tabWidget");
        tabWidget->setEnabled(true);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(tabWidget->sizePolicy().hasHeightForWidth());
        tabWidget->setSizePolicy(sizePolicy);
        tab_queries = new QWidget();
        tab_queries->setObjectName("tab_queries");
        sizePolicy.setHeightForWidth(tab_queries->sizePolicy().hasHeightForWidth());
        tab_queries->setSizePolicy(sizePolicy);
        verticalLayout = new QVBoxLayout(tab_queries);
        verticalLayout->setObjectName("verticalLayout");
        gridLayout_queryButtons = new QGridLayout();
        gridLayout_queryButtons->setObjectName("gridLayout_queryButtons");
        backPushButton = new QPushButton(tab_queries);
        backPushButton->setObjectName("backPushButton");

        gridLayout_queryButtons->addWidget(backPushButton, 1, 0, 1, 1);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_queryButtons->addItem(horizontalSpacer, 0, 7, 1, 1);

        wikipediaPushButton = new QPushButton(tab_queries);
        wikipediaPushButton->setObjectName("wikipediaPushButton");
        wikipediaPushButton->setText(QString::fromUtf8("Wikipedia"));

        gridLayout_queryButtons->addWidget(wikipediaPushButton, 0, 0, 1, 1);

        custom1PushButton = new QPushButton(tab_queries);
        custom1PushButton->setObjectName("custom1PushButton");

        gridLayout_queryButtons->addWidget(custom1PushButton, 0, 4, 1, 1);

        aavsoPushButton = new QPushButton(tab_queries);
        aavsoPushButton->setObjectName("aavsoPushButton");
        aavsoPushButton->setText(QString::fromUtf8("AAVSO"));

        gridLayout_queryButtons->addWidget(aavsoPushButton, 0, 1, 1, 1);

        ancientSkiesPushButton = new QPushButton(tab_queries);
        ancientSkiesPushButton->setObjectName("ancientSkiesPushButton");
        ancientSkiesPushButton->setText(QString::fromUtf8("ASE"));

        gridLayout_queryButtons->addWidget(ancientSkiesPushButton, 0, 3, 1, 1);

        gcvsPushButton = new QPushButton(tab_queries);
        gcvsPushButton->setObjectName("gcvsPushButton");
        gcvsPushButton->setText(QString::fromUtf8("GCVS"));

        gridLayout_queryButtons->addWidget(gcvsPushButton, 0, 2, 1, 1);

        forwardPushButton = new QPushButton(tab_queries);
        forwardPushButton->setObjectName("forwardPushButton");

        gridLayout_queryButtons->addWidget(forwardPushButton, 1, 1, 1, 1);

        custom3PushButton = new QPushButton(tab_queries);
        custom3PushButton->setObjectName("custom3PushButton");

        gridLayout_queryButtons->addWidget(custom3PushButton, 0, 6, 1, 1);

        custom2PushButton = new QPushButton(tab_queries);
        custom2PushButton->setObjectName("custom2PushButton");

        gridLayout_queryButtons->addWidget(custom2PushButton, 0, 5, 1, 1);


        verticalLayout->addLayout(gridLayout_queryButtons);

        tabWidget->addTab(tab_queries, QString());
        tab_about = new QWidget();
        tab_about->setObjectName("tab_about");
        sizePolicy.setHeightForWidth(tab_about->sizePolicy().hasHeightForWidth());
        tab_about->setSizePolicy(sizePolicy);
        verticalLayout_2 = new QVBoxLayout(tab_about);
        verticalLayout_2->setObjectName("verticalLayout_2");
        aboutTextBrowser = new QTextBrowser(tab_about);
        aboutTextBrowser->setObjectName("aboutTextBrowser");
        sizePolicy.setHeightForWidth(aboutTextBrowser->sizePolicy().hasHeightForWidth());
        aboutTextBrowser->setSizePolicy(sizePolicy);
        aboutTextBrowser->setOpenExternalLinks(true);

        verticalLayout_2->addWidget(aboutTextBrowser);

        tabWidget->addTab(tab_about, QString());

        gridLayout_3->addWidget(tabWidget, 0, 2, 1, 1);


        verticalLayout_5->addWidget(widget);


        vboxLayout->addWidget(viewContent);


        retranslateUi(onlineQueriesDialogForm);

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(onlineQueriesDialogForm);
    } // setupUi

    void retranslateUi(QWidget *onlineQueriesDialogForm)
    {
        titleBar->setTitle(QCoreApplication::translate("onlineQueriesDialogForm", "Online Queries", nullptr));
        backPushButton->setText(QCoreApplication::translate("onlineQueriesDialogForm", "Back", nullptr));
        custom1PushButton->setText(QCoreApplication::translate("onlineQueriesDialogForm", "(Custom 1)", nullptr));
        forwardPushButton->setText(QCoreApplication::translate("onlineQueriesDialogForm", "Forward", nullptr));
        custom3PushButton->setText(QCoreApplication::translate("onlineQueriesDialogForm", "(Custom 3)", nullptr));
        custom2PushButton->setText(QCoreApplication::translate("onlineQueriesDialogForm", "(Custom 2)", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_queries), QCoreApplication::translate("onlineQueriesDialogForm", "Queries", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_about), QCoreApplication::translate("onlineQueriesDialogForm", "About", nullptr));
        (void)onlineQueriesDialogForm;
    } // retranslateUi

};

namespace Ui {
    class onlineQueriesDialogForm: public Ui_onlineQueriesDialogForm {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_ONLINEQUERIESDIALOG_H
