/****************************************************************************
** Meta object code from reading C++ file 'JapaneseCalendar.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../stellarium/plugins/Calendars/src/JapaneseCalendar.hpp"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'JapaneseCalendar.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16JapaneseCalendarE_t {};
} // unnamed namespace

template <> constexpr inline auto JapaneseCalendar::qt_create_metaobjectdata<qt_meta_tag_ZN16JapaneseCalendarE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JapaneseCalendar",
        "retranslate",
        "",
        "setJD",
        "JD",
        "setDate",
        "QList<int>",
        "parts5",
        "getDateStrings",
        "getFormattedDateString",
        "getSolarTermStrings",
        "std::pair<QString,QString>",
        "getFormattedSolarTermsString",
        "fixedFromJapanese",
        "japaneseFromFixed",
        "rd",
        "currentMajorSolarTerm",
        "japaneseLocation",
        "StelLocation",
        "rd_t",
        "solarLongitudeOnOrAfter",
        "lambda",
        "majorSolarTermOnOrAfter",
        "currentMinorSolarTerm",
        "minorSolarTermOnOrAfter",
        "midnightInJapan",
        "winterSolsticeOnOrBefore",
        "newMoonOnOrAfter",
        "newMoonBefore",
        "noMajorSolarTerm",
        "priorLeapMonth",
        "mP",
        "m",
        "newYearInSui",
        "newYearOnOrBefore",
        "japaneseYear",
        "cycle",
        "year"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'retranslate'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setJD'
        QtMocHelpers::SlotData<void(double)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 4 },
        }}),
        // Slot 'setDate'
        QtMocHelpers::SlotData<void(const QVector<int> &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'getDateStrings'
        QtMocHelpers::SlotData<QStringList() const>(8, 2, QMC::AccessPublic, QMetaType::QStringList),
        // Slot 'getFormattedDateString'
        QtMocHelpers::SlotData<QString() const>(9, 2, QMC::AccessPublic, QMetaType::QString),
        // Slot 'getSolarTermStrings'
        QtMocHelpers::SlotData<QPair<QString,QString>() const>(10, 2, QMC::AccessPublic, 0x80000000 | 11),
        // Slot 'getFormattedSolarTermsString'
        QtMocHelpers::SlotData<QString() const>(12, 2, QMC::AccessPublic, QMetaType::QString),
        // Slot 'fixedFromJapanese'
        QtMocHelpers::SlotData<int(const QVector<int> &)>(13, 2, QMC::AccessPublic, QMetaType::Int, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'japaneseFromFixed'
        QtMocHelpers::SlotData<QVector<int>(int)>(14, 2, QMC::AccessPublic, 0x80000000 | 6, {{
            { QMetaType::Int, 15 },
        }}),
        // Slot 'currentMajorSolarTerm'
        QtMocHelpers::SlotData<int(int)>(16, 2, QMC::AccessPublic, QMetaType::Int, {{
            { QMetaType::Int, 15 },
        }}),
        // Slot 'japaneseLocation'
        QtMocHelpers::SlotData<StelLocation(double)>(17, 2, QMC::AccessPublic, 0x80000000 | 18, {{
            { QMetaType::Double, 19 },
        }}),
        // Slot 'solarLongitudeOnOrAfter'
        QtMocHelpers::SlotData<double(double, double)>(20, 2, QMC::AccessPublic, QMetaType::Double, {{
            { QMetaType::Double, 21 }, { QMetaType::Double, 19 },
        }}),
        // Slot 'majorSolarTermOnOrAfter'
        QtMocHelpers::SlotData<int(int)>(22, 2, QMC::AccessPublic, QMetaType::Int, {{
            { QMetaType::Int, 15 },
        }}),
        // Slot 'currentMinorSolarTerm'
        QtMocHelpers::SlotData<int(int)>(23, 2, QMC::AccessPublic, QMetaType::Int, {{
            { QMetaType::Int, 15 },
        }}),
        // Slot 'minorSolarTermOnOrAfter'
        QtMocHelpers::SlotData<int(int)>(24, 2, QMC::AccessPublic, QMetaType::Int, {{
            { QMetaType::Int, 15 },
        }}),
        // Slot 'midnightInJapan'
        QtMocHelpers::SlotData<double(int)>(25, 2, QMC::AccessPublic, QMetaType::Double, {{
            { QMetaType::Int, 15 },
        }}),
        // Slot 'winterSolsticeOnOrBefore'
        QtMocHelpers::SlotData<int(int)>(26, 2, QMC::AccessPublic, QMetaType::Int, {{
            { QMetaType::Int, 15 },
        }}),
        // Slot 'newMoonOnOrAfter'
        QtMocHelpers::SlotData<int(int)>(27, 2, QMC::AccessPublic, QMetaType::Int, {{
            { QMetaType::Int, 15 },
        }}),
        // Slot 'newMoonBefore'
        QtMocHelpers::SlotData<int(int)>(28, 2, QMC::AccessPublic, QMetaType::Int, {{
            { QMetaType::Int, 15 },
        }}),
        // Slot 'noMajorSolarTerm'
        QtMocHelpers::SlotData<bool(int)>(29, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::Int, 15 },
        }}),
        // Slot 'priorLeapMonth'
        QtMocHelpers::SlotData<bool(int, int)>(30, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::Int, 31 }, { QMetaType::Int, 32 },
        }}),
        // Slot 'newYearInSui'
        QtMocHelpers::SlotData<int(int)>(33, 2, QMC::AccessPublic, QMetaType::Int, {{
            { QMetaType::Int, 15 },
        }}),
        // Slot 'newYearOnOrBefore'
        QtMocHelpers::SlotData<int(int)>(34, 2, QMC::AccessPublic, QMetaType::Int, {{
            { QMetaType::Int, 15 },
        }}),
        // Slot 'japaneseYear'
        QtMocHelpers::SlotData<int(int, int)>(35, 2, QMC::AccessPublic, QMetaType::Int, {{
            { QMetaType::Int, 36 }, { QMetaType::Int, 37 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JapaneseCalendar, qt_meta_tag_ZN16JapaneseCalendarE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject JapaneseCalendar::staticMetaObject = { {
    QMetaObject::SuperData::link<ChineseCalendar::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16JapaneseCalendarE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16JapaneseCalendarE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16JapaneseCalendarE_t>.metaTypes,
    nullptr
} };

void JapaneseCalendar::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JapaneseCalendar *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->retranslate(); break;
        case 1: _t->setJD((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 2: _t->setDate((*reinterpret_cast<std::add_pointer_t<QList<int>>>(_a[1]))); break;
        case 3: { QStringList _r = _t->getDateStrings();
            if (_a[0]) *reinterpret_cast<QStringList*>(_a[0]) = std::move(_r); }  break;
        case 4: { QString _r = _t->getFormattedDateString();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 5: { std::pair<QString,QString> _r = _t->getSolarTermStrings();
            if (_a[0]) *reinterpret_cast<std::pair<QString,QString>*>(_a[0]) = std::move(_r); }  break;
        case 6: { QString _r = _t->getFormattedSolarTermsString();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 7: { int _r = _t->fixedFromJapanese((*reinterpret_cast<std::add_pointer_t<QList<int>>>(_a[1])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 8: { QList<int> _r = _t->japaneseFromFixed((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QList<int>*>(_a[0]) = std::move(_r); }  break;
        case 9: { int _r = _t->currentMajorSolarTerm((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 10: { StelLocation _r = _t->japaneseLocation((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])));
            if (_a[0]) *reinterpret_cast<StelLocation*>(_a[0]) = std::move(_r); }  break;
        case 11: { double _r = _t->solarLongitudeOnOrAfter((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])));
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 12: { int _r = _t->majorSolarTermOnOrAfter((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 13: { int _r = _t->currentMinorSolarTerm((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 14: { int _r = _t->minorSolarTermOnOrAfter((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 15: { double _r = _t->midnightInJapan((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 16: { int _r = _t->winterSolsticeOnOrBefore((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 17: { int _r = _t->newMoonOnOrAfter((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 18: { int _r = _t->newMoonBefore((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 19: { bool _r = _t->noMajorSolarTerm((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 20: { bool _r = _t->priorLeapMonth((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 21: { int _r = _t->newYearInSui((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 22: { int _r = _t->newYearOnOrBefore((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 23: { int _r = _t->japaneseYear((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<int> >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<int> >(); break;
            }
            break;
        }
    }
}

const QMetaObject *JapaneseCalendar::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JapaneseCalendar::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16JapaneseCalendarE_t>.strings))
        return static_cast<void*>(this);
    return ChineseCalendar::qt_metacast(_clname);
}

int JapaneseCalendar::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ChineseCalendar::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 24)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 24;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 24)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 24;
    }
    return _id;
}
QT_WARNING_POP
