/********************************************************************************
** Form generated from reading UI file 'MSSearchDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MSSEARCHDIALOG_H
#define UI_MSSEARCHDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTreeWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "Dialog.hpp"
#include "gui/StelCloseButton.hpp"

QT_BEGIN_NAMESPACE

class Ui_MSSearchDialog
{
public:
    QVBoxLayout *verticalLayout_2;
    TitleBar *titleBar;
    QVBoxLayout *verticalLayout;
    QGroupBox *search;
    QHBoxLayout *horizontalLayout;
    QLabel *labelFrom;
    QSpinBox *fromYearSpinBox;
    QPushButton *searchButton;
    QGroupBox *results;
    QVBoxLayout *verticalLayout_5;
    QTreeWidget *listEvents;

    void setupUi(QWidget *MSSearchDialog)
    {
        if (MSSearchDialog->objectName().isEmpty())
            MSSearchDialog->setObjectName("MSSearchDialog");
        MSSearchDialog->resize(713, 449);
        verticalLayout_2 = new QVBoxLayout(MSSearchDialog);
        verticalLayout_2->setSpacing(0);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        titleBar = new TitleBar(MSSearchDialog);
        titleBar->setObjectName("titleBar");

        verticalLayout_2->addWidget(titleBar);

        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, -1, 0, 0);
        search = new QGroupBox(MSSearchDialog);
        search->setObjectName("search");
        search->setMaximumSize(QSize(16777215, 16777215));
        horizontalLayout = new QHBoxLayout(search);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        labelFrom = new QLabel(search);
        labelFrom->setObjectName("labelFrom");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(labelFrom->sizePolicy().hasHeightForWidth());
        labelFrom->setSizePolicy(sizePolicy);
        labelFrom->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        horizontalLayout->addWidget(labelFrom);

        fromYearSpinBox = new QSpinBox(search);
        fromYearSpinBox->setObjectName("fromYearSpinBox");
        fromYearSpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        fromYearSpinBox->setMinimum(-1000);
        fromYearSpinBox->setMaximum(5000);

        horizontalLayout->addWidget(fromYearSpinBox);

        searchButton = new QPushButton(search);
        searchButton->setObjectName("searchButton");
        searchButton->setCheckable(false);

        horizontalLayout->addWidget(searchButton);


        verticalLayout->addWidget(search);

        results = new QGroupBox(MSSearchDialog);
        results->setObjectName("results");
        verticalLayout_5 = new QVBoxLayout(results);
        verticalLayout_5->setSpacing(0);
        verticalLayout_5->setObjectName("verticalLayout_5");
        verticalLayout_5->setContentsMargins(0, 0, 0, 0);
        listEvents = new QTreeWidget(results);
        listEvents->setObjectName("listEvents");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::MinimumExpanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(listEvents->sizePolicy().hasHeightForWidth());
        listEvents->setSizePolicy(sizePolicy1);
        listEvents->setMinimumSize(QSize(0, 100));
        listEvents->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        listEvents->setEditTriggers(QAbstractItemView::NoEditTriggers);
        listEvents->setRootIsDecorated(false);
        listEvents->setUniformRowHeights(true);
        listEvents->setItemsExpandable(false);
        listEvents->setSortingEnabled(true);
        listEvents->setExpandsOnDoubleClick(false);
        listEvents->setColumnCount(0);
        listEvents->header()->setProperty("showSortIndicator", QVariant(true));

        verticalLayout_5->addWidget(listEvents);


        verticalLayout->addWidget(results);


        verticalLayout_2->addLayout(verticalLayout);


        retranslateUi(MSSearchDialog);

        searchButton->setDefault(false);


        QMetaObject::connectSlotsByName(MSSearchDialog);
    } // setupUi

    void retranslateUi(QWidget *MSSearchDialog)
    {
        MSSearchDialog->setWindowTitle(QCoreApplication::translate("MSSearchDialog", "Meteor Showers Configuration", nullptr));
        titleBar->setTitle(QCoreApplication::translate("MSSearchDialog", "Meteor Showers: Search", nullptr));
        search->setTitle(QCoreApplication::translate("MSSearchDialog", "Search", nullptr));
        labelFrom->setText(QCoreApplication::translate("MSSearchDialog", "Year:", nullptr));
        searchButton->setText(QCoreApplication::translate("MSSearchDialog", "Search", nullptr));
        results->setTitle(QCoreApplication::translate("MSSearchDialog", "Results", nullptr));
    } // retranslateUi

};

namespace Ui {
    class MSSearchDialog: public Ui_MSSearchDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_MSSEARCHDIALOG_H
