/****************************************************************************
** Meta object code from reading C++ file 'AstroCalcDialog.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../stellarium/src/gui/AstroCalcDialog.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'AstroCalcDialog.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15AstroCalcDialogE_t {};
} // unnamed namespace

template <> constexpr inline auto AstroCalcDialog::qt_create_metaobjectdata<qt_meta_tag_ZN15AstroCalcDialogE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "AstroCalcDialog",
        "graphDayChanged",
        "",
        "retranslate",
        "currentCelestialPositions",
        "populateCelestialCategoryList",
        "saveCelestialPositions",
        "selectCurrentCelestialPosition",
        "QModelIndex",
        "modelIndex",
        "currentHECPositions",
        "drawHECGraph",
        "selectedObject",
        "saveHECPositions",
        "selectCurrentHECPosition",
        "markCurrentHECPosition",
        "saveHECFlagMinorPlanets",
        "b",
        "saveHECFlagBrightComets",
        "saveHECBrightCometMagnitudeLimit",
        "mag",
        "saveCelestialPositionsMagnitudeLimit",
        "saveCelestialPositionsHorizontalCoordinatesFlag",
        "saveCelestialPositionsCategory",
        "index",
        "generateEphemeris",
        "cleanupEphemeris",
        "selectCurrentEphemeride",
        "saveEphemeris",
        "onChangedEphemerisPosition",
        "reGenerateEphemeris",
        "setDateTimeNow",
        "saveIgnoreDateTestFlag",
        "generateRTS",
        "cleanupRTS",
        "selectCurrentRTS",
        "QTreeWidgetItem*",
        "item",
        "idx",
        "saveRTS",
        "setRTSCelestialBodyName",
        "generateLunarEclipses",
        "cleanupLunarEclipses",
        "selectCurrentLunarEclipse",
        "selectCurrentLunarEclipseDate",
        "selectCurrentLunarEclipseContact",
        "saveLunarEclipses",
        "saveLunarEclipseCircumstances",
        "generateSolarEclipses",
        "cleanupSolarEclipses",
        "selectCurrentSolarEclipse",
        "selectCurrentSolarEclipseDate",
        "selectCurrentSolarEclipseContact",
        "saveSolarEclipses",
        "saveSolarEclipseCircumstances",
        "saveSolarEclipseMap",
        "local",
        "generateSolarEclipsesLocal",
        "cleanupSolarEclipsesLocal",
        "selectCurrentSolarEclipseLocal",
        "saveSolarEclipsesLocal",
        "generateTransits",
        "cleanupTransits",
        "selectCurrentTransit",
        "saveTransits",
        "saveEclipseFiltersState",
        "saveEphemerisCelestialBody",
        "saveEphemerisSecondaryCelestialBody",
        "saveEphemerisTimeStep",
        "saveEphemerisTimeUnit",
        "saveEphemerisTimeDuration",
        "duration",
        "initEphemerisFlagNakedEyePlanets",
        "saveEphemerisFlagNakedEyePlanets",
        "flag",
        "setMonthDuration",
        "updateGeneratedEphemeris",
        "calculatePhenomena",
        "cleanupPhenomena",
        "selectCurrentPhenomen",
        "savePhenomena",
        "savePhenomenaAngularSeparation",
        "populateGroupCelestialBodyList",
        "savePhenomenaCelestialBody",
        "savePhenomenaCelestialGroup",
        "savePhenomenaOppositionFlag",
        "savePhenomenaPerihelionAphelionFlag",
        "savePhenomenaElongationsQuadraturesFlag",
        "saveFirstCelestialBody",
        "saveSecondCelestialBody",
        "computePlanetaryData",
        "drawDistanceGraph",
        "drawLunarElongationGraph",
        "drawLunarElongationLimitLine",
        "saveLunarElongationLimit",
        "limit",
        "drawAltVsTimeDiagram",
        "drawCurrentTimeDiagram",
        "saveAltVsTimeSunFlag",
        "state",
        "saveAltVsTimeMoonFlag",
        "saveAltVsTimePositiveFlag",
        "saveAltVsTimePositiveLimit",
        "drawAziVsTimeDiagram",
        "handleVisibleEnabled",
        "saveGraphsCelestialBody",
        "saveGraphsFirstId",
        "saveGraphsSecondId",
        "updateGraphsDuration",
        "updateGraphsStep",
        "step",
        "drawXVsTimeGraphs",
        "updateXVsTimeGraphs",
        "drawMonthlyElevationGraph",
        "updateMonthlyElevationTime",
        "syncMonthlyElevationTime",
        "saveMonthlyElevationPositiveFlag",
        "saveMonthlyElevationPositiveLimit",
        "saveWutMagnitudeLimit",
        "saveWutMinAngularSizeLimit",
        "saveWutMaxAngularSizeLimit",
        "saveWutMinAltitude",
        "saveWutAngularSizeFlag",
        "saveWutTimeInterval",
        "calculateWutObjects",
        "selectWutObject",
        "saveWutObjects",
        "searchWutClear",
        "populateWutGroups",
        "updateAstroCalcData",
        "updateEquatorialData",
        "changePage",
        "QListWidgetItem*",
        "current",
        "previous",
        "changePCTab",
        "changeGraphsTab",
        "changeEclipsesTab",
        "changePositionsTab",
        "updateSolarSystemData",
        "populateCelestialNames",
        "showExtraEphemerisDialog",
        "showCustomStepsDialog",
        "updateAlmanacWidgetVisibility",
        "saveGraph",
        "QChartView*",
        "graph",
        "updateMinMaxDateRange",
        "populateSelectedObject",
        "selectStoredObject"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'graphDayChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'retranslate'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'currentCelestialPositions'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'populateCelestialCategoryList'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'saveCelestialPositions'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'selectCurrentCelestialPosition'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(7, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'currentHECPositions'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'drawHECGraph'
        QtMocHelpers::SlotData<void(const QString &)>(11, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 12 },
        }}),
        // Slot 'drawHECGraph'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPrivate | QMC::MethodCloned, QMetaType::Void),
        // Slot 'saveHECPositions'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'selectCurrentHECPosition'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(14, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'markCurrentHECPosition'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(15, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'saveHECFlagMinorPlanets'
        QtMocHelpers::SlotData<void(bool)>(16, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 17 },
        }}),
        // Slot 'saveHECFlagBrightComets'
        QtMocHelpers::SlotData<void(bool)>(18, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 17 },
        }}),
        // Slot 'saveHECBrightCometMagnitudeLimit'
        QtMocHelpers::SlotData<void(double)>(19, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 20 },
        }}),
        // Slot 'saveCelestialPositionsMagnitudeLimit'
        QtMocHelpers::SlotData<void(double)>(21, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 20 },
        }}),
        // Slot 'saveCelestialPositionsHorizontalCoordinatesFlag'
        QtMocHelpers::SlotData<void(bool)>(22, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 17 },
        }}),
        // Slot 'saveCelestialPositionsCategory'
        QtMocHelpers::SlotData<void(int)>(23, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 24 },
        }}),
        // Slot 'generateEphemeris'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'cleanupEphemeris'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'selectCurrentEphemeride'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(27, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'saveEphemeris'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onChangedEphemerisPosition'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'reGenerateEphemeris'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setDateTimeNow'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'saveIgnoreDateTestFlag'
        QtMocHelpers::SlotData<void(bool)>(32, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 17 },
        }}),
        // Slot 'generateRTS'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'cleanupRTS'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'selectCurrentRTS'
        QtMocHelpers::SlotData<void(QTreeWidgetItem *, int)>(35, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 36, 37 }, { QMetaType::Int, 38 },
        }}),
        // Slot 'saveRTS'
        QtMocHelpers::SlotData<void()>(39, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setRTSCelestialBodyName'
        QtMocHelpers::SlotData<void()>(40, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'generateLunarEclipses'
        QtMocHelpers::SlotData<void()>(41, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'cleanupLunarEclipses'
        QtMocHelpers::SlotData<void()>(42, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'selectCurrentLunarEclipse'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(43, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'selectCurrentLunarEclipseDate'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(44, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'selectCurrentLunarEclipseContact'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(45, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'saveLunarEclipses'
        QtMocHelpers::SlotData<void()>(46, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'saveLunarEclipseCircumstances'
        QtMocHelpers::SlotData<void()>(47, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'generateSolarEclipses'
        QtMocHelpers::SlotData<void()>(48, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'cleanupSolarEclipses'
        QtMocHelpers::SlotData<void()>(49, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'selectCurrentSolarEclipse'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(50, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'selectCurrentSolarEclipseDate'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(51, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'selectCurrentSolarEclipseContact'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(52, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'saveSolarEclipses'
        QtMocHelpers::SlotData<void()>(53, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'saveSolarEclipseCircumstances'
        QtMocHelpers::SlotData<void()>(54, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'saveSolarEclipseMap'
        QtMocHelpers::SlotData<void(bool)>(55, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 56 },
        }}),
        // Slot 'generateSolarEclipsesLocal'
        QtMocHelpers::SlotData<void()>(57, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'cleanupSolarEclipsesLocal'
        QtMocHelpers::SlotData<void()>(58, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'selectCurrentSolarEclipseLocal'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(59, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'saveSolarEclipsesLocal'
        QtMocHelpers::SlotData<void()>(60, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'generateTransits'
        QtMocHelpers::SlotData<void()>(61, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'cleanupTransits'
        QtMocHelpers::SlotData<void()>(62, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'selectCurrentTransit'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(63, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'saveTransits'
        QtMocHelpers::SlotData<void()>(64, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'saveEclipseFiltersState'
        QtMocHelpers::SlotData<void()>(65, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'saveEphemerisCelestialBody'
        QtMocHelpers::SlotData<void(int)>(66, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 24 },
        }}),
        // Slot 'saveEphemerisSecondaryCelestialBody'
        QtMocHelpers::SlotData<void(int)>(67, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 24 },
        }}),
        // Slot 'saveEphemerisTimeStep'
        QtMocHelpers::SlotData<void(int)>(68, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 24 },
        }}),
        // Slot 'saveEphemerisTimeUnit'
        QtMocHelpers::SlotData<void(int)>(69, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 24 },
        }}),
        // Slot 'saveEphemerisTimeDuration'
        QtMocHelpers::SlotData<void(int)>(70, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 71 },
        }}),
        // Slot 'initEphemerisFlagNakedEyePlanets'
        QtMocHelpers::SlotData<void()>(72, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'saveEphemerisFlagNakedEyePlanets'
        QtMocHelpers::SlotData<void(bool)>(73, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 74 },
        }}),
        // Slot 'setMonthDuration'
        QtMocHelpers::SlotData<void()>(75, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateGeneratedEphemeris'
        QtMocHelpers::SlotData<void()>(76, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'calculatePhenomena'
        QtMocHelpers::SlotData<void()>(77, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'cleanupPhenomena'
        QtMocHelpers::SlotData<void()>(78, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'selectCurrentPhenomen'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(79, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'savePhenomena'
        QtMocHelpers::SlotData<void()>(80, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'savePhenomenaAngularSeparation'
        QtMocHelpers::SlotData<void()>(81, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'populateGroupCelestialBodyList'
        QtMocHelpers::SlotData<void()>(82, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'savePhenomenaCelestialBody'
        QtMocHelpers::SlotData<void(int)>(83, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 24 },
        }}),
        // Slot 'savePhenomenaCelestialGroup'
        QtMocHelpers::SlotData<void(int)>(84, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 24 },
        }}),
        // Slot 'savePhenomenaOppositionFlag'
        QtMocHelpers::SlotData<void(bool)>(85, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 17 },
        }}),
        // Slot 'savePhenomenaPerihelionAphelionFlag'
        QtMocHelpers::SlotData<void(bool)>(86, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 17 },
        }}),
        // Slot 'savePhenomenaElongationsQuadraturesFlag'
        QtMocHelpers::SlotData<void(bool)>(87, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 17 },
        }}),
        // Slot 'saveFirstCelestialBody'
        QtMocHelpers::SlotData<void(int)>(88, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 24 },
        }}),
        // Slot 'saveSecondCelestialBody'
        QtMocHelpers::SlotData<void(int)>(89, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 24 },
        }}),
        // Slot 'computePlanetaryData'
        QtMocHelpers::SlotData<void()>(90, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'drawDistanceGraph'
        QtMocHelpers::SlotData<void()>(91, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'drawLunarElongationGraph'
        QtMocHelpers::SlotData<void()>(92, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'drawLunarElongationLimitLine'
        QtMocHelpers::SlotData<void()>(93, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'saveLunarElongationLimit'
        QtMocHelpers::SlotData<void(int)>(94, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 95 },
        }}),
        // Slot 'drawAltVsTimeDiagram'
        QtMocHelpers::SlotData<void()>(96, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'drawCurrentTimeDiagram'
        QtMocHelpers::SlotData<void()>(97, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'saveAltVsTimeSunFlag'
        QtMocHelpers::SlotData<void(bool)>(98, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 99 },
        }}),
        // Slot 'saveAltVsTimeMoonFlag'
        QtMocHelpers::SlotData<void(bool)>(100, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 99 },
        }}),
        // Slot 'saveAltVsTimePositiveFlag'
        QtMocHelpers::SlotData<void(bool)>(101, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 99 },
        }}),
        // Slot 'saveAltVsTimePositiveLimit'
        QtMocHelpers::SlotData<void(int)>(102, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 95 },
        }}),
        // Slot 'drawAziVsTimeDiagram'
        QtMocHelpers::SlotData<void()>(103, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'handleVisibleEnabled'
        QtMocHelpers::SlotData<void()>(104, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'saveGraphsCelestialBody'
        QtMocHelpers::SlotData<void(int)>(105, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 24 },
        }}),
        // Slot 'saveGraphsFirstId'
        QtMocHelpers::SlotData<void(int)>(106, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 24 },
        }}),
        // Slot 'saveGraphsSecondId'
        QtMocHelpers::SlotData<void(int)>(107, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 24 },
        }}),
        // Slot 'updateGraphsDuration'
        QtMocHelpers::SlotData<void(int)>(108, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 71 },
        }}),
        // Slot 'updateGraphsStep'
        QtMocHelpers::SlotData<void(int)>(109, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 110 },
        }}),
        // Slot 'drawXVsTimeGraphs'
        QtMocHelpers::SlotData<void()>(111, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateXVsTimeGraphs'
        QtMocHelpers::SlotData<void()>(112, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'drawMonthlyElevationGraph'
        QtMocHelpers::SlotData<void()>(113, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateMonthlyElevationTime'
        QtMocHelpers::SlotData<void()>(114, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'syncMonthlyElevationTime'
        QtMocHelpers::SlotData<void()>(115, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'saveMonthlyElevationPositiveFlag'
        QtMocHelpers::SlotData<void(bool)>(116, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 99 },
        }}),
        // Slot 'saveMonthlyElevationPositiveLimit'
        QtMocHelpers::SlotData<void(int)>(117, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 95 },
        }}),
        // Slot 'saveWutMagnitudeLimit'
        QtMocHelpers::SlotData<void(double)>(118, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 20 },
        }}),
        // Slot 'saveWutMinAngularSizeLimit'
        QtMocHelpers::SlotData<void()>(119, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'saveWutMaxAngularSizeLimit'
        QtMocHelpers::SlotData<void()>(120, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'saveWutMinAltitude'
        QtMocHelpers::SlotData<void()>(121, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'saveWutAngularSizeFlag'
        QtMocHelpers::SlotData<void(bool)>(122, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 99 },
        }}),
        // Slot 'saveWutTimeInterval'
        QtMocHelpers::SlotData<void(int)>(123, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 24 },
        }}),
        // Slot 'calculateWutObjects'
        QtMocHelpers::SlotData<void()>(124, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'selectWutObject'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(125, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 8, 24 },
        }}),
        // Slot 'saveWutObjects'
        QtMocHelpers::SlotData<void()>(126, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'searchWutClear'
        QtMocHelpers::SlotData<void()>(127, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'populateWutGroups'
        QtMocHelpers::SlotData<void()>(128, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateAstroCalcData'
        QtMocHelpers::SlotData<void()>(129, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateEquatorialData'
        QtMocHelpers::SlotData<void()>(130, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'changePage'
        QtMocHelpers::SlotData<void(QListWidgetItem *, QListWidgetItem *)>(131, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 132, 133 }, { 0x80000000 | 132, 134 },
        }}),
        // Slot 'changePCTab'
        QtMocHelpers::SlotData<void(int)>(135, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 24 },
        }}),
        // Slot 'changeGraphsTab'
        QtMocHelpers::SlotData<void(int)>(136, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 24 },
        }}),
        // Slot 'changeEclipsesTab'
        QtMocHelpers::SlotData<void(int)>(137, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 24 },
        }}),
        // Slot 'changePositionsTab'
        QtMocHelpers::SlotData<void(int)>(138, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 24 },
        }}),
        // Slot 'updateSolarSystemData'
        QtMocHelpers::SlotData<void()>(139, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'populateCelestialNames'
        QtMocHelpers::SlotData<void()>(140, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'showExtraEphemerisDialog'
        QtMocHelpers::SlotData<void()>(141, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'showCustomStepsDialog'
        QtMocHelpers::SlotData<void()>(142, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateAlmanacWidgetVisibility'
        QtMocHelpers::SlotData<void()>(143, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'saveGraph'
        QtMocHelpers::SlotData<void(QChartView *)>(144, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 145, 146 },
        }}),
        // Slot 'updateMinMaxDateRange'
        QtMocHelpers::SlotData<void()>(147, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'populateSelectedObject'
        QtMocHelpers::SlotData<void()>(148, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'selectStoredObject'
        QtMocHelpers::SlotData<void()>(149, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<AstroCalcDialog, qt_meta_tag_ZN15AstroCalcDialogE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject AstroCalcDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<StelDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15AstroCalcDialogE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15AstroCalcDialogE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15AstroCalcDialogE_t>.metaTypes,
    nullptr
} };

void AstroCalcDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AstroCalcDialog *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->graphDayChanged(); break;
        case 1: _t->retranslate(); break;
        case 2: _t->currentCelestialPositions(); break;
        case 3: _t->populateCelestialCategoryList(); break;
        case 4: _t->saveCelestialPositions(); break;
        case 5: _t->selectCurrentCelestialPosition((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 6: _t->currentHECPositions(); break;
        case 7: _t->drawHECGraph((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 8: _t->drawHECGraph(); break;
        case 9: _t->saveHECPositions(); break;
        case 10: _t->selectCurrentHECPosition((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 11: _t->markCurrentHECPosition((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 12: _t->saveHECFlagMinorPlanets((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 13: _t->saveHECFlagBrightComets((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 14: _t->saveHECBrightCometMagnitudeLimit((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 15: _t->saveCelestialPositionsMagnitudeLimit((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 16: _t->saveCelestialPositionsHorizontalCoordinatesFlag((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 17: _t->saveCelestialPositionsCategory((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 18: _t->generateEphemeris(); break;
        case 19: _t->cleanupEphemeris(); break;
        case 20: _t->selectCurrentEphemeride((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 21: _t->saveEphemeris(); break;
        case 22: _t->onChangedEphemerisPosition(); break;
        case 23: _t->reGenerateEphemeris(); break;
        case 24: _t->setDateTimeNow(); break;
        case 25: _t->saveIgnoreDateTestFlag((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 26: _t->generateRTS(); break;
        case 27: _t->cleanupRTS(); break;
        case 28: _t->selectCurrentRTS((*reinterpret_cast<std::add_pointer_t<QTreeWidgetItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 29: _t->saveRTS(); break;
        case 30: _t->setRTSCelestialBodyName(); break;
        case 31: _t->generateLunarEclipses(); break;
        case 32: _t->cleanupLunarEclipses(); break;
        case 33: _t->selectCurrentLunarEclipse((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 34: _t->selectCurrentLunarEclipseDate((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 35: _t->selectCurrentLunarEclipseContact((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 36: _t->saveLunarEclipses(); break;
        case 37: _t->saveLunarEclipseCircumstances(); break;
        case 38: _t->generateSolarEclipses(); break;
        case 39: _t->cleanupSolarEclipses(); break;
        case 40: _t->selectCurrentSolarEclipse((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 41: _t->selectCurrentSolarEclipseDate((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 42: _t->selectCurrentSolarEclipseContact((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 43: _t->saveSolarEclipses(); break;
        case 44: _t->saveSolarEclipseCircumstances(); break;
        case 45: _t->saveSolarEclipseMap((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 46: _t->generateSolarEclipsesLocal(); break;
        case 47: _t->cleanupSolarEclipsesLocal(); break;
        case 48: _t->selectCurrentSolarEclipseLocal((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 49: _t->saveSolarEclipsesLocal(); break;
        case 50: _t->generateTransits(); break;
        case 51: _t->cleanupTransits(); break;
        case 52: _t->selectCurrentTransit((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 53: _t->saveTransits(); break;
        case 54: _t->saveEclipseFiltersState(); break;
        case 55: _t->saveEphemerisCelestialBody((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 56: _t->saveEphemerisSecondaryCelestialBody((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 57: _t->saveEphemerisTimeStep((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 58: _t->saveEphemerisTimeUnit((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 59: _t->saveEphemerisTimeDuration((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 60: _t->initEphemerisFlagNakedEyePlanets(); break;
        case 61: _t->saveEphemerisFlagNakedEyePlanets((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 62: _t->setMonthDuration(); break;
        case 63: _t->updateGeneratedEphemeris(); break;
        case 64: _t->calculatePhenomena(); break;
        case 65: _t->cleanupPhenomena(); break;
        case 66: _t->selectCurrentPhenomen((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 67: _t->savePhenomena(); break;
        case 68: _t->savePhenomenaAngularSeparation(); break;
        case 69: _t->populateGroupCelestialBodyList(); break;
        case 70: _t->savePhenomenaCelestialBody((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 71: _t->savePhenomenaCelestialGroup((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 72: _t->savePhenomenaOppositionFlag((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 73: _t->savePhenomenaPerihelionAphelionFlag((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 74: _t->savePhenomenaElongationsQuadraturesFlag((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 75: _t->saveFirstCelestialBody((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 76: _t->saveSecondCelestialBody((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 77: _t->computePlanetaryData(); break;
        case 78: _t->drawDistanceGraph(); break;
        case 79: _t->drawLunarElongationGraph(); break;
        case 80: _t->drawLunarElongationLimitLine(); break;
        case 81: _t->saveLunarElongationLimit((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 82: _t->drawAltVsTimeDiagram(); break;
        case 83: _t->drawCurrentTimeDiagram(); break;
        case 84: _t->saveAltVsTimeSunFlag((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 85: _t->saveAltVsTimeMoonFlag((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 86: _t->saveAltVsTimePositiveFlag((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 87: _t->saveAltVsTimePositiveLimit((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 88: _t->drawAziVsTimeDiagram(); break;
        case 89: _t->handleVisibleEnabled(); break;
        case 90: _t->saveGraphsCelestialBody((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 91: _t->saveGraphsFirstId((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 92: _t->saveGraphsSecondId((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 93: _t->updateGraphsDuration((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 94: _t->updateGraphsStep((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 95: _t->drawXVsTimeGraphs(); break;
        case 96: _t->updateXVsTimeGraphs(); break;
        case 97: _t->drawMonthlyElevationGraph(); break;
        case 98: _t->updateMonthlyElevationTime(); break;
        case 99: _t->syncMonthlyElevationTime(); break;
        case 100: _t->saveMonthlyElevationPositiveFlag((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 101: _t->saveMonthlyElevationPositiveLimit((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 102: _t->saveWutMagnitudeLimit((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 103: _t->saveWutMinAngularSizeLimit(); break;
        case 104: _t->saveWutMaxAngularSizeLimit(); break;
        case 105: _t->saveWutMinAltitude(); break;
        case 106: _t->saveWutAngularSizeFlag((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 107: _t->saveWutTimeInterval((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 108: _t->calculateWutObjects(); break;
        case 109: _t->selectWutObject((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 110: _t->saveWutObjects(); break;
        case 111: _t->searchWutClear(); break;
        case 112: _t->populateWutGroups(); break;
        case 113: _t->updateAstroCalcData(); break;
        case 114: _t->updateEquatorialData(); break;
        case 115: _t->changePage((*reinterpret_cast<std::add_pointer_t<QListWidgetItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QListWidgetItem*>>(_a[2]))); break;
        case 116: _t->changePCTab((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 117: _t->changeGraphsTab((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 118: _t->changeEclipsesTab((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 119: _t->changePositionsTab((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 120: _t->updateSolarSystemData(); break;
        case 121: _t->populateCelestialNames(); break;
        case 122: _t->showExtraEphemerisDialog(); break;
        case 123: _t->showCustomStepsDialog(); break;
        case 124: _t->updateAlmanacWidgetVisibility(); break;
        case 125: _t->saveGraph((*reinterpret_cast<std::add_pointer_t<QChartView*>>(_a[1]))); break;
        case 126: _t->updateMinMaxDateRange(); break;
        case 127: _t->populateSelectedObject(); break;
        case 128: _t->selectStoredObject(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 125:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QChartView* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (AstroCalcDialog::*)()>(_a, &AstroCalcDialog::graphDayChanged, 0))
            return;
    }
}

const QMetaObject *AstroCalcDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *AstroCalcDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15AstroCalcDialogE_t>.strings))
        return static_cast<void*>(this);
    return StelDialog::qt_metacast(_clname);
}

int AstroCalcDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StelDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 129)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 129;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 129)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 129;
    }
    return _id;
}

// SIGNAL 0
void AstroCalcDialog::graphDayChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
