/****************************************************************************
** Meta object code from reading C++ file 'TelescopeControl.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../stellarium/plugins/TelescopeControl/src/TelescopeControl.hpp"
#include <QtCore/qmetatype.h>
#include <QtCore/qplugin.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'TelescopeControl.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16TelescopeControlE_t {};
} // unnamed namespace

template <> constexpr inline auto TelescopeControl::qt_create_metaobjectdata<qt_meta_tag_ZN16TelescopeControlE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TelescopeControl",
        "clientConnected",
        "",
        "slot",
        "name",
        "clientDisconnected",
        "flagTelescopeReticlesChanged",
        "b",
        "flagTelescopeLabelsChanged",
        "flagTelescopeCirclesChanged",
        "reticleColorChanged",
        "Vec3f",
        "c",
        "labelColorChanged",
        "circleColorChanged",
        "flagUseTelescopeServerLogsChanged",
        "useTelescopeServerExecutables",
        "getFlagUseServerExecutablesChanged",
        "serverExecutablesDirectoryPathChanged",
        "s",
        "setFlagTelescopeReticles",
        "getFlagTelescopeReticles",
        "setFlagTelescopeLabels",
        "getFlagTelescopeLabels",
        "setFlagTelescopeCircles",
        "getFlagTelescopeCircles",
        "setReticleColor",
        "getReticleColor",
        "getLabelColor",
        "setLabelColor",
        "setCircleColor",
        "getCircleColor",
        "setFontSize",
        "fontSize",
        "slewTelescopeToSelectedObject",
        "idx",
        "syncTelescopeWithSelectedObject",
        "slewTelescopeToViewDirection",
        "abortTelescopeSlew",
        "centeringScreenByTelescope",
        "setFlagUseTelescopeServerLogs",
        "translateActionDescriptions",
        "flagTelescopeReticles",
        "flagTelescopeLabels",
        "flagTelescopeCircles",
        "reticleColor",
        "labelColor",
        "circleColor",
        "useTelescopeServerLogs",
        "serverExecutablesDirectoryPath",
        "ConnectionType",
        "ConnectionNA",
        "ConnectionVirtual",
        "ConnectionInternal",
        "ConnectionLocal",
        "ConnectionRemote",
        "ConnectionRTS2",
        "ConnectionINDI",
        "ConnectionASCOM",
        "ConnectionTypeCount",
        "TelescopeStatus",
        "StatusNA",
        "StatusStarting",
        "StatusConnecting",
        "StatusConnected",
        "StatusDisconnected",
        "StatusStopped",
        "Equinox",
        "EquinoxJ2000",
        "EquinoxJNow"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'clientConnected'
        QtMocHelpers::SignalData<void(int, QString)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { QMetaType::QString, 4 },
        }}),
        // Signal 'clientDisconnected'
        QtMocHelpers::SignalData<void(int)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'flagTelescopeReticlesChanged'
        QtMocHelpers::SignalData<void(bool)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 7 },
        }}),
        // Signal 'flagTelescopeLabelsChanged'
        QtMocHelpers::SignalData<void(bool)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 7 },
        }}),
        // Signal 'flagTelescopeCirclesChanged'
        QtMocHelpers::SignalData<void(bool)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 7 },
        }}),
        // Signal 'reticleColorChanged'
        QtMocHelpers::SignalData<void(const Vec3f &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Signal 'labelColorChanged'
        QtMocHelpers::SignalData<void(const Vec3f &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Signal 'circleColorChanged'
        QtMocHelpers::SignalData<void(const Vec3f &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Signal 'flagUseTelescopeServerLogsChanged'
        QtMocHelpers::SignalData<void(bool)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 7 },
        }}),
        // Signal 'useTelescopeServerExecutables'
        QtMocHelpers::SignalData<void(bool)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 7 },
        }}),
        // Signal 'getFlagUseServerExecutablesChanged'
        QtMocHelpers::SignalData<void(bool)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 7 },
        }}),
        // Signal 'serverExecutablesDirectoryPathChanged'
        QtMocHelpers::SignalData<void(const QString &)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 19 },
        }}),
        // Slot 'setFlagTelescopeReticles'
        QtMocHelpers::SlotData<void(bool)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 7 },
        }}),
        // Slot 'getFlagTelescopeReticles'
        QtMocHelpers::SlotData<bool() const>(21, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFlagTelescopeLabels'
        QtMocHelpers::SlotData<void(bool)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 7 },
        }}),
        // Slot 'getFlagTelescopeLabels'
        QtMocHelpers::SlotData<bool() const>(23, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFlagTelescopeCircles'
        QtMocHelpers::SlotData<void(bool)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 7 },
        }}),
        // Slot 'getFlagTelescopeCircles'
        QtMocHelpers::SlotData<bool() const>(25, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setReticleColor'
        QtMocHelpers::SlotData<void(const Vec3f &)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'getReticleColor'
        QtMocHelpers::SlotData<void() const>(27, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'getLabelColor'
        QtMocHelpers::SlotData<void() const>(28, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setLabelColor'
        QtMocHelpers::SlotData<void(const Vec3f &)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'setCircleColor'
        QtMocHelpers::SlotData<void(const Vec3f &)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'getCircleColor'
        QtMocHelpers::SlotData<void() const>(31, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setFontSize'
        QtMocHelpers::SlotData<void(int)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 33 },
        }}),
        // Slot 'slewTelescopeToSelectedObject'
        QtMocHelpers::SlotData<void(const int)>(34, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 35 },
        }}),
        // Slot 'syncTelescopeWithSelectedObject'
        QtMocHelpers::SlotData<void(const int)>(36, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 35 },
        }}),
        // Slot 'slewTelescopeToViewDirection'
        QtMocHelpers::SlotData<void(const int)>(37, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 35 },
        }}),
        // Slot 'abortTelescopeSlew'
        QtMocHelpers::SlotData<void(const int)>(38, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 35 },
        }}),
        // Slot 'centeringScreenByTelescope'
        QtMocHelpers::SlotData<void(const int)>(39, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 35 },
        }}),
        // Slot 'setFlagUseTelescopeServerLogs'
        QtMocHelpers::SlotData<void(bool)>(40, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 7 },
        }}),
        // Slot 'translateActionDescriptions'
        QtMocHelpers::SlotData<void()>(41, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'flagTelescopeReticles'
        QtMocHelpers::PropertyData<bool>(42, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'flagTelescopeLabels'
        QtMocHelpers::PropertyData<bool>(43, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'flagTelescopeCircles'
        QtMocHelpers::PropertyData<bool>(44, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'reticleColor'
        QtMocHelpers::PropertyData<Vec3f>(45, 0x80000000 | 11, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 5),
        // property 'labelColor'
        QtMocHelpers::PropertyData<Vec3f>(46, 0x80000000 | 11, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 6),
        // property 'circleColor'
        QtMocHelpers::PropertyData<Vec3f>(47, 0x80000000 | 11, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 7),
        // property 'useTelescopeServerLogs'
        QtMocHelpers::PropertyData<bool>(48, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 8),
        // property 'useTelescopeServerExecutables'
        QtMocHelpers::PropertyData<bool>(16, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 10),
        // property 'serverExecutablesDirectoryPath'
        QtMocHelpers::PropertyData<QString>(49, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 11),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'ConnectionType'
        QtMocHelpers::EnumData<enum ConnectionType>(50, 50, QMC::EnumFlags{}).add({
            {   51, ConnectionType::ConnectionNA },
            {   52, ConnectionType::ConnectionVirtual },
            {   53, ConnectionType::ConnectionInternal },
            {   54, ConnectionType::ConnectionLocal },
            {   55, ConnectionType::ConnectionRemote },
            {   56, ConnectionType::ConnectionRTS2 },
            {   57, ConnectionType::ConnectionINDI },
            {   58, ConnectionType::ConnectionASCOM },
            {   59, ConnectionType::ConnectionTypeCount },
        }),
        // enum 'TelescopeStatus'
        QtMocHelpers::EnumData<enum TelescopeStatus>(60, 60, QMC::EnumFlags{}).add({
            {   61, TelescopeStatus::StatusNA },
            {   62, TelescopeStatus::StatusStarting },
            {   63, TelescopeStatus::StatusConnecting },
            {   64, TelescopeStatus::StatusConnected },
            {   65, TelescopeStatus::StatusDisconnected },
            {   66, TelescopeStatus::StatusStopped },
        }),
        // enum 'Equinox'
        QtMocHelpers::EnumData<enum Equinox>(67, 67, QMC::EnumFlags{}).add({
            {   68, Equinox::EquinoxJ2000 },
            {   69, Equinox::EquinoxJNow },
        }),
    };
    return QtMocHelpers::metaObjectData<TelescopeControl, qt_meta_tag_ZN16TelescopeControlE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject TelescopeControl::staticMetaObject = { {
    QMetaObject::SuperData::link<StelObjectModule::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16TelescopeControlE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16TelescopeControlE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16TelescopeControlE_t>.metaTypes,
    nullptr
} };

void TelescopeControl::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TelescopeControl *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->clientConnected((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 1: _t->clientDisconnected((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->flagTelescopeReticlesChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->flagTelescopeLabelsChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->flagTelescopeCirclesChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->reticleColorChanged((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1]))); break;
        case 6: _t->labelColorChanged((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1]))); break;
        case 7: _t->circleColorChanged((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1]))); break;
        case 8: _t->flagUseTelescopeServerLogsChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 9: _t->useTelescopeServerExecutables((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 10: _t->getFlagUseServerExecutablesChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 11: _t->serverExecutablesDirectoryPathChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 12: _t->setFlagTelescopeReticles((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 13: { bool _r = _t->getFlagTelescopeReticles();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 14: _t->setFlagTelescopeLabels((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 15: { bool _r = _t->getFlagTelescopeLabels();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 16: _t->setFlagTelescopeCircles((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 17: { bool _r = _t->getFlagTelescopeCircles();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 18: _t->setReticleColor((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1]))); break;
        case 19: _t->getReticleColor(); break;
        case 20: _t->getLabelColor(); break;
        case 21: _t->setLabelColor((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1]))); break;
        case 22: _t->setCircleColor((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1]))); break;
        case 23: _t->getCircleColor(); break;
        case 24: _t->setFontSize((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 25: _t->slewTelescopeToSelectedObject((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 26: _t->syncTelescopeWithSelectedObject((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 27: _t->slewTelescopeToViewDirection((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 28: _t->abortTelescopeSlew((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 29: _t->centeringScreenByTelescope((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 30: _t->setFlagUseTelescopeServerLogs((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 31: _t->translateActionDescriptions(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 18:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 21:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 22:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TelescopeControl::*)(int , QString )>(_a, &TelescopeControl::clientConnected, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (TelescopeControl::*)(int )>(_a, &TelescopeControl::clientDisconnected, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (TelescopeControl::*)(bool )>(_a, &TelescopeControl::flagTelescopeReticlesChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (TelescopeControl::*)(bool )>(_a, &TelescopeControl::flagTelescopeLabelsChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (TelescopeControl::*)(bool )>(_a, &TelescopeControl::flagTelescopeCirclesChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (TelescopeControl::*)(const Vec3f & )>(_a, &TelescopeControl::reticleColorChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (TelescopeControl::*)(const Vec3f & )>(_a, &TelescopeControl::labelColorChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (TelescopeControl::*)(const Vec3f & )>(_a, &TelescopeControl::circleColorChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (TelescopeControl::*)(bool )>(_a, &TelescopeControl::flagUseTelescopeServerLogsChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (TelescopeControl::*)(bool )>(_a, &TelescopeControl::useTelescopeServerExecutables, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (TelescopeControl::*)(bool )>(_a, &TelescopeControl::getFlagUseServerExecutablesChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (TelescopeControl::*)(const QString & )>(_a, &TelescopeControl::serverExecutablesDirectoryPathChanged, 11))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 5:
        case 4:
        case 3:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Vec3f >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->getFlagTelescopeReticles(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->getFlagTelescopeLabels(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->getFlagTelescopeCircles(); break;
        case 3: *reinterpret_cast<Vec3f*>(_v) = _t->getReticleColor(); break;
        case 4: *reinterpret_cast<Vec3f*>(_v) = _t->getLabelColor(); break;
        case 5: *reinterpret_cast<Vec3f*>(_v) = _t->getCircleColor(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->getFlagUseTelescopeServerLogs(); break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->getFlagUseServerExecutables(); break;
        case 8: *reinterpret_cast<QString*>(_v) = _t->getServerExecutablesDirectoryPath(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setFlagTelescopeReticles(*reinterpret_cast<bool*>(_v)); break;
        case 1: _t->setFlagTelescopeLabels(*reinterpret_cast<bool*>(_v)); break;
        case 2: _t->setFlagTelescopeCircles(*reinterpret_cast<bool*>(_v)); break;
        case 3: _t->setReticleColor(*reinterpret_cast<Vec3f*>(_v)); break;
        case 4: _t->setLabelColor(*reinterpret_cast<Vec3f*>(_v)); break;
        case 5: _t->setCircleColor(*reinterpret_cast<Vec3f*>(_v)); break;
        case 6: _t->setFlagUseTelescopeServerLogs(*reinterpret_cast<bool*>(_v)); break;
        case 7: _t->setFlagUseServerExecutables(*reinterpret_cast<bool*>(_v)); break;
        case 8: _t->setServerExecutablesDirectoryPath(*reinterpret_cast<QString*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *TelescopeControl::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TelescopeControl::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16TelescopeControlE_t>.strings))
        return static_cast<void*>(this);
    return StelObjectModule::qt_metacast(_clname);
}

int TelescopeControl::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StelObjectModule::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 32)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 32;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 32)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 32;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void TelescopeControl::clientConnected(int _t1, QString _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void TelescopeControl::clientDisconnected(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void TelescopeControl::flagTelescopeReticlesChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void TelescopeControl::flagTelescopeLabelsChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void TelescopeControl::flagTelescopeCirclesChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void TelescopeControl::reticleColorChanged(const Vec3f & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void TelescopeControl::labelColorChanged(const Vec3f & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void TelescopeControl::circleColorChanged(const Vec3f & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void TelescopeControl::flagUseTelescopeServerLogsChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void TelescopeControl::useTelescopeServerExecutables(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void TelescopeControl::getFlagUseServerExecutablesChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void TelescopeControl::serverExecutablesDirectoryPathChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN35TelescopeControlStelPluginInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto TelescopeControlStelPluginInterface::qt_create_metaobjectdata<qt_meta_tag_ZN35TelescopeControlStelPluginInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TelescopeControlStelPluginInterface"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TelescopeControlStelPluginInterface, qt_meta_tag_ZN35TelescopeControlStelPluginInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject TelescopeControlStelPluginInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN35TelescopeControlStelPluginInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN35TelescopeControlStelPluginInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN35TelescopeControlStelPluginInterfaceE_t>.metaTypes,
    nullptr
} };

void TelescopeControlStelPluginInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TelescopeControlStelPluginInterface *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *TelescopeControlStelPluginInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TelescopeControlStelPluginInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN35TelescopeControlStelPluginInterfaceE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "StelPluginInterface"))
        return static_cast< StelPluginInterface*>(this);
    if (!strcmp(_clname, "org.stellarium.StelPluginInterface/2.0"))
        return static_cast< StelPluginInterface*>(this);
    return QObject::qt_metacast(_clname);
}

int TelescopeControlStelPluginInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}

#ifdef QT_MOC_EXPORT_PLUGIN_V2
static constexpr unsigned char qt_pluginMetaDataV2_TelescopeControlStelPluginInterface[] = {
    0xbf, 
    // "IID"
    0x02,  0x78,  0x26,  'o',  'r',  'g',  '.',  's', 
    't',  'e',  'l',  'l',  'a',  'r',  'i',  'u', 
    'm',  '.',  'S',  't',  'e',  'l',  'P',  'l', 
    'u',  'g',  'i',  'n',  'I',  'n',  't',  'e', 
    'r',  'f',  'a',  'c',  'e',  '/',  '2',  '.', 
    '0', 
    // "className"
    0x03,  0x78,  0x23,  'T',  'e',  'l',  'e',  's', 
    'c',  'o',  'p',  'e',  'C',  'o',  'n',  't', 
    'r',  'o',  'l',  'S',  't',  'e',  'l',  'P', 
    'l',  'u',  'g',  'i',  'n',  'I',  'n',  't', 
    'e',  'r',  'f',  'a',  'c',  'e', 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN_V2(TelescopeControlStelPluginInterface, TelescopeControlStelPluginInterface, qt_pluginMetaDataV2_TelescopeControlStelPluginInterface)
#else
QT_PLUGIN_METADATA_SECTION
Q_CONSTINIT static constexpr unsigned char qt_pluginMetaData_TelescopeControlStelPluginInterface[] = {
    'Q', 'T', 'M', 'E', 'T', 'A', 'D', 'A', 'T', 'A', ' ', '!',
    // metadata version, Qt version, architectural requirements
    0, QT_VERSION_MAJOR, QT_VERSION_MINOR, qPluginArchRequirements(),
    0xbf, 
    // "IID"
    0x02,  0x78,  0x26,  'o',  'r',  'g',  '.',  's', 
    't',  'e',  'l',  'l',  'a',  'r',  'i',  'u', 
    'm',  '.',  'S',  't',  'e',  'l',  'P',  'l', 
    'u',  'g',  'i',  'n',  'I',  'n',  't',  'e', 
    'r',  'f',  'a',  'c',  'e',  '/',  '2',  '.', 
    '0', 
    // "className"
    0x03,  0x78,  0x23,  'T',  'e',  'l',  'e',  's', 
    'c',  'o',  'p',  'e',  'C',  'o',  'n',  't', 
    'r',  'o',  'l',  'S',  't',  'e',  'l',  'P', 
    'l',  'u',  'g',  'i',  'n',  'I',  'n',  't', 
    'e',  'r',  'f',  'a',  'c',  'e', 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN(TelescopeControlStelPluginInterface, TelescopeControlStelPluginInterface)
#endif  // QT_MOC_EXPORT_PLUGIN_V2

QT_WARNING_POP
