/****************************************************************************
** Meta object code from reading C++ file 'ShortcutsDialog.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../stellarium/src/gui/ShortcutsDialog.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ShortcutsDialog.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN20ShortcutsFilterModelE_t {};
} // unnamed namespace

template <> constexpr inline auto ShortcutsFilterModel::qt_create_metaobjectdata<qt_meta_tag_ZN20ShortcutsFilterModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ShortcutsFilterModel"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ShortcutsFilterModel, qt_meta_tag_ZN20ShortcutsFilterModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ShortcutsFilterModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QSortFilterProxyModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20ShortcutsFilterModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20ShortcutsFilterModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN20ShortcutsFilterModelE_t>.metaTypes,
    nullptr
} };

void ShortcutsFilterModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ShortcutsFilterModel *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *ShortcutsFilterModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ShortcutsFilterModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20ShortcutsFilterModelE_t>.strings))
        return static_cast<void*>(this);
    return QSortFilterProxyModel::qt_metacast(_clname);
}

int ShortcutsFilterModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QSortFilterProxyModel::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN15ShortcutsDialogE_t {};
} // unnamed namespace

template <> constexpr inline auto ShortcutsDialog::qt_create_metaobjectdata<qt_meta_tag_ZN15ShortcutsDialogE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ShortcutsDialog",
        "resetCollisions",
        "",
        "retranslate",
        "initEditors",
        "prefixMatchKeySequence",
        "QKeySequence",
        "ks1",
        "ks2",
        "findCollidingItems",
        "QList<QStandardItem*>",
        "ks",
        "handleCollisions",
        "ShortcutLineEdit*",
        "currentEdit",
        "handleChanges",
        "applyChanges",
        "switchToEditors",
        "QModelIndex",
        "index",
        "updateShortcutsItem",
        "StelAction*",
        "action",
        "QStandardItem*",
        "shortcutItem",
        "restoreDefaultShortcuts",
        "restoreAllDefaultShortcuts",
        "updateTreeData"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'resetCollisions'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'retranslate'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'initEditors'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'prefixMatchKeySequence'
        QtMocHelpers::SlotData<bool(const QKeySequence &, const QKeySequence &)>(5, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 6, 7 }, { 0x80000000 | 6, 8 },
        }}),
        // Slot 'findCollidingItems'
        QtMocHelpers::SlotData<QList<QStandardItem*>(QKeySequence)>(9, 2, QMC::AccessPublic, 0x80000000 | 10, {{
            { 0x80000000 | 6, 11 },
        }}),
        // Slot 'handleCollisions'
        QtMocHelpers::SlotData<void(ShortcutLineEdit *)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
        // Slot 'handleChanges'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'applyChanges'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'switchToEditors'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 18, 19 },
        }}),
        // Slot 'updateShortcutsItem'
        QtMocHelpers::SlotData<void(StelAction *, QStandardItem *)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 21, 22 }, { 0x80000000 | 23, 24 },
        }}),
        // Slot 'updateShortcutsItem'
        QtMocHelpers::SlotData<void(StelAction *)>(20, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 21, 22 },
        }}),
        // Slot 'restoreDefaultShortcuts'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'restoreAllDefaultShortcuts'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'updateTreeData'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ShortcutsDialog, qt_meta_tag_ZN15ShortcutsDialogE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ShortcutsDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<StelDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15ShortcutsDialogE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15ShortcutsDialogE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15ShortcutsDialogE_t>.metaTypes,
    nullptr
} };

void ShortcutsDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ShortcutsDialog *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->resetCollisions(); break;
        case 1: _t->retranslate(); break;
        case 2: _t->initEditors(); break;
        case 3: { bool _r = _t->prefixMatchKeySequence((*reinterpret_cast<std::add_pointer_t<QKeySequence>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QKeySequence>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 4: { QList<QStandardItem*> _r = _t->findCollidingItems((*reinterpret_cast<std::add_pointer_t<QKeySequence>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QList<QStandardItem*>*>(_a[0]) = std::move(_r); }  break;
        case 5: _t->handleCollisions((*reinterpret_cast<std::add_pointer_t<ShortcutLineEdit*>>(_a[1]))); break;
        case 6: _t->handleChanges(); break;
        case 7: _t->applyChanges(); break;
        case 8: _t->switchToEditors((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 9: _t->updateShortcutsItem((*reinterpret_cast<std::add_pointer_t<StelAction*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QStandardItem*>>(_a[2]))); break;
        case 10: _t->updateShortcutsItem((*reinterpret_cast<std::add_pointer_t<StelAction*>>(_a[1]))); break;
        case 11: _t->restoreDefaultShortcuts(); break;
        case 12: _t->restoreAllDefaultShortcuts(); break;
        case 13: _t->updateTreeData(); break;
        default: ;
        }
    }
}

const QMetaObject *ShortcutsDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ShortcutsDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15ShortcutsDialogE_t>.strings))
        return static_cast<void*>(this);
    return StelDialog::qt_metacast(_clname);
}

int ShortcutsDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StelDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 14;
    }
    return _id;
}
QT_WARNING_POP
