/********************************************************************************
** Form generated from reading UI file 'TelescopeClientINDIWidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TELESCOPECLIENTINDIWIDGET_H
#define UI_TELESCOPECLIENTINDIWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_TelescopeClientINDIWidget
{
public:
    QVBoxLayout *verticalLayout;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout_2;
    QGridLayout *serverSettings;
    QLabel *labelTCPPort;
    QSpinBox *spinBoxTCPPort;
    QLineEdit *lineEditHostName;
    QLabel *labelHost;
    QPushButton *connectButton;
    QLabel *label;
    QComboBox *devicesComboBox;

    void setupUi(QWidget *TelescopeClientINDIWidget)
    {
        if (TelescopeClientINDIWidget->objectName().isEmpty())
            TelescopeClientINDIWidget->setObjectName("TelescopeClientINDIWidget");
        TelescopeClientINDIWidget->resize(609, 176);
        TelescopeClientINDIWidget->setWindowTitle(QString::fromUtf8("Form"));
        verticalLayout = new QVBoxLayout(TelescopeClientINDIWidget);
        verticalLayout->setSpacing(0);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        groupBox = new QGroupBox(TelescopeClientINDIWidget);
        groupBox->setObjectName("groupBox");
        verticalLayout_2 = new QVBoxLayout(groupBox);
        verticalLayout_2->setSpacing(6);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        serverSettings = new QGridLayout();
        serverSettings->setSpacing(6);
        serverSettings->setObjectName("serverSettings");
        labelTCPPort = new QLabel(groupBox);
        labelTCPPort->setObjectName("labelTCPPort");
        labelTCPPort->setEnabled(true);

        serverSettings->addWidget(labelTCPPort, 0, 1, 1, 1);

        spinBoxTCPPort = new QSpinBox(groupBox);
        spinBoxTCPPort->setObjectName("spinBoxTCPPort");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(spinBoxTCPPort->sizePolicy().hasHeightForWidth());
        spinBoxTCPPort->setSizePolicy(sizePolicy);
        spinBoxTCPPort->setMinimum(1024);
        spinBoxTCPPort->setMaximum(65535);
        spinBoxTCPPort->setValue(7624);

        serverSettings->addWidget(spinBoxTCPPort, 1, 1, 1, 1);

        lineEditHostName = new QLineEdit(groupBox);
        lineEditHostName->setObjectName("lineEditHostName");
        lineEditHostName->setEnabled(true);
        lineEditHostName->setText(QString::fromUtf8("localhost"));
        lineEditHostName->setMaxLength(255);

        serverSettings->addWidget(lineEditHostName, 1, 0, 1, 1);

        labelHost = new QLabel(groupBox);
        labelHost->setObjectName("labelHost");
        labelHost->setEnabled(true);

        serverSettings->addWidget(labelHost, 0, 0, 1, 1);


        verticalLayout_2->addLayout(serverSettings);

        connectButton = new QPushButton(groupBox);
        connectButton->setObjectName("connectButton");
        connectButton->setCheckable(true);
        connectButton->setChecked(false);

        verticalLayout_2->addWidget(connectButton);

        label = new QLabel(groupBox);
        label->setObjectName("label");

        verticalLayout_2->addWidget(label);

        devicesComboBox = new QComboBox(groupBox);
        devicesComboBox->setObjectName("devicesComboBox");
        devicesComboBox->setEditable(false);
        devicesComboBox->setInsertPolicy(QComboBox::NoInsert);

        verticalLayout_2->addWidget(devicesComboBox);


        verticalLayout->addWidget(groupBox);


        retranslateUi(TelescopeClientINDIWidget);

        QMetaObject::connectSlotsByName(TelescopeClientINDIWidget);
    } // setupUi

    void retranslateUi(QWidget *TelescopeClientINDIWidget)
    {
        groupBox->setTitle(QCoreApplication::translate("TelescopeClientINDIWidget", "INDI Settings", nullptr));
        labelTCPPort->setText(QCoreApplication::translate("TelescopeClientINDIWidget", "TCP port:", nullptr));
#if QT_CONFIG(tooltip)
        lineEditHostName->setToolTip(QCoreApplication::translate("TelescopeClientINDIWidget", "Host name or IPv4 address of the machine that hosts the telescope server", nullptr));
#endif // QT_CONFIG(tooltip)
        labelHost->setText(QCoreApplication::translate("TelescopeClientINDIWidget", "Host:", nullptr));
        connectButton->setText(QCoreApplication::translate("TelescopeClientINDIWidget", "Refresh devices", nullptr));
        label->setText(QCoreApplication::translate("TelescopeClientINDIWidget", "Devices:", nullptr));
        (void)TelescopeClientINDIWidget;
    } // retranslateUi

};

namespace Ui {
    class TelescopeClientINDIWidget: public Ui_TelescopeClientINDIWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_TELESCOPECLIENTINDIWIDGET_H
