/********************************************************************************
** Form generated from reading UI file 'helpDialogGui.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_HELPDIALOGGUI_H
#define UI_HELPDIALOGGUI_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QTextBrowser>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "Dialog.hpp"
#include "gui/StelCloseButton.hpp"

QT_BEGIN_NAMESPACE

class Ui_helpDialogForm
{
public:
    QVBoxLayout *verticalLayout_3;
    TitleBar *titleBar;
    QFrame *viewContent;
    QGridLayout *gridLayout;
    QListWidget *stackListWidget;
    QStackedWidget *stackedWidget;
    QWidget *pageHelp;
    QVBoxLayout *verticalLayout_5;
    QTextBrowser *helpBrowser;
    QPushButton *editShortcutsButton;
    QWidget *pageAbout;
    QVBoxLayout *verticalLayout_2;
    QTextBrowser *aboutBrowser;
    QPushButton *checkUpdatesButton;
    QWidget *pageLog;
    QVBoxLayout *verticalLayout;
    QLabel *logPathLabel;
    QTextBrowser *logBrowser;
    QPushButton *refreshButton;
    QWidget *pageConfig;
    QVBoxLayout *verticalLayout_4;
    QLabel *configPathLabel;
    QTextBrowser *configBrowser;
    QPushButton *refreshConfigButton;

    void setupUi(QWidget *helpDialogForm)
    {
        if (helpDialogForm->objectName().isEmpty())
            helpDialogForm->setObjectName("helpDialogForm");
        helpDialogForm->resize(476, 441);
        helpDialogForm->setMinimumSize(QSize(250, 250));
        helpDialogForm->setStyleSheet(QString::fromUtf8(""));
        verticalLayout_3 = new QVBoxLayout(helpDialogForm);
        verticalLayout_3->setSpacing(0);
        verticalLayout_3->setObjectName("verticalLayout_3");
        verticalLayout_3->setContentsMargins(0, 0, 0, 0);
        titleBar = new TitleBar(helpDialogForm);
        titleBar->setObjectName("titleBar");

        verticalLayout_3->addWidget(titleBar);

        viewContent = new QFrame(helpDialogForm);
        viewContent->setObjectName("viewContent");
        viewContent->setFrameShape(QFrame::StyledPanel);
        gridLayout = new QGridLayout(viewContent);
        gridLayout->setSpacing(0);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setContentsMargins(0, 0, 0, 0);
        stackListWidget = new QListWidget(viewContent);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/graphicGui/tabHelp.png"), QSize(), QIcon::Mode::Normal, QIcon::State::On);
        icon.addFile(QString::fromUtf8(":/graphicGui/tabHelp-selected.png"), QSize(), QIcon::Mode::Selected, QIcon::State::On);
        QListWidgetItem *__qlistwidgetitem = new QListWidgetItem(stackListWidget);
        __qlistwidgetitem->setIcon(icon);
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/graphicGui/tabInfo.png"), QSize(), QIcon::Mode::Normal, QIcon::State::On);
        icon1.addFile(QString::fromUtf8(":/graphicGui/tabInfo-selected.png"), QSize(), QIcon::Mode::Selected, QIcon::State::On);
        QListWidgetItem *__qlistwidgetitem1 = new QListWidgetItem(stackListWidget);
        __qlistwidgetitem1->setIcon(icon1);
        QIcon icon2;
        icon2.addFile(QString::fromUtf8(":/graphicGui/tabLogs.png"), QSize(), QIcon::Mode::Normal, QIcon::State::On);
        icon2.addFile(QString::fromUtf8(":/graphicGui/tabLogs-selected.png"), QSize(), QIcon::Mode::Selected, QIcon::State::On);
        QListWidgetItem *__qlistwidgetitem2 = new QListWidgetItem(stackListWidget);
        __qlistwidgetitem2->setIcon(icon2);
        QIcon icon3;
        icon3.addFile(QString::fromUtf8(":/graphicGui/tabTools.png"), QSize(), QIcon::Mode::Normal, QIcon::State::On);
        icon3.addFile(QString::fromUtf8(":/graphicGui/tabTools-selected.png"), QSize(), QIcon::Mode::Selected, QIcon::State::On);
        QListWidgetItem *__qlistwidgetitem3 = new QListWidgetItem(stackListWidget);
        __qlistwidgetitem3->setIcon(icon3);
        stackListWidget->setObjectName("stackListWidget");
        stackListWidget->setFocusPolicy(Qt::NoFocus);
        stackListWidget->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        stackListWidget->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        stackListWidget->setSelectionMode(QAbstractItemView::SingleSelection);
        stackListWidget->setMovement(QListView::Static);
        stackListWidget->setFlow(QListView::LeftToRight);
        stackListWidget->setProperty("isWrapping", QVariant(false));
        stackListWidget->setResizeMode(QListView::Adjust);
        stackListWidget->setLayoutMode(QListView::SinglePass);
        stackListWidget->setSpacing(0);
        stackListWidget->setViewMode(QListView::IconMode);
        stackListWidget->setUniformItemSizes(false);
        stackListWidget->setWordWrap(false);
        stackListWidget->setSelectionRectVisible(false);

        gridLayout->addWidget(stackListWidget, 0, 0, 1, 1);

        stackedWidget = new QStackedWidget(viewContent);
        stackedWidget->setObjectName("stackedWidget");
        stackedWidget->setFrameShape(QFrame::StyledPanel);
        pageHelp = new QWidget();
        pageHelp->setObjectName("pageHelp");
        verticalLayout_5 = new QVBoxLayout(pageHelp);
        verticalLayout_5->setSpacing(4);
        verticalLayout_5->setObjectName("verticalLayout_5");
        verticalLayout_5->setContentsMargins(4, 4, 4, 4);
        helpBrowser = new QTextBrowser(pageHelp);
        helpBrowser->setObjectName("helpBrowser");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(helpBrowser->sizePolicy().hasHeightForWidth());
        helpBrowser->setSizePolicy(sizePolicy);
        helpBrowser->setMouseTracking(true);
        helpBrowser->setContextMenuPolicy(Qt::DefaultContextMenu);
        helpBrowser->setAcceptDrops(false);
        helpBrowser->setOpenExternalLinks(true);

        verticalLayout_5->addWidget(helpBrowser);

        editShortcutsButton = new QPushButton(pageHelp);
        editShortcutsButton->setObjectName("editShortcutsButton");

        verticalLayout_5->addWidget(editShortcutsButton);

        stackedWidget->addWidget(pageHelp);
        pageAbout = new QWidget();
        pageAbout->setObjectName("pageAbout");
        verticalLayout_2 = new QVBoxLayout(pageAbout);
        verticalLayout_2->setSpacing(4);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(4, 4, 4, 4);
        aboutBrowser = new QTextBrowser(pageAbout);
        aboutBrowser->setObjectName("aboutBrowser");
        aboutBrowser->setOpenExternalLinks(true);

        verticalLayout_2->addWidget(aboutBrowser);

        checkUpdatesButton = new QPushButton(pageAbout);
        checkUpdatesButton->setObjectName("checkUpdatesButton");

        verticalLayout_2->addWidget(checkUpdatesButton);

        stackedWidget->addWidget(pageAbout);
        pageLog = new QWidget();
        pageLog->setObjectName("pageLog");
        verticalLayout = new QVBoxLayout(pageLog);
        verticalLayout->setSpacing(4);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(4, 4, 4, 4);
        logPathLabel = new QLabel(pageLog);
        logPathLabel->setObjectName("logPathLabel");
        logPathLabel->setText(QString::fromUtf8("xxx"));
        logPathLabel->setWordWrap(true);
        logPathLabel->setTextInteractionFlags(Qt::TextSelectableByMouse);

        verticalLayout->addWidget(logPathLabel);

        logBrowser = new QTextBrowser(pageLog);
        logBrowser->setObjectName("logBrowser");
        logBrowser->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        logBrowser->setLineWrapMode(QTextEdit::WidgetWidth);
        logBrowser->setAcceptRichText(false);
        logBrowser->setTextInteractionFlags(Qt::TextSelectableByKeyboard|Qt::TextSelectableByMouse);
        logBrowser->setOpenLinks(false);

        verticalLayout->addWidget(logBrowser);

        refreshButton = new QPushButton(pageLog);
        refreshButton->setObjectName("refreshButton");

        verticalLayout->addWidget(refreshButton);

        stackedWidget->addWidget(pageLog);
        pageConfig = new QWidget();
        pageConfig->setObjectName("pageConfig");
        verticalLayout_4 = new QVBoxLayout(pageConfig);
        verticalLayout_4->setSpacing(4);
        verticalLayout_4->setObjectName("verticalLayout_4");
        verticalLayout_4->setContentsMargins(4, 4, 4, 4);
        configPathLabel = new QLabel(pageConfig);
        configPathLabel->setObjectName("configPathLabel");
        configPathLabel->setText(QString::fromUtf8("xxx"));

        verticalLayout_4->addWidget(configPathLabel);

        configBrowser = new QTextBrowser(pageConfig);
        configBrowser->setObjectName("configBrowser");

        verticalLayout_4->addWidget(configBrowser);

        refreshConfigButton = new QPushButton(pageConfig);
        refreshConfigButton->setObjectName("refreshConfigButton");

        verticalLayout_4->addWidget(refreshConfigButton);

        stackedWidget->addWidget(pageConfig);

        gridLayout->addWidget(stackedWidget, 1, 0, 1, 1);


        verticalLayout_3->addWidget(viewContent);


        retranslateUi(helpDialogForm);

        stackedWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(helpDialogForm);
    } // setupUi

    void retranslateUi(QWidget *helpDialogForm)
    {
        titleBar->setTitle(QCoreApplication::translate("helpDialogForm", "Help", nullptr));

        const bool __sortingEnabled = stackListWidget->isSortingEnabled();
        stackListWidget->setSortingEnabled(false);
        QListWidgetItem *___qlistwidgetitem = stackListWidget->item(0);
        ___qlistwidgetitem->setText(QCoreApplication::translate("helpDialogForm", "Help", nullptr));
        QListWidgetItem *___qlistwidgetitem1 = stackListWidget->item(1);
        ___qlistwidgetitem1->setText(QCoreApplication::translate("helpDialogForm", "About", nullptr));
        QListWidgetItem *___qlistwidgetitem2 = stackListWidget->item(2);
        ___qlistwidgetitem2->setText(QCoreApplication::translate("helpDialogForm", "Log", nullptr));
        QListWidgetItem *___qlistwidgetitem3 = stackListWidget->item(3);
        ___qlistwidgetitem3->setText(QCoreApplication::translate("helpDialogForm", "Config", nullptr));
        stackListWidget->setSortingEnabled(__sortingEnabled);

        editShortcutsButton->setText(QCoreApplication::translate("helpDialogForm", "Edit keyboard shortcuts...", nullptr));
        checkUpdatesButton->setText(QCoreApplication::translate("helpDialogForm", "Check updates", nullptr));
        refreshButton->setText(QCoreApplication::translate("helpDialogForm", "Refresh", nullptr));
        refreshConfigButton->setText(QCoreApplication::translate("helpDialogForm", "Refresh", nullptr));
        (void)helpDialogForm;
    } // retranslateUi

};

namespace Ui {
    class helpDialogForm: public Ui_helpDialogForm {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_HELPDIALOGGUI_H
