/****************************************************************************
** Meta object code from reading C++ file 'StelSkyCultureMgr.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../stellarium/src/core/StelSkyCultureMgr.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'StelSkyCultureMgr.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14StelSkyCultureE_t {};
} // unnamed namespace

template <> constexpr inline auto StelSkyCulture::qt_create_metaobjectdata<qt_meta_tag_ZN14StelSkyCultureE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "StelSkyCulture",
        "BoundariesType",
        "None",
        "IAU",
        "Own",
        "CLASSIFICATION",
        "INCOMPLETE",
        "PERSONAL",
        "TRADITIONAL",
        "ETHNOGRAPHIC",
        "HISTORICAL",
        "SINGLE",
        "COMPARATIVE"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'BoundariesType'
        QtMocHelpers::EnumData<enum BoundariesType>(1, 1, QMC::EnumIsScoped).add({
            {    2, BoundariesType::None },
            {    3, BoundariesType::IAU },
            {    4, BoundariesType::Own },
        }),
        // enum 'CLASSIFICATION'
        QtMocHelpers::EnumData<enum CLASSIFICATION>(5, 5, QMC::EnumFlags{}).add({
            {    6, CLASSIFICATION::INCOMPLETE },
            {    7, CLASSIFICATION::PERSONAL },
            {    8, CLASSIFICATION::TRADITIONAL },
            {    9, CLASSIFICATION::ETHNOGRAPHIC },
            {   10, CLASSIFICATION::HISTORICAL },
            {   11, CLASSIFICATION::SINGLE },
            {   12, CLASSIFICATION::COMPARATIVE },
        }),
    };
    return QtMocHelpers::metaObjectData<StelSkyCulture, qt_meta_tag_ZN14StelSkyCultureE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject StelSkyCulture::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14StelSkyCultureE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14StelSkyCultureE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14StelSkyCultureE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN17StelSkyCultureMgrE_t {};
} // unnamed namespace

template <> constexpr inline auto StelSkyCultureMgr::qt_create_metaobjectdata<qt_meta_tag_ZN17StelSkyCultureMgrE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "StelSkyCultureMgr",
        "defaultSkyCultureIDChanged",
        "",
        "id",
        "currentSkyCultureIDChanged",
        "currentSkyCultureChanged",
        "StelSkyCulture",
        "culture",
        "skyCultureListChanged",
        "infoLabelStyleChanged",
        "StelObject::CulturalDisplayStyle",
        "style",
        "screenLabelStyleChanged",
        "zodiacLabelStyleChanged",
        "lunarSystemLabelStyleChanged",
        "flagOverrideUseCommonNamesChanged",
        "override",
        "flagUseAbbreviatedNamesChanged",
        "b",
        "getCurrentSkyCultureEnglishName",
        "getCurrentSkyCultureNameI18",
        "setCurrentSkyCultureNameI18",
        "cultureName",
        "getCurrentSkyCultureID",
        "setCurrentSkyCultureID",
        "reloadSkyCulture",
        "getCurrentSkyCultureBoundariesType",
        "StelSkyCulture::BoundariesType",
        "getCurrentSkyCultureClassificationIdx",
        "StelSkyCulture::CLASSIFICATION",
        "getCurrentSkyCultureHtmlClassification",
        "getCurrentSkyCultureHtmlLicense",
        "getCurrentSkyCultureHtmlRegion",
        "getCurrentSkyCultureHtmlDescription",
        "getCurrentSkyCultureNarration",
        "getDefaultSkyCultureID",
        "setDefaultSkyCultureID",
        "getSkyCultureListEnglish",
        "getSkyCultureListI18",
        "getSkyCultureRegionMapI18",
        "QMultiMap<QString,QString>",
        "getSkyCultureTimeLimitMapI18",
        "QMap<QString,std::pair<int,int>>",
        "getSkyCultureListIDs",
        "getDirToNameMap",
        "QMap<QString,StelSkyCulture>",
        "getDirToI18Map",
        "QMap<QString,QString>",
        "convertCulturalDisplayStyleFromCSVstring",
        "csv",
        "convertCulturalDisplayStyleToCSVstring",
        "getScreenLabelStyle",
        "getScreenLabelStyleString",
        "setScreenLabelStyle",
        "getInfoLabelStyle",
        "getInfoLabelStyleString",
        "setInfoLabelStyle",
        "getZodiacLabelStyle",
        "getZodiacLabelStyleString",
        "setZodiacLabelStyle",
        "getLunarSystemLabelStyle",
        "getLunarSystemLabelStyleString",
        "setLunarSystemLabelStyle",
        "getFlagOverrideUseCommonNames",
        "setFlagOverrideUseCommonNames",
        "getFlagUseAbbreviatedNames",
        "setFlagUseAbbreviatedNames",
        "currentSkycultureUsesCommonNames",
        "analyzeScreenLabel",
        "currentSkyCultureID",
        "defaultSkyCultureID",
        "screenLabelStyle",
        "infoLabelStyle",
        "zodiacLabelStyle",
        "lunarSystemLabelStyle",
        "flagUseAbbreviatedNames",
        "flagOverrideUseCommonNames"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'defaultSkyCultureIDChanged'
        QtMocHelpers::SignalData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'currentSkyCultureIDChanged'
        QtMocHelpers::SignalData<void(const QString &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'currentSkyCultureChanged'
        QtMocHelpers::SignalData<void(const StelSkyCulture &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'skyCultureListChanged'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'infoLabelStyleChanged'
        QtMocHelpers::SignalData<void(const StelObject::CulturalDisplayStyle)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Signal 'screenLabelStyleChanged'
        QtMocHelpers::SignalData<void(const StelObject::CulturalDisplayStyle)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Signal 'zodiacLabelStyleChanged'
        QtMocHelpers::SignalData<void(const StelObject::CulturalDisplayStyle)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Signal 'lunarSystemLabelStyleChanged'
        QtMocHelpers::SignalData<void(const StelObject::CulturalDisplayStyle)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Signal 'flagOverrideUseCommonNamesChanged'
        QtMocHelpers::SignalData<void(bool)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 16 },
        }}),
        // Signal 'flagUseAbbreviatedNamesChanged'
        QtMocHelpers::SignalData<void(bool)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 18 },
        }}),
        // Slot 'getCurrentSkyCultureEnglishName'
        QtMocHelpers::SlotData<QString() const>(19, 2, QMC::AccessPublic, QMetaType::QString),
        // Slot 'getCurrentSkyCultureNameI18'
        QtMocHelpers::SlotData<QString() const>(20, 2, QMC::AccessPublic, QMetaType::QString),
        // Slot 'setCurrentSkyCultureNameI18'
        QtMocHelpers::SlotData<bool(const QString &)>(21, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 22 },
        }}),
        // Slot 'getCurrentSkyCultureID'
        QtMocHelpers::SlotData<QString() const>(23, 2, QMC::AccessPublic, QMetaType::QString),
        // Slot 'setCurrentSkyCultureID'
        QtMocHelpers::SlotData<bool(const QString &)>(24, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'reloadSkyCulture'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'getCurrentSkyCultureBoundariesType'
        QtMocHelpers::SlotData<StelSkyCulture::BoundariesType() const>(26, 2, QMC::AccessPublic, 0x80000000 | 27),
        // Slot 'getCurrentSkyCultureClassificationIdx'
        QtMocHelpers::SlotData<StelSkyCulture::CLASSIFICATION() const>(28, 2, QMC::AccessPublic, 0x80000000 | 29),
        // Slot 'getCurrentSkyCultureHtmlClassification'
        QtMocHelpers::SlotData<QString() const>(30, 2, QMC::AccessPublic, QMetaType::QString),
        // Slot 'getCurrentSkyCultureHtmlLicense'
        QtMocHelpers::SlotData<QString() const>(31, 2, QMC::AccessPublic, QMetaType::QString),
        // Slot 'getCurrentSkyCultureHtmlRegion'
        QtMocHelpers::SlotData<QString() const>(32, 2, QMC::AccessPublic, QMetaType::QString),
        // Slot 'getCurrentSkyCultureHtmlDescription'
        QtMocHelpers::SlotData<QString()>(33, 2, QMC::AccessPublic, QMetaType::QString),
        // Slot 'getCurrentSkyCultureNarration'
        QtMocHelpers::SlotData<QString()>(34, 2, QMC::AccessPublic, QMetaType::QString),
        // Slot 'getDefaultSkyCultureID'
        QtMocHelpers::SlotData<QString()>(35, 2, QMC::AccessPublic, QMetaType::QString),
        // Slot 'setDefaultSkyCultureID'
        QtMocHelpers::SlotData<bool(const QString &)>(36, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'getSkyCultureListEnglish'
        QtMocHelpers::SlotData<QStringList() const>(37, 2, QMC::AccessPublic, QMetaType::QStringList),
        // Slot 'getSkyCultureListI18'
        QtMocHelpers::SlotData<QStringList() const>(38, 2, QMC::AccessPublic, QMetaType::QStringList),
        // Slot 'getSkyCultureRegionMapI18'
        QtMocHelpers::SlotData<QMultiMap<QString,QString>() const>(39, 2, QMC::AccessPublic, 0x80000000 | 40),
        // Slot 'getSkyCultureTimeLimitMapI18'
        QtMocHelpers::SlotData<QMap<QString,QPair<int,int> >() const>(41, 2, QMC::AccessPublic, 0x80000000 | 42),
        // Slot 'getSkyCultureListIDs'
        QtMocHelpers::SlotData<QStringList() const>(43, 2, QMC::AccessPublic, QMetaType::QStringList),
        // Slot 'getDirToNameMap'
        QtMocHelpers::SlotData<QMap<QString,StelSkyCulture>() const>(44, 2, QMC::AccessPublic, 0x80000000 | 45),
        // Slot 'getDirToI18Map'
        QtMocHelpers::SlotData<QMap<QString,QString>() const>(46, 2, QMC::AccessPublic, 0x80000000 | 47),
        // Slot 'convertCulturalDisplayStyleFromCSVstring'
        QtMocHelpers::SlotData<StelObject::CulturalDisplayStyle(const QString &)>(48, 2, QMC::AccessPublic, 0x80000000 | 10, {{
            { QMetaType::QString, 49 },
        }}),
        // Slot 'convertCulturalDisplayStyleToCSVstring'
        QtMocHelpers::SlotData<QString(const StelObject::CulturalDisplayStyle)>(50, 2, QMC::AccessPublic, QMetaType::QString, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Slot 'getScreenLabelStyle'
        QtMocHelpers::SlotData<StelObject::CulturalDisplayStyle() const>(51, 2, QMC::AccessPublic, 0x80000000 | 10),
        // Slot 'getScreenLabelStyleString'
        QtMocHelpers::SlotData<QString() const>(52, 2, QMC::AccessPublic, QMetaType::QString),
        // Slot 'setScreenLabelStyle'
        QtMocHelpers::SlotData<void(const StelObject::CulturalDisplayStyle)>(53, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Slot 'setScreenLabelStyle'
        QtMocHelpers::SlotData<void(const QString &)>(53, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 11 },
        }}),
        // Slot 'getInfoLabelStyle'
        QtMocHelpers::SlotData<StelObject::CulturalDisplayStyle() const>(54, 2, QMC::AccessPublic, 0x80000000 | 10),
        // Slot 'getInfoLabelStyleString'
        QtMocHelpers::SlotData<QString() const>(55, 2, QMC::AccessPublic, QMetaType::QString),
        // Slot 'setInfoLabelStyle'
        QtMocHelpers::SlotData<void(const StelObject::CulturalDisplayStyle)>(56, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Slot 'setInfoLabelStyle'
        QtMocHelpers::SlotData<void(const QString &)>(56, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 11 },
        }}),
        // Slot 'getZodiacLabelStyle'
        QtMocHelpers::SlotData<StelObject::CulturalDisplayStyle() const>(57, 2, QMC::AccessPublic, 0x80000000 | 10),
        // Slot 'getZodiacLabelStyleString'
        QtMocHelpers::SlotData<QString() const>(58, 2, QMC::AccessPublic, QMetaType::QString),
        // Slot 'setZodiacLabelStyle'
        QtMocHelpers::SlotData<void(const StelObject::CulturalDisplayStyle)>(59, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Slot 'setZodiacLabelStyle'
        QtMocHelpers::SlotData<void(const QString &)>(59, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 11 },
        }}),
        // Slot 'getLunarSystemLabelStyle'
        QtMocHelpers::SlotData<StelObject::CulturalDisplayStyle() const>(60, 2, QMC::AccessPublic, 0x80000000 | 10),
        // Slot 'getLunarSystemLabelStyleString'
        QtMocHelpers::SlotData<QString() const>(61, 2, QMC::AccessPublic, QMetaType::QString),
        // Slot 'setLunarSystemLabelStyle'
        QtMocHelpers::SlotData<void(const StelObject::CulturalDisplayStyle)>(62, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Slot 'setLunarSystemLabelStyle'
        QtMocHelpers::SlotData<void(const QString &)>(62, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 11 },
        }}),
        // Slot 'getFlagOverrideUseCommonNames'
        QtMocHelpers::SlotData<bool() const>(63, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFlagOverrideUseCommonNames'
        QtMocHelpers::SlotData<void(bool)>(64, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 16 },
        }}),
        // Slot 'getFlagUseAbbreviatedNames'
        QtMocHelpers::SlotData<bool() const>(65, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFlagUseAbbreviatedNames'
        QtMocHelpers::SlotData<void(bool)>(66, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 18 },
        }}),
        // Slot 'currentSkycultureUsesCommonNames'
        QtMocHelpers::SlotData<bool() const>(67, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'analyzeScreenLabel'
        QtMocHelpers::SlotData<void() const>(68, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'currentSkyCultureID'
        QtMocHelpers::PropertyData<QString>(69, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'defaultSkyCultureID'
        QtMocHelpers::PropertyData<QString>(70, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'screenLabelStyle'
        QtMocHelpers::PropertyData<StelObject::CulturalDisplayStyle>(71, 0x80000000 | 10, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 5),
        // property 'infoLabelStyle'
        QtMocHelpers::PropertyData<StelObject::CulturalDisplayStyle>(72, 0x80000000 | 10, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 4),
        // property 'zodiacLabelStyle'
        QtMocHelpers::PropertyData<StelObject::CulturalDisplayStyle>(73, 0x80000000 | 10, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 6),
        // property 'lunarSystemLabelStyle'
        QtMocHelpers::PropertyData<StelObject::CulturalDisplayStyle>(74, 0x80000000 | 10, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 7),
        // property 'flagUseAbbreviatedNames'
        QtMocHelpers::PropertyData<bool>(75, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 9),
        // property 'flagOverrideUseCommonNames'
        QtMocHelpers::PropertyData<bool>(76, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 8),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<StelSkyCultureMgr, qt_meta_tag_ZN17StelSkyCultureMgrE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN17StelSkyCultureMgrE[] = {
    QMetaObject::SuperData::link<StelObject::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject StelSkyCultureMgr::staticMetaObject = { {
    QMetaObject::SuperData::link<StelModule::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17StelSkyCultureMgrE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17StelSkyCultureMgrE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN17StelSkyCultureMgrE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17StelSkyCultureMgrE_t>.metaTypes,
    nullptr
} };

void StelSkyCultureMgr::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<StelSkyCultureMgr *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->defaultSkyCultureIDChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->currentSkyCultureIDChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->currentSkyCultureChanged((*reinterpret_cast<std::add_pointer_t<StelSkyCulture>>(_a[1]))); break;
        case 3: _t->skyCultureListChanged(); break;
        case 4: _t->infoLabelStyleChanged((*reinterpret_cast<std::add_pointer_t<StelObject::CulturalDisplayStyle>>(_a[1]))); break;
        case 5: _t->screenLabelStyleChanged((*reinterpret_cast<std::add_pointer_t<StelObject::CulturalDisplayStyle>>(_a[1]))); break;
        case 6: _t->zodiacLabelStyleChanged((*reinterpret_cast<std::add_pointer_t<StelObject::CulturalDisplayStyle>>(_a[1]))); break;
        case 7: _t->lunarSystemLabelStyleChanged((*reinterpret_cast<std::add_pointer_t<StelObject::CulturalDisplayStyle>>(_a[1]))); break;
        case 8: _t->flagOverrideUseCommonNamesChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 9: _t->flagUseAbbreviatedNamesChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 10: { QString _r = _t->getCurrentSkyCultureEnglishName();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 11: { QString _r = _t->getCurrentSkyCultureNameI18();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 12: { bool _r = _t->setCurrentSkyCultureNameI18((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 13: { QString _r = _t->getCurrentSkyCultureID();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 14: { bool _r = _t->setCurrentSkyCultureID((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 15: _t->reloadSkyCulture(); break;
        case 16: { StelSkyCulture::BoundariesType _r = _t->getCurrentSkyCultureBoundariesType();
            if (_a[0]) *reinterpret_cast<StelSkyCulture::BoundariesType*>(_a[0]) = std::move(_r); }  break;
        case 17: { StelSkyCulture::CLASSIFICATION _r = _t->getCurrentSkyCultureClassificationIdx();
            if (_a[0]) *reinterpret_cast<StelSkyCulture::CLASSIFICATION*>(_a[0]) = std::move(_r); }  break;
        case 18: { QString _r = _t->getCurrentSkyCultureHtmlClassification();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 19: { QString _r = _t->getCurrentSkyCultureHtmlLicense();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 20: { QString _r = _t->getCurrentSkyCultureHtmlRegion();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 21: { QString _r = _t->getCurrentSkyCultureHtmlDescription();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 22: { QString _r = _t->getCurrentSkyCultureNarration();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 23: { QString _r = _t->getDefaultSkyCultureID();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 24: { bool _r = _t->setDefaultSkyCultureID((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 25: { QStringList _r = _t->getSkyCultureListEnglish();
            if (_a[0]) *reinterpret_cast<QStringList*>(_a[0]) = std::move(_r); }  break;
        case 26: { QStringList _r = _t->getSkyCultureListI18();
            if (_a[0]) *reinterpret_cast<QStringList*>(_a[0]) = std::move(_r); }  break;
        case 27: { QMultiMap<QString,QString> _r = _t->getSkyCultureRegionMapI18();
            if (_a[0]) *reinterpret_cast<QMultiMap<QString,QString>*>(_a[0]) = std::move(_r); }  break;
        case 28: { QMap<QString,std::pair<int,int>> _r = _t->getSkyCultureTimeLimitMapI18();
            if (_a[0]) *reinterpret_cast<QMap<QString,std::pair<int,int>>*>(_a[0]) = std::move(_r); }  break;
        case 29: { QStringList _r = _t->getSkyCultureListIDs();
            if (_a[0]) *reinterpret_cast<QStringList*>(_a[0]) = std::move(_r); }  break;
        case 30: { QMap<QString,StelSkyCulture> _r = _t->getDirToNameMap();
            if (_a[0]) *reinterpret_cast<QMap<QString,StelSkyCulture>*>(_a[0]) = std::move(_r); }  break;
        case 31: { QMap<QString,QString> _r = _t->getDirToI18Map();
            if (_a[0]) *reinterpret_cast<QMap<QString,QString>*>(_a[0]) = std::move(_r); }  break;
        case 32: { StelObject::CulturalDisplayStyle _r = _t->convertCulturalDisplayStyleFromCSVstring((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<StelObject::CulturalDisplayStyle*>(_a[0]) = std::move(_r); }  break;
        case 33: { QString _r = _t->convertCulturalDisplayStyleToCSVstring((*reinterpret_cast<std::add_pointer_t<StelObject::CulturalDisplayStyle>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 34: { StelObject::CulturalDisplayStyle _r = _t->getScreenLabelStyle();
            if (_a[0]) *reinterpret_cast<StelObject::CulturalDisplayStyle*>(_a[0]) = std::move(_r); }  break;
        case 35: { QString _r = _t->getScreenLabelStyleString();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 36: _t->setScreenLabelStyle((*reinterpret_cast<std::add_pointer_t<StelObject::CulturalDisplayStyle>>(_a[1]))); break;
        case 37: _t->setScreenLabelStyle((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 38: { StelObject::CulturalDisplayStyle _r = _t->getInfoLabelStyle();
            if (_a[0]) *reinterpret_cast<StelObject::CulturalDisplayStyle*>(_a[0]) = std::move(_r); }  break;
        case 39: { QString _r = _t->getInfoLabelStyleString();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 40: _t->setInfoLabelStyle((*reinterpret_cast<std::add_pointer_t<StelObject::CulturalDisplayStyle>>(_a[1]))); break;
        case 41: _t->setInfoLabelStyle((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 42: { StelObject::CulturalDisplayStyle _r = _t->getZodiacLabelStyle();
            if (_a[0]) *reinterpret_cast<StelObject::CulturalDisplayStyle*>(_a[0]) = std::move(_r); }  break;
        case 43: { QString _r = _t->getZodiacLabelStyleString();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 44: _t->setZodiacLabelStyle((*reinterpret_cast<std::add_pointer_t<StelObject::CulturalDisplayStyle>>(_a[1]))); break;
        case 45: _t->setZodiacLabelStyle((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 46: { StelObject::CulturalDisplayStyle _r = _t->getLunarSystemLabelStyle();
            if (_a[0]) *reinterpret_cast<StelObject::CulturalDisplayStyle*>(_a[0]) = std::move(_r); }  break;
        case 47: { QString _r = _t->getLunarSystemLabelStyleString();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 48: _t->setLunarSystemLabelStyle((*reinterpret_cast<std::add_pointer_t<StelObject::CulturalDisplayStyle>>(_a[1]))); break;
        case 49: _t->setLunarSystemLabelStyle((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 50: { bool _r = _t->getFlagOverrideUseCommonNames();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 51: _t->setFlagOverrideUseCommonNames((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 52: { bool _r = _t->getFlagUseAbbreviatedNames();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 53: _t->setFlagUseAbbreviatedNames((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 54: { bool _r = _t->currentSkycultureUsesCommonNames();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 55: _t->analyzeScreenLabel(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (StelSkyCultureMgr::*)(const QString & )>(_a, &StelSkyCultureMgr::defaultSkyCultureIDChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelSkyCultureMgr::*)(const QString & )>(_a, &StelSkyCultureMgr::currentSkyCultureIDChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelSkyCultureMgr::*)(const StelSkyCulture & )>(_a, &StelSkyCultureMgr::currentSkyCultureChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelSkyCultureMgr::*)()>(_a, &StelSkyCultureMgr::skyCultureListChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelSkyCultureMgr::*)(const StelObject::CulturalDisplayStyle )>(_a, &StelSkyCultureMgr::infoLabelStyleChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelSkyCultureMgr::*)(const StelObject::CulturalDisplayStyle )>(_a, &StelSkyCultureMgr::screenLabelStyleChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelSkyCultureMgr::*)(const StelObject::CulturalDisplayStyle )>(_a, &StelSkyCultureMgr::zodiacLabelStyleChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelSkyCultureMgr::*)(const StelObject::CulturalDisplayStyle )>(_a, &StelSkyCultureMgr::lunarSystemLabelStyleChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelSkyCultureMgr::*)(bool )>(_a, &StelSkyCultureMgr::flagOverrideUseCommonNamesChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelSkyCultureMgr::*)(bool )>(_a, &StelSkyCultureMgr::flagUseAbbreviatedNamesChanged, 9))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->getCurrentSkyCultureID(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->getDefaultSkyCultureID(); break;
        case 2: *reinterpret_cast<StelObject::CulturalDisplayStyle*>(_v) = _t->getScreenLabelStyle(); break;
        case 3: *reinterpret_cast<StelObject::CulturalDisplayStyle*>(_v) = _t->getInfoLabelStyle(); break;
        case 4: *reinterpret_cast<StelObject::CulturalDisplayStyle*>(_v) = _t->getZodiacLabelStyle(); break;
        case 5: *reinterpret_cast<StelObject::CulturalDisplayStyle*>(_v) = _t->getLunarSystemLabelStyle(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->getFlagUseAbbreviatedNames(); break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->getFlagOverrideUseCommonNames(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setCurrentSkyCultureID(*reinterpret_cast<QString*>(_v)); break;
        case 1: _t->setDefaultSkyCultureID(*reinterpret_cast<QString*>(_v)); break;
        case 2: _t->setScreenLabelStyle(*reinterpret_cast<StelObject::CulturalDisplayStyle*>(_v)); break;
        case 3: _t->setInfoLabelStyle(*reinterpret_cast<StelObject::CulturalDisplayStyle*>(_v)); break;
        case 4: _t->setZodiacLabelStyle(*reinterpret_cast<StelObject::CulturalDisplayStyle*>(_v)); break;
        case 5: _t->setLunarSystemLabelStyle(*reinterpret_cast<StelObject::CulturalDisplayStyle*>(_v)); break;
        case 6: _t->setFlagUseAbbreviatedNames(*reinterpret_cast<bool*>(_v)); break;
        case 7: _t->setFlagOverrideUseCommonNames(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *StelSkyCultureMgr::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *StelSkyCultureMgr::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17StelSkyCultureMgrE_t>.strings))
        return static_cast<void*>(this);
    return StelModule::qt_metacast(_clname);
}

int StelSkyCultureMgr::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StelModule::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 56)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 56;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 56)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 56;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void StelSkyCultureMgr::defaultSkyCultureIDChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void StelSkyCultureMgr::currentSkyCultureIDChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void StelSkyCultureMgr::currentSkyCultureChanged(const StelSkyCulture & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void StelSkyCultureMgr::skyCultureListChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void StelSkyCultureMgr::infoLabelStyleChanged(const StelObject::CulturalDisplayStyle _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void StelSkyCultureMgr::screenLabelStyleChanged(const StelObject::CulturalDisplayStyle _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void StelSkyCultureMgr::zodiacLabelStyleChanged(const StelObject::CulturalDisplayStyle _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void StelSkyCultureMgr::lunarSystemLabelStyleChanged(const StelObject::CulturalDisplayStyle _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void StelSkyCultureMgr::flagOverrideUseCommonNamesChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void StelSkyCultureMgr::flagUseAbbreviatedNamesChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}
QT_WARNING_POP
