/********************************************************************************
** Form generated from reading UI file 'slewDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SLEWDIALOG_H
#define UI_SLEWDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "AngleSpinBox.hpp"
#include "Dialog.hpp"

QT_BEGIN_NAMESPACE

class Ui_slewDialog
{
public:
    QVBoxLayout *verticalLayout_4;
    TitleBar *titleBar;
    QLabel *labelNoTelescopes;
    QGroupBox *groupBoxSlew;
    QVBoxLayout *verticalLayout_2;
    QFrame *frameCoordinates;
    QGridLayout *gridLayout;
    QLabel *labelRA;
    QLabel *labelDec;
    AngleSpinBox *spinBoxDec;
    AngleSpinBox *spinBoxRA;
    QHBoxLayout *horizontalLayout_2;
    QPushButton *pushButtonCurrent;
    QPushButton *pushButtonCenter;
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout_3;
    QComboBox *comboBoxStoredPoints;
    QToolButton *toolButtonStoredPoints;
    QFrame *frameFormat;
    QHBoxLayout *horizontalLayout;
    QRadioButton *radioButtonHMS;
    QRadioButton *radioButtonDMS;
    QRadioButton *radioButtonDecimal;
    QFrame *frameTelescope;
    QVBoxLayout *verticalLayout_3;
    QComboBox *comboBoxTelescope;
    QPushButton *pushButtonSlew;
    QHBoxLayout *horizontalLayout_4;
    QPushButton *pushButtonSync;
    QPushButton *pushButtonAbort;
    QVBoxLayout *controlWidgetLayout;
    QSpacerItem *verticalSpacer;
    QPushButton *pushButtonConfigure;

    void setupUi(QWidget *slewDialog)
    {
        if (slewDialog->objectName().isEmpty())
            slewDialog->setObjectName("slewDialog");
        slewDialog->resize(402, 447);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(slewDialog->sizePolicy().hasHeightForWidth());
        slewDialog->setSizePolicy(sizePolicy);
        slewDialog->setMinimumSize(QSize(400, 0));
        verticalLayout_4 = new QVBoxLayout(slewDialog);
        verticalLayout_4->setSpacing(0);
        verticalLayout_4->setObjectName("verticalLayout_4");
        verticalLayout_4->setSizeConstraint(QLayout::SetDefaultConstraint);
        verticalLayout_4->setContentsMargins(0, 0, 0, 0);
        titleBar = new TitleBar(slewDialog);
        titleBar->setObjectName("titleBar");

        verticalLayout_4->addWidget(titleBar);

        labelNoTelescopes = new QLabel(slewDialog);
        labelNoTelescopes->setObjectName("labelNoTelescopes");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(labelNoTelescopes->sizePolicy().hasHeightForWidth());
        labelNoTelescopes->setSizePolicy(sizePolicy1);
        labelNoTelescopes->setAlignment(Qt::AlignCenter);
        labelNoTelescopes->setTextInteractionFlags(Qt::NoTextInteraction);

        verticalLayout_4->addWidget(labelNoTelescopes);

        groupBoxSlew = new QGroupBox(slewDialog);
        groupBoxSlew->setObjectName("groupBoxSlew");
        sizePolicy.setHeightForWidth(groupBoxSlew->sizePolicy().hasHeightForWidth());
        groupBoxSlew->setSizePolicy(sizePolicy);
        verticalLayout_2 = new QVBoxLayout(groupBoxSlew);
        verticalLayout_2->setObjectName("verticalLayout_2");
        frameCoordinates = new QFrame(groupBoxSlew);
        frameCoordinates->setObjectName("frameCoordinates");
        sizePolicy.setHeightForWidth(frameCoordinates->sizePolicy().hasHeightForWidth());
        frameCoordinates->setSizePolicy(sizePolicy);
        gridLayout = new QGridLayout(frameCoordinates);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setContentsMargins(0, 0, 0, 0);
        labelRA = new QLabel(frameCoordinates);
        labelRA->setObjectName("labelRA");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Preferred);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(labelRA->sizePolicy().hasHeightForWidth());
        labelRA->setSizePolicy(sizePolicy2);

        gridLayout->addWidget(labelRA, 0, 0, 1, 1);

        labelDec = new QLabel(frameCoordinates);
        labelDec->setObjectName("labelDec");
        sizePolicy2.setHeightForWidth(labelDec->sizePolicy().hasHeightForWidth());
        labelDec->setSizePolicy(sizePolicy2);

        gridLayout->addWidget(labelDec, 1, 0, 1, 1);

        spinBoxDec = new AngleSpinBox(frameCoordinates);
        spinBoxDec->setObjectName("spinBoxDec");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(spinBoxDec->sizePolicy().hasHeightForWidth());
        spinBoxDec->setSizePolicy(sizePolicy3);
        spinBoxDec->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(spinBoxDec, 1, 2, 1, 1);

        spinBoxRA = new AngleSpinBox(frameCoordinates);
        spinBoxRA->setObjectName("spinBoxRA");
        QSizePolicy sizePolicy4(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy4.setHorizontalStretch(0);
        sizePolicy4.setVerticalStretch(0);
        sizePolicy4.setHeightForWidth(spinBoxRA->sizePolicy().hasHeightForWidth());
        spinBoxRA->setSizePolicy(sizePolicy4);
        spinBoxRA->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(spinBoxRA, 0, 2, 1, 1);


        verticalLayout_2->addWidget(frameCoordinates);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        pushButtonCurrent = new QPushButton(groupBoxSlew);
        pushButtonCurrent->setObjectName("pushButtonCurrent");

        horizontalLayout_2->addWidget(pushButtonCurrent);

        pushButtonCenter = new QPushButton(groupBoxSlew);
        pushButtonCenter->setObjectName("pushButtonCenter");

        horizontalLayout_2->addWidget(pushButtonCenter);


        verticalLayout_2->addLayout(horizontalLayout_2);

        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName("verticalLayout");
        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        comboBoxStoredPoints = new QComboBox(groupBoxSlew);
        comboBoxStoredPoints->setObjectName("comboBoxStoredPoints");

        horizontalLayout_3->addWidget(comboBoxStoredPoints);

        toolButtonStoredPoints = new QToolButton(groupBoxSlew);
        toolButtonStoredPoints->setObjectName("toolButtonStoredPoints");

        horizontalLayout_3->addWidget(toolButtonStoredPoints);


        verticalLayout->addLayout(horizontalLayout_3);


        verticalLayout_2->addLayout(verticalLayout);

        frameFormat = new QFrame(groupBoxSlew);
        frameFormat->setObjectName("frameFormat");
        sizePolicy.setHeightForWidth(frameFormat->sizePolicy().hasHeightForWidth());
        frameFormat->setSizePolicy(sizePolicy);
        frameFormat->setFrameShape(QFrame::NoFrame);
        frameFormat->setFrameShadow(QFrame::Plain);
        horizontalLayout = new QHBoxLayout(frameFormat);
        horizontalLayout->setSpacing(6);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        radioButtonHMS = new QRadioButton(frameFormat);
        radioButtonHMS->setObjectName("radioButtonHMS");

        horizontalLayout->addWidget(radioButtonHMS);

        radioButtonDMS = new QRadioButton(frameFormat);
        radioButtonDMS->setObjectName("radioButtonDMS");

        horizontalLayout->addWidget(radioButtonDMS);

        radioButtonDecimal = new QRadioButton(frameFormat);
        radioButtonDecimal->setObjectName("radioButtonDecimal");

        horizontalLayout->addWidget(radioButtonDecimal);


        verticalLayout_2->addWidget(frameFormat);

        frameTelescope = new QFrame(groupBoxSlew);
        frameTelescope->setObjectName("frameTelescope");
        QSizePolicy sizePolicy5(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::MinimumExpanding);
        sizePolicy5.setHorizontalStretch(0);
        sizePolicy5.setVerticalStretch(0);
        sizePolicy5.setHeightForWidth(frameTelescope->sizePolicy().hasHeightForWidth());
        frameTelescope->setSizePolicy(sizePolicy5);
        verticalLayout_3 = new QVBoxLayout(frameTelescope);
        verticalLayout_3->setSpacing(6);
        verticalLayout_3->setObjectName("verticalLayout_3");
        verticalLayout_3->setContentsMargins(0, 0, 0, 0);
        comboBoxTelescope = new QComboBox(frameTelescope);
        comboBoxTelescope->setObjectName("comboBoxTelescope");
        QSizePolicy sizePolicy6(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Fixed);
        sizePolicy6.setHorizontalStretch(0);
        sizePolicy6.setVerticalStretch(0);
        sizePolicy6.setHeightForWidth(comboBoxTelescope->sizePolicy().hasHeightForWidth());
        comboBoxTelescope->setSizePolicy(sizePolicy6);
        comboBoxTelescope->setInsertPolicy(QComboBox::InsertAlphabetically);

        verticalLayout_3->addWidget(comboBoxTelescope);

        pushButtonSlew = new QPushButton(frameTelescope);
        pushButtonSlew->setObjectName("pushButtonSlew");
        pushButtonSlew->setMinimumSize(QSize(50, 48));

        verticalLayout_3->addWidget(pushButtonSlew);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        pushButtonSync = new QPushButton(frameTelescope);
        pushButtonSync->setObjectName("pushButtonSync");
        pushButtonSync->setMinimumSize(QSize(50, 24));

        horizontalLayout_4->addWidget(pushButtonSync);

        pushButtonAbort = new QPushButton(frameTelescope);
        pushButtonAbort->setObjectName("pushButtonAbort");
        pushButtonAbort->setMinimumSize(QSize(50, 24));

        horizontalLayout_4->addWidget(pushButtonAbort);


        verticalLayout_3->addLayout(horizontalLayout_4);


        verticalLayout_2->addWidget(frameTelescope);

        controlWidgetLayout = new QVBoxLayout();
        controlWidgetLayout->setObjectName("controlWidgetLayout");

        verticalLayout_2->addLayout(controlWidgetLayout);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_2->addItem(verticalSpacer);


        verticalLayout_4->addWidget(groupBoxSlew);

        pushButtonConfigure = new QPushButton(slewDialog);
        pushButtonConfigure->setObjectName("pushButtonConfigure");
        sizePolicy4.setHeightForWidth(pushButtonConfigure->sizePolicy().hasHeightForWidth());
        pushButtonConfigure->setSizePolicy(sizePolicy4);
        pushButtonConfigure->setMinimumSize(QSize(0, 34));
        pushButtonConfigure->setStyleSheet(QString::fromUtf8("margin-left:10px;\n"
"margin-right:10px;\n"
"margin-bottom:10px;"));

        verticalLayout_4->addWidget(pushButtonConfigure);

#if QT_CONFIG(shortcut)
        labelRA->setBuddy(spinBoxRA);
        labelDec->setBuddy(spinBoxDec);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(spinBoxRA, spinBoxDec);
        QWidget::setTabOrder(spinBoxDec, radioButtonHMS);
        QWidget::setTabOrder(radioButtonHMS, radioButtonDMS);
        QWidget::setTabOrder(radioButtonDMS, radioButtonDecimal);
        QWidget::setTabOrder(radioButtonDecimal, comboBoxTelescope);
        QWidget::setTabOrder(comboBoxTelescope, pushButtonSlew);
        QWidget::setTabOrder(pushButtonSlew, pushButtonConfigure);

        retranslateUi(slewDialog);

        pushButtonSlew->setDefault(true);


        QMetaObject::connectSlotsByName(slewDialog);
    } // setupUi

    void retranslateUi(QWidget *slewDialog)
    {
        titleBar->setProperty("title", QVariant(QCoreApplication::translate("slewDialog", "Slew telescope to", nullptr)));
        labelNoTelescopes->setText(QCoreApplication::translate("slewDialog", "There are no active devices.", nullptr));
        groupBoxSlew->setTitle(QCoreApplication::translate("slewDialog", "Slew telescope to coordinates", nullptr));
        labelRA->setText(QCoreApplication::translate("slewDialog", "&Right Ascension (J2000):", nullptr));
        labelDec->setText(QCoreApplication::translate("slewDialog", "De&clination (J2000):", nullptr));
#if QT_CONFIG(tooltip)
        pushButtonCurrent->setToolTip(QCoreApplication::translate("slewDialog", "Take coordinates from current object", nullptr));
#endif // QT_CONFIG(tooltip)
        pushButtonCurrent->setText(QCoreApplication::translate("slewDialog", "&Current object", nullptr));
#if QT_CONFIG(tooltip)
        pushButtonCenter->setToolTip(QCoreApplication::translate("slewDialog", "Take coordinates of screen center", nullptr));
#endif // QT_CONFIG(tooltip)
        pushButtonCenter->setText(QCoreApplication::translate("slewDialog", "Center of the screen", nullptr));
#if QT_CONFIG(tooltip)
        comboBoxStoredPoints->setToolTip(QCoreApplication::translate("slewDialog", "Select preconfigured point", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        toolButtonStoredPoints->setToolTip(QCoreApplication::translate("slewDialog", "Configure point list", nullptr));
#endif // QT_CONFIG(tooltip)
        toolButtonStoredPoints->setText(QCoreApplication::translate("slewDialog", "...", nullptr));
#if QT_CONFIG(tooltip)
        radioButtonHMS->setToolTip(QCoreApplication::translate("slewDialog", "Hours-minutes-seconds format", nullptr));
#endif // QT_CONFIG(tooltip)
        radioButtonHMS->setText(QCoreApplication::translate("slewDialog", "&HMS", nullptr));
#if QT_CONFIG(tooltip)
        radioButtonDMS->setToolTip(QCoreApplication::translate("slewDialog", "Degrees-minutes-seconds format", nullptr));
#endif // QT_CONFIG(tooltip)
        radioButtonDMS->setText(QCoreApplication::translate("slewDialog", "&DMS", nullptr));
#if QT_CONFIG(tooltip)
        radioButtonDecimal->setToolTip(QCoreApplication::translate("slewDialog", "Decimal degrees", nullptr));
#endif // QT_CONFIG(tooltip)
        radioButtonDecimal->setText(QCoreApplication::translate("slewDialog", "D&ecimal", nullptr));
#if QT_CONFIG(tooltip)
        pushButtonSlew->setToolTip(QCoreApplication::translate("slewDialog", "Slew to above coordinates", nullptr));
#endif // QT_CONFIG(tooltip)
        pushButtonSlew->setText(QCoreApplication::translate("slewDialog", "&Slew", nullptr));
#if QT_CONFIG(tooltip)
        pushButtonSync->setToolTip(QCoreApplication::translate("slewDialog", "Sync telescope to above coordinates (if supported by instrument)", nullptr));
#endif // QT_CONFIG(tooltip)
        pushButtonSync->setText(QCoreApplication::translate("slewDialog", "Sync", nullptr));
#if QT_CONFIG(tooltip)
        pushButtonAbort->setToolTip(QCoreApplication::translate("slewDialog", "Stop all telescope motions (if supported)", nullptr));
#endif // QT_CONFIG(tooltip)
        pushButtonAbort->setText(QCoreApplication::translate("slewDialog", "Abort slew", nullptr));
        pushButtonConfigure->setText(QCoreApplication::translate("slewDialog", "Configure &telescopes...", nullptr));
        (void)slewDialog;
    } // retranslateUi

};

namespace Ui {
    class slewDialog: public Ui_slewDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SLEWDIALOG_H
