/****************************************************************************
** Meta object code from reading C++ file 'AngleSpinBox.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../stellarium/src/gui/AngleSpinBox.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'AngleSpinBox.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12AngleSpinBoxE_t {};
} // unnamed namespace

template <> constexpr inline auto AngleSpinBox::qt_create_metaobjectdata<qt_meta_tag_ZN12AngleSpinBoxE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "AngleSpinBox",
        "valueChanged",
        "",
        "valueChangedDeg",
        "valueChangedRad",
        "clear",
        "setRadians",
        "radians",
        "setDegrees",
        "degrees",
        "updateValue",
        "DisplayFormat",
        "DMSLetters",
        "DMSSymbols",
        "DMSLettersUnsigned",
        "DMSSymbolsUnsigned",
        "HMSLetters",
        "HMSSymbols",
        "DecimalDeg",
        "PrefixType",
        "Normal",
        "NormalPlus",
        "Longitude",
        "Latitude",
        "Unknown"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'valueChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'valueChangedDeg'
        QtMocHelpers::SignalData<void(double)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Signal 'valueChangedRad'
        QtMocHelpers::SignalData<void(double)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Slot 'clear'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setRadians'
        QtMocHelpers::SlotData<void(double)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 7 },
        }}),
        // Slot 'setDegrees'
        QtMocHelpers::SlotData<void(double)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 9 },
        }}),
        // Slot 'updateValue'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'DisplayFormat'
        QtMocHelpers::EnumData<enum DisplayFormat>(11, 11, QMC::EnumFlags{}).add({
            {   12, DisplayFormat::DMSLetters },
            {   13, DisplayFormat::DMSSymbols },
            {   14, DisplayFormat::DMSLettersUnsigned },
            {   15, DisplayFormat::DMSSymbolsUnsigned },
            {   16, DisplayFormat::HMSLetters },
            {   17, DisplayFormat::HMSSymbols },
            {   18, DisplayFormat::DecimalDeg },
        }),
        // enum 'PrefixType'
        QtMocHelpers::EnumData<enum PrefixType>(19, 19, QMC::EnumFlags{}).add({
            {   20, PrefixType::Normal },
            {   21, PrefixType::NormalPlus },
            {   22, PrefixType::Longitude },
            {   23, PrefixType::Latitude },
            {   24, PrefixType::Unknown },
        }),
    };
    return QtMocHelpers::metaObjectData<AngleSpinBox, qt_meta_tag_ZN12AngleSpinBoxE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject AngleSpinBox::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractSpinBox::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12AngleSpinBoxE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12AngleSpinBoxE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12AngleSpinBoxE_t>.metaTypes,
    nullptr
} };

void AngleSpinBox::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AngleSpinBox *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->valueChanged(); break;
        case 1: _t->valueChangedDeg((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 2: _t->valueChangedRad((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 3: _t->clear(); break;
        case 4: _t->setRadians((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 5: _t->setDegrees((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 6: _t->updateValue(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (AngleSpinBox::*)()>(_a, &AngleSpinBox::valueChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (AngleSpinBox::*)(double )>(_a, &AngleSpinBox::valueChangedDeg, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (AngleSpinBox::*)(double )>(_a, &AngleSpinBox::valueChangedRad, 2))
            return;
    }
}

const QMetaObject *AngleSpinBox::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *AngleSpinBox::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12AngleSpinBoxE_t>.strings))
        return static_cast<void*>(this);
    return QAbstractSpinBox::qt_metacast(_clname);
}

int AngleSpinBox::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractSpinBox::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void AngleSpinBox::valueChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void AngleSpinBox::valueChangedDeg(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void AngleSpinBox::valueChangedRad(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}
QT_WARNING_POP
