/********************************************************************************
** Form generated from reading UI file 'nebulaTexturesDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_NEBULATEXTURESDIALOG_H
#define UI_NEBULATEXTURESDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTextBrowser>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "AngleSpinBox.hpp"
#include "Dialog.hpp"

QT_BEGIN_NAMESPACE

class Ui_nebulaTexturesDialog
{
public:
    QVBoxLayout *verticalLayout_2;
    TitleBar *titleBar;
    QTabWidget *tabs;
    QWidget *tab_2;
    QVBoxLayout *verticalLayout_6;
    QVBoxLayout *verticalLayout_5;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout_8;
    QGridLayout *gridLayout_2;
    QPushButton *openFileButton;
    QLineEdit *lineEditImagePath;
    QLabel *label_imagePath;
    QLabel *label_apiKey;
    QPushButton *cancelButton;
    QPushButton *solveButton;
    QLineEdit *lineEditApiKey;
    QCheckBox *checkBoxKeepApi;
    QLabel *statusText;
    QGroupBox *groupBox_3;
    QVBoxLayout *verticalLayout_9;
    QGridLayout *gridLayout_3;
    QLabel *label_tld;
    QLabel *label_blr;
    QLabel *label_bld;
    QLabel *label_rr;
    QLabel *label_tlr;
    AngleSpinBox *topLeftX;
    AngleSpinBox *topLeftY;
    AngleSpinBox *bottomLeftX;
    AngleSpinBox *bottomLeftY;
    AngleSpinBox *referY;
    AngleSpinBox *referX;
    QLabel *label_rd;
    QLabel *label_trr;
    QLabel *label_trd;
    QLabel *label_brr;
    QLabel *label_brd;
    AngleSpinBox *topRightX;
    AngleSpinBox *topRightY;
    AngleSpinBox *bottomRightX;
    AngleSpinBox *bottomRightY;
    QPushButton *goPushButton;
    QPushButton *recoverCoordsButton;
    QGroupBox *groupBox_5;
    QGridLayout *gridLayout;
    QCheckBox *disableDefault;
    QPushButton *renderButton;
    QComboBox *brightComboBox;
    QVBoxLayout *verticalLayout_3;
    QPushButton *addCustomTextureButton;
    QPushButton *restoreDefaultsButton;
    QSpacerItem *verticalSpacer;
    QWidget *tab;
    QVBoxLayout *verticalLayout_4;
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout_2;
    QCheckBox *checkBoxShow;
    QCheckBox *checkBoxAvoid;
    QPushButton *reloadButton;
    QListWidget *listWidget;
    QPushButton *removeTextureButton;
    QWidget *aboutTab;
    QVBoxLayout *verticalLayout_11;
    QTextBrowser *aboutTextBrowser;

    void setupUi(QWidget *nebulaTexturesDialog)
    {
        if (nebulaTexturesDialog->objectName().isEmpty())
            nebulaTexturesDialog->setObjectName("nebulaTexturesDialog");
        nebulaTexturesDialog->resize(519, 502);
        verticalLayout_2 = new QVBoxLayout(nebulaTexturesDialog);
        verticalLayout_2->setSpacing(0);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        titleBar = new TitleBar(nebulaTexturesDialog);
        titleBar->setObjectName("titleBar");

        verticalLayout_2->addWidget(titleBar);

        tabs = new QTabWidget(nebulaTexturesDialog);
        tabs->setObjectName("tabs");
        tabs->setDocumentMode(false);
        tab_2 = new QWidget();
        tab_2->setObjectName("tab_2");
        verticalLayout_6 = new QVBoxLayout(tab_2);
        verticalLayout_6->setObjectName("verticalLayout_6");
        verticalLayout_6->setContentsMargins(0, 0, 0, 0);
        verticalLayout_5 = new QVBoxLayout();
        verticalLayout_5->setObjectName("verticalLayout_5");
        groupBox = new QGroupBox(tab_2);
        groupBox->setObjectName("groupBox");
        verticalLayout_8 = new QVBoxLayout(groupBox);
        verticalLayout_8->setObjectName("verticalLayout_8");
        verticalLayout_8->setContentsMargins(0, 0, 0, 0);
        gridLayout_2 = new QGridLayout();
        gridLayout_2->setObjectName("gridLayout_2");
        openFileButton = new QPushButton(groupBox);
        openFileButton->setObjectName("openFileButton");

        gridLayout_2->addWidget(openFileButton, 0, 2, 1, 1);

        lineEditImagePath = new QLineEdit(groupBox);
        lineEditImagePath->setObjectName("lineEditImagePath");

        gridLayout_2->addWidget(lineEditImagePath, 0, 1, 1, 1);

        label_imagePath = new QLabel(groupBox);
        label_imagePath->setObjectName("label_imagePath");

        gridLayout_2->addWidget(label_imagePath, 0, 0, 1, 1);

        label_apiKey = new QLabel(groupBox);
        label_apiKey->setObjectName("label_apiKey");
        label_apiKey->setText(QString::fromUtf8("Astrometry API key:"));
        label_apiKey->setTextFormat(Qt::RichText);
        label_apiKey->setOpenExternalLinks(true);
        label_apiKey->setTextInteractionFlags(Qt::TextBrowserInteraction);

        gridLayout_2->addWidget(label_apiKey, 1, 0, 1, 1);

        cancelButton = new QPushButton(groupBox);
        cancelButton->setObjectName("cancelButton");

        gridLayout_2->addWidget(cancelButton, 2, 2, 1, 1);

        solveButton = new QPushButton(groupBox);
        solveButton->setObjectName("solveButton");

        gridLayout_2->addWidget(solveButton, 2, 1, 1, 1);

        lineEditApiKey = new QLineEdit(groupBox);
        lineEditApiKey->setObjectName("lineEditApiKey");
        lineEditApiKey->setEchoMode(QLineEdit::PasswordEchoOnEdit);

        gridLayout_2->addWidget(lineEditApiKey, 1, 1, 1, 1);

        checkBoxKeepApi = new QCheckBox(groupBox);
        checkBoxKeepApi->setObjectName("checkBoxKeepApi");
        checkBoxKeepApi->setChecked(true);

        gridLayout_2->addWidget(checkBoxKeepApi, 1, 2, 1, 1);


        verticalLayout_8->addLayout(gridLayout_2);

        statusText = new QLabel(groupBox);
        statusText->setObjectName("statusText");
        statusText->setMaximumSize(QSize(16777215, 20));

        verticalLayout_8->addWidget(statusText);


        verticalLayout_5->addWidget(groupBox);


        verticalLayout_6->addLayout(verticalLayout_5);

        groupBox_3 = new QGroupBox(tab_2);
        groupBox_3->setObjectName("groupBox_3");
        verticalLayout_9 = new QVBoxLayout(groupBox_3);
        verticalLayout_9->setObjectName("verticalLayout_9");
        verticalLayout_9->setContentsMargins(0, 0, 0, 0);
        gridLayout_3 = new QGridLayout();
        gridLayout_3->setObjectName("gridLayout_3");
        label_tld = new QLabel(groupBox_3);
        label_tld->setObjectName("label_tld");
        QSizePolicy sizePolicy(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(label_tld->sizePolicy().hasHeightForWidth());
        label_tld->setSizePolicy(sizePolicy);

        gridLayout_3->addWidget(label_tld, 1, 0, 1, 1);

        label_blr = new QLabel(groupBox_3);
        label_blr->setObjectName("label_blr");
        sizePolicy.setHeightForWidth(label_blr->sizePolicy().hasHeightForWidth());
        label_blr->setSizePolicy(sizePolicy);

        gridLayout_3->addWidget(label_blr, 4, 0, 1, 1);

        label_bld = new QLabel(groupBox_3);
        label_bld->setObjectName("label_bld");
        sizePolicy.setHeightForWidth(label_bld->sizePolicy().hasHeightForWidth());
        label_bld->setSizePolicy(sizePolicy);

        gridLayout_3->addWidget(label_bld, 5, 0, 1, 1);

        label_rr = new QLabel(groupBox_3);
        label_rr->setObjectName("label_rr");
        sizePolicy.setHeightForWidth(label_rr->sizePolicy().hasHeightForWidth());
        label_rr->setSizePolicy(sizePolicy);

        gridLayout_3->addWidget(label_rr, 2, 2, 1, 1);

        label_tlr = new QLabel(groupBox_3);
        label_tlr->setObjectName("label_tlr");
        sizePolicy.setHeightForWidth(label_tlr->sizePolicy().hasHeightForWidth());
        label_tlr->setSizePolicy(sizePolicy);

        gridLayout_3->addWidget(label_tlr, 0, 0, 1, 1);

        topLeftX = new AngleSpinBox(groupBox_3);
        topLeftX->setObjectName("topLeftX");
        topLeftX->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_3->addWidget(topLeftX, 0, 1, 1, 1);

        topLeftY = new AngleSpinBox(groupBox_3);
        topLeftY->setObjectName("topLeftY");
        topLeftY->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_3->addWidget(topLeftY, 1, 1, 1, 1);

        bottomLeftX = new AngleSpinBox(groupBox_3);
        bottomLeftX->setObjectName("bottomLeftX");
        bottomLeftX->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_3->addWidget(bottomLeftX, 4, 1, 1, 1);

        bottomLeftY = new AngleSpinBox(groupBox_3);
        bottomLeftY->setObjectName("bottomLeftY");
        bottomLeftY->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_3->addWidget(bottomLeftY, 5, 1, 1, 1);

        referY = new AngleSpinBox(groupBox_3);
        referY->setObjectName("referY");
        referY->setEnabled(true);
        referY->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_3->addWidget(referY, 3, 3, 1, 1);

        referX = new AngleSpinBox(groupBox_3);
        referX->setObjectName("referX");
        referX->setEnabled(true);
        referX->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_3->addWidget(referX, 2, 3, 1, 1);

        label_rd = new QLabel(groupBox_3);
        label_rd->setObjectName("label_rd");
        sizePolicy.setHeightForWidth(label_rd->sizePolicy().hasHeightForWidth());
        label_rd->setSizePolicy(sizePolicy);

        gridLayout_3->addWidget(label_rd, 3, 2, 1, 1);

        label_trr = new QLabel(groupBox_3);
        label_trr->setObjectName("label_trr");
        sizePolicy.setHeightForWidth(label_trr->sizePolicy().hasHeightForWidth());
        label_trr->setSizePolicy(sizePolicy);

        gridLayout_3->addWidget(label_trr, 0, 4, 1, 1);

        label_trd = new QLabel(groupBox_3);
        label_trd->setObjectName("label_trd");
        sizePolicy.setHeightForWidth(label_trd->sizePolicy().hasHeightForWidth());
        label_trd->setSizePolicy(sizePolicy);

        gridLayout_3->addWidget(label_trd, 1, 4, 1, 1);

        label_brr = new QLabel(groupBox_3);
        label_brr->setObjectName("label_brr");
        sizePolicy.setHeightForWidth(label_brr->sizePolicy().hasHeightForWidth());
        label_brr->setSizePolicy(sizePolicy);

        gridLayout_3->addWidget(label_brr, 4, 4, 1, 1);

        label_brd = new QLabel(groupBox_3);
        label_brd->setObjectName("label_brd");
        sizePolicy.setHeightForWidth(label_brd->sizePolicy().hasHeightForWidth());
        label_brd->setSizePolicy(sizePolicy);

        gridLayout_3->addWidget(label_brd, 5, 4, 1, 1);

        topRightX = new AngleSpinBox(groupBox_3);
        topRightX->setObjectName("topRightX");
        topRightX->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_3->addWidget(topRightX, 0, 5, 1, 1);

        topRightY = new AngleSpinBox(groupBox_3);
        topRightY->setObjectName("topRightY");
        topRightY->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_3->addWidget(topRightY, 1, 5, 1, 1);

        bottomRightX = new AngleSpinBox(groupBox_3);
        bottomRightX->setObjectName("bottomRightX");
        bottomRightX->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_3->addWidget(bottomRightX, 4, 5, 1, 1);

        bottomRightY = new AngleSpinBox(groupBox_3);
        bottomRightY->setObjectName("bottomRightY");
        bottomRightY->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_3->addWidget(bottomRightY, 5, 5, 1, 1);

        goPushButton = new QPushButton(groupBox_3);
        goPushButton->setObjectName("goPushButton");

        gridLayout_3->addWidget(goPushButton, 4, 2, 1, 2);

        recoverCoordsButton = new QPushButton(groupBox_3);
        recoverCoordsButton->setObjectName("recoverCoordsButton");

        gridLayout_3->addWidget(recoverCoordsButton, 1, 2, 1, 2);


        verticalLayout_9->addLayout(gridLayout_3);


        verticalLayout_6->addWidget(groupBox_3);

        groupBox_5 = new QGroupBox(tab_2);
        groupBox_5->setObjectName("groupBox_5");
        gridLayout = new QGridLayout(groupBox_5);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setContentsMargins(0, 0, 0, 0);
        disableDefault = new QCheckBox(groupBox_5);
        disableDefault->setObjectName("disableDefault");
        disableDefault->setChecked(true);

        gridLayout->addWidget(disableDefault, 0, 2, 1, 1);

        renderButton = new QPushButton(groupBox_5);
        renderButton->setObjectName("renderButton");
        renderButton->setEnabled(true);
        renderButton->setCheckable(true);

        gridLayout->addWidget(renderButton, 0, 0, 1, 1);

        brightComboBox = new QComboBox(groupBox_5);
        brightComboBox->setObjectName("brightComboBox");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(brightComboBox->sizePolicy().hasHeightForWidth());
        brightComboBox->setSizePolicy(sizePolicy1);
        brightComboBox->setEditable(true);

        gridLayout->addWidget(brightComboBox, 0, 1, 1, 1);


        verticalLayout_6->addWidget(groupBox_5);

        verticalLayout_3 = new QVBoxLayout();
        verticalLayout_3->setObjectName("verticalLayout_3");
        verticalLayout_3->setContentsMargins(9, -1, 9, -1);
        addCustomTextureButton = new QPushButton(tab_2);
        addCustomTextureButton->setObjectName("addCustomTextureButton");

        verticalLayout_3->addWidget(addCustomTextureButton);

        restoreDefaultsButton = new QPushButton(tab_2);
        restoreDefaultsButton->setObjectName("restoreDefaultsButton");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(restoreDefaultsButton->sizePolicy().hasHeightForWidth());
        restoreDefaultsButton->setSizePolicy(sizePolicy2);

        verticalLayout_3->addWidget(restoreDefaultsButton);


        verticalLayout_6->addLayout(verticalLayout_3);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_6->addItem(verticalSpacer);

        tabs->addTab(tab_2, QString());
        tab = new QWidget();
        tab->setObjectName("tab");
        verticalLayout_4 = new QVBoxLayout(tab);
        verticalLayout_4->setObjectName("verticalLayout_4");
        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName("verticalLayout");
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        checkBoxShow = new QCheckBox(tab);
        checkBoxShow->setObjectName("checkBoxShow");
        checkBoxShow->setChecked(true);

        horizontalLayout_2->addWidget(checkBoxShow);

        checkBoxAvoid = new QCheckBox(tab);
        checkBoxAvoid->setObjectName("checkBoxAvoid");
        checkBoxAvoid->setChecked(true);

        horizontalLayout_2->addWidget(checkBoxAvoid);

        reloadButton = new QPushButton(tab);
        reloadButton->setObjectName("reloadButton");

        horizontalLayout_2->addWidget(reloadButton);


        verticalLayout->addLayout(horizontalLayout_2);

        listWidget = new QListWidget(tab);
        listWidget->setObjectName("listWidget");

        verticalLayout->addWidget(listWidget);

        removeTextureButton = new QPushButton(tab);
        removeTextureButton->setObjectName("removeTextureButton");

        verticalLayout->addWidget(removeTextureButton);


        verticalLayout_4->addLayout(verticalLayout);

        tabs->addTab(tab, QString());
        aboutTab = new QWidget();
        aboutTab->setObjectName("aboutTab");
        verticalLayout_11 = new QVBoxLayout(aboutTab);
        verticalLayout_11->setSpacing(9);
        verticalLayout_11->setObjectName("verticalLayout_11");
        aboutTextBrowser = new QTextBrowser(aboutTab);
        aboutTextBrowser->setObjectName("aboutTextBrowser");
        aboutTextBrowser->setOpenExternalLinks(true);
        aboutTextBrowser->setOpenLinks(true);

        verticalLayout_11->addWidget(aboutTextBrowser);

        tabs->addTab(aboutTab, QString());

        verticalLayout_2->addWidget(tabs);

        QWidget::setTabOrder(tabs, aboutTextBrowser);

        retranslateUi(nebulaTexturesDialog);

        tabs->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(nebulaTexturesDialog);
    } // setupUi

    void retranslateUi(QWidget *nebulaTexturesDialog)
    {
        nebulaTexturesDialog->setWindowTitle(QCoreApplication::translate("nebulaTexturesDialog", "Nebula Textures Plug-in Configuration", nullptr));
        titleBar->setProperty("title", QVariant(QCoreApplication::translate("nebulaTexturesDialog", "Nebula Textures Plug-in Configuration", nullptr)));
        groupBox->setTitle(QCoreApplication::translate("nebulaTexturesDialog", "Online plate-solving", nullptr));
        openFileButton->setText(QCoreApplication::translate("nebulaTexturesDialog", "Open File", nullptr));
        label_imagePath->setText(QCoreApplication::translate("nebulaTexturesDialog", "Image Filepath:", nullptr));
        cancelButton->setText(QCoreApplication::translate("nebulaTexturesDialog", "Cancel", nullptr));
        solveButton->setText(QCoreApplication::translate("nebulaTexturesDialog", "Online Solve", nullptr));
        checkBoxKeepApi->setText(QCoreApplication::translate("nebulaTexturesDialog", "Remember me", nullptr));
        statusText->setText(QCoreApplication::translate("nebulaTexturesDialog", "Select and upload astrophotography image to solve coordinates with Astrometry.net", nullptr));
        groupBox_3->setTitle(QCoreApplication::translate("nebulaTexturesDialog", "Goto celestial center and Adjust corner coordinates", nullptr));
#if QT_CONFIG(tooltip)
        label_tld->setToolTip(QCoreApplication::translate("nebulaTexturesDialog", "Declination of top left corner of image", nullptr));
#endif // QT_CONFIG(tooltip)
        label_tld->setText(QCoreApplication::translate("nebulaTexturesDialog", "Dec (\342\206\226)", nullptr));
#if QT_CONFIG(tooltip)
        label_blr->setToolTip(QCoreApplication::translate("nebulaTexturesDialog", "Right ascension of bottom left corner of image", nullptr));
#endif // QT_CONFIG(tooltip)
        label_blr->setText(QCoreApplication::translate("nebulaTexturesDialog", "RA (\342\206\231)", nullptr));
#if QT_CONFIG(tooltip)
        label_bld->setToolTip(QCoreApplication::translate("nebulaTexturesDialog", "Declination of bottom left corner of image", nullptr));
#endif // QT_CONFIG(tooltip)
        label_bld->setText(QCoreApplication::translate("nebulaTexturesDialog", "Dec (\342\206\231)", nullptr));
#if QT_CONFIG(tooltip)
        label_rr->setToolTip(QCoreApplication::translate("nebulaTexturesDialog", "Right ascension of center of image", nullptr));
#endif // QT_CONFIG(tooltip)
        label_rr->setText(QCoreApplication::translate("nebulaTexturesDialog", "RA (Center)", nullptr));
#if QT_CONFIG(tooltip)
        label_tlr->setToolTip(QCoreApplication::translate("nebulaTexturesDialog", "Right ascension of top left corner of image", nullptr));
#endif // QT_CONFIG(tooltip)
        label_tlr->setText(QCoreApplication::translate("nebulaTexturesDialog", "RA (\342\206\226)", nullptr));
#if QT_CONFIG(tooltip)
        label_rd->setToolTip(QCoreApplication::translate("nebulaTexturesDialog", "Declination of center of image", nullptr));
#endif // QT_CONFIG(tooltip)
        label_rd->setText(QCoreApplication::translate("nebulaTexturesDialog", "Dec (Center)", nullptr));
#if QT_CONFIG(tooltip)
        label_trr->setToolTip(QCoreApplication::translate("nebulaTexturesDialog", "Right ascension of top right corner of image", nullptr));
#endif // QT_CONFIG(tooltip)
        label_trr->setText(QCoreApplication::translate("nebulaTexturesDialog", "RA (\342\206\227)", nullptr));
#if QT_CONFIG(tooltip)
        label_trd->setToolTip(QCoreApplication::translate("nebulaTexturesDialog", "Declination of top right corner of image", nullptr));
#endif // QT_CONFIG(tooltip)
        label_trd->setText(QCoreApplication::translate("nebulaTexturesDialog", "Dec (\342\206\227)", nullptr));
#if QT_CONFIG(tooltip)
        label_brr->setToolTip(QCoreApplication::translate("nebulaTexturesDialog", "Right ascension of bottom right corner of image", nullptr));
#endif // QT_CONFIG(tooltip)
        label_brr->setText(QCoreApplication::translate("nebulaTexturesDialog", "RA (\342\206\230)", nullptr));
#if QT_CONFIG(tooltip)
        label_brd->setToolTip(QCoreApplication::translate("nebulaTexturesDialog", "Declination of bottom right corner of image", nullptr));
#endif // QT_CONFIG(tooltip)
        label_brd->setText(QCoreApplication::translate("nebulaTexturesDialog", "Dec (\342\206\230)", nullptr));
        goPushButton->setText(QCoreApplication::translate("nebulaTexturesDialog", "Goto Center", nullptr));
        recoverCoordsButton->setText(QCoreApplication::translate("nebulaTexturesDialog", "Reset", nullptr));
        groupBox_5->setTitle(QCoreApplication::translate("nebulaTexturesDialog", "Rendering tester", nullptr));
        disableDefault->setText(QCoreApplication::translate("nebulaTexturesDialog", "Disable default textures", nullptr));
        renderButton->setText(QCoreApplication::translate("nebulaTexturesDialog", "Test this texture", nullptr));
#if QT_CONFIG(tooltip)
        brightComboBox->setToolTip(QCoreApplication::translate("nebulaTexturesDialog", "Brightness", nullptr));
#endif // QT_CONFIG(tooltip)
        addCustomTextureButton->setText(QCoreApplication::translate("nebulaTexturesDialog", "Add the Texture to Custom Storage", nullptr));
        restoreDefaultsButton->setText(QCoreApplication::translate("nebulaTexturesDialog", "Restore defaults", nullptr));
        tabs->setTabText(tabs->indexOf(tab_2), QCoreApplication::translate("nebulaTexturesDialog", "Import", nullptr));
        checkBoxShow->setText(QCoreApplication::translate("nebulaTexturesDialog", "Show Custom Textures", nullptr));
        checkBoxAvoid->setText(QCoreApplication::translate("nebulaTexturesDialog", "Avoid Area Conflict", nullptr));
        reloadButton->setText(QCoreApplication::translate("nebulaTexturesDialog", "Reload", nullptr));
        removeTextureButton->setText(QCoreApplication::translate("nebulaTexturesDialog", "Remove select item in Custom Textures", nullptr));
        tabs->setTabText(tabs->indexOf(tab), QCoreApplication::translate("nebulaTexturesDialog", "Manage", nullptr));
        tabs->setTabText(tabs->indexOf(aboutTab), QCoreApplication::translate("nebulaTexturesDialog", "About", "tab in plugin windows"));
    } // retranslateUi

};

namespace Ui {
    class nebulaTexturesDialog: public Ui_nebulaTexturesDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_NEBULATEXTURESDIALOG_H
