/********************************************************************************
** Form generated from reading UI file 'equationOfTimeWindow.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_EQUATIONOFTIMEWINDOW_H
#define UI_EQUATIONOFTIMEWINDOW_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTextBrowser>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "Dialog.hpp"
#include "gui/ColorButton.hpp"

QT_BEGIN_NAMESPACE

class Ui_equationOfTimeWindowForm
{
public:
    QVBoxLayout *verticalLayout;
    TitleBar *titleBar;
    QTabWidget *tabWidget;
    QWidget *tabEquationOfTime;
    QVBoxLayout *verticalLayoutTimeZone;
    QGroupBox *groupBoxEOT;
    QVBoxLayout *verticalLayout_2;
    QLabel *labelEOTDescription;
    QFrame *line;
    QCheckBox *checkBoxEnableAtStartup;
    QCheckBox *checkBoxShowButton;
    QCheckBox *checkBoxMsFormat;
    QCheckBox *checkBoxInvertedValue;
    QHBoxLayout *horizontalLayout;
    ColorButton *textColorButton;
    QLabel *labelFontSize;
    QSpinBox *spinBoxFontSize;
    QSpacerItem *verticalSpacerTimeZone;
    QHBoxLayout *horizontalLayout_2;
    QPushButton *pushButtonReset;
    QSpacerItem *horizontalSpacer;
    QPushButton *pushButtonSave;
    QWidget *tabAbout;
    QVBoxLayout *verticalLayout_3;
    QTextBrowser *aboutTextBrowser;

    void setupUi(QWidget *equationOfTimeWindowForm)
    {
        if (equationOfTimeWindowForm->objectName().isEmpty())
            equationOfTimeWindowForm->setObjectName("equationOfTimeWindowForm");
        equationOfTimeWindowForm->resize(662, 461);
        verticalLayout = new QVBoxLayout(equationOfTimeWindowForm);
        verticalLayout->setSpacing(0);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        titleBar = new TitleBar(equationOfTimeWindowForm);
        titleBar->setObjectName("titleBar");

        verticalLayout->addWidget(titleBar);

        tabWidget = new QTabWidget(equationOfTimeWindowForm);
        tabWidget->setObjectName("tabWidget");
        tabEquationOfTime = new QWidget();
        tabEquationOfTime->setObjectName("tabEquationOfTime");
        verticalLayoutTimeZone = new QVBoxLayout(tabEquationOfTime);
        verticalLayoutTimeZone->setSpacing(0);
        verticalLayoutTimeZone->setObjectName("verticalLayoutTimeZone");
        verticalLayoutTimeZone->setContentsMargins(0, 0, 0, 0);
        groupBoxEOT = new QGroupBox(tabEquationOfTime);
        groupBoxEOT->setObjectName("groupBoxEOT");
        groupBoxEOT->setMinimumSize(QSize(0, 400));
        groupBoxEOT->setAlignment(Qt::AlignCenter);
        verticalLayout_2 = new QVBoxLayout(groupBoxEOT);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        labelEOTDescription = new QLabel(groupBoxEOT);
        labelEOTDescription->setObjectName("labelEOTDescription");
        QSizePolicy sizePolicy(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(labelEOTDescription->sizePolicy().hasHeightForWidth());
        labelEOTDescription->setSizePolicy(sizePolicy);
        labelEOTDescription->setWordWrap(true);

        verticalLayout_2->addWidget(labelEOTDescription);

        line = new QFrame(groupBoxEOT);
        line->setObjectName("line");
        line->setMinimumSize(QSize(0, 1));
        line->setMaximumSize(QSize(16777215, 1));
        line->setBaseSize(QSize(0, 1));
        line->setFrameShadow(QFrame::Plain);
        line->setLineWidth(1);
        line->setMidLineWidth(1);
        line->setFrameShape(QFrame::Shape::HLine);

        verticalLayout_2->addWidget(line);

        checkBoxEnableAtStartup = new QCheckBox(groupBoxEOT);
        checkBoxEnableAtStartup->setObjectName("checkBoxEnableAtStartup");

        verticalLayout_2->addWidget(checkBoxEnableAtStartup);

        checkBoxShowButton = new QCheckBox(groupBoxEOT);
        checkBoxShowButton->setObjectName("checkBoxShowButton");

        verticalLayout_2->addWidget(checkBoxShowButton);

        checkBoxMsFormat = new QCheckBox(groupBoxEOT);
        checkBoxMsFormat->setObjectName("checkBoxMsFormat");

        verticalLayout_2->addWidget(checkBoxMsFormat);

        checkBoxInvertedValue = new QCheckBox(groupBoxEOT);
        checkBoxInvertedValue->setObjectName("checkBoxInvertedValue");

        verticalLayout_2->addWidget(checkBoxInvertedValue);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        textColorButton = new ColorButton(groupBoxEOT);
        textColorButton->setObjectName("textColorButton");

        horizontalLayout->addWidget(textColorButton);

        labelFontSize = new QLabel(groupBoxEOT);
        labelFontSize->setObjectName("labelFontSize");

        horizontalLayout->addWidget(labelFontSize);

        spinBoxFontSize = new QSpinBox(groupBoxEOT);
        spinBoxFontSize->setObjectName("spinBoxFontSize");
        spinBoxFontSize->setMinimum(10);
        spinBoxFontSize->setMaximum(50);
        spinBoxFontSize->setValue(20);

        horizontalLayout->addWidget(spinBoxFontSize);


        verticalLayout_2->addLayout(horizontalLayout);

        verticalSpacerTimeZone = new QSpacerItem(0, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_2->addItem(verticalSpacerTimeZone);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        pushButtonReset = new QPushButton(groupBoxEOT);
        pushButtonReset->setObjectName("pushButtonReset");

        horizontalLayout_2->addWidget(pushButtonReset);

        horizontalSpacer = new QSpacerItem(20, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer);

        pushButtonSave = new QPushButton(groupBoxEOT);
        pushButtonSave->setObjectName("pushButtonSave");

        horizontalLayout_2->addWidget(pushButtonSave);


        verticalLayout_2->addLayout(horizontalLayout_2);


        verticalLayoutTimeZone->addWidget(groupBoxEOT);

        tabWidget->addTab(tabEquationOfTime, QString());
        tabAbout = new QWidget();
        tabAbout->setObjectName("tabAbout");
        verticalLayout_3 = new QVBoxLayout(tabAbout);
        verticalLayout_3->setObjectName("verticalLayout_3");
        aboutTextBrowser = new QTextBrowser(tabAbout);
        aboutTextBrowser->setObjectName("aboutTextBrowser");
        aboutTextBrowser->setOpenExternalLinks(true);

        verticalLayout_3->addWidget(aboutTextBrowser);

        tabWidget->addTab(tabAbout, QString());

        verticalLayout->addWidget(tabWidget);


        retranslateUi(equationOfTimeWindowForm);

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(equationOfTimeWindowForm);
    } // setupUi

    void retranslateUi(QWidget *equationOfTimeWindowForm)
    {
        titleBar->setTitle(QCoreApplication::translate("equationOfTimeWindowForm", "Equation of Time", nullptr));
        groupBoxEOT->setTitle(QCoreApplication::translate("equationOfTimeWindowForm", "Equation of Time", nullptr));
        labelEOTDescription->setText(QCoreApplication::translate("equationOfTimeWindowForm", "The equation of time describes the discrepancy between two kinds of solar time. These are apparent solar time, which directly tracks the motion of the sun, and mean solar time, which tracks a fictitious \"mean\" sun with noons 24 hours apart. There is no universally accepted definition of the sign of the equation of time. Some publications show it as positive when a sundial is ahead of a clock; others when the clock is ahead of the sundial. In the English-speaking world, the former usage is the more common, but is not always followed. Anyone who makes use of a published table or graph should first check its sign usage.", nullptr));
        checkBoxEnableAtStartup->setText(QCoreApplication::translate("equationOfTimeWindowForm", "Enable display at startup", nullptr));
        checkBoxShowButton->setText(QCoreApplication::translate("equationOfTimeWindowForm", "Show plug-ins button on toolbar", nullptr));
        checkBoxMsFormat->setText(QCoreApplication::translate("equationOfTimeWindowForm", "Use minutes and seconds", nullptr));
        checkBoxInvertedValue->setText(QCoreApplication::translate("equationOfTimeWindowForm", "Change sign of equation", nullptr));
#if QT_CONFIG(tooltip)
        textColorButton->setToolTip(QCoreApplication::translate("equationOfTimeWindowForm", "Color of text", nullptr));
#endif // QT_CONFIG(tooltip)
        labelFontSize->setText(QCoreApplication::translate("equationOfTimeWindowForm", "Font size:", nullptr));
        pushButtonReset->setText(QCoreApplication::translate("equationOfTimeWindowForm", "Restore defaults", nullptr));
        pushButtonSave->setText(QCoreApplication::translate("equationOfTimeWindowForm", "Save settings", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabEquationOfTime), QCoreApplication::translate("equationOfTimeWindowForm", "Equation of Time", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabAbout), QCoreApplication::translate("equationOfTimeWindowForm", "About", "tab in plugin windows"));
        (void)equationOfTimeWindowForm;
    } // retranslateUi

};

namespace Ui {
    class equationOfTimeWindowForm: public Ui_equationOfTimeWindowForm {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_EQUATIONOFTIMEWINDOW_H
