/****************************************************************************
** Meta object code from reading C++ file 'StelVideoMgr.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../stellarium/src/core/StelVideoMgr.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'StelVideoMgr.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12StelVideoMgrE_t {};
} // unnamed namespace

template <> constexpr inline auto StelVideoMgr::qt_create_metaobjectdata<qt_meta_tag_ZN12StelVideoMgrE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "StelVideoMgr",
        "init",
        "",
        "update",
        "deltaTime",
        "loadVideo",
        "filename",
        "id",
        "x",
        "y",
        "show",
        "alpha",
        "playVideo",
        "keepVisibleAtEnd",
        "playVideoPopout",
        "fromX",
        "fromY",
        "atCenterX",
        "atCenterY",
        "finalSizeX",
        "finalSizeY",
        "popupDuration",
        "frozenInTransition",
        "pauseVideo",
        "stopVideo",
        "dropVideo",
        "seekVideo",
        "ms",
        "pause",
        "setVideoXY",
        "relative",
        "setVideoAlpha",
        "resizeVideo",
        "w",
        "h",
        "showVideo",
        "getVideoDuration",
        "getVideoPosition",
        "getVideoResolution",
        "QSize",
        "getVideoWidth",
        "getVideoHeight",
        "muteVideo",
        "setVideoVolume",
        "newVolume",
        "getVideoVolume",
        "isVideoPlaying",
        "handleAudioAvailableChanged",
        "available",
        "handleDurationChanged",
        "duration",
        "handleMediaStatusChanged",
        "QMediaPlayer::MediaStatus",
        "status",
        "handleMutedChanged",
        "muted",
        "handleSeekableChanged",
        "seekable",
        "handleStateChanged",
        "QMediaPlayer::PlaybackState",
        "state",
        "handleBufferProgressChanged",
        "filled",
        "handleErrorMsg",
        "QMediaPlayer::Error",
        "error",
        "errorString",
        "handleSourceChanged",
        "QUrl",
        "media",
        "handleVideoAvailableChanged",
        "videoAvailable",
        "handleVolumeChanged",
        "volume",
        "handleMetaDataChanged"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'init'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'update'
        QtMocHelpers::SlotData<void(double)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 4 },
        }}),
        // Slot 'loadVideo'
        QtMocHelpers::SlotData<void(const QString &, const QString &, const float, const float, const bool, const float)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 }, { QMetaType::QString, 7 }, { QMetaType::Float, 8 }, { QMetaType::Float, 9 },
            { QMetaType::Bool, 10 }, { QMetaType::Float, 11 },
        }}),
        // Slot 'playVideo'
        QtMocHelpers::SlotData<void(const QString &, const bool)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 }, { QMetaType::Bool, 13 },
        }}),
        // Slot 'playVideo'
        QtMocHelpers::SlotData<void(const QString &)>(12, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
        // Slot 'playVideoPopout'
        QtMocHelpers::SlotData<void(const QString &, float, float, float, float, float, float, float, bool)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 }, { QMetaType::Float, 15 }, { QMetaType::Float, 16 }, { QMetaType::Float, 17 },
            { QMetaType::Float, 18 }, { QMetaType::Float, 19 }, { QMetaType::Float, 20 }, { QMetaType::Float, 21 },
            { QMetaType::Bool, 22 },
        }}),
        // Slot 'pauseVideo'
        QtMocHelpers::SlotData<void(const QString &)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
        // Slot 'stopVideo'
        QtMocHelpers::SlotData<void(const QString &)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
        // Slot 'dropVideo'
        QtMocHelpers::SlotData<void(const QString &)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
        // Slot 'seekVideo'
        QtMocHelpers::SlotData<void(const QString &, const qint64, bool)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 }, { QMetaType::LongLong, 27 }, { QMetaType::Bool, 28 },
        }}),
        // Slot 'seekVideo'
        QtMocHelpers::SlotData<void(const QString &, const qint64)>(26, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 7 }, { QMetaType::LongLong, 27 },
        }}),
        // Slot 'setVideoXY'
        QtMocHelpers::SlotData<void(const QString &, const float, const float, const bool)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 }, { QMetaType::Float, 8 }, { QMetaType::Float, 9 }, { QMetaType::Bool, 30 },
        }}),
        // Slot 'setVideoXY'
        QtMocHelpers::SlotData<void(const QString &, const float, const float)>(29, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 7 }, { QMetaType::Float, 8 }, { QMetaType::Float, 9 },
        }}),
        // Slot 'setVideoAlpha'
        QtMocHelpers::SlotData<void(const QString &, const float)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 }, { QMetaType::Float, 11 },
        }}),
        // Slot 'resizeVideo'
        QtMocHelpers::SlotData<void(const QString &, float, float)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 }, { QMetaType::Float, 33 }, { QMetaType::Float, 34 },
        }}),
        // Slot 'resizeVideo'
        QtMocHelpers::SlotData<void(const QString &, float)>(32, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 7 }, { QMetaType::Float, 33 },
        }}),
        // Slot 'resizeVideo'
        QtMocHelpers::SlotData<void(const QString &)>(32, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
        // Slot 'showVideo'
        QtMocHelpers::SlotData<void(const QString &, const bool)>(35, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 }, { QMetaType::Bool, 10 },
        }}),
        // Slot 'getVideoDuration'
        QtMocHelpers::SlotData<qint64(const QString &) const>(36, 2, QMC::AccessPublic, QMetaType::LongLong, {{
            { QMetaType::QString, 7 },
        }}),
        // Slot 'getVideoPosition'
        QtMocHelpers::SlotData<qint64(const QString &) const>(37, 2, QMC::AccessPublic, QMetaType::LongLong, {{
            { QMetaType::QString, 7 },
        }}),
        // Slot 'getVideoResolution'
        QtMocHelpers::SlotData<QSize(const QString &) const>(38, 2, QMC::AccessPublic, 0x80000000 | 39, {{
            { QMetaType::QString, 7 },
        }}),
        // Slot 'getVideoWidth'
        QtMocHelpers::SlotData<int(const QString &) const>(40, 2, QMC::AccessPublic, QMetaType::Int, {{
            { QMetaType::QString, 7 },
        }}),
        // Slot 'getVideoHeight'
        QtMocHelpers::SlotData<int(const QString &) const>(41, 2, QMC::AccessPublic, QMetaType::Int, {{
            { QMetaType::QString, 7 },
        }}),
        // Slot 'muteVideo'
        QtMocHelpers::SlotData<void(const QString &, bool)>(42, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 }, { QMetaType::Bool, 42 },
        }}),
        // Slot 'muteVideo'
        QtMocHelpers::SlotData<void(const QString &)>(42, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
        // Slot 'setVideoVolume'
        QtMocHelpers::SlotData<void(const QString &, int)>(43, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 }, { QMetaType::Int, 44 },
        }}),
        // Slot 'getVideoVolume'
        QtMocHelpers::SlotData<int(const QString &) const>(45, 2, QMC::AccessPublic, QMetaType::Int, {{
            { QMetaType::QString, 7 },
        }}),
        // Slot 'isVideoPlaying'
        QtMocHelpers::SlotData<bool(const QString &) const>(46, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 7 },
        }}),
        // Slot 'handleAudioAvailableChanged'
        QtMocHelpers::SlotData<void(bool)>(47, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 48 },
        }}),
        // Slot 'handleDurationChanged'
        QtMocHelpers::SlotData<void(qint64)>(49, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::LongLong, 50 },
        }}),
        // Slot 'handleMediaStatusChanged'
        QtMocHelpers::SlotData<void(QMediaPlayer::MediaStatus)>(51, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 52, 53 },
        }}),
        // Slot 'handleMutedChanged'
        QtMocHelpers::SlotData<void(bool)>(54, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 55 },
        }}),
        // Slot 'handleSeekableChanged'
        QtMocHelpers::SlotData<void(bool)>(56, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 57 },
        }}),
        // Slot 'handleStateChanged'
        QtMocHelpers::SlotData<void(QMediaPlayer::PlaybackState)>(58, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 59, 60 },
        }}),
        // Slot 'handleBufferProgressChanged'
        QtMocHelpers::SlotData<void(float)>(61, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Float, 62 },
        }}),
        // Slot 'handleErrorMsg'
        QtMocHelpers::SlotData<void(QMediaPlayer::Error, const QString &)>(63, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 64, 65 }, { QMetaType::QString, 66 },
        }}),
        // Slot 'handleSourceChanged'
        QtMocHelpers::SlotData<void(const QUrl &)>(67, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 68, 69 },
        }}),
        // Slot 'handleVideoAvailableChanged'
        QtMocHelpers::SlotData<void(bool)>(70, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 71 },
        }}),
        // Slot 'handleVolumeChanged'
        QtMocHelpers::SlotData<void(int)>(72, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 73 },
        }}),
        // Slot 'handleMetaDataChanged'
        QtMocHelpers::SlotData<void()>(74, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<StelVideoMgr, qt_meta_tag_ZN12StelVideoMgrE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject StelVideoMgr::staticMetaObject = { {
    QMetaObject::SuperData::link<StelModule::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12StelVideoMgrE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12StelVideoMgrE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12StelVideoMgrE_t>.metaTypes,
    nullptr
} };

void StelVideoMgr::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<StelVideoMgr *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->init(); break;
        case 1: _t->update((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 2: _t->loadVideo((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[6]))); break;
        case 3: _t->playVideo((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 4: _t->playVideo((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->playVideoPopout((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[8])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[9]))); break;
        case 6: _t->pauseVideo((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: _t->stopVideo((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 8: _t->dropVideo((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 9: _t->seekVideo((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qint64>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 10: _t->seekVideo((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qint64>>(_a[2]))); break;
        case 11: _t->setVideoXY((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4]))); break;
        case 12: _t->setVideoXY((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[3]))); break;
        case 13: _t->setVideoAlpha((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[2]))); break;
        case 14: _t->resizeVideo((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[3]))); break;
        case 15: _t->resizeVideo((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[2]))); break;
        case 16: _t->resizeVideo((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 17: _t->showVideo((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 18: { qint64 _r = _t->getVideoDuration((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<qint64*>(_a[0]) = std::move(_r); }  break;
        case 19: { qint64 _r = _t->getVideoPosition((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<qint64*>(_a[0]) = std::move(_r); }  break;
        case 20: { QSize _r = _t->getVideoResolution((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QSize*>(_a[0]) = std::move(_r); }  break;
        case 21: { int _r = _t->getVideoWidth((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 22: { int _r = _t->getVideoHeight((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 23: _t->muteVideo((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 24: _t->muteVideo((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 25: _t->setVideoVolume((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 26: { int _r = _t->getVideoVolume((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 27: { bool _r = _t->isVideoPlaying((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 28: _t->handleAudioAvailableChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 29: _t->handleDurationChanged((*reinterpret_cast<std::add_pointer_t<qint64>>(_a[1]))); break;
        case 30: _t->handleMediaStatusChanged((*reinterpret_cast<std::add_pointer_t<QMediaPlayer::MediaStatus>>(_a[1]))); break;
        case 31: _t->handleMutedChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 32: _t->handleSeekableChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 33: _t->handleStateChanged((*reinterpret_cast<std::add_pointer_t<QMediaPlayer::PlaybackState>>(_a[1]))); break;
        case 34: _t->handleBufferProgressChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 35: _t->handleErrorMsg((*reinterpret_cast<std::add_pointer_t<QMediaPlayer::Error>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 36: _t->handleSourceChanged((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 37: _t->handleVideoAvailableChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 38: _t->handleVolumeChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 39: _t->handleMetaDataChanged(); break;
        default: ;
        }
    }
}

const QMetaObject *StelVideoMgr::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *StelVideoMgr::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12StelVideoMgrE_t>.strings))
        return static_cast<void*>(this);
    return StelModule::qt_metacast(_clname);
}

int StelVideoMgr::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StelModule::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 40)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 40;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 40)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 40;
    }
    return _id;
}
QT_WARNING_POP
