/****************************************************************************
** Meta object code from reading C++ file 'SyncClient.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../stellarium/plugins/RemoteSync/src/SyncClient.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'SyncClient.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10SyncClientE_t {};
} // unnamed namespace

template <> constexpr inline auto SyncClient::qt_create_metaobjectdata<qt_meta_tag_ZN10SyncClientE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "SyncClient",
        "connected",
        "",
        "disconnected",
        "cleanExit",
        "connectToServer",
        "host",
        "port",
        "disconnectFromServer",
        "serverDisconnected",
        "clean",
        "socketConnected",
        "emitServerError",
        "errorStr",
        "SyncOptions",
        "SyncOption",
        "NONE",
        "SyncTime",
        "SyncLocation",
        "SyncSelection",
        "SyncStelProperty",
        "SyncView",
        "SkipGUIProps",
        "ALL"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'connected'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'disconnected'
        QtMocHelpers::SignalData<void(bool)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 4 },
        }}),
        // Slot 'connectToServer'
        QtMocHelpers::SlotData<void(const QString &, const int)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 }, { QMetaType::Int, 7 },
        }}),
        // Slot 'disconnectFromServer'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'serverDisconnected'
        QtMocHelpers::SlotData<void(bool)>(9, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 10 },
        }}),
        // Slot 'socketConnected'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'emitServerError'
        QtMocHelpers::SlotData<void(const QString &)>(12, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 13 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // flag 'SyncOptions'
        QtMocHelpers::EnumData<SyncOptions>(14, 15, QMC::EnumIsFlag).add({
            {   16, SyncOption::NONE },
            {   17, SyncOption::SyncTime },
            {   18, SyncOption::SyncLocation },
            {   19, SyncOption::SyncSelection },
            {   20, SyncOption::SyncStelProperty },
            {   21, SyncOption::SyncView },
            {   22, SyncOption::SkipGUIProps },
            {   23, SyncOption::ALL },
        }),
    };
    return QtMocHelpers::metaObjectData<SyncClient, qt_meta_tag_ZN10SyncClientE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject SyncClient::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10SyncClientE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10SyncClientE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10SyncClientE_t>.metaTypes,
    nullptr
} };

void SyncClient::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SyncClient *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->connected(); break;
        case 1: _t->disconnected((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->connectToServer((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 3: _t->disconnectFromServer(); break;
        case 4: _t->serverDisconnected((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->socketConnected(); break;
        case 6: _t->emitServerError((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (SyncClient::*)()>(_a, &SyncClient::connected, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (SyncClient::*)(bool )>(_a, &SyncClient::disconnected, 1))
            return;
    }
}

const QMetaObject *SyncClient::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SyncClient::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10SyncClientE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int SyncClient::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void SyncClient::connected()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void SyncClient::disconnected(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}
QT_WARNING_POP
