/****************************************************************************
** Meta object code from reading C++ file 'StelScriptMgr.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../stellarium/src/scripting/StelScriptMgr.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'StelScriptMgr.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13StelScriptMgrE_t {};
} // unnamed namespace

template <> constexpr inline auto StelScriptMgr::qt_create_metaobjectdata<qt_meta_tag_ZN13StelScriptMgrE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "StelScriptMgr",
        "runningScriptIdChanged",
        "",
        "id",
        "scriptRunning",
        "scriptPaused",
        "scriptStopped",
        "scriptDebug",
        "scriptOutput",
        "flagAllowExternalScreenshotDirChanged",
        "flag",
        "flagAllowWriteAbsolutePathsChanged",
        "getHtmlDescription",
        "s",
        "generateDocumentTags",
        "getName",
        "getAuthor",
        "getLicense",
        "getVersion",
        "getDescription",
        "getShortcut",
        "runScript",
        "fileName",
        "includePath",
        "runScriptDirect",
        "scriptId",
        "scriptCode",
        "int&",
        "errLoc",
        "runPreprocessedScript",
        "preprocessedScript",
        "prepareScript",
        "QString&",
        "script",
        "stopScript",
        "setScriptRate",
        "r",
        "getScriptRate",
        "debug",
        "msg",
        "output",
        "resetOutput",
        "saveOutputAs",
        "filename",
        "pauseScript",
        "resumeScript",
        "getFlagAllowExternalScreenshotDir",
        "setFlagAllowExternalScreenshotDir",
        "getFlagAllowWriteAbsolutePaths",
        "setFlagAllowWriteAbsolutePaths",
        "scriptEnded",
        "runningScriptId",
        "flagAllowExternalScreenshotDir",
        "flagAllowWriteAbsolutePaths"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'runningScriptIdChanged'
        QtMocHelpers::SignalData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'scriptRunning'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'scriptPaused'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'scriptStopped'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'scriptDebug'
        QtMocHelpers::SignalData<void(const QString &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Signal 'scriptOutput'
        QtMocHelpers::SignalData<void(const QString &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Signal 'flagAllowExternalScreenshotDirChanged'
        QtMocHelpers::SignalData<void(bool)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 10 },
        }}),
        // Signal 'flagAllowWriteAbsolutePathsChanged'
        QtMocHelpers::SignalData<void(bool)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 10 },
        }}),
        // Slot 'getHtmlDescription'
        QtMocHelpers::SlotData<QString(const QString &, bool)>(12, 2, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::QString, 13 }, { QMetaType::Bool, 14 },
        }}),
        // Slot 'getHtmlDescription'
        QtMocHelpers::SlotData<QString(const QString &)>(12, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::QString, {{
            { QMetaType::QString, 13 },
        }}),
        // Slot 'getName'
        QtMocHelpers::SlotData<QString(const QString &)>(15, 2, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::QString, 13 },
        }}),
        // Slot 'getAuthor'
        QtMocHelpers::SlotData<QString(const QString &)>(16, 2, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::QString, 13 },
        }}),
        // Slot 'getLicense'
        QtMocHelpers::SlotData<QString(const QString &)>(17, 2, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::QString, 13 },
        }}),
        // Slot 'getVersion'
        QtMocHelpers::SlotData<QString(const QString &)>(18, 2, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::QString, 13 },
        }}),
        // Slot 'getDescription'
        QtMocHelpers::SlotData<QString(const QString &)>(19, 2, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::QString, 13 },
        }}),
        // Slot 'getShortcut'
        QtMocHelpers::SlotData<QString(const QString &)>(20, 2, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::QString, 13 },
        }}),
        // Slot 'runScript'
        QtMocHelpers::SlotData<bool(const QString &, const QString &)>(21, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 22 }, { QMetaType::QString, 23 },
        }}),
        // Slot 'runScript'
        QtMocHelpers::SlotData<bool(const QString &)>(21, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool, {{
            { QMetaType::QString, 22 },
        }}),
        // Slot 'runScriptDirect'
        QtMocHelpers::SlotData<bool(const QString, const QString &, int &, const QString &)>(24, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 25 }, { QMetaType::QString, 26 }, { 0x80000000 | 27, 28 }, { QMetaType::QString, 23 },
        }}),
        // Slot 'runScriptDirect'
        QtMocHelpers::SlotData<bool(const QString, const QString &, int &)>(24, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool, {{
            { QMetaType::QString, 25 }, { QMetaType::QString, 26 }, { 0x80000000 | 27, 28 },
        }}),
        // Slot 'runScriptDirect'
        QtMocHelpers::SlotData<bool(const QString &, const QString &)>(24, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 26 }, { QMetaType::QString, 23 },
        }}),
        // Slot 'runScriptDirect'
        QtMocHelpers::SlotData<bool(const QString &)>(24, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool, {{
            { QMetaType::QString, 26 },
        }}),
        // Slot 'runPreprocessedScript'
        QtMocHelpers::SlotData<bool(const QString &, const QString &)>(29, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 30 }, { QMetaType::QString, 25 },
        }}),
        // Slot 'prepareScript'
        QtMocHelpers::SlotData<bool(QString &, const QString &, const QString &)>(31, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 32, 33 }, { QMetaType::QString, 22 }, { QMetaType::QString, 23 },
        }}),
        // Slot 'prepareScript'
        QtMocHelpers::SlotData<bool(QString &, const QString &)>(31, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool, {{
            { 0x80000000 | 32, 33 }, { QMetaType::QString, 22 },
        }}),
        // Slot 'stopScript'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setScriptRate'
        QtMocHelpers::SlotData<void(double)>(35, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 36 },
        }}),
        // Slot 'getScriptRate'
        QtMocHelpers::SlotData<double() const>(37, 2, QMC::AccessPublic, QMetaType::Double),
        // Slot 'debug'
        QtMocHelpers::SlotData<void(const QString &)>(38, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 39 },
        }}),
        // Slot 'output'
        QtMocHelpers::SlotData<void(const QString &)>(40, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 39 },
        }}),
        // Slot 'resetOutput'
        QtMocHelpers::SlotData<void()>(41, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'saveOutputAs'
        QtMocHelpers::SlotData<void(const QString &)>(42, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 43 },
        }}),
        // Slot 'pauseScript'
        QtMocHelpers::SlotData<void()>(44, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'resumeScript'
        QtMocHelpers::SlotData<void()>(45, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'getFlagAllowExternalScreenshotDir'
        QtMocHelpers::SlotData<bool() const>(46, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFlagAllowExternalScreenshotDir'
        QtMocHelpers::SlotData<void(bool)>(47, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 10 },
        }}),
        // Slot 'getFlagAllowWriteAbsolutePaths'
        QtMocHelpers::SlotData<bool() const>(48, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFlagAllowWriteAbsolutePaths'
        QtMocHelpers::SlotData<void(bool)>(49, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 10 },
        }}),
        // Slot 'scriptEnded'
        QtMocHelpers::SlotData<void()>(50, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'runningScriptId'
        QtMocHelpers::PropertyData<QString>(51, QMetaType::QString, QMC::DefaultPropertyFlags, 0),
        // property 'flagAllowExternalScreenshotDir'
        QtMocHelpers::PropertyData<bool>(52, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 6),
        // property 'flagAllowWriteAbsolutePaths'
        QtMocHelpers::PropertyData<bool>(53, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 7),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<StelScriptMgr, qt_meta_tag_ZN13StelScriptMgrE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject StelScriptMgr::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13StelScriptMgrE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13StelScriptMgrE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13StelScriptMgrE_t>.metaTypes,
    nullptr
} };

void StelScriptMgr::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<StelScriptMgr *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->runningScriptIdChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->scriptRunning(); break;
        case 2: _t->scriptPaused(); break;
        case 3: _t->scriptStopped(); break;
        case 4: _t->scriptDebug((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->scriptOutput((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->flagAllowExternalScreenshotDirChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->flagAllowWriteAbsolutePathsChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: { QString _r = _t->getHtmlDescription((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 9: { QString _r = _t->getHtmlDescription((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 10: { QString _r = _t->getName((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 11: { QString _r = _t->getAuthor((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 12: { QString _r = _t->getLicense((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 13: { QString _r = _t->getVersion((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 14: { QString _r = _t->getDescription((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 15: { QString _r = _t->getShortcut((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 16: { bool _r = _t->runScript((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 17: { bool _r = _t->runScript((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 18: { bool _r = _t->runScriptDirect((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int&>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 19: { bool _r = _t->runScriptDirect((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int&>>(_a[3])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 20: { bool _r = _t->runScriptDirect((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 21: { bool _r = _t->runScriptDirect((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 22: { bool _r = _t->runPreprocessedScript((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 23: { bool _r = _t->prepareScript((*reinterpret_cast<std::add_pointer_t<QString&>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 24: { bool _r = _t->prepareScript((*reinterpret_cast<std::add_pointer_t<QString&>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 25: _t->stopScript(); break;
        case 26: _t->setScriptRate((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 27: { double _r = _t->getScriptRate();
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 28: _t->debug((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 29: _t->output((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 30: _t->resetOutput(); break;
        case 31: _t->saveOutputAs((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 32: _t->pauseScript(); break;
        case 33: _t->resumeScript(); break;
        case 34: { bool _r = _t->getFlagAllowExternalScreenshotDir();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 35: _t->setFlagAllowExternalScreenshotDir((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 36: { bool _r = _t->getFlagAllowWriteAbsolutePaths();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 37: _t->setFlagAllowWriteAbsolutePaths((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 38: _t->scriptEnded(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (StelScriptMgr::*)(const QString & )>(_a, &StelScriptMgr::runningScriptIdChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelScriptMgr::*)()>(_a, &StelScriptMgr::scriptRunning, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelScriptMgr::*)()>(_a, &StelScriptMgr::scriptPaused, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelScriptMgr::*)()>(_a, &StelScriptMgr::scriptStopped, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelScriptMgr::*)(const QString & )>(_a, &StelScriptMgr::scriptDebug, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelScriptMgr::*)(const QString & )>(_a, &StelScriptMgr::scriptOutput, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelScriptMgr::*)(bool )>(_a, &StelScriptMgr::flagAllowExternalScreenshotDirChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelScriptMgr::*)(bool )>(_a, &StelScriptMgr::flagAllowWriteAbsolutePathsChanged, 7))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->runningScriptId(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->getFlagAllowExternalScreenshotDir(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->getFlagAllowWriteAbsolutePaths(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setFlagAllowExternalScreenshotDir(*reinterpret_cast<bool*>(_v)); break;
        case 2: _t->setFlagAllowWriteAbsolutePaths(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *StelScriptMgr::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *StelScriptMgr::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13StelScriptMgrE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int StelScriptMgr::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 39)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 39;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 39)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 39;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void StelScriptMgr::runningScriptIdChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void StelScriptMgr::scriptRunning()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void StelScriptMgr::scriptPaused()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void StelScriptMgr::scriptStopped()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void StelScriptMgr::scriptDebug(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void StelScriptMgr::scriptOutput(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void StelScriptMgr::flagAllowExternalScreenshotDirChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void StelScriptMgr::flagAllowWriteAbsolutePathsChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}
QT_WARNING_POP
