/********************************************************************************
** Form generated from reading UI file 'archaeoLinesDialogLocations.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_ARCHAEOLINESDIALOGLOCATIONS_H
#define UI_ARCHAEOLINESDIALOGLOCATIONS_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "Dialog.hpp"
#include "gui/StelCloseButton.hpp"

QT_BEGIN_NAMESPACE

class Ui_archaeoLinesDialogLocations
{
public:
    QVBoxLayout *vboxLayout;
    TitleBar *titleBar;
    QFrame *locationContent;
    QVBoxLayout *verticalLayout;
    QListView *citiesListView;
    QLineEdit *citySearchLineEdit;
    QHBoxLayout *horizontalLayout;

    void setupUi(QWidget *archaeoLinesDialogLocations)
    {
        if (archaeoLinesDialogLocations->objectName().isEmpty())
            archaeoLinesDialogLocations->setObjectName("archaeoLinesDialogLocations");
        archaeoLinesDialogLocations->resize(258, 362);
        archaeoLinesDialogLocations->setStyleSheet(QString::fromUtf8(""));
        vboxLayout = new QVBoxLayout(archaeoLinesDialogLocations);
        vboxLayout->setSpacing(0);
        vboxLayout->setObjectName("vboxLayout");
        vboxLayout->setContentsMargins(0, 0, 0, 0);
        titleBar = new TitleBar(archaeoLinesDialogLocations);
        titleBar->setObjectName("titleBar");

        vboxLayout->addWidget(titleBar);

        locationContent = new QFrame(archaeoLinesDialogLocations);
        locationContent->setObjectName("locationContent");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(locationContent->sizePolicy().hasHeightForWidth());
        locationContent->setSizePolicy(sizePolicy);
        locationContent->setFrameShape(QFrame::StyledPanel);
        verticalLayout = new QVBoxLayout(locationContent);
        verticalLayout->setSpacing(0);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        citiesListView = new QListView(locationContent);
        citiesListView->setObjectName("citiesListView");
        citiesListView->setFocusPolicy(Qt::NoFocus);
        citiesListView->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        citiesListView->setEditTriggers(QAbstractItemView::NoEditTriggers);
        citiesListView->setUniformItemSizes(true);

        verticalLayout->addWidget(citiesListView);

        citySearchLineEdit = new QLineEdit(locationContent);
        citySearchLineEdit->setObjectName("citySearchLineEdit");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(citySearchLineEdit->sizePolicy().hasHeightForWidth());
        citySearchLineEdit->setSizePolicy(sizePolicy1);
        citySearchLineEdit->setAlignment(Qt::AlignBottom|Qt::AlignLeading|Qt::AlignLeft);

        verticalLayout->addWidget(citySearchLineEdit);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");

        verticalLayout->addLayout(horizontalLayout);


        vboxLayout->addWidget(locationContent);


        retranslateUi(archaeoLinesDialogLocations);

        QMetaObject::connectSlotsByName(archaeoLinesDialogLocations);
    } // setupUi

    void retranslateUi(QWidget *archaeoLinesDialogLocations)
    {
        titleBar->setTitle(QCoreApplication::translate("archaeoLinesDialogLocations", "Locations", nullptr));
#if QT_CONFIG(tooltip)
        citiesListView->setToolTip(QCoreApplication::translate("archaeoLinesDialogLocations", "Standard list of locations contains only English names (without diacritics!)", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        citySearchLineEdit->setToolTip(QCoreApplication::translate("archaeoLinesDialogLocations", "Standard list of locations contains only English names (without diacritics!)", nullptr));
#endif // QT_CONFIG(tooltip)
        citySearchLineEdit->setText(QString());
        (void)archaeoLinesDialogLocations;
    } // retranslateUi

};

namespace Ui {
    class archaeoLinesDialogLocations: public Ui_archaeoLinesDialogLocations {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_ARCHAEOLINESDIALOGLOCATIONS_H
