/********************************************************************************
** Form generated from reading UI file 'remoteControlDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_REMOTECONTROLDIALOG_H
#define UI_REMOTECONTROLDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTextBrowser>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "Dialog.hpp"
#include "gui/StelCloseButton.hpp"

QT_BEGIN_NAMESPACE

class Ui_remoteControlDialog
{
public:
    QVBoxLayout *verticalLayout_2;
    TitleBar *titleBar;
    QTabWidget *tabs;
    QWidget *settingsTab;
    QFormLayout *formLayout;
    QLabel *labelPortNumber;
    QSpacerItem *verticalSpacer_3;
    QHBoxLayout *horizontalLayout_7;
    QPushButton *restoreDefaultsButton;
    QSpacerItem *horizontalSpacer;
    QPushButton *saveSettingsButton;
    QCheckBox *passwordCheckBox;
    QLabel *labelPassword;
    QLineEdit *passwordEdit;
    QLabel *labelNote;
    QSpinBox *portNumberSpinBox;
    QCheckBox *enableCorsCheckbox;
    QLineEdit *corsOriginEdit;
    QLabel *labelCors;
    QWidget *restartPanel;
    QVBoxLayout *verticalLayout;
    QLabel *label_4;
    QPushButton *resetButton;
    QCheckBox *enabledCheckbox;
    QCheckBox *activateOnStartCheckBox;
    QLabel *label_RemoteRunningState;
    QWidget *aboutTab;
    QVBoxLayout *verticalLayout_11;
    QTextBrowser *aboutTextBrowser;

    void setupUi(QWidget *remoteControlDialog)
    {
        if (remoteControlDialog->objectName().isEmpty())
            remoteControlDialog->setObjectName("remoteControlDialog");
        remoteControlDialog->resize(666, 525);
        verticalLayout_2 = new QVBoxLayout(remoteControlDialog);
        verticalLayout_2->setSpacing(0);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        titleBar = new TitleBar(remoteControlDialog);
        titleBar->setObjectName("titleBar");

        verticalLayout_2->addWidget(titleBar);

        tabs = new QTabWidget(remoteControlDialog);
        tabs->setObjectName("tabs");
        tabs->setDocumentMode(false);
        settingsTab = new QWidget();
        settingsTab->setObjectName("settingsTab");
        formLayout = new QFormLayout(settingsTab);
        formLayout->setObjectName("formLayout");
        labelPortNumber = new QLabel(settingsTab);
        labelPortNumber->setObjectName("labelPortNumber");

        formLayout->setWidget(7, QFormLayout::ItemRole::LabelRole, labelPortNumber);

        verticalSpacer_3 = new QSpacerItem(20, 120, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        formLayout->setItem(15, QFormLayout::ItemRole::LabelRole, verticalSpacer_3);

        horizontalLayout_7 = new QHBoxLayout();
        horizontalLayout_7->setObjectName("horizontalLayout_7");
        restoreDefaultsButton = new QPushButton(settingsTab);
        restoreDefaultsButton->setObjectName("restoreDefaultsButton");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(restoreDefaultsButton->sizePolicy().hasHeightForWidth());
        restoreDefaultsButton->setSizePolicy(sizePolicy);

        horizontalLayout_7->addWidget(restoreDefaultsButton);

        horizontalSpacer = new QSpacerItem(20, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout_7->addItem(horizontalSpacer);

        saveSettingsButton = new QPushButton(settingsTab);
        saveSettingsButton->setObjectName("saveSettingsButton");
        sizePolicy.setHeightForWidth(saveSettingsButton->sizePolicy().hasHeightForWidth());
        saveSettingsButton->setSizePolicy(sizePolicy);

        horizontalLayout_7->addWidget(saveSettingsButton);


        formLayout->setLayout(16, QFormLayout::ItemRole::SpanningRole, horizontalLayout_7);

        passwordCheckBox = new QCheckBox(settingsTab);
        passwordCheckBox->setObjectName("passwordCheckBox");

        formLayout->setWidget(8, QFormLayout::ItemRole::SpanningRole, passwordCheckBox);

        labelPassword = new QLabel(settingsTab);
        labelPassword->setObjectName("labelPassword");

        formLayout->setWidget(9, QFormLayout::ItemRole::LabelRole, labelPassword);

        passwordEdit = new QLineEdit(settingsTab);
        passwordEdit->setObjectName("passwordEdit");

        formLayout->setWidget(9, QFormLayout::ItemRole::FieldRole, passwordEdit);

        labelNote = new QLabel(settingsTab);
        labelNote->setObjectName("labelNote");
        QFont font;
        font.setItalic(true);
        labelNote->setFont(font);

        formLayout->setWidget(10, QFormLayout::ItemRole::SpanningRole, labelNote);

        portNumberSpinBox = new QSpinBox(settingsTab);
        portNumberSpinBox->setObjectName("portNumberSpinBox");
        portNumberSpinBox->setMinimum(1);
        portNumberSpinBox->setMaximum(65535);

        formLayout->setWidget(7, QFormLayout::ItemRole::FieldRole, portNumberSpinBox);

        enableCorsCheckbox = new QCheckBox(settingsTab);
        enableCorsCheckbox->setObjectName("enableCorsCheckbox");

        formLayout->setWidget(11, QFormLayout::ItemRole::LabelRole, enableCorsCheckbox);

        corsOriginEdit = new QLineEdit(settingsTab);
        corsOriginEdit->setObjectName("corsOriginEdit");

        formLayout->setWidget(11, QFormLayout::ItemRole::FieldRole, corsOriginEdit);

        labelCors = new QLabel(settingsTab);
        labelCors->setObjectName("labelCors");

        formLayout->setWidget(12, QFormLayout::ItemRole::SpanningRole, labelCors);

        restartPanel = new QWidget(settingsTab);
        restartPanel->setObjectName("restartPanel");
        verticalLayout = new QVBoxLayout(restartPanel);
        verticalLayout->setObjectName("verticalLayout");
        label_4 = new QLabel(restartPanel);
        label_4->setObjectName("label_4");

        verticalLayout->addWidget(label_4);

        resetButton = new QPushButton(restartPanel);
        resetButton->setObjectName("resetButton");

        verticalLayout->addWidget(resetButton);


        formLayout->setWidget(13, QFormLayout::ItemRole::SpanningRole, restartPanel);

        enabledCheckbox = new QCheckBox(settingsTab);
        enabledCheckbox->setObjectName("enabledCheckbox");

        formLayout->setWidget(2, QFormLayout::ItemRole::LabelRole, enabledCheckbox);

        activateOnStartCheckBox = new QCheckBox(settingsTab);
        activateOnStartCheckBox->setObjectName("activateOnStartCheckBox");
        activateOnStartCheckBox->setEnabled(true);

        formLayout->setWidget(2, QFormLayout::ItemRole::FieldRole, activateOnStartCheckBox);

        label_RemoteRunningState = new QLabel(settingsTab);
        label_RemoteRunningState->setObjectName("label_RemoteRunningState");

        formLayout->setWidget(6, QFormLayout::ItemRole::LabelRole, label_RemoteRunningState);

        tabs->addTab(settingsTab, QString());
        aboutTab = new QWidget();
        aboutTab->setObjectName("aboutTab");
        verticalLayout_11 = new QVBoxLayout(aboutTab);
        verticalLayout_11->setSpacing(9);
        verticalLayout_11->setObjectName("verticalLayout_11");
        aboutTextBrowser = new QTextBrowser(aboutTab);
        aboutTextBrowser->setObjectName("aboutTextBrowser");
        aboutTextBrowser->setOpenExternalLinks(true);
        aboutTextBrowser->setOpenLinks(true);

        verticalLayout_11->addWidget(aboutTextBrowser);

        tabs->addTab(aboutTab, QString());

        verticalLayout_2->addWidget(tabs);

#if QT_CONFIG(shortcut)
        labelPortNumber->setBuddy(portNumberSpinBox);
        labelPassword->setBuddy(passwordEdit);
#endif // QT_CONFIG(shortcut)

        retranslateUi(remoteControlDialog);

        tabs->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(remoteControlDialog);
    } // setupUi

    void retranslateUi(QWidget *remoteControlDialog)
    {
        remoteControlDialog->setWindowTitle(QCoreApplication::translate("remoteControlDialog", "Remote Control Configuration", nullptr));
        titleBar->setTitle(QCoreApplication::translate("remoteControlDialog", "Remote Control Plug-in Configuration", nullptr));
        labelPortNumber->setText(QCoreApplication::translate("remoteControlDialog", "Port Number:", nullptr));
        restoreDefaultsButton->setText(QCoreApplication::translate("remoteControlDialog", "Restore defaults", nullptr));
        saveSettingsButton->setText(QCoreApplication::translate("remoteControlDialog", "Save settings", nullptr));
        passwordCheckBox->setText(QCoreApplication::translate("remoteControlDialog", "Access requires authentication", nullptr));
        labelPassword->setText(QCoreApplication::translate("remoteControlDialog", "Password", nullptr));
        labelNote->setText(QCoreApplication::translate("remoteControlDialog", "<html><head/><body><p>Note: The user name field in the browser must be left empty. </p><p>The password is transmitted over an insecure channel.</p></body></html>", nullptr));
        enableCorsCheckbox->setText(QCoreApplication::translate("remoteControlDialog", "Enable CORS for the following origin", nullptr));
        labelCors->setText(QCoreApplication::translate("remoteControlDialog", "Cross-Origin Resource Sharing lets the specified website access and control your Stellarium.<br/>Specify \"*\" to let any website take control - do this at your own risk.", nullptr));
        label_4->setText(QCoreApplication::translate("remoteControlDialog", "Your changes require a restart of the server.", nullptr));
        resetButton->setText(QCoreApplication::translate("remoteControlDialog", "Restart server", nullptr));
        enabledCheckbox->setText(QCoreApplication::translate("remoteControlDialog", "Server enabled", nullptr));
        activateOnStartCheckBox->setText(QCoreApplication::translate("remoteControlDialog", "Enable automatically on startup", nullptr));
        label_RemoteRunningState->setText(QCoreApplication::translate("remoteControlDialog", "Server disabled - or Server listening on IP: aaa.bbb.ccc.ddd", nullptr));
        tabs->setTabText(tabs->indexOf(settingsTab), QCoreApplication::translate("remoteControlDialog", "Remote Control", nullptr));
        tabs->setTabText(tabs->indexOf(aboutTab), QCoreApplication::translate("remoteControlDialog", "About", "tab in plugin windows"));
    } // retranslateUi

};

namespace Ui {
    class remoteControlDialog: public Ui_remoteControlDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_REMOTECONTROLDIALOG_H
