/********************************************************************************
** Form generated from reading UI file 'scmSkyCultureExportDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SCMSKYCULTUREEXPORTDIALOG_H
#define UI_SCMSKYCULTUREEXPORTDIALOG_H

#include <Dialog.hpp>
#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPlainTextEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_scmSkyCultureExportDialog
{
public:
    QVBoxLayout *verticalLayout;
    TitleBar *titleBar;
    QFrame *Content;
    QVBoxLayout *contentLayout;
    QLabel *titleLbl;
    QPlainTextEdit *plainTextEdit;
    QSpacerItem *verticalSpacer;
    QLabel *label;
    QCheckBox *mergeLinesCB;
    QHBoxLayout *btnLayout;
    QPushButton *exportBtn;
    QPushButton *exportAndExitBtn;
    QPushButton *cancelBtn;

    void setupUi(QWidget *scmSkyCultureExportDialog)
    {
        if (scmSkyCultureExportDialog->objectName().isEmpty())
            scmSkyCultureExportDialog->setObjectName("scmSkyCultureExportDialog");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(scmSkyCultureExportDialog->sizePolicy().hasHeightForWidth());
        scmSkyCultureExportDialog->setSizePolicy(sizePolicy);
        scmSkyCultureExportDialog->setStyleSheet(QString::fromUtf8(""));
        verticalLayout = new QVBoxLayout(scmSkyCultureExportDialog);
        verticalLayout->setSpacing(0);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        titleBar = new TitleBar(scmSkyCultureExportDialog);
        titleBar->setObjectName("titleBar");

        verticalLayout->addWidget(titleBar);

        Content = new QFrame(scmSkyCultureExportDialog);
        Content->setObjectName("Content");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(Content->sizePolicy().hasHeightForWidth());
        Content->setSizePolicy(sizePolicy1);
        Content->setFrameShape(QFrame::StyledPanel);
        contentLayout = new QVBoxLayout(Content);
        contentLayout->setSpacing(10);
        contentLayout->setObjectName("contentLayout");
        contentLayout->setContentsMargins(10, 10, 10, 10);
        titleLbl = new QLabel(Content);
        titleLbl->setObjectName("titleLbl");
        titleLbl->setAlignment(Qt::AlignCenter);
        titleLbl->setWordWrap(true);
        titleLbl->setMargin(0);

        contentLayout->addWidget(titleLbl);

        plainTextEdit = new QPlainTextEdit(Content);
        plainTextEdit->setObjectName("plainTextEdit");
        sizePolicy.setHeightForWidth(plainTextEdit->sizePolicy().hasHeightForWidth());
        plainTextEdit->setSizePolicy(sizePolicy);
        plainTextEdit->setReadOnly(true);

        contentLayout->addWidget(plainTextEdit);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        contentLayout->addItem(verticalSpacer);

        label = new QLabel(Content);
        label->setObjectName("label");
        label->setWordWrap(true);

        contentLayout->addWidget(label);

        mergeLinesCB = new QCheckBox(Content);
        mergeLinesCB->setObjectName("mergeLinesCB");

        contentLayout->addWidget(mergeLinesCB);


        verticalLayout->addWidget(Content, 0, Qt::AlignTop);

        btnLayout = new QHBoxLayout();
        btnLayout->setSpacing(5);
        btnLayout->setObjectName("btnLayout");
        btnLayout->setContentsMargins(10, 10, 10, 10);
        exportBtn = new QPushButton(scmSkyCultureExportDialog);
        exportBtn->setObjectName("exportBtn");

        btnLayout->addWidget(exportBtn);

        exportAndExitBtn = new QPushButton(scmSkyCultureExportDialog);
        exportAndExitBtn->setObjectName("exportAndExitBtn");

        btnLayout->addWidget(exportAndExitBtn);

        cancelBtn = new QPushButton(scmSkyCultureExportDialog);
        cancelBtn->setObjectName("cancelBtn");

        btnLayout->addWidget(cancelBtn);


        verticalLayout->addLayout(btnLayout);


        retranslateUi(scmSkyCultureExportDialog);

        QMetaObject::connectSlotsByName(scmSkyCultureExportDialog);
    } // setupUi

    void retranslateUi(QWidget *scmSkyCultureExportDialog)
    {
        titleBar->setProperty("title", QVariant(QCoreApplication::translate("scmSkyCultureExportDialog", "SCM: Export", nullptr)));
        titleLbl->setText(QCoreApplication::translate("scmSkyCultureExportDialog", "Before exporting the Sky Culture, please make sure that:", nullptr));
        plainTextEdit->setPlainText(QCoreApplication::translate("scmSkyCultureExportDialog", "1. You fully agree that the Sky Culture will be subject to the license you have selected.\n"
"2. You have described the Sky Culture as elaborately as possible.\n"
"3. You have the necessary rights to use and share all included content (e.g., artwork, descriptions).\n"
"4. All information provided is accurate to the best of your knowledge.\n"
"5. The content is shared in good faith, with respect for cultural significance and sensitivities.\n"
"6. You have provided proper attributions where required (e.g., sources of images, texts, or names).", nullptr));
        label->setText(QCoreApplication::translate("scmSkyCultureExportDialog", "On export, you will be prompted to select your personal skycultures directory. A new directory for the sky culture will be created inside.", nullptr));
        mergeLinesCB->setText(QCoreApplication::translate("scmSkyCultureExportDialog", "Merge constellation lines into polylines on export", nullptr));
        exportBtn->setText(QCoreApplication::translate("scmSkyCultureExportDialog", "Export", nullptr));
        exportAndExitBtn->setText(QCoreApplication::translate("scmSkyCultureExportDialog", "Export and Exit", nullptr));
        cancelBtn->setText(QCoreApplication::translate("scmSkyCultureExportDialog", "Cancel", nullptr));
        (void)scmSkyCultureExportDialog;
    } // retranslateUi

};

namespace Ui {
    class scmSkyCultureExportDialog: public Ui_scmSkyCultureExportDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SCMSKYCULTUREEXPORTDIALOG_H
