/********************************************************************************
** Form generated from reading UI file 'customDeltaTEquationDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CUSTOMDELTATEQUATIONDIALOG_H
#define UI_CUSTOMDELTATEQUATIONDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "Dialog.hpp"
#include "gui/StelCloseButton.hpp"

QT_BEGIN_NAMESPACE

class Ui_customDeltaTEquationDialogForm
{
public:
    QVBoxLayout *verticalLayout;
    TitleBar *titleBar;
    QGroupBox *customDeltaTEquationGroupBox;
    QVBoxLayout *verticalLayout_3;
    QLabel *labelDescription;
    QLabel *labelEquation;
    QLabel *labelSubEquation;
    QHBoxLayout *horizontalLayout;
    QLabel *labelCoefficientA;
    QLineEdit *lineEditCoefficientA;
    QLabel *labelCoefficientB;
    QLineEdit *lineEditCoefficientB;
    QLabel *labelCoefficientC;
    QLineEdit *lineEditCoefficientC;
    QHBoxLayout *horizontalLayout_5;
    QLabel *labelYear;
    QLineEdit *lineEditYear;
    QLabel *labelNDot;
    QLineEdit *lineEditNDot;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *customDeltaTEquationDialogForm)
    {
        if (customDeltaTEquationDialogForm->objectName().isEmpty())
            customDeltaTEquationDialogForm->setObjectName("customDeltaTEquationDialogForm");
        customDeltaTEquationDialogForm->resize(335, 204);
        verticalLayout = new QVBoxLayout(customDeltaTEquationDialogForm);
        verticalLayout->setSpacing(0);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        titleBar = new TitleBar(customDeltaTEquationDialogForm);
        titleBar->setObjectName("titleBar");

        verticalLayout->addWidget(titleBar);

        customDeltaTEquationGroupBox = new QGroupBox(customDeltaTEquationDialogForm);
        customDeltaTEquationGroupBox->setObjectName("customDeltaTEquationGroupBox");
        verticalLayout_3 = new QVBoxLayout(customDeltaTEquationGroupBox);
        verticalLayout_3->setObjectName("verticalLayout_3");
        verticalLayout_3->setContentsMargins(0, 0, 0, 0);
        labelDescription = new QLabel(customDeltaTEquationGroupBox);
        labelDescription->setObjectName("labelDescription");
        QSizePolicy sizePolicy(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::MinimumExpanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(labelDescription->sizePolicy().hasHeightForWidth());
        labelDescription->setSizePolicy(sizePolicy);
        labelDescription->setText(QString::fromUtf8("description"));
        labelDescription->setWordWrap(true);

        verticalLayout_3->addWidget(labelDescription);

        labelEquation = new QLabel(customDeltaTEquationGroupBox);
        labelEquation->setObjectName("labelEquation");
        labelEquation->setText(QString::fromUtf8("equation"));
        labelEquation->setTextFormat(Qt::RichText);
        labelEquation->setAlignment(Qt::AlignCenter);

        verticalLayout_3->addWidget(labelEquation);

        labelSubEquation = new QLabel(customDeltaTEquationGroupBox);
        labelSubEquation->setObjectName("labelSubEquation");
        labelSubEquation->setText(QString::fromUtf8("subequation"));

        verticalLayout_3->addWidget(labelSubEquation);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        labelCoefficientA = new QLabel(customDeltaTEquationGroupBox);
        labelCoefficientA->setObjectName("labelCoefficientA");
        labelCoefficientA->setText(QString::fromUtf8("a:"));

        horizontalLayout->addWidget(labelCoefficientA);

        lineEditCoefficientA = new QLineEdit(customDeltaTEquationGroupBox);
        lineEditCoefficientA->setObjectName("lineEditCoefficientA");
        lineEditCoefficientA->setInputMethodHints(Qt::ImhDigitsOnly);
        lineEditCoefficientA->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        horizontalLayout->addWidget(lineEditCoefficientA);

        labelCoefficientB = new QLabel(customDeltaTEquationGroupBox);
        labelCoefficientB->setObjectName("labelCoefficientB");
        labelCoefficientB->setText(QString::fromUtf8("b:"));

        horizontalLayout->addWidget(labelCoefficientB);

        lineEditCoefficientB = new QLineEdit(customDeltaTEquationGroupBox);
        lineEditCoefficientB->setObjectName("lineEditCoefficientB");
        lineEditCoefficientB->setInputMethodHints(Qt::ImhDigitsOnly);
        lineEditCoefficientB->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        horizontalLayout->addWidget(lineEditCoefficientB);

        labelCoefficientC = new QLabel(customDeltaTEquationGroupBox);
        labelCoefficientC->setObjectName("labelCoefficientC");
        labelCoefficientC->setText(QString::fromUtf8("c:"));

        horizontalLayout->addWidget(labelCoefficientC);

        lineEditCoefficientC = new QLineEdit(customDeltaTEquationGroupBox);
        lineEditCoefficientC->setObjectName("lineEditCoefficientC");
        lineEditCoefficientC->setInputMethodHints(Qt::ImhDigitsOnly);
        lineEditCoefficientC->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        horizontalLayout->addWidget(lineEditCoefficientC);


        verticalLayout_3->addLayout(horizontalLayout);

        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        labelYear = new QLabel(customDeltaTEquationGroupBox);
        labelYear->setObjectName("labelYear");
        labelYear->setText(QString::fromUtf8("y:"));

        horizontalLayout_5->addWidget(labelYear);

        lineEditYear = new QLineEdit(customDeltaTEquationGroupBox);
        lineEditYear->setObjectName("lineEditYear");
        lineEditYear->setInputMethodHints(Qt::ImhDigitsOnly);
        lineEditYear->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        horizontalLayout_5->addWidget(lineEditYear);

        labelNDot = new QLabel(customDeltaTEquationGroupBox);
        labelNDot->setObjectName("labelNDot");
        labelNDot->setText(QString::fromUtf8("ndot:"));

        horizontalLayout_5->addWidget(labelNDot);

        lineEditNDot = new QLineEdit(customDeltaTEquationGroupBox);
        lineEditNDot->setObjectName("lineEditNDot");
        lineEditNDot->setInputMethodHints(Qt::ImhDigitsOnly);
        lineEditNDot->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        horizontalLayout_5->addWidget(lineEditNDot);


        verticalLayout_3->addLayout(horizontalLayout_5);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_3->addItem(verticalSpacer);


        verticalLayout->addWidget(customDeltaTEquationGroupBox);


        retranslateUi(customDeltaTEquationDialogForm);

        QMetaObject::connectSlotsByName(customDeltaTEquationDialogForm);
    } // setupUi

    void retranslateUi(QWidget *customDeltaTEquationDialogForm)
    {
        titleBar->setTitle(QString());
        customDeltaTEquationGroupBox->setTitle(QCoreApplication::translate("customDeltaTEquationDialogForm", "Custom equation", nullptr));
        (void)customDeltaTEquationDialogForm;
    } // retranslateUi

};

namespace Ui {
    class customDeltaTEquationDialogForm: public Ui_customDeltaTEquationDialogForm {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_CUSTOMDELTATEQUATIONDIALOG_H
