/****************************************************************************
** Meta object code from reading C++ file 'MainService.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../stellarium/plugins/RemoteControl/src/MainService.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'MainService.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11MainServiceE_t {};
} // unnamed namespace

template <> constexpr inline auto MainService::qt_create_metaobjectdata<qt_meta_tag_ZN11MainServiceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MainService",
        "getSelectedObject",
        "StelObjectP",
        "",
        "getInfoString",
        "focusObject",
        "name",
        "MainService::SelectionMode",
        "mode",
        "focusPosition",
        "Vec3d",
        "pos",
        "updateMovement",
        "x",
        "y",
        "xUpdated",
        "yUpdated",
        "updateView",
        "az",
        "alt",
        "azUpdated",
        "altUpdated",
        "setFov",
        "fov",
        "setWindowSize",
        "width",
        "height",
        "actionToggled",
        "id",
        "val",
        "propertyChanged",
        "StelProperty*",
        "prop",
        "QVariant",
        "SelectionMode",
        "Center",
        "Zoom",
        "Mark"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'getSelectedObject'
        QtMocHelpers::SlotData<StelObjectP()>(1, 3, QMC::AccessPrivate, 0x80000000 | 2),
        // Slot 'getInfoString'
        QtMocHelpers::SlotData<QString()>(4, 3, QMC::AccessPrivate, QMetaType::QString),
        // Slot 'focusObject'
        QtMocHelpers::SlotData<bool(const QString &, MainService::SelectionMode)>(5, 3, QMC::AccessPrivate, QMetaType::Bool, {{
            { QMetaType::QString, 6 }, { 0x80000000 | 7, 8 },
        }}),
        // Slot 'focusPosition'
        QtMocHelpers::SlotData<void(const Vec3d &)>(9, 3, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Slot 'updateMovement'
        QtMocHelpers::SlotData<void(double, double, bool, bool)>(12, 3, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 13 }, { QMetaType::Double, 14 }, { QMetaType::Bool, 15 }, { QMetaType::Bool, 16 },
        }}),
        // Slot 'updateView'
        QtMocHelpers::SlotData<void(double, double, bool, bool)>(17, 3, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 18 }, { QMetaType::Double, 19 }, { QMetaType::Bool, 20 }, { QMetaType::Bool, 21 },
        }}),
        // Slot 'setFov'
        QtMocHelpers::SlotData<void(double)>(22, 3, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 23 },
        }}),
        // Slot 'setWindowSize'
        QtMocHelpers::SlotData<void(const int, const int)>(24, 3, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 25 }, { QMetaType::Int, 26 },
        }}),
        // Slot 'actionToggled'
        QtMocHelpers::SlotData<void(const QString &, bool)>(27, 3, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 28 }, { QMetaType::Bool, 29 },
        }}),
        // Slot 'propertyChanged'
        QtMocHelpers::SlotData<void(StelProperty *, const QVariant &)>(30, 3, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 31, 32 }, { 0x80000000 | 33, 29 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'SelectionMode'
        QtMocHelpers::EnumData<enum SelectionMode>(34, 34, QMC::EnumFlags{}).add({
            {   35, SelectionMode::Center },
            {   36, SelectionMode::Zoom },
            {   37, SelectionMode::Mark },
        }),
    };
    return QtMocHelpers::metaObjectData<MainService, qt_meta_tag_ZN11MainServiceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MainService::staticMetaObject = { {
    QMetaObject::SuperData::link<AbstractAPIService::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11MainServiceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11MainServiceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11MainServiceE_t>.metaTypes,
    nullptr
} };

void MainService::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MainService *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { StelObjectP _r = _t->getSelectedObject();
            if (_a[0]) *reinterpret_cast<StelObjectP*>(_a[0]) = std::move(_r); }  break;
        case 1: { QString _r = _t->getInfoString();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 2: { bool _r = _t->focusObject((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<MainService::SelectionMode>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 3: _t->focusPosition((*reinterpret_cast<std::add_pointer_t<Vec3d>>(_a[1]))); break;
        case 4: _t->updateMovement((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4]))); break;
        case 5: _t->updateView((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4]))); break;
        case 6: _t->setFov((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 7: _t->setWindowSize((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 8: _t->actionToggled((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 9: _t->propertyChanged((*reinterpret_cast<std::add_pointer_t<StelProperty*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3d >(); break;
            }
            break;
        }
    }
}

const QMetaObject *MainService::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MainService::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11MainServiceE_t>.strings))
        return static_cast<void*>(this);
    return AbstractAPIService::qt_metacast(_clname);
}

int MainService::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = AbstractAPIService::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    return _id;
}
QT_WARNING_POP
