/********************************************************************************
** Form generated from reading UI file 'mpcImportWindow.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MPCIMPORTWINDOW_H
#define UI_MPCIMPORTWINDOW_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListView>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "Dialog.hpp"
#include "gui/StelCloseButton.hpp"

QT_BEGIN_NAMESPACE

class Ui_mpcImportWindow
{
public:
    QVBoxLayout *verticalLayout_2;
    TitleBar *titleBar;
    QStackedWidget *stackedWidget;
    QWidget *pageSource;
    QVBoxLayout *verticalLayout_7;
    QTabWidget *tabWidget;
    QWidget *tabLists;
    QVBoxLayout *verticalLayout;
    QGroupBox *groupBoxType;
    QVBoxLayout *verticalLayoutType_3;
    QRadioButton *radioButtonAsteroids;
    QRadioButton *radioButtonComets;
    QGroupBox *groupBoxSource;
    QVBoxLayout *verticalLayout_8;
    QRadioButton *radioButtonURL;
    QFrame *frameURL;
    QVBoxLayout *verticalLayout_9;
    QLabel *labelSelect;
    QComboBox *comboBoxBookmarks;
    QLabel *labelEnter;
    QFrame *frameBookmarkURL;
    QHBoxLayout *horizontalLayoutBookmarkURL;
    QLineEdit *lineEditURL;
    QCheckBox *checkBoxAddBookmark;
    QFrame *frameBookmarkTitle;
    QHBoxLayout *horizontalLayoutBookmarkName;
    QLabel *labelBookmarkTitle;
    QLineEdit *lineEditBookmarkTitle;
    QRadioButton *radioButtonFile;
    QFrame *frameFile;
    QHBoxLayout *horizontalLayoutFile_2;
    QLineEdit *lineEditFilePath;
    QPushButton *pushButtonBrowse;
    QPushButton *pushButtonAcquire;
    QPushButton *pushButtonAbortDownload;
    QSpacerItem *verticalSpacer;
    QSpacerItem *verticalSpacer_2;
    QWidget *tabSearch;
    QVBoxLayout *verticalLayout_11;
    QGroupBox *groupBoxSearch;
    QVBoxLayout *verticalLayout_10;
    QLabel *labelQueryLink;
    QFrame *frameSearch;
    QHBoxLayout *horizontalLayout;
    QLineEdit *lineEditQuery;
    QPushButton *pushButtonSendQuery;
    QPushButton *pushButtonAbortQuery;
    QLabel *labelQueryMessage;
    QLabel *labelQueryInstructions;
    QSpacerItem *verticalSpacerSearch;
    QWidget *pageResult;
    QVBoxLayout *verticalLayout_3;
    QGroupBox *groupBoxObjects;
    QVBoxLayout *verticalLayout_4;
    QLabel *labelInstructions;
    QRadioButton *radioButtonOverwrite;
    QRadioButton *radioButtonUpdate;
    QCheckBox *checkBoxOnlyOrbitalElements;
    QLineEdit *lineEditSearch;
    QLabel *labelNumberOfObjects;
    QListView *listViewObjects;
    QFrame *frameMarkButtons;
    QHBoxLayout *horizontalLayoutMarkButtons;
    QPushButton *pushButtonMarkAll;
    QPushButton *pushButtonMarkNone;
    QPushButton *pushButtonDiscard;
    QPushButton *pushButtonAdd;

    void setupUi(QWidget *mpcImportWindow)
    {
        if (mpcImportWindow->objectName().isEmpty())
            mpcImportWindow->setObjectName("mpcImportWindow");
        mpcImportWindow->resize(476, 515);
        verticalLayout_2 = new QVBoxLayout(mpcImportWindow);
        verticalLayout_2->setSpacing(0);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        titleBar = new TitleBar(mpcImportWindow);
        titleBar->setObjectName("titleBar");

        verticalLayout_2->addWidget(titleBar);

        stackedWidget = new QStackedWidget(mpcImportWindow);
        stackedWidget->setObjectName("stackedWidget");
        pageSource = new QWidget();
        pageSource->setObjectName("pageSource");
        verticalLayout_7 = new QVBoxLayout(pageSource);
        verticalLayout_7->setSpacing(0);
        verticalLayout_7->setObjectName("verticalLayout_7");
        verticalLayout_7->setContentsMargins(0, 0, 0, 0);
        tabWidget = new QTabWidget(pageSource);
        tabWidget->setObjectName("tabWidget");
        tabLists = new QWidget();
        tabLists->setObjectName("tabLists");
        verticalLayout = new QVBoxLayout(tabLists);
        verticalLayout->setSpacing(0);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        groupBoxType = new QGroupBox(tabLists);
        groupBoxType->setObjectName("groupBoxType");
        verticalLayoutType_3 = new QVBoxLayout(groupBoxType);
        verticalLayoutType_3->setObjectName("verticalLayoutType_3");
        verticalLayoutType_3->setContentsMargins(0, 0, 0, 0);
        radioButtonAsteroids = new QRadioButton(groupBoxType);
        radioButtonAsteroids->setObjectName("radioButtonAsteroids");

        verticalLayoutType_3->addWidget(radioButtonAsteroids);

        radioButtonComets = new QRadioButton(groupBoxType);
        radioButtonComets->setObjectName("radioButtonComets");

        verticalLayoutType_3->addWidget(radioButtonComets);


        verticalLayout->addWidget(groupBoxType);

        groupBoxSource = new QGroupBox(tabLists);
        groupBoxSource->setObjectName("groupBoxSource");
        verticalLayout_8 = new QVBoxLayout(groupBoxSource);
        verticalLayout_8->setObjectName("verticalLayout_8");
        verticalLayout_8->setContentsMargins(0, 0, 0, 0);
        radioButtonURL = new QRadioButton(groupBoxSource);
        radioButtonURL->setObjectName("radioButtonURL");

        verticalLayout_8->addWidget(radioButtonURL);

        frameURL = new QFrame(groupBoxSource);
        frameURL->setObjectName("frameURL");
        frameURL->setFrameShape(QFrame::StyledPanel);
        frameURL->setFrameShadow(QFrame::Raised);
        verticalLayout_9 = new QVBoxLayout(frameURL);
        verticalLayout_9->setObjectName("verticalLayout_9");
        verticalLayout_9->setContentsMargins(-1, 0, -1, 0);
        labelSelect = new QLabel(frameURL);
        labelSelect->setObjectName("labelSelect");

        verticalLayout_9->addWidget(labelSelect);

        comboBoxBookmarks = new QComboBox(frameURL);
        comboBoxBookmarks->setObjectName("comboBoxBookmarks");
        comboBoxBookmarks->setEditable(true);
        comboBoxBookmarks->setInsertPolicy(QComboBox::NoInsert);

        verticalLayout_9->addWidget(comboBoxBookmarks);

        labelEnter = new QLabel(frameURL);
        labelEnter->setObjectName("labelEnter");

        verticalLayout_9->addWidget(labelEnter);

        frameBookmarkURL = new QFrame(frameURL);
        frameBookmarkURL->setObjectName("frameBookmarkURL");
        horizontalLayoutBookmarkURL = new QHBoxLayout(frameBookmarkURL);
        horizontalLayoutBookmarkURL->setObjectName("horizontalLayoutBookmarkURL");
        horizontalLayoutBookmarkURL->setContentsMargins(0, 0, 0, 0);
        lineEditURL = new QLineEdit(frameBookmarkURL);
        lineEditURL->setObjectName("lineEditURL");
        lineEditURL->setInputMethodHints(Qt::ImhUrlCharactersOnly);
        lineEditURL->setText(QString::fromUtf8("http://"));

        horizontalLayoutBookmarkURL->addWidget(lineEditURL);


        verticalLayout_9->addWidget(frameBookmarkURL);

        checkBoxAddBookmark = new QCheckBox(frameURL);
        checkBoxAddBookmark->setObjectName("checkBoxAddBookmark");

        verticalLayout_9->addWidget(checkBoxAddBookmark);

        frameBookmarkTitle = new QFrame(frameURL);
        frameBookmarkTitle->setObjectName("frameBookmarkTitle");
        horizontalLayoutBookmarkName = new QHBoxLayout(frameBookmarkTitle);
        horizontalLayoutBookmarkName->setObjectName("horizontalLayoutBookmarkName");
        horizontalLayoutBookmarkName->setContentsMargins(0, 0, 0, 0);
        labelBookmarkTitle = new QLabel(frameBookmarkTitle);
        labelBookmarkTitle->setObjectName("labelBookmarkTitle");

        horizontalLayoutBookmarkName->addWidget(labelBookmarkTitle);

        lineEditBookmarkTitle = new QLineEdit(frameBookmarkTitle);
        lineEditBookmarkTitle->setObjectName("lineEditBookmarkTitle");

        horizontalLayoutBookmarkName->addWidget(lineEditBookmarkTitle);


        verticalLayout_9->addWidget(frameBookmarkTitle);


        verticalLayout_8->addWidget(frameURL);

        radioButtonFile = new QRadioButton(groupBoxSource);
        radioButtonFile->setObjectName("radioButtonFile");

        verticalLayout_8->addWidget(radioButtonFile);

        frameFile = new QFrame(groupBoxSource);
        frameFile->setObjectName("frameFile");
        horizontalLayoutFile_2 = new QHBoxLayout(frameFile);
        horizontalLayoutFile_2->setObjectName("horizontalLayoutFile_2");
        horizontalLayoutFile_2->setContentsMargins(-1, 0, -1, 0);
        lineEditFilePath = new QLineEdit(frameFile);
        lineEditFilePath->setObjectName("lineEditFilePath");
        lineEditFilePath->setInputMethodHints(Qt::ImhNone);

        horizontalLayoutFile_2->addWidget(lineEditFilePath);

        pushButtonBrowse = new QPushButton(frameFile);
        pushButtonBrowse->setObjectName("pushButtonBrowse");
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/graphicGui/uibtFolder.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        pushButtonBrowse->setIcon(icon);

        horizontalLayoutFile_2->addWidget(pushButtonBrowse);


        verticalLayout_8->addWidget(frameFile);

        pushButtonAcquire = new QPushButton(groupBoxSource);
        pushButtonAcquire->setObjectName("pushButtonAcquire");

        verticalLayout_8->addWidget(pushButtonAcquire);

        pushButtonAbortDownload = new QPushButton(groupBoxSource);
        pushButtonAbortDownload->setObjectName("pushButtonAbortDownload");

        verticalLayout_8->addWidget(pushButtonAbortDownload);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_8->addItem(verticalSpacer);


        verticalLayout->addWidget(groupBoxSource);

        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer_2);

        tabWidget->addTab(tabLists, QString());
        tabSearch = new QWidget();
        tabSearch->setObjectName("tabSearch");
        verticalLayout_11 = new QVBoxLayout(tabSearch);
        verticalLayout_11->setObjectName("verticalLayout_11");
        groupBoxSearch = new QGroupBox(tabSearch);
        groupBoxSearch->setObjectName("groupBoxSearch");
        verticalLayout_10 = new QVBoxLayout(groupBoxSearch);
        verticalLayout_10->setObjectName("verticalLayout_10");
        verticalLayout_10->setContentsMargins(0, 0, 0, 0);
        labelQueryLink = new QLabel(groupBoxSearch);
        labelQueryLink->setObjectName("labelQueryLink");
        labelQueryLink->setText(QString::fromUtf8("Query the MPC's <a href=\"https://www.minorplanetcenter.net/iau/MPEph/MPEph.html\">Minor Planet &amp; Comet Ephemeris Service</a>:"));
        labelQueryLink->setWordWrap(true);
        labelQueryLink->setOpenExternalLinks(true);

        verticalLayout_10->addWidget(labelQueryLink);

        frameSearch = new QFrame(groupBoxSearch);
        frameSearch->setObjectName("frameSearch");
        frameSearch->setFrameShape(QFrame::StyledPanel);
        frameSearch->setFrameShadow(QFrame::Raised);
        horizontalLayout = new QHBoxLayout(frameSearch);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        lineEditQuery = new QLineEdit(frameSearch);
        lineEditQuery->setObjectName("lineEditQuery");

        horizontalLayout->addWidget(lineEditQuery);

        pushButtonSendQuery = new QPushButton(frameSearch);
        pushButtonSendQuery->setObjectName("pushButtonSendQuery");
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/graphicGui/uibtSearch.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        pushButtonSendQuery->setIcon(icon1);

        horizontalLayout->addWidget(pushButtonSendQuery);

        pushButtonAbortQuery = new QPushButton(frameSearch);
        pushButtonAbortQuery->setObjectName("pushButtonAbortQuery");
        QIcon icon2;
        icon2.addFile(QString::fromUtf8(":/graphicGui/uibtClose.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        pushButtonAbortQuery->setIcon(icon2);

        horizontalLayout->addWidget(pushButtonAbortQuery);


        verticalLayout_10->addWidget(frameSearch);

        labelQueryMessage = new QLabel(groupBoxSearch);
        labelQueryMessage->setObjectName("labelQueryMessage");
        labelQueryMessage->setAlignment(Qt::AlignCenter);

        verticalLayout_10->addWidget(labelQueryMessage);

        labelQueryInstructions = new QLabel(groupBoxSearch);
        labelQueryInstructions->setObjectName("labelQueryInstructions");
        labelQueryInstructions->setText(QString::fromUtf8("Only one result will be returned if the query is successful.<br>Both comets and asteroids can be identified with their number, name or IAU designation.<br>Comet <i>names</i> need to be prefixed with <b>C/</b> or <b>P/</b>. If more than one comet matches a name, only the first result will be returned. For example, \"C/Halley\" will return 1P/Halley, Halley's Comet, but \"Halley\" will return the asteroid (2688) Halley."));
        labelQueryInstructions->setWordWrap(true);

        verticalLayout_10->addWidget(labelQueryInstructions);


        verticalLayout_11->addWidget(groupBoxSearch);

        verticalSpacerSearch = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_11->addItem(verticalSpacerSearch);

        tabWidget->addTab(tabSearch, QString());

        verticalLayout_7->addWidget(tabWidget);

        stackedWidget->addWidget(pageSource);
        pageResult = new QWidget();
        pageResult->setObjectName("pageResult");
        verticalLayout_3 = new QVBoxLayout(pageResult);
        verticalLayout_3->setSpacing(0);
        verticalLayout_3->setObjectName("verticalLayout_3");
        verticalLayout_3->setContentsMargins(0, 0, 0, 0);
        groupBoxObjects = new QGroupBox(pageResult);
        groupBoxObjects->setObjectName("groupBoxObjects");
        verticalLayout_4 = new QVBoxLayout(groupBoxObjects);
        verticalLayout_4->setObjectName("verticalLayout_4");
        verticalLayout_4->setContentsMargins(0, 0, 0, 0);
        labelInstructions = new QLabel(groupBoxObjects);
        labelInstructions->setObjectName("labelInstructions");
        labelInstructions->setWordWrap(true);

        verticalLayout_4->addWidget(labelInstructions);

        radioButtonOverwrite = new QRadioButton(groupBoxObjects);
        radioButtonOverwrite->setObjectName("radioButtonOverwrite");

        verticalLayout_4->addWidget(radioButtonOverwrite);

        radioButtonUpdate = new QRadioButton(groupBoxObjects);
        radioButtonUpdate->setObjectName("radioButtonUpdate");

        verticalLayout_4->addWidget(radioButtonUpdate);

        checkBoxOnlyOrbitalElements = new QCheckBox(groupBoxObjects);
        checkBoxOnlyOrbitalElements->setObjectName("checkBoxOnlyOrbitalElements");
        checkBoxOnlyOrbitalElements->setChecked(true);

        verticalLayout_4->addWidget(checkBoxOnlyOrbitalElements);

        lineEditSearch = new QLineEdit(groupBoxObjects);
        lineEditSearch->setObjectName("lineEditSearch");

        verticalLayout_4->addWidget(lineEditSearch);

        labelNumberOfObjects = new QLabel(groupBoxObjects);
        labelNumberOfObjects->setObjectName("labelNumberOfObjects");

        verticalLayout_4->addWidget(labelNumberOfObjects);

        listViewObjects = new QListView(groupBoxObjects);
        listViewObjects->setObjectName("listViewObjects");
        listViewObjects->setSelectionMode(QAbstractItemView::NoSelection);
        listViewObjects->setUniformItemSizes(true);

        verticalLayout_4->addWidget(listViewObjects);

        frameMarkButtons = new QFrame(groupBoxObjects);
        frameMarkButtons->setObjectName("frameMarkButtons");
        frameMarkButtons->setFrameShape(QFrame::StyledPanel);
        frameMarkButtons->setFrameShadow(QFrame::Raised);
        horizontalLayoutMarkButtons = new QHBoxLayout(frameMarkButtons);
        horizontalLayoutMarkButtons->setObjectName("horizontalLayoutMarkButtons");
        horizontalLayoutMarkButtons->setContentsMargins(0, 0, 0, 0);
        pushButtonMarkAll = new QPushButton(frameMarkButtons);
        pushButtonMarkAll->setObjectName("pushButtonMarkAll");

        horizontalLayoutMarkButtons->addWidget(pushButtonMarkAll);

        pushButtonMarkNone = new QPushButton(frameMarkButtons);
        pushButtonMarkNone->setObjectName("pushButtonMarkNone");

        horizontalLayoutMarkButtons->addWidget(pushButtonMarkNone);

        pushButtonDiscard = new QPushButton(frameMarkButtons);
        pushButtonDiscard->setObjectName("pushButtonDiscard");

        horizontalLayoutMarkButtons->addWidget(pushButtonDiscard);


        verticalLayout_4->addWidget(frameMarkButtons);

        pushButtonAdd = new QPushButton(groupBoxObjects);
        pushButtonAdd->setObjectName("pushButtonAdd");

        verticalLayout_4->addWidget(pushButtonAdd);


        verticalLayout_3->addWidget(groupBoxObjects);

        stackedWidget->addWidget(pageResult);

        verticalLayout_2->addWidget(stackedWidget);

        QWidget::setTabOrder(tabWidget, radioButtonAsteroids);
        QWidget::setTabOrder(radioButtonAsteroids, radioButtonComets);
        QWidget::setTabOrder(radioButtonComets, radioButtonURL);
        QWidget::setTabOrder(radioButtonURL, comboBoxBookmarks);
        QWidget::setTabOrder(comboBoxBookmarks, lineEditURL);
        QWidget::setTabOrder(lineEditURL, checkBoxAddBookmark);
        QWidget::setTabOrder(checkBoxAddBookmark, lineEditBookmarkTitle);
        QWidget::setTabOrder(lineEditBookmarkTitle, radioButtonFile);
        QWidget::setTabOrder(radioButtonFile, lineEditFilePath);
        QWidget::setTabOrder(lineEditFilePath, pushButtonBrowse);
        QWidget::setTabOrder(pushButtonBrowse, pushButtonAcquire);
        QWidget::setTabOrder(pushButtonAcquire, pushButtonAbortDownload);
        QWidget::setTabOrder(pushButtonAbortDownload, radioButtonOverwrite);
        QWidget::setTabOrder(radioButtonOverwrite, radioButtonUpdate);
        QWidget::setTabOrder(radioButtonUpdate, checkBoxOnlyOrbitalElements);
        QWidget::setTabOrder(checkBoxOnlyOrbitalElements, lineEditSearch);
        QWidget::setTabOrder(lineEditSearch, listViewObjects);
        QWidget::setTabOrder(listViewObjects, pushButtonMarkAll);
        QWidget::setTabOrder(pushButtonMarkAll, pushButtonMarkNone);
        QWidget::setTabOrder(pushButtonMarkNone, pushButtonDiscard);
        QWidget::setTabOrder(pushButtonDiscard, pushButtonAdd);
        QWidget::setTabOrder(pushButtonAdd, pushButtonSendQuery);
        QWidget::setTabOrder(pushButtonSendQuery, lineEditQuery);
        QWidget::setTabOrder(lineEditQuery, pushButtonAbortQuery);

        retranslateUi(mpcImportWindow);
        QObject::connect(radioButtonFile, &QRadioButton::clicked, pushButtonBrowse, qOverload<>(&QPushButton::setFocus));
        QObject::connect(radioButtonURL, &QRadioButton::clicked, lineEditURL, qOverload<>(&QLineEdit::setFocus));
        QObject::connect(radioButtonUpdate, &QRadioButton::toggled, checkBoxOnlyOrbitalElements, &QCheckBox::setEnabled);
        QObject::connect(checkBoxAddBookmark, &QCheckBox::toggled, frameBookmarkTitle, &QFrame::setVisible);

        stackedWidget->setCurrentIndex(0);
        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(mpcImportWindow);
    } // setupUi

    void retranslateUi(QWidget *mpcImportWindow)
    {
        titleBar->setTitle(QCoreApplication::translate("mpcImportWindow", "Import data", nullptr));
        groupBoxType->setTitle(QCoreApplication::translate("mpcImportWindow", "Select the type", nullptr));
        radioButtonAsteroids->setText(QCoreApplication::translate("mpcImportWindow", "Asteroids", nullptr));
        radioButtonComets->setText(QCoreApplication::translate("mpcImportWindow", "Comets", nullptr));
        groupBoxSource->setTitle(QCoreApplication::translate("mpcImportWindow", "Select the source", nullptr));
        radioButtonURL->setText(QCoreApplication::translate("mpcImportWindow", "Download a list of objects from the Internet", nullptr));
        labelSelect->setText(QCoreApplication::translate("mpcImportWindow", "Select a source from the list:", nullptr));
        labelEnter->setText(QCoreApplication::translate("mpcImportWindow", "Or enter a URL:", nullptr));
        checkBoxAddBookmark->setText(QCoreApplication::translate("mpcImportWindow", "Add this URL to the bookmarks list", nullptr));
        labelBookmarkTitle->setText(QCoreApplication::translate("mpcImportWindow", "Bookmark title:", nullptr));
        radioButtonFile->setText(QCoreApplication::translate("mpcImportWindow", "A file containing a list of objects", nullptr));
        pushButtonAcquire->setText(QCoreApplication::translate("mpcImportWindow", "Get orbital elements", nullptr));
        pushButtonAbortDownload->setText(QCoreApplication::translate("mpcImportWindow", "Abort download", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabLists), QCoreApplication::translate("mpcImportWindow", "Lists", nullptr));
        groupBoxSearch->setTitle(QCoreApplication::translate("mpcImportWindow", "Online search", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabSearch), QCoreApplication::translate("mpcImportWindow", "Online search", nullptr));
        groupBoxObjects->setTitle(QCoreApplication::translate("mpcImportWindow", "Objects found", nullptr));
        labelInstructions->setText(QCoreApplication::translate("mpcImportWindow", "Mark the objects you wish to be imported. <i>italics</i> indicate objects with names matching those of already loaded objects. <br/><br/>Note that adding a large number of objects may cause Stellarium to run slowly.", nullptr));
#if QT_CONFIG(tooltip)
        radioButtonOverwrite->setToolTip(QCoreApplication::translate("mpcImportWindow", "This will update existing objects only.", nullptr));
#endif // QT_CONFIG(tooltip)
        radioButtonOverwrite->setText(QCoreApplication::translate("mpcImportWindow", "Overwrite existing objects only", nullptr));
#if QT_CONFIG(tooltip)
        radioButtonUpdate->setToolTip(QCoreApplication::translate("mpcImportWindow", "This will add new objects AND update existing ones.", nullptr));
#endif // QT_CONFIG(tooltip)
        radioButtonUpdate->setText(QCoreApplication::translate("mpcImportWindow", "Add new and update existing objects", nullptr));
#if QT_CONFIG(tooltip)
        checkBoxOnlyOrbitalElements->setToolTip(QCoreApplication::translate("mpcImportWindow", "If not checked, also update the type (comet, asteroid, ...) and magnitude data.", nullptr));
#endif // QT_CONFIG(tooltip)
        checkBoxOnlyOrbitalElements->setText(QCoreApplication::translate("mpcImportWindow", "Update only the orbital elements", nullptr));
        pushButtonMarkAll->setText(QCoreApplication::translate("mpcImportWindow", "Mark all", nullptr));
        pushButtonMarkNone->setText(QCoreApplication::translate("mpcImportWindow", "Mark none", nullptr));
        pushButtonDiscard->setText(QCoreApplication::translate("mpcImportWindow", "Discard", nullptr));
        pushButtonAdd->setText(QCoreApplication::translate("mpcImportWindow", "Add objects", nullptr));
        (void)mpcImportWindow;
    } // retranslateUi

};

namespace Ui {
    class mpcImportWindow: public Ui_mpcImportWindow {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_MPCIMPORTWINDOW_H
