/****************************************************************************
** Meta object code from reading C++ file 'FrenchAstronomicalCalendar.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../stellarium/plugins/Calendars/src/FrenchAstronomicalCalendar.hpp"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'FrenchAstronomicalCalendar.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN26FrenchAstronomicalCalendarE_t {};
} // unnamed namespace

template <> constexpr inline auto FrenchAstronomicalCalendar::qt_create_metaobjectdata<qt_meta_tag_ZN26FrenchAstronomicalCalendarE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "FrenchAstronomicalCalendar",
        "setJD",
        "",
        "JD",
        "setDate",
        "QList<int>",
        "parts",
        "isLeap",
        "year",
        "fixedFromFrenchAstronomical",
        "french",
        "frenchAstronomicalFromFixed",
        "rd",
        "midnightInParis",
        "frenchNewYearOnOrBefore"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'setJD'
        QtMocHelpers::SlotData<void(double)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setDate'
        QtMocHelpers::SlotData<void(const QVector<int> &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Slot 'isLeap'
        QtMocHelpers::SlotData<bool(int)>(7, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::Int, 8 },
        }}),
        // Slot 'fixedFromFrenchAstronomical'
        QtMocHelpers::SlotData<int(const QVector<int> &)>(9, 2, QMC::AccessPublic, QMetaType::Int, {{
            { 0x80000000 | 5, 10 },
        }}),
        // Slot 'frenchAstronomicalFromFixed'
        QtMocHelpers::SlotData<QVector<int>(int)>(11, 2, QMC::AccessPublic, 0x80000000 | 5, {{
            { QMetaType::Int, 12 },
        }}),
        // Slot 'midnightInParis'
        QtMocHelpers::SlotData<double(int)>(13, 2, QMC::AccessPublic, QMetaType::Double, {{
            { QMetaType::Int, 12 },
        }}),
        // Slot 'frenchNewYearOnOrBefore'
        QtMocHelpers::SlotData<int(int)>(14, 2, QMC::AccessPublic, QMetaType::Int, {{
            { QMetaType::Int, 12 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<FrenchAstronomicalCalendar, qt_meta_tag_ZN26FrenchAstronomicalCalendarE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject FrenchAstronomicalCalendar::staticMetaObject = { {
    QMetaObject::SuperData::link<FrenchArithmeticCalendar::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26FrenchAstronomicalCalendarE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26FrenchAstronomicalCalendarE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN26FrenchAstronomicalCalendarE_t>.metaTypes,
    nullptr
} };

void FrenchAstronomicalCalendar::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<FrenchAstronomicalCalendar *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setJD((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 1: _t->setDate((*reinterpret_cast<std::add_pointer_t<QList<int>>>(_a[1]))); break;
        case 2: { bool _r = _t->isLeap((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 3: { int _r = _t->fixedFromFrenchAstronomical((*reinterpret_cast<std::add_pointer_t<QList<int>>>(_a[1])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 4: { QList<int> _r = _t->frenchAstronomicalFromFixed((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QList<int>*>(_a[0]) = std::move(_r); }  break;
        case 5: { double _r = _t->midnightInParis((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 6: { int _r = _t->frenchNewYearOnOrBefore((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<int> >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<int> >(); break;
            }
            break;
        }
    }
}

const QMetaObject *FrenchAstronomicalCalendar::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FrenchAstronomicalCalendar::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26FrenchAstronomicalCalendarE_t>.strings))
        return static_cast<void*>(this);
    return FrenchArithmeticCalendar::qt_metacast(_clname);
}

int FrenchAstronomicalCalendar::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = FrenchArithmeticCalendar::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}
QT_WARNING_POP
