/****************************************************************************
** Meta object code from reading C++ file 'Exoplanets.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../stellarium/plugins/Exoplanets/src/Exoplanets.hpp"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>
#include <QtCore/qplugin.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'Exoplanets.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10ExoplanetsE_t {};
} // unnamed namespace

template <> constexpr inline auto Exoplanets::qt_create_metaobjectdata<qt_meta_tag_ZN10ExoplanetsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Exoplanets",
        "updateStateChanged",
        "",
        "Exoplanets::UpdateState",
        "state",
        "jsonUpdateComplete",
        "flagExoplanetsVisibilityChanged",
        "b",
        "markerColorChanged",
        "Vec3f",
        "habitableColorChanged",
        "displayModeChanged",
        "timelineModeChanged",
        "habitableModeChanged",
        "flagShowExoplanetsDesignationsChanged",
        "flagShowExoplanetsNumbersChanged",
        "updateJSON",
        "setFlagShowExoplanets",
        "getFlagShowExoplanets",
        "setFlagShowExoplanetsDesignations",
        "getFlagShowExoplanetsDesignations",
        "setFlagShowExoplanetsNumbers",
        "getFlagShowExoplanetsNumbers",
        "setFlagShowExoplanetsButton",
        "getFlagShowExoplanetsButton",
        "getDisplayMode",
        "setDisplayMode",
        "getTimelineMode",
        "setTimelineMode",
        "getHabitableMode",
        "setHabitableMode",
        "getMarkerColor",
        "setMarkerColor",
        "c",
        "getHabitableColor",
        "setHabitableColor",
        "getCountPlanetarySystems",
        "getCountAllExoplanets",
        "getCountHabitableExoplanets",
        "setCurrentTemperatureScale",
        "Exoplanets::TemperatureScale",
        "tscale",
        "getCurrentTemperatureScale",
        "getCurrentTemperatureScaleKey",
        "setCurrentTemperatureScaleKey",
        "key",
        "setFontSize",
        "s",
        "checkForUpdate",
        "updateDownloadProgress",
        "bytesReceived",
        "bytesTotal",
        "downloadComplete",
        "QNetworkReply*",
        "reply",
        "displayMessage",
        "message",
        "hexColor",
        "reloadCatalog",
        "saveSettings",
        "setFlagSyncShowLabels",
        "showExoplanets",
        "markerColor",
        "habitableColor",
        "flagDisplayMode",
        "flagTimelineMode",
        "flagHabitableMode",
        "flagShowExoplanetsDesignations",
        "flagShowExoplanetsNumbers",
        "UpdateState",
        "Updating",
        "CompleteNoUpdates",
        "CompleteUpdates",
        "DownloadError",
        "OtherError",
        "TemperatureScale",
        "Kelvin",
        "Celsius",
        "Fahrenheit"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'updateStateChanged'
        QtMocHelpers::SignalData<void(Exoplanets::UpdateState)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'jsonUpdateComplete'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'flagExoplanetsVisibilityChanged'
        QtMocHelpers::SignalData<void(bool)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 7 },
        }}),
        // Signal 'markerColorChanged'
        QtMocHelpers::SignalData<void(Vec3f)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 2 },
        }}),
        // Signal 'habitableColorChanged'
        QtMocHelpers::SignalData<void(Vec3f)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 2 },
        }}),
        // Signal 'displayModeChanged'
        QtMocHelpers::SignalData<void(bool)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'timelineModeChanged'
        QtMocHelpers::SignalData<void(bool)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'habitableModeChanged'
        QtMocHelpers::SignalData<void(bool)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'flagShowExoplanetsDesignationsChanged'
        QtMocHelpers::SignalData<void(bool)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'flagShowExoplanetsNumbersChanged'
        QtMocHelpers::SignalData<void(bool)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'updateJSON'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setFlagShowExoplanets'
        QtMocHelpers::SlotData<void(bool)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 7 },
        }}),
        // Slot 'getFlagShowExoplanets'
        QtMocHelpers::SlotData<bool() const>(18, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFlagShowExoplanetsDesignations'
        QtMocHelpers::SlotData<void(bool)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 7 },
        }}),
        // Slot 'getFlagShowExoplanetsDesignations'
        QtMocHelpers::SlotData<bool() const>(20, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFlagShowExoplanetsNumbers'
        QtMocHelpers::SlotData<void(bool)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 7 },
        }}),
        // Slot 'getFlagShowExoplanetsNumbers'
        QtMocHelpers::SlotData<bool() const>(22, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFlagShowExoplanetsButton'
        QtMocHelpers::SlotData<void(bool)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 7 },
        }}),
        // Slot 'getFlagShowExoplanetsButton'
        QtMocHelpers::SlotData<bool() const>(24, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'getDisplayMode'
        QtMocHelpers::SlotData<bool() const>(25, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setDisplayMode'
        QtMocHelpers::SlotData<void(bool)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 7 },
        }}),
        // Slot 'getTimelineMode'
        QtMocHelpers::SlotData<bool() const>(27, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setTimelineMode'
        QtMocHelpers::SlotData<void(bool)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 7 },
        }}),
        // Slot 'getHabitableMode'
        QtMocHelpers::SlotData<bool() const>(29, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setHabitableMode'
        QtMocHelpers::SlotData<void(bool)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 7 },
        }}),
        // Slot 'getMarkerColor'
        QtMocHelpers::SlotData<Vec3f() const>(31, 2, QMC::AccessPublic, 0x80000000 | 9),
        // Slot 'setMarkerColor'
        QtMocHelpers::SlotData<void(const Vec3f &)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 33 },
        }}),
        // Slot 'getHabitableColor'
        QtMocHelpers::SlotData<Vec3f() const>(34, 2, QMC::AccessPublic, 0x80000000 | 9),
        // Slot 'setHabitableColor'
        QtMocHelpers::SlotData<void(const Vec3f &)>(35, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 33 },
        }}),
        // Slot 'getCountPlanetarySystems'
        QtMocHelpers::SlotData<int() const>(36, 2, QMC::AccessPublic, QMetaType::Int),
        // Slot 'getCountAllExoplanets'
        QtMocHelpers::SlotData<int() const>(37, 2, QMC::AccessPublic, QMetaType::Int),
        // Slot 'getCountHabitableExoplanets'
        QtMocHelpers::SlotData<int() const>(38, 2, QMC::AccessPublic, QMetaType::Int),
        // Slot 'setCurrentTemperatureScale'
        QtMocHelpers::SlotData<void(Exoplanets::TemperatureScale)>(39, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 40, 41 },
        }}),
        // Slot 'getCurrentTemperatureScale'
        QtMocHelpers::SlotData<Exoplanets::TemperatureScale() const>(42, 2, QMC::AccessPublic, 0x80000000 | 40),
        // Slot 'getCurrentTemperatureScaleKey'
        QtMocHelpers::SlotData<QString() const>(43, 2, QMC::AccessPublic, QMetaType::QString),
        // Slot 'setCurrentTemperatureScaleKey'
        QtMocHelpers::SlotData<void(const QString &)>(44, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 45 },
        }}),
        // Slot 'setFontSize'
        QtMocHelpers::SlotData<void(int)>(46, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 47 },
        }}),
        // Slot 'checkForUpdate'
        QtMocHelpers::SlotData<void()>(48, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateDownloadProgress'
        QtMocHelpers::SlotData<void(qint64, qint64)>(49, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::LongLong, 50 }, { QMetaType::LongLong, 51 },
        }}),
        // Slot 'downloadComplete'
        QtMocHelpers::SlotData<void(QNetworkReply *)>(52, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 53, 54 },
        }}),
        // Slot 'displayMessage'
        QtMocHelpers::SlotData<void(const QString &, const QString &)>(55, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 56 }, { QMetaType::QString, 57 },
        }}),
        // Slot 'displayMessage'
        QtMocHelpers::SlotData<void(const QString &)>(55, 2, QMC::AccessPrivate | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 56 },
        }}),
        // Slot 'reloadCatalog'
        QtMocHelpers::SlotData<void()>(58, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'saveSettings'
        QtMocHelpers::SlotData<void()>(59, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setFlagSyncShowLabels'
        QtMocHelpers::SlotData<void(bool)>(60, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 7 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'showExoplanets'
        QtMocHelpers::PropertyData<bool>(61, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 2),
        // property 'markerColor'
        QtMocHelpers::PropertyData<Vec3f>(62, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 3),
        // property 'habitableColor'
        QtMocHelpers::PropertyData<Vec3f>(63, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 4),
        // property 'flagDisplayMode'
        QtMocHelpers::PropertyData<bool>(64, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 5),
        // property 'flagTimelineMode'
        QtMocHelpers::PropertyData<bool>(65, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 6),
        // property 'flagHabitableMode'
        QtMocHelpers::PropertyData<bool>(66, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 7),
        // property 'flagShowExoplanetsDesignations'
        QtMocHelpers::PropertyData<bool>(67, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 8),
        // property 'flagShowExoplanetsNumbers'
        QtMocHelpers::PropertyData<bool>(68, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 9),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'UpdateState'
        QtMocHelpers::EnumData<enum UpdateState>(69, 69, QMC::EnumFlags{}).add({
            {   70, UpdateState::Updating },
            {   71, UpdateState::CompleteNoUpdates },
            {   72, UpdateState::CompleteUpdates },
            {   73, UpdateState::DownloadError },
            {   74, UpdateState::OtherError },
        }),
        // enum 'TemperatureScale'
        QtMocHelpers::EnumData<enum TemperatureScale>(75, 75, QMC::EnumFlags{}).add({
            {   76, TemperatureScale::Kelvin },
            {   77, TemperatureScale::Celsius },
            {   78, TemperatureScale::Fahrenheit },
        }),
    };
    return QtMocHelpers::metaObjectData<Exoplanets, qt_meta_tag_ZN10ExoplanetsE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Exoplanets::staticMetaObject = { {
    QMetaObject::SuperData::link<StelObjectModule::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10ExoplanetsE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10ExoplanetsE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10ExoplanetsE_t>.metaTypes,
    nullptr
} };

void Exoplanets::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Exoplanets *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->updateStateChanged((*reinterpret_cast<std::add_pointer_t<Exoplanets::UpdateState>>(_a[1]))); break;
        case 1: _t->jsonUpdateComplete(); break;
        case 2: _t->flagExoplanetsVisibilityChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->markerColorChanged((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1]))); break;
        case 4: _t->habitableColorChanged((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1]))); break;
        case 5: _t->displayModeChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->timelineModeChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->habitableModeChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->flagShowExoplanetsDesignationsChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 9: _t->flagShowExoplanetsNumbersChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 10: _t->updateJSON(); break;
        case 11: _t->setFlagShowExoplanets((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 12: { bool _r = _t->getFlagShowExoplanets();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 13: _t->setFlagShowExoplanetsDesignations((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 14: { bool _r = _t->getFlagShowExoplanetsDesignations();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 15: _t->setFlagShowExoplanetsNumbers((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 16: { bool _r = _t->getFlagShowExoplanetsNumbers();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 17: _t->setFlagShowExoplanetsButton((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 18: { bool _r = _t->getFlagShowExoplanetsButton();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 19: { bool _r = _t->getDisplayMode();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 20: _t->setDisplayMode((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 21: { bool _r = _t->getTimelineMode();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 22: _t->setTimelineMode((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 23: { bool _r = _t->getHabitableMode();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 24: _t->setHabitableMode((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 25: { Vec3f _r = _t->getMarkerColor();
            if (_a[0]) *reinterpret_cast<Vec3f*>(_a[0]) = std::move(_r); }  break;
        case 26: _t->setMarkerColor((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1]))); break;
        case 27: { Vec3f _r = _t->getHabitableColor();
            if (_a[0]) *reinterpret_cast<Vec3f*>(_a[0]) = std::move(_r); }  break;
        case 28: _t->setHabitableColor((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1]))); break;
        case 29: { int _r = _t->getCountPlanetarySystems();
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 30: { int _r = _t->getCountAllExoplanets();
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 31: { int _r = _t->getCountHabitableExoplanets();
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 32: _t->setCurrentTemperatureScale((*reinterpret_cast<std::add_pointer_t<Exoplanets::TemperatureScale>>(_a[1]))); break;
        case 33: { Exoplanets::TemperatureScale _r = _t->getCurrentTemperatureScale();
            if (_a[0]) *reinterpret_cast<Exoplanets::TemperatureScale*>(_a[0]) = std::move(_r); }  break;
        case 34: { QString _r = _t->getCurrentTemperatureScaleKey();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 35: _t->setCurrentTemperatureScaleKey((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 36: _t->setFontSize((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 37: _t->checkForUpdate(); break;
        case 38: _t->updateDownloadProgress((*reinterpret_cast<std::add_pointer_t<qint64>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qint64>>(_a[2]))); break;
        case 39: _t->downloadComplete((*reinterpret_cast<std::add_pointer_t<QNetworkReply*>>(_a[1]))); break;
        case 40: _t->displayMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 41: _t->displayMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 42: _t->reloadCatalog(); break;
        case 43: _t->saveSettings(); break;
        case 44: _t->setFlagSyncShowLabels((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 26:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 28:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 39:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QNetworkReply* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Exoplanets::*)(Exoplanets::UpdateState )>(_a, &Exoplanets::updateStateChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Exoplanets::*)()>(_a, &Exoplanets::jsonUpdateComplete, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Exoplanets::*)(bool )>(_a, &Exoplanets::flagExoplanetsVisibilityChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Exoplanets::*)(Vec3f )>(_a, &Exoplanets::markerColorChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Exoplanets::*)(Vec3f )>(_a, &Exoplanets::habitableColorChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (Exoplanets::*)(bool )>(_a, &Exoplanets::displayModeChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (Exoplanets::*)(bool )>(_a, &Exoplanets::timelineModeChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (Exoplanets::*)(bool )>(_a, &Exoplanets::habitableModeChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (Exoplanets::*)(bool )>(_a, &Exoplanets::flagShowExoplanetsDesignationsChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (Exoplanets::*)(bool )>(_a, &Exoplanets::flagShowExoplanetsNumbersChanged, 9))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 2:
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Vec3f >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->getFlagShowExoplanets(); break;
        case 1: *reinterpret_cast<Vec3f*>(_v) = _t->getMarkerColor(); break;
        case 2: *reinterpret_cast<Vec3f*>(_v) = _t->getHabitableColor(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->getDisplayMode(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->getTimelineMode(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->getHabitableMode(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->getFlagShowExoplanetsDesignations(); break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->getFlagShowExoplanetsNumbers(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setFlagShowExoplanets(*reinterpret_cast<bool*>(_v)); break;
        case 1: _t->setMarkerColor(*reinterpret_cast<Vec3f*>(_v)); break;
        case 2: _t->setHabitableColor(*reinterpret_cast<Vec3f*>(_v)); break;
        case 3: _t->setDisplayMode(*reinterpret_cast<bool*>(_v)); break;
        case 4: _t->setTimelineMode(*reinterpret_cast<bool*>(_v)); break;
        case 5: _t->setHabitableMode(*reinterpret_cast<bool*>(_v)); break;
        case 6: _t->setFlagShowExoplanetsDesignations(*reinterpret_cast<bool*>(_v)); break;
        case 7: _t->setFlagShowExoplanetsNumbers(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *Exoplanets::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Exoplanets::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10ExoplanetsE_t>.strings))
        return static_cast<void*>(this);
    return StelObjectModule::qt_metacast(_clname);
}

int Exoplanets::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StelObjectModule::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 45)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 45;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 45)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 45;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void Exoplanets::updateStateChanged(Exoplanets::UpdateState _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void Exoplanets::jsonUpdateComplete()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void Exoplanets::flagExoplanetsVisibilityChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void Exoplanets::markerColorChanged(Vec3f _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void Exoplanets::habitableColorChanged(Vec3f _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void Exoplanets::displayModeChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void Exoplanets::timelineModeChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void Exoplanets::habitableModeChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void Exoplanets::flagShowExoplanetsDesignationsChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void Exoplanets::flagShowExoplanetsNumbersChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN29ExoplanetsStelPluginInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto ExoplanetsStelPluginInterface::qt_create_metaobjectdata<qt_meta_tag_ZN29ExoplanetsStelPluginInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ExoplanetsStelPluginInterface"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ExoplanetsStelPluginInterface, qt_meta_tag_ZN29ExoplanetsStelPluginInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ExoplanetsStelPluginInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN29ExoplanetsStelPluginInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN29ExoplanetsStelPluginInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN29ExoplanetsStelPluginInterfaceE_t>.metaTypes,
    nullptr
} };

void ExoplanetsStelPluginInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ExoplanetsStelPluginInterface *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *ExoplanetsStelPluginInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ExoplanetsStelPluginInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN29ExoplanetsStelPluginInterfaceE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "StelPluginInterface"))
        return static_cast< StelPluginInterface*>(this);
    if (!strcmp(_clname, "org.stellarium.StelPluginInterface/2.0"))
        return static_cast< StelPluginInterface*>(this);
    return QObject::qt_metacast(_clname);
}

int ExoplanetsStelPluginInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}

#ifdef QT_MOC_EXPORT_PLUGIN_V2
static constexpr unsigned char qt_pluginMetaDataV2_ExoplanetsStelPluginInterface[] = {
    0xbf, 
    // "IID"
    0x02,  0x78,  0x26,  'o',  'r',  'g',  '.',  's', 
    't',  'e',  'l',  'l',  'a',  'r',  'i',  'u', 
    'm',  '.',  'S',  't',  'e',  'l',  'P',  'l', 
    'u',  'g',  'i',  'n',  'I',  'n',  't',  'e', 
    'r',  'f',  'a',  'c',  'e',  '/',  '2',  '.', 
    '0', 
    // "className"
    0x03,  0x78,  0x1d,  'E',  'x',  'o',  'p',  'l', 
    'a',  'n',  'e',  't',  's',  'S',  't',  'e', 
    'l',  'P',  'l',  'u',  'g',  'i',  'n',  'I', 
    'n',  't',  'e',  'r',  'f',  'a',  'c',  'e', 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN_V2(ExoplanetsStelPluginInterface, ExoplanetsStelPluginInterface, qt_pluginMetaDataV2_ExoplanetsStelPluginInterface)
#else
QT_PLUGIN_METADATA_SECTION
Q_CONSTINIT static constexpr unsigned char qt_pluginMetaData_ExoplanetsStelPluginInterface[] = {
    'Q', 'T', 'M', 'E', 'T', 'A', 'D', 'A', 'T', 'A', ' ', '!',
    // metadata version, Qt version, architectural requirements
    0, QT_VERSION_MAJOR, QT_VERSION_MINOR, qPluginArchRequirements(),
    0xbf, 
    // "IID"
    0x02,  0x78,  0x26,  'o',  'r',  'g',  '.',  's', 
    't',  'e',  'l',  'l',  'a',  'r',  'i',  'u', 
    'm',  '.',  'S',  't',  'e',  'l',  'P',  'l', 
    'u',  'g',  'i',  'n',  'I',  'n',  't',  'e', 
    'r',  'f',  'a',  'c',  'e',  '/',  '2',  '.', 
    '0', 
    // "className"
    0x03,  0x78,  0x1d,  'E',  'x',  'o',  'p',  'l', 
    'a',  'n',  'e',  't',  's',  'S',  't',  'e', 
    'l',  'P',  'l',  'u',  'g',  'i',  'n',  'I', 
    'n',  't',  'e',  'r',  'f',  'a',  'c',  'e', 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN(ExoplanetsStelPluginInterface, ExoplanetsStelPluginInterface)
#endif  // QT_MOC_EXPORT_PLUGIN_V2

QT_WARNING_POP
