/****************************************************************************
** Meta object code from reading C++ file 'Calendars.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../stellarium/plugins/Calendars/src/Calendars.hpp"
#include <QtCore/qmetatype.h>
#include <QtCore/qplugin.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'Calendars.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9CalendarsE_t {};
} // unnamed namespace

template <> constexpr inline auto Calendars::qt_create_metaobjectdata<qt_meta_tag_ZN9CalendarsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Calendars",
        "enabledChanged",
        "",
        "b",
        "flagTextColorOverrideChanged",
        "textColorChanged",
        "Vec3f&",
        "color",
        "showJulianChanged",
        "showRevisedJulianChanged",
        "showGregorianChanged",
        "showISOChanged",
        "showIcelandicChanged",
        "showRomanChanged",
        "showOlympicChanged",
        "showEgyptianChanged",
        "showArmenianChanged",
        "showZoroastrianChanged",
        "showCopticChanged",
        "showEthiopicChanged",
        "showChineseChanged",
        "showJapaneseChanged",
        "showKoreanChanged",
        "showVietnameseChanged",
        "showIslamicChanged",
        "showHebrewChanged",
        "showOldHinduSolarChanged",
        "showOldHinduLunarChanged",
        "showNewHinduSolarChanged",
        "showNewHinduLunarChanged",
        "showAstroHinduSolarChanged",
        "showAstroHinduLunarChanged",
        "showMayaLongCountChanged",
        "showMayaHaabChanged",
        "showMayaTzolkinChanged",
        "showAztecXihuitlChanged",
        "showAztecTonalpohualliChanged",
        "showBalineseChanged",
        "showFrenchAstronomicalChanged",
        "showFrenchArithmeticChanged",
        "showPersianArithmeticChanged",
        "showPersianAstronomicalChanged",
        "showBahaiArithmeticChanged",
        "showBahaiAstronomicalChanged",
        "showTibetanChanged",
        "isEnabled",
        "getFlagTextColorOverride",
        "setFlagTextColorOverride",
        "getTextColor",
        "Vec3f",
        "setTextColor",
        "newColor",
        "exportCalendars",
        "enable",
        "isJulianDisplayed",
        "showJulian",
        "isRevisedJulianDisplayed",
        "showRevisedJulian",
        "isGregorianDisplayed",
        "showGregorian",
        "isISODisplayed",
        "showISO",
        "isIcelandicDisplayed",
        "showIcelandic",
        "isRomanDisplayed",
        "showRoman",
        "isOlympicDisplayed",
        "showOlympic",
        "isEgyptianDisplayed",
        "showEgyptian",
        "isArmenianDisplayed",
        "showArmenian",
        "isZoroastrianDisplayed",
        "showZoroastrian",
        "isCopticDisplayed",
        "showCoptic",
        "isEthiopicDisplayed",
        "showEthiopic",
        "isChineseDisplayed",
        "showChinese",
        "isJapaneseDisplayed",
        "showJapanese",
        "isKoreanDisplayed",
        "showKorean",
        "isVietnameseDisplayed",
        "showVietnamese",
        "isIslamicDisplayed",
        "showIslamic",
        "isHebrewDisplayed",
        "showHebrew",
        "isOldHinduSolarDisplayed",
        "showOldHinduSolar",
        "isOldHinduLunarDisplayed",
        "showOldHinduLunar",
        "isNewHinduSolarDisplayed",
        "showNewHinduSolar",
        "isNewHinduLunarDisplayed",
        "showNewHinduLunar",
        "isAstroHinduSolarDisplayed",
        "showAstroHinduSolar",
        "isAstroHinduLunarDisplayed",
        "showAstroHinduLunar",
        "isMayaLongCountDisplayed",
        "showMayaLongCount",
        "isMayaHaabDisplayed",
        "showMayaHaab",
        "isMayaTzolkinDisplayed",
        "showMayaTzolkin",
        "isAztecXihuitlDisplayed",
        "showAztecXihuitl",
        "isAztecTonalpohualliDisplayed",
        "showAztecTonalpohualli",
        "isBalineseDisplayed",
        "showBalinese",
        "isFrenchAstronomicalDisplayed",
        "showFrenchAstronomical",
        "isFrenchArithmeticDisplayed",
        "showFrenchArithmetic",
        "isPersianArithmeticDisplayed",
        "showPersianArithmetic",
        "isPersianAstronomicalDisplayed",
        "showPersianAstronomical",
        "isBahaiArithmeticDisplayed",
        "showBahaiArithmetic",
        "isBahaiAstronomicalDisplayed",
        "showBahaiAstronomical",
        "isTibetanDisplayed",
        "showTibetan",
        "enabled",
        "flagTextColorOverride",
        "textColor",
        "flagShowJulian",
        "flagShowRevisedJulian",
        "flagShowGregorian",
        "flagShowISO",
        "flagShowIcelandic",
        "flagShowRoman",
        "flagShowOlympic",
        "flagShowEgyptian",
        "flagShowArmenian",
        "flagShowZoroastrian",
        "flagShowCoptic",
        "flagShowEthiopic",
        "flagShowChinese",
        "flagShowJapanese",
        "flagShowKorean",
        "flagShowVietnamese",
        "flagShowIslamic",
        "flagShowHebrew",
        "flagShowOldHinduSolar",
        "flagShowOldHinduLunar",
        "flagShowNewHinduSolar",
        "flagShowNewHinduLunar",
        "flagShowAstroHinduSolar",
        "flagShowAstroHinduLunar",
        "flagShowMayaLongCount",
        "flagShowMayaHaab",
        "flagShowMayaTzolkin",
        "flagShowAztecXihuitl",
        "flagShowAztecTonalpohualli",
        "flagShowBalinese",
        "flagShowFrenchAstronomical",
        "flagShowFrenchArithmetic",
        "flagShowPersianArithmetic",
        "flagShowPersianAstronomical",
        "flagShowBahaiArithmetic",
        "flagShowBahaiAstronomical",
        "flagShowTibetan"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'enabledChanged'
        QtMocHelpers::SignalData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagTextColorOverrideChanged'
        QtMocHelpers::SignalData<void(bool)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'textColorChanged'
        QtMocHelpers::SignalData<void(Vec3f &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'showJulianChanged'
        QtMocHelpers::SignalData<void(bool)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'showRevisedJulianChanged'
        QtMocHelpers::SignalData<void(bool)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'showGregorianChanged'
        QtMocHelpers::SignalData<void(bool)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'showISOChanged'
        QtMocHelpers::SignalData<void(bool)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'showIcelandicChanged'
        QtMocHelpers::SignalData<void(bool)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'showRomanChanged'
        QtMocHelpers::SignalData<void(bool)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'showOlympicChanged'
        QtMocHelpers::SignalData<void(bool)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'showEgyptianChanged'
        QtMocHelpers::SignalData<void(bool)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'showArmenianChanged'
        QtMocHelpers::SignalData<void(bool)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'showZoroastrianChanged'
        QtMocHelpers::SignalData<void(bool)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'showCopticChanged'
        QtMocHelpers::SignalData<void(bool)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'showEthiopicChanged'
        QtMocHelpers::SignalData<void(bool)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'showChineseChanged'
        QtMocHelpers::SignalData<void(bool)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'showJapaneseChanged'
        QtMocHelpers::SignalData<void(bool)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'showKoreanChanged'
        QtMocHelpers::SignalData<void(bool)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'showVietnameseChanged'
        QtMocHelpers::SignalData<void(bool)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'showIslamicChanged'
        QtMocHelpers::SignalData<void(bool)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'showHebrewChanged'
        QtMocHelpers::SignalData<void(bool)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'showOldHinduSolarChanged'
        QtMocHelpers::SignalData<void(bool)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'showOldHinduLunarChanged'
        QtMocHelpers::SignalData<void(bool)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'showNewHinduSolarChanged'
        QtMocHelpers::SignalData<void(bool)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'showNewHinduLunarChanged'
        QtMocHelpers::SignalData<void(bool)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'showAstroHinduSolarChanged'
        QtMocHelpers::SignalData<void(bool)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'showAstroHinduLunarChanged'
        QtMocHelpers::SignalData<void(bool)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'showMayaLongCountChanged'
        QtMocHelpers::SignalData<void(bool)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'showMayaHaabChanged'
        QtMocHelpers::SignalData<void(bool)>(33, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'showMayaTzolkinChanged'
        QtMocHelpers::SignalData<void(bool)>(34, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'showAztecXihuitlChanged'
        QtMocHelpers::SignalData<void(bool)>(35, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'showAztecTonalpohualliChanged'
        QtMocHelpers::SignalData<void(bool)>(36, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'showBalineseChanged'
        QtMocHelpers::SignalData<void(bool)>(37, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'showFrenchAstronomicalChanged'
        QtMocHelpers::SignalData<void(bool)>(38, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'showFrenchArithmeticChanged'
        QtMocHelpers::SignalData<void(bool)>(39, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'showPersianArithmeticChanged'
        QtMocHelpers::SignalData<void(bool)>(40, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'showPersianAstronomicalChanged'
        QtMocHelpers::SignalData<void(bool)>(41, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'showBahaiArithmeticChanged'
        QtMocHelpers::SignalData<void(bool)>(42, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'showBahaiAstronomicalChanged'
        QtMocHelpers::SignalData<void(bool)>(43, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'showTibetanChanged'
        QtMocHelpers::SignalData<void(bool)>(44, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'isEnabled'
        QtMocHelpers::SlotData<bool() const>(45, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'getFlagTextColorOverride'
        QtMocHelpers::SlotData<bool() const>(46, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFlagTextColorOverride'
        QtMocHelpers::SlotData<void(bool)>(47, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'getTextColor'
        QtMocHelpers::SlotData<Vec3f() const>(48, 2, QMC::AccessPublic, 0x80000000 | 49),
        // Slot 'setTextColor'
        QtMocHelpers::SlotData<void(const Vec3f &)>(50, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 49, 51 },
        }}),
        // Slot 'exportCalendars'
        QtMocHelpers::SlotData<void()>(52, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'enable'
        QtMocHelpers::SlotData<void(bool)>(53, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'isJulianDisplayed'
        QtMocHelpers::SlotData<bool() const>(54, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'showJulian'
        QtMocHelpers::SlotData<void(bool)>(55, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'isRevisedJulianDisplayed'
        QtMocHelpers::SlotData<bool() const>(56, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'showRevisedJulian'
        QtMocHelpers::SlotData<void(bool)>(57, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'isGregorianDisplayed'
        QtMocHelpers::SlotData<bool() const>(58, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'showGregorian'
        QtMocHelpers::SlotData<void(bool)>(59, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'isISODisplayed'
        QtMocHelpers::SlotData<bool() const>(60, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'showISO'
        QtMocHelpers::SlotData<void(bool)>(61, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'isIcelandicDisplayed'
        QtMocHelpers::SlotData<bool() const>(62, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'showIcelandic'
        QtMocHelpers::SlotData<void(bool)>(63, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'isRomanDisplayed'
        QtMocHelpers::SlotData<bool() const>(64, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'showRoman'
        QtMocHelpers::SlotData<void(bool)>(65, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'isOlympicDisplayed'
        QtMocHelpers::SlotData<bool() const>(66, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'showOlympic'
        QtMocHelpers::SlotData<void(bool)>(67, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'isEgyptianDisplayed'
        QtMocHelpers::SlotData<bool() const>(68, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'showEgyptian'
        QtMocHelpers::SlotData<void(bool)>(69, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'isArmenianDisplayed'
        QtMocHelpers::SlotData<bool() const>(70, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'showArmenian'
        QtMocHelpers::SlotData<void(bool)>(71, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'isZoroastrianDisplayed'
        QtMocHelpers::SlotData<bool() const>(72, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'showZoroastrian'
        QtMocHelpers::SlotData<void(bool)>(73, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'isCopticDisplayed'
        QtMocHelpers::SlotData<bool() const>(74, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'showCoptic'
        QtMocHelpers::SlotData<void(bool)>(75, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'isEthiopicDisplayed'
        QtMocHelpers::SlotData<bool() const>(76, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'showEthiopic'
        QtMocHelpers::SlotData<void(bool)>(77, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'isChineseDisplayed'
        QtMocHelpers::SlotData<bool() const>(78, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'showChinese'
        QtMocHelpers::SlotData<void(bool)>(79, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'isJapaneseDisplayed'
        QtMocHelpers::SlotData<bool() const>(80, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'showJapanese'
        QtMocHelpers::SlotData<void(bool)>(81, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'isKoreanDisplayed'
        QtMocHelpers::SlotData<bool() const>(82, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'showKorean'
        QtMocHelpers::SlotData<void(bool)>(83, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'isVietnameseDisplayed'
        QtMocHelpers::SlotData<bool() const>(84, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'showVietnamese'
        QtMocHelpers::SlotData<void(bool)>(85, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'isIslamicDisplayed'
        QtMocHelpers::SlotData<bool() const>(86, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'showIslamic'
        QtMocHelpers::SlotData<void(bool)>(87, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'isHebrewDisplayed'
        QtMocHelpers::SlotData<bool() const>(88, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'showHebrew'
        QtMocHelpers::SlotData<void(bool)>(89, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'isOldHinduSolarDisplayed'
        QtMocHelpers::SlotData<bool() const>(90, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'showOldHinduSolar'
        QtMocHelpers::SlotData<void(bool)>(91, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'isOldHinduLunarDisplayed'
        QtMocHelpers::SlotData<bool() const>(92, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'showOldHinduLunar'
        QtMocHelpers::SlotData<void(bool)>(93, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'isNewHinduSolarDisplayed'
        QtMocHelpers::SlotData<bool() const>(94, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'showNewHinduSolar'
        QtMocHelpers::SlotData<void(bool)>(95, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'isNewHinduLunarDisplayed'
        QtMocHelpers::SlotData<bool() const>(96, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'showNewHinduLunar'
        QtMocHelpers::SlotData<void(bool)>(97, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'isAstroHinduSolarDisplayed'
        QtMocHelpers::SlotData<bool() const>(98, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'showAstroHinduSolar'
        QtMocHelpers::SlotData<void(bool)>(99, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'isAstroHinduLunarDisplayed'
        QtMocHelpers::SlotData<bool() const>(100, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'showAstroHinduLunar'
        QtMocHelpers::SlotData<void(bool)>(101, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'isMayaLongCountDisplayed'
        QtMocHelpers::SlotData<bool() const>(102, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'showMayaLongCount'
        QtMocHelpers::SlotData<void(bool)>(103, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'isMayaHaabDisplayed'
        QtMocHelpers::SlotData<bool() const>(104, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'showMayaHaab'
        QtMocHelpers::SlotData<void(bool)>(105, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'isMayaTzolkinDisplayed'
        QtMocHelpers::SlotData<bool() const>(106, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'showMayaTzolkin'
        QtMocHelpers::SlotData<void(bool)>(107, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'isAztecXihuitlDisplayed'
        QtMocHelpers::SlotData<bool() const>(108, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'showAztecXihuitl'
        QtMocHelpers::SlotData<void(bool)>(109, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'isAztecTonalpohualliDisplayed'
        QtMocHelpers::SlotData<bool() const>(110, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'showAztecTonalpohualli'
        QtMocHelpers::SlotData<void(bool)>(111, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'isBalineseDisplayed'
        QtMocHelpers::SlotData<bool() const>(112, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'showBalinese'
        QtMocHelpers::SlotData<void(bool)>(113, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'isFrenchAstronomicalDisplayed'
        QtMocHelpers::SlotData<bool() const>(114, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'showFrenchAstronomical'
        QtMocHelpers::SlotData<void(bool)>(115, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'isFrenchArithmeticDisplayed'
        QtMocHelpers::SlotData<bool() const>(116, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'showFrenchArithmetic'
        QtMocHelpers::SlotData<void(bool)>(117, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'isPersianArithmeticDisplayed'
        QtMocHelpers::SlotData<bool() const>(118, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'showPersianArithmetic'
        QtMocHelpers::SlotData<void(bool)>(119, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'isPersianAstronomicalDisplayed'
        QtMocHelpers::SlotData<bool() const>(120, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'showPersianAstronomical'
        QtMocHelpers::SlotData<void(bool)>(121, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'isBahaiArithmeticDisplayed'
        QtMocHelpers::SlotData<bool() const>(122, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'showBahaiArithmetic'
        QtMocHelpers::SlotData<void(bool)>(123, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'isBahaiAstronomicalDisplayed'
        QtMocHelpers::SlotData<bool() const>(124, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'showBahaiAstronomical'
        QtMocHelpers::SlotData<void(bool)>(125, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'isTibetanDisplayed'
        QtMocHelpers::SlotData<bool() const>(126, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'showTibetan'
        QtMocHelpers::SlotData<void(bool)>(127, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'enabled'
        QtMocHelpers::PropertyData<bool>(128, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 0),
        // property 'flagTextColorOverride'
        QtMocHelpers::PropertyData<bool>(129, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'textColor'
        QtMocHelpers::PropertyData<Vec3f>(130, 0x80000000 | 49, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 2),
        // property 'flagShowJulian'
        QtMocHelpers::PropertyData<bool>(131, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 3),
        // property 'flagShowRevisedJulian'
        QtMocHelpers::PropertyData<bool>(132, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 4),
        // property 'flagShowGregorian'
        QtMocHelpers::PropertyData<bool>(133, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 5),
        // property 'flagShowISO'
        QtMocHelpers::PropertyData<bool>(134, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 6),
        // property 'flagShowIcelandic'
        QtMocHelpers::PropertyData<bool>(135, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 7),
        // property 'flagShowRoman'
        QtMocHelpers::PropertyData<bool>(136, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 8),
        // property 'flagShowOlympic'
        QtMocHelpers::PropertyData<bool>(137, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 9),
        // property 'flagShowEgyptian'
        QtMocHelpers::PropertyData<bool>(138, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 10),
        // property 'flagShowArmenian'
        QtMocHelpers::PropertyData<bool>(139, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 11),
        // property 'flagShowZoroastrian'
        QtMocHelpers::PropertyData<bool>(140, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 12),
        // property 'flagShowCoptic'
        QtMocHelpers::PropertyData<bool>(141, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 13),
        // property 'flagShowEthiopic'
        QtMocHelpers::PropertyData<bool>(142, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 14),
        // property 'flagShowChinese'
        QtMocHelpers::PropertyData<bool>(143, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 15),
        // property 'flagShowJapanese'
        QtMocHelpers::PropertyData<bool>(144, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 16),
        // property 'flagShowKorean'
        QtMocHelpers::PropertyData<bool>(145, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 17),
        // property 'flagShowVietnamese'
        QtMocHelpers::PropertyData<bool>(146, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 18),
        // property 'flagShowIslamic'
        QtMocHelpers::PropertyData<bool>(147, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 19),
        // property 'flagShowHebrew'
        QtMocHelpers::PropertyData<bool>(148, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 20),
        // property 'flagShowOldHinduSolar'
        QtMocHelpers::PropertyData<bool>(149, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 21),
        // property 'flagShowOldHinduLunar'
        QtMocHelpers::PropertyData<bool>(150, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 22),
        // property 'flagShowNewHinduSolar'
        QtMocHelpers::PropertyData<bool>(151, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 23),
        // property 'flagShowNewHinduLunar'
        QtMocHelpers::PropertyData<bool>(152, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 24),
        // property 'flagShowAstroHinduSolar'
        QtMocHelpers::PropertyData<bool>(153, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 25),
        // property 'flagShowAstroHinduLunar'
        QtMocHelpers::PropertyData<bool>(154, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 26),
        // property 'flagShowMayaLongCount'
        QtMocHelpers::PropertyData<bool>(155, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 27),
        // property 'flagShowMayaHaab'
        QtMocHelpers::PropertyData<bool>(156, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 28),
        // property 'flagShowMayaTzolkin'
        QtMocHelpers::PropertyData<bool>(157, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 29),
        // property 'flagShowAztecXihuitl'
        QtMocHelpers::PropertyData<bool>(158, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 30),
        // property 'flagShowAztecTonalpohualli'
        QtMocHelpers::PropertyData<bool>(159, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 31),
        // property 'flagShowBalinese'
        QtMocHelpers::PropertyData<bool>(160, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 32),
        // property 'flagShowFrenchAstronomical'
        QtMocHelpers::PropertyData<bool>(161, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 33),
        // property 'flagShowFrenchArithmetic'
        QtMocHelpers::PropertyData<bool>(162, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 34),
        // property 'flagShowPersianArithmetic'
        QtMocHelpers::PropertyData<bool>(163, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 35),
        // property 'flagShowPersianAstronomical'
        QtMocHelpers::PropertyData<bool>(164, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 36),
        // property 'flagShowBahaiArithmetic'
        QtMocHelpers::PropertyData<bool>(165, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 37),
        // property 'flagShowBahaiAstronomical'
        QtMocHelpers::PropertyData<bool>(166, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 38),
        // property 'flagShowTibetan'
        QtMocHelpers::PropertyData<bool>(167, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 39),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Calendars, qt_meta_tag_ZN9CalendarsE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Calendars::staticMetaObject = { {
    QMetaObject::SuperData::link<StelModule::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9CalendarsE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9CalendarsE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9CalendarsE_t>.metaTypes,
    nullptr
} };

void Calendars::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Calendars *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->enabledChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->flagTextColorOverrideChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->textColorChanged((*reinterpret_cast<std::add_pointer_t<Vec3f&>>(_a[1]))); break;
        case 3: _t->showJulianChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->showRevisedJulianChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->showGregorianChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->showISOChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->showIcelandicChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->showRomanChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 9: _t->showOlympicChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 10: _t->showEgyptianChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 11: _t->showArmenianChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 12: _t->showZoroastrianChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 13: _t->showCopticChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 14: _t->showEthiopicChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 15: _t->showChineseChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 16: _t->showJapaneseChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 17: _t->showKoreanChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 18: _t->showVietnameseChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 19: _t->showIslamicChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 20: _t->showHebrewChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 21: _t->showOldHinduSolarChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 22: _t->showOldHinduLunarChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 23: _t->showNewHinduSolarChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 24: _t->showNewHinduLunarChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 25: _t->showAstroHinduSolarChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 26: _t->showAstroHinduLunarChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 27: _t->showMayaLongCountChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 28: _t->showMayaHaabChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 29: _t->showMayaTzolkinChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 30: _t->showAztecXihuitlChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 31: _t->showAztecTonalpohualliChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 32: _t->showBalineseChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 33: _t->showFrenchAstronomicalChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 34: _t->showFrenchArithmeticChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 35: _t->showPersianArithmeticChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 36: _t->showPersianAstronomicalChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 37: _t->showBahaiArithmeticChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 38: _t->showBahaiAstronomicalChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 39: _t->showTibetanChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 40: { bool _r = _t->isEnabled();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 41: { bool _r = _t->getFlagTextColorOverride();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 42: _t->setFlagTextColorOverride((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 43: { Vec3f _r = _t->getTextColor();
            if (_a[0]) *reinterpret_cast<Vec3f*>(_a[0]) = std::move(_r); }  break;
        case 44: _t->setTextColor((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1]))); break;
        case 45: _t->exportCalendars(); break;
        case 46: _t->enable((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 47: { bool _r = _t->isJulianDisplayed();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 48: _t->showJulian((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 49: { bool _r = _t->isRevisedJulianDisplayed();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 50: _t->showRevisedJulian((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 51: { bool _r = _t->isGregorianDisplayed();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 52: _t->showGregorian((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 53: { bool _r = _t->isISODisplayed();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 54: _t->showISO((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 55: { bool _r = _t->isIcelandicDisplayed();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 56: _t->showIcelandic((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 57: { bool _r = _t->isRomanDisplayed();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 58: _t->showRoman((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 59: { bool _r = _t->isOlympicDisplayed();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 60: _t->showOlympic((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 61: { bool _r = _t->isEgyptianDisplayed();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 62: _t->showEgyptian((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 63: { bool _r = _t->isArmenianDisplayed();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 64: _t->showArmenian((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 65: { bool _r = _t->isZoroastrianDisplayed();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 66: _t->showZoroastrian((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 67: { bool _r = _t->isCopticDisplayed();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 68: _t->showCoptic((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 69: { bool _r = _t->isEthiopicDisplayed();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 70: _t->showEthiopic((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 71: { bool _r = _t->isChineseDisplayed();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 72: _t->showChinese((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 73: { bool _r = _t->isJapaneseDisplayed();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 74: _t->showJapanese((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 75: { bool _r = _t->isKoreanDisplayed();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 76: _t->showKorean((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 77: { bool _r = _t->isVietnameseDisplayed();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 78: _t->showVietnamese((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 79: { bool _r = _t->isIslamicDisplayed();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 80: _t->showIslamic((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 81: { bool _r = _t->isHebrewDisplayed();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 82: _t->showHebrew((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 83: { bool _r = _t->isOldHinduSolarDisplayed();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 84: _t->showOldHinduSolar((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 85: { bool _r = _t->isOldHinduLunarDisplayed();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 86: _t->showOldHinduLunar((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 87: { bool _r = _t->isNewHinduSolarDisplayed();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 88: _t->showNewHinduSolar((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 89: { bool _r = _t->isNewHinduLunarDisplayed();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 90: _t->showNewHinduLunar((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 91: { bool _r = _t->isAstroHinduSolarDisplayed();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 92: _t->showAstroHinduSolar((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 93: { bool _r = _t->isAstroHinduLunarDisplayed();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 94: _t->showAstroHinduLunar((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 95: { bool _r = _t->isMayaLongCountDisplayed();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 96: _t->showMayaLongCount((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 97: { bool _r = _t->isMayaHaabDisplayed();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 98: _t->showMayaHaab((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 99: { bool _r = _t->isMayaTzolkinDisplayed();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 100: _t->showMayaTzolkin((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 101: { bool _r = _t->isAztecXihuitlDisplayed();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 102: _t->showAztecXihuitl((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 103: { bool _r = _t->isAztecTonalpohualliDisplayed();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 104: _t->showAztecTonalpohualli((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 105: { bool _r = _t->isBalineseDisplayed();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 106: _t->showBalinese((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 107: { bool _r = _t->isFrenchAstronomicalDisplayed();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 108: _t->showFrenchAstronomical((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 109: { bool _r = _t->isFrenchArithmeticDisplayed();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 110: _t->showFrenchArithmetic((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 111: { bool _r = _t->isPersianArithmeticDisplayed();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 112: _t->showPersianArithmetic((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 113: { bool _r = _t->isPersianAstronomicalDisplayed();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 114: _t->showPersianAstronomical((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 115: { bool _r = _t->isBahaiArithmeticDisplayed();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 116: _t->showBahaiArithmetic((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 117: { bool _r = _t->isBahaiAstronomicalDisplayed();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 118: _t->showBahaiAstronomical((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 119: { bool _r = _t->isTibetanDisplayed();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 120: _t->showTibetan((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 44:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Calendars::*)(bool )>(_a, &Calendars::enabledChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Calendars::*)(bool )>(_a, &Calendars::flagTextColorOverrideChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Calendars::*)(Vec3f & )>(_a, &Calendars::textColorChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Calendars::*)(bool )>(_a, &Calendars::showJulianChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Calendars::*)(bool )>(_a, &Calendars::showRevisedJulianChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (Calendars::*)(bool )>(_a, &Calendars::showGregorianChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (Calendars::*)(bool )>(_a, &Calendars::showISOChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (Calendars::*)(bool )>(_a, &Calendars::showIcelandicChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (Calendars::*)(bool )>(_a, &Calendars::showRomanChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (Calendars::*)(bool )>(_a, &Calendars::showOlympicChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (Calendars::*)(bool )>(_a, &Calendars::showEgyptianChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (Calendars::*)(bool )>(_a, &Calendars::showArmenianChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (Calendars::*)(bool )>(_a, &Calendars::showZoroastrianChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (Calendars::*)(bool )>(_a, &Calendars::showCopticChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (Calendars::*)(bool )>(_a, &Calendars::showEthiopicChanged, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (Calendars::*)(bool )>(_a, &Calendars::showChineseChanged, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (Calendars::*)(bool )>(_a, &Calendars::showJapaneseChanged, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (Calendars::*)(bool )>(_a, &Calendars::showKoreanChanged, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (Calendars::*)(bool )>(_a, &Calendars::showVietnameseChanged, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (Calendars::*)(bool )>(_a, &Calendars::showIslamicChanged, 19))
            return;
        if (QtMocHelpers::indexOfMethod<void (Calendars::*)(bool )>(_a, &Calendars::showHebrewChanged, 20))
            return;
        if (QtMocHelpers::indexOfMethod<void (Calendars::*)(bool )>(_a, &Calendars::showOldHinduSolarChanged, 21))
            return;
        if (QtMocHelpers::indexOfMethod<void (Calendars::*)(bool )>(_a, &Calendars::showOldHinduLunarChanged, 22))
            return;
        if (QtMocHelpers::indexOfMethod<void (Calendars::*)(bool )>(_a, &Calendars::showNewHinduSolarChanged, 23))
            return;
        if (QtMocHelpers::indexOfMethod<void (Calendars::*)(bool )>(_a, &Calendars::showNewHinduLunarChanged, 24))
            return;
        if (QtMocHelpers::indexOfMethod<void (Calendars::*)(bool )>(_a, &Calendars::showAstroHinduSolarChanged, 25))
            return;
        if (QtMocHelpers::indexOfMethod<void (Calendars::*)(bool )>(_a, &Calendars::showAstroHinduLunarChanged, 26))
            return;
        if (QtMocHelpers::indexOfMethod<void (Calendars::*)(bool )>(_a, &Calendars::showMayaLongCountChanged, 27))
            return;
        if (QtMocHelpers::indexOfMethod<void (Calendars::*)(bool )>(_a, &Calendars::showMayaHaabChanged, 28))
            return;
        if (QtMocHelpers::indexOfMethod<void (Calendars::*)(bool )>(_a, &Calendars::showMayaTzolkinChanged, 29))
            return;
        if (QtMocHelpers::indexOfMethod<void (Calendars::*)(bool )>(_a, &Calendars::showAztecXihuitlChanged, 30))
            return;
        if (QtMocHelpers::indexOfMethod<void (Calendars::*)(bool )>(_a, &Calendars::showAztecTonalpohualliChanged, 31))
            return;
        if (QtMocHelpers::indexOfMethod<void (Calendars::*)(bool )>(_a, &Calendars::showBalineseChanged, 32))
            return;
        if (QtMocHelpers::indexOfMethod<void (Calendars::*)(bool )>(_a, &Calendars::showFrenchAstronomicalChanged, 33))
            return;
        if (QtMocHelpers::indexOfMethod<void (Calendars::*)(bool )>(_a, &Calendars::showFrenchArithmeticChanged, 34))
            return;
        if (QtMocHelpers::indexOfMethod<void (Calendars::*)(bool )>(_a, &Calendars::showPersianArithmeticChanged, 35))
            return;
        if (QtMocHelpers::indexOfMethod<void (Calendars::*)(bool )>(_a, &Calendars::showPersianAstronomicalChanged, 36))
            return;
        if (QtMocHelpers::indexOfMethod<void (Calendars::*)(bool )>(_a, &Calendars::showBahaiArithmeticChanged, 37))
            return;
        if (QtMocHelpers::indexOfMethod<void (Calendars::*)(bool )>(_a, &Calendars::showBahaiAstronomicalChanged, 38))
            return;
        if (QtMocHelpers::indexOfMethod<void (Calendars::*)(bool )>(_a, &Calendars::showTibetanChanged, 39))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 2:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Vec3f >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->isEnabled(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->getFlagTextColorOverride(); break;
        case 2: *reinterpret_cast<Vec3f*>(_v) = _t->getTextColor(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->isJulianDisplayed(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->isRevisedJulianDisplayed(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->isGregorianDisplayed(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->isISODisplayed(); break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->isIcelandicDisplayed(); break;
        case 8: *reinterpret_cast<bool*>(_v) = _t->isRomanDisplayed(); break;
        case 9: *reinterpret_cast<bool*>(_v) = _t->isOlympicDisplayed(); break;
        case 10: *reinterpret_cast<bool*>(_v) = _t->isEgyptianDisplayed(); break;
        case 11: *reinterpret_cast<bool*>(_v) = _t->isArmenianDisplayed(); break;
        case 12: *reinterpret_cast<bool*>(_v) = _t->isZoroastrianDisplayed(); break;
        case 13: *reinterpret_cast<bool*>(_v) = _t->isCopticDisplayed(); break;
        case 14: *reinterpret_cast<bool*>(_v) = _t->isEthiopicDisplayed(); break;
        case 15: *reinterpret_cast<bool*>(_v) = _t->isChineseDisplayed(); break;
        case 16: *reinterpret_cast<bool*>(_v) = _t->isJapaneseDisplayed(); break;
        case 17: *reinterpret_cast<bool*>(_v) = _t->isKoreanDisplayed(); break;
        case 18: *reinterpret_cast<bool*>(_v) = _t->isVietnameseDisplayed(); break;
        case 19: *reinterpret_cast<bool*>(_v) = _t->isIslamicDisplayed(); break;
        case 20: *reinterpret_cast<bool*>(_v) = _t->isHebrewDisplayed(); break;
        case 21: *reinterpret_cast<bool*>(_v) = _t->isOldHinduSolarDisplayed(); break;
        case 22: *reinterpret_cast<bool*>(_v) = _t->isOldHinduLunarDisplayed(); break;
        case 23: *reinterpret_cast<bool*>(_v) = _t->isNewHinduSolarDisplayed(); break;
        case 24: *reinterpret_cast<bool*>(_v) = _t->isNewHinduLunarDisplayed(); break;
        case 25: *reinterpret_cast<bool*>(_v) = _t->isAstroHinduSolarDisplayed(); break;
        case 26: *reinterpret_cast<bool*>(_v) = _t->isAstroHinduLunarDisplayed(); break;
        case 27: *reinterpret_cast<bool*>(_v) = _t->isMayaLongCountDisplayed(); break;
        case 28: *reinterpret_cast<bool*>(_v) = _t->isMayaHaabDisplayed(); break;
        case 29: *reinterpret_cast<bool*>(_v) = _t->isMayaTzolkinDisplayed(); break;
        case 30: *reinterpret_cast<bool*>(_v) = _t->isAztecXihuitlDisplayed(); break;
        case 31: *reinterpret_cast<bool*>(_v) = _t->isAztecTonalpohualliDisplayed(); break;
        case 32: *reinterpret_cast<bool*>(_v) = _t->isBalineseDisplayed(); break;
        case 33: *reinterpret_cast<bool*>(_v) = _t->isFrenchAstronomicalDisplayed(); break;
        case 34: *reinterpret_cast<bool*>(_v) = _t->isFrenchArithmeticDisplayed(); break;
        case 35: *reinterpret_cast<bool*>(_v) = _t->isPersianArithmeticDisplayed(); break;
        case 36: *reinterpret_cast<bool*>(_v) = _t->isPersianAstronomicalDisplayed(); break;
        case 37: *reinterpret_cast<bool*>(_v) = _t->isBahaiArithmeticDisplayed(); break;
        case 38: *reinterpret_cast<bool*>(_v) = _t->isBahaiAstronomicalDisplayed(); break;
        case 39: *reinterpret_cast<bool*>(_v) = _t->isTibetanDisplayed(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->enable(*reinterpret_cast<bool*>(_v)); break;
        case 1: _t->setFlagTextColorOverride(*reinterpret_cast<bool*>(_v)); break;
        case 2: _t->setTextColor(*reinterpret_cast<Vec3f*>(_v)); break;
        case 3: _t->showJulian(*reinterpret_cast<bool*>(_v)); break;
        case 4: _t->showRevisedJulian(*reinterpret_cast<bool*>(_v)); break;
        case 5: _t->showGregorian(*reinterpret_cast<bool*>(_v)); break;
        case 6: _t->showISO(*reinterpret_cast<bool*>(_v)); break;
        case 7: _t->showIcelandic(*reinterpret_cast<bool*>(_v)); break;
        case 8: _t->showRoman(*reinterpret_cast<bool*>(_v)); break;
        case 9: _t->showOlympic(*reinterpret_cast<bool*>(_v)); break;
        case 10: _t->showEgyptian(*reinterpret_cast<bool*>(_v)); break;
        case 11: _t->showArmenian(*reinterpret_cast<bool*>(_v)); break;
        case 12: _t->showZoroastrian(*reinterpret_cast<bool*>(_v)); break;
        case 13: _t->showCoptic(*reinterpret_cast<bool*>(_v)); break;
        case 14: _t->showEthiopic(*reinterpret_cast<bool*>(_v)); break;
        case 15: _t->showChinese(*reinterpret_cast<bool*>(_v)); break;
        case 16: _t->showJapanese(*reinterpret_cast<bool*>(_v)); break;
        case 17: _t->showKorean(*reinterpret_cast<bool*>(_v)); break;
        case 18: _t->showVietnamese(*reinterpret_cast<bool*>(_v)); break;
        case 19: _t->showIslamic(*reinterpret_cast<bool*>(_v)); break;
        case 20: _t->showHebrew(*reinterpret_cast<bool*>(_v)); break;
        case 21: _t->showOldHinduSolar(*reinterpret_cast<bool*>(_v)); break;
        case 22: _t->showOldHinduLunar(*reinterpret_cast<bool*>(_v)); break;
        case 23: _t->showNewHinduSolar(*reinterpret_cast<bool*>(_v)); break;
        case 24: _t->showNewHinduLunar(*reinterpret_cast<bool*>(_v)); break;
        case 25: _t->showAstroHinduSolar(*reinterpret_cast<bool*>(_v)); break;
        case 26: _t->showAstroHinduLunar(*reinterpret_cast<bool*>(_v)); break;
        case 27: _t->showMayaLongCount(*reinterpret_cast<bool*>(_v)); break;
        case 28: _t->showMayaHaab(*reinterpret_cast<bool*>(_v)); break;
        case 29: _t->showMayaTzolkin(*reinterpret_cast<bool*>(_v)); break;
        case 30: _t->showAztecXihuitl(*reinterpret_cast<bool*>(_v)); break;
        case 31: _t->showAztecTonalpohualli(*reinterpret_cast<bool*>(_v)); break;
        case 32: _t->showBalinese(*reinterpret_cast<bool*>(_v)); break;
        case 33: _t->showFrenchAstronomical(*reinterpret_cast<bool*>(_v)); break;
        case 34: _t->showFrenchArithmetic(*reinterpret_cast<bool*>(_v)); break;
        case 35: _t->showPersianArithmetic(*reinterpret_cast<bool*>(_v)); break;
        case 36: _t->showPersianAstronomical(*reinterpret_cast<bool*>(_v)); break;
        case 37: _t->showBahaiArithmetic(*reinterpret_cast<bool*>(_v)); break;
        case 38: _t->showBahaiAstronomical(*reinterpret_cast<bool*>(_v)); break;
        case 39: _t->showTibetan(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *Calendars::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Calendars::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9CalendarsE_t>.strings))
        return static_cast<void*>(this);
    return StelModule::qt_metacast(_clname);
}

int Calendars::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StelModule::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 121)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 121;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 121)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 121;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 40;
    }
    return _id;
}

// SIGNAL 0
void Calendars::enabledChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void Calendars::flagTextColorOverrideChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void Calendars::textColorChanged(Vec3f & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void Calendars::showJulianChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void Calendars::showRevisedJulianChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void Calendars::showGregorianChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void Calendars::showISOChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void Calendars::showIcelandicChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void Calendars::showRomanChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void Calendars::showOlympicChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void Calendars::showEgyptianChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void Calendars::showArmenianChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void Calendars::showZoroastrianChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void Calendars::showCopticChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1);
}

// SIGNAL 14
void Calendars::showEthiopicChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1);
}

// SIGNAL 15
void Calendars::showChineseChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 15, nullptr, _t1);
}

// SIGNAL 16
void Calendars::showJapaneseChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 16, nullptr, _t1);
}

// SIGNAL 17
void Calendars::showKoreanChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 17, nullptr, _t1);
}

// SIGNAL 18
void Calendars::showVietnameseChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 18, nullptr, _t1);
}

// SIGNAL 19
void Calendars::showIslamicChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 19, nullptr, _t1);
}

// SIGNAL 20
void Calendars::showHebrewChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 20, nullptr, _t1);
}

// SIGNAL 21
void Calendars::showOldHinduSolarChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 21, nullptr, _t1);
}

// SIGNAL 22
void Calendars::showOldHinduLunarChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 22, nullptr, _t1);
}

// SIGNAL 23
void Calendars::showNewHinduSolarChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 23, nullptr, _t1);
}

// SIGNAL 24
void Calendars::showNewHinduLunarChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 24, nullptr, _t1);
}

// SIGNAL 25
void Calendars::showAstroHinduSolarChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 25, nullptr, _t1);
}

// SIGNAL 26
void Calendars::showAstroHinduLunarChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 26, nullptr, _t1);
}

// SIGNAL 27
void Calendars::showMayaLongCountChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 27, nullptr, _t1);
}

// SIGNAL 28
void Calendars::showMayaHaabChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 28, nullptr, _t1);
}

// SIGNAL 29
void Calendars::showMayaTzolkinChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 29, nullptr, _t1);
}

// SIGNAL 30
void Calendars::showAztecXihuitlChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 30, nullptr, _t1);
}

// SIGNAL 31
void Calendars::showAztecTonalpohualliChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 31, nullptr, _t1);
}

// SIGNAL 32
void Calendars::showBalineseChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 32, nullptr, _t1);
}

// SIGNAL 33
void Calendars::showFrenchAstronomicalChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 33, nullptr, _t1);
}

// SIGNAL 34
void Calendars::showFrenchArithmeticChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 34, nullptr, _t1);
}

// SIGNAL 35
void Calendars::showPersianArithmeticChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 35, nullptr, _t1);
}

// SIGNAL 36
void Calendars::showPersianAstronomicalChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 36, nullptr, _t1);
}

// SIGNAL 37
void Calendars::showBahaiArithmeticChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 37, nullptr, _t1);
}

// SIGNAL 38
void Calendars::showBahaiAstronomicalChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 38, nullptr, _t1);
}

// SIGNAL 39
void Calendars::showTibetanChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 39, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN28CalendarsStelPluginInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto CalendarsStelPluginInterface::qt_create_metaobjectdata<qt_meta_tag_ZN28CalendarsStelPluginInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "CalendarsStelPluginInterface"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CalendarsStelPluginInterface, qt_meta_tag_ZN28CalendarsStelPluginInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject CalendarsStelPluginInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28CalendarsStelPluginInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28CalendarsStelPluginInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN28CalendarsStelPluginInterfaceE_t>.metaTypes,
    nullptr
} };

void CalendarsStelPluginInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CalendarsStelPluginInterface *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *CalendarsStelPluginInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CalendarsStelPluginInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28CalendarsStelPluginInterfaceE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "StelPluginInterface"))
        return static_cast< StelPluginInterface*>(this);
    if (!strcmp(_clname, "org.stellarium.StelPluginInterface/2.0"))
        return static_cast< StelPluginInterface*>(this);
    return QObject::qt_metacast(_clname);
}

int CalendarsStelPluginInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}

#ifdef QT_MOC_EXPORT_PLUGIN_V2
static constexpr unsigned char qt_pluginMetaDataV2_CalendarsStelPluginInterface[] = {
    0xbf, 
    // "IID"
    0x02,  0x78,  0x26,  'o',  'r',  'g',  '.',  's', 
    't',  'e',  'l',  'l',  'a',  'r',  'i',  'u', 
    'm',  '.',  'S',  't',  'e',  'l',  'P',  'l', 
    'u',  'g',  'i',  'n',  'I',  'n',  't',  'e', 
    'r',  'f',  'a',  'c',  'e',  '/',  '2',  '.', 
    '0', 
    // "className"
    0x03,  0x78,  0x1c,  'C',  'a',  'l',  'e',  'n', 
    'd',  'a',  'r',  's',  'S',  't',  'e',  'l', 
    'P',  'l',  'u',  'g',  'i',  'n',  'I',  'n', 
    't',  'e',  'r',  'f',  'a',  'c',  'e', 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN_V2(CalendarsStelPluginInterface, CalendarsStelPluginInterface, qt_pluginMetaDataV2_CalendarsStelPluginInterface)
#else
QT_PLUGIN_METADATA_SECTION
Q_CONSTINIT static constexpr unsigned char qt_pluginMetaData_CalendarsStelPluginInterface[] = {
    'Q', 'T', 'M', 'E', 'T', 'A', 'D', 'A', 'T', 'A', ' ', '!',
    // metadata version, Qt version, architectural requirements
    0, QT_VERSION_MAJOR, QT_VERSION_MINOR, qPluginArchRequirements(),
    0xbf, 
    // "IID"
    0x02,  0x78,  0x26,  'o',  'r',  'g',  '.',  's', 
    't',  'e',  'l',  'l',  'a',  'r',  'i',  'u', 
    'm',  '.',  'S',  't',  'e',  'l',  'P',  'l', 
    'u',  'g',  'i',  'n',  'I',  'n',  't',  'e', 
    'r',  'f',  'a',  'c',  'e',  '/',  '2',  '.', 
    '0', 
    // "className"
    0x03,  0x78,  0x1c,  'C',  'a',  'l',  'e',  'n', 
    'd',  'a',  'r',  's',  'S',  't',  'e',  'l', 
    'P',  'l',  'u',  'g',  'i',  'n',  'I',  'n', 
    't',  'e',  'r',  'f',  'a',  'c',  'e', 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN(CalendarsStelPluginInterface, CalendarsStelPluginInterface)
#endif  // QT_MOC_EXPORT_PLUGIN_V2

QT_WARNING_POP
