/********************************************************************************
** Form generated from reading UI file 'pointerCoordinatesWindow.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_POINTERCOORDINATESWINDOW_H
#define UI_POINTERCOORDINATESWINDOW_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTextBrowser>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "Dialog.hpp"
#include "gui/ColorButton.hpp"

QT_BEGIN_NAMESPACE

class Ui_pointerCoordinatesWindowForm
{
public:
    QVBoxLayout *verticalLayout;
    TitleBar *titleBar;
    QTabWidget *tabWidget;
    QWidget *tabCoordinates;
    QVBoxLayout *verticalLayoutTimeZone;
    QGroupBox *groupBoxCoordinates;
    QVBoxLayout *verticalLayout_2;
    QCheckBox *checkBoxEnableAtStartup;
    QCheckBox *checkBoxShowButton;
    QCheckBox *checkBoxConstellation;
    QCheckBox *checkBoxCrossedLines;
    QCheckBox *checkBoxElongation;
    QHBoxLayout *horizontalLayout_4;
    QLabel *labelCoordinateSystem;
    QComboBox *coordinateSystemComboBox;
    QHBoxLayout *horizontalLayout;
    ColorButton *textColorButton;
    QLabel *labelFontSize;
    QSpinBox *spinBoxFontSize;
    QHBoxLayout *horizontalLayout_3;
    QLabel *placeLabel;
    QComboBox *placeComboBox;
    QHBoxLayout *horizontalLayoutCustomCoords;
    QSpacerItem *horizontalSpacer;
    QLabel *labelCustomCoords;
    QSpinBox *spinBoxX;
    QSpinBox *spinBoxY;
    QSpacerItem *verticalSpacer;
    QHBoxLayout *horizontalLayout_2;
    QPushButton *pushButtonReset;
    QSpacerItem *horizontalSpacer_2;
    QPushButton *pushButtonSave;
    QWidget *tabAbout;
    QVBoxLayout *verticalLayout_3;
    QTextBrowser *aboutTextBrowser;

    void setupUi(QWidget *pointerCoordinatesWindowForm)
    {
        if (pointerCoordinatesWindowForm->objectName().isEmpty())
            pointerCoordinatesWindowForm->setObjectName("pointerCoordinatesWindowForm");
        pointerCoordinatesWindowForm->resize(506, 461);
        verticalLayout = new QVBoxLayout(pointerCoordinatesWindowForm);
        verticalLayout->setSpacing(0);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        titleBar = new TitleBar(pointerCoordinatesWindowForm);
        titleBar->setObjectName("titleBar");

        verticalLayout->addWidget(titleBar);

        tabWidget = new QTabWidget(pointerCoordinatesWindowForm);
        tabWidget->setObjectName("tabWidget");
        tabCoordinates = new QWidget();
        tabCoordinates->setObjectName("tabCoordinates");
        verticalLayoutTimeZone = new QVBoxLayout(tabCoordinates);
        verticalLayoutTimeZone->setSpacing(0);
        verticalLayoutTimeZone->setObjectName("verticalLayoutTimeZone");
        verticalLayoutTimeZone->setContentsMargins(0, 0, 0, 0);
        groupBoxCoordinates = new QGroupBox(tabCoordinates);
        groupBoxCoordinates->setObjectName("groupBoxCoordinates");
        groupBoxCoordinates->setMinimumSize(QSize(0, 400));
        groupBoxCoordinates->setAlignment(Qt::AlignCenter);
        verticalLayout_2 = new QVBoxLayout(groupBoxCoordinates);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        checkBoxEnableAtStartup = new QCheckBox(groupBoxCoordinates);
        checkBoxEnableAtStartup->setObjectName("checkBoxEnableAtStartup");

        verticalLayout_2->addWidget(checkBoxEnableAtStartup);

        checkBoxShowButton = new QCheckBox(groupBoxCoordinates);
        checkBoxShowButton->setObjectName("checkBoxShowButton");

        verticalLayout_2->addWidget(checkBoxShowButton);

        checkBoxConstellation = new QCheckBox(groupBoxCoordinates);
        checkBoxConstellation->setObjectName("checkBoxConstellation");

        verticalLayout_2->addWidget(checkBoxConstellation);

        checkBoxCrossedLines = new QCheckBox(groupBoxCoordinates);
        checkBoxCrossedLines->setObjectName("checkBoxCrossedLines");

        verticalLayout_2->addWidget(checkBoxCrossedLines);

        checkBoxElongation = new QCheckBox(groupBoxCoordinates);
        checkBoxElongation->setObjectName("checkBoxElongation");

        verticalLayout_2->addWidget(checkBoxElongation);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        labelCoordinateSystem = new QLabel(groupBoxCoordinates);
        labelCoordinateSystem->setObjectName("labelCoordinateSystem");

        horizontalLayout_4->addWidget(labelCoordinateSystem);

        coordinateSystemComboBox = new QComboBox(groupBoxCoordinates);
        coordinateSystemComboBox->setObjectName("coordinateSystemComboBox");
        coordinateSystemComboBox->setEditable(false);
        coordinateSystemComboBox->setInsertPolicy(QComboBox::NoInsert);

        horizontalLayout_4->addWidget(coordinateSystemComboBox);


        verticalLayout_2->addLayout(horizontalLayout_4);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        textColorButton = new ColorButton(groupBoxCoordinates);
        textColorButton->setObjectName("textColorButton");

        horizontalLayout->addWidget(textColorButton);

        labelFontSize = new QLabel(groupBoxCoordinates);
        labelFontSize->setObjectName("labelFontSize");

        horizontalLayout->addWidget(labelFontSize);

        spinBoxFontSize = new QSpinBox(groupBoxCoordinates);
        spinBoxFontSize->setObjectName("spinBoxFontSize");
        spinBoxFontSize->setMinimum(10);
        spinBoxFontSize->setMaximum(50);
        spinBoxFontSize->setValue(14);

        horizontalLayout->addWidget(spinBoxFontSize);


        verticalLayout_2->addLayout(horizontalLayout);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        placeLabel = new QLabel(groupBoxCoordinates);
        placeLabel->setObjectName("placeLabel");

        horizontalLayout_3->addWidget(placeLabel);

        placeComboBox = new QComboBox(groupBoxCoordinates);
        placeComboBox->setObjectName("placeComboBox");
        placeComboBox->setEditable(false);
        placeComboBox->setInsertPolicy(QComboBox::NoInsert);
        placeComboBox->setSizeAdjustPolicy(QComboBox::AdjustToContents);

        horizontalLayout_3->addWidget(placeComboBox);


        verticalLayout_2->addLayout(horizontalLayout_3);

        horizontalLayoutCustomCoords = new QHBoxLayout();
        horizontalLayoutCustomCoords->setObjectName("horizontalLayoutCustomCoords");
        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayoutCustomCoords->addItem(horizontalSpacer);

        labelCustomCoords = new QLabel(groupBoxCoordinates);
        labelCustomCoords->setObjectName("labelCustomCoords");
        labelCustomCoords->setEnabled(true);

        horizontalLayoutCustomCoords->addWidget(labelCustomCoords);

        spinBoxX = new QSpinBox(groupBoxCoordinates);
        spinBoxX->setObjectName("spinBoxX");
        spinBoxX->setEnabled(true);
        spinBoxX->setMinimum(1);
        spinBoxX->setMaximum(9999);

        horizontalLayoutCustomCoords->addWidget(spinBoxX);

        spinBoxY = new QSpinBox(groupBoxCoordinates);
        spinBoxY->setObjectName("spinBoxY");
        spinBoxY->setEnabled(true);
        spinBoxY->setMinimum(1);
        spinBoxY->setMaximum(9999);

        horizontalLayoutCustomCoords->addWidget(spinBoxY);


        verticalLayout_2->addLayout(horizontalLayoutCustomCoords);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_2->addItem(verticalSpacer);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        pushButtonReset = new QPushButton(groupBoxCoordinates);
        pushButtonReset->setObjectName("pushButtonReset");

        horizontalLayout_2->addWidget(pushButtonReset);

        horizontalSpacer_2 = new QSpacerItem(20, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_2);

        pushButtonSave = new QPushButton(groupBoxCoordinates);
        pushButtonSave->setObjectName("pushButtonSave");

        horizontalLayout_2->addWidget(pushButtonSave);


        verticalLayout_2->addLayout(horizontalLayout_2);


        verticalLayoutTimeZone->addWidget(groupBoxCoordinates);

        tabWidget->addTab(tabCoordinates, QString());
        tabAbout = new QWidget();
        tabAbout->setObjectName("tabAbout");
        verticalLayout_3 = new QVBoxLayout(tabAbout);
        verticalLayout_3->setObjectName("verticalLayout_3");
        aboutTextBrowser = new QTextBrowser(tabAbout);
        aboutTextBrowser->setObjectName("aboutTextBrowser");
        aboutTextBrowser->setOpenExternalLinks(true);

        verticalLayout_3->addWidget(aboutTextBrowser);

        tabWidget->addTab(tabAbout, QString());

        verticalLayout->addWidget(tabWidget);


        retranslateUi(pointerCoordinatesWindowForm);

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(pointerCoordinatesWindowForm);
    } // setupUi

    void retranslateUi(QWidget *pointerCoordinatesWindowForm)
    {
        titleBar->setTitle(QCoreApplication::translate("pointerCoordinatesWindowForm", "Pointer Coordinates", nullptr));
        groupBoxCoordinates->setTitle(QCoreApplication::translate("pointerCoordinatesWindowForm", "Pointer Coordinates", nullptr));
        checkBoxEnableAtStartup->setText(QCoreApplication::translate("pointerCoordinatesWindowForm", "Enable display of coordinates at startup", nullptr));
        checkBoxShowButton->setText(QCoreApplication::translate("pointerCoordinatesWindowForm", "Show plug-in button on toolbar", nullptr));
#if QT_CONFIG(tooltip)
        checkBoxConstellation->setToolTip(QCoreApplication::translate("pointerCoordinatesWindowForm", "3-letter IAU constellation abbreviation", nullptr));
#endif // QT_CONFIG(tooltip)
        checkBoxConstellation->setText(QCoreApplication::translate("pointerCoordinatesWindowForm", "Show IAU constellation code", nullptr));
        checkBoxCrossedLines->setText(QCoreApplication::translate("pointerCoordinatesWindowForm", "Show crossed lines under mouse cursor", nullptr));
        checkBoxElongation->setText(QCoreApplication::translate("pointerCoordinatesWindowForm", "Show elongation from the Sun in ecliptic longitude", nullptr));
#if QT_CONFIG(tooltip)
        labelCoordinateSystem->setToolTip(QCoreApplication::translate("pointerCoordinatesWindowForm", "Which coordinate system will be used for displaying", nullptr));
#endif // QT_CONFIG(tooltip)
        labelCoordinateSystem->setText(QCoreApplication::translate("pointerCoordinatesWindowForm", "Coordinates:", nullptr));
#if QT_CONFIG(tooltip)
        textColorButton->setToolTip(QCoreApplication::translate("pointerCoordinatesWindowForm", "The color of the screen text", nullptr));
#endif // QT_CONFIG(tooltip)
        textColorButton->setText(QString());
        labelFontSize->setText(QCoreApplication::translate("pointerCoordinatesWindowForm", "Font size:", nullptr));
#if QT_CONFIG(tooltip)
        placeLabel->setToolTip(QCoreApplication::translate("pointerCoordinatesWindowForm", "Place of the string with coordinates of the mouse pointer", nullptr));
#endif // QT_CONFIG(tooltip)
        placeLabel->setText(QCoreApplication::translate("pointerCoordinatesWindowForm", "Place of the string:", nullptr));
#if QT_CONFIG(tooltip)
        spinBoxX->setToolTip(QCoreApplication::translate("pointerCoordinatesWindowForm", "Coordinate X", nullptr));
#endif // QT_CONFIG(tooltip)
        spinBoxX->setSpecialValueText(QString());
#if QT_CONFIG(tooltip)
        spinBoxY->setToolTip(QCoreApplication::translate("pointerCoordinatesWindowForm", "Coordinate Y", nullptr));
#endif // QT_CONFIG(tooltip)
        spinBoxY->setSpecialValueText(QString());
        pushButtonReset->setText(QCoreApplication::translate("pointerCoordinatesWindowForm", "Restore defaults", nullptr));
        pushButtonSave->setText(QCoreApplication::translate("pointerCoordinatesWindowForm", "Save settings", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabCoordinates), QCoreApplication::translate("pointerCoordinatesWindowForm", "Pointer Coordinates", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabAbout), QCoreApplication::translate("pointerCoordinatesWindowForm", "About", "tab in plugin windows"));
        (void)pointerCoordinatesWindowForm;
    } // retranslateUi

};

namespace Ui {
    class pointerCoordinatesWindowForm: public Ui_pointerCoordinatesWindowForm {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_POINTERCOORDINATESWINDOW_H
