/********************************************************************************
** Form generated from reading UI file 'obsListDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_OBSLISTDIALOG_H
#define UI_OBSLISTDIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTreeView>
#include <QtWidgets/QWidget>
#include "Dialog.hpp"
#include "gui/StelCloseButton.hpp"

QT_BEGIN_NAMESPACE

class Ui_obsListDialogForm
{
public:
    QGridLayout *gridLayout_2;
    TitleBar *titleBar;
    QWidget *widget;
    QGridLayout *gridLayout;
    QTreeView *treeView;
    QHBoxLayout *horizontalLayout_editButtons;
    QPushButton *addObjectButton;
    QPushButton *removeObjectButton;
    QPushButton *saveButton;
    QSpacerItem *horizontalSpacer_edit;
    QPushButton *cancelButton;
    QHBoxLayout *horizontalLayout_Description;
    QLabel *descriptionLineLabel;
    QSpacerItem *horizontalSpacer_description;
    QLineEdit *descriptionLineEdit;
    QHBoxLayout *horizontalLayout_checkBoxDefault;
    QCheckBox *defaultListCheckBox;
    QSpacerItem *horizontalSpacer_default_date;
    QLabel *creationDateLabel;
    QLineEdit *creationDateLineEdit;
    QLabel *lastEditLabel;
    QLineEdit *lastEditLineEdit;
    QHBoxLayout *horizontalLayout_listName;
    QLabel *listNameLabel;
    QSpacerItem *horizontalSpacer_listName;
    QLineEdit *listNameLineEdit;
    QHBoxLayout *horizontalLayout_listCombo;
    QLabel *obsListComboLabel;
    QComboBox *obsListComboBox;
    QHBoxLayout *horizontalLayout_Buttons;
    QPushButton *highlightAllButton;
    QPushButton *clearHighlightButton;
    QPushButton *newListButton;
    QPushButton *editListButton;
    QPushButton *exportListButton;
    QPushButton *importListButton;
    QPushButton *deleteListButton;
    QHBoxLayout *horizontalLayout_options;
    QLabel *alsoLoadLabel;
    QLabel *alsoStoreLabel;
    QCheckBox *coordinatesCheckBox;
    QCheckBox *jdCheckBox;
    QCheckBox *locationCheckBox;
    QCheckBox *landscapeCheckBox;
    QCheckBox *fovCheckBox;
    QSpacerItem *horizontalSpacer_options;
    QHBoxLayout *horizontalLayout_obsListDir;
    QLabel *label;
    QLineEdit *obsListDirEdit;
    QPushButton *obsListBrowseButton;

    void setupUi(QWidget *obsListDialogForm)
    {
        if (obsListDialogForm->objectName().isEmpty())
            obsListDialogForm->setObjectName("obsListDialogForm");
        obsListDialogForm->resize(760, 448);
        gridLayout_2 = new QGridLayout(obsListDialogForm);
        gridLayout_2->setSpacing(0);
        gridLayout_2->setObjectName("gridLayout_2");
        gridLayout_2->setContentsMargins(0, 0, 0, 0);
        titleBar = new TitleBar(obsListDialogForm);
        titleBar->setObjectName("titleBar");

        gridLayout_2->addWidget(titleBar, 0, 0, 1, 1);

        widget = new QWidget(obsListDialogForm);
        widget->setObjectName("widget");
        gridLayout = new QGridLayout(widget);
        gridLayout->setObjectName("gridLayout");
        treeView = new QTreeView(widget);
        treeView->setObjectName("treeView");
        treeView->setToolTipDuration(-1);

        gridLayout->addWidget(treeView, 6, 0, 1, 1);

        horizontalLayout_editButtons = new QHBoxLayout();
        horizontalLayout_editButtons->setObjectName("horizontalLayout_editButtons");
        addObjectButton = new QPushButton(widget);
        addObjectButton->setObjectName("addObjectButton");

        horizontalLayout_editButtons->addWidget(addObjectButton);

        removeObjectButton = new QPushButton(widget);
        removeObjectButton->setObjectName("removeObjectButton");

        horizontalLayout_editButtons->addWidget(removeObjectButton);

        saveButton = new QPushButton(widget);
        saveButton->setObjectName("saveButton");

        horizontalLayout_editButtons->addWidget(saveButton);

        horizontalSpacer_edit = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_editButtons->addItem(horizontalSpacer_edit);

        cancelButton = new QPushButton(widget);
        cancelButton->setObjectName("cancelButton");

        horizontalLayout_editButtons->addWidget(cancelButton);


        gridLayout->addLayout(horizontalLayout_editButtons, 9, 0, 1, 1);

        horizontalLayout_Description = new QHBoxLayout();
        horizontalLayout_Description->setObjectName("horizontalLayout_Description");
        descriptionLineLabel = new QLabel(widget);
        descriptionLineLabel->setObjectName("descriptionLineLabel");

        horizontalLayout_Description->addWidget(descriptionLineLabel);

        horizontalSpacer_description = new QSpacerItem(62, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout_Description->addItem(horizontalSpacer_description);

        descriptionLineEdit = new QLineEdit(widget);
        descriptionLineEdit->setObjectName("descriptionLineEdit");

        horizontalLayout_Description->addWidget(descriptionLineEdit);


        gridLayout->addLayout(horizontalLayout_Description, 4, 0, 1, 1);

        horizontalLayout_checkBoxDefault = new QHBoxLayout();
        horizontalLayout_checkBoxDefault->setObjectName("horizontalLayout_checkBoxDefault");
        defaultListCheckBox = new QCheckBox(widget);
        defaultListCheckBox->setObjectName("defaultListCheckBox");
        defaultListCheckBox->setToolTipDuration(3);

        horizontalLayout_checkBoxDefault->addWidget(defaultListCheckBox);

        horizontalSpacer_default_date = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_checkBoxDefault->addItem(horizontalSpacer_default_date);

        creationDateLabel = new QLabel(widget);
        creationDateLabel->setObjectName("creationDateLabel");

        horizontalLayout_checkBoxDefault->addWidget(creationDateLabel);

        creationDateLineEdit = new QLineEdit(widget);
        creationDateLineEdit->setObjectName("creationDateLineEdit");

        horizontalLayout_checkBoxDefault->addWidget(creationDateLineEdit);

        lastEditLabel = new QLabel(widget);
        lastEditLabel->setObjectName("lastEditLabel");

        horizontalLayout_checkBoxDefault->addWidget(lastEditLabel);

        lastEditLineEdit = new QLineEdit(widget);
        lastEditLineEdit->setObjectName("lastEditLineEdit");

        horizontalLayout_checkBoxDefault->addWidget(lastEditLineEdit);


        gridLayout->addLayout(horizontalLayout_checkBoxDefault, 5, 0, 1, 1);

        horizontalLayout_listName = new QHBoxLayout();
        horizontalLayout_listName->setObjectName("horizontalLayout_listName");
        listNameLabel = new QLabel(widget);
        listNameLabel->setObjectName("listNameLabel");

        horizontalLayout_listName->addWidget(listNameLabel);

        horizontalSpacer_listName = new QSpacerItem(39, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout_listName->addItem(horizontalSpacer_listName);

        listNameLineEdit = new QLineEdit(widget);
        listNameLineEdit->setObjectName("listNameLineEdit");

        horizontalLayout_listName->addWidget(listNameLineEdit);


        gridLayout->addLayout(horizontalLayout_listName, 3, 0, 1, 1);

        horizontalLayout_listCombo = new QHBoxLayout();
        horizontalLayout_listCombo->setObjectName("horizontalLayout_listCombo");
        obsListComboLabel = new QLabel(widget);
        obsListComboLabel->setObjectName("obsListComboLabel");
        obsListComboLabel->setMaximumSize(QSize(50, 16777215));

        horizontalLayout_listCombo->addWidget(obsListComboLabel);

        obsListComboBox = new QComboBox(widget);
        obsListComboBox->setObjectName("obsListComboBox");

        horizontalLayout_listCombo->addWidget(obsListComboBox);


        gridLayout->addLayout(horizontalLayout_listCombo, 2, 0, 1, 1);

        horizontalLayout_Buttons = new QHBoxLayout();
        horizontalLayout_Buttons->setObjectName("horizontalLayout_Buttons");
        horizontalLayout_Buttons->setContentsMargins(-1, 0, -1, -1);
        highlightAllButton = new QPushButton(widget);
        highlightAllButton->setObjectName("highlightAllButton");
        highlightAllButton->setToolTipDuration(3);

        horizontalLayout_Buttons->addWidget(highlightAllButton);

        clearHighlightButton = new QPushButton(widget);
        clearHighlightButton->setObjectName("clearHighlightButton");
        clearHighlightButton->setToolTipDuration(3);

        horizontalLayout_Buttons->addWidget(clearHighlightButton);

        newListButton = new QPushButton(widget);
        newListButton->setObjectName("newListButton");
        newListButton->setToolTipDuration(3);

        horizontalLayout_Buttons->addWidget(newListButton);

        editListButton = new QPushButton(widget);
        editListButton->setObjectName("editListButton");
        editListButton->setToolTipDuration(3);

        horizontalLayout_Buttons->addWidget(editListButton);

        exportListButton = new QPushButton(widget);
        exportListButton->setObjectName("exportListButton");
        exportListButton->setToolTipDuration(3);

        horizontalLayout_Buttons->addWidget(exportListButton);

        importListButton = new QPushButton(widget);
        importListButton->setObjectName("importListButton");
        importListButton->setToolTipDuration(3);

        horizontalLayout_Buttons->addWidget(importListButton);

        deleteListButton = new QPushButton(widget);
        deleteListButton->setObjectName("deleteListButton");

        horizontalLayout_Buttons->addWidget(deleteListButton);


        gridLayout->addLayout(horizontalLayout_Buttons, 8, 0, 1, 1);

        horizontalLayout_options = new QHBoxLayout();
        horizontalLayout_options->setObjectName("horizontalLayout_options");
        alsoLoadLabel = new QLabel(widget);
        alsoLoadLabel->setObjectName("alsoLoadLabel");

        horizontalLayout_options->addWidget(alsoLoadLabel);

        alsoStoreLabel = new QLabel(widget);
        alsoStoreLabel->setObjectName("alsoStoreLabel");

        horizontalLayout_options->addWidget(alsoStoreLabel);

        coordinatesCheckBox = new QCheckBox(widget);
        coordinatesCheckBox->setObjectName("coordinatesCheckBox");
        coordinatesCheckBox->setToolTipDuration(3);

        horizontalLayout_options->addWidget(coordinatesCheckBox);

        jdCheckBox = new QCheckBox(widget);
        jdCheckBox->setObjectName("jdCheckBox");

        horizontalLayout_options->addWidget(jdCheckBox);

        locationCheckBox = new QCheckBox(widget);
        locationCheckBox->setObjectName("locationCheckBox");

        horizontalLayout_options->addWidget(locationCheckBox);

        landscapeCheckBox = new QCheckBox(widget);
        landscapeCheckBox->setObjectName("landscapeCheckBox");
        landscapeCheckBox->setToolTipDuration(3);

        horizontalLayout_options->addWidget(landscapeCheckBox);

        fovCheckBox = new QCheckBox(widget);
        fovCheckBox->setObjectName("fovCheckBox");
        fovCheckBox->setToolTipDuration(3);

        horizontalLayout_options->addWidget(fovCheckBox);

        horizontalSpacer_options = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_options->addItem(horizontalSpacer_options);

        horizontalLayout_obsListDir = new QHBoxLayout();
        horizontalLayout_obsListDir->setObjectName("horizontalLayout_obsListDir");
        label = new QLabel(widget);
        label->setObjectName("label");

        horizontalLayout_obsListDir->addWidget(label);

        obsListDirEdit = new QLineEdit(widget);
        obsListDirEdit->setObjectName("obsListDirEdit");

        horizontalLayout_obsListDir->addWidget(obsListDirEdit);

        obsListBrowseButton = new QPushButton(widget);
        obsListBrowseButton->setObjectName("obsListBrowseButton");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(obsListBrowseButton->sizePolicy().hasHeightForWidth());
        obsListBrowseButton->setSizePolicy(sizePolicy);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/graphicGui/uibtFolder.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        icon.addFile(QString::fromUtf8(":/graphicGui/uibtFolder-disabled.png"), QSize(), QIcon::Mode::Disabled, QIcon::State::Off);
        obsListBrowseButton->setIcon(icon);

        horizontalLayout_obsListDir->addWidget(obsListBrowseButton);


        horizontalLayout_options->addLayout(horizontalLayout_obsListDir);


        gridLayout->addLayout(horizontalLayout_options, 7, 0, 1, 1);


        gridLayout_2->addWidget(widget, 1, 0, 1, 1);


        retranslateUi(obsListDialogForm);

        QMetaObject::connectSlotsByName(obsListDialogForm);
    } // setupUi

    void retranslateUi(QWidget *obsListDialogForm)
    {
        titleBar->setTitle(QCoreApplication::translate("obsListDialogForm", "Observing lists", nullptr));
        addObjectButton->setText(QCoreApplication::translate("obsListDialogForm", "Add object", nullptr));
        removeObjectButton->setText(QCoreApplication::translate("obsListDialogForm", "Remove object", nullptr));
        saveButton->setText(QCoreApplication::translate("obsListDialogForm", "Save list", nullptr));
        cancelButton->setText(QCoreApplication::translate("obsListDialogForm", "Cancel", nullptr));
        descriptionLineLabel->setText(QCoreApplication::translate("obsListDialogForm", "Description:", nullptr));
#if QT_CONFIG(tooltip)
        defaultListCheckBox->setToolTip(QCoreApplication::translate("obsListDialogForm", "Activate to set list as default", nullptr));
#endif // QT_CONFIG(tooltip)
        defaultListCheckBox->setText(QCoreApplication::translate("obsListDialogForm", "Default list", nullptr));
        creationDateLabel->setText(QCoreApplication::translate("obsListDialogForm", "Creation date:", nullptr));
        lastEditLabel->setText(QCoreApplication::translate("obsListDialogForm", "Last edit:", nullptr));
        listNameLabel->setText(QCoreApplication::translate("obsListDialogForm", "List name:", nullptr));
        obsListComboLabel->setText(QCoreApplication::translate("obsListDialogForm", "List:", nullptr));
#if QT_CONFIG(tooltip)
        highlightAllButton->setToolTip(QCoreApplication::translate("obsListDialogForm", "Highlight all the objects of the selected list", nullptr));
#endif // QT_CONFIG(tooltip)
        highlightAllButton->setText(QCoreApplication::translate("obsListDialogForm", "Highlight all", nullptr));
#if QT_CONFIG(tooltip)
        clearHighlightButton->setToolTip(QCoreApplication::translate("obsListDialogForm", "Clear all the highlight objects", nullptr));
#endif // QT_CONFIG(tooltip)
        clearHighlightButton->setText(QCoreApplication::translate("obsListDialogForm", "Clear highlight(s)", nullptr));
#if QT_CONFIG(tooltip)
        newListButton->setToolTip(QCoreApplication::translate("obsListDialogForm", "Create a new observing list", nullptr));
#endif // QT_CONFIG(tooltip)
        newListButton->setText(QCoreApplication::translate("obsListDialogForm", "New list", nullptr));
#if QT_CONFIG(tooltip)
        editListButton->setToolTip(QCoreApplication::translate("obsListDialogForm", "Edit the selected list", nullptr));
#endif // QT_CONFIG(tooltip)
        editListButton->setText(QCoreApplication::translate("obsListDialogForm", "Edit list", nullptr));
#if QT_CONFIG(tooltip)
        exportListButton->setToolTip(QCoreApplication::translate("obsListDialogForm", "Export list", nullptr));
#endif // QT_CONFIG(tooltip)
        exportListButton->setText(QCoreApplication::translate("obsListDialogForm", "Export list...", nullptr));
#if QT_CONFIG(tooltip)
        importListButton->setToolTip(QCoreApplication::translate("obsListDialogForm", "Import list", nullptr));
#endif // QT_CONFIG(tooltip)
        importListButton->setText(QCoreApplication::translate("obsListDialogForm", "Import list...", nullptr));
#if QT_CONFIG(tooltip)
        deleteListButton->setToolTip(QCoreApplication::translate("obsListDialogForm", "Delete the selected list", nullptr));
#endif // QT_CONFIG(tooltip)
        deleteListButton->setText(QCoreApplication::translate("obsListDialogForm", "Delete list", nullptr));
        alsoLoadLabel->setText(QCoreApplication::translate("obsListDialogForm", "Also load:", nullptr));
        alsoStoreLabel->setText(QCoreApplication::translate("obsListDialogForm", "Also store:", nullptr));
#if QT_CONFIG(tooltip)
        coordinatesCheckBox->setToolTip(QCoreApplication::translate("obsListDialogForm", "Store/load equatorial coordinates (J2000.0)", nullptr));
#endif // QT_CONFIG(tooltip)
        coordinatesCheckBox->setText(QCoreApplication::translate("obsListDialogForm", "Coordinates (RA and DEC)", nullptr));
#if QT_CONFIG(tooltip)
        jdCheckBox->setToolTip(QCoreApplication::translate("obsListDialogForm", "Store/load with date/time (JD)", nullptr));
#endif // QT_CONFIG(tooltip)
        jdCheckBox->setText(QCoreApplication::translate("obsListDialogForm", "Date", nullptr));
#if QT_CONFIG(tooltip)
        locationCheckBox->setToolTip(QCoreApplication::translate("obsListDialogForm", "Store/load with location", nullptr));
#endif // QT_CONFIG(tooltip)
        locationCheckBox->setText(QCoreApplication::translate("obsListDialogForm", "Location", nullptr));
#if QT_CONFIG(tooltip)
        landscapeCheckBox->setToolTip(QCoreApplication::translate("obsListDialogForm", "Store/load with landscape", nullptr));
#endif // QT_CONFIG(tooltip)
        landscapeCheckBox->setText(QCoreApplication::translate("obsListDialogForm", "Landscape", nullptr));
#if QT_CONFIG(tooltip)
        fovCheckBox->setToolTip(QCoreApplication::translate("obsListDialogForm", "Store/load with field of view", nullptr));
#endif // QT_CONFIG(tooltip)
        fovCheckBox->setText(QCoreApplication::translate("obsListDialogForm", "FOV", nullptr));
#if QT_CONFIG(tooltip)
        label->setToolTip(QCoreApplication::translate("obsListDialogForm", "Observing Lists Directory", nullptr));
#endif // QT_CONFIG(tooltip)
        label->setText(QCoreApplication::translate("obsListDialogForm", "Dir", nullptr));
        obsListBrowseButton->setText(QString());
        (void)obsListDialogForm;
    } // retranslateUi

};

namespace Ui {
    class obsListDialogForm: public Ui_obsListDialogForm {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_OBSLISTDIALOG_H
