/********************************************************************************
** Form generated from reading UI file 'scriptConsole.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SCRIPTCONSOLE_H
#define UI_SCRIPTCONSOLE_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPlainTextEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTextBrowser>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "Dialog.hpp"

QT_BEGIN_NAMESPACE

class Ui_scriptConsoleForm
{
public:
    QVBoxLayout *verticalLayout_2;
    TitleBar *titleBar;
    QFrame *frame;
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout;
    QToolButton *loadButton;
    QToolButton *saveButton;
    QToolButton *clearButton;
    QPushButton *preprocessSSCButton;
    QSpacerItem *horizontalSpacer;
    QLabel *quickrunLabel;
    QComboBox *quickrunCombo;
    QSpacerItem *horizontalSpacer_2;
    QToolButton *runButton;
    QToolButton *stopButton;
    QTabWidget *tabs;
    QWidget *scriptTab;
    QVBoxLayout *verticalLayout_3;
    QPlainTextEdit *scriptEdit;
    QWidget *logTab;
    QVBoxLayout *verticalLayout_5;
    QTextBrowser *logBrowser;
    QWidget *outputTab;
    QVBoxLayout *verticalLayout_4;
    QTextBrowser *outputBrowser;
    QWidget *settingsTab;
    QVBoxLayout *verticalLayout_6;
    QCheckBox *useUserDirCheckBox;
    QCheckBox *closeWindowAtScriptRunCheckbox;
    QCheckBox *clearOutputCheckbox;
    QCheckBox *allowScreenshotDirCheckBox;
    QCheckBox *allowStoreAbsoluteCheckBox;
    QSpacerItem *verticalSpacer;
    QHBoxLayout *horizontalLayout_2;
    QLabel *rowColumnLabel;
    QLabel *label;
    QLineEdit *includeEdit;
    QToolButton *includeBrowseButton;
    ResizeFrame *frame_4;

    void setupUi(QWidget *scriptConsoleForm)
    {
        if (scriptConsoleForm->objectName().isEmpty())
            scriptConsoleForm->setObjectName("scriptConsoleForm");
        scriptConsoleForm->resize(733, 432);
        verticalLayout_2 = new QVBoxLayout(scriptConsoleForm);
        verticalLayout_2->setSpacing(0);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        titleBar = new TitleBar(scriptConsoleForm);
        titleBar->setObjectName("titleBar");

        verticalLayout_2->addWidget(titleBar);

        frame = new QFrame(scriptConsoleForm);
        frame->setObjectName("frame");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(1);
        sizePolicy.setHeightForWidth(frame->sizePolicy().hasHeightForWidth());
        frame->setSizePolicy(sizePolicy);
        frame->setFrameShape(QFrame::StyledPanel);
        verticalLayout = new QVBoxLayout(frame);
        verticalLayout->setSpacing(0);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(3);
        horizontalLayout->setObjectName("horizontalLayout");
        loadButton = new QToolButton(frame);
        loadButton->setObjectName("loadButton");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(loadButton->sizePolicy().hasHeightForWidth());
        loadButton->setSizePolicy(sizePolicy1);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/graphicGui/uibtFolder.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        loadButton->setIcon(icon);
#if QT_CONFIG(shortcut)
        loadButton->setShortcut(QString::fromUtf8("Ctrl+Shift+O"));
#endif // QT_CONFIG(shortcut)

        horizontalLayout->addWidget(loadButton);

        saveButton = new QToolButton(frame);
        saveButton->setObjectName("saveButton");
        sizePolicy1.setHeightForWidth(saveButton->sizePolicy().hasHeightForWidth());
        saveButton->setSizePolicy(sizePolicy1);
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/graphicGui/uibtSave.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        saveButton->setIcon(icon1);
#if QT_CONFIG(shortcut)
        saveButton->setShortcut(QString::fromUtf8("Ctrl+Shift+S"));
#endif // QT_CONFIG(shortcut)

        horizontalLayout->addWidget(saveButton);

        clearButton = new QToolButton(frame);
        clearButton->setObjectName("clearButton");
        sizePolicy1.setHeightForWidth(clearButton->sizePolicy().hasHeightForWidth());
        clearButton->setSizePolicy(sizePolicy1);
        QIcon icon2;
        icon2.addFile(QString::fromUtf8(":/graphicGui/uibtDelete.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        clearButton->setIcon(icon2);

        horizontalLayout->addWidget(clearButton);

        preprocessSSCButton = new QPushButton(frame);
        preprocessSSCButton->setObjectName("preprocessSSCButton");
        preprocessSSCButton->setText(QString::fromUtf8("SSC"));
        QIcon icon3;
        icon3.addFile(QString::fromUtf8(":/graphicGui/uibtProcess.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        preprocessSSCButton->setIcon(icon3);

        horizontalLayout->addWidget(preprocessSSCButton);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        quickrunLabel = new QLabel(frame);
        quickrunLabel->setObjectName("quickrunLabel");

        horizontalLayout->addWidget(quickrunLabel);

        quickrunCombo = new QComboBox(frame);
        quickrunCombo->setObjectName("quickrunCombo");

        horizontalLayout->addWidget(quickrunCombo);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_2);

        runButton = new QToolButton(frame);
        runButton->setObjectName("runButton");
        sizePolicy1.setHeightForWidth(runButton->sizePolicy().hasHeightForWidth());
        runButton->setSizePolicy(sizePolicy1);
        QIcon icon4;
        icon4.addFile(QString::fromUtf8(":/graphicGui/uibtStart.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        icon4.addFile(QString::fromUtf8(":/graphicGui/uibtStart-disabled.png"), QSize(), QIcon::Mode::Disabled, QIcon::State::Off);
        runButton->setIcon(icon4);
#if QT_CONFIG(shortcut)
        runButton->setShortcut(QString::fromUtf8("Ctrl+Return"));
#endif // QT_CONFIG(shortcut)

        horizontalLayout->addWidget(runButton);

        stopButton = new QToolButton(frame);
        stopButton->setObjectName("stopButton");
        stopButton->setEnabled(false);
        sizePolicy1.setHeightForWidth(stopButton->sizePolicy().hasHeightForWidth());
        stopButton->setSizePolicy(sizePolicy1);
        QIcon icon5;
        icon5.addFile(QString::fromUtf8(":/graphicGui/uibtStop.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        icon5.addFile(QString::fromUtf8(":/graphicGui/uibtStop-disabled.png"), QSize(), QIcon::Mode::Disabled, QIcon::State::Off);
        stopButton->setIcon(icon5);

        horizontalLayout->addWidget(stopButton);


        verticalLayout->addLayout(horizontalLayout);

        tabs = new QTabWidget(frame);
        tabs->setObjectName("tabs");
        tabs->setTabPosition(QTabWidget::North);
        scriptTab = new QWidget();
        scriptTab->setObjectName("scriptTab");
        verticalLayout_3 = new QVBoxLayout(scriptTab);
        verticalLayout_3->setObjectName("verticalLayout_3");
        verticalLayout_3->setContentsMargins(0, 0, 0, 0);
        scriptEdit = new QPlainTextEdit(scriptTab);
        scriptEdit->setObjectName("scriptEdit");
        scriptEdit->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOn);
        scriptEdit->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOn);
        scriptEdit->setLineWrapMode(QPlainTextEdit::NoWrap);

        verticalLayout_3->addWidget(scriptEdit);

        tabs->addTab(scriptTab, QString());
        logTab = new QWidget();
        logTab->setObjectName("logTab");
        verticalLayout_5 = new QVBoxLayout(logTab);
        verticalLayout_5->setObjectName("verticalLayout_5");
        verticalLayout_5->setContentsMargins(0, 0, 0, 0);
        logBrowser = new QTextBrowser(logTab);
        logBrowser->setObjectName("logBrowser");

        verticalLayout_5->addWidget(logBrowser);

        tabs->addTab(logTab, QString());
        outputTab = new QWidget();
        outputTab->setObjectName("outputTab");
        verticalLayout_4 = new QVBoxLayout(outputTab);
        verticalLayout_4->setObjectName("verticalLayout_4");
        verticalLayout_4->setContentsMargins(0, 0, 0, 0);
        outputBrowser = new QTextBrowser(outputTab);
        outputBrowser->setObjectName("outputBrowser");

        verticalLayout_4->addWidget(outputBrowser);

        tabs->addTab(outputTab, QString());
        settingsTab = new QWidget();
        settingsTab->setObjectName("settingsTab");
        verticalLayout_6 = new QVBoxLayout(settingsTab);
        verticalLayout_6->setObjectName("verticalLayout_6");
        useUserDirCheckBox = new QCheckBox(settingsTab);
        useUserDirCheckBox->setObjectName("useUserDirCheckBox");

        verticalLayout_6->addWidget(useUserDirCheckBox);

        closeWindowAtScriptRunCheckbox = new QCheckBox(settingsTab);
        closeWindowAtScriptRunCheckbox->setObjectName("closeWindowAtScriptRunCheckbox");

        verticalLayout_6->addWidget(closeWindowAtScriptRunCheckbox);

        clearOutputCheckbox = new QCheckBox(settingsTab);
        clearOutputCheckbox->setObjectName("clearOutputCheckbox");

        verticalLayout_6->addWidget(clearOutputCheckbox);

        allowScreenshotDirCheckBox = new QCheckBox(settingsTab);
        allowScreenshotDirCheckBox->setObjectName("allowScreenshotDirCheckBox");

        verticalLayout_6->addWidget(allowScreenshotDirCheckBox);

        allowStoreAbsoluteCheckBox = new QCheckBox(settingsTab);
        allowStoreAbsoluteCheckBox->setObjectName("allowStoreAbsoluteCheckBox");

        verticalLayout_6->addWidget(allowStoreAbsoluteCheckBox);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_6->addItem(verticalSpacer);

        tabs->addTab(settingsTab, QString());

        verticalLayout->addWidget(tabs);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setSpacing(2);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        horizontalLayout_2->setContentsMargins(4, -1, -1, -1);
        rowColumnLabel = new QLabel(frame);
        rowColumnLabel->setObjectName("rowColumnLabel");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Preferred);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(rowColumnLabel->sizePolicy().hasHeightForWidth());
        rowColumnLabel->setSizePolicy(sizePolicy2);

        horizontalLayout_2->addWidget(rowColumnLabel);

        label = new QLabel(frame);
        label->setObjectName("label");

        horizontalLayout_2->addWidget(label);

        includeEdit = new QLineEdit(frame);
        includeEdit->setObjectName("includeEdit");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Fixed);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(includeEdit->sizePolicy().hasHeightForWidth());
        includeEdit->setSizePolicy(sizePolicy3);

        horizontalLayout_2->addWidget(includeEdit);

        includeBrowseButton = new QToolButton(frame);
        includeBrowseButton->setObjectName("includeBrowseButton");
        sizePolicy1.setHeightForWidth(includeBrowseButton->sizePolicy().hasHeightForWidth());
        includeBrowseButton->setSizePolicy(sizePolicy1);

        horizontalLayout_2->addWidget(includeBrowseButton);

        frame_4 = new ResizeFrame(frame);
        frame_4->setObjectName("frame_4");
        sizePolicy1.setHeightForWidth(frame_4->sizePolicy().hasHeightForWidth());
        frame_4->setSizePolicy(sizePolicy1);
        frame_4->setMinimumSize(QSize(20, 20));
        frame_4->setMaximumSize(QSize(24, 24));
        QFont font;
        font.setFamilies({QString::fromUtf8("Sans Serif")});
        frame_4->setFont(font);
        frame_4->setCursor(QCursor(Qt::CursorShape::SizeFDiagCursor));
        frame_4->setFrameShape(QFrame::StyledPanel);

        horizontalLayout_2->addWidget(frame_4);


        verticalLayout->addLayout(horizontalLayout_2);


        verticalLayout_2->addWidget(frame);

        frame->raise();
        titleBar->raise();

        retranslateUi(scriptConsoleForm);

        tabs->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(scriptConsoleForm);
    } // setupUi

    void retranslateUi(QWidget *scriptConsoleForm)
    {
        titleBar->setProperty("title", QVariant(QCoreApplication::translate("scriptConsoleForm", "Script console", nullptr)));
#if QT_CONFIG(tooltip)
        loadButton->setToolTip(QCoreApplication::translate("scriptConsoleForm", "Load script from file", nullptr));
#endif // QT_CONFIG(tooltip)
        loadButton->setText(QString());
#if QT_CONFIG(tooltip)
        saveButton->setToolTip(QCoreApplication::translate("scriptConsoleForm", "Save script to file", nullptr));
#endif // QT_CONFIG(tooltip)
        saveButton->setText(QString());
#if QT_CONFIG(tooltip)
        clearButton->setToolTip(QCoreApplication::translate("scriptConsoleForm", "Clear", nullptr));
#endif // QT_CONFIG(tooltip)
        clearButton->setText(QString());
#if QT_CONFIG(tooltip)
        preprocessSSCButton->setToolTip(QCoreApplication::translate("scriptConsoleForm", "Pre-process script using SSC preprocessor", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        quickrunLabel->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
        quickrunLabel->setText(QCoreApplication::translate("scriptConsoleForm", "Execute:", nullptr));
#if QT_CONFIG(tooltip)
        quickrunCombo->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        runButton->setToolTip(QCoreApplication::translate("scriptConsoleForm", "Run script", nullptr));
#endif // QT_CONFIG(tooltip)
        runButton->setText(QString());
#if QT_CONFIG(tooltip)
        stopButton->setToolTip(QCoreApplication::translate("scriptConsoleForm", "Stop script", nullptr));
#endif // QT_CONFIG(tooltip)
        stopButton->setText(QString());
        tabs->setTabText(tabs->indexOf(scriptTab), QCoreApplication::translate("scriptConsoleForm", "Script", nullptr));
        tabs->setTabText(tabs->indexOf(logTab), QCoreApplication::translate("scriptConsoleForm", "Log", nullptr));
        tabs->setTabText(tabs->indexOf(outputTab), QCoreApplication::translate("scriptConsoleForm", "Output", nullptr));
        useUserDirCheckBox->setText(QCoreApplication::translate("scriptConsoleForm", "Load scripts from the User Directory by default", nullptr));
        closeWindowAtScriptRunCheckbox->setText(QCoreApplication::translate("scriptConsoleForm", "Close window when script runs", nullptr));
        clearOutputCheckbox->setText(QCoreApplication::translate("scriptConsoleForm", "Clear output before script runs", nullptr));
#if QT_CONFIG(tooltip)
        allowScreenshotDirCheckBox->setToolTip(QCoreApplication::translate("scriptConsoleForm", "If disabled, all screenshots go to the directory set in the Configuration dialog.", nullptr));
#endif // QT_CONFIG(tooltip)
        allowScreenshotDirCheckBox->setText(QCoreApplication::translate("scriptConsoleForm", "Allow scripts to store screenshots to other directories", nullptr));
#if QT_CONFIG(tooltip)
        allowStoreAbsoluteCheckBox->setToolTip(QCoreApplication::translate("scriptConsoleForm", "If disabled, all output goes to the user data directory. Enabling this bears some risks!", nullptr));
#endif // QT_CONFIG(tooltip)
        allowStoreAbsoluteCheckBox->setText(QCoreApplication::translate("scriptConsoleForm", "Allow scripts to store output to arbitrary directories", nullptr));
        tabs->setTabText(tabs->indexOf(settingsTab), QCoreApplication::translate("scriptConsoleForm", "Settings", nullptr));
#if QT_CONFIG(tooltip)
        rowColumnLabel->setToolTip(QCoreApplication::translate("scriptConsoleForm", "Cursor position", nullptr));
#endif // QT_CONFIG(tooltip)
        rowColumnLabel->setText(QCoreApplication::translate("scriptConsoleForm", "R:0 C:0", nullptr));
        label->setText(QCoreApplication::translate("scriptConsoleForm", "Include dir:", nullptr));
        includeBrowseButton->setText(QCoreApplication::translate("scriptConsoleForm", "...", nullptr));
        (void)scriptConsoleForm;
    } // retranslateUi

};

namespace Ui {
    class scriptConsoleForm: public Ui_scriptConsoleForm {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SCRIPTCONSOLE_H
