/********************************************************************************
** Form generated from reading UI file 'angleMeasureDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_ANGLEMEASUREDIALOG_H
#define UI_ANGLEMEASUREDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTextBrowser>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "Dialog.hpp"
#include "gui/ColorButton.hpp"

QT_BEGIN_NAMESPACE

class Ui_angleMeasureDialog
{
public:
    QVBoxLayout *verticalLayout_2;
    TitleBar *titleBar;
    QTabWidget *tabs;
    QWidget *settingsTab;
    QVBoxLayout *verticalLayout_3;
    QGroupBox *showEquatorial_GroupBox;
    QGridLayout *gridLayout;
    QHBoxLayout *equatorialHorizontalLayout;
    QCheckBox *showPositionAngleCheckBox;
    QSpacerItem *equatorialLineHorizontalSpacer;
    ColorButton *equatorialTextColorToolButton;
    QLabel *equatorialTextColorLabel;
    ColorButton *equatorialLineColorToolButton;
    QLabel *equatorialLineColorLabel;
    QGroupBox *showHorizontal_GroupBox;
    QGridLayout *gridLayout_2;
    QHBoxLayout *horizontalLayout;
    QCheckBox *showPositionAngleHorizontalCheckBox;
    QSpacerItem *horizontalLineHorizontalSpacer;
    ColorButton *horizontalTextColorToolButton;
    QLabel *horizontalTextColorLabel;
    ColorButton *horizontalLineColorToolButton;
    QLabel *horizontalLineColorLabel;
    QCheckBox *azAltStartOnSkyCheckBox;
    QCheckBox *azAltEndOnSkyCheckBox;
    QGroupBox *optionsBox;
    QGridLayout *gridLayout_3;
    QCheckBox *followCursorCheckBox;
    QCheckBox *useDmsFormatCheckBox;
    QCheckBox *panAndSelectCheckBox;
    QSpacerItem *verticalSpacer_3;
    QPushButton *restoreDefaultsButton;
    QWidget *aboutTab;
    QVBoxLayout *verticalLayout_11;
    QTextBrowser *aboutTextBrowser;

    void setupUi(QWidget *angleMeasureDialog)
    {
        if (angleMeasureDialog->objectName().isEmpty())
            angleMeasureDialog->setObjectName("angleMeasureDialog");
        angleMeasureDialog->resize(836, 511);
        verticalLayout_2 = new QVBoxLayout(angleMeasureDialog);
        verticalLayout_2->setSpacing(0);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        titleBar = new TitleBar(angleMeasureDialog);
        titleBar->setObjectName("titleBar");

        verticalLayout_2->addWidget(titleBar);

        tabs = new QTabWidget(angleMeasureDialog);
        tabs->setObjectName("tabs");
        tabs->setDocumentMode(false);
        settingsTab = new QWidget();
        settingsTab->setObjectName("settingsTab");
        verticalLayout_3 = new QVBoxLayout(settingsTab);
        verticalLayout_3->setObjectName("verticalLayout_3");
        showEquatorial_GroupBox = new QGroupBox(settingsTab);
        showEquatorial_GroupBox->setObjectName("showEquatorial_GroupBox");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(showEquatorial_GroupBox->sizePolicy().hasHeightForWidth());
        showEquatorial_GroupBox->setSizePolicy(sizePolicy);
        showEquatorial_GroupBox->setFlat(true);
        showEquatorial_GroupBox->setCheckable(true);
        gridLayout = new QGridLayout(showEquatorial_GroupBox);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setContentsMargins(0, 0, 0, 0);
        equatorialHorizontalLayout = new QHBoxLayout();
        equatorialHorizontalLayout->setObjectName("equatorialHorizontalLayout");
        showPositionAngleCheckBox = new QCheckBox(showEquatorial_GroupBox);
        showPositionAngleCheckBox->setObjectName("showPositionAngleCheckBox");
        showPositionAngleCheckBox->setEnabled(true);

        equatorialHorizontalLayout->addWidget(showPositionAngleCheckBox);

        equatorialLineHorizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        equatorialHorizontalLayout->addItem(equatorialLineHorizontalSpacer);

        equatorialTextColorToolButton = new ColorButton(showEquatorial_GroupBox);
        equatorialTextColorToolButton->setObjectName("equatorialTextColorToolButton");

        equatorialHorizontalLayout->addWidget(equatorialTextColorToolButton);

        equatorialTextColorLabel = new QLabel(showEquatorial_GroupBox);
        equatorialTextColorLabel->setObjectName("equatorialTextColorLabel");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(equatorialTextColorLabel->sizePolicy().hasHeightForWidth());
        equatorialTextColorLabel->setSizePolicy(sizePolicy1);

        equatorialHorizontalLayout->addWidget(equatorialTextColorLabel);

        equatorialLineColorToolButton = new ColorButton(showEquatorial_GroupBox);
        equatorialLineColorToolButton->setObjectName("equatorialLineColorToolButton");

        equatorialHorizontalLayout->addWidget(equatorialLineColorToolButton);

        equatorialLineColorLabel = new QLabel(showEquatorial_GroupBox);
        equatorialLineColorLabel->setObjectName("equatorialLineColorLabel");
        sizePolicy1.setHeightForWidth(equatorialLineColorLabel->sizePolicy().hasHeightForWidth());
        equatorialLineColorLabel->setSizePolicy(sizePolicy1);

        equatorialHorizontalLayout->addWidget(equatorialLineColorLabel);


        gridLayout->addLayout(equatorialHorizontalLayout, 1, 0, 1, 1);


        verticalLayout_3->addWidget(showEquatorial_GroupBox);

        showHorizontal_GroupBox = new QGroupBox(settingsTab);
        showHorizontal_GroupBox->setObjectName("showHorizontal_GroupBox");
        showHorizontal_GroupBox->setFlat(true);
        showHorizontal_GroupBox->setCheckable(true);
        gridLayout_2 = new QGridLayout(showHorizontal_GroupBox);
        gridLayout_2->setObjectName("gridLayout_2");
        gridLayout_2->setContentsMargins(0, 0, 0, 0);
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        showPositionAngleHorizontalCheckBox = new QCheckBox(showHorizontal_GroupBox);
        showPositionAngleHorizontalCheckBox->setObjectName("showPositionAngleHorizontalCheckBox");

        horizontalLayout->addWidget(showPositionAngleHorizontalCheckBox);

        horizontalLineHorizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalLineHorizontalSpacer);

        horizontalTextColorToolButton = new ColorButton(showHorizontal_GroupBox);
        horizontalTextColorToolButton->setObjectName("horizontalTextColorToolButton");

        horizontalLayout->addWidget(horizontalTextColorToolButton);

        horizontalTextColorLabel = new QLabel(showHorizontal_GroupBox);
        horizontalTextColorLabel->setObjectName("horizontalTextColorLabel");
        sizePolicy1.setHeightForWidth(horizontalTextColorLabel->sizePolicy().hasHeightForWidth());
        horizontalTextColorLabel->setSizePolicy(sizePolicy1);

        horizontalLayout->addWidget(horizontalTextColorLabel);

        horizontalLineColorToolButton = new ColorButton(showHorizontal_GroupBox);
        horizontalLineColorToolButton->setObjectName("horizontalLineColorToolButton");

        horizontalLayout->addWidget(horizontalLineColorToolButton);

        horizontalLineColorLabel = new QLabel(showHorizontal_GroupBox);
        horizontalLineColorLabel->setObjectName("horizontalLineColorLabel");
        sizePolicy1.setHeightForWidth(horizontalLineColorLabel->sizePolicy().hasHeightForWidth());
        horizontalLineColorLabel->setSizePolicy(sizePolicy1);

        horizontalLayout->addWidget(horizontalLineColorLabel);


        gridLayout_2->addLayout(horizontalLayout, 3, 0, 1, 1);

        azAltStartOnSkyCheckBox = new QCheckBox(showHorizontal_GroupBox);
        azAltStartOnSkyCheckBox->setObjectName("azAltStartOnSkyCheckBox");

        gridLayout_2->addWidget(azAltStartOnSkyCheckBox, 5, 0, 1, 1);

        azAltEndOnSkyCheckBox = new QCheckBox(showHorizontal_GroupBox);
        azAltEndOnSkyCheckBox->setObjectName("azAltEndOnSkyCheckBox");

        gridLayout_2->addWidget(azAltEndOnSkyCheckBox, 6, 0, 1, 1);


        verticalLayout_3->addWidget(showHorizontal_GroupBox);

        optionsBox = new QGroupBox(settingsTab);
        optionsBox->setObjectName("optionsBox");
        optionsBox->setFlat(true);
        gridLayout_3 = new QGridLayout(optionsBox);
        gridLayout_3->setObjectName("gridLayout_3");
        gridLayout_3->setContentsMargins(0, 0, 0, 0);
        followCursorCheckBox = new QCheckBox(optionsBox);
        followCursorCheckBox->setObjectName("followCursorCheckBox");

        gridLayout_3->addWidget(followCursorCheckBox, 1, 0, 1, 1);

        useDmsFormatCheckBox = new QCheckBox(optionsBox);
        useDmsFormatCheckBox->setObjectName("useDmsFormatCheckBox");

        gridLayout_3->addWidget(useDmsFormatCheckBox, 0, 0, 1, 1);

        panAndSelectCheckBox = new QCheckBox(optionsBox);
        panAndSelectCheckBox->setObjectName("panAndSelectCheckBox");

        gridLayout_3->addWidget(panAndSelectCheckBox, 2, 0, 1, 1);


        verticalLayout_3->addWidget(optionsBox);

        verticalSpacer_3 = new QSpacerItem(20, 120, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_3->addItem(verticalSpacer_3);

        restoreDefaultsButton = new QPushButton(settingsTab);
        restoreDefaultsButton->setObjectName("restoreDefaultsButton");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(restoreDefaultsButton->sizePolicy().hasHeightForWidth());
        restoreDefaultsButton->setSizePolicy(sizePolicy2);

        verticalLayout_3->addWidget(restoreDefaultsButton);

        tabs->addTab(settingsTab, QString());
        aboutTab = new QWidget();
        aboutTab->setObjectName("aboutTab");
        verticalLayout_11 = new QVBoxLayout(aboutTab);
        verticalLayout_11->setSpacing(9);
        verticalLayout_11->setObjectName("verticalLayout_11");
        aboutTextBrowser = new QTextBrowser(aboutTab);
        aboutTextBrowser->setObjectName("aboutTextBrowser");
        aboutTextBrowser->setOpenExternalLinks(true);
        aboutTextBrowser->setOpenLinks(true);

        verticalLayout_11->addWidget(aboutTextBrowser);

        tabs->addTab(aboutTab, QString());

        verticalLayout_2->addWidget(tabs);

        QWidget::setTabOrder(tabs, showEquatorial_GroupBox);
        QWidget::setTabOrder(showEquatorial_GroupBox, showPositionAngleCheckBox);
        QWidget::setTabOrder(showPositionAngleCheckBox, equatorialTextColorToolButton);
        QWidget::setTabOrder(equatorialTextColorToolButton, equatorialLineColorToolButton);
        QWidget::setTabOrder(equatorialLineColorToolButton, showHorizontal_GroupBox);
        QWidget::setTabOrder(showHorizontal_GroupBox, showPositionAngleHorizontalCheckBox);
        QWidget::setTabOrder(showPositionAngleHorizontalCheckBox, horizontalTextColorToolButton);
        QWidget::setTabOrder(horizontalTextColorToolButton, horizontalLineColorToolButton);
        QWidget::setTabOrder(horizontalLineColorToolButton, azAltStartOnSkyCheckBox);
        QWidget::setTabOrder(azAltStartOnSkyCheckBox, azAltEndOnSkyCheckBox);
        QWidget::setTabOrder(azAltEndOnSkyCheckBox, useDmsFormatCheckBox);
        QWidget::setTabOrder(useDmsFormatCheckBox, followCursorCheckBox);
        QWidget::setTabOrder(followCursorCheckBox, restoreDefaultsButton);
        QWidget::setTabOrder(restoreDefaultsButton, aboutTextBrowser);

        retranslateUi(angleMeasureDialog);

        tabs->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(angleMeasureDialog);
    } // setupUi

    void retranslateUi(QWidget *angleMeasureDialog)
    {
        angleMeasureDialog->setWindowTitle(QCoreApplication::translate("angleMeasureDialog", "Angle Measure Plug-in Configuration", nullptr));
        titleBar->setTitle(QCoreApplication::translate("angleMeasureDialog", "Angle Measure Plug-in Configuration", nullptr));
        showEquatorial_GroupBox->setTitle(QCoreApplication::translate("angleMeasureDialog", "Show in Equatorial Coordinates", nullptr));
        showPositionAngleCheckBox->setText(QCoreApplication::translate("angleMeasureDialog", "Display with position angle", nullptr));
        equatorialTextColorToolButton->setText(QString());
        equatorialTextColorLabel->setText(QCoreApplication::translate("angleMeasureDialog", "Text", nullptr));
        equatorialLineColorToolButton->setText(QString());
        equatorialLineColorLabel->setText(QCoreApplication::translate("angleMeasureDialog", "Line", nullptr));
        showHorizontal_GroupBox->setTitle(QCoreApplication::translate("angleMeasureDialog", "Show in Horizontal Coordinates", nullptr));
#if QT_CONFIG(tooltip)
        showPositionAngleHorizontalCheckBox->setToolTip(QCoreApplication::translate("angleMeasureDialog", "(e.g. for Dobson starhopping)", nullptr));
#endif // QT_CONFIG(tooltip)
        showPositionAngleHorizontalCheckBox->setText(QCoreApplication::translate("angleMeasureDialog", "Display with position angle ", nullptr));
        horizontalTextColorToolButton->setText(QString());
        horizontalTextColorLabel->setText(QCoreApplication::translate("angleMeasureDialog", "Text", nullptr));
        horizontalLineColorToolButton->setText(QString());
        horizontalLineColorLabel->setText(QCoreApplication::translate("angleMeasureDialog", "Line", nullptr));
        azAltStartOnSkyCheckBox->setText(QCoreApplication::translate("angleMeasureDialog", "Attach start point to rotating sky", nullptr));
        azAltEndOnSkyCheckBox->setText(QCoreApplication::translate("angleMeasureDialog", "Attach end point to rotating sky", nullptr));
        optionsBox->setTitle(QCoreApplication::translate("angleMeasureDialog", "Options", nullptr));
#if QT_CONFIG(tooltip)
        followCursorCheckBox->setToolTip(QCoreApplication::translate("angleMeasureDialog", "When set, measurements are shown near the end of the line instead of its starting point.", nullptr));
#endif // QT_CONFIG(tooltip)
        followCursorCheckBox->setText(QCoreApplication::translate("angleMeasureDialog", "Show measurements near mouse", nullptr));
        useDmsFormatCheckBox->setText(QCoreApplication::translate("angleMeasureDialog", "Use dms format for angles", nullptr));
#if QT_CONFIG(tooltip)
        panAndSelectCheckBox->setToolTip(QCoreApplication::translate("angleMeasureDialog", "When set, only the right mouse button is used to measure angles, the left mouse button pans and selects as usual", nullptr));
#endif // QT_CONFIG(tooltip)
        panAndSelectCheckBox->setText(QCoreApplication::translate("angleMeasureDialog", "Allow snap to selected object", nullptr));
        restoreDefaultsButton->setText(QCoreApplication::translate("angleMeasureDialog", "Restore defaults", nullptr));
        tabs->setTabText(tabs->indexOf(settingsTab), QCoreApplication::translate("angleMeasureDialog", "Angle Measure", nullptr));
        tabs->setTabText(tabs->indexOf(aboutTab), QCoreApplication::translate("angleMeasureDialog", "About", "tab in plugin windows"));
    } // retranslateUi

};

namespace Ui {
    class angleMeasureDialog: public Ui_angleMeasureDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_ANGLEMEASUREDIALOG_H
