/********************************************************************************
** Form generated from reading UI file 'archaeoLinesDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_ARCHAEOLINESDIALOG_H
#define UI_ARCHAEOLINESDIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTextBrowser>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "AngleSpinBox.hpp"
#include "Dialog.hpp"
#include "gui/ColorButton.hpp"

QT_BEGIN_NAMESPACE

class Ui_archaeoLinesDialog
{
public:
    QVBoxLayout *verticalLayout_2;
    TitleBar *titleBar;
    QTabWidget *tabs;
    QWidget *settingsTab;
    QVBoxLayout *verticalLayout_3;
    QGridLayout *gridLayout;
    QCheckBox *crossquarterCheckBox;
    QComboBox *currentPlanetComboBox;
    QSpinBox *lineWidthSpinBox;
    ColorButton *selectedObjectAzimuthColorToolButton;
    QCheckBox *currentSunCheckBox;
    QCheckBox *currentMoonCheckBox;
    QCheckBox *majorStandstillCheckBox;
    QCheckBox *minorStandstillCheckBox;
    ColorButton *currentMoonColorToolButton;
    QCheckBox *equinoxCheckBox;
    ColorButton *selectedObjectColorToolButton;
    QCheckBox *polarCirclesCheckBox;
    ColorButton *equinoxColorToolButton;
    QCheckBox *selectedObjectHourAngleCheckBox;
    ColorButton *currentSunColorToolButton;
    QCheckBox *selectedObjectCheckBox;
    ColorButton *crossquarterColorToolButton;
    ColorButton *minorStandstillColorToolButton;
    ColorButton *currentPlanetColorToolButton;
    ColorButton *selectedObjectHourAngleColorToolButton;
    QLabel *lineWidthLabel;
    QCheckBox *zenithPassageCheckBox;
    ColorButton *solsticesColorToolButton;
    QCheckBox *solsticesCheckBox;
    ColorButton *majorStandstillColorToolButton;
    QCheckBox *selectedObjectAzimuthCheckBox;
    QCheckBox *nadirPassageCheckBox;
    ColorButton *nadirPassageColorToolButton;
    ColorButton *zenithPassageColorToolButton;
    ColorButton *polarCirclesColorToolButton;
    QSpacerItem *verticalSpacer_3;
    QHBoxLayout *horizontalLayout_7;
    QPushButton *restoreDefaultsButton;
    QWidget *customLinesTab;
    QVBoxLayout *verticalLayout;
    QGridLayout *gridLayout_2;
    QLabel *geographicLocation1LongitudeLabel;
    QPushButton *customDeclination1PushButton;
    ColorButton *customDeclination2ColorToolButton;
    AngleSpinBox *customAzimuth2DoubleSpinBox;
    QCheckBox *geographicLocation1CheckBox;
    QLineEdit *geographicLocation2LineEdit;
    QLineEdit *customAzimuth2LineEdit;
    QPushButton *customDeclination2PushButton;
    QPushButton *customAzimuth2PushButton;
    QLabel *geographicLocation2LatitudeLabel;
    AngleSpinBox *customDeclination2DoubleSpinBox;
    ColorButton *customDeclination1ColorToolButton;
    AngleSpinBox *geographicLocation2LongitudeDoubleSpinBox;
    QCheckBox *geographicLocation2CheckBox;
    QLineEdit *customAzimuth1LineEdit;
    QLineEdit *customDeclination2LineEdit;
    ColorButton *customAzimuth2ColorToolButton;
    QLineEdit *geographicLocation1LineEdit;
    QCheckBox *customDeclination1CheckBox;
    ColorButton *geographicLocation1ColorToolButton;
    QPushButton *customAzimuth1PushButton;
    AngleSpinBox *geographicLocation1LatitudeDoubleSpinBox;
    QCheckBox *customAzimuth1CheckBox;
    QPushButton *geographicLocation2PickPushButton;
    QPushButton *geographicLocation1PickPushButton;
    ColorButton *customAzimuth1ColorToolButton;
    QLabel *geographicLocation1LatitudeLabel;
    ColorButton *geographicLocation2ColorToolButton;
    AngleSpinBox *geographicLocation1LongitudeDoubleSpinBox;
    QLabel *geographicLocation2LongitudeLabel;
    QCheckBox *customDeclination2CheckBox;
    AngleSpinBox *customAzimuth1DoubleSpinBox;
    QLineEdit *customDeclination1LineEdit;
    AngleSpinBox *geographicLocation2LatitudeDoubleSpinBox;
    QCheckBox *customAzimuth2CheckBox;
    AngleSpinBox *customDeclination1DoubleSpinBox;
    QCheckBox *customAltitude1CheckBox;
    QCheckBox *customAltitude2CheckBox;
    AngleSpinBox *customAltitude1DoubleSpinBox;
    AngleSpinBox *customAltitude2DoubleSpinBox;
    QLineEdit *customAltitude1LineEdit;
    QLineEdit *customAltitude2LineEdit;
    QPushButton *customAltitude1PushButton;
    QPushButton *customAltitude2PushButton;
    ColorButton *customAltitude1ColorToolButton;
    ColorButton *customAltitude2ColorToolButton;
    QSpacerItem *verticalSpacer;
    QPushButton *restoreDefaultsButtonCL;
    QWidget *aboutTab;
    QVBoxLayout *verticalLayout_11;
    QTextBrowser *aboutTextBrowser;

    void setupUi(QWidget *archaeoLinesDialog)
    {
        if (archaeoLinesDialog->objectName().isEmpty())
            archaeoLinesDialog->setObjectName("archaeoLinesDialog");
        archaeoLinesDialog->resize(535, 393);
        verticalLayout_2 = new QVBoxLayout(archaeoLinesDialog);
        verticalLayout_2->setSpacing(0);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        titleBar = new TitleBar(archaeoLinesDialog);
        titleBar->setObjectName("titleBar");

        verticalLayout_2->addWidget(titleBar);

        tabs = new QTabWidget(archaeoLinesDialog);
        tabs->setObjectName("tabs");
        tabs->setDocumentMode(false);
        settingsTab = new QWidget();
        settingsTab->setObjectName("settingsTab");
        verticalLayout_3 = new QVBoxLayout(settingsTab);
        verticalLayout_3->setObjectName("verticalLayout_3");
        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        crossquarterCheckBox = new QCheckBox(settingsTab);
        crossquarterCheckBox->setObjectName("crossquarterCheckBox");

        gridLayout->addWidget(crossquarterCheckBox, 2, 1, 1, 1);

        currentPlanetComboBox = new QComboBox(settingsTab);
        currentPlanetComboBox->addItem(QString());
        currentPlanetComboBox->addItem(QString());
        currentPlanetComboBox->addItem(QString());
        currentPlanetComboBox->addItem(QString());
        currentPlanetComboBox->addItem(QString());
        currentPlanetComboBox->addItem(QString());
        currentPlanetComboBox->setObjectName("currentPlanetComboBox");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(currentPlanetComboBox->sizePolicy().hasHeightForWidth());
        currentPlanetComboBox->setSizePolicy(sizePolicy);
        currentPlanetComboBox->setMinimumSize(QSize(150, 0));

        gridLayout->addWidget(currentPlanetComboBox, 2, 3, 1, 1);

        lineWidthSpinBox = new QSpinBox(settingsTab);
        lineWidthSpinBox->setObjectName("lineWidthSpinBox");
        lineWidthSpinBox->setMinimum(1);
        lineWidthSpinBox->setMaximum(8);
        lineWidthSpinBox->setValue(1);

        gridLayout->addWidget(lineWidthSpinBox, 6, 2, 1, 1);

        selectedObjectAzimuthColorToolButton = new ColorButton(settingsTab);
        selectedObjectAzimuthColorToolButton->setObjectName("selectedObjectAzimuthColorToolButton");
        selectedObjectAzimuthColorToolButton->setText(QString::fromUtf8(""));

        gridLayout->addWidget(selectedObjectAzimuthColorToolButton, 4, 2, 1, 1);

        currentSunCheckBox = new QCheckBox(settingsTab);
        currentSunCheckBox->setObjectName("currentSunCheckBox");

        gridLayout->addWidget(currentSunCheckBox, 0, 3, 1, 1);

        currentMoonCheckBox = new QCheckBox(settingsTab);
        currentMoonCheckBox->setObjectName("currentMoonCheckBox");

        gridLayout->addWidget(currentMoonCheckBox, 1, 3, 1, 1);

        majorStandstillCheckBox = new QCheckBox(settingsTab);
        majorStandstillCheckBox->setObjectName("majorStandstillCheckBox");

        gridLayout->addWidget(majorStandstillCheckBox, 3, 1, 1, 1);

        minorStandstillCheckBox = new QCheckBox(settingsTab);
        minorStandstillCheckBox->setObjectName("minorStandstillCheckBox");

        gridLayout->addWidget(minorStandstillCheckBox, 4, 1, 1, 1);

        currentMoonColorToolButton = new ColorButton(settingsTab);
        currentMoonColorToolButton->setObjectName("currentMoonColorToolButton");
        currentMoonColorToolButton->setText(QString::fromUtf8(""));

        gridLayout->addWidget(currentMoonColorToolButton, 1, 2, 1, 1);

        equinoxCheckBox = new QCheckBox(settingsTab);
        equinoxCheckBox->setObjectName("equinoxCheckBox");

        gridLayout->addWidget(equinoxCheckBox, 0, 1, 1, 1);

        selectedObjectColorToolButton = new ColorButton(settingsTab);
        selectedObjectColorToolButton->setObjectName("selectedObjectColorToolButton");
        selectedObjectColorToolButton->setText(QString::fromUtf8(""));

        gridLayout->addWidget(selectedObjectColorToolButton, 3, 2, 1, 1);

        polarCirclesCheckBox = new QCheckBox(settingsTab);
        polarCirclesCheckBox->setObjectName("polarCirclesCheckBox");

        gridLayout->addWidget(polarCirclesCheckBox, 5, 1, 1, 1);

        equinoxColorToolButton = new ColorButton(settingsTab);
        equinoxColorToolButton->setObjectName("equinoxColorToolButton");
        equinoxColorToolButton->setText(QString::fromUtf8(""));

        gridLayout->addWidget(equinoxColorToolButton, 0, 0, 1, 1);

        selectedObjectHourAngleCheckBox = new QCheckBox(settingsTab);
        selectedObjectHourAngleCheckBox->setObjectName("selectedObjectHourAngleCheckBox");

        gridLayout->addWidget(selectedObjectHourAngleCheckBox, 5, 3, 1, 1);

        currentSunColorToolButton = new ColorButton(settingsTab);
        currentSunColorToolButton->setObjectName("currentSunColorToolButton");
        currentSunColorToolButton->setText(QString::fromUtf8(""));

        gridLayout->addWidget(currentSunColorToolButton, 0, 2, 1, 1);

        selectedObjectCheckBox = new QCheckBox(settingsTab);
        selectedObjectCheckBox->setObjectName("selectedObjectCheckBox");

        gridLayout->addWidget(selectedObjectCheckBox, 3, 3, 1, 1);

        crossquarterColorToolButton = new ColorButton(settingsTab);
        crossquarterColorToolButton->setObjectName("crossquarterColorToolButton");
        crossquarterColorToolButton->setText(QString::fromUtf8(""));

        gridLayout->addWidget(crossquarterColorToolButton, 2, 0, 1, 1);

        minorStandstillColorToolButton = new ColorButton(settingsTab);
        minorStandstillColorToolButton->setObjectName("minorStandstillColorToolButton");
        minorStandstillColorToolButton->setText(QString::fromUtf8(""));

        gridLayout->addWidget(minorStandstillColorToolButton, 4, 0, 1, 1);

        currentPlanetColorToolButton = new ColorButton(settingsTab);
        currentPlanetColorToolButton->setObjectName("currentPlanetColorToolButton");
        currentPlanetColorToolButton->setText(QString::fromUtf8(""));

        gridLayout->addWidget(currentPlanetColorToolButton, 2, 2, 1, 1);

        selectedObjectHourAngleColorToolButton = new ColorButton(settingsTab);
        selectedObjectHourAngleColorToolButton->setObjectName("selectedObjectHourAngleColorToolButton");
        selectedObjectHourAngleColorToolButton->setText(QString::fromUtf8(""));

        gridLayout->addWidget(selectedObjectHourAngleColorToolButton, 5, 2, 1, 1);

        lineWidthLabel = new QLabel(settingsTab);
        lineWidthLabel->setObjectName("lineWidthLabel");

        gridLayout->addWidget(lineWidthLabel, 6, 3, 1, 1);

        zenithPassageCheckBox = new QCheckBox(settingsTab);
        zenithPassageCheckBox->setObjectName("zenithPassageCheckBox");

        gridLayout->addWidget(zenithPassageCheckBox, 6, 1, 1, 1);

        solsticesColorToolButton = new ColorButton(settingsTab);
        solsticesColorToolButton->setObjectName("solsticesColorToolButton");
        solsticesColorToolButton->setText(QString::fromUtf8(""));

        gridLayout->addWidget(solsticesColorToolButton, 1, 0, 1, 1);

        solsticesCheckBox = new QCheckBox(settingsTab);
        solsticesCheckBox->setObjectName("solsticesCheckBox");

        gridLayout->addWidget(solsticesCheckBox, 1, 1, 1, 1);

        majorStandstillColorToolButton = new ColorButton(settingsTab);
        majorStandstillColorToolButton->setObjectName("majorStandstillColorToolButton");
        majorStandstillColorToolButton->setText(QString::fromUtf8(""));

        gridLayout->addWidget(majorStandstillColorToolButton, 3, 0, 1, 1);

        selectedObjectAzimuthCheckBox = new QCheckBox(settingsTab);
        selectedObjectAzimuthCheckBox->setObjectName("selectedObjectAzimuthCheckBox");

        gridLayout->addWidget(selectedObjectAzimuthCheckBox, 4, 3, 1, 1);

        nadirPassageCheckBox = new QCheckBox(settingsTab);
        nadirPassageCheckBox->setObjectName("nadirPassageCheckBox");

        gridLayout->addWidget(nadirPassageCheckBox, 7, 1, 1, 1);

        nadirPassageColorToolButton = new ColorButton(settingsTab);
        nadirPassageColorToolButton->setObjectName("nadirPassageColorToolButton");
        nadirPassageColorToolButton->setText(QString::fromUtf8(""));

        gridLayout->addWidget(nadirPassageColorToolButton, 7, 0, 1, 1);

        zenithPassageColorToolButton = new ColorButton(settingsTab);
        zenithPassageColorToolButton->setObjectName("zenithPassageColorToolButton");
        zenithPassageColorToolButton->setText(QString::fromUtf8(""));

        gridLayout->addWidget(zenithPassageColorToolButton, 6, 0, 1, 1);

        polarCirclesColorToolButton = new ColorButton(settingsTab);
        polarCirclesColorToolButton->setObjectName("polarCirclesColorToolButton");

        gridLayout->addWidget(polarCirclesColorToolButton, 5, 0, 1, 1);


        verticalLayout_3->addLayout(gridLayout);

        verticalSpacer_3 = new QSpacerItem(20, 120, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_3->addItem(verticalSpacer_3);

        horizontalLayout_7 = new QHBoxLayout();
        horizontalLayout_7->setObjectName("horizontalLayout_7");
        restoreDefaultsButton = new QPushButton(settingsTab);
        restoreDefaultsButton->setObjectName("restoreDefaultsButton");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(restoreDefaultsButton->sizePolicy().hasHeightForWidth());
        restoreDefaultsButton->setSizePolicy(sizePolicy1);

        horizontalLayout_7->addWidget(restoreDefaultsButton);


        verticalLayout_3->addLayout(horizontalLayout_7);

        tabs->addTab(settingsTab, QString());
        customLinesTab = new QWidget();
        customLinesTab->setObjectName("customLinesTab");
        verticalLayout = new QVBoxLayout(customLinesTab);
        verticalLayout->setObjectName("verticalLayout");
        gridLayout_2 = new QGridLayout();
        gridLayout_2->setObjectName("gridLayout_2");
        geographicLocation1LongitudeLabel = new QLabel(customLinesTab);
        geographicLocation1LongitudeLabel->setObjectName("geographicLocation1LongitudeLabel");
        geographicLocation1LongitudeLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_2->addWidget(geographicLocation1LongitudeLabel, 1, 1, 1, 1);

        customDeclination1PushButton = new QPushButton(customLinesTab);
        customDeclination1PushButton->setObjectName("customDeclination1PushButton");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Maximum, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(customDeclination1PushButton->sizePolicy().hasHeightForWidth());
        customDeclination1PushButton->setSizePolicy(sizePolicy2);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/archaeoLines/textures/telescope.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        customDeclination1PushButton->setIcon(icon);

        gridLayout_2->addWidget(customDeclination1PushButton, 8, 5, 1, 1);

        customDeclination2ColorToolButton = new ColorButton(customLinesTab);
        customDeclination2ColorToolButton->setObjectName("customDeclination2ColorToolButton");
        customDeclination2ColorToolButton->setText(QString::fromUtf8(""));

        gridLayout_2->addWidget(customDeclination2ColorToolButton, 9, 0, 1, 1);

        customAzimuth2DoubleSpinBox = new AngleSpinBox(customLinesTab);
        customAzimuth2DoubleSpinBox->setObjectName("customAzimuth2DoubleSpinBox");
        customAzimuth2DoubleSpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_2->addWidget(customAzimuth2DoubleSpinBox, 5, 2, 1, 1);

        geographicLocation1CheckBox = new QCheckBox(customLinesTab);
        geographicLocation1CheckBox->setObjectName("geographicLocation1CheckBox");

        gridLayout_2->addWidget(geographicLocation1CheckBox, 0, 1, 1, 1);

        geographicLocation2LineEdit = new QLineEdit(customLinesTab);
        geographicLocation2LineEdit->setObjectName("geographicLocation2LineEdit");

        gridLayout_2->addWidget(geographicLocation2LineEdit, 2, 2, 1, 3);

        customAzimuth2LineEdit = new QLineEdit(customLinesTab);
        customAzimuth2LineEdit->setObjectName("customAzimuth2LineEdit");

        gridLayout_2->addWidget(customAzimuth2LineEdit, 5, 3, 1, 2);

        customDeclination2PushButton = new QPushButton(customLinesTab);
        customDeclination2PushButton->setObjectName("customDeclination2PushButton");
        sizePolicy2.setHeightForWidth(customDeclination2PushButton->sizePolicy().hasHeightForWidth());
        customDeclination2PushButton->setSizePolicy(sizePolicy2);
        customDeclination2PushButton->setIcon(icon);

        gridLayout_2->addWidget(customDeclination2PushButton, 9, 5, 1, 1);

        customAzimuth2PushButton = new QPushButton(customLinesTab);
        customAzimuth2PushButton->setObjectName("customAzimuth2PushButton");
        sizePolicy2.setHeightForWidth(customAzimuth2PushButton->sizePolicy().hasHeightForWidth());
        customAzimuth2PushButton->setSizePolicy(sizePolicy2);
        customAzimuth2PushButton->setIcon(icon);

        gridLayout_2->addWidget(customAzimuth2PushButton, 5, 5, 1, 1);

        geographicLocation2LatitudeLabel = new QLabel(customLinesTab);
        geographicLocation2LatitudeLabel->setObjectName("geographicLocation2LatitudeLabel");
        geographicLocation2LatitudeLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_2->addWidget(geographicLocation2LatitudeLabel, 3, 3, 1, 1);

        customDeclination2DoubleSpinBox = new AngleSpinBox(customLinesTab);
        customDeclination2DoubleSpinBox->setObjectName("customDeclination2DoubleSpinBox");
        customDeclination2DoubleSpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_2->addWidget(customDeclination2DoubleSpinBox, 9, 2, 1, 1);

        customDeclination1ColorToolButton = new ColorButton(customLinesTab);
        customDeclination1ColorToolButton->setObjectName("customDeclination1ColorToolButton");
        customDeclination1ColorToolButton->setText(QString::fromUtf8(""));

        gridLayout_2->addWidget(customDeclination1ColorToolButton, 8, 0, 1, 1);

        geographicLocation2LongitudeDoubleSpinBox = new AngleSpinBox(customLinesTab);
        geographicLocation2LongitudeDoubleSpinBox->setObjectName("geographicLocation2LongitudeDoubleSpinBox");
        geographicLocation2LongitudeDoubleSpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_2->addWidget(geographicLocation2LongitudeDoubleSpinBox, 3, 2, 1, 1);

        geographicLocation2CheckBox = new QCheckBox(customLinesTab);
        geographicLocation2CheckBox->setObjectName("geographicLocation2CheckBox");

        gridLayout_2->addWidget(geographicLocation2CheckBox, 2, 1, 1, 1);

        customAzimuth1LineEdit = new QLineEdit(customLinesTab);
        customAzimuth1LineEdit->setObjectName("customAzimuth1LineEdit");

        gridLayout_2->addWidget(customAzimuth1LineEdit, 4, 3, 1, 2);

        customDeclination2LineEdit = new QLineEdit(customLinesTab);
        customDeclination2LineEdit->setObjectName("customDeclination2LineEdit");

        gridLayout_2->addWidget(customDeclination2LineEdit, 9, 3, 1, 2);

        customAzimuth2ColorToolButton = new ColorButton(customLinesTab);
        customAzimuth2ColorToolButton->setObjectName("customAzimuth2ColorToolButton");
        customAzimuth2ColorToolButton->setText(QString::fromUtf8(""));

        gridLayout_2->addWidget(customAzimuth2ColorToolButton, 5, 0, 1, 1);

        geographicLocation1LineEdit = new QLineEdit(customLinesTab);
        geographicLocation1LineEdit->setObjectName("geographicLocation1LineEdit");

        gridLayout_2->addWidget(geographicLocation1LineEdit, 0, 2, 1, 3);

        customDeclination1CheckBox = new QCheckBox(customLinesTab);
        customDeclination1CheckBox->setObjectName("customDeclination1CheckBox");

        gridLayout_2->addWidget(customDeclination1CheckBox, 8, 1, 1, 1);

        geographicLocation1ColorToolButton = new ColorButton(customLinesTab);
        geographicLocation1ColorToolButton->setObjectName("geographicLocation1ColorToolButton");
        geographicLocation1ColorToolButton->setText(QString::fromUtf8(""));

        gridLayout_2->addWidget(geographicLocation1ColorToolButton, 0, 0, 1, 1);

        customAzimuth1PushButton = new QPushButton(customLinesTab);
        customAzimuth1PushButton->setObjectName("customAzimuth1PushButton");
        sizePolicy2.setHeightForWidth(customAzimuth1PushButton->sizePolicy().hasHeightForWidth());
        customAzimuth1PushButton->setSizePolicy(sizePolicy2);
        customAzimuth1PushButton->setIcon(icon);

        gridLayout_2->addWidget(customAzimuth1PushButton, 4, 5, 1, 1);

        geographicLocation1LatitudeDoubleSpinBox = new AngleSpinBox(customLinesTab);
        geographicLocation1LatitudeDoubleSpinBox->setObjectName("geographicLocation1LatitudeDoubleSpinBox");
        geographicLocation1LatitudeDoubleSpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_2->addWidget(geographicLocation1LatitudeDoubleSpinBox, 1, 4, 1, 1);

        customAzimuth1CheckBox = new QCheckBox(customLinesTab);
        customAzimuth1CheckBox->setObjectName("customAzimuth1CheckBox");

        gridLayout_2->addWidget(customAzimuth1CheckBox, 4, 1, 1, 1);

        geographicLocation2PickPushButton = new QPushButton(customLinesTab);
        geographicLocation2PickPushButton->setObjectName("geographicLocation2PickPushButton");
        sizePolicy2.setHeightForWidth(geographicLocation2PickPushButton->sizePolicy().hasHeightForWidth());
        geographicLocation2PickPushButton->setSizePolicy(sizePolicy2);

        gridLayout_2->addWidget(geographicLocation2PickPushButton, 3, 5, 1, 1);

        geographicLocation1PickPushButton = new QPushButton(customLinesTab);
        geographicLocation1PickPushButton->setObjectName("geographicLocation1PickPushButton");
        sizePolicy2.setHeightForWidth(geographicLocation1PickPushButton->sizePolicy().hasHeightForWidth());
        geographicLocation1PickPushButton->setSizePolicy(sizePolicy2);

        gridLayout_2->addWidget(geographicLocation1PickPushButton, 1, 5, 1, 1);

        customAzimuth1ColorToolButton = new ColorButton(customLinesTab);
        customAzimuth1ColorToolButton->setObjectName("customAzimuth1ColorToolButton");
        customAzimuth1ColorToolButton->setText(QString::fromUtf8(""));

        gridLayout_2->addWidget(customAzimuth1ColorToolButton, 4, 0, 1, 1);

        geographicLocation1LatitudeLabel = new QLabel(customLinesTab);
        geographicLocation1LatitudeLabel->setObjectName("geographicLocation1LatitudeLabel");
        geographicLocation1LatitudeLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_2->addWidget(geographicLocation1LatitudeLabel, 1, 3, 1, 1);

        geographicLocation2ColorToolButton = new ColorButton(customLinesTab);
        geographicLocation2ColorToolButton->setObjectName("geographicLocation2ColorToolButton");
        geographicLocation2ColorToolButton->setText(QString::fromUtf8(""));

        gridLayout_2->addWidget(geographicLocation2ColorToolButton, 2, 0, 1, 1);

        geographicLocation1LongitudeDoubleSpinBox = new AngleSpinBox(customLinesTab);
        geographicLocation1LongitudeDoubleSpinBox->setObjectName("geographicLocation1LongitudeDoubleSpinBox");
        geographicLocation1LongitudeDoubleSpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_2->addWidget(geographicLocation1LongitudeDoubleSpinBox, 1, 2, 1, 1);

        geographicLocation2LongitudeLabel = new QLabel(customLinesTab);
        geographicLocation2LongitudeLabel->setObjectName("geographicLocation2LongitudeLabel");
        geographicLocation2LongitudeLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_2->addWidget(geographicLocation2LongitudeLabel, 3, 1, 1, 1);

        customDeclination2CheckBox = new QCheckBox(customLinesTab);
        customDeclination2CheckBox->setObjectName("customDeclination2CheckBox");

        gridLayout_2->addWidget(customDeclination2CheckBox, 9, 1, 1, 1);

        customAzimuth1DoubleSpinBox = new AngleSpinBox(customLinesTab);
        customAzimuth1DoubleSpinBox->setObjectName("customAzimuth1DoubleSpinBox");
        customAzimuth1DoubleSpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_2->addWidget(customAzimuth1DoubleSpinBox, 4, 2, 1, 1);

        customDeclination1LineEdit = new QLineEdit(customLinesTab);
        customDeclination1LineEdit->setObjectName("customDeclination1LineEdit");

        gridLayout_2->addWidget(customDeclination1LineEdit, 8, 3, 1, 2);

        geographicLocation2LatitudeDoubleSpinBox = new AngleSpinBox(customLinesTab);
        geographicLocation2LatitudeDoubleSpinBox->setObjectName("geographicLocation2LatitudeDoubleSpinBox");
        geographicLocation2LatitudeDoubleSpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_2->addWidget(geographicLocation2LatitudeDoubleSpinBox, 3, 4, 1, 1);

        customAzimuth2CheckBox = new QCheckBox(customLinesTab);
        customAzimuth2CheckBox->setObjectName("customAzimuth2CheckBox");

        gridLayout_2->addWidget(customAzimuth2CheckBox, 5, 1, 1, 1);

        customDeclination1DoubleSpinBox = new AngleSpinBox(customLinesTab);
        customDeclination1DoubleSpinBox->setObjectName("customDeclination1DoubleSpinBox");
        customDeclination1DoubleSpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_2->addWidget(customDeclination1DoubleSpinBox, 8, 2, 1, 1);

        customAltitude1CheckBox = new QCheckBox(customLinesTab);
        customAltitude1CheckBox->setObjectName("customAltitude1CheckBox");

        gridLayout_2->addWidget(customAltitude1CheckBox, 6, 1, 1, 1);

        customAltitude2CheckBox = new QCheckBox(customLinesTab);
        customAltitude2CheckBox->setObjectName("customAltitude2CheckBox");

        gridLayout_2->addWidget(customAltitude2CheckBox, 7, 1, 1, 1);

        customAltitude1DoubleSpinBox = new AngleSpinBox(customLinesTab);
        customAltitude1DoubleSpinBox->setObjectName("customAltitude1DoubleSpinBox");
        customAltitude1DoubleSpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_2->addWidget(customAltitude1DoubleSpinBox, 6, 2, 1, 1);

        customAltitude2DoubleSpinBox = new AngleSpinBox(customLinesTab);
        customAltitude2DoubleSpinBox->setObjectName("customAltitude2DoubleSpinBox");
        customAltitude2DoubleSpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_2->addWidget(customAltitude2DoubleSpinBox, 7, 2, 1, 1);

        customAltitude1LineEdit = new QLineEdit(customLinesTab);
        customAltitude1LineEdit->setObjectName("customAltitude1LineEdit");

        gridLayout_2->addWidget(customAltitude1LineEdit, 6, 3, 1, 2);

        customAltitude2LineEdit = new QLineEdit(customLinesTab);
        customAltitude2LineEdit->setObjectName("customAltitude2LineEdit");

        gridLayout_2->addWidget(customAltitude2LineEdit, 7, 3, 1, 2);

        customAltitude1PushButton = new QPushButton(customLinesTab);
        customAltitude1PushButton->setObjectName("customAltitude1PushButton");
        sizePolicy2.setHeightForWidth(customAltitude1PushButton->sizePolicy().hasHeightForWidth());
        customAltitude1PushButton->setSizePolicy(sizePolicy2);
        customAltitude1PushButton->setIcon(icon);

        gridLayout_2->addWidget(customAltitude1PushButton, 6, 5, 1, 1);

        customAltitude2PushButton = new QPushButton(customLinesTab);
        customAltitude2PushButton->setObjectName("customAltitude2PushButton");
        sizePolicy2.setHeightForWidth(customAltitude2PushButton->sizePolicy().hasHeightForWidth());
        customAltitude2PushButton->setSizePolicy(sizePolicy2);
        customAltitude2PushButton->setIcon(icon);

        gridLayout_2->addWidget(customAltitude2PushButton, 7, 5, 1, 1);

        customAltitude1ColorToolButton = new ColorButton(customLinesTab);
        customAltitude1ColorToolButton->setObjectName("customAltitude1ColorToolButton");
        customAltitude1ColorToolButton->setText(QString::fromUtf8(""));

        gridLayout_2->addWidget(customAltitude1ColorToolButton, 6, 0, 1, 1);

        customAltitude2ColorToolButton = new ColorButton(customLinesTab);
        customAltitude2ColorToolButton->setObjectName("customAltitude2ColorToolButton");
        customAltitude2ColorToolButton->setText(QString::fromUtf8(""));

        gridLayout_2->addWidget(customAltitude2ColorToolButton, 7, 0, 1, 1);


        verticalLayout->addLayout(gridLayout_2);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);

        restoreDefaultsButtonCL = new QPushButton(customLinesTab);
        restoreDefaultsButtonCL->setObjectName("restoreDefaultsButtonCL");

        verticalLayout->addWidget(restoreDefaultsButtonCL);

        tabs->addTab(customLinesTab, QString());
        aboutTab = new QWidget();
        aboutTab->setObjectName("aboutTab");
        verticalLayout_11 = new QVBoxLayout(aboutTab);
        verticalLayout_11->setSpacing(9);
        verticalLayout_11->setObjectName("verticalLayout_11");
        aboutTextBrowser = new QTextBrowser(aboutTab);
        aboutTextBrowser->setObjectName("aboutTextBrowser");
        aboutTextBrowser->setOpenExternalLinks(true);
        aboutTextBrowser->setOpenLinks(true);

        verticalLayout_11->addWidget(aboutTextBrowser);

        tabs->addTab(aboutTab, QString());

        verticalLayout_2->addWidget(tabs);


        retranslateUi(archaeoLinesDialog);

        tabs->setCurrentIndex(0);
        currentPlanetComboBox->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(archaeoLinesDialog);
    } // setupUi

    void retranslateUi(QWidget *archaeoLinesDialog)
    {
        archaeoLinesDialog->setWindowTitle(QCoreApplication::translate("archaeoLinesDialog", "ArchaeoLines Configuration", nullptr));
        titleBar->setTitle(QCoreApplication::translate("archaeoLinesDialog", "ArchaeoLines Plug-in Configuration", nullptr));
        crossquarterCheckBox->setText(QCoreApplication::translate("archaeoLinesDialog", "Crossquarter Lines", nullptr));
        currentPlanetComboBox->setItemText(0, QCoreApplication::translate("archaeoLinesDialog", "Current Planet: None", nullptr));
        currentPlanetComboBox->setItemText(1, QCoreApplication::translate("archaeoLinesDialog", "Current Planet: Mercury", nullptr));
        currentPlanetComboBox->setItemText(2, QCoreApplication::translate("archaeoLinesDialog", "Current Planet: Venus", nullptr));
        currentPlanetComboBox->setItemText(3, QCoreApplication::translate("archaeoLinesDialog", "Current Planet: Mars", nullptr));
        currentPlanetComboBox->setItemText(4, QCoreApplication::translate("archaeoLinesDialog", "Current Planet: Jupiter", nullptr));
        currentPlanetComboBox->setItemText(5, QCoreApplication::translate("archaeoLinesDialog", "Current Planet: Saturn", nullptr));

        lineWidthSpinBox->setPrefix(QString());
        currentSunCheckBox->setText(QCoreApplication::translate("archaeoLinesDialog", "Current Sun", nullptr));
        currentMoonCheckBox->setText(QCoreApplication::translate("archaeoLinesDialog", "Current Moon", nullptr));
        majorStandstillCheckBox->setText(QCoreApplication::translate("archaeoLinesDialog", "Major Lunar Standstill lines", nullptr));
        minorStandstillCheckBox->setText(QCoreApplication::translate("archaeoLinesDialog", "Minor Lunar Standstill lines", nullptr));
        equinoxCheckBox->setText(QCoreApplication::translate("archaeoLinesDialog", "Equinox line (Equator)", nullptr));
        polarCirclesCheckBox->setText(QCoreApplication::translate("archaeoLinesDialog", "Polar Circles", nullptr));
        selectedObjectHourAngleCheckBox->setText(QCoreApplication::translate("archaeoLinesDialog", "Selected Object Hour Angle", nullptr));
        selectedObjectCheckBox->setText(QCoreApplication::translate("archaeoLinesDialog", "Selected Object", nullptr));
#if QT_CONFIG(tooltip)
        crossquarterColorToolButton->setToolTip(QCoreApplication::translate("archaeoLinesDialog", "Change color", nullptr));
#endif // QT_CONFIG(tooltip)
        lineWidthLabel->setText(QCoreApplication::translate("archaeoLinesDialog", "Line thickness", nullptr));
        zenithPassageCheckBox->setText(QCoreApplication::translate("archaeoLinesDialog", "Declination of Zenith passage", nullptr));
        solsticesCheckBox->setText(QCoreApplication::translate("archaeoLinesDialog", "Solstice Lines", nullptr));
        selectedObjectAzimuthCheckBox->setText(QCoreApplication::translate("archaeoLinesDialog", "Selected Object Azimuth", nullptr));
        nadirPassageCheckBox->setText(QCoreApplication::translate("archaeoLinesDialog", "Declination of Nadir passage", nullptr));
        polarCirclesColorToolButton->setText(QString());
        restoreDefaultsButton->setText(QCoreApplication::translate("archaeoLinesDialog", "Restore defaults", nullptr));
        tabs->setTabText(tabs->indexOf(settingsTab), QCoreApplication::translate("archaeoLinesDialog", "ArchaeoLines", nullptr));
        geographicLocation1LongitudeLabel->setText(QCoreApplication::translate("archaeoLinesDialog", "Longitude", nullptr));
#if QT_CONFIG(tooltip)
        customDeclination1PushButton->setToolTip(QCoreApplication::translate("archaeoLinesDialog", "From selected object (if any)", nullptr));
#endif // QT_CONFIG(tooltip)
        customDeclination1PushButton->setText(QString());
        geographicLocation1CheckBox->setText(QCoreApplication::translate("archaeoLinesDialog", "Geographic Location 1", nullptr));
#if QT_CONFIG(tooltip)
        geographicLocation2LineEdit->setToolTip(QCoreApplication::translate("archaeoLinesDialog", "Enter a label (location name) for this line", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        customAzimuth2LineEdit->setToolTip(QCoreApplication::translate("archaeoLinesDialog", "Enter a label for this line", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        customDeclination2PushButton->setToolTip(QCoreApplication::translate("archaeoLinesDialog", "From selected object (if any)", nullptr));
#endif // QT_CONFIG(tooltip)
        customDeclination2PushButton->setText(QString());
#if QT_CONFIG(tooltip)
        customAzimuth2PushButton->setToolTip(QCoreApplication::translate("archaeoLinesDialog", "From selected object (if any)", nullptr));
#endif // QT_CONFIG(tooltip)
        customAzimuth2PushButton->setText(QString());
        geographicLocation2LatitudeLabel->setText(QCoreApplication::translate("archaeoLinesDialog", "Latitude", nullptr));
        geographicLocation2CheckBox->setText(QCoreApplication::translate("archaeoLinesDialog", "Geographic Location 2", nullptr));
#if QT_CONFIG(tooltip)
        customAzimuth1LineEdit->setToolTip(QCoreApplication::translate("archaeoLinesDialog", "Enter a label for this line", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        geographicLocation1LineEdit->setToolTip(QCoreApplication::translate("archaeoLinesDialog", "Enter a label (location name) for this line", nullptr));
#endif // QT_CONFIG(tooltip)
        customDeclination1CheckBox->setText(QCoreApplication::translate("archaeoLinesDialog", "Custom Declination 1", nullptr));
#if QT_CONFIG(tooltip)
        customAzimuth1PushButton->setToolTip(QCoreApplication::translate("archaeoLinesDialog", "From selected object (if any)", nullptr));
#endif // QT_CONFIG(tooltip)
        customAzimuth1PushButton->setText(QString());
#if QT_CONFIG(tooltip)
        customAzimuth1CheckBox->setToolTip(QCoreApplication::translate("archaeoLinesDialog", "Always counted from North", nullptr));
#endif // QT_CONFIG(tooltip)
        customAzimuth1CheckBox->setText(QCoreApplication::translate("archaeoLinesDialog", "Custom Azimuth 1", nullptr));
#if QT_CONFIG(tooltip)
        geographicLocation2PickPushButton->setToolTip(QCoreApplication::translate("archaeoLinesDialog", "Select from location list", nullptr));
#endif // QT_CONFIG(tooltip)
        geographicLocation2PickPushButton->setText(QCoreApplication::translate("archaeoLinesDialog", "Pick...", nullptr));
#if QT_CONFIG(tooltip)
        geographicLocation1PickPushButton->setToolTip(QCoreApplication::translate("archaeoLinesDialog", "Select from location list", nullptr));
#endif // QT_CONFIG(tooltip)
        geographicLocation1PickPushButton->setText(QCoreApplication::translate("archaeoLinesDialog", "Pick...", nullptr));
        geographicLocation1LatitudeLabel->setText(QCoreApplication::translate("archaeoLinesDialog", "Latitude", nullptr));
        geographicLocation2LongitudeLabel->setText(QCoreApplication::translate("archaeoLinesDialog", "Longitude", nullptr));
        customDeclination2CheckBox->setText(QCoreApplication::translate("archaeoLinesDialog", "Custom Declination 2", nullptr));
#if QT_CONFIG(tooltip)
        customAzimuth2CheckBox->setToolTip(QCoreApplication::translate("archaeoLinesDialog", "Always counted from North", nullptr));
#endif // QT_CONFIG(tooltip)
        customAzimuth2CheckBox->setText(QCoreApplication::translate("archaeoLinesDialog", "Custom Azimuth 2", nullptr));
        customAltitude1CheckBox->setText(QCoreApplication::translate("archaeoLinesDialog", "Custom Altitude 1", nullptr));
        customAltitude2CheckBox->setText(QCoreApplication::translate("archaeoLinesDialog", "Custom Altitude 2", nullptr));
#if QT_CONFIG(tooltip)
        customAltitude1LineEdit->setToolTip(QCoreApplication::translate("archaeoLinesDialog", "Enter a label for this line", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        customAltitude2LineEdit->setToolTip(QCoreApplication::translate("archaeoLinesDialog", "Enter a label for this line", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        customAltitude1PushButton->setToolTip(QCoreApplication::translate("archaeoLinesDialog", "From selected object (if any)", nullptr));
#endif // QT_CONFIG(tooltip)
        customAltitude1PushButton->setText(QString());
#if QT_CONFIG(tooltip)
        customAltitude2PushButton->setToolTip(QCoreApplication::translate("archaeoLinesDialog", "From selected object (if any)", nullptr));
#endif // QT_CONFIG(tooltip)
        customAltitude2PushButton->setText(QString());
        restoreDefaultsButtonCL->setText(QCoreApplication::translate("archaeoLinesDialog", "Restore default settings", nullptr));
        tabs->setTabText(tabs->indexOf(customLinesTab), QCoreApplication::translate("archaeoLinesDialog", "Custom Lines", nullptr));
        tabs->setTabText(tabs->indexOf(aboutTab), QCoreApplication::translate("archaeoLinesDialog", "About", "tab in plugin windows"));
    } // retranslateUi

};

namespace Ui {
    class archaeoLinesDialog: public Ui_archaeoLinesDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_ARCHAEOLINESDIALOG_H
