// This file is autogenerated. Changes will be overwritten.
#include "EWIEGA46WW/moc_INDIConnection.cpp"
#include "EWIEGA46WW/moc_INDIControlWidget.cpp"
#include "EWIEGA46WW/moc_TelescopeClientINDI.cpp"
#include "EWIEGA46WW/moc_TelescopeClientINDIWidget.cpp"
