/****************************************************************************
** Meta object code from reading C++ file 'AstroCalcChart.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../stellarium/src/gui/AstroCalcChart.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'AstroCalcChart.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14AstroCalcChartE_t {};
} // unnamed namespace

template <> constexpr inline auto AstroCalcChart::qt_create_metaobjectdata<qt_meta_tag_ZN14AstroCalcChartE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "AstroCalcChart",
        "retranslate",
        "",
        "showToolTip",
        "QPointF",
        "point",
        "show",
        "Series",
        "AltVsTime",
        "CurrentTime",
        "TransitTime",
        "SunElevation",
        "CivilTwilight",
        "NauticalTwilight",
        "AstroTwilight",
        "Moon",
        "AzVsTime",
        "MonthlyElevation",
        "AngularSize1",
        "Declination1",
        "Distance1",
        "Elongation1",
        "HeliocentricDistance1",
        "Magnitude1",
        "PhaseAngle1",
        "Phase1",
        "RightAscension1",
        "TransitAltitude1",
        "AngularSize2",
        "Declination2",
        "Distance2",
        "Elongation2",
        "HeliocentricDistance2",
        "Magnitude2",
        "PhaseAngle2",
        "Phase2",
        "RightAscension2",
        "TransitAltitude2",
        "LunarElongation",
        "LunarElongationLimit",
        "pcDistanceAU",
        "pcDistanceDeg"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'retranslate'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showToolTip'
        QtMocHelpers::SlotData<void(const QPointF &, const bool)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 }, { QMetaType::Bool, 6 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Series'
        QtMocHelpers::EnumData<enum Series>(7, 7, QMC::EnumFlags{}).add({
            {    8, Series::AltVsTime },
            {    9, Series::CurrentTime },
            {   10, Series::TransitTime },
            {   11, Series::SunElevation },
            {   12, Series::CivilTwilight },
            {   13, Series::NauticalTwilight },
            {   14, Series::AstroTwilight },
            {   15, Series::Moon },
            {   16, Series::AzVsTime },
            {   17, Series::MonthlyElevation },
            {   18, Series::AngularSize1 },
            {   19, Series::Declination1 },
            {   20, Series::Distance1 },
            {   21, Series::Elongation1 },
            {   22, Series::HeliocentricDistance1 },
            {   23, Series::Magnitude1 },
            {   24, Series::PhaseAngle1 },
            {   25, Series::Phase1 },
            {   26, Series::RightAscension1 },
            {   27, Series::TransitAltitude1 },
            {   28, Series::AngularSize2 },
            {   29, Series::Declination2 },
            {   30, Series::Distance2 },
            {   31, Series::Elongation2 },
            {   32, Series::HeliocentricDistance2 },
            {   33, Series::Magnitude2 },
            {   34, Series::PhaseAngle2 },
            {   35, Series::Phase2 },
            {   36, Series::RightAscension2 },
            {   37, Series::TransitAltitude2 },
            {   38, Series::LunarElongation },
            {   39, Series::LunarElongationLimit },
            {   40, Series::pcDistanceAU },
            {   41, Series::pcDistanceDeg },
        }),
    };
    return QtMocHelpers::metaObjectData<AstroCalcChart, qt_meta_tag_ZN14AstroCalcChartE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject AstroCalcChart::staticMetaObject = { {
    QMetaObject::SuperData::link<QChart::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14AstroCalcChartE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14AstroCalcChartE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14AstroCalcChartE_t>.metaTypes,
    nullptr
} };

void AstroCalcChart::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AstroCalcChart *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->retranslate(); break;
        case 1: _t->showToolTip((*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        default: ;
        }
    }
}

const QMetaObject *AstroCalcChart::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *AstroCalcChart::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14AstroCalcChartE_t>.strings))
        return static_cast<void*>(this);
    return QChart::qt_metacast(_clname);
}

int AstroCalcChart::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QChart::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}
QT_WARNING_POP
