/********************************************************************************
** Form generated from reading UI file 'INDIControlWidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_INDICONTROLWIDGET_H
#define UI_INDICONTROLWIDGET_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSlider>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_INDIControlWidget
{
public:
    QHBoxLayout *horizontalLayout;
    QGridLayout *gridLayout;
    QToolButton *moveUp;
    QToolButton *moveRight;
    QToolButton *moveDown;
    QToolButton *moveLeft;
    QSpacerItem *horizontalSpacer;
    QVBoxLayout *verticalLayout_2;
    QSpacerItem *verticalSpacer_2;
    QHBoxLayout *horizontalLayout_2;
    QLabel *label_2;
    QSpacerItem *horizontalSpacer_2;
    QLabel *label;
    QSlider *speedSlider;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *INDIControlWidget)
    {
        if (INDIControlWidget->objectName().isEmpty())
            INDIControlWidget->setObjectName("INDIControlWidget");
        INDIControlWidget->resize(400, 112);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(INDIControlWidget->sizePolicy().hasHeightForWidth());
        INDIControlWidget->setSizePolicy(sizePolicy);
        INDIControlWidget->setWindowTitle(QString::fromUtf8("Form"));
        horizontalLayout = new QHBoxLayout(INDIControlWidget);
        horizontalLayout->setObjectName("horizontalLayout");
        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        moveUp = new QToolButton(INDIControlWidget);
        moveUp->setObjectName("moveUp");
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/telescopeControl/nv_spinup.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        moveUp->setIcon(icon);

        gridLayout->addWidget(moveUp, 0, 1, 1, 1);

        moveRight = new QToolButton(INDIControlWidget);
        moveRight->setObjectName("moveRight");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(moveRight->sizePolicy().hasHeightForWidth());
        moveRight->setSizePolicy(sizePolicy1);
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/telescopeControl/nv_spinright.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        moveRight->setIcon(icon1);

        gridLayout->addWidget(moveRight, 1, 2, 1, 1);

        moveDown = new QToolButton(INDIControlWidget);
        moveDown->setObjectName("moveDown");
        sizePolicy1.setHeightForWidth(moveDown->sizePolicy().hasHeightForWidth());
        moveDown->setSizePolicy(sizePolicy1);
        QIcon icon2;
        icon2.addFile(QString::fromUtf8(":/telescopeControl/nv_spindown.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        moveDown->setIcon(icon2);

        gridLayout->addWidget(moveDown, 2, 1, 1, 1);

        moveLeft = new QToolButton(INDIControlWidget);
        moveLeft->setObjectName("moveLeft");
        sizePolicy1.setHeightForWidth(moveLeft->sizePolicy().hasHeightForWidth());
        moveLeft->setSizePolicy(sizePolicy1);
        QIcon icon3;
        icon3.addFile(QString::fromUtf8(":/telescopeControl/nv_spinleft.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        moveLeft->setIcon(icon3);

        gridLayout->addWidget(moveLeft, 1, 0, 1, 1);


        horizontalLayout->addLayout(gridLayout);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        verticalLayout_2 = new QVBoxLayout();
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_2->addItem(verticalSpacer_2);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        label_2 = new QLabel(INDIControlWidget);
        label_2->setObjectName("label_2");

        horizontalLayout_2->addWidget(label_2);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_2);

        label = new QLabel(INDIControlWidget);
        label->setObjectName("label");

        horizontalLayout_2->addWidget(label);


        verticalLayout_2->addLayout(horizontalLayout_2);

        speedSlider = new QSlider(INDIControlWidget);
        speedSlider->setObjectName("speedSlider");
        speedSlider->setMinimumSize(QSize(100, 0));
        speedSlider->setMinimum(1);
        speedSlider->setMaximum(4);
        speedSlider->setSingleStep(1);
        speedSlider->setPageStep(1);
        speedSlider->setValue(2);
        speedSlider->setOrientation(Qt::Horizontal);

        verticalLayout_2->addWidget(speedSlider);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_2->addItem(verticalSpacer);


        horizontalLayout->addLayout(verticalLayout_2);


        retranslateUi(INDIControlWidget);

        QMetaObject::connectSlotsByName(INDIControlWidget);
    } // setupUi

    void retranslateUi(QWidget *INDIControlWidget)
    {
#if QT_CONFIG(tooltip)
        moveUp->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
        moveUp->setText(QString());
#if QT_CONFIG(tooltip)
        moveRight->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
        moveRight->setText(QString());
#if QT_CONFIG(tooltip)
        moveDown->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
        moveDown->setText(QString());
#if QT_CONFIG(tooltip)
        moveLeft->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
        moveLeft->setText(QString());
        label_2->setText(QCoreApplication::translate("INDIControlWidget", "Min", nullptr));
        label->setText(QCoreApplication::translate("INDIControlWidget", "Max", nullptr));
        (void)INDIControlWidget;
    } // retranslateUi

};

namespace Ui {
    class INDIControlWidget: public Ui_INDIControlWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_INDICONTROLWIDGET_H
