/********************************************************************************
** Form generated from reading UI file 'lensDistortionEstimatorDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LENSDISTORTIONESTIMATORDIALOG_H
#define UI_LENSDISTORTIONESTIMATORDIALOG_H

#include <QtCharts/QChartView>
#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QSplitter>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTextBrowser>
#include <QtWidgets/QTextEdit>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QTreeWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "Dialog.hpp"
#include "gui/ColorButton.hpp"

QT_BEGIN_NAMESPACE

class Ui_LDEDialog
{
public:
    QVBoxLayout *verticalLayout_2;
    TitleBar *titleBar;
    QTabWidget *tabs;
    QWidget *imageLoadTab;
    QGridLayout *gridLayout;
    QSpacerItem *verticalSpacer_2;
    QGroupBox *groupBox;
    QGridLayout *gridLayout_2;
    QLabel *label_7;
    QHBoxLayout *horizontalLayout_2;
    QDoubleSpinBox *imageCenterShiftX;
    QLabel *label_8;
    QDoubleSpinBox *imageCenterShiftY;
    QDoubleSpinBox *projectionCenterAzimuth;
    QLabel *label_4;
    QDoubleSpinBox *imageSmallerSideFoV;
    QLabel *label_3;
    QDoubleSpinBox *imageFieldRotation;
    QLabel *label_2;
    QComboBox *distortionModelCB;
    QLabel *label_5;
    QLabel *label;
    QDoubleSpinBox *distortionTerm2;
    QLabel *label_9;
    QDoubleSpinBox *projectionCenterElevation;
    QDoubleSpinBox *distortionTerm1;
    QDoubleSpinBox *distortionTerm3;
    QPushButton *resetPlacementBtn;
    QPushButton *resetDistortionBtn;
    QHBoxLayout *horizontalLayout;
    QLabel *label_6;
    QLineEdit *imageFilePath;
    QToolButton *imageBrowseBtn;
    QHBoxLayout *horizontalLayout_3;
    QLabel *warnings;
    QPushButton *fixWarningBtn;
    QHBoxLayout *horizontalLayout_4;
    QCheckBox *subtractBG;
    QSpinBox *bgRed;
    QSpinBox *bgGreen;
    QSpinBox *bgBlue;
    QSpacerItem *horizontalSpacer;
    QLabel *label_14;
    QDoubleSpinBox *imageBrightness;
    QWidget *fittingTab;
    QVBoxLayout *verticalLayout_4;
    QGridLayout *gridLayout_4;
    QPushButton *repositionBtn;
    QPushButton *pickImagePointBtn;
    QPushButton *optimizeBtn;
    QSplitter *listChartSplitter;
    QWidget *layoutWidget;
    QGridLayout *gridLayout_3;
    QTreeWidget *imagePointsPicked;
    QPushButton *importPointsBtn;
    QPushButton *removePointBtn;
    QPushButton *exportPointsBtn;
    QChartView *errorsChartView;
    QWidget *lensfunModelTab;
    QGridLayout *gridLayout_5;
    QTextEdit *lensfunModelXML;
    QDoubleSpinBox *lensFocalLength;
    QLabel *label_12;
    QLabel *label_15;
    QLabel *label_16;
    QPushButton *generateLensfunModelBtn;
    QLabel *label_13;
    QLineEdit *lensModel;
    QLineEdit *lensMount;
    QLineEdit *lensMake;
    QWidget *settingsTab;
    QVBoxLayout *verticalLayout;
    QGroupBox *imageEnabled;
    QGridLayout *gridLayout_6;
    QSpacerItem *horizontalSpacer_2;
    ColorButton *imageAxesColor;
    QLabel *label_10;
    QSpacerItem *verticalSpacer;
    QCheckBox *markPickedPoints;
    ColorButton *projectionCenterMarkerColor;
    ColorButton *selectedPointMarkerColor;
    ColorButton *pointMarkerColor;
    QCheckBox *imageAxesEnabled;
    QLabel *label_11;
    QCheckBox *markProjectionCenter;
    QCheckBox *imgPosResetOnLoadingEnabled;
    QPushButton *restoreDefaultsBtn;
    QWidget *aboutTab;
    QVBoxLayout *verticalLayout_11;
    QTextBrowser *about;

    void setupUi(QWidget *LDEDialog)
    {
        if (LDEDialog->objectName().isEmpty())
            LDEDialog->setObjectName("LDEDialog");
        LDEDialog->resize(300, 300);
        verticalLayout_2 = new QVBoxLayout(LDEDialog);
        verticalLayout_2->setSpacing(0);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        titleBar = new TitleBar(LDEDialog);
        titleBar->setObjectName("titleBar");

        verticalLayout_2->addWidget(titleBar);

        tabs = new QTabWidget(LDEDialog);
        tabs->setObjectName("tabs");
        tabs->setDocumentMode(false);
        imageLoadTab = new QWidget();
        imageLoadTab->setObjectName("imageLoadTab");
        gridLayout = new QGridLayout(imageLoadTab);
        gridLayout->setObjectName("gridLayout");
        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(verticalSpacer_2, 5, 0, 1, 1);

        groupBox = new QGroupBox(imageLoadTab);
        groupBox->setObjectName("groupBox");
        gridLayout_2 = new QGridLayout(groupBox);
        gridLayout_2->setObjectName("gridLayout_2");
        label_7 = new QLabel(groupBox);
        label_7->setObjectName("label_7");

        gridLayout_2->addWidget(label_7, 6, 0, 1, 1);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        imageCenterShiftX = new QDoubleSpinBox(groupBox);
        imageCenterShiftX->setObjectName("imageCenterShiftX");
        imageCenterShiftX->setMinimum(-500.000000000000000);
        imageCenterShiftX->setMaximum(500.000000000000000);
        imageCenterShiftX->setValue(0.000000000000000);

        horizontalLayout_2->addWidget(imageCenterShiftX);

        label_8 = new QLabel(groupBox);
        label_8->setObjectName("label_8");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(label_8->sizePolicy().hasHeightForWidth());
        label_8->setSizePolicy(sizePolicy);
        label_8->setText(QString::fromUtf8(","));

        horizontalLayout_2->addWidget(label_8);

        imageCenterShiftY = new QDoubleSpinBox(groupBox);
        imageCenterShiftY->setObjectName("imageCenterShiftY");
        imageCenterShiftY->setMinimum(-500.000000000000000);
        imageCenterShiftY->setMaximum(500.000000000000000);
        imageCenterShiftY->setValue(0.000000000000000);

        horizontalLayout_2->addWidget(imageCenterShiftY);


        gridLayout_2->addLayout(horizontalLayout_2, 5, 1, 1, 3);

        projectionCenterAzimuth = new QDoubleSpinBox(groupBox);
        projectionCenterAzimuth->setObjectName("projectionCenterAzimuth");
        projectionCenterAzimuth->setSuffix(QString::fromUtf8("\302\260"));
        projectionCenterAzimuth->setDecimals(7);
        projectionCenterAzimuth->setMinimum(-360.000000000000000);
        projectionCenterAzimuth->setMaximum(360.000000000000000);
        projectionCenterAzimuth->setSingleStep(0.100000000000000);

        gridLayout_2->addWidget(projectionCenterAzimuth, 0, 1, 1, 2);

        label_4 = new QLabel(groupBox);
        label_4->setObjectName("label_4");

        gridLayout_2->addWidget(label_4, 4, 0, 1, 1);

        imageSmallerSideFoV = new QDoubleSpinBox(groupBox);
        imageSmallerSideFoV->setObjectName("imageSmallerSideFoV");
        imageSmallerSideFoV->setSuffix(QString::fromUtf8("\302\260"));
        imageSmallerSideFoV->setDecimals(7);
        imageSmallerSideFoV->setMinimum(0.001000000000000);
        imageSmallerSideFoV->setMaximum(179.000000000000000);
        imageSmallerSideFoV->setValue(60.000000000000000);

        gridLayout_2->addWidget(imageSmallerSideFoV, 4, 1, 1, 2);

        label_3 = new QLabel(groupBox);
        label_3->setObjectName("label_3");

        gridLayout_2->addWidget(label_3, 3, 0, 1, 1);

        imageFieldRotation = new QDoubleSpinBox(groupBox);
        imageFieldRotation->setObjectName("imageFieldRotation");
        imageFieldRotation->setSuffix(QString::fromUtf8("\302\260"));
        imageFieldRotation->setDecimals(7);
        imageFieldRotation->setMinimum(-360.000000000000000);
        imageFieldRotation->setMaximum(360.000000000000000);
        imageFieldRotation->setSingleStep(0.100000000000000);

        gridLayout_2->addWidget(imageFieldRotation, 3, 1, 1, 2);

        label_2 = new QLabel(groupBox);
        label_2->setObjectName("label_2");

        gridLayout_2->addWidget(label_2, 1, 0, 1, 1);

        distortionModelCB = new QComboBox(groupBox);
        distortionModelCB->addItem(QString());
        distortionModelCB->addItem(QString());
        distortionModelCB->addItem(QString());
        distortionModelCB->setObjectName("distortionModelCB");

        gridLayout_2->addWidget(distortionModelCB, 6, 1, 1, 3);

        label_5 = new QLabel(groupBox);
        label_5->setObjectName("label_5");

        gridLayout_2->addWidget(label_5, 5, 0, 1, 1);

        label = new QLabel(groupBox);
        label->setObjectName("label");

        gridLayout_2->addWidget(label, 0, 0, 1, 1);

        distortionTerm2 = new QDoubleSpinBox(groupBox);
        distortionTerm2->setObjectName("distortionTerm2");
        distortionTerm2->setDecimals(7);
        distortionTerm2->setMinimum(-0.100000000000000);
        distortionTerm2->setMaximum(0.100000000000000);
        distortionTerm2->setSingleStep(0.001000000000000);

        gridLayout_2->addWidget(distortionTerm2, 7, 2, 1, 1);

        label_9 = new QLabel(groupBox);
        label_9->setObjectName("label_9");

        gridLayout_2->addWidget(label_9, 7, 0, 1, 1);

        projectionCenterElevation = new QDoubleSpinBox(groupBox);
        projectionCenterElevation->setObjectName("projectionCenterElevation");
        projectionCenterElevation->setSuffix(QString::fromUtf8("\302\260"));
        projectionCenterElevation->setDecimals(7);
        projectionCenterElevation->setMinimum(-90.000000000000000);
        projectionCenterElevation->setMaximum(90.000000000000000);
        projectionCenterElevation->setSingleStep(0.100000000000000);

        gridLayout_2->addWidget(projectionCenterElevation, 1, 1, 1, 2);

        distortionTerm1 = new QDoubleSpinBox(groupBox);
        distortionTerm1->setObjectName("distortionTerm1");
        distortionTerm1->setDecimals(7);
        distortionTerm1->setMinimum(-0.100000000000000);
        distortionTerm1->setMaximum(0.100000000000000);
        distortionTerm1->setSingleStep(0.001000000000000);

        gridLayout_2->addWidget(distortionTerm1, 7, 1, 1, 1);

        distortionTerm3 = new QDoubleSpinBox(groupBox);
        distortionTerm3->setObjectName("distortionTerm3");
        distortionTerm3->setDecimals(7);
        distortionTerm3->setMinimum(-0.100000000000000);
        distortionTerm3->setMaximum(0.100000000000000);
        distortionTerm3->setSingleStep(0.001000000000000);

        gridLayout_2->addWidget(distortionTerm3, 7, 3, 1, 1);

        resetPlacementBtn = new QPushButton(groupBox);
        resetPlacementBtn->setObjectName("resetPlacementBtn");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(resetPlacementBtn->sizePolicy().hasHeightForWidth());
        resetPlacementBtn->setSizePolicy(sizePolicy1);

        gridLayout_2->addWidget(resetPlacementBtn, 0, 3, 2, 1);

        resetDistortionBtn = new QPushButton(groupBox);
        resetDistortionBtn->setObjectName("resetDistortionBtn");
        sizePolicy1.setHeightForWidth(resetDistortionBtn->sizePolicy().hasHeightForWidth());
        resetDistortionBtn->setSizePolicy(sizePolicy1);

        gridLayout_2->addWidget(resetDistortionBtn, 3, 3, 2, 1);


        gridLayout->addWidget(groupBox, 3, 0, 1, 1);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        label_6 = new QLabel(imageLoadTab);
        label_6->setObjectName("label_6");

        horizontalLayout->addWidget(label_6);

        imageFilePath = new QLineEdit(imageLoadTab);
        imageFilePath->setObjectName("imageFilePath");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(imageFilePath->sizePolicy().hasHeightForWidth());
        imageFilePath->setSizePolicy(sizePolicy2);

        horizontalLayout->addWidget(imageFilePath);

        imageBrowseBtn = new QToolButton(imageLoadTab);
        imageBrowseBtn->setObjectName("imageBrowseBtn");
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/graphicGui/uibtFolder.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        icon.addFile(QString::fromUtf8(":/graphicGui/uibtFolder-disabled.png"), QSize(), QIcon::Mode::Disabled, QIcon::State::Off);
        imageBrowseBtn->setIcon(icon);

        horizontalLayout->addWidget(imageBrowseBtn);


        gridLayout->addLayout(horizontalLayout, 0, 0, 1, 1);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        warnings = new QLabel(imageLoadTab);
        warnings->setObjectName("warnings");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(warnings->sizePolicy().hasHeightForWidth());
        warnings->setSizePolicy(sizePolicy3);
        warnings->setWordWrap(true);
        warnings->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextSelectableByKeyboard|Qt::TextSelectableByMouse);

        horizontalLayout_3->addWidget(warnings);

        fixWarningBtn = new QPushButton(imageLoadTab);
        fixWarningBtn->setObjectName("fixWarningBtn");

        horizontalLayout_3->addWidget(fixWarningBtn);


        gridLayout->addLayout(horizontalLayout_3, 1, 0, 1, 1);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        subtractBG = new QCheckBox(imageLoadTab);
        subtractBG->setObjectName("subtractBG");

        horizontalLayout_4->addWidget(subtractBG);

        bgRed = new QSpinBox(imageLoadTab);
        bgRed->setObjectName("bgRed");
        bgRed->setMinimum(0);
        bgRed->setMaximum(255);

        horizontalLayout_4->addWidget(bgRed);

        bgGreen = new QSpinBox(imageLoadTab);
        bgGreen->setObjectName("bgGreen");
        bgGreen->setMinimum(0);
        bgGreen->setMaximum(255);

        horizontalLayout_4->addWidget(bgGreen);

        bgBlue = new QSpinBox(imageLoadTab);
        bgBlue->setObjectName("bgBlue");
        bgBlue->setMinimum(0);
        bgBlue->setMaximum(255);

        horizontalLayout_4->addWidget(bgBlue);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_4->addItem(horizontalSpacer);

        label_14 = new QLabel(imageLoadTab);
        label_14->setObjectName("label_14");

        horizontalLayout_4->addWidget(label_14);

        imageBrightness = new QDoubleSpinBox(imageLoadTab);
        imageBrightness->setObjectName("imageBrightness");
        imageBrightness->setSuffix(QString::fromUtf8("%"));
        imageBrightness->setDecimals(1);
        imageBrightness->setMaximum(100.000000000000000);
        imageBrightness->setValue(100.000000000000000);

        horizontalLayout_4->addWidget(imageBrightness);


        gridLayout->addLayout(horizontalLayout_4, 4, 0, 1, 1);

        tabs->addTab(imageLoadTab, QString());
        fittingTab = new QWidget();
        fittingTab->setObjectName("fittingTab");
        verticalLayout_4 = new QVBoxLayout(fittingTab);
        verticalLayout_4->setObjectName("verticalLayout_4");
        gridLayout_4 = new QGridLayout();
        gridLayout_4->setObjectName("gridLayout_4");
        repositionBtn = new QPushButton(fittingTab);
        repositionBtn->setObjectName("repositionBtn");
        repositionBtn->setEnabled(false);

        gridLayout_4->addWidget(repositionBtn, 0, 0, 1, 1);

        pickImagePointBtn = new QPushButton(fittingTab);
        pickImagePointBtn->setObjectName("pickImagePointBtn");
        pickImagePointBtn->setCheckable(true);

        gridLayout_4->addWidget(pickImagePointBtn, 1, 0, 1, 1);

        optimizeBtn = new QPushButton(fittingTab);
        optimizeBtn->setObjectName("optimizeBtn");
        optimizeBtn->setEnabled(false);
        sizePolicy1.setHeightForWidth(optimizeBtn->sizePolicy().hasHeightForWidth());
        optimizeBtn->setSizePolicy(sizePolicy1);

        gridLayout_4->addWidget(optimizeBtn, 0, 1, 2, 1);


        verticalLayout_4->addLayout(gridLayout_4);

        listChartSplitter = new QSplitter(fittingTab);
        listChartSplitter->setObjectName("listChartSplitter");
        listChartSplitter->setOrientation(Qt::Horizontal);
        layoutWidget = new QWidget(listChartSplitter);
        layoutWidget->setObjectName("layoutWidget");
        gridLayout_3 = new QGridLayout(layoutWidget);
        gridLayout_3->setObjectName("gridLayout_3");
        gridLayout_3->setContentsMargins(0, 0, 0, 0);
        imagePointsPicked = new QTreeWidget(layoutWidget);
        imagePointsPicked->setObjectName("imagePointsPicked");
        QSizePolicy sizePolicy4(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy4.setHorizontalStretch(0);
        sizePolicy4.setVerticalStretch(0);
        sizePolicy4.setHeightForWidth(imagePointsPicked->sizePolicy().hasHeightForWidth());
        imagePointsPicked->setSizePolicy(sizePolicy4);
        imagePointsPicked->setDragEnabled(true);
        imagePointsPicked->setDragDropMode(QAbstractItemView::InternalMove);
        imagePointsPicked->setDefaultDropAction(Qt::MoveAction);

        gridLayout_3->addWidget(imagePointsPicked, 0, 0, 1, 2);

        importPointsBtn = new QPushButton(layoutWidget);
        importPointsBtn->setObjectName("importPointsBtn");

        gridLayout_3->addWidget(importPointsBtn, 2, 0, 1, 1);

        removePointBtn = new QPushButton(layoutWidget);
        removePointBtn->setObjectName("removePointBtn");
        removePointBtn->setEnabled(false);

        gridLayout_3->addWidget(removePointBtn, 1, 0, 1, 2);

        exportPointsBtn = new QPushButton(layoutWidget);
        exportPointsBtn->setObjectName("exportPointsBtn");
        exportPointsBtn->setEnabled(false);

        gridLayout_3->addWidget(exportPointsBtn, 2, 1, 1, 1);

        listChartSplitter->addWidget(layoutWidget);
        errorsChartView = new QChartView(listChartSplitter);
        errorsChartView->setObjectName("errorsChartView");
        QSizePolicy sizePolicy5(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy5.setHorizontalStretch(0);
        sizePolicy5.setVerticalStretch(0);
        sizePolicy5.setHeightForWidth(errorsChartView->sizePolicy().hasHeightForWidth());
        errorsChartView->setSizePolicy(sizePolicy5);
        listChartSplitter->addWidget(errorsChartView);

        verticalLayout_4->addWidget(listChartSplitter);

        verticalLayout_4->setStretch(1, 1);
        tabs->addTab(fittingTab, QString());
        lensfunModelTab = new QWidget();
        lensfunModelTab->setObjectName("lensfunModelTab");
        gridLayout_5 = new QGridLayout(lensfunModelTab);
        gridLayout_5->setObjectName("gridLayout_5");
        lensfunModelXML = new QTextEdit(lensfunModelTab);
        lensfunModelXML->setObjectName("lensfunModelXML");
        lensfunModelXML->setReadOnly(true);
        lensfunModelXML->setAcceptRichText(false);

        gridLayout_5->addWidget(lensfunModelXML, 6, 0, 1, 3);

        lensFocalLength = new QDoubleSpinBox(lensfunModelTab);
        lensFocalLength->setObjectName("lensFocalLength");
        lensFocalLength->setMinimum(1.000000000000000);
        lensFocalLength->setMaximum(9999.000000000000000);
        lensFocalLength->setSingleStep(0.100000000000000);
        lensFocalLength->setValue(45.000000000000000);

        gridLayout_5->addWidget(lensFocalLength, 3, 2, 1, 1);

        label_12 = new QLabel(lensfunModelTab);
        label_12->setObjectName("label_12");

        gridLayout_5->addWidget(label_12, 0, 0, 1, 1);

        label_15 = new QLabel(lensfunModelTab);
        label_15->setObjectName("label_15");

        gridLayout_5->addWidget(label_15, 2, 0, 1, 1);

        label_16 = new QLabel(lensfunModelTab);
        label_16->setObjectName("label_16");

        gridLayout_5->addWidget(label_16, 3, 0, 1, 1);

        generateLensfunModelBtn = new QPushButton(lensfunModelTab);
        generateLensfunModelBtn->setObjectName("generateLensfunModelBtn");
        generateLensfunModelBtn->setEnabled(false);

        gridLayout_5->addWidget(generateLensfunModelBtn, 5, 0, 1, 3);

        label_13 = new QLabel(lensfunModelTab);
        label_13->setObjectName("label_13");

        gridLayout_5->addWidget(label_13, 1, 0, 1, 1);

        lensModel = new QLineEdit(lensfunModelTab);
        lensModel->setObjectName("lensModel");

        gridLayout_5->addWidget(lensModel, 1, 2, 1, 1);

        lensMount = new QLineEdit(lensfunModelTab);
        lensMount->setObjectName("lensMount");

        gridLayout_5->addWidget(lensMount, 2, 2, 1, 1);

        lensMake = new QLineEdit(lensfunModelTab);
        lensMake->setObjectName("lensMake");

        gridLayout_5->addWidget(lensMake, 0, 2, 1, 1);

        tabs->addTab(lensfunModelTab, QString());
        settingsTab = new QWidget();
        settingsTab->setObjectName("settingsTab");
        verticalLayout = new QVBoxLayout(settingsTab);
        verticalLayout->setObjectName("verticalLayout");
        imageEnabled = new QGroupBox(settingsTab);
        imageEnabled->setObjectName("imageEnabled");
        imageEnabled->setFlat(true);
        imageEnabled->setCheckable(true);
        gridLayout_6 = new QGridLayout(imageEnabled);
        gridLayout_6->setObjectName("gridLayout_6");
        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_6->addItem(horizontalSpacer_2, 0, 1, 1, 1);

        imageAxesColor = new ColorButton(imageEnabled);
        imageAxesColor->setObjectName("imageAxesColor");

        gridLayout_6->addWidget(imageAxesColor, 2, 2, 1, 1);

        label_10 = new QLabel(imageEnabled);
        label_10->setObjectName("label_10");

        gridLayout_6->addWidget(label_10, 0, 3, 1, 1);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_6->addItem(verticalSpacer, 4, 1, 1, 1);

        markPickedPoints = new QCheckBox(imageEnabled);
        markPickedPoints->setObjectName("markPickedPoints");

        gridLayout_6->addWidget(markPickedPoints, 0, 0, 1, 1);

        projectionCenterMarkerColor = new ColorButton(imageEnabled);
        projectionCenterMarkerColor->setObjectName("projectionCenterMarkerColor");

        gridLayout_6->addWidget(projectionCenterMarkerColor, 1, 2, 1, 1);

        selectedPointMarkerColor = new ColorButton(imageEnabled);
        selectedPointMarkerColor->setObjectName("selectedPointMarkerColor");

        gridLayout_6->addWidget(selectedPointMarkerColor, 0, 4, 1, 1);

        pointMarkerColor = new ColorButton(imageEnabled);
        pointMarkerColor->setObjectName("pointMarkerColor");

        gridLayout_6->addWidget(pointMarkerColor, 0, 2, 1, 1);

        imageAxesEnabled = new QCheckBox(imageEnabled);
        imageAxesEnabled->setObjectName("imageAxesEnabled");

        gridLayout_6->addWidget(imageAxesEnabled, 2, 0, 1, 1);

        label_11 = new QLabel(imageEnabled);
        label_11->setObjectName("label_11");

        gridLayout_6->addWidget(label_11, 0, 5, 1, 1);

        markProjectionCenter = new QCheckBox(imageEnabled);
        markProjectionCenter->setObjectName("markProjectionCenter");

        gridLayout_6->addWidget(markProjectionCenter, 1, 0, 1, 1);

        imgPosResetOnLoadingEnabled = new QCheckBox(imageEnabled);
        imgPosResetOnLoadingEnabled->setObjectName("imgPosResetOnLoadingEnabled");

        gridLayout_6->addWidget(imgPosResetOnLoadingEnabled, 3, 0, 1, 1);


        verticalLayout->addWidget(imageEnabled);

        restoreDefaultsBtn = new QPushButton(settingsTab);
        restoreDefaultsBtn->setObjectName("restoreDefaultsBtn");

        verticalLayout->addWidget(restoreDefaultsBtn);

        tabs->addTab(settingsTab, QString());
        aboutTab = new QWidget();
        aboutTab->setObjectName("aboutTab");
        verticalLayout_11 = new QVBoxLayout(aboutTab);
        verticalLayout_11->setSpacing(9);
        verticalLayout_11->setObjectName("verticalLayout_11");
        about = new QTextBrowser(aboutTab);
        about->setObjectName("about");
        about->setOpenExternalLinks(true);
        about->setOpenLinks(true);

        verticalLayout_11->addWidget(about);

        tabs->addTab(aboutTab, QString());

        verticalLayout_2->addWidget(tabs);

#if QT_CONFIG(shortcut)
        label_4->setBuddy(imageSmallerSideFoV);
        label_3->setBuddy(imageFieldRotation);
        label_2->setBuddy(projectionCenterElevation);
        label->setBuddy(projectionCenterAzimuth);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(tabs, imageFilePath);
        QWidget::setTabOrder(imageFilePath, imageBrowseBtn);
        QWidget::setTabOrder(imageBrowseBtn, fixWarningBtn);
        QWidget::setTabOrder(fixWarningBtn, projectionCenterAzimuth);
        QWidget::setTabOrder(projectionCenterAzimuth, projectionCenterElevation);
        QWidget::setTabOrder(projectionCenterElevation, imageFieldRotation);
        QWidget::setTabOrder(imageFieldRotation, imageSmallerSideFoV);
        QWidget::setTabOrder(imageSmallerSideFoV, imageCenterShiftX);
        QWidget::setTabOrder(imageCenterShiftX, imageCenterShiftY);
        QWidget::setTabOrder(imageCenterShiftY, distortionModelCB);
        QWidget::setTabOrder(distortionModelCB, distortionTerm1);
        QWidget::setTabOrder(distortionTerm1, distortionTerm2);
        QWidget::setTabOrder(distortionTerm2, distortionTerm3);
        QWidget::setTabOrder(distortionTerm3, subtractBG);
        QWidget::setTabOrder(subtractBG, bgRed);
        QWidget::setTabOrder(bgRed, bgGreen);
        QWidget::setTabOrder(bgGreen, bgBlue);
        QWidget::setTabOrder(bgBlue, imageBrightness);
        QWidget::setTabOrder(imageBrightness, repositionBtn);
        QWidget::setTabOrder(repositionBtn, optimizeBtn);
        QWidget::setTabOrder(optimizeBtn, pickImagePointBtn);
        QWidget::setTabOrder(pickImagePointBtn, imagePointsPicked);
        QWidget::setTabOrder(imagePointsPicked, removePointBtn);
        QWidget::setTabOrder(removePointBtn, importPointsBtn);
        QWidget::setTabOrder(importPointsBtn, exportPointsBtn);
        QWidget::setTabOrder(exportPointsBtn, errorsChartView);
        QWidget::setTabOrder(errorsChartView, imageEnabled);
        QWidget::setTabOrder(imageEnabled, markPickedPoints);
        QWidget::setTabOrder(markPickedPoints, pointMarkerColor);
        QWidget::setTabOrder(pointMarkerColor, selectedPointMarkerColor);
        QWidget::setTabOrder(selectedPointMarkerColor, markProjectionCenter);
        QWidget::setTabOrder(markProjectionCenter, projectionCenterMarkerColor);
        QWidget::setTabOrder(projectionCenterMarkerColor, imageAxesEnabled);
        QWidget::setTabOrder(imageAxesEnabled, imageAxesColor);
        QWidget::setTabOrder(imageAxesColor, restoreDefaultsBtn);
        QWidget::setTabOrder(restoreDefaultsBtn, about);
        QWidget::setTabOrder(about, lensMake);
        QWidget::setTabOrder(lensMake, lensModel);
        QWidget::setTabOrder(lensModel, lensMount);
        QWidget::setTabOrder(lensMount, lensFocalLength);
        QWidget::setTabOrder(lensFocalLength, generateLensfunModelBtn);
        QWidget::setTabOrder(generateLensfunModelBtn, lensfunModelXML);

        retranslateUi(LDEDialog);

        tabs->setCurrentIndex(0);
        distortionModelCB->setCurrentIndex(2);


        QMetaObject::connectSlotsByName(LDEDialog);
    } // setupUi

    void retranslateUi(QWidget *LDEDialog)
    {
        LDEDialog->setWindowTitle(QCoreApplication::translate("LDEDialog", "Lens Distortion Estimator", nullptr));
        titleBar->setProperty("title", QVariant(QCoreApplication::translate("LDEDialog", "Lens Distortion Estimator", nullptr)));
        groupBox->setTitle(QCoreApplication::translate("LDEDialog", "Image placement", nullptr));
        label_7->setText(QCoreApplication::translate("LDEDialog", "Distortion model:", nullptr));
        imageCenterShiftX->setSuffix(QCoreApplication::translate("LDEDialog", " px", nullptr));
        imageCenterShiftY->setSuffix(QCoreApplication::translate("LDEDialog", " px", nullptr));
        label_4->setText(QCoreApplication::translate("LDEDialog", "Field of view in smaller direction:", nullptr));
        label_3->setText(QCoreApplication::translate("LDEDialog", "Image rotation angle:", nullptr));
        label_2->setText(QCoreApplication::translate("LDEDialog", "Center of projection elevation:", nullptr));
        distortionModelCB->setItemText(0, QCoreApplication::translate("LDEDialog", "3rd order polynomial", nullptr));
        distortionModelCB->setItemText(1, QCoreApplication::translate("LDEDialog", "5th order polynomial", nullptr));
        distortionModelCB->setItemText(2, QCoreApplication::translate("LDEDialog", "PTLens", nullptr));

        label_5->setText(QCoreApplication::translate("LDEDialog", "Image center shift:", nullptr));
        label->setText(QCoreApplication::translate("LDEDialog", "Center of projection azimuth:", nullptr));
        label_9->setText(QCoreApplication::translate("LDEDialog", "Distortion coefficients:", nullptr));
        resetPlacementBtn->setText(QCoreApplication::translate("LDEDialog", "Reset\n"
"placement", nullptr));
        resetDistortionBtn->setText(QCoreApplication::translate("LDEDialog", "Reset\n"
"distortion", nullptr));
        label_6->setText(QCoreApplication::translate("LDEDialog", "Image to load:", nullptr));
#if QT_CONFIG(tooltip)
        imageBrowseBtn->setToolTip(QCoreApplication::translate("LDEDialog", "Browse for an image...", nullptr));
#endif // QT_CONFIG(tooltip)
        imageBrowseBtn->setText(QString());
        warnings->setText(QString());
        fixWarningBtn->setText(QCoreApplication::translate("LDEDialog", "Fix", "A verb used on a button that fixes a problem."));
        subtractBG->setText(QCoreApplication::translate("LDEDialog", "Subtract background RGB:", nullptr));
        label_14->setText(QCoreApplication::translate("LDEDialog", "Image brightness:", nullptr));
        tabs->setTabText(tabs->indexOf(imageLoadTab), QCoreApplication::translate("LDEDialog", "Image Loading", nullptr));
        repositionBtn->setText(QCoreApplication::translate("LDEDialog", "Reposition image by first two points", nullptr));
        pickImagePointBtn->setText(QCoreApplication::translate("LDEDialog", "Pick an image point for selected object...", nullptr));
        optimizeBtn->setText(QCoreApplication::translate("LDEDialog", "Optimize", nullptr));
        QTreeWidgetItem *___qtreewidgetitem = imagePointsPicked->headerItem();
        ___qtreewidgetitem->setText(2, QCoreApplication::translate("LDEDialog", "y, px", nullptr));
        ___qtreewidgetitem->setText(1, QCoreApplication::translate("LDEDialog", "x, px", nullptr));
        ___qtreewidgetitem->setText(0, QCoreApplication::translate("LDEDialog", "Object", nullptr));
        importPointsBtn->setText(QCoreApplication::translate("LDEDialog", "Import list...", nullptr));
        removePointBtn->setText(QCoreApplication::translate("LDEDialog", "Remove point", nullptr));
        exportPointsBtn->setText(QCoreApplication::translate("LDEDialog", "Export list...", nullptr));
        tabs->setTabText(tabs->indexOf(fittingTab), QCoreApplication::translate("LDEDialog", "Fitting", nullptr));
        lensFocalLength->setSuffix(QCoreApplication::translate("LDEDialog", " mm", nullptr));
        label_12->setText(QCoreApplication::translate("LDEDialog", "Make:", nullptr));
#if QT_CONFIG(tooltip)
        label_15->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
        label_15->setText(QCoreApplication::translate("LDEDialog", "Mount:", "Lens mount on a camera"));
        label_16->setText(QCoreApplication::translate("LDEDialog", "Focal length:", nullptr));
        generateLensfunModelBtn->setText(QCoreApplication::translate("LDEDialog", "Generate", nullptr));
        label_13->setText(QCoreApplication::translate("LDEDialog", "Model:", nullptr));
        tabs->setTabText(tabs->indexOf(lensfunModelTab), QCoreApplication::translate("LDEDialog", "Model for lensfun", nullptr));
        imageEnabled->setTitle(QCoreApplication::translate("LDEDialog", "Show image", nullptr));
        imageAxesColor->setText(QString());
        label_10->setText(QCoreApplication::translate("LDEDialog", "normal", "As in \"normal\" points, the opposite being \"selected\""));
        markPickedPoints->setText(QCoreApplication::translate("LDEDialog", "Mark picked points", nullptr));
        projectionCenterMarkerColor->setText(QString());
        selectedPointMarkerColor->setText(QString());
        pointMarkerColor->setText(QString());
        imageAxesEnabled->setText(QCoreApplication::translate("LDEDialog", "Mark image axes", nullptr));
        label_11->setText(QCoreApplication::translate("LDEDialog", "selected", "As in \"selected points\""));
        markProjectionCenter->setText(QCoreApplication::translate("LDEDialog", "Mark center of projection", nullptr));
        imgPosResetOnLoadingEnabled->setText(QCoreApplication::translate("LDEDialog", "Reset image position on loading", nullptr));
        restoreDefaultsBtn->setText(QCoreApplication::translate("LDEDialog", "Restore defaults", nullptr));
        tabs->setTabText(tabs->indexOf(settingsTab), QCoreApplication::translate("LDEDialog", "Settings", nullptr));
        tabs->setTabText(tabs->indexOf(aboutTab), QCoreApplication::translate("LDEDialog", "About", "tab in plugin windows"));
    } // retranslateUi

};

namespace Ui {
    class LDEDialog: public Ui_LDEDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_LENSDISTORTIONESTIMATORDIALOG_H
