/********************************************************************************
** Form generated from reading UI file 'telescopeDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TELESCOPEDIALOG_H
#define UI_TELESCOPEDIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTextBrowser>
#include <QtWidgets/QTreeView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "Dialog.hpp"
#include "gui/ColorButton.hpp"

QT_BEGIN_NAMESPACE

class Ui_telescopeDialogForm
{
public:
    QVBoxLayout *verticalLayoutTelescopeDialog;
    TitleBar *titleBar;
    QTabWidget *tabs;
    QWidget *tabTelescopes;
    QVBoxLayout *verticalLayoutTabTelescopes;
    QGroupBox *groupBoxTelescopes;
    QVBoxLayout *verticalLayout_2;
    QTreeView *telescopeTreeView;
    QFrame *frameButtons;
    QHBoxLayout *horizontalLayout;
    QPushButton *pushButtonChangeStatus;
    QPushButton *pushButtonConfigure;
    QPushButton *pushButtonAdd;
    QPushButton *pushButtonRemove;
    QSpacerItem *horizontalSpacer;
    QLabel *labelWarning;
    QWidget *tabOptions;
    QVBoxLayout *verticalLayout_3;
    QGroupBox *groupBoxGUI;
    QGridLayout *gridLayout_2;
    QCheckBox *checkBoxLabels;
    QCheckBox *checkBoxCircles;
    QCheckBox *checkBoxReticles;
    ColorButton *reticleColorButton;
    ColorButton *circleColorButton;
    ColorButton *labelColorButton;
    QGroupBox *groupBoxTelescopeServers;
    QVBoxLayout *verticalLayout_6;
    QCheckBox *checkBoxEnableLogs;
    QCheckBox *checkBoxUseExecutables;
    QFrame *frameExecutablesDirectory;
    QHBoxLayout *horizontalLayoutExecutablesDirectory;
    QLabel *labelExecutablesDirectory;
    QLineEdit *lineEditExecutablesDirectory;
    QPushButton *pushButtonPickExecutablesDirectory;
    QSpacerItem *verticalSpacer;
    QWidget *tabHelp;
    QVBoxLayout *verticalLayoutHelp;
    QTextBrowser *textBrowserHelp;
    QWidget *tabAbout;
    QVBoxLayout *verticalLayout;
    QTextBrowser *textBrowserAbout;

    void setupUi(QWidget *telescopeDialogForm)
    {
        if (telescopeDialogForm->objectName().isEmpty())
            telescopeDialogForm->setObjectName("telescopeDialogForm");
        telescopeDialogForm->resize(530, 420);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(telescopeDialogForm->sizePolicy().hasHeightForWidth());
        telescopeDialogForm->setSizePolicy(sizePolicy);
        verticalLayoutTelescopeDialog = new QVBoxLayout(telescopeDialogForm);
        verticalLayoutTelescopeDialog->setSpacing(0);
        verticalLayoutTelescopeDialog->setObjectName("verticalLayoutTelescopeDialog");
        verticalLayoutTelescopeDialog->setSizeConstraint(QLayout::SetNoConstraint);
        verticalLayoutTelescopeDialog->setContentsMargins(0, 0, 0, 0);
        titleBar = new TitleBar(telescopeDialogForm);
        titleBar->setObjectName("titleBar");

        verticalLayoutTelescopeDialog->addWidget(titleBar);

        tabs = new QTabWidget(telescopeDialogForm);
        tabs->setObjectName("tabs");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Expanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(tabs->sizePolicy().hasHeightForWidth());
        tabs->setSizePolicy(sizePolicy1);
        tabTelescopes = new QWidget();
        tabTelescopes->setObjectName("tabTelescopes");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::MinimumExpanding);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(tabTelescopes->sizePolicy().hasHeightForWidth());
        tabTelescopes->setSizePolicy(sizePolicy2);
        verticalLayoutTabTelescopes = new QVBoxLayout(tabTelescopes);
        verticalLayoutTabTelescopes->setSpacing(0);
        verticalLayoutTabTelescopes->setObjectName("verticalLayoutTabTelescopes");
        verticalLayoutTabTelescopes->setContentsMargins(0, 0, 0, 0);
        groupBoxTelescopes = new QGroupBox(tabTelescopes);
        groupBoxTelescopes->setObjectName("groupBoxTelescopes");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(groupBoxTelescopes->sizePolicy().hasHeightForWidth());
        groupBoxTelescopes->setSizePolicy(sizePolicy3);
        groupBoxTelescopes->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
        verticalLayout_2 = new QVBoxLayout(groupBoxTelescopes);
        verticalLayout_2->setSpacing(0);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        telescopeTreeView = new QTreeView(groupBoxTelescopes);
        telescopeTreeView->setObjectName("telescopeTreeView");
        QSizePolicy sizePolicy4(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::MinimumExpanding);
        sizePolicy4.setHorizontalStretch(0);
        sizePolicy4.setVerticalStretch(0);
        sizePolicy4.setHeightForWidth(telescopeTreeView->sizePolicy().hasHeightForWidth());
        telescopeTreeView->setSizePolicy(sizePolicy4);
        telescopeTreeView->setMinimumSize(QSize(0, 100));
        telescopeTreeView->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        telescopeTreeView->setEditTriggers(QAbstractItemView::NoEditTriggers);
        telescopeTreeView->setSelectionMode(QAbstractItemView::SingleSelection);
        telescopeTreeView->setSelectionBehavior(QAbstractItemView::SelectRows);
        telescopeTreeView->setRootIsDecorated(false);
        telescopeTreeView->setUniformRowHeights(true);
        telescopeTreeView->setItemsExpandable(false);
        telescopeTreeView->setSortingEnabled(true);
        telescopeTreeView->setAllColumnsShowFocus(true);
        telescopeTreeView->setExpandsOnDoubleClick(false);

        verticalLayout_2->addWidget(telescopeTreeView);

        frameButtons = new QFrame(groupBoxTelescopes);
        frameButtons->setObjectName("frameButtons");
        QSizePolicy sizePolicy5(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);
        sizePolicy5.setHorizontalStretch(0);
        sizePolicy5.setVerticalStretch(0);
        sizePolicy5.setHeightForWidth(frameButtons->sizePolicy().hasHeightForWidth());
        frameButtons->setSizePolicy(sizePolicy5);
        frameButtons->setMinimumSize(QSize(0, 40));
        horizontalLayout = new QHBoxLayout(frameButtons);
        horizontalLayout->setSpacing(10);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(10, 0, 10, 0);
        pushButtonChangeStatus = new QPushButton(frameButtons);
        pushButtonChangeStatus->setObjectName("pushButtonChangeStatus");
        pushButtonChangeStatus->setMinimumSize(QSize(0, 24));
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/graphicGui/uibtStart.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        icon.addFile(QString::fromUtf8(":/graphicGui/uibtStart-disabled.png"), QSize(), QIcon::Mode::Disabled, QIcon::State::Off);
        pushButtonChangeStatus->setIcon(icon);

        horizontalLayout->addWidget(pushButtonChangeStatus);

        pushButtonConfigure = new QPushButton(frameButtons);
        pushButtonConfigure->setObjectName("pushButtonConfigure");
        pushButtonConfigure->setMinimumSize(QSize(0, 24));
        pushButtonConfigure->setLayoutDirection(Qt::RightToLeft);
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/graphicGui/uibtSettings.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        icon1.addFile(QString::fromUtf8(":/graphicGui/uibtSettings-disabled.png"), QSize(), QIcon::Mode::Disabled, QIcon::State::Off);
        pushButtonConfigure->setIcon(icon1);

        horizontalLayout->addWidget(pushButtonConfigure);

        pushButtonAdd = new QPushButton(frameButtons);
        pushButtonAdd->setObjectName("pushButtonAdd");
        pushButtonAdd->setMinimumSize(QSize(0, 24));
        pushButtonAdd->setLayoutDirection(Qt::RightToLeft);
        QIcon icon2;
        icon2.addFile(QString::fromUtf8(":/graphicGui/uibtCreate.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        icon2.addFile(QString::fromUtf8(":/graphicGui/uibtCreate-disabled.png"), QSize(), QIcon::Mode::Disabled, QIcon::State::Off);
        pushButtonAdd->setIcon(icon2);

        horizontalLayout->addWidget(pushButtonAdd);

        pushButtonRemove = new QPushButton(frameButtons);
        pushButtonRemove->setObjectName("pushButtonRemove");
        pushButtonRemove->setEnabled(true);
        pushButtonRemove->setMinimumSize(QSize(0, 24));
        QIcon icon3;
        icon3.addFile(QString::fromUtf8(":/graphicGui/uibtDelete.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        icon3.addFile(QString::fromUtf8(":/graphicGui/uibtDelete-disabled.png"), QSize(), QIcon::Mode::Disabled, QIcon::State::Off);
        pushButtonRemove->setIcon(icon3);

        horizontalLayout->addWidget(pushButtonRemove);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);


        verticalLayout_2->addWidget(frameButtons);

        labelWarning = new QLabel(groupBoxTelescopes);
        labelWarning->setObjectName("labelWarning");
        sizePolicy3.setHeightForWidth(labelWarning->sizePolicy().hasHeightForWidth());
        labelWarning->setSizePolicy(sizePolicy3);
        labelWarning->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
        labelWarning->setWordWrap(true);
        labelWarning->setTextInteractionFlags(Qt::NoTextInteraction);

        verticalLayout_2->addWidget(labelWarning);


        verticalLayoutTabTelescopes->addWidget(groupBoxTelescopes);

        tabs->addTab(tabTelescopes, QString());
        tabOptions = new QWidget();
        tabOptions->setObjectName("tabOptions");
        sizePolicy3.setHeightForWidth(tabOptions->sizePolicy().hasHeightForWidth());
        tabOptions->setSizePolicy(sizePolicy3);
        verticalLayout_3 = new QVBoxLayout(tabOptions);
        verticalLayout_3->setObjectName("verticalLayout_3");
        groupBoxGUI = new QGroupBox(tabOptions);
        groupBoxGUI->setObjectName("groupBoxGUI");
        groupBoxGUI->setFlat(false);
        groupBoxGUI->setCheckable(false);
        gridLayout_2 = new QGridLayout(groupBoxGUI);
        gridLayout_2->setSpacing(4);
        gridLayout_2->setObjectName("gridLayout_2");
        gridLayout_2->setContentsMargins(0, 0, 0, 0);
        checkBoxLabels = new QCheckBox(groupBoxGUI);
        checkBoxLabels->setObjectName("checkBoxLabels");
        QSizePolicy sizePolicy6(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy6.setHorizontalStretch(0);
        sizePolicy6.setVerticalStretch(0);
        sizePolicy6.setHeightForWidth(checkBoxLabels->sizePolicy().hasHeightForWidth());
        checkBoxLabels->setSizePolicy(sizePolicy6);
        checkBoxLabels->setMinimumSize(QSize(0, 26));
        checkBoxLabels->setChecked(true);

        gridLayout_2->addWidget(checkBoxLabels, 0, 1, 1, 1);

        checkBoxCircles = new QCheckBox(groupBoxGUI);
        checkBoxCircles->setObjectName("checkBoxCircles");
        sizePolicy6.setHeightForWidth(checkBoxCircles->sizePolicy().hasHeightForWidth());
        checkBoxCircles->setSizePolicy(sizePolicy6);
        checkBoxCircles->setMinimumSize(QSize(0, 26));
        checkBoxCircles->setChecked(true);

        gridLayout_2->addWidget(checkBoxCircles, 2, 1, 1, 1);

        checkBoxReticles = new QCheckBox(groupBoxGUI);
        checkBoxReticles->setObjectName("checkBoxReticles");
        sizePolicy6.setHeightForWidth(checkBoxReticles->sizePolicy().hasHeightForWidth());
        checkBoxReticles->setSizePolicy(sizePolicy6);
        checkBoxReticles->setMinimumSize(QSize(0, 26));
        checkBoxReticles->setChecked(true);

        gridLayout_2->addWidget(checkBoxReticles, 1, 1, 1, 1);

        reticleColorButton = new ColorButton(groupBoxGUI);
        reticleColorButton->setObjectName("reticleColorButton");

        gridLayout_2->addWidget(reticleColorButton, 1, 0, 1, 1);

        circleColorButton = new ColorButton(groupBoxGUI);
        circleColorButton->setObjectName("circleColorButton");

        gridLayout_2->addWidget(circleColorButton, 2, 0, 1, 1);

        labelColorButton = new ColorButton(groupBoxGUI);
        labelColorButton->setObjectName("labelColorButton");

        gridLayout_2->addWidget(labelColorButton, 0, 0, 1, 1);


        verticalLayout_3->addWidget(groupBoxGUI);

        groupBoxTelescopeServers = new QGroupBox(tabOptions);
        groupBoxTelescopeServers->setObjectName("groupBoxTelescopeServers");
        verticalLayout_6 = new QVBoxLayout(groupBoxTelescopeServers);
        verticalLayout_6->setSpacing(4);
        verticalLayout_6->setObjectName("verticalLayout_6");
        verticalLayout_6->setContentsMargins(0, 0, 0, 0);
        checkBoxEnableLogs = new QCheckBox(groupBoxTelescopeServers);
        checkBoxEnableLogs->setObjectName("checkBoxEnableLogs");
        checkBoxEnableLogs->setMinimumSize(QSize(0, 24));

        verticalLayout_6->addWidget(checkBoxEnableLogs);

        checkBoxUseExecutables = new QCheckBox(groupBoxTelescopeServers);
        checkBoxUseExecutables->setObjectName("checkBoxUseExecutables");
        checkBoxUseExecutables->setEnabled(true);
        checkBoxUseExecutables->setMinimumSize(QSize(0, 24));
        checkBoxUseExecutables->setChecked(true);

        verticalLayout_6->addWidget(checkBoxUseExecutables);

        frameExecutablesDirectory = new QFrame(groupBoxTelescopeServers);
        frameExecutablesDirectory->setObjectName("frameExecutablesDirectory");
        horizontalLayoutExecutablesDirectory = new QHBoxLayout(frameExecutablesDirectory);
        horizontalLayoutExecutablesDirectory->setObjectName("horizontalLayoutExecutablesDirectory");
        horizontalLayoutExecutablesDirectory->setContentsMargins(0, 0, 0, 0);
        labelExecutablesDirectory = new QLabel(frameExecutablesDirectory);
        labelExecutablesDirectory->setObjectName("labelExecutablesDirectory");

        horizontalLayoutExecutablesDirectory->addWidget(labelExecutablesDirectory);

        lineEditExecutablesDirectory = new QLineEdit(frameExecutablesDirectory);
        lineEditExecutablesDirectory->setObjectName("lineEditExecutablesDirectory");
        lineEditExecutablesDirectory->setReadOnly(true);

        horizontalLayoutExecutablesDirectory->addWidget(lineEditExecutablesDirectory);

        pushButtonPickExecutablesDirectory = new QPushButton(frameExecutablesDirectory);
        pushButtonPickExecutablesDirectory->setObjectName("pushButtonPickExecutablesDirectory");
        QSizePolicy sizePolicy7(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy7.setHorizontalStretch(0);
        sizePolicy7.setVerticalStretch(0);
        sizePolicy7.setHeightForWidth(pushButtonPickExecutablesDirectory->sizePolicy().hasHeightForWidth());
        pushButtonPickExecutablesDirectory->setSizePolicy(sizePolicy7);
        QIcon icon4;
        icon4.addFile(QString::fromUtf8(":/graphicGui/uibtFolder.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        icon4.addFile(QString::fromUtf8(":/graphicGui/uibtFolder-disabled.png"), QSize(), QIcon::Mode::Disabled, QIcon::State::Off);
        pushButtonPickExecutablesDirectory->setIcon(icon4);

        horizontalLayoutExecutablesDirectory->addWidget(pushButtonPickExecutablesDirectory);


        verticalLayout_6->addWidget(frameExecutablesDirectory);


        verticalLayout_3->addWidget(groupBoxTelescopeServers);

        verticalSpacer = new QSpacerItem(0, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_3->addItem(verticalSpacer);

        tabs->addTab(tabOptions, QString());
        tabHelp = new QWidget();
        tabHelp->setObjectName("tabHelp");
        sizePolicy2.setHeightForWidth(tabHelp->sizePolicy().hasHeightForWidth());
        tabHelp->setSizePolicy(sizePolicy2);
        tabHelp->setMinimumSize(QSize(0, 10));
        verticalLayoutHelp = new QVBoxLayout(tabHelp);
        verticalLayoutHelp->setObjectName("verticalLayoutHelp");
        textBrowserHelp = new QTextBrowser(tabHelp);
        textBrowserHelp->setObjectName("textBrowserHelp");
        textBrowserHelp->setMinimumSize(QSize(0, 10));
        textBrowserHelp->setOpenExternalLinks(true);

        verticalLayoutHelp->addWidget(textBrowserHelp);

        tabs->addTab(tabHelp, QString());
        tabAbout = new QWidget();
        tabAbout->setObjectName("tabAbout");
        sizePolicy2.setHeightForWidth(tabAbout->sizePolicy().hasHeightForWidth());
        tabAbout->setSizePolicy(sizePolicy2);
        verticalLayout = new QVBoxLayout(tabAbout);
        verticalLayout->setObjectName("verticalLayout");
        textBrowserAbout = new QTextBrowser(tabAbout);
        textBrowserAbout->setObjectName("textBrowserAbout");
        sizePolicy2.setHeightForWidth(textBrowserAbout->sizePolicy().hasHeightForWidth());
        textBrowserAbout->setSizePolicy(sizePolicy2);
        textBrowserAbout->setOpenExternalLinks(true);

        verticalLayout->addWidget(textBrowserAbout);

        tabs->addTab(tabAbout, QString());

        verticalLayoutTelescopeDialog->addWidget(tabs);


        retranslateUi(telescopeDialogForm);

        tabs->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(telescopeDialogForm);
    } // setupUi

    void retranslateUi(QWidget *telescopeDialogForm)
    {
        titleBar->setTitle(QCoreApplication::translate("telescopeDialogForm", "Telescopes", nullptr));
        groupBoxTelescopes->setTitle(QCoreApplication::translate("telescopeDialogForm", "Telescopes Controlled", nullptr));
#if QT_CONFIG(tooltip)
        pushButtonConfigure->setToolTip(QCoreApplication::translate("telescopeDialogForm", "Configure the selected telescope", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        pushButtonAdd->setToolTip(QCoreApplication::translate("telescopeDialogForm", "Add a new telescope", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        pushButtonRemove->setToolTip(QCoreApplication::translate("telescopeDialogForm", "Remove the selected telescope", nullptr));
#endif // QT_CONFIG(tooltip)
        tabs->setTabText(tabs->indexOf(tabTelescopes), QCoreApplication::translate("telescopeDialogForm", "Telescopes", nullptr));
        groupBoxGUI->setTitle(QCoreApplication::translate("telescopeDialogForm", "GUI", nullptr));
        checkBoxLabels->setText(QCoreApplication::translate("telescopeDialogForm", "Show telescope labels", nullptr));
        checkBoxCircles->setText(QCoreApplication::translate("telescopeDialogForm", "Show field of view indicators", nullptr));
        checkBoxReticles->setText(QCoreApplication::translate("telescopeDialogForm", "Show telescope reticles", nullptr));
        reticleColorButton->setText(QString());
        circleColorButton->setText(QString());
        labelColorButton->setText(QString());
        groupBoxTelescopeServers->setTitle(QCoreApplication::translate("telescopeDialogForm", "Files", nullptr));
        checkBoxEnableLogs->setText(QCoreApplication::translate("telescopeDialogForm", "Log telescope driver messages to files", nullptr));
        checkBoxUseExecutables->setText(QCoreApplication::translate("telescopeDialogForm", "Use telescope server executables (override built-in drivers)", nullptr));
        labelExecutablesDirectory->setText(QCoreApplication::translate("telescopeDialogForm", "Executables directory:", nullptr));
        tabs->setTabText(tabs->indexOf(tabOptions), QCoreApplication::translate("telescopeDialogForm", "Options", nullptr));
        tabs->setTabText(tabs->indexOf(tabHelp), QCoreApplication::translate("telescopeDialogForm", "Help", nullptr));
        tabs->setTabText(tabs->indexOf(tabAbout), QCoreApplication::translate("telescopeDialogForm", "About", "tab in plugin windows"));
        (void)telescopeDialogForm;
    } // retranslateUi

};

namespace Ui {
    class telescopeDialogForm: public Ui_telescopeDialogForm {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_TELESCOPEDIALOG_H
