/****************************************************************************
** Meta object code from reading C++ file 'CCD.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../stellarium/plugins/Oculars/src/CCD.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'CCD.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3CCDE_t {};
} // unnamed namespace

template <> constexpr inline auto CCD::qt_create_metaobjectdata<qt_meta_tag_ZN3CCDE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "CCD",
        "",
        "QObject",
        "other",
        "name",
        "resolutionX",
        "resolutionY",
        "chipWidth",
        "chipHeight",
        "chipRotAngle",
        "binningX",
        "binningY",
        "hasOAG",
        "prismHeight",
        "prismWidth",
        "prismDistance",
        "prismPosAngle"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'name'
        QtMocHelpers::PropertyData<QString>(4, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'resolutionX'
        QtMocHelpers::PropertyData<int>(5, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'resolutionY'
        QtMocHelpers::PropertyData<int>(6, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'chipWidth'
        QtMocHelpers::PropertyData<double>(7, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'chipHeight'
        QtMocHelpers::PropertyData<double>(8, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'chipRotAngle'
        QtMocHelpers::PropertyData<double>(9, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'binningX'
        QtMocHelpers::PropertyData<int>(10, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'binningY'
        QtMocHelpers::PropertyData<int>(11, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'hasOAG'
        QtMocHelpers::PropertyData<double>(12, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'prismHeight'
        QtMocHelpers::PropertyData<double>(13, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'prismWidth'
        QtMocHelpers::PropertyData<double>(14, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'prismDistance'
        QtMocHelpers::PropertyData<double>(15, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'prismPosAngle'
        QtMocHelpers::PropertyData<double>(16, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
    };
    QtMocHelpers::UintData qt_enums {
    };
    using Constructor = QtMocHelpers::NoType;
    QtMocHelpers::UintData qt_constructors {
        QtMocHelpers::ConstructorData<Constructor(const QObject &)>(1, QMC::AccessPublic, {{
            { 0x80000000 | 2, 3 },
        }}),
    };
    return QtMocHelpers::metaObjectData<CCD, qt_meta_tag_ZN3CCDE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors);
}
Q_CONSTINIT const QMetaObject CCD::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3CCDE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3CCDE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3CCDE_t>.metaTypes,
    nullptr
} };

void CCD::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CCD *>(_o);
    if (_c == QMetaObject::CreateInstance) {
        switch (_id) {
        case 0: { CCD *_r = new CCD((*reinterpret_cast<std::add_pointer_t<QObject>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QObject**>(_a[0]) = _r; } break;
        default: break;
        }
    }
    if (_c == QMetaObject::ConstructInPlace) {
        switch (_id) {
        case 0: { new (_a[0]) CCD((*reinterpret_cast<std::add_pointer_t<QObject>>(_a[1]))); } break;
        default: break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->name(); break;
        case 1: *reinterpret_cast<int*>(_v) = _t->resolutionX(); break;
        case 2: *reinterpret_cast<int*>(_v) = _t->resolutionY(); break;
        case 3: *reinterpret_cast<double*>(_v) = _t->chipWidth(); break;
        case 4: *reinterpret_cast<double*>(_v) = _t->chipHeight(); break;
        case 5: *reinterpret_cast<double*>(_v) = _t->chipRotAngle(); break;
        case 6: *reinterpret_cast<int*>(_v) = _t->binningX(); break;
        case 7: *reinterpret_cast<int*>(_v) = _t->binningY(); break;
        case 8: *reinterpret_cast<double*>(_v) = _t->hasOAG(); break;
        case 9: *reinterpret_cast<double*>(_v) = _t->prismHeight(); break;
        case 10: *reinterpret_cast<double*>(_v) = _t->prismWidth(); break;
        case 11: *reinterpret_cast<double*>(_v) = _t->prismDistance(); break;
        case 12: *reinterpret_cast<double*>(_v) = _t->prismPosAngle(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setName(*reinterpret_cast<QString*>(_v)); break;
        case 1: _t->setResolutionX(*reinterpret_cast<int*>(_v)); break;
        case 2: _t->setResolutionY(*reinterpret_cast<int*>(_v)); break;
        case 3: _t->setChipWidth(*reinterpret_cast<double*>(_v)); break;
        case 4: _t->setChipHeight(*reinterpret_cast<double*>(_v)); break;
        case 5: _t->setChipRotAngle(*reinterpret_cast<double*>(_v)); break;
        case 6: _t->setBinningX(*reinterpret_cast<int*>(_v)); break;
        case 7: _t->setBinningY(*reinterpret_cast<int*>(_v)); break;
        case 8: _t->setHasOAG(*reinterpret_cast<double*>(_v)); break;
        case 9: _t->setPrismHeight(*reinterpret_cast<double*>(_v)); break;
        case 10: _t->setPrismWidth(*reinterpret_cast<double*>(_v)); break;
        case 11: _t->setPrismDistance(*reinterpret_cast<double*>(_v)); break;
        case 12: _t->setPrismPosAngle(*reinterpret_cast<double*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *CCD::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CCD::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3CCDE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int CCD::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    return _id;
}
QT_WARNING_POP
