/********************************************************************************
** Form generated from reading UI file 'scmSkyCultureDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SCMSKYCULTUREDIALOG_H
#define UI_SCMSKYCULTUREDIALOG_H

#include <Dialog.hpp>
#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QScrollArea>
#include <QtWidgets/QSlider>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTextEdit>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QTreeWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "ScmAdvancedSpinBox.hpp"
#include "ScmGeoLocGraphicsView.hpp"
#include "ScmMultiselectionComboBox.hpp"

QT_BEGIN_NAMESPACE

class Ui_scmSkyCultureDialog
{
public:
    QVBoxLayout *verticalLayout;
    TitleBar *titleBar;
    QFrame *Content;
    QVBoxLayout *verticalLayout_3;
    QTabWidget *tabs;
    QWidget *overviewTab;
    QVBoxLayout *verticalLayout_2;
    QLabel *currentSCNameLbl;
    QLineEdit *skyCultureNameLE;
    QLabel *constellationsLbl;
    QListWidget *constellationsList;
    QGridLayout *overviewConstellationBtnLayout;
    QPushButton *AddConstellationBtn;
    QPushButton *AddDarkConstellationBtn;
    QPushButton *EditConstellationBtn;
    QPushButton *RemoveConstellationBtn;
    QVBoxLayout *overviewSCBtnLayout;
    QLabel *selectLicenseLbl;
    QComboBox *licenseCB;
    QPushButton *ExportSkyCultureBtn;
    QWidget *descriptionTab;
    QVBoxLayout *descriptionTabLayout;
    QScrollArea *descriptionScrollArea;
    QWidget *descriptionScrollContent;
    QVBoxLayout *descriptionLayout;
    QLabel *descriptionTabLbl;
    QLabel *descriptionTabExplanationLbl;
    QGroupBox *generalInfoGroup;
    QVBoxLayout *generalInfoLayout;
    QLabel *classificationLbl;
    QComboBox *classificationCB;
    QLabel *introLbl;
    QTextEdit *introTE;
    QLabel *regionLabel;
    QComboBox *regionCB;
    QGroupBox *celestialObjectsGroup;
    QVBoxLayout *celestialObjectsLayout;
    QLabel *cultureDescriptionLbl;
    QTextEdit *cultureDescriptionTE;
    QHBoxLayout *hboxLayout;
    QLabel *skyLbl;
    QTextEdit *skyTE;
    QLabel *moonSunLbl;
    QTextEdit *moonSunTE;
    QLabel *planetsLbl;
    QTextEdit *planetsTE;
    QLabel *zodiacLbl;
    QTextEdit *zodiacTE;
    QLabel *milkyWayLbl;
    QTextEdit *milkyWayTE;
    QLabel *otherObjectsLbl;
    QTextEdit *otherObjectsTE;
    QGroupBox *creditsGroup;
    QVBoxLayout *creditsLayout;
    QLabel *referencesLbl;
    QGridLayout *referencesLayout;
    QPushButton *addRefBtn;
    QPushButton *moveRefUpBtn;
    QPushButton *moveRefDownBtn;
    QPushButton *removeRefBtn;
    QTreeWidget *referencesList;
    QLabel *authorsLbl;
    QTextEdit *authorsTE;
    QLabel *aboutLbl;
    QTextEdit *aboutTE;
    QLabel *acknowledgementsLbl;
    QTextEdit *acknowledgementsTE;
    QSpacerItem *verticalSpacer_2;
    QWidget *geoLocTab;
    QVBoxLayout *geoLocVerticalLayout;
    QWidget *mapPage;
    QGridLayout *mapGridLayout;
    QPushButton *helpLabel;
    ScmGeoLocGraphicsView *scmGeoLocGraphicsView;
    QWidget *mapForegroundWidget;
    QFrame *addPolygonFrame;
    QGridLayout *addPolygonGridLayout;
    QWidget *addPolygonBackgroundWidget;
    QLabel *beginTimeLabel;
    ScmAdvancedSpinBox *beginTimeSpinBox;
    QLabel *endTimeLabel;
    ScmAdvancedSpinBox *endTimeSpinBox;
    QDialogButtonBox *addPolygonDialogButtonBox;
    QGroupBox *skyCultureTimeOptionsGroupBox;
    QGridLayout *skyCultureTimeOptionsGridLayout;
    QGridLayout *skyCultureTimeSliderGridLayout;
    QLabel *skyCultureCurrentTimeLabel;
    QSlider *skyCultureTimeSlider;
    QSpinBox *skyCultureCurrentTimeSpinBox;
    QGridLayout *polygonInfoGridLayout;
    QLabel *polygonCountLabel;
    QLabel *polygonCountValueLabel;
    QLabel *cultureBeginTimeLabel;
    QLabel *cultureBeginTimeValueLabel;
    QLabel *cultureEndTimeLabel;
    QLabel *cultureEndTimeValueLabel;
    QPushButton *removePolygonButton;
    QTreeWidget *polygonInfoTreeWidget;
    QToolButton *mapHelpToolButton;

    void setupUi(QWidget *scmSkyCultureDialog)
    {
        if (scmSkyCultureDialog->objectName().isEmpty())
            scmSkyCultureDialog->setObjectName("scmSkyCultureDialog");
        scmSkyCultureDialog->resize(846, 700);
        scmSkyCultureDialog->setStyleSheet(QString::fromUtf8(""));
        verticalLayout = new QVBoxLayout(scmSkyCultureDialog);
        verticalLayout->setSpacing(0);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        titleBar = new TitleBar(scmSkyCultureDialog);
        titleBar->setObjectName("titleBar");

        verticalLayout->addWidget(titleBar);

        Content = new QFrame(scmSkyCultureDialog);
        Content->setObjectName("Content");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(Content->sizePolicy().hasHeightForWidth());
        Content->setSizePolicy(sizePolicy);
        Content->setFrameShape(QFrame::NoFrame);
        verticalLayout_3 = new QVBoxLayout(Content);
        verticalLayout_3->setSpacing(0);
        verticalLayout_3->setObjectName("verticalLayout_3");
        verticalLayout_3->setContentsMargins(0, 0, 0, 0);
        tabs = new QTabWidget(Content);
        tabs->setObjectName("tabs");
        tabs->setIconSize(QSize(32, 32));
        overviewTab = new QWidget();
        overviewTab->setObjectName("overviewTab");
        verticalLayout_2 = new QVBoxLayout(overviewTab);
        verticalLayout_2->setObjectName("verticalLayout_2");
        currentSCNameLbl = new QLabel(overviewTab);
        currentSCNameLbl->setObjectName("currentSCNameLbl");

        verticalLayout_2->addWidget(currentSCNameLbl);

        skyCultureNameLE = new QLineEdit(overviewTab);
        skyCultureNameLE->setObjectName("skyCultureNameLE");

        verticalLayout_2->addWidget(skyCultureNameLE);

        constellationsLbl = new QLabel(overviewTab);
        constellationsLbl->setObjectName("constellationsLbl");

        verticalLayout_2->addWidget(constellationsLbl);

        constellationsList = new QListWidget(overviewTab);
        constellationsList->setObjectName("constellationsList");
        constellationsList->setStyleSheet(QString::fromUtf8("\n"
"              QListWidget, QListWidget::viewport {\n"
"                  margin: 0px;\n"
"                  padding: 0px;\n"
"                  border: none;\n"
"                  outline: none;\n"
"              }\n"
"              QListWidget::item {\n"
"                  margin: 0px;\n"
"                  padding: 4px;\n"
"                  border: none;\n"
"                  outline: none;\n"
"              }\n"
"             "));

        verticalLayout_2->addWidget(constellationsList);

        overviewConstellationBtnLayout = new QGridLayout();
        overviewConstellationBtnLayout->setObjectName("overviewConstellationBtnLayout");
        AddConstellationBtn = new QPushButton(overviewTab);
        AddConstellationBtn->setObjectName("AddConstellationBtn");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(AddConstellationBtn->sizePolicy().hasHeightForWidth());
        AddConstellationBtn->setSizePolicy(sizePolicy1);

        overviewConstellationBtnLayout->addWidget(AddConstellationBtn, 0, 0, 1, 1);

        AddDarkConstellationBtn = new QPushButton(overviewTab);
        AddDarkConstellationBtn->setObjectName("AddDarkConstellationBtn");
        sizePolicy1.setHeightForWidth(AddDarkConstellationBtn->sizePolicy().hasHeightForWidth());
        AddDarkConstellationBtn->setSizePolicy(sizePolicy1);

        overviewConstellationBtnLayout->addWidget(AddDarkConstellationBtn, 0, 1, 1, 1);

        EditConstellationBtn = new QPushButton(overviewTab);
        EditConstellationBtn->setObjectName("EditConstellationBtn");
        sizePolicy1.setHeightForWidth(EditConstellationBtn->sizePolicy().hasHeightForWidth());
        EditConstellationBtn->setSizePolicy(sizePolicy1);

        overviewConstellationBtnLayout->addWidget(EditConstellationBtn, 1, 0, 1, 1);

        RemoveConstellationBtn = new QPushButton(overviewTab);
        RemoveConstellationBtn->setObjectName("RemoveConstellationBtn");
        sizePolicy1.setHeightForWidth(RemoveConstellationBtn->sizePolicy().hasHeightForWidth());
        RemoveConstellationBtn->setSizePolicy(sizePolicy1);

        overviewConstellationBtnLayout->addWidget(RemoveConstellationBtn, 1, 1, 1, 1);


        verticalLayout_2->addLayout(overviewConstellationBtnLayout);

        overviewSCBtnLayout = new QVBoxLayout();
        overviewSCBtnLayout->setObjectName("overviewSCBtnLayout");
        selectLicenseLbl = new QLabel(overviewTab);
        selectLicenseLbl->setObjectName("selectLicenseLbl");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(selectLicenseLbl->sizePolicy().hasHeightForWidth());
        selectLicenseLbl->setSizePolicy(sizePolicy2);

        overviewSCBtnLayout->addWidget(selectLicenseLbl);

        licenseCB = new QComboBox(overviewTab);
        licenseCB->setObjectName("licenseCB");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(licenseCB->sizePolicy().hasHeightForWidth());
        licenseCB->setSizePolicy(sizePolicy3);

        overviewSCBtnLayout->addWidget(licenseCB);

        ExportSkyCultureBtn = new QPushButton(overviewTab);
        ExportSkyCultureBtn->setObjectName("ExportSkyCultureBtn");

        overviewSCBtnLayout->addWidget(ExportSkyCultureBtn);


        verticalLayout_2->addLayout(overviewSCBtnLayout);

        tabs->addTab(overviewTab, QString());
        descriptionTab = new QWidget();
        descriptionTab->setObjectName("descriptionTab");
        descriptionTabLayout = new QVBoxLayout(descriptionTab);
        descriptionTabLayout->setSpacing(6);
        descriptionTabLayout->setObjectName("descriptionTabLayout");
        descriptionTabLayout->setContentsMargins(6, 6, 6, 6);
        descriptionScrollArea = new QScrollArea(descriptionTab);
        descriptionScrollArea->setObjectName("descriptionScrollArea");
        descriptionScrollArea->setWidgetResizable(true);
        descriptionScrollContent = new QWidget();
        descriptionScrollContent->setObjectName("descriptionScrollContent");
        descriptionScrollContent->setGeometry(QRect(0, -1481, 800, 2342));
        descriptionLayout = new QVBoxLayout(descriptionScrollContent);
        descriptionLayout->setSpacing(12);
        descriptionLayout->setObjectName("descriptionLayout");
        descriptionLayout->setContentsMargins(10, 10, 10, 10);
        descriptionTabLbl = new QLabel(descriptionScrollContent);
        descriptionTabLbl->setObjectName("descriptionTabLbl");
        descriptionTabLbl->setWordWrap(true);

        descriptionLayout->addWidget(descriptionTabLbl);

        descriptionTabExplanationLbl = new QLabel(descriptionScrollContent);
        descriptionTabExplanationLbl->setObjectName("descriptionTabExplanationLbl");
        descriptionTabExplanationLbl->setWordWrap(true);

        descriptionLayout->addWidget(descriptionTabExplanationLbl);

        generalInfoGroup = new QGroupBox(descriptionScrollContent);
        generalInfoGroup->setObjectName("generalInfoGroup");
        generalInfoGroup->setFlat(true);
        generalInfoLayout = new QVBoxLayout(generalInfoGroup);
        generalInfoLayout->setObjectName("generalInfoLayout");
        generalInfoLayout->setContentsMargins(0, 0, 0, 0);
        classificationLbl = new QLabel(generalInfoGroup);
        classificationLbl->setObjectName("classificationLbl");

        generalInfoLayout->addWidget(classificationLbl);

        classificationCB = new QComboBox(generalInfoGroup);
        classificationCB->setObjectName("classificationCB");

        generalInfoLayout->addWidget(classificationCB);

        introLbl = new QLabel(generalInfoGroup);
        introLbl->setObjectName("introLbl");

        generalInfoLayout->addWidget(introLbl);

        introTE = new QTextEdit(generalInfoGroup);
        introTE->setObjectName("introTE");
        introTE->setSizeAdjustPolicy(QAbstractScrollArea::AdjustToContents);

        generalInfoLayout->addWidget(introTE);

        regionLabel = new QLabel(generalInfoGroup);
        regionLabel->setObjectName("regionLabel");

        generalInfoLayout->addWidget(regionLabel);

        regionCB = new QComboBox(generalInfoGroup);
        regionCB->setObjectName("regionCB");

        generalInfoLayout->addWidget(regionCB);


        descriptionLayout->addWidget(generalInfoGroup);

        celestialObjectsGroup = new QGroupBox(descriptionScrollContent);
        celestialObjectsGroup->setObjectName("celestialObjectsGroup");
        celestialObjectsGroup->setFlat(true);
        celestialObjectsLayout = new QVBoxLayout(celestialObjectsGroup);
        celestialObjectsLayout->setObjectName("celestialObjectsLayout");
        celestialObjectsLayout->setContentsMargins(0, 0, 0, 0);
        cultureDescriptionLbl = new QLabel(celestialObjectsGroup);
        cultureDescriptionLbl->setObjectName("cultureDescriptionLbl");

        celestialObjectsLayout->addWidget(cultureDescriptionLbl);

        cultureDescriptionTE = new QTextEdit(celestialObjectsGroup);
        cultureDescriptionTE->setObjectName("cultureDescriptionTE");
        cultureDescriptionTE->setSizeAdjustPolicy(QAbstractScrollArea::AdjustToContents);
        cultureDescriptionTE->setAcceptRichText(false);

        celestialObjectsLayout->addWidget(cultureDescriptionTE);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setObjectName("hboxLayout");
        skyLbl = new QLabel(celestialObjectsGroup);
        skyLbl->setObjectName("skyLbl");

        hboxLayout->addWidget(skyLbl);


        celestialObjectsLayout->addLayout(hboxLayout);

        skyTE = new QTextEdit(celestialObjectsGroup);
        skyTE->setObjectName("skyTE");
        skyTE->setSizeAdjustPolicy(QAbstractScrollArea::AdjustToContents);
        skyTE->setAcceptRichText(false);

        celestialObjectsLayout->addWidget(skyTE);

        moonSunLbl = new QLabel(celestialObjectsGroup);
        moonSunLbl->setObjectName("moonSunLbl");

        celestialObjectsLayout->addWidget(moonSunLbl);

        moonSunTE = new QTextEdit(celestialObjectsGroup);
        moonSunTE->setObjectName("moonSunTE");
        moonSunTE->setSizeAdjustPolicy(QAbstractScrollArea::AdjustToContents);
        moonSunTE->setAcceptRichText(false);

        celestialObjectsLayout->addWidget(moonSunTE);

        planetsLbl = new QLabel(celestialObjectsGroup);
        planetsLbl->setObjectName("planetsLbl");

        celestialObjectsLayout->addWidget(planetsLbl);

        planetsTE = new QTextEdit(celestialObjectsGroup);
        planetsTE->setObjectName("planetsTE");
        planetsTE->setSizeAdjustPolicy(QAbstractScrollArea::AdjustToContents);
        planetsTE->setAcceptRichText(false);

        celestialObjectsLayout->addWidget(planetsTE);

        zodiacLbl = new QLabel(celestialObjectsGroup);
        zodiacLbl->setObjectName("zodiacLbl");

        celestialObjectsLayout->addWidget(zodiacLbl);

        zodiacTE = new QTextEdit(celestialObjectsGroup);
        zodiacTE->setObjectName("zodiacTE");
        zodiacTE->setSizeAdjustPolicy(QAbstractScrollArea::AdjustToContents);
        zodiacTE->setAcceptRichText(false);

        celestialObjectsLayout->addWidget(zodiacTE);

        milkyWayLbl = new QLabel(celestialObjectsGroup);
        milkyWayLbl->setObjectName("milkyWayLbl");

        celestialObjectsLayout->addWidget(milkyWayLbl);

        milkyWayTE = new QTextEdit(celestialObjectsGroup);
        milkyWayTE->setObjectName("milkyWayTE");
        milkyWayTE->setSizeAdjustPolicy(QAbstractScrollArea::AdjustToContents);
        milkyWayTE->setAcceptRichText(false);

        celestialObjectsLayout->addWidget(milkyWayTE);

        otherObjectsLbl = new QLabel(celestialObjectsGroup);
        otherObjectsLbl->setObjectName("otherObjectsLbl");

        celestialObjectsLayout->addWidget(otherObjectsLbl);

        otherObjectsTE = new QTextEdit(celestialObjectsGroup);
        otherObjectsTE->setObjectName("otherObjectsTE");
        otherObjectsTE->setSizeAdjustPolicy(QAbstractScrollArea::AdjustToContents);
        otherObjectsTE->setAcceptRichText(false);

        celestialObjectsLayout->addWidget(otherObjectsTE);


        descriptionLayout->addWidget(celestialObjectsGroup);

        creditsGroup = new QGroupBox(descriptionScrollContent);
        creditsGroup->setObjectName("creditsGroup");
        creditsGroup->setFlat(true);
        creditsLayout = new QVBoxLayout(creditsGroup);
        creditsLayout->setObjectName("creditsLayout");
        creditsLayout->setContentsMargins(0, 0, 0, 0);
        referencesLbl = new QLabel(creditsGroup);
        referencesLbl->setObjectName("referencesLbl");

        creditsLayout->addWidget(referencesLbl);

        referencesLayout = new QGridLayout();
        referencesLayout->setObjectName("referencesLayout");
        addRefBtn = new QPushButton(creditsGroup);
        addRefBtn->setObjectName("addRefBtn");

        referencesLayout->addWidget(addRefBtn, 2, 0, 1, 1);

        moveRefUpBtn = new QPushButton(creditsGroup);
        moveRefUpBtn->setObjectName("moveRefUpBtn");
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/graphicGui/uibtUp.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        icon.addFile(QString::fromUtf8(":/graphicGui/uibtUp-disabled.png"), QSize(), QIcon::Mode::Disabled, QIcon::State::Off);
        moveRefUpBtn->setIcon(icon);

        referencesLayout->addWidget(moveRefUpBtn, 2, 1, 1, 1);

        moveRefDownBtn = new QPushButton(creditsGroup);
        moveRefDownBtn->setObjectName("moveRefDownBtn");
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/graphicGui/uibtDown.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        icon1.addFile(QString::fromUtf8(":/graphicGui/uibtDown-disabled.png"), QSize(), QIcon::Mode::Disabled, QIcon::State::Off);
        moveRefDownBtn->setIcon(icon1);

        referencesLayout->addWidget(moveRefDownBtn, 2, 2, 1, 1);

        removeRefBtn = new QPushButton(creditsGroup);
        removeRefBtn->setObjectName("removeRefBtn");

        referencesLayout->addWidget(removeRefBtn, 2, 3, 1, 1);

        referencesList = new QTreeWidget(creditsGroup);
        referencesList->headerItem()->setText(0, QString());
        referencesList->headerItem()->setText(1, QString());
        referencesList->setObjectName("referencesList");
        referencesList->setRootIsDecorated(false);
        referencesList->setUniformRowHeights(true);
        referencesList->setHeaderHidden(true);

        referencesLayout->addWidget(referencesList, 0, 0, 1, 4);


        creditsLayout->addLayout(referencesLayout);

        authorsLbl = new QLabel(creditsGroup);
        authorsLbl->setObjectName("authorsLbl");

        creditsLayout->addWidget(authorsLbl);

        authorsTE = new QTextEdit(creditsGroup);
        authorsTE->setObjectName("authorsTE");
        authorsTE->setSizeAdjustPolicy(QAbstractScrollArea::AdjustToContents);
        authorsTE->setAcceptRichText(false);

        creditsLayout->addWidget(authorsTE);

        aboutLbl = new QLabel(creditsGroup);
        aboutLbl->setObjectName("aboutLbl");

        creditsLayout->addWidget(aboutLbl);

        aboutTE = new QTextEdit(creditsGroup);
        aboutTE->setObjectName("aboutTE");
        aboutTE->setSizeAdjustPolicy(QAbstractScrollArea::AdjustToContents);
        aboutTE->setAcceptRichText(false);

        creditsLayout->addWidget(aboutTE);

        acknowledgementsLbl = new QLabel(creditsGroup);
        acknowledgementsLbl->setObjectName("acknowledgementsLbl");

        creditsLayout->addWidget(acknowledgementsLbl);

        acknowledgementsTE = new QTextEdit(creditsGroup);
        acknowledgementsTE->setObjectName("acknowledgementsTE");
        acknowledgementsTE->setSizeAdjustPolicy(QAbstractScrollArea::AdjustToContents);
        acknowledgementsTE->setAcceptRichText(false);

        creditsLayout->addWidget(acknowledgementsTE);


        descriptionLayout->addWidget(creditsGroup);

        verticalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        descriptionLayout->addItem(verticalSpacer_2);

        descriptionScrollArea->setWidget(descriptionScrollContent);

        descriptionTabLayout->addWidget(descriptionScrollArea);

        tabs->addTab(descriptionTab, QString());
        geoLocTab = new QWidget();
        geoLocTab->setObjectName("geoLocTab");
        geoLocVerticalLayout = new QVBoxLayout(geoLocTab);
        geoLocVerticalLayout->setObjectName("geoLocVerticalLayout");
        mapPage = new QWidget(geoLocTab);
        mapPage->setObjectName("mapPage");
        mapGridLayout = new QGridLayout(mapPage);
        mapGridLayout->setObjectName("mapGridLayout");
        mapGridLayout->setContentsMargins(0, 0, 0, 0);
        helpLabel = new QPushButton(mapPage);
        helpLabel->setObjectName("helpLabel");
        helpLabel->setStyleSheet(QString::fromUtf8("\n"
"                  border: 1px solid black;\n"
"                  margin: 2px;\n"
"                  padding: 10px;\n"
"                  background-color: rgb(143, 143, 143);\n"
"                  text-align: left;\n"
"                "));

        mapGridLayout->addWidget(helpLabel, 2, 2, 1, 1, Qt::AlignRight|Qt::AlignBottom);

        scmGeoLocGraphicsView = new ScmGeoLocGraphicsView(mapPage);
        scmGeoLocGraphicsView->setObjectName("scmGeoLocGraphicsView");

        mapGridLayout->addWidget(scmGeoLocGraphicsView, 0, 0, 3, 3);

        mapForegroundWidget = new QWidget(mapPage);
        mapForegroundWidget->setObjectName("mapForegroundWidget");
        sizePolicy.setHeightForWidth(mapForegroundWidget->sizePolicy().hasHeightForWidth());
        mapForegroundWidget->setSizePolicy(sizePolicy);
        mapForegroundWidget->setStyleSheet(QString::fromUtf8("\n"
"                   background-color: rgba(0, 0, 0, 180);\n"
"                "));

        mapGridLayout->addWidget(mapForegroundWidget, 0, 0, 3, 3);

        addPolygonFrame = new QFrame(mapPage);
        addPolygonFrame->setObjectName("addPolygonFrame");
        QSizePolicy sizePolicy4(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy4.setHorizontalStretch(0);
        sizePolicy4.setVerticalStretch(0);
        sizePolicy4.setHeightForWidth(addPolygonFrame->sizePolicy().hasHeightForWidth());
        addPolygonFrame->setSizePolicy(sizePolicy4);
        addPolygonFrame->setFrameShape(QFrame::Shape::StyledPanel);
        addPolygonGridLayout = new QGridLayout(addPolygonFrame);
        addPolygonGridLayout->setObjectName("addPolygonGridLayout");
        addPolygonGridLayout->setHorizontalSpacing(15);
        addPolygonGridLayout->setVerticalSpacing(15);
        addPolygonGridLayout->setContentsMargins(2, 2, 2, 2);
        addPolygonBackgroundWidget = new QWidget(addPolygonFrame);
        addPolygonBackgroundWidget->setObjectName("addPolygonBackgroundWidget");
        sizePolicy.setHeightForWidth(addPolygonBackgroundWidget->sizePolicy().hasHeightForWidth());
        addPolygonBackgroundWidget->setSizePolicy(sizePolicy);
        addPolygonBackgroundWidget->setStyleSheet(QString::fromUtf8("\n"
"                      background-color: rgb(100, 100, 100);\n"
"                      border: 1px solid rgb(0, 0, 0);\n"
"                      border-radius: 10px;\n"
"                   "));

        addPolygonGridLayout->addWidget(addPolygonBackgroundWidget, 0, 0, 5, 6);

        beginTimeLabel = new QLabel(addPolygonFrame);
        beginTimeLabel->setObjectName("beginTimeLabel");

        addPolygonGridLayout->addWidget(beginTimeLabel, 1, 1, 1, 1);

        beginTimeSpinBox = new ScmAdvancedSpinBox(addPolygonFrame);
        beginTimeSpinBox->setObjectName("beginTimeSpinBox");
        QSizePolicy sizePolicy5(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy5.setHorizontalStretch(0);
        sizePolicy5.setVerticalStretch(0);
        sizePolicy5.setHeightForWidth(beginTimeSpinBox->sizePolicy().hasHeightForWidth());
        beginTimeSpinBox->setSizePolicy(sizePolicy5);
        beginTimeSpinBox->setMinimumSize(QSize(100, 0));
        beginTimeSpinBox->setButtonSymbols(QAbstractSpinBox::NoButtons);
        beginTimeSpinBox->setMinimum(-2000);
        beginTimeSpinBox->setMaximum(2000);

        addPolygonGridLayout->addWidget(beginTimeSpinBox, 1, 2, 1, 1);

        endTimeLabel = new QLabel(addPolygonFrame);
        endTimeLabel->setObjectName("endTimeLabel");

        addPolygonGridLayout->addWidget(endTimeLabel, 2, 1, 1, 1);

        endTimeSpinBox = new ScmAdvancedSpinBox(addPolygonFrame);
        endTimeSpinBox->setObjectName("endTimeSpinBox");
        sizePolicy5.setHeightForWidth(endTimeSpinBox->sizePolicy().hasHeightForWidth());
        endTimeSpinBox->setSizePolicy(sizePolicy5);
        endTimeSpinBox->setMinimumSize(QSize(100, 0));
        endTimeSpinBox->setButtonSymbols(QAbstractSpinBox::NoButtons);
        endTimeSpinBox->setMinimum(-2000);
        endTimeSpinBox->setMaximum(2000);

        addPolygonGridLayout->addWidget(endTimeSpinBox, 2, 2, 1, 1);

        addPolygonDialogButtonBox = new QDialogButtonBox(addPolygonFrame);
        addPolygonDialogButtonBox->setObjectName("addPolygonDialogButtonBox");
        sizePolicy2.setHeightForWidth(addPolygonDialogButtonBox->sizePolicy().hasHeightForWidth());
        addPolygonDialogButtonBox->setSizePolicy(sizePolicy2);
        addPolygonDialogButtonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        addPolygonGridLayout->addWidget(addPolygonDialogButtonBox, 3, 1, 1, 2);


        mapGridLayout->addWidget(addPolygonFrame, 1, 1, 1, 1);


        geoLocVerticalLayout->addWidget(mapPage);

        skyCultureTimeOptionsGroupBox = new QGroupBox(geoLocTab);
        skyCultureTimeOptionsGroupBox->setObjectName("skyCultureTimeOptionsGroupBox");
        skyCultureTimeOptionsGroupBox->setEnabled(true);
        QSizePolicy sizePolicy6(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);
        sizePolicy6.setHorizontalStretch(0);
        sizePolicy6.setVerticalStretch(0);
        sizePolicy6.setHeightForWidth(skyCultureTimeOptionsGroupBox->sizePolicy().hasHeightForWidth());
        skyCultureTimeOptionsGroupBox->setSizePolicy(sizePolicy6);
        skyCultureTimeOptionsGridLayout = new QGridLayout(skyCultureTimeOptionsGroupBox);
        skyCultureTimeOptionsGridLayout->setObjectName("skyCultureTimeOptionsGridLayout");
        skyCultureTimeOptionsGridLayout->setVerticalSpacing(20);
        skyCultureTimeOptionsGridLayout->setContentsMargins(0, 0, 0, 0);
        skyCultureTimeSliderGridLayout = new QGridLayout();
        skyCultureTimeSliderGridLayout->setObjectName("skyCultureTimeSliderGridLayout");
        skyCultureTimeSliderGridLayout->setHorizontalSpacing(10);
        skyCultureCurrentTimeLabel = new QLabel(skyCultureTimeOptionsGroupBox);
        skyCultureCurrentTimeLabel->setObjectName("skyCultureCurrentTimeLabel");

        skyCultureTimeSliderGridLayout->addWidget(skyCultureCurrentTimeLabel, 0, 0, 1, 1);

        skyCultureTimeSlider = new QSlider(skyCultureTimeOptionsGroupBox);
        skyCultureTimeSlider->setObjectName("skyCultureTimeSlider");
        skyCultureTimeSlider->setMinimum(-2000);
        skyCultureTimeSlider->setMaximum(2000);
        skyCultureTimeSlider->setSingleStep(1);
        skyCultureTimeSlider->setOrientation(Qt::Horizontal);
        skyCultureTimeSlider->setTickPosition(QSlider::TicksBelow);
        skyCultureTimeSlider->setTickInterval(150);

        skyCultureTimeSliderGridLayout->addWidget(skyCultureTimeSlider, 0, 1, 1, 1);

        skyCultureCurrentTimeSpinBox = new QSpinBox(skyCultureTimeOptionsGroupBox);
        skyCultureCurrentTimeSpinBox->setObjectName("skyCultureCurrentTimeSpinBox");
        skyCultureCurrentTimeSpinBox->setAlignment(Qt::AlignCenter);
        skyCultureCurrentTimeSpinBox->setMinimum(-2000);
        skyCultureCurrentTimeSpinBox->setMaximum(2000);
        skyCultureCurrentTimeSpinBox->setStyleSheet(QString::fromUtf8("\n"
"                  QSpinBox\n"
"                  {\n"
"                      width: 65px;\n"
"                  }\n"
"                  "));

        skyCultureTimeSliderGridLayout->addWidget(skyCultureCurrentTimeSpinBox, 0, 2, 1, 1);


        skyCultureTimeOptionsGridLayout->addLayout(skyCultureTimeSliderGridLayout, 0, 0, 1, 2);

        polygonInfoGridLayout = new QGridLayout();
        polygonInfoGridLayout->setObjectName("polygonInfoGridLayout");
        polygonInfoGridLayout->setHorizontalSpacing(40);
        polygonInfoGridLayout->setVerticalSpacing(10);
        polygonInfoGridLayout->setContentsMargins(10, 10, 10, 0);
        polygonCountLabel = new QLabel(skyCultureTimeOptionsGroupBox);
        polygonCountLabel->setObjectName("polygonCountLabel");

        polygonInfoGridLayout->addWidget(polygonCountLabel, 0, 1, 1, 1);

        polygonCountValueLabel = new QLabel(skyCultureTimeOptionsGroupBox);
        polygonCountValueLabel->setObjectName("polygonCountValueLabel");
        polygonCountValueLabel->setAlignment(Qt::AlignCenter);

        polygonInfoGridLayout->addWidget(polygonCountValueLabel, 0, 2, 1, 1);

        cultureBeginTimeLabel = new QLabel(skyCultureTimeOptionsGroupBox);
        cultureBeginTimeLabel->setObjectName("cultureBeginTimeLabel");

        polygonInfoGridLayout->addWidget(cultureBeginTimeLabel, 1, 1, 1, 1);

        cultureBeginTimeValueLabel = new QLabel(skyCultureTimeOptionsGroupBox);
        cultureBeginTimeValueLabel->setObjectName("cultureBeginTimeValueLabel");
        cultureBeginTimeValueLabel->setAlignment(Qt::AlignCenter);

        polygonInfoGridLayout->addWidget(cultureBeginTimeValueLabel, 1, 2, 1, 1);

        cultureEndTimeLabel = new QLabel(skyCultureTimeOptionsGroupBox);
        cultureEndTimeLabel->setObjectName("cultureEndTimeLabel");

        polygonInfoGridLayout->addWidget(cultureEndTimeLabel, 2, 1, 1, 1);

        cultureEndTimeValueLabel = new QLabel(skyCultureTimeOptionsGroupBox);
        cultureEndTimeValueLabel->setObjectName("cultureEndTimeValueLabel");
        cultureEndTimeValueLabel->setAlignment(Qt::AlignCenter);

        polygonInfoGridLayout->addWidget(cultureEndTimeValueLabel, 2, 2, 1, 1);

        removePolygonButton = new QPushButton(skyCultureTimeOptionsGroupBox);
        removePolygonButton->setObjectName("removePolygonButton");
        sizePolicy1.setHeightForWidth(removePolygonButton->sizePolicy().hasHeightForWidth());
        removePolygonButton->setSizePolicy(sizePolicy1);

        polygonInfoGridLayout->addWidget(removePolygonButton, 5, 1, 1, 2);

        polygonInfoTreeWidget = new QTreeWidget(skyCultureTimeOptionsGroupBox);
        polygonInfoTreeWidget->setObjectName("polygonInfoTreeWidget");
        sizePolicy3.setHeightForWidth(polygonInfoTreeWidget->sizePolicy().hasHeightForWidth());
        polygonInfoTreeWidget->setSizePolicy(sizePolicy3);
        polygonInfoTreeWidget->setMinimumSize(QSize(400, 0));
        polygonInfoTreeWidget->setColumnCount(3);

        polygonInfoGridLayout->addWidget(polygonInfoTreeWidget, 0, 0, 6, 1);


        skyCultureTimeOptionsGridLayout->addLayout(polygonInfoGridLayout, 1, 0, 1, 1);

        mapHelpToolButton = new QToolButton(skyCultureTimeOptionsGroupBox);
        mapHelpToolButton->setObjectName("mapHelpToolButton");
        sizePolicy4.setHeightForWidth(mapHelpToolButton->sizePolicy().hasHeightForWidth());
        mapHelpToolButton->setSizePolicy(sizePolicy4);
        mapHelpToolButton->setFocusPolicy(Qt::NoFocus);
        QIcon icon2;
        icon2.addFile(QString::fromUtf8(":/graphicGui/tabHelp-selected.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        icon2.addFile(QString::fromUtf8(":/graphicGui/tabHelp-selected.png"), QSize(), QIcon::Mode::Disabled, QIcon::State::Off);
        mapHelpToolButton->setIcon(icon2);
        mapHelpToolButton->setStyleSheet(QString::fromUtf8("\n"
"                  background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 rgb(113, 114, 115), stop: 1 rgb(71, 73, 74));\n"
"                "));
        mapHelpToolButton->setCursor(QCursor(Qt::CursorShape::WhatsThisCursor));
        mapHelpToolButton->setCheckable(false);

        skyCultureTimeOptionsGridLayout->addWidget(mapHelpToolButton, 1, 1, 1, 1, Qt::AlignBottom);


        geoLocVerticalLayout->addWidget(skyCultureTimeOptionsGroupBox);

        tabs->addTab(geoLocTab, QString());

        verticalLayout_3->addWidget(tabs);


        verticalLayout->addWidget(Content);


        retranslateUi(scmSkyCultureDialog);

        tabs->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(scmSkyCultureDialog);
    } // setupUi

    void retranslateUi(QWidget *scmSkyCultureDialog)
    {
        titleBar->setProperty("title", QVariant(QCoreApplication::translate("scmSkyCultureDialog", "Sky Culture Maker", nullptr)));
        currentSCNameLbl->setText(QCoreApplication::translate("scmSkyCultureDialog", "Name of the Sky Culture", nullptr));
        constellationsLbl->setText(QCoreApplication::translate("scmSkyCultureDialog", "Constellations", nullptr));
        AddConstellationBtn->setText(QCoreApplication::translate("scmSkyCultureDialog", "Add Constellation", nullptr));
        AddDarkConstellationBtn->setText(QCoreApplication::translate("scmSkyCultureDialog", "Add Dark Constellation", nullptr));
        EditConstellationBtn->setText(QCoreApplication::translate("scmSkyCultureDialog", "Edit", nullptr));
        RemoveConstellationBtn->setText(QCoreApplication::translate("scmSkyCultureDialog", "Remove", nullptr));
        selectLicenseLbl->setText(QCoreApplication::translate("scmSkyCultureDialog", "License of the Sky Culture", nullptr));
        ExportSkyCultureBtn->setText(QCoreApplication::translate("scmSkyCultureDialog", "Export Sky Culture", nullptr));
        tabs->setTabText(tabs->indexOf(overviewTab), QCoreApplication::translate("scmSkyCultureDialog", "Overview", nullptr));
        descriptionTabLbl->setText(QCoreApplication::translate("scmSkyCultureDialog", "Please describe the Sky Culture", nullptr));
        descriptionTabExplanationLbl->setText(QCoreApplication::translate("scmSkyCultureDialog", "Mandatory fields are marked with an asterisk (*).", nullptr));
        generalInfoGroup->setTitle(QCoreApplication::translate("scmSkyCultureDialog", "General Information", nullptr));
        classificationLbl->setText(QCoreApplication::translate("scmSkyCultureDialog", "Classification of the Sky Culture*", nullptr));
        introLbl->setText(QCoreApplication::translate("scmSkyCultureDialog", "Introduction:*", nullptr));
        introTE->setPlaceholderText(QCoreApplication::translate("scmSkyCultureDialog", " A summary that could ideally be shown on a small screen entirely.", nullptr));
        regionLabel->setText(QCoreApplication::translate("scmSkyCultureDialog", "Region of the Sky Culture*", nullptr));
        celestialObjectsGroup->setTitle(QCoreApplication::translate("scmSkyCultureDialog", "Description (What can the user see here?)", nullptr));
        cultureDescriptionLbl->setText(QCoreApplication::translate("scmSkyCultureDialog", "Culture Description:*", nullptr));
        cultureDescriptionTE->setPlaceholderText(QCoreApplication::translate("scmSkyCultureDialog", "Give a short introduction about the culture: time, location, circumstances of life. Please describe specifics of the source (book/clay tablet/parchment/...) or of the culture: e.g. sun-focussed, moon-focussed, climate of the region, geographic specifics. Did members of the culture build pyramids or any other well-known thing? What did they do with astronomy (e.g. time keeping, calendar making, weather predictions...)? Are there connections to other cultures? Is this version a didactic reduction or is it derived from a specific other source?", nullptr));
        skyLbl->setText(QCoreApplication::translate("scmSkyCultureDialog", "Sky", nullptr));
        skyTE->setPlaceholderText(QCoreApplication::translate("scmSkyCultureDialog", "What special features did/does this culture use, what shall the user watch out for? Is it worthwhile to click a specific button rather than others - e.g. divisions of the ecliptic, super-constellations, dark constellations, all-sky-constellations, zoom into the Pleiades, etc.? Do the constellations change with the months or remain the same throughout the year? What is actually 'correct' about your presentation? Did they really use stick figures? Did they paint images (or just use names of stars)? Did they use stars at all or prefer dark/bright clouds of the Milky Way?", nullptr));
        moonSunLbl->setText(QCoreApplication::translate("scmSkyCultureDialog", "Moon & Sun:", nullptr));
        moonSunTE->setPlaceholderText(QCoreApplication::translate("scmSkyCultureDialog", "How did they determine their calendar? Did they observe phases?", nullptr));
        planetsLbl->setText(QCoreApplication::translate("scmSkyCultureDialog", "Planets:", nullptr));
        planetsTE->setPlaceholderText(QCoreApplication::translate("scmSkyCultureDialog", "Were the planets deified? What deities connect, what meaning do they have?", nullptr));
        zodiacLbl->setText(QCoreApplication::translate("scmSkyCultureDialog", "Zodiac / Lunar System:", nullptr));
        zodiacTE->setPlaceholderText(QCoreApplication::translate("scmSkyCultureDialog", "Did they have the (Bab.) zodiac system or any other division of the \"path of the Moon\"? If so, was it a (equal or unequal) division of the ecliptic or of any other type?", nullptr));
        milkyWayLbl->setText(QCoreApplication::translate("scmSkyCultureDialog", "Milky Way:", nullptr));
        milkyWayTE->setPlaceholderText(QCoreApplication::translate("scmSkyCultureDialog", "Was the Milky Way deified? What meaning did it have?", nullptr));
        otherObjectsLbl->setText(QCoreApplication::translate("scmSkyCultureDialog", "Other Celestial Objects:", nullptr));
        otherObjectsTE->setPlaceholderText(QCoreApplication::translate("scmSkyCultureDialog", "What other celestial objects were significant? E.g what about shooting stars, aurorae? Did they observe stellar transients, e.g. supernovae, that were visible only in a specific year?", nullptr));
        creditsGroup->setTitle(QCoreApplication::translate("scmSkyCultureDialog", "Credits and References", nullptr));
        referencesLbl->setText(QCoreApplication::translate("scmSkyCultureDialog", "References:*", nullptr));
        addRefBtn->setText(QCoreApplication::translate("scmSkyCultureDialog", "Add", nullptr));
        moveRefUpBtn->setText(QString());
        moveRefDownBtn->setText(QString());
        removeRefBtn->setText(QCoreApplication::translate("scmSkyCultureDialog", "Remove", nullptr));
        authorsLbl->setText(QCoreApplication::translate("scmSkyCultureDialog", "Authors:*", nullptr));
        authorsTE->setPlaceholderText(QCoreApplication::translate("scmSkyCultureDialog", "Who are you and what is your background (native person, researcher, ...)? Ideally, please provide an email-address as well.", nullptr));
        aboutLbl->setText(QCoreApplication::translate("scmSkyCultureDialog", "About the sky culture:*", nullptr));
        aboutTE->setPlaceholderText(QCoreApplication::translate("scmSkyCultureDialog", "How do you know about this Sky Culture? Is this contribution based on your own or other people's research? Is it based on interviews with Elders or reading ancient text, etc.?", nullptr));
        acknowledgementsLbl->setText(QCoreApplication::translate("scmSkyCultureDialog", "Acknowledgements:", nullptr));
        acknowledgementsTE->setPlaceholderText(QCoreApplication::translate("scmSkyCultureDialog", "What acknowledgements would you like to make? E.g. to interviewed people, organizations, or resources that helped you?", nullptr));
        tabs->setTabText(tabs->indexOf(descriptionTab), QCoreApplication::translate("scmSkyCultureDialog", "Description", nullptr));
        beginTimeLabel->setText(QCoreApplication::translate("scmSkyCultureDialog", "start Time", nullptr));
        endTimeLabel->setText(QCoreApplication::translate("scmSkyCultureDialog", "end Time", nullptr));
        skyCultureTimeOptionsGroupBox->setTitle(QCoreApplication::translate("scmSkyCultureDialog", "Options", nullptr));
        skyCultureCurrentTimeLabel->setText(QCoreApplication::translate("scmSkyCultureDialog", "Current time", nullptr));
        polygonCountLabel->setText(QCoreApplication::translate("scmSkyCultureDialog", "Number of digitized polygons:", nullptr));
        polygonCountValueLabel->setText(QCoreApplication::translate("scmSkyCultureDialog", "0", nullptr));
        cultureBeginTimeLabel->setText(QCoreApplication::translate("scmSkyCultureDialog", "Start time of the culture:", nullptr));
        cultureBeginTimeValueLabel->setText(QString());
        cultureEndTimeLabel->setText(QCoreApplication::translate("scmSkyCultureDialog", "End time of the culture:", nullptr));
        cultureEndTimeValueLabel->setText(QString());
        removePolygonButton->setText(QCoreApplication::translate("scmSkyCultureDialog", "Delete selected polygon", nullptr));
        QTreeWidgetItem *___qtreewidgetitem = polygonInfoTreeWidget->headerItem();
        ___qtreewidgetitem->setText(2, QCoreApplication::translate("scmSkyCultureDialog", "Vertex count", nullptr));
        ___qtreewidgetitem->setText(1, QCoreApplication::translate("scmSkyCultureDialog", "End time", nullptr));
        ___qtreewidgetitem->setText(0, QCoreApplication::translate("scmSkyCultureDialog", "Start time", nullptr));
        tabs->setTabText(tabs->indexOf(geoLocTab), QCoreApplication::translate("scmSkyCultureDialog", "Location", nullptr));
        (void)scmSkyCultureDialog;
    } // retranslateUi

};

namespace Ui {
    class scmSkyCultureDialog: public Ui_scmSkyCultureDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SCMSKYCULTUREDIALOG_H
