/********************************************************************************
** Form generated from reading UI file 'satellitesCommDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SATELLITESCOMMDIALOG_H
#define UI_SATELLITESCOMMDIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QTreeWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "Dialog.hpp"

QT_BEGIN_NAMESPACE

class Ui_satellitesCommDialog
{
public:
    QVBoxLayout *verticalLayout;
    TitleBar *titleBar;
    QGroupBox *groupBox;
    QGridLayout *gridLayout_2;
    QLabel *satelliteLabel;
    QSpacerItem *verticalSpacer;
    QTreeWidget *communicationsTreeWidget;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *horizontalSpacer;
    QToolButton *addCommLinkButton;
    QToolButton *removeCommLinkButton;
    QGridLayout *gridLayout;
    QLabel *label;
    QDoubleSpinBox *frequencySpinBox;
    QLineEdit *modulationLineEdit;
    QLineEdit *descriptionLineEdit;
    QFrame *line;

    void setupUi(QWidget *satellitesCommDialog)
    {
        if (satellitesCommDialog->objectName().isEmpty())
            satellitesCommDialog->setObjectName("satellitesCommDialog");
        satellitesCommDialog->resize(486, 253);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(satellitesCommDialog->sizePolicy().hasHeightForWidth());
        satellitesCommDialog->setSizePolicy(sizePolicy);
        verticalLayout = new QVBoxLayout(satellitesCommDialog);
        verticalLayout->setSpacing(0);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        titleBar = new TitleBar(satellitesCommDialog);
        titleBar->setObjectName("titleBar");

        verticalLayout->addWidget(titleBar);

        groupBox = new QGroupBox(satellitesCommDialog);
        groupBox->setObjectName("groupBox");
        gridLayout_2 = new QGridLayout(groupBox);
        gridLayout_2->setObjectName("gridLayout_2");
        gridLayout_2->setContentsMargins(0, 0, 0, 0);
        satelliteLabel = new QLabel(groupBox);
        satelliteLabel->setObjectName("satelliteLabel");
        satelliteLabel->setAlignment(Qt::AlignCenter);

        gridLayout_2->addWidget(satelliteLabel, 0, 0, 1, 1);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_2->addItem(verticalSpacer, 5, 0, 1, 1);

        communicationsTreeWidget = new QTreeWidget(groupBox);
        communicationsTreeWidget->setObjectName("communicationsTreeWidget");
        communicationsTreeWidget->setEditTriggers(QAbstractItemView::NoEditTriggers);
        communicationsTreeWidget->setRootIsDecorated(false);
        communicationsTreeWidget->setUniformRowHeights(true);
        communicationsTreeWidget->setItemsExpandable(false);
        communicationsTreeWidget->setSortingEnabled(true);
        communicationsTreeWidget->setExpandsOnDoubleClick(false);
        communicationsTreeWidget->setColumnCount(0);

        gridLayout_2->addWidget(communicationsTreeWidget, 1, 0, 1, 1);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        addCommLinkButton = new QToolButton(groupBox);
        addCommLinkButton->setObjectName("addCommLinkButton");
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/graphicGui/uibtAdd.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        icon.addFile(QString::fromUtf8(":/graphicGui/uibtAdd.png"), QSize(), QIcon::Mode::Disabled, QIcon::State::Off);
        addCommLinkButton->setIcon(icon);

        horizontalLayout->addWidget(addCommLinkButton);

        removeCommLinkButton = new QToolButton(groupBox);
        removeCommLinkButton->setObjectName("removeCommLinkButton");
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/graphicGui/uibtRemove.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        icon1.addFile(QString::fromUtf8(":/graphicGui/uibtRemove.png"), QSize(), QIcon::Mode::Disabled, QIcon::State::Off);
        removeCommLinkButton->setIcon(icon1);

        horizontalLayout->addWidget(removeCommLinkButton);


        gridLayout_2->addLayout(horizontalLayout, 4, 0, 1, 1);

        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        label = new QLabel(groupBox);
        label->setObjectName("label");

        gridLayout->addWidget(label, 0, 0, 1, 1);

        frequencySpinBox = new QDoubleSpinBox(groupBox);
        frequencySpinBox->setObjectName("frequencySpinBox");
        frequencySpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        frequencySpinBox->setDecimals(3);
        frequencySpinBox->setMaximum(100000.000000000000000);

        gridLayout->addWidget(frequencySpinBox, 1, 1, 1, 1);

        modulationLineEdit = new QLineEdit(groupBox);
        modulationLineEdit->setObjectName("modulationLineEdit");
        modulationLineEdit->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(modulationLineEdit, 1, 2, 1, 1);

        descriptionLineEdit = new QLineEdit(groupBox);
        descriptionLineEdit->setObjectName("descriptionLineEdit");

        gridLayout->addWidget(descriptionLineEdit, 0, 1, 1, 2);


        gridLayout_2->addLayout(gridLayout, 2, 0, 1, 1);

        line = new QFrame(groupBox);
        line->setObjectName("line");
        line->setMinimumSize(QSize(0, 1));
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        gridLayout_2->addWidget(line, 3, 0, 1, 1);


        verticalLayout->addWidget(groupBox);


        retranslateUi(satellitesCommDialog);

        QMetaObject::connectSlotsByName(satellitesCommDialog);
    } // setupUi

    void retranslateUi(QWidget *satellitesCommDialog)
    {
        titleBar->setProperty("title", QVariant(QCoreApplication::translate("satellitesCommDialog", "Frequencies", nullptr)));
        groupBox->setTitle(QCoreApplication::translate("satellitesCommDialog", "Communication data of selected satellite", nullptr));
        label->setText(QCoreApplication::translate("satellitesCommDialog", "Channel description:", nullptr));
        (void)satellitesCommDialog;
    } // retranslateUi

};

namespace Ui {
    class satellitesCommDialog: public Ui_satellitesCommDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SATELLITESCOMMDIALOG_H
