/****************************************************************************
** Meta object code from reading C++ file 'StelLocationMgr.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../stellarium/src/core/StelLocationMgr.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'StelLocationMgr.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15StelLocationMgrE_t {};
} // unnamed namespace

template <> constexpr inline auto StelLocationMgr::qt_create_metaobjectdata<qt_meta_tag_ZN15StelLocationMgrE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "StelLocationMgr",
        "locationListChanged",
        "",
        "gpsQueryFinished",
        "success",
        "locationForString",
        "StelLocation",
        "s",
        "locationFromIP",
        "getAllTimezoneNames",
        "getColorForCoordinates",
        "QColor",
        "lng",
        "lat",
        "locationFromGPS",
        "interval",
        "sanitizeTimezoneStringForLocationDB",
        "tzString",
        "sanitizeTimezoneStringFromLocationDB",
        "dbString",
        "changeLocationFromNetworkLookup",
        "changePlanetMapForLocation",
        "loc",
        "changeLocationFromGPSQuery",
        "gpsQueryError",
        "err",
        "positionUpdated",
        "QGeoPositionInfo",
        "gpsPos",
        "positionUpdatedFromOS",
        "info"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'locationListChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'gpsQueryFinished'
        QtMocHelpers::SignalData<void(bool)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 4 },
        }}),
        // Slot 'locationForString'
        QtMocHelpers::SlotData<const StelLocation(const QString &) const>(5, 2, QMC::AccessPublic, 0x80000000 | 6, {{
            { QMetaType::QString, 7 },
        }}),
        // Slot 'locationFromIP'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'getAllTimezoneNames'
        QtMocHelpers::SlotData<QStringList() const>(9, 2, QMC::AccessPublic, QMetaType::QStringList),
        // Slot 'getColorForCoordinates'
        QtMocHelpers::SlotData<QColor(const double, const double) const>(10, 2, QMC::AccessPublic, 0x80000000 | 11, {{
            { QMetaType::Double, 12 }, { QMetaType::Double, 13 },
        }}),
        // Slot 'locationFromGPS'
        QtMocHelpers::SlotData<void(int)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 15 },
        }}),
        // Slot 'locationFromGPS'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'sanitizeTimezoneStringForLocationDB'
        QtMocHelpers::SlotData<QString(QString)>(16, 2, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::QString, 17 },
        }}),
        // Slot 'sanitizeTimezoneStringFromLocationDB'
        QtMocHelpers::SlotData<QString(QString)>(18, 2, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::QString, 19 },
        }}),
        // Slot 'changeLocationFromNetworkLookup'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'changePlanetMapForLocation'
        QtMocHelpers::SlotData<void(StelLocation)>(21, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 6, 22 },
        }}),
        // Slot 'changeLocationFromGPSQuery'
        QtMocHelpers::SlotData<void(const StelLocation &)>(23, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 6, 22 },
        }}),
        // Slot 'gpsQueryError'
        QtMocHelpers::SlotData<void(const QString &)>(24, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 25 },
        }}),
        // Slot 'positionUpdated'
        QtMocHelpers::SlotData<void(QGeoPositionInfo)>(26, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 27, 28 },
        }}),
        // Slot 'positionUpdatedFromOS'
        QtMocHelpers::SlotData<void(const QGeoPositionInfo &)>(29, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 27, 30 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<StelLocationMgr, qt_meta_tag_ZN15StelLocationMgrE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject StelLocationMgr::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15StelLocationMgrE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15StelLocationMgrE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15StelLocationMgrE_t>.metaTypes,
    nullptr
} };

void StelLocationMgr::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<StelLocationMgr *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->locationListChanged(); break;
        case 1: _t->gpsQueryFinished((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: { StelLocation _r = _t->locationForString((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<StelLocation*>(_a[0]) = std::move(_r); }  break;
        case 3: _t->locationFromIP(); break;
        case 4: { QStringList _r = _t->getAllTimezoneNames();
            if (_a[0]) *reinterpret_cast<QStringList*>(_a[0]) = std::move(_r); }  break;
        case 5: { QColor _r = _t->getColorForCoordinates((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QColor*>(_a[0]) = std::move(_r); }  break;
        case 6: _t->locationFromGPS((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 7: _t->locationFromGPS(); break;
        case 8: { QString _r = _t->sanitizeTimezoneStringForLocationDB((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 9: { QString _r = _t->sanitizeTimezoneStringFromLocationDB((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 10: _t->changeLocationFromNetworkLookup(); break;
        case 11: _t->changePlanetMapForLocation((*reinterpret_cast<std::add_pointer_t<StelLocation>>(_a[1]))); break;
        case 12: _t->changeLocationFromGPSQuery((*reinterpret_cast<std::add_pointer_t<StelLocation>>(_a[1]))); break;
        case 13: _t->gpsQueryError((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 14: _t->positionUpdated((*reinterpret_cast<std::add_pointer_t<QGeoPositionInfo>>(_a[1]))); break;
        case 15: _t->positionUpdatedFromOS((*reinterpret_cast<std::add_pointer_t<QGeoPositionInfo>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< StelLocation >(); break;
            }
            break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< StelLocation >(); break;
            }
            break;
        case 14:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QGeoPositionInfo >(); break;
            }
            break;
        case 15:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QGeoPositionInfo >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (StelLocationMgr::*)()>(_a, &StelLocationMgr::locationListChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelLocationMgr::*)(bool )>(_a, &StelLocationMgr::gpsQueryFinished, 1))
            return;
    }
}

const QMetaObject *StelLocationMgr::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *StelLocationMgr::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15StelLocationMgrE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int StelLocationMgr::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    return _id;
}

// SIGNAL 0
void StelLocationMgr::locationListChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void StelLocationMgr::gpsQueryFinished(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}
QT_WARNING_POP
