/****************************************************************************
** Meta object code from reading C++ file 'StelApp.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../stellarium/src/core/StelApp.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'StelApp.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN7StelAppE_t {};
} // unnamed namespace

template <> constexpr inline auto StelApp::qt_create_metaobjectdata<qt_meta_tag_ZN7StelAppE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "StelApp",
        "visionNightModeChanged",
        "",
        "flagShowDecimalDegreesChanged",
        "flagUseAzimuthFromSouthChanged",
        "flagUseNegativeHourAnglesChanged",
        "flagUsePolarDistanceChanged",
        "flagUseCCSDesignationChanged",
        "flagUseFormattingOutputChanged",
        "flagOverwriteInfoColorChanged",
        "colorSchemeChanged",
        "languageChanged",
        "screenFontSizeChanged",
        "guiFontSizeChanged",
        "screenButtonScaleChanged",
        "fontChanged",
        "QFont",
        "overwriteInfoColorChanged",
        "Vec3f",
        "color",
        "daylightInfoColorChanged",
        "flagImmediateSaveChanged",
        "progressBarAdded",
        "const StelProgressController*",
        "progressBarRemoved",
        "aboutToQuit",
        "glWindowHasBeenResized",
        "QRectF",
        "rect",
        "glPhysicalWindowHasBeenResized",
        "setVisionModeNight",
        "getVisionModeNight",
        "setFlagOverwriteInfoColor",
        "getFlagOverwriteInfoColor",
        "setFlagShowDecimalDegrees",
        "b",
        "getFlagShowDecimalDegrees",
        "setFlagSouthAzimuthUsage",
        "use",
        "getFlagSouthAzimuthUsage",
        "setFlagUseNegativeHourAngles",
        "getFlagUseNegativeHourAngles",
        "getFlagPolarDistanceUsage",
        "setFlagPolarDistanceUsage",
        "setFlagUseFormattingOutput",
        "getFlagUseFormattingOutput",
        "setFlagUseCCSDesignation",
        "getFlagUseCCSDesignation",
        "setOverwriteInfoColor",
        "getOverwriteInfoColor",
        "setDaylightInfoColor",
        "getDaylightInfoColor",
        "setFlagImmediateSave",
        "getFlagImmediateSave",
        "getFps",
        "setAppFont",
        "font",
        "getDefaultFBO",
        "ensureGLContextCurrent",
        "getTotalRunTime",
        "getAnimationTime",
        "reportFileDownloadFinished",
        "QNetworkReply*",
        "reply",
        "quit",
        "nightMode",
        "flagShowDecimalDegrees",
        "flagUseAzimuthFromSouth",
        "flagUseNegativeHourAngles",
        "flagUsePolarDistance",
        "flagUseCCSDesignation",
        "flagUseFormattingOutput",
        "flagOverwriteInfoColor",
        "overwriteInfoColor",
        "daylightInfoColor",
        "screenFontSize",
        "guiFontSize",
        "screenButtonScale",
        "flagImmediateSave",
        "version"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'visionNightModeChanged'
        QtMocHelpers::SignalData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'flagShowDecimalDegreesChanged'
        QtMocHelpers::SignalData<void(bool)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'flagUseAzimuthFromSouthChanged'
        QtMocHelpers::SignalData<void(bool)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'flagUseNegativeHourAnglesChanged'
        QtMocHelpers::SignalData<void(bool)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'flagUsePolarDistanceChanged'
        QtMocHelpers::SignalData<void(bool)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'flagUseCCSDesignationChanged'
        QtMocHelpers::SignalData<void(bool)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'flagUseFormattingOutputChanged'
        QtMocHelpers::SignalData<void(bool)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'flagOverwriteInfoColorChanged'
        QtMocHelpers::SignalData<void(bool)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'colorSchemeChanged'
        QtMocHelpers::SignalData<void(const QString &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Signal 'languageChanged'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'screenFontSizeChanged'
        QtMocHelpers::SignalData<void(int)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Signal 'guiFontSizeChanged'
        QtMocHelpers::SignalData<void(int)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Signal 'screenButtonScaleChanged'
        QtMocHelpers::SignalData<void(double)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Signal 'fontChanged'
        QtMocHelpers::SignalData<void(const QFont &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 2 },
        }}),
        // Signal 'overwriteInfoColorChanged'
        QtMocHelpers::SignalData<void(const Vec3f &)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 18, 19 },
        }}),
        // Signal 'daylightInfoColorChanged'
        QtMocHelpers::SignalData<void(const Vec3f &)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 18, 19 },
        }}),
        // Signal 'flagImmediateSaveChanged'
        QtMocHelpers::SignalData<void(bool)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'progressBarAdded'
        QtMocHelpers::SignalData<void(const StelProgressController *)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 23, 2 },
        }}),
        // Signal 'progressBarRemoved'
        QtMocHelpers::SignalData<void(const StelProgressController *)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 23, 2 },
        }}),
        // Signal 'aboutToQuit'
        QtMocHelpers::SignalData<void()>(25, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'glWindowHasBeenResized'
        QtMocHelpers::SlotData<void(const QRectF &)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 27, 28 },
        }}),
        // Slot 'glPhysicalWindowHasBeenResized'
        QtMocHelpers::SlotData<void(const QRectF &)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 27, 28 },
        }}),
        // Slot 'setVisionModeNight'
        QtMocHelpers::SlotData<void(bool)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'getVisionModeNight'
        QtMocHelpers::SlotData<bool() const>(31, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFlagOverwriteInfoColor'
        QtMocHelpers::SlotData<void(bool)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'getFlagOverwriteInfoColor'
        QtMocHelpers::SlotData<bool() const>(33, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFlagShowDecimalDegrees'
        QtMocHelpers::SlotData<void(bool)>(34, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 35 },
        }}),
        // Slot 'getFlagShowDecimalDegrees'
        QtMocHelpers::SlotData<bool() const>(36, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFlagSouthAzimuthUsage'
        QtMocHelpers::SlotData<void(bool)>(37, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 38 },
        }}),
        // Slot 'getFlagSouthAzimuthUsage'
        QtMocHelpers::SlotData<bool() const>(39, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFlagUseNegativeHourAngles'
        QtMocHelpers::SlotData<void(bool)>(40, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 38 },
        }}),
        // Slot 'getFlagUseNegativeHourAngles'
        QtMocHelpers::SlotData<bool() const>(41, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'getFlagPolarDistanceUsage'
        QtMocHelpers::SlotData<bool() const>(42, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFlagPolarDistanceUsage'
        QtMocHelpers::SlotData<void(bool)>(43, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 38 },
        }}),
        // Slot 'setFlagUseFormattingOutput'
        QtMocHelpers::SlotData<void(bool)>(44, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 35 },
        }}),
        // Slot 'getFlagUseFormattingOutput'
        QtMocHelpers::SlotData<bool() const>(45, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFlagUseCCSDesignation'
        QtMocHelpers::SlotData<void(bool)>(46, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 35 },
        }}),
        // Slot 'getFlagUseCCSDesignation'
        QtMocHelpers::SlotData<bool() const>(47, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setOverwriteInfoColor'
        QtMocHelpers::SlotData<void(const Vec3f &)>(48, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 18, 19 },
        }}),
        // Slot 'getOverwriteInfoColor'
        QtMocHelpers::SlotData<Vec3f() const>(49, 2, QMC::AccessPublic, 0x80000000 | 18),
        // Slot 'setDaylightInfoColor'
        QtMocHelpers::SlotData<void(const Vec3f &)>(50, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 18, 19 },
        }}),
        // Slot 'getDaylightInfoColor'
        QtMocHelpers::SlotData<Vec3f() const>(51, 2, QMC::AccessPublic, 0x80000000 | 18),
        // Slot 'setFlagImmediateSave'
        QtMocHelpers::SlotData<void(bool)>(52, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 35 },
        }}),
        // Slot 'getFlagImmediateSave'
        QtMocHelpers::SlotData<bool() const>(53, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'getFps'
        QtMocHelpers::SlotData<float() const>(54, 2, QMC::AccessPublic, QMetaType::Float),
        // Slot 'setAppFont'
        QtMocHelpers::SlotData<void(QFont)>(55, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 56 },
        }}),
        // Slot 'getDefaultFBO'
        QtMocHelpers::SlotData<quint32() const>(57, 2, QMC::AccessPublic, QMetaType::UInt),
        // Slot 'ensureGLContextCurrent'
        QtMocHelpers::SlotData<void()>(58, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'getTotalRunTime'
        QtMocHelpers::SlotData<double()>(59, 2, QMC::AccessPublic, QMetaType::Double),
        // Slot 'getAnimationTime'
        QtMocHelpers::SlotData<double()>(60, 2, QMC::AccessPublic, QMetaType::Double),
        // Slot 'reportFileDownloadFinished'
        QtMocHelpers::SlotData<void(QNetworkReply *)>(61, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 62, 63 },
        }}),
        // Slot 'quit'
        QtMocHelpers::SlotData<void()>(64, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'nightMode'
        QtMocHelpers::PropertyData<bool>(65, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 0),
        // property 'flagShowDecimalDegrees'
        QtMocHelpers::PropertyData<bool>(66, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'flagUseAzimuthFromSouth'
        QtMocHelpers::PropertyData<bool>(67, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 2),
        // property 'flagUseNegativeHourAngles'
        QtMocHelpers::PropertyData<bool>(68, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'flagUsePolarDistance'
        QtMocHelpers::PropertyData<bool>(69, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 4),
        // property 'flagUseCCSDesignation'
        QtMocHelpers::PropertyData<bool>(70, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
        // property 'flagUseFormattingOutput'
        QtMocHelpers::PropertyData<bool>(71, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 6),
        // property 'flagOverwriteInfoColor'
        QtMocHelpers::PropertyData<bool>(72, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 7),
        // property 'overwriteInfoColor'
        QtMocHelpers::PropertyData<Vec3f>(73, 0x80000000 | 18, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 14),
        // property 'daylightInfoColor'
        QtMocHelpers::PropertyData<Vec3f>(74, 0x80000000 | 18, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 15),
        // property 'screenFontSize'
        QtMocHelpers::PropertyData<int>(75, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 10),
        // property 'guiFontSize'
        QtMocHelpers::PropertyData<int>(76, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 11),
        // property 'screenButtonScale'
        QtMocHelpers::PropertyData<double>(77, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 12),
        // property 'flagImmediateSave'
        QtMocHelpers::PropertyData<bool>(78, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 16),
        // property 'version'
        QtMocHelpers::PropertyData<QString>(79, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<StelApp, qt_meta_tag_ZN7StelAppE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject StelApp::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7StelAppE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7StelAppE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN7StelAppE_t>.metaTypes,
    nullptr
} };

void StelApp::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<StelApp *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->visionNightModeChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->flagShowDecimalDegreesChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->flagUseAzimuthFromSouthChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->flagUseNegativeHourAnglesChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->flagUsePolarDistanceChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->flagUseCCSDesignationChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->flagUseFormattingOutputChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->flagOverwriteInfoColorChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->colorSchemeChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 9: _t->languageChanged(); break;
        case 10: _t->screenFontSizeChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 11: _t->guiFontSizeChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 12: _t->screenButtonScaleChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 13: _t->fontChanged((*reinterpret_cast<std::add_pointer_t<QFont>>(_a[1]))); break;
        case 14: _t->overwriteInfoColorChanged((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1]))); break;
        case 15: _t->daylightInfoColorChanged((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1]))); break;
        case 16: _t->flagImmediateSaveChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 17: _t->progressBarAdded((*reinterpret_cast<std::add_pointer_t<const StelProgressController*>>(_a[1]))); break;
        case 18: _t->progressBarRemoved((*reinterpret_cast<std::add_pointer_t<const StelProgressController*>>(_a[1]))); break;
        case 19: _t->aboutToQuit(); break;
        case 20: _t->glWindowHasBeenResized((*reinterpret_cast<std::add_pointer_t<QRectF>>(_a[1]))); break;
        case 21: _t->glPhysicalWindowHasBeenResized((*reinterpret_cast<std::add_pointer_t<QRectF>>(_a[1]))); break;
        case 22: _t->setVisionModeNight((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 23: { bool _r = _t->getVisionModeNight();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 24: _t->setFlagOverwriteInfoColor((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 25: { bool _r = _t->getFlagOverwriteInfoColor();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 26: _t->setFlagShowDecimalDegrees((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 27: { bool _r = _t->getFlagShowDecimalDegrees();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 28: _t->setFlagSouthAzimuthUsage((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 29: { bool _r = _t->getFlagSouthAzimuthUsage();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 30: _t->setFlagUseNegativeHourAngles((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 31: { bool _r = _t->getFlagUseNegativeHourAngles();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 32: { bool _r = _t->getFlagPolarDistanceUsage();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 33: _t->setFlagPolarDistanceUsage((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 34: _t->setFlagUseFormattingOutput((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 35: { bool _r = _t->getFlagUseFormattingOutput();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 36: _t->setFlagUseCCSDesignation((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 37: { bool _r = _t->getFlagUseCCSDesignation();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 38: _t->setOverwriteInfoColor((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1]))); break;
        case 39: { Vec3f _r = _t->getOverwriteInfoColor();
            if (_a[0]) *reinterpret_cast<Vec3f*>(_a[0]) = std::move(_r); }  break;
        case 40: _t->setDaylightInfoColor((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1]))); break;
        case 41: { Vec3f _r = _t->getDaylightInfoColor();
            if (_a[0]) *reinterpret_cast<Vec3f*>(_a[0]) = std::move(_r); }  break;
        case 42: _t->setFlagImmediateSave((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 43: { bool _r = _t->getFlagImmediateSave();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 44: { float _r = _t->getFps();
            if (_a[0]) *reinterpret_cast<float*>(_a[0]) = std::move(_r); }  break;
        case 45: _t->setAppFont((*reinterpret_cast<std::add_pointer_t<QFont>>(_a[1]))); break;
        case 46: { quint32 _r = _t->getDefaultFBO();
            if (_a[0]) *reinterpret_cast<quint32*>(_a[0]) = std::move(_r); }  break;
        case 47: _t->ensureGLContextCurrent(); break;
        case 48: { double _r = _t->getTotalRunTime();
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 49: { double _r = _t->getAnimationTime();
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 50: _t->reportFileDownloadFinished((*reinterpret_cast<std::add_pointer_t<QNetworkReply*>>(_a[1]))); break;
        case 51: _t->quit(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 14:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 15:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 38:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 40:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (StelApp::*)(bool )>(_a, &StelApp::visionNightModeChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelApp::*)(bool )>(_a, &StelApp::flagShowDecimalDegreesChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelApp::*)(bool )>(_a, &StelApp::flagUseAzimuthFromSouthChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelApp::*)(bool )>(_a, &StelApp::flagUseNegativeHourAnglesChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelApp::*)(bool )>(_a, &StelApp::flagUsePolarDistanceChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelApp::*)(bool )>(_a, &StelApp::flagUseCCSDesignationChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelApp::*)(bool )>(_a, &StelApp::flagUseFormattingOutputChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelApp::*)(bool )>(_a, &StelApp::flagOverwriteInfoColorChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelApp::*)(const QString & )>(_a, &StelApp::colorSchemeChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelApp::*)()>(_a, &StelApp::languageChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelApp::*)(int )>(_a, &StelApp::screenFontSizeChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelApp::*)(int )>(_a, &StelApp::guiFontSizeChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelApp::*)(double )>(_a, &StelApp::screenButtonScaleChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelApp::*)(const QFont & )>(_a, &StelApp::fontChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelApp::*)(const Vec3f & )>(_a, &StelApp::overwriteInfoColorChanged, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelApp::*)(const Vec3f & )>(_a, &StelApp::daylightInfoColorChanged, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelApp::*)(bool )>(_a, &StelApp::flagImmediateSaveChanged, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelApp::*)(const StelProgressController * )>(_a, &StelApp::progressBarAdded, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelApp::*)(const StelProgressController * )>(_a, &StelApp::progressBarRemoved, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelApp::*)()>(_a, &StelApp::aboutToQuit, 19))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 9:
        case 8:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Vec3f >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->getVisionModeNight(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->getFlagShowDecimalDegrees(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->getFlagSouthAzimuthUsage(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->getFlagUseNegativeHourAngles(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->getFlagPolarDistanceUsage(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->getFlagUseCCSDesignation(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->getFlagUseFormattingOutput(); break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->getFlagOverwriteInfoColor(); break;
        case 8: *reinterpret_cast<Vec3f*>(_v) = _t->getOverwriteInfoColor(); break;
        case 9: *reinterpret_cast<Vec3f*>(_v) = _t->getDaylightInfoColor(); break;
        case 10: *reinterpret_cast<int*>(_v) = _t->getScreenFontSize(); break;
        case 11: *reinterpret_cast<int*>(_v) = _t->getGuiFontSize(); break;
        case 12: *reinterpret_cast<double*>(_v) = _t->getScreenButtonScale(); break;
        case 13: *reinterpret_cast<bool*>(_v) = _t->getFlagImmediateSave(); break;
        case 14: *reinterpret_cast<QString*>(_v) = _t->getVersion(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setVisionModeNight(*reinterpret_cast<bool*>(_v)); break;
        case 1: _t->setFlagShowDecimalDegrees(*reinterpret_cast<bool*>(_v)); break;
        case 2: _t->setFlagSouthAzimuthUsage(*reinterpret_cast<bool*>(_v)); break;
        case 3: _t->setFlagUseNegativeHourAngles(*reinterpret_cast<bool*>(_v)); break;
        case 4: _t->setFlagPolarDistanceUsage(*reinterpret_cast<bool*>(_v)); break;
        case 5: _t->setFlagUseCCSDesignation(*reinterpret_cast<bool*>(_v)); break;
        case 6: _t->setFlagUseFormattingOutput(*reinterpret_cast<bool*>(_v)); break;
        case 7: _t->setFlagOverwriteInfoColor(*reinterpret_cast<bool*>(_v)); break;
        case 8: _t->setOverwriteInfoColor(*reinterpret_cast<Vec3f*>(_v)); break;
        case 9: _t->setDaylightInfoColor(*reinterpret_cast<Vec3f*>(_v)); break;
        case 10: _t->setScreenFontSize(*reinterpret_cast<int*>(_v)); break;
        case 11: _t->setGuiFontSize(*reinterpret_cast<int*>(_v)); break;
        case 12: _t->setScreenButtonScale(*reinterpret_cast<double*>(_v)); break;
        case 13: _t->setFlagImmediateSave(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *StelApp::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *StelApp::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7StelAppE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int StelApp::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 52)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 52;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 52)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 52;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    return _id;
}

// SIGNAL 0
void StelApp::visionNightModeChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void StelApp::flagShowDecimalDegreesChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void StelApp::flagUseAzimuthFromSouthChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void StelApp::flagUseNegativeHourAnglesChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void StelApp::flagUsePolarDistanceChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void StelApp::flagUseCCSDesignationChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void StelApp::flagUseFormattingOutputChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void StelApp::flagOverwriteInfoColorChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void StelApp::colorSchemeChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void StelApp::languageChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void StelApp::screenFontSizeChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void StelApp::guiFontSizeChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void StelApp::screenButtonScaleChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void StelApp::fontChanged(const QFont & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1);
}

// SIGNAL 14
void StelApp::overwriteInfoColorChanged(const Vec3f & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1);
}

// SIGNAL 15
void StelApp::daylightInfoColorChanged(const Vec3f & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 15, nullptr, _t1);
}

// SIGNAL 16
void StelApp::flagImmediateSaveChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 16, nullptr, _t1);
}

// SIGNAL 17
void StelApp::progressBarAdded(const StelProgressController * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 17, nullptr, _t1);
}

// SIGNAL 18
void StelApp::progressBarRemoved(const StelProgressController * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 18, nullptr, _t1);
}

// SIGNAL 19
void StelApp::aboutToQuit()
{
    QMetaObject::activate(this, &staticMetaObject, 19, nullptr);
}
QT_WARNING_POP
