/********************************************************************************
** Form generated from reading UI file 'scenery3dDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SCENERY3DDIALOG_H
#define UI_SCENERY3DDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSlider>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTextBrowser>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "Dialog.hpp"
#include "gui/StelCloseButton.hpp"

QT_BEGIN_NAMESPACE

class Ui_scenery3dDialogForm
{
public:
    QVBoxLayout *vboxLayout;
    TitleBar *titleBar;
    QFrame *viewContent;
    QVBoxLayout *verticalLayout_5;
    QWidget *widget;
    QGridLayout *gridLayout_3;
    QGridLayout *gridLayout_2;
    QTextBrowser *scenery3dTextBrowser;
    QListWidget *scenery3dListWidget;
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout;
    QCheckBox *checkBoxDefaultScene;
    QSpacerItem *horizontalSpacer_4;
    QPushButton *pushButtonOpenStoredViewDialog;
    QSpacerItem *horizontalSpacer;
    QTabWidget *tabWidget;
    QWidget *tab_VisualOptions;
    QFormLayout *formLayout_4;
    QCheckBox *checkBoxTorchlight;
    QLabel *labelTorchStrength;
    QLabel *labelTorchRange;
    QSlider *sliderTorchRange;
    QLabel *labelDirectionalLightPush;
    QDoubleSpinBox *directionalLightPushSpinBox;
    QSlider *sliderTorchStrength;
    QGroupBox *gridCoordinatesGroupBox;
    QHBoxLayout *horizontalLayout_4;
    QRadioButton *radioButton_GCTopRight;
    QRadioButton *radioButton_GCBottom;
    QCheckBox *overdrawLandscapePolylineCheckBox;
    QCheckBox *ignoreInitialViewCheckBox;
    QWidget *tab_QualityPerformanceOptions;
    QHBoxLayout *horizontalLayout_2;
    QFormLayout *formLayout;
    QCheckBox *checkBoxEnablePixelLight;
    QCheckBox *checkBoxEnableShadows;
    QFormLayout *formLayout_6;
    QCheckBox *checkBoxSimpleShadows;
    QCheckBox *checkBoxEnableBump;
    QLabel *label_5;
    QComboBox *comboBoxCubemapSize;
    QLabel *labelShadowmapSize;
    QComboBox *comboBoxShadowmapSize;
    QLabel *labelFilterQuality;
    QComboBox *comboBoxShadowFiltering;
    QFormLayout *formLayout_5;
    QCheckBox *checkBoxPCSS;
    QFormLayout *formLayout_2;
    QLabel *label_3;
    QComboBox *comboBoxCubemapMode;
    QCheckBox *checkBoxCubemapShadows;
    QCheckBox *checkBoxEnableLazyDrawing;
    QFormLayout *formLayout_3;
    QLabel *labelLazyDrawingInterval;
    QDoubleSpinBox *spinLazyDrawingInterval;
    QCheckBox *checkBoxDominantFace;
    QCheckBox *checkBoxSecondDominantFace;

    void setupUi(QWidget *scenery3dDialogForm)
    {
        if (scenery3dDialogForm->objectName().isEmpty())
            scenery3dDialogForm->setObjectName("scenery3dDialogForm");
        scenery3dDialogForm->resize(861, 550);
        QFont font;
        scenery3dDialogForm->setFont(font);
        scenery3dDialogForm->setAutoFillBackground(false);
        scenery3dDialogForm->setStyleSheet(QString::fromUtf8(""));
        vboxLayout = new QVBoxLayout(scenery3dDialogForm);
        vboxLayout->setSpacing(0);
        vboxLayout->setObjectName("vboxLayout");
        vboxLayout->setContentsMargins(0, 0, 0, 0);
        titleBar = new TitleBar(scenery3dDialogForm);
        titleBar->setObjectName("titleBar");

        vboxLayout->addWidget(titleBar);

        viewContent = new QFrame(scenery3dDialogForm);
        viewContent->setObjectName("viewContent");
        viewContent->setFrameShape(QFrame::StyledPanel);
        verticalLayout_5 = new QVBoxLayout(viewContent);
        verticalLayout_5->setSpacing(0);
        verticalLayout_5->setObjectName("verticalLayout_5");
        verticalLayout_5->setContentsMargins(0, 0, 0, 0);
        widget = new QWidget(viewContent);
        widget->setObjectName("widget");
        gridLayout_3 = new QGridLayout(widget);
        gridLayout_3->setSpacing(0);
        gridLayout_3->setObjectName("gridLayout_3");
        gridLayout_3->setContentsMargins(0, 0, 0, 0);
        gridLayout_2 = new QGridLayout();
        gridLayout_2->setObjectName("gridLayout_2");
        scenery3dTextBrowser = new QTextBrowser(widget);
        scenery3dTextBrowser->setObjectName("scenery3dTextBrowser");
        scenery3dTextBrowser->setEnabled(true);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(scenery3dTextBrowser->sizePolicy().hasHeightForWidth());
        scenery3dTextBrowser->setSizePolicy(sizePolicy);
        scenery3dTextBrowser->setFrameShape(QFrame::StyledPanel);
        scenery3dTextBrowser->setLineWidth(1);
        scenery3dTextBrowser->setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded);
        scenery3dTextBrowser->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        scenery3dTextBrowser->setHtml(QString::fromUtf8("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><meta charset=\"utf-8\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"hr { height: 1px; border-width: 0; }\n"
"li.unchecked::marker { content: \"\\2610\"; }\n"
"li.checked::marker { content: \"\\2612\"; }\n"
"</style></head><body style=\" font-family:'Segoe UI'; font-size:9pt; font-weight:400; font-style:normal;\">\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-family:'Sans Serif';\"><br /></p></body></html>"));
        scenery3dTextBrowser->setOpenExternalLinks(true);

        gridLayout_2->addWidget(scenery3dTextBrowser, 0, 1, 1, 1);

        scenery3dListWidget = new QListWidget(widget);
        scenery3dListWidget->setObjectName("scenery3dListWidget");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Expanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(scenery3dListWidget->sizePolicy().hasHeightForWidth());
        scenery3dListWidget->setSizePolicy(sizePolicy1);
        scenery3dListWidget->setMinimumSize(QSize(180, 0));
        scenery3dListWidget->setMaximumSize(QSize(120, 16777215));
        scenery3dListWidget->setFocusPolicy(Qt::NoFocus);
        scenery3dListWidget->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);

        gridLayout_2->addWidget(scenery3dListWidget, 0, 0, 2, 1);

        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(3, 3, 3, 3);
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        checkBoxDefaultScene = new QCheckBox(widget);
        checkBoxDefaultScene->setObjectName("checkBoxDefaultScene");

        horizontalLayout->addWidget(checkBoxDefaultScene);

        horizontalSpacer_4 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_4);

        pushButtonOpenStoredViewDialog = new QPushButton(widget);
        pushButtonOpenStoredViewDialog->setObjectName("pushButtonOpenStoredViewDialog");

        horizontalLayout->addWidget(pushButtonOpenStoredViewDialog);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);


        verticalLayout->addLayout(horizontalLayout);

        tabWidget = new QTabWidget(widget);
        tabWidget->setObjectName("tabWidget");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(tabWidget->sizePolicy().hasHeightForWidth());
        tabWidget->setSizePolicy(sizePolicy2);
        tab_VisualOptions = new QWidget();
        tab_VisualOptions->setObjectName("tab_VisualOptions");
        formLayout_4 = new QFormLayout(tab_VisualOptions);
        formLayout_4->setObjectName("formLayout_4");
        formLayout_4->setFieldGrowthPolicy(QFormLayout::AllNonFixedFieldsGrow);
        checkBoxTorchlight = new QCheckBox(tab_VisualOptions);
        checkBoxTorchlight->setObjectName("checkBoxTorchlight");

        formLayout_4->setWidget(0, QFormLayout::ItemRole::SpanningRole, checkBoxTorchlight);

        labelTorchStrength = new QLabel(tab_VisualOptions);
        labelTorchStrength->setObjectName("labelTorchStrength");

        formLayout_4->setWidget(2, QFormLayout::ItemRole::LabelRole, labelTorchStrength);

        labelTorchRange = new QLabel(tab_VisualOptions);
        labelTorchRange->setObjectName("labelTorchRange");

        formLayout_4->setWidget(3, QFormLayout::ItemRole::LabelRole, labelTorchRange);

        sliderTorchRange = new QSlider(tab_VisualOptions);
        sliderTorchRange->setObjectName("sliderTorchRange");
        sliderTorchRange->setMaximum(10000);
        sliderTorchRange->setSingleStep(100);
        sliderTorchRange->setPageStep(1000);
        sliderTorchRange->setOrientation(Qt::Horizontal);

        formLayout_4->setWidget(3, QFormLayout::ItemRole::FieldRole, sliderTorchRange);

        labelDirectionalLightPush = new QLabel(tab_VisualOptions);
        labelDirectionalLightPush->setObjectName("labelDirectionalLightPush");

        formLayout_4->setWidget(4, QFormLayout::ItemRole::LabelRole, labelDirectionalLightPush);

        directionalLightPushSpinBox = new QDoubleSpinBox(tab_VisualOptions);
        directionalLightPushSpinBox->setObjectName("directionalLightPushSpinBox");
        directionalLightPushSpinBox->setMinimum(0.200000000000000);
        directionalLightPushSpinBox->setMaximum(20.000000000000000);
        directionalLightPushSpinBox->setSingleStep(0.050000000000000);
        directionalLightPushSpinBox->setValue(1.000000000000000);

        formLayout_4->setWidget(4, QFormLayout::ItemRole::FieldRole, directionalLightPushSpinBox);

        sliderTorchStrength = new QSlider(tab_VisualOptions);
        sliderTorchStrength->setObjectName("sliderTorchStrength");
        sliderTorchStrength->setMaximum(100);
        sliderTorchStrength->setOrientation(Qt::Horizontal);

        formLayout_4->setWidget(2, QFormLayout::ItemRole::FieldRole, sliderTorchStrength);

        gridCoordinatesGroupBox = new QGroupBox(tab_VisualOptions);
        gridCoordinatesGroupBox->setObjectName("gridCoordinatesGroupBox");
        gridCoordinatesGroupBox->setCheckable(true);
        gridCoordinatesGroupBox->setChecked(true);
        horizontalLayout_4 = new QHBoxLayout(gridCoordinatesGroupBox);
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        radioButton_GCTopRight = new QRadioButton(gridCoordinatesGroupBox);
        radioButton_GCTopRight->setObjectName("radioButton_GCTopRight");

        horizontalLayout_4->addWidget(radioButton_GCTopRight);

        radioButton_GCBottom = new QRadioButton(gridCoordinatesGroupBox);
        radioButton_GCBottom->setObjectName("radioButton_GCBottom");

        horizontalLayout_4->addWidget(radioButton_GCBottom);


        formLayout_4->setWidget(5, QFormLayout::ItemRole::FieldRole, gridCoordinatesGroupBox);

        overdrawLandscapePolylineCheckBox = new QCheckBox(tab_VisualOptions);
        overdrawLandscapePolylineCheckBox->setObjectName("overdrawLandscapePolylineCheckBox");

        formLayout_4->setWidget(5, QFormLayout::ItemRole::LabelRole, overdrawLandscapePolylineCheckBox);

        ignoreInitialViewCheckBox = new QCheckBox(tab_VisualOptions);
        ignoreInitialViewCheckBox->setObjectName("ignoreInitialViewCheckBox");

        formLayout_4->setWidget(7, QFormLayout::ItemRole::LabelRole, ignoreInitialViewCheckBox);

        tabWidget->addTab(tab_VisualOptions, QString());
        tab_QualityPerformanceOptions = new QWidget();
        tab_QualityPerformanceOptions->setObjectName("tab_QualityPerformanceOptions");
        horizontalLayout_2 = new QHBoxLayout(tab_QualityPerformanceOptions);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        formLayout = new QFormLayout();
        formLayout->setObjectName("formLayout");
        formLayout->setFieldGrowthPolicy(QFormLayout::AllNonFixedFieldsGrow);
        checkBoxEnablePixelLight = new QCheckBox(tab_QualityPerformanceOptions);
        checkBoxEnablePixelLight->setObjectName("checkBoxEnablePixelLight");

        formLayout->setWidget(0, QFormLayout::ItemRole::SpanningRole, checkBoxEnablePixelLight);

        checkBoxEnableShadows = new QCheckBox(tab_QualityPerformanceOptions);
        checkBoxEnableShadows->setObjectName("checkBoxEnableShadows");
        checkBoxEnableShadows->setEnabled(true);
        checkBoxEnableShadows->setChecked(false);
        checkBoxEnableShadows->setTristate(false);

        formLayout->setWidget(1, QFormLayout::ItemRole::SpanningRole, checkBoxEnableShadows);

        formLayout_6 = new QFormLayout();
        formLayout_6->setObjectName("formLayout_6");
        formLayout_6->setContentsMargins(20, -1, -1, -1);
        checkBoxSimpleShadows = new QCheckBox(tab_QualityPerformanceOptions);
        checkBoxSimpleShadows->setObjectName("checkBoxSimpleShadows");

        formLayout_6->setWidget(0, QFormLayout::ItemRole::SpanningRole, checkBoxSimpleShadows);


        formLayout->setLayout(2, QFormLayout::ItemRole::SpanningRole, formLayout_6);

        checkBoxEnableBump = new QCheckBox(tab_QualityPerformanceOptions);
        checkBoxEnableBump->setObjectName("checkBoxEnableBump");
        checkBoxEnableBump->setEnabled(true);
        checkBoxEnableBump->setChecked(false);
        checkBoxEnableBump->setTristate(false);

        formLayout->setWidget(3, QFormLayout::ItemRole::SpanningRole, checkBoxEnableBump);

        label_5 = new QLabel(tab_QualityPerformanceOptions);
        label_5->setObjectName("label_5");

        formLayout->setWidget(4, QFormLayout::ItemRole::LabelRole, label_5);

        comboBoxCubemapSize = new QComboBox(tab_QualityPerformanceOptions);
        comboBoxCubemapSize->setObjectName("comboBoxCubemapSize");

        formLayout->setWidget(4, QFormLayout::ItemRole::FieldRole, comboBoxCubemapSize);

        labelShadowmapSize = new QLabel(tab_QualityPerformanceOptions);
        labelShadowmapSize->setObjectName("labelShadowmapSize");

        formLayout->setWidget(5, QFormLayout::ItemRole::LabelRole, labelShadowmapSize);

        comboBoxShadowmapSize = new QComboBox(tab_QualityPerformanceOptions);
        comboBoxShadowmapSize->setObjectName("comboBoxShadowmapSize");

        formLayout->setWidget(5, QFormLayout::ItemRole::FieldRole, comboBoxShadowmapSize);

        labelFilterQuality = new QLabel(tab_QualityPerformanceOptions);
        labelFilterQuality->setObjectName("labelFilterQuality");

        formLayout->setWidget(6, QFormLayout::ItemRole::LabelRole, labelFilterQuality);

        comboBoxShadowFiltering = new QComboBox(tab_QualityPerformanceOptions);
        comboBoxShadowFiltering->addItem(QString());
        comboBoxShadowFiltering->addItem(QString());
        comboBoxShadowFiltering->addItem(QString());
        comboBoxShadowFiltering->addItem(QString());
        comboBoxShadowFiltering->addItem(QString());
        comboBoxShadowFiltering->addItem(QString());
        comboBoxShadowFiltering->setObjectName("comboBoxShadowFiltering");

        formLayout->setWidget(6, QFormLayout::ItemRole::FieldRole, comboBoxShadowFiltering);

        formLayout_5 = new QFormLayout();
        formLayout_5->setObjectName("formLayout_5");
        formLayout_5->setContentsMargins(20, -1, -1, -1);
        checkBoxPCSS = new QCheckBox(tab_QualityPerformanceOptions);
        checkBoxPCSS->setObjectName("checkBoxPCSS");

        formLayout_5->setWidget(0, QFormLayout::ItemRole::SpanningRole, checkBoxPCSS);


        formLayout->setLayout(7, QFormLayout::ItemRole::SpanningRole, formLayout_5);


        horizontalLayout_2->addLayout(formLayout);

        formLayout_2 = new QFormLayout();
        formLayout_2->setObjectName("formLayout_2");
        formLayout_2->setFieldGrowthPolicy(QFormLayout::AllNonFixedFieldsGrow);
        label_3 = new QLabel(tab_QualityPerformanceOptions);
        label_3->setObjectName("label_3");

        formLayout_2->setWidget(0, QFormLayout::ItemRole::LabelRole, label_3);

        comboBoxCubemapMode = new QComboBox(tab_QualityPerformanceOptions);
        comboBoxCubemapMode->setObjectName("comboBoxCubemapMode");

        formLayout_2->setWidget(0, QFormLayout::ItemRole::FieldRole, comboBoxCubemapMode);

        checkBoxCubemapShadows = new QCheckBox(tab_QualityPerformanceOptions);
        checkBoxCubemapShadows->setObjectName("checkBoxCubemapShadows");

        formLayout_2->setWidget(1, QFormLayout::ItemRole::SpanningRole, checkBoxCubemapShadows);

        checkBoxEnableLazyDrawing = new QCheckBox(tab_QualityPerformanceOptions);
        checkBoxEnableLazyDrawing->setObjectName("checkBoxEnableLazyDrawing");

        formLayout_2->setWidget(2, QFormLayout::ItemRole::SpanningRole, checkBoxEnableLazyDrawing);

        formLayout_3 = new QFormLayout();
        formLayout_3->setObjectName("formLayout_3");
        formLayout_3->setContentsMargins(20, -1, -1, -1);
        labelLazyDrawingInterval = new QLabel(tab_QualityPerformanceOptions);
        labelLazyDrawingInterval->setObjectName("labelLazyDrawingInterval");

        formLayout_3->setWidget(0, QFormLayout::ItemRole::LabelRole, labelLazyDrawingInterval);

        spinLazyDrawingInterval = new QDoubleSpinBox(tab_QualityPerformanceOptions);
        spinLazyDrawingInterval->setObjectName("spinLazyDrawingInterval");
        spinLazyDrawingInterval->setMinimum(0.100000000000000);
        spinLazyDrawingInterval->setMaximum(600.000000000000000);
        spinLazyDrawingInterval->setSingleStep(0.100000000000000);

        formLayout_3->setWidget(0, QFormLayout::ItemRole::FieldRole, spinLazyDrawingInterval);

        checkBoxDominantFace = new QCheckBox(tab_QualityPerformanceOptions);
        checkBoxDominantFace->setObjectName("checkBoxDominantFace");

        formLayout_3->setWidget(1, QFormLayout::ItemRole::SpanningRole, checkBoxDominantFace);

        checkBoxSecondDominantFace = new QCheckBox(tab_QualityPerformanceOptions);
        checkBoxSecondDominantFace->setObjectName("checkBoxSecondDominantFace");
        checkBoxSecondDominantFace->setEnabled(true);

        formLayout_3->setWidget(2, QFormLayout::ItemRole::SpanningRole, checkBoxSecondDominantFace);


        formLayout_2->setLayout(3, QFormLayout::ItemRole::SpanningRole, formLayout_3);


        horizontalLayout_2->addLayout(formLayout_2);

        tabWidget->addTab(tab_QualityPerformanceOptions, QString());

        verticalLayout->addWidget(tabWidget);


        gridLayout_2->addLayout(verticalLayout, 1, 1, 1, 1);


        gridLayout_3->addLayout(gridLayout_2, 0, 2, 1, 1);


        verticalLayout_5->addWidget(widget);


        vboxLayout->addWidget(viewContent);

#if QT_CONFIG(shortcut)
        labelTorchStrength->setBuddy(sliderTorchStrength);
        labelTorchRange->setBuddy(sliderTorchRange);
        label_5->setBuddy(comboBoxCubemapSize);
        labelShadowmapSize->setBuddy(comboBoxShadowmapSize);
        labelFilterQuality->setBuddy(comboBoxShadowFiltering);
        label_3->setBuddy(comboBoxCubemapMode);
        labelLazyDrawingInterval->setBuddy(spinLazyDrawingInterval);
#endif // QT_CONFIG(shortcut)

        retranslateUi(scenery3dDialogForm);

        tabWidget->setCurrentIndex(0);
        comboBoxShadowmapSize->setCurrentIndex(-1);


        QMetaObject::connectSlotsByName(scenery3dDialogForm);
    } // setupUi

    void retranslateUi(QWidget *scenery3dDialogForm)
    {
        titleBar->setTitle(QCoreApplication::translate("scenery3dDialogForm", "3D Scenery", nullptr));
        checkBoxDefaultScene->setText(QCoreApplication::translate("scenery3dDialogForm", "Set as default scene", nullptr));
        pushButtonOpenStoredViewDialog->setText(QCoreApplication::translate("scenery3dDialogForm", "Load/save viewpoints", nullptr));
        checkBoxTorchlight->setText(QCoreApplication::translate("scenery3dDialogForm", "Torchlight (%1)", nullptr));
        labelTorchStrength->setText(QCoreApplication::translate("scenery3dDialogForm", "Torch strength:", nullptr));
        labelTorchRange->setText(QCoreApplication::translate("scenery3dDialogForm", "Torch range:", nullptr));
        labelDirectionalLightPush->setText(QCoreApplication::translate("scenery3dDialogForm", "Directional light enhancement", nullptr));
#if QT_CONFIG(tooltip)
        directionalLightPushSpinBox->setToolTip(QCoreApplication::translate("scenery3dDialogForm", "Increase the power of directional light, e.g. to enhance contrast for patches of light cast through small holes.", nullptr));
#endif // QT_CONFIG(tooltip)
        gridCoordinatesGroupBox->setTitle(QCoreApplication::translate("scenery3dDialogForm", "Show grid coordinates (%1)", nullptr));
#if QT_CONFIG(tooltip)
        radioButton_GCTopRight->setToolTip(QCoreApplication::translate("scenery3dDialogForm", "Show coordinates in the top right screen corner", nullptr));
#endif // QT_CONFIG(tooltip)
        radioButton_GCTopRight->setText(QCoreApplication::translate("scenery3dDialogForm", "Top Right", nullptr));
#if QT_CONFIG(tooltip)
        radioButton_GCBottom->setToolTip(QCoreApplication::translate("scenery3dDialogForm", "Show coordinates centered in the bottom of the screen", nullptr));
#endif // QT_CONFIG(tooltip)
        radioButton_GCBottom->setText(QCoreApplication::translate("scenery3dDialogForm", "Bottom Center", nullptr));
#if QT_CONFIG(tooltip)
        overdrawLandscapePolylineCheckBox->setToolTip(QCoreApplication::translate("scenery3dDialogForm", "Overdraw horizon polygon defined in your landscape. This displays the inaccuracies in a very large planar model compared to Earth curvature, of course only for a single point of observation.", nullptr));
#endif // QT_CONFIG(tooltip)
        overdrawLandscapePolylineCheckBox->setText(QCoreApplication::translate("scenery3dDialogForm", "Draw horizon polyline in foreground", nullptr));
#if QT_CONFIG(tooltip)
        ignoreInitialViewCheckBox->setToolTip(QCoreApplication::translate("scenery3dDialogForm", "Do no use the configured direction and field of view. This may be helpful in a planetarium environment.", nullptr));
#endif // QT_CONFIG(tooltip)
        ignoreInitialViewCheckBox->setText(QCoreApplication::translate("scenery3dDialogForm", "Ignore initial view settings", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_VisualOptions), QCoreApplication::translate("scenery3dDialogForm", "Visual options", nullptr));
        checkBoxEnablePixelLight->setText(QCoreApplication::translate("scenery3dDialogForm", "Per-Pixel lighting", nullptr));
        checkBoxEnableShadows->setText(QCoreApplication::translate("scenery3dDialogForm", "Shadows", nullptr));
        checkBoxSimpleShadows->setText(QCoreApplication::translate("scenery3dDialogForm", "Simple shadows", nullptr));
        checkBoxEnableBump->setText(QCoreApplication::translate("scenery3dDialogForm", "Bump mapping", nullptr));
        label_5->setText(QCoreApplication::translate("scenery3dDialogForm", "Cubemap size:", nullptr));
        labelShadowmapSize->setText(QCoreApplication::translate("scenery3dDialogForm", "Shadowmap size:", nullptr));
        labelFilterQuality->setText(QCoreApplication::translate("scenery3dDialogForm", "Filtering Quality:", nullptr));
        comboBoxShadowFiltering->setItemText(0, QCoreApplication::translate("scenery3dDialogForm", "Off", nullptr));
        comboBoxShadowFiltering->setItemText(1, QCoreApplication::translate("scenery3dDialogForm", "Hardware", nullptr));
        comboBoxShadowFiltering->setItemText(2, QCoreApplication::translate("scenery3dDialogForm", "Low", nullptr));
        comboBoxShadowFiltering->setItemText(3, QCoreApplication::translate("scenery3dDialogForm", "Low + Hardware", nullptr));
        comboBoxShadowFiltering->setItemText(4, QCoreApplication::translate("scenery3dDialogForm", "High (slow)", nullptr));
        comboBoxShadowFiltering->setItemText(5, QCoreApplication::translate("scenery3dDialogForm", "High + Hardware (slow)", nullptr));

        checkBoxPCSS->setText(QCoreApplication::translate("scenery3dDialogForm", "PCSS", nullptr));
        label_3->setText(QCoreApplication::translate("scenery3dDialogForm", "Cubemapping mode:", nullptr));
        checkBoxCubemapShadows->setText(QCoreApplication::translate("scenery3dDialogForm", "Full cubemap shadows", nullptr));
        checkBoxEnableLazyDrawing->setText(QCoreApplication::translate("scenery3dDialogForm", "Lazy cubemap re-rendering", nullptr));
        labelLazyDrawingInterval->setText(QCoreApplication::translate("scenery3dDialogForm", "Interval:", nullptr));
        checkBoxDominantFace->setText(QCoreApplication::translate("scenery3dDialogForm", "Only update dominant face when moving", nullptr));
        checkBoxSecondDominantFace->setText(QCoreApplication::translate("scenery3dDialogForm", "Also update second-most dominant face", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_QualityPerformanceOptions), QCoreApplication::translate("scenery3dDialogForm", "Quality/Performance settings", nullptr));
        (void)scenery3dDialogForm;
    } // retranslateUi

};

namespace Ui {
    class scenery3dDialogForm: public Ui_scenery3dDialogForm {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SCENERY3DDIALOG_H
