/****************************************************************************
** Meta object code from reading C++ file 'PointerCoordinates.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../stellarium/plugins/PointerCoordinates/src/PointerCoordinates.hpp"
#include <QtCore/qmetatype.h>
#include <QtCore/qplugin.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'PointerCoordinates.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN18PointerCoordinatesE_t {};
} // unnamed namespace

template <> constexpr inline auto PointerCoordinates::qt_create_metaobjectdata<qt_meta_tag_ZN18PointerCoordinatesE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "PointerCoordinates",
        "flagCoordinatesVisibilityChanged",
        "",
        "b",
        "flagEnableAtStartupChanged",
        "flagShowCoordinatesButtonChanged",
        "flagShowConstellationChanged",
        "flagShowCrossedLinesChanged",
        "flagShowElongationChanged",
        "fontSizeChanged",
        "i",
        "fontColorChanged",
        "Vec3f",
        "saveSettings",
        "enableCoordinates",
        "setFlagEnableAtStartup",
        "setFontSize",
        "size",
        "setFlagShowCoordinatesButton",
        "setFlagShowCrossedLines",
        "setCurrentCoordinatesPlace",
        "PointerCoordinates::CoordinatesPlace",
        "place",
        "getCurrentCoordinatesPlace",
        "getCurrentCoordinatesPlaceKey",
        "setCurrentCoordinatesPlaceKey",
        "key",
        "setCurrentCoordinateSystem",
        "PointerCoordinates::CoordinateSystem",
        "cs",
        "getCurrentCoordinateSystem",
        "getCurrentCoordinateSystemKey",
        "setCurrentCoordinateSystemKey",
        "setCustomCoordinatesPlace",
        "x",
        "y",
        "setFlagShowConstellation",
        "setFlagShowElongation",
        "getFontColor",
        "setFontColor",
        "c",
        "enabled",
        "enableAtStartup",
        "showCoordinatesButton",
        "showConstellation",
        "showCrossedLines",
        "showElongation",
        "fontSize",
        "fontColor",
        "CoordinatesPlace",
        "TopCenter",
        "TopRight",
        "RightBottomCorner",
        "NearMouseCursor",
        "Custom",
        "CoordinateSystem",
        "RaDecJ2000",
        "RaDec",
        "HourAngle",
        "Ecliptic",
        "EclipticJ2000",
        "AltAzi",
        "Galactic",
        "Supergalactic"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'flagCoordinatesVisibilityChanged'
        QtMocHelpers::SignalData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagEnableAtStartupChanged'
        QtMocHelpers::SignalData<void(bool)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagShowCoordinatesButtonChanged'
        QtMocHelpers::SignalData<void(bool)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagShowConstellationChanged'
        QtMocHelpers::SignalData<void(bool)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagShowCrossedLinesChanged'
        QtMocHelpers::SignalData<void(bool)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagShowElongationChanged'
        QtMocHelpers::SignalData<void(bool)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'fontSizeChanged'
        QtMocHelpers::SignalData<void(int)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 10 },
        }}),
        // Signal 'fontColorChanged'
        QtMocHelpers::SignalData<void(Vec3f)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 2 },
        }}),
        // Slot 'saveSettings'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'enableCoordinates'
        QtMocHelpers::SlotData<void(bool)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'setFlagEnableAtStartup'
        QtMocHelpers::SlotData<void(bool)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'setFontSize'
        QtMocHelpers::SlotData<void(int)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 17 },
        }}),
        // Slot 'setFlagShowCoordinatesButton'
        QtMocHelpers::SlotData<void(bool)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'setFlagShowCrossedLines'
        QtMocHelpers::SlotData<void(bool)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'setCurrentCoordinatesPlace'
        QtMocHelpers::SlotData<void(PointerCoordinates::CoordinatesPlace)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 21, 22 },
        }}),
        // Slot 'getCurrentCoordinatesPlace'
        QtMocHelpers::SlotData<PointerCoordinates::CoordinatesPlace() const>(23, 2, QMC::AccessPublic, 0x80000000 | 21),
        // Slot 'getCurrentCoordinatesPlaceKey'
        QtMocHelpers::SlotData<QString() const>(24, 2, QMC::AccessPublic, QMetaType::QString),
        // Slot 'setCurrentCoordinatesPlaceKey'
        QtMocHelpers::SlotData<void(const QString &)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 26 },
        }}),
        // Slot 'setCurrentCoordinateSystem'
        QtMocHelpers::SlotData<void(PointerCoordinates::CoordinateSystem)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 28, 29 },
        }}),
        // Slot 'getCurrentCoordinateSystem'
        QtMocHelpers::SlotData<PointerCoordinates::CoordinateSystem() const>(30, 2, QMC::AccessPublic, 0x80000000 | 28),
        // Slot 'getCurrentCoordinateSystemKey'
        QtMocHelpers::SlotData<QString() const>(31, 2, QMC::AccessPublic, QMetaType::QString),
        // Slot 'setCurrentCoordinateSystemKey'
        QtMocHelpers::SlotData<void(const QString &)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 26 },
        }}),
        // Slot 'setCustomCoordinatesPlace'
        QtMocHelpers::SlotData<void(int, int)>(33, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 34 }, { QMetaType::Int, 35 },
        }}),
        // Slot 'setFlagShowConstellation'
        QtMocHelpers::SlotData<void(bool)>(36, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'setFlagShowElongation'
        QtMocHelpers::SlotData<void(bool)>(37, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'getFontColor'
        QtMocHelpers::SlotData<Vec3f() const>(38, 2, QMC::AccessPublic, 0x80000000 | 12),
        // Slot 'setFontColor'
        QtMocHelpers::SlotData<void(const Vec3f &)>(39, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 40 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'enabled'
        QtMocHelpers::PropertyData<bool>(41, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 0),
        // property 'enableAtStartup'
        QtMocHelpers::PropertyData<bool>(42, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 1),
        // property 'showCoordinatesButton'
        QtMocHelpers::PropertyData<bool>(43, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 2),
        // property 'showConstellation'
        QtMocHelpers::PropertyData<bool>(44, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 3),
        // property 'showCrossedLines'
        QtMocHelpers::PropertyData<bool>(45, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 4),
        // property 'showElongation'
        QtMocHelpers::PropertyData<bool>(46, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 5),
        // property 'fontSize'
        QtMocHelpers::PropertyData<int>(47, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 6),
        // property 'fontColor'
        QtMocHelpers::PropertyData<Vec3f>(48, 0x80000000 | 12, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 7),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'CoordinatesPlace'
        QtMocHelpers::EnumData<enum CoordinatesPlace>(49, 49, QMC::EnumFlags{}).add({
            {   50, CoordinatesPlace::TopCenter },
            {   51, CoordinatesPlace::TopRight },
            {   52, CoordinatesPlace::RightBottomCorner },
            {   53, CoordinatesPlace::NearMouseCursor },
            {   54, CoordinatesPlace::Custom },
        }),
        // enum 'CoordinateSystem'
        QtMocHelpers::EnumData<enum CoordinateSystem>(55, 55, QMC::EnumFlags{}).add({
            {   56, CoordinateSystem::RaDecJ2000 },
            {   57, CoordinateSystem::RaDec },
            {   58, CoordinateSystem::HourAngle },
            {   59, CoordinateSystem::Ecliptic },
            {   60, CoordinateSystem::EclipticJ2000 },
            {   61, CoordinateSystem::AltAzi },
            {   62, CoordinateSystem::Galactic },
            {   63, CoordinateSystem::Supergalactic },
        }),
    };
    return QtMocHelpers::metaObjectData<PointerCoordinates, qt_meta_tag_ZN18PointerCoordinatesE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject PointerCoordinates::staticMetaObject = { {
    QMetaObject::SuperData::link<StelModule::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18PointerCoordinatesE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18PointerCoordinatesE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18PointerCoordinatesE_t>.metaTypes,
    nullptr
} };

void PointerCoordinates::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PointerCoordinates *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->flagCoordinatesVisibilityChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->flagEnableAtStartupChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->flagShowCoordinatesButtonChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->flagShowConstellationChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->flagShowCrossedLinesChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->flagShowElongationChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->fontSizeChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 7: _t->fontColorChanged((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1]))); break;
        case 8: _t->saveSettings(); break;
        case 9: _t->enableCoordinates((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 10: _t->setFlagEnableAtStartup((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 11: _t->setFontSize((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 12: _t->setFlagShowCoordinatesButton((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 13: _t->setFlagShowCrossedLines((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 14: _t->setCurrentCoordinatesPlace((*reinterpret_cast<std::add_pointer_t<PointerCoordinates::CoordinatesPlace>>(_a[1]))); break;
        case 15: { PointerCoordinates::CoordinatesPlace _r = _t->getCurrentCoordinatesPlace();
            if (_a[0]) *reinterpret_cast<PointerCoordinates::CoordinatesPlace*>(_a[0]) = std::move(_r); }  break;
        case 16: { QString _r = _t->getCurrentCoordinatesPlaceKey();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 17: _t->setCurrentCoordinatesPlaceKey((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 18: _t->setCurrentCoordinateSystem((*reinterpret_cast<std::add_pointer_t<PointerCoordinates::CoordinateSystem>>(_a[1]))); break;
        case 19: { PointerCoordinates::CoordinateSystem _r = _t->getCurrentCoordinateSystem();
            if (_a[0]) *reinterpret_cast<PointerCoordinates::CoordinateSystem*>(_a[0]) = std::move(_r); }  break;
        case 20: { QString _r = _t->getCurrentCoordinateSystemKey();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 21: _t->setCurrentCoordinateSystemKey((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 22: _t->setCustomCoordinatesPlace((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 23: _t->setFlagShowConstellation((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 24: _t->setFlagShowElongation((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 25: { Vec3f _r = _t->getFontColor();
            if (_a[0]) *reinterpret_cast<Vec3f*>(_a[0]) = std::move(_r); }  break;
        case 26: _t->setFontColor((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 26:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (PointerCoordinates::*)(bool )>(_a, &PointerCoordinates::flagCoordinatesVisibilityChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (PointerCoordinates::*)(bool )>(_a, &PointerCoordinates::flagEnableAtStartupChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (PointerCoordinates::*)(bool )>(_a, &PointerCoordinates::flagShowCoordinatesButtonChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (PointerCoordinates::*)(bool )>(_a, &PointerCoordinates::flagShowConstellationChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (PointerCoordinates::*)(bool )>(_a, &PointerCoordinates::flagShowCrossedLinesChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (PointerCoordinates::*)(bool )>(_a, &PointerCoordinates::flagShowElongationChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (PointerCoordinates::*)(int )>(_a, &PointerCoordinates::fontSizeChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (PointerCoordinates::*)(Vec3f )>(_a, &PointerCoordinates::fontColorChanged, 7))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 7:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Vec3f >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->isEnabled(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->getFlagEnableAtStartup(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->getFlagShowCoordinatesButton(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->getFlagShowConstellation(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->getFlagShowCrossedLines(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->getFlagShowElongation(); break;
        case 6: *reinterpret_cast<int*>(_v) = _t->getFontSize(); break;
        case 7: *reinterpret_cast<Vec3f*>(_v) = _t->getFontColor(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->enableCoordinates(*reinterpret_cast<bool*>(_v)); break;
        case 1: _t->setFlagEnableAtStartup(*reinterpret_cast<bool*>(_v)); break;
        case 2: _t->setFlagShowCoordinatesButton(*reinterpret_cast<bool*>(_v)); break;
        case 3: _t->setFlagShowConstellation(*reinterpret_cast<bool*>(_v)); break;
        case 4: _t->setFlagShowCrossedLines(*reinterpret_cast<bool*>(_v)); break;
        case 5: _t->setFlagShowElongation(*reinterpret_cast<bool*>(_v)); break;
        case 6: _t->setFontSize(*reinterpret_cast<int*>(_v)); break;
        case 7: _t->setFontColor(*reinterpret_cast<Vec3f*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *PointerCoordinates::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PointerCoordinates::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18PointerCoordinatesE_t>.strings))
        return static_cast<void*>(this);
    return StelModule::qt_metacast(_clname);
}

int PointerCoordinates::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StelModule::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 27)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 27;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 27)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 27;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void PointerCoordinates::flagCoordinatesVisibilityChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void PointerCoordinates::flagEnableAtStartupChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void PointerCoordinates::flagShowCoordinatesButtonChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void PointerCoordinates::flagShowConstellationChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void PointerCoordinates::flagShowCrossedLinesChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void PointerCoordinates::flagShowElongationChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void PointerCoordinates::fontSizeChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void PointerCoordinates::fontColorChanged(Vec3f _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN37PointerCoordinatesStelPluginInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto PointerCoordinatesStelPluginInterface::qt_create_metaobjectdata<qt_meta_tag_ZN37PointerCoordinatesStelPluginInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "PointerCoordinatesStelPluginInterface"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PointerCoordinatesStelPluginInterface, qt_meta_tag_ZN37PointerCoordinatesStelPluginInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject PointerCoordinatesStelPluginInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN37PointerCoordinatesStelPluginInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN37PointerCoordinatesStelPluginInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN37PointerCoordinatesStelPluginInterfaceE_t>.metaTypes,
    nullptr
} };

void PointerCoordinatesStelPluginInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PointerCoordinatesStelPluginInterface *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *PointerCoordinatesStelPluginInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PointerCoordinatesStelPluginInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN37PointerCoordinatesStelPluginInterfaceE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "StelPluginInterface"))
        return static_cast< StelPluginInterface*>(this);
    if (!strcmp(_clname, "org.stellarium.StelPluginInterface/2.0"))
        return static_cast< StelPluginInterface*>(this);
    return QObject::qt_metacast(_clname);
}

int PointerCoordinatesStelPluginInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}

#ifdef QT_MOC_EXPORT_PLUGIN_V2
static constexpr unsigned char qt_pluginMetaDataV2_PointerCoordinatesStelPluginInterface[] = {
    0xbf, 
    // "IID"
    0x02,  0x78,  0x26,  'o',  'r',  'g',  '.',  's', 
    't',  'e',  'l',  'l',  'a',  'r',  'i',  'u', 
    'm',  '.',  'S',  't',  'e',  'l',  'P',  'l', 
    'u',  'g',  'i',  'n',  'I',  'n',  't',  'e', 
    'r',  'f',  'a',  'c',  'e',  '/',  '2',  '.', 
    '0', 
    // "className"
    0x03,  0x78,  0x25,  'P',  'o',  'i',  'n',  't', 
    'e',  'r',  'C',  'o',  'o',  'r',  'd',  'i', 
    'n',  'a',  't',  'e',  's',  'S',  't',  'e', 
    'l',  'P',  'l',  'u',  'g',  'i',  'n',  'I', 
    'n',  't',  'e',  'r',  'f',  'a',  'c',  'e', 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN_V2(PointerCoordinatesStelPluginInterface, PointerCoordinatesStelPluginInterface, qt_pluginMetaDataV2_PointerCoordinatesStelPluginInterface)
#else
QT_PLUGIN_METADATA_SECTION
Q_CONSTINIT static constexpr unsigned char qt_pluginMetaData_PointerCoordinatesStelPluginInterface[] = {
    'Q', 'T', 'M', 'E', 'T', 'A', 'D', 'A', 'T', 'A', ' ', '!',
    // metadata version, Qt version, architectural requirements
    0, QT_VERSION_MAJOR, QT_VERSION_MINOR, qPluginArchRequirements(),
    0xbf, 
    // "IID"
    0x02,  0x78,  0x26,  'o',  'r',  'g',  '.',  's', 
    't',  'e',  'l',  'l',  'a',  'r',  'i',  'u', 
    'm',  '.',  'S',  't',  'e',  'l',  'P',  'l', 
    'u',  'g',  'i',  'n',  'I',  'n',  't',  'e', 
    'r',  'f',  'a',  'c',  'e',  '/',  '2',  '.', 
    '0', 
    // "className"
    0x03,  0x78,  0x25,  'P',  'o',  'i',  'n',  't', 
    'e',  'r',  'C',  'o',  'o',  'r',  'd',  'i', 
    'n',  'a',  't',  'e',  's',  'S',  't',  'e', 
    'l',  'P',  'l',  'u',  'g',  'i',  'n',  'I', 
    'n',  't',  'e',  'r',  'f',  'a',  'c',  'e', 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN(PointerCoordinatesStelPluginInterface, PointerCoordinatesStelPluginInterface)
#endif  // QT_MOC_EXPORT_PLUGIN_V2

QT_WARNING_POP
