/****************************************************************************
** Meta object code from reading C++ file 'StelGui.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../stellarium/src/gui/StelGui.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'StelGui.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN7StelGuiE_t {};
} // unnamed namespace

template <> constexpr inline auto StelGui::qt_create_metaobjectdata<qt_meta_tag_ZN7StelGuiE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "StelGui",
        "visibleChanged",
        "",
        "b",
        "autoHideHorizontalButtonBarChanged",
        "autoHideVerticalButtonBarChanged",
        "flagUseButtonsBackgroundChanged",
        "flagUseKineticScrollingChanged",
        "flagEnableFocusOnDaySpinnerChanged",
        "flagShowQuitButtonChanged",
        "flagShowFlipButtonsChanged",
        "flagShowNebulaBackgroundButtonChanged",
        "flagShowDSSButtonChanged",
        "flagShowHiPSButtonChanged",
        "flagShowGotoSelectedObjectButtonChanged",
        "flagShowNightmodeButtonChanged",
        "flagShowFullscreenButtonChanged",
        "flagShowObsListButtonChanged",
        "flagShowICRSGridButtonChanged",
        "flagShowGalacticGridButtonChanged",
        "flagShowEclipticGridButtonChanged",
        "flagShowConstellationBoundariesButtonChanged",
        "flagShowConstellationArtsButtonChanged",
        "flagShowAsterismLinesButtonChanged",
        "flagShowAsterismLabelsButtonChanged",
        "flagShowCardinalButtonChanged",
        "flagShowCompassButtonChanged",
        "infoStringChanged",
        "guiStyleChanged",
        "style",
        "htmlStyleChanged",
        "setFlagShowFlipButtons",
        "setFlagShowNebulaBackgroundButton",
        "setFlagShowObsListButton",
        "setFlagShowICRSGridButton",
        "setFlagShowGalacticGridButton",
        "setFlagShowEclipticGridButton",
        "setFlagShowConstellationBoundariesButton",
        "setFlagShowConstellationArtsButton",
        "setFlagShowAsterismLinesButton",
        "setFlagShowAsterismLabelsButton",
        "setFlagShowDSSButton",
        "setFlagShowHiPSButton",
        "setFlagShowGotoSelectedObjectButton",
        "setFlagShowNightmodeButton",
        "setFlagShowFullscreenButton",
        "setFlagShowQuitButton",
        "setFlagShowCardinalButton",
        "setFlagShowCompassButton",
        "setFlagUseButtonsBackground",
        "setFlagUseKineticScrolling",
        "setFlagEnableFocusOnDaySpinner",
        "setAutoHideHorizontalButtonBar",
        "setAutoHideVerticalButtonBar",
        "setGuiVisible",
        "setScriptKeys",
        "increaseScriptSpeed",
        "decreaseScriptSpeed",
        "setRealScriptSpeed",
        "stopScript",
        "displayAllInfo",
        "displayDefaultInfo",
        "displayShortInfo",
        "displayNoneInfo",
        "displayCustomInfo",
        "reloadStyle",
        "scriptStarted",
        "scriptStopped",
        "setStelStyle",
        "quit",
        "updateI18n",
        "copySelectedObjectInfo",
        "visible",
        "autoHideHorizontalButtonBar",
        "autoHideVerticalButtonBar",
        "flagShowQuitButton",
        "flagShowFlipButtons",
        "flagShowNebulaBackgroundButton",
        "flagShowDSSButton",
        "flagShowHiPSButton",
        "flagShowGotoSelectedObjectButton",
        "flagShowNightmodeButton",
        "flagShowFullscreenButton",
        "flagShowObsListButton",
        "flagShowICRSGridButton",
        "flagShowGalacticGridButton",
        "flagShowEclipticGridButton",
        "flagShowConstellationBoundariesButton",
        "flagShowConstellationArtsButton",
        "flagShowAsterismLinesButton",
        "flagShowAsterismLabelsButton",
        "flagUseButtonsBackground",
        "flagUseKineticScrolling",
        "flagEnableFocusOnDaySpinner",
        "flagShowCardinalButton",
        "flagShowCompassButton"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'visibleChanged'
        QtMocHelpers::SignalData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'autoHideHorizontalButtonBarChanged'
        QtMocHelpers::SignalData<void(bool)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'autoHideVerticalButtonBarChanged'
        QtMocHelpers::SignalData<void(bool)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagUseButtonsBackgroundChanged'
        QtMocHelpers::SignalData<void(bool)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagUseKineticScrollingChanged'
        QtMocHelpers::SignalData<void(bool)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagEnableFocusOnDaySpinnerChanged'
        QtMocHelpers::SignalData<void(bool)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagShowQuitButtonChanged'
        QtMocHelpers::SignalData<void(bool)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagShowFlipButtonsChanged'
        QtMocHelpers::SignalData<void(bool)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagShowNebulaBackgroundButtonChanged'
        QtMocHelpers::SignalData<void(bool)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagShowDSSButtonChanged'
        QtMocHelpers::SignalData<void(bool)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagShowHiPSButtonChanged'
        QtMocHelpers::SignalData<void(bool)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagShowGotoSelectedObjectButtonChanged'
        QtMocHelpers::SignalData<void(bool)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagShowNightmodeButtonChanged'
        QtMocHelpers::SignalData<void(bool)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagShowFullscreenButtonChanged'
        QtMocHelpers::SignalData<void(bool)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagShowObsListButtonChanged'
        QtMocHelpers::SignalData<void(bool)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagShowICRSGridButtonChanged'
        QtMocHelpers::SignalData<void(bool)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagShowGalacticGridButtonChanged'
        QtMocHelpers::SignalData<void(bool)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagShowEclipticGridButtonChanged'
        QtMocHelpers::SignalData<void(bool)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagShowConstellationBoundariesButtonChanged'
        QtMocHelpers::SignalData<void(bool)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagShowConstellationArtsButtonChanged'
        QtMocHelpers::SignalData<void(bool)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagShowAsterismLinesButtonChanged'
        QtMocHelpers::SignalData<void(bool)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagShowAsterismLabelsButtonChanged'
        QtMocHelpers::SignalData<void(bool)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagShowCardinalButtonChanged'
        QtMocHelpers::SignalData<void(bool)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagShowCompassButtonChanged'
        QtMocHelpers::SignalData<void(bool)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'infoStringChanged'
        QtMocHelpers::SignalData<void()>(27, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'guiStyleChanged'
        QtMocHelpers::SignalData<void(const QString &)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 29 },
        }}),
        // Signal 'htmlStyleChanged'
        QtMocHelpers::SignalData<void(const QString &)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 29 },
        }}),
        // Slot 'setFlagShowFlipButtons'
        QtMocHelpers::SlotData<void(bool)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'setFlagShowNebulaBackgroundButton'
        QtMocHelpers::SlotData<void(bool)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'setFlagShowObsListButton'
        QtMocHelpers::SlotData<void(bool)>(33, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'setFlagShowICRSGridButton'
        QtMocHelpers::SlotData<void(bool)>(34, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'setFlagShowGalacticGridButton'
        QtMocHelpers::SlotData<void(bool)>(35, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'setFlagShowEclipticGridButton'
        QtMocHelpers::SlotData<void(bool)>(36, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'setFlagShowConstellationBoundariesButton'
        QtMocHelpers::SlotData<void(bool)>(37, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'setFlagShowConstellationArtsButton'
        QtMocHelpers::SlotData<void(bool)>(38, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'setFlagShowAsterismLinesButton'
        QtMocHelpers::SlotData<void(bool)>(39, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'setFlagShowAsterismLabelsButton'
        QtMocHelpers::SlotData<void(bool)>(40, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'setFlagShowDSSButton'
        QtMocHelpers::SlotData<void(bool)>(41, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'setFlagShowHiPSButton'
        QtMocHelpers::SlotData<void(bool)>(42, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'setFlagShowGotoSelectedObjectButton'
        QtMocHelpers::SlotData<void(bool)>(43, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'setFlagShowNightmodeButton'
        QtMocHelpers::SlotData<void(bool)>(44, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'setFlagShowFullscreenButton'
        QtMocHelpers::SlotData<void(bool)>(45, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'setFlagShowQuitButton'
        QtMocHelpers::SlotData<void(bool)>(46, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'setFlagShowCardinalButton'
        QtMocHelpers::SlotData<void(bool)>(47, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'setFlagShowCompassButton'
        QtMocHelpers::SlotData<void(bool)>(48, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'setFlagUseButtonsBackground'
        QtMocHelpers::SlotData<void(bool)>(49, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'setFlagUseKineticScrolling'
        QtMocHelpers::SlotData<void(bool)>(50, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'setFlagEnableFocusOnDaySpinner'
        QtMocHelpers::SlotData<void(bool)>(51, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'setAutoHideHorizontalButtonBar'
        QtMocHelpers::SlotData<void(bool)>(52, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'setAutoHideVerticalButtonBar'
        QtMocHelpers::SlotData<void(bool)>(53, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'setGuiVisible'
        QtMocHelpers::SlotData<void(bool)>(54, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'setScriptKeys'
        QtMocHelpers::SlotData<void(bool)>(55, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'increaseScriptSpeed'
        QtMocHelpers::SlotData<void()>(56, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'decreaseScriptSpeed'
        QtMocHelpers::SlotData<void()>(57, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setRealScriptSpeed'
        QtMocHelpers::SlotData<void()>(58, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'stopScript'
        QtMocHelpers::SlotData<void()>(59, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'displayAllInfo'
        QtMocHelpers::SlotData<void()>(60, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'displayDefaultInfo'
        QtMocHelpers::SlotData<void()>(61, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'displayShortInfo'
        QtMocHelpers::SlotData<void()>(62, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'displayNoneInfo'
        QtMocHelpers::SlotData<void()>(63, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'displayCustomInfo'
        QtMocHelpers::SlotData<void()>(64, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'reloadStyle'
        QtMocHelpers::SlotData<void()>(65, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'scriptStarted'
        QtMocHelpers::SlotData<void()>(66, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'scriptStopped'
        QtMocHelpers::SlotData<void()>(67, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setStelStyle'
        QtMocHelpers::SlotData<void(const QString &)>(68, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 29 },
        }}),
        // Slot 'quit'
        QtMocHelpers::SlotData<void()>(69, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateI18n'
        QtMocHelpers::SlotData<void()>(70, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'copySelectedObjectInfo'
        QtMocHelpers::SlotData<void()>(71, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'visible'
        QtMocHelpers::PropertyData<bool>(72, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'autoHideHorizontalButtonBar'
        QtMocHelpers::PropertyData<bool>(73, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'autoHideVerticalButtonBar'
        QtMocHelpers::PropertyData<bool>(74, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'flagShowQuitButton'
        QtMocHelpers::PropertyData<bool>(75, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 6),
        // property 'flagShowFlipButtons'
        QtMocHelpers::PropertyData<bool>(76, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 7),
        // property 'flagShowNebulaBackgroundButton'
        QtMocHelpers::PropertyData<bool>(77, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 8),
        // property 'flagShowDSSButton'
        QtMocHelpers::PropertyData<bool>(78, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 9),
        // property 'flagShowHiPSButton'
        QtMocHelpers::PropertyData<bool>(79, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 10),
        // property 'flagShowGotoSelectedObjectButton'
        QtMocHelpers::PropertyData<bool>(80, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 11),
        // property 'flagShowNightmodeButton'
        QtMocHelpers::PropertyData<bool>(81, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 12),
        // property 'flagShowFullscreenButton'
        QtMocHelpers::PropertyData<bool>(82, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 13),
        // property 'flagShowObsListButton'
        QtMocHelpers::PropertyData<bool>(83, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 14),
        // property 'flagShowICRSGridButton'
        QtMocHelpers::PropertyData<bool>(84, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 15),
        // property 'flagShowGalacticGridButton'
        QtMocHelpers::PropertyData<bool>(85, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 16),
        // property 'flagShowEclipticGridButton'
        QtMocHelpers::PropertyData<bool>(86, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 17),
        // property 'flagShowConstellationBoundariesButton'
        QtMocHelpers::PropertyData<bool>(87, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 18),
        // property 'flagShowConstellationArtsButton'
        QtMocHelpers::PropertyData<bool>(88, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 19),
        // property 'flagShowAsterismLinesButton'
        QtMocHelpers::PropertyData<bool>(89, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 20),
        // property 'flagShowAsterismLabelsButton'
        QtMocHelpers::PropertyData<bool>(90, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 21),
        // property 'flagUseButtonsBackground'
        QtMocHelpers::PropertyData<bool>(91, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'flagUseKineticScrolling'
        QtMocHelpers::PropertyData<bool>(92, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'flagEnableFocusOnDaySpinner'
        QtMocHelpers::PropertyData<bool>(93, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
        // property 'flagShowCardinalButton'
        QtMocHelpers::PropertyData<bool>(94, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 22),
        // property 'flagShowCompassButton'
        QtMocHelpers::PropertyData<bool>(95, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 23),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<StelGui, qt_meta_tag_ZN7StelGuiE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject StelGui::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7StelGuiE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7StelGuiE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN7StelGuiE_t>.metaTypes,
    nullptr
} };

void StelGui::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<StelGui *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->visibleChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->autoHideHorizontalButtonBarChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->autoHideVerticalButtonBarChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->flagUseButtonsBackgroundChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->flagUseKineticScrollingChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->flagEnableFocusOnDaySpinnerChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->flagShowQuitButtonChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->flagShowFlipButtonsChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->flagShowNebulaBackgroundButtonChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 9: _t->flagShowDSSButtonChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 10: _t->flagShowHiPSButtonChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 11: _t->flagShowGotoSelectedObjectButtonChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 12: _t->flagShowNightmodeButtonChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 13: _t->flagShowFullscreenButtonChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 14: _t->flagShowObsListButtonChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 15: _t->flagShowICRSGridButtonChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 16: _t->flagShowGalacticGridButtonChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 17: _t->flagShowEclipticGridButtonChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 18: _t->flagShowConstellationBoundariesButtonChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 19: _t->flagShowConstellationArtsButtonChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 20: _t->flagShowAsterismLinesButtonChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 21: _t->flagShowAsterismLabelsButtonChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 22: _t->flagShowCardinalButtonChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 23: _t->flagShowCompassButtonChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 24: _t->infoStringChanged(); break;
        case 25: _t->guiStyleChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 26: _t->htmlStyleChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 27: _t->setFlagShowFlipButtons((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 28: _t->setFlagShowNebulaBackgroundButton((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 29: _t->setFlagShowObsListButton((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 30: _t->setFlagShowICRSGridButton((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 31: _t->setFlagShowGalacticGridButton((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 32: _t->setFlagShowEclipticGridButton((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 33: _t->setFlagShowConstellationBoundariesButton((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 34: _t->setFlagShowConstellationArtsButton((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 35: _t->setFlagShowAsterismLinesButton((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 36: _t->setFlagShowAsterismLabelsButton((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 37: _t->setFlagShowDSSButton((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 38: _t->setFlagShowHiPSButton((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 39: _t->setFlagShowGotoSelectedObjectButton((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 40: _t->setFlagShowNightmodeButton((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 41: _t->setFlagShowFullscreenButton((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 42: _t->setFlagShowQuitButton((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 43: _t->setFlagShowCardinalButton((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 44: _t->setFlagShowCompassButton((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 45: _t->setFlagUseButtonsBackground((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 46: _t->setFlagUseKineticScrolling((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 47: _t->setFlagEnableFocusOnDaySpinner((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 48: _t->setAutoHideHorizontalButtonBar((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 49: _t->setAutoHideVerticalButtonBar((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 50: _t->setGuiVisible((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 51: _t->setScriptKeys((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 52: _t->increaseScriptSpeed(); break;
        case 53: _t->decreaseScriptSpeed(); break;
        case 54: _t->setRealScriptSpeed(); break;
        case 55: _t->stopScript(); break;
        case 56: _t->displayAllInfo(); break;
        case 57: _t->displayDefaultInfo(); break;
        case 58: _t->displayShortInfo(); break;
        case 59: _t->displayNoneInfo(); break;
        case 60: _t->displayCustomInfo(); break;
        case 61: _t->reloadStyle(); break;
        case 62: _t->scriptStarted(); break;
        case 63: _t->scriptStopped(); break;
        case 64: _t->setStelStyle((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 65: _t->quit(); break;
        case 66: _t->updateI18n(); break;
        case 67: _t->copySelectedObjectInfo(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (StelGui::*)(bool )>(_a, &StelGui::visibleChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelGui::*)(bool )>(_a, &StelGui::autoHideHorizontalButtonBarChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelGui::*)(bool )>(_a, &StelGui::autoHideVerticalButtonBarChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelGui::*)(bool )>(_a, &StelGui::flagUseButtonsBackgroundChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelGui::*)(bool )>(_a, &StelGui::flagUseKineticScrollingChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelGui::*)(bool )>(_a, &StelGui::flagEnableFocusOnDaySpinnerChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelGui::*)(bool )>(_a, &StelGui::flagShowQuitButtonChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelGui::*)(bool )>(_a, &StelGui::flagShowFlipButtonsChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelGui::*)(bool )>(_a, &StelGui::flagShowNebulaBackgroundButtonChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelGui::*)(bool )>(_a, &StelGui::flagShowDSSButtonChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelGui::*)(bool )>(_a, &StelGui::flagShowHiPSButtonChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelGui::*)(bool )>(_a, &StelGui::flagShowGotoSelectedObjectButtonChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelGui::*)(bool )>(_a, &StelGui::flagShowNightmodeButtonChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelGui::*)(bool )>(_a, &StelGui::flagShowFullscreenButtonChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelGui::*)(bool )>(_a, &StelGui::flagShowObsListButtonChanged, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelGui::*)(bool )>(_a, &StelGui::flagShowICRSGridButtonChanged, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelGui::*)(bool )>(_a, &StelGui::flagShowGalacticGridButtonChanged, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelGui::*)(bool )>(_a, &StelGui::flagShowEclipticGridButtonChanged, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelGui::*)(bool )>(_a, &StelGui::flagShowConstellationBoundariesButtonChanged, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelGui::*)(bool )>(_a, &StelGui::flagShowConstellationArtsButtonChanged, 19))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelGui::*)(bool )>(_a, &StelGui::flagShowAsterismLinesButtonChanged, 20))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelGui::*)(bool )>(_a, &StelGui::flagShowAsterismLabelsButtonChanged, 21))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelGui::*)(bool )>(_a, &StelGui::flagShowCardinalButtonChanged, 22))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelGui::*)(bool )>(_a, &StelGui::flagShowCompassButtonChanged, 23))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelGui::*)()>(_a, &StelGui::infoStringChanged, 24))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelGui::*)(const QString & )>(_a, &StelGui::guiStyleChanged, 25))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelGui::*)(const QString & )>(_a, &StelGui::htmlStyleChanged, 26))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->getVisible(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->getAutoHideHorizontalButtonBar(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->getAutoHideVerticalButtonBar(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->getFlagShowQuitButton(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->getFlagShowFlipButtons(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->getFlagShowNebulaBackgroundButton(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->getFlagShowDSSButton(); break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->getFlagShowHiPSButton(); break;
        case 8: *reinterpret_cast<bool*>(_v) = _t->getFlagShowGotoSelectedObjectButton(); break;
        case 9: *reinterpret_cast<bool*>(_v) = _t->getFlagShowNightmodeButton(); break;
        case 10: *reinterpret_cast<bool*>(_v) = _t->getFlagShowFullscreenButton(); break;
        case 11: *reinterpret_cast<bool*>(_v) = _t->getFlagShowObsListButton(); break;
        case 12: *reinterpret_cast<bool*>(_v) = _t->getFlagShowICRSGridButton(); break;
        case 13: *reinterpret_cast<bool*>(_v) = _t->getFlagShowGalacticGridButton(); break;
        case 14: *reinterpret_cast<bool*>(_v) = _t->getFlagShowEclipticGridButton(); break;
        case 15: *reinterpret_cast<bool*>(_v) = _t->getFlagShowConstellationBoundariesButton(); break;
        case 16: *reinterpret_cast<bool*>(_v) = _t->getFlagShowConstellationArtsButton(); break;
        case 17: *reinterpret_cast<bool*>(_v) = _t->getFlagShowAsterismLinesButton(); break;
        case 18: *reinterpret_cast<bool*>(_v) = _t->getFlagShowAsterismLabelsButton(); break;
        case 19: *reinterpret_cast<bool*>(_v) = _t->getFlagUseButtonsBackground(); break;
        case 20: *reinterpret_cast<bool*>(_v) = _t->getFlagUseKineticScrolling(); break;
        case 21: *reinterpret_cast<bool*>(_v) = _t->getFlagEnableFocusOnDaySpinner(); break;
        case 22: *reinterpret_cast<bool*>(_v) = _t->getFlagShowCardinalButton(); break;
        case 23: *reinterpret_cast<bool*>(_v) = _t->getFlagShowCompassButton(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setVisible(*reinterpret_cast<bool*>(_v)); break;
        case 1: _t->setAutoHideHorizontalButtonBar(*reinterpret_cast<bool*>(_v)); break;
        case 2: _t->setAutoHideVerticalButtonBar(*reinterpret_cast<bool*>(_v)); break;
        case 3: _t->setFlagShowQuitButton(*reinterpret_cast<bool*>(_v)); break;
        case 4: _t->setFlagShowFlipButtons(*reinterpret_cast<bool*>(_v)); break;
        case 5: _t->setFlagShowNebulaBackgroundButton(*reinterpret_cast<bool*>(_v)); break;
        case 6: _t->setFlagShowDSSButton(*reinterpret_cast<bool*>(_v)); break;
        case 7: _t->setFlagShowHiPSButton(*reinterpret_cast<bool*>(_v)); break;
        case 8: _t->setFlagShowGotoSelectedObjectButton(*reinterpret_cast<bool*>(_v)); break;
        case 9: _t->setFlagShowNightmodeButton(*reinterpret_cast<bool*>(_v)); break;
        case 10: _t->setFlagShowFullscreenButton(*reinterpret_cast<bool*>(_v)); break;
        case 11: _t->setFlagShowObsListButton(*reinterpret_cast<bool*>(_v)); break;
        case 12: _t->setFlagShowICRSGridButton(*reinterpret_cast<bool*>(_v)); break;
        case 13: _t->setFlagShowGalacticGridButton(*reinterpret_cast<bool*>(_v)); break;
        case 14: _t->setFlagShowEclipticGridButton(*reinterpret_cast<bool*>(_v)); break;
        case 15: _t->setFlagShowConstellationBoundariesButton(*reinterpret_cast<bool*>(_v)); break;
        case 16: _t->setFlagShowConstellationArtsButton(*reinterpret_cast<bool*>(_v)); break;
        case 17: _t->setFlagShowAsterismLinesButton(*reinterpret_cast<bool*>(_v)); break;
        case 18: _t->setFlagShowAsterismLabelsButton(*reinterpret_cast<bool*>(_v)); break;
        case 19: _t->setFlagUseButtonsBackground(*reinterpret_cast<bool*>(_v)); break;
        case 20: _t->setFlagUseKineticScrolling(*reinterpret_cast<bool*>(_v)); break;
        case 21: _t->setFlagEnableFocusOnDaySpinner(*reinterpret_cast<bool*>(_v)); break;
        case 22: _t->setFlagShowCardinalButton(*reinterpret_cast<bool*>(_v)); break;
        case 23: _t->setFlagShowCompassButton(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *StelGui::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *StelGui::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7StelGuiE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "StelGuiBase"))
        return static_cast< StelGuiBase*>(this);
    return QObject::qt_metacast(_clname);
}

int StelGui::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 68)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 68;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 68)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 68;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 24;
    }
    return _id;
}

// SIGNAL 0
void StelGui::visibleChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void StelGui::autoHideHorizontalButtonBarChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void StelGui::autoHideVerticalButtonBarChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void StelGui::flagUseButtonsBackgroundChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void StelGui::flagUseKineticScrollingChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void StelGui::flagEnableFocusOnDaySpinnerChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void StelGui::flagShowQuitButtonChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void StelGui::flagShowFlipButtonsChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void StelGui::flagShowNebulaBackgroundButtonChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void StelGui::flagShowDSSButtonChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void StelGui::flagShowHiPSButtonChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void StelGui::flagShowGotoSelectedObjectButtonChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void StelGui::flagShowNightmodeButtonChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void StelGui::flagShowFullscreenButtonChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1);
}

// SIGNAL 14
void StelGui::flagShowObsListButtonChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1);
}

// SIGNAL 15
void StelGui::flagShowICRSGridButtonChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 15, nullptr, _t1);
}

// SIGNAL 16
void StelGui::flagShowGalacticGridButtonChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 16, nullptr, _t1);
}

// SIGNAL 17
void StelGui::flagShowEclipticGridButtonChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 17, nullptr, _t1);
}

// SIGNAL 18
void StelGui::flagShowConstellationBoundariesButtonChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 18, nullptr, _t1);
}

// SIGNAL 19
void StelGui::flagShowConstellationArtsButtonChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 19, nullptr, _t1);
}

// SIGNAL 20
void StelGui::flagShowAsterismLinesButtonChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 20, nullptr, _t1);
}

// SIGNAL 21
void StelGui::flagShowAsterismLabelsButtonChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 21, nullptr, _t1);
}

// SIGNAL 22
void StelGui::flagShowCardinalButtonChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 22, nullptr, _t1);
}

// SIGNAL 23
void StelGui::flagShowCompassButtonChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 23, nullptr, _t1);
}

// SIGNAL 24
void StelGui::infoStringChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 24, nullptr);
}

// SIGNAL 25
void StelGui::guiStyleChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 25, nullptr, _t1);
}

// SIGNAL 26
void StelGui::htmlStyleChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 26, nullptr, _t1);
}
QT_WARNING_POP
