/********************************************************************************
** Form generated from reading UI file 'storedViewDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_STOREDVIEWDIALOG_H
#define UI_STOREDVIEWDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListView>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "Dialog.hpp"
#include "StoredViewDialog_p.hpp"
#include "gui/StelCloseButton.hpp"

QT_BEGIN_NAMESPACE

class Ui_storedViewDialogForm
{
public:
    QVBoxLayout *vboxLayout;
    TitleBar *titleBar;
    QFrame *viewContent;
    QGridLayout *gridLayout;
    QWidget *widget;
    QGridLayout *gridLayout_3;
    QVBoxLayout *verticalLayout;
    QListView *listView;
    QPushButton *pushButtonLoadView;
    QPushButton *pushButtonDeleteView;
    QPushButton *pushButtonAddView;
    QCheckBox *useDateCheckBox;
    QGridLayout *gridLayout_2;
    QLineEdit *lineEditTitle;
    QLabel *label;
    QLabel *label_2;
    CustomTextEdit *textEditDescription;

    void setupUi(QWidget *storedViewDialogForm)
    {
        if (storedViewDialogForm->objectName().isEmpty())
            storedViewDialogForm->setObjectName("storedViewDialogForm");
        storedViewDialogForm->resize(349, 359);
        QFont font;
        storedViewDialogForm->setFont(font);
        storedViewDialogForm->setAutoFillBackground(false);
        storedViewDialogForm->setStyleSheet(QString::fromUtf8(""));
        vboxLayout = new QVBoxLayout(storedViewDialogForm);
        vboxLayout->setSpacing(0);
        vboxLayout->setObjectName("vboxLayout");
        vboxLayout->setContentsMargins(0, 0, 0, 0);
        titleBar = new TitleBar(storedViewDialogForm);
        titleBar->setObjectName("titleBar");

        vboxLayout->addWidget(titleBar);

        viewContent = new QFrame(storedViewDialogForm);
        viewContent->setObjectName("viewContent");
        viewContent->setFrameShape(QFrame::StyledPanel);
        gridLayout = new QGridLayout(viewContent);
        gridLayout->setObjectName("gridLayout");
        widget = new QWidget(viewContent);
        widget->setObjectName("widget");
        gridLayout_3 = new QGridLayout(widget);
        gridLayout_3->setObjectName("gridLayout_3");
        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName("verticalLayout");
        listView = new QListView(widget);
        listView->setObjectName("listView");

        verticalLayout->addWidget(listView);

        pushButtonLoadView = new QPushButton(widget);
        pushButtonLoadView->setObjectName("pushButtonLoadView");
        pushButtonLoadView->setEnabled(false);

        verticalLayout->addWidget(pushButtonLoadView);

        pushButtonDeleteView = new QPushButton(widget);
        pushButtonDeleteView->setObjectName("pushButtonDeleteView");
        pushButtonDeleteView->setEnabled(false);

        verticalLayout->addWidget(pushButtonDeleteView);

        pushButtonAddView = new QPushButton(widget);
        pushButtonAddView->setObjectName("pushButtonAddView");
        pushButtonAddView->setEnabled(false);

        verticalLayout->addWidget(pushButtonAddView);

        useDateCheckBox = new QCheckBox(widget);
        useDateCheckBox->setObjectName("useDateCheckBox");

        verticalLayout->addWidget(useDateCheckBox);


        gridLayout_3->addLayout(verticalLayout, 0, 0, 1, 1);

        gridLayout_2 = new QGridLayout();
        gridLayout_2->setObjectName("gridLayout_2");
        lineEditTitle = new QLineEdit(widget);
        lineEditTitle->setObjectName("lineEditTitle");
        lineEditTitle->setReadOnly(true);

        gridLayout_2->addWidget(lineEditTitle, 2, 0, 1, 1);

        label = new QLabel(widget);
        label->setObjectName("label");

        gridLayout_2->addWidget(label, 1, 0, 1, 1);

        label_2 = new QLabel(widget);
        label_2->setObjectName("label_2");

        gridLayout_2->addWidget(label_2, 3, 0, 1, 1);

        textEditDescription = new CustomTextEdit(widget);
        textEditDescription->setObjectName("textEditDescription");

        gridLayout_2->addWidget(textEditDescription, 4, 0, 1, 1);


        gridLayout_3->addLayout(gridLayout_2, 0, 1, 1, 1);


        gridLayout->addWidget(widget, 0, 0, 1, 1);


        vboxLayout->addWidget(viewContent);


        retranslateUi(storedViewDialogForm);

        QMetaObject::connectSlotsByName(storedViewDialogForm);
    } // setupUi

    void retranslateUi(QWidget *storedViewDialogForm)
    {
        titleBar->setTitle(QCoreApplication::translate("storedViewDialogForm", "Stored views", nullptr));
        pushButtonLoadView->setText(QCoreApplication::translate("storedViewDialogForm", "Load view", nullptr));
        pushButtonDeleteView->setText(QCoreApplication::translate("storedViewDialogForm", "Delete view", nullptr));
        pushButtonAddView->setText(QCoreApplication::translate("storedViewDialogForm", "Add current view", nullptr));
        useDateCheckBox->setText(QCoreApplication::translate("storedViewDialogForm", "Load/store with date", nullptr));
        label->setText(QCoreApplication::translate("storedViewDialogForm", "Title:", nullptr));
        label_2->setText(QCoreApplication::translate("storedViewDialogForm", "Description:", nullptr));
        (void)storedViewDialogForm;
    } // retranslateUi

};

namespace Ui {
    class storedViewDialogForm: public Ui_storedViewDialogForm {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_STOREDVIEWDIALOG_H
