/****************************************************************************
** Meta object code from reading C++ file 'TelescopeConfigurationDialog.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../../stellarium/plugins/TelescopeControl/src/gui/TelescopeConfigurationDialog.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'TelescopeConfigurationDialog.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN28TelescopeConfigurationDialogE_t {};
} // unnamed namespace

template <> constexpr inline auto TelescopeConfigurationDialog::qt_create_metaobjectdata<qt_meta_tag_ZN28TelescopeConfigurationDialogE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TelescopeConfigurationDialog",
        "changesSaved",
        "",
        "name",
        "TelescopeControl::ConnectionType",
        "type",
        "changesDiscarded",
        "retranslate",
        "buttonSavePressed",
        "buttonDiscardPressed",
        "toggleTypeLocal",
        "toggleTypeConnection",
        "toggleTypeVirtual",
        "toggleTypeRTS2",
        "toggleTypeINDI",
        "enabled",
        "deviceModelSelected",
        "modelIndex"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'changesSaved'
        QtMocHelpers::SignalData<void(QString, TelescopeControl::ConnectionType)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { 0x80000000 | 4, 5 },
        }}),
        // Signal 'changesDiscarded'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'retranslate'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'buttonSavePressed'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'buttonDiscardPressed'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'toggleTypeLocal'
        QtMocHelpers::SlotData<void(bool)>(10, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'toggleTypeConnection'
        QtMocHelpers::SlotData<void(bool)>(11, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'toggleTypeVirtual'
        QtMocHelpers::SlotData<void(bool)>(12, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'toggleTypeRTS2'
        QtMocHelpers::SlotData<void(bool)>(13, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'toggleTypeINDI'
        QtMocHelpers::SlotData<void(bool)>(14, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 15 },
        }}),
        // Slot 'deviceModelSelected'
        QtMocHelpers::SlotData<void(int)>(16, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 17 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TelescopeConfigurationDialog, qt_meta_tag_ZN28TelescopeConfigurationDialogE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject TelescopeConfigurationDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<StelDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28TelescopeConfigurationDialogE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28TelescopeConfigurationDialogE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN28TelescopeConfigurationDialogE_t>.metaTypes,
    nullptr
} };

void TelescopeConfigurationDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TelescopeConfigurationDialog *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->changesSaved((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<TelescopeControl::ConnectionType>>(_a[2]))); break;
        case 1: _t->changesDiscarded(); break;
        case 2: _t->retranslate(); break;
        case 3: _t->buttonSavePressed(); break;
        case 4: _t->buttonDiscardPressed(); break;
        case 5: _t->toggleTypeLocal((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->toggleTypeConnection((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->toggleTypeVirtual((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->toggleTypeRTS2((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 9: _t->toggleTypeINDI((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 10: _t->deviceModelSelected((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TelescopeConfigurationDialog::*)(QString , TelescopeControl::ConnectionType )>(_a, &TelescopeConfigurationDialog::changesSaved, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (TelescopeConfigurationDialog::*)()>(_a, &TelescopeConfigurationDialog::changesDiscarded, 1))
            return;
    }
}

const QMetaObject *TelescopeConfigurationDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TelescopeConfigurationDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28TelescopeConfigurationDialogE_t>.strings))
        return static_cast<void*>(this);
    return StelDialog::qt_metacast(_clname);
}

int TelescopeConfigurationDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StelDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void TelescopeConfigurationDialog::changesSaved(QString _t1, TelescopeControl::ConnectionType _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void TelescopeConfigurationDialog::changesDiscarded()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
