/****************************************************************************
** Meta object code from reading C++ file 'SlewDialog.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../../stellarium/plugins/TelescopeControl/src/gui/SlewDialog.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'SlewDialog.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10SlewDialogE_t {};
} // unnamed namespace

template <> constexpr inline auto SlewDialog::qt_create_metaobjectdata<qt_meta_tag_ZN10SlewDialogE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "SlewDialog",
        "retranslate",
        "",
        "showConfiguration",
        "slew",
        "sync",
        "abort",
        "addTelescope",
        "slot",
        "name",
        "removeTelescope",
        "setFormatHMS",
        "set",
        "setFormatDMS",
        "setFormatDecimal",
        "getCurrentObjectInfo",
        "getCenterInfo",
        "editStoredPoints",
        "addStoredPointToComboBox",
        "number",
        "radiansRA",
        "radiansDec",
        "removeStoredPointFromComboBox",
        "clearStoredPointsFromComboBox",
        "getStoredPointInfo",
        "onMove",
        "angle",
        "speed",
        "onCurrentTelescopeChanged"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'retranslate'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showConfiguration'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slew'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'sync'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'abort'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'addTelescope'
        QtMocHelpers::SlotData<void(int, QString)>(7, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 8 }, { QMetaType::QString, 9 },
        }}),
        // Slot 'removeTelescope'
        QtMocHelpers::SlotData<void(int)>(10, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 8 },
        }}),
        // Slot 'setFormatHMS'
        QtMocHelpers::SlotData<void(bool)>(11, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 12 },
        }}),
        // Slot 'setFormatDMS'
        QtMocHelpers::SlotData<void(bool)>(13, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 12 },
        }}),
        // Slot 'setFormatDecimal'
        QtMocHelpers::SlotData<void(bool)>(14, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 12 },
        }}),
        // Slot 'getCurrentObjectInfo'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'getCenterInfo'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'editStoredPoints'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'addStoredPointToComboBox'
        QtMocHelpers::SlotData<void(int, QString, double, double)>(18, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 19 }, { QMetaType::QString, 9 }, { QMetaType::Double, 20 }, { QMetaType::Double, 21 },
        }}),
        // Slot 'removeStoredPointFromComboBox'
        QtMocHelpers::SlotData<void(int)>(22, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 19 },
        }}),
        // Slot 'clearStoredPointsFromComboBox'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'getStoredPointInfo'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onMove'
        QtMocHelpers::SlotData<void(double, double)>(25, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 26 }, { QMetaType::Double, 27 },
        }}),
        // Slot 'onCurrentTelescopeChanged'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<SlewDialog, qt_meta_tag_ZN10SlewDialogE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject SlewDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<StelDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10SlewDialogE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10SlewDialogE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10SlewDialogE_t>.metaTypes,
    nullptr
} };

void SlewDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SlewDialog *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->retranslate(); break;
        case 1: _t->showConfiguration(); break;
        case 2: _t->slew(); break;
        case 3: _t->sync(); break;
        case 4: _t->abort(); break;
        case 5: _t->addTelescope((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 6: _t->removeTelescope((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 7: _t->setFormatHMS((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->setFormatDMS((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 9: _t->setFormatDecimal((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 10: _t->getCurrentObjectInfo(); break;
        case 11: _t->getCenterInfo(); break;
        case 12: _t->editStoredPoints(); break;
        case 13: _t->addStoredPointToComboBox((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[4]))); break;
        case 14: _t->removeStoredPointFromComboBox((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 15: _t->clearStoredPointsFromComboBox(); break;
        case 16: _t->getStoredPointInfo(); break;
        case 17: _t->onMove((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 18: _t->onCurrentTelescopeChanged(); break;
        default: ;
        }
    }
}

const QMetaObject *SlewDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SlewDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10SlewDialogE_t>.strings))
        return static_cast<void*>(this);
    return StelDialog::qt_metacast(_clname);
}

int SlewDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StelDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 19)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 19;
    }
    return _id;
}
QT_WARNING_POP
