/********************************************************************************
** Form generated from reading UI file 'exoplanetsDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_EXOPLANETSDIALOG_H
#define UI_EXOPLANETSDIALOG_H

#include <QtCharts/QChartView>
#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDateTimeEdit>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTextBrowser>
#include <QtWidgets/QTreeWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "Dialog.hpp"
#include "gui/ColorButton.hpp"

QT_BEGIN_NAMESPACE

class Ui_exoplanetsDialog
{
public:
    QVBoxLayout *verticalLayoutEP;
    TitleBar *titleBar;
    QTabWidget *tabs;
    QWidget *settingsTab;
    QVBoxLayout *verticalLayout_3;
    QGroupBox *updatesGroup;
    QVBoxLayout *verticalLayout_12;
    QCheckBox *internetUpdatesCheckbox;
    QGridLayout *gridLayout_4;
    QHBoxLayout *horizontalLayout_6;
    QLabel *lastUpdateLabel;
    QSpacerItem *horizontalSpacer_2;
    QDateTimeEdit *lastUpdateDateTimeEdit;
    QHBoxLayout *horizontalLayout_10;
    QLabel *updateFrequencyLabel;
    QSpacerItem *horizontalSpacer_3;
    QSpinBox *updateFrequencySpinBox;
    QLabel *nextUpdateLabel;
    QPushButton *updateButton;
    QGroupBox *displayBox;
    QVBoxLayout *verticalLayout;
    QCheckBox *displayModeCheckBox;
    QCheckBox *timelineModeCheckBox;
    QCheckBox *habitableModeCheckBox;
    QCheckBox *displayShowDesignationsCheckBox;
    QCheckBox *displayShowNumbersCheckBox;
    QCheckBox *displayAtStartupCheckBox;
    QCheckBox *displayShowExoplanetsButton;
    QHBoxLayout *horizontalLayout;
    QLabel *label;
    ColorButton *exoplanetMarkerColor;
    QHBoxLayout *horizontalLayout_4;
    QLabel *label_2;
    ColorButton *habitableExoplanetMarkerColor;
    QHBoxLayout *horizontalLayout_5;
    QLabel *temperatureScaleLabel;
    QComboBox *temperatureScaleComboBox;
    QSpacerItem *verticalSpacer_3;
    QHBoxLayout *horizontalLayout_7;
    QPushButton *restoreDefaultsButton;
    QSpacerItem *horizontalSpacer;
    QPushButton *saveSettingsButton;
    QWidget *diagramTab;
    QGridLayout *gridLayout_3;
    QGroupBox *diagramGroupBox;
    QGridLayout *gridLayout_2;
    QVBoxLayout *verticalLayout_5;
    QHBoxLayout *horizontalLayout_9;
    QLabel *labelY;
    QCheckBox *checkBoxLogY;
    QComboBox *comboAxisY;
    QHBoxLayout *horizontalLayout_3;
    QLineEdit *minY;
    QLabel *labelMinMaxY;
    QLineEdit *maxY;
    QVBoxLayout *verticalLayout_4;
    QHBoxLayout *horizontalLayout_8;
    QLabel *labelX;
    QCheckBox *checkBoxLogX;
    QComboBox *comboAxisX;
    QHBoxLayout *horizontalLayout_2;
    QLineEdit *minX;
    QLabel *labelMinMaxX;
    QLineEdit *maxX;
    QChartView *chartView;
    QWidget *tableTab;
    QGridLayout *gridLayout;
    QTreeWidget *exoplanetsTreeWidget;
    QWidget *infoTab;
    QVBoxLayout *verticalLayoutInfoBox;
    QTextBrowser *infoTextBrowser;
    QWidget *websitesTab;
    QVBoxLayout *verticalLayoutWebsitesBox;
    QTextBrowser *websitesTextBrowser;
    QWidget *aboutTab;
    QVBoxLayout *verticalLayoutAboutBox;
    QTextBrowser *aboutTextBrowser;

    void setupUi(QWidget *exoplanetsDialog)
    {
        if (exoplanetsDialog->objectName().isEmpty())
            exoplanetsDialog->setObjectName("exoplanetsDialog");
        exoplanetsDialog->resize(530, 568);
        verticalLayoutEP = new QVBoxLayout(exoplanetsDialog);
        verticalLayoutEP->setSpacing(0);
        verticalLayoutEP->setObjectName("verticalLayoutEP");
        verticalLayoutEP->setContentsMargins(0, 0, 0, 0);
        titleBar = new TitleBar(exoplanetsDialog);
        titleBar->setObjectName("titleBar");

        verticalLayoutEP->addWidget(titleBar);

        tabs = new QTabWidget(exoplanetsDialog);
        tabs->setObjectName("tabs");
        settingsTab = new QWidget();
        settingsTab->setObjectName("settingsTab");
        verticalLayout_3 = new QVBoxLayout(settingsTab);
        verticalLayout_3->setObjectName("verticalLayout_3");
        updatesGroup = new QGroupBox(settingsTab);
        updatesGroup->setObjectName("updatesGroup");
        updatesGroup->setFlat(true);
        updatesGroup->setCheckable(false);
        verticalLayout_12 = new QVBoxLayout(updatesGroup);
        verticalLayout_12->setObjectName("verticalLayout_12");
        verticalLayout_12->setContentsMargins(0, 0, 0, 0);
        internetUpdatesCheckbox = new QCheckBox(updatesGroup);
        internetUpdatesCheckbox->setObjectName("internetUpdatesCheckbox");

        verticalLayout_12->addWidget(internetUpdatesCheckbox);

        gridLayout_4 = new QGridLayout();
        gridLayout_4->setObjectName("gridLayout_4");
        horizontalLayout_6 = new QHBoxLayout();
        horizontalLayout_6->setObjectName("horizontalLayout_6");
        lastUpdateLabel = new QLabel(updatesGroup);
        lastUpdateLabel->setObjectName("lastUpdateLabel");

        horizontalLayout_6->addWidget(lastUpdateLabel);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_6->addItem(horizontalSpacer_2);

        lastUpdateDateTimeEdit = new QDateTimeEdit(updatesGroup);
        lastUpdateDateTimeEdit->setObjectName("lastUpdateDateTimeEdit");
        lastUpdateDateTimeEdit->setEnabled(false);
        lastUpdateDateTimeEdit->setFrame(false);
        lastUpdateDateTimeEdit->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        lastUpdateDateTimeEdit->setButtonSymbols(QAbstractSpinBox::NoButtons);

        horizontalLayout_6->addWidget(lastUpdateDateTimeEdit);


        gridLayout_4->addLayout(horizontalLayout_6, 0, 0, 1, 1);

        horizontalLayout_10 = new QHBoxLayout();
        horizontalLayout_10->setObjectName("horizontalLayout_10");
        updateFrequencyLabel = new QLabel(updatesGroup);
        updateFrequencyLabel->setObjectName("updateFrequencyLabel");

        horizontalLayout_10->addWidget(updateFrequencyLabel);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_10->addItem(horizontalSpacer_3);

        updateFrequencySpinBox = new QSpinBox(updatesGroup);
        updateFrequencySpinBox->setObjectName("updateFrequencySpinBox");
        updateFrequencySpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        updateFrequencySpinBox->setMinimum(1);
        updateFrequencySpinBox->setMaximum(9999);
        updateFrequencySpinBox->setValue(1);

        horizontalLayout_10->addWidget(updateFrequencySpinBox);


        gridLayout_4->addLayout(horizontalLayout_10, 0, 1, 1, 1);

        nextUpdateLabel = new QLabel(updatesGroup);
        nextUpdateLabel->setObjectName("nextUpdateLabel");

        gridLayout_4->addWidget(nextUpdateLabel, 1, 0, 1, 1);

        updateButton = new QPushButton(updatesGroup);
        updateButton->setObjectName("updateButton");

        gridLayout_4->addWidget(updateButton, 1, 1, 1, 1);


        verticalLayout_12->addLayout(gridLayout_4);


        verticalLayout_3->addWidget(updatesGroup);

        displayBox = new QGroupBox(settingsTab);
        displayBox->setObjectName("displayBox");
        verticalLayout = new QVBoxLayout(displayBox);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        displayModeCheckBox = new QCheckBox(displayBox);
        displayModeCheckBox->setObjectName("displayModeCheckBox");

        verticalLayout->addWidget(displayModeCheckBox);

        timelineModeCheckBox = new QCheckBox(displayBox);
        timelineModeCheckBox->setObjectName("timelineModeCheckBox");

        verticalLayout->addWidget(timelineModeCheckBox);

        habitableModeCheckBox = new QCheckBox(displayBox);
        habitableModeCheckBox->setObjectName("habitableModeCheckBox");

        verticalLayout->addWidget(habitableModeCheckBox);

        displayShowDesignationsCheckBox = new QCheckBox(displayBox);
        displayShowDesignationsCheckBox->setObjectName("displayShowDesignationsCheckBox");

        verticalLayout->addWidget(displayShowDesignationsCheckBox);

        displayShowNumbersCheckBox = new QCheckBox(displayBox);
        displayShowNumbersCheckBox->setObjectName("displayShowNumbersCheckBox");

        verticalLayout->addWidget(displayShowNumbersCheckBox);

        displayAtStartupCheckBox = new QCheckBox(displayBox);
        displayAtStartupCheckBox->setObjectName("displayAtStartupCheckBox");

        verticalLayout->addWidget(displayAtStartupCheckBox);

        displayShowExoplanetsButton = new QCheckBox(displayBox);
        displayShowExoplanetsButton->setObjectName("displayShowExoplanetsButton");

        verticalLayout->addWidget(displayShowExoplanetsButton);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        label = new QLabel(displayBox);
        label->setObjectName("label");

        horizontalLayout->addWidget(label);

        exoplanetMarkerColor = new ColorButton(displayBox);
        exoplanetMarkerColor->setObjectName("exoplanetMarkerColor");

        horizontalLayout->addWidget(exoplanetMarkerColor);


        verticalLayout->addLayout(horizontalLayout);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        label_2 = new QLabel(displayBox);
        label_2->setObjectName("label_2");

        horizontalLayout_4->addWidget(label_2);

        habitableExoplanetMarkerColor = new ColorButton(displayBox);
        habitableExoplanetMarkerColor->setObjectName("habitableExoplanetMarkerColor");

        horizontalLayout_4->addWidget(habitableExoplanetMarkerColor);


        verticalLayout->addLayout(horizontalLayout_4);

        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        temperatureScaleLabel = new QLabel(displayBox);
        temperatureScaleLabel->setObjectName("temperatureScaleLabel");

        horizontalLayout_5->addWidget(temperatureScaleLabel);

        temperatureScaleComboBox = new QComboBox(displayBox);
        temperatureScaleComboBox->setObjectName("temperatureScaleComboBox");
        temperatureScaleComboBox->setEditable(true);
        temperatureScaleComboBox->setInsertPolicy(QComboBox::NoInsert);

        horizontalLayout_5->addWidget(temperatureScaleComboBox);


        verticalLayout->addLayout(horizontalLayout_5);


        verticalLayout_3->addWidget(displayBox);

        verticalSpacer_3 = new QSpacerItem(20, 120, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_3->addItem(verticalSpacer_3);

        horizontalLayout_7 = new QHBoxLayout();
        horizontalLayout_7->setObjectName("horizontalLayout_7");
        restoreDefaultsButton = new QPushButton(settingsTab);
        restoreDefaultsButton->setObjectName("restoreDefaultsButton");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(restoreDefaultsButton->sizePolicy().hasHeightForWidth());
        restoreDefaultsButton->setSizePolicy(sizePolicy);

        horizontalLayout_7->addWidget(restoreDefaultsButton);

        horizontalSpacer = new QSpacerItem(20, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout_7->addItem(horizontalSpacer);

        saveSettingsButton = new QPushButton(settingsTab);
        saveSettingsButton->setObjectName("saveSettingsButton");
        sizePolicy.setHeightForWidth(saveSettingsButton->sizePolicy().hasHeightForWidth());
        saveSettingsButton->setSizePolicy(sizePolicy);

        horizontalLayout_7->addWidget(saveSettingsButton);


        verticalLayout_3->addLayout(horizontalLayout_7);

        tabs->addTab(settingsTab, QString());
        diagramTab = new QWidget();
        diagramTab->setObjectName("diagramTab");
        gridLayout_3 = new QGridLayout(diagramTab);
        gridLayout_3->setObjectName("gridLayout_3");
        gridLayout_3->setContentsMargins(6, 6, 6, 6);
        diagramGroupBox = new QGroupBox(diagramTab);
        diagramGroupBox->setObjectName("diagramGroupBox");
        gridLayout_2 = new QGridLayout(diagramGroupBox);
        gridLayout_2->setObjectName("gridLayout_2");
        gridLayout_2->setContentsMargins(0, 0, 0, 0);
        verticalLayout_5 = new QVBoxLayout();
        verticalLayout_5->setObjectName("verticalLayout_5");
        horizontalLayout_9 = new QHBoxLayout();
        horizontalLayout_9->setObjectName("horizontalLayout_9");
        labelY = new QLabel(diagramGroupBox);
        labelY->setObjectName("labelY");
        QFont font;
        font.setBold(true);
        labelY->setFont(font);

        horizontalLayout_9->addWidget(labelY);

        checkBoxLogY = new QCheckBox(diagramGroupBox);
        checkBoxLogY->setObjectName("checkBoxLogY");

        horizontalLayout_9->addWidget(checkBoxLogY, 0, Qt::AlignRight);


        verticalLayout_5->addLayout(horizontalLayout_9);

        comboAxisY = new QComboBox(diagramGroupBox);
        comboAxisY->setObjectName("comboAxisY");
        comboAxisY->setEditable(true);
        comboAxisY->setInsertPolicy(QComboBox::NoInsert);

        verticalLayout_5->addWidget(comboAxisY);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        minY = new QLineEdit(diagramGroupBox);
        minY->setObjectName("minY");
        minY->setInputMethodHints(Qt::ImhDigitsOnly);

        horizontalLayout_3->addWidget(minY);

        labelMinMaxY = new QLabel(diagramGroupBox);
        labelMinMaxY->setObjectName("labelMinMaxY");
        labelMinMaxY->setText(QString::fromUtf8("-"));

        horizontalLayout_3->addWidget(labelMinMaxY);

        maxY = new QLineEdit(diagramGroupBox);
        maxY->setObjectName("maxY");
        maxY->setInputMethodHints(Qt::ImhDigitsOnly);

        horizontalLayout_3->addWidget(maxY);


        verticalLayout_5->addLayout(horizontalLayout_3);


        gridLayout_2->addLayout(verticalLayout_5, 0, 1, 1, 1);

        verticalLayout_4 = new QVBoxLayout();
        verticalLayout_4->setObjectName("verticalLayout_4");
        horizontalLayout_8 = new QHBoxLayout();
        horizontalLayout_8->setObjectName("horizontalLayout_8");
        labelX = new QLabel(diagramGroupBox);
        labelX->setObjectName("labelX");
        labelX->setFont(font);

        horizontalLayout_8->addWidget(labelX);

        checkBoxLogX = new QCheckBox(diagramGroupBox);
        checkBoxLogX->setObjectName("checkBoxLogX");

        horizontalLayout_8->addWidget(checkBoxLogX, 0, Qt::AlignRight);


        verticalLayout_4->addLayout(horizontalLayout_8);

        comboAxisX = new QComboBox(diagramGroupBox);
        comboAxisX->setObjectName("comboAxisX");
        comboAxisX->setEditable(true);
        comboAxisX->setInsertPolicy(QComboBox::NoInsert);

        verticalLayout_4->addWidget(comboAxisX);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        minX = new QLineEdit(diagramGroupBox);
        minX->setObjectName("minX");
        minX->setInputMethodHints(Qt::ImhDigitsOnly);

        horizontalLayout_2->addWidget(minX);

        labelMinMaxX = new QLabel(diagramGroupBox);
        labelMinMaxX->setObjectName("labelMinMaxX");
        labelMinMaxX->setText(QString::fromUtf8("-"));

        horizontalLayout_2->addWidget(labelMinMaxX);

        maxX = new QLineEdit(diagramGroupBox);
        maxX->setObjectName("maxX");
        maxX->setInputMethodHints(Qt::ImhDigitsOnly);

        horizontalLayout_2->addWidget(maxX);


        verticalLayout_4->addLayout(horizontalLayout_2);


        gridLayout_2->addLayout(verticalLayout_4, 0, 0, 1, 1);


        gridLayout_3->addWidget(diagramGroupBox, 0, 0, 2, 1);

        chartView = new QChartView(diagramTab);
        chartView->setObjectName("chartView");

        gridLayout_3->addWidget(chartView, 2, 0, 1, 1);

        tabs->addTab(diagramTab, QString());
        tableTab = new QWidget();
        tableTab->setObjectName("tableTab");
        gridLayout = new QGridLayout(tableTab);
        gridLayout->setObjectName("gridLayout");
        exoplanetsTreeWidget = new QTreeWidget(tableTab);
        exoplanetsTreeWidget->setObjectName("exoplanetsTreeWidget");
        exoplanetsTreeWidget->setRootIsDecorated(false);
        exoplanetsTreeWidget->setUniformRowHeights(true);
        exoplanetsTreeWidget->setItemsExpandable(false);
        exoplanetsTreeWidget->setSortingEnabled(true);
        exoplanetsTreeWidget->setExpandsOnDoubleClick(false);
        exoplanetsTreeWidget->setColumnCount(0);

        gridLayout->addWidget(exoplanetsTreeWidget, 0, 0, 1, 1);

        tabs->addTab(tableTab, QString());
        infoTab = new QWidget();
        infoTab->setObjectName("infoTab");
        verticalLayoutInfoBox = new QVBoxLayout(infoTab);
        verticalLayoutInfoBox->setSpacing(9);
        verticalLayoutInfoBox->setObjectName("verticalLayoutInfoBox");
        infoTextBrowser = new QTextBrowser(infoTab);
        infoTextBrowser->setObjectName("infoTextBrowser");
        infoTextBrowser->setOpenExternalLinks(true);
        infoTextBrowser->setOpenLinks(true);

        verticalLayoutInfoBox->addWidget(infoTextBrowser);

        tabs->addTab(infoTab, QString());
        websitesTab = new QWidget();
        websitesTab->setObjectName("websitesTab");
        verticalLayoutWebsitesBox = new QVBoxLayout(websitesTab);
        verticalLayoutWebsitesBox->setSpacing(9);
        verticalLayoutWebsitesBox->setObjectName("verticalLayoutWebsitesBox");
        websitesTextBrowser = new QTextBrowser(websitesTab);
        websitesTextBrowser->setObjectName("websitesTextBrowser");
        websitesTextBrowser->setOpenExternalLinks(true);
        websitesTextBrowser->setOpenLinks(true);

        verticalLayoutWebsitesBox->addWidget(websitesTextBrowser);

        tabs->addTab(websitesTab, QString());
        aboutTab = new QWidget();
        aboutTab->setObjectName("aboutTab");
        verticalLayoutAboutBox = new QVBoxLayout(aboutTab);
        verticalLayoutAboutBox->setSpacing(9);
        verticalLayoutAboutBox->setObjectName("verticalLayoutAboutBox");
        aboutTextBrowser = new QTextBrowser(aboutTab);
        aboutTextBrowser->setObjectName("aboutTextBrowser");
        aboutTextBrowser->setOpenExternalLinks(true);
        aboutTextBrowser->setOpenLinks(true);

        verticalLayoutAboutBox->addWidget(aboutTextBrowser);

        tabs->addTab(aboutTab, QString());

        verticalLayoutEP->addWidget(tabs);


        retranslateUi(exoplanetsDialog);

        tabs->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(exoplanetsDialog);
    } // setupUi

    void retranslateUi(QWidget *exoplanetsDialog)
    {
        exoplanetsDialog->setWindowTitle(QCoreApplication::translate("exoplanetsDialog", "Exoplanets Configuration", nullptr));
        titleBar->setTitle(QCoreApplication::translate("exoplanetsDialog", "Exoplanets Plug-in Configuration", nullptr));
        updatesGroup->setTitle(QCoreApplication::translate("exoplanetsDialog", "Update exoplanets data from Internet", nullptr));
        internetUpdatesCheckbox->setText(QCoreApplication::translate("exoplanetsDialog", "Update from Internet sources", nullptr));
        lastUpdateLabel->setText(QCoreApplication::translate("exoplanetsDialog", "Last update:", nullptr));
        updateFrequencyLabel->setText(QCoreApplication::translate("exoplanetsDialog", "Update frequency:", nullptr));
        nextUpdateLabel->setText(QCoreApplication::translate("exoplanetsDialog", "[next update info]", nullptr));
        updateButton->setText(QCoreApplication::translate("exoplanetsDialog", "Update now", nullptr));
        displayBox->setTitle(QCoreApplication::translate("exoplanetsDialog", "Settings for exoplanets", nullptr));
#if QT_CONFIG(tooltip)
        displayModeCheckBox->setToolTip(QCoreApplication::translate("exoplanetsDialog", "Plot all systems with exoplanets without labels", nullptr));
#endif // QT_CONFIG(tooltip)
        displayModeCheckBox->setText(QCoreApplication::translate("exoplanetsDialog", "Enable display of distribution for exoplanets", nullptr));
#if QT_CONFIG(tooltip)
        timelineModeCheckBox->setToolTip(QCoreApplication::translate("exoplanetsDialog", "Display exoplanets since their discovery", nullptr));
#endif // QT_CONFIG(tooltip)
        timelineModeCheckBox->setText(QCoreApplication::translate("exoplanetsDialog", "Enable timeline discovery of exoplanets", nullptr));
#if QT_CONFIG(tooltip)
        habitableModeCheckBox->setToolTip(QCoreApplication::translate("exoplanetsDialog", "Display of the planetary systems, which contains the potential habitable exoplanets", nullptr));
#endif // QT_CONFIG(tooltip)
        habitableModeCheckBox->setText(QCoreApplication::translate("exoplanetsDialog", "Enable display of the potential habitable exoplanets only", nullptr));
        displayShowDesignationsCheckBox->setText(QCoreApplication::translate("exoplanetsDialog", "Enable display of designations for exoplanets", nullptr));
        displayShowNumbersCheckBox->setText(QCoreApplication::translate("exoplanetsDialog", "Enable display number of exoplanets in the system", nullptr));
        displayAtStartupCheckBox->setText(QCoreApplication::translate("exoplanetsDialog", "Enable display at startup", nullptr));
        displayShowExoplanetsButton->setText(QCoreApplication::translate("exoplanetsDialog", "Show exoplanets button on toolbar", nullptr));
        label->setText(QCoreApplication::translate("exoplanetsDialog", "Exoplanets marker color", nullptr));
#if QT_CONFIG(tooltip)
        exoplanetMarkerColor->setToolTip(QCoreApplication::translate("exoplanetsDialog", "Exoplanets marker color", nullptr));
#endif // QT_CONFIG(tooltip)
        label_2->setText(QCoreApplication::translate("exoplanetsDialog", "Marker color for potential habitable exoplanets", nullptr));
#if QT_CONFIG(tooltip)
        habitableExoplanetMarkerColor->setToolTip(QCoreApplication::translate("exoplanetsDialog", "Marker color for potential habitable exoplanets", nullptr));
#endif // QT_CONFIG(tooltip)
        temperatureScaleLabel->setText(QCoreApplication::translate("exoplanetsDialog", "Temperature scale:", nullptr));
        restoreDefaultsButton->setText(QCoreApplication::translate("exoplanetsDialog", "Restore defaults", nullptr));
        saveSettingsButton->setText(QCoreApplication::translate("exoplanetsDialog", "Save settings", nullptr));
        tabs->setTabText(tabs->indexOf(settingsTab), QCoreApplication::translate("exoplanetsDialog", "Settings", nullptr));
        diagramGroupBox->setTitle(QCoreApplication::translate("exoplanetsDialog", "Diagrams: scatter plot", nullptr));
        labelY->setText(QCoreApplication::translate("exoplanetsDialog", "Y axis", nullptr));
#if QT_CONFIG(tooltip)
        checkBoxLogY->setToolTip(QCoreApplication::translate("exoplanetsDialog", "Logarithmic scale", nullptr));
#endif // QT_CONFIG(tooltip)
        checkBoxLogY->setText(QCoreApplication::translate("exoplanetsDialog", "log scale", nullptr));
        minY->setPlaceholderText(QCoreApplication::translate("exoplanetsDialog", "min", nullptr));
        maxY->setPlaceholderText(QCoreApplication::translate("exoplanetsDialog", "max", nullptr));
        labelX->setText(QCoreApplication::translate("exoplanetsDialog", "X axis", nullptr));
#if QT_CONFIG(tooltip)
        checkBoxLogX->setToolTip(QCoreApplication::translate("exoplanetsDialog", "Logarithmic scale", nullptr));
#endif // QT_CONFIG(tooltip)
        checkBoxLogX->setText(QCoreApplication::translate("exoplanetsDialog", "log scale", nullptr));
        minX->setText(QString());
        minX->setPlaceholderText(QCoreApplication::translate("exoplanetsDialog", "min", nullptr));
        maxX->setPlaceholderText(QCoreApplication::translate("exoplanetsDialog", "max", nullptr));
        tabs->setTabText(tabs->indexOf(diagramTab), QCoreApplication::translate("exoplanetsDialog", "Diagram", "tab in plugin windows"));
        tabs->setTabText(tabs->indexOf(tableTab), QCoreApplication::translate("exoplanetsDialog", "Table", nullptr));
        tabs->setTabText(tabs->indexOf(infoTab), QCoreApplication::translate("exoplanetsDialog", "Info", "tab in plugin windows"));
        tabs->setTabText(tabs->indexOf(websitesTab), QCoreApplication::translate("exoplanetsDialog", "Websites", "tab in plugin windows"));
        tabs->setTabText(tabs->indexOf(aboutTab), QCoreApplication::translate("exoplanetsDialog", "About", "tab in plugin windows"));
    } // retranslateUi

};

namespace Ui {
    class exoplanetsDialog: public Ui_exoplanetsDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_EXOPLANETSDIALOG_H
