/****************************************************************************
** Meta object code from reading C++ file 'StelObject.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../stellarium/src/core/StelObject.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'StelObject.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10StelObjectE_t {};
} // unnamed namespace

template <> constexpr inline auto StelObject::qt_create_metaobjectdata<qt_meta_tag_ZN10StelObjectE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "StelObject",
        "setExtraInfoString",
        "",
        "InfoStringGroup",
        "flags",
        "str",
        "addToExtraInfoString",
        "StelObject::InfoStringGroup",
        "getExtraInfoStrings",
        "removeExtraInfoStrings",
        "InfoStringGroupFlags",
        "None",
        "Name",
        "CatalogNumber",
        "Magnitude",
        "RaDecJ2000",
        "RaDecOfDate",
        "AltAzi",
        "Distance",
        "Elongation",
        "Size",
        "Velocity",
        "ProperMotion",
        "Extra",
        "HourAngle",
        "AbsoluteMagnitude",
        "GalacticCoord",
        "SupergalacticCoord",
        "OtherCoord",
        "ObjectType",
        "EclipticCoordJ2000",
        "EclipticCoordOfDate",
        "IAUConstellation",
        "CulturalConstellation",
        "SiderealTime",
        "RTSTime",
        "SolarLunarPosition",
        "Script",
        "DebugAid",
        "NoFont",
        "PlainText",
        "CulturalDisplayStyle",
        "NONE",
        "Modern",
        "IPA",
        "IPA_Modern",
        "Translated",
        "Translated_Modern",
        "Translated_IPA",
        "Translated_IPA_Modern",
        "Translit",
        "Translit_Modern",
        "Translit_IPA",
        "Translit_IPA_Modern",
        "Translit_Translated",
        "Translit_Translated_Modern",
        "Translit_Translated_IPA",
        "Translit_Translated_IPA_Modern",
        "Pronounce",
        "Pronounce_Modern",
        "Pronounce_IPA",
        "Pronounce_IPA_Modern",
        "Pronounce_Translated",
        "Pronounce_Translated_Modern",
        "Pronounce_Translated_IPA",
        "Pronounce_Translated_IPA_Modern",
        "Pronounce_Translit",
        "Pronounce_Translit_Modern",
        "Pronounce_Translit_IPA",
        "Pronounce_Translit_IPA_Modern",
        "Pronounce_Translit_Translated",
        "Pronounce_Translit_Translated_Modern",
        "Pronounce_Translit_Translated_IPA",
        "Pronounce_Translit_Translated_IPA_Modern",
        "Native",
        "Native_Modern",
        "Native_IPA",
        "Native_IPA_Modern",
        "Native_Translated",
        "Native_Translated_Modern",
        "Native_Translated_IPA",
        "Native_Translated_IPA_Modern",
        "Native_Translit",
        "Native_Translit_Modern",
        "Native_Translit_IPA",
        "Native_Translit_IPA_Modern",
        "Native_Translit_Translated",
        "Native_Translit_Translated_Modern",
        "Native_Translit_Translated_IPA",
        "Native_Translit_Translated_IPA_Modern",
        "Native_Pronounce",
        "Native_Pronounce_Modern",
        "Native_Pronounce_IPA",
        "Native_Pronounce_IPA_Modern",
        "Native_Pronounce_Translated",
        "Native_Pronounce_Translated_Modern",
        "Native_Pronounce_Translated_IPA",
        "Native_Pronounce_Translated_IPA_Modern",
        "Native_Pronounce_Translit",
        "Native_Pronounce_Translit_Modern",
        "Native_Pronounce_Translit_IPA",
        "Native_Pronounce_Translit_IPA_Modern",
        "Native_Pronounce_Translit_Translated",
        "Native_Pronounce_Translit_Translated_Modern",
        "Native_Pronounce_Translit_Translated_IPA",
        "Native_Pronounce_Translit_Translated_IPA_Modern",
        "Byname",
        "Byname_Modern",
        "IPA_Byname",
        "IPA_Byname_Modern",
        "Translated_Byname",
        "Translated_Byname_Modern",
        "Translated_IPA_Byname",
        "Translated_IPA_Byname_Modern",
        "Translit_Byname",
        "Translit_Byname_Modern",
        "Translit_IPA_Byname",
        "Translit_IPA_Byname_Modern",
        "Translit_Translated_Byname",
        "Translit_Translated_Byname_Modern",
        "Translit_Translated_IPA_Byname",
        "Translit_Translated_IPA_Byname_Modern",
        "Pronounce_Byname",
        "Pronounce_Byname_Modern",
        "Pronounce_IPA_Byname",
        "Pronounce_IPA_Byname_Modern",
        "Pronounce_Translated_Byname",
        "Pronounce_Translated_Byname_Modern",
        "Pronounce_Translated_IPA_Byname",
        "Pronounce_Translated_IPA_Byname_Modern",
        "Pronounce_Translit_Byname",
        "Pronounce_Translit_Byname_Modern",
        "Pronounce_Translit_IPA_Byname",
        "Pronounce_Translit_IPA_Byname_Modern",
        "Pronounce_Translit_Translated_Byname",
        "Pronounce_Translit_Translated_Byname_Modern",
        "Pronounce_Translit_Translated_IPA_Byname",
        "Pronounce_Translit_Translated_IPA_Byname_Modern",
        "Native_Byname",
        "Native_Byname_Modern",
        "Native_IPA_Byname",
        "Native_IPA_Byname_Modern",
        "Native_Translated_Byname",
        "Native_Translated_Byname_Modern",
        "Native_Translated_IPA_Byname",
        "Native_Translated_IPA_Byname_Modern",
        "Native_Translit_Byname",
        "Native_Translit_Byname_Modern",
        "Native_Translit_IPA_Byname",
        "Native_Translit_IPA_Byname_Modern",
        "Native_Translit_Translated_Byname",
        "Native_Translit_Translated_Byname_Modern",
        "Native_Translit_Translated_IPA_Byname",
        "Native_Translit_Translated_IPA_Byname_Modern",
        "Native_Pronounce_Byname",
        "Native_Pronounce_Byname_Modern",
        "Native_Pronounce_IPA_Byname",
        "Native_Pronounce_IPA_Byname_Modern",
        "Native_Pronounce_Translated_Byname",
        "Native_Pronounce_Translated_Byname_Modern",
        "Native_Pronounce_Translated_IPA_Byname",
        "Native_Pronounce_Translated_IPA_Byname_Modern",
        "Native_Pronounce_Translit_Byname",
        "Native_Pronounce_Translit_Byname_Modern",
        "Native_Pronounce_Translit_IPA_Byname",
        "Native_Pronounce_Translit_IPA_Byname_Modern",
        "Native_Pronounce_Translit_Translated_Byname",
        "Native_Pronounce_Translit_Translated_Byname_Modern",
        "Native_Pronounce_Translit_Translated_IPA_Byname",
        "Native_Pronounce_Translit_Translated_IPA_Byname_Modern",
        "CulturalNameSpecial",
        "Morning",
        "Evening"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'setExtraInfoString'
        QtMocHelpers::SlotData<void(const InfoStringGroup &, const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::QString, 5 },
        }}),
        // Slot 'addToExtraInfoString'
        QtMocHelpers::SlotData<void(const StelObject::InfoStringGroup &, const QString &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 4 }, { QMetaType::QString, 5 },
        }}),
        // Slot 'getExtraInfoStrings'
        QtMocHelpers::SlotData<QStringList(const InfoStringGroup &) const>(8, 2, QMC::AccessPublic, QMetaType::QStringList, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'removeExtraInfoStrings'
        QtMocHelpers::SlotData<void(const InfoStringGroup &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // flag 'InfoStringGroup'
        QtMocHelpers::EnumData<InfoStringGroup>(3, 10, QMC::EnumIsFlag).add({
            {   11, InfoStringGroupFlags::None },
            {   12, InfoStringGroupFlags::Name },
            {   13, InfoStringGroupFlags::CatalogNumber },
            {   14, InfoStringGroupFlags::Magnitude },
            {   15, InfoStringGroupFlags::RaDecJ2000 },
            {   16, InfoStringGroupFlags::RaDecOfDate },
            {   17, InfoStringGroupFlags::AltAzi },
            {   18, InfoStringGroupFlags::Distance },
            {   19, InfoStringGroupFlags::Elongation },
            {   20, InfoStringGroupFlags::Size },
            {   21, InfoStringGroupFlags::Velocity },
            {   22, InfoStringGroupFlags::ProperMotion },
            {   23, InfoStringGroupFlags::Extra },
            {   24, InfoStringGroupFlags::HourAngle },
            {   25, InfoStringGroupFlags::AbsoluteMagnitude },
            {   26, InfoStringGroupFlags::GalacticCoord },
            {   27, InfoStringGroupFlags::SupergalacticCoord },
            {   28, InfoStringGroupFlags::OtherCoord },
            {   29, InfoStringGroupFlags::ObjectType },
            {   30, InfoStringGroupFlags::EclipticCoordJ2000 },
            {   31, InfoStringGroupFlags::EclipticCoordOfDate },
            {   32, InfoStringGroupFlags::IAUConstellation },
            {   33, InfoStringGroupFlags::CulturalConstellation },
            {   34, InfoStringGroupFlags::SiderealTime },
            {   35, InfoStringGroupFlags::RTSTime },
            {   36, InfoStringGroupFlags::SolarLunarPosition },
            {   37, InfoStringGroupFlags::Script },
            {   38, InfoStringGroupFlags::DebugAid },
            {   39, InfoStringGroupFlags::NoFont },
            {   40, InfoStringGroupFlags::PlainText },
        }),
        // enum 'CulturalDisplayStyle'
        QtMocHelpers::EnumData<enum CulturalDisplayStyle>(41, 41, QMC::EnumIsScoped).add({
            {   42, CulturalDisplayStyle::NONE },
            {   43, CulturalDisplayStyle::Modern },
            {   44, CulturalDisplayStyle::IPA },
            {   45, CulturalDisplayStyle::IPA_Modern },
            {   46, CulturalDisplayStyle::Translated },
            {   47, CulturalDisplayStyle::Translated_Modern },
            {   48, CulturalDisplayStyle::Translated_IPA },
            {   49, CulturalDisplayStyle::Translated_IPA_Modern },
            {   50, CulturalDisplayStyle::Translit },
            {   51, CulturalDisplayStyle::Translit_Modern },
            {   52, CulturalDisplayStyle::Translit_IPA },
            {   53, CulturalDisplayStyle::Translit_IPA_Modern },
            {   54, CulturalDisplayStyle::Translit_Translated },
            {   55, CulturalDisplayStyle::Translit_Translated_Modern },
            {   56, CulturalDisplayStyle::Translit_Translated_IPA },
            {   57, CulturalDisplayStyle::Translit_Translated_IPA_Modern },
            {   58, CulturalDisplayStyle::Pronounce },
            {   59, CulturalDisplayStyle::Pronounce_Modern },
            {   60, CulturalDisplayStyle::Pronounce_IPA },
            {   61, CulturalDisplayStyle::Pronounce_IPA_Modern },
            {   62, CulturalDisplayStyle::Pronounce_Translated },
            {   63, CulturalDisplayStyle::Pronounce_Translated_Modern },
            {   64, CulturalDisplayStyle::Pronounce_Translated_IPA },
            {   65, CulturalDisplayStyle::Pronounce_Translated_IPA_Modern },
            {   66, CulturalDisplayStyle::Pronounce_Translit },
            {   67, CulturalDisplayStyle::Pronounce_Translit_Modern },
            {   68, CulturalDisplayStyle::Pronounce_Translit_IPA },
            {   69, CulturalDisplayStyle::Pronounce_Translit_IPA_Modern },
            {   70, CulturalDisplayStyle::Pronounce_Translit_Translated },
            {   71, CulturalDisplayStyle::Pronounce_Translit_Translated_Modern },
            {   72, CulturalDisplayStyle::Pronounce_Translit_Translated_IPA },
            {   73, CulturalDisplayStyle::Pronounce_Translit_Translated_IPA_Modern },
            {   74, CulturalDisplayStyle::Native },
            {   75, CulturalDisplayStyle::Native_Modern },
            {   76, CulturalDisplayStyle::Native_IPA },
            {   77, CulturalDisplayStyle::Native_IPA_Modern },
            {   78, CulturalDisplayStyle::Native_Translated },
            {   79, CulturalDisplayStyle::Native_Translated_Modern },
            {   80, CulturalDisplayStyle::Native_Translated_IPA },
            {   81, CulturalDisplayStyle::Native_Translated_IPA_Modern },
            {   82, CulturalDisplayStyle::Native_Translit },
            {   83, CulturalDisplayStyle::Native_Translit_Modern },
            {   84, CulturalDisplayStyle::Native_Translit_IPA },
            {   85, CulturalDisplayStyle::Native_Translit_IPA_Modern },
            {   86, CulturalDisplayStyle::Native_Translit_Translated },
            {   87, CulturalDisplayStyle::Native_Translit_Translated_Modern },
            {   88, CulturalDisplayStyle::Native_Translit_Translated_IPA },
            {   89, CulturalDisplayStyle::Native_Translit_Translated_IPA_Modern },
            {   90, CulturalDisplayStyle::Native_Pronounce },
            {   91, CulturalDisplayStyle::Native_Pronounce_Modern },
            {   92, CulturalDisplayStyle::Native_Pronounce_IPA },
            {   93, CulturalDisplayStyle::Native_Pronounce_IPA_Modern },
            {   94, CulturalDisplayStyle::Native_Pronounce_Translated },
            {   95, CulturalDisplayStyle::Native_Pronounce_Translated_Modern },
            {   96, CulturalDisplayStyle::Native_Pronounce_Translated_IPA },
            {   97, CulturalDisplayStyle::Native_Pronounce_Translated_IPA_Modern },
            {   98, CulturalDisplayStyle::Native_Pronounce_Translit },
            {   99, CulturalDisplayStyle::Native_Pronounce_Translit_Modern },
            {  100, CulturalDisplayStyle::Native_Pronounce_Translit_IPA },
            {  101, CulturalDisplayStyle::Native_Pronounce_Translit_IPA_Modern },
            {  102, CulturalDisplayStyle::Native_Pronounce_Translit_Translated },
            {  103, CulturalDisplayStyle::Native_Pronounce_Translit_Translated_Modern },
            {  104, CulturalDisplayStyle::Native_Pronounce_Translit_Translated_IPA },
            {  105, CulturalDisplayStyle::Native_Pronounce_Translit_Translated_IPA_Modern },
            {  106, CulturalDisplayStyle::Byname },
            {  107, CulturalDisplayStyle::Byname_Modern },
            {  108, CulturalDisplayStyle::IPA_Byname },
            {  109, CulturalDisplayStyle::IPA_Byname_Modern },
            {  110, CulturalDisplayStyle::Translated_Byname },
            {  111, CulturalDisplayStyle::Translated_Byname_Modern },
            {  112, CulturalDisplayStyle::Translated_IPA_Byname },
            {  113, CulturalDisplayStyle::Translated_IPA_Byname_Modern },
            {  114, CulturalDisplayStyle::Translit_Byname },
            {  115, CulturalDisplayStyle::Translit_Byname_Modern },
            {  116, CulturalDisplayStyle::Translit_IPA_Byname },
            {  117, CulturalDisplayStyle::Translit_IPA_Byname_Modern },
            {  118, CulturalDisplayStyle::Translit_Translated_Byname },
            {  119, CulturalDisplayStyle::Translit_Translated_Byname_Modern },
            {  120, CulturalDisplayStyle::Translit_Translated_IPA_Byname },
            {  121, CulturalDisplayStyle::Translit_Translated_IPA_Byname_Modern },
            {  122, CulturalDisplayStyle::Pronounce_Byname },
            {  123, CulturalDisplayStyle::Pronounce_Byname_Modern },
            {  124, CulturalDisplayStyle::Pronounce_IPA_Byname },
            {  125, CulturalDisplayStyle::Pronounce_IPA_Byname_Modern },
            {  126, CulturalDisplayStyle::Pronounce_Translated_Byname },
            {  127, CulturalDisplayStyle::Pronounce_Translated_Byname_Modern },
            {  128, CulturalDisplayStyle::Pronounce_Translated_IPA_Byname },
            {  129, CulturalDisplayStyle::Pronounce_Translated_IPA_Byname_Modern },
            {  130, CulturalDisplayStyle::Pronounce_Translit_Byname },
            {  131, CulturalDisplayStyle::Pronounce_Translit_Byname_Modern },
            {  132, CulturalDisplayStyle::Pronounce_Translit_IPA_Byname },
            {  133, CulturalDisplayStyle::Pronounce_Translit_IPA_Byname_Modern },
            {  134, CulturalDisplayStyle::Pronounce_Translit_Translated_Byname },
            {  135, CulturalDisplayStyle::Pronounce_Translit_Translated_Byname_Modern },
            {  136, CulturalDisplayStyle::Pronounce_Translit_Translated_IPA_Byname },
            {  137, CulturalDisplayStyle::Pronounce_Translit_Translated_IPA_Byname_Modern },
            {  138, CulturalDisplayStyle::Native_Byname },
            {  139, CulturalDisplayStyle::Native_Byname_Modern },
            {  140, CulturalDisplayStyle::Native_IPA_Byname },
            {  141, CulturalDisplayStyle::Native_IPA_Byname_Modern },
            {  142, CulturalDisplayStyle::Native_Translated_Byname },
            {  143, CulturalDisplayStyle::Native_Translated_Byname_Modern },
            {  144, CulturalDisplayStyle::Native_Translated_IPA_Byname },
            {  145, CulturalDisplayStyle::Native_Translated_IPA_Byname_Modern },
            {  146, CulturalDisplayStyle::Native_Translit_Byname },
            {  147, CulturalDisplayStyle::Native_Translit_Byname_Modern },
            {  148, CulturalDisplayStyle::Native_Translit_IPA_Byname },
            {  149, CulturalDisplayStyle::Native_Translit_IPA_Byname_Modern },
            {  150, CulturalDisplayStyle::Native_Translit_Translated_Byname },
            {  151, CulturalDisplayStyle::Native_Translit_Translated_Byname_Modern },
            {  152, CulturalDisplayStyle::Native_Translit_Translated_IPA_Byname },
            {  153, CulturalDisplayStyle::Native_Translit_Translated_IPA_Byname_Modern },
            {  154, CulturalDisplayStyle::Native_Pronounce_Byname },
            {  155, CulturalDisplayStyle::Native_Pronounce_Byname_Modern },
            {  156, CulturalDisplayStyle::Native_Pronounce_IPA_Byname },
            {  157, CulturalDisplayStyle::Native_Pronounce_IPA_Byname_Modern },
            {  158, CulturalDisplayStyle::Native_Pronounce_Translated_Byname },
            {  159, CulturalDisplayStyle::Native_Pronounce_Translated_Byname_Modern },
            {  160, CulturalDisplayStyle::Native_Pronounce_Translated_IPA_Byname },
            {  161, CulturalDisplayStyle::Native_Pronounce_Translated_IPA_Byname_Modern },
            {  162, CulturalDisplayStyle::Native_Pronounce_Translit_Byname },
            {  163, CulturalDisplayStyle::Native_Pronounce_Translit_Byname_Modern },
            {  164, CulturalDisplayStyle::Native_Pronounce_Translit_IPA_Byname },
            {  165, CulturalDisplayStyle::Native_Pronounce_Translit_IPA_Byname_Modern },
            {  166, CulturalDisplayStyle::Native_Pronounce_Translit_Translated_Byname },
            {  167, CulturalDisplayStyle::Native_Pronounce_Translit_Translated_Byname_Modern },
            {  168, CulturalDisplayStyle::Native_Pronounce_Translit_Translated_IPA_Byname },
            {  169, CulturalDisplayStyle::Native_Pronounce_Translit_Translated_IPA_Byname_Modern },
        }),
        // enum 'CulturalNameSpecial'
        QtMocHelpers::EnumData<enum CulturalNameSpecial>(170, 170, QMC::EnumIsScoped).add({
            {   11, CulturalNameSpecial::None },
            {  171, CulturalNameSpecial::Morning },
            {  172, CulturalNameSpecial::Evening },
        }),
    };
    return QtMocHelpers::metaObjectData<StelObject, qt_meta_tag_ZN10StelObjectE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject StelObject::staticMetaObject = { {
    QtPrivate::MetaObjectForType<StelRegionObject>::value,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10StelObjectE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10StelObjectE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10StelObjectE_t>.metaTypes,
    nullptr
} };

QT_WARNING_POP
