/****************************************************************************
** Meta object code from reading C++ file 'MarkerMgr.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../stellarium/src/core/modules/MarkerMgr.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'MarkerMgr.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9MarkerMgrE_t {};
} // unnamed namespace

template <> constexpr inline auto MarkerMgr::qt_create_metaobjectdata<qt_meta_tag_ZN9MarkerMgrE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MarkerMgr",
        "markerObject",
        "",
        "objectName",
        "visible",
        "mtype",
        "color",
        "size",
        "autoDelete",
        "autoDeleteTimeoutMs",
        "markerEquatorial",
        "RA",
        "Dec",
        "j2000epoch",
        "withAberration",
        "markerHorizon",
        "az",
        "alt",
        "getMarkerShow",
        "id",
        "setMarkerShow",
        "show",
        "deleteMarker",
        "deleteAllMarkers",
        "markerVisibleTimeout",
        "markerDeleteTimeout"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'markerObject'
        QtMocHelpers::SlotData<int(const QString &, bool, const QString &, const QString &, const float, bool, int)>(1, 2, QMC::AccessPublic, QMetaType::Int, {{
            { QMetaType::QString, 3 }, { QMetaType::Bool, 4 }, { QMetaType::QString, 5 }, { QMetaType::QString, 6 },
            { QMetaType::Float, 7 }, { QMetaType::Bool, 8 }, { QMetaType::Int, 9 },
        }}),
        // Slot 'markerObject'
        QtMocHelpers::SlotData<int(const QString &, bool, const QString &, const QString &, const float, bool)>(1, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { QMetaType::QString, 3 }, { QMetaType::Bool, 4 }, { QMetaType::QString, 5 }, { QMetaType::QString, 6 },
            { QMetaType::Float, 7 }, { QMetaType::Bool, 8 },
        }}),
        // Slot 'markerObject'
        QtMocHelpers::SlotData<int(const QString &, bool, const QString &, const QString &, const float)>(1, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { QMetaType::QString, 3 }, { QMetaType::Bool, 4 }, { QMetaType::QString, 5 }, { QMetaType::QString, 6 },
            { QMetaType::Float, 7 },
        }}),
        // Slot 'markerObject'
        QtMocHelpers::SlotData<int(const QString &, bool, const QString &, const QString &)>(1, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { QMetaType::QString, 3 }, { QMetaType::Bool, 4 }, { QMetaType::QString, 5 }, { QMetaType::QString, 6 },
        }}),
        // Slot 'markerObject'
        QtMocHelpers::SlotData<int(const QString &, bool, const QString &)>(1, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { QMetaType::QString, 3 }, { QMetaType::Bool, 4 }, { QMetaType::QString, 5 },
        }}),
        // Slot 'markerObject'
        QtMocHelpers::SlotData<int(const QString &, bool)>(1, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { QMetaType::QString, 3 }, { QMetaType::Bool, 4 },
        }}),
        // Slot 'markerObject'
        QtMocHelpers::SlotData<int(const QString &)>(1, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'markerEquatorial'
        QtMocHelpers::SlotData<int(const QString &, const QString &, bool, bool, const QString &, const QString &, const float, bool, int, bool)>(10, 2, QMC::AccessPublic, QMetaType::Int, {{
            { QMetaType::QString, 11 }, { QMetaType::QString, 12 }, { QMetaType::Bool, 13 }, { QMetaType::Bool, 4 },
            { QMetaType::QString, 5 }, { QMetaType::QString, 6 }, { QMetaType::Float, 7 }, { QMetaType::Bool, 8 },
            { QMetaType::Int, 9 }, { QMetaType::Bool, 14 },
        }}),
        // Slot 'markerEquatorial'
        QtMocHelpers::SlotData<int(const QString &, const QString &, bool, bool, const QString &, const QString &, const float, bool, int)>(10, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { QMetaType::QString, 11 }, { QMetaType::QString, 12 }, { QMetaType::Bool, 13 }, { QMetaType::Bool, 4 },
            { QMetaType::QString, 5 }, { QMetaType::QString, 6 }, { QMetaType::Float, 7 }, { QMetaType::Bool, 8 },
            { QMetaType::Int, 9 },
        }}),
        // Slot 'markerEquatorial'
        QtMocHelpers::SlotData<int(const QString &, const QString &, bool, bool, const QString &, const QString &, const float, bool)>(10, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { QMetaType::QString, 11 }, { QMetaType::QString, 12 }, { QMetaType::Bool, 13 }, { QMetaType::Bool, 4 },
            { QMetaType::QString, 5 }, { QMetaType::QString, 6 }, { QMetaType::Float, 7 }, { QMetaType::Bool, 8 },
        }}),
        // Slot 'markerEquatorial'
        QtMocHelpers::SlotData<int(const QString &, const QString &, bool, bool, const QString &, const QString &, const float)>(10, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { QMetaType::QString, 11 }, { QMetaType::QString, 12 }, { QMetaType::Bool, 13 }, { QMetaType::Bool, 4 },
            { QMetaType::QString, 5 }, { QMetaType::QString, 6 }, { QMetaType::Float, 7 },
        }}),
        // Slot 'markerEquatorial'
        QtMocHelpers::SlotData<int(const QString &, const QString &, bool, bool, const QString &, const QString &)>(10, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { QMetaType::QString, 11 }, { QMetaType::QString, 12 }, { QMetaType::Bool, 13 }, { QMetaType::Bool, 4 },
            { QMetaType::QString, 5 }, { QMetaType::QString, 6 },
        }}),
        // Slot 'markerEquatorial'
        QtMocHelpers::SlotData<int(const QString &, const QString &, bool, bool, const QString &)>(10, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { QMetaType::QString, 11 }, { QMetaType::QString, 12 }, { QMetaType::Bool, 13 }, { QMetaType::Bool, 4 },
            { QMetaType::QString, 5 },
        }}),
        // Slot 'markerEquatorial'
        QtMocHelpers::SlotData<int(const QString &, const QString &, bool, bool)>(10, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { QMetaType::QString, 11 }, { QMetaType::QString, 12 }, { QMetaType::Bool, 13 }, { QMetaType::Bool, 4 },
        }}),
        // Slot 'markerEquatorial'
        QtMocHelpers::SlotData<int(const QString &, const QString &, bool)>(10, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { QMetaType::QString, 11 }, { QMetaType::QString, 12 }, { QMetaType::Bool, 13 },
        }}),
        // Slot 'markerEquatorial'
        QtMocHelpers::SlotData<int(const QString &, const QString &)>(10, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { QMetaType::QString, 11 }, { QMetaType::QString, 12 },
        }}),
        // Slot 'markerHorizon'
        QtMocHelpers::SlotData<int(const QString &, const QString &, bool, const QString &, const QString &, const float, bool, int)>(15, 2, QMC::AccessPublic, QMetaType::Int, {{
            { QMetaType::QString, 16 }, { QMetaType::QString, 17 }, { QMetaType::Bool, 4 }, { QMetaType::QString, 5 },
            { QMetaType::QString, 6 }, { QMetaType::Float, 7 }, { QMetaType::Bool, 8 }, { QMetaType::Int, 9 },
        }}),
        // Slot 'markerHorizon'
        QtMocHelpers::SlotData<int(const QString &, const QString &, bool, const QString &, const QString &, const float, bool)>(15, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { QMetaType::QString, 16 }, { QMetaType::QString, 17 }, { QMetaType::Bool, 4 }, { QMetaType::QString, 5 },
            { QMetaType::QString, 6 }, { QMetaType::Float, 7 }, { QMetaType::Bool, 8 },
        }}),
        // Slot 'markerHorizon'
        QtMocHelpers::SlotData<int(const QString &, const QString &, bool, const QString &, const QString &, const float)>(15, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { QMetaType::QString, 16 }, { QMetaType::QString, 17 }, { QMetaType::Bool, 4 }, { QMetaType::QString, 5 },
            { QMetaType::QString, 6 }, { QMetaType::Float, 7 },
        }}),
        // Slot 'markerHorizon'
        QtMocHelpers::SlotData<int(const QString &, const QString &, bool, const QString &, const QString &)>(15, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { QMetaType::QString, 16 }, { QMetaType::QString, 17 }, { QMetaType::Bool, 4 }, { QMetaType::QString, 5 },
            { QMetaType::QString, 6 },
        }}),
        // Slot 'markerHorizon'
        QtMocHelpers::SlotData<int(const QString &, const QString &, bool, const QString &)>(15, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { QMetaType::QString, 16 }, { QMetaType::QString, 17 }, { QMetaType::Bool, 4 }, { QMetaType::QString, 5 },
        }}),
        // Slot 'markerHorizon'
        QtMocHelpers::SlotData<int(const QString &, const QString &, bool)>(15, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { QMetaType::QString, 16 }, { QMetaType::QString, 17 }, { QMetaType::Bool, 4 },
        }}),
        // Slot 'markerHorizon'
        QtMocHelpers::SlotData<int(const QString &, const QString &)>(15, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { QMetaType::QString, 16 }, { QMetaType::QString, 17 },
        }}),
        // Slot 'getMarkerShow'
        QtMocHelpers::SlotData<bool(int) const>(18, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::Int, 19 },
        }}),
        // Slot 'setMarkerShow'
        QtMocHelpers::SlotData<void(int, bool)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 19 }, { QMetaType::Bool, 21 },
        }}),
        // Slot 'deleteMarker'
        QtMocHelpers::SlotData<void(int)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 19 },
        }}),
        // Slot 'deleteAllMarkers'
        QtMocHelpers::SlotData<int()>(23, 2, QMC::AccessPublic, QMetaType::Int),
        // Slot 'markerVisibleTimeout'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'markerDeleteTimeout'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<MarkerMgr, qt_meta_tag_ZN9MarkerMgrE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MarkerMgr::staticMetaObject = { {
    QMetaObject::SuperData::link<StelModule::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9MarkerMgrE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9MarkerMgrE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9MarkerMgrE_t>.metaTypes,
    nullptr
} };

void MarkerMgr::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MarkerMgr *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { int _r = _t->markerObject((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[7])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 1: { int _r = _t->markerObject((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[6])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 2: { int _r = _t->markerObject((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[5])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 3: { int _r = _t->markerObject((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 4: { int _r = _t->markerObject((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 5: { int _r = _t->markerObject((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 6: { int _r = _t->markerObject((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 7: { int _r = _t->markerEquatorial((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[8])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[9])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[10])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 8: { int _r = _t->markerEquatorial((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[8])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[9])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 9: { int _r = _t->markerEquatorial((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[8])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 10: { int _r = _t->markerEquatorial((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[7])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 11: { int _r = _t->markerEquatorial((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[6])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 12: { int _r = _t->markerEquatorial((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 13: { int _r = _t->markerEquatorial((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 14: { int _r = _t->markerEquatorial((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 15: { int _r = _t->markerEquatorial((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 16: { int _r = _t->markerHorizon((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[8])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 17: { int _r = _t->markerHorizon((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[7])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 18: { int _r = _t->markerHorizon((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[6])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 19: { int _r = _t->markerHorizon((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 20: { int _r = _t->markerHorizon((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 21: { int _r = _t->markerHorizon((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 22: { int _r = _t->markerHorizon((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 23: { bool _r = _t->getMarkerShow((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 24: _t->setMarkerShow((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 25: _t->deleteMarker((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 26: { int _r = _t->deleteAllMarkers();
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 27: _t->markerVisibleTimeout(); break;
        case 28: _t->markerDeleteTimeout(); break;
        default: ;
        }
    }
}

const QMetaObject *MarkerMgr::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MarkerMgr::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9MarkerMgrE_t>.strings))
        return static_cast<void*>(this);
    return StelModule::qt_metacast(_clname);
}

int MarkerMgr::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StelModule::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 29)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 29;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 29)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 29;
    }
    return _id;
}
QT_WARNING_POP
