/****************************************************************************
** Meta object code from reading C++ file 'HipsMgr.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../stellarium/src/core/modules/HipsMgr.hpp"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'HipsMgr.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN7HipsMgrE_t {};
} // unnamed namespace

template <> constexpr inline auto HipsMgr::qt_create_metaobjectdata<qt_meta_tag_ZN7HipsMgrE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "HipsMgr",
        "showChanged",
        "",
        "value",
        "surveysChanged",
        "stateChanged",
        "State",
        "gotNewSurvey",
        "HipsSurveyP",
        "survey",
        "toggleDialog",
        "loadSources",
        "getFlagShow",
        "setFlagShow",
        "b",
        "restoreVisibleSurveys",
        "getSurveyByUrl",
        "url",
        "surveys",
        "QList<HipsSurveyP>",
        "flagShow",
        "state",
        "loaded",
        "Created",
        "Loading",
        "Loaded"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'showChanged'
        QtMocHelpers::SignalData<void(bool) const>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'surveysChanged'
        QtMocHelpers::SignalData<void() const>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'stateChanged'
        QtMocHelpers::SignalData<void(enum State) const>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 3 },
        }}),
        // Signal 'gotNewSurvey'
        QtMocHelpers::SignalData<void(HipsSurveyP) const>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Signal 'toggleDialog'
        QtMocHelpers::SignalData<void() const>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'loadSources'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'getFlagShow'
        QtMocHelpers::SlotData<bool() const>(12, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFlagShow'
        QtMocHelpers::SlotData<void(bool)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 14 },
        }}),
        // Slot 'restoreVisibleSurveys'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPrivate, QMetaType::Void),
        // Method 'getSurveyByUrl'
        QtMocHelpers::MethodData<HipsSurveyP(const QString &)>(16, 2, QMC::AccessPublic, 0x80000000 | 8, {{
            { QMetaType::QString, 17 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'surveys'
        QtMocHelpers::PropertyData<QList<HipsSurveyP>>(18, 0x80000000 | 19, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag, 1),
        // property 'flagShow'
        QtMocHelpers::PropertyData<bool>(20, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'state'
        QtMocHelpers::PropertyData<enum State>(21, 0x80000000 | 6, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 2),
        // property 'loaded'
        QtMocHelpers::PropertyData<bool>(22, QMetaType::Bool, QMC::DefaultPropertyFlags, 2),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'State'
        QtMocHelpers::EnumData<enum State>(6, 6, QMC::EnumFlags{}).add({
            {   23, State::Created },
            {   24, State::Loading },
            {   25, State::Loaded },
        }),
    };
    return QtMocHelpers::metaObjectData<HipsMgr, qt_meta_tag_ZN7HipsMgrE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject HipsMgr::staticMetaObject = { {
    QMetaObject::SuperData::link<StelModule::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7HipsMgrE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7HipsMgrE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN7HipsMgrE_t>.metaTypes,
    nullptr
} };

void HipsMgr::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<HipsMgr *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->showChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->surveysChanged(); break;
        case 2: _t->stateChanged((*reinterpret_cast<std::add_pointer_t<enum State>>(_a[1]))); break;
        case 3: _t->gotNewSurvey((*reinterpret_cast<std::add_pointer_t<HipsSurveyP>>(_a[1]))); break;
        case 4: _t->toggleDialog(); break;
        case 5: _t->loadSources(); break;
        case 6: { bool _r = _t->getFlagShow();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 7: _t->setFlagShow((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->restoreVisibleSurveys(); break;
        case 9: { HipsSurveyP _r = _t->getSurveyByUrl((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<HipsSurveyP*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< HipsSurveyP >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (HipsMgr::*)(bool ) const>(_a, &HipsMgr::showChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (HipsMgr::*)() const>(_a, &HipsMgr::surveysChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (HipsMgr::*)(State ) const>(_a, &HipsMgr::stateChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (HipsMgr::*)(HipsSurveyP ) const>(_a, &HipsMgr::gotNewSurvey, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (HipsMgr::*)() const>(_a, &HipsMgr::toggleDialog, 4))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<HipsSurveyP> >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QList<HipsSurveyP>*>(_v) = _t->surveys; break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->getFlagShow(); break;
        case 2: *reinterpret_cast<enum State*>(_v) = _t->getState(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->isLoaded(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0:
            if (QtMocHelpers::setProperty(_t->surveys, *reinterpret_cast<QList<HipsSurveyP>*>(_v)))
                Q_EMIT _t->surveysChanged();
            break;
        case 1: _t->setFlagShow(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *HipsMgr::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *HipsMgr::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7HipsMgrE_t>.strings))
        return static_cast<void*>(this);
    return StelModule::qt_metacast(_clname);
}

int HipsMgr::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StelModule::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void HipsMgr::showChanged(bool _t1)const
{
    QMetaObject::activate<void>(const_cast< HipsMgr *>(this), &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void HipsMgr::surveysChanged()const
{
    QMetaObject::activate(const_cast< HipsMgr *>(this), &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void HipsMgr::stateChanged(State _t1)const
{
    QMetaObject::activate<void>(const_cast< HipsMgr *>(this), &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void HipsMgr::gotNewSurvey(HipsSurveyP _t1)const
{
    QMetaObject::activate<void>(const_cast< HipsMgr *>(this), &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void HipsMgr::toggleDialog()const
{
    QMetaObject::activate(const_cast< HipsMgr *>(this), &staticMetaObject, 4, nullptr);
}
QT_WARNING_POP
