/****************************************************************************
** Meta object code from reading C++ file 'SpecialMarkersMgr.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../stellarium/src/core/modules/SpecialMarkersMgr.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'SpecialMarkersMgr.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN17SpecialMarkersMgrE_t {};
} // unnamed namespace

template <> constexpr inline auto SpecialMarkersMgr::qt_create_metaobjectdata<qt_meta_tag_ZN17SpecialMarkersMgrE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "SpecialMarkersMgr",
        "fovCenterMarkerDisplayedChanged",
        "",
        "displayed",
        "fovCenterMarkerColorChanged",
        "Vec3f",
        "newColor",
        "fovCircularMarkerDisplayedChanged",
        "fovCircularMarkerSizeChanged",
        "size",
        "fovCircularMarkerColorChanged",
        "fovRectangularMarkerDisplayedChanged",
        "fovRectangularMarkerWidthChanged",
        "fovRectangularMarkerHeightChanged",
        "fovRectangularMarkerRotationAngleChanged",
        "fovRectangularMarkerColorChanged",
        "compassMarksDisplayedChanged",
        "compassMarksColorChanged",
        "setFlagFOVCenterMarker",
        "getFlagFOVCenterMarker",
        "getColorFOVCenterMarker",
        "setColorFOVCenterMarker",
        "setFlagFOVCircularMarker",
        "getFlagFOVCircularMarker",
        "setFOVCircularMarkerSize",
        "getFOVCircularMarkerSize",
        "getColorFOVCircularMarker",
        "setColorFOVCircularMarker",
        "setFlagFOVRectangularMarker",
        "getFlagFOVRectangularMarker",
        "setFOVRectangularMarkerWidth",
        "getFOVRectangularMarkerWidth",
        "setFOVRectangularMarkerHeight",
        "getFOVRectangularMarkerHeight",
        "setFOVRectangularMarkerRotationAngle",
        "pa",
        "getFOVRectangularMarkerRotationAngle",
        "getColorFOVRectangularMarker",
        "setColorFOVRectangularMarker",
        "setFlagCompassMarks",
        "getFlagCompassMarks",
        "getColorCompassMarks",
        "setColorCompassMarks",
        "fovCenterMarkerDisplayed",
        "fovCenterMarkerColor",
        "fovCircularMarkerDisplayed",
        "fovCircularMarkerSize",
        "fovCircularMarkerColor",
        "fovRectangularMarkerDisplayed",
        "fovRectangularMarkerWidth",
        "fovRectangularMarkerHeight",
        "fovRectangularMarkerRotationAngle",
        "fovRectangularMarkerColor",
        "compassMarksDisplayed",
        "compassMarksColor"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'fovCenterMarkerDisplayedChanged'
        QtMocHelpers::SignalData<void(const bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'fovCenterMarkerColorChanged'
        QtMocHelpers::SignalData<void(const Vec3f &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Signal 'fovCircularMarkerDisplayedChanged'
        QtMocHelpers::SignalData<void(const bool)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'fovCircularMarkerSizeChanged'
        QtMocHelpers::SignalData<void(const double)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 9 },
        }}),
        // Signal 'fovCircularMarkerColorChanged'
        QtMocHelpers::SignalData<void(const Vec3f &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Signal 'fovRectangularMarkerDisplayedChanged'
        QtMocHelpers::SignalData<void(const bool)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'fovRectangularMarkerWidthChanged'
        QtMocHelpers::SignalData<void(const double)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 9 },
        }}),
        // Signal 'fovRectangularMarkerHeightChanged'
        QtMocHelpers::SignalData<void(const double)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 9 },
        }}),
        // Signal 'fovRectangularMarkerRotationAngleChanged'
        QtMocHelpers::SignalData<void(const double)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 9 },
        }}),
        // Signal 'fovRectangularMarkerColorChanged'
        QtMocHelpers::SignalData<void(const Vec3f &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Signal 'compassMarksDisplayedChanged'
        QtMocHelpers::SignalData<void(const bool)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'compassMarksColorChanged'
        QtMocHelpers::SignalData<void(const Vec3f &)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Slot 'setFlagFOVCenterMarker'
        QtMocHelpers::SlotData<void(const bool)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'getFlagFOVCenterMarker'
        QtMocHelpers::SlotData<bool() const>(19, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'getColorFOVCenterMarker'
        QtMocHelpers::SlotData<Vec3f() const>(20, 2, QMC::AccessPublic, 0x80000000 | 5),
        // Slot 'setColorFOVCenterMarker'
        QtMocHelpers::SlotData<void(const Vec3f &)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Slot 'setFlagFOVCircularMarker'
        QtMocHelpers::SlotData<void(const bool)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'getFlagFOVCircularMarker'
        QtMocHelpers::SlotData<bool() const>(23, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFOVCircularMarkerSize'
        QtMocHelpers::SlotData<void(const double)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 9 },
        }}),
        // Slot 'getFOVCircularMarkerSize'
        QtMocHelpers::SlotData<double() const>(25, 2, QMC::AccessPublic, QMetaType::Double),
        // Slot 'getColorFOVCircularMarker'
        QtMocHelpers::SlotData<Vec3f() const>(26, 2, QMC::AccessPublic, 0x80000000 | 5),
        // Slot 'setColorFOVCircularMarker'
        QtMocHelpers::SlotData<void(const Vec3f &)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Slot 'setFlagFOVRectangularMarker'
        QtMocHelpers::SlotData<void(const bool)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'getFlagFOVRectangularMarker'
        QtMocHelpers::SlotData<bool() const>(29, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFOVRectangularMarkerWidth'
        QtMocHelpers::SlotData<void(const double)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 9 },
        }}),
        // Slot 'getFOVRectangularMarkerWidth'
        QtMocHelpers::SlotData<double() const>(31, 2, QMC::AccessPublic, QMetaType::Double),
        // Slot 'setFOVRectangularMarkerHeight'
        QtMocHelpers::SlotData<void(const double)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 9 },
        }}),
        // Slot 'getFOVRectangularMarkerHeight'
        QtMocHelpers::SlotData<double() const>(33, 2, QMC::AccessPublic, QMetaType::Double),
        // Slot 'setFOVRectangularMarkerRotationAngle'
        QtMocHelpers::SlotData<void(const double)>(34, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 35 },
        }}),
        // Slot 'getFOVRectangularMarkerRotationAngle'
        QtMocHelpers::SlotData<double() const>(36, 2, QMC::AccessPublic, QMetaType::Double),
        // Slot 'getColorFOVRectangularMarker'
        QtMocHelpers::SlotData<Vec3f() const>(37, 2, QMC::AccessPublic, 0x80000000 | 5),
        // Slot 'setColorFOVRectangularMarker'
        QtMocHelpers::SlotData<void(const Vec3f &)>(38, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Slot 'setFlagCompassMarks'
        QtMocHelpers::SlotData<void(const bool)>(39, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'getFlagCompassMarks'
        QtMocHelpers::SlotData<bool() const>(40, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'getColorCompassMarks'
        QtMocHelpers::SlotData<Vec3f() const>(41, 2, QMC::AccessPublic, 0x80000000 | 5),
        // Slot 'setColorCompassMarks'
        QtMocHelpers::SlotData<void(const Vec3f &)>(42, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'fovCenterMarkerDisplayed'
        QtMocHelpers::PropertyData<bool>(43, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 0),
        // property 'fovCenterMarkerColor'
        QtMocHelpers::PropertyData<Vec3f>(44, 0x80000000 | 5, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag, 1),
        // property 'fovCircularMarkerDisplayed'
        QtMocHelpers::PropertyData<bool>(45, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 2),
        // property 'fovCircularMarkerSize'
        QtMocHelpers::PropertyData<double>(46, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable, 3),
        // property 'fovCircularMarkerColor'
        QtMocHelpers::PropertyData<Vec3f>(47, 0x80000000 | 5, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag, 4),
        // property 'fovRectangularMarkerDisplayed'
        QtMocHelpers::PropertyData<bool>(48, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 5),
        // property 'fovRectangularMarkerWidth'
        QtMocHelpers::PropertyData<double>(49, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable, 6),
        // property 'fovRectangularMarkerHeight'
        QtMocHelpers::PropertyData<double>(50, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable, 7),
        // property 'fovRectangularMarkerRotationAngle'
        QtMocHelpers::PropertyData<double>(51, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable, 8),
        // property 'fovRectangularMarkerColor'
        QtMocHelpers::PropertyData<Vec3f>(52, 0x80000000 | 5, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag, 9),
        // property 'compassMarksDisplayed'
        QtMocHelpers::PropertyData<bool>(53, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 10),
        // property 'compassMarksColor'
        QtMocHelpers::PropertyData<Vec3f>(54, 0x80000000 | 5, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag, 11),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<SpecialMarkersMgr, qt_meta_tag_ZN17SpecialMarkersMgrE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject SpecialMarkersMgr::staticMetaObject = { {
    QMetaObject::SuperData::link<StelModule::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17SpecialMarkersMgrE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17SpecialMarkersMgrE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17SpecialMarkersMgrE_t>.metaTypes,
    nullptr
} };

void SpecialMarkersMgr::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SpecialMarkersMgr *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->fovCenterMarkerDisplayedChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->fovCenterMarkerColorChanged((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1]))); break;
        case 2: _t->fovCircularMarkerDisplayedChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->fovCircularMarkerSizeChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 4: _t->fovCircularMarkerColorChanged((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1]))); break;
        case 5: _t->fovRectangularMarkerDisplayedChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->fovRectangularMarkerWidthChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 7: _t->fovRectangularMarkerHeightChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 8: _t->fovRectangularMarkerRotationAngleChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 9: _t->fovRectangularMarkerColorChanged((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1]))); break;
        case 10: _t->compassMarksDisplayedChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 11: _t->compassMarksColorChanged((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1]))); break;
        case 12: _t->setFlagFOVCenterMarker((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 13: { bool _r = _t->getFlagFOVCenterMarker();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 14: { Vec3f _r = _t->getColorFOVCenterMarker();
            if (_a[0]) *reinterpret_cast<Vec3f*>(_a[0]) = std::move(_r); }  break;
        case 15: _t->setColorFOVCenterMarker((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1]))); break;
        case 16: _t->setFlagFOVCircularMarker((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 17: { bool _r = _t->getFlagFOVCircularMarker();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 18: _t->setFOVCircularMarkerSize((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 19: { double _r = _t->getFOVCircularMarkerSize();
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 20: { Vec3f _r = _t->getColorFOVCircularMarker();
            if (_a[0]) *reinterpret_cast<Vec3f*>(_a[0]) = std::move(_r); }  break;
        case 21: _t->setColorFOVCircularMarker((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1]))); break;
        case 22: _t->setFlagFOVRectangularMarker((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 23: { bool _r = _t->getFlagFOVRectangularMarker();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 24: _t->setFOVRectangularMarkerWidth((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 25: { double _r = _t->getFOVRectangularMarkerWidth();
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 26: _t->setFOVRectangularMarkerHeight((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 27: { double _r = _t->getFOVRectangularMarkerHeight();
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 28: _t->setFOVRectangularMarkerRotationAngle((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 29: { double _r = _t->getFOVRectangularMarkerRotationAngle();
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 30: { Vec3f _r = _t->getColorFOVRectangularMarker();
            if (_a[0]) *reinterpret_cast<Vec3f*>(_a[0]) = std::move(_r); }  break;
        case 31: _t->setColorFOVRectangularMarker((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1]))); break;
        case 32: _t->setFlagCompassMarks((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 33: { bool _r = _t->getFlagCompassMarks();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 34: { Vec3f _r = _t->getColorCompassMarks();
            if (_a[0]) *reinterpret_cast<Vec3f*>(_a[0]) = std::move(_r); }  break;
        case 35: _t->setColorCompassMarks((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 15:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 21:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 31:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 35:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (SpecialMarkersMgr::*)(const bool )>(_a, &SpecialMarkersMgr::fovCenterMarkerDisplayedChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (SpecialMarkersMgr::*)(const Vec3f & )>(_a, &SpecialMarkersMgr::fovCenterMarkerColorChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (SpecialMarkersMgr::*)(const bool )>(_a, &SpecialMarkersMgr::fovCircularMarkerDisplayedChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (SpecialMarkersMgr::*)(const double )>(_a, &SpecialMarkersMgr::fovCircularMarkerSizeChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (SpecialMarkersMgr::*)(const Vec3f & )>(_a, &SpecialMarkersMgr::fovCircularMarkerColorChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (SpecialMarkersMgr::*)(const bool )>(_a, &SpecialMarkersMgr::fovRectangularMarkerDisplayedChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (SpecialMarkersMgr::*)(const double )>(_a, &SpecialMarkersMgr::fovRectangularMarkerWidthChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (SpecialMarkersMgr::*)(const double )>(_a, &SpecialMarkersMgr::fovRectangularMarkerHeightChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (SpecialMarkersMgr::*)(const double )>(_a, &SpecialMarkersMgr::fovRectangularMarkerRotationAngleChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (SpecialMarkersMgr::*)(const Vec3f & )>(_a, &SpecialMarkersMgr::fovRectangularMarkerColorChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (SpecialMarkersMgr::*)(const bool )>(_a, &SpecialMarkersMgr::compassMarksDisplayedChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (SpecialMarkersMgr::*)(const Vec3f & )>(_a, &SpecialMarkersMgr::compassMarksColorChanged, 11))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 11:
        case 9:
        case 4:
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Vec3f >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->getFlagFOVCenterMarker(); break;
        case 1: *reinterpret_cast<Vec3f*>(_v) = _t->getColorFOVCenterMarker(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->getFlagFOVCircularMarker(); break;
        case 3: *reinterpret_cast<double*>(_v) = _t->getFOVCircularMarkerSize(); break;
        case 4: *reinterpret_cast<Vec3f*>(_v) = _t->getColorFOVCircularMarker(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->getFlagFOVRectangularMarker(); break;
        case 6: *reinterpret_cast<double*>(_v) = _t->getFOVRectangularMarkerWidth(); break;
        case 7: *reinterpret_cast<double*>(_v) = _t->getFOVRectangularMarkerHeight(); break;
        case 8: *reinterpret_cast<double*>(_v) = _t->getFOVRectangularMarkerRotationAngle(); break;
        case 9: *reinterpret_cast<Vec3f*>(_v) = _t->getColorFOVRectangularMarker(); break;
        case 10: *reinterpret_cast<bool*>(_v) = _t->getFlagCompassMarks(); break;
        case 11: *reinterpret_cast<Vec3f*>(_v) = _t->getColorCompassMarks(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setFlagFOVCenterMarker(*reinterpret_cast<bool*>(_v)); break;
        case 1: _t->setColorFOVCenterMarker(*reinterpret_cast<Vec3f*>(_v)); break;
        case 2: _t->setFlagFOVCircularMarker(*reinterpret_cast<bool*>(_v)); break;
        case 3: _t->setFOVCircularMarkerSize(*reinterpret_cast<double*>(_v)); break;
        case 4: _t->setColorFOVCircularMarker(*reinterpret_cast<Vec3f*>(_v)); break;
        case 5: _t->setFlagFOVRectangularMarker(*reinterpret_cast<bool*>(_v)); break;
        case 6: _t->setFOVRectangularMarkerWidth(*reinterpret_cast<double*>(_v)); break;
        case 7: _t->setFOVRectangularMarkerHeight(*reinterpret_cast<double*>(_v)); break;
        case 8: _t->setFOVRectangularMarkerRotationAngle(*reinterpret_cast<double*>(_v)); break;
        case 9: _t->setColorFOVRectangularMarker(*reinterpret_cast<Vec3f*>(_v)); break;
        case 10: _t->setFlagCompassMarks(*reinterpret_cast<bool*>(_v)); break;
        case 11: _t->setColorCompassMarks(*reinterpret_cast<Vec3f*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *SpecialMarkersMgr::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SpecialMarkersMgr::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17SpecialMarkersMgrE_t>.strings))
        return static_cast<void*>(this);
    return StelModule::qt_metacast(_clname);
}

int SpecialMarkersMgr::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StelModule::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 36)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 36;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 36)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 36;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void SpecialMarkersMgr::fovCenterMarkerDisplayedChanged(const bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void SpecialMarkersMgr::fovCenterMarkerColorChanged(const Vec3f & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void SpecialMarkersMgr::fovCircularMarkerDisplayedChanged(const bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void SpecialMarkersMgr::fovCircularMarkerSizeChanged(const double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void SpecialMarkersMgr::fovCircularMarkerColorChanged(const Vec3f & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void SpecialMarkersMgr::fovRectangularMarkerDisplayedChanged(const bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void SpecialMarkersMgr::fovRectangularMarkerWidthChanged(const double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void SpecialMarkersMgr::fovRectangularMarkerHeightChanged(const double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void SpecialMarkersMgr::fovRectangularMarkerRotationAngleChanged(const double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void SpecialMarkersMgr::fovRectangularMarkerColorChanged(const Vec3f & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void SpecialMarkersMgr::compassMarksDisplayedChanged(const bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void SpecialMarkersMgr::compassMarksColorChanged(const Vec3f & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}
QT_WARNING_POP
