/********************************************************************************
** Form generated from reading UI file 'dsoColorsDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DSOCOLORSDIALOG_H
#define UI_DSOCOLORSDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "Dialog.hpp"
#include "gui/ColorButton.hpp"

QT_BEGIN_NAMESPACE

class Ui_ConfigureDSOColorsDialogForm
{
public:
    QVBoxLayout *verticalLayout;
    TitleBar *titleBar;
    QGroupBox *colorsGroupBox;
    QGridLayout *gridLayout_18;
    ColorButton *colorDSOBLLacObjects;
    QLabel *labelDSOColorPossibleQuasars;
    ColorButton *colorDSOStars;
    QLabel *labelDSOColorPossiblePlanetaryNebulae;
    QLabel *labelDSOColorNebulae;
    ColorButton *colorDSOEmissionLineStars;
    QLabel *labelDSOColorReflectionNebulae;
    QLabel *labelDSOColorRadioGalaxies;
    QLabel *labelDSOColorActiveGalaxies;
    ColorButton *colorDSOBlazars;
    QLabel *labelDSOColorStarClusters;
    QLabel *labelDSOColorEmissionObjects;
    ColorButton *colorDSOInteractingGalaxies;
    QLabel *labelDSOColorGlobularStarClusters;
    ColorButton *colorDSOYoungStellarObjects;
    ColorButton *colorDSOStarClouds;
    QLabel *labelDSOColorSupernovaRemnants;
    ColorButton *colorDSOBipolarNebulae;
    ColorButton *colorDSOSupernovaCandidates;
    QLabel *labelDSOColorPlanetaryNebulae;
    ColorButton *colorDSOInterstellarMatter;
    ColorButton *colorDSOGalaxies;
    QLabel *labelDSOColorGalaxies;
    ColorButton *colorDSOStellarAssociations;
    QLabel *labelDSOColorOpenStarClusters;
    QLabel *labelDSOColorBlazars;
    ColorButton *colorDSODarkNebulae;
    ColorButton *colorDSOEmissionNebulae;
    ColorButton *colorDSOActiveGalaxies;
    QLabel *labelDSOColorBipolarNebulae;
    ColorButton *colorDSOLabels;
    ColorButton *colorDSOMolecularClouds;
    QLabel *labelDSOColorBLLacObjects;
    ColorButton *colorDSOStarClusters;
    QLabel *labelDSOColorInterstellarMatter;
    ColorButton *colorDSOGlobularStarClusters;
    ColorButton *colorDSOPossiblePlanetaryNebulae;
    QLabel *labelDSOColorYSOs;
    QLabel *labelDSOColorHydrogenRegions;
    ColorButton *colorDSONebulae;
    QLabel *labelDSOColorSupernovaCandidates;
    ColorButton *colorDSOPlanetaryNebulae;
    ColorButton *colorDSOSupernovaRemnantCandidates;
    ColorButton *colorDSOPossibleQuasars;
    QLabel *labelDSOColorInteractingGalaxies;
    ColorButton *colorDSOQuasars;
    ColorButton *colorDSOReflectionNebulae;
    QLabel *labelDSOColorStellarAssociations;
    QLabel *labelDSOColorProtoplanetaryNebulae;
    QLabel *labelDSOColorDarkNebulae;
    QLabel *labelDSOColorMarkers;
    ColorButton *colorDSOMarkers;
    QLabel *labelDSOColorEmissionNebulae;
    ColorButton *colorDSORadioGalaxies;
    QLabel *labelDSOColorEmissionLineStars;
    QLabel *labelDSOColorQuasars;
    ColorButton *colorDSOEmissionObjects;
    QLabel *labelDSOColorStarClouds;
    QLabel *labelDSOColorMolecularClouds;
    QLabel *labelDSOColorSupernovaRemnantCandidates;
    ColorButton *colorDSOOpenStarClusters;
    ColorButton *colorDSOSymbioticStars;
    ColorButton *colorDSOProtoplanetaryNebulae;
    ColorButton *colorDSOHydrogenRegions;
    QLabel *labelDSOColorGalaxyClusters;
    QLabel *labelDSOColorSymbioticStars;
    ColorButton *colorDSOSupernovaRemnants;
    QLabel *labelDSOColorLabels;
    QLabel *labelDSOColorStars;
    ColorButton *colorDSOGalaxyClusters;
    QLabel *labelDSOColorRegions;
    ColorButton *colorDSORegions;

    void setupUi(QWidget *ConfigureDSOColorsDialogForm)
    {
        if (ConfigureDSOColorsDialogForm->objectName().isEmpty())
            ConfigureDSOColorsDialogForm->setObjectName("ConfigureDSOColorsDialogForm");
        ConfigureDSOColorsDialogForm->resize(845, 459);
        verticalLayout = new QVBoxLayout(ConfigureDSOColorsDialogForm);
        verticalLayout->setSpacing(0);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        titleBar = new TitleBar(ConfigureDSOColorsDialogForm);
        titleBar->setObjectName("titleBar");

        verticalLayout->addWidget(titleBar);

        colorsGroupBox = new QGroupBox(ConfigureDSOColorsDialogForm);
        colorsGroupBox->setObjectName("colorsGroupBox");
        gridLayout_18 = new QGridLayout(colorsGroupBox);
        gridLayout_18->setObjectName("gridLayout_18");
        gridLayout_18->setContentsMargins(0, 0, 0, 0);
        colorDSOBLLacObjects = new ColorButton(colorsGroupBox);
        colorDSOBLLacObjects->setObjectName("colorDSOBLLacObjects");

        gridLayout_18->addWidget(colorDSOBLLacObjects, 2, 4, 1, 1);

        labelDSOColorPossibleQuasars = new QLabel(colorsGroupBox);
        labelDSOColorPossibleQuasars->setObjectName("labelDSOColorPossibleQuasars");

        gridLayout_18->addWidget(labelDSOColorPossibleQuasars, 2, 3, 1, 1);

        colorDSOStars = new ColorButton(colorsGroupBox);
        colorDSOStars->setObjectName("colorDSOStars");

        gridLayout_18->addWidget(colorDSOStars, 5, 0, 1, 1);

        labelDSOColorPossiblePlanetaryNebulae = new QLabel(colorsGroupBox);
        labelDSOColorPossiblePlanetaryNebulae->setObjectName("labelDSOColorPossiblePlanetaryNebulae");

        gridLayout_18->addWidget(labelDSOColorPossiblePlanetaryNebulae, 6, 3, 1, 1);

        labelDSOColorNebulae = new QLabel(colorsGroupBox);
        labelDSOColorNebulae->setObjectName("labelDSOColorNebulae");

        gridLayout_18->addWidget(labelDSOColorNebulae, 5, 5, 1, 1);

        colorDSOEmissionLineStars = new ColorButton(colorsGroupBox);
        colorDSOEmissionLineStars->setObjectName("colorDSOEmissionLineStars");

        gridLayout_18->addWidget(colorDSOEmissionLineStars, 10, 4, 1, 1);

        labelDSOColorReflectionNebulae = new QLabel(colorsGroupBox);
        labelDSOColorReflectionNebulae->setObjectName("labelDSOColorReflectionNebulae");

        gridLayout_18->addWidget(labelDSOColorReflectionNebulae, 7, 3, 1, 1);

        labelDSOColorRadioGalaxies = new QLabel(colorsGroupBox);
        labelDSOColorRadioGalaxies->setObjectName("labelDSOColorRadioGalaxies");

        gridLayout_18->addWidget(labelDSOColorRadioGalaxies, 1, 3, 1, 1);

        labelDSOColorActiveGalaxies = new QLabel(colorsGroupBox);
        labelDSOColorActiveGalaxies->setObjectName("labelDSOColorActiveGalaxies");

        gridLayout_18->addWidget(labelDSOColorActiveGalaxies, 1, 1, 1, 1);

        colorDSOBlazars = new ColorButton(colorsGroupBox);
        colorDSOBlazars->setObjectName("colorDSOBlazars");

        gridLayout_18->addWidget(colorDSOBlazars, 3, 0, 1, 1);

        labelDSOColorStarClusters = new QLabel(colorsGroupBox);
        labelDSOColorStarClusters->setObjectName("labelDSOColorStarClusters");

        gridLayout_18->addWidget(labelDSOColorStarClusters, 3, 3, 1, 1);

        labelDSOColorEmissionObjects = new QLabel(colorsGroupBox);
        labelDSOColorEmissionObjects->setObjectName("labelDSOColorEmissionObjects");

        gridLayout_18->addWidget(labelDSOColorEmissionObjects, 9, 3, 1, 1);

        colorDSOInteractingGalaxies = new ColorButton(colorsGroupBox);
        colorDSOInteractingGalaxies->setObjectName("colorDSOInteractingGalaxies");

        gridLayout_18->addWidget(colorDSOInteractingGalaxies, 1, 4, 1, 1);

        labelDSOColorGlobularStarClusters = new QLabel(colorsGroupBox);
        labelDSOColorGlobularStarClusters->setObjectName("labelDSOColorGlobularStarClusters");

        gridLayout_18->addWidget(labelDSOColorGlobularStarClusters, 4, 1, 1, 1);

        colorDSOYoungStellarObjects = new ColorButton(colorsGroupBox);
        colorDSOYoungStellarObjects->setObjectName("colorDSOYoungStellarObjects");

        gridLayout_18->addWidget(colorDSOYoungStellarObjects, 9, 4, 1, 1);

        colorDSOStarClouds = new ColorButton(colorsGroupBox);
        colorDSOStarClouds->setObjectName("colorDSOStarClouds");

        gridLayout_18->addWidget(colorDSOStarClouds, 4, 4, 1, 1);

        labelDSOColorSupernovaRemnants = new QLabel(colorsGroupBox);
        labelDSOColorSupernovaRemnants->setObjectName("labelDSOColorSupernovaRemnants");

        gridLayout_18->addWidget(labelDSOColorSupernovaRemnants, 10, 1, 1, 1);

        colorDSOBipolarNebulae = new ColorButton(colorsGroupBox);
        colorDSOBipolarNebulae->setObjectName("colorDSOBipolarNebulae");

        gridLayout_18->addWidget(colorDSOBipolarNebulae, 7, 4, 1, 1);

        colorDSOSupernovaCandidates = new ColorButton(colorsGroupBox);
        colorDSOSupernovaCandidates->setObjectName("colorDSOSupernovaCandidates");

        gridLayout_18->addWidget(colorDSOSupernovaCandidates, 11, 0, 1, 1);

        labelDSOColorPlanetaryNebulae = new QLabel(colorsGroupBox);
        labelDSOColorPlanetaryNebulae->setObjectName("labelDSOColorPlanetaryNebulae");

        gridLayout_18->addWidget(labelDSOColorPlanetaryNebulae, 6, 1, 1, 1);

        colorDSOInterstellarMatter = new ColorButton(colorsGroupBox);
        colorDSOInterstellarMatter->setObjectName("colorDSOInterstellarMatter");

        gridLayout_18->addWidget(colorDSOInterstellarMatter, 9, 0, 1, 1);

        colorDSOGalaxies = new ColorButton(colorsGroupBox);
        colorDSOGalaxies->setObjectName("colorDSOGalaxies");

        gridLayout_18->addWidget(colorDSOGalaxies, 0, 4, 1, 1);

        labelDSOColorGalaxies = new QLabel(colorsGroupBox);
        labelDSOColorGalaxies->setObjectName("labelDSOColorGalaxies");

        gridLayout_18->addWidget(labelDSOColorGalaxies, 0, 5, 1, 1);

        colorDSOStellarAssociations = new ColorButton(colorsGroupBox);
        colorDSOStellarAssociations->setObjectName("colorDSOStellarAssociations");

        gridLayout_18->addWidget(colorDSOStellarAssociations, 4, 2, 1, 1);

        labelDSOColorOpenStarClusters = new QLabel(colorsGroupBox);
        labelDSOColorOpenStarClusters->setObjectName("labelDSOColorOpenStarClusters");

        gridLayout_18->addWidget(labelDSOColorOpenStarClusters, 3, 5, 1, 1);

        labelDSOColorBlazars = new QLabel(colorsGroupBox);
        labelDSOColorBlazars->setObjectName("labelDSOColorBlazars");

        gridLayout_18->addWidget(labelDSOColorBlazars, 3, 1, 1, 1);

        colorDSODarkNebulae = new ColorButton(colorsGroupBox);
        colorDSODarkNebulae->setObjectName("colorDSODarkNebulae");

        gridLayout_18->addWidget(colorDSODarkNebulae, 7, 0, 1, 1);

        colorDSOEmissionNebulae = new ColorButton(colorsGroupBox);
        colorDSOEmissionNebulae->setObjectName("colorDSOEmissionNebulae");

        gridLayout_18->addWidget(colorDSOEmissionNebulae, 8, 0, 1, 1);

        colorDSOActiveGalaxies = new ColorButton(colorsGroupBox);
        colorDSOActiveGalaxies->setObjectName("colorDSOActiveGalaxies");

        gridLayout_18->addWidget(colorDSOActiveGalaxies, 1, 0, 1, 1);

        labelDSOColorBipolarNebulae = new QLabel(colorsGroupBox);
        labelDSOColorBipolarNebulae->setObjectName("labelDSOColorBipolarNebulae");

        gridLayout_18->addWidget(labelDSOColorBipolarNebulae, 7, 5, 1, 1);

        colorDSOLabels = new ColorButton(colorsGroupBox);
        colorDSOLabels->setObjectName("colorDSOLabels");

        gridLayout_18->addWidget(colorDSOLabels, 0, 0, 1, 1);

        colorDSOMolecularClouds = new ColorButton(colorsGroupBox);
        colorDSOMolecularClouds->setObjectName("colorDSOMolecularClouds");

        gridLayout_18->addWidget(colorDSOMolecularClouds, 8, 4, 1, 1);

        labelDSOColorBLLacObjects = new QLabel(colorsGroupBox);
        labelDSOColorBLLacObjects->setObjectName("labelDSOColorBLLacObjects");

        gridLayout_18->addWidget(labelDSOColorBLLacObjects, 2, 5, 1, 1);

        colorDSOStarClusters = new ColorButton(colorsGroupBox);
        colorDSOStarClusters->setObjectName("colorDSOStarClusters");

        gridLayout_18->addWidget(colorDSOStarClusters, 3, 2, 1, 1);

        labelDSOColorInterstellarMatter = new QLabel(colorsGroupBox);
        labelDSOColorInterstellarMatter->setObjectName("labelDSOColorInterstellarMatter");

        gridLayout_18->addWidget(labelDSOColorInterstellarMatter, 9, 1, 1, 1);

        colorDSOGlobularStarClusters = new ColorButton(colorsGroupBox);
        colorDSOGlobularStarClusters->setObjectName("colorDSOGlobularStarClusters");

        gridLayout_18->addWidget(colorDSOGlobularStarClusters, 4, 0, 1, 1);

        colorDSOPossiblePlanetaryNebulae = new ColorButton(colorsGroupBox);
        colorDSOPossiblePlanetaryNebulae->setObjectName("colorDSOPossiblePlanetaryNebulae");

        gridLayout_18->addWidget(colorDSOPossiblePlanetaryNebulae, 6, 2, 1, 1);

        labelDSOColorYSOs = new QLabel(colorsGroupBox);
        labelDSOColorYSOs->setObjectName("labelDSOColorYSOs");

        gridLayout_18->addWidget(labelDSOColorYSOs, 9, 5, 1, 1);

        labelDSOColorHydrogenRegions = new QLabel(colorsGroupBox);
        labelDSOColorHydrogenRegions->setObjectName("labelDSOColorHydrogenRegions");

        gridLayout_18->addWidget(labelDSOColorHydrogenRegions, 8, 3, 1, 1);

        colorDSONebulae = new ColorButton(colorsGroupBox);
        colorDSONebulae->setObjectName("colorDSONebulae");

        gridLayout_18->addWidget(colorDSONebulae, 5, 4, 1, 1);

        labelDSOColorSupernovaCandidates = new QLabel(colorsGroupBox);
        labelDSOColorSupernovaCandidates->setObjectName("labelDSOColorSupernovaCandidates");

        gridLayout_18->addWidget(labelDSOColorSupernovaCandidates, 11, 1, 1, 1);

        colorDSOPlanetaryNebulae = new ColorButton(colorsGroupBox);
        colorDSOPlanetaryNebulae->setObjectName("colorDSOPlanetaryNebulae");

        gridLayout_18->addWidget(colorDSOPlanetaryNebulae, 6, 0, 1, 1);

        colorDSOSupernovaRemnantCandidates = new ColorButton(colorsGroupBox);
        colorDSOSupernovaRemnantCandidates->setObjectName("colorDSOSupernovaRemnantCandidates");

        gridLayout_18->addWidget(colorDSOSupernovaRemnantCandidates, 11, 2, 1, 1);

        colorDSOPossibleQuasars = new ColorButton(colorsGroupBox);
        colorDSOPossibleQuasars->setObjectName("colorDSOPossibleQuasars");

        gridLayout_18->addWidget(colorDSOPossibleQuasars, 2, 2, 1, 1);

        labelDSOColorInteractingGalaxies = new QLabel(colorsGroupBox);
        labelDSOColorInteractingGalaxies->setObjectName("labelDSOColorInteractingGalaxies");

        gridLayout_18->addWidget(labelDSOColorInteractingGalaxies, 1, 5, 1, 1);

        colorDSOQuasars = new ColorButton(colorsGroupBox);
        colorDSOQuasars->setObjectName("colorDSOQuasars");

        gridLayout_18->addWidget(colorDSOQuasars, 2, 0, 1, 1);

        colorDSOReflectionNebulae = new ColorButton(colorsGroupBox);
        colorDSOReflectionNebulae->setObjectName("colorDSOReflectionNebulae");

        gridLayout_18->addWidget(colorDSOReflectionNebulae, 7, 2, 1, 1);

        labelDSOColorStellarAssociations = new QLabel(colorsGroupBox);
        labelDSOColorStellarAssociations->setObjectName("labelDSOColorStellarAssociations");

        gridLayout_18->addWidget(labelDSOColorStellarAssociations, 4, 3, 1, 1);

        labelDSOColorProtoplanetaryNebulae = new QLabel(colorsGroupBox);
        labelDSOColorProtoplanetaryNebulae->setObjectName("labelDSOColorProtoplanetaryNebulae");

        gridLayout_18->addWidget(labelDSOColorProtoplanetaryNebulae, 6, 5, 1, 1);

        labelDSOColorDarkNebulae = new QLabel(colorsGroupBox);
        labelDSOColorDarkNebulae->setObjectName("labelDSOColorDarkNebulae");

        gridLayout_18->addWidget(labelDSOColorDarkNebulae, 7, 1, 1, 1);

        labelDSOColorMarkers = new QLabel(colorsGroupBox);
        labelDSOColorMarkers->setObjectName("labelDSOColorMarkers");

        gridLayout_18->addWidget(labelDSOColorMarkers, 0, 3, 1, 1);

        colorDSOMarkers = new ColorButton(colorsGroupBox);
        colorDSOMarkers->setObjectName("colorDSOMarkers");

        gridLayout_18->addWidget(colorDSOMarkers, 0, 2, 1, 1);

        labelDSOColorEmissionNebulae = new QLabel(colorsGroupBox);
        labelDSOColorEmissionNebulae->setObjectName("labelDSOColorEmissionNebulae");

        gridLayout_18->addWidget(labelDSOColorEmissionNebulae, 8, 1, 1, 1);

        colorDSORadioGalaxies = new ColorButton(colorsGroupBox);
        colorDSORadioGalaxies->setObjectName("colorDSORadioGalaxies");

        gridLayout_18->addWidget(colorDSORadioGalaxies, 1, 2, 1, 1);

        labelDSOColorEmissionLineStars = new QLabel(colorsGroupBox);
        labelDSOColorEmissionLineStars->setObjectName("labelDSOColorEmissionLineStars");

        gridLayout_18->addWidget(labelDSOColorEmissionLineStars, 10, 5, 1, 1);

        labelDSOColorQuasars = new QLabel(colorsGroupBox);
        labelDSOColorQuasars->setObjectName("labelDSOColorQuasars");

        gridLayout_18->addWidget(labelDSOColorQuasars, 2, 1, 1, 1);

        colorDSOEmissionObjects = new ColorButton(colorsGroupBox);
        colorDSOEmissionObjects->setObjectName("colorDSOEmissionObjects");

        gridLayout_18->addWidget(colorDSOEmissionObjects, 9, 2, 1, 1);

        labelDSOColorStarClouds = new QLabel(colorsGroupBox);
        labelDSOColorStarClouds->setObjectName("labelDSOColorStarClouds");

        gridLayout_18->addWidget(labelDSOColorStarClouds, 4, 5, 1, 1);

        labelDSOColorMolecularClouds = new QLabel(colorsGroupBox);
        labelDSOColorMolecularClouds->setObjectName("labelDSOColorMolecularClouds");

        gridLayout_18->addWidget(labelDSOColorMolecularClouds, 8, 5, 1, 1);

        labelDSOColorSupernovaRemnantCandidates = new QLabel(colorsGroupBox);
        labelDSOColorSupernovaRemnantCandidates->setObjectName("labelDSOColorSupernovaRemnantCandidates");

        gridLayout_18->addWidget(labelDSOColorSupernovaRemnantCandidates, 11, 3, 1, 1);

        colorDSOOpenStarClusters = new ColorButton(colorsGroupBox);
        colorDSOOpenStarClusters->setObjectName("colorDSOOpenStarClusters");

        gridLayout_18->addWidget(colorDSOOpenStarClusters, 3, 4, 1, 1);

        colorDSOSymbioticStars = new ColorButton(colorsGroupBox);
        colorDSOSymbioticStars->setObjectName("colorDSOSymbioticStars");

        gridLayout_18->addWidget(colorDSOSymbioticStars, 10, 2, 1, 1);

        colorDSOProtoplanetaryNebulae = new ColorButton(colorsGroupBox);
        colorDSOProtoplanetaryNebulae->setObjectName("colorDSOProtoplanetaryNebulae");

        gridLayout_18->addWidget(colorDSOProtoplanetaryNebulae, 6, 4, 1, 1);

        colorDSOHydrogenRegions = new ColorButton(colorsGroupBox);
        colorDSOHydrogenRegions->setObjectName("colorDSOHydrogenRegions");

        gridLayout_18->addWidget(colorDSOHydrogenRegions, 8, 2, 1, 1);

        labelDSOColorGalaxyClusters = new QLabel(colorsGroupBox);
        labelDSOColorGalaxyClusters->setObjectName("labelDSOColorGalaxyClusters");

        gridLayout_18->addWidget(labelDSOColorGalaxyClusters, 11, 5, 1, 1);

        labelDSOColorSymbioticStars = new QLabel(colorsGroupBox);
        labelDSOColorSymbioticStars->setObjectName("labelDSOColorSymbioticStars");

        gridLayout_18->addWidget(labelDSOColorSymbioticStars, 10, 3, 1, 1);

        colorDSOSupernovaRemnants = new ColorButton(colorsGroupBox);
        colorDSOSupernovaRemnants->setObjectName("colorDSOSupernovaRemnants");

        gridLayout_18->addWidget(colorDSOSupernovaRemnants, 10, 0, 1, 1);

        labelDSOColorLabels = new QLabel(colorsGroupBox);
        labelDSOColorLabels->setObjectName("labelDSOColorLabels");

        gridLayout_18->addWidget(labelDSOColorLabels, 0, 1, 1, 1);

        labelDSOColorStars = new QLabel(colorsGroupBox);
        labelDSOColorStars->setObjectName("labelDSOColorStars");

        gridLayout_18->addWidget(labelDSOColorStars, 5, 1, 1, 1);

        colorDSOGalaxyClusters = new ColorButton(colorsGroupBox);
        colorDSOGalaxyClusters->setObjectName("colorDSOGalaxyClusters");

        gridLayout_18->addWidget(colorDSOGalaxyClusters, 11, 4, 1, 1);

        labelDSOColorRegions = new QLabel(colorsGroupBox);
        labelDSOColorRegions->setObjectName("labelDSOColorRegions");

        gridLayout_18->addWidget(labelDSOColorRegions, 5, 3, 1, 1);

        colorDSORegions = new ColorButton(colorsGroupBox);
        colorDSORegions->setObjectName("colorDSORegions");

        gridLayout_18->addWidget(colorDSORegions, 5, 2, 1, 1);


        verticalLayout->addWidget(colorsGroupBox);


        retranslateUi(ConfigureDSOColorsDialogForm);

        QMetaObject::connectSlotsByName(ConfigureDSOColorsDialogForm);
    } // setupUi

    void retranslateUi(QWidget *ConfigureDSOColorsDialogForm)
    {
        titleBar->setTitle(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Color settings", nullptr));
        colorsGroupBox->setTitle(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Colors of labels and markers of deep-sky objects", nullptr));
#if QT_CONFIG(tooltip)
        colorDSOBLLacObjects->setToolTip(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Color of markers of BL Lac objects", nullptr));
#endif // QT_CONFIG(tooltip)
        labelDSOColorPossibleQuasars->setText(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Possible quasars", nullptr));
#if QT_CONFIG(tooltip)
        colorDSOStars->setToolTip(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Color of markers of stars", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        labelDSOColorPossiblePlanetaryNebulae->setToolTip(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Possible planetary nebulae", nullptr));
#endif // QT_CONFIG(tooltip)
        labelDSOColorPossiblePlanetaryNebulae->setText(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Possible pl. nebulae", nullptr));
        labelDSOColorNebulae->setText(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Nebulae", nullptr));
#if QT_CONFIG(tooltip)
        colorDSOEmissionLineStars->setToolTip(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Color of markers of emission-line stars", nullptr));
#endif // QT_CONFIG(tooltip)
        labelDSOColorReflectionNebulae->setText(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Reflection nebulae", nullptr));
        labelDSOColorRadioGalaxies->setText(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Radio galaxies", nullptr));
        labelDSOColorActiveGalaxies->setText(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Active galaxies", nullptr));
#if QT_CONFIG(tooltip)
        colorDSOBlazars->setToolTip(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Color of markers of blazars", nullptr));
#endif // QT_CONFIG(tooltip)
        labelDSOColorStarClusters->setText(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Star clusters", nullptr));
        labelDSOColorEmissionObjects->setText(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Emission objects", nullptr));
#if QT_CONFIG(tooltip)
        colorDSOInteractingGalaxies->setToolTip(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Color of markers of interacting galaxies", nullptr));
#endif // QT_CONFIG(tooltip)
        labelDSOColorGlobularStarClusters->setText(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Globular star clusters", nullptr));
#if QT_CONFIG(tooltip)
        colorDSOYoungStellarObjects->setToolTip(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Color of markers of young stellar objects", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        colorDSOStarClouds->setToolTip(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Color of markers of star clouds", nullptr));
#endif // QT_CONFIG(tooltip)
        labelDSOColorSupernovaRemnants->setText(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Supernova remnants", nullptr));
#if QT_CONFIG(tooltip)
        colorDSOBipolarNebulae->setToolTip(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Color of markers of bipolar nebulae", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        colorDSOSupernovaCandidates->setToolTip(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Color of markers of supernova candidates", nullptr));
#endif // QT_CONFIG(tooltip)
        labelDSOColorPlanetaryNebulae->setText(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Planetary nebulae", nullptr));
#if QT_CONFIG(tooltip)
        colorDSOInterstellarMatter->setToolTip(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Color of markers of interstellar matter", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        colorDSOGalaxies->setToolTip(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Color of markers of galaxies", nullptr));
#endif // QT_CONFIG(tooltip)
        labelDSOColorGalaxies->setText(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Galaxies", nullptr));
#if QT_CONFIG(tooltip)
        colorDSOStellarAssociations->setToolTip(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Color of markers of stellar associations", nullptr));
#endif // QT_CONFIG(tooltip)
        labelDSOColorOpenStarClusters->setText(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Open star clusters", nullptr));
        labelDSOColorBlazars->setText(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Blazars", nullptr));
#if QT_CONFIG(tooltip)
        colorDSODarkNebulae->setToolTip(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Color of markers of dark nebulae", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        colorDSOEmissionNebulae->setToolTip(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Color of markers of emission nebulae", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        colorDSOActiveGalaxies->setToolTip(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Color of markers of active galaxies", nullptr));
#endif // QT_CONFIG(tooltip)
        labelDSOColorBipolarNebulae->setText(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Bipolar nebulae", nullptr));
#if QT_CONFIG(tooltip)
        colorDSOLabels->setToolTip(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Color of labels of deep-sky objects", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        colorDSOMolecularClouds->setToolTip(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Color of markers of molecular clouds", nullptr));
#endif // QT_CONFIG(tooltip)
        labelDSOColorBLLacObjects->setText(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "BL Lac objects", nullptr));
#if QT_CONFIG(tooltip)
        colorDSOStarClusters->setToolTip(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Color of markers of star clusters", nullptr));
#endif // QT_CONFIG(tooltip)
        labelDSOColorInterstellarMatter->setText(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Interstellar matter", nullptr));
#if QT_CONFIG(tooltip)
        colorDSOGlobularStarClusters->setToolTip(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Color of markers of globular star clusters", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        colorDSOPossiblePlanetaryNebulae->setToolTip(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Color of markers of possible planetary nebulae", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        labelDSOColorYSOs->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
        labelDSOColorYSOs->setText(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Young stellar objects", nullptr));
        labelDSOColorHydrogenRegions->setText(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Hydrogen regions", nullptr));
#if QT_CONFIG(tooltip)
        colorDSONebulae->setToolTip(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Color of markers of nebulae", nullptr));
#endif // QT_CONFIG(tooltip)
        labelDSOColorSupernovaCandidates->setText(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Supernova candidates", nullptr));
#if QT_CONFIG(tooltip)
        colorDSOPlanetaryNebulae->setToolTip(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Color of markers of planetary nebulae", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        colorDSOSupernovaRemnantCandidates->setToolTip(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Color of markers of supernova remnant candidates", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        colorDSOPossibleQuasars->setToolTip(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Color of markers of possible quasars", nullptr));
#endif // QT_CONFIG(tooltip)
        labelDSOColorInteractingGalaxies->setText(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Interacting galaxies", nullptr));
#if QT_CONFIG(tooltip)
        colorDSOQuasars->setToolTip(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Color of markers of quasars", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        colorDSOReflectionNebulae->setToolTip(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Color of markers of reflection nebulae", nullptr));
#endif // QT_CONFIG(tooltip)
        labelDSOColorStellarAssociations->setText(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Stellar associations", nullptr));
        labelDSOColorProtoplanetaryNebulae->setText(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Protoplanetary nebulae", nullptr));
        labelDSOColorDarkNebulae->setText(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Dark nebulae", nullptr));
#if QT_CONFIG(tooltip)
        labelDSOColorMarkers->setToolTip(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Markers of deep-sky objects", nullptr));
#endif // QT_CONFIG(tooltip)
        labelDSOColorMarkers->setText(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Markers of DSOs", nullptr));
#if QT_CONFIG(tooltip)
        colorDSOMarkers->setToolTip(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Color of markers of deep-sky objects", nullptr));
#endif // QT_CONFIG(tooltip)
        labelDSOColorEmissionNebulae->setText(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Emission nebulae", nullptr));
#if QT_CONFIG(tooltip)
        colorDSORadioGalaxies->setToolTip(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Color of markers of radio galaxies", nullptr));
#endif // QT_CONFIG(tooltip)
        labelDSOColorEmissionLineStars->setText(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Emission-line stars", nullptr));
        labelDSOColorQuasars->setText(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Quasars", nullptr));
#if QT_CONFIG(tooltip)
        colorDSOEmissionObjects->setToolTip(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Color of markers of emission objects", nullptr));
#endif // QT_CONFIG(tooltip)
        labelDSOColorStarClouds->setText(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Star clouds", nullptr));
        labelDSOColorMolecularClouds->setText(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Molecular clouds", nullptr));
        labelDSOColorSupernovaRemnantCandidates->setText(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Supernova remnant candidates", nullptr));
#if QT_CONFIG(tooltip)
        colorDSOOpenStarClusters->setToolTip(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Color of markers of open star clusters", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        colorDSOSymbioticStars->setToolTip(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Color of markers of symbiotic stars", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        colorDSOProtoplanetaryNebulae->setToolTip(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Color of markers of protoplanetary nebulae", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        colorDSOHydrogenRegions->setToolTip(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Color of markers of hydrogen regions", nullptr));
#endif // QT_CONFIG(tooltip)
        labelDSOColorGalaxyClusters->setText(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Clusters of galaxies", nullptr));
        labelDSOColorSymbioticStars->setText(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Symbiotic stars", nullptr));
#if QT_CONFIG(tooltip)
        colorDSOSupernovaRemnants->setToolTip(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Color of markers of supernova remnants", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        labelDSOColorLabels->setToolTip(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Labels of deep-sky objects", nullptr));
#endif // QT_CONFIG(tooltip)
        labelDSOColorLabels->setText(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Labels of DSOs", nullptr));
        labelDSOColorStars->setText(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Stars", nullptr));
#if QT_CONFIG(tooltip)
        colorDSOGalaxyClusters->setToolTip(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Color of markers for clusters of galaxies", nullptr));
#endif // QT_CONFIG(tooltip)
        labelDSOColorRegions->setText(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Regions of the sky", nullptr));
#if QT_CONFIG(tooltip)
        colorDSORegions->setToolTip(QCoreApplication::translate("ConfigureDSOColorsDialogForm", "Color of markers of regions of the sky", nullptr));
#endif // QT_CONFIG(tooltip)
        (void)ConfigureDSOColorsDialogForm;
    } // retranslateUi

};

namespace Ui {
    class ConfigureDSOColorsDialogForm: public Ui_ConfigureDSOColorsDialogForm {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DSOCOLORSDIALOG_H
