/****************************************************************************
** Meta object code from reading C++ file 'Scenery3dDialog.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../stellarium/plugins/Scenery3d/src/gui/Scenery3dDialog.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'Scenery3dDialog.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15Scenery3dDialogE_t {};
} // unnamed namespace

template <> constexpr inline auto Scenery3dDialog::qt_create_metaobjectdata<qt_meta_tag_ZN15Scenery3dDialogE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Scenery3dDialog",
        "retranslate",
        "",
        "on_comboBoxShadowFiltering_currentIndexChanged",
        "index",
        "on_comboBoxCubemapMode_currentIndexChanged",
        "on_sliderTorchStrength_valueChanged",
        "value",
        "on_sliderTorchRange_valueChanged",
        "on_checkBoxDefaultScene_stateChanged",
        "on_comboBoxCubemapSize_currentIndexChanged",
        "on_comboBoxShadowmapSize_currentIndexChanged",
        "scenery3dChanged",
        "QListWidgetItem*",
        "item",
        "updateTorchStrength",
        "val",
        "updateTorchRange",
        "updateLazyDrawingInterval",
        "updateShadowFilterQuality",
        "S3DEnum::ShadowFilterQuality",
        "quality",
        "updateSecondDominantFaceEnabled",
        "updateCurrentScene",
        "SceneInfo",
        "sceneInfo",
        "initResolutionCombobox",
        "QComboBox*",
        "cb",
        "setResolutionCombobox",
        "updateShortcutStrings",
        "updateToolTipStrings",
        "setCoordinateTextStyle"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'retranslate'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_comboBoxShadowFiltering_currentIndexChanged'
        QtMocHelpers::SlotData<void(int)>(3, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 4 },
        }}),
        // Slot 'on_comboBoxCubemapMode_currentIndexChanged'
        QtMocHelpers::SlotData<void(int)>(5, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 4 },
        }}),
        // Slot 'on_sliderTorchStrength_valueChanged'
        QtMocHelpers::SlotData<void(int)>(6, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 7 },
        }}),
        // Slot 'on_sliderTorchRange_valueChanged'
        QtMocHelpers::SlotData<void(int)>(8, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 7 },
        }}),
        // Slot 'on_checkBoxDefaultScene_stateChanged'
        QtMocHelpers::SlotData<void(int)>(9, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 7 },
        }}),
        // Slot 'on_comboBoxCubemapSize_currentIndexChanged'
        QtMocHelpers::SlotData<void(int)>(10, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 4 },
        }}),
        // Slot 'on_comboBoxShadowmapSize_currentIndexChanged'
        QtMocHelpers::SlotData<void(int)>(11, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 4 },
        }}),
        // Slot 'scenery3dChanged'
        QtMocHelpers::SlotData<void(QListWidgetItem *)>(12, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
        // Slot 'updateTorchStrength'
        QtMocHelpers::SlotData<void(float)>(15, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Float, 16 },
        }}),
        // Slot 'updateTorchRange'
        QtMocHelpers::SlotData<void(float)>(17, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Float, 16 },
        }}),
        // Slot 'updateLazyDrawingInterval'
        QtMocHelpers::SlotData<void(double)>(18, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 16 },
        }}),
        // Slot 'updateShadowFilterQuality'
        QtMocHelpers::SlotData<void(S3DEnum::ShadowFilterQuality)>(19, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 20, 21 },
        }}),
        // Slot 'updateSecondDominantFaceEnabled'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateCurrentScene'
        QtMocHelpers::SlotData<void(const SceneInfo &)>(23, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 24, 25 },
        }}),
        // Slot 'initResolutionCombobox'
        QtMocHelpers::SlotData<void(QComboBox *)>(26, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 27, 28 },
        }}),
        // Slot 'setResolutionCombobox'
        QtMocHelpers::SlotData<void(QComboBox *, uint)>(29, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 27, 28 }, { QMetaType::UInt, 16 },
        }}),
        // Slot 'updateShortcutStrings'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateToolTipStrings'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setCoordinateTextStyle'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Scenery3dDialog, qt_meta_tag_ZN15Scenery3dDialogE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Scenery3dDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<StelDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15Scenery3dDialogE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15Scenery3dDialogE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15Scenery3dDialogE_t>.metaTypes,
    nullptr
} };

void Scenery3dDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Scenery3dDialog *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->retranslate(); break;
        case 1: _t->on_comboBoxShadowFiltering_currentIndexChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->on_comboBoxCubemapMode_currentIndexChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 3: _t->on_sliderTorchStrength_valueChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->on_sliderTorchRange_valueChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 5: _t->on_checkBoxDefaultScene_stateChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 6: _t->on_comboBoxCubemapSize_currentIndexChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 7: _t->on_comboBoxShadowmapSize_currentIndexChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 8: _t->scenery3dChanged((*reinterpret_cast<std::add_pointer_t<QListWidgetItem*>>(_a[1]))); break;
        case 9: _t->updateTorchStrength((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 10: _t->updateTorchRange((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 11: _t->updateLazyDrawingInterval((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 12: _t->updateShadowFilterQuality((*reinterpret_cast<std::add_pointer_t<S3DEnum::ShadowFilterQuality>>(_a[1]))); break;
        case 13: _t->updateSecondDominantFaceEnabled(); break;
        case 14: _t->updateCurrentScene((*reinterpret_cast<std::add_pointer_t<SceneInfo>>(_a[1]))); break;
        case 15: _t->initResolutionCombobox((*reinterpret_cast<std::add_pointer_t<QComboBox*>>(_a[1]))); break;
        case 16: _t->setResolutionCombobox((*reinterpret_cast<std::add_pointer_t<QComboBox*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[2]))); break;
        case 17: _t->updateShortcutStrings(); break;
        case 18: _t->updateToolTipStrings(); break;
        case 19: _t->setCoordinateTextStyle(); break;
        default: ;
        }
    }
}

const QMetaObject *Scenery3dDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Scenery3dDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15Scenery3dDialogE_t>.strings))
        return static_cast<void*>(this);
    return StelDialog::qt_metacast(_clname);
}

int Scenery3dDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StelDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 20)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 20;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 20)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 20;
    }
    return _id;
}
QT_WARNING_POP
