/********************************************************************************
** Form generated from reading UI file 'solarSystemManagerWindow.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SOLARSYSTEMMANAGERWINDOW_H
#define UI_SOLARSYSTEMMANAGERWINDOW_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTextBrowser>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "Dialog.hpp"
#include "gui/StelCloseButton.hpp"

QT_BEGIN_NAMESPACE

class Ui_solarSystemManagerWindow
{
public:
    QVBoxLayout *verticalLayout;
    TitleBar *titleBar;
    QTabWidget *tabWidget;
    QWidget *tabConfigurationFile;
    QVBoxLayout *verticalLayoutTabConfigurationFile;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayoutWarning;
    QLabel *labelFileWarning;
    QLineEdit *lineEditUserFilePath;
    QLabel *labelResetWarning;
    QPushButton *pushButtonReset;
    QGroupBox *groupBoxSystemFile;
    QVBoxLayout *verticalLayoutImportExport;
    QLabel *label;
    QPushButton *pushButtonCopyFile;
    QPushButton *pushButtonReplaceFile;
    QPushButton *pushButtonAddFile;
    QWidget *tabSolarSystem;
    QVBoxLayout *verticalLayoutTabSolarSystem;
    QGroupBox *groupBoxObjectsList;
    QVBoxLayout *verticalLayout_2;
    QListWidget *listWidgetObjects;
    QPushButton *pushButtonRemove;
    QGroupBox *groupBoxMPC;
    QVBoxLayout *verticalLayoutMPC;
    QLabel *labelMPC;
    QPushButton *pushButtonImportMPC;
    QWidget *tabAbout;
    QVBoxLayout *verticalLayout_4;
    QTextBrowser *aboutTextBrowser;

    void setupUi(QWidget *solarSystemManagerWindow)
    {
        if (solarSystemManagerWindow->objectName().isEmpty())
            solarSystemManagerWindow->setObjectName("solarSystemManagerWindow");
        solarSystemManagerWindow->resize(394, 361);
        verticalLayout = new QVBoxLayout(solarSystemManagerWindow);
        verticalLayout->setSpacing(0);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        titleBar = new TitleBar(solarSystemManagerWindow);
        titleBar->setObjectName("titleBar");

        verticalLayout->addWidget(titleBar);

        tabWidget = new QTabWidget(solarSystemManagerWindow);
        tabWidget->setObjectName("tabWidget");
        tabConfigurationFile = new QWidget();
        tabConfigurationFile->setObjectName("tabConfigurationFile");
        verticalLayoutTabConfigurationFile = new QVBoxLayout(tabConfigurationFile);
        verticalLayoutTabConfigurationFile->setSpacing(0);
        verticalLayoutTabConfigurationFile->setObjectName("verticalLayoutTabConfigurationFile");
        verticalLayoutTabConfigurationFile->setContentsMargins(0, 0, 0, 0);
        groupBox = new QGroupBox(tabConfigurationFile);
        groupBox->setObjectName("groupBox");
        verticalLayoutWarning = new QVBoxLayout(groupBox);
        verticalLayoutWarning->setObjectName("verticalLayoutWarning");
        verticalLayoutWarning->setContentsMargins(0, 0, 0, 0);
        labelFileWarning = new QLabel(groupBox);
        labelFileWarning->setObjectName("labelFileWarning");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Minimum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(labelFileWarning->sizePolicy().hasHeightForWidth());
        labelFileWarning->setSizePolicy(sizePolicy);
        labelFileWarning->setWordWrap(true);

        verticalLayoutWarning->addWidget(labelFileWarning);

        lineEditUserFilePath = new QLineEdit(groupBox);
        lineEditUserFilePath->setObjectName("lineEditUserFilePath");
        lineEditUserFilePath->setReadOnly(true);

        verticalLayoutWarning->addWidget(lineEditUserFilePath);

        labelResetWarning = new QLabel(groupBox);
        labelResetWarning->setObjectName("labelResetWarning");
        labelResetWarning->setWordWrap(true);

        verticalLayoutWarning->addWidget(labelResetWarning);

        pushButtonReset = new QPushButton(groupBox);
        pushButtonReset->setObjectName("pushButtonReset");

        verticalLayoutWarning->addWidget(pushButtonReset);


        verticalLayoutTabConfigurationFile->addWidget(groupBox);

        groupBoxSystemFile = new QGroupBox(tabConfigurationFile);
        groupBoxSystemFile->setObjectName("groupBoxSystemFile");
        verticalLayoutImportExport = new QVBoxLayout(groupBoxSystemFile);
        verticalLayoutImportExport->setObjectName("verticalLayoutImportExport");
        verticalLayoutImportExport->setContentsMargins(0, 0, 0, 0);
        label = new QLabel(groupBoxSystemFile);
        label->setObjectName("label");
        label->setWordWrap(true);

        verticalLayoutImportExport->addWidget(label);

        pushButtonCopyFile = new QPushButton(groupBoxSystemFile);
        pushButtonCopyFile->setObjectName("pushButtonCopyFile");

        verticalLayoutImportExport->addWidget(pushButtonCopyFile);

        pushButtonReplaceFile = new QPushButton(groupBoxSystemFile);
        pushButtonReplaceFile->setObjectName("pushButtonReplaceFile");

        verticalLayoutImportExport->addWidget(pushButtonReplaceFile);

        pushButtonAddFile = new QPushButton(groupBoxSystemFile);
        pushButtonAddFile->setObjectName("pushButtonAddFile");

        verticalLayoutImportExport->addWidget(pushButtonAddFile);


        verticalLayoutTabConfigurationFile->addWidget(groupBoxSystemFile);

        tabWidget->addTab(tabConfigurationFile, QString());
        tabSolarSystem = new QWidget();
        tabSolarSystem->setObjectName("tabSolarSystem");
        verticalLayoutTabSolarSystem = new QVBoxLayout(tabSolarSystem);
        verticalLayoutTabSolarSystem->setSpacing(0);
        verticalLayoutTabSolarSystem->setObjectName("verticalLayoutTabSolarSystem");
        verticalLayoutTabSolarSystem->setContentsMargins(0, 0, 0, 0);
        groupBoxObjectsList = new QGroupBox(tabSolarSystem);
        groupBoxObjectsList->setObjectName("groupBoxObjectsList");
        verticalLayout_2 = new QVBoxLayout(groupBoxObjectsList);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        listWidgetObjects = new QListWidget(groupBoxObjectsList);
        listWidgetObjects->setObjectName("listWidgetObjects");
        listWidgetObjects->setSelectionMode(QAbstractItemView::ExtendedSelection);
        listWidgetObjects->setSortingEnabled(true);

        verticalLayout_2->addWidget(listWidgetObjects);

        pushButtonRemove = new QPushButton(groupBoxObjectsList);
        pushButtonRemove->setObjectName("pushButtonRemove");

        verticalLayout_2->addWidget(pushButtonRemove);


        verticalLayoutTabSolarSystem->addWidget(groupBoxObjectsList);

        groupBoxMPC = new QGroupBox(tabSolarSystem);
        groupBoxMPC->setObjectName("groupBoxMPC");
        verticalLayoutMPC = new QVBoxLayout(groupBoxMPC);
        verticalLayoutMPC->setObjectName("verticalLayoutMPC");
        verticalLayoutMPC->setContentsMargins(0, 0, 0, 0);
        labelMPC = new QLabel(groupBoxMPC);
        labelMPC->setObjectName("labelMPC");
        labelMPC->setText(QString::fromUtf8("You can import comet and asteroid data formatted in the export formats of the IAU's <a href=\"https://www.minorplanetcenter.net/\">Minor Planet Center</a> (MPC). You can import files with lists of objects, download such lists from the Internet or search the online Minor Planet and Comet Ephemeris Service (MPES)."));
        labelMPC->setTextFormat(Qt::AutoText);
        labelMPC->setWordWrap(true);
        labelMPC->setOpenExternalLinks(true);

        verticalLayoutMPC->addWidget(labelMPC);

        pushButtonImportMPC = new QPushButton(groupBoxMPC);
        pushButtonImportMPC->setObjectName("pushButtonImportMPC");

        verticalLayoutMPC->addWidget(pushButtonImportMPC);


        verticalLayoutTabSolarSystem->addWidget(groupBoxMPC);

        tabWidget->addTab(tabSolarSystem, QString());
        tabAbout = new QWidget();
        tabAbout->setObjectName("tabAbout");
        verticalLayout_4 = new QVBoxLayout(tabAbout);
        verticalLayout_4->setObjectName("verticalLayout_4");
        aboutTextBrowser = new QTextBrowser(tabAbout);
        aboutTextBrowser->setObjectName("aboutTextBrowser");
        aboutTextBrowser->setOpenExternalLinks(true);

        verticalLayout_4->addWidget(aboutTextBrowser);

        tabWidget->addTab(tabAbout, QString());

        verticalLayout->addWidget(tabWidget);


        retranslateUi(solarSystemManagerWindow);

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(solarSystemManagerWindow);
    } // setupUi

    void retranslateUi(QWidget *solarSystemManagerWindow)
    {
        titleBar->setProperty("title", QVariant(QCoreApplication::translate("solarSystemManagerWindow", "Minor Solar System objects", nullptr)));
        groupBox->setTitle(QCoreApplication::translate("solarSystemManagerWindow", "Note", nullptr));
        labelFileWarning->setText(QCoreApplication::translate("solarSystemManagerWindow", "This plug-in edits a custom Solar System configuration file for the minor bodies. If something goes wrong and Stellarium crashes and/or doesn't start, you can manually delete that file from:", nullptr));
        labelResetWarning->setText(QCoreApplication::translate("solarSystemManagerWindow", "You can also reset all changes and return to the default configuration:", nullptr));
        pushButtonReset->setText(QCoreApplication::translate("solarSystemManagerWindow", "Restore defaults", nullptr));
        groupBoxSystemFile->setTitle(QCoreApplication::translate("solarSystemManagerWindow", "Export/replace the Solar System minor objects file", nullptr));
        label->setText(QCoreApplication::translate("solarSystemManagerWindow", "You can create a backup copy of your custom Solar System configuration file in a convenient location, or replace it with such a copy.", nullptr));
        pushButtonCopyFile->setText(QCoreApplication::translate("solarSystemManagerWindow", "Export the Solar System minor objects to file...", nullptr));
        pushButtonReplaceFile->setText(QCoreApplication::translate("solarSystemManagerWindow", "Import and replace the Solar System minor objects from file...", nullptr));
        pushButtonAddFile->setText(QCoreApplication::translate("solarSystemManagerWindow", "Import and add Solar System minor objects from file...", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabConfigurationFile), QCoreApplication::translate("solarSystemManagerWindow", "Configuration file", nullptr));
        groupBoxObjectsList->setTitle(QCoreApplication::translate("solarSystemManagerWindow", "Minor Solar System objects", nullptr));
        pushButtonRemove->setText(QCoreApplication::translate("solarSystemManagerWindow", "Remove", nullptr));
        groupBoxMPC->setTitle(QCoreApplication::translate("solarSystemManagerWindow", "Add new Minor Solar System objects", nullptr));
        pushButtonImportMPC->setText(QCoreApplication::translate("solarSystemManagerWindow", "Import orbital elements in MPC format...", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabSolarSystem), QCoreApplication::translate("solarSystemManagerWindow", "Solar System", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabAbout), QCoreApplication::translate("solarSystemManagerWindow", "About", "tab in plugin windows"));
        (void)solarSystemManagerWindow;
    } // retranslateUi

};

namespace Ui {
    class solarSystemManagerWindow: public Ui_solarSystemManagerWindow {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SOLARSYSTEMMANAGERWINDOW_H
