/****************************************************************************
** Meta object code from reading C++ file 'StelDialog_p.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../stellarium/src/gui/StelDialog_p.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'StelDialog_p.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN43QAbstractButtonStelPropertyConnectionHelperE_t {};
} // unnamed namespace

template <> constexpr inline auto QAbstractButtonStelPropertyConnectionHelper::qt_create_metaobjectdata<qt_meta_tag_ZN43QAbstractButtonStelPropertyConnectionHelperE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QAbstractButtonStelPropertyConnectionHelper",
        "onPropertyChanged",
        "",
        "QVariant",
        "value"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'onPropertyChanged'
        QtMocHelpers::SlotData<void(const QVariant &)>(1, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QAbstractButtonStelPropertyConnectionHelper, qt_meta_tag_ZN43QAbstractButtonStelPropertyConnectionHelperE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QAbstractButtonStelPropertyConnectionHelper::staticMetaObject = { {
    QMetaObject::SuperData::link<StelPropertyProxy::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN43QAbstractButtonStelPropertyConnectionHelperE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN43QAbstractButtonStelPropertyConnectionHelperE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN43QAbstractButtonStelPropertyConnectionHelperE_t>.metaTypes,
    nullptr
} };

void QAbstractButtonStelPropertyConnectionHelper::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QAbstractButtonStelPropertyConnectionHelper *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->onPropertyChanged((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *QAbstractButtonStelPropertyConnectionHelper::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QAbstractButtonStelPropertyConnectionHelper::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN43QAbstractButtonStelPropertyConnectionHelperE_t>.strings))
        return static_cast<void*>(this);
    return StelPropertyProxy::qt_metacast(_clname);
}

int QAbstractButtonStelPropertyConnectionHelper::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StelPropertyProxy::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN37QGroupBoxStelPropertyConnectionHelperE_t {};
} // unnamed namespace

template <> constexpr inline auto QGroupBoxStelPropertyConnectionHelper::qt_create_metaobjectdata<qt_meta_tag_ZN37QGroupBoxStelPropertyConnectionHelperE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QGroupBoxStelPropertyConnectionHelper",
        "onPropertyChanged",
        "",
        "QVariant",
        "value"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'onPropertyChanged'
        QtMocHelpers::SlotData<void(const QVariant &)>(1, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QGroupBoxStelPropertyConnectionHelper, qt_meta_tag_ZN37QGroupBoxStelPropertyConnectionHelperE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QGroupBoxStelPropertyConnectionHelper::staticMetaObject = { {
    QMetaObject::SuperData::link<StelPropertyProxy::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN37QGroupBoxStelPropertyConnectionHelperE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN37QGroupBoxStelPropertyConnectionHelperE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN37QGroupBoxStelPropertyConnectionHelperE_t>.metaTypes,
    nullptr
} };

void QGroupBoxStelPropertyConnectionHelper::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QGroupBoxStelPropertyConnectionHelper *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->onPropertyChanged((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *QGroupBoxStelPropertyConnectionHelper::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QGroupBoxStelPropertyConnectionHelper::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN37QGroupBoxStelPropertyConnectionHelperE_t>.strings))
        return static_cast<void*>(this);
    return StelPropertyProxy::qt_metacast(_clname);
}

int QGroupBoxStelPropertyConnectionHelper::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StelPropertyProxy::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN37QComboBoxStelPropertyConnectionHelperE_t {};
} // unnamed namespace

template <> constexpr inline auto QComboBoxStelPropertyConnectionHelper::qt_create_metaobjectdata<qt_meta_tag_ZN37QComboBoxStelPropertyConnectionHelperE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QComboBoxStelPropertyConnectionHelper",
        "onPropertyChanged",
        "",
        "QVariant",
        "value"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'onPropertyChanged'
        QtMocHelpers::SlotData<void(const QVariant &)>(1, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QComboBoxStelPropertyConnectionHelper, qt_meta_tag_ZN37QComboBoxStelPropertyConnectionHelperE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QComboBoxStelPropertyConnectionHelper::staticMetaObject = { {
    QMetaObject::SuperData::link<StelPropertyProxy::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN37QComboBoxStelPropertyConnectionHelperE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN37QComboBoxStelPropertyConnectionHelperE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN37QComboBoxStelPropertyConnectionHelperE_t>.metaTypes,
    nullptr
} };

void QComboBoxStelPropertyConnectionHelper::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QComboBoxStelPropertyConnectionHelper *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->onPropertyChanged((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *QComboBoxStelPropertyConnectionHelper::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QComboBoxStelPropertyConnectionHelper::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN37QComboBoxStelPropertyConnectionHelperE_t>.strings))
        return static_cast<void*>(this);
    return StelPropertyProxy::qt_metacast(_clname);
}

int QComboBoxStelPropertyConnectionHelper::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StelPropertyProxy::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN43QComboBoxStelStringPropertyConnectionHelperE_t {};
} // unnamed namespace

template <> constexpr inline auto QComboBoxStelStringPropertyConnectionHelper::qt_create_metaobjectdata<qt_meta_tag_ZN43QComboBoxStelStringPropertyConnectionHelperE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QComboBoxStelStringPropertyConnectionHelper",
        "onPropertyChanged",
        "",
        "QVariant",
        "value"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'onPropertyChanged'
        QtMocHelpers::SlotData<void(const QVariant &)>(1, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QComboBoxStelStringPropertyConnectionHelper, qt_meta_tag_ZN43QComboBoxStelStringPropertyConnectionHelperE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QComboBoxStelStringPropertyConnectionHelper::staticMetaObject = { {
    QMetaObject::SuperData::link<StelPropertyProxy::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN43QComboBoxStelStringPropertyConnectionHelperE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN43QComboBoxStelStringPropertyConnectionHelperE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN43QComboBoxStelStringPropertyConnectionHelperE_t>.metaTypes,
    nullptr
} };

void QComboBoxStelStringPropertyConnectionHelper::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QComboBoxStelStringPropertyConnectionHelper *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->onPropertyChanged((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *QComboBoxStelStringPropertyConnectionHelper::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QComboBoxStelStringPropertyConnectionHelper::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN43QComboBoxStelStringPropertyConnectionHelperE_t>.strings))
        return static_cast<void*>(this);
    return StelPropertyProxy::qt_metacast(_clname);
}

int QComboBoxStelStringPropertyConnectionHelper::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StelPropertyProxy::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN37QLineEditStelPropertyConnectionHelperE_t {};
} // unnamed namespace

template <> constexpr inline auto QLineEditStelPropertyConnectionHelper::qt_create_metaobjectdata<qt_meta_tag_ZN37QLineEditStelPropertyConnectionHelperE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QLineEditStelPropertyConnectionHelper",
        "onPropertyChanged",
        "",
        "QVariant",
        "value"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'onPropertyChanged'
        QtMocHelpers::SlotData<void(const QVariant &)>(1, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QLineEditStelPropertyConnectionHelper, qt_meta_tag_ZN37QLineEditStelPropertyConnectionHelperE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QLineEditStelPropertyConnectionHelper::staticMetaObject = { {
    QMetaObject::SuperData::link<StelPropertyProxy::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN37QLineEditStelPropertyConnectionHelperE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN37QLineEditStelPropertyConnectionHelperE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN37QLineEditStelPropertyConnectionHelperE_t>.metaTypes,
    nullptr
} };

void QLineEditStelPropertyConnectionHelper::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QLineEditStelPropertyConnectionHelper *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->onPropertyChanged((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *QLineEditStelPropertyConnectionHelper::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QLineEditStelPropertyConnectionHelper::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN37QLineEditStelPropertyConnectionHelperE_t>.strings))
        return static_cast<void*>(this);
    return StelPropertyProxy::qt_metacast(_clname);
}

int QLineEditStelPropertyConnectionHelper::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StelPropertyProxy::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN36QSpinBoxStelPropertyConnectionHelperE_t {};
} // unnamed namespace

template <> constexpr inline auto QSpinBoxStelPropertyConnectionHelper::qt_create_metaobjectdata<qt_meta_tag_ZN36QSpinBoxStelPropertyConnectionHelperE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QSpinBoxStelPropertyConnectionHelper",
        "onPropertyChanged",
        "",
        "QVariant",
        "value"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'onPropertyChanged'
        QtMocHelpers::SlotData<void(const QVariant &)>(1, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QSpinBoxStelPropertyConnectionHelper, qt_meta_tag_ZN36QSpinBoxStelPropertyConnectionHelperE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QSpinBoxStelPropertyConnectionHelper::staticMetaObject = { {
    QMetaObject::SuperData::link<StelPropertyProxy::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN36QSpinBoxStelPropertyConnectionHelperE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN36QSpinBoxStelPropertyConnectionHelperE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN36QSpinBoxStelPropertyConnectionHelperE_t>.metaTypes,
    nullptr
} };

void QSpinBoxStelPropertyConnectionHelper::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QSpinBoxStelPropertyConnectionHelper *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->onPropertyChanged((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *QSpinBoxStelPropertyConnectionHelper::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QSpinBoxStelPropertyConnectionHelper::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN36QSpinBoxStelPropertyConnectionHelperE_t>.strings))
        return static_cast<void*>(this);
    return StelPropertyProxy::qt_metacast(_clname);
}

int QSpinBoxStelPropertyConnectionHelper::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StelPropertyProxy::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN42QDoubleSpinBoxStelPropertyConnectionHelperE_t {};
} // unnamed namespace

template <> constexpr inline auto QDoubleSpinBoxStelPropertyConnectionHelper::qt_create_metaobjectdata<qt_meta_tag_ZN42QDoubleSpinBoxStelPropertyConnectionHelperE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QDoubleSpinBoxStelPropertyConnectionHelper",
        "onPropertyChanged",
        "",
        "QVariant",
        "value"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'onPropertyChanged'
        QtMocHelpers::SlotData<void(const QVariant &)>(1, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QDoubleSpinBoxStelPropertyConnectionHelper, qt_meta_tag_ZN42QDoubleSpinBoxStelPropertyConnectionHelperE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QDoubleSpinBoxStelPropertyConnectionHelper::staticMetaObject = { {
    QMetaObject::SuperData::link<StelPropertyProxy::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN42QDoubleSpinBoxStelPropertyConnectionHelperE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN42QDoubleSpinBoxStelPropertyConnectionHelperE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN42QDoubleSpinBoxStelPropertyConnectionHelperE_t>.metaTypes,
    nullptr
} };

void QDoubleSpinBoxStelPropertyConnectionHelper::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QDoubleSpinBoxStelPropertyConnectionHelper *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->onPropertyChanged((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *QDoubleSpinBoxStelPropertyConnectionHelper::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QDoubleSpinBoxStelPropertyConnectionHelper::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN42QDoubleSpinBoxStelPropertyConnectionHelperE_t>.strings))
        return static_cast<void*>(this);
    return StelPropertyProxy::qt_metacast(_clname);
}

int QDoubleSpinBoxStelPropertyConnectionHelper::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StelPropertyProxy::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN40AngleSpinBoxStelPropertyConnectionHelperE_t {};
} // unnamed namespace

template <> constexpr inline auto AngleSpinBoxStelPropertyConnectionHelper::qt_create_metaobjectdata<qt_meta_tag_ZN40AngleSpinBoxStelPropertyConnectionHelperE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "AngleSpinBoxStelPropertyConnectionHelper",
        "onPropertyChanged",
        "",
        "QVariant",
        "value"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'onPropertyChanged'
        QtMocHelpers::SlotData<void(const QVariant &)>(1, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<AngleSpinBoxStelPropertyConnectionHelper, qt_meta_tag_ZN40AngleSpinBoxStelPropertyConnectionHelperE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject AngleSpinBoxStelPropertyConnectionHelper::staticMetaObject = { {
    QMetaObject::SuperData::link<StelPropertyProxy::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN40AngleSpinBoxStelPropertyConnectionHelperE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN40AngleSpinBoxStelPropertyConnectionHelperE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN40AngleSpinBoxStelPropertyConnectionHelperE_t>.metaTypes,
    nullptr
} };

void AngleSpinBoxStelPropertyConnectionHelper::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AngleSpinBoxStelPropertyConnectionHelper *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->onPropertyChanged((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *AngleSpinBoxStelPropertyConnectionHelper::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *AngleSpinBoxStelPropertyConnectionHelper::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN40AngleSpinBoxStelPropertyConnectionHelperE_t>.strings))
        return static_cast<void*>(this);
    return StelPropertyProxy::qt_metacast(_clname);
}

int AngleSpinBoxStelPropertyConnectionHelper::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StelPropertyProxy::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN35QSliderStelPropertyConnectionHelperE_t {};
} // unnamed namespace

template <> constexpr inline auto QSliderStelPropertyConnectionHelper::qt_create_metaobjectdata<qt_meta_tag_ZN35QSliderStelPropertyConnectionHelperE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QSliderStelPropertyConnectionHelper",
        "onPropertyChanged",
        "",
        "QVariant",
        "value",
        "sliderIntValueChanged",
        "val"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'onPropertyChanged'
        QtMocHelpers::SlotData<void(const QVariant &)>(1, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'sliderIntValueChanged'
        QtMocHelpers::SlotData<void(int)>(5, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 6 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QSliderStelPropertyConnectionHelper, qt_meta_tag_ZN35QSliderStelPropertyConnectionHelperE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QSliderStelPropertyConnectionHelper::staticMetaObject = { {
    QMetaObject::SuperData::link<StelPropertyProxy::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN35QSliderStelPropertyConnectionHelperE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN35QSliderStelPropertyConnectionHelperE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN35QSliderStelPropertyConnectionHelperE_t>.metaTypes,
    nullptr
} };

void QSliderStelPropertyConnectionHelper::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QSliderStelPropertyConnectionHelper *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->onPropertyChanged((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1]))); break;
        case 1: _t->sliderIntValueChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *QSliderStelPropertyConnectionHelper::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QSliderStelPropertyConnectionHelper::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN35QSliderStelPropertyConnectionHelperE_t>.strings))
        return static_cast<void*>(this);
    return StelPropertyProxy::qt_metacast(_clname);
}

int QSliderStelPropertyConnectionHelper::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StelPropertyProxy::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}
QT_WARNING_POP
