/****************************************************************************
** Meta object code from reading C++ file 'MosaicCamera.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../stellarium/plugins/MosaicCamera/src/MosaicCamera.hpp"
#include <QtCore/qmetatype.h>
#include <QtCore/qplugin.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'MosaicCamera.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12MosaicCameraE_t {};
} // unnamed namespace

template <> constexpr inline auto MosaicCamera::qt_create_metaobjectdata<qt_meta_tag_ZN12MosaicCameraE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MosaicCamera",
        "flagMosaicCameraVisibilityChanged",
        "",
        "b",
        "flagShowButtonChanged",
        "currentCameraChanged",
        "cameraName",
        "currentRAChanged",
        "ra",
        "currentDecChanged",
        "dec",
        "currentRotationChanged",
        "rotation",
        "currentVisibilityChanged",
        "visible",
        "setRA",
        "setDec",
        "setRotation",
        "setVisibility",
        "setPosition",
        "setRADecToView",
        "setRADecToObject",
        "setViewToCamera",
        "incrementRotation",
        "decrementRotation",
        "nextCamera",
        "previousCamera",
        "enableMosaicCamera",
        "setFlagShowButton",
        "setCurrentCamera",
        "setCurrentRA",
        "setCurrentDec",
        "setCurrentRotation",
        "setCurrentVisibility",
        "saveSettings",
        "restoreDefaults",
        "getRA",
        "getDec",
        "getRotation",
        "getVisibility",
        "enabled",
        "showButton",
        "currentCamera"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'flagMosaicCameraVisibilityChanged'
        QtMocHelpers::SignalData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagShowButtonChanged'
        QtMocHelpers::SignalData<void(bool)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'currentCameraChanged'
        QtMocHelpers::SignalData<void(const QString &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 },
        }}),
        // Signal 'currentRAChanged'
        QtMocHelpers::SignalData<void(double)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 8 },
        }}),
        // Signal 'currentDecChanged'
        QtMocHelpers::SignalData<void(double)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 10 },
        }}),
        // Signal 'currentRotationChanged'
        QtMocHelpers::SignalData<void(double)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 12 },
        }}),
        // Signal 'currentVisibilityChanged'
        QtMocHelpers::SignalData<void(bool)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 14 },
        }}),
        // Slot 'setRA'
        QtMocHelpers::SlotData<void(const QString &, double)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 }, { QMetaType::Double, 8 },
        }}),
        // Slot 'setDec'
        QtMocHelpers::SlotData<void(const QString &, double)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 }, { QMetaType::Double, 10 },
        }}),
        // Slot 'setRotation'
        QtMocHelpers::SlotData<void(const QString &, double)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 }, { QMetaType::Double, 12 },
        }}),
        // Slot 'setVisibility'
        QtMocHelpers::SlotData<void(const QString &, bool)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 }, { QMetaType::Bool, 14 },
        }}),
        // Slot 'setPosition'
        QtMocHelpers::SlotData<void(const QString &, double, double, double)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 }, { QMetaType::Double, 8 }, { QMetaType::Double, 10 }, { QMetaType::Double, 12 },
        }}),
        // Slot 'setRADecToView'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setRADecToObject'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setViewToCamera'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'incrementRotation'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'decrementRotation'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'nextCamera'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'previousCamera'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'enableMosaicCamera'
        QtMocHelpers::SlotData<void(bool)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'setFlagShowButton'
        QtMocHelpers::SlotData<void(bool)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'setCurrentCamera'
        QtMocHelpers::SlotData<void(const QString &)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 },
        }}),
        // Slot 'setCurrentRA'
        QtMocHelpers::SlotData<void(double)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 8 },
        }}),
        // Slot 'setCurrentDec'
        QtMocHelpers::SlotData<void(double)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 10 },
        }}),
        // Slot 'setCurrentRotation'
        QtMocHelpers::SlotData<void(double)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 12 },
        }}),
        // Slot 'setCurrentVisibility'
        QtMocHelpers::SlotData<void(bool)>(33, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 14 },
        }}),
        // Slot 'saveSettings'
        QtMocHelpers::SlotData<void() const>(34, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'restoreDefaults'
        QtMocHelpers::SlotData<void()>(35, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'getRA'
        QtMocHelpers::MethodData<double(const QString &) const>(36, 2, QMC::AccessPublic, QMetaType::Double, {{
            { QMetaType::QString, 6 },
        }}),
        // Method 'getDec'
        QtMocHelpers::MethodData<double(const QString &) const>(37, 2, QMC::AccessPublic, QMetaType::Double, {{
            { QMetaType::QString, 6 },
        }}),
        // Method 'getRotation'
        QtMocHelpers::MethodData<double(const QString &) const>(38, 2, QMC::AccessPublic, QMetaType::Double, {{
            { QMetaType::QString, 6 },
        }}),
        // Method 'getVisibility'
        QtMocHelpers::MethodData<bool(const QString &) const>(39, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 6 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'enabled'
        QtMocHelpers::PropertyData<bool>(40, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 0),
        // property 'showButton'
        QtMocHelpers::PropertyData<bool>(41, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 1),
        // property 'currentCamera'
        QtMocHelpers::PropertyData<QString>(42, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'ra'
        QtMocHelpers::PropertyData<double>(8, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable, 3),
        // property 'dec'
        QtMocHelpers::PropertyData<double>(10, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable, 4),
        // property 'rotation'
        QtMocHelpers::PropertyData<double>(12, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable, 5),
        // property 'visible'
        QtMocHelpers::PropertyData<bool>(14, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 6),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<MosaicCamera, qt_meta_tag_ZN12MosaicCameraE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MosaicCamera::staticMetaObject = { {
    QMetaObject::SuperData::link<StelModule::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12MosaicCameraE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12MosaicCameraE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12MosaicCameraE_t>.metaTypes,
    nullptr
} };

void MosaicCamera::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MosaicCamera *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->flagMosaicCameraVisibilityChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->flagShowButtonChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->currentCameraChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->currentRAChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 4: _t->currentDecChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 5: _t->currentRotationChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 6: _t->currentVisibilityChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->setRA((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 8: _t->setDec((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 9: _t->setRotation((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 10: _t->setVisibility((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 11: _t->setPosition((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[4]))); break;
        case 12: _t->setRADecToView(); break;
        case 13: _t->setRADecToObject(); break;
        case 14: _t->setViewToCamera(); break;
        case 15: _t->incrementRotation(); break;
        case 16: _t->decrementRotation(); break;
        case 17: _t->nextCamera(); break;
        case 18: _t->previousCamera(); break;
        case 19: _t->enableMosaicCamera((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 20: _t->setFlagShowButton((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 21: _t->setCurrentCamera((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 22: _t->setCurrentRA((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 23: _t->setCurrentDec((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 24: _t->setCurrentRotation((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 25: _t->setCurrentVisibility((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 26: _t->saveSettings(); break;
        case 27: _t->restoreDefaults(); break;
        case 28: { double _r = _t->getRA((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 29: { double _r = _t->getDec((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 30: { double _r = _t->getRotation((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 31: { bool _r = _t->getVisibility((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (MosaicCamera::*)(bool )>(_a, &MosaicCamera::flagMosaicCameraVisibilityChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (MosaicCamera::*)(bool )>(_a, &MosaicCamera::flagShowButtonChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (MosaicCamera::*)(const QString & )>(_a, &MosaicCamera::currentCameraChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (MosaicCamera::*)(double )>(_a, &MosaicCamera::currentRAChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (MosaicCamera::*)(double )>(_a, &MosaicCamera::currentDecChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (MosaicCamera::*)(double )>(_a, &MosaicCamera::currentRotationChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (MosaicCamera::*)(bool )>(_a, &MosaicCamera::currentVisibilityChanged, 6))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->isEnabled(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->getFlagShowButton(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->getCurrentCamera(); break;
        case 3: *reinterpret_cast<double*>(_v) = _t->getCurrentRA(); break;
        case 4: *reinterpret_cast<double*>(_v) = _t->getCurrentDec(); break;
        case 5: *reinterpret_cast<double*>(_v) = _t->getCurrentRotation(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->getCurrentVisibility(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->enableMosaicCamera(*reinterpret_cast<bool*>(_v)); break;
        case 1: _t->setFlagShowButton(*reinterpret_cast<bool*>(_v)); break;
        case 2: _t->setCurrentCamera(*reinterpret_cast<QString*>(_v)); break;
        case 3: _t->setCurrentRA(*reinterpret_cast<double*>(_v)); break;
        case 4: _t->setCurrentDec(*reinterpret_cast<double*>(_v)); break;
        case 5: _t->setCurrentRotation(*reinterpret_cast<double*>(_v)); break;
        case 6: _t->setCurrentVisibility(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *MosaicCamera::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MosaicCamera::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12MosaicCameraE_t>.strings))
        return static_cast<void*>(this);
    return StelModule::qt_metacast(_clname);
}

int MosaicCamera::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StelModule::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 32)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 32;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 32)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 32;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void MosaicCamera::flagMosaicCameraVisibilityChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void MosaicCamera::flagShowButtonChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void MosaicCamera::currentCameraChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void MosaicCamera::currentRAChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void MosaicCamera::currentDecChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void MosaicCamera::currentRotationChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void MosaicCamera::currentVisibilityChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN31MosaicCameraStelPluginInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto MosaicCameraStelPluginInterface::qt_create_metaobjectdata<qt_meta_tag_ZN31MosaicCameraStelPluginInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MosaicCameraStelPluginInterface"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<MosaicCameraStelPluginInterface, qt_meta_tag_ZN31MosaicCameraStelPluginInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MosaicCameraStelPluginInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN31MosaicCameraStelPluginInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN31MosaicCameraStelPluginInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN31MosaicCameraStelPluginInterfaceE_t>.metaTypes,
    nullptr
} };

void MosaicCameraStelPluginInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MosaicCameraStelPluginInterface *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *MosaicCameraStelPluginInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MosaicCameraStelPluginInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN31MosaicCameraStelPluginInterfaceE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "StelPluginInterface"))
        return static_cast< StelPluginInterface*>(this);
    if (!strcmp(_clname, "org.stellarium.StelPluginInterface/2.0"))
        return static_cast< StelPluginInterface*>(this);
    return QObject::qt_metacast(_clname);
}

int MosaicCameraStelPluginInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}

#ifdef QT_MOC_EXPORT_PLUGIN_V2
static constexpr unsigned char qt_pluginMetaDataV2_MosaicCameraStelPluginInterface[] = {
    0xbf, 
    // "IID"
    0x02,  0x78,  0x26,  'o',  'r',  'g',  '.',  's', 
    't',  'e',  'l',  'l',  'a',  'r',  'i',  'u', 
    'm',  '.',  'S',  't',  'e',  'l',  'P',  'l', 
    'u',  'g',  'i',  'n',  'I',  'n',  't',  'e', 
    'r',  'f',  'a',  'c',  'e',  '/',  '2',  '.', 
    '0', 
    // "className"
    0x03,  0x78,  0x1f,  'M',  'o',  's',  'a',  'i', 
    'c',  'C',  'a',  'm',  'e',  'r',  'a',  'S', 
    't',  'e',  'l',  'P',  'l',  'u',  'g',  'i', 
    'n',  'I',  'n',  't',  'e',  'r',  'f',  'a', 
    'c',  'e', 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN_V2(MosaicCameraStelPluginInterface, MosaicCameraStelPluginInterface, qt_pluginMetaDataV2_MosaicCameraStelPluginInterface)
#else
QT_PLUGIN_METADATA_SECTION
Q_CONSTINIT static constexpr unsigned char qt_pluginMetaData_MosaicCameraStelPluginInterface[] = {
    'Q', 'T', 'M', 'E', 'T', 'A', 'D', 'A', 'T', 'A', ' ', '!',
    // metadata version, Qt version, architectural requirements
    0, QT_VERSION_MAJOR, QT_VERSION_MINOR, qPluginArchRequirements(),
    0xbf, 
    // "IID"
    0x02,  0x78,  0x26,  'o',  'r',  'g',  '.',  's', 
    't',  'e',  'l',  'l',  'a',  'r',  'i',  'u', 
    'm',  '.',  'S',  't',  'e',  'l',  'P',  'l', 
    'u',  'g',  'i',  'n',  'I',  'n',  't',  'e', 
    'r',  'f',  'a',  'c',  'e',  '/',  '2',  '.', 
    '0', 
    // "className"
    0x03,  0x78,  0x1f,  'M',  'o',  's',  'a',  'i', 
    'c',  'C',  'a',  'm',  'e',  'r',  'a',  'S', 
    't',  'e',  'l',  'P',  'l',  'u',  'g',  'i', 
    'n',  'I',  'n',  't',  'e',  'r',  'f',  'a', 
    'c',  'e', 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN(MosaicCameraStelPluginInterface, MosaicCameraStelPluginInterface)
#endif  // QT_MOC_EXPORT_PLUGIN_V2

QT_WARNING_POP
