/********************************************************************************
** Form generated from reading UI file 'astroCalcAlmanacWidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_ASTROCALCALMANACWIDGET_H
#define UI_ASTROCALCALMANACWIDGET_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_astroCalcAlmanacWidget
{
public:
    QGridLayout *gridLayout;
    QTabWidget *tabWidget;
    QWidget *tabSpecificTime;
    QGridLayout *gridLayoutSpecificTime;
    QLabel *labelLT2;
    QLabel *labelCustomMoonrise;
    QLabel *labelAstronomicalTwilightEnd;
    QPushButton *buttonNauticalDawn;
    QLabel *labelCivilTwilight;
    QLabel *labelJuneSolsticeDuration;
    QLabel *labelNauticalTwilightEnd;
    QPushButton *buttonMoonrise;
    QLabel *labelDecemberSolsticeLT;
    QLabel *labelCivilTwilightEnd;
    QLabel *labelMoonRise;
    QPushButton *buttonCustomSunset;
    QPushButton *buttonCustomMoonrise;
    QLabel *labelCustomMoonset;
    QLabel *labelAstronomicalTwilightDuration;
    QFrame *lineDelimiterLeftBlock;
    QLabel *labelSunrise;
    QLabel *labelDecemberSolsticeDuration;
    QLabel *labelSun;
    QLabel *labelMarchEquinoxLT;
    QLabel *labelNauticalTwilightDuration;
    QLabel *labelRise;
    QPushButton *buttonSunset;
    QLabel *labelMoonSet;
    QFrame *lineDelimiterCustomData;
    QPushButton *buttonMoonset;
    QPushButton *buttonCivilDusk;
    QPushButton *buttonSunrise;
    QLabel *labelMoon;
    QPushButton *buttonCustomSunrise;
    QHBoxLayout *DecemberSolsticeLayout;
    QLabel *labelDecemberSolstice;
    QPushButton *buttonDecemberSolsticePrevious;
    QPushButton *buttonDecemberSolsticeCurrent;
    QPushButton *buttonDecemberSolsticeNext;
    QLabel *labelDurationColumn1;
    QLabel *labelAfterSunset;
    QLabel *labelCustomSunset;
    QLabel *labelDawn;
    QLabel *labelDuration1;
    QLabel *labelNightDuration;
    QHBoxLayout *YearLayout;
    QLabel *labelYear;
    QLabel *labelCurrentYear;
    QLabel *labelYearDuration;
    QFrame *lineDelimiterSeasonsTop;
    QPushButton *buttonAstronomicalDawn;
    QLabel *labelCustomSun;
    QDoubleSpinBox *spinBoxCustomSunAltitude;
    QPushButton *buttonCustomMoonset;
    QLabel *labelSeptemberEquinoxDuration;
    QFrame *lineDelimiterSections;
    QLabel *labelNauticalTwilightBegin;
    QSpinBox *spinBoxMinutes;
    QDoubleSpinBox *spinBoxCustomMoonAltitude;
    QLabel *labelNauticalTwilight;
    QLabel *labelDuration2;
    QLabel *labelMarchEquinoxDuration;
    QLabel *labelJuneSolsticeJD;
    QPushButton *buttonAstronomicalDusk;
    QLabel *labelTwilight;
    QLabel *labelJD1;
    QLabel *labelJuneSolsticeLT;
    QLabel *labelSeptemberEquinoxJD;
    QHBoxLayout *JuneSolsticeLayout;
    QLabel *labelJuneSolstice;
    QPushButton *buttonJuneSolsticePrevious;
    QPushButton *buttonJuneSolsticeCurrent;
    QPushButton *buttonJuneSolsticeNext;
    QLabel *labelDayDuration;
    QFrame *lineDelimiterToday;
    QLabel *labelNight;
    QHBoxLayout *SeptemberEquinoxLayout;
    QLabel *labelSeptemberEquinox;
    QPushButton *buttonSeptemberEquinoxPrevious;
    QPushButton *buttonSeptemberEquinoxCurrent;
    QPushButton *buttonSeptemberEquinoxNext;
    QLabel *labelCivilTwilightBegin;
    QLabel *labelSunset;
    QPushButton *buttonAfterSunset;
    QLabel *labelSeptemberEquinoxLT;
    QHBoxLayout *MarchEquinoxLayout;
    QLabel *labelMarchEquinox;
    QPushButton *buttonMarchEquinoxPrevious;
    QPushButton *buttonMarchEquinoxCurrent;
    QPushButton *buttonMarchEquinoxNext;
    QLabel *labelCustomSunrise;
    QLabel *labelCustomMoon;
    QFrame *lineDelimiterSeasonsBottom;
    QLabel *labelCivilTwilightDuration;
    QLabel *labelSet;
    QLabel *labelDurationColumn2;
    QLabel *labelSeasons;
    QFrame *lineDelimiterSeasonsMiddle;
    QPushButton *buttonNauticalDusk;
    QPushButton *buttonBeforeSunrise;
    QLabel *labelBeforeSunrise;
    QHBoxLayout *horizontalLayout;
    QPushButton *buttonYesterday;
    QLabel *labelToday;
    QPushButton *buttonTomorrow;
    QLabel *labelAstronomicalTwilight;
    QLabel *labelAstronomicalTwilightBegin;
    QLabel *labelLT1;
    QLabel *labelJD2;
    QLabel *labelDecemberSolsticeJD;
    QLabel *labelDusk;
    QLabel *labelMarchEquinoxJD;
    QPushButton *buttonCivilDawn;
    QSpacerItem *verticalSpacer_2;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *astroCalcAlmanacWidget)
    {
        if (astroCalcAlmanacWidget->objectName().isEmpty())
            astroCalcAlmanacWidget->setObjectName("astroCalcAlmanacWidget");
        astroCalcAlmanacWidget->resize(638, 324);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(astroCalcAlmanacWidget->sizePolicy().hasHeightForWidth());
        astroCalcAlmanacWidget->setSizePolicy(sizePolicy);
        astroCalcAlmanacWidget->setWindowTitle(QString::fromUtf8("Form"));
        gridLayout = new QGridLayout(astroCalcAlmanacWidget);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setContentsMargins(0, 0, 0, 0);
        tabWidget = new QTabWidget(astroCalcAlmanacWidget);
        tabWidget->setObjectName("tabWidget");
        tabWidget->setTabPosition(QTabWidget::South);
        tabSpecificTime = new QWidget();
        tabSpecificTime->setObjectName("tabSpecificTime");
        sizePolicy.setHeightForWidth(tabSpecificTime->sizePolicy().hasHeightForWidth());
        tabSpecificTime->setSizePolicy(sizePolicy);
        gridLayoutSpecificTime = new QGridLayout(tabSpecificTime);
        gridLayoutSpecificTime->setObjectName("gridLayoutSpecificTime");
        labelLT2 = new QLabel(tabSpecificTime);
        labelLT2->setObjectName("labelLT2");
        labelLT2->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayoutSpecificTime->addWidget(labelLT2, 8, 1, 1, 1);

        labelCustomMoonrise = new QLabel(tabSpecificTime);
        labelCustomMoonrise->setObjectName("labelCustomMoonrise");
        labelCustomMoonrise->setAlignment(Qt::AlignCenter);

        gridLayoutSpecificTime->addWidget(labelCustomMoonrise, 14, 8, 1, 2);

        labelAstronomicalTwilightEnd = new QLabel(tabSpecificTime);
        labelAstronomicalTwilightEnd->setObjectName("labelAstronomicalTwilightEnd");
        labelAstronomicalTwilightEnd->setAlignment(Qt::AlignCenter);
        labelAstronomicalTwilightEnd->setTextInteractionFlags(Qt::TextSelectableByMouse);

        gridLayoutSpecificTime->addWidget(labelAstronomicalTwilightEnd, 3, 11, 1, 2);

        buttonNauticalDawn = new QPushButton(tabSpecificTime);
        buttonNauticalDawn->setObjectName("buttonNauticalDawn");
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/graphicGui/bbtTime-off.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        icon.addFile(QString::fromUtf8(":/graphicGui/bbtTime-off.png"), QSize(), QIcon::Mode::Disabled, QIcon::State::Off);
        buttonNauticalDawn->setIcon(icon);

        gridLayoutSpecificTime->addWidget(buttonNauticalDawn, 4, 10, 1, 1);

        labelCivilTwilight = new QLabel(tabSpecificTime);
        labelCivilTwilight->setObjectName("labelCivilTwilight");
        labelCivilTwilight->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayoutSpecificTime->addWidget(labelCivilTwilight, 5, 7, 1, 1);

        labelJuneSolsticeDuration = new QLabel(tabSpecificTime);
        labelJuneSolsticeDuration->setObjectName("labelJuneSolsticeDuration");
        labelJuneSolsticeDuration->setTextInteractionFlags(Qt::TextSelectableByMouse);

        gridLayoutSpecificTime->addWidget(labelJuneSolsticeDuration, 5, 5, 1, 1);

        labelNauticalTwilightEnd = new QLabel(tabSpecificTime);
        labelNauticalTwilightEnd->setObjectName("labelNauticalTwilightEnd");
        labelNauticalTwilightEnd->setAlignment(Qt::AlignCenter);
        labelNauticalTwilightEnd->setTextInteractionFlags(Qt::TextSelectableByMouse);

        gridLayoutSpecificTime->addWidget(labelNauticalTwilightEnd, 4, 11, 1, 2);

        buttonMoonrise = new QPushButton(tabSpecificTime);
        buttonMoonrise->setObjectName("buttonMoonrise");
        buttonMoonrise->setIcon(icon);

        gridLayoutSpecificTime->addWidget(buttonMoonrise, 9, 10, 1, 1);

        labelDecemberSolsticeLT = new QLabel(tabSpecificTime);
        labelDecemberSolsticeLT->setObjectName("labelDecemberSolsticeLT");
        labelDecemberSolsticeLT->setTextInteractionFlags(Qt::TextSelectableByMouse);

        gridLayoutSpecificTime->addWidget(labelDecemberSolsticeLT, 8, 5, 1, 1);

        labelCivilTwilightEnd = new QLabel(tabSpecificTime);
        labelCivilTwilightEnd->setObjectName("labelCivilTwilightEnd");
        labelCivilTwilightEnd->setAlignment(Qt::AlignCenter);
        labelCivilTwilightEnd->setTextInteractionFlags(Qt::TextSelectableByMouse);

        gridLayoutSpecificTime->addWidget(labelCivilTwilightEnd, 5, 11, 1, 2);

        labelMoonRise = new QLabel(tabSpecificTime);
        labelMoonRise->setObjectName("labelMoonRise");
        labelMoonRise->setAlignment(Qt::AlignCenter);
        labelMoonRise->setTextInteractionFlags(Qt::TextSelectableByMouse);

        gridLayoutSpecificTime->addWidget(labelMoonRise, 9, 8, 1, 2);

        buttonCustomSunset = new QPushButton(tabSpecificTime);
        buttonCustomSunset->setObjectName("buttonCustomSunset");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(buttonCustomSunset->sizePolicy().hasHeightForWidth());
        buttonCustomSunset->setSizePolicy(sizePolicy1);
        buttonCustomSunset->setIcon(icon);

        gridLayoutSpecificTime->addWidget(buttonCustomSunset, 13, 13, 1, 1);

        buttonCustomMoonrise = new QPushButton(tabSpecificTime);
        buttonCustomMoonrise->setObjectName("buttonCustomMoonrise");
        sizePolicy1.setHeightForWidth(buttonCustomMoonrise->sizePolicy().hasHeightForWidth());
        buttonCustomMoonrise->setSizePolicy(sizePolicy1);
        buttonCustomMoonrise->setIcon(icon);

        gridLayoutSpecificTime->addWidget(buttonCustomMoonrise, 14, 10, 1, 1);

        labelCustomMoonset = new QLabel(tabSpecificTime);
        labelCustomMoonset->setObjectName("labelCustomMoonset");
        labelCustomMoonset->setAlignment(Qt::AlignCenter);

        gridLayoutSpecificTime->addWidget(labelCustomMoonset, 14, 11, 1, 2);

        labelAstronomicalTwilightDuration = new QLabel(tabSpecificTime);
        labelAstronomicalTwilightDuration->setObjectName("labelAstronomicalTwilightDuration");
        labelAstronomicalTwilightDuration->setAlignment(Qt::AlignCenter);
        labelAstronomicalTwilightDuration->setTextInteractionFlags(Qt::TextSelectableByMouse);

        gridLayoutSpecificTime->addWidget(labelAstronomicalTwilightDuration, 3, 14, 1, 1);

        lineDelimiterLeftBlock = new QFrame(tabSpecificTime);
        lineDelimiterLeftBlock->setObjectName("lineDelimiterLeftBlock");
        lineDelimiterLeftBlock->setMinimumSize(QSize(1, 0));
        lineDelimiterLeftBlock->setMaximumSize(QSize(1, 16777215));
        lineDelimiterLeftBlock->setFrameShape(QFrame::Shape::VLine);
        lineDelimiterLeftBlock->setFrameShadow(QFrame::Shadow::Sunken);

        gridLayoutSpecificTime->addWidget(lineDelimiterLeftBlock, 1, 2, 10, 1);

        labelSunrise = new QLabel(tabSpecificTime);
        labelSunrise->setObjectName("labelSunrise");
        labelSunrise->setAlignment(Qt::AlignCenter);
        labelSunrise->setTextInteractionFlags(Qt::TextSelectableByMouse);

        gridLayoutSpecificTime->addWidget(labelSunrise, 8, 8, 1, 2);

        labelDecemberSolsticeDuration = new QLabel(tabSpecificTime);
        labelDecemberSolsticeDuration->setObjectName("labelDecemberSolsticeDuration");
        labelDecemberSolsticeDuration->setTextInteractionFlags(Qt::TextSelectableByMouse);

        gridLayoutSpecificTime->addWidget(labelDecemberSolsticeDuration, 10, 5, 1, 1);

        labelSun = new QLabel(tabSpecificTime);
        labelSun->setObjectName("labelSun");
        labelSun->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayoutSpecificTime->addWidget(labelSun, 8, 7, 1, 1);

        labelMarchEquinoxLT = new QLabel(tabSpecificTime);
        labelMarchEquinoxLT->setObjectName("labelMarchEquinoxLT");
        labelMarchEquinoxLT->setTextInteractionFlags(Qt::TextSelectableByMouse);

        gridLayoutSpecificTime->addWidget(labelMarchEquinoxLT, 3, 3, 1, 1);

        labelNauticalTwilightDuration = new QLabel(tabSpecificTime);
        labelNauticalTwilightDuration->setObjectName("labelNauticalTwilightDuration");
        labelNauticalTwilightDuration->setAlignment(Qt::AlignCenter);
        labelNauticalTwilightDuration->setTextInteractionFlags(Qt::TextSelectableByMouse);

        gridLayoutSpecificTime->addWidget(labelNauticalTwilightDuration, 4, 14, 1, 1);

        labelRise = new QLabel(tabSpecificTime);
        labelRise->setObjectName("labelRise");
        labelRise->setAlignment(Qt::AlignCenter);

        gridLayoutSpecificTime->addWidget(labelRise, 7, 8, 1, 3);

        buttonSunset = new QPushButton(tabSpecificTime);
        buttonSunset->setObjectName("buttonSunset");
        buttonSunset->setIcon(icon);

        gridLayoutSpecificTime->addWidget(buttonSunset, 8, 13, 1, 1);

        labelMoonSet = new QLabel(tabSpecificTime);
        labelMoonSet->setObjectName("labelMoonSet");
        labelMoonSet->setAlignment(Qt::AlignCenter);
        labelMoonSet->setTextInteractionFlags(Qt::TextSelectableByMouse);

        gridLayoutSpecificTime->addWidget(labelMoonSet, 9, 11, 1, 2);

        lineDelimiterCustomData = new QFrame(tabSpecificTime);
        lineDelimiterCustomData->setObjectName("lineDelimiterCustomData");
        lineDelimiterCustomData->setMinimumSize(QSize(0, 1));
        lineDelimiterCustomData->setMaximumSize(QSize(16777215, 1));
        lineDelimiterCustomData->setFrameShape(QFrame::Shape::HLine);
        lineDelimiterCustomData->setFrameShadow(QFrame::Shadow::Sunken);

        gridLayoutSpecificTime->addWidget(lineDelimiterCustomData, 11, 7, 1, 8);

        buttonMoonset = new QPushButton(tabSpecificTime);
        buttonMoonset->setObjectName("buttonMoonset");
        buttonMoonset->setIcon(icon);

        gridLayoutSpecificTime->addWidget(buttonMoonset, 9, 13, 1, 1);

        buttonCivilDusk = new QPushButton(tabSpecificTime);
        buttonCivilDusk->setObjectName("buttonCivilDusk");
        buttonCivilDusk->setIcon(icon);

        gridLayoutSpecificTime->addWidget(buttonCivilDusk, 5, 13, 1, 1);

        buttonSunrise = new QPushButton(tabSpecificTime);
        buttonSunrise->setObjectName("buttonSunrise");
        buttonSunrise->setIcon(icon);

        gridLayoutSpecificTime->addWidget(buttonSunrise, 8, 10, 1, 1);

        labelMoon = new QLabel(tabSpecificTime);
        labelMoon->setObjectName("labelMoon");
        labelMoon->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayoutSpecificTime->addWidget(labelMoon, 9, 7, 1, 1);

        buttonCustomSunrise = new QPushButton(tabSpecificTime);
        buttonCustomSunrise->setObjectName("buttonCustomSunrise");
        sizePolicy1.setHeightForWidth(buttonCustomSunrise->sizePolicy().hasHeightForWidth());
        buttonCustomSunrise->setSizePolicy(sizePolicy1);
        buttonCustomSunrise->setIcon(icon);

        gridLayoutSpecificTime->addWidget(buttonCustomSunrise, 13, 10, 1, 1);

        DecemberSolsticeLayout = new QHBoxLayout();
        DecemberSolsticeLayout->setObjectName("DecemberSolsticeLayout");
        labelDecemberSolstice = new QLabel(tabSpecificTime);
        labelDecemberSolstice->setObjectName("labelDecemberSolstice");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(labelDecemberSolstice->sizePolicy().hasHeightForWidth());
        labelDecemberSolstice->setSizePolicy(sizePolicy2);

        DecemberSolsticeLayout->addWidget(labelDecemberSolstice);

        buttonDecemberSolsticePrevious = new QPushButton(tabSpecificTime);
        buttonDecemberSolsticePrevious->setObjectName("buttonDecemberSolsticePrevious");
        sizePolicy1.setHeightForWidth(buttonDecemberSolsticePrevious->sizePolicy().hasHeightForWidth());
        buttonDecemberSolsticePrevious->setSizePolicy(sizePolicy1);
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/graphicGui/btTimeRewind-off.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        icon1.addFile(QString::fromUtf8(":/graphicGui/btTimeRewind-off.png"), QSize(), QIcon::Mode::Disabled, QIcon::State::Off);
        buttonDecemberSolsticePrevious->setIcon(icon1);

        DecemberSolsticeLayout->addWidget(buttonDecemberSolsticePrevious);

        buttonDecemberSolsticeCurrent = new QPushButton(tabSpecificTime);
        buttonDecemberSolsticeCurrent->setObjectName("buttonDecemberSolsticeCurrent");
        sizePolicy1.setHeightForWidth(buttonDecemberSolsticeCurrent->sizePolicy().hasHeightForWidth());
        buttonDecemberSolsticeCurrent->setSizePolicy(sizePolicy1);
        buttonDecemberSolsticeCurrent->setIcon(icon);

        DecemberSolsticeLayout->addWidget(buttonDecemberSolsticeCurrent);

        buttonDecemberSolsticeNext = new QPushButton(tabSpecificTime);
        buttonDecemberSolsticeNext->setObjectName("buttonDecemberSolsticeNext");
        sizePolicy1.setHeightForWidth(buttonDecemberSolsticeNext->sizePolicy().hasHeightForWidth());
        buttonDecemberSolsticeNext->setSizePolicy(sizePolicy1);
        QIcon icon2;
        icon2.addFile(QString::fromUtf8(":/graphicGui/btTimeForward-off.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        icon2.addFile(QString::fromUtf8(":/graphicGui/btTimeForward-off.png"), QSize(), QIcon::Mode::Disabled, QIcon::State::Off);
        buttonDecemberSolsticeNext->setIcon(icon2);

        DecemberSolsticeLayout->addWidget(buttonDecemberSolsticeNext);


        gridLayoutSpecificTime->addLayout(DecemberSolsticeLayout, 7, 5, 1, 1);

        labelDurationColumn1 = new QLabel(tabSpecificTime);
        labelDurationColumn1->setObjectName("labelDurationColumn1");
        labelDurationColumn1->setAlignment(Qt::AlignCenter);

        gridLayoutSpecificTime->addWidget(labelDurationColumn1, 1, 14, 1, 1);

        labelAfterSunset = new QLabel(tabSpecificTime);
        labelAfterSunset->setObjectName("labelAfterSunset");
        labelAfterSunset->setAlignment(Qt::AlignCenter);
        labelAfterSunset->setTextInteractionFlags(Qt::TextSelectableByMouse);

        gridLayoutSpecificTime->addWidget(labelAfterSunset, 12, 11, 1, 2);

        labelCustomSunset = new QLabel(tabSpecificTime);
        labelCustomSunset->setObjectName("labelCustomSunset");
        labelCustomSunset->setAlignment(Qt::AlignCenter);
        labelCustomSunset->setTextInteractionFlags(Qt::TextSelectableByMouse);

        gridLayoutSpecificTime->addWidget(labelCustomSunset, 13, 12, 1, 1);

        labelDawn = new QLabel(tabSpecificTime);
        labelDawn->setObjectName("labelDawn");
        labelDawn->setAlignment(Qt::AlignCenter);

        gridLayoutSpecificTime->addWidget(labelDawn, 1, 8, 1, 3);

        labelDuration1 = new QLabel(tabSpecificTime);
        labelDuration1->setObjectName("labelDuration1");
        labelDuration1->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayoutSpecificTime->addWidget(labelDuration1, 5, 1, 1, 1);

        labelNightDuration = new QLabel(tabSpecificTime);
        labelNightDuration->setObjectName("labelNightDuration");
        QFont font;
        font.setItalic(true);
        labelNightDuration->setFont(font);
        labelNightDuration->setAlignment(Qt::AlignCenter);
        labelNightDuration->setTextInteractionFlags(Qt::TextSelectableByMouse);

        gridLayoutSpecificTime->addWidget(labelNightDuration, 10, 14, 1, 1);

        YearLayout = new QHBoxLayout();
        YearLayout->setObjectName("YearLayout");
        labelYear = new QLabel(tabSpecificTime);
        labelYear->setObjectName("labelYear");
        labelYear->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        YearLayout->addWidget(labelYear);

        labelCurrentYear = new QLabel(tabSpecificTime);
        labelCurrentYear->setObjectName("labelCurrentYear");
        QFont font1;
        font1.setBold(true);
        labelCurrentYear->setFont(font1);
        labelCurrentYear->setAlignment(Qt::AlignCenter);

        YearLayout->addWidget(labelCurrentYear);

        labelYearDuration = new QLabel(tabSpecificTime);
        labelYearDuration->setObjectName("labelYearDuration");
        labelYearDuration->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        YearLayout->addWidget(labelYearDuration);


        gridLayoutSpecificTime->addLayout(YearLayout, 1, 1, 1, 1);

        lineDelimiterSeasonsTop = new QFrame(tabSpecificTime);
        lineDelimiterSeasonsTop->setObjectName("lineDelimiterSeasonsTop");
        lineDelimiterSeasonsTop->setMinimumSize(QSize(1, 0));
        lineDelimiterSeasonsTop->setMaximumSize(QSize(1, 16777215));
        lineDelimiterSeasonsTop->setFrameShape(QFrame::Shape::VLine);
        lineDelimiterSeasonsTop->setFrameShadow(QFrame::Shadow::Sunken);

        gridLayoutSpecificTime->addWidget(lineDelimiterSeasonsTop, 1, 4, 5, 1);

        buttonAstronomicalDawn = new QPushButton(tabSpecificTime);
        buttonAstronomicalDawn->setObjectName("buttonAstronomicalDawn");
        buttonAstronomicalDawn->setIcon(icon);

        gridLayoutSpecificTime->addWidget(buttonAstronomicalDawn, 3, 10, 1, 1);

        labelCustomSun = new QLabel(tabSpecificTime);
        labelCustomSun->setObjectName("labelCustomSun");
        labelCustomSun->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayoutSpecificTime->addWidget(labelCustomSun, 13, 7, 1, 1);

        spinBoxCustomSunAltitude = new QDoubleSpinBox(tabSpecificTime);
        spinBoxCustomSunAltitude->setObjectName("spinBoxCustomSunAltitude");
        spinBoxCustomSunAltitude->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        spinBoxCustomSunAltitude->setMinimum(-90.000000000000000);
        spinBoxCustomSunAltitude->setMaximum(90.000000000000000);
        spinBoxCustomSunAltitude->setValue(-7.000000000000000);

        gridLayoutSpecificTime->addWidget(spinBoxCustomSunAltitude, 13, 14, 1, 1);

        buttonCustomMoonset = new QPushButton(tabSpecificTime);
        buttonCustomMoonset->setObjectName("buttonCustomMoonset");
        sizePolicy1.setHeightForWidth(buttonCustomMoonset->sizePolicy().hasHeightForWidth());
        buttonCustomMoonset->setSizePolicy(sizePolicy1);
        buttonCustomMoonset->setIcon(icon);

        gridLayoutSpecificTime->addWidget(buttonCustomMoonset, 14, 13, 1, 1);

        labelSeptemberEquinoxDuration = new QLabel(tabSpecificTime);
        labelSeptemberEquinoxDuration->setObjectName("labelSeptemberEquinoxDuration");
        labelSeptemberEquinoxDuration->setTextInteractionFlags(Qt::TextSelectableByMouse);

        gridLayoutSpecificTime->addWidget(labelSeptemberEquinoxDuration, 10, 3, 1, 1);

        lineDelimiterSections = new QFrame(tabSpecificTime);
        lineDelimiterSections->setObjectName("lineDelimiterSections");
        lineDelimiterSections->setMinimumSize(QSize(1, 0));
        lineDelimiterSections->setMaximumSize(QSize(1, 16777215));
        lineDelimiterSections->setFrameShape(QFrame::Shape::VLine);
        lineDelimiterSections->setFrameShadow(QFrame::Shadow::Sunken);

        gridLayoutSpecificTime->addWidget(lineDelimiterSections, 0, 6, 15, 1);

        labelNauticalTwilightBegin = new QLabel(tabSpecificTime);
        labelNauticalTwilightBegin->setObjectName("labelNauticalTwilightBegin");
        labelNauticalTwilightBegin->setAlignment(Qt::AlignCenter);
        labelNauticalTwilightBegin->setTextInteractionFlags(Qt::TextSelectableByMouse);

        gridLayoutSpecificTime->addWidget(labelNauticalTwilightBegin, 4, 8, 1, 2);

        spinBoxMinutes = new QSpinBox(tabSpecificTime);
        spinBoxMinutes->setObjectName("spinBoxMinutes");
        spinBoxMinutes->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        spinBoxMinutes->setMinimum(1);
        spinBoxMinutes->setMaximum(120);
        spinBoxMinutes->setValue(60);

        gridLayoutSpecificTime->addWidget(spinBoxMinutes, 12, 14, 1, 1);

        spinBoxCustomMoonAltitude = new QDoubleSpinBox(tabSpecificTime);
        spinBoxCustomMoonAltitude->setObjectName("spinBoxCustomMoonAltitude");
        spinBoxCustomMoonAltitude->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        spinBoxCustomMoonAltitude->setMinimum(-90.000000000000000);
        spinBoxCustomMoonAltitude->setMaximum(90.000000000000000);
        spinBoxCustomMoonAltitude->setValue(18.000000000000000);

        gridLayoutSpecificTime->addWidget(spinBoxCustomMoonAltitude, 14, 14, 1, 1);

        labelNauticalTwilight = new QLabel(tabSpecificTime);
        labelNauticalTwilight->setObjectName("labelNauticalTwilight");
        labelNauticalTwilight->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayoutSpecificTime->addWidget(labelNauticalTwilight, 4, 7, 1, 1);

        labelDuration2 = new QLabel(tabSpecificTime);
        labelDuration2->setObjectName("labelDuration2");
        labelDuration2->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayoutSpecificTime->addWidget(labelDuration2, 10, 1, 1, 1);

        labelMarchEquinoxDuration = new QLabel(tabSpecificTime);
        labelMarchEquinoxDuration->setObjectName("labelMarchEquinoxDuration");
        labelMarchEquinoxDuration->setTextInteractionFlags(Qt::TextSelectableByMouse);

        gridLayoutSpecificTime->addWidget(labelMarchEquinoxDuration, 5, 3, 1, 1);

        labelJuneSolsticeJD = new QLabel(tabSpecificTime);
        labelJuneSolsticeJD->setObjectName("labelJuneSolsticeJD");
        labelJuneSolsticeJD->setTextInteractionFlags(Qt::TextSelectableByMouse);

        gridLayoutSpecificTime->addWidget(labelJuneSolsticeJD, 4, 5, 1, 1);

        buttonAstronomicalDusk = new QPushButton(tabSpecificTime);
        buttonAstronomicalDusk->setObjectName("buttonAstronomicalDusk");
        buttonAstronomicalDusk->setIcon(icon);

        gridLayoutSpecificTime->addWidget(buttonAstronomicalDusk, 3, 13, 1, 1);

        labelTwilight = new QLabel(tabSpecificTime);
        labelTwilight->setObjectName("labelTwilight");
        labelTwilight->setAlignment(Qt::AlignCenter);

        gridLayoutSpecificTime->addWidget(labelTwilight, 1, 7, 1, 1);

        labelJD1 = new QLabel(tabSpecificTime);
        labelJD1->setObjectName("labelJD1");
        labelJD1->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayoutSpecificTime->addWidget(labelJD1, 4, 1, 1, 1);

        labelJuneSolsticeLT = new QLabel(tabSpecificTime);
        labelJuneSolsticeLT->setObjectName("labelJuneSolsticeLT");
        labelJuneSolsticeLT->setTextInteractionFlags(Qt::TextSelectableByMouse);

        gridLayoutSpecificTime->addWidget(labelJuneSolsticeLT, 3, 5, 1, 1);

        labelSeptemberEquinoxJD = new QLabel(tabSpecificTime);
        labelSeptemberEquinoxJD->setObjectName("labelSeptemberEquinoxJD");
        labelSeptemberEquinoxJD->setTextInteractionFlags(Qt::TextSelectableByMouse);

        gridLayoutSpecificTime->addWidget(labelSeptemberEquinoxJD, 9, 3, 1, 1);

        JuneSolsticeLayout = new QHBoxLayout();
        JuneSolsticeLayout->setObjectName("JuneSolsticeLayout");
        labelJuneSolstice = new QLabel(tabSpecificTime);
        labelJuneSolstice->setObjectName("labelJuneSolstice");
        sizePolicy2.setHeightForWidth(labelJuneSolstice->sizePolicy().hasHeightForWidth());
        labelJuneSolstice->setSizePolicy(sizePolicy2);

        JuneSolsticeLayout->addWidget(labelJuneSolstice);

        buttonJuneSolsticePrevious = new QPushButton(tabSpecificTime);
        buttonJuneSolsticePrevious->setObjectName("buttonJuneSolsticePrevious");
        sizePolicy1.setHeightForWidth(buttonJuneSolsticePrevious->sizePolicy().hasHeightForWidth());
        buttonJuneSolsticePrevious->setSizePolicy(sizePolicy1);
        buttonJuneSolsticePrevious->setIcon(icon1);

        JuneSolsticeLayout->addWidget(buttonJuneSolsticePrevious);

        buttonJuneSolsticeCurrent = new QPushButton(tabSpecificTime);
        buttonJuneSolsticeCurrent->setObjectName("buttonJuneSolsticeCurrent");
        sizePolicy1.setHeightForWidth(buttonJuneSolsticeCurrent->sizePolicy().hasHeightForWidth());
        buttonJuneSolsticeCurrent->setSizePolicy(sizePolicy1);
        buttonJuneSolsticeCurrent->setIcon(icon);

        JuneSolsticeLayout->addWidget(buttonJuneSolsticeCurrent);

        buttonJuneSolsticeNext = new QPushButton(tabSpecificTime);
        buttonJuneSolsticeNext->setObjectName("buttonJuneSolsticeNext");
        sizePolicy1.setHeightForWidth(buttonJuneSolsticeNext->sizePolicy().hasHeightForWidth());
        buttonJuneSolsticeNext->setSizePolicy(sizePolicy1);
        buttonJuneSolsticeNext->setIcon(icon2);

        JuneSolsticeLayout->addWidget(buttonJuneSolsticeNext);


        gridLayoutSpecificTime->addLayout(JuneSolsticeLayout, 1, 5, 1, 1);

        labelDayDuration = new QLabel(tabSpecificTime);
        labelDayDuration->setObjectName("labelDayDuration");
        labelDayDuration->setAlignment(Qt::AlignCenter);
        labelDayDuration->setTextInteractionFlags(Qt::TextSelectableByMouse);

        gridLayoutSpecificTime->addWidget(labelDayDuration, 8, 14, 1, 1);

        lineDelimiterToday = new QFrame(tabSpecificTime);
        lineDelimiterToday->setObjectName("lineDelimiterToday");
        lineDelimiterToday->setMinimumSize(QSize(0, 1));
        lineDelimiterToday->setMaximumSize(QSize(16777215, 1));
        lineDelimiterToday->setFrameShape(QFrame::Shape::HLine);
        lineDelimiterToday->setFrameShadow(QFrame::Shadow::Sunken);

        gridLayoutSpecificTime->addWidget(lineDelimiterToday, 6, 7, 1, 8);

        labelNight = new QLabel(tabSpecificTime);
        labelNight->setObjectName("labelNight");
        labelNight->setFont(font);
        labelNight->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayoutSpecificTime->addWidget(labelNight, 10, 7, 1, 7);

        SeptemberEquinoxLayout = new QHBoxLayout();
        SeptemberEquinoxLayout->setObjectName("SeptemberEquinoxLayout");
        labelSeptemberEquinox = new QLabel(tabSpecificTime);
        labelSeptemberEquinox->setObjectName("labelSeptemberEquinox");
        sizePolicy2.setHeightForWidth(labelSeptemberEquinox->sizePolicy().hasHeightForWidth());
        labelSeptemberEquinox->setSizePolicy(sizePolicy2);

        SeptemberEquinoxLayout->addWidget(labelSeptemberEquinox);

        buttonSeptemberEquinoxPrevious = new QPushButton(tabSpecificTime);
        buttonSeptemberEquinoxPrevious->setObjectName("buttonSeptemberEquinoxPrevious");
        buttonSeptemberEquinoxPrevious->setIcon(icon1);

        SeptemberEquinoxLayout->addWidget(buttonSeptemberEquinoxPrevious);

        buttonSeptemberEquinoxCurrent = new QPushButton(tabSpecificTime);
        buttonSeptemberEquinoxCurrent->setObjectName("buttonSeptemberEquinoxCurrent");
        sizePolicy1.setHeightForWidth(buttonSeptemberEquinoxCurrent->sizePolicy().hasHeightForWidth());
        buttonSeptemberEquinoxCurrent->setSizePolicy(sizePolicy1);
        buttonSeptemberEquinoxCurrent->setIcon(icon);

        SeptemberEquinoxLayout->addWidget(buttonSeptemberEquinoxCurrent);

        buttonSeptemberEquinoxNext = new QPushButton(tabSpecificTime);
        buttonSeptemberEquinoxNext->setObjectName("buttonSeptemberEquinoxNext");
        sizePolicy1.setHeightForWidth(buttonSeptemberEquinoxNext->sizePolicy().hasHeightForWidth());
        buttonSeptemberEquinoxNext->setSizePolicy(sizePolicy1);
        buttonSeptemberEquinoxNext->setIcon(icon2);

        SeptemberEquinoxLayout->addWidget(buttonSeptemberEquinoxNext);


        gridLayoutSpecificTime->addLayout(SeptemberEquinoxLayout, 7, 3, 1, 1);

        labelCivilTwilightBegin = new QLabel(tabSpecificTime);
        labelCivilTwilightBegin->setObjectName("labelCivilTwilightBegin");
        labelCivilTwilightBegin->setAlignment(Qt::AlignCenter);
        labelCivilTwilightBegin->setTextInteractionFlags(Qt::TextSelectableByMouse);

        gridLayoutSpecificTime->addWidget(labelCivilTwilightBegin, 5, 8, 1, 2);

        labelSunset = new QLabel(tabSpecificTime);
        labelSunset->setObjectName("labelSunset");
        labelSunset->setAlignment(Qt::AlignCenter);
        labelSunset->setTextInteractionFlags(Qt::TextSelectableByMouse);

        gridLayoutSpecificTime->addWidget(labelSunset, 8, 11, 1, 2);

        buttonAfterSunset = new QPushButton(tabSpecificTime);
        buttonAfterSunset->setObjectName("buttonAfterSunset");
        sizePolicy1.setHeightForWidth(buttonAfterSunset->sizePolicy().hasHeightForWidth());
        buttonAfterSunset->setSizePolicy(sizePolicy1);
        buttonAfterSunset->setIcon(icon);

        gridLayoutSpecificTime->addWidget(buttonAfterSunset, 12, 13, 1, 1);

        labelSeptemberEquinoxLT = new QLabel(tabSpecificTime);
        labelSeptemberEquinoxLT->setObjectName("labelSeptemberEquinoxLT");
        labelSeptemberEquinoxLT->setTextInteractionFlags(Qt::TextSelectableByMouse);

        gridLayoutSpecificTime->addWidget(labelSeptemberEquinoxLT, 8, 3, 1, 1);

        MarchEquinoxLayout = new QHBoxLayout();
        MarchEquinoxLayout->setObjectName("MarchEquinoxLayout");
        labelMarchEquinox = new QLabel(tabSpecificTime);
        labelMarchEquinox->setObjectName("labelMarchEquinox");
        sizePolicy2.setHeightForWidth(labelMarchEquinox->sizePolicy().hasHeightForWidth());
        labelMarchEquinox->setSizePolicy(sizePolicy2);

        MarchEquinoxLayout->addWidget(labelMarchEquinox);

        buttonMarchEquinoxPrevious = new QPushButton(tabSpecificTime);
        buttonMarchEquinoxPrevious->setObjectName("buttonMarchEquinoxPrevious");
        sizePolicy1.setHeightForWidth(buttonMarchEquinoxPrevious->sizePolicy().hasHeightForWidth());
        buttonMarchEquinoxPrevious->setSizePolicy(sizePolicy1);
        buttonMarchEquinoxPrevious->setIcon(icon1);

        MarchEquinoxLayout->addWidget(buttonMarchEquinoxPrevious);

        buttonMarchEquinoxCurrent = new QPushButton(tabSpecificTime);
        buttonMarchEquinoxCurrent->setObjectName("buttonMarchEquinoxCurrent");
        sizePolicy1.setHeightForWidth(buttonMarchEquinoxCurrent->sizePolicy().hasHeightForWidth());
        buttonMarchEquinoxCurrent->setSizePolicy(sizePolicy1);
        buttonMarchEquinoxCurrent->setIcon(icon);

        MarchEquinoxLayout->addWidget(buttonMarchEquinoxCurrent);

        buttonMarchEquinoxNext = new QPushButton(tabSpecificTime);
        buttonMarchEquinoxNext->setObjectName("buttonMarchEquinoxNext");
        sizePolicy1.setHeightForWidth(buttonMarchEquinoxNext->sizePolicy().hasHeightForWidth());
        buttonMarchEquinoxNext->setSizePolicy(sizePolicy1);
        buttonMarchEquinoxNext->setIcon(icon2);

        MarchEquinoxLayout->addWidget(buttonMarchEquinoxNext);


        gridLayoutSpecificTime->addLayout(MarchEquinoxLayout, 1, 3, 1, 1);

        labelCustomSunrise = new QLabel(tabSpecificTime);
        labelCustomSunrise->setObjectName("labelCustomSunrise");
        labelCustomSunrise->setAlignment(Qt::AlignCenter);

        gridLayoutSpecificTime->addWidget(labelCustomSunrise, 13, 8, 1, 2);

        labelCustomMoon = new QLabel(tabSpecificTime);
        labelCustomMoon->setObjectName("labelCustomMoon");
        labelCustomMoon->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayoutSpecificTime->addWidget(labelCustomMoon, 14, 7, 1, 1);

        lineDelimiterSeasonsBottom = new QFrame(tabSpecificTime);
        lineDelimiterSeasonsBottom->setObjectName("lineDelimiterSeasonsBottom");
        lineDelimiterSeasonsBottom->setMinimumSize(QSize(1, 0));
        lineDelimiterSeasonsBottom->setMaximumSize(QSize(1, 16777215));
        lineDelimiterSeasonsBottom->setFrameShape(QFrame::Shape::VLine);
        lineDelimiterSeasonsBottom->setFrameShadow(QFrame::Shadow::Sunken);

        gridLayoutSpecificTime->addWidget(lineDelimiterSeasonsBottom, 7, 4, 4, 1);

        labelCivilTwilightDuration = new QLabel(tabSpecificTime);
        labelCivilTwilightDuration->setObjectName("labelCivilTwilightDuration");
        labelCivilTwilightDuration->setAlignment(Qt::AlignCenter);
        labelCivilTwilightDuration->setTextInteractionFlags(Qt::TextSelectableByMouse);

        gridLayoutSpecificTime->addWidget(labelCivilTwilightDuration, 5, 14, 1, 1);

        labelSet = new QLabel(tabSpecificTime);
        labelSet->setObjectName("labelSet");
        labelSet->setAlignment(Qt::AlignCenter);

        gridLayoutSpecificTime->addWidget(labelSet, 7, 11, 1, 3);

        labelDurationColumn2 = new QLabel(tabSpecificTime);
        labelDurationColumn2->setObjectName("labelDurationColumn2");

        gridLayoutSpecificTime->addWidget(labelDurationColumn2, 7, 14, 1, 1);

        labelSeasons = new QLabel(tabSpecificTime);
        labelSeasons->setObjectName("labelSeasons");
        labelSeasons->setFont(font1);
        labelSeasons->setAlignment(Qt::AlignCenter);

        gridLayoutSpecificTime->addWidget(labelSeasons, 0, 1, 1, 5);

        lineDelimiterSeasonsMiddle = new QFrame(tabSpecificTime);
        lineDelimiterSeasonsMiddle->setObjectName("lineDelimiterSeasonsMiddle");
        lineDelimiterSeasonsMiddle->setMinimumSize(QSize(0, 1));
        lineDelimiterSeasonsMiddle->setMaximumSize(QSize(16777215, 1));
        lineDelimiterSeasonsMiddle->setFrameShape(QFrame::Shape::HLine);
        lineDelimiterSeasonsMiddle->setFrameShadow(QFrame::Shadow::Sunken);

        gridLayoutSpecificTime->addWidget(lineDelimiterSeasonsMiddle, 6, 3, 1, 3);

        buttonNauticalDusk = new QPushButton(tabSpecificTime);
        buttonNauticalDusk->setObjectName("buttonNauticalDusk");
        buttonNauticalDusk->setIcon(icon);

        gridLayoutSpecificTime->addWidget(buttonNauticalDusk, 4, 13, 1, 1);

        buttonBeforeSunrise = new QPushButton(tabSpecificTime);
        buttonBeforeSunrise->setObjectName("buttonBeforeSunrise");
        sizePolicy1.setHeightForWidth(buttonBeforeSunrise->sizePolicy().hasHeightForWidth());
        buttonBeforeSunrise->setSizePolicy(sizePolicy1);
        buttonBeforeSunrise->setIcon(icon);

        gridLayoutSpecificTime->addWidget(buttonBeforeSunrise, 12, 10, 1, 1);

        labelBeforeSunrise = new QLabel(tabSpecificTime);
        labelBeforeSunrise->setObjectName("labelBeforeSunrise");
        labelBeforeSunrise->setAlignment(Qt::AlignCenter);

        gridLayoutSpecificTime->addWidget(labelBeforeSunrise, 12, 8, 1, 2);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        buttonYesterday = new QPushButton(tabSpecificTime);
        buttonYesterday->setObjectName("buttonYesterday");
        sizePolicy1.setHeightForWidth(buttonYesterday->sizePolicy().hasHeightForWidth());
        buttonYesterday->setSizePolicy(sizePolicy1);
        buttonYesterday->setIcon(icon1);

        horizontalLayout->addWidget(buttonYesterday);

        labelToday = new QLabel(tabSpecificTime);
        labelToday->setObjectName("labelToday");
        labelToday->setFont(font1);
        labelToday->setAlignment(Qt::AlignCenter);

        horizontalLayout->addWidget(labelToday);

        buttonTomorrow = new QPushButton(tabSpecificTime);
        buttonTomorrow->setObjectName("buttonTomorrow");
        sizePolicy1.setHeightForWidth(buttonTomorrow->sizePolicy().hasHeightForWidth());
        buttonTomorrow->setSizePolicy(sizePolicy1);
        QIcon icon3;
        icon3.addFile(QString::fromUtf8(":/graphicGui/btTimeForward-off.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        icon3.addFile(QString::fromUtf8(":/graphicGui/btTimeForward-off.png"), QSize(), QIcon::Mode::Disabled, QIcon::State::On);
        buttonTomorrow->setIcon(icon3);

        horizontalLayout->addWidget(buttonTomorrow);


        gridLayoutSpecificTime->addLayout(horizontalLayout, 0, 7, 1, 8);

        labelAstronomicalTwilight = new QLabel(tabSpecificTime);
        labelAstronomicalTwilight->setObjectName("labelAstronomicalTwilight");
        labelAstronomicalTwilight->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayoutSpecificTime->addWidget(labelAstronomicalTwilight, 3, 7, 1, 1);

        labelAstronomicalTwilightBegin = new QLabel(tabSpecificTime);
        labelAstronomicalTwilightBegin->setObjectName("labelAstronomicalTwilightBegin");
        labelAstronomicalTwilightBegin->setAlignment(Qt::AlignCenter);
        labelAstronomicalTwilightBegin->setTextInteractionFlags(Qt::TextSelectableByMouse);

        gridLayoutSpecificTime->addWidget(labelAstronomicalTwilightBegin, 3, 8, 1, 2);

        labelLT1 = new QLabel(tabSpecificTime);
        labelLT1->setObjectName("labelLT1");
        sizePolicy2.setHeightForWidth(labelLT1->sizePolicy().hasHeightForWidth());
        labelLT1->setSizePolicy(sizePolicy2);
        labelLT1->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayoutSpecificTime->addWidget(labelLT1, 3, 1, 1, 1);

        labelJD2 = new QLabel(tabSpecificTime);
        labelJD2->setObjectName("labelJD2");
        labelJD2->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayoutSpecificTime->addWidget(labelJD2, 9, 1, 1, 1);

        labelDecemberSolsticeJD = new QLabel(tabSpecificTime);
        labelDecemberSolsticeJD->setObjectName("labelDecemberSolsticeJD");
        labelDecemberSolsticeJD->setTextInteractionFlags(Qt::TextSelectableByMouse);

        gridLayoutSpecificTime->addWidget(labelDecemberSolsticeJD, 9, 5, 1, 1);

        labelDusk = new QLabel(tabSpecificTime);
        labelDusk->setObjectName("labelDusk");
        labelDusk->setAlignment(Qt::AlignCenter);

        gridLayoutSpecificTime->addWidget(labelDusk, 1, 11, 1, 3);

        labelMarchEquinoxJD = new QLabel(tabSpecificTime);
        labelMarchEquinoxJD->setObjectName("labelMarchEquinoxJD");
        labelMarchEquinoxJD->setTextInteractionFlags(Qt::TextSelectableByMouse);

        gridLayoutSpecificTime->addWidget(labelMarchEquinoxJD, 4, 3, 1, 1);

        buttonCivilDawn = new QPushButton(tabSpecificTime);
        buttonCivilDawn->setObjectName("buttonCivilDawn");
        buttonCivilDawn->setIcon(icon);

        gridLayoutSpecificTime->addWidget(buttonCivilDawn, 5, 10, 1, 1);

        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayoutSpecificTime->addItem(verticalSpacer_2, 15, 7, 1, 1);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayoutSpecificTime->addItem(verticalSpacer, 11, 1, 4, 1);

        tabWidget->addTab(tabSpecificTime, QString());

        gridLayout->addWidget(tabWidget, 0, 0, 1, 1);


        retranslateUi(astroCalcAlmanacWidget);

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(astroCalcAlmanacWidget);
    } // setupUi

    void retranslateUi(QWidget *astroCalcAlmanacWidget)
    {
        labelLT2->setText(QCoreApplication::translate("astroCalcAlmanacWidget", "Local Time", nullptr));
        labelCustomMoonrise->setText(QString());
        labelAstronomicalTwilightEnd->setText(QString());
#if QT_CONFIG(tooltip)
        buttonNauticalDawn->setToolTip(QCoreApplication::translate("astroCalcAlmanacWidget", "Morning nautical twilight", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonNauticalDawn->setText(QString());
        labelCivilTwilight->setText(QCoreApplication::translate("astroCalcAlmanacWidget", "Civil", "twilight"));
        labelNauticalTwilightEnd->setText(QString());
#if QT_CONFIG(tooltip)
        buttonMoonrise->setToolTip(QCoreApplication::translate("astroCalcAlmanacWidget", "Today's moonrise", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonMoonrise->setText(QString());
        labelCivilTwilightEnd->setText(QString());
        labelMoonRise->setText(QString());
#if QT_CONFIG(tooltip)
        buttonCustomSunset->setToolTip(QCoreApplication::translate("astroCalcAlmanacWidget", "Today's Sun at custom altitude (set)", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonCustomSunset->setText(QString());
#if QT_CONFIG(tooltip)
        buttonCustomMoonrise->setToolTip(QCoreApplication::translate("astroCalcAlmanacWidget", "Today's Moon at custom altitude (rise)", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonCustomMoonrise->setText(QString());
        labelCustomMoonset->setText(QString());
        labelAstronomicalTwilightDuration->setText(QString());
        labelSunrise->setText(QString());
        labelSun->setText(QCoreApplication::translate("astroCalcAlmanacWidget", "Sun", nullptr));
        labelNauticalTwilightDuration->setText(QString());
        labelRise->setText(QCoreApplication::translate("astroCalcAlmanacWidget", "Rise", nullptr));
#if QT_CONFIG(tooltip)
        buttonSunset->setToolTip(QCoreApplication::translate("astroCalcAlmanacWidget", "Today's sunset", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonSunset->setText(QString());
        labelMoonSet->setText(QString());
#if QT_CONFIG(tooltip)
        buttonMoonset->setToolTip(QCoreApplication::translate("astroCalcAlmanacWidget", "Today's moonset", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonMoonset->setText(QString());
#if QT_CONFIG(tooltip)
        buttonCivilDusk->setToolTip(QCoreApplication::translate("astroCalcAlmanacWidget", "Evening civil twilight", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonCivilDusk->setText(QString());
#if QT_CONFIG(tooltip)
        buttonSunrise->setToolTip(QCoreApplication::translate("astroCalcAlmanacWidget", "Today's sunrise", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonSunrise->setText(QString());
        labelMoon->setText(QCoreApplication::translate("astroCalcAlmanacWidget", "Moon", nullptr));
#if QT_CONFIG(tooltip)
        buttonCustomSunrise->setToolTip(QCoreApplication::translate("astroCalcAlmanacWidget", "Today's Sun at custom altitude (rise)", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonCustomSunrise->setText(QString());
        labelDecemberSolstice->setText(QCoreApplication::translate("astroCalcAlmanacWidget", "Winter", "season"));
#if QT_CONFIG(tooltip)
        buttonDecemberSolsticePrevious->setToolTip(QCoreApplication::translate("astroCalcAlmanacWidget", "Previous December Solstice", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        buttonDecemberSolsticeCurrent->setToolTip(QCoreApplication::translate("astroCalcAlmanacWidget", "Current December Solstice", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        buttonDecemberSolsticeNext->setToolTip(QCoreApplication::translate("astroCalcAlmanacWidget", "Next December Solstice", nullptr));
#endif // QT_CONFIG(tooltip)
        labelDurationColumn1->setText(QCoreApplication::translate("astroCalcAlmanacWidget", "Duration", nullptr));
        labelAfterSunset->setText(QString());
        labelCustomSunset->setText(QString());
        labelDawn->setText(QCoreApplication::translate("astroCalcAlmanacWidget", "Dawn", nullptr));
        labelDuration1->setText(QCoreApplication::translate("astroCalcAlmanacWidget", "Duration", nullptr));
#if QT_CONFIG(tooltip)
        labelNightDuration->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
        labelNightDuration->setText(QString());
        labelYear->setText(QCoreApplication::translate("astroCalcAlmanacWidget", "Year", nullptr));
        labelCurrentYear->setText(QString());
        labelYearDuration->setText(QString());
#if QT_CONFIG(tooltip)
        buttonAstronomicalDawn->setToolTip(QCoreApplication::translate("astroCalcAlmanacWidget", "Morning astronomical twilight", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonAstronomicalDawn->setText(QString());
        labelCustomSun->setText(QCoreApplication::translate("astroCalcAlmanacWidget", "Sun", nullptr));
#if QT_CONFIG(tooltip)
        spinBoxCustomSunAltitude->setToolTip(QCoreApplication::translate("astroCalcAlmanacWidget", "Custom altitude of the Sun", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        buttonCustomMoonset->setToolTip(QCoreApplication::translate("astroCalcAlmanacWidget", "Today's Moon at custom altitude (set)", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonCustomMoonset->setText(QString());
        labelNauticalTwilightBegin->setText(QString());
#if QT_CONFIG(tooltip)
        spinBoxMinutes->setToolTip(QCoreApplication::translate("astroCalcAlmanacWidget", "Minutes before sunrise/after sunset", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        spinBoxCustomMoonAltitude->setToolTip(QCoreApplication::translate("astroCalcAlmanacWidget", "Custom altitude of the Moon", nullptr));
#endif // QT_CONFIG(tooltip)
        labelNauticalTwilight->setText(QCoreApplication::translate("astroCalcAlmanacWidget", "Nautical", "twilight"));
        labelDuration2->setText(QCoreApplication::translate("astroCalcAlmanacWidget", "Duration", nullptr));
#if QT_CONFIG(tooltip)
        buttonAstronomicalDusk->setToolTip(QCoreApplication::translate("astroCalcAlmanacWidget", "Evening astronomical twilight", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonAstronomicalDusk->setText(QString());
        labelTwilight->setText(QCoreApplication::translate("astroCalcAlmanacWidget", "Twilight", nullptr));
        labelJD1->setText(QCoreApplication::translate("astroCalcAlmanacWidget", "Julian Day", nullptr));
        labelJuneSolstice->setText(QCoreApplication::translate("astroCalcAlmanacWidget", "Summer", "season"));
#if QT_CONFIG(tooltip)
        buttonJuneSolsticePrevious->setToolTip(QCoreApplication::translate("astroCalcAlmanacWidget", "Previous June Solstice", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        buttonJuneSolsticeCurrent->setToolTip(QCoreApplication::translate("astroCalcAlmanacWidget", "Current June Solstice", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        buttonJuneSolsticeNext->setToolTip(QCoreApplication::translate("astroCalcAlmanacWidget", "Next June Solstice", nullptr));
#endif // QT_CONFIG(tooltip)
        labelDayDuration->setText(QString());
        labelNight->setText(QCoreApplication::translate("astroCalcAlmanacWidget", "Astronomical Night", nullptr));
        labelSeptemberEquinox->setText(QCoreApplication::translate("astroCalcAlmanacWidget", "Fall", "season"));
#if QT_CONFIG(tooltip)
        buttonSeptemberEquinoxPrevious->setToolTip(QCoreApplication::translate("astroCalcAlmanacWidget", "Previous September Equinox", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        buttonSeptemberEquinoxCurrent->setToolTip(QCoreApplication::translate("astroCalcAlmanacWidget", "Current September Equinox", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        buttonSeptemberEquinoxNext->setToolTip(QCoreApplication::translate("astroCalcAlmanacWidget", "Next September Equinox", nullptr));
#endif // QT_CONFIG(tooltip)
        labelCivilTwilightBegin->setText(QString());
        labelSunset->setText(QString());
#if QT_CONFIG(tooltip)
        buttonAfterSunset->setToolTip(QCoreApplication::translate("astroCalcAlmanacWidget", "Defined minutes after today's sunset", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonAfterSunset->setText(QString());
        labelMarchEquinox->setText(QCoreApplication::translate("astroCalcAlmanacWidget", "Spring", "season"));
#if QT_CONFIG(tooltip)
        buttonMarchEquinoxPrevious->setToolTip(QCoreApplication::translate("astroCalcAlmanacWidget", "Previous March Equinox", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        buttonMarchEquinoxCurrent->setToolTip(QCoreApplication::translate("astroCalcAlmanacWidget", "Current March Equinox", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        buttonMarchEquinoxNext->setToolTip(QCoreApplication::translate("astroCalcAlmanacWidget", "Next March Equinox", nullptr));
#endif // QT_CONFIG(tooltip)
        labelCustomSunrise->setText(QString());
        labelCustomMoon->setText(QCoreApplication::translate("astroCalcAlmanacWidget", "Moon", nullptr));
        labelCivilTwilightDuration->setText(QString());
        labelSet->setText(QCoreApplication::translate("astroCalcAlmanacWidget", "Set", nullptr));
        labelDurationColumn2->setText(QCoreApplication::translate("astroCalcAlmanacWidget", "Duration", nullptr));
        labelSeasons->setText(QCoreApplication::translate("astroCalcAlmanacWidget", "Seasons", nullptr));
#if QT_CONFIG(tooltip)
        buttonNauticalDusk->setToolTip(QCoreApplication::translate("astroCalcAlmanacWidget", "Evening nautical twilight", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonNauticalDusk->setText(QString());
#if QT_CONFIG(tooltip)
        buttonBeforeSunrise->setToolTip(QCoreApplication::translate("astroCalcAlmanacWidget", "Defined minutes before today's sunrise", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonBeforeSunrise->setText(QString());
        labelBeforeSunrise->setText(QString());
#if QT_CONFIG(tooltip)
        buttonYesterday->setToolTip(QCoreApplication::translate("astroCalcAlmanacWidget", "Yesterday", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonYesterday->setText(QString());
        labelToday->setText(QCoreApplication::translate("astroCalcAlmanacWidget", "Today", nullptr));
#if QT_CONFIG(tooltip)
        buttonTomorrow->setToolTip(QCoreApplication::translate("astroCalcAlmanacWidget", "Tomorrow", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonTomorrow->setText(QString());
        labelAstronomicalTwilight->setText(QCoreApplication::translate("astroCalcAlmanacWidget", "Astronomical", "twilight"));
        labelAstronomicalTwilightBegin->setText(QString());
        labelLT1->setText(QCoreApplication::translate("astroCalcAlmanacWidget", "Local Time", nullptr));
        labelJD2->setText(QCoreApplication::translate("astroCalcAlmanacWidget", "Julian Day", nullptr));
        labelDusk->setText(QCoreApplication::translate("astroCalcAlmanacWidget", "Dusk", nullptr));
#if QT_CONFIG(tooltip)
        buttonCivilDawn->setToolTip(QCoreApplication::translate("astroCalcAlmanacWidget", "Morning civil twilight", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonCivilDawn->setText(QString());
        tabWidget->setTabText(tabWidget->indexOf(tabSpecificTime), QCoreApplication::translate("astroCalcAlmanacWidget", "Specific Time", nullptr));
        (void)astroCalcAlmanacWidget;
    } // retranslateUi

};

namespace Ui {
    class astroCalcAlmanacWidget: public Ui_astroCalcAlmanacWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_ASTROCALCALMANACWIDGET_H
