/********************************************************************************
** Form generated from reading UI file 'orbitColorsDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_ORBITCOLORSDIALOG_H
#define UI_ORBITCOLORSDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "Dialog.hpp"
#include "gui/ColorButton.hpp"

QT_BEGIN_NAMESPACE

class Ui_ConfigureOrbitColorsDialogForm
{
public:
    QVBoxLayout *verticalLayout;
    TitleBar *titleBar;
    QGroupBox *ocsGroupBox;
    QVBoxLayout *verticalLayout_2;
    QRadioButton *oneColorRadioButton;
    QRadioButton *groupsRadioButton;
    QRadioButton *majorPlanetsRadioButton;
    QRadioButton *majorPlanetsAndMinorTypeRadioButton;
    QGroupBox *colorsGroupBox;
    QGridLayout *gridLayout_18;
    QLabel *labelGroupsMajorPlanetsColor;
    QLabel *labelGroupsOCOColor;
    QLabel *labelGroupsMinorPlanetsColor;
    QLabel *labelGroupsMoonsColor;
    QLabel *labelMPNeptune;
    QLabel *labelMPJupiter;
    QLabel *labelMPEarth;
    QLabel *labelGroupsDwarfPlanetsColor;
    QLabel *labelMPSaturn;
    QLabel *labelGroupsCometsColor;
    QLabel *labelGroupsPlutinosColor;
    QLabel *labelGenericOrbitColor;
    QLabel *labelGroupsInterstellarColor;
    QLabel *labelGroupsCubewanosColor;
    QLabel *labelMPVenus;
    QLabel *labelGroupsSednoidsColor;
    QLabel *labelGroupsSDOColor;
    QLabel *labelMPUranus;
    QLabel *labelMPMars;
    QLabel *labelMPMercury;
    ColorButton *colorGenericOrbits;
    ColorButton *colorGroupsMajorPlanetsOrbits;
    ColorButton *colorGroupsMinorPlanetsOrbits;
    ColorButton *colorGroupsDwarfPlanetsOrbits;
    ColorButton *colorGroupsPlutinosOrbits;
    ColorButton *colorGroupsSDOOrbits;
    ColorButton *colorGroupsInterstellarOrbits;
    ColorButton *colorGroupsMoonsOrbits;
    ColorButton *colorGroupsCometsOrbits;
    ColorButton *colorGroupsCubewanosOrbits;
    ColorButton *colorGroupsSednoidsOrbits;
    ColorButton *colorGroupsOCOOrbits;
    ColorButton *colorMPMercuryOrbit;
    ColorButton *colorMPEarthOrbit;
    ColorButton *colorMPJupiterOrbit;
    ColorButton *colorMPVenusOrbit;
    ColorButton *colorMPMarsOrbit;
    ColorButton *colorMPSaturnOrbit;
    ColorButton *colorMPUranusOrbit;
    ColorButton *colorMPNeptuneOrbit;
    QFrame *line1;
    QFrame *line2;

    void setupUi(QWidget *ConfigureOrbitColorsDialogForm)
    {
        if (ConfigureOrbitColorsDialogForm->objectName().isEmpty())
            ConfigureOrbitColorsDialogForm->setObjectName("ConfigureOrbitColorsDialogForm");
        ConfigureOrbitColorsDialogForm->resize(470, 482);
        verticalLayout = new QVBoxLayout(ConfigureOrbitColorsDialogForm);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        titleBar = new TitleBar(ConfigureOrbitColorsDialogForm);
        titleBar->setObjectName("titleBar");

        verticalLayout->addWidget(titleBar);

        ocsGroupBox = new QGroupBox(ConfigureOrbitColorsDialogForm);
        ocsGroupBox->setObjectName("ocsGroupBox");
        verticalLayout_2 = new QVBoxLayout(ocsGroupBox);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        oneColorRadioButton = new QRadioButton(ocsGroupBox);
        oneColorRadioButton->setObjectName("oneColorRadioButton");

        verticalLayout_2->addWidget(oneColorRadioButton);

        groupsRadioButton = new QRadioButton(ocsGroupBox);
        groupsRadioButton->setObjectName("groupsRadioButton");

        verticalLayout_2->addWidget(groupsRadioButton);

        majorPlanetsRadioButton = new QRadioButton(ocsGroupBox);
        majorPlanetsRadioButton->setObjectName("majorPlanetsRadioButton");

        verticalLayout_2->addWidget(majorPlanetsRadioButton);

        majorPlanetsAndMinorTypeRadioButton = new QRadioButton(ocsGroupBox);
        majorPlanetsAndMinorTypeRadioButton->setObjectName("majorPlanetsAndMinorTypeRadioButton");

        verticalLayout_2->addWidget(majorPlanetsAndMinorTypeRadioButton);


        verticalLayout->addWidget(ocsGroupBox);

        colorsGroupBox = new QGroupBox(ConfigureOrbitColorsDialogForm);
        colorsGroupBox->setObjectName("colorsGroupBox");
        colorsGroupBox->setBaseSize(QSize(0, 2));
        gridLayout_18 = new QGridLayout(colorsGroupBox);
        gridLayout_18->setObjectName("gridLayout_18");
        gridLayout_18->setContentsMargins(0, 0, 0, 0);
        labelGroupsMajorPlanetsColor = new QLabel(colorsGroupBox);
        labelGroupsMajorPlanetsColor->setObjectName("labelGroupsMajorPlanetsColor");

        gridLayout_18->addWidget(labelGroupsMajorPlanetsColor, 2, 1, 1, 1);

        labelGroupsOCOColor = new QLabel(colorsGroupBox);
        labelGroupsOCOColor->setObjectName("labelGroupsOCOColor");

        gridLayout_18->addWidget(labelGroupsOCOColor, 6, 3, 1, 1);

        labelGroupsMinorPlanetsColor = new QLabel(colorsGroupBox);
        labelGroupsMinorPlanetsColor->setObjectName("labelGroupsMinorPlanetsColor");

        gridLayout_18->addWidget(labelGroupsMinorPlanetsColor, 3, 1, 1, 1);

        labelGroupsMoonsColor = new QLabel(colorsGroupBox);
        labelGroupsMoonsColor->setObjectName("labelGroupsMoonsColor");

        gridLayout_18->addWidget(labelGroupsMoonsColor, 2, 3, 1, 1);

        labelMPNeptune = new QLabel(colorsGroupBox);
        labelMPNeptune->setObjectName("labelMPNeptune");

        gridLayout_18->addWidget(labelMPNeptune, 12, 3, 1, 1);

        labelMPJupiter = new QLabel(colorsGroupBox);
        labelMPJupiter->setObjectName("labelMPJupiter");

        gridLayout_18->addWidget(labelMPJupiter, 11, 1, 1, 1);

        labelMPEarth = new QLabel(colorsGroupBox);
        labelMPEarth->setObjectName("labelMPEarth");

        gridLayout_18->addWidget(labelMPEarth, 10, 1, 1, 1);

        labelGroupsDwarfPlanetsColor = new QLabel(colorsGroupBox);
        labelGroupsDwarfPlanetsColor->setObjectName("labelGroupsDwarfPlanetsColor");

        gridLayout_18->addWidget(labelGroupsDwarfPlanetsColor, 4, 1, 1, 1);

        labelMPSaturn = new QLabel(colorsGroupBox);
        labelMPSaturn->setObjectName("labelMPSaturn");

        gridLayout_18->addWidget(labelMPSaturn, 11, 3, 1, 1);

        labelGroupsCometsColor = new QLabel(colorsGroupBox);
        labelGroupsCometsColor->setObjectName("labelGroupsCometsColor");

        gridLayout_18->addWidget(labelGroupsCometsColor, 3, 3, 1, 1);

        labelGroupsPlutinosColor = new QLabel(colorsGroupBox);
        labelGroupsPlutinosColor->setObjectName("labelGroupsPlutinosColor");

        gridLayout_18->addWidget(labelGroupsPlutinosColor, 5, 1, 1, 1);

        labelGenericOrbitColor = new QLabel(colorsGroupBox);
        labelGenericOrbitColor->setObjectName("labelGenericOrbitColor");

        gridLayout_18->addWidget(labelGenericOrbitColor, 0, 1, 1, 1);

        labelGroupsInterstellarColor = new QLabel(colorsGroupBox);
        labelGroupsInterstellarColor->setObjectName("labelGroupsInterstellarColor");

        gridLayout_18->addWidget(labelGroupsInterstellarColor, 7, 1, 1, 1);

        labelGroupsCubewanosColor = new QLabel(colorsGroupBox);
        labelGroupsCubewanosColor->setObjectName("labelGroupsCubewanosColor");

        gridLayout_18->addWidget(labelGroupsCubewanosColor, 4, 3, 1, 1);

        labelMPVenus = new QLabel(colorsGroupBox);
        labelMPVenus->setObjectName("labelMPVenus");

        gridLayout_18->addWidget(labelMPVenus, 9, 3, 1, 1);

        labelGroupsSednoidsColor = new QLabel(colorsGroupBox);
        labelGroupsSednoidsColor->setObjectName("labelGroupsSednoidsColor");

        gridLayout_18->addWidget(labelGroupsSednoidsColor, 5, 3, 1, 1);

        labelGroupsSDOColor = new QLabel(colorsGroupBox);
        labelGroupsSDOColor->setObjectName("labelGroupsSDOColor");

        gridLayout_18->addWidget(labelGroupsSDOColor, 6, 1, 1, 1);

        labelMPUranus = new QLabel(colorsGroupBox);
        labelMPUranus->setObjectName("labelMPUranus");

        gridLayout_18->addWidget(labelMPUranus, 12, 1, 1, 1);

        labelMPMars = new QLabel(colorsGroupBox);
        labelMPMars->setObjectName("labelMPMars");

        gridLayout_18->addWidget(labelMPMars, 10, 3, 1, 1);

        labelMPMercury = new QLabel(colorsGroupBox);
        labelMPMercury->setObjectName("labelMPMercury");

        gridLayout_18->addWidget(labelMPMercury, 9, 1, 1, 1);

        colorGenericOrbits = new ColorButton(colorsGroupBox);
        colorGenericOrbits->setObjectName("colorGenericOrbits");

        gridLayout_18->addWidget(colorGenericOrbits, 0, 0, 1, 1);

        colorGroupsMajorPlanetsOrbits = new ColorButton(colorsGroupBox);
        colorGroupsMajorPlanetsOrbits->setObjectName("colorGroupsMajorPlanetsOrbits");

        gridLayout_18->addWidget(colorGroupsMajorPlanetsOrbits, 2, 0, 1, 1);

        colorGroupsMinorPlanetsOrbits = new ColorButton(colorsGroupBox);
        colorGroupsMinorPlanetsOrbits->setObjectName("colorGroupsMinorPlanetsOrbits");

        gridLayout_18->addWidget(colorGroupsMinorPlanetsOrbits, 3, 0, 1, 1);

        colorGroupsDwarfPlanetsOrbits = new ColorButton(colorsGroupBox);
        colorGroupsDwarfPlanetsOrbits->setObjectName("colorGroupsDwarfPlanetsOrbits");

        gridLayout_18->addWidget(colorGroupsDwarfPlanetsOrbits, 4, 0, 1, 1);

        colorGroupsPlutinosOrbits = new ColorButton(colorsGroupBox);
        colorGroupsPlutinosOrbits->setObjectName("colorGroupsPlutinosOrbits");

        gridLayout_18->addWidget(colorGroupsPlutinosOrbits, 5, 0, 1, 1);

        colorGroupsSDOOrbits = new ColorButton(colorsGroupBox);
        colorGroupsSDOOrbits->setObjectName("colorGroupsSDOOrbits");

        gridLayout_18->addWidget(colorGroupsSDOOrbits, 6, 0, 1, 1);

        colorGroupsInterstellarOrbits = new ColorButton(colorsGroupBox);
        colorGroupsInterstellarOrbits->setObjectName("colorGroupsInterstellarOrbits");

        gridLayout_18->addWidget(colorGroupsInterstellarOrbits, 7, 0, 1, 1);

        colorGroupsMoonsOrbits = new ColorButton(colorsGroupBox);
        colorGroupsMoonsOrbits->setObjectName("colorGroupsMoonsOrbits");

        gridLayout_18->addWidget(colorGroupsMoonsOrbits, 2, 2, 1, 1);

        colorGroupsCometsOrbits = new ColorButton(colorsGroupBox);
        colorGroupsCometsOrbits->setObjectName("colorGroupsCometsOrbits");

        gridLayout_18->addWidget(colorGroupsCometsOrbits, 3, 2, 1, 1);

        colorGroupsCubewanosOrbits = new ColorButton(colorsGroupBox);
        colorGroupsCubewanosOrbits->setObjectName("colorGroupsCubewanosOrbits");

        gridLayout_18->addWidget(colorGroupsCubewanosOrbits, 4, 2, 1, 1);

        colorGroupsSednoidsOrbits = new ColorButton(colorsGroupBox);
        colorGroupsSednoidsOrbits->setObjectName("colorGroupsSednoidsOrbits");

        gridLayout_18->addWidget(colorGroupsSednoidsOrbits, 5, 2, 1, 1);

        colorGroupsOCOOrbits = new ColorButton(colorsGroupBox);
        colorGroupsOCOOrbits->setObjectName("colorGroupsOCOOrbits");

        gridLayout_18->addWidget(colorGroupsOCOOrbits, 6, 2, 1, 1);

        colorMPMercuryOrbit = new ColorButton(colorsGroupBox);
        colorMPMercuryOrbit->setObjectName("colorMPMercuryOrbit");

        gridLayout_18->addWidget(colorMPMercuryOrbit, 9, 0, 1, 1);

        colorMPEarthOrbit = new ColorButton(colorsGroupBox);
        colorMPEarthOrbit->setObjectName("colorMPEarthOrbit");

        gridLayout_18->addWidget(colorMPEarthOrbit, 10, 0, 1, 1);

        colorMPJupiterOrbit = new ColorButton(colorsGroupBox);
        colorMPJupiterOrbit->setObjectName("colorMPJupiterOrbit");

        gridLayout_18->addWidget(colorMPJupiterOrbit, 11, 0, 1, 1);

        colorMPVenusOrbit = new ColorButton(colorsGroupBox);
        colorMPVenusOrbit->setObjectName("colorMPVenusOrbit");

        gridLayout_18->addWidget(colorMPVenusOrbit, 9, 2, 1, 1);

        colorMPMarsOrbit = new ColorButton(colorsGroupBox);
        colorMPMarsOrbit->setObjectName("colorMPMarsOrbit");

        gridLayout_18->addWidget(colorMPMarsOrbit, 10, 2, 1, 1);

        colorMPSaturnOrbit = new ColorButton(colorsGroupBox);
        colorMPSaturnOrbit->setObjectName("colorMPSaturnOrbit");

        gridLayout_18->addWidget(colorMPSaturnOrbit, 11, 2, 1, 1);

        colorMPUranusOrbit = new ColorButton(colorsGroupBox);
        colorMPUranusOrbit->setObjectName("colorMPUranusOrbit");

        gridLayout_18->addWidget(colorMPUranusOrbit, 12, 0, 1, 1);

        colorMPNeptuneOrbit = new ColorButton(colorsGroupBox);
        colorMPNeptuneOrbit->setObjectName("colorMPNeptuneOrbit");

        gridLayout_18->addWidget(colorMPNeptuneOrbit, 12, 2, 1, 1);

        line1 = new QFrame(colorsGroupBox);
        line1->setObjectName("line1");
        line1->setMinimumSize(QSize(0, 1));
        line1->setMaximumSize(QSize(16777215, 1));
        line1->setFrameShadow(QFrame::Plain);
        line1->setFrameShape(QFrame::Shape::HLine);

        gridLayout_18->addWidget(line1, 1, 0, 1, 5);

        line2 = new QFrame(colorsGroupBox);
        line2->setObjectName("line2");
        line2->setMinimumSize(QSize(0, 1));
        line2->setMaximumSize(QSize(16777215, 1));
        line2->setFrameShadow(QFrame::Plain);
        line2->setFrameShape(QFrame::Shape::HLine);

        gridLayout_18->addWidget(line2, 8, 0, 1, 5);


        verticalLayout->addWidget(colorsGroupBox);


        retranslateUi(ConfigureOrbitColorsDialogForm);

        QMetaObject::connectSlotsByName(ConfigureOrbitColorsDialogForm);
    } // setupUi

    void retranslateUi(QWidget *ConfigureOrbitColorsDialogForm)
    {
        titleBar->setTitle(QCoreApplication::translate("ConfigureOrbitColorsDialogForm", "Color settings", nullptr));
        ocsGroupBox->setTitle(QCoreApplication::translate("ConfigureOrbitColorsDialogForm", "Style of orbit colors", nullptr));
        oneColorRadioButton->setText(QCoreApplication::translate("ConfigureOrbitColorsDialogForm", "One color for all orbits", nullptr));
        groupsRadioButton->setText(QCoreApplication::translate("ConfigureOrbitColorsDialogForm", "Separate colors for orbits by object type", nullptr));
        majorPlanetsRadioButton->setText(QCoreApplication::translate("ConfigureOrbitColorsDialogForm", "Separate colors for orbits of major planets only", nullptr));
        majorPlanetsAndMinorTypeRadioButton->setText(QCoreApplication::translate("ConfigureOrbitColorsDialogForm", "Separate colors for orbits of major planets and by minor body object type", nullptr));
        colorsGroupBox->setTitle(QCoreApplication::translate("ConfigureOrbitColorsDialogForm", "Colors of orbits of Solar system bodies", nullptr));
        labelGroupsMajorPlanetsColor->setText(QCoreApplication::translate("ConfigureOrbitColorsDialogForm", "Orbits of major planets", nullptr));
        labelGroupsOCOColor->setText(QCoreApplication::translate("ConfigureOrbitColorsDialogForm", "Orbits of Oort cloud objects", nullptr));
#if QT_CONFIG(tooltip)
        labelGroupsMinorPlanetsColor->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
        labelGroupsMinorPlanetsColor->setText(QCoreApplication::translate("ConfigureOrbitColorsDialogForm", "Orbits of minor planets", nullptr));
        labelGroupsMoonsColor->setText(QCoreApplication::translate("ConfigureOrbitColorsDialogForm", "Orbits of moons of planets", nullptr));
#if QT_CONFIG(tooltip)
        labelMPNeptune->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
        labelMPNeptune->setText(QCoreApplication::translate("ConfigureOrbitColorsDialogForm", "Orbit of Neptune", nullptr));
        labelMPJupiter->setText(QCoreApplication::translate("ConfigureOrbitColorsDialogForm", "Orbit of Jupiter", nullptr));
        labelMPEarth->setText(QCoreApplication::translate("ConfigureOrbitColorsDialogForm", "Orbit of Earth", nullptr));
        labelGroupsDwarfPlanetsColor->setText(QCoreApplication::translate("ConfigureOrbitColorsDialogForm", "Orbits of dwarf planets", nullptr));
#if QT_CONFIG(tooltip)
        labelMPSaturn->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
        labelMPSaturn->setText(QCoreApplication::translate("ConfigureOrbitColorsDialogForm", "Orbit of Saturn", nullptr));
        labelGroupsCometsColor->setText(QCoreApplication::translate("ConfigureOrbitColorsDialogForm", "Orbits of comets", nullptr));
        labelGroupsPlutinosColor->setText(QCoreApplication::translate("ConfigureOrbitColorsDialogForm", "Orbits of plutinos", nullptr));
#if QT_CONFIG(tooltip)
        labelGenericOrbitColor->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
        labelGenericOrbitColor->setText(QCoreApplication::translate("ConfigureOrbitColorsDialogForm", "Generic orbit", nullptr));
        labelGroupsInterstellarColor->setText(QCoreApplication::translate("ConfigureOrbitColorsDialogForm", "Orbits (trajectories) of interstellar objects", nullptr));
        labelGroupsCubewanosColor->setText(QCoreApplication::translate("ConfigureOrbitColorsDialogForm", "Orbits of cubewanos", nullptr));
        labelMPVenus->setText(QCoreApplication::translate("ConfigureOrbitColorsDialogForm", "Orbit of Venus", nullptr));
        labelGroupsSednoidsColor->setText(QCoreApplication::translate("ConfigureOrbitColorsDialogForm", "Orbits of sednoids", nullptr));
        labelGroupsSDOColor->setText(QCoreApplication::translate("ConfigureOrbitColorsDialogForm", "Orbits of scattered disk objects", nullptr));
        labelMPUranus->setText(QCoreApplication::translate("ConfigureOrbitColorsDialogForm", "Orbit of Uranus", nullptr));
        labelMPMars->setText(QCoreApplication::translate("ConfigureOrbitColorsDialogForm", "Orbit of Mars", nullptr));
        labelMPMercury->setText(QCoreApplication::translate("ConfigureOrbitColorsDialogForm", "Orbit of Mercury", nullptr));
#if QT_CONFIG(tooltip)
        colorGenericOrbits->setToolTip(QCoreApplication::translate("ConfigureOrbitColorsDialogForm", "Color of generic orbit", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        colorGroupsMajorPlanetsOrbits->setToolTip(QCoreApplication::translate("ConfigureOrbitColorsDialogForm", "Color of major planet orbits", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        colorGroupsMinorPlanetsOrbits->setToolTip(QCoreApplication::translate("ConfigureOrbitColorsDialogForm", "Color of minor planet orbits", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        colorGroupsDwarfPlanetsOrbits->setToolTip(QCoreApplication::translate("ConfigureOrbitColorsDialogForm", "Color of dwarf planet orbits", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        colorGroupsPlutinosOrbits->setToolTip(QCoreApplication::translate("ConfigureOrbitColorsDialogForm", "Color of plutino orbits", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        colorGroupsSDOOrbits->setToolTip(QCoreApplication::translate("ConfigureOrbitColorsDialogForm", "Color of scattered disk object orbits", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        colorGroupsInterstellarOrbits->setToolTip(QCoreApplication::translate("ConfigureOrbitColorsDialogForm", "Color of interstellar objects' orbits", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        colorGroupsMoonsOrbits->setToolTip(QCoreApplication::translate("ConfigureOrbitColorsDialogForm", "Color of planet moon orbits", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        colorGroupsCometsOrbits->setToolTip(QCoreApplication::translate("ConfigureOrbitColorsDialogForm", "Color of comet orbits", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        colorGroupsCubewanosOrbits->setToolTip(QCoreApplication::translate("ConfigureOrbitColorsDialogForm", "Color of cubewano orbits", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        colorGroupsSednoidsOrbits->setToolTip(QCoreApplication::translate("ConfigureOrbitColorsDialogForm", "Color of sednoid orbits", nullptr));
#endif // QT_CONFIG(tooltip)
        colorGroupsSednoidsOrbits->setText(QString());
#if QT_CONFIG(tooltip)
        colorGroupsOCOOrbits->setToolTip(QCoreApplication::translate("ConfigureOrbitColorsDialogForm", "Color of Oort cloud object orbits", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        colorMPMercuryOrbit->setToolTip(QCoreApplication::translate("ConfigureOrbitColorsDialogForm", "Color of Mercury orbit", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        colorMPEarthOrbit->setToolTip(QCoreApplication::translate("ConfigureOrbitColorsDialogForm", "Color of Earth orbit", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        colorMPJupiterOrbit->setToolTip(QCoreApplication::translate("ConfigureOrbitColorsDialogForm", "Color of Jupiter orbit", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        colorMPVenusOrbit->setToolTip(QCoreApplication::translate("ConfigureOrbitColorsDialogForm", "Color of Venus orbit", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        colorMPMarsOrbit->setToolTip(QCoreApplication::translate("ConfigureOrbitColorsDialogForm", "Color of Mars orbit", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        colorMPSaturnOrbit->setToolTip(QCoreApplication::translate("ConfigureOrbitColorsDialogForm", "Color of Saturn orbit", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        colorMPUranusOrbit->setToolTip(QCoreApplication::translate("ConfigureOrbitColorsDialogForm", "Color of Uranus orbit", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        colorMPNeptuneOrbit->setToolTip(QCoreApplication::translate("ConfigureOrbitColorsDialogForm", "Color of Neptune orbit", nullptr));
#endif // QT_CONFIG(tooltip)
        (void)ConfigureOrbitColorsDialogForm;
    } // retranslateUi

};

namespace Ui {
    class ConfigureOrbitColorsDialogForm: public Ui_ConfigureOrbitColorsDialogForm {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_ORBITCOLORSDIALOG_H
