/****************************************************************************
** Meta object code from reading C++ file 'MeteorShowersMgr.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../stellarium/plugins/MeteorShowers/src/MeteorShowersMgr.hpp"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>
#include <QtCore/qplugin.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'MeteorShowersMgr.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16MeteorShowersMgrE_t {};
} // unnamed namespace

template <> constexpr inline auto MeteorShowersMgr::qt_create_metaobjectdata<qt_meta_tag_ZN16MeteorShowersMgrE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MeteorShowersMgr",
        "updateStateChanged",
        "",
        "MeteorShowersMgr::UpdateState",
        "state",
        "jsonUpdateComplete",
        "enablePluginChanged",
        "b",
        "enableLabelsChanged",
        "colorARGChanged",
        "Vec3f",
        "c",
        "colorARCChanged",
        "colorIRChanged",
        "setEnableAtStartup",
        "getEnableAtStartup",
        "setShowEnableButton",
        "show",
        "getShowEnableButton",
        "setShowSearchButton",
        "getShowSearchButton",
        "setEnableMarker",
        "getEnableMarker",
        "setActiveRadiantOnly",
        "getActiveRadiantOnly",
        "setEnableLabels",
        "getEnableLabels",
        "setFontSize",
        "pixelSize",
        "getFontSize",
        "setUpdateFrequencyHours",
        "hours",
        "getUpdateFrequencyHours",
        "setEnableAutoUpdates",
        "getEnableAutoUpdates",
        "setColorARG",
        "rgb",
        "getColorARG",
        "setColorARC",
        "getColorARC",
        "setColorIR",
        "getColorIR",
        "updateCatalog",
        "restoreDefaultSettings",
        "displayMessage",
        "message",
        "hexColor",
        "checkForUpdates",
        "updateDownloadProgress",
        "bytesReceived",
        "bytesTotal",
        "downloadComplete",
        "QNetworkReply*",
        "reply",
        "locationChanged",
        "StelLocation",
        "location",
        "saveSettings",
        "enablePlugin",
        "enableLabels",
        "colorARG",
        "colorARC",
        "colorIR"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'updateStateChanged'
        QtMocHelpers::SignalData<void(MeteorShowersMgr::UpdateState)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'jsonUpdateComplete'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'enablePluginChanged'
        QtMocHelpers::SignalData<void(bool)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 7 },
        }}),
        // Signal 'enableLabelsChanged'
        QtMocHelpers::SignalData<void(bool)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 7 },
        }}),
        // Signal 'colorARGChanged'
        QtMocHelpers::SignalData<void(Vec3f)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Signal 'colorARCChanged'
        QtMocHelpers::SignalData<void(Vec3f)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Signal 'colorIRChanged'
        QtMocHelpers::SignalData<void(Vec3f)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Slot 'setEnableAtStartup'
        QtMocHelpers::SlotData<void(const bool &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 7 },
        }}),
        // Slot 'getEnableAtStartup'
        QtMocHelpers::SlotData<bool()>(15, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setShowEnableButton'
        QtMocHelpers::SlotData<void(const bool &)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 17 },
        }}),
        // Slot 'getShowEnableButton'
        QtMocHelpers::SlotData<bool()>(18, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setShowSearchButton'
        QtMocHelpers::SlotData<void(const bool &)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 17 },
        }}),
        // Slot 'getShowSearchButton'
        QtMocHelpers::SlotData<bool()>(20, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setEnableMarker'
        QtMocHelpers::SlotData<void(const bool &)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 7 },
        }}),
        // Slot 'getEnableMarker'
        QtMocHelpers::SlotData<bool()>(22, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setActiveRadiantOnly'
        QtMocHelpers::SlotData<void(const bool &)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 7 },
        }}),
        // Slot 'getActiveRadiantOnly'
        QtMocHelpers::SlotData<bool()>(24, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setEnableLabels'
        QtMocHelpers::SlotData<void(const bool &)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 7 },
        }}),
        // Slot 'getEnableLabels'
        QtMocHelpers::SlotData<bool()>(26, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFontSize'
        QtMocHelpers::SlotData<void(int)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 28 },
        }}),
        // Slot 'getFontSize'
        QtMocHelpers::SlotData<int()>(29, 2, QMC::AccessPublic, QMetaType::Int),
        // Slot 'setUpdateFrequencyHours'
        QtMocHelpers::SlotData<void(const int &)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 31 },
        }}),
        // Slot 'getUpdateFrequencyHours'
        QtMocHelpers::SlotData<int()>(32, 2, QMC::AccessPublic, QMetaType::Int),
        // Slot 'setEnableAutoUpdates'
        QtMocHelpers::SlotData<void(const bool &)>(33, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 7 },
        }}),
        // Slot 'getEnableAutoUpdates'
        QtMocHelpers::SlotData<bool()>(34, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setColorARG'
        QtMocHelpers::SlotData<void(const Vec3f &)>(35, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 36 },
        }}),
        // Slot 'getColorARG'
        QtMocHelpers::SlotData<Vec3f()>(37, 2, QMC::AccessPublic, 0x80000000 | 10),
        // Slot 'setColorARC'
        QtMocHelpers::SlotData<void(const Vec3f &)>(38, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 36 },
        }}),
        // Slot 'getColorARC'
        QtMocHelpers::SlotData<Vec3f()>(39, 2, QMC::AccessPublic, 0x80000000 | 10),
        // Slot 'setColorIR'
        QtMocHelpers::SlotData<void(const Vec3f &)>(40, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 36 },
        }}),
        // Slot 'getColorIR'
        QtMocHelpers::SlotData<Vec3f()>(41, 2, QMC::AccessPublic, 0x80000000 | 10),
        // Slot 'updateCatalog'
        QtMocHelpers::SlotData<void()>(42, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'restoreDefaultSettings'
        QtMocHelpers::SlotData<void()>(43, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'displayMessage'
        QtMocHelpers::SlotData<void(const QString &, const QString &)>(44, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 45 }, { QMetaType::QString, 46 },
        }}),
        // Slot 'displayMessage'
        QtMocHelpers::SlotData<void(const QString &)>(44, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 45 },
        }}),
        // Slot 'checkForUpdates'
        QtMocHelpers::SlotData<void()>(47, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateDownloadProgress'
        QtMocHelpers::SlotData<void(qint64, qint64)>(48, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::LongLong, 49 }, { QMetaType::LongLong, 50 },
        }}),
        // Slot 'downloadComplete'
        QtMocHelpers::SlotData<void(QNetworkReply *)>(51, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 52, 53 },
        }}),
        // Slot 'locationChanged'
        QtMocHelpers::SlotData<void(const StelLocation &)>(54, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 55, 56 },
        }}),
        // Slot 'saveSettings'
        QtMocHelpers::SlotData<void()>(57, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'enablePlugin'
        QtMocHelpers::PropertyData<bool>(58, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 2),
        // property 'enableLabels'
        QtMocHelpers::PropertyData<bool>(59, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'colorARG'
        QtMocHelpers::PropertyData<Vec3f>(60, 0x80000000 | 10, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 4),
        // property 'colorARC'
        QtMocHelpers::PropertyData<Vec3f>(61, 0x80000000 | 10, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 5),
        // property 'colorIR'
        QtMocHelpers::PropertyData<Vec3f>(62, 0x80000000 | 10, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 6),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<MeteorShowersMgr, qt_meta_tag_ZN16MeteorShowersMgrE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MeteorShowersMgr::staticMetaObject = { {
    QMetaObject::SuperData::link<StelModule::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16MeteorShowersMgrE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16MeteorShowersMgrE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16MeteorShowersMgrE_t>.metaTypes,
    nullptr
} };

void MeteorShowersMgr::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MeteorShowersMgr *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->updateStateChanged((*reinterpret_cast<std::add_pointer_t<MeteorShowersMgr::UpdateState>>(_a[1]))); break;
        case 1: _t->jsonUpdateComplete(); break;
        case 2: _t->enablePluginChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->enableLabelsChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->colorARGChanged((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1]))); break;
        case 5: _t->colorARCChanged((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1]))); break;
        case 6: _t->colorIRChanged((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1]))); break;
        case 7: _t->setEnableAtStartup((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: { bool _r = _t->getEnableAtStartup();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 9: _t->setShowEnableButton((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 10: { bool _r = _t->getShowEnableButton();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 11: _t->setShowSearchButton((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 12: { bool _r = _t->getShowSearchButton();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 13: _t->setEnableMarker((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 14: { bool _r = _t->getEnableMarker();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 15: _t->setActiveRadiantOnly((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 16: { bool _r = _t->getActiveRadiantOnly();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 17: _t->setEnableLabels((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 18: { bool _r = _t->getEnableLabels();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 19: _t->setFontSize((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 20: { int _r = _t->getFontSize();
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 21: _t->setUpdateFrequencyHours((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 22: { int _r = _t->getUpdateFrequencyHours();
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 23: _t->setEnableAutoUpdates((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 24: { bool _r = _t->getEnableAutoUpdates();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 25: _t->setColorARG((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1]))); break;
        case 26: { Vec3f _r = _t->getColorARG();
            if (_a[0]) *reinterpret_cast<Vec3f*>(_a[0]) = std::move(_r); }  break;
        case 27: _t->setColorARC((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1]))); break;
        case 28: { Vec3f _r = _t->getColorARC();
            if (_a[0]) *reinterpret_cast<Vec3f*>(_a[0]) = std::move(_r); }  break;
        case 29: _t->setColorIR((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1]))); break;
        case 30: { Vec3f _r = _t->getColorIR();
            if (_a[0]) *reinterpret_cast<Vec3f*>(_a[0]) = std::move(_r); }  break;
        case 31: _t->updateCatalog(); break;
        case 32: _t->restoreDefaultSettings(); break;
        case 33: _t->displayMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 34: _t->displayMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 35: _t->checkForUpdates(); break;
        case 36: _t->updateDownloadProgress((*reinterpret_cast<std::add_pointer_t<qint64>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qint64>>(_a[2]))); break;
        case 37: _t->downloadComplete((*reinterpret_cast<std::add_pointer_t<QNetworkReply*>>(_a[1]))); break;
        case 38: _t->locationChanged((*reinterpret_cast<std::add_pointer_t<StelLocation>>(_a[1]))); break;
        case 39: _t->saveSettings(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 25:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 27:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 29:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 37:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QNetworkReply* >(); break;
            }
            break;
        case 38:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< StelLocation >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (MeteorShowersMgr::*)(MeteorShowersMgr::UpdateState )>(_a, &MeteorShowersMgr::updateStateChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (MeteorShowersMgr::*)()>(_a, &MeteorShowersMgr::jsonUpdateComplete, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (MeteorShowersMgr::*)(bool )>(_a, &MeteorShowersMgr::enablePluginChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (MeteorShowersMgr::*)(bool )>(_a, &MeteorShowersMgr::enableLabelsChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (MeteorShowersMgr::*)(Vec3f )>(_a, &MeteorShowersMgr::colorARGChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (MeteorShowersMgr::*)(Vec3f )>(_a, &MeteorShowersMgr::colorARCChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (MeteorShowersMgr::*)(Vec3f )>(_a, &MeteorShowersMgr::colorIRChanged, 6))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 4:
        case 3:
        case 2:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Vec3f >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->getEnablePlugin(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->getEnableLabels(); break;
        case 2: *reinterpret_cast<Vec3f*>(_v) = _t->getColorARG(); break;
        case 3: *reinterpret_cast<Vec3f*>(_v) = _t->getColorARC(); break;
        case 4: *reinterpret_cast<Vec3f*>(_v) = _t->getColorIR(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->actionEnablePlugin(*reinterpret_cast<bool*>(_v)); break;
        case 1: _t->setEnableLabels(*reinterpret_cast<bool*>(_v)); break;
        case 2: _t->setColorARG(*reinterpret_cast<Vec3f*>(_v)); break;
        case 3: _t->setColorARC(*reinterpret_cast<Vec3f*>(_v)); break;
        case 4: _t->setColorIR(*reinterpret_cast<Vec3f*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *MeteorShowersMgr::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MeteorShowersMgr::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16MeteorShowersMgrE_t>.strings))
        return static_cast<void*>(this);
    return StelModule::qt_metacast(_clname);
}

int MeteorShowersMgr::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StelModule::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 40)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 40;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 40)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 40;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void MeteorShowersMgr::updateStateChanged(MeteorShowersMgr::UpdateState _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void MeteorShowersMgr::jsonUpdateComplete()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void MeteorShowersMgr::enablePluginChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void MeteorShowersMgr::enableLabelsChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void MeteorShowersMgr::colorARGChanged(Vec3f _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void MeteorShowersMgr::colorARCChanged(Vec3f _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void MeteorShowersMgr::colorIRChanged(Vec3f _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN32MeteorShowersStelPluginInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto MeteorShowersStelPluginInterface::qt_create_metaobjectdata<qt_meta_tag_ZN32MeteorShowersStelPluginInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MeteorShowersStelPluginInterface"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<MeteorShowersStelPluginInterface, qt_meta_tag_ZN32MeteorShowersStelPluginInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MeteorShowersStelPluginInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN32MeteorShowersStelPluginInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN32MeteorShowersStelPluginInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN32MeteorShowersStelPluginInterfaceE_t>.metaTypes,
    nullptr
} };

void MeteorShowersStelPluginInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MeteorShowersStelPluginInterface *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *MeteorShowersStelPluginInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MeteorShowersStelPluginInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN32MeteorShowersStelPluginInterfaceE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "StelPluginInterface"))
        return static_cast< StelPluginInterface*>(this);
    if (!strcmp(_clname, "org.stellarium.StelPluginInterface/2.0"))
        return static_cast< StelPluginInterface*>(this);
    return QObject::qt_metacast(_clname);
}

int MeteorShowersStelPluginInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}

#ifdef QT_MOC_EXPORT_PLUGIN_V2
static constexpr unsigned char qt_pluginMetaDataV2_MeteorShowersStelPluginInterface[] = {
    0xbf, 
    // "IID"
    0x02,  0x78,  0x26,  'o',  'r',  'g',  '.',  's', 
    't',  'e',  'l',  'l',  'a',  'r',  'i',  'u', 
    'm',  '.',  'S',  't',  'e',  'l',  'P',  'l', 
    'u',  'g',  'i',  'n',  'I',  'n',  't',  'e', 
    'r',  'f',  'a',  'c',  'e',  '/',  '2',  '.', 
    '0', 
    // "className"
    0x03,  0x78,  0x20,  'M',  'e',  't',  'e',  'o', 
    'r',  'S',  'h',  'o',  'w',  'e',  'r',  's', 
    'S',  't',  'e',  'l',  'P',  'l',  'u',  'g', 
    'i',  'n',  'I',  'n',  't',  'e',  'r',  'f', 
    'a',  'c',  'e', 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN_V2(MeteorShowersStelPluginInterface, MeteorShowersStelPluginInterface, qt_pluginMetaDataV2_MeteorShowersStelPluginInterface)
#else
QT_PLUGIN_METADATA_SECTION
Q_CONSTINIT static constexpr unsigned char qt_pluginMetaData_MeteorShowersStelPluginInterface[] = {
    'Q', 'T', 'M', 'E', 'T', 'A', 'D', 'A', 'T', 'A', ' ', '!',
    // metadata version, Qt version, architectural requirements
    0, QT_VERSION_MAJOR, QT_VERSION_MINOR, qPluginArchRequirements(),
    0xbf, 
    // "IID"
    0x02,  0x78,  0x26,  'o',  'r',  'g',  '.',  's', 
    't',  'e',  'l',  'l',  'a',  'r',  'i',  'u', 
    'm',  '.',  'S',  't',  'e',  'l',  'P',  'l', 
    'u',  'g',  'i',  'n',  'I',  'n',  't',  'e', 
    'r',  'f',  'a',  'c',  'e',  '/',  '2',  '.', 
    '0', 
    // "className"
    0x03,  0x78,  0x20,  'M',  'e',  't',  'e',  'o', 
    'r',  'S',  'h',  'o',  'w',  'e',  'r',  's', 
    'S',  't',  'e',  'l',  'P',  'l',  'u',  'g', 
    'i',  'n',  'I',  'n',  't',  'e',  'r',  'f', 
    'a',  'c',  'e', 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN(MeteorShowersStelPluginInterface, MeteorShowersStelPluginInterface)
#endif  // QT_MOC_EXPORT_PLUGIN_V2

QT_WARNING_POP
