/****************************************************************************
** Meta object code from reading C++ file 'StelMainScriptAPI.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../stellarium/src/scripting/StelMainScriptAPI.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'StelMainScriptAPI.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN17StelMainScriptAPIE_t {};
} // unnamed namespace

template <> constexpr inline auto StelMainScriptAPI::qt_create_metaobjectdata<qt_meta_tag_ZN17StelMainScriptAPIE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "StelMainScriptAPI",
        "requestLoadSkyImage",
        "",
        "id",
        "filename",
        "lon0",
        "lat0",
        "lon1",
        "lat1",
        "lon2",
        "lat2",
        "lon3",
        "lat3",
        "minRes",
        "maxBright",
        "visible",
        "StelCore::FrameType",
        "frameType",
        "withAberration",
        "requestRemoveSkyImage",
        "requestLoadSound",
        "requestPlaySound",
        "requestPauseSound",
        "requestStopSound",
        "requestDropSound",
        "requestLoadVideo",
        "x",
        "y",
        "show",
        "alpha",
        "requestPlayVideo",
        "keepVisibleAtEnd",
        "requestPlayVideoPopout",
        "fromX",
        "fromY",
        "atCenterX",
        "atCenterY",
        "finalSizeX",
        "finalSizeY",
        "popupDuration",
        "frozenInTransition",
        "requestPauseVideo",
        "requestStopVideo",
        "requestDropVideo",
        "requestSeekVideo",
        "ms",
        "pause",
        "requestSetVideoXY",
        "relative",
        "requestSetVideoAlpha",
        "requestResizeVideo",
        "w",
        "h",
        "requestShowVideo",
        "requestSetProjectionMode",
        "requestSetSkyCulture",
        "requestSetDiskViewport",
        "b",
        "requestExit",
        "requestSetHomePosition",
        "vec3d",
        "Vec3d",
        "z",
        "vec3f",
        "Vec3f",
        "color",
        "Color",
        "cstr",
        "useQtScript",
        "setJDay",
        "JD",
        "getJDay",
        "setMJDay",
        "MJD",
        "getMJDay",
        "setDate",
        "dateStr",
        "spec",
        "dateIsDT",
        "getDate",
        "getDeltaT",
        "getDeltaTsec",
        "getDeltaTAlgorithm",
        "setDeltaTAlgorithm",
        "algorithmName",
        "setTimeRate",
        "ts",
        "getTimeRate",
        "isRealTime",
        "setRealTime",
        "isPlanetocentricCalculations",
        "setPlanetocentricCalculations",
        "f",
        "selectObjectByName",
        "name",
        "pointer",
        "selectConstellationByName",
        "getObjectInfo",
        "QVariantMap",
        "getSelectedObjectInfo",
        "addToSelectedObjectInfoString",
        "str",
        "replace",
        "getRTS",
        "objectName",
        "altitude",
        "setStelProperty",
        "propertyName",
        "QVariant",
        "propertyValue",
        "getStelProperty",
        "clear",
        "state",
        "getViewAltitudeAngle",
        "getViewAzimuthAngle",
        "getViewRaAngle",
        "getViewDecAngle",
        "getViewRaJ2000Angle",
        "getViewDecJ2000Angle",
        "moveToObject",
        "duration",
        "moveToSelectedObject",
        "moveToAltAzi",
        "alt",
        "azi",
        "moveToRaDec",
        "ra",
        "dec",
        "moveToRaDecJ2000",
        "moveToGalLongLat",
        "lon",
        "lat",
        "setObserverLocation",
        "longitude",
        "latitude",
        "planet",
        "getObserverLocation",
        "getObserverLocationInfo",
        "setTimezone",
        "tz",
        "markAsCustom",
        "getAllTimezoneNames",
        "geo2utm",
        "QList<double>",
        "zone",
        "utm2geo",
        "easting",
        "northing",
        "north",
        "screenshot",
        "prefix",
        "invert",
        "dir",
        "overwrite",
        "format",
        "setGuiVisible",
        "setSelectedObjectMarkerVisible",
        "setGuiStyle",
        "cssStyle",
        "setMinFps",
        "m",
        "getMinFps",
        "setMaxFps",
        "getMaxFps",
        "getMountMode",
        "setMountMode",
        "mode",
        "getNightMode",
        "setNightMode",
        "getProjectionMode",
        "setProjectionMode",
        "setWindowSize",
        "Vec2d",
        "width",
        "height",
        "getDiskViewport",
        "setDiskViewport",
        "setSphericMirror",
        "setViewportStretch",
        "stretch",
        "getAllSkyCultureIDs",
        "getSkyCulture",
        "setSkyCulture",
        "getSkyCultureName",
        "getSkyCultureNameI18n",
        "getFlagGravityLabels",
        "getFlipHorz",
        "setFlipHorz",
        "getFlipVert",
        "setFlipVert",
        "setFlagGravityLabels",
        "loadSkyImage",
        "frame",
        "angSize",
        "rotation",
        "removeSkyImage",
        "getScreenXYFromAltAzi",
        "say",
        "text",
        "loadSound",
        "playSound",
        "pauseSound",
        "stopSound",
        "dropSound",
        "getSoundPosition",
        "getSoundDuration",
        "loadVideo",
        "playVideo",
        "playVideoPopout",
        "pauseVideo",
        "stopVideo",
        "dropVideo",
        "seekVideo",
        "setVideoXY",
        "setVideoAlpha",
        "resizeVideo",
        "showVideo",
        "getVideoDuration",
        "getVideoPosition",
        "getScreenWidth",
        "getScreenHeight",
        "getScriptRate",
        "setScriptRate",
        "r",
        "pauseScript",
        "setSelectedObjectInfo",
        "level",
        "exit",
        "quitStellarium",
        "getPropertyList",
        "debug",
        "s",
        "output",
        "mapToString",
        "map",
        "resetOutput",
        "saveOutputAs",
        "getAppLanguage",
        "setAppLanguage",
        "langCode",
        "translate",
        "englishText",
        "goHome",
        "getBortleScaleIndex",
        "setBortleScaleIndex",
        "index",
        "refraction",
        "apparent",
        "jdFromDateString",
        "dt",
        "wait",
        "t",
        "waitFor",
        "getEnv",
        "var",
        "isModuleLoaded",
        "moduleID",
        "getPlatformName",
        "isMediaPlaybackSupported",
        "setDisplayMaxLuminance",
        "cdPerSqM",
        "getDisplayMaxLuminance",
        "setDisplayAdaptationLuminance",
        "getDisplayAdaptationLuminance",
        "setDisplayGamma",
        "gamma",
        "getDisplayGamma",
        "JDay",
        "timeSpeed"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'requestLoadSkyImage'
        QtMocHelpers::SignalData<void(const QString &, const QString &, double, double, double, double, double, double, double, double, double, double, bool, const StelCore::FrameType, bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 4 }, { QMetaType::Double, 5 }, { QMetaType::Double, 6 },
            { QMetaType::Double, 7 }, { QMetaType::Double, 8 }, { QMetaType::Double, 9 }, { QMetaType::Double, 10 },
            { QMetaType::Double, 11 }, { QMetaType::Double, 12 }, { QMetaType::Double, 13 }, { QMetaType::Double, 14 },
            { QMetaType::Bool, 15 }, { 0x80000000 | 16, 17 }, { QMetaType::Bool, 18 },
        }}),
        // Signal 'requestRemoveSkyImage'
        QtMocHelpers::SignalData<void(const QString &)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'requestLoadSound'
        QtMocHelpers::SignalData<void(const QString &, const QString &)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 4 }, { QMetaType::QString, 3 },
        }}),
        // Signal 'requestPlaySound'
        QtMocHelpers::SignalData<void(const QString &)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'requestPauseSound'
        QtMocHelpers::SignalData<void(const QString &)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'requestStopSound'
        QtMocHelpers::SignalData<void(const QString &)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'requestDropSound'
        QtMocHelpers::SignalData<void(const QString &)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'requestLoadVideo'
        QtMocHelpers::SignalData<void(const QString &, const QString &, float, float, bool, float)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 4 }, { QMetaType::QString, 3 }, { QMetaType::Float, 26 }, { QMetaType::Float, 27 },
            { QMetaType::Bool, 28 }, { QMetaType::Float, 29 },
        }}),
        // Signal 'requestPlayVideo'
        QtMocHelpers::SignalData<void(const QString &, const bool)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::Bool, 31 },
        }}),
        // Signal 'requestPlayVideoPopout'
        QtMocHelpers::SignalData<void(const QString &, float, float, float, float, float, float, float, bool)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::Float, 33 }, { QMetaType::Float, 34 }, { QMetaType::Float, 35 },
            { QMetaType::Float, 36 }, { QMetaType::Float, 37 }, { QMetaType::Float, 38 }, { QMetaType::Float, 39 },
            { QMetaType::Bool, 40 },
        }}),
        // Signal 'requestPauseVideo'
        QtMocHelpers::SignalData<void(const QString &)>(41, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'requestStopVideo'
        QtMocHelpers::SignalData<void(const QString &)>(42, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'requestDropVideo'
        QtMocHelpers::SignalData<void(const QString &)>(43, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'requestSeekVideo'
        QtMocHelpers::SignalData<void(const QString &, qint64, bool)>(44, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::LongLong, 45 }, { QMetaType::Bool, 46 },
        }}),
        // Signal 'requestSeekVideo'
        QtMocHelpers::SignalData<void(const QString &, qint64)>(44, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::LongLong, 45 },
        }}),
        // Signal 'requestSetVideoXY'
        QtMocHelpers::SignalData<void(const QString &, float, float, bool)>(47, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::Float, 26 }, { QMetaType::Float, 27 }, { QMetaType::Bool, 48 },
        }}),
        // Signal 'requestSetVideoXY'
        QtMocHelpers::SignalData<void(const QString &, float, float)>(47, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::Float, 26 }, { QMetaType::Float, 27 },
        }}),
        // Signal 'requestSetVideoAlpha'
        QtMocHelpers::SignalData<void(const QString &, float)>(49, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::Float, 29 },
        }}),
        // Signal 'requestResizeVideo'
        QtMocHelpers::SignalData<void(const QString &, float, float)>(50, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::Float, 51 }, { QMetaType::Float, 52 },
        }}),
        // Signal 'requestShowVideo'
        QtMocHelpers::SignalData<void(const QString &, bool)>(53, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::Bool, 28 },
        }}),
        // Signal 'requestSetProjectionMode'
        QtMocHelpers::SignalData<void(QString)>(54, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'requestSetSkyCulture'
        QtMocHelpers::SignalData<void(QString)>(55, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'requestSetDiskViewport'
        QtMocHelpers::SignalData<void(bool)>(56, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 57 },
        }}),
        // Signal 'requestExit'
        QtMocHelpers::SignalData<void()>(58, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'requestSetHomePosition'
        QtMocHelpers::SignalData<void()>(59, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'vec3d'
        QtMocHelpers::SlotData<Vec3d(const double, const double, const double)>(60, 2, QMC::AccessPublic, 0x80000000 | 61, {{
            { QMetaType::Double, 26 }, { QMetaType::Double, 27 }, { QMetaType::Double, 62 },
        }}),
        // Slot 'vec3f'
        QtMocHelpers::SlotData<Vec3f(const float, const float, const float)>(63, 2, QMC::AccessPublic, 0x80000000 | 64, {{
            { QMetaType::Float, 26 }, { QMetaType::Float, 27 }, { QMetaType::Float, 62 },
        }}),
        // Slot 'color'
        QtMocHelpers::SlotData<Color(const QString &)>(65, 2, QMC::AccessPublic, 0x80000000 | 66, {{
            { QMetaType::QString, 67 },
        }}),
        // Slot 'useQtScript'
        QtMocHelpers::SlotData<bool()>(68, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setJDay'
        QtMocHelpers::SlotData<void(double)>(69, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 70 },
        }}),
        // Slot 'getJDay'
        QtMocHelpers::SlotData<double()>(71, 2, QMC::AccessPublic, QMetaType::Double),
        // Slot 'setMJDay'
        QtMocHelpers::SlotData<void(double)>(72, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 73 },
        }}),
        // Slot 'getMJDay'
        QtMocHelpers::SlotData<double()>(74, 2, QMC::AccessPublic, QMetaType::Double),
        // Slot 'setDate'
        QtMocHelpers::SlotData<void(const QString &, const QString &, const bool &)>(75, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 76 }, { QMetaType::QString, 77 }, { QMetaType::Bool, 78 },
        }}),
        // Slot 'setDate'
        QtMocHelpers::SlotData<void(const QString &, const QString &)>(75, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 76 }, { QMetaType::QString, 77 },
        }}),
        // Slot 'setDate'
        QtMocHelpers::SlotData<void(const QString &)>(75, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 76 },
        }}),
        // Slot 'getDate'
        QtMocHelpers::SlotData<QString(const QString &)>(79, 2, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::QString, 77 },
        }}),
        // Slot 'getDate'
        QtMocHelpers::SlotData<QString()>(79, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::QString),
        // Slot 'getDeltaT'
        QtMocHelpers::SlotData<QString()>(80, 2, QMC::AccessPublic, QMetaType::QString),
        // Slot 'getDeltaTsec'
        QtMocHelpers::SlotData<double()>(81, 2, QMC::AccessPublic, QMetaType::Double),
        // Slot 'getDeltaTAlgorithm'
        QtMocHelpers::SlotData<QString()>(82, 2, QMC::AccessPublic, QMetaType::QString),
        // Slot 'setDeltaTAlgorithm'
        QtMocHelpers::SlotData<void(QString)>(83, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 84 },
        }}),
        // Slot 'setTimeRate'
        QtMocHelpers::SlotData<void(double)>(85, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 86 },
        }}),
        // Slot 'getTimeRate'
        QtMocHelpers::SlotData<double()>(87, 2, QMC::AccessPublic, QMetaType::Double),
        // Slot 'isRealTime'
        QtMocHelpers::SlotData<bool()>(88, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setRealTime'
        QtMocHelpers::SlotData<void()>(89, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'isPlanetocentricCalculations'
        QtMocHelpers::SlotData<bool()>(90, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setPlanetocentricCalculations'
        QtMocHelpers::SlotData<void(bool)>(91, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 92 },
        }}),
        // Slot 'selectObjectByName'
        QtMocHelpers::SlotData<void(const QString &, bool)>(93, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 94 }, { QMetaType::Bool, 95 },
        }}),
        // Slot 'selectObjectByName'
        QtMocHelpers::SlotData<void(const QString &)>(93, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 94 },
        }}),
        // Slot 'selectConstellationByName'
        QtMocHelpers::SlotData<void(const QString &)>(96, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 94 },
        }}),
        // Slot 'getObjectInfo'
        QtMocHelpers::SlotData<QVariantMap(const QString &)>(97, 2, QMC::AccessPublic, 0x80000000 | 98, {{
            { QMetaType::QString, 94 },
        }}),
        // Slot 'getSelectedObjectInfo'
        QtMocHelpers::SlotData<QVariantMap()>(99, 2, QMC::AccessPublic, 0x80000000 | 98),
        // Slot 'addToSelectedObjectInfoString'
        QtMocHelpers::SlotData<void(const QString &, bool)>(100, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 101 }, { QMetaType::Bool, 102 },
        }}),
        // Slot 'addToSelectedObjectInfoString'
        QtMocHelpers::SlotData<void(const QString &)>(100, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 101 },
        }}),
        // Slot 'getRTS'
        QtMocHelpers::SlotData<QVariantMap(const QString &, const double)>(103, 2, QMC::AccessPublic, 0x80000000 | 98, {{
            { QMetaType::QString, 104 }, { QMetaType::Double, 105 },
        }}),
        // Slot 'getRTS'
        QtMocHelpers::SlotData<QVariantMap(const QString &)>(103, 2, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 98, {{
            { QMetaType::QString, 104 },
        }}),
        // Slot 'setStelProperty'
        QtMocHelpers::SlotData<void(const QString &, QVariant)>(106, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 107 }, { 0x80000000 | 108, 109 },
        }}),
        // Slot 'getStelProperty'
        QtMocHelpers::SlotData<QVariant(const QString &)>(110, 2, QMC::AccessPublic, 0x80000000 | 108, {{
            { QMetaType::QString, 107 },
        }}),
        // Slot 'clear'
        QtMocHelpers::SlotData<void(const QString &)>(111, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 112 },
        }}),
        // Slot 'clear'
        QtMocHelpers::SlotData<void()>(111, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'getViewAltitudeAngle'
        QtMocHelpers::SlotData<double()>(113, 2, QMC::AccessPublic, QMetaType::Double),
        // Slot 'getViewAzimuthAngle'
        QtMocHelpers::SlotData<double()>(114, 2, QMC::AccessPublic, QMetaType::Double),
        // Slot 'getViewRaAngle'
        QtMocHelpers::SlotData<double()>(115, 2, QMC::AccessPublic, QMetaType::Double),
        // Slot 'getViewDecAngle'
        QtMocHelpers::SlotData<double()>(116, 2, QMC::AccessPublic, QMetaType::Double),
        // Slot 'getViewRaJ2000Angle'
        QtMocHelpers::SlotData<double()>(117, 2, QMC::AccessPublic, QMetaType::Double),
        // Slot 'getViewDecJ2000Angle'
        QtMocHelpers::SlotData<double()>(118, 2, QMC::AccessPublic, QMetaType::Double),
        // Slot 'moveToObject'
        QtMocHelpers::SlotData<void(const QString &, float)>(119, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 94 }, { QMetaType::Float, 120 },
        }}),
        // Slot 'moveToObject'
        QtMocHelpers::SlotData<void(const QString &)>(119, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 94 },
        }}),
        // Slot 'moveToSelectedObject'
        QtMocHelpers::SlotData<void(float)>(121, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 120 },
        }}),
        // Slot 'moveToSelectedObject'
        QtMocHelpers::SlotData<void()>(121, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'moveToAltAzi'
        QtMocHelpers::SlotData<void(const QString &, const QString &, float)>(122, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 123 }, { QMetaType::QString, 124 }, { QMetaType::Float, 120 },
        }}),
        // Slot 'moveToAltAzi'
        QtMocHelpers::SlotData<void(const QString &, const QString &)>(122, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 123 }, { QMetaType::QString, 124 },
        }}),
        // Slot 'moveToRaDec'
        QtMocHelpers::SlotData<void(const QString &, const QString &, float)>(125, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 126 }, { QMetaType::QString, 127 }, { QMetaType::Float, 120 },
        }}),
        // Slot 'moveToRaDec'
        QtMocHelpers::SlotData<void(const QString &, const QString &)>(125, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 126 }, { QMetaType::QString, 127 },
        }}),
        // Slot 'moveToRaDecJ2000'
        QtMocHelpers::SlotData<void(const QString &, const QString &, float)>(128, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 126 }, { QMetaType::QString, 127 }, { QMetaType::Float, 120 },
        }}),
        // Slot 'moveToRaDecJ2000'
        QtMocHelpers::SlotData<void(const QString &, const QString &)>(128, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 126 }, { QMetaType::QString, 127 },
        }}),
        // Slot 'moveToGalLongLat'
        QtMocHelpers::SlotData<void(const QString &, const QString &, float)>(129, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 130 }, { QMetaType::QString, 131 }, { QMetaType::Float, 120 },
        }}),
        // Slot 'moveToGalLongLat'
        QtMocHelpers::SlotData<void(const QString &, const QString &)>(129, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 130 }, { QMetaType::QString, 131 },
        }}),
        // Slot 'setObserverLocation'
        QtMocHelpers::SlotData<void(double, double, double, double, const QString &, const QString &)>(132, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 133 }, { QMetaType::Double, 134 }, { QMetaType::Double, 105 }, { QMetaType::Double, 120 },
            { QMetaType::QString, 94 }, { QMetaType::QString, 135 },
        }}),
        // Slot 'setObserverLocation'
        QtMocHelpers::SlotData<void(double, double, double, double, const QString &)>(132, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Double, 133 }, { QMetaType::Double, 134 }, { QMetaType::Double, 105 }, { QMetaType::Double, 120 },
            { QMetaType::QString, 94 },
        }}),
        // Slot 'setObserverLocation'
        QtMocHelpers::SlotData<void(double, double, double, double)>(132, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Double, 133 }, { QMetaType::Double, 134 }, { QMetaType::Double, 105 }, { QMetaType::Double, 120 },
        }}),
        // Slot 'setObserverLocation'
        QtMocHelpers::SlotData<void(double, double, double)>(132, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Double, 133 }, { QMetaType::Double, 134 }, { QMetaType::Double, 105 },
        }}),
        // Slot 'setObserverLocation'
        QtMocHelpers::SlotData<void(const QString &, double)>(132, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::Double, 120 },
        }}),
        // Slot 'setObserverLocation'
        QtMocHelpers::SlotData<void(const QString &)>(132, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'getObserverLocation'
        QtMocHelpers::SlotData<QString()>(136, 2, QMC::AccessPublic, QMetaType::QString),
        // Slot 'getObserverLocationInfo'
        QtMocHelpers::SlotData<QVariantMap()>(137, 2, QMC::AccessPublic, 0x80000000 | 98),
        // Slot 'setTimezone'
        QtMocHelpers::SlotData<void(QString, int)>(138, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 139 }, { QMetaType::Int, 140 },
        }}),
        // Slot 'setTimezone'
        QtMocHelpers::SlotData<void(QString)>(138, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 139 },
        }}),
        // Slot 'getAllTimezoneNames'
        QtMocHelpers::SlotData<QStringList()>(141, 2, QMC::AccessPublic, QMetaType::QStringList),
        // Slot 'geo2utm'
        QtMocHelpers::SlotData<QList<double>(const double, const double, const int)>(142, 2, QMC::AccessPublic, 0x80000000 | 143, {{
            { QMetaType::Double, 133 }, { QMetaType::Double, 134 }, { QMetaType::Int, 144 },
        }}),
        // Slot 'geo2utm'
        QtMocHelpers::SlotData<QList<double>(const double, const double)>(142, 2, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 143, {{
            { QMetaType::Double, 133 }, { QMetaType::Double, 134 },
        }}),
        // Slot 'utm2geo'
        QtMocHelpers::SlotData<QList<double>(const double, const double, const int, const bool)>(145, 2, QMC::AccessPublic, 0x80000000 | 143, {{
            { QMetaType::Double, 146 }, { QMetaType::Double, 147 }, { QMetaType::Int, 144 }, { QMetaType::Bool, 148 },
        }}),
        // Slot 'screenshot'
        QtMocHelpers::SlotData<void(const QString &, bool, const QString &, const bool, const QString &)>(149, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 150 }, { QMetaType::Bool, 151 }, { QMetaType::QString, 152 }, { QMetaType::Bool, 153 },
            { QMetaType::QString, 154 },
        }}),
        // Slot 'screenshot'
        QtMocHelpers::SlotData<void(const QString &, bool, const QString &, const bool)>(149, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 150 }, { QMetaType::Bool, 151 }, { QMetaType::QString, 152 }, { QMetaType::Bool, 153 },
        }}),
        // Slot 'screenshot'
        QtMocHelpers::SlotData<void(const QString &, bool, const QString &)>(149, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 150 }, { QMetaType::Bool, 151 }, { QMetaType::QString, 152 },
        }}),
        // Slot 'screenshot'
        QtMocHelpers::SlotData<void(const QString &, bool)>(149, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 150 }, { QMetaType::Bool, 151 },
        }}),
        // Slot 'screenshot'
        QtMocHelpers::SlotData<void(const QString &)>(149, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 150 },
        }}),
        // Slot 'setGuiVisible'
        QtMocHelpers::SlotData<void(bool)>(155, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 57 },
        }}),
        // Slot 'setSelectedObjectMarkerVisible'
        QtMocHelpers::SlotData<void(bool)>(156, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 57 },
        }}),
        // Slot 'setGuiStyle'
        QtMocHelpers::SlotData<void(const QString &)>(157, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 158 },
        }}),
        // Slot 'setMinFps'
        QtMocHelpers::SlotData<void(float)>(159, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 160 },
        }}),
        // Slot 'getMinFps'
        QtMocHelpers::SlotData<float()>(161, 2, QMC::AccessPublic, QMetaType::Float),
        // Slot 'setMaxFps'
        QtMocHelpers::SlotData<void(float)>(162, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 160 },
        }}),
        // Slot 'getMaxFps'
        QtMocHelpers::SlotData<float()>(163, 2, QMC::AccessPublic, QMetaType::Float),
        // Slot 'getMountMode'
        QtMocHelpers::SlotData<QString()>(164, 2, QMC::AccessPublic, QMetaType::QString),
        // Slot 'setMountMode'
        QtMocHelpers::SlotData<void(const QString &)>(165, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 166 },
        }}),
        // Slot 'getNightMode'
        QtMocHelpers::SlotData<bool()>(167, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setNightMode'
        QtMocHelpers::SlotData<void(bool)>(168, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 57 },
        }}),
        // Slot 'getProjectionMode'
        QtMocHelpers::SlotData<QString()>(169, 2, QMC::AccessPublic, QMetaType::QString),
        // Slot 'setProjectionMode'
        QtMocHelpers::SlotData<void(const QString &)>(170, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'setWindowSize'
        QtMocHelpers::SlotData<Vec2d(int, int)>(171, 2, QMC::AccessPublic, 0x80000000 | 172, {{
            { QMetaType::Int, 173 }, { QMetaType::Int, 174 },
        }}),
        // Slot 'getDiskViewport'
        QtMocHelpers::SlotData<bool()>(175, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setDiskViewport'
        QtMocHelpers::SlotData<void(bool)>(176, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 57 },
        }}),
        // Slot 'setSphericMirror'
        QtMocHelpers::SlotData<void(bool)>(177, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 57 },
        }}),
        // Slot 'setViewportStretch'
        QtMocHelpers::SlotData<void(const float)>(178, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 179 },
        }}),
        // Slot 'getAllSkyCultureIDs'
        QtMocHelpers::SlotData<QStringList()>(180, 2, QMC::AccessPublic, QMetaType::QStringList),
        // Slot 'getSkyCulture'
        QtMocHelpers::SlotData<QString()>(181, 2, QMC::AccessPublic, QMetaType::QString),
        // Slot 'setSkyCulture'
        QtMocHelpers::SlotData<void(const QString &)>(182, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'getSkyCultureName'
        QtMocHelpers::SlotData<QString()>(183, 2, QMC::AccessPublic, QMetaType::QString),
        // Slot 'getSkyCultureNameI18n'
        QtMocHelpers::SlotData<QString()>(184, 2, QMC::AccessPublic, QMetaType::QString),
        // Slot 'getFlagGravityLabels'
        QtMocHelpers::SlotData<bool()>(185, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'getFlipHorz'
        QtMocHelpers::SlotData<bool()>(186, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFlipHorz'
        QtMocHelpers::SlotData<void(bool)>(187, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 57 },
        }}),
        // Slot 'getFlipVert'
        QtMocHelpers::SlotData<bool()>(188, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFlipVert'
        QtMocHelpers::SlotData<void(bool)>(189, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 57 },
        }}),
        // Slot 'setFlagGravityLabels'
        QtMocHelpers::SlotData<void(bool)>(190, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 57 },
        }}),
        // Slot 'loadSkyImage'
        QtMocHelpers::SlotData<void(const QString &, const QString &, double, double, double, double, double, double, double, double, double, double, bool, const QString &, bool)>(191, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 4 }, { QMetaType::Double, 5 }, { QMetaType::Double, 6 },
            { QMetaType::Double, 7 }, { QMetaType::Double, 8 }, { QMetaType::Double, 9 }, { QMetaType::Double, 10 },
            { QMetaType::Double, 11 }, { QMetaType::Double, 12 }, { QMetaType::Double, 13 }, { QMetaType::Double, 14 },
            { QMetaType::Bool, 15 }, { QMetaType::QString, 192 }, { QMetaType::Bool, 18 },
        }}),
        // Slot 'loadSkyImage'
        QtMocHelpers::SlotData<void(const QString &, const QString &, double, double, double, double, double, double, double, double, double, double, bool, const QString &)>(191, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 4 }, { QMetaType::Double, 5 }, { QMetaType::Double, 6 },
            { QMetaType::Double, 7 }, { QMetaType::Double, 8 }, { QMetaType::Double, 9 }, { QMetaType::Double, 10 },
            { QMetaType::Double, 11 }, { QMetaType::Double, 12 }, { QMetaType::Double, 13 }, { QMetaType::Double, 14 },
            { QMetaType::Bool, 15 }, { QMetaType::QString, 192 },
        }}),
        // Slot 'loadSkyImage'
        QtMocHelpers::SlotData<void(const QString &, const QString &, double, double, double, double, double, double, double, double, double, double, bool)>(191, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 4 }, { QMetaType::Double, 5 }, { QMetaType::Double, 6 },
            { QMetaType::Double, 7 }, { QMetaType::Double, 8 }, { QMetaType::Double, 9 }, { QMetaType::Double, 10 },
            { QMetaType::Double, 11 }, { QMetaType::Double, 12 }, { QMetaType::Double, 13 }, { QMetaType::Double, 14 },
            { QMetaType::Bool, 15 },
        }}),
        // Slot 'loadSkyImage'
        QtMocHelpers::SlotData<void(const QString &, const QString &, double, double, double, double, double, double, double, double, double, double)>(191, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 4 }, { QMetaType::Double, 5 }, { QMetaType::Double, 6 },
            { QMetaType::Double, 7 }, { QMetaType::Double, 8 }, { QMetaType::Double, 9 }, { QMetaType::Double, 10 },
            { QMetaType::Double, 11 }, { QMetaType::Double, 12 }, { QMetaType::Double, 13 }, { QMetaType::Double, 14 },
        }}),
        // Slot 'loadSkyImage'
        QtMocHelpers::SlotData<void(const QString &, const QString &, double, double, double, double, double, double, double, double, double)>(191, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 4 }, { QMetaType::Double, 5 }, { QMetaType::Double, 6 },
            { QMetaType::Double, 7 }, { QMetaType::Double, 8 }, { QMetaType::Double, 9 }, { QMetaType::Double, 10 },
            { QMetaType::Double, 11 }, { QMetaType::Double, 12 }, { QMetaType::Double, 13 },
        }}),
        // Slot 'loadSkyImage'
        QtMocHelpers::SlotData<void(const QString &, const QString &, double, double, double, double, double, double, double, double)>(191, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 4 }, { QMetaType::Double, 5 }, { QMetaType::Double, 6 },
            { QMetaType::Double, 7 }, { QMetaType::Double, 8 }, { QMetaType::Double, 9 }, { QMetaType::Double, 10 },
            { QMetaType::Double, 11 }, { QMetaType::Double, 12 },
        }}),
        // Slot 'loadSkyImage'
        QtMocHelpers::SlotData<void(const QString &, const QString &, const QString &, const QString &, const QString &, const QString &, const QString &, const QString &, const QString &, const QString &, double, double, bool, const QString &, bool)>(191, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 4 }, { QMetaType::QString, 5 }, { QMetaType::QString, 6 },
            { QMetaType::QString, 7 }, { QMetaType::QString, 8 }, { QMetaType::QString, 9 }, { QMetaType::QString, 10 },
            { QMetaType::QString, 11 }, { QMetaType::QString, 12 }, { QMetaType::Double, 13 }, { QMetaType::Double, 14 },
            { QMetaType::Bool, 15 }, { QMetaType::QString, 192 }, { QMetaType::Bool, 18 },
        }}),
        // Slot 'loadSkyImage'
        QtMocHelpers::SlotData<void(const QString &, const QString &, const QString &, const QString &, const QString &, const QString &, const QString &, const QString &, const QString &, const QString &, double, double, bool, const QString &)>(191, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 4 }, { QMetaType::QString, 5 }, { QMetaType::QString, 6 },
            { QMetaType::QString, 7 }, { QMetaType::QString, 8 }, { QMetaType::QString, 9 }, { QMetaType::QString, 10 },
            { QMetaType::QString, 11 }, { QMetaType::QString, 12 }, { QMetaType::Double, 13 }, { QMetaType::Double, 14 },
            { QMetaType::Bool, 15 }, { QMetaType::QString, 192 },
        }}),
        // Slot 'loadSkyImage'
        QtMocHelpers::SlotData<void(const QString &, const QString &, const QString &, const QString &, const QString &, const QString &, const QString &, const QString &, const QString &, const QString &, double, double, bool)>(191, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 4 }, { QMetaType::QString, 5 }, { QMetaType::QString, 6 },
            { QMetaType::QString, 7 }, { QMetaType::QString, 8 }, { QMetaType::QString, 9 }, { QMetaType::QString, 10 },
            { QMetaType::QString, 11 }, { QMetaType::QString, 12 }, { QMetaType::Double, 13 }, { QMetaType::Double, 14 },
            { QMetaType::Bool, 15 },
        }}),
        // Slot 'loadSkyImage'
        QtMocHelpers::SlotData<void(const QString &, const QString &, const QString &, const QString &, const QString &, const QString &, const QString &, const QString &, const QString &, const QString &, double, double)>(191, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 4 }, { QMetaType::QString, 5 }, { QMetaType::QString, 6 },
            { QMetaType::QString, 7 }, { QMetaType::QString, 8 }, { QMetaType::QString, 9 }, { QMetaType::QString, 10 },
            { QMetaType::QString, 11 }, { QMetaType::QString, 12 }, { QMetaType::Double, 13 }, { QMetaType::Double, 14 },
        }}),
        // Slot 'loadSkyImage'
        QtMocHelpers::SlotData<void(const QString &, const QString &, const QString &, const QString &, const QString &, const QString &, const QString &, const QString &, const QString &, const QString &, double)>(191, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 4 }, { QMetaType::QString, 5 }, { QMetaType::QString, 6 },
            { QMetaType::QString, 7 }, { QMetaType::QString, 8 }, { QMetaType::QString, 9 }, { QMetaType::QString, 10 },
            { QMetaType::QString, 11 }, { QMetaType::QString, 12 }, { QMetaType::Double, 13 },
        }}),
        // Slot 'loadSkyImage'
        QtMocHelpers::SlotData<void(const QString &, const QString &, const QString &, const QString &, const QString &, const QString &, const QString &, const QString &, const QString &, const QString &)>(191, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 4 }, { QMetaType::QString, 5 }, { QMetaType::QString, 6 },
            { QMetaType::QString, 7 }, { QMetaType::QString, 8 }, { QMetaType::QString, 9 }, { QMetaType::QString, 10 },
            { QMetaType::QString, 11 }, { QMetaType::QString, 12 },
        }}),
        // Slot 'loadSkyImage'
        QtMocHelpers::SlotData<void(const QString &, const QString &, double, double, double, double, double, double, bool, const QString &, bool)>(191, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 4 }, { QMetaType::Double, 130 }, { QMetaType::Double, 131 },
            { QMetaType::Double, 193 }, { QMetaType::Double, 194 }, { QMetaType::Double, 13 }, { QMetaType::Double, 14 },
            { QMetaType::Bool, 15 }, { QMetaType::QString, 192 }, { QMetaType::Bool, 18 },
        }}),
        // Slot 'loadSkyImage'
        QtMocHelpers::SlotData<void(const QString &, const QString &, double, double, double, double, double, double, bool, const QString &)>(191, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 4 }, { QMetaType::Double, 130 }, { QMetaType::Double, 131 },
            { QMetaType::Double, 193 }, { QMetaType::Double, 194 }, { QMetaType::Double, 13 }, { QMetaType::Double, 14 },
            { QMetaType::Bool, 15 }, { QMetaType::QString, 192 },
        }}),
        // Slot 'loadSkyImage'
        QtMocHelpers::SlotData<void(const QString &, const QString &, double, double, double, double, double, double, bool)>(191, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 4 }, { QMetaType::Double, 130 }, { QMetaType::Double, 131 },
            { QMetaType::Double, 193 }, { QMetaType::Double, 194 }, { QMetaType::Double, 13 }, { QMetaType::Double, 14 },
            { QMetaType::Bool, 15 },
        }}),
        // Slot 'loadSkyImage'
        QtMocHelpers::SlotData<void(const QString &, const QString &, double, double, double, double, double, double)>(191, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 4 }, { QMetaType::Double, 130 }, { QMetaType::Double, 131 },
            { QMetaType::Double, 193 }, { QMetaType::Double, 194 }, { QMetaType::Double, 13 }, { QMetaType::Double, 14 },
        }}),
        // Slot 'loadSkyImage'
        QtMocHelpers::SlotData<void(const QString &, const QString &, double, double, double, double, double)>(191, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 4 }, { QMetaType::Double, 130 }, { QMetaType::Double, 131 },
            { QMetaType::Double, 193 }, { QMetaType::Double, 194 }, { QMetaType::Double, 13 },
        }}),
        // Slot 'loadSkyImage'
        QtMocHelpers::SlotData<void(const QString &, const QString &, double, double, double, double)>(191, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 4 }, { QMetaType::Double, 130 }, { QMetaType::Double, 131 },
            { QMetaType::Double, 193 }, { QMetaType::Double, 194 },
        }}),
        // Slot 'loadSkyImage'
        QtMocHelpers::SlotData<void(const QString &, const QString &, const QString &, const QString &, double, double, double, double, bool, const QString &, bool)>(191, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 4 }, { QMetaType::QString, 130 }, { QMetaType::QString, 131 },
            { QMetaType::Double, 193 }, { QMetaType::Double, 194 }, { QMetaType::Double, 13 }, { QMetaType::Double, 14 },
            { QMetaType::Bool, 15 }, { QMetaType::QString, 192 }, { QMetaType::Bool, 18 },
        }}),
        // Slot 'loadSkyImage'
        QtMocHelpers::SlotData<void(const QString &, const QString &, const QString &, const QString &, double, double, double, double, bool, const QString &)>(191, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 4 }, { QMetaType::QString, 130 }, { QMetaType::QString, 131 },
            { QMetaType::Double, 193 }, { QMetaType::Double, 194 }, { QMetaType::Double, 13 }, { QMetaType::Double, 14 },
            { QMetaType::Bool, 15 }, { QMetaType::QString, 192 },
        }}),
        // Slot 'loadSkyImage'
        QtMocHelpers::SlotData<void(const QString &, const QString &, const QString &, const QString &, double, double, double, double, bool)>(191, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 4 }, { QMetaType::QString, 130 }, { QMetaType::QString, 131 },
            { QMetaType::Double, 193 }, { QMetaType::Double, 194 }, { QMetaType::Double, 13 }, { QMetaType::Double, 14 },
            { QMetaType::Bool, 15 },
        }}),
        // Slot 'loadSkyImage'
        QtMocHelpers::SlotData<void(const QString &, const QString &, const QString &, const QString &, double, double, double, double)>(191, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 4 }, { QMetaType::QString, 130 }, { QMetaType::QString, 131 },
            { QMetaType::Double, 193 }, { QMetaType::Double, 194 }, { QMetaType::Double, 13 }, { QMetaType::Double, 14 },
        }}),
        // Slot 'loadSkyImage'
        QtMocHelpers::SlotData<void(const QString &, const QString &, const QString &, const QString &, double, double, double)>(191, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 4 }, { QMetaType::QString, 130 }, { QMetaType::QString, 131 },
            { QMetaType::Double, 193 }, { QMetaType::Double, 194 }, { QMetaType::Double, 13 },
        }}),
        // Slot 'loadSkyImage'
        QtMocHelpers::SlotData<void(const QString &, const QString &, const QString &, const QString &, double, double)>(191, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 4 }, { QMetaType::QString, 130 }, { QMetaType::QString, 131 },
            { QMetaType::Double, 193 }, { QMetaType::Double, 194 },
        }}),
        // Slot 'removeSkyImage'
        QtMocHelpers::SlotData<void(const QString &)>(195, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'getScreenXYFromAltAzi'
        QtMocHelpers::SlotData<QVariantMap(const QString &, const QString &)>(196, 2, QMC::AccessPublic, 0x80000000 | 98, {{
            { QMetaType::QString, 123 }, { QMetaType::QString, 124 },
        }}),
        // Slot 'say'
        QtMocHelpers::SlotData<void(const QString &)>(197, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 198 },
        }}),
        // Slot 'loadSound'
        QtMocHelpers::SlotData<void(const QString &, const QString &)>(199, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 4 }, { QMetaType::QString, 3 },
        }}),
        // Slot 'playSound'
        QtMocHelpers::SlotData<void(const QString &)>(200, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'pauseSound'
        QtMocHelpers::SlotData<void(const QString &)>(201, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'stopSound'
        QtMocHelpers::SlotData<void(const QString &)>(202, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'dropSound'
        QtMocHelpers::SlotData<void(const QString &)>(203, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'getSoundPosition'
        QtMocHelpers::SlotData<qint64(const QString &)>(204, 2, QMC::AccessPublic, QMetaType::LongLong, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'getSoundDuration'
        QtMocHelpers::SlotData<qint64(const QString &)>(205, 2, QMC::AccessPublic, QMetaType::LongLong, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'loadVideo'
        QtMocHelpers::SlotData<void(const QString &, const QString &, float, float, bool, float)>(206, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 4 }, { QMetaType::QString, 3 }, { QMetaType::Float, 26 }, { QMetaType::Float, 27 },
            { QMetaType::Bool, 28 }, { QMetaType::Float, 29 },
        }}),
        // Slot 'loadVideo'
        QtMocHelpers::SlotData<void(const QString &, const QString &, float, float, bool)>(206, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 4 }, { QMetaType::QString, 3 }, { QMetaType::Float, 26 }, { QMetaType::Float, 27 },
            { QMetaType::Bool, 28 },
        }}),
        // Slot 'loadVideo'
        QtMocHelpers::SlotData<void(const QString &, const QString &, float, float)>(206, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 4 }, { QMetaType::QString, 3 }, { QMetaType::Float, 26 }, { QMetaType::Float, 27 },
        }}),
        // Slot 'playVideo'
        QtMocHelpers::SlotData<void(const QString &, bool)>(207, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::Bool, 31 },
        }}),
        // Slot 'playVideo'
        QtMocHelpers::SlotData<void(const QString &)>(207, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'playVideoPopout'
        QtMocHelpers::SlotData<void(const QString &, float, float, float, float, float, float, float, bool)>(208, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::Float, 33 }, { QMetaType::Float, 34 }, { QMetaType::Float, 35 },
            { QMetaType::Float, 36 }, { QMetaType::Float, 37 }, { QMetaType::Float, 38 }, { QMetaType::Float, 39 },
            { QMetaType::Bool, 40 },
        }}),
        // Slot 'pauseVideo'
        QtMocHelpers::SlotData<void(const QString &)>(209, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'stopVideo'
        QtMocHelpers::SlotData<void(const QString &)>(210, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'dropVideo'
        QtMocHelpers::SlotData<void(const QString &)>(211, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'seekVideo'
        QtMocHelpers::SlotData<void(const QString &, qint64, bool)>(212, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::LongLong, 45 }, { QMetaType::Bool, 46 },
        }}),
        // Slot 'seekVideo'
        QtMocHelpers::SlotData<void(const QString &, qint64)>(212, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::LongLong, 45 },
        }}),
        // Slot 'setVideoXY'
        QtMocHelpers::SlotData<void(const QString &, float, float, bool)>(213, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::Float, 26 }, { QMetaType::Float, 27 }, { QMetaType::Bool, 48 },
        }}),
        // Slot 'setVideoXY'
        QtMocHelpers::SlotData<void(const QString &, float, float)>(213, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::Float, 26 }, { QMetaType::Float, 27 },
        }}),
        // Slot 'setVideoAlpha'
        QtMocHelpers::SlotData<void(const QString &, float)>(214, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::Float, 29 },
        }}),
        // Slot 'resizeVideo'
        QtMocHelpers::SlotData<void(const QString &, float, float)>(215, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::Float, 51 }, { QMetaType::Float, 52 },
        }}),
        // Slot 'showVideo'
        QtMocHelpers::SlotData<void(const QString &, bool)>(216, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::Bool, 28 },
        }}),
        // Slot 'showVideo'
        QtMocHelpers::SlotData<void(const QString &)>(216, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'getVideoDuration'
        QtMocHelpers::SlotData<qint64(const QString &)>(217, 2, QMC::AccessPublic, QMetaType::LongLong, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'getVideoPosition'
        QtMocHelpers::SlotData<qint64(const QString &)>(218, 2, QMC::AccessPublic, QMetaType::LongLong, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'getScreenWidth'
        QtMocHelpers::SlotData<int()>(219, 2, QMC::AccessPublic, QMetaType::Int),
        // Slot 'getScreenHeight'
        QtMocHelpers::SlotData<int()>(220, 2, QMC::AccessPublic, QMetaType::Int),
        // Slot 'getScriptRate'
        QtMocHelpers::SlotData<double()>(221, 2, QMC::AccessPublic, QMetaType::Double),
        // Slot 'setScriptRate'
        QtMocHelpers::SlotData<void(double)>(222, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 223 },
        }}),
        // Slot 'pauseScript'
        QtMocHelpers::SlotData<void()>(224, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setSelectedObjectInfo'
        QtMocHelpers::SlotData<void(const QString &)>(225, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 226 },
        }}),
        // Slot 'exit'
        QtMocHelpers::SlotData<void()>(227, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'quitStellarium'
        QtMocHelpers::SlotData<void()>(228, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'getPropertyList'
        QtMocHelpers::SlotData<QStringList()>(229, 2, QMC::AccessPublic, QMetaType::QStringList),
        // Slot 'debug'
        QtMocHelpers::SlotData<void(const QString &)>(230, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 231 },
        }}),
        // Slot 'output'
        QtMocHelpers::SlotData<void(const QString &)>(232, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 231 },
        }}),
        // Slot 'mapToString'
        QtMocHelpers::SlotData<QString(const QVariantMap &)>(233, 2, QMC::AccessPublic, QMetaType::QString, {{
            { 0x80000000 | 98, 234 },
        }}),
        // Slot 'resetOutput'
        QtMocHelpers::SlotData<void()>(235, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'saveOutputAs'
        QtMocHelpers::SlotData<void(const QString &)>(236, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 4 },
        }}),
        // Slot 'getAppLanguage'
        QtMocHelpers::SlotData<QString()>(237, 2, QMC::AccessPublic, QMetaType::QString),
        // Slot 'setAppLanguage'
        QtMocHelpers::SlotData<void(QString)>(238, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 239 },
        }}),
        // Slot 'translate'
        QtMocHelpers::SlotData<QString(QString)>(240, 2, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::QString, 241 },
        }}),
        // Slot 'goHome'
        QtMocHelpers::SlotData<void()>(242, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'getBortleScaleIndex'
        QtMocHelpers::SlotData<int()>(243, 2, QMC::AccessPublic, QMetaType::Int),
        // Slot 'setBortleScaleIndex'
        QtMocHelpers::SlotData<void(int)>(244, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 245 },
        }}),
        // Slot 'refraction'
        QtMocHelpers::SlotData<double(double, bool)>(246, 2, QMC::AccessPublic, QMetaType::Double, {{
            { QMetaType::Double, 105 }, { QMetaType::Bool, 247 },
        }}),
        // Slot 'refraction'
        QtMocHelpers::SlotData<double(double)>(246, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Double, {{
            { QMetaType::Double, 105 },
        }}),
        // Slot 'jdFromDateString'
        QtMocHelpers::SlotData<double(const QString &, const QString &)>(248, 2, QMC::AccessPublic, QMetaType::Double, {{
            { QMetaType::QString, 249 }, { QMetaType::QString, 77 },
        }}),
        // Slot 'wait'
        QtMocHelpers::SlotData<void(double)>(250, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 251 },
        }}),
        // Slot 'waitFor'
        QtMocHelpers::SlotData<void(const QString &, const QString &)>(252, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 249 }, { QMetaType::QString, 77 },
        }}),
        // Slot 'waitFor'
        QtMocHelpers::SlotData<void(const QString &)>(252, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 249 },
        }}),
        // Slot 'getEnv'
        QtMocHelpers::SlotData<QString(const QString &)>(253, 2, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::QString, 254 },
        }}),
        // Slot 'isModuleLoaded'
        QtMocHelpers::SlotData<bool(const QString &)>(255, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 256 },
        }}),
        // Slot 'getPlatformName'
        QtMocHelpers::SlotData<QString()>(257, 2, QMC::AccessPublic, QMetaType::QString),
        // Slot 'isMediaPlaybackSupported'
        QtMocHelpers::SlotData<bool()>(258, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setDisplayMaxLuminance'
        QtMocHelpers::SlotData<void(double)>(259, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 260 },
        }}),
        // Slot 'getDisplayMaxLuminance'
        QtMocHelpers::SlotData<double()>(261, 2, QMC::AccessPublic, QMetaType::Double),
        // Slot 'setDisplayAdaptationLuminance'
        QtMocHelpers::SlotData<void(double)>(262, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 260 },
        }}),
        // Slot 'getDisplayAdaptationLuminance'
        QtMocHelpers::SlotData<double()>(263, 2, QMC::AccessPublic, QMetaType::Double),
        // Slot 'setDisplayGamma'
        QtMocHelpers::SlotData<void(double)>(264, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 265 },
        }}),
        // Slot 'getDisplayGamma'
        QtMocHelpers::SlotData<double()>(266, 2, QMC::AccessPublic, QMetaType::Double),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'JDay'
        QtMocHelpers::PropertyData<double>(267, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'timeSpeed'
        QtMocHelpers::PropertyData<double>(268, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<StelMainScriptAPI, qt_meta_tag_ZN17StelMainScriptAPIE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject StelMainScriptAPI::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17StelMainScriptAPIE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17StelMainScriptAPIE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17StelMainScriptAPIE_t>.metaTypes,
    nullptr
} };

void StelMainScriptAPI::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<StelMainScriptAPI *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->requestLoadSkyImage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[8])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[9])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[10])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[11])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[12])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[13])),(*reinterpret_cast<std::add_pointer_t<StelCore::FrameType>>(_a[14])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[15]))); break;
        case 1: _t->requestRemoveSkyImage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->requestLoadSound((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 3: _t->requestPlaySound((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: _t->requestPauseSound((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->requestStopSound((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->requestDropSound((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: _t->requestLoadVideo((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[6]))); break;
        case 8: _t->requestPlayVideo((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 9: _t->requestPlayVideoPopout((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[8])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[9]))); break;
        case 10: _t->requestPauseVideo((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 11: _t->requestStopVideo((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 12: _t->requestDropVideo((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 13: _t->requestSeekVideo((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qint64>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 14: _t->requestSeekVideo((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qint64>>(_a[2]))); break;
        case 15: _t->requestSetVideoXY((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4]))); break;
        case 16: _t->requestSetVideoXY((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[3]))); break;
        case 17: _t->requestSetVideoAlpha((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[2]))); break;
        case 18: _t->requestResizeVideo((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[3]))); break;
        case 19: _t->requestShowVideo((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 20: _t->requestSetProjectionMode((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 21: _t->requestSetSkyCulture((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 22: _t->requestSetDiskViewport((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 23: _t->requestExit(); break;
        case 24: _t->requestSetHomePosition(); break;
        case 25: { Vec3d _r = _t->vec3d((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3])));
            if (_a[0]) *reinterpret_cast<Vec3d*>(_a[0]) = std::move(_r); }  break;
        case 26: { Vec3f _r = _t->vec3f((*reinterpret_cast<std::add_pointer_t<float>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[3])));
            if (_a[0]) *reinterpret_cast<Vec3f*>(_a[0]) = std::move(_r); }  break;
        case 27: { Color _r = _t->color((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<Color*>(_a[0]) = std::move(_r); }  break;
        case 28: { bool _r = _t->useQtScript();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 29: _t->setJDay((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 30: { double _r = _t->getJDay();
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 31: _t->setMJDay((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 32: { double _r = _t->getMJDay();
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 33: _t->setDate((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 34: _t->setDate((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 35: _t->setDate((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 36: { QString _r = _t->getDate((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 37: { QString _r = _t->getDate();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 38: { QString _r = _t->getDeltaT();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 39: { double _r = _t->getDeltaTsec();
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 40: { QString _r = _t->getDeltaTAlgorithm();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 41: _t->setDeltaTAlgorithm((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 42: _t->setTimeRate((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 43: { double _r = _t->getTimeRate();
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 44: { bool _r = _t->isRealTime();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 45: _t->setRealTime(); break;
        case 46: { bool _r = _t->isPlanetocentricCalculations();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 47: _t->setPlanetocentricCalculations((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 48: _t->selectObjectByName((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 49: _t->selectObjectByName((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 50: _t->selectConstellationByName((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 51: { QVariantMap _r = _t->getObjectInfo((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QVariantMap*>(_a[0]) = std::move(_r); }  break;
        case 52: { QVariantMap _r = _t->getSelectedObjectInfo();
            if (_a[0]) *reinterpret_cast<QVariantMap*>(_a[0]) = std::move(_r); }  break;
        case 53: _t->addToSelectedObjectInfoString((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 54: _t->addToSelectedObjectInfoString((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 55: { QVariantMap _r = _t->getRTS((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QVariantMap*>(_a[0]) = std::move(_r); }  break;
        case 56: { QVariantMap _r = _t->getRTS((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QVariantMap*>(_a[0]) = std::move(_r); }  break;
        case 57: _t->setStelProperty((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 58: { QVariant _r = _t->getStelProperty((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QVariant*>(_a[0]) = std::move(_r); }  break;
        case 59: _t->clear((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 60: _t->clear(); break;
        case 61: { double _r = _t->getViewAltitudeAngle();
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 62: { double _r = _t->getViewAzimuthAngle();
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 63: { double _r = _t->getViewRaAngle();
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 64: { double _r = _t->getViewDecAngle();
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 65: { double _r = _t->getViewRaJ2000Angle();
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 66: { double _r = _t->getViewDecJ2000Angle();
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 67: _t->moveToObject((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[2]))); break;
        case 68: _t->moveToObject((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 69: _t->moveToSelectedObject((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 70: _t->moveToSelectedObject(); break;
        case 71: _t->moveToAltAzi((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[3]))); break;
        case 72: _t->moveToAltAzi((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 73: _t->moveToRaDec((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[3]))); break;
        case 74: _t->moveToRaDec((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 75: _t->moveToRaDecJ2000((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[3]))); break;
        case 76: _t->moveToRaDecJ2000((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 77: _t->moveToGalLongLat((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[3]))); break;
        case 78: _t->moveToGalLongLat((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 79: _t->setObserverLocation((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[6]))); break;
        case 80: _t->setObserverLocation((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5]))); break;
        case 81: _t->setObserverLocation((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[4]))); break;
        case 82: _t->setObserverLocation((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3]))); break;
        case 83: _t->setObserverLocation((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 84: _t->setObserverLocation((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 85: { QString _r = _t->getObserverLocation();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 86: { QVariantMap _r = _t->getObserverLocationInfo();
            if (_a[0]) *reinterpret_cast<QVariantMap*>(_a[0]) = std::move(_r); }  break;
        case 87: _t->setTimezone((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 88: _t->setTimezone((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 89: { QStringList _r = _t->getAllTimezoneNames();
            if (_a[0]) *reinterpret_cast<QStringList*>(_a[0]) = std::move(_r); }  break;
        case 90: { QList<double> _r = _t->geo2utm((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QList<double>*>(_a[0]) = std::move(_r); }  break;
        case 91: { QList<double> _r = _t->geo2utm((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QList<double>*>(_a[0]) = std::move(_r); }  break;
        case 92: { QList<double> _r = _t->utm2geo((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])));
            if (_a[0]) *reinterpret_cast<QList<double>*>(_a[0]) = std::move(_r); }  break;
        case 93: _t->screenshot((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5]))); break;
        case 94: _t->screenshot((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4]))); break;
        case 95: _t->screenshot((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 96: _t->screenshot((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 97: _t->screenshot((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 98: _t->setGuiVisible((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 99: _t->setSelectedObjectMarkerVisible((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 100: _t->setGuiStyle((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 101: _t->setMinFps((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 102: { float _r = _t->getMinFps();
            if (_a[0]) *reinterpret_cast<float*>(_a[0]) = std::move(_r); }  break;
        case 103: _t->setMaxFps((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 104: { float _r = _t->getMaxFps();
            if (_a[0]) *reinterpret_cast<float*>(_a[0]) = std::move(_r); }  break;
        case 105: { QString _r = _t->getMountMode();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 106: _t->setMountMode((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 107: { bool _r = _t->getNightMode();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 108: _t->setNightMode((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 109: { QString _r = _t->getProjectionMode();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 110: _t->setProjectionMode((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 111: { Vec2d _r = _t->setWindowSize((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])));
            if (_a[0]) *reinterpret_cast<Vec2d*>(_a[0]) = std::move(_r); }  break;
        case 112: { bool _r = _t->getDiskViewport();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 113: _t->setDiskViewport((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 114: _t->setSphericMirror((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 115: _t->setViewportStretch((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 116: { QStringList _r = _t->getAllSkyCultureIDs();
            if (_a[0]) *reinterpret_cast<QStringList*>(_a[0]) = std::move(_r); }  break;
        case 117: { QString _r = _t->getSkyCulture();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 118: _t->setSkyCulture((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 119: { QString _r = _t->getSkyCultureName();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 120: { QString _r = _t->getSkyCultureNameI18n();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 121: { bool _r = _t->getFlagGravityLabels();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 122: { bool _r = _t->getFlipHorz();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 123: _t->setFlipHorz((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 124: { bool _r = _t->getFlipVert();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 125: _t->setFlipVert((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 126: _t->setFlagGravityLabels((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 127: _t->loadSkyImage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[8])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[9])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[10])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[11])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[12])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[13])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[14])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[15]))); break;
        case 128: _t->loadSkyImage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[8])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[9])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[10])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[11])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[12])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[13])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[14]))); break;
        case 129: _t->loadSkyImage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[8])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[9])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[10])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[11])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[12])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[13]))); break;
        case 130: _t->loadSkyImage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[8])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[9])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[10])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[11])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[12]))); break;
        case 131: _t->loadSkyImage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[8])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[9])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[10])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[11]))); break;
        case 132: _t->loadSkyImage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[8])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[9])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[10]))); break;
        case 133: _t->loadSkyImage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[8])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[9])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[10])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[11])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[12])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[13])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[14])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[15]))); break;
        case 134: _t->loadSkyImage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[8])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[9])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[10])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[11])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[12])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[13])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[14]))); break;
        case 135: _t->loadSkyImage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[8])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[9])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[10])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[11])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[12])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[13]))); break;
        case 136: _t->loadSkyImage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[8])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[9])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[10])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[11])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[12]))); break;
        case 137: _t->loadSkyImage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[8])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[9])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[10])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[11]))); break;
        case 138: _t->loadSkyImage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[8])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[9])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[10]))); break;
        case 139: _t->loadSkyImage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[8])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[9])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[10])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[11]))); break;
        case 140: _t->loadSkyImage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[8])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[9])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[10]))); break;
        case 141: _t->loadSkyImage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[8])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[9]))); break;
        case 142: _t->loadSkyImage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[8]))); break;
        case 143: _t->loadSkyImage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[7]))); break;
        case 144: _t->loadSkyImage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[6]))); break;
        case 145: _t->loadSkyImage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[8])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[9])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[10])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[11]))); break;
        case 146: _t->loadSkyImage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[8])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[9])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[10]))); break;
        case 147: _t->loadSkyImage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[8])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[9]))); break;
        case 148: _t->loadSkyImage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[8]))); break;
        case 149: _t->loadSkyImage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[7]))); break;
        case 150: _t->loadSkyImage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[6]))); break;
        case 151: _t->removeSkyImage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 152: { QVariantMap _r = _t->getScreenXYFromAltAzi((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QVariantMap*>(_a[0]) = std::move(_r); }  break;
        case 153: _t->say((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 154: _t->loadSound((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 155: _t->playSound((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 156: _t->pauseSound((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 157: _t->stopSound((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 158: _t->dropSound((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 159: { qint64 _r = _t->getSoundPosition((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<qint64*>(_a[0]) = std::move(_r); }  break;
        case 160: { qint64 _r = _t->getSoundDuration((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<qint64*>(_a[0]) = std::move(_r); }  break;
        case 161: _t->loadVideo((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[6]))); break;
        case 162: _t->loadVideo((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[5]))); break;
        case 163: _t->loadVideo((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[4]))); break;
        case 164: _t->playVideo((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 165: _t->playVideo((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 166: _t->playVideoPopout((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[8])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[9]))); break;
        case 167: _t->pauseVideo((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 168: _t->stopVideo((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 169: _t->dropVideo((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 170: _t->seekVideo((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qint64>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 171: _t->seekVideo((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qint64>>(_a[2]))); break;
        case 172: _t->setVideoXY((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4]))); break;
        case 173: _t->setVideoXY((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[3]))); break;
        case 174: _t->setVideoAlpha((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[2]))); break;
        case 175: _t->resizeVideo((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[3]))); break;
        case 176: _t->showVideo((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 177: _t->showVideo((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 178: { qint64 _r = _t->getVideoDuration((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<qint64*>(_a[0]) = std::move(_r); }  break;
        case 179: { qint64 _r = _t->getVideoPosition((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<qint64*>(_a[0]) = std::move(_r); }  break;
        case 180: { int _r = _t->getScreenWidth();
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 181: { int _r = _t->getScreenHeight();
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 182: { double _r = _t->getScriptRate();
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 183: _t->setScriptRate((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 184: _t->pauseScript(); break;
        case 185: _t->setSelectedObjectInfo((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 186: _t->exit(); break;
        case 187: _t->quitStellarium(); break;
        case 188: { QStringList _r = _t->getPropertyList();
            if (_a[0]) *reinterpret_cast<QStringList*>(_a[0]) = std::move(_r); }  break;
        case 189: _t->debug((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 190: _t->output((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 191: { QString _r = _t->mapToString((*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 192: _t->resetOutput(); break;
        case 193: _t->saveOutputAs((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 194: { QString _r = _t->getAppLanguage();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 195: _t->setAppLanguage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 196: { QString _r = _t->translate((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 197: _t->goHome(); break;
        case 198: { int _r = _t->getBortleScaleIndex();
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 199: _t->setBortleScaleIndex((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 200: { double _r = _t->refraction((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])));
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 201: { double _r = _t->refraction((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])));
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 202: { double _r = _t->jdFromDateString((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 203: _t->wait((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 204: _t->waitFor((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 205: _t->waitFor((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 206: { QString _r = _t->getEnv((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 207: { bool _r = _t->isModuleLoaded((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 208: { QString _r = _t->getPlatformName();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 209: { bool _r = _t->isMediaPlaybackSupported();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 210: _t->setDisplayMaxLuminance((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 211: { double _r = _t->getDisplayMaxLuminance();
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 212: _t->setDisplayAdaptationLuminance((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 213: { double _r = _t->getDisplayAdaptationLuminance();
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 214: _t->setDisplayGamma((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 215: { double _r = _t->getDisplayGamma();
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (StelMainScriptAPI::*)(const QString & , const QString & , double , double , double , double , double , double , double , double , double , double , bool , const StelCore::FrameType , bool )>(_a, &StelMainScriptAPI::requestLoadSkyImage, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelMainScriptAPI::*)(const QString & )>(_a, &StelMainScriptAPI::requestRemoveSkyImage, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelMainScriptAPI::*)(const QString & , const QString & )>(_a, &StelMainScriptAPI::requestLoadSound, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelMainScriptAPI::*)(const QString & )>(_a, &StelMainScriptAPI::requestPlaySound, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelMainScriptAPI::*)(const QString & )>(_a, &StelMainScriptAPI::requestPauseSound, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelMainScriptAPI::*)(const QString & )>(_a, &StelMainScriptAPI::requestStopSound, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelMainScriptAPI::*)(const QString & )>(_a, &StelMainScriptAPI::requestDropSound, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelMainScriptAPI::*)(const QString & , const QString & , float , float , bool , float )>(_a, &StelMainScriptAPI::requestLoadVideo, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelMainScriptAPI::*)(const QString & , const bool )>(_a, &StelMainScriptAPI::requestPlayVideo, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelMainScriptAPI::*)(const QString & , float , float , float , float , float , float , float , bool )>(_a, &StelMainScriptAPI::requestPlayVideoPopout, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelMainScriptAPI::*)(const QString & )>(_a, &StelMainScriptAPI::requestPauseVideo, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelMainScriptAPI::*)(const QString & )>(_a, &StelMainScriptAPI::requestStopVideo, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelMainScriptAPI::*)(const QString & )>(_a, &StelMainScriptAPI::requestDropVideo, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelMainScriptAPI::*)(const QString & , qint64 , bool )>(_a, &StelMainScriptAPI::requestSeekVideo, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelMainScriptAPI::*)(const QString & , float , float , bool )>(_a, &StelMainScriptAPI::requestSetVideoXY, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelMainScriptAPI::*)(const QString & , float )>(_a, &StelMainScriptAPI::requestSetVideoAlpha, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelMainScriptAPI::*)(const QString & , float , float )>(_a, &StelMainScriptAPI::requestResizeVideo, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelMainScriptAPI::*)(const QString & , bool )>(_a, &StelMainScriptAPI::requestShowVideo, 19))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelMainScriptAPI::*)(QString )>(_a, &StelMainScriptAPI::requestSetProjectionMode, 20))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelMainScriptAPI::*)(QString )>(_a, &StelMainScriptAPI::requestSetSkyCulture, 21))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelMainScriptAPI::*)(bool )>(_a, &StelMainScriptAPI::requestSetDiskViewport, 22))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelMainScriptAPI::*)()>(_a, &StelMainScriptAPI::requestExit, 23))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelMainScriptAPI::*)()>(_a, &StelMainScriptAPI::requestSetHomePosition, 24))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<double*>(_v) = _t->getJDay(); break;
        case 1: *reinterpret_cast<double*>(_v) = _t->getTimeRate(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setJDay(*reinterpret_cast<double*>(_v)); break;
        case 1: _t->setTimeRate(*reinterpret_cast<double*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *StelMainScriptAPI::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *StelMainScriptAPI::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17StelMainScriptAPIE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int StelMainScriptAPI::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 216)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 216;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 216)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 216;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void StelMainScriptAPI::requestLoadSkyImage(const QString & _t1, const QString & _t2, double _t3, double _t4, double _t5, double _t6, double _t7, double _t8, double _t9, double _t10, double _t11, double _t12, bool _t13, const StelCore::FrameType _t14, bool _t15)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3, _t4, _t5, _t6, _t7, _t8, _t9, _t10, _t11, _t12, _t13, _t14, _t15);
}

// SIGNAL 1
void StelMainScriptAPI::requestRemoveSkyImage(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void StelMainScriptAPI::requestLoadSound(const QString & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void StelMainScriptAPI::requestPlaySound(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void StelMainScriptAPI::requestPauseSound(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void StelMainScriptAPI::requestStopSound(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void StelMainScriptAPI::requestDropSound(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void StelMainScriptAPI::requestLoadVideo(const QString & _t1, const QString & _t2, float _t3, float _t4, bool _t5, float _t6)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1, _t2, _t3, _t4, _t5, _t6);
}

// SIGNAL 8
void StelMainScriptAPI::requestPlayVideo(const QString & _t1, const bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1, _t2);
}

// SIGNAL 9
void StelMainScriptAPI::requestPlayVideoPopout(const QString & _t1, float _t2, float _t3, float _t4, float _t5, float _t6, float _t7, float _t8, bool _t9)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1, _t2, _t3, _t4, _t5, _t6, _t7, _t8, _t9);
}

// SIGNAL 10
void StelMainScriptAPI::requestPauseVideo(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void StelMainScriptAPI::requestStopVideo(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void StelMainScriptAPI::requestDropVideo(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void StelMainScriptAPI::requestSeekVideo(const QString & _t1, qint64 _t2, bool _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1, _t2, _t3);
}

// SIGNAL 15
void StelMainScriptAPI::requestSetVideoXY(const QString & _t1, float _t2, float _t3, bool _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 15, nullptr, _t1, _t2, _t3, _t4);
}

// SIGNAL 17
void StelMainScriptAPI::requestSetVideoAlpha(const QString & _t1, float _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 17, nullptr, _t1, _t2);
}

// SIGNAL 18
void StelMainScriptAPI::requestResizeVideo(const QString & _t1, float _t2, float _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 18, nullptr, _t1, _t2, _t3);
}

// SIGNAL 19
void StelMainScriptAPI::requestShowVideo(const QString & _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 19, nullptr, _t1, _t2);
}

// SIGNAL 20
void StelMainScriptAPI::requestSetProjectionMode(QString _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 20, nullptr, _t1);
}

// SIGNAL 21
void StelMainScriptAPI::requestSetSkyCulture(QString _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 21, nullptr, _t1);
}

// SIGNAL 22
void StelMainScriptAPI::requestSetDiskViewport(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 22, nullptr, _t1);
}

// SIGNAL 23
void StelMainScriptAPI::requestExit()
{
    QMetaObject::activate(this, &staticMetaObject, 23, nullptr);
}

// SIGNAL 24
void StelMainScriptAPI::requestSetHomePosition()
{
    QMetaObject::activate(this, &staticMetaObject, 24, nullptr);
}
QT_WARNING_POP
