/********************************************************************************
** Form generated from reading UI file 'MSConfigDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MSCONFIGDIALOG_H
#define UI_MSCONFIGDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDateTimeEdit>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTextBrowser>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "Dialog.hpp"
#include "gui/ColorButton.hpp"

QT_BEGIN_NAMESPACE

class Ui_MSConfigDialog
{
public:
    QVBoxLayout *verticalLayout_2;
    TitleBar *titleBar;
    QTabWidget *tabs;
    QWidget *generalTab;
    QVBoxLayout *verticalLayout_3;
    QGroupBox *displayMeteorShowerBox;
    QGridLayout *gridLayout_8;
    QCheckBox *enableAtStartUp;
    QCheckBox *showEnableButton;
    QCheckBox *showSearchButton;
    QSpacerItem *verticalSpacer_3;
    QHBoxLayout *horizontalLayout_7;
    QPushButton *bRestoreDefaults;
    QWidget *radiantTab;
    QVBoxLayout *verticalLayout_4;
    QGroupBox *enableMarker;
    QGridLayout *gridLayout_9;
    QCheckBox *enableLabels;
    QCheckBox *activeRadiantsOnly;
    QSpinBox *fontSize;
    QGroupBox *groupBox;
    QGridLayout *gridLayout_5;
    QGroupBox *activeGenericData;
    QGridLayout *gridLayout_3;
    ColorButton *setColorARG;
    QGroupBox *inactive;
    QGridLayout *gridLayout_4;
    ColorButton *setColorIR;
    QGroupBox *activeConfirmedData;
    QGridLayout *gridLayout_2;
    ColorButton *setColorARC;
    QSpacerItem *verticalSpacer;
    QWidget *updatesTab;
    QVBoxLayout *updatesTab_2;
    QGroupBox *enableUpdates;
    QVBoxLayout *verticalLayout_12;
    QHBoxLayout *horizontalLayout;
    QLabel *lUpdateFrequency;
    QSpacerItem *horizontalSpacer;
    QSpinBox *updateFrequency;
    QHBoxLayout *horizontalLayout_2;
    QLabel *lNextUpdate;
    QSpacerItem *horizontalSpacer_2;
    QDateTimeEdit *nextUpdate;
    QGroupBox *groupBox_2;
    QVBoxLayout *verticalLayout;
    QGridLayout *gridLayout_6;
    QLabel *lLastUpdate;
    QLabel *lStatus;
    QDateTimeEdit *lastUpdate;
    QLabel *status;
    QSpacerItem *verticalSpacer_31;
    QPushButton *bUpdate;
    QWidget *aboutTab;
    QVBoxLayout *verticalLayout_11;
    QTextBrowser *about;

    void setupUi(QWidget *MSConfigDialog)
    {
        if (MSConfigDialog->objectName().isEmpty())
            MSConfigDialog->setObjectName("MSConfigDialog");
        MSConfigDialog->resize(530, 404);
        verticalLayout_2 = new QVBoxLayout(MSConfigDialog);
        verticalLayout_2->setSpacing(0);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        titleBar = new TitleBar(MSConfigDialog);
        titleBar->setObjectName("titleBar");

        verticalLayout_2->addWidget(titleBar);

        tabs = new QTabWidget(MSConfigDialog);
        tabs->setObjectName("tabs");
        tabs->setDocumentMode(false);
        generalTab = new QWidget();
        generalTab->setObjectName("generalTab");
        verticalLayout_3 = new QVBoxLayout(generalTab);
        verticalLayout_3->setObjectName("verticalLayout_3");
        displayMeteorShowerBox = new QGroupBox(generalTab);
        displayMeteorShowerBox->setObjectName("displayMeteorShowerBox");
        displayMeteorShowerBox->setFlat(true);
        displayMeteorShowerBox->setCheckable(false);
        gridLayout_8 = new QGridLayout(displayMeteorShowerBox);
        gridLayout_8->setObjectName("gridLayout_8");
        enableAtStartUp = new QCheckBox(displayMeteorShowerBox);
        enableAtStartUp->setObjectName("enableAtStartUp");

        gridLayout_8->addWidget(enableAtStartUp, 0, 0, 1, 1);

        showEnableButton = new QCheckBox(displayMeteorShowerBox);
        showEnableButton->setObjectName("showEnableButton");

        gridLayout_8->addWidget(showEnableButton, 1, 0, 1, 1);

        showSearchButton = new QCheckBox(displayMeteorShowerBox);
        showSearchButton->setObjectName("showSearchButton");

        gridLayout_8->addWidget(showSearchButton, 2, 0, 1, 1);


        verticalLayout_3->addWidget(displayMeteorShowerBox);

        verticalSpacer_3 = new QSpacerItem(20, 120, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_3->addItem(verticalSpacer_3);

        horizontalLayout_7 = new QHBoxLayout();
        horizontalLayout_7->setObjectName("horizontalLayout_7");
        bRestoreDefaults = new QPushButton(generalTab);
        bRestoreDefaults->setObjectName("bRestoreDefaults");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(bRestoreDefaults->sizePolicy().hasHeightForWidth());
        bRestoreDefaults->setSizePolicy(sizePolicy);

        horizontalLayout_7->addWidget(bRestoreDefaults);


        verticalLayout_3->addLayout(horizontalLayout_7);

        tabs->addTab(generalTab, QString());
        radiantTab = new QWidget();
        radiantTab->setObjectName("radiantTab");
        radiantTab->setEnabled(true);
        verticalLayout_4 = new QVBoxLayout(radiantTab);
        verticalLayout_4->setObjectName("verticalLayout_4");
        enableMarker = new QGroupBox(radiantTab);
        enableMarker->setObjectName("enableMarker");
        enableMarker->setFlat(true);
        enableMarker->setCheckable(true);
        gridLayout_9 = new QGridLayout(enableMarker);
        gridLayout_9->setObjectName("gridLayout_9");
        enableLabels = new QCheckBox(enableMarker);
        enableLabels->setObjectName("enableLabels");

        gridLayout_9->addWidget(enableLabels, 1, 0, 1, 1);

        activeRadiantsOnly = new QCheckBox(enableMarker);
        activeRadiantsOnly->setObjectName("activeRadiantsOnly");

        gridLayout_9->addWidget(activeRadiantsOnly, 0, 0, 1, 1);

        fontSize = new QSpinBox(enableMarker);
        fontSize->setObjectName("fontSize");

        gridLayout_9->addWidget(fontSize, 1, 1, 1, 1);


        verticalLayout_4->addWidget(enableMarker);

        groupBox = new QGroupBox(radiantTab);
        groupBox->setObjectName("groupBox");
        gridLayout_5 = new QGridLayout(groupBox);
        gridLayout_5->setSpacing(0);
        gridLayout_5->setObjectName("gridLayout_5");
        gridLayout_5->setContentsMargins(0, 0, 0, 0);
        activeGenericData = new QGroupBox(groupBox);
        activeGenericData->setObjectName("activeGenericData");
        activeGenericData->setMinimumSize(QSize(130, 100));
        activeGenericData->setAlignment(Qt::AlignCenter);
        gridLayout_3 = new QGridLayout(activeGenericData);
        gridLayout_3->setSpacing(0);
        gridLayout_3->setObjectName("gridLayout_3");
        gridLayout_3->setContentsMargins(0, 0, 0, 0);
        setColorARG = new ColorButton(activeGenericData);
        setColorARG->setObjectName("setColorARG");

        gridLayout_3->addWidget(setColorARG, 0, 0, 1, 1);


        gridLayout_5->addWidget(activeGenericData, 0, 1, 1, 1, Qt::AlignHCenter);

        inactive = new QGroupBox(groupBox);
        inactive->setObjectName("inactive");
        inactive->setMinimumSize(QSize(130, 100));
        inactive->setAlignment(Qt::AlignCenter);
        gridLayout_4 = new QGridLayout(inactive);
        gridLayout_4->setSpacing(0);
        gridLayout_4->setObjectName("gridLayout_4");
        gridLayout_4->setContentsMargins(0, 0, 0, 0);
        setColorIR = new ColorButton(inactive);
        setColorIR->setObjectName("setColorIR");

        gridLayout_4->addWidget(setColorIR, 0, 0, 1, 1);


        gridLayout_5->addWidget(inactive, 0, 2, 1, 1, Qt::AlignHCenter);

        activeConfirmedData = new QGroupBox(groupBox);
        activeConfirmedData->setObjectName("activeConfirmedData");
        activeConfirmedData->setMinimumSize(QSize(130, 100));
        activeConfirmedData->setAlignment(Qt::AlignCenter);
        gridLayout_2 = new QGridLayout(activeConfirmedData);
        gridLayout_2->setSpacing(0);
        gridLayout_2->setObjectName("gridLayout_2");
        gridLayout_2->setContentsMargins(0, 0, 0, 0);
        setColorARC = new ColorButton(activeConfirmedData);
        setColorARC->setObjectName("setColorARC");

        gridLayout_2->addWidget(setColorARC, 0, 0, 1, 1);


        gridLayout_5->addWidget(activeConfirmedData, 0, 0, 1, 1, Qt::AlignHCenter);


        verticalLayout_4->addWidget(groupBox);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_4->addItem(verticalSpacer);

        tabs->addTab(radiantTab, QString());
        updatesTab = new QWidget();
        updatesTab->setObjectName("updatesTab");
        updatesTab_2 = new QVBoxLayout(updatesTab);
        updatesTab_2->setObjectName("updatesTab_2");
        enableUpdates = new QGroupBox(updatesTab);
        enableUpdates->setObjectName("enableUpdates");
        enableUpdates->setFlat(true);
        enableUpdates->setCheckable(true);
        enableUpdates->setChecked(true);
        verticalLayout_12 = new QVBoxLayout(enableUpdates);
        verticalLayout_12->setObjectName("verticalLayout_12");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        lUpdateFrequency = new QLabel(enableUpdates);
        lUpdateFrequency->setObjectName("lUpdateFrequency");

        horizontalLayout->addWidget(lUpdateFrequency);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        updateFrequency = new QSpinBox(enableUpdates);
        updateFrequency->setObjectName("updateFrequency");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(updateFrequency->sizePolicy().hasHeightForWidth());
        updateFrequency->setSizePolicy(sizePolicy1);
        updateFrequency->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        updateFrequency->setMinimum(1);
        updateFrequency->setMaximum(9999);
        updateFrequency->setValue(1);

        horizontalLayout->addWidget(updateFrequency);


        verticalLayout_12->addLayout(horizontalLayout);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        lNextUpdate = new QLabel(enableUpdates);
        lNextUpdate->setObjectName("lNextUpdate");

        horizontalLayout_2->addWidget(lNextUpdate);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_2);

        nextUpdate = new QDateTimeEdit(enableUpdates);
        nextUpdate->setObjectName("nextUpdate");
        nextUpdate->setEnabled(false);
        nextUpdate->setFrame(false);
        nextUpdate->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        nextUpdate->setButtonSymbols(QAbstractSpinBox::NoButtons);

        horizontalLayout_2->addWidget(nextUpdate);


        verticalLayout_12->addLayout(horizontalLayout_2);


        updatesTab_2->addWidget(enableUpdates);

        groupBox_2 = new QGroupBox(updatesTab);
        groupBox_2->setObjectName("groupBox_2");
        verticalLayout = new QVBoxLayout(groupBox_2);
        verticalLayout->setObjectName("verticalLayout");
        gridLayout_6 = new QGridLayout();
        gridLayout_6->setObjectName("gridLayout_6");
        lLastUpdate = new QLabel(groupBox_2);
        lLastUpdate->setObjectName("lLastUpdate");

        gridLayout_6->addWidget(lLastUpdate, 0, 0, 1, 1);

        lStatus = new QLabel(groupBox_2);
        lStatus->setObjectName("lStatus");

        gridLayout_6->addWidget(lStatus, 1, 0, 1, 1);

        lastUpdate = new QDateTimeEdit(groupBox_2);
        lastUpdate->setObjectName("lastUpdate");
        lastUpdate->setEnabled(false);
        sizePolicy1.setHeightForWidth(lastUpdate->sizePolicy().hasHeightForWidth());
        lastUpdate->setSizePolicy(sizePolicy1);
        lastUpdate->setFrame(false);
        lastUpdate->setButtonSymbols(QAbstractSpinBox::NoButtons);

        gridLayout_6->addWidget(lastUpdate, 0, 1, 1, 1);

        status = new QLabel(groupBox_2);
        status->setObjectName("status");

        gridLayout_6->addWidget(status, 1, 1, 1, 1);


        verticalLayout->addLayout(gridLayout_6);


        updatesTab_2->addWidget(groupBox_2);

        verticalSpacer_31 = new QSpacerItem(20, 120, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        updatesTab_2->addItem(verticalSpacer_31);

        bUpdate = new QPushButton(updatesTab);
        bUpdate->setObjectName("bUpdate");
        sizePolicy1.setHeightForWidth(bUpdate->sizePolicy().hasHeightForWidth());
        bUpdate->setSizePolicy(sizePolicy1);

        updatesTab_2->addWidget(bUpdate);

        tabs->addTab(updatesTab, QString());
        aboutTab = new QWidget();
        aboutTab->setObjectName("aboutTab");
        verticalLayout_11 = new QVBoxLayout(aboutTab);
        verticalLayout_11->setSpacing(9);
        verticalLayout_11->setObjectName("verticalLayout_11");
        about = new QTextBrowser(aboutTab);
        about->setObjectName("about");
        about->setOpenExternalLinks(true);
        about->setOpenLinks(true);

        verticalLayout_11->addWidget(about);

        tabs->addTab(aboutTab, QString());

        verticalLayout_2->addWidget(tabs);

        tabs->raise();
        titleBar->raise();

        retranslateUi(MSConfigDialog);

        tabs->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(MSConfigDialog);
    } // setupUi

    void retranslateUi(QWidget *MSConfigDialog)
    {
        MSConfigDialog->setWindowTitle(QCoreApplication::translate("MSConfigDialog", "Meteor Showers Configuration", nullptr));
        titleBar->setTitle(QCoreApplication::translate("MSConfigDialog", "Meteor Showers: Settings", nullptr));
#if QT_CONFIG(tooltip)
        displayMeteorShowerBox->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
        displayMeteorShowerBox->setTitle(QCoreApplication::translate("MSConfigDialog", "General settings", nullptr));
        enableAtStartUp->setText(QCoreApplication::translate("MSConfigDialog", "Enable plugin at startup", nullptr));
        showEnableButton->setText(QCoreApplication::translate("MSConfigDialog", "Show a button to enable/disable the plugin", nullptr));
        showSearchButton->setText(QCoreApplication::translate("MSConfigDialog", "Show a button to open the search panel", nullptr));
        bRestoreDefaults->setText(QCoreApplication::translate("MSConfigDialog", "Restore defaults", nullptr));
        tabs->setTabText(tabs->indexOf(generalTab), QCoreApplication::translate("MSConfigDialog", "General", nullptr));
        enableMarker->setTitle(QCoreApplication::translate("MSConfigDialog", "Show radiant marker", nullptr));
        enableLabels->setText(QCoreApplication::translate("MSConfigDialog", "Show radiant labels", nullptr));
        activeRadiantsOnly->setText(QCoreApplication::translate("MSConfigDialog", "Show active radiants only", nullptr));
#if QT_CONFIG(tooltip)
        fontSize->setToolTip(QCoreApplication::translate("MSConfigDialog", "Label font size (pixels)", nullptr));
#endif // QT_CONFIG(tooltip)
        groupBox->setTitle(QCoreApplication::translate("MSConfigDialog", "Set radiant color", nullptr));
#if QT_CONFIG(tooltip)
        activeGenericData->setToolTip(QCoreApplication::translate("MSConfigDialog", "Active Radiant - Generic Data", nullptr));
#endif // QT_CONFIG(tooltip)
        activeGenericData->setTitle(QCoreApplication::translate("MSConfigDialog", "Generic", nullptr));
        setColorARG->setText(QString());
#if QT_CONFIG(tooltip)
        inactive->setToolTip(QCoreApplication::translate("MSConfigDialog", "Inactive Radiant", nullptr));
#endif // QT_CONFIG(tooltip)
        inactive->setTitle(QCoreApplication::translate("MSConfigDialog", "Inactive", nullptr));
        setColorIR->setText(QString());
#if QT_CONFIG(tooltip)
        activeConfirmedData->setToolTip(QCoreApplication::translate("MSConfigDialog", "Active Radiant - Confirmed Data", nullptr));
#endif // QT_CONFIG(tooltip)
        activeConfirmedData->setTitle(QCoreApplication::translate("MSConfigDialog", "Confirmed", nullptr));
        setColorARC->setText(QString());
        tabs->setTabText(tabs->indexOf(radiantTab), QCoreApplication::translate("MSConfigDialog", "Radiant", nullptr));
        enableUpdates->setTitle(QCoreApplication::translate("MSConfigDialog", "Enable automatic updates", nullptr));
        lUpdateFrequency->setText(QCoreApplication::translate("MSConfigDialog", "Update frequency:", nullptr));
        lNextUpdate->setText(QCoreApplication::translate("MSConfigDialog", "Next update:", nullptr));
        groupBox_2->setTitle(QCoreApplication::translate("MSConfigDialog", "Last attempt to update", nullptr));
        lLastUpdate->setText(QCoreApplication::translate("MSConfigDialog", "Date:", nullptr));
        lStatus->setText(QCoreApplication::translate("MSConfigDialog", "Status:", nullptr));
        status->setText(QString());
        bUpdate->setText(QCoreApplication::translate("MSConfigDialog", "Update now", nullptr));
        tabs->setTabText(tabs->indexOf(updatesTab), QCoreApplication::translate("MSConfigDialog", "Update", nullptr));
        tabs->setTabText(tabs->indexOf(aboutTab), QCoreApplication::translate("MSConfigDialog", "About", "tab in plugin windows"));
    } // retranslateUi

};

namespace Ui {
    class MSConfigDialog: public Ui_MSConfigDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_MSCONFIGDIALOG_H
