/****************************************************************************
** Meta object code from reading C++ file 'ObsListDialog.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../stellarium/src/gui/ObsListDialog.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ObsListDialog.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13ObsListDialogE_t {};
} // unnamed namespace

template <> constexpr inline auto ObsListDialog::qt_create_metaobjectdata<qt_meta_tag_ZN13ObsListDialogE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ObsListDialog",
        "flagUseJDChanged",
        "",
        "b",
        "flagUseLandscapeChanged",
        "flagUseLocationChanged",
        "flagUseFovChanged",
        "retranslate",
        "getFlagUseJD",
        "getFlagUseLandscape",
        "getFlagUseLocation",
        "getFlagUseFov",
        "setFlagUseJD",
        "setFlagUseLandscape",
        "setFlagUseLocation",
        "setFlagUseFov",
        "highlightAll",
        "clearHighlights",
        "loadSelectedObservingList",
        "selectedIndex",
        "selectAndGoToObject",
        "QModelIndex",
        "index",
        "newListButtonPressed",
        "editListButtonPressed",
        "deleteListButtonPressed",
        "addObjectButtonPressed",
        "removeObjectButtonPressed",
        "exportListButtonPressed",
        "importListButtonPressed",
        "saveButtonPressed",
        "cancelButtonPressed",
        "headerClicked",
        "defaultClicked",
        "browseForObsListDir",
        "selectObsListDir",
        "flagUseJD",
        "flagUseLandscape",
        "flagUseLocation",
        "flagUseFov",
        "ObsListColumns",
        "ColumnUUID",
        "ColumnDesignation",
        "ColumnNameI18n",
        "ColumnType",
        "ColumnRa",
        "ColumnDec",
        "ColumnMagnitude",
        "ColumnConstellation",
        "ColumnDate",
        "ColumnLocation",
        "ColumnLandscapeID",
        "ColumnCount"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'flagUseJDChanged'
        QtMocHelpers::SignalData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagUseLandscapeChanged'
        QtMocHelpers::SignalData<void(bool)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagUseLocationChanged'
        QtMocHelpers::SignalData<void(bool)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagUseFovChanged'
        QtMocHelpers::SignalData<void(bool)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'retranslate'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'getFlagUseJD'
        QtMocHelpers::SlotData<bool()>(8, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'getFlagUseLandscape'
        QtMocHelpers::SlotData<bool()>(9, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'getFlagUseLocation'
        QtMocHelpers::SlotData<bool()>(10, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'getFlagUseFov'
        QtMocHelpers::SlotData<bool()>(11, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFlagUseJD'
        QtMocHelpers::SlotData<void(bool)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'setFlagUseLandscape'
        QtMocHelpers::SlotData<void(bool)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'setFlagUseLocation'
        QtMocHelpers::SlotData<void(bool)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'setFlagUseFov'
        QtMocHelpers::SlotData<void(bool)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'highlightAll'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'clearHighlights'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'loadSelectedObservingList'
        QtMocHelpers::SlotData<void(int)>(18, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 19 },
        }}),
        // Slot 'selectAndGoToObject'
        QtMocHelpers::SlotData<void(QModelIndex)>(20, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 21, 22 },
        }}),
        // Slot 'newListButtonPressed'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'editListButtonPressed'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'deleteListButtonPressed'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'addObjectButtonPressed'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'removeObjectButtonPressed'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'exportListButtonPressed'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'importListButtonPressed'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'saveButtonPressed'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'cancelButtonPressed'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'headerClicked'
        QtMocHelpers::SlotData<void(int)>(32, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 22 },
        }}),
        // Slot 'defaultClicked'
        QtMocHelpers::SlotData<void(bool)>(33, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'browseForObsListDir'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'selectObsListDir'
        QtMocHelpers::SlotData<void()>(35, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'flagUseJD'
        QtMocHelpers::PropertyData<bool>(36, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'flagUseLandscape'
        QtMocHelpers::PropertyData<bool>(37, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'flagUseLocation'
        QtMocHelpers::PropertyData<bool>(38, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'flagUseFov'
        QtMocHelpers::PropertyData<bool>(39, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'ObsListColumns'
        QtMocHelpers::EnumData<enum ObsListColumns>(40, 40, QMC::EnumFlags{}).add({
            {   41, ObsListColumns::ColumnUUID },
            {   42, ObsListColumns::ColumnDesignation },
            {   43, ObsListColumns::ColumnNameI18n },
            {   44, ObsListColumns::ColumnType },
            {   45, ObsListColumns::ColumnRa },
            {   46, ObsListColumns::ColumnDec },
            {   47, ObsListColumns::ColumnMagnitude },
            {   48, ObsListColumns::ColumnConstellation },
            {   49, ObsListColumns::ColumnDate },
            {   50, ObsListColumns::ColumnLocation },
            {   51, ObsListColumns::ColumnLandscapeID },
            {   52, ObsListColumns::ColumnCount },
        }),
    };
    return QtMocHelpers::metaObjectData<ObsListDialog, qt_meta_tag_ZN13ObsListDialogE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ObsListDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<StelDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13ObsListDialogE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13ObsListDialogE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13ObsListDialogE_t>.metaTypes,
    nullptr
} };

void ObsListDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ObsListDialog *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->flagUseJDChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->flagUseLandscapeChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->flagUseLocationChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->flagUseFovChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->retranslate(); break;
        case 5: { bool _r = _t->getFlagUseJD();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 6: { bool _r = _t->getFlagUseLandscape();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 7: { bool _r = _t->getFlagUseLocation();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 8: { bool _r = _t->getFlagUseFov();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 9: _t->setFlagUseJD((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 10: _t->setFlagUseLandscape((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 11: _t->setFlagUseLocation((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 12: _t->setFlagUseFov((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 13: _t->highlightAll(); break;
        case 14: _t->clearHighlights(); break;
        case 15: _t->loadSelectedObservingList((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 16: _t->selectAndGoToObject((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 17: _t->newListButtonPressed(); break;
        case 18: _t->editListButtonPressed(); break;
        case 19: _t->deleteListButtonPressed(); break;
        case 20: _t->addObjectButtonPressed(); break;
        case 21: _t->removeObjectButtonPressed(); break;
        case 22: _t->exportListButtonPressed(); break;
        case 23: _t->importListButtonPressed(); break;
        case 24: _t->saveButtonPressed(); break;
        case 25: _t->cancelButtonPressed(); break;
        case 26: _t->headerClicked((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 27: _t->defaultClicked((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 28: _t->browseForObsListDir(); break;
        case 29: _t->selectObsListDir(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ObsListDialog::*)(bool )>(_a, &ObsListDialog::flagUseJDChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ObsListDialog::*)(bool )>(_a, &ObsListDialog::flagUseLandscapeChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ObsListDialog::*)(bool )>(_a, &ObsListDialog::flagUseLocationChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ObsListDialog::*)(bool )>(_a, &ObsListDialog::flagUseFovChanged, 3))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->getFlagUseJD(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->getFlagUseLandscape(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->getFlagUseLocation(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->getFlagUseFov(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setFlagUseJD(*reinterpret_cast<bool*>(_v)); break;
        case 1: _t->setFlagUseLandscape(*reinterpret_cast<bool*>(_v)); break;
        case 2: _t->setFlagUseLocation(*reinterpret_cast<bool*>(_v)); break;
        case 3: _t->setFlagUseFov(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *ObsListDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ObsListDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13ObsListDialogE_t>.strings))
        return static_cast<void*>(this);
    return StelDialog::qt_metacast(_clname);
}

int ObsListDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StelDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 30)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 30;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 30)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 30;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void ObsListDialog::flagUseJDChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void ObsListDialog::flagUseLandscapeChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void ObsListDialog::flagUseLocationChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void ObsListDialog::flagUseFovChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}
QT_WARNING_POP
