/********************************************************************************
** Form generated from reading UI file 'dateTimeDialogGui.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DATETIMEDIALOGGUI_H
#define UI_DATETIMEDIALOGGUI_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QWidget>
#include "Dialog.hpp"
#include "ExternalStepSpinBox.hpp"
#include "gui/StelCloseButton.hpp"

QT_BEGIN_NAMESPACE

class Ui_dateTimeDialogForm
{
public:
    QGridLayout *gridLayout;
    TitleBar *titleBar;
    QTabWidget *dateTimeTabWidget;
    QWidget *dateTimeTab;
    QGridLayout *gridLayout_2;
    QFrame *dateFrame;
    QHBoxLayout *horizontalLayout;
    QSpinBox *spinner_year;
    QLabel *dateDelimiterLabel1;
    ExternalStepSpinBox *spinner_month;
    QLabel *dateDelimiterLabel2;
    ExternalStepSpinBox *spinner_day;
    QFrame *timeFrame;
    QHBoxLayout *horizontalLayout_2;
    ExternalStepSpinBox *spinner_hour;
    QLabel *label_3;
    ExternalStepSpinBox *spinner_minute;
    QLabel *label_4;
    ExternalStepSpinBox *spinner_second;
    QWidget *julianDateTab;
    QGridLayout *gridLayout_3;
    QFrame *jdFrame;
    QHBoxLayout *horizontalLayout_3;
    QLabel *label_5;
    QDoubleSpinBox *spinner_jd;
    QFrame *mjdFrame;
    QHBoxLayout *horizontalLayout_5;
    QLabel *label_6;
    QDoubleSpinBox *spinner_mjd;

    void setupUi(QWidget *dateTimeDialogForm)
    {
        if (dateTimeDialogForm->objectName().isEmpty())
            dateTimeDialogForm->setObjectName("dateTimeDialogForm");
        dateTimeDialogForm->resize(334, 85);
        dateTimeDialogForm->setStyleSheet(QString::fromUtf8(""));
        gridLayout = new QGridLayout(dateTimeDialogForm);
        gridLayout->setSpacing(0);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setContentsMargins(0, 0, 0, 0);
        titleBar = new TitleBar(dateTimeDialogForm);
        titleBar->setObjectName("titleBar");

        gridLayout->addWidget(titleBar, 0, 0, 1, 1);

        dateTimeTabWidget = new QTabWidget(dateTimeDialogForm);
        dateTimeTabWidget->setObjectName("dateTimeTabWidget");
        dateTimeTab = new QWidget();
        dateTimeTab->setObjectName("dateTimeTab");
        gridLayout_2 = new QGridLayout(dateTimeTab);
        gridLayout_2->setObjectName("gridLayout_2");
        gridLayout_2->setContentsMargins(0, 0, 0, 0);
        dateFrame = new QFrame(dateTimeTab);
        dateFrame->setObjectName("dateFrame");
        dateFrame->setFrameShape(QFrame::StyledPanel);
        horizontalLayout = new QHBoxLayout(dateFrame);
        horizontalLayout->setSpacing(0);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        spinner_year = new QSpinBox(dateFrame);
        spinner_year->setObjectName("spinner_year");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(spinner_year->sizePolicy().hasHeightForWidth());
        spinner_year->setSizePolicy(sizePolicy);
        spinner_year->setFocusPolicy(Qt::StrongFocus);
        spinner_year->setContextMenuPolicy(Qt::NoContextMenu);
        spinner_year->setWrapping(false);
        spinner_year->setFrame(true);
        spinner_year->setAlignment(Qt::AlignCenter);
        spinner_year->setMinimum(-200000);
        spinner_year->setMaximum(200000);

        horizontalLayout->addWidget(spinner_year);

        dateDelimiterLabel1 = new QLabel(dateFrame);
        dateDelimiterLabel1->setObjectName("dateDelimiterLabel1");
        dateDelimiterLabel1->setEnabled(true);
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(dateDelimiterLabel1->sizePolicy().hasHeightForWidth());
        dateDelimiterLabel1->setSizePolicy(sizePolicy1);
        dateDelimiterLabel1->setMaximumSize(QSize(10, 16777215));
        dateDelimiterLabel1->setText(QString::fromUtf8("\342\200\222"));
        dateDelimiterLabel1->setAlignment(Qt::AlignCenter);

        horizontalLayout->addWidget(dateDelimiterLabel1);

        spinner_month = new ExternalStepSpinBox(dateFrame);
        spinner_month->setObjectName("spinner_month");
        sizePolicy.setHeightForWidth(spinner_month->sizePolicy().hasHeightForWidth());
        spinner_month->setSizePolicy(sizePolicy);
        spinner_month->setFocusPolicy(Qt::StrongFocus);
        spinner_month->setContextMenuPolicy(Qt::NoContextMenu);
        spinner_month->setFrame(true);
        spinner_month->setAlignment(Qt::AlignCenter);
        spinner_month->setMinimum(1);
        spinner_month->setMaximum(12);

        horizontalLayout->addWidget(spinner_month);

        dateDelimiterLabel2 = new QLabel(dateFrame);
        dateDelimiterLabel2->setObjectName("dateDelimiterLabel2");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(dateDelimiterLabel2->sizePolicy().hasHeightForWidth());
        dateDelimiterLabel2->setSizePolicy(sizePolicy2);
        dateDelimiterLabel2->setMaximumSize(QSize(10, 16777215));
        dateDelimiterLabel2->setText(QString::fromUtf8("\342\200\222"));
        dateDelimiterLabel2->setAlignment(Qt::AlignCenter);

        horizontalLayout->addWidget(dateDelimiterLabel2);

        spinner_day = new ExternalStepSpinBox(dateFrame);
        spinner_day->setObjectName("spinner_day");
        sizePolicy.setHeightForWidth(spinner_day->sizePolicy().hasHeightForWidth());
        spinner_day->setSizePolicy(sizePolicy);
        spinner_day->setFocusPolicy(Qt::StrongFocus);
        spinner_day->setContextMenuPolicy(Qt::NoContextMenu);
        spinner_day->setFrame(true);
        spinner_day->setAlignment(Qt::AlignCenter);
        spinner_day->setMinimum(1);
        spinner_day->setMaximum(31);

        horizontalLayout->addWidget(spinner_day);


        gridLayout_2->addWidget(dateFrame, 0, 0, 1, 1);

        timeFrame = new QFrame(dateTimeTab);
        timeFrame->setObjectName("timeFrame");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(timeFrame->sizePolicy().hasHeightForWidth());
        timeFrame->setSizePolicy(sizePolicy3);
        QFont font;
        timeFrame->setFont(font);
        timeFrame->setFrameShape(QFrame::StyledPanel);
        horizontalLayout_2 = new QHBoxLayout(timeFrame);
        horizontalLayout_2->setSpacing(0);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        horizontalLayout_2->setContentsMargins(0, 0, 0, 0);
        spinner_hour = new ExternalStepSpinBox(timeFrame);
        spinner_hour->setObjectName("spinner_hour");
        sizePolicy.setHeightForWidth(spinner_hour->sizePolicy().hasHeightForWidth());
        spinner_hour->setSizePolicy(sizePolicy);
        spinner_hour->setFocusPolicy(Qt::StrongFocus);
        spinner_hour->setContextMenuPolicy(Qt::NoContextMenu);
        spinner_hour->setAlignment(Qt::AlignCenter);
        spinner_hour->setMinimum(0);
        spinner_hour->setMaximum(23);

        horizontalLayout_2->addWidget(spinner_hour);

        label_3 = new QLabel(timeFrame);
        label_3->setObjectName("label_3");
        sizePolicy2.setHeightForWidth(label_3->sizePolicy().hasHeightForWidth());
        label_3->setSizePolicy(sizePolicy2);
        label_3->setMaximumSize(QSize(10, 16777215));
        label_3->setText(QString::fromUtf8(":"));
        label_3->setAlignment(Qt::AlignCenter);

        horizontalLayout_2->addWidget(label_3);

        spinner_minute = new ExternalStepSpinBox(timeFrame);
        spinner_minute->setObjectName("spinner_minute");
        sizePolicy.setHeightForWidth(spinner_minute->sizePolicy().hasHeightForWidth());
        spinner_minute->setSizePolicy(sizePolicy);
        spinner_minute->setFocusPolicy(Qt::StrongFocus);
        spinner_minute->setContextMenuPolicy(Qt::NoContextMenu);
        spinner_minute->setAlignment(Qt::AlignCenter);
        spinner_minute->setMinimum(0);
        spinner_minute->setMaximum(59);

        horizontalLayout_2->addWidget(spinner_minute);

        label_4 = new QLabel(timeFrame);
        label_4->setObjectName("label_4");
        sizePolicy2.setHeightForWidth(label_4->sizePolicy().hasHeightForWidth());
        label_4->setSizePolicy(sizePolicy2);
        label_4->setMaximumSize(QSize(10, 16777215));
        label_4->setAutoFillBackground(false);
        label_4->setText(QString::fromUtf8(":"));
        label_4->setAlignment(Qt::AlignCenter);

        horizontalLayout_2->addWidget(label_4);

        spinner_second = new ExternalStepSpinBox(timeFrame);
        spinner_second->setObjectName("spinner_second");
        sizePolicy.setHeightForWidth(spinner_second->sizePolicy().hasHeightForWidth());
        spinner_second->setSizePolicy(sizePolicy);
        spinner_second->setFocusPolicy(Qt::StrongFocus);
        spinner_second->setContextMenuPolicy(Qt::NoContextMenu);
        spinner_second->setFrame(true);
        spinner_second->setAlignment(Qt::AlignCenter);
        spinner_second->setButtonSymbols(QAbstractSpinBox::UpDownArrows);
        spinner_second->setAccelerated(true);
        spinner_second->setMinimum(0);
        spinner_second->setMaximum(59);

        horizontalLayout_2->addWidget(spinner_second);


        gridLayout_2->addWidget(timeFrame, 0, 1, 1, 1);

        dateTimeTabWidget->addTab(dateTimeTab, QString());
        julianDateTab = new QWidget();
        julianDateTab->setObjectName("julianDateTab");
        gridLayout_3 = new QGridLayout(julianDateTab);
        gridLayout_3->setObjectName("gridLayout_3");
        gridLayout_3->setContentsMargins(0, 0, 0, 0);
        jdFrame = new QFrame(julianDateTab);
        jdFrame->setObjectName("jdFrame");
        jdFrame->setFrameShape(QFrame::StyledPanel);
        jdFrame->setFrameShadow(QFrame::Plain);
        horizontalLayout_3 = new QHBoxLayout(jdFrame);
        horizontalLayout_3->setSpacing(0);
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        horizontalLayout_3->setContentsMargins(5, 0, 0, 0);
        label_5 = new QLabel(jdFrame);
        label_5->setObjectName("label_5");
        label_5->setText(QString::fromUtf8("JD:"));
        label_5->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);

        horizontalLayout_3->addWidget(label_5);

        spinner_jd = new QDoubleSpinBox(jdFrame);
        spinner_jd->setObjectName("spinner_jd");
        spinner_jd->setFocusPolicy(Qt::StrongFocus);
        spinner_jd->setContextMenuPolicy(Qt::NoContextMenu);
        spinner_jd->setAlignment(Qt::AlignCenter);
        spinner_jd->setDecimals(5);
        spinner_jd->setMinimum(-100000000.000000000000000);
        spinner_jd->setMaximum(100000000.000000000000000);
        spinner_jd->setSingleStep(0.000010000000000);

        horizontalLayout_3->addWidget(spinner_jd);


        gridLayout_3->addWidget(jdFrame, 0, 1, 1, 1);

        mjdFrame = new QFrame(julianDateTab);
        mjdFrame->setObjectName("mjdFrame");
        mjdFrame->setFrameShape(QFrame::StyledPanel);
        mjdFrame->setFrameShadow(QFrame::Plain);
        horizontalLayout_5 = new QHBoxLayout(mjdFrame);
        horizontalLayout_5->setSpacing(0);
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        horizontalLayout_5->setContentsMargins(5, 0, 0, 0);
        label_6 = new QLabel(mjdFrame);
        label_6->setObjectName("label_6");
        label_6->setText(QString::fromUtf8("MJD:"));
        label_6->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);

        horizontalLayout_5->addWidget(label_6);

        spinner_mjd = new QDoubleSpinBox(mjdFrame);
        spinner_mjd->setObjectName("spinner_mjd");
        spinner_mjd->setFocusPolicy(Qt::StrongFocus);
        spinner_mjd->setContextMenuPolicy(Qt::NoContextMenu);
        spinner_mjd->setAlignment(Qt::AlignCenter);
        spinner_mjd->setDecimals(5);
        spinner_mjd->setMinimum(-100000000.000000000000000);
        spinner_mjd->setMaximum(100000000.000000000000000);
        spinner_mjd->setSingleStep(0.000010000000000);

        horizontalLayout_5->addWidget(spinner_mjd);


        gridLayout_3->addWidget(mjdFrame, 0, 2, 1, 1);

        dateTimeTabWidget->addTab(julianDateTab, QString());

        gridLayout->addWidget(dateTimeTabWidget, 1, 0, 1, 1);


        retranslateUi(dateTimeDialogForm);

        dateTimeTabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(dateTimeDialogForm);
    } // setupUi

    void retranslateUi(QWidget *dateTimeDialogForm)
    {
        titleBar->setProperty("title", QVariant(QCoreApplication::translate("dateTimeDialogForm", "Date and Time", nullptr)));
        spinner_second->setPrefix(QString());
        dateTimeTabWidget->setTabText(dateTimeTabWidget->indexOf(dateTimeTab), QCoreApplication::translate("dateTimeDialogForm", "Date and Time", nullptr));
#if QT_CONFIG(tooltip)
        label_5->setToolTip(QCoreApplication::translate("dateTimeDialogForm", "Julian Day", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        label_6->setToolTip(QCoreApplication::translate("dateTimeDialogForm", "Modified Julian Day", nullptr));
#endif // QT_CONFIG(tooltip)
        dateTimeTabWidget->setTabText(dateTimeTabWidget->indexOf(julianDateTab), QCoreApplication::translate("dateTimeDialogForm", "Julian Day", nullptr));
        (void)dateTimeDialogForm;
    } // retranslateUi

};

namespace Ui {
    class dateTimeDialogForm: public Ui_dateTimeDialogForm {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DATETIMEDIALOGGUI_H
