/********************************************************************************
** Form generated from reading UI file 'configureScreenshotsDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CONFIGURESCREENSHOTSDIALOG_H
#define UI_CONFIGURESCREENSHOTSDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "Dialog.hpp"
#include "gui/StelCloseButton.hpp"

QT_BEGIN_NAMESPACE

class Ui_configureScreenshotsDialogForm
{
public:
    QVBoxLayout *verticalLayout;
    TitleBar *titleBar;
    QGroupBox *useDateTimeFileMask;
    QGridLayout *gridLayout;
    QLabel *labelMask;
    QSpacerItem *verticalSpacer;
    QLineEdit *maskLineEdit;

    void setupUi(QWidget *configureScreenshotsDialogForm)
    {
        if (configureScreenshotsDialogForm->objectName().isEmpty())
            configureScreenshotsDialogForm->setObjectName("configureScreenshotsDialogForm");
        configureScreenshotsDialogForm->resize(335, 204);
        verticalLayout = new QVBoxLayout(configureScreenshotsDialogForm);
        verticalLayout->setSpacing(0);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        titleBar = new TitleBar(configureScreenshotsDialogForm);
        titleBar->setObjectName("titleBar");

        verticalLayout->addWidget(titleBar);

        useDateTimeFileMask = new QGroupBox(configureScreenshotsDialogForm);
        useDateTimeFileMask->setObjectName("useDateTimeFileMask");
        useDateTimeFileMask->setCheckable(true);
        gridLayout = new QGridLayout(useDateTimeFileMask);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setContentsMargins(0, 0, 0, 0);
        labelMask = new QLabel(useDateTimeFileMask);
        labelMask->setObjectName("labelMask");

        gridLayout->addWidget(labelMask, 0, 0, 1, 1);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(verticalSpacer, 1, 0, 1, 1);

        maskLineEdit = new QLineEdit(useDateTimeFileMask);
        maskLineEdit->setObjectName("maskLineEdit");
        maskLineEdit->setInputMethodHints(Qt::ImhDate|Qt::ImhTime);
        maskLineEdit->setInputMask(QString::fromUtf8(""));
        maskLineEdit->setText(QString::fromUtf8(""));

        gridLayout->addWidget(maskLineEdit, 0, 1, 1, 1);


        verticalLayout->addWidget(useDateTimeFileMask);


        retranslateUi(configureScreenshotsDialogForm);

        QMetaObject::connectSlotsByName(configureScreenshotsDialogForm);
    } // setupUi

    void retranslateUi(QWidget *configureScreenshotsDialogForm)
    {
        titleBar->setTitle(QCoreApplication::translate("configureScreenshotsDialogForm", "Screenshots naming", nullptr));
        useDateTimeFileMask->setTitle(QCoreApplication::translate("configureScreenshotsDialogForm", "Use time mask for naming screenshots", nullptr));
        labelMask->setText(QCoreApplication::translate("configureScreenshotsDialogForm", "Filename mask:", nullptr));
        (void)configureScreenshotsDialogForm;
    } // retranslateUi

};

namespace Ui {
    class configureScreenshotsDialogForm: public Ui_configureScreenshotsDialogForm {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_CONFIGURESCREENSHOTSDIALOG_H
