/****************************************************************************
** Meta object code from reading C++ file 'StelLocationMgr_p.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../stellarium/src/core/StelLocationMgr_p.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'StelLocationMgr_p.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15GPSLookupHelperE_t {};
} // unnamed namespace

template <> constexpr inline auto GPSLookupHelper::qt_create_metaobjectdata<qt_meta_tag_ZN15GPSLookupHelperE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "GPSLookupHelper",
        "queryFinished",
        "",
        "StelLocation",
        "loc",
        "queryError",
        "msg",
        "query",
        "setPeriodicQuery",
        "interval"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'queryFinished'
        QtMocHelpers::SignalData<void(const StelLocation &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'queryError'
        QtMocHelpers::SignalData<void(const QString &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 },
        }}),
        // Slot 'query'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setPeriodicQuery'
        QtMocHelpers::SlotData<void(int)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 9 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<GPSLookupHelper, qt_meta_tag_ZN15GPSLookupHelperE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject GPSLookupHelper::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15GPSLookupHelperE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15GPSLookupHelperE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15GPSLookupHelperE_t>.metaTypes,
    nullptr
} };

void GPSLookupHelper::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<GPSLookupHelper *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->queryFinished((*reinterpret_cast<std::add_pointer_t<StelLocation>>(_a[1]))); break;
        case 1: _t->queryError((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->query(); break;
        case 3: _t->setPeriodicQuery((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< StelLocation >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (GPSLookupHelper::*)(const StelLocation & )>(_a, &GPSLookupHelper::queryFinished, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (GPSLookupHelper::*)(const QString & )>(_a, &GPSLookupHelper::queryError, 1))
            return;
    }
}

const QMetaObject *GPSLookupHelper::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *GPSLookupHelper::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15GPSLookupHelperE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int GPSLookupHelper::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void GPSLookupHelper::queryFinished(const StelLocation & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void GPSLookupHelper::queryError(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN18LibGPSLookupHelperE_t {};
} // unnamed namespace

template <> constexpr inline auto LibGPSLookupHelper::qt_create_metaobjectdata<qt_meta_tag_ZN18LibGPSLookupHelperE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "LibGPSLookupHelper",
        "query",
        "",
        "setPeriodicQuery",
        "interval"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'query'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setPeriodicQuery'
        QtMocHelpers::SlotData<void(int)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<LibGPSLookupHelper, qt_meta_tag_ZN18LibGPSLookupHelperE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject LibGPSLookupHelper::staticMetaObject = { {
    QMetaObject::SuperData::link<GPSLookupHelper::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18LibGPSLookupHelperE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18LibGPSLookupHelperE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18LibGPSLookupHelperE_t>.metaTypes,
    nullptr
} };

void LibGPSLookupHelper::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<LibGPSLookupHelper *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->query(); break;
        case 1: _t->setPeriodicQuery((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *LibGPSLookupHelper::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *LibGPSLookupHelper::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18LibGPSLookupHelperE_t>.strings))
        return static_cast<void*>(this);
    return GPSLookupHelper::qt_metacast(_clname);
}

int LibGPSLookupHelper::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = GPSLookupHelper::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN16NMEALookupHelperE_t {};
} // unnamed namespace

template <> constexpr inline auto NMEALookupHelper::qt_create_metaobjectdata<qt_meta_tag_ZN16NMEALookupHelperE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "NMEALookupHelper",
        "query",
        "",
        "setPeriodicQuery",
        "interval",
        "nmeaUpdated",
        "QGeoPositionInfo",
        "update",
        "nmeaError",
        "QGeoPositionInfoSource::Error",
        "error"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'query'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setPeriodicQuery'
        QtMocHelpers::SlotData<void(int)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 4 },
        }}),
        // Slot 'nmeaUpdated'
        QtMocHelpers::SlotData<void(const QGeoPositionInfo &)>(5, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'nmeaError'
        QtMocHelpers::SlotData<void(QGeoPositionInfoSource::Error)>(8, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<NMEALookupHelper, qt_meta_tag_ZN16NMEALookupHelperE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject NMEALookupHelper::staticMetaObject = { {
    QMetaObject::SuperData::link<GPSLookupHelper::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16NMEALookupHelperE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16NMEALookupHelperE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16NMEALookupHelperE_t>.metaTypes,
    nullptr
} };

void NMEALookupHelper::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<NMEALookupHelper *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->query(); break;
        case 1: _t->setPeriodicQuery((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->nmeaUpdated((*reinterpret_cast<std::add_pointer_t<QGeoPositionInfo>>(_a[1]))); break;
        case 3: _t->nmeaError((*reinterpret_cast<std::add_pointer_t<QGeoPositionInfoSource::Error>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QGeoPositionInfo >(); break;
            }
            break;
        }
    }
}

const QMetaObject *NMEALookupHelper::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *NMEALookupHelper::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16NMEALookupHelperE_t>.strings))
        return static_cast<void*>(this);
    return GPSLookupHelper::qt_metacast(_clname);
}

int NMEALookupHelper::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = GPSLookupHelper::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    return _id;
}
QT_WARNING_POP
