/****************************************************************************
** Meta object code from reading C++ file 'Oculars.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../stellarium/plugins/Oculars/src/Oculars.hpp"
#include <QtCore/qmetatype.h>
#include <QtCore/qplugin.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'Oculars.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN7OcularsE_t {};
} // unnamed namespace

template <> constexpr inline auto Oculars::qt_create_metaobjectdata<qt_meta_tag_ZN7OcularsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Oculars",
        "enableOcularChanged",
        "",
        "value",
        "enableCrosshairsChanged",
        "enableCCDChanged",
        "enableTelradChanged",
        "selectedCCDChanged",
        "selectedOcularChanged",
        "selectedTelescopeChanged",
        "selectedLensChanged",
        "selectedCCDRotationAngleChanged",
        "selectedCCDPrismPositionAngleChanged",
        "flagGuiPanelEnabledChanged",
        "guiPanelFontSizeChanged",
        "textColorChanged",
        "Vec3f",
        "color",
        "lineColorChanged",
        "focuserColorChanged",
        "flagHideGridsLinesChanged",
        "flagAutosetMountForCCDChanged",
        "flagScalingFOVForTelradChanged",
        "telradFOVChanged",
        "Vec4f",
        "fov",
        "flagScalingFOVForCCDChanged",
        "flagMaxExposureTimeForCCDChanged",
        "flagUseSemiTransparencyChanged",
        "transparencyMaskChanged",
        "flagShowResolutionCriteriaChanged",
        "arrowButtonScaleChanged",
        "flagInitDirectionUsageChanged",
        "flagInitFOVUsageChanged",
        "flagRequireSelectionChanged",
        "flagAutoLimitMagnitudeChanged",
        "flagDMSDegreesChanged",
        "flagHorizontalCoordinatesChanged",
        "flagScaleImageCircleChanged",
        "flagShowOcularsButtonChanged",
        "flagShowCcdCropOverlayChanged",
        "ccdCropOverlayHSizeChanged",
        "ccdCropOverlayVSizeChanged",
        "flagShowCcdCropOverlayPixelGridChanged",
        "flagShowContourChanged",
        "flagShowCardinalsChanged",
        "flagAlignCrosshairChanged",
        "flagShowFocuserOverlayChanged",
        "flagUseSmallFocuserOverlayChanged",
        "flagUseMediumFocuserOverlayChanged",
        "flagUseLargeFocuserOverlayChanged",
        "flagShowOAGLimitsChanged",
        "getSettings",
        "QSettings*",
        "updateLists",
        "ccdRotationReset",
        "prismPositionAngleReset",
        "decrementCCDIndex",
        "decrementOcularIndex",
        "decrementTelescopeIndex",
        "decrementLensIndex",
        "displayPopupMenu",
        "enableOcular",
        "b",
        "getEnableOcular",
        "incrementCCDIndex",
        "incrementOcularIndex",
        "incrementTelescopeIndex",
        "incrementLensIndex",
        "disableLens",
        "rotateReticleClockwise",
        "rotateReticleCounterclockwise",
        "rotateCCD",
        "amount",
        "getSelectedCCDRotationAngle",
        "setSelectedCCDRotationAngle",
        "angle",
        "rotatePrism",
        "getSelectedCCDPrismPositionAngle",
        "setSelectedCCDPrismPositionAngle",
        "selectCCDAtIndex",
        "index",
        "getSelectedCCDIndex",
        "selectOcularAtIndex",
        "getSelectedOcularIndex",
        "selectTelescopeAtIndex",
        "getSelectedTelescopeIndex",
        "selectLensAtIndex",
        "getSelectedLensIndex",
        "toggleCCD",
        "show",
        "getEnableCCD",
        "toggleCrosshairs",
        "getEnableCrosshairs",
        "toggleTelrad",
        "getEnableTelrad",
        "enableGuiPanel",
        "enable",
        "getFlagGuiPanelEnabled",
        "setGuiPanelFontSize",
        "size",
        "getGuiPanelFontSize",
        "setTextColor",
        "getTextColor",
        "setLineColor",
        "getLineColor",
        "setFocuserColor",
        "getFocuserColor",
        "setTelradFOV",
        "getTelradFOV",
        "setFlagDMSDegrees",
        "getFlagDMSDegrees",
        "setFlagHorizontalCoordinates",
        "getFlagHorizontalCoordinates",
        "setFlagRequireSelection",
        "getFlagRequireSelection",
        "setFlagAutoLimitMagnitude",
        "getFlagAutoLimitMagnitude",
        "setMagLimitStarsOcularsManual",
        "mag",
        "getMagLimitStarsOcularsManual",
        "setFlagInitFovUsage",
        "getFlagInitFovUsage",
        "setFlagInitDirectionUsage",
        "getFlagInitDirectionUsage",
        "setFlagAutosetMountForCCD",
        "getFlagAutosetMountForCCD",
        "setFlagScalingFOVForTelrad",
        "getFlagScalingFOVForTelrad",
        "setFlagScalingFOVForCCD",
        "getFlagScalingFOVForCCD",
        "setFlagMaxExposureTimeForCCD",
        "getFlagMaxExposureTimeForCCD",
        "setFlagUseSemiTransparency",
        "getFlagUseSemiTransparency",
        "setTransparencyMask",
        "v",
        "getTransparencyMask",
        "setFlagShowResolutionCriteria",
        "getFlagShowResolutionCriteria",
        "setCcdCropOverlayHSize",
        "getCcdCropOverlayHSize",
        "setCcdCropOverlayVSize",
        "getCcdCropOverlayVSize",
        "setArrowButtonScale",
        "val",
        "getArrowButtonScale",
        "setFlagHideGridsLines",
        "getFlagHideGridsLines",
        "setFlagScaleImageCircle",
        "state",
        "getFlagScaleImageCircle",
        "setFlagShowOcularsButton",
        "getFlagShowOcularsButton",
        "setFontSize",
        "fontSize",
        "guiPanelFontSize",
        "setFontSizeFromApp",
        "setFlagShowCcdCropOverlay",
        "getFlagShowCcdCropOverlay",
        "setFlagShowCcdCropOverlayPixelGrid",
        "getFlagShowCcdCropOverlayPixelGrid",
        "setFlagShowContour",
        "getFlagShowContour",
        "setFlagShowCardinals",
        "getFlagShowCardinals",
        "setFlagAlignCrosshair",
        "getFlagAlignCrosshair",
        "setFlagShowFocuserOverlay",
        "getFlagShowFocuserOverlay",
        "setFlagUseSmallFocuserOverlay",
        "getFlagUseSmallFocuserOverlay",
        "setFlagUseMediumFocuserOverlay",
        "getFlagUseMediumFocuserOverlay",
        "setFlagUseLargeFocuserOverlay",
        "getFlagUseLargeFocuserOverlay",
        "setFlagShowOAGLimits",
        "getFlagShowOAGLimits",
        "instrumentChanged",
        "determineMaxEyepieceAngle",
        "setScreenFOVForCCD",
        "retranslateGui",
        "updateOcularReticle",
        "togglePixelGrid",
        "toggleCropOverlay",
        "toggleFocuserOverlay",
        "handleAutoLimitToggle",
        "on",
        "handleStarMagLimitToggle",
        "updateLatestSelectedSSO",
        "enableCrosshairs",
        "enableCCD",
        "enableTelrad",
        "selectedCCDIndex",
        "selectedOcularIndex",
        "selectedTelescopeIndex",
        "selectedLensIndex",
        "selectedCCDRotationAngle",
        "selectedCCDPrismPositionAngle",
        "flagGuiPanelEnabled",
        "flagInitFOVUsage",
        "flagInitDirectionUsage",
        "flagShowResolutionCriteria",
        "flagRequireSelection",
        "flagAutoLimitMagnitude",
        "flagHideGridsLines",
        "flagScaleImageCircle",
        "flagSemiTransparency",
        "transparencyMask",
        "flagDMSDegrees",
        "flagHorizontalCoordinates",
        "flagAutosetMountForCCD",
        "flagScalingFOVForTelrad",
        "telradFOV",
        "flagScalingFOVForCCD",
        "flagMaxExposureTimeForCCD",
        "flagShowOcularsButton",
        "flagShowContour",
        "flagShowCardinals",
        "flagAlignCrosshair",
        "arrowButtonScale",
        "textColor",
        "lineColor",
        "flagShowCcdCropOverlay",
        "flagShowCcdCropOverlayPixelGrid",
        "ccdCropOverlayHSize",
        "ccdCropOverlayVSize",
        "flagShowFocuserOverlay",
        "flagUseSmallFocuserOverlay",
        "flagUseMediumFocuserOverlay",
        "flagUseLargeFocuserOverlay",
        "focuserColor",
        "flagShowOAGLimits"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'enableOcularChanged'
        QtMocHelpers::SignalData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'enableCrosshairsChanged'
        QtMocHelpers::SignalData<void(bool)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'enableCCDChanged'
        QtMocHelpers::SignalData<void(bool)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'enableTelradChanged'
        QtMocHelpers::SignalData<void(bool)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'selectedCCDChanged'
        QtMocHelpers::SignalData<void(int)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'selectedOcularChanged'
        QtMocHelpers::SignalData<void(int)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'selectedTelescopeChanged'
        QtMocHelpers::SignalData<void(int)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'selectedLensChanged'
        QtMocHelpers::SignalData<void(int)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'selectedCCDRotationAngleChanged'
        QtMocHelpers::SignalData<void(double)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Signal 'selectedCCDPrismPositionAngleChanged'
        QtMocHelpers::SignalData<void(double)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Signal 'flagGuiPanelEnabledChanged'
        QtMocHelpers::SignalData<void(bool)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'guiPanelFontSizeChanged'
        QtMocHelpers::SignalData<void(int)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'textColorChanged'
        QtMocHelpers::SignalData<void(const Vec3f &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 17 },
        }}),
        // Signal 'lineColorChanged'
        QtMocHelpers::SignalData<void(const Vec3f &)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 17 },
        }}),
        // Signal 'focuserColorChanged'
        QtMocHelpers::SignalData<void(const Vec3f &)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 17 },
        }}),
        // Signal 'flagHideGridsLinesChanged'
        QtMocHelpers::SignalData<void(bool)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagAutosetMountForCCDChanged'
        QtMocHelpers::SignalData<void(bool)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagScalingFOVForTelradChanged'
        QtMocHelpers::SignalData<void(bool)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'telradFOVChanged'
        QtMocHelpers::SignalData<void(Vec4f)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 24, 25 },
        }}),
        // Signal 'flagScalingFOVForCCDChanged'
        QtMocHelpers::SignalData<void(bool)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagMaxExposureTimeForCCDChanged'
        QtMocHelpers::SignalData<void(bool)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagUseSemiTransparencyChanged'
        QtMocHelpers::SignalData<void(bool)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'transparencyMaskChanged'
        QtMocHelpers::SignalData<void(int)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'flagShowResolutionCriteriaChanged'
        QtMocHelpers::SignalData<void(bool)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'arrowButtonScaleChanged'
        QtMocHelpers::SignalData<void(int)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'flagInitDirectionUsageChanged'
        QtMocHelpers::SignalData<void(bool)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagInitFOVUsageChanged'
        QtMocHelpers::SignalData<void(bool)>(33, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagRequireSelectionChanged'
        QtMocHelpers::SignalData<void(bool)>(34, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagAutoLimitMagnitudeChanged'
        QtMocHelpers::SignalData<void(bool)>(35, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagDMSDegreesChanged'
        QtMocHelpers::SignalData<void(bool)>(36, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagHorizontalCoordinatesChanged'
        QtMocHelpers::SignalData<void(bool)>(37, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagScaleImageCircleChanged'
        QtMocHelpers::SignalData<void(bool)>(38, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagShowOcularsButtonChanged'
        QtMocHelpers::SignalData<void(bool)>(39, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagShowCcdCropOverlayChanged'
        QtMocHelpers::SignalData<void(bool)>(40, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'ccdCropOverlayHSizeChanged'
        QtMocHelpers::SignalData<void(int)>(41, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'ccdCropOverlayVSizeChanged'
        QtMocHelpers::SignalData<void(int)>(42, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'flagShowCcdCropOverlayPixelGridChanged'
        QtMocHelpers::SignalData<void(bool)>(43, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagShowContourChanged'
        QtMocHelpers::SignalData<void(bool)>(44, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagShowCardinalsChanged'
        QtMocHelpers::SignalData<void(bool)>(45, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagAlignCrosshairChanged'
        QtMocHelpers::SignalData<void(bool)>(46, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagShowFocuserOverlayChanged'
        QtMocHelpers::SignalData<void(bool)>(47, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagUseSmallFocuserOverlayChanged'
        QtMocHelpers::SignalData<void(bool)>(48, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagUseMediumFocuserOverlayChanged'
        QtMocHelpers::SignalData<void(bool)>(49, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagUseLargeFocuserOverlayChanged'
        QtMocHelpers::SignalData<void(bool)>(50, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagShowOAGLimitsChanged'
        QtMocHelpers::SignalData<void(bool)>(51, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'getSettings'
        QtMocHelpers::SlotData<QSettings *()>(52, 2, QMC::AccessPublic, 0x80000000 | 53),
        // Slot 'updateLists'
        QtMocHelpers::SlotData<void()>(54, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'ccdRotationReset'
        QtMocHelpers::SlotData<void()>(55, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'prismPositionAngleReset'
        QtMocHelpers::SlotData<void()>(56, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'decrementCCDIndex'
        QtMocHelpers::SlotData<void()>(57, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'decrementOcularIndex'
        QtMocHelpers::SlotData<void()>(58, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'decrementTelescopeIndex'
        QtMocHelpers::SlotData<void()>(59, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'decrementLensIndex'
        QtMocHelpers::SlotData<void()>(60, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'displayPopupMenu'
        QtMocHelpers::SlotData<void()>(61, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'enableOcular'
        QtMocHelpers::SlotData<void(bool)>(62, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 63 },
        }}),
        // Slot 'getEnableOcular'
        QtMocHelpers::SlotData<bool() const>(64, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'incrementCCDIndex'
        QtMocHelpers::SlotData<void()>(65, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'incrementOcularIndex'
        QtMocHelpers::SlotData<void()>(66, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'incrementTelescopeIndex'
        QtMocHelpers::SlotData<void()>(67, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'incrementLensIndex'
        QtMocHelpers::SlotData<void()>(68, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'disableLens'
        QtMocHelpers::SlotData<void()>(69, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'rotateReticleClockwise'
        QtMocHelpers::SlotData<void()>(70, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'rotateReticleCounterclockwise'
        QtMocHelpers::SlotData<void()>(71, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'rotateCCD'
        QtMocHelpers::SlotData<void(int)>(72, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 73 },
        }}),
        // Slot 'getSelectedCCDRotationAngle'
        QtMocHelpers::SlotData<double() const>(74, 2, QMC::AccessPublic, QMetaType::Double),
        // Slot 'setSelectedCCDRotationAngle'
        QtMocHelpers::SlotData<void(double)>(75, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 76 },
        }}),
        // Slot 'rotatePrism'
        QtMocHelpers::SlotData<void(int)>(77, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 73 },
        }}),
        // Slot 'getSelectedCCDPrismPositionAngle'
        QtMocHelpers::SlotData<double() const>(78, 2, QMC::AccessPublic, QMetaType::Double),
        // Slot 'setSelectedCCDPrismPositionAngle'
        QtMocHelpers::SlotData<void(double)>(79, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 76 },
        }}),
        // Slot 'selectCCDAtIndex'
        QtMocHelpers::SlotData<void(int)>(80, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 81 },
        }}),
        // Slot 'getSelectedCCDIndex'
        QtMocHelpers::SlotData<int() const>(82, 2, QMC::AccessPublic, QMetaType::Int),
        // Slot 'selectOcularAtIndex'
        QtMocHelpers::SlotData<void(int)>(83, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 81 },
        }}),
        // Slot 'getSelectedOcularIndex'
        QtMocHelpers::SlotData<int() const>(84, 2, QMC::AccessPublic, QMetaType::Int),
        // Slot 'selectTelescopeAtIndex'
        QtMocHelpers::SlotData<void(int)>(85, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 81 },
        }}),
        // Slot 'getSelectedTelescopeIndex'
        QtMocHelpers::SlotData<int() const>(86, 2, QMC::AccessPublic, QMetaType::Int),
        // Slot 'selectLensAtIndex'
        QtMocHelpers::SlotData<void(int)>(87, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 81 },
        }}),
        // Slot 'getSelectedLensIndex'
        QtMocHelpers::SlotData<int() const>(88, 2, QMC::AccessPublic, QMetaType::Int),
        // Slot 'toggleCCD'
        QtMocHelpers::SlotData<void(bool)>(89, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 90 },
        }}),
        // Slot 'toggleCCD'
        QtMocHelpers::SlotData<void()>(89, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'getEnableCCD'
        QtMocHelpers::SlotData<bool() const>(91, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'toggleCrosshairs'
        QtMocHelpers::SlotData<void(bool)>(92, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 90 },
        }}),
        // Slot 'toggleCrosshairs'
        QtMocHelpers::SlotData<void()>(92, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'getEnableCrosshairs'
        QtMocHelpers::SlotData<bool() const>(93, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'toggleTelrad'
        QtMocHelpers::SlotData<void(bool)>(94, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 90 },
        }}),
        // Slot 'getEnableTelrad'
        QtMocHelpers::SlotData<bool() const>(95, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'toggleTelrad'
        QtMocHelpers::SlotData<void()>(94, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'enableGuiPanel'
        QtMocHelpers::SlotData<void(bool)>(96, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 97 },
        }}),
        // Slot 'enableGuiPanel'
        QtMocHelpers::SlotData<void()>(96, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'getFlagGuiPanelEnabled'
        QtMocHelpers::SlotData<bool() const>(98, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setGuiPanelFontSize'
        QtMocHelpers::SlotData<void(int)>(99, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 100 },
        }}),
        // Slot 'getGuiPanelFontSize'
        QtMocHelpers::SlotData<int() const>(101, 2, QMC::AccessPublic, QMetaType::Int),
        // Slot 'setTextColor'
        QtMocHelpers::SlotData<void(const Vec3f &)>(102, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 17 },
        }}),
        // Slot 'getTextColor'
        QtMocHelpers::SlotData<Vec3f() const>(103, 2, QMC::AccessPublic, 0x80000000 | 16),
        // Slot 'setLineColor'
        QtMocHelpers::SlotData<void(const Vec3f &)>(104, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 17 },
        }}),
        // Slot 'getLineColor'
        QtMocHelpers::SlotData<Vec3f() const>(105, 2, QMC::AccessPublic, 0x80000000 | 16),
        // Slot 'setFocuserColor'
        QtMocHelpers::SlotData<void(const Vec3f &)>(106, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 17 },
        }}),
        // Slot 'getFocuserColor'
        QtMocHelpers::SlotData<Vec3f() const>(107, 2, QMC::AccessPublic, 0x80000000 | 16),
        // Slot 'setTelradFOV'
        QtMocHelpers::SlotData<void(const Vec4f &)>(108, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 24, 25 },
        }}),
        // Slot 'getTelradFOV'
        QtMocHelpers::SlotData<Vec4f() const>(109, 2, QMC::AccessPublic, 0x80000000 | 24),
        // Slot 'setFlagDMSDegrees'
        QtMocHelpers::SlotData<void(const bool)>(110, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 63 },
        }}),
        // Slot 'getFlagDMSDegrees'
        QtMocHelpers::SlotData<bool() const>(111, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFlagHorizontalCoordinates'
        QtMocHelpers::SlotData<void(const bool)>(112, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 63 },
        }}),
        // Slot 'getFlagHorizontalCoordinates'
        QtMocHelpers::SlotData<bool() const>(113, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFlagRequireSelection'
        QtMocHelpers::SlotData<void(const bool)>(114, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 63 },
        }}),
        // Slot 'getFlagRequireSelection'
        QtMocHelpers::SlotData<bool() const>(115, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFlagAutoLimitMagnitude'
        QtMocHelpers::SlotData<void(const bool)>(116, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 63 },
        }}),
        // Slot 'getFlagAutoLimitMagnitude'
        QtMocHelpers::SlotData<bool() const>(117, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setMagLimitStarsOcularsManual'
        QtMocHelpers::SlotData<void(double)>(118, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 119 },
        }}),
        // Slot 'getMagLimitStarsOcularsManual'
        QtMocHelpers::SlotData<double() const>(120, 2, QMC::AccessPublic, QMetaType::Double),
        // Slot 'setFlagInitFovUsage'
        QtMocHelpers::SlotData<void(const bool)>(121, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 63 },
        }}),
        // Slot 'getFlagInitFovUsage'
        QtMocHelpers::SlotData<bool() const>(122, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFlagInitDirectionUsage'
        QtMocHelpers::SlotData<void(const bool)>(123, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 63 },
        }}),
        // Slot 'getFlagInitDirectionUsage'
        QtMocHelpers::SlotData<bool() const>(124, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFlagAutosetMountForCCD'
        QtMocHelpers::SlotData<void(const bool)>(125, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 63 },
        }}),
        // Slot 'getFlagAutosetMountForCCD'
        QtMocHelpers::SlotData<bool() const>(126, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFlagScalingFOVForTelrad'
        QtMocHelpers::SlotData<void(const bool)>(127, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 63 },
        }}),
        // Slot 'getFlagScalingFOVForTelrad'
        QtMocHelpers::SlotData<bool() const>(128, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFlagScalingFOVForCCD'
        QtMocHelpers::SlotData<void(const bool)>(129, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 63 },
        }}),
        // Slot 'getFlagScalingFOVForCCD'
        QtMocHelpers::SlotData<bool() const>(130, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFlagMaxExposureTimeForCCD'
        QtMocHelpers::SlotData<void(const bool)>(131, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 63 },
        }}),
        // Slot 'getFlagMaxExposureTimeForCCD'
        QtMocHelpers::SlotData<bool() const>(132, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFlagUseSemiTransparency'
        QtMocHelpers::SlotData<void(const bool)>(133, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 63 },
        }}),
        // Slot 'getFlagUseSemiTransparency'
        QtMocHelpers::SlotData<bool() const>(134, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setTransparencyMask'
        QtMocHelpers::SlotData<void(const int)>(135, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 136 },
        }}),
        // Slot 'getTransparencyMask'
        QtMocHelpers::SlotData<int() const>(137, 2, QMC::AccessPublic, QMetaType::Int),
        // Slot 'setFlagShowResolutionCriteria'
        QtMocHelpers::SlotData<void(const bool)>(138, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 63 },
        }}),
        // Slot 'getFlagShowResolutionCriteria'
        QtMocHelpers::SlotData<bool() const>(139, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setCcdCropOverlayHSize'
        QtMocHelpers::SlotData<void(int)>(140, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 100 },
        }}),
        // Slot 'getCcdCropOverlayHSize'
        QtMocHelpers::SlotData<int() const>(141, 2, QMC::AccessPublic, QMetaType::Int),
        // Slot 'setCcdCropOverlayVSize'
        QtMocHelpers::SlotData<void(int)>(142, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 100 },
        }}),
        // Slot 'getCcdCropOverlayVSize'
        QtMocHelpers::SlotData<int() const>(143, 2, QMC::AccessPublic, QMetaType::Int),
        // Slot 'setArrowButtonScale'
        QtMocHelpers::SlotData<void(const int)>(144, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 145 },
        }}),
        // Slot 'getArrowButtonScale'
        QtMocHelpers::SlotData<int() const>(146, 2, QMC::AccessPublic, QMetaType::Int),
        // Slot 'setFlagHideGridsLines'
        QtMocHelpers::SlotData<void(const bool)>(147, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 63 },
        }}),
        // Slot 'getFlagHideGridsLines'
        QtMocHelpers::SlotData<bool() const>(148, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFlagScaleImageCircle'
        QtMocHelpers::SlotData<void(bool)>(149, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 150 },
        }}),
        // Slot 'getFlagScaleImageCircle'
        QtMocHelpers::SlotData<bool() const>(151, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFlagShowOcularsButton'
        QtMocHelpers::SlotData<void(bool)>(152, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 63 },
        }}),
        // Slot 'getFlagShowOcularsButton'
        QtMocHelpers::SlotData<bool()>(153, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFontSize'
        QtMocHelpers::SlotData<void(int, int)>(154, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 155 }, { QMetaType::Int, 156 },
        }}),
        // Slot 'setFontSizeFromApp'
        QtMocHelpers::SlotData<void(int)>(157, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 100 },
        }}),
        // Slot 'setFlagShowCcdCropOverlay'
        QtMocHelpers::SlotData<void(const bool)>(158, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 63 },
        }}),
        // Slot 'getFlagShowCcdCropOverlay'
        QtMocHelpers::SlotData<bool() const>(159, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFlagShowCcdCropOverlayPixelGrid'
        QtMocHelpers::SlotData<void(const bool)>(160, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 63 },
        }}),
        // Slot 'getFlagShowCcdCropOverlayPixelGrid'
        QtMocHelpers::SlotData<bool() const>(161, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFlagShowContour'
        QtMocHelpers::SlotData<void(const bool)>(162, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 63 },
        }}),
        // Slot 'getFlagShowContour'
        QtMocHelpers::SlotData<bool() const>(163, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFlagShowCardinals'
        QtMocHelpers::SlotData<void(const bool)>(164, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 63 },
        }}),
        // Slot 'getFlagShowCardinals'
        QtMocHelpers::SlotData<bool() const>(165, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFlagAlignCrosshair'
        QtMocHelpers::SlotData<void(const bool)>(166, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 63 },
        }}),
        // Slot 'getFlagAlignCrosshair'
        QtMocHelpers::SlotData<bool() const>(167, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFlagShowFocuserOverlay'
        QtMocHelpers::SlotData<void(const bool)>(168, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 63 },
        }}),
        // Slot 'getFlagShowFocuserOverlay'
        QtMocHelpers::SlotData<bool() const>(169, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFlagUseSmallFocuserOverlay'
        QtMocHelpers::SlotData<void(const bool)>(170, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 63 },
        }}),
        // Slot 'getFlagUseSmallFocuserOverlay'
        QtMocHelpers::SlotData<bool() const>(171, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFlagUseMediumFocuserOverlay'
        QtMocHelpers::SlotData<void(const bool)>(172, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 63 },
        }}),
        // Slot 'getFlagUseMediumFocuserOverlay'
        QtMocHelpers::SlotData<bool() const>(173, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFlagUseLargeFocuserOverlay'
        QtMocHelpers::SlotData<void(const bool)>(174, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 63 },
        }}),
        // Slot 'getFlagUseLargeFocuserOverlay'
        QtMocHelpers::SlotData<bool() const>(175, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFlagShowOAGLimits'
        QtMocHelpers::SlotData<void(const bool)>(176, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 63 },
        }}),
        // Slot 'getFlagShowOAGLimits'
        QtMocHelpers::SlotData<bool() const>(177, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'instrumentChanged'
        QtMocHelpers::SlotData<void()>(178, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'determineMaxEyepieceAngle'
        QtMocHelpers::SlotData<void()>(179, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setScreenFOVForCCD'
        QtMocHelpers::SlotData<void()>(180, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'retranslateGui'
        QtMocHelpers::SlotData<void()>(181, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateOcularReticle'
        QtMocHelpers::SlotData<void()>(182, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'togglePixelGrid'
        QtMocHelpers::SlotData<void()>(183, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'toggleCropOverlay'
        QtMocHelpers::SlotData<void()>(184, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'toggleFocuserOverlay'
        QtMocHelpers::SlotData<void()>(185, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'handleAutoLimitToggle'
        QtMocHelpers::SlotData<void(bool)>(186, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 187 },
        }}),
        // Slot 'handleStarMagLimitToggle'
        QtMocHelpers::SlotData<void(bool)>(188, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 187 },
        }}),
        // Slot 'updateLatestSelectedSSO'
        QtMocHelpers::SlotData<void()>(189, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'enableOcular'
        QtMocHelpers::PropertyData<bool>(62, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 0),
        // property 'enableCrosshairs'
        QtMocHelpers::PropertyData<bool>(190, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 1),
        // property 'enableCCD'
        QtMocHelpers::PropertyData<bool>(191, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 2),
        // property 'enableTelrad'
        QtMocHelpers::PropertyData<bool>(192, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 3),
        // property 'selectedCCDIndex'
        QtMocHelpers::PropertyData<int>(193, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable, 4),
        // property 'selectedOcularIndex'
        QtMocHelpers::PropertyData<int>(194, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable, 5),
        // property 'selectedTelescopeIndex'
        QtMocHelpers::PropertyData<int>(195, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable, 6),
        // property 'selectedLensIndex'
        QtMocHelpers::PropertyData<int>(196, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable, 7),
        // property 'selectedCCDRotationAngle'
        QtMocHelpers::PropertyData<double>(197, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 8),
        // property 'selectedCCDPrismPositionAngle'
        QtMocHelpers::PropertyData<double>(198, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 9),
        // property 'flagGuiPanelEnabled'
        QtMocHelpers::PropertyData<bool>(199, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 10),
        // property 'flagInitFOVUsage'
        QtMocHelpers::PropertyData<bool>(200, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 26),
        // property 'flagInitDirectionUsage'
        QtMocHelpers::PropertyData<bool>(201, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 25),
        // property 'flagShowResolutionCriteria'
        QtMocHelpers::PropertyData<bool>(202, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 23),
        // property 'flagRequireSelection'
        QtMocHelpers::PropertyData<bool>(203, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 27),
        // property 'flagAutoLimitMagnitude'
        QtMocHelpers::PropertyData<bool>(204, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 28),
        // property 'flagHideGridsLines'
        QtMocHelpers::PropertyData<bool>(205, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 15),
        // property 'flagScaleImageCircle'
        QtMocHelpers::PropertyData<bool>(206, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 31),
        // property 'flagSemiTransparency'
        QtMocHelpers::PropertyData<bool>(207, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 21),
        // property 'transparencyMask'
        QtMocHelpers::PropertyData<int>(208, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 22),
        // property 'flagDMSDegrees'
        QtMocHelpers::PropertyData<bool>(209, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 29),
        // property 'flagHorizontalCoordinates'
        QtMocHelpers::PropertyData<bool>(210, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 30),
        // property 'flagAutosetMountForCCD'
        QtMocHelpers::PropertyData<bool>(211, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 16),
        // property 'flagScalingFOVForTelrad'
        QtMocHelpers::PropertyData<bool>(212, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 17),
        // property 'telradFOV'
        QtMocHelpers::PropertyData<Vec4f>(213, 0x80000000 | 24, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 18),
        // property 'flagScalingFOVForCCD'
        QtMocHelpers::PropertyData<bool>(214, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 19),
        // property 'flagMaxExposureTimeForCCD'
        QtMocHelpers::PropertyData<bool>(215, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 20),
        // property 'flagShowOcularsButton'
        QtMocHelpers::PropertyData<bool>(216, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 32),
        // property 'flagShowContour'
        QtMocHelpers::PropertyData<bool>(217, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 37),
        // property 'flagShowCardinals'
        QtMocHelpers::PropertyData<bool>(218, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 38),
        // property 'flagAlignCrosshair'
        QtMocHelpers::PropertyData<bool>(219, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 39),
        // property 'arrowButtonScale'
        QtMocHelpers::PropertyData<int>(220, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 24),
        // property 'guiPanelFontSize'
        QtMocHelpers::PropertyData<int>(156, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 11),
        // property 'textColor'
        QtMocHelpers::PropertyData<Vec3f>(221, 0x80000000 | 16, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 12),
        // property 'lineColor'
        QtMocHelpers::PropertyData<Vec3f>(222, 0x80000000 | 16, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 12),
        // property 'flagShowCcdCropOverlay'
        QtMocHelpers::PropertyData<bool>(223, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 33),
        // property 'flagShowCcdCropOverlayPixelGrid'
        QtMocHelpers::PropertyData<bool>(224, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 36),
        // property 'ccdCropOverlayHSize'
        QtMocHelpers::PropertyData<int>(225, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 34),
        // property 'ccdCropOverlayVSize'
        QtMocHelpers::PropertyData<int>(226, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 35),
        // property 'flagShowFocuserOverlay'
        QtMocHelpers::PropertyData<bool>(227, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 40),
        // property 'flagUseSmallFocuserOverlay'
        QtMocHelpers::PropertyData<bool>(228, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 41),
        // property 'flagUseMediumFocuserOverlay'
        QtMocHelpers::PropertyData<bool>(229, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 42),
        // property 'flagUseLargeFocuserOverlay'
        QtMocHelpers::PropertyData<bool>(230, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 43),
        // property 'focuserColor'
        QtMocHelpers::PropertyData<Vec3f>(231, 0x80000000 | 16, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 14),
        // property 'flagShowOAGLimits'
        QtMocHelpers::PropertyData<bool>(232, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 44),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Oculars, qt_meta_tag_ZN7OcularsE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Oculars::staticMetaObject = { {
    QMetaObject::SuperData::link<StelModule::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7OcularsE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7OcularsE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN7OcularsE_t>.metaTypes,
    nullptr
} };

void Oculars::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Oculars *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->enableOcularChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->enableCrosshairsChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->enableCCDChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->enableTelradChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->selectedCCDChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 5: _t->selectedOcularChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 6: _t->selectedTelescopeChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 7: _t->selectedLensChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 8: _t->selectedCCDRotationAngleChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 9: _t->selectedCCDPrismPositionAngleChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 10: _t->flagGuiPanelEnabledChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 11: _t->guiPanelFontSizeChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 12: _t->textColorChanged((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1]))); break;
        case 13: _t->lineColorChanged((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1]))); break;
        case 14: _t->focuserColorChanged((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1]))); break;
        case 15: _t->flagHideGridsLinesChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 16: _t->flagAutosetMountForCCDChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 17: _t->flagScalingFOVForTelradChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 18: _t->telradFOVChanged((*reinterpret_cast<std::add_pointer_t<Vec4f>>(_a[1]))); break;
        case 19: _t->flagScalingFOVForCCDChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 20: _t->flagMaxExposureTimeForCCDChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 21: _t->flagUseSemiTransparencyChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 22: _t->transparencyMaskChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 23: _t->flagShowResolutionCriteriaChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 24: _t->arrowButtonScaleChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 25: _t->flagInitDirectionUsageChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 26: _t->flagInitFOVUsageChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 27: _t->flagRequireSelectionChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 28: _t->flagAutoLimitMagnitudeChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 29: _t->flagDMSDegreesChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 30: _t->flagHorizontalCoordinatesChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 31: _t->flagScaleImageCircleChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 32: _t->flagShowOcularsButtonChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 33: _t->flagShowCcdCropOverlayChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 34: _t->ccdCropOverlayHSizeChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 35: _t->ccdCropOverlayVSizeChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 36: _t->flagShowCcdCropOverlayPixelGridChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 37: _t->flagShowContourChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 38: _t->flagShowCardinalsChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 39: _t->flagAlignCrosshairChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 40: _t->flagShowFocuserOverlayChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 41: _t->flagUseSmallFocuserOverlayChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 42: _t->flagUseMediumFocuserOverlayChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 43: _t->flagUseLargeFocuserOverlayChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 44: _t->flagShowOAGLimitsChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 45: { QSettings* _r = _t->getSettings();
            if (_a[0]) *reinterpret_cast<QSettings**>(_a[0]) = std::move(_r); }  break;
        case 46: _t->updateLists(); break;
        case 47: _t->ccdRotationReset(); break;
        case 48: _t->prismPositionAngleReset(); break;
        case 49: _t->decrementCCDIndex(); break;
        case 50: _t->decrementOcularIndex(); break;
        case 51: _t->decrementTelescopeIndex(); break;
        case 52: _t->decrementLensIndex(); break;
        case 53: _t->displayPopupMenu(); break;
        case 54: _t->enableOcular((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 55: { bool _r = _t->getEnableOcular();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 56: _t->incrementCCDIndex(); break;
        case 57: _t->incrementOcularIndex(); break;
        case 58: _t->incrementTelescopeIndex(); break;
        case 59: _t->incrementLensIndex(); break;
        case 60: _t->disableLens(); break;
        case 61: _t->rotateReticleClockwise(); break;
        case 62: _t->rotateReticleCounterclockwise(); break;
        case 63: _t->rotateCCD((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 64: { double _r = _t->getSelectedCCDRotationAngle();
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 65: _t->setSelectedCCDRotationAngle((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 66: _t->rotatePrism((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 67: { double _r = _t->getSelectedCCDPrismPositionAngle();
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 68: _t->setSelectedCCDPrismPositionAngle((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 69: _t->selectCCDAtIndex((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 70: { int _r = _t->getSelectedCCDIndex();
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 71: _t->selectOcularAtIndex((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 72: { int _r = _t->getSelectedOcularIndex();
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 73: _t->selectTelescopeAtIndex((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 74: { int _r = _t->getSelectedTelescopeIndex();
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 75: _t->selectLensAtIndex((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 76: { int _r = _t->getSelectedLensIndex();
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 77: _t->toggleCCD((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 78: _t->toggleCCD(); break;
        case 79: { bool _r = _t->getEnableCCD();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 80: _t->toggleCrosshairs((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 81: _t->toggleCrosshairs(); break;
        case 82: { bool _r = _t->getEnableCrosshairs();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 83: _t->toggleTelrad((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 84: { bool _r = _t->getEnableTelrad();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 85: _t->toggleTelrad(); break;
        case 86: _t->enableGuiPanel((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 87: _t->enableGuiPanel(); break;
        case 88: { bool _r = _t->getFlagGuiPanelEnabled();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 89: _t->setGuiPanelFontSize((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 90: { int _r = _t->getGuiPanelFontSize();
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 91: _t->setTextColor((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1]))); break;
        case 92: { Vec3f _r = _t->getTextColor();
            if (_a[0]) *reinterpret_cast<Vec3f*>(_a[0]) = std::move(_r); }  break;
        case 93: _t->setLineColor((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1]))); break;
        case 94: { Vec3f _r = _t->getLineColor();
            if (_a[0]) *reinterpret_cast<Vec3f*>(_a[0]) = std::move(_r); }  break;
        case 95: _t->setFocuserColor((*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[1]))); break;
        case 96: { Vec3f _r = _t->getFocuserColor();
            if (_a[0]) *reinterpret_cast<Vec3f*>(_a[0]) = std::move(_r); }  break;
        case 97: _t->setTelradFOV((*reinterpret_cast<std::add_pointer_t<Vec4f>>(_a[1]))); break;
        case 98: { Vec4f _r = _t->getTelradFOV();
            if (_a[0]) *reinterpret_cast<Vec4f*>(_a[0]) = std::move(_r); }  break;
        case 99: _t->setFlagDMSDegrees((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 100: { bool _r = _t->getFlagDMSDegrees();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 101: _t->setFlagHorizontalCoordinates((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 102: { bool _r = _t->getFlagHorizontalCoordinates();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 103: _t->setFlagRequireSelection((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 104: { bool _r = _t->getFlagRequireSelection();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 105: _t->setFlagAutoLimitMagnitude((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 106: { bool _r = _t->getFlagAutoLimitMagnitude();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 107: _t->setMagLimitStarsOcularsManual((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 108: { double _r = _t->getMagLimitStarsOcularsManual();
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 109: _t->setFlagInitFovUsage((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 110: { bool _r = _t->getFlagInitFovUsage();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 111: _t->setFlagInitDirectionUsage((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 112: { bool _r = _t->getFlagInitDirectionUsage();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 113: _t->setFlagAutosetMountForCCD((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 114: { bool _r = _t->getFlagAutosetMountForCCD();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 115: _t->setFlagScalingFOVForTelrad((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 116: { bool _r = _t->getFlagScalingFOVForTelrad();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 117: _t->setFlagScalingFOVForCCD((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 118: { bool _r = _t->getFlagScalingFOVForCCD();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 119: _t->setFlagMaxExposureTimeForCCD((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 120: { bool _r = _t->getFlagMaxExposureTimeForCCD();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 121: _t->setFlagUseSemiTransparency((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 122: { bool _r = _t->getFlagUseSemiTransparency();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 123: _t->setTransparencyMask((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 124: { int _r = _t->getTransparencyMask();
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 125: _t->setFlagShowResolutionCriteria((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 126: { bool _r = _t->getFlagShowResolutionCriteria();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 127: _t->setCcdCropOverlayHSize((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 128: { int _r = _t->getCcdCropOverlayHSize();
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 129: _t->setCcdCropOverlayVSize((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 130: { int _r = _t->getCcdCropOverlayVSize();
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 131: _t->setArrowButtonScale((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 132: { int _r = _t->getArrowButtonScale();
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 133: _t->setFlagHideGridsLines((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 134: { bool _r = _t->getFlagHideGridsLines();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 135: _t->setFlagScaleImageCircle((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 136: { bool _r = _t->getFlagScaleImageCircle();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 137: _t->setFlagShowOcularsButton((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 138: { bool _r = _t->getFlagShowOcularsButton();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 139: _t->setFontSize((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 140: _t->setFontSizeFromApp((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 141: _t->setFlagShowCcdCropOverlay((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 142: { bool _r = _t->getFlagShowCcdCropOverlay();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 143: _t->setFlagShowCcdCropOverlayPixelGrid((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 144: { bool _r = _t->getFlagShowCcdCropOverlayPixelGrid();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 145: _t->setFlagShowContour((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 146: { bool _r = _t->getFlagShowContour();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 147: _t->setFlagShowCardinals((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 148: { bool _r = _t->getFlagShowCardinals();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 149: _t->setFlagAlignCrosshair((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 150: { bool _r = _t->getFlagAlignCrosshair();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 151: _t->setFlagShowFocuserOverlay((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 152: { bool _r = _t->getFlagShowFocuserOverlay();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 153: _t->setFlagUseSmallFocuserOverlay((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 154: { bool _r = _t->getFlagUseSmallFocuserOverlay();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 155: _t->setFlagUseMediumFocuserOverlay((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 156: { bool _r = _t->getFlagUseMediumFocuserOverlay();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 157: _t->setFlagUseLargeFocuserOverlay((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 158: { bool _r = _t->getFlagUseLargeFocuserOverlay();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 159: _t->setFlagShowOAGLimits((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 160: { bool _r = _t->getFlagShowOAGLimits();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 161: _t->instrumentChanged(); break;
        case 162: _t->determineMaxEyepieceAngle(); break;
        case 163: _t->setScreenFOVForCCD(); break;
        case 164: _t->retranslateGui(); break;
        case 165: _t->updateOcularReticle(); break;
        case 166: _t->togglePixelGrid(); break;
        case 167: _t->toggleCropOverlay(); break;
        case 168: _t->toggleFocuserOverlay(); break;
        case 169: _t->handleAutoLimitToggle((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 170: _t->handleStarMagLimitToggle((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 171: _t->updateLatestSelectedSSO(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 13:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 14:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 18:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec4f >(); break;
            }
            break;
        case 91:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 93:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 95:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 97:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec4f >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Oculars::*)(bool )>(_a, &Oculars::enableOcularChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Oculars::*)(bool )>(_a, &Oculars::enableCrosshairsChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Oculars::*)(bool )>(_a, &Oculars::enableCCDChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Oculars::*)(bool )>(_a, &Oculars::enableTelradChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Oculars::*)(int )>(_a, &Oculars::selectedCCDChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (Oculars::*)(int )>(_a, &Oculars::selectedOcularChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (Oculars::*)(int )>(_a, &Oculars::selectedTelescopeChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (Oculars::*)(int )>(_a, &Oculars::selectedLensChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (Oculars::*)(double )>(_a, &Oculars::selectedCCDRotationAngleChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (Oculars::*)(double )>(_a, &Oculars::selectedCCDPrismPositionAngleChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (Oculars::*)(bool )>(_a, &Oculars::flagGuiPanelEnabledChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (Oculars::*)(int )>(_a, &Oculars::guiPanelFontSizeChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (Oculars::*)(const Vec3f & )>(_a, &Oculars::textColorChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (Oculars::*)(const Vec3f & )>(_a, &Oculars::lineColorChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (Oculars::*)(const Vec3f & )>(_a, &Oculars::focuserColorChanged, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (Oculars::*)(bool )>(_a, &Oculars::flagHideGridsLinesChanged, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (Oculars::*)(bool )>(_a, &Oculars::flagAutosetMountForCCDChanged, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (Oculars::*)(bool )>(_a, &Oculars::flagScalingFOVForTelradChanged, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (Oculars::*)(Vec4f )>(_a, &Oculars::telradFOVChanged, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (Oculars::*)(bool )>(_a, &Oculars::flagScalingFOVForCCDChanged, 19))
            return;
        if (QtMocHelpers::indexOfMethod<void (Oculars::*)(bool )>(_a, &Oculars::flagMaxExposureTimeForCCDChanged, 20))
            return;
        if (QtMocHelpers::indexOfMethod<void (Oculars::*)(bool )>(_a, &Oculars::flagUseSemiTransparencyChanged, 21))
            return;
        if (QtMocHelpers::indexOfMethod<void (Oculars::*)(int )>(_a, &Oculars::transparencyMaskChanged, 22))
            return;
        if (QtMocHelpers::indexOfMethod<void (Oculars::*)(bool )>(_a, &Oculars::flagShowResolutionCriteriaChanged, 23))
            return;
        if (QtMocHelpers::indexOfMethod<void (Oculars::*)(int )>(_a, &Oculars::arrowButtonScaleChanged, 24))
            return;
        if (QtMocHelpers::indexOfMethod<void (Oculars::*)(bool )>(_a, &Oculars::flagInitDirectionUsageChanged, 25))
            return;
        if (QtMocHelpers::indexOfMethod<void (Oculars::*)(bool )>(_a, &Oculars::flagInitFOVUsageChanged, 26))
            return;
        if (QtMocHelpers::indexOfMethod<void (Oculars::*)(bool )>(_a, &Oculars::flagRequireSelectionChanged, 27))
            return;
        if (QtMocHelpers::indexOfMethod<void (Oculars::*)(bool )>(_a, &Oculars::flagAutoLimitMagnitudeChanged, 28))
            return;
        if (QtMocHelpers::indexOfMethod<void (Oculars::*)(bool )>(_a, &Oculars::flagDMSDegreesChanged, 29))
            return;
        if (QtMocHelpers::indexOfMethod<void (Oculars::*)(bool )>(_a, &Oculars::flagHorizontalCoordinatesChanged, 30))
            return;
        if (QtMocHelpers::indexOfMethod<void (Oculars::*)(bool )>(_a, &Oculars::flagScaleImageCircleChanged, 31))
            return;
        if (QtMocHelpers::indexOfMethod<void (Oculars::*)(bool )>(_a, &Oculars::flagShowOcularsButtonChanged, 32))
            return;
        if (QtMocHelpers::indexOfMethod<void (Oculars::*)(bool )>(_a, &Oculars::flagShowCcdCropOverlayChanged, 33))
            return;
        if (QtMocHelpers::indexOfMethod<void (Oculars::*)(int )>(_a, &Oculars::ccdCropOverlayHSizeChanged, 34))
            return;
        if (QtMocHelpers::indexOfMethod<void (Oculars::*)(int )>(_a, &Oculars::ccdCropOverlayVSizeChanged, 35))
            return;
        if (QtMocHelpers::indexOfMethod<void (Oculars::*)(bool )>(_a, &Oculars::flagShowCcdCropOverlayPixelGridChanged, 36))
            return;
        if (QtMocHelpers::indexOfMethod<void (Oculars::*)(bool )>(_a, &Oculars::flagShowContourChanged, 37))
            return;
        if (QtMocHelpers::indexOfMethod<void (Oculars::*)(bool )>(_a, &Oculars::flagShowCardinalsChanged, 38))
            return;
        if (QtMocHelpers::indexOfMethod<void (Oculars::*)(bool )>(_a, &Oculars::flagAlignCrosshairChanged, 39))
            return;
        if (QtMocHelpers::indexOfMethod<void (Oculars::*)(bool )>(_a, &Oculars::flagShowFocuserOverlayChanged, 40))
            return;
        if (QtMocHelpers::indexOfMethod<void (Oculars::*)(bool )>(_a, &Oculars::flagUseSmallFocuserOverlayChanged, 41))
            return;
        if (QtMocHelpers::indexOfMethod<void (Oculars::*)(bool )>(_a, &Oculars::flagUseMediumFocuserOverlayChanged, 42))
            return;
        if (QtMocHelpers::indexOfMethod<void (Oculars::*)(bool )>(_a, &Oculars::flagUseLargeFocuserOverlayChanged, 43))
            return;
        if (QtMocHelpers::indexOfMethod<void (Oculars::*)(bool )>(_a, &Oculars::flagShowOAGLimitsChanged, 44))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 43:
        case 34:
        case 33:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Vec3f >(); break;
        case 24:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Vec4f >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->getEnableOcular(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->getEnableCrosshairs(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->getEnableCCD(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->getEnableTelrad(); break;
        case 4: *reinterpret_cast<int*>(_v) = _t->getSelectedCCDIndex(); break;
        case 5: *reinterpret_cast<int*>(_v) = _t->getSelectedOcularIndex(); break;
        case 6: *reinterpret_cast<int*>(_v) = _t->getSelectedTelescopeIndex(); break;
        case 7: *reinterpret_cast<int*>(_v) = _t->getSelectedLensIndex(); break;
        case 8: *reinterpret_cast<double*>(_v) = _t->getSelectedCCDRotationAngle(); break;
        case 9: *reinterpret_cast<double*>(_v) = _t->getSelectedCCDPrismPositionAngle(); break;
        case 10: *reinterpret_cast<bool*>(_v) = _t->getFlagGuiPanelEnabled(); break;
        case 11: *reinterpret_cast<bool*>(_v) = _t->getFlagInitFovUsage(); break;
        case 12: *reinterpret_cast<bool*>(_v) = _t->getFlagInitDirectionUsage(); break;
        case 13: *reinterpret_cast<bool*>(_v) = _t->getFlagShowResolutionCriteria(); break;
        case 14: *reinterpret_cast<bool*>(_v) = _t->getFlagRequireSelection(); break;
        case 15: *reinterpret_cast<bool*>(_v) = _t->getFlagAutoLimitMagnitude(); break;
        case 16: *reinterpret_cast<bool*>(_v) = _t->getFlagHideGridsLines(); break;
        case 17: *reinterpret_cast<bool*>(_v) = _t->getFlagScaleImageCircle(); break;
        case 18: *reinterpret_cast<bool*>(_v) = _t->getFlagUseSemiTransparency(); break;
        case 19: *reinterpret_cast<int*>(_v) = _t->getTransparencyMask(); break;
        case 20: *reinterpret_cast<bool*>(_v) = _t->getFlagDMSDegrees(); break;
        case 21: *reinterpret_cast<bool*>(_v) = _t->getFlagHorizontalCoordinates(); break;
        case 22: *reinterpret_cast<bool*>(_v) = _t->getFlagAutosetMountForCCD(); break;
        case 23: *reinterpret_cast<bool*>(_v) = _t->getFlagScalingFOVForTelrad(); break;
        case 24: *reinterpret_cast<Vec4f*>(_v) = _t->getTelradFOV(); break;
        case 25: *reinterpret_cast<bool*>(_v) = _t->getFlagScalingFOVForCCD(); break;
        case 26: *reinterpret_cast<bool*>(_v) = _t->getFlagMaxExposureTimeForCCD(); break;
        case 27: *reinterpret_cast<bool*>(_v) = _t->getFlagShowOcularsButton(); break;
        case 28: *reinterpret_cast<bool*>(_v) = _t->getFlagShowContour(); break;
        case 29: *reinterpret_cast<bool*>(_v) = _t->getFlagShowCardinals(); break;
        case 30: *reinterpret_cast<bool*>(_v) = _t->getFlagAlignCrosshair(); break;
        case 31: *reinterpret_cast<int*>(_v) = _t->getArrowButtonScale(); break;
        case 32: *reinterpret_cast<int*>(_v) = _t->getGuiPanelFontSize(); break;
        case 33: *reinterpret_cast<Vec3f*>(_v) = _t->getTextColor(); break;
        case 34: *reinterpret_cast<Vec3f*>(_v) = _t->getLineColor(); break;
        case 35: *reinterpret_cast<bool*>(_v) = _t->getFlagShowCcdCropOverlay(); break;
        case 36: *reinterpret_cast<bool*>(_v) = _t->getFlagShowCcdCropOverlayPixelGrid(); break;
        case 37: *reinterpret_cast<int*>(_v) = _t->getCcdCropOverlayHSize(); break;
        case 38: *reinterpret_cast<int*>(_v) = _t->getCcdCropOverlayVSize(); break;
        case 39: *reinterpret_cast<bool*>(_v) = _t->getFlagShowFocuserOverlay(); break;
        case 40: *reinterpret_cast<bool*>(_v) = _t->getFlagUseSmallFocuserOverlay(); break;
        case 41: *reinterpret_cast<bool*>(_v) = _t->getFlagUseMediumFocuserOverlay(); break;
        case 42: *reinterpret_cast<bool*>(_v) = _t->getFlagUseLargeFocuserOverlay(); break;
        case 43: *reinterpret_cast<Vec3f*>(_v) = _t->getFocuserColor(); break;
        case 44: *reinterpret_cast<bool*>(_v) = _t->getFlagShowOAGLimits(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->enableOcular(*reinterpret_cast<bool*>(_v)); break;
        case 1: _t->toggleCrosshairs(*reinterpret_cast<bool*>(_v)); break;
        case 2: _t->toggleCCD(*reinterpret_cast<bool*>(_v)); break;
        case 3: _t->toggleTelrad(*reinterpret_cast<bool*>(_v)); break;
        case 4: _t->selectCCDAtIndex(*reinterpret_cast<int*>(_v)); break;
        case 5: _t->selectOcularAtIndex(*reinterpret_cast<int*>(_v)); break;
        case 6: _t->selectTelescopeAtIndex(*reinterpret_cast<int*>(_v)); break;
        case 7: _t->selectLensAtIndex(*reinterpret_cast<int*>(_v)); break;
        case 8: _t->setSelectedCCDRotationAngle(*reinterpret_cast<double*>(_v)); break;
        case 9: _t->setSelectedCCDPrismPositionAngle(*reinterpret_cast<double*>(_v)); break;
        case 10: _t->enableGuiPanel(*reinterpret_cast<bool*>(_v)); break;
        case 11: _t->setFlagInitFovUsage(*reinterpret_cast<bool*>(_v)); break;
        case 12: _t->setFlagInitDirectionUsage(*reinterpret_cast<bool*>(_v)); break;
        case 13: _t->setFlagShowResolutionCriteria(*reinterpret_cast<bool*>(_v)); break;
        case 14: _t->setFlagRequireSelection(*reinterpret_cast<bool*>(_v)); break;
        case 15: _t->setFlagAutoLimitMagnitude(*reinterpret_cast<bool*>(_v)); break;
        case 16: _t->setFlagHideGridsLines(*reinterpret_cast<bool*>(_v)); break;
        case 17: _t->setFlagScaleImageCircle(*reinterpret_cast<bool*>(_v)); break;
        case 18: _t->setFlagUseSemiTransparency(*reinterpret_cast<bool*>(_v)); break;
        case 19: _t->setTransparencyMask(*reinterpret_cast<int*>(_v)); break;
        case 20: _t->setFlagDMSDegrees(*reinterpret_cast<bool*>(_v)); break;
        case 21: _t->setFlagHorizontalCoordinates(*reinterpret_cast<bool*>(_v)); break;
        case 22: _t->setFlagAutosetMountForCCD(*reinterpret_cast<bool*>(_v)); break;
        case 23: _t->setFlagScalingFOVForTelrad(*reinterpret_cast<bool*>(_v)); break;
        case 24: _t->setTelradFOV(*reinterpret_cast<Vec4f*>(_v)); break;
        case 25: _t->setFlagScalingFOVForCCD(*reinterpret_cast<bool*>(_v)); break;
        case 26: _t->setFlagMaxExposureTimeForCCD(*reinterpret_cast<bool*>(_v)); break;
        case 27: _t->setFlagShowOcularsButton(*reinterpret_cast<bool*>(_v)); break;
        case 28: _t->setFlagShowContour(*reinterpret_cast<bool*>(_v)); break;
        case 29: _t->setFlagShowCardinals(*reinterpret_cast<bool*>(_v)); break;
        case 30: _t->setFlagAlignCrosshair(*reinterpret_cast<bool*>(_v)); break;
        case 31: _t->setArrowButtonScale(*reinterpret_cast<int*>(_v)); break;
        case 32: _t->setGuiPanelFontSize(*reinterpret_cast<int*>(_v)); break;
        case 33: _t->setTextColor(*reinterpret_cast<Vec3f*>(_v)); break;
        case 34: _t->setLineColor(*reinterpret_cast<Vec3f*>(_v)); break;
        case 35: _t->setFlagShowCcdCropOverlay(*reinterpret_cast<bool*>(_v)); break;
        case 36: _t->setFlagShowCcdCropOverlayPixelGrid(*reinterpret_cast<bool*>(_v)); break;
        case 37: _t->setCcdCropOverlayHSize(*reinterpret_cast<int*>(_v)); break;
        case 38: _t->setCcdCropOverlayVSize(*reinterpret_cast<int*>(_v)); break;
        case 39: _t->setFlagShowFocuserOverlay(*reinterpret_cast<bool*>(_v)); break;
        case 40: _t->setFlagUseSmallFocuserOverlay(*reinterpret_cast<bool*>(_v)); break;
        case 41: _t->setFlagUseMediumFocuserOverlay(*reinterpret_cast<bool*>(_v)); break;
        case 42: _t->setFlagUseLargeFocuserOverlay(*reinterpret_cast<bool*>(_v)); break;
        case 43: _t->setFocuserColor(*reinterpret_cast<Vec3f*>(_v)); break;
        case 44: _t->setFlagShowOAGLimits(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *Oculars::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Oculars::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7OcularsE_t>.strings))
        return static_cast<void*>(this);
    return StelModule::qt_metacast(_clname);
}

int Oculars::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StelModule::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 172)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 172;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 172)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 172;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 45;
    }
    return _id;
}

// SIGNAL 0
void Oculars::enableOcularChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void Oculars::enableCrosshairsChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void Oculars::enableCCDChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void Oculars::enableTelradChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void Oculars::selectedCCDChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void Oculars::selectedOcularChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void Oculars::selectedTelescopeChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void Oculars::selectedLensChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void Oculars::selectedCCDRotationAngleChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void Oculars::selectedCCDPrismPositionAngleChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void Oculars::flagGuiPanelEnabledChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void Oculars::guiPanelFontSizeChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void Oculars::textColorChanged(const Vec3f & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void Oculars::lineColorChanged(const Vec3f & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1);
}

// SIGNAL 14
void Oculars::focuserColorChanged(const Vec3f & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1);
}

// SIGNAL 15
void Oculars::flagHideGridsLinesChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 15, nullptr, _t1);
}

// SIGNAL 16
void Oculars::flagAutosetMountForCCDChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 16, nullptr, _t1);
}

// SIGNAL 17
void Oculars::flagScalingFOVForTelradChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 17, nullptr, _t1);
}

// SIGNAL 18
void Oculars::telradFOVChanged(Vec4f _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 18, nullptr, _t1);
}

// SIGNAL 19
void Oculars::flagScalingFOVForCCDChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 19, nullptr, _t1);
}

// SIGNAL 20
void Oculars::flagMaxExposureTimeForCCDChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 20, nullptr, _t1);
}

// SIGNAL 21
void Oculars::flagUseSemiTransparencyChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 21, nullptr, _t1);
}

// SIGNAL 22
void Oculars::transparencyMaskChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 22, nullptr, _t1);
}

// SIGNAL 23
void Oculars::flagShowResolutionCriteriaChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 23, nullptr, _t1);
}

// SIGNAL 24
void Oculars::arrowButtonScaleChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 24, nullptr, _t1);
}

// SIGNAL 25
void Oculars::flagInitDirectionUsageChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 25, nullptr, _t1);
}

// SIGNAL 26
void Oculars::flagInitFOVUsageChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 26, nullptr, _t1);
}

// SIGNAL 27
void Oculars::flagRequireSelectionChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 27, nullptr, _t1);
}

// SIGNAL 28
void Oculars::flagAutoLimitMagnitudeChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 28, nullptr, _t1);
}

// SIGNAL 29
void Oculars::flagDMSDegreesChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 29, nullptr, _t1);
}

// SIGNAL 30
void Oculars::flagHorizontalCoordinatesChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 30, nullptr, _t1);
}

// SIGNAL 31
void Oculars::flagScaleImageCircleChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 31, nullptr, _t1);
}

// SIGNAL 32
void Oculars::flagShowOcularsButtonChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 32, nullptr, _t1);
}

// SIGNAL 33
void Oculars::flagShowCcdCropOverlayChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 33, nullptr, _t1);
}

// SIGNAL 34
void Oculars::ccdCropOverlayHSizeChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 34, nullptr, _t1);
}

// SIGNAL 35
void Oculars::ccdCropOverlayVSizeChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 35, nullptr, _t1);
}

// SIGNAL 36
void Oculars::flagShowCcdCropOverlayPixelGridChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 36, nullptr, _t1);
}

// SIGNAL 37
void Oculars::flagShowContourChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 37, nullptr, _t1);
}

// SIGNAL 38
void Oculars::flagShowCardinalsChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 38, nullptr, _t1);
}

// SIGNAL 39
void Oculars::flagAlignCrosshairChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 39, nullptr, _t1);
}

// SIGNAL 40
void Oculars::flagShowFocuserOverlayChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 40, nullptr, _t1);
}

// SIGNAL 41
void Oculars::flagUseSmallFocuserOverlayChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 41, nullptr, _t1);
}

// SIGNAL 42
void Oculars::flagUseMediumFocuserOverlayChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 42, nullptr, _t1);
}

// SIGNAL 43
void Oculars::flagUseLargeFocuserOverlayChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 43, nullptr, _t1);
}

// SIGNAL 44
void Oculars::flagShowOAGLimitsChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 44, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN26OcularsStelPluginInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto OcularsStelPluginInterface::qt_create_metaobjectdata<qt_meta_tag_ZN26OcularsStelPluginInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OcularsStelPluginInterface"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<OcularsStelPluginInterface, qt_meta_tag_ZN26OcularsStelPluginInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OcularsStelPluginInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26OcularsStelPluginInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26OcularsStelPluginInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN26OcularsStelPluginInterfaceE_t>.metaTypes,
    nullptr
} };

void OcularsStelPluginInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<OcularsStelPluginInterface *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *OcularsStelPluginInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OcularsStelPluginInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26OcularsStelPluginInterfaceE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "StelPluginInterface"))
        return static_cast< StelPluginInterface*>(this);
    if (!strcmp(_clname, "org.stellarium.StelPluginInterface/2.0"))
        return static_cast< StelPluginInterface*>(this);
    return QObject::qt_metacast(_clname);
}

int OcularsStelPluginInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}

#ifdef QT_MOC_EXPORT_PLUGIN_V2
static constexpr unsigned char qt_pluginMetaDataV2_OcularsStelPluginInterface[] = {
    0xbf, 
    // "IID"
    0x02,  0x78,  0x26,  'o',  'r',  'g',  '.',  's', 
    't',  'e',  'l',  'l',  'a',  'r',  'i',  'u', 
    'm',  '.',  'S',  't',  'e',  'l',  'P',  'l', 
    'u',  'g',  'i',  'n',  'I',  'n',  't',  'e', 
    'r',  'f',  'a',  'c',  'e',  '/',  '2',  '.', 
    '0', 
    // "className"
    0x03,  0x78,  0x1a,  'O',  'c',  'u',  'l',  'a', 
    'r',  's',  'S',  't',  'e',  'l',  'P',  'l', 
    'u',  'g',  'i',  'n',  'I',  'n',  't',  'e', 
    'r',  'f',  'a',  'c',  'e', 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN_V2(OcularsStelPluginInterface, OcularsStelPluginInterface, qt_pluginMetaDataV2_OcularsStelPluginInterface)
#else
QT_PLUGIN_METADATA_SECTION
Q_CONSTINIT static constexpr unsigned char qt_pluginMetaData_OcularsStelPluginInterface[] = {
    'Q', 'T', 'M', 'E', 'T', 'A', 'D', 'A', 'T', 'A', ' ', '!',
    // metadata version, Qt version, architectural requirements
    0, QT_VERSION_MAJOR, QT_VERSION_MINOR, qPluginArchRequirements(),
    0xbf, 
    // "IID"
    0x02,  0x78,  0x26,  'o',  'r',  'g',  '.',  's', 
    't',  'e',  'l',  'l',  'a',  'r',  'i',  'u', 
    'm',  '.',  'S',  't',  'e',  'l',  'P',  'l', 
    'u',  'g',  'i',  'n',  'I',  'n',  't',  'e', 
    'r',  'f',  'a',  'c',  'e',  '/',  '2',  '.', 
    '0', 
    // "className"
    0x03,  0x78,  0x1a,  'O',  'c',  'u',  'l',  'a', 
    'r',  's',  'S',  't',  'e',  'l',  'P',  'l', 
    'u',  'g',  'i',  'n',  'I',  'n',  't',  'e', 
    'r',  'f',  'a',  'c',  'e', 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN(OcularsStelPluginInterface, OcularsStelPluginInterface)
#endif  // QT_MOC_EXPORT_PLUGIN_V2

QT_WARNING_POP
