/****************************************************************************
** Meta object code from reading C++ file 'Skylight.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../stellarium/src/core/modules/Skylight.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'Skylight.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8SkylightE_t {};
} // unnamed namespace

template <> constexpr inline auto Skylight::qt_create_metaobjectdata<qt_meta_tag_ZN8SkylightE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Skylight",
        "AYtChanged",
        "",
        "val",
        "BYtChanged",
        "CYtChanged",
        "DYtChanged",
        "EYtChanged",
        "AYcChanged",
        "BYcChanged",
        "CYcChanged",
        "DYcChanged",
        "EYcChanged",
        "AxtChanged",
        "BxtChanged",
        "CxtChanged",
        "DxtChanged",
        "ExtChanged",
        "AxcChanged",
        "BxcChanged",
        "CxcChanged",
        "DxcChanged",
        "ExcChanged",
        "AytChanged",
        "BytChanged",
        "CytChanged",
        "DytChanged",
        "EytChanged",
        "AycChanged",
        "BycChanged",
        "CycChanged",
        "DycChanged",
        "EycChanged",
        "zX11Changed",
        "zX12Changed",
        "zX13Changed",
        "zX21Changed",
        "zX22Changed",
        "zX23Changed",
        "zX24Changed",
        "zX31Changed",
        "zX32Changed",
        "zX33Changed",
        "zX34Changed",
        "zY11Changed",
        "zY12Changed",
        "zY13Changed",
        "zY21Changed",
        "zY22Changed",
        "zY23Changed",
        "zY24Changed",
        "zY31Changed",
        "zY32Changed",
        "zY33Changed",
        "zY34Changed",
        "turbidityChanged",
        "flagSchaeferChanged",
        "flagGuiPublicChanged",
        "setAYt",
        "setBYt",
        "setCYt",
        "setDYt",
        "setEYt",
        "setAYc",
        "setBYc",
        "setCYc",
        "setDYc",
        "setEYc",
        "setAxt",
        "setBxt",
        "setCxt",
        "setDxt",
        "setExt",
        "setAxc",
        "setBxc",
        "setCxc",
        "setDxc",
        "setExc",
        "setAyt",
        "setByt",
        "setCyt",
        "setDyt",
        "setEyt",
        "setAyc",
        "setByc",
        "setCyc",
        "setDyc",
        "setEyc",
        "setZX11",
        "setZX12",
        "setZX13",
        "setZX21",
        "setZX22",
        "setZX23",
        "setZX24",
        "setZX31",
        "setZX32",
        "setZX33",
        "setZX34",
        "setZY11",
        "setZY12",
        "setZY13",
        "setZY21",
        "setZY22",
        "setZY23",
        "setZY24",
        "setZY31",
        "setZY32",
        "setZY33",
        "setZY34",
        "setT",
        "newT",
        "setFlagSchaefer",
        "setFlagGuiPublic",
        "AYt",
        "BYt",
        "CYt",
        "DYt",
        "EYt",
        "AYc",
        "BYc",
        "CYc",
        "DYc",
        "EYc",
        "Axt",
        "Bxt",
        "Cxt",
        "Dxt",
        "Ext",
        "Axc",
        "Bxc",
        "Cxc",
        "Dxc",
        "Exc",
        "Ayt",
        "Byt",
        "Cyt",
        "Dyt",
        "Eyt",
        "Ayc",
        "Byc",
        "Cyc",
        "Dyc",
        "Eyc",
        "zX11",
        "zX12",
        "zX13",
        "zX21",
        "zX22",
        "zX23",
        "zX24",
        "zX31",
        "zX32",
        "zX33",
        "zX34",
        "zY11",
        "zY12",
        "zY13",
        "zY21",
        "zY22",
        "zY23",
        "zY24",
        "zY31",
        "zY32",
        "zY33",
        "zY34",
        "T",
        "flagSchaefer",
        "flagGuiPublic"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'AYtChanged'
        QtMocHelpers::SignalData<void(double)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Signal 'BYtChanged'
        QtMocHelpers::SignalData<void(double)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Signal 'CYtChanged'
        QtMocHelpers::SignalData<void(double)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Signal 'DYtChanged'
        QtMocHelpers::SignalData<void(double)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Signal 'EYtChanged'
        QtMocHelpers::SignalData<void(double)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Signal 'AYcChanged'
        QtMocHelpers::SignalData<void(double)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Signal 'BYcChanged'
        QtMocHelpers::SignalData<void(double)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Signal 'CYcChanged'
        QtMocHelpers::SignalData<void(double)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Signal 'DYcChanged'
        QtMocHelpers::SignalData<void(double)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Signal 'EYcChanged'
        QtMocHelpers::SignalData<void(double)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Signal 'AxtChanged'
        QtMocHelpers::SignalData<void(double)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Signal 'BxtChanged'
        QtMocHelpers::SignalData<void(double)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Signal 'CxtChanged'
        QtMocHelpers::SignalData<void(double)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Signal 'DxtChanged'
        QtMocHelpers::SignalData<void(double)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Signal 'ExtChanged'
        QtMocHelpers::SignalData<void(double)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Signal 'AxcChanged'
        QtMocHelpers::SignalData<void(double)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Signal 'BxcChanged'
        QtMocHelpers::SignalData<void(double)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Signal 'CxcChanged'
        QtMocHelpers::SignalData<void(double)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Signal 'DxcChanged'
        QtMocHelpers::SignalData<void(double)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Signal 'ExcChanged'
        QtMocHelpers::SignalData<void(double)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Signal 'AytChanged'
        QtMocHelpers::SignalData<void(double)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Signal 'BytChanged'
        QtMocHelpers::SignalData<void(double)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Signal 'CytChanged'
        QtMocHelpers::SignalData<void(double)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Signal 'DytChanged'
        QtMocHelpers::SignalData<void(double)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Signal 'EytChanged'
        QtMocHelpers::SignalData<void(double)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Signal 'AycChanged'
        QtMocHelpers::SignalData<void(double)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Signal 'BycChanged'
        QtMocHelpers::SignalData<void(double)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Signal 'CycChanged'
        QtMocHelpers::SignalData<void(double)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Signal 'DycChanged'
        QtMocHelpers::SignalData<void(double)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Signal 'EycChanged'
        QtMocHelpers::SignalData<void(double)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Signal 'zX11Changed'
        QtMocHelpers::SignalData<void(double)>(33, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Signal 'zX12Changed'
        QtMocHelpers::SignalData<void(double)>(34, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Signal 'zX13Changed'
        QtMocHelpers::SignalData<void(double)>(35, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Signal 'zX21Changed'
        QtMocHelpers::SignalData<void(double)>(36, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Signal 'zX22Changed'
        QtMocHelpers::SignalData<void(double)>(37, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Signal 'zX23Changed'
        QtMocHelpers::SignalData<void(double)>(38, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Signal 'zX24Changed'
        QtMocHelpers::SignalData<void(double)>(39, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Signal 'zX31Changed'
        QtMocHelpers::SignalData<void(double)>(40, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Signal 'zX32Changed'
        QtMocHelpers::SignalData<void(double)>(41, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Signal 'zX33Changed'
        QtMocHelpers::SignalData<void(double)>(42, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Signal 'zX34Changed'
        QtMocHelpers::SignalData<void(double)>(43, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Signal 'zY11Changed'
        QtMocHelpers::SignalData<void(double)>(44, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Signal 'zY12Changed'
        QtMocHelpers::SignalData<void(double)>(45, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Signal 'zY13Changed'
        QtMocHelpers::SignalData<void(double)>(46, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Signal 'zY21Changed'
        QtMocHelpers::SignalData<void(double)>(47, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Signal 'zY22Changed'
        QtMocHelpers::SignalData<void(double)>(48, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Signal 'zY23Changed'
        QtMocHelpers::SignalData<void(double)>(49, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Signal 'zY24Changed'
        QtMocHelpers::SignalData<void(double)>(50, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Signal 'zY31Changed'
        QtMocHelpers::SignalData<void(double)>(51, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Signal 'zY32Changed'
        QtMocHelpers::SignalData<void(double)>(52, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Signal 'zY33Changed'
        QtMocHelpers::SignalData<void(double)>(53, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Signal 'zY34Changed'
        QtMocHelpers::SignalData<void(double)>(54, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Signal 'turbidityChanged'
        QtMocHelpers::SignalData<void(double)>(55, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Signal 'flagSchaeferChanged'
        QtMocHelpers::SignalData<void(bool)>(56, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagGuiPublicChanged'
        QtMocHelpers::SignalData<void(bool)>(57, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'setAYt'
        QtMocHelpers::SlotData<void(const double)>(58, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setBYt'
        QtMocHelpers::SlotData<void(const double)>(59, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setCYt'
        QtMocHelpers::SlotData<void(const double)>(60, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setDYt'
        QtMocHelpers::SlotData<void(const double)>(61, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setEYt'
        QtMocHelpers::SlotData<void(const double)>(62, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setAYc'
        QtMocHelpers::SlotData<void(const double)>(63, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setBYc'
        QtMocHelpers::SlotData<void(const double)>(64, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setCYc'
        QtMocHelpers::SlotData<void(const double)>(65, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setDYc'
        QtMocHelpers::SlotData<void(const double)>(66, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setEYc'
        QtMocHelpers::SlotData<void(const double)>(67, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setAxt'
        QtMocHelpers::SlotData<void(const double)>(68, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setBxt'
        QtMocHelpers::SlotData<void(const double)>(69, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setCxt'
        QtMocHelpers::SlotData<void(const double)>(70, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setDxt'
        QtMocHelpers::SlotData<void(const double)>(71, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setExt'
        QtMocHelpers::SlotData<void(const double)>(72, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setAxc'
        QtMocHelpers::SlotData<void(const double)>(73, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setBxc'
        QtMocHelpers::SlotData<void(const double)>(74, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setCxc'
        QtMocHelpers::SlotData<void(const double)>(75, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setDxc'
        QtMocHelpers::SlotData<void(const double)>(76, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setExc'
        QtMocHelpers::SlotData<void(const double)>(77, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setAyt'
        QtMocHelpers::SlotData<void(const double)>(78, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setByt'
        QtMocHelpers::SlotData<void(const double)>(79, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setCyt'
        QtMocHelpers::SlotData<void(const double)>(80, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setDyt'
        QtMocHelpers::SlotData<void(const double)>(81, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setEyt'
        QtMocHelpers::SlotData<void(const double)>(82, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setAyc'
        QtMocHelpers::SlotData<void(const double)>(83, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setByc'
        QtMocHelpers::SlotData<void(const double)>(84, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setCyc'
        QtMocHelpers::SlotData<void(const double)>(85, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setDyc'
        QtMocHelpers::SlotData<void(const double)>(86, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setEyc'
        QtMocHelpers::SlotData<void(const double)>(87, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setZX11'
        QtMocHelpers::SlotData<void(const double)>(88, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setZX12'
        QtMocHelpers::SlotData<void(const double)>(89, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setZX13'
        QtMocHelpers::SlotData<void(const double)>(90, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setZX21'
        QtMocHelpers::SlotData<void(const double)>(91, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setZX22'
        QtMocHelpers::SlotData<void(const double)>(92, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setZX23'
        QtMocHelpers::SlotData<void(const double)>(93, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setZX24'
        QtMocHelpers::SlotData<void(const double)>(94, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setZX31'
        QtMocHelpers::SlotData<void(const double)>(95, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setZX32'
        QtMocHelpers::SlotData<void(const double)>(96, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setZX33'
        QtMocHelpers::SlotData<void(const double)>(97, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setZX34'
        QtMocHelpers::SlotData<void(const double)>(98, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setZY11'
        QtMocHelpers::SlotData<void(const double)>(99, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setZY12'
        QtMocHelpers::SlotData<void(const double)>(100, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setZY13'
        QtMocHelpers::SlotData<void(const double)>(101, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setZY21'
        QtMocHelpers::SlotData<void(const double)>(102, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setZY22'
        QtMocHelpers::SlotData<void(const double)>(103, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setZY23'
        QtMocHelpers::SlotData<void(const double)>(104, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setZY24'
        QtMocHelpers::SlotData<void(const double)>(105, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setZY31'
        QtMocHelpers::SlotData<void(const double)>(106, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setZY32'
        QtMocHelpers::SlotData<void(const double)>(107, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setZY33'
        QtMocHelpers::SlotData<void(const double)>(108, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setZY34'
        QtMocHelpers::SlotData<void(const double)>(109, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setT'
        QtMocHelpers::SlotData<void(double)>(110, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 111 },
        }}),
        // Slot 'setFlagSchaefer'
        QtMocHelpers::SlotData<void(bool)>(112, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'setFlagGuiPublic'
        QtMocHelpers::SlotData<void(bool)>(113, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'AYt'
        QtMocHelpers::PropertyData<double>(114, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'BYt'
        QtMocHelpers::PropertyData<double>(115, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'CYt'
        QtMocHelpers::PropertyData<double>(116, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'DYt'
        QtMocHelpers::PropertyData<double>(117, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'EYt'
        QtMocHelpers::PropertyData<double>(118, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'AYc'
        QtMocHelpers::PropertyData<double>(119, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
        // property 'BYc'
        QtMocHelpers::PropertyData<double>(120, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 6),
        // property 'CYc'
        QtMocHelpers::PropertyData<double>(121, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 7),
        // property 'DYc'
        QtMocHelpers::PropertyData<double>(122, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 8),
        // property 'EYc'
        QtMocHelpers::PropertyData<double>(123, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 9),
        // property 'Axt'
        QtMocHelpers::PropertyData<double>(124, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 10),
        // property 'Bxt'
        QtMocHelpers::PropertyData<double>(125, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 11),
        // property 'Cxt'
        QtMocHelpers::PropertyData<double>(126, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 12),
        // property 'Dxt'
        QtMocHelpers::PropertyData<double>(127, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 13),
        // property 'Ext'
        QtMocHelpers::PropertyData<double>(128, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 14),
        // property 'Axc'
        QtMocHelpers::PropertyData<double>(129, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 15),
        // property 'Bxc'
        QtMocHelpers::PropertyData<double>(130, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 16),
        // property 'Cxc'
        QtMocHelpers::PropertyData<double>(131, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 17),
        // property 'Dxc'
        QtMocHelpers::PropertyData<double>(132, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 18),
        // property 'Exc'
        QtMocHelpers::PropertyData<double>(133, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 19),
        // property 'Ayt'
        QtMocHelpers::PropertyData<double>(134, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 20),
        // property 'Byt'
        QtMocHelpers::PropertyData<double>(135, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 21),
        // property 'Cyt'
        QtMocHelpers::PropertyData<double>(136, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 22),
        // property 'Dyt'
        QtMocHelpers::PropertyData<double>(137, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 23),
        // property 'Eyt'
        QtMocHelpers::PropertyData<double>(138, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 24),
        // property 'Ayc'
        QtMocHelpers::PropertyData<double>(139, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 25),
        // property 'Byc'
        QtMocHelpers::PropertyData<double>(140, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 26),
        // property 'Cyc'
        QtMocHelpers::PropertyData<double>(141, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 27),
        // property 'Dyc'
        QtMocHelpers::PropertyData<double>(142, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 28),
        // property 'Eyc'
        QtMocHelpers::PropertyData<double>(143, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 29),
        // property 'zX11'
        QtMocHelpers::PropertyData<double>(144, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 30),
        // property 'zX12'
        QtMocHelpers::PropertyData<double>(145, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 31),
        // property 'zX13'
        QtMocHelpers::PropertyData<double>(146, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 32),
        // property 'zX21'
        QtMocHelpers::PropertyData<double>(147, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 33),
        // property 'zX22'
        QtMocHelpers::PropertyData<double>(148, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 34),
        // property 'zX23'
        QtMocHelpers::PropertyData<double>(149, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 35),
        // property 'zX24'
        QtMocHelpers::PropertyData<double>(150, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 36),
        // property 'zX31'
        QtMocHelpers::PropertyData<double>(151, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 37),
        // property 'zX32'
        QtMocHelpers::PropertyData<double>(152, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 38),
        // property 'zX33'
        QtMocHelpers::PropertyData<double>(153, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 39),
        // property 'zX34'
        QtMocHelpers::PropertyData<double>(154, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 40),
        // property 'zY11'
        QtMocHelpers::PropertyData<double>(155, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 41),
        // property 'zY12'
        QtMocHelpers::PropertyData<double>(156, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 42),
        // property 'zY13'
        QtMocHelpers::PropertyData<double>(157, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 43),
        // property 'zY21'
        QtMocHelpers::PropertyData<double>(158, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 44),
        // property 'zY22'
        QtMocHelpers::PropertyData<double>(159, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 45),
        // property 'zY23'
        QtMocHelpers::PropertyData<double>(160, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 46),
        // property 'zY24'
        QtMocHelpers::PropertyData<double>(161, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 47),
        // property 'zY31'
        QtMocHelpers::PropertyData<double>(162, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 48),
        // property 'zY32'
        QtMocHelpers::PropertyData<double>(163, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 49),
        // property 'zY33'
        QtMocHelpers::PropertyData<double>(164, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 50),
        // property 'zY34'
        QtMocHelpers::PropertyData<double>(165, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 51),
        // property 'T'
        QtMocHelpers::PropertyData<double>(166, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 52),
        // property 'flagSchaefer'
        QtMocHelpers::PropertyData<bool>(167, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 53),
        // property 'flagGuiPublic'
        QtMocHelpers::PropertyData<bool>(168, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 54),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Skylight, qt_meta_tag_ZN8SkylightE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Skylight::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8SkylightE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8SkylightE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8SkylightE_t>.metaTypes,
    nullptr
} };

void Skylight::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Skylight *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->AYtChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 1: _t->BYtChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 2: _t->CYtChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 3: _t->DYtChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 4: _t->EYtChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 5: _t->AYcChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 6: _t->BYcChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 7: _t->CYcChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 8: _t->DYcChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 9: _t->EYcChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 10: _t->AxtChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 11: _t->BxtChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 12: _t->CxtChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 13: _t->DxtChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 14: _t->ExtChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 15: _t->AxcChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 16: _t->BxcChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 17: _t->CxcChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 18: _t->DxcChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 19: _t->ExcChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 20: _t->AytChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 21: _t->BytChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 22: _t->CytChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 23: _t->DytChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 24: _t->EytChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 25: _t->AycChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 26: _t->BycChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 27: _t->CycChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 28: _t->DycChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 29: _t->EycChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 30: _t->zX11Changed((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 31: _t->zX12Changed((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 32: _t->zX13Changed((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 33: _t->zX21Changed((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 34: _t->zX22Changed((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 35: _t->zX23Changed((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 36: _t->zX24Changed((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 37: _t->zX31Changed((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 38: _t->zX32Changed((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 39: _t->zX33Changed((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 40: _t->zX34Changed((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 41: _t->zY11Changed((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 42: _t->zY12Changed((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 43: _t->zY13Changed((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 44: _t->zY21Changed((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 45: _t->zY22Changed((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 46: _t->zY23Changed((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 47: _t->zY24Changed((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 48: _t->zY31Changed((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 49: _t->zY32Changed((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 50: _t->zY33Changed((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 51: _t->zY34Changed((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 52: _t->turbidityChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 53: _t->flagSchaeferChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 54: _t->flagGuiPublicChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 55: _t->setAYt((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 56: _t->setBYt((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 57: _t->setCYt((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 58: _t->setDYt((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 59: _t->setEYt((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 60: _t->setAYc((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 61: _t->setBYc((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 62: _t->setCYc((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 63: _t->setDYc((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 64: _t->setEYc((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 65: _t->setAxt((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 66: _t->setBxt((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 67: _t->setCxt((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 68: _t->setDxt((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 69: _t->setExt((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 70: _t->setAxc((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 71: _t->setBxc((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 72: _t->setCxc((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 73: _t->setDxc((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 74: _t->setExc((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 75: _t->setAyt((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 76: _t->setByt((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 77: _t->setCyt((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 78: _t->setDyt((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 79: _t->setEyt((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 80: _t->setAyc((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 81: _t->setByc((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 82: _t->setCyc((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 83: _t->setDyc((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 84: _t->setEyc((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 85: _t->setZX11((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 86: _t->setZX12((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 87: _t->setZX13((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 88: _t->setZX21((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 89: _t->setZX22((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 90: _t->setZX23((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 91: _t->setZX24((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 92: _t->setZX31((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 93: _t->setZX32((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 94: _t->setZX33((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 95: _t->setZX34((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 96: _t->setZY11((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 97: _t->setZY12((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 98: _t->setZY13((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 99: _t->setZY21((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 100: _t->setZY22((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 101: _t->setZY23((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 102: _t->setZY24((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 103: _t->setZY31((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 104: _t->setZY32((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 105: _t->setZY33((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 106: _t->setZY34((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 107: _t->setT((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 108: _t->setFlagSchaefer((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 109: _t->setFlagGuiPublic((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Skylight::*)(double )>(_a, &Skylight::AYtChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Skylight::*)(double )>(_a, &Skylight::BYtChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Skylight::*)(double )>(_a, &Skylight::CYtChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Skylight::*)(double )>(_a, &Skylight::DYtChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Skylight::*)(double )>(_a, &Skylight::EYtChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (Skylight::*)(double )>(_a, &Skylight::AYcChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (Skylight::*)(double )>(_a, &Skylight::BYcChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (Skylight::*)(double )>(_a, &Skylight::CYcChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (Skylight::*)(double )>(_a, &Skylight::DYcChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (Skylight::*)(double )>(_a, &Skylight::EYcChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (Skylight::*)(double )>(_a, &Skylight::AxtChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (Skylight::*)(double )>(_a, &Skylight::BxtChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (Skylight::*)(double )>(_a, &Skylight::CxtChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (Skylight::*)(double )>(_a, &Skylight::DxtChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (Skylight::*)(double )>(_a, &Skylight::ExtChanged, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (Skylight::*)(double )>(_a, &Skylight::AxcChanged, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (Skylight::*)(double )>(_a, &Skylight::BxcChanged, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (Skylight::*)(double )>(_a, &Skylight::CxcChanged, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (Skylight::*)(double )>(_a, &Skylight::DxcChanged, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (Skylight::*)(double )>(_a, &Skylight::ExcChanged, 19))
            return;
        if (QtMocHelpers::indexOfMethod<void (Skylight::*)(double )>(_a, &Skylight::AytChanged, 20))
            return;
        if (QtMocHelpers::indexOfMethod<void (Skylight::*)(double )>(_a, &Skylight::BytChanged, 21))
            return;
        if (QtMocHelpers::indexOfMethod<void (Skylight::*)(double )>(_a, &Skylight::CytChanged, 22))
            return;
        if (QtMocHelpers::indexOfMethod<void (Skylight::*)(double )>(_a, &Skylight::DytChanged, 23))
            return;
        if (QtMocHelpers::indexOfMethod<void (Skylight::*)(double )>(_a, &Skylight::EytChanged, 24))
            return;
        if (QtMocHelpers::indexOfMethod<void (Skylight::*)(double )>(_a, &Skylight::AycChanged, 25))
            return;
        if (QtMocHelpers::indexOfMethod<void (Skylight::*)(double )>(_a, &Skylight::BycChanged, 26))
            return;
        if (QtMocHelpers::indexOfMethod<void (Skylight::*)(double )>(_a, &Skylight::CycChanged, 27))
            return;
        if (QtMocHelpers::indexOfMethod<void (Skylight::*)(double )>(_a, &Skylight::DycChanged, 28))
            return;
        if (QtMocHelpers::indexOfMethod<void (Skylight::*)(double )>(_a, &Skylight::EycChanged, 29))
            return;
        if (QtMocHelpers::indexOfMethod<void (Skylight::*)(double )>(_a, &Skylight::zX11Changed, 30))
            return;
        if (QtMocHelpers::indexOfMethod<void (Skylight::*)(double )>(_a, &Skylight::zX12Changed, 31))
            return;
        if (QtMocHelpers::indexOfMethod<void (Skylight::*)(double )>(_a, &Skylight::zX13Changed, 32))
            return;
        if (QtMocHelpers::indexOfMethod<void (Skylight::*)(double )>(_a, &Skylight::zX21Changed, 33))
            return;
        if (QtMocHelpers::indexOfMethod<void (Skylight::*)(double )>(_a, &Skylight::zX22Changed, 34))
            return;
        if (QtMocHelpers::indexOfMethod<void (Skylight::*)(double )>(_a, &Skylight::zX23Changed, 35))
            return;
        if (QtMocHelpers::indexOfMethod<void (Skylight::*)(double )>(_a, &Skylight::zX24Changed, 36))
            return;
        if (QtMocHelpers::indexOfMethod<void (Skylight::*)(double )>(_a, &Skylight::zX31Changed, 37))
            return;
        if (QtMocHelpers::indexOfMethod<void (Skylight::*)(double )>(_a, &Skylight::zX32Changed, 38))
            return;
        if (QtMocHelpers::indexOfMethod<void (Skylight::*)(double )>(_a, &Skylight::zX33Changed, 39))
            return;
        if (QtMocHelpers::indexOfMethod<void (Skylight::*)(double )>(_a, &Skylight::zX34Changed, 40))
            return;
        if (QtMocHelpers::indexOfMethod<void (Skylight::*)(double )>(_a, &Skylight::zY11Changed, 41))
            return;
        if (QtMocHelpers::indexOfMethod<void (Skylight::*)(double )>(_a, &Skylight::zY12Changed, 42))
            return;
        if (QtMocHelpers::indexOfMethod<void (Skylight::*)(double )>(_a, &Skylight::zY13Changed, 43))
            return;
        if (QtMocHelpers::indexOfMethod<void (Skylight::*)(double )>(_a, &Skylight::zY21Changed, 44))
            return;
        if (QtMocHelpers::indexOfMethod<void (Skylight::*)(double )>(_a, &Skylight::zY22Changed, 45))
            return;
        if (QtMocHelpers::indexOfMethod<void (Skylight::*)(double )>(_a, &Skylight::zY23Changed, 46))
            return;
        if (QtMocHelpers::indexOfMethod<void (Skylight::*)(double )>(_a, &Skylight::zY24Changed, 47))
            return;
        if (QtMocHelpers::indexOfMethod<void (Skylight::*)(double )>(_a, &Skylight::zY31Changed, 48))
            return;
        if (QtMocHelpers::indexOfMethod<void (Skylight::*)(double )>(_a, &Skylight::zY32Changed, 49))
            return;
        if (QtMocHelpers::indexOfMethod<void (Skylight::*)(double )>(_a, &Skylight::zY33Changed, 50))
            return;
        if (QtMocHelpers::indexOfMethod<void (Skylight::*)(double )>(_a, &Skylight::zY34Changed, 51))
            return;
        if (QtMocHelpers::indexOfMethod<void (Skylight::*)(double )>(_a, &Skylight::turbidityChanged, 52))
            return;
        if (QtMocHelpers::indexOfMethod<void (Skylight::*)(bool )>(_a, &Skylight::flagSchaeferChanged, 53))
            return;
        if (QtMocHelpers::indexOfMethod<void (Skylight::*)(bool )>(_a, &Skylight::flagGuiPublicChanged, 54))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<double*>(_v) = _t->getAYt(); break;
        case 1: *reinterpret_cast<double*>(_v) = _t->getBYt(); break;
        case 2: *reinterpret_cast<double*>(_v) = _t->getCYt(); break;
        case 3: *reinterpret_cast<double*>(_v) = _t->getDYt(); break;
        case 4: *reinterpret_cast<double*>(_v) = _t->getEYt(); break;
        case 5: *reinterpret_cast<double*>(_v) = _t->getAYc(); break;
        case 6: *reinterpret_cast<double*>(_v) = _t->getBYc(); break;
        case 7: *reinterpret_cast<double*>(_v) = _t->getCYc(); break;
        case 8: *reinterpret_cast<double*>(_v) = _t->getDYc(); break;
        case 9: *reinterpret_cast<double*>(_v) = _t->getEYc(); break;
        case 10: *reinterpret_cast<double*>(_v) = _t->getAxt(); break;
        case 11: *reinterpret_cast<double*>(_v) = _t->getBxt(); break;
        case 12: *reinterpret_cast<double*>(_v) = _t->getCxt(); break;
        case 13: *reinterpret_cast<double*>(_v) = _t->getDxt(); break;
        case 14: *reinterpret_cast<double*>(_v) = _t->getExt(); break;
        case 15: *reinterpret_cast<double*>(_v) = _t->getAxc(); break;
        case 16: *reinterpret_cast<double*>(_v) = _t->getBxc(); break;
        case 17: *reinterpret_cast<double*>(_v) = _t->getCxc(); break;
        case 18: *reinterpret_cast<double*>(_v) = _t->getDxc(); break;
        case 19: *reinterpret_cast<double*>(_v) = _t->getExc(); break;
        case 20: *reinterpret_cast<double*>(_v) = _t->getAyt(); break;
        case 21: *reinterpret_cast<double*>(_v) = _t->getByt(); break;
        case 22: *reinterpret_cast<double*>(_v) = _t->getCyt(); break;
        case 23: *reinterpret_cast<double*>(_v) = _t->getDyt(); break;
        case 24: *reinterpret_cast<double*>(_v) = _t->getEyt(); break;
        case 25: *reinterpret_cast<double*>(_v) = _t->getAyc(); break;
        case 26: *reinterpret_cast<double*>(_v) = _t->getByc(); break;
        case 27: *reinterpret_cast<double*>(_v) = _t->getCyc(); break;
        case 28: *reinterpret_cast<double*>(_v) = _t->getDyc(); break;
        case 29: *reinterpret_cast<double*>(_v) = _t->getEyc(); break;
        case 30: *reinterpret_cast<double*>(_v) = _t->getZX11(); break;
        case 31: *reinterpret_cast<double*>(_v) = _t->getZX12(); break;
        case 32: *reinterpret_cast<double*>(_v) = _t->getZX13(); break;
        case 33: *reinterpret_cast<double*>(_v) = _t->getZX21(); break;
        case 34: *reinterpret_cast<double*>(_v) = _t->getZX22(); break;
        case 35: *reinterpret_cast<double*>(_v) = _t->getZX23(); break;
        case 36: *reinterpret_cast<double*>(_v) = _t->getZX24(); break;
        case 37: *reinterpret_cast<double*>(_v) = _t->getZX31(); break;
        case 38: *reinterpret_cast<double*>(_v) = _t->getZX32(); break;
        case 39: *reinterpret_cast<double*>(_v) = _t->getZX33(); break;
        case 40: *reinterpret_cast<double*>(_v) = _t->getZX34(); break;
        case 41: *reinterpret_cast<double*>(_v) = _t->getZY11(); break;
        case 42: *reinterpret_cast<double*>(_v) = _t->getZY12(); break;
        case 43: *reinterpret_cast<double*>(_v) = _t->getZY13(); break;
        case 44: *reinterpret_cast<double*>(_v) = _t->getZY21(); break;
        case 45: *reinterpret_cast<double*>(_v) = _t->getZY22(); break;
        case 46: *reinterpret_cast<double*>(_v) = _t->getZY23(); break;
        case 47: *reinterpret_cast<double*>(_v) = _t->getZY24(); break;
        case 48: *reinterpret_cast<double*>(_v) = _t->getZY31(); break;
        case 49: *reinterpret_cast<double*>(_v) = _t->getZY32(); break;
        case 50: *reinterpret_cast<double*>(_v) = _t->getZY33(); break;
        case 51: *reinterpret_cast<double*>(_v) = _t->getZY34(); break;
        case 52: *reinterpret_cast<double*>(_v) = _t->getT(); break;
        case 53: *reinterpret_cast<bool*>(_v) = _t->getFlagSchaefer(); break;
        case 54: *reinterpret_cast<bool*>(_v) = _t->getFlagGuiPublic(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setAYt(*reinterpret_cast<double*>(_v)); break;
        case 1: _t->setBYt(*reinterpret_cast<double*>(_v)); break;
        case 2: _t->setCYt(*reinterpret_cast<double*>(_v)); break;
        case 3: _t->setDYt(*reinterpret_cast<double*>(_v)); break;
        case 4: _t->setEYt(*reinterpret_cast<double*>(_v)); break;
        case 5: _t->setAYc(*reinterpret_cast<double*>(_v)); break;
        case 6: _t->setBYc(*reinterpret_cast<double*>(_v)); break;
        case 7: _t->setCYc(*reinterpret_cast<double*>(_v)); break;
        case 8: _t->setDYc(*reinterpret_cast<double*>(_v)); break;
        case 9: _t->setEYc(*reinterpret_cast<double*>(_v)); break;
        case 10: _t->setAxt(*reinterpret_cast<double*>(_v)); break;
        case 11: _t->setBxt(*reinterpret_cast<double*>(_v)); break;
        case 12: _t->setCxt(*reinterpret_cast<double*>(_v)); break;
        case 13: _t->setDxt(*reinterpret_cast<double*>(_v)); break;
        case 14: _t->setExt(*reinterpret_cast<double*>(_v)); break;
        case 15: _t->setAxc(*reinterpret_cast<double*>(_v)); break;
        case 16: _t->setBxc(*reinterpret_cast<double*>(_v)); break;
        case 17: _t->setCxc(*reinterpret_cast<double*>(_v)); break;
        case 18: _t->setDxc(*reinterpret_cast<double*>(_v)); break;
        case 19: _t->setExc(*reinterpret_cast<double*>(_v)); break;
        case 20: _t->setAyt(*reinterpret_cast<double*>(_v)); break;
        case 21: _t->setByt(*reinterpret_cast<double*>(_v)); break;
        case 22: _t->setCyt(*reinterpret_cast<double*>(_v)); break;
        case 23: _t->setDyt(*reinterpret_cast<double*>(_v)); break;
        case 24: _t->setEyt(*reinterpret_cast<double*>(_v)); break;
        case 25: _t->setAyc(*reinterpret_cast<double*>(_v)); break;
        case 26: _t->setByc(*reinterpret_cast<double*>(_v)); break;
        case 27: _t->setCyc(*reinterpret_cast<double*>(_v)); break;
        case 28: _t->setDyc(*reinterpret_cast<double*>(_v)); break;
        case 29: _t->setEyc(*reinterpret_cast<double*>(_v)); break;
        case 30: _t->setZX11(*reinterpret_cast<double*>(_v)); break;
        case 31: _t->setZX12(*reinterpret_cast<double*>(_v)); break;
        case 32: _t->setZX13(*reinterpret_cast<double*>(_v)); break;
        case 33: _t->setZX21(*reinterpret_cast<double*>(_v)); break;
        case 34: _t->setZX22(*reinterpret_cast<double*>(_v)); break;
        case 35: _t->setZX23(*reinterpret_cast<double*>(_v)); break;
        case 36: _t->setZX24(*reinterpret_cast<double*>(_v)); break;
        case 37: _t->setZX31(*reinterpret_cast<double*>(_v)); break;
        case 38: _t->setZX32(*reinterpret_cast<double*>(_v)); break;
        case 39: _t->setZX33(*reinterpret_cast<double*>(_v)); break;
        case 40: _t->setZX34(*reinterpret_cast<double*>(_v)); break;
        case 41: _t->setZY11(*reinterpret_cast<double*>(_v)); break;
        case 42: _t->setZY12(*reinterpret_cast<double*>(_v)); break;
        case 43: _t->setZY13(*reinterpret_cast<double*>(_v)); break;
        case 44: _t->setZY21(*reinterpret_cast<double*>(_v)); break;
        case 45: _t->setZY22(*reinterpret_cast<double*>(_v)); break;
        case 46: _t->setZY23(*reinterpret_cast<double*>(_v)); break;
        case 47: _t->setZY24(*reinterpret_cast<double*>(_v)); break;
        case 48: _t->setZY31(*reinterpret_cast<double*>(_v)); break;
        case 49: _t->setZY32(*reinterpret_cast<double*>(_v)); break;
        case 50: _t->setZY33(*reinterpret_cast<double*>(_v)); break;
        case 51: _t->setZY34(*reinterpret_cast<double*>(_v)); break;
        case 52: _t->setT(*reinterpret_cast<double*>(_v)); break;
        case 53: _t->setFlagSchaefer(*reinterpret_cast<bool*>(_v)); break;
        case 54: _t->setFlagGuiPublic(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *Skylight::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Skylight::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8SkylightE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Skylight::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 110)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 110;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 110)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 110;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 55;
    }
    return _id;
}

// SIGNAL 0
void Skylight::AYtChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void Skylight::BYtChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void Skylight::CYtChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void Skylight::DYtChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void Skylight::EYtChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void Skylight::AYcChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void Skylight::BYcChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void Skylight::CYcChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void Skylight::DYcChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void Skylight::EYcChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void Skylight::AxtChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void Skylight::BxtChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void Skylight::CxtChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void Skylight::DxtChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1);
}

// SIGNAL 14
void Skylight::ExtChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1);
}

// SIGNAL 15
void Skylight::AxcChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 15, nullptr, _t1);
}

// SIGNAL 16
void Skylight::BxcChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 16, nullptr, _t1);
}

// SIGNAL 17
void Skylight::CxcChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 17, nullptr, _t1);
}

// SIGNAL 18
void Skylight::DxcChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 18, nullptr, _t1);
}

// SIGNAL 19
void Skylight::ExcChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 19, nullptr, _t1);
}

// SIGNAL 20
void Skylight::AytChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 20, nullptr, _t1);
}

// SIGNAL 21
void Skylight::BytChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 21, nullptr, _t1);
}

// SIGNAL 22
void Skylight::CytChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 22, nullptr, _t1);
}

// SIGNAL 23
void Skylight::DytChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 23, nullptr, _t1);
}

// SIGNAL 24
void Skylight::EytChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 24, nullptr, _t1);
}

// SIGNAL 25
void Skylight::AycChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 25, nullptr, _t1);
}

// SIGNAL 26
void Skylight::BycChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 26, nullptr, _t1);
}

// SIGNAL 27
void Skylight::CycChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 27, nullptr, _t1);
}

// SIGNAL 28
void Skylight::DycChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 28, nullptr, _t1);
}

// SIGNAL 29
void Skylight::EycChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 29, nullptr, _t1);
}

// SIGNAL 30
void Skylight::zX11Changed(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 30, nullptr, _t1);
}

// SIGNAL 31
void Skylight::zX12Changed(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 31, nullptr, _t1);
}

// SIGNAL 32
void Skylight::zX13Changed(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 32, nullptr, _t1);
}

// SIGNAL 33
void Skylight::zX21Changed(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 33, nullptr, _t1);
}

// SIGNAL 34
void Skylight::zX22Changed(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 34, nullptr, _t1);
}

// SIGNAL 35
void Skylight::zX23Changed(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 35, nullptr, _t1);
}

// SIGNAL 36
void Skylight::zX24Changed(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 36, nullptr, _t1);
}

// SIGNAL 37
void Skylight::zX31Changed(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 37, nullptr, _t1);
}

// SIGNAL 38
void Skylight::zX32Changed(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 38, nullptr, _t1);
}

// SIGNAL 39
void Skylight::zX33Changed(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 39, nullptr, _t1);
}

// SIGNAL 40
void Skylight::zX34Changed(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 40, nullptr, _t1);
}

// SIGNAL 41
void Skylight::zY11Changed(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 41, nullptr, _t1);
}

// SIGNAL 42
void Skylight::zY12Changed(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 42, nullptr, _t1);
}

// SIGNAL 43
void Skylight::zY13Changed(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 43, nullptr, _t1);
}

// SIGNAL 44
void Skylight::zY21Changed(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 44, nullptr, _t1);
}

// SIGNAL 45
void Skylight::zY22Changed(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 45, nullptr, _t1);
}

// SIGNAL 46
void Skylight::zY23Changed(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 46, nullptr, _t1);
}

// SIGNAL 47
void Skylight::zY24Changed(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 47, nullptr, _t1);
}

// SIGNAL 48
void Skylight::zY31Changed(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 48, nullptr, _t1);
}

// SIGNAL 49
void Skylight::zY32Changed(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 49, nullptr, _t1);
}

// SIGNAL 50
void Skylight::zY33Changed(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 50, nullptr, _t1);
}

// SIGNAL 51
void Skylight::zY34Changed(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 51, nullptr, _t1);
}

// SIGNAL 52
void Skylight::turbidityChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 52, nullptr, _t1);
}

// SIGNAL 53
void Skylight::flagSchaeferChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 53, nullptr, _t1);
}

// SIGNAL 54
void Skylight::flagGuiPublicChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 54, nullptr, _t1);
}
QT_WARNING_POP
