/****************************************************************************
** Meta object code from reading C++ file 'StelMovementMgr.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../stellarium/src/core/StelMovementMgr.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'StelMovementMgr.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15StelMovementMgrE_t {};
} // unnamed namespace

template <> constexpr inline auto StelMovementMgr::qt_create_metaobjectdata<qt_meta_tag_ZN15StelMovementMgrE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "StelMovementMgr",
        "flagTrackingChanged",
        "",
        "b",
        "equatorialMountChanged",
        "flagIndicationMountModeChanged",
        "flagAutoZoomOutResetsDirectionChanged",
        "viewportHorizontalOffsetTargetChanged",
        "f",
        "viewportVerticalOffsetTargetChanged",
        "flagEnableMouseNavigationChanged",
        "flagEnableMouseZoomingChanged",
        "flagEnableMoveKeysChanged",
        "flagEnableZoomKeysChanged",
        "userMaxFovChanged",
        "fov",
        "currentFovChanged",
        "currentDirectionChanged",
        "resetInitViewPos",
        "toggleMountMode",
        "setEquatorialMount",
        "setFlagTracking",
        "getFlagTracking",
        "setFlagLockEquPos",
        "getFlagLockEquPos",
        "panView",
        "deltaAz",
        "deltaAlt",
        "setAutoMoveDuration",
        "getAutoMoveDuration",
        "setFlagAutoZoomOutResetsDirection",
        "getFlagAutoZoomOutResetsDirection",
        "getFlagEnableZoomKeys",
        "setFlagEnableZoomKeys",
        "getFlagEnableMoveKeys",
        "setFlagEnableMoveKeys",
        "getFlagEnableMoveAtScreenEdge",
        "setFlagEnableMoveAtScreenEdge",
        "getFlagEnableMouseNavigation",
        "setFlagEnableMouseNavigation",
        "getFlagEnableMouseZooming",
        "setFlagEnableMouseZooming",
        "getFlagIndicationMountMode",
        "setFlagIndicationMountMode",
        "moveToJ2000",
        "Vec3d",
        "aim",
        "aimUp",
        "moveDuration",
        "StelMovementMgr::ZoomingMode",
        "zooming",
        "moveToAltAzi",
        "zoomTo",
        "aimFov",
        "zoomDuration",
        "getCurrentFov",
        "getInitFov",
        "setInitFov",
        "getInitViewingDirection",
        "setInitViewDirectionToCurrent",
        "getViewDirectionJ2000",
        "setViewDirectionJ2000",
        "v",
        "setMaxFov",
        "max",
        "getMaxFov",
        "getMinFov",
        "autoZoomIn",
        "allowManualZoom",
        "autoZoomOut",
        "full",
        "deselection",
        "reSelectLastObject",
        "getAimFov",
        "turnRight",
        "s",
        "turnLeft",
        "turnUp",
        "turnDown",
        "moveSlow",
        "zoomIn",
        "zoomOut",
        "smoothPan",
        "deltaX",
        "deltaY",
        "ptime",
        "lookEast",
        "zero",
        "lookWest",
        "lookNorth",
        "lookSouth",
        "lookZenith",
        "lookNadir",
        "lookTowardsNCP",
        "lookTowardsSCP",
        "moveViewport",
        "offsetX",
        "offsetY",
        "duration",
        "setMountMode",
        "StelMovementMgr::MountMode",
        "m",
        "getMountMode",
        "getEquatorialMount",
        "setInhibitAllAutomoves",
        "inhibit",
        "getViewportOffsetTarget",
        "Vec2d",
        "getViewportHorizontalOffsetTarget",
        "getViewportVerticalOffsetTarget",
        "setViewportHorizontalOffsetTarget",
        "setViewportVerticalOffsetTarget",
        "setUserMaxFov",
        "getUserMaxFov",
        "setFov",
        "selectedObjectChange",
        "StelModule::StelModuleSelectAction",
        "action",
        "handleViewportOffsetMovement",
        "value",
        "setFOVDeg",
        "bindingFOVActions",
        "equatorialMount",
        "tracking",
        "flagIndicationMountMode",
        "viewportHorizontalOffsetTarget",
        "viewportVerticalOffsetTarget",
        "flagAutoZoomOutResetsDirection",
        "flagEnableMouseNavigation",
        "flagEnableMouseZooming",
        "flagEnableMoveKeys",
        "flagEnableZoomKeys",
        "userMaxFov",
        "currentFov",
        "MountMode",
        "MountAltAzimuthal",
        "MountEquinoxEquatorial",
        "MountGalactic",
        "MountSupergalactic",
        "ZoomingMode",
        "ZoomOut",
        "ZoomNone",
        "ZoomIn"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'flagTrackingChanged'
        QtMocHelpers::SignalData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'equatorialMountChanged'
        QtMocHelpers::SignalData<void(bool)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagIndicationMountModeChanged'
        QtMocHelpers::SignalData<void(bool)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagAutoZoomOutResetsDirectionChanged'
        QtMocHelpers::SignalData<void(bool)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'viewportHorizontalOffsetTargetChanged'
        QtMocHelpers::SignalData<void(double)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 8 },
        }}),
        // Signal 'viewportVerticalOffsetTargetChanged'
        QtMocHelpers::SignalData<void(double)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 8 },
        }}),
        // Signal 'flagEnableMouseNavigationChanged'
        QtMocHelpers::SignalData<void(bool)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagEnableMouseZoomingChanged'
        QtMocHelpers::SignalData<void(bool)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagEnableMoveKeysChanged'
        QtMocHelpers::SignalData<void(bool)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagEnableZoomKeysChanged'
        QtMocHelpers::SignalData<void(bool)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'userMaxFovChanged'
        QtMocHelpers::SignalData<void(double)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 15 },
        }}),
        // Signal 'currentFovChanged'
        QtMocHelpers::SignalData<void(double)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 15 },
        }}),
        // Signal 'currentDirectionChanged'
        QtMocHelpers::SignalData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'resetInitViewPos'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'toggleMountMode'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setEquatorialMount'
        QtMocHelpers::SlotData<void(bool)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'setFlagTracking'
        QtMocHelpers::SlotData<void(bool)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'setFlagTracking'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'getFlagTracking'
        QtMocHelpers::SlotData<bool() const>(22, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFlagLockEquPos'
        QtMocHelpers::SlotData<void(bool)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'getFlagLockEquPos'
        QtMocHelpers::SlotData<bool() const>(24, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'panView'
        QtMocHelpers::SlotData<void(const double, const double)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 26 }, { QMetaType::Double, 27 },
        }}),
        // Slot 'setAutoMoveDuration'
        QtMocHelpers::SlotData<void(float)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 8 },
        }}),
        // Slot 'getAutoMoveDuration'
        QtMocHelpers::SlotData<float() const>(29, 2, QMC::AccessPublic, QMetaType::Float),
        // Slot 'setFlagAutoZoomOutResetsDirection'
        QtMocHelpers::SlotData<void(bool)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'getFlagAutoZoomOutResetsDirection'
        QtMocHelpers::SlotData<bool() const>(31, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'getFlagEnableZoomKeys'
        QtMocHelpers::SlotData<bool() const>(32, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFlagEnableZoomKeys'
        QtMocHelpers::SlotData<void(bool)>(33, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'getFlagEnableMoveKeys'
        QtMocHelpers::SlotData<bool() const>(34, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFlagEnableMoveKeys'
        QtMocHelpers::SlotData<void(bool)>(35, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'getFlagEnableMoveAtScreenEdge'
        QtMocHelpers::SlotData<bool() const>(36, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFlagEnableMoveAtScreenEdge'
        QtMocHelpers::SlotData<void(bool)>(37, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'getFlagEnableMouseNavigation'
        QtMocHelpers::SlotData<bool() const>(38, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFlagEnableMouseNavigation'
        QtMocHelpers::SlotData<void(bool)>(39, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'getFlagEnableMouseZooming'
        QtMocHelpers::SlotData<bool() const>(40, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFlagEnableMouseZooming'
        QtMocHelpers::SlotData<void(bool)>(41, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'getFlagIndicationMountMode'
        QtMocHelpers::SlotData<bool() const>(42, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFlagIndicationMountMode'
        QtMocHelpers::SlotData<void(bool)>(43, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'moveToJ2000'
        QtMocHelpers::SlotData<void(const Vec3d &, const Vec3d &, float, StelMovementMgr::ZoomingMode)>(44, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 45, 46 }, { 0x80000000 | 45, 47 }, { QMetaType::Float, 48 }, { 0x80000000 | 49, 50 },
        }}),
        // Slot 'moveToJ2000'
        QtMocHelpers::SlotData<void(const Vec3d &, const Vec3d &, float)>(44, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 45, 46 }, { 0x80000000 | 45, 47 }, { QMetaType::Float, 48 },
        }}),
        // Slot 'moveToJ2000'
        QtMocHelpers::SlotData<void(const Vec3d &, const Vec3d &)>(44, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 45, 46 }, { 0x80000000 | 45, 47 },
        }}),
        // Slot 'moveToAltAzi'
        QtMocHelpers::SlotData<void(const Vec3d &, const Vec3d &, float, StelMovementMgr::ZoomingMode)>(51, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 45, 46 }, { 0x80000000 | 45, 47 }, { QMetaType::Float, 48 }, { 0x80000000 | 49, 50 },
        }}),
        // Slot 'moveToAltAzi'
        QtMocHelpers::SlotData<void(const Vec3d &, const Vec3d &, float)>(51, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 45, 46 }, { 0x80000000 | 45, 47 }, { QMetaType::Float, 48 },
        }}),
        // Slot 'moveToAltAzi'
        QtMocHelpers::SlotData<void(const Vec3d &, const Vec3d &)>(51, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 45, 46 }, { 0x80000000 | 45, 47 },
        }}),
        // Slot 'zoomTo'
        QtMocHelpers::SlotData<void(double, float)>(52, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 53 }, { QMetaType::Float, 54 },
        }}),
        // Slot 'zoomTo'
        QtMocHelpers::SlotData<void(double)>(52, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Double, 53 },
        }}),
        // Slot 'getCurrentFov'
        QtMocHelpers::SlotData<double() const>(55, 2, QMC::AccessPublic, QMetaType::Double),
        // Slot 'getInitFov'
        QtMocHelpers::SlotData<double() const>(56, 2, QMC::AccessPublic, QMetaType::Double),
        // Slot 'setInitFov'
        QtMocHelpers::SlotData<void(double)>(57, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 15 },
        }}),
        // Slot 'getInitViewingDirection'
        QtMocHelpers::SlotData<const Vec3d() const>(58, 2, QMC::AccessPublic, 0x80000000 | 45),
        // Slot 'setInitViewDirectionToCurrent'
        QtMocHelpers::SlotData<void()>(59, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'getViewDirectionJ2000'
        QtMocHelpers::SlotData<Vec3d() const>(60, 2, QMC::AccessPublic, 0x80000000 | 45),
        // Slot 'setViewDirectionJ2000'
        QtMocHelpers::SlotData<void(const Vec3d &)>(61, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 45, 62 },
        }}),
        // Slot 'setMaxFov'
        QtMocHelpers::SlotData<void(double)>(63, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 64 },
        }}),
        // Slot 'getMaxFov'
        QtMocHelpers::SlotData<double() const>(65, 2, QMC::AccessPublic, QMetaType::Double),
        // Slot 'getMinFov'
        QtMocHelpers::SlotData<double() const>(66, 2, QMC::AccessPublic, QMetaType::Double),
        // Slot 'autoZoomIn'
        QtMocHelpers::SlotData<void(float, bool)>(67, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 48 }, { QMetaType::Bool, 68 },
        }}),
        // Slot 'autoZoomIn'
        QtMocHelpers::SlotData<void(float)>(67, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Float, 48 },
        }}),
        // Slot 'autoZoomIn'
        QtMocHelpers::SlotData<void()>(67, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'autoZoomOut'
        QtMocHelpers::SlotData<void(float, bool)>(69, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 48 }, { QMetaType::Bool, 70 },
        }}),
        // Slot 'autoZoomOut'
        QtMocHelpers::SlotData<void(float)>(69, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Float, 48 },
        }}),
        // Slot 'autoZoomOut'
        QtMocHelpers::SlotData<void()>(69, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'deselection'
        QtMocHelpers::SlotData<void()>(71, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'reSelectLastObject'
        QtMocHelpers::SlotData<void()>(72, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'getAimFov'
        QtMocHelpers::SlotData<double() const>(73, 2, QMC::AccessPublic, QMetaType::Double),
        // Slot 'turnRight'
        QtMocHelpers::SlotData<void(bool)>(74, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 75 },
        }}),
        // Slot 'turnLeft'
        QtMocHelpers::SlotData<void(bool)>(76, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 75 },
        }}),
        // Slot 'turnUp'
        QtMocHelpers::SlotData<void(bool)>(77, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 75 },
        }}),
        // Slot 'turnDown'
        QtMocHelpers::SlotData<void(bool)>(78, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 75 },
        }}),
        // Slot 'moveSlow'
        QtMocHelpers::SlotData<void(bool)>(79, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'zoomIn'
        QtMocHelpers::SlotData<void(bool)>(80, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 75 },
        }}),
        // Slot 'zoomOut'
        QtMocHelpers::SlotData<void(bool)>(81, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 75 },
        }}),
        // Slot 'smoothPan'
        QtMocHelpers::SlotData<void(double, double, double, bool)>(82, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 83 }, { QMetaType::Double, 84 }, { QMetaType::Double, 85 }, { QMetaType::Bool, 75 },
        }}),
        // Slot 'lookEast'
        QtMocHelpers::SlotData<void(bool)>(86, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 87 },
        }}),
        // Slot 'lookEast'
        QtMocHelpers::SlotData<void()>(86, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'lookWest'
        QtMocHelpers::SlotData<void(bool)>(88, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 87 },
        }}),
        // Slot 'lookWest'
        QtMocHelpers::SlotData<void()>(88, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'lookNorth'
        QtMocHelpers::SlotData<void(bool)>(89, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 87 },
        }}),
        // Slot 'lookNorth'
        QtMocHelpers::SlotData<void()>(89, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'lookSouth'
        QtMocHelpers::SlotData<void(bool)>(90, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 87 },
        }}),
        // Slot 'lookSouth'
        QtMocHelpers::SlotData<void()>(90, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'lookZenith'
        QtMocHelpers::SlotData<void()>(91, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'lookNadir'
        QtMocHelpers::SlotData<void()>(92, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'lookTowardsNCP'
        QtMocHelpers::SlotData<void()>(93, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'lookTowardsSCP'
        QtMocHelpers::SlotData<void()>(94, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'moveViewport'
        QtMocHelpers::SlotData<void(double, double, const float)>(95, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 96 }, { QMetaType::Double, 97 }, { QMetaType::Float, 98 },
        }}),
        // Slot 'moveViewport'
        QtMocHelpers::SlotData<void(double, double)>(95, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Double, 96 }, { QMetaType::Double, 97 },
        }}),
        // Slot 'setMountMode'
        QtMocHelpers::SlotData<void(StelMovementMgr::MountMode)>(99, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 100, 101 },
        }}),
        // Slot 'getMountMode'
        QtMocHelpers::SlotData<StelMovementMgr::MountMode() const>(102, 2, QMC::AccessPublic, 0x80000000 | 100),
        // Slot 'getEquatorialMount'
        QtMocHelpers::SlotData<bool() const>(103, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setInhibitAllAutomoves'
        QtMocHelpers::SlotData<void(bool)>(104, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 105 },
        }}),
        // Slot 'getViewportOffsetTarget'
        QtMocHelpers::SlotData<Vec2d() const>(106, 2, QMC::AccessPublic, 0x80000000 | 107),
        // Slot 'getViewportHorizontalOffsetTarget'
        QtMocHelpers::SlotData<double() const>(108, 2, QMC::AccessPublic, QMetaType::Double),
        // Slot 'getViewportVerticalOffsetTarget'
        QtMocHelpers::SlotData<double() const>(109, 2, QMC::AccessPublic, QMetaType::Double),
        // Slot 'setViewportHorizontalOffsetTarget'
        QtMocHelpers::SlotData<void(double)>(110, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 8 },
        }}),
        // Slot 'setViewportVerticalOffsetTarget'
        QtMocHelpers::SlotData<void(double)>(111, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 8 },
        }}),
        // Slot 'setUserMaxFov'
        QtMocHelpers::SlotData<void(double)>(112, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 64 },
        }}),
        // Slot 'getUserMaxFov'
        QtMocHelpers::SlotData<double() const>(113, 2, QMC::AccessPublic, QMetaType::Double),
        // Slot 'setFov'
        QtMocHelpers::SlotData<void(double)>(114, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 8 },
        }}),
        // Slot 'selectedObjectChange'
        QtMocHelpers::SlotData<void(StelModule::StelModuleSelectAction)>(115, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 116, 117 },
        }}),
        // Slot 'handleViewportOffsetMovement'
        QtMocHelpers::SlotData<void(qreal)>(118, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QReal, 119 },
        }}),
        // Slot 'setFOVDeg'
        QtMocHelpers::SlotData<void(float)>(120, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Float, 15 },
        }}),
        // Slot 'bindingFOVActions'
        QtMocHelpers::SlotData<void()>(121, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'equatorialMount'
        QtMocHelpers::PropertyData<bool>(122, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'tracking'
        QtMocHelpers::PropertyData<bool>(123, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 0),
        // property 'flagIndicationMountMode'
        QtMocHelpers::PropertyData<bool>(124, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'viewportHorizontalOffsetTarget'
        QtMocHelpers::PropertyData<double>(125, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'viewportVerticalOffsetTarget'
        QtMocHelpers::PropertyData<double>(126, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
        // property 'flagAutoZoomOutResetsDirection'
        QtMocHelpers::PropertyData<bool>(127, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'flagEnableMouseNavigation'
        QtMocHelpers::PropertyData<bool>(128, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 6),
        // property 'flagEnableMouseZooming'
        QtMocHelpers::PropertyData<bool>(129, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 7),
        // property 'flagEnableMoveKeys'
        QtMocHelpers::PropertyData<bool>(130, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 8),
        // property 'flagEnableZoomKeys'
        QtMocHelpers::PropertyData<bool>(131, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 9),
        // property 'userMaxFov'
        QtMocHelpers::PropertyData<double>(132, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 10),
        // property 'currentFov'
        QtMocHelpers::PropertyData<double>(133, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable, 11),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'MountMode'
        QtMocHelpers::EnumData<enum MountMode>(134, 134, QMC::EnumFlags{}).add({
            {  135, MountMode::MountAltAzimuthal },
            {  136, MountMode::MountEquinoxEquatorial },
            {  137, MountMode::MountGalactic },
            {  138, MountMode::MountSupergalactic },
        }),
        // enum 'ZoomingMode'
        QtMocHelpers::EnumData<enum ZoomingMode>(139, 139, QMC::EnumFlags{}).add({
            {  140, ZoomingMode::ZoomOut },
            {  141, ZoomingMode::ZoomNone },
            {  142, ZoomingMode::ZoomIn },
        }),
    };
    return QtMocHelpers::metaObjectData<StelMovementMgr, qt_meta_tag_ZN15StelMovementMgrE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject StelMovementMgr::staticMetaObject = { {
    QMetaObject::SuperData::link<StelModule::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15StelMovementMgrE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15StelMovementMgrE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15StelMovementMgrE_t>.metaTypes,
    nullptr
} };

void StelMovementMgr::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<StelMovementMgr *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->flagTrackingChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->equatorialMountChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->flagIndicationMountModeChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->flagAutoZoomOutResetsDirectionChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->viewportHorizontalOffsetTargetChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 5: _t->viewportVerticalOffsetTargetChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 6: _t->flagEnableMouseNavigationChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->flagEnableMouseZoomingChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->flagEnableMoveKeysChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 9: _t->flagEnableZoomKeysChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 10: _t->userMaxFovChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 11: _t->currentFovChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 12: _t->currentDirectionChanged(); break;
        case 13: _t->resetInitViewPos(); break;
        case 14: _t->toggleMountMode(); break;
        case 15: _t->setEquatorialMount((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 16: _t->setFlagTracking((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 17: _t->setFlagTracking(); break;
        case 18: { bool _r = _t->getFlagTracking();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 19: _t->setFlagLockEquPos((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 20: { bool _r = _t->getFlagLockEquPos();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 21: _t->panView((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 22: _t->setAutoMoveDuration((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 23: { float _r = _t->getAutoMoveDuration();
            if (_a[0]) *reinterpret_cast<float*>(_a[0]) = std::move(_r); }  break;
        case 24: _t->setFlagAutoZoomOutResetsDirection((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 25: { bool _r = _t->getFlagAutoZoomOutResetsDirection();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 26: { bool _r = _t->getFlagEnableZoomKeys();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 27: _t->setFlagEnableZoomKeys((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 28: { bool _r = _t->getFlagEnableMoveKeys();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 29: _t->setFlagEnableMoveKeys((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 30: { bool _r = _t->getFlagEnableMoveAtScreenEdge();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 31: _t->setFlagEnableMoveAtScreenEdge((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 32: { bool _r = _t->getFlagEnableMouseNavigation();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 33: _t->setFlagEnableMouseNavigation((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 34: { bool _r = _t->getFlagEnableMouseZooming();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 35: _t->setFlagEnableMouseZooming((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 36: { bool _r = _t->getFlagIndicationMountMode();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 37: _t->setFlagIndicationMountMode((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 38: _t->moveToJ2000((*reinterpret_cast<std::add_pointer_t<Vec3d>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Vec3d>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<StelMovementMgr::ZoomingMode>>(_a[4]))); break;
        case 39: _t->moveToJ2000((*reinterpret_cast<std::add_pointer_t<Vec3d>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Vec3d>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[3]))); break;
        case 40: _t->moveToJ2000((*reinterpret_cast<std::add_pointer_t<Vec3d>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Vec3d>>(_a[2]))); break;
        case 41: _t->moveToAltAzi((*reinterpret_cast<std::add_pointer_t<Vec3d>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Vec3d>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<StelMovementMgr::ZoomingMode>>(_a[4]))); break;
        case 42: _t->moveToAltAzi((*reinterpret_cast<std::add_pointer_t<Vec3d>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Vec3d>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[3]))); break;
        case 43: _t->moveToAltAzi((*reinterpret_cast<std::add_pointer_t<Vec3d>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Vec3d>>(_a[2]))); break;
        case 44: _t->zoomTo((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[2]))); break;
        case 45: _t->zoomTo((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 46: { double _r = _t->getCurrentFov();
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 47: { double _r = _t->getInitFov();
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 48: _t->setInitFov((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 49: { Vec3d _r = _t->getInitViewingDirection();
            if (_a[0]) *reinterpret_cast<Vec3d*>(_a[0]) = std::move(_r); }  break;
        case 50: _t->setInitViewDirectionToCurrent(); break;
        case 51: { Vec3d _r = _t->getViewDirectionJ2000();
            if (_a[0]) *reinterpret_cast<Vec3d*>(_a[0]) = std::move(_r); }  break;
        case 52: _t->setViewDirectionJ2000((*reinterpret_cast<std::add_pointer_t<Vec3d>>(_a[1]))); break;
        case 53: _t->setMaxFov((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 54: { double _r = _t->getMaxFov();
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 55: { double _r = _t->getMinFov();
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 56: _t->autoZoomIn((*reinterpret_cast<std::add_pointer_t<float>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 57: _t->autoZoomIn((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 58: _t->autoZoomIn(); break;
        case 59: _t->autoZoomOut((*reinterpret_cast<std::add_pointer_t<float>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 60: _t->autoZoomOut((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 61: _t->autoZoomOut(); break;
        case 62: _t->deselection(); break;
        case 63: _t->reSelectLastObject(); break;
        case 64: { double _r = _t->getAimFov();
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 65: _t->turnRight((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 66: _t->turnLeft((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 67: _t->turnUp((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 68: _t->turnDown((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 69: _t->moveSlow((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 70: _t->zoomIn((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 71: _t->zoomOut((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 72: _t->smoothPan((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4]))); break;
        case 73: _t->lookEast((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 74: _t->lookEast(); break;
        case 75: _t->lookWest((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 76: _t->lookWest(); break;
        case 77: _t->lookNorth((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 78: _t->lookNorth(); break;
        case 79: _t->lookSouth((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 80: _t->lookSouth(); break;
        case 81: _t->lookZenith(); break;
        case 82: _t->lookNadir(); break;
        case 83: _t->lookTowardsNCP(); break;
        case 84: _t->lookTowardsSCP(); break;
        case 85: _t->moveViewport((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[3]))); break;
        case 86: _t->moveViewport((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 87: _t->setMountMode((*reinterpret_cast<std::add_pointer_t<StelMovementMgr::MountMode>>(_a[1]))); break;
        case 88: { StelMovementMgr::MountMode _r = _t->getMountMode();
            if (_a[0]) *reinterpret_cast<StelMovementMgr::MountMode*>(_a[0]) = std::move(_r); }  break;
        case 89: { bool _r = _t->getEquatorialMount();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 90: _t->setInhibitAllAutomoves((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 91: { Vec2d _r = _t->getViewportOffsetTarget();
            if (_a[0]) *reinterpret_cast<Vec2d*>(_a[0]) = std::move(_r); }  break;
        case 92: { double _r = _t->getViewportHorizontalOffsetTarget();
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 93: { double _r = _t->getViewportVerticalOffsetTarget();
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 94: _t->setViewportHorizontalOffsetTarget((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 95: _t->setViewportVerticalOffsetTarget((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 96: _t->setUserMaxFov((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 97: { double _r = _t->getUserMaxFov();
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 98: _t->setFov((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 99: _t->selectedObjectChange((*reinterpret_cast<std::add_pointer_t<StelModule::StelModuleSelectAction>>(_a[1]))); break;
        case 100: _t->handleViewportOffsetMovement((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 101: _t->setFOVDeg((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 102: _t->bindingFOVActions(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 38:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3d >(); break;
            }
            break;
        case 39:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3d >(); break;
            }
            break;
        case 40:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3d >(); break;
            }
            break;
        case 41:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3d >(); break;
            }
            break;
        case 42:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3d >(); break;
            }
            break;
        case 43:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3d >(); break;
            }
            break;
        case 52:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3d >(); break;
            }
            break;
        case 99:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< StelModule::StelModuleSelectAction >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (StelMovementMgr::*)(bool )>(_a, &StelMovementMgr::flagTrackingChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelMovementMgr::*)(bool )>(_a, &StelMovementMgr::equatorialMountChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelMovementMgr::*)(bool )>(_a, &StelMovementMgr::flagIndicationMountModeChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelMovementMgr::*)(bool )>(_a, &StelMovementMgr::flagAutoZoomOutResetsDirectionChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelMovementMgr::*)(double )>(_a, &StelMovementMgr::viewportHorizontalOffsetTargetChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelMovementMgr::*)(double )>(_a, &StelMovementMgr::viewportVerticalOffsetTargetChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelMovementMgr::*)(bool )>(_a, &StelMovementMgr::flagEnableMouseNavigationChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelMovementMgr::*)(bool )>(_a, &StelMovementMgr::flagEnableMouseZoomingChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelMovementMgr::*)(bool )>(_a, &StelMovementMgr::flagEnableMoveKeysChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelMovementMgr::*)(bool )>(_a, &StelMovementMgr::flagEnableZoomKeysChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelMovementMgr::*)(double )>(_a, &StelMovementMgr::userMaxFovChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelMovementMgr::*)(double )>(_a, &StelMovementMgr::currentFovChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelMovementMgr::*)()>(_a, &StelMovementMgr::currentDirectionChanged, 12))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->getEquatorialMount(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->getFlagTracking(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->getFlagIndicationMountMode(); break;
        case 3: *reinterpret_cast<double*>(_v) = _t->getViewportHorizontalOffsetTarget(); break;
        case 4: *reinterpret_cast<double*>(_v) = _t->getViewportVerticalOffsetTarget(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->getFlagAutoZoomOutResetsDirection(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->getFlagEnableMouseNavigation(); break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->getFlagEnableMouseZooming(); break;
        case 8: *reinterpret_cast<bool*>(_v) = _t->getFlagEnableMoveKeys(); break;
        case 9: *reinterpret_cast<bool*>(_v) = _t->getFlagEnableZoomKeys(); break;
        case 10: *reinterpret_cast<double*>(_v) = _t->getUserMaxFov(); break;
        case 11: *reinterpret_cast<double*>(_v) = _t->getCurrentFov(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setEquatorialMount(*reinterpret_cast<bool*>(_v)); break;
        case 1: _t->setFlagTracking(*reinterpret_cast<bool*>(_v)); break;
        case 2: _t->setFlagIndicationMountMode(*reinterpret_cast<bool*>(_v)); break;
        case 3: _t->setViewportHorizontalOffsetTarget(*reinterpret_cast<double*>(_v)); break;
        case 4: _t->setViewportVerticalOffsetTarget(*reinterpret_cast<double*>(_v)); break;
        case 5: _t->setFlagAutoZoomOutResetsDirection(*reinterpret_cast<bool*>(_v)); break;
        case 6: _t->setFlagEnableMouseNavigation(*reinterpret_cast<bool*>(_v)); break;
        case 7: _t->setFlagEnableMouseZooming(*reinterpret_cast<bool*>(_v)); break;
        case 8: _t->setFlagEnableMoveKeys(*reinterpret_cast<bool*>(_v)); break;
        case 9: _t->setFlagEnableZoomKeys(*reinterpret_cast<bool*>(_v)); break;
        case 10: _t->setUserMaxFov(*reinterpret_cast<double*>(_v)); break;
        case 11: _t->setFov(*reinterpret_cast<double*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *StelMovementMgr::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *StelMovementMgr::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15StelMovementMgrE_t>.strings))
        return static_cast<void*>(this);
    return StelModule::qt_metacast(_clname);
}

int StelMovementMgr::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StelModule::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 103)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 103;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 103)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 103;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void StelMovementMgr::flagTrackingChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void StelMovementMgr::equatorialMountChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void StelMovementMgr::flagIndicationMountModeChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void StelMovementMgr::flagAutoZoomOutResetsDirectionChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void StelMovementMgr::viewportHorizontalOffsetTargetChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void StelMovementMgr::viewportVerticalOffsetTargetChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void StelMovementMgr::flagEnableMouseNavigationChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void StelMovementMgr::flagEnableMouseZoomingChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void StelMovementMgr::flagEnableMoveKeysChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void StelMovementMgr::flagEnableZoomKeysChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void StelMovementMgr::userMaxFovChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void StelMovementMgr::currentFovChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void StelMovementMgr::currentDirectionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}
QT_WARNING_POP
