/********************************************************************************
** Form generated from reading UI file 'timeNavigatorDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TIMENAVIGATORDIALOG_H
#define UI_TIMENAVIGATORDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTextBrowser>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "Dialog.hpp"

QT_BEGIN_NAMESPACE

class Ui_timeNavigatorDialog
{
public:
    QVBoxLayout *mainVBox;
    TitleBar *titleBar;
    QTabWidget *tabWidget;
    QWidget *tabTimeSteps;
    QVBoxLayout *vboxTimeSteps;
    QLabel *labelTimeStepsHeader;
    QGroupBox *groupDays;
    QGridLayout *gridDays;
    QHBoxLayout *hboxLayout;
    QToolButton *btnSubSolarDay;
    QLabel *labelSolarDay;
    QToolButton *btnAddSolarDay;
    QHBoxLayout *hboxLayout1;
    QToolButton *btnSubSolarWeek;
    QLabel *labelSolarWeek;
    QToolButton *btnAddSolarWeek;
    QHBoxLayout *hboxLayout2;
    QToolButton *btnSubSiderealDay;
    QLabel *labelSiderealDay;
    QToolButton *btnAddSiderealDay;
    QHBoxLayout *hboxLayout3;
    QToolButton *btnSubSiderealWeek;
    QLabel *labelSiderealWeek;
    QToolButton *btnAddSiderealWeek;
    QGroupBox *groupMonths;
    QGridLayout *gridMonths;
    QHBoxLayout *hboxLayout4;
    QToolButton *btnSubSynodicMonth;
    QLabel *labelSynodicMonth;
    QToolButton *btnAddSynodicMonth;
    QHBoxLayout *hboxLayout5;
    QToolButton *btnSubMeanTropicalMonth;
    QLabel *labelMeanTropicalMonth;
    QToolButton *btnAddMeanTropicalMonth;
    QHBoxLayout *hboxLayout6;
    QToolButton *btnSubDraconicMonth;
    QLabel *labelDraconicMonth;
    QToolButton *btnAddDraconicMonth;
    QHBoxLayout *hboxLayout7;
    QToolButton *btnSubAnomalisticMonth;
    QLabel *labelAnomalisticMonth;
    QToolButton *btnAddAnomalisticMonth;
    QHBoxLayout *hboxLayout8;
    QToolButton *btnSubCalendarMonth;
    QLabel *labelCalendarMonth;
    QToolButton *btnAddCalendarMonth;
    QGroupBox *groupYears;
    QGridLayout *gridYears;
    QHBoxLayout *hboxLayout9;
    QToolButton *btnSubMeanTropicalYear;
    QLabel *labelMeanTropicalYear;
    QToolButton *btnAddMeanTropicalYear;
    QHBoxLayout *hboxLayout10;
    QToolButton *btnSubTropicalYear;
    QLabel *labelTropicalYear;
    QToolButton *btnAddTropicalYear;
    QHBoxLayout *hboxLayout11;
    QToolButton *btnSubSiderealYear;
    QLabel *labelSiderealYear;
    QToolButton *btnAddSiderealYear;
    QHBoxLayout *hboxLayout12;
    QToolButton *btnSubDraconicYear;
    QLabel *labelDraconicYear;
    QToolButton *btnAddDraconicYear;
    QHBoxLayout *hboxLayout13;
    QToolButton *btnSubAnomalisticYear;
    QLabel *labelAnomalisticYear;
    QToolButton *btnAddAnomalisticYear;
    QHBoxLayout *hboxLayout14;
    QToolButton *btnSubJulianYear;
    QLabel *labelJulianYear;
    QToolButton *btnAddJulianYear;
    QHBoxLayout *hboxLayout15;
    QToolButton *btnSubGaussianYear;
    QLabel *labelGaussianYear;
    QToolButton *btnAddGaussianYear;
    QHBoxLayout *hboxLayout16;
    QToolButton *btnSubCalendarYear;
    QLabel *labelCalendarYear;
    QToolButton *btnAddCalendarYear;
    QGroupBox *groupLongCycles;
    QGridLayout *gridLongCycles;
    QHBoxLayout *hboxLayout17;
    QToolButton *btnSubMetonicCycle;
    QLabel *labelMetonicCycle;
    QToolButton *btnAddMetonicCycle;
    QHBoxLayout *hboxLayout18;
    QToolButton *btnSubSaros;
    QLabel *labelSaros;
    QToolButton *btnAddSaros;
    QSpacerItem *verticalSpacerTimeSteps;
    QWidget *tabSpecificTime;
    QVBoxLayout *vboxSpecificTime;
    QGroupBox *groupRTS;
    QGridLayout *gridRTS;
    QLabel *labelSelectedObject;
    QLabel *labelRTSPrev;
    QLabel *labelRTSThisDate;
    QLabel *labelRTSNext;
    QPushButton *btnPreviousRising;
    QPushButton *btnThisDateRising;
    QPushButton *btnNextRising;
    QPushButton *btnPreviousTransit;
    QPushButton *btnThisDateTransit;
    QPushButton *btnNextTransit;
    QPushButton *btnPreviousSetting;
    QPushButton *btnThisDateSetting;
    QPushButton *btnNextSetting;
    QGroupBox *groupTwilight;
    QGridLayout *gridTwilight;
    QHBoxLayout *hboxTwilightAlt;
    QLabel *labelTwilightAlt;
    QDoubleSpinBox *spinBoxTwilightAltitude;
    QSpacerItem *spacerItem;
    QLabel *labelTwilightPrev;
    QLabel *labelTwilightThisDate;
    QLabel *labelTwilightNext;
    QPushButton *btnPreviousMorningTwilight;
    QPushButton *btnThisDateMorningTwilight;
    QPushButton *btnNextMorningTwilight;
    QPushButton *btnPreviousEveningTwilight;
    QPushButton *btnThisDateEveningTwilight;
    QPushButton *btnNextEveningTwilight;
    QGroupBox *groupAtAltitude;
    QGridLayout *gridAtAltitude;
    QLabel *labelAltPrev;
    QLabel *labelAltThisDate;
    QLabel *labelAltNext;
    QPushButton *btnPreviousMorningAtAltitude;
    QPushButton *btnThisDateMorningAtAltitude;
    QPushButton *btnNextMorningAtAltitude;
    QPushButton *btnPreviousEveningAtAltitude;
    QPushButton *btnThisDateEveningAtAltitude;
    QPushButton *btnNextEveningAtAltitude;
    QSpacerItem *verticalSpacerSpecificTime;
    QWidget *tabPlanetaryEvents;
    QVBoxLayout *vboxPlanetaryEvents;
    QWidget *tabSettings;
    QVBoxLayout *vboxSettings;
    QCheckBox *checkBoxShowButton;
    QCheckBox *checkBoxSelectObject;
    QCheckBox *checkBoxCenterView;
    QSpacerItem *verticalSpacerSettings;
    QHBoxLayout *hboxSettingsButtons;
    QPushButton *btnRestoreDefaults;
    QSpacerItem *spacerItem1;
    QPushButton *btnSaveSettings;
    QWidget *tabAbout;
    QVBoxLayout *vboxAbout;
    QTextBrowser *aboutTextBrowser;

    void setupUi(QWidget *timeNavigatorDialog)
    {
        if (timeNavigatorDialog->objectName().isEmpty())
            timeNavigatorDialog->setObjectName("timeNavigatorDialog");
        timeNavigatorDialog->resize(500, 620);
        mainVBox = new QVBoxLayout(timeNavigatorDialog);
        mainVBox->setSpacing(0);
        mainVBox->setObjectName("mainVBox");
        mainVBox->setContentsMargins(0, 0, 0, 0);
        titleBar = new TitleBar(timeNavigatorDialog);
        titleBar->setObjectName("titleBar");

        mainVBox->addWidget(titleBar);

        tabWidget = new QTabWidget(timeNavigatorDialog);
        tabWidget->setObjectName("tabWidget");
        tabTimeSteps = new QWidget();
        tabTimeSteps->setObjectName("tabTimeSteps");
        vboxTimeSteps = new QVBoxLayout(tabTimeSteps);
        vboxTimeSteps->setObjectName("vboxTimeSteps");
        labelTimeStepsHeader = new QLabel(tabTimeSteps);
        labelTimeStepsHeader->setObjectName("labelTimeStepsHeader");
        labelTimeStepsHeader->setWordWrap(true);

        vboxTimeSteps->addWidget(labelTimeStepsHeader);

        groupDays = new QGroupBox(tabTimeSteps);
        groupDays->setObjectName("groupDays");
        gridDays = new QGridLayout(groupDays);
        gridDays->setObjectName("gridDays");
        gridDays->setContentsMargins(0, 0, 0, 0);
        hboxLayout = new QHBoxLayout();
        hboxLayout->setObjectName("hboxLayout");
        btnSubSolarDay = new QToolButton(groupDays);
        btnSubSolarDay->setObjectName("btnSubSolarDay");
        btnSubSolarDay->setText(QString::fromUtf8("\342\227\200"));

        hboxLayout->addWidget(btnSubSolarDay);

        labelSolarDay = new QLabel(groupDays);
        labelSolarDay->setObjectName("labelSolarDay");
        labelSolarDay->setAlignment(Qt::AlignCenter);

        hboxLayout->addWidget(labelSolarDay);

        btnAddSolarDay = new QToolButton(groupDays);
        btnAddSolarDay->setObjectName("btnAddSolarDay");
        btnAddSolarDay->setText(QString::fromUtf8("\342\226\266"));

        hboxLayout->addWidget(btnAddSolarDay);


        gridDays->addLayout(hboxLayout, 0, 0, 1, 1);

        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setObjectName("hboxLayout1");
        btnSubSolarWeek = new QToolButton(groupDays);
        btnSubSolarWeek->setObjectName("btnSubSolarWeek");
        btnSubSolarWeek->setText(QString::fromUtf8("\342\227\200"));

        hboxLayout1->addWidget(btnSubSolarWeek);

        labelSolarWeek = new QLabel(groupDays);
        labelSolarWeek->setObjectName("labelSolarWeek");
        labelSolarWeek->setAlignment(Qt::AlignCenter);

        hboxLayout1->addWidget(labelSolarWeek);

        btnAddSolarWeek = new QToolButton(groupDays);
        btnAddSolarWeek->setObjectName("btnAddSolarWeek");
        btnAddSolarWeek->setText(QString::fromUtf8("\342\226\266"));

        hboxLayout1->addWidget(btnAddSolarWeek);


        gridDays->addLayout(hboxLayout1, 0, 1, 1, 1);

        hboxLayout2 = new QHBoxLayout();
        hboxLayout2->setObjectName("hboxLayout2");
        btnSubSiderealDay = new QToolButton(groupDays);
        btnSubSiderealDay->setObjectName("btnSubSiderealDay");
        btnSubSiderealDay->setText(QString::fromUtf8("\342\227\200"));

        hboxLayout2->addWidget(btnSubSiderealDay);

        labelSiderealDay = new QLabel(groupDays);
        labelSiderealDay->setObjectName("labelSiderealDay");
        labelSiderealDay->setAlignment(Qt::AlignCenter);

        hboxLayout2->addWidget(labelSiderealDay);

        btnAddSiderealDay = new QToolButton(groupDays);
        btnAddSiderealDay->setObjectName("btnAddSiderealDay");
        btnAddSiderealDay->setText(QString::fromUtf8("\342\226\266"));

        hboxLayout2->addWidget(btnAddSiderealDay);


        gridDays->addLayout(hboxLayout2, 1, 0, 1, 1);

        hboxLayout3 = new QHBoxLayout();
        hboxLayout3->setObjectName("hboxLayout3");
        btnSubSiderealWeek = new QToolButton(groupDays);
        btnSubSiderealWeek->setObjectName("btnSubSiderealWeek");
        btnSubSiderealWeek->setText(QString::fromUtf8("\342\227\200"));

        hboxLayout3->addWidget(btnSubSiderealWeek);

        labelSiderealWeek = new QLabel(groupDays);
        labelSiderealWeek->setObjectName("labelSiderealWeek");
        labelSiderealWeek->setAlignment(Qt::AlignCenter);

        hboxLayout3->addWidget(labelSiderealWeek);

        btnAddSiderealWeek = new QToolButton(groupDays);
        btnAddSiderealWeek->setObjectName("btnAddSiderealWeek");
        btnAddSiderealWeek->setText(QString::fromUtf8("\342\226\266"));

        hboxLayout3->addWidget(btnAddSiderealWeek);


        gridDays->addLayout(hboxLayout3, 1, 1, 1, 1);


        vboxTimeSteps->addWidget(groupDays);

        groupMonths = new QGroupBox(tabTimeSteps);
        groupMonths->setObjectName("groupMonths");
        gridMonths = new QGridLayout(groupMonths);
        gridMonths->setObjectName("gridMonths");
        gridMonths->setContentsMargins(0, 0, 0, 0);
        hboxLayout4 = new QHBoxLayout();
        hboxLayout4->setObjectName("hboxLayout4");
        btnSubSynodicMonth = new QToolButton(groupMonths);
        btnSubSynodicMonth->setObjectName("btnSubSynodicMonth");
        btnSubSynodicMonth->setText(QString::fromUtf8("\342\227\200"));

        hboxLayout4->addWidget(btnSubSynodicMonth);

        labelSynodicMonth = new QLabel(groupMonths);
        labelSynodicMonth->setObjectName("labelSynodicMonth");
        labelSynodicMonth->setAlignment(Qt::AlignCenter);

        hboxLayout4->addWidget(labelSynodicMonth);

        btnAddSynodicMonth = new QToolButton(groupMonths);
        btnAddSynodicMonth->setObjectName("btnAddSynodicMonth");
        btnAddSynodicMonth->setText(QString::fromUtf8("\342\226\266"));

        hboxLayout4->addWidget(btnAddSynodicMonth);


        gridMonths->addLayout(hboxLayout4, 0, 0, 1, 1);

        hboxLayout5 = new QHBoxLayout();
        hboxLayout5->setObjectName("hboxLayout5");
        btnSubMeanTropicalMonth = new QToolButton(groupMonths);
        btnSubMeanTropicalMonth->setObjectName("btnSubMeanTropicalMonth");
        btnSubMeanTropicalMonth->setText(QString::fromUtf8("\342\227\200"));

        hboxLayout5->addWidget(btnSubMeanTropicalMonth);

        labelMeanTropicalMonth = new QLabel(groupMonths);
        labelMeanTropicalMonth->setObjectName("labelMeanTropicalMonth");
        labelMeanTropicalMonth->setAlignment(Qt::AlignCenter);

        hboxLayout5->addWidget(labelMeanTropicalMonth);

        btnAddMeanTropicalMonth = new QToolButton(groupMonths);
        btnAddMeanTropicalMonth->setObjectName("btnAddMeanTropicalMonth");
        btnAddMeanTropicalMonth->setText(QString::fromUtf8("\342\226\266"));

        hboxLayout5->addWidget(btnAddMeanTropicalMonth);


        gridMonths->addLayout(hboxLayout5, 0, 1, 1, 1);

        hboxLayout6 = new QHBoxLayout();
        hboxLayout6->setObjectName("hboxLayout6");
        btnSubDraconicMonth = new QToolButton(groupMonths);
        btnSubDraconicMonth->setObjectName("btnSubDraconicMonth");
        btnSubDraconicMonth->setText(QString::fromUtf8("\342\227\200"));

        hboxLayout6->addWidget(btnSubDraconicMonth);

        labelDraconicMonth = new QLabel(groupMonths);
        labelDraconicMonth->setObjectName("labelDraconicMonth");
        labelDraconicMonth->setAlignment(Qt::AlignCenter);

        hboxLayout6->addWidget(labelDraconicMonth);

        btnAddDraconicMonth = new QToolButton(groupMonths);
        btnAddDraconicMonth->setObjectName("btnAddDraconicMonth");
        btnAddDraconicMonth->setText(QString::fromUtf8("\342\226\266"));

        hboxLayout6->addWidget(btnAddDraconicMonth);


        gridMonths->addLayout(hboxLayout6, 1, 0, 1, 1);

        hboxLayout7 = new QHBoxLayout();
        hboxLayout7->setObjectName("hboxLayout7");
        btnSubAnomalisticMonth = new QToolButton(groupMonths);
        btnSubAnomalisticMonth->setObjectName("btnSubAnomalisticMonth");
        btnSubAnomalisticMonth->setText(QString::fromUtf8("\342\227\200"));

        hboxLayout7->addWidget(btnSubAnomalisticMonth);

        labelAnomalisticMonth = new QLabel(groupMonths);
        labelAnomalisticMonth->setObjectName("labelAnomalisticMonth");
        labelAnomalisticMonth->setAlignment(Qt::AlignCenter);

        hboxLayout7->addWidget(labelAnomalisticMonth);

        btnAddAnomalisticMonth = new QToolButton(groupMonths);
        btnAddAnomalisticMonth->setObjectName("btnAddAnomalisticMonth");
        btnAddAnomalisticMonth->setText(QString::fromUtf8("\342\226\266"));

        hboxLayout7->addWidget(btnAddAnomalisticMonth);


        gridMonths->addLayout(hboxLayout7, 1, 1, 1, 1);

        hboxLayout8 = new QHBoxLayout();
        hboxLayout8->setObjectName("hboxLayout8");
        btnSubCalendarMonth = new QToolButton(groupMonths);
        btnSubCalendarMonth->setObjectName("btnSubCalendarMonth");
        btnSubCalendarMonth->setText(QString::fromUtf8("\342\227\200"));

        hboxLayout8->addWidget(btnSubCalendarMonth);

        labelCalendarMonth = new QLabel(groupMonths);
        labelCalendarMonth->setObjectName("labelCalendarMonth");
        labelCalendarMonth->setAlignment(Qt::AlignCenter);

        hboxLayout8->addWidget(labelCalendarMonth);

        btnAddCalendarMonth = new QToolButton(groupMonths);
        btnAddCalendarMonth->setObjectName("btnAddCalendarMonth");
        btnAddCalendarMonth->setText(QString::fromUtf8("\342\226\266"));

        hboxLayout8->addWidget(btnAddCalendarMonth);


        gridMonths->addLayout(hboxLayout8, 2, 0, 1, 1);


        vboxTimeSteps->addWidget(groupMonths);

        groupYears = new QGroupBox(tabTimeSteps);
        groupYears->setObjectName("groupYears");
        gridYears = new QGridLayout(groupYears);
        gridYears->setObjectName("gridYears");
        gridYears->setContentsMargins(0, 0, 0, 0);
        hboxLayout9 = new QHBoxLayout();
        hboxLayout9->setObjectName("hboxLayout9");
        btnSubMeanTropicalYear = new QToolButton(groupYears);
        btnSubMeanTropicalYear->setObjectName("btnSubMeanTropicalYear");
        btnSubMeanTropicalYear->setText(QString::fromUtf8("\342\227\200"));

        hboxLayout9->addWidget(btnSubMeanTropicalYear);

        labelMeanTropicalYear = new QLabel(groupYears);
        labelMeanTropicalYear->setObjectName("labelMeanTropicalYear");
        labelMeanTropicalYear->setAlignment(Qt::AlignCenter);

        hboxLayout9->addWidget(labelMeanTropicalYear);

        btnAddMeanTropicalYear = new QToolButton(groupYears);
        btnAddMeanTropicalYear->setObjectName("btnAddMeanTropicalYear");
        btnAddMeanTropicalYear->setText(QString::fromUtf8("\342\226\266"));

        hboxLayout9->addWidget(btnAddMeanTropicalYear);


        gridYears->addLayout(hboxLayout9, 0, 0, 1, 1);

        hboxLayout10 = new QHBoxLayout();
        hboxLayout10->setObjectName("hboxLayout10");
        btnSubTropicalYear = new QToolButton(groupYears);
        btnSubTropicalYear->setObjectName("btnSubTropicalYear");
        btnSubTropicalYear->setText(QString::fromUtf8("\342\227\200"));

        hboxLayout10->addWidget(btnSubTropicalYear);

        labelTropicalYear = new QLabel(groupYears);
        labelTropicalYear->setObjectName("labelTropicalYear");
        labelTropicalYear->setAlignment(Qt::AlignCenter);

        hboxLayout10->addWidget(labelTropicalYear);

        btnAddTropicalYear = new QToolButton(groupYears);
        btnAddTropicalYear->setObjectName("btnAddTropicalYear");
        btnAddTropicalYear->setText(QString::fromUtf8("\342\226\266"));

        hboxLayout10->addWidget(btnAddTropicalYear);


        gridYears->addLayout(hboxLayout10, 0, 1, 1, 1);

        hboxLayout11 = new QHBoxLayout();
        hboxLayout11->setObjectName("hboxLayout11");
        btnSubSiderealYear = new QToolButton(groupYears);
        btnSubSiderealYear->setObjectName("btnSubSiderealYear");
        btnSubSiderealYear->setText(QString::fromUtf8("\342\227\200"));

        hboxLayout11->addWidget(btnSubSiderealYear);

        labelSiderealYear = new QLabel(groupYears);
        labelSiderealYear->setObjectName("labelSiderealYear");
        labelSiderealYear->setAlignment(Qt::AlignCenter);

        hboxLayout11->addWidget(labelSiderealYear);

        btnAddSiderealYear = new QToolButton(groupYears);
        btnAddSiderealYear->setObjectName("btnAddSiderealYear");
        btnAddSiderealYear->setText(QString::fromUtf8("\342\226\266"));

        hboxLayout11->addWidget(btnAddSiderealYear);


        gridYears->addLayout(hboxLayout11, 1, 0, 1, 1);

        hboxLayout12 = new QHBoxLayout();
        hboxLayout12->setObjectName("hboxLayout12");
        btnSubDraconicYear = new QToolButton(groupYears);
        btnSubDraconicYear->setObjectName("btnSubDraconicYear");
        btnSubDraconicYear->setText(QString::fromUtf8("\342\227\200"));

        hboxLayout12->addWidget(btnSubDraconicYear);

        labelDraconicYear = new QLabel(groupYears);
        labelDraconicYear->setObjectName("labelDraconicYear");
        labelDraconicYear->setAlignment(Qt::AlignCenter);

        hboxLayout12->addWidget(labelDraconicYear);

        btnAddDraconicYear = new QToolButton(groupYears);
        btnAddDraconicYear->setObjectName("btnAddDraconicYear");
        btnAddDraconicYear->setText(QString::fromUtf8("\342\226\266"));

        hboxLayout12->addWidget(btnAddDraconicYear);


        gridYears->addLayout(hboxLayout12, 1, 1, 1, 1);

        hboxLayout13 = new QHBoxLayout();
        hboxLayout13->setObjectName("hboxLayout13");
        btnSubAnomalisticYear = new QToolButton(groupYears);
        btnSubAnomalisticYear->setObjectName("btnSubAnomalisticYear");
        btnSubAnomalisticYear->setText(QString::fromUtf8("\342\227\200"));

        hboxLayout13->addWidget(btnSubAnomalisticYear);

        labelAnomalisticYear = new QLabel(groupYears);
        labelAnomalisticYear->setObjectName("labelAnomalisticYear");
        labelAnomalisticYear->setAlignment(Qt::AlignCenter);

        hboxLayout13->addWidget(labelAnomalisticYear);

        btnAddAnomalisticYear = new QToolButton(groupYears);
        btnAddAnomalisticYear->setObjectName("btnAddAnomalisticYear");
        btnAddAnomalisticYear->setText(QString::fromUtf8("\342\226\266"));

        hboxLayout13->addWidget(btnAddAnomalisticYear);


        gridYears->addLayout(hboxLayout13, 2, 0, 1, 1);

        hboxLayout14 = new QHBoxLayout();
        hboxLayout14->setObjectName("hboxLayout14");
        btnSubJulianYear = new QToolButton(groupYears);
        btnSubJulianYear->setObjectName("btnSubJulianYear");
        btnSubJulianYear->setText(QString::fromUtf8("\342\227\200"));

        hboxLayout14->addWidget(btnSubJulianYear);

        labelJulianYear = new QLabel(groupYears);
        labelJulianYear->setObjectName("labelJulianYear");
        labelJulianYear->setAlignment(Qt::AlignCenter);

        hboxLayout14->addWidget(labelJulianYear);

        btnAddJulianYear = new QToolButton(groupYears);
        btnAddJulianYear->setObjectName("btnAddJulianYear");
        btnAddJulianYear->setText(QString::fromUtf8("\342\226\266"));

        hboxLayout14->addWidget(btnAddJulianYear);


        gridYears->addLayout(hboxLayout14, 2, 1, 1, 1);

        hboxLayout15 = new QHBoxLayout();
        hboxLayout15->setObjectName("hboxLayout15");
        btnSubGaussianYear = new QToolButton(groupYears);
        btnSubGaussianYear->setObjectName("btnSubGaussianYear");
        btnSubGaussianYear->setText(QString::fromUtf8("\342\227\200"));

        hboxLayout15->addWidget(btnSubGaussianYear);

        labelGaussianYear = new QLabel(groupYears);
        labelGaussianYear->setObjectName("labelGaussianYear");
        labelGaussianYear->setAlignment(Qt::AlignCenter);

        hboxLayout15->addWidget(labelGaussianYear);

        btnAddGaussianYear = new QToolButton(groupYears);
        btnAddGaussianYear->setObjectName("btnAddGaussianYear");
        btnAddGaussianYear->setText(QString::fromUtf8("\342\226\266"));

        hboxLayout15->addWidget(btnAddGaussianYear);


        gridYears->addLayout(hboxLayout15, 3, 0, 1, 1);

        hboxLayout16 = new QHBoxLayout();
        hboxLayout16->setObjectName("hboxLayout16");
        btnSubCalendarYear = new QToolButton(groupYears);
        btnSubCalendarYear->setObjectName("btnSubCalendarYear");
        btnSubCalendarYear->setText(QString::fromUtf8("\342\227\200"));

        hboxLayout16->addWidget(btnSubCalendarYear);

        labelCalendarYear = new QLabel(groupYears);
        labelCalendarYear->setObjectName("labelCalendarYear");
        labelCalendarYear->setAlignment(Qt::AlignCenter);

        hboxLayout16->addWidget(labelCalendarYear);

        btnAddCalendarYear = new QToolButton(groupYears);
        btnAddCalendarYear->setObjectName("btnAddCalendarYear");
        btnAddCalendarYear->setText(QString::fromUtf8("\342\226\266"));

        hboxLayout16->addWidget(btnAddCalendarYear);


        gridYears->addLayout(hboxLayout16, 3, 1, 1, 1);


        vboxTimeSteps->addWidget(groupYears);

        groupLongCycles = new QGroupBox(tabTimeSteps);
        groupLongCycles->setObjectName("groupLongCycles");
        gridLongCycles = new QGridLayout(groupLongCycles);
        gridLongCycles->setObjectName("gridLongCycles");
        gridLongCycles->setContentsMargins(0, 0, 0, 0);
        hboxLayout17 = new QHBoxLayout();
        hboxLayout17->setObjectName("hboxLayout17");
        btnSubMetonicCycle = new QToolButton(groupLongCycles);
        btnSubMetonicCycle->setObjectName("btnSubMetonicCycle");
        btnSubMetonicCycle->setText(QString::fromUtf8("\342\227\200"));

        hboxLayout17->addWidget(btnSubMetonicCycle);

        labelMetonicCycle = new QLabel(groupLongCycles);
        labelMetonicCycle->setObjectName("labelMetonicCycle");
        labelMetonicCycle->setAlignment(Qt::AlignCenter);

        hboxLayout17->addWidget(labelMetonicCycle);

        btnAddMetonicCycle = new QToolButton(groupLongCycles);
        btnAddMetonicCycle->setObjectName("btnAddMetonicCycle");
        btnAddMetonicCycle->setText(QString::fromUtf8("\342\226\266"));

        hboxLayout17->addWidget(btnAddMetonicCycle);


        gridLongCycles->addLayout(hboxLayout17, 0, 0, 1, 1);

        hboxLayout18 = new QHBoxLayout();
        hboxLayout18->setObjectName("hboxLayout18");
        btnSubSaros = new QToolButton(groupLongCycles);
        btnSubSaros->setObjectName("btnSubSaros");
        btnSubSaros->setText(QString::fromUtf8("\342\227\200"));

        hboxLayout18->addWidget(btnSubSaros);

        labelSaros = new QLabel(groupLongCycles);
        labelSaros->setObjectName("labelSaros");
        labelSaros->setAlignment(Qt::AlignCenter);

        hboxLayout18->addWidget(labelSaros);

        btnAddSaros = new QToolButton(groupLongCycles);
        btnAddSaros->setObjectName("btnAddSaros");
        btnAddSaros->setText(QString::fromUtf8("\342\226\266"));

        hboxLayout18->addWidget(btnAddSaros);


        gridLongCycles->addLayout(hboxLayout18, 0, 1, 1, 1);


        vboxTimeSteps->addWidget(groupLongCycles);

        verticalSpacerTimeSteps = new QSpacerItem(20, 10, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxTimeSteps->addItem(verticalSpacerTimeSteps);

        tabWidget->addTab(tabTimeSteps, QString());
        tabSpecificTime = new QWidget();
        tabSpecificTime->setObjectName("tabSpecificTime");
        vboxSpecificTime = new QVBoxLayout(tabSpecificTime);
        vboxSpecificTime->setObjectName("vboxSpecificTime");
        groupRTS = new QGroupBox(tabSpecificTime);
        groupRTS->setObjectName("groupRTS");
        gridRTS = new QGridLayout(groupRTS);
        gridRTS->setObjectName("gridRTS");
        gridRTS->setContentsMargins(0, 0, 0, 0);
        labelSelectedObject = new QLabel(groupRTS);
        labelSelectedObject->setObjectName("labelSelectedObject");
        labelSelectedObject->setAlignment(Qt::AlignCenter);

        gridRTS->addWidget(labelSelectedObject, 0, 0, 1, 3);

        labelRTSPrev = new QLabel(groupRTS);
        labelRTSPrev->setObjectName("labelRTSPrev");
        labelRTSPrev->setAlignment(Qt::AlignCenter);

        gridRTS->addWidget(labelRTSPrev, 1, 0, 1, 1);

        labelRTSThisDate = new QLabel(groupRTS);
        labelRTSThisDate->setObjectName("labelRTSThisDate");
        labelRTSThisDate->setAlignment(Qt::AlignCenter);

        gridRTS->addWidget(labelRTSThisDate, 1, 1, 1, 1);

        labelRTSNext = new QLabel(groupRTS);
        labelRTSNext->setObjectName("labelRTSNext");
        labelRTSNext->setAlignment(Qt::AlignCenter);

        gridRTS->addWidget(labelRTSNext, 1, 2, 1, 1);

        btnPreviousRising = new QPushButton(groupRTS);
        btnPreviousRising->setObjectName("btnPreviousRising");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(btnPreviousRising->sizePolicy().hasHeightForWidth());
        btnPreviousRising->setSizePolicy(sizePolicy);

        gridRTS->addWidget(btnPreviousRising, 2, 0, 1, 1);

        btnThisDateRising = new QPushButton(groupRTS);
        btnThisDateRising->setObjectName("btnThisDateRising");
        sizePolicy.setHeightForWidth(btnThisDateRising->sizePolicy().hasHeightForWidth());
        btnThisDateRising->setSizePolicy(sizePolicy);

        gridRTS->addWidget(btnThisDateRising, 2, 1, 1, 1);

        btnNextRising = new QPushButton(groupRTS);
        btnNextRising->setObjectName("btnNextRising");
        sizePolicy.setHeightForWidth(btnNextRising->sizePolicy().hasHeightForWidth());
        btnNextRising->setSizePolicy(sizePolicy);

        gridRTS->addWidget(btnNextRising, 2, 2, 1, 1);

        btnPreviousTransit = new QPushButton(groupRTS);
        btnPreviousTransit->setObjectName("btnPreviousTransit");
        sizePolicy.setHeightForWidth(btnPreviousTransit->sizePolicy().hasHeightForWidth());
        btnPreviousTransit->setSizePolicy(sizePolicy);

        gridRTS->addWidget(btnPreviousTransit, 3, 0, 1, 1);

        btnThisDateTransit = new QPushButton(groupRTS);
        btnThisDateTransit->setObjectName("btnThisDateTransit");
        sizePolicy.setHeightForWidth(btnThisDateTransit->sizePolicy().hasHeightForWidth());
        btnThisDateTransit->setSizePolicy(sizePolicy);

        gridRTS->addWidget(btnThisDateTransit, 3, 1, 1, 1);

        btnNextTransit = new QPushButton(groupRTS);
        btnNextTransit->setObjectName("btnNextTransit");
        sizePolicy.setHeightForWidth(btnNextTransit->sizePolicy().hasHeightForWidth());
        btnNextTransit->setSizePolicy(sizePolicy);

        gridRTS->addWidget(btnNextTransit, 3, 2, 1, 1);

        btnPreviousSetting = new QPushButton(groupRTS);
        btnPreviousSetting->setObjectName("btnPreviousSetting");
        sizePolicy.setHeightForWidth(btnPreviousSetting->sizePolicy().hasHeightForWidth());
        btnPreviousSetting->setSizePolicy(sizePolicy);

        gridRTS->addWidget(btnPreviousSetting, 4, 0, 1, 1);

        btnThisDateSetting = new QPushButton(groupRTS);
        btnThisDateSetting->setObjectName("btnThisDateSetting");
        sizePolicy.setHeightForWidth(btnThisDateSetting->sizePolicy().hasHeightForWidth());
        btnThisDateSetting->setSizePolicy(sizePolicy);

        gridRTS->addWidget(btnThisDateSetting, 4, 1, 1, 1);

        btnNextSetting = new QPushButton(groupRTS);
        btnNextSetting->setObjectName("btnNextSetting");
        sizePolicy.setHeightForWidth(btnNextSetting->sizePolicy().hasHeightForWidth());
        btnNextSetting->setSizePolicy(sizePolicy);

        gridRTS->addWidget(btnNextSetting, 4, 2, 1, 1);


        vboxSpecificTime->addWidget(groupRTS);

        groupTwilight = new QGroupBox(tabSpecificTime);
        groupTwilight->setObjectName("groupTwilight");
        gridTwilight = new QGridLayout(groupTwilight);
        gridTwilight->setObjectName("gridTwilight");
        gridTwilight->setContentsMargins(0, 0, 0, 0);
        hboxTwilightAlt = new QHBoxLayout();
        hboxTwilightAlt->setObjectName("hboxTwilightAlt");
        labelTwilightAlt = new QLabel(groupTwilight);
        labelTwilightAlt->setObjectName("labelTwilightAlt");

        hboxTwilightAlt->addWidget(labelTwilightAlt);

        spinBoxTwilightAltitude = new QDoubleSpinBox(groupTwilight);
        spinBoxTwilightAltitude->setObjectName("spinBoxTwilightAltitude");
        spinBoxTwilightAltitude->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        spinBoxTwilightAltitude->setSuffix(QString::fromUtf8("\302\260"));
        spinBoxTwilightAltitude->setDecimals(2);
        spinBoxTwilightAltitude->setMinimum(-20.000000000000000);
        spinBoxTwilightAltitude->setMaximum(0.000000000000000);
        spinBoxTwilightAltitude->setSingleStep(0.100000000000000);

        hboxTwilightAlt->addWidget(spinBoxTwilightAltitude);

        spacerItem = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxTwilightAlt->addItem(spacerItem);


        gridTwilight->addLayout(hboxTwilightAlt, 0, 0, 1, 3);

        labelTwilightPrev = new QLabel(groupTwilight);
        labelTwilightPrev->setObjectName("labelTwilightPrev");
        labelTwilightPrev->setAlignment(Qt::AlignCenter);

        gridTwilight->addWidget(labelTwilightPrev, 1, 0, 1, 1);

        labelTwilightThisDate = new QLabel(groupTwilight);
        labelTwilightThisDate->setObjectName("labelTwilightThisDate");
        labelTwilightThisDate->setAlignment(Qt::AlignCenter);

        gridTwilight->addWidget(labelTwilightThisDate, 1, 1, 1, 1);

        labelTwilightNext = new QLabel(groupTwilight);
        labelTwilightNext->setObjectName("labelTwilightNext");
        labelTwilightNext->setAlignment(Qt::AlignCenter);

        gridTwilight->addWidget(labelTwilightNext, 1, 2, 1, 1);

        btnPreviousMorningTwilight = new QPushButton(groupTwilight);
        btnPreviousMorningTwilight->setObjectName("btnPreviousMorningTwilight");
        sizePolicy.setHeightForWidth(btnPreviousMorningTwilight->sizePolicy().hasHeightForWidth());
        btnPreviousMorningTwilight->setSizePolicy(sizePolicy);

        gridTwilight->addWidget(btnPreviousMorningTwilight, 2, 0, 1, 1);

        btnThisDateMorningTwilight = new QPushButton(groupTwilight);
        btnThisDateMorningTwilight->setObjectName("btnThisDateMorningTwilight");
        sizePolicy.setHeightForWidth(btnThisDateMorningTwilight->sizePolicy().hasHeightForWidth());
        btnThisDateMorningTwilight->setSizePolicy(sizePolicy);

        gridTwilight->addWidget(btnThisDateMorningTwilight, 2, 1, 1, 1);

        btnNextMorningTwilight = new QPushButton(groupTwilight);
        btnNextMorningTwilight->setObjectName("btnNextMorningTwilight");
        sizePolicy.setHeightForWidth(btnNextMorningTwilight->sizePolicy().hasHeightForWidth());
        btnNextMorningTwilight->setSizePolicy(sizePolicy);

        gridTwilight->addWidget(btnNextMorningTwilight, 2, 2, 1, 1);

        btnPreviousEveningTwilight = new QPushButton(groupTwilight);
        btnPreviousEveningTwilight->setObjectName("btnPreviousEveningTwilight");
        sizePolicy.setHeightForWidth(btnPreviousEveningTwilight->sizePolicy().hasHeightForWidth());
        btnPreviousEveningTwilight->setSizePolicy(sizePolicy);

        gridTwilight->addWidget(btnPreviousEveningTwilight, 3, 0, 1, 1);

        btnThisDateEveningTwilight = new QPushButton(groupTwilight);
        btnThisDateEveningTwilight->setObjectName("btnThisDateEveningTwilight");
        sizePolicy.setHeightForWidth(btnThisDateEveningTwilight->sizePolicy().hasHeightForWidth());
        btnThisDateEveningTwilight->setSizePolicy(sizePolicy);

        gridTwilight->addWidget(btnThisDateEveningTwilight, 3, 1, 1, 1);

        btnNextEveningTwilight = new QPushButton(groupTwilight);
        btnNextEveningTwilight->setObjectName("btnNextEveningTwilight");
        sizePolicy.setHeightForWidth(btnNextEveningTwilight->sizePolicy().hasHeightForWidth());
        btnNextEveningTwilight->setSizePolicy(sizePolicy);

        gridTwilight->addWidget(btnNextEveningTwilight, 3, 2, 1, 1);


        vboxSpecificTime->addWidget(groupTwilight);

        groupAtAltitude = new QGroupBox(tabSpecificTime);
        groupAtAltitude->setObjectName("groupAtAltitude");
        gridAtAltitude = new QGridLayout(groupAtAltitude);
        gridAtAltitude->setObjectName("gridAtAltitude");
        gridAtAltitude->setContentsMargins(0, 0, 0, 0);
        labelAltPrev = new QLabel(groupAtAltitude);
        labelAltPrev->setObjectName("labelAltPrev");
        labelAltPrev->setAlignment(Qt::AlignCenter);

        gridAtAltitude->addWidget(labelAltPrev, 0, 0, 1, 1);

        labelAltThisDate = new QLabel(groupAtAltitude);
        labelAltThisDate->setObjectName("labelAltThisDate");
        labelAltThisDate->setAlignment(Qt::AlignCenter);

        gridAtAltitude->addWidget(labelAltThisDate, 0, 1, 1, 1);

        labelAltNext = new QLabel(groupAtAltitude);
        labelAltNext->setObjectName("labelAltNext");
        labelAltNext->setAlignment(Qt::AlignCenter);

        gridAtAltitude->addWidget(labelAltNext, 0, 2, 1, 1);

        btnPreviousMorningAtAltitude = new QPushButton(groupAtAltitude);
        btnPreviousMorningAtAltitude->setObjectName("btnPreviousMorningAtAltitude");
        sizePolicy.setHeightForWidth(btnPreviousMorningAtAltitude->sizePolicy().hasHeightForWidth());
        btnPreviousMorningAtAltitude->setSizePolicy(sizePolicy);

        gridAtAltitude->addWidget(btnPreviousMorningAtAltitude, 1, 0, 1, 1);

        btnThisDateMorningAtAltitude = new QPushButton(groupAtAltitude);
        btnThisDateMorningAtAltitude->setObjectName("btnThisDateMorningAtAltitude");
        sizePolicy.setHeightForWidth(btnThisDateMorningAtAltitude->sizePolicy().hasHeightForWidth());
        btnThisDateMorningAtAltitude->setSizePolicy(sizePolicy);

        gridAtAltitude->addWidget(btnThisDateMorningAtAltitude, 1, 1, 1, 1);

        btnNextMorningAtAltitude = new QPushButton(groupAtAltitude);
        btnNextMorningAtAltitude->setObjectName("btnNextMorningAtAltitude");
        sizePolicy.setHeightForWidth(btnNextMorningAtAltitude->sizePolicy().hasHeightForWidth());
        btnNextMorningAtAltitude->setSizePolicy(sizePolicy);

        gridAtAltitude->addWidget(btnNextMorningAtAltitude, 1, 2, 1, 1);

        btnPreviousEveningAtAltitude = new QPushButton(groupAtAltitude);
        btnPreviousEveningAtAltitude->setObjectName("btnPreviousEveningAtAltitude");
        sizePolicy.setHeightForWidth(btnPreviousEveningAtAltitude->sizePolicy().hasHeightForWidth());
        btnPreviousEveningAtAltitude->setSizePolicy(sizePolicy);

        gridAtAltitude->addWidget(btnPreviousEveningAtAltitude, 2, 0, 1, 1);

        btnThisDateEveningAtAltitude = new QPushButton(groupAtAltitude);
        btnThisDateEveningAtAltitude->setObjectName("btnThisDateEveningAtAltitude");
        sizePolicy.setHeightForWidth(btnThisDateEveningAtAltitude->sizePolicy().hasHeightForWidth());
        btnThisDateEveningAtAltitude->setSizePolicy(sizePolicy);

        gridAtAltitude->addWidget(btnThisDateEveningAtAltitude, 2, 1, 1, 1);

        btnNextEveningAtAltitude = new QPushButton(groupAtAltitude);
        btnNextEveningAtAltitude->setObjectName("btnNextEveningAtAltitude");
        sizePolicy.setHeightForWidth(btnNextEveningAtAltitude->sizePolicy().hasHeightForWidth());
        btnNextEveningAtAltitude->setSizePolicy(sizePolicy);

        gridAtAltitude->addWidget(btnNextEveningAtAltitude, 2, 2, 1, 1);


        vboxSpecificTime->addWidget(groupAtAltitude);

        verticalSpacerSpecificTime = new QSpacerItem(20, 10, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxSpecificTime->addItem(verticalSpacerSpecificTime);

        tabWidget->addTab(tabSpecificTime, QString());
        tabPlanetaryEvents = new QWidget();
        tabPlanetaryEvents->setObjectName("tabPlanetaryEvents");
        vboxPlanetaryEvents = new QVBoxLayout(tabPlanetaryEvents);
        vboxPlanetaryEvents->setSpacing(4);
        vboxPlanetaryEvents->setObjectName("vboxPlanetaryEvents");
        vboxPlanetaryEvents->setContentsMargins(4, 4, 4, 4);
        tabWidget->addTab(tabPlanetaryEvents, QString());
        tabSettings = new QWidget();
        tabSettings->setObjectName("tabSettings");
        vboxSettings = new QVBoxLayout(tabSettings);
        vboxSettings->setObjectName("vboxSettings");
        checkBoxShowButton = new QCheckBox(tabSettings);
        checkBoxShowButton->setObjectName("checkBoxShowButton");

        vboxSettings->addWidget(checkBoxShowButton);

        checkBoxSelectObject = new QCheckBox(tabSettings);
        checkBoxSelectObject->setObjectName("checkBoxSelectObject");

        vboxSettings->addWidget(checkBoxSelectObject);

        checkBoxCenterView = new QCheckBox(tabSettings);
        checkBoxCenterView->setObjectName("checkBoxCenterView");

        vboxSettings->addWidget(checkBoxCenterView);

        verticalSpacerSettings = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxSettings->addItem(verticalSpacerSettings);

        hboxSettingsButtons = new QHBoxLayout();
        hboxSettingsButtons->setObjectName("hboxSettingsButtons");
        btnRestoreDefaults = new QPushButton(tabSettings);
        btnRestoreDefaults->setObjectName("btnRestoreDefaults");

        hboxSettingsButtons->addWidget(btnRestoreDefaults);

        spacerItem1 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxSettingsButtons->addItem(spacerItem1);

        btnSaveSettings = new QPushButton(tabSettings);
        btnSaveSettings->setObjectName("btnSaveSettings");

        hboxSettingsButtons->addWidget(btnSaveSettings);


        vboxSettings->addLayout(hboxSettingsButtons);

        tabWidget->addTab(tabSettings, QString());
        tabAbout = new QWidget();
        tabAbout->setObjectName("tabAbout");
        vboxAbout = new QVBoxLayout(tabAbout);
        vboxAbout->setObjectName("vboxAbout");
        aboutTextBrowser = new QTextBrowser(tabAbout);
        aboutTextBrowser->setObjectName("aboutTextBrowser");
        aboutTextBrowser->setOpenExternalLinks(true);

        vboxAbout->addWidget(aboutTextBrowser);

        tabWidget->addTab(tabAbout, QString());

        mainVBox->addWidget(tabWidget);


        retranslateUi(timeNavigatorDialog);

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(timeNavigatorDialog);
    } // setupUi

    void retranslateUi(QWidget *timeNavigatorDialog)
    {
        titleBar->setProperty("title", QVariant(QCoreApplication::translate("timeNavigatorDialog", "Time Navigator", nullptr)));
        labelTimeStepsHeader->setText(QCoreApplication::translate("timeNavigatorDialog", "Step the simulation time forward (\342\226\266) or backward (\342\227\200) by a period:", nullptr));
        groupDays->setTitle(QCoreApplication::translate("timeNavigatorDialog", "Days", nullptr));
#if QT_CONFIG(tooltip)
        btnSubSolarDay->setToolTip(QCoreApplication::translate("timeNavigatorDialog", "Subtract 1 solar day", nullptr));
#endif // QT_CONFIG(tooltip)
        labelSolarDay->setText(QCoreApplication::translate("timeNavigatorDialog", "Solar day", nullptr));
#if QT_CONFIG(tooltip)
        btnAddSolarDay->setToolTip(QCoreApplication::translate("timeNavigatorDialog", "Add 1 solar day", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        btnSubSolarWeek->setToolTip(QCoreApplication::translate("timeNavigatorDialog", "Subtract 7 solar days", nullptr));
#endif // QT_CONFIG(tooltip)
        labelSolarWeek->setText(QCoreApplication::translate("timeNavigatorDialog", "Solar week", nullptr));
#if QT_CONFIG(tooltip)
        btnAddSolarWeek->setToolTip(QCoreApplication::translate("timeNavigatorDialog", "Add 7 solar days", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        btnSubSiderealDay->setToolTip(QCoreApplication::translate("timeNavigatorDialog", "Subtract 1 sidereal day", nullptr));
#endif // QT_CONFIG(tooltip)
        labelSiderealDay->setText(QCoreApplication::translate("timeNavigatorDialog", "Sidereal day", nullptr));
#if QT_CONFIG(tooltip)
        btnAddSiderealDay->setToolTip(QCoreApplication::translate("timeNavigatorDialog", "Add 1 sidereal day", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        btnSubSiderealWeek->setToolTip(QCoreApplication::translate("timeNavigatorDialog", "Subtract 7 sidereal days", nullptr));
#endif // QT_CONFIG(tooltip)
        labelSiderealWeek->setText(QCoreApplication::translate("timeNavigatorDialog", "Sidereal week", nullptr));
#if QT_CONFIG(tooltip)
        btnAddSiderealWeek->setToolTip(QCoreApplication::translate("timeNavigatorDialog", "Add 7 sidereal days", nullptr));
#endif // QT_CONFIG(tooltip)
        groupMonths->setTitle(QCoreApplication::translate("timeNavigatorDialog", "Months", nullptr));
#if QT_CONFIG(tooltip)
        btnSubSynodicMonth->setToolTip(QCoreApplication::translate("timeNavigatorDialog", "Subtract 1 synodic month (~29.53 days)", nullptr));
#endif // QT_CONFIG(tooltip)
        labelSynodicMonth->setText(QCoreApplication::translate("timeNavigatorDialog", "Synodic month", nullptr));
#if QT_CONFIG(tooltip)
        btnAddSynodicMonth->setToolTip(QCoreApplication::translate("timeNavigatorDialog", "Add 1 synodic month (~29.53 days)", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        btnSubMeanTropicalMonth->setToolTip(QCoreApplication::translate("timeNavigatorDialog", "Subtract 1 mean tropical month (~27.32 days)", nullptr));
#endif // QT_CONFIG(tooltip)
        labelMeanTropicalMonth->setText(QCoreApplication::translate("timeNavigatorDialog", "Mean tropical month", nullptr));
#if QT_CONFIG(tooltip)
        btnAddMeanTropicalMonth->setToolTip(QCoreApplication::translate("timeNavigatorDialog", "Add 1 mean tropical month (~27.32 days)", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        btnSubDraconicMonth->setToolTip(QCoreApplication::translate("timeNavigatorDialog", "Subtract 1 draconic month (~27.21 days)", nullptr));
#endif // QT_CONFIG(tooltip)
        labelDraconicMonth->setText(QCoreApplication::translate("timeNavigatorDialog", "Draconic month", nullptr));
#if QT_CONFIG(tooltip)
        btnAddDraconicMonth->setToolTip(QCoreApplication::translate("timeNavigatorDialog", "Add 1 draconic month (~27.21 days)", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        btnSubAnomalisticMonth->setToolTip(QCoreApplication::translate("timeNavigatorDialog", "Subtract 1 anomalistic month (~27.55 days)", nullptr));
#endif // QT_CONFIG(tooltip)
        labelAnomalisticMonth->setText(QCoreApplication::translate("timeNavigatorDialog", "Anomalistic month", nullptr));
#if QT_CONFIG(tooltip)
        btnAddAnomalisticMonth->setToolTip(QCoreApplication::translate("timeNavigatorDialog", "Add 1 anomalistic month (~27.55 days)", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        btnSubCalendarMonth->setToolTip(QCoreApplication::translate("timeNavigatorDialog", "Subtract 1 calendar month", nullptr));
#endif // QT_CONFIG(tooltip)
        labelCalendarMonth->setText(QCoreApplication::translate("timeNavigatorDialog", "Calendar month", nullptr));
#if QT_CONFIG(tooltip)
        btnAddCalendarMonth->setToolTip(QCoreApplication::translate("timeNavigatorDialog", "Add 1 calendar month", nullptr));
#endif // QT_CONFIG(tooltip)
        groupYears->setTitle(QCoreApplication::translate("timeNavigatorDialog", "Years", nullptr));
#if QT_CONFIG(tooltip)
        btnSubMeanTropicalYear->setToolTip(QCoreApplication::translate("timeNavigatorDialog", "Subtract 1 mean tropical year (~365.24 days)", nullptr));
#endif // QT_CONFIG(tooltip)
        labelMeanTropicalYear->setText(QCoreApplication::translate("timeNavigatorDialog", "Mean tropical year", nullptr));
#if QT_CONFIG(tooltip)
        btnAddMeanTropicalYear->setToolTip(QCoreApplication::translate("timeNavigatorDialog", "Add 1 mean tropical year (~365.24 days)", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        btnSubTropicalYear->setToolTip(QCoreApplication::translate("timeNavigatorDialog", "Subtract 1 tropical year (varies with epoch)", nullptr));
#endif // QT_CONFIG(tooltip)
        labelTropicalYear->setText(QCoreApplication::translate("timeNavigatorDialog", "Tropical year", nullptr));
#if QT_CONFIG(tooltip)
        btnAddTropicalYear->setToolTip(QCoreApplication::translate("timeNavigatorDialog", "Add 1 tropical year (varies with epoch)", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        btnSubSiderealYear->setToolTip(QCoreApplication::translate("timeNavigatorDialog", "Subtract 1 sidereal year", nullptr));
#endif // QT_CONFIG(tooltip)
        labelSiderealYear->setText(QCoreApplication::translate("timeNavigatorDialog", "Sidereal year", nullptr));
#if QT_CONFIG(tooltip)
        btnAddSiderealYear->setToolTip(QCoreApplication::translate("timeNavigatorDialog", "Add 1 sidereal year", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        btnSubDraconicYear->setToolTip(QCoreApplication::translate("timeNavigatorDialog", "Subtract 1 draconic year (~346.62 days)", nullptr));
#endif // QT_CONFIG(tooltip)
        labelDraconicYear->setText(QCoreApplication::translate("timeNavigatorDialog", "Draconic year", nullptr));
#if QT_CONFIG(tooltip)
        btnAddDraconicYear->setToolTip(QCoreApplication::translate("timeNavigatorDialog", "Add 1 draconic year (~346.62 days)", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        btnSubAnomalisticYear->setToolTip(QCoreApplication::translate("timeNavigatorDialog", "Subtract 1 anomalistic year (~365.26 days)", nullptr));
#endif // QT_CONFIG(tooltip)
        labelAnomalisticYear->setText(QCoreApplication::translate("timeNavigatorDialog", "Anomalistic year", nullptr));
#if QT_CONFIG(tooltip)
        btnAddAnomalisticYear->setToolTip(QCoreApplication::translate("timeNavigatorDialog", "Add 1 anomalistic year (~365.26 days)", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        btnSubJulianYear->setToolTip(QCoreApplication::translate("timeNavigatorDialog", "Subtract 1 Julian year (365.25 days)", nullptr));
#endif // QT_CONFIG(tooltip)
        labelJulianYear->setText(QCoreApplication::translate("timeNavigatorDialog", "Julian year", nullptr));
#if QT_CONFIG(tooltip)
        btnAddJulianYear->setToolTip(QCoreApplication::translate("timeNavigatorDialog", "Add 1 Julian year (365.25 days)", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        btnSubGaussianYear->setToolTip(QCoreApplication::translate("timeNavigatorDialog", "Subtract 1 Gaussian year (~365.26 days)", nullptr));
#endif // QT_CONFIG(tooltip)
        labelGaussianYear->setText(QCoreApplication::translate("timeNavigatorDialog", "Gaussian year", nullptr));
#if QT_CONFIG(tooltip)
        btnAddGaussianYear->setToolTip(QCoreApplication::translate("timeNavigatorDialog", "Add 1 Gaussian year (~365.26 days)", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        btnSubCalendarYear->setToolTip(QCoreApplication::translate("timeNavigatorDialog", "Subtract 1 calendar year", nullptr));
#endif // QT_CONFIG(tooltip)
        labelCalendarYear->setText(QCoreApplication::translate("timeNavigatorDialog", "Calendar year", nullptr));
#if QT_CONFIG(tooltip)
        btnAddCalendarYear->setToolTip(QCoreApplication::translate("timeNavigatorDialog", "Add 1 calendar year", nullptr));
#endif // QT_CONFIG(tooltip)
        groupLongCycles->setTitle(QCoreApplication::translate("timeNavigatorDialog", "Long cycles", nullptr));
#if QT_CONFIG(tooltip)
        btnSubMetonicCycle->setToolTip(QCoreApplication::translate("timeNavigatorDialog", "Subtract 1 Metonic cycle (235 synodic months \342\211\210 19 years)", nullptr));
#endif // QT_CONFIG(tooltip)
        labelMetonicCycle->setText(QCoreApplication::translate("timeNavigatorDialog", "Metonic cycle", nullptr));
#if QT_CONFIG(tooltip)
        btnAddMetonicCycle->setToolTip(QCoreApplication::translate("timeNavigatorDialog", "Add 1 Metonic cycle (235 synodic months \342\211\210 19 years)", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        btnSubSaros->setToolTip(QCoreApplication::translate("timeNavigatorDialog", "Subtract 1 saros (223 synodic months \342\211\210 18 years 11 days)", nullptr));
#endif // QT_CONFIG(tooltip)
        labelSaros->setText(QCoreApplication::translate("timeNavigatorDialog", "Saros", nullptr));
#if QT_CONFIG(tooltip)
        btnAddSaros->setToolTip(QCoreApplication::translate("timeNavigatorDialog", "Add 1 saros (223 synodic months \342\211\210 18 years 11 days)", nullptr));
#endif // QT_CONFIG(tooltip)
        tabWidget->setTabText(tabWidget->indexOf(tabTimeSteps), QCoreApplication::translate("timeNavigatorDialog", "Time Steps", nullptr));
        groupRTS->setTitle(QCoreApplication::translate("timeNavigatorDialog", "Selected object", nullptr));
        labelSelectedObject->setText(QCoreApplication::translate("timeNavigatorDialog", "No object selected", nullptr));
        labelRTSPrev->setText(QCoreApplication::translate("timeNavigatorDialog", "Previous", nullptr));
        labelRTSThisDate->setText(QCoreApplication::translate("timeNavigatorDialog", "This date", nullptr));
        labelRTSNext->setText(QCoreApplication::translate("timeNavigatorDialog", "Next", nullptr));
#if QT_CONFIG(tooltip)
        btnPreviousRising->setToolTip(QCoreApplication::translate("timeNavigatorDialog", "Previous rising of the selected object", nullptr));
#endif // QT_CONFIG(tooltip)
        btnPreviousRising->setText(QCoreApplication::translate("timeNavigatorDialog", "Rising", nullptr));
#if QT_CONFIG(tooltip)
        btnThisDateRising->setToolTip(QCoreApplication::translate("timeNavigatorDialog", "This date's rising of the selected object", nullptr));
#endif // QT_CONFIG(tooltip)
        btnThisDateRising->setText(QCoreApplication::translate("timeNavigatorDialog", "Rising", nullptr));
#if QT_CONFIG(tooltip)
        btnNextRising->setToolTip(QCoreApplication::translate("timeNavigatorDialog", "Next rising of the selected object", nullptr));
#endif // QT_CONFIG(tooltip)
        btnNextRising->setText(QCoreApplication::translate("timeNavigatorDialog", "Rising", nullptr));
#if QT_CONFIG(tooltip)
        btnPreviousTransit->setToolTip(QCoreApplication::translate("timeNavigatorDialog", "Previous transit of the selected object", nullptr));
#endif // QT_CONFIG(tooltip)
        btnPreviousTransit->setText(QCoreApplication::translate("timeNavigatorDialog", "Transit", nullptr));
#if QT_CONFIG(tooltip)
        btnThisDateTransit->setToolTip(QCoreApplication::translate("timeNavigatorDialog", "This date's transit of the selected object", nullptr));
#endif // QT_CONFIG(tooltip)
        btnThisDateTransit->setText(QCoreApplication::translate("timeNavigatorDialog", "Transit", nullptr));
#if QT_CONFIG(tooltip)
        btnNextTransit->setToolTip(QCoreApplication::translate("timeNavigatorDialog", "Next transit of the selected object", nullptr));
#endif // QT_CONFIG(tooltip)
        btnNextTransit->setText(QCoreApplication::translate("timeNavigatorDialog", "Transit", nullptr));
#if QT_CONFIG(tooltip)
        btnPreviousSetting->setToolTip(QCoreApplication::translate("timeNavigatorDialog", "Previous setting of the selected object", nullptr));
#endif // QT_CONFIG(tooltip)
        btnPreviousSetting->setText(QCoreApplication::translate("timeNavigatorDialog", "Setting", nullptr));
#if QT_CONFIG(tooltip)
        btnThisDateSetting->setToolTip(QCoreApplication::translate("timeNavigatorDialog", "This date's setting of the selected object", nullptr));
#endif // QT_CONFIG(tooltip)
        btnThisDateSetting->setText(QCoreApplication::translate("timeNavigatorDialog", "Setting", nullptr));
#if QT_CONFIG(tooltip)
        btnNextSetting->setToolTip(QCoreApplication::translate("timeNavigatorDialog", "Next setting of the selected object", nullptr));
#endif // QT_CONFIG(tooltip)
        btnNextSetting->setText(QCoreApplication::translate("timeNavigatorDialog", "Setting", nullptr));
        groupTwilight->setTitle(QCoreApplication::translate("timeNavigatorDialog", "Twilight", nullptr));
        labelTwilightAlt->setText(QCoreApplication::translate("timeNavigatorDialog", "Solar altitude:", nullptr));
#if QT_CONFIG(tooltip)
        spinBoxTwilightAltitude->setToolTip(QCoreApplication::translate("timeNavigatorDialog", "Solar altitude used for twilight calculations (synced with View \342\206\222 Sky)", nullptr));
#endif // QT_CONFIG(tooltip)
        labelTwilightPrev->setText(QCoreApplication::translate("timeNavigatorDialog", "Previous", nullptr));
        labelTwilightThisDate->setText(QCoreApplication::translate("timeNavigatorDialog", "This date", nullptr));
        labelTwilightNext->setText(QCoreApplication::translate("timeNavigatorDialog", "Next", nullptr));
#if QT_CONFIG(tooltip)
        btnPreviousMorningTwilight->setToolTip(QCoreApplication::translate("timeNavigatorDialog", "Previous morning twilight", nullptr));
#endif // QT_CONFIG(tooltip)
        btnPreviousMorningTwilight->setText(QCoreApplication::translate("timeNavigatorDialog", "Morning", nullptr));
#if QT_CONFIG(tooltip)
        btnThisDateMorningTwilight->setToolTip(QCoreApplication::translate("timeNavigatorDialog", "This date's morning twilight", nullptr));
#endif // QT_CONFIG(tooltip)
        btnThisDateMorningTwilight->setText(QCoreApplication::translate("timeNavigatorDialog", "Morning", nullptr));
#if QT_CONFIG(tooltip)
        btnNextMorningTwilight->setToolTip(QCoreApplication::translate("timeNavigatorDialog", "Next morning twilight", nullptr));
#endif // QT_CONFIG(tooltip)
        btnNextMorningTwilight->setText(QCoreApplication::translate("timeNavigatorDialog", "Morning", nullptr));
#if QT_CONFIG(tooltip)
        btnPreviousEveningTwilight->setToolTip(QCoreApplication::translate("timeNavigatorDialog", "Previous evening twilight", nullptr));
#endif // QT_CONFIG(tooltip)
        btnPreviousEveningTwilight->setText(QCoreApplication::translate("timeNavigatorDialog", "Evening", nullptr));
#if QT_CONFIG(tooltip)
        btnThisDateEveningTwilight->setToolTip(QCoreApplication::translate("timeNavigatorDialog", "This date's evening twilight", nullptr));
#endif // QT_CONFIG(tooltip)
        btnThisDateEveningTwilight->setText(QCoreApplication::translate("timeNavigatorDialog", "Evening", nullptr));
#if QT_CONFIG(tooltip)
        btnNextEveningTwilight->setToolTip(QCoreApplication::translate("timeNavigatorDialog", "Next evening twilight", nullptr));
#endif // QT_CONFIG(tooltip)
        btnNextEveningTwilight->setText(QCoreApplication::translate("timeNavigatorDialog", "Evening", nullptr));
        groupAtAltitude->setTitle(QCoreApplication::translate("timeNavigatorDialog", "Selected object at current altitude", nullptr));
        labelAltPrev->setText(QCoreApplication::translate("timeNavigatorDialog", "Previous", nullptr));
        labelAltThisDate->setText(QCoreApplication::translate("timeNavigatorDialog", "This date", nullptr));
        labelAltNext->setText(QCoreApplication::translate("timeNavigatorDialog", "Next", nullptr));
#if QT_CONFIG(tooltip)
        btnPreviousMorningAtAltitude->setToolTip(QCoreApplication::translate("timeNavigatorDialog", "Selected object at its current altitude, previous morning", nullptr));
#endif // QT_CONFIG(tooltip)
        btnPreviousMorningAtAltitude->setText(QCoreApplication::translate("timeNavigatorDialog", "Morning", nullptr));
#if QT_CONFIG(tooltip)
        btnThisDateMorningAtAltitude->setToolTip(QCoreApplication::translate("timeNavigatorDialog", "Selected object at its current altitude this date's morning", nullptr));
#endif // QT_CONFIG(tooltip)
        btnThisDateMorningAtAltitude->setText(QCoreApplication::translate("timeNavigatorDialog", "Morning", nullptr));
#if QT_CONFIG(tooltip)
        btnNextMorningAtAltitude->setToolTip(QCoreApplication::translate("timeNavigatorDialog", "Selected object at its current altitude, next morning", nullptr));
#endif // QT_CONFIG(tooltip)
        btnNextMorningAtAltitude->setText(QCoreApplication::translate("timeNavigatorDialog", "Morning", nullptr));
#if QT_CONFIG(tooltip)
        btnPreviousEveningAtAltitude->setToolTip(QCoreApplication::translate("timeNavigatorDialog", "Selected object at its current altitude, previous evening", nullptr));
#endif // QT_CONFIG(tooltip)
        btnPreviousEveningAtAltitude->setText(QCoreApplication::translate("timeNavigatorDialog", "Evening", nullptr));
#if QT_CONFIG(tooltip)
        btnThisDateEveningAtAltitude->setToolTip(QCoreApplication::translate("timeNavigatorDialog", "Selected object at its current altitude this date's evening", nullptr));
#endif // QT_CONFIG(tooltip)
        btnThisDateEveningAtAltitude->setText(QCoreApplication::translate("timeNavigatorDialog", "Evening", nullptr));
#if QT_CONFIG(tooltip)
        btnNextEveningAtAltitude->setToolTip(QCoreApplication::translate("timeNavigatorDialog", "Selected object at its current altitude, next evening", nullptr));
#endif // QT_CONFIG(tooltip)
        btnNextEveningAtAltitude->setText(QCoreApplication::translate("timeNavigatorDialog", "Evening", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabSpecificTime), QCoreApplication::translate("timeNavigatorDialog", "Specific Time", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabPlanetaryEvents), QCoreApplication::translate("timeNavigatorDialog", "Planetary Events", nullptr));
        checkBoxShowButton->setText(QCoreApplication::translate("timeNavigatorDialog", "Show plug-in button on toolbar", nullptr));
        checkBoxSelectObject->setText(QCoreApplication::translate("timeNavigatorDialog", "Select object at event", nullptr));
        checkBoxCenterView->setText(QCoreApplication::translate("timeNavigatorDialog", "Center view on object at event", nullptr));
        btnRestoreDefaults->setText(QCoreApplication::translate("timeNavigatorDialog", "Restore defaults", nullptr));
        btnSaveSettings->setText(QCoreApplication::translate("timeNavigatorDialog", "Save settings", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabSettings), QCoreApplication::translate("timeNavigatorDialog", "Settings", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabAbout), QCoreApplication::translate("timeNavigatorDialog", "About", "tab in plugin windows"));
        (void)timeNavigatorDialog;
    } // retranslateUi

};

namespace Ui {
    class timeNavigatorDialog: public Ui_timeNavigatorDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_TIMENAVIGATORDIALOG_H
