/********************************************************************************
** Form generated from reading UI file 'manualImportWindow.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MANUALIMPORTWINDOW_H
#define UI_MANUALIMPORTWINDOW_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QScrollArea>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "Dialog.hpp"
#include "gui/StelCloseButton.hpp"

QT_BEGIN_NAMESPACE

class Ui_manualImportWindow
{
public:
    QGridLayout *gridLayoutWindow;
    TitleBar *titleBar;
    QGroupBox *groupBoxObjectType;
    QVBoxLayout *verticalLayoutObjectType;
    QRadioButton *radioButtonSolarSystemObject;
    QRadioButton *radioButtonMinorPlanet;
    QRadioButton *radioButtonComet;
    QRadioButton *radioButtonSatellite;
    QGroupBox *groupBoxName;
    QGridLayout *gridLayoutName;
    QLabel *labelName;
    QLineEdit *lineEditName;
    QLabel *labelIdentifier;
    QLineEdit *lineEditIdentifier;
    QCheckBox *checkBoxMinorPlanetNumber;
    QLineEdit *lineEditMinorPlanetNumber;
    QScrollArea *scrollArea;
    QWidget *scrollAreaWidgetContents;
    QVBoxLayout *verticalLayoutScroll;
    QGroupBox *groupBoxProperties;
    QGridLayout *gridLayout_4;
    QLabel *labelParent;
    QComboBox *comboBoxParent;
    QLabel *labelRadius;
    QLineEdit *lineEditRadius;
    QLabel *labelOblateness;
    QLineEdit *lineEditOblateness;
    QLabel *labelAlbedo;
    QLineEdit *lineEditAlbedo;
    QLabel *labelColor;
    QPushButton *pushButtonSelectColor;
    QCheckBox *checkBoxLighting;
    QCheckBox *checkBoxAtmosphere;
    QCheckBox *checkBoxHidden;
    QLabel *labelTexture;
    QLineEdit *lineEditTexture;
    QPushButton *pushButtonSelectTexture;
    QFrame *frameMagnitude;
    QGridLayout *gridLayoutMagnitude;
    QLabel *labelAbsoluteMagnitude;
    QLineEdit *lineEditAbsoluteMagnitude;
    QLabel *labelSlopeParameter;
    QLineEdit *lineEditSlopeParameter;
    QPushButton *pushButtonCalculateRadius;
    QLabel *labelMagnitudeSystem;
    QLineEdit *lineEditColor;
    QGroupBox *groupBoxOrbitalElements;
    QGridLayout *gridLayoutOrbitalElements;
    QLabel *labelOrbitType;
    QRadioButton *radioButtonCometaryOrbit;
    QRadioButton *radioButtonEllipticOrbit;
    QLabel *labelEccentricity;
    QLineEdit *lineEditEccentricity;
    QLabel *labelInclination;
    QLineEdit *lineEditInclination;
    QLabel *labelLongitudeOfTheAscendingNode;
    QLineEdit *lineEditLongitudeOfTheAscendingNode;
    QFrame *framePeriapsis;
    QGridLayout *gridLayoutPeriapsis;
    QRadioButton *radioButtonArgumentOfPeriapsis;
    QLineEdit *lineEditArgumentOfPeriapsis;
    QRadioButton *radioButtonLongitudeOfPeriapsis;
    QLineEdit *lineEditLongitudeOfPeriapsis;
    QFrame *frameSemiMajorAxis;
    QGridLayout *gridLayoutSemiMajorAxis;
    QLineEdit *lineEditSemiMajorAxis;
    QLineEdit *lineEditPeriapsisDistance;
    QRadioButton *radioButtonSemiMajorAxis;
    QRadioButton *radioButtonPeriapsisDistance;
    QFrame *framePeriod;
    QGridLayout *gridLayoutPeriod;
    QCheckBox *checkBoxPeriod;
    QLineEdit *lineEditPeriod;
    QCheckBox *checkBoxMeanMotion;
    QLineEdit *lineEditMeanMotion;
    QFrame *frameEpoch;
    QGridLayout *gridLayoutEpoch;
    QLineEdit *lineEditEpoch;
    QLineEdit *lineEditTimeOfPeriapsis;
    QRadioButton *radioButtonEpoch;
    QRadioButton *radioButtonTimeOfPeriapsis;
    QPushButton *pushButtonEpochJ2000;
    QFrame *frameMeanAnomaly;
    QGridLayout *gridLayout_2;
    QLineEdit *lineEditMeanAnomaly;
    QLineEdit *lineEditMeanLongitude;
    QRadioButton *radioButtonMeanAnomaly;
    QRadioButton *radioButtonMeanLongitude;
    QRadioButton *radioButtonOrbitFunction;
    QComboBox *comboBoxOrbitFunction;
    QCheckBox *checkBoxOrbitVisualizationPeriod;
    QLineEdit *lineEditOrbitVisualizationPeriod;
    QGroupBox *groupBoxRotationalElements;
    QGridLayout *gridLayout;
    QCheckBox *checkBoxObliquity;
    QLineEdit *lineEditObliquity;
    QCheckBox *checkBoxNorthPoleRA;
    QLineEdit *lineEditNorthPoleRA;
    QCheckBox *checkBoxNorthPoleDec;
    QLineEdit *lineEditNorthPoleDec;
    QCheckBox *checkBoxRotationalPeriod;
    QLineEdit *lineEditRotationalPeriod;
    QCheckBox *checkBoxPrecessionRate;
    QLineEdit *lineEditPrecessionRate;
    QCheckBox *checkBoxRotationalEpoch;
    QLineEdit *lineEditRotationalEpoch;
    QCheckBox *checkBoxOffset;
    QLineEdit *lineEditOffset;
    QCheckBox *checkBoxEquatorAscendingNode;
    QPushButton *pushButtonRotationalEpochJ2000;
    QLineEdit *lineEditEquatorAscendingNode;
    QGroupBox *groupBoxRings;
    QGridLayout *gridLayout_3;
    QLabel *labelInnerSize;
    QLabel *labelOuterSize;
    QLabel *labelRingTexture;
    QLineEdit *lineEditRingTexture;
    QPushButton *pushButtonSelectRingTexture;
    QLineEdit *lineEditRingInnerSize;
    QLineEdit *lineEditRingOuterSize;
    QFrame *frameButton;
    QHBoxLayout *horizontalLayoutButton;
    QLabel *labelErrorMessage;
    QPushButton *pushButtonSave;

    void setupUi(QWidget *manualImportWindow)
    {
        if (manualImportWindow->objectName().isEmpty())
            manualImportWindow->setObjectName("manualImportWindow");
        manualImportWindow->resize(600, 500);
        gridLayoutWindow = new QGridLayout(manualImportWindow);
        gridLayoutWindow->setSpacing(0);
        gridLayoutWindow->setContentsMargins(0, 0, 0, 0);
        gridLayoutWindow->setObjectName("gridLayoutWindow");
        titleBar = new TitleBar(manualImportWindow);
        titleBar->setObjectName("titleBar");
        titleBar->setTitle(QString::fromUtf8("Define an object"));

        gridLayoutWindow->addWidget(titleBar, 0, 0, 1, 2);

        groupBoxObjectType = new QGroupBox(manualImportWindow);
        groupBoxObjectType->setObjectName("groupBoxObjectType");
        groupBoxObjectType->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
        verticalLayoutObjectType = new QVBoxLayout(groupBoxObjectType);
        verticalLayoutObjectType->setContentsMargins(0, 0, 0, 0);
        verticalLayoutObjectType->setObjectName("verticalLayoutObjectType");
        radioButtonSolarSystemObject = new QRadioButton(groupBoxObjectType);
        radioButtonSolarSystemObject->setObjectName("radioButtonSolarSystemObject");

        verticalLayoutObjectType->addWidget(radioButtonSolarSystemObject);

        radioButtonMinorPlanet = new QRadioButton(groupBoxObjectType);
        radioButtonMinorPlanet->setObjectName("radioButtonMinorPlanet");

        verticalLayoutObjectType->addWidget(radioButtonMinorPlanet);

        radioButtonComet = new QRadioButton(groupBoxObjectType);
        radioButtonComet->setObjectName("radioButtonComet");

        verticalLayoutObjectType->addWidget(radioButtonComet);

        radioButtonSatellite = new QRadioButton(groupBoxObjectType);
        radioButtonSatellite->setObjectName("radioButtonSatellite");

        verticalLayoutObjectType->addWidget(radioButtonSatellite);


        gridLayoutWindow->addWidget(groupBoxObjectType, 1, 1, 1, 1);

        groupBoxName = new QGroupBox(manualImportWindow);
        groupBoxName->setObjectName("groupBoxName");
        groupBoxName->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
        gridLayoutName = new QGridLayout(groupBoxName);
        gridLayoutName->setContentsMargins(0, 0, 0, 0);
        gridLayoutName->setObjectName("gridLayoutName");
        labelName = new QLabel(groupBoxName);
        labelName->setObjectName("labelName");

        gridLayoutName->addWidget(labelName, 0, 0, 1, 1);

        lineEditName = new QLineEdit(groupBoxName);
        lineEditName->setObjectName("lineEditName");

        gridLayoutName->addWidget(lineEditName, 0, 1, 1, 1);

        labelIdentifier = new QLabel(groupBoxName);
        labelIdentifier->setObjectName("labelIdentifier");

        gridLayoutName->addWidget(labelIdentifier, 1, 0, 1, 1);

        lineEditIdentifier = new QLineEdit(groupBoxName);
        lineEditIdentifier->setObjectName("lineEditIdentifier");

        gridLayoutName->addWidget(lineEditIdentifier, 1, 1, 1, 1);

        checkBoxMinorPlanetNumber = new QCheckBox(groupBoxName);
        checkBoxMinorPlanetNumber->setObjectName("checkBoxMinorPlanetNumber");

        gridLayoutName->addWidget(checkBoxMinorPlanetNumber, 2, 0, 1, 1);

        lineEditMinorPlanetNumber = new QLineEdit(groupBoxName);
        lineEditMinorPlanetNumber->setObjectName("lineEditMinorPlanetNumber");

        gridLayoutName->addWidget(lineEditMinorPlanetNumber, 2, 1, 1, 1);


        gridLayoutWindow->addWidget(groupBoxName, 1, 0, 1, 1);

        scrollArea = new QScrollArea(manualImportWindow);
        scrollArea->setObjectName("scrollArea");
        scrollArea->setWidgetResizable(true);
        scrollArea->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
        scrollAreaWidgetContents = new QWidget();
        scrollAreaWidgetContents->setObjectName("scrollAreaWidgetContents");
        scrollAreaWidgetContents->setGeometry(QRect(0, 0, 582, 1436));
        verticalLayoutScroll = new QVBoxLayout(scrollAreaWidgetContents);
        verticalLayoutScroll->setContentsMargins(0, 0, 0, 0);
        verticalLayoutScroll->setObjectName("verticalLayoutScroll");
        groupBoxProperties = new QGroupBox(scrollAreaWidgetContents);
        groupBoxProperties->setObjectName("groupBoxProperties");
        gridLayout_4 = new QGridLayout(groupBoxProperties);
        gridLayout_4->setObjectName("gridLayout_4");
        labelParent = new QLabel(groupBoxProperties);
        labelParent->setObjectName("labelParent");

        gridLayout_4->addWidget(labelParent, 0, 0, 1, 1);

        comboBoxParent = new QComboBox(groupBoxProperties);
        comboBoxParent->setObjectName("comboBoxParent");

        gridLayout_4->addWidget(comboBoxParent, 0, 2, 1, 2);

        labelRadius = new QLabel(groupBoxProperties);
        labelRadius->setObjectName("labelRadius");

        gridLayout_4->addWidget(labelRadius, 1, 0, 1, 1);

        lineEditRadius = new QLineEdit(groupBoxProperties);
        lineEditRadius->setObjectName("lineEditRadius");

        gridLayout_4->addWidget(lineEditRadius, 1, 2, 1, 2);

        labelOblateness = new QLabel(groupBoxProperties);
        labelOblateness->setObjectName("labelOblateness");

        gridLayout_4->addWidget(labelOblateness, 2, 0, 1, 1);

        lineEditOblateness = new QLineEdit(groupBoxProperties);
        lineEditOblateness->setObjectName("lineEditOblateness");

        gridLayout_4->addWidget(lineEditOblateness, 2, 2, 1, 2);

        labelAlbedo = new QLabel(groupBoxProperties);
        labelAlbedo->setObjectName("labelAlbedo");

        gridLayout_4->addWidget(labelAlbedo, 3, 0, 1, 1);

        lineEditAlbedo = new QLineEdit(groupBoxProperties);
        lineEditAlbedo->setObjectName("lineEditAlbedo");

        gridLayout_4->addWidget(lineEditAlbedo, 3, 2, 1, 2);

        labelColor = new QLabel(groupBoxProperties);
        labelColor->setObjectName("labelColor");

        gridLayout_4->addWidget(labelColor, 4, 0, 1, 1);

        pushButtonSelectColor = new QPushButton(groupBoxProperties);
        pushButtonSelectColor->setObjectName("pushButtonSelectColor");

        gridLayout_4->addWidget(pushButtonSelectColor, 4, 3, 1, 1);

        checkBoxLighting = new QCheckBox(groupBoxProperties);
        checkBoxLighting->setObjectName("checkBoxLighting");

        gridLayout_4->addWidget(checkBoxLighting, 5, 0, 1, 4);

        checkBoxAtmosphere = new QCheckBox(groupBoxProperties);
        checkBoxAtmosphere->setObjectName("checkBoxAtmosphere");

        gridLayout_4->addWidget(checkBoxAtmosphere, 6, 0, 1, 4);

        checkBoxHidden = new QCheckBox(groupBoxProperties);
        checkBoxHidden->setObjectName("checkBoxHidden");

        gridLayout_4->addWidget(checkBoxHidden, 7, 0, 1, 4);

        labelTexture = new QLabel(groupBoxProperties);
        labelTexture->setObjectName("labelTexture");

        gridLayout_4->addWidget(labelTexture, 8, 0, 1, 1);

        lineEditTexture = new QLineEdit(groupBoxProperties);
        lineEditTexture->setObjectName("lineEditTexture");
        lineEditTexture->setReadOnly(true);

        gridLayout_4->addWidget(lineEditTexture, 8, 2, 1, 1);

        pushButtonSelectTexture = new QPushButton(groupBoxProperties);
        pushButtonSelectTexture->setObjectName("pushButtonSelectTexture");

        gridLayout_4->addWidget(pushButtonSelectTexture, 8, 3, 1, 1);

        frameMagnitude = new QFrame(groupBoxProperties);
        frameMagnitude->setObjectName("frameMagnitude");
        gridLayoutMagnitude = new QGridLayout(frameMagnitude);
        gridLayoutMagnitude->setContentsMargins(0, 0, 0, 0);
        gridLayoutMagnitude->setObjectName("gridLayoutMagnitude");
        labelAbsoluteMagnitude = new QLabel(frameMagnitude);
        labelAbsoluteMagnitude->setObjectName("labelAbsoluteMagnitude");

        gridLayoutMagnitude->addWidget(labelAbsoluteMagnitude, 1, 0, 1, 1);

        lineEditAbsoluteMagnitude = new QLineEdit(frameMagnitude);
        lineEditAbsoluteMagnitude->setObjectName("lineEditAbsoluteMagnitude");

        gridLayoutMagnitude->addWidget(lineEditAbsoluteMagnitude, 1, 1, 1, 1);

        labelSlopeParameter = new QLabel(frameMagnitude);
        labelSlopeParameter->setObjectName("labelSlopeParameter");

        gridLayoutMagnitude->addWidget(labelSlopeParameter, 1, 2, 1, 1);

        lineEditSlopeParameter = new QLineEdit(frameMagnitude);
        lineEditSlopeParameter->setObjectName("lineEditSlopeParameter");

        gridLayoutMagnitude->addWidget(lineEditSlopeParameter, 1, 3, 1, 1);

        pushButtonCalculateRadius = new QPushButton(frameMagnitude);
        pushButtonCalculateRadius->setObjectName("pushButtonCalculateRadius");

        gridLayoutMagnitude->addWidget(pushButtonCalculateRadius, 2, 0, 1, 4);

        labelMagnitudeSystem = new QLabel(frameMagnitude);
        labelMagnitudeSystem->setObjectName("labelMagnitudeSystem");

        gridLayoutMagnitude->addWidget(labelMagnitudeSystem, 0, 0, 1, 4);


        gridLayout_4->addWidget(frameMagnitude, 9, 0, 1, 4);

        lineEditColor = new QLineEdit(groupBoxProperties);
        lineEditColor->setObjectName("lineEditColor");

        gridLayout_4->addWidget(lineEditColor, 4, 2, 1, 1);


        verticalLayoutScroll->addWidget(groupBoxProperties);

        groupBoxOrbitalElements = new QGroupBox(scrollAreaWidgetContents);
        groupBoxOrbitalElements->setObjectName("groupBoxOrbitalElements");
        gridLayoutOrbitalElements = new QGridLayout(groupBoxOrbitalElements);
        gridLayoutOrbitalElements->setContentsMargins(0, 0, 0, 0);
        gridLayoutOrbitalElements->setObjectName("gridLayoutOrbitalElements");
        labelOrbitType = new QLabel(groupBoxOrbitalElements);
        labelOrbitType->setObjectName("labelOrbitType");

        gridLayoutOrbitalElements->addWidget(labelOrbitType, 0, 0, 1, 2);

        radioButtonCometaryOrbit = new QRadioButton(groupBoxOrbitalElements);
        radioButtonCometaryOrbit->setObjectName("radioButtonCometaryOrbit");

        gridLayoutOrbitalElements->addWidget(radioButtonCometaryOrbit, 1, 0, 1, 2);

        radioButtonEllipticOrbit = new QRadioButton(groupBoxOrbitalElements);
        radioButtonEllipticOrbit->setObjectName("radioButtonEllipticOrbit");

        gridLayoutOrbitalElements->addWidget(radioButtonEllipticOrbit, 2, 0, 1, 2);

        labelEccentricity = new QLabel(groupBoxOrbitalElements);
        labelEccentricity->setObjectName("labelEccentricity");

        gridLayoutOrbitalElements->addWidget(labelEccentricity, 4, 0, 1, 1);

        lineEditEccentricity = new QLineEdit(groupBoxOrbitalElements);
        lineEditEccentricity->setObjectName("lineEditEccentricity");

        gridLayoutOrbitalElements->addWidget(lineEditEccentricity, 4, 1, 1, 1);

        labelInclination = new QLabel(groupBoxOrbitalElements);
        labelInclination->setObjectName("labelInclination");

        gridLayoutOrbitalElements->addWidget(labelInclination, 5, 0, 1, 1);

        lineEditInclination = new QLineEdit(groupBoxOrbitalElements);
        lineEditInclination->setObjectName("lineEditInclination");

        gridLayoutOrbitalElements->addWidget(lineEditInclination, 5, 1, 1, 1);

        labelLongitudeOfTheAscendingNode = new QLabel(groupBoxOrbitalElements);
        labelLongitudeOfTheAscendingNode->setObjectName("labelLongitudeOfTheAscendingNode");

        gridLayoutOrbitalElements->addWidget(labelLongitudeOfTheAscendingNode, 6, 0, 1, 1);

        lineEditLongitudeOfTheAscendingNode = new QLineEdit(groupBoxOrbitalElements);
        lineEditLongitudeOfTheAscendingNode->setObjectName("lineEditLongitudeOfTheAscendingNode");

        gridLayoutOrbitalElements->addWidget(lineEditLongitudeOfTheAscendingNode, 6, 1, 1, 1);

        framePeriapsis = new QFrame(groupBoxOrbitalElements);
        framePeriapsis->setObjectName("framePeriapsis");
        gridLayoutPeriapsis = new QGridLayout(framePeriapsis);
        gridLayoutPeriapsis->setContentsMargins(0, 0, 0, 0);
        gridLayoutPeriapsis->setObjectName("gridLayoutPeriapsis");
        radioButtonArgumentOfPeriapsis = new QRadioButton(framePeriapsis);
        radioButtonArgumentOfPeriapsis->setObjectName("radioButtonArgumentOfPeriapsis");

        gridLayoutPeriapsis->addWidget(radioButtonArgumentOfPeriapsis, 0, 0, 1, 1);

        lineEditArgumentOfPeriapsis = new QLineEdit(framePeriapsis);
        lineEditArgumentOfPeriapsis->setObjectName("lineEditArgumentOfPeriapsis");

        gridLayoutPeriapsis->addWidget(lineEditArgumentOfPeriapsis, 0, 1, 1, 1);

        radioButtonLongitudeOfPeriapsis = new QRadioButton(framePeriapsis);
        radioButtonLongitudeOfPeriapsis->setObjectName("radioButtonLongitudeOfPeriapsis");

        gridLayoutPeriapsis->addWidget(radioButtonLongitudeOfPeriapsis, 1, 0, 1, 1);

        lineEditLongitudeOfPeriapsis = new QLineEdit(framePeriapsis);
        lineEditLongitudeOfPeriapsis->setObjectName("lineEditLongitudeOfPeriapsis");

        gridLayoutPeriapsis->addWidget(lineEditLongitudeOfPeriapsis, 1, 1, 1, 1);


        gridLayoutOrbitalElements->addWidget(framePeriapsis, 7, 0, 1, 2);

        frameSemiMajorAxis = new QFrame(groupBoxOrbitalElements);
        frameSemiMajorAxis->setObjectName("frameSemiMajorAxis");
        gridLayoutSemiMajorAxis = new QGridLayout(frameSemiMajorAxis);
        gridLayoutSemiMajorAxis->setContentsMargins(0, 0, 0, 0);
        gridLayoutSemiMajorAxis->setObjectName("gridLayoutSemiMajorAxis");
        lineEditSemiMajorAxis = new QLineEdit(frameSemiMajorAxis);
        lineEditSemiMajorAxis->setObjectName("lineEditSemiMajorAxis");

        gridLayoutSemiMajorAxis->addWidget(lineEditSemiMajorAxis, 0, 1, 1, 1);

        lineEditPeriapsisDistance = new QLineEdit(frameSemiMajorAxis);
        lineEditPeriapsisDistance->setObjectName("lineEditPeriapsisDistance");

        gridLayoutSemiMajorAxis->addWidget(lineEditPeriapsisDistance, 1, 1, 1, 1);

        radioButtonSemiMajorAxis = new QRadioButton(frameSemiMajorAxis);
        radioButtonSemiMajorAxis->setObjectName("radioButtonSemiMajorAxis");

        gridLayoutSemiMajorAxis->addWidget(radioButtonSemiMajorAxis, 0, 0, 1, 1);

        radioButtonPeriapsisDistance = new QRadioButton(frameSemiMajorAxis);
        radioButtonPeriapsisDistance->setObjectName("radioButtonPeriapsisDistance");

        gridLayoutSemiMajorAxis->addWidget(radioButtonPeriapsisDistance, 1, 0, 1, 1);


        gridLayoutOrbitalElements->addWidget(frameSemiMajorAxis, 8, 0, 1, 2);

        framePeriod = new QFrame(groupBoxOrbitalElements);
        framePeriod->setObjectName("framePeriod");
        gridLayoutPeriod = new QGridLayout(framePeriod);
        gridLayoutPeriod->setContentsMargins(0, 0, 0, 0);
        gridLayoutPeriod->setObjectName("gridLayoutPeriod");
        checkBoxPeriod = new QCheckBox(framePeriod);
        checkBoxPeriod->setObjectName("checkBoxPeriod");

        gridLayoutPeriod->addWidget(checkBoxPeriod, 0, 0, 1, 1);

        lineEditPeriod = new QLineEdit(framePeriod);
        lineEditPeriod->setObjectName("lineEditPeriod");

        gridLayoutPeriod->addWidget(lineEditPeriod, 0, 1, 1, 1);

        checkBoxMeanMotion = new QCheckBox(framePeriod);
        checkBoxMeanMotion->setObjectName("checkBoxMeanMotion");

        gridLayoutPeriod->addWidget(checkBoxMeanMotion, 1, 0, 1, 1);

        lineEditMeanMotion = new QLineEdit(framePeriod);
        lineEditMeanMotion->setObjectName("lineEditMeanMotion");

        gridLayoutPeriod->addWidget(lineEditMeanMotion, 1, 1, 1, 1);


        gridLayoutOrbitalElements->addWidget(framePeriod, 9, 0, 1, 2);

        frameEpoch = new QFrame(groupBoxOrbitalElements);
        frameEpoch->setObjectName("frameEpoch");
        gridLayoutEpoch = new QGridLayout(frameEpoch);
        gridLayoutEpoch->setContentsMargins(0, 0, 0, 0);
        gridLayoutEpoch->setObjectName("gridLayoutEpoch");
        lineEditEpoch = new QLineEdit(frameEpoch);
        lineEditEpoch->setObjectName("lineEditEpoch");

        gridLayoutEpoch->addWidget(lineEditEpoch, 0, 1, 1, 1);

        lineEditTimeOfPeriapsis = new QLineEdit(frameEpoch);
        lineEditTimeOfPeriapsis->setObjectName("lineEditTimeOfPeriapsis");

        gridLayoutEpoch->addWidget(lineEditTimeOfPeriapsis, 1, 1, 1, 2);

        radioButtonEpoch = new QRadioButton(frameEpoch);
        radioButtonEpoch->setObjectName("radioButtonEpoch");

        gridLayoutEpoch->addWidget(radioButtonEpoch, 0, 0, 1, 1);

        radioButtonTimeOfPeriapsis = new QRadioButton(frameEpoch);
        radioButtonTimeOfPeriapsis->setObjectName("radioButtonTimeOfPeriapsis");

        gridLayoutEpoch->addWidget(radioButtonTimeOfPeriapsis, 1, 0, 1, 1);

        pushButtonEpochJ2000 = new QPushButton(frameEpoch);
        pushButtonEpochJ2000->setObjectName("pushButtonEpochJ2000");

        gridLayoutEpoch->addWidget(pushButtonEpochJ2000, 0, 2, 1, 1);


        gridLayoutOrbitalElements->addWidget(frameEpoch, 10, 0, 1, 2);

        frameMeanAnomaly = new QFrame(groupBoxOrbitalElements);
        frameMeanAnomaly->setObjectName("frameMeanAnomaly");
        gridLayout_2 = new QGridLayout(frameMeanAnomaly);
        gridLayout_2->setContentsMargins(0, 0, 0, 0);
        gridLayout_2->setObjectName("gridLayout_2");
        lineEditMeanAnomaly = new QLineEdit(frameMeanAnomaly);
        lineEditMeanAnomaly->setObjectName("lineEditMeanAnomaly");

        gridLayout_2->addWidget(lineEditMeanAnomaly, 1, 1, 1, 1);

        lineEditMeanLongitude = new QLineEdit(frameMeanAnomaly);
        lineEditMeanLongitude->setObjectName("lineEditMeanLongitude");

        gridLayout_2->addWidget(lineEditMeanLongitude, 2, 1, 1, 1);

        radioButtonMeanAnomaly = new QRadioButton(frameMeanAnomaly);
        radioButtonMeanAnomaly->setObjectName("radioButtonMeanAnomaly");

        gridLayout_2->addWidget(radioButtonMeanAnomaly, 1, 0, 1, 1);

        radioButtonMeanLongitude = new QRadioButton(frameMeanAnomaly);
        radioButtonMeanLongitude->setObjectName("radioButtonMeanLongitude");

        gridLayout_2->addWidget(radioButtonMeanLongitude, 2, 0, 1, 1);


        gridLayoutOrbitalElements->addWidget(frameMeanAnomaly, 11, 0, 1, 2);

        radioButtonOrbitFunction = new QRadioButton(groupBoxOrbitalElements);
        radioButtonOrbitFunction->setObjectName("radioButtonOrbitFunction");

        gridLayoutOrbitalElements->addWidget(radioButtonOrbitFunction, 3, 0, 1, 1);

        comboBoxOrbitFunction = new QComboBox(groupBoxOrbitalElements);
        comboBoxOrbitFunction->setObjectName("comboBoxOrbitFunction");

        gridLayoutOrbitalElements->addWidget(comboBoxOrbitFunction, 3, 1, 1, 1);

        checkBoxOrbitVisualizationPeriod = new QCheckBox(groupBoxOrbitalElements);
        checkBoxOrbitVisualizationPeriod->setObjectName("checkBoxOrbitVisualizationPeriod");

        gridLayoutOrbitalElements->addWidget(checkBoxOrbitVisualizationPeriod, 12, 0, 1, 1);

        lineEditOrbitVisualizationPeriod = new QLineEdit(groupBoxOrbitalElements);
        lineEditOrbitVisualizationPeriod->setObjectName("lineEditOrbitVisualizationPeriod");

        gridLayoutOrbitalElements->addWidget(lineEditOrbitVisualizationPeriod, 12, 1, 1, 1);


        verticalLayoutScroll->addWidget(groupBoxOrbitalElements);

        groupBoxRotationalElements = new QGroupBox(scrollAreaWidgetContents);
        groupBoxRotationalElements->setObjectName("groupBoxRotationalElements");
        gridLayout = new QGridLayout(groupBoxRotationalElements);
        gridLayout->setObjectName("gridLayout");
        checkBoxObliquity = new QCheckBox(groupBoxRotationalElements);
        checkBoxObliquity->setObjectName("checkBoxObliquity");

        gridLayout->addWidget(checkBoxObliquity, 0, 0, 1, 1);

        lineEditObliquity = new QLineEdit(groupBoxRotationalElements);
        lineEditObliquity->setObjectName("lineEditObliquity");

        gridLayout->addWidget(lineEditObliquity, 0, 1, 1, 3);

        checkBoxNorthPoleRA = new QCheckBox(groupBoxRotationalElements);
        checkBoxNorthPoleRA->setObjectName("checkBoxNorthPoleRA");

        gridLayout->addWidget(checkBoxNorthPoleRA, 2, 0, 1, 1);

        lineEditNorthPoleRA = new QLineEdit(groupBoxRotationalElements);
        lineEditNorthPoleRA->setObjectName("lineEditNorthPoleRA");

        gridLayout->addWidget(lineEditNorthPoleRA, 2, 1, 1, 3);

        checkBoxNorthPoleDec = new QCheckBox(groupBoxRotationalElements);
        checkBoxNorthPoleDec->setObjectName("checkBoxNorthPoleDec");

        gridLayout->addWidget(checkBoxNorthPoleDec, 3, 0, 1, 1);

        lineEditNorthPoleDec = new QLineEdit(groupBoxRotationalElements);
        lineEditNorthPoleDec->setObjectName("lineEditNorthPoleDec");

        gridLayout->addWidget(lineEditNorthPoleDec, 3, 1, 1, 3);

        checkBoxRotationalPeriod = new QCheckBox(groupBoxRotationalElements);
        checkBoxRotationalPeriod->setObjectName("checkBoxRotationalPeriod");

        gridLayout->addWidget(checkBoxRotationalPeriod, 4, 0, 1, 1);

        lineEditRotationalPeriod = new QLineEdit(groupBoxRotationalElements);
        lineEditRotationalPeriod->setObjectName("lineEditRotationalPeriod");

        gridLayout->addWidget(lineEditRotationalPeriod, 4, 1, 1, 3);

        checkBoxPrecessionRate = new QCheckBox(groupBoxRotationalElements);
        checkBoxPrecessionRate->setObjectName("checkBoxPrecessionRate");

        gridLayout->addWidget(checkBoxPrecessionRate, 5, 0, 1, 1);

        lineEditPrecessionRate = new QLineEdit(groupBoxRotationalElements);
        lineEditPrecessionRate->setObjectName("lineEditPrecessionRate");

        gridLayout->addWidget(lineEditPrecessionRate, 5, 1, 1, 3);

        checkBoxRotationalEpoch = new QCheckBox(groupBoxRotationalElements);
        checkBoxRotationalEpoch->setObjectName("checkBoxRotationalEpoch");

        gridLayout->addWidget(checkBoxRotationalEpoch, 6, 0, 1, 1);

        lineEditRotationalEpoch = new QLineEdit(groupBoxRotationalElements);
        lineEditRotationalEpoch->setObjectName("lineEditRotationalEpoch");

        gridLayout->addWidget(lineEditRotationalEpoch, 6, 1, 1, 1);

        checkBoxOffset = new QCheckBox(groupBoxRotationalElements);
        checkBoxOffset->setObjectName("checkBoxOffset");

        gridLayout->addWidget(checkBoxOffset, 7, 0, 1, 1);

        lineEditOffset = new QLineEdit(groupBoxRotationalElements);
        lineEditOffset->setObjectName("lineEditOffset");

        gridLayout->addWidget(lineEditOffset, 7, 1, 1, 3);

        checkBoxEquatorAscendingNode = new QCheckBox(groupBoxRotationalElements);
        checkBoxEquatorAscendingNode->setObjectName("checkBoxEquatorAscendingNode");

        gridLayout->addWidget(checkBoxEquatorAscendingNode, 1, 0, 1, 3);

        pushButtonRotationalEpochJ2000 = new QPushButton(groupBoxRotationalElements);
        pushButtonRotationalEpochJ2000->setObjectName("pushButtonRotationalEpochJ2000");

        gridLayout->addWidget(pushButtonRotationalEpochJ2000, 6, 3, 1, 1);

        lineEditEquatorAscendingNode = new QLineEdit(groupBoxRotationalElements);
        lineEditEquatorAscendingNode->setObjectName("lineEditEquatorAscendingNode");

        gridLayout->addWidget(lineEditEquatorAscendingNode, 1, 3, 1, 1);


        verticalLayoutScroll->addWidget(groupBoxRotationalElements);

        groupBoxRings = new QGroupBox(scrollAreaWidgetContents);
        groupBoxRings->setObjectName("groupBoxRings");
        groupBoxRings->setCheckable(true);
        groupBoxRings->setChecked(false);
        gridLayout_3 = new QGridLayout(groupBoxRings);
        gridLayout_3->setObjectName("gridLayout_3");
        labelInnerSize = new QLabel(groupBoxRings);
        labelInnerSize->setObjectName("labelInnerSize");

        gridLayout_3->addWidget(labelInnerSize, 0, 0, 1, 1);

        labelOuterSize = new QLabel(groupBoxRings);
        labelOuterSize->setObjectName("labelOuterSize");

        gridLayout_3->addWidget(labelOuterSize, 1, 0, 1, 1);

        labelRingTexture = new QLabel(groupBoxRings);
        labelRingTexture->setObjectName("labelRingTexture");

        gridLayout_3->addWidget(labelRingTexture, 2, 0, 1, 1);

        lineEditRingTexture = new QLineEdit(groupBoxRings);
        lineEditRingTexture->setObjectName("lineEditRingTexture");
        lineEditRingTexture->setReadOnly(true);

        gridLayout_3->addWidget(lineEditRingTexture, 2, 1, 1, 1);

        pushButtonSelectRingTexture = new QPushButton(groupBoxRings);
        pushButtonSelectRingTexture->setObjectName("pushButtonSelectRingTexture");

        gridLayout_3->addWidget(pushButtonSelectRingTexture, 2, 2, 1, 1);

        lineEditRingInnerSize = new QLineEdit(groupBoxRings);
        lineEditRingInnerSize->setObjectName("lineEditRingInnerSize");

        gridLayout_3->addWidget(lineEditRingInnerSize, 0, 1, 1, 2);

        lineEditRingOuterSize = new QLineEdit(groupBoxRings);
        lineEditRingOuterSize->setObjectName("lineEditRingOuterSize");

        gridLayout_3->addWidget(lineEditRingOuterSize, 1, 1, 1, 2);


        verticalLayoutScroll->addWidget(groupBoxRings);

        scrollArea->setWidget(scrollAreaWidgetContents);

        gridLayoutWindow->addWidget(scrollArea, 2, 0, 1, 2);

        frameButton = new QFrame(manualImportWindow);
        frameButton->setObjectName("frameButton");
        horizontalLayoutButton = new QHBoxLayout(frameButton);
        horizontalLayoutButton->setObjectName("horizontalLayoutButton");
        labelErrorMessage = new QLabel(frameButton);
        labelErrorMessage->setObjectName("labelErrorMessage");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(labelErrorMessage->sizePolicy().hasHeightForWidth());
        labelErrorMessage->setSizePolicy(sizePolicy);

        horizontalLayoutButton->addWidget(labelErrorMessage);

        pushButtonSave = new QPushButton(frameButton);
        pushButtonSave->setObjectName("pushButtonSave");

        horizontalLayoutButton->addWidget(pushButtonSave);


        gridLayoutWindow->addWidget(frameButton, 3, 0, 1, 2);

        QWidget::setTabOrder(lineEditName, lineEditIdentifier);
        QWidget::setTabOrder(lineEditIdentifier, checkBoxMinorPlanetNumber);
        QWidget::setTabOrder(checkBoxMinorPlanetNumber, lineEditMinorPlanetNumber);
        QWidget::setTabOrder(lineEditMinorPlanetNumber, radioButtonSolarSystemObject);
        QWidget::setTabOrder(radioButtonSolarSystemObject, radioButtonMinorPlanet);
        QWidget::setTabOrder(radioButtonMinorPlanet, radioButtonComet);
        QWidget::setTabOrder(radioButtonComet, radioButtonSatellite);
        QWidget::setTabOrder(radioButtonSatellite, scrollArea);
        QWidget::setTabOrder(scrollArea, comboBoxParent);
        QWidget::setTabOrder(comboBoxParent, lineEditRadius);
        QWidget::setTabOrder(lineEditRadius, lineEditOblateness);
        QWidget::setTabOrder(lineEditOblateness, lineEditAlbedo);
        QWidget::setTabOrder(lineEditAlbedo, lineEditColor);
        QWidget::setTabOrder(lineEditColor, pushButtonSelectColor);
        QWidget::setTabOrder(pushButtonSelectColor, checkBoxLighting);
        QWidget::setTabOrder(checkBoxLighting, checkBoxAtmosphere);
        QWidget::setTabOrder(checkBoxAtmosphere, checkBoxHidden);
        QWidget::setTabOrder(checkBoxHidden, lineEditTexture);
        QWidget::setTabOrder(lineEditTexture, pushButtonSelectTexture);
        QWidget::setTabOrder(pushButtonSelectTexture, lineEditAbsoluteMagnitude);
        QWidget::setTabOrder(lineEditAbsoluteMagnitude, lineEditSlopeParameter);
        QWidget::setTabOrder(lineEditSlopeParameter, pushButtonCalculateRadius);
        QWidget::setTabOrder(pushButtonCalculateRadius, radioButtonCometaryOrbit);
        QWidget::setTabOrder(radioButtonCometaryOrbit, radioButtonEllipticOrbit);
        QWidget::setTabOrder(radioButtonEllipticOrbit, radioButtonOrbitFunction);
        QWidget::setTabOrder(radioButtonOrbitFunction, comboBoxOrbitFunction);
        QWidget::setTabOrder(comboBoxOrbitFunction, lineEditEccentricity);
        QWidget::setTabOrder(lineEditEccentricity, lineEditInclination);
        QWidget::setTabOrder(lineEditInclination, lineEditLongitudeOfTheAscendingNode);
        QWidget::setTabOrder(lineEditLongitudeOfTheAscendingNode, radioButtonArgumentOfPeriapsis);
        QWidget::setTabOrder(radioButtonArgumentOfPeriapsis, lineEditArgumentOfPeriapsis);
        QWidget::setTabOrder(lineEditArgumentOfPeriapsis, radioButtonLongitudeOfPeriapsis);
        QWidget::setTabOrder(radioButtonLongitudeOfPeriapsis, lineEditLongitudeOfPeriapsis);
        QWidget::setTabOrder(lineEditLongitudeOfPeriapsis, radioButtonSemiMajorAxis);
        QWidget::setTabOrder(radioButtonSemiMajorAxis, lineEditSemiMajorAxis);
        QWidget::setTabOrder(lineEditSemiMajorAxis, radioButtonPeriapsisDistance);
        QWidget::setTabOrder(radioButtonPeriapsisDistance, lineEditPeriapsisDistance);
        QWidget::setTabOrder(lineEditPeriapsisDistance, checkBoxPeriod);
        QWidget::setTabOrder(checkBoxPeriod, lineEditPeriod);
        QWidget::setTabOrder(lineEditPeriod, checkBoxMeanMotion);
        QWidget::setTabOrder(checkBoxMeanMotion, lineEditMeanMotion);
        QWidget::setTabOrder(lineEditMeanMotion, radioButtonEpoch);
        QWidget::setTabOrder(radioButtonEpoch, lineEditEpoch);
        QWidget::setTabOrder(lineEditEpoch, pushButtonEpochJ2000);
        QWidget::setTabOrder(pushButtonEpochJ2000, radioButtonTimeOfPeriapsis);
        QWidget::setTabOrder(radioButtonTimeOfPeriapsis, lineEditTimeOfPeriapsis);
        QWidget::setTabOrder(lineEditTimeOfPeriapsis, radioButtonMeanAnomaly);
        QWidget::setTabOrder(radioButtonMeanAnomaly, lineEditMeanAnomaly);
        QWidget::setTabOrder(lineEditMeanAnomaly, radioButtonMeanLongitude);
        QWidget::setTabOrder(radioButtonMeanLongitude, lineEditMeanLongitude);
        QWidget::setTabOrder(lineEditMeanLongitude, checkBoxOrbitVisualizationPeriod);
        QWidget::setTabOrder(checkBoxOrbitVisualizationPeriod, lineEditOrbitVisualizationPeriod);
        QWidget::setTabOrder(lineEditOrbitVisualizationPeriod, checkBoxObliquity);
        QWidget::setTabOrder(checkBoxObliquity, lineEditObliquity);
        QWidget::setTabOrder(lineEditObliquity, checkBoxEquatorAscendingNode);
        QWidget::setTabOrder(checkBoxEquatorAscendingNode, lineEditEquatorAscendingNode);
        QWidget::setTabOrder(lineEditEquatorAscendingNode, checkBoxNorthPoleRA);
        QWidget::setTabOrder(checkBoxNorthPoleRA, lineEditNorthPoleRA);
        QWidget::setTabOrder(lineEditNorthPoleRA, checkBoxNorthPoleDec);
        QWidget::setTabOrder(checkBoxNorthPoleDec, lineEditNorthPoleDec);
        QWidget::setTabOrder(lineEditNorthPoleDec, checkBoxRotationalPeriod);
        QWidget::setTabOrder(checkBoxRotationalPeriod, lineEditRotationalPeriod);
        QWidget::setTabOrder(lineEditRotationalPeriod, checkBoxPrecessionRate);
        QWidget::setTabOrder(checkBoxPrecessionRate, lineEditPrecessionRate);
        QWidget::setTabOrder(lineEditPrecessionRate, checkBoxRotationalEpoch);
        QWidget::setTabOrder(checkBoxRotationalEpoch, lineEditRotationalEpoch);
        QWidget::setTabOrder(lineEditRotationalEpoch, pushButtonRotationalEpochJ2000);
        QWidget::setTabOrder(pushButtonRotationalEpochJ2000, checkBoxOffset);
        QWidget::setTabOrder(checkBoxOffset, lineEditOffset);
        QWidget::setTabOrder(lineEditOffset, groupBoxRings);
        QWidget::setTabOrder(groupBoxRings, lineEditRingInnerSize);
        QWidget::setTabOrder(lineEditRingInnerSize, lineEditRingOuterSize);
        QWidget::setTabOrder(lineEditRingOuterSize, lineEditRingTexture);
        QWidget::setTabOrder(lineEditRingTexture, pushButtonSelectRingTexture);
        QWidget::setTabOrder(pushButtonSelectRingTexture, pushButtonSave);

        retranslateUi(manualImportWindow);

        QMetaObject::connectSlotsByName(manualImportWindow);
    } // setupUi

    void retranslateUi(QWidget *manualImportWindow)
    {
        groupBoxObjectType->setTitle(QCoreApplication::translate("manualImportWindow", "Object type", nullptr));
        radioButtonSolarSystemObject->setText(QCoreApplication::translate("manualImportWindow", "Solar System object", nullptr));
        radioButtonMinorPlanet->setText(QCoreApplication::translate("manualImportWindow", "Minor planet (asteroid)", nullptr));
        radioButtonComet->setText(QCoreApplication::translate("manualImportWindow", "Comet", nullptr));
        radioButtonSatellite->setText(QCoreApplication::translate("manualImportWindow", "Satellite", nullptr));
        groupBoxName->setTitle(QCoreApplication::translate("manualImportWindow", "Name", nullptr));
        labelName->setText(QCoreApplication::translate("manualImportWindow", "Name", nullptr));
        labelIdentifier->setText(QCoreApplication::translate("manualImportWindow", "Identifier", nullptr));
        checkBoxMinorPlanetNumber->setText(QCoreApplication::translate("manualImportWindow", "Minor planet number", nullptr));
        groupBoxProperties->setTitle(QCoreApplication::translate("manualImportWindow", "Properties", nullptr));
        labelParent->setText(QCoreApplication::translate("manualImportWindow", "Parent:", nullptr));
        labelRadius->setText(QCoreApplication::translate("manualImportWindow", "Radius:", nullptr));
        labelOblateness->setText(QCoreApplication::translate("manualImportWindow", "Oblateness:", nullptr));
        labelAlbedo->setText(QCoreApplication::translate("manualImportWindow", "Albedo:", nullptr));
        labelColor->setText(QCoreApplication::translate("manualImportWindow", "Color:", nullptr));
        checkBoxLighting->setText(QCoreApplication::translate("manualImportWindow", "Lighting (TODO: More clear name)", nullptr));
        checkBoxAtmosphere->setText(QCoreApplication::translate("manualImportWindow", "Atmosphere", nullptr));
        checkBoxHidden->setText(QCoreApplication::translate("manualImportWindow", "Hidden", nullptr));
        labelTexture->setText(QCoreApplication::translate("manualImportWindow", "Texture:", nullptr));
        pushButtonSelectTexture->setText(QCoreApplication::translate("manualImportWindow", "...", nullptr));
        labelAbsoluteMagnitude->setText(QCoreApplication::translate("manualImportWindow", "Absolute magnitude:", nullptr));
        labelSlopeParameter->setText(QCoreApplication::translate("manualImportWindow", "Slope parameter:", nullptr));
        pushButtonCalculateRadius->setText(QCoreApplication::translate("manualImportWindow", "Calculate the object's radius from the absolute magnitude and the current albedo", nullptr));
        labelMagnitudeSystem->setText(QCoreApplication::translate("manualImportWindow", "H-G two parameter magnitude system for minor planets:", nullptr));
        groupBoxOrbitalElements->setTitle(QCoreApplication::translate("manualImportWindow", "Orbital elements", nullptr));
        labelOrbitType->setText(QCoreApplication::translate("manualImportWindow", "Orbital function type:", nullptr));
        radioButtonCometaryOrbit->setText(QCoreApplication::translate("manualImportWindow", "Cometary orbit", nullptr));
        radioButtonEllipticOrbit->setText(QCoreApplication::translate("manualImportWindow", "Elliptic orbit", nullptr));
        labelEccentricity->setText(QCoreApplication::translate("manualImportWindow", "Eccentricity (%1):", nullptr));
        lineEditEccentricity->setText(QCoreApplication::translate("manualImportWindow", "0", nullptr));
        labelInclination->setText(QCoreApplication::translate("manualImportWindow", "Inclination (degrees):", nullptr));
        labelLongitudeOfTheAscendingNode->setText(QCoreApplication::translate("manualImportWindow", "Longitude of the ascending node %1:", nullptr));
        radioButtonArgumentOfPeriapsis->setText(QCoreApplication::translate("manualImportWindow", "Argument of the periapsis %1:", nullptr));
        radioButtonLongitudeOfPeriapsis->setText(QCoreApplication::translate("manualImportWindow", "Longitude of the periapsis %1:", nullptr));
        radioButtonSemiMajorAxis->setText(QCoreApplication::translate("manualImportWindow", "Semi-major axis %1:", nullptr));
        radioButtonPeriapsisDistance->setText(QCoreApplication::translate("manualImportWindow", "Periapsis distance %1:", nullptr));
        checkBoxPeriod->setText(QCoreApplication::translate("manualImportWindow", "Period:", nullptr));
        checkBoxMeanMotion->setText(QCoreApplication::translate("manualImportWindow", "Mean motion:", nullptr));
        radioButtonEpoch->setText(QCoreApplication::translate("manualImportWindow", "Epoch:", nullptr));
        radioButtonTimeOfPeriapsis->setText(QCoreApplication::translate("manualImportWindow", "Time of periapsis passage:", nullptr));
        pushButtonEpochJ2000->setText(QCoreApplication::translate("manualImportWindow", "J2000.0", nullptr));
        radioButtonMeanAnomaly->setText(QCoreApplication::translate("manualImportWindow", "Mean anomaly:", nullptr));
        radioButtonMeanLongitude->setText(QCoreApplication::translate("manualImportWindow", "Mean longitude:", nullptr));
        radioButtonOrbitFunction->setText(QCoreApplication::translate("manualImportWindow", "Object-specific:", nullptr));
        checkBoxOrbitVisualizationPeriod->setText(QCoreApplication::translate("manualImportWindow", "Orbit visualization period:", nullptr));
        groupBoxRotationalElements->setTitle(QCoreApplication::translate("manualImportWindow", "Rotational elements", nullptr));
        checkBoxObliquity->setText(QCoreApplication::translate("manualImportWindow", "Obliquity:", nullptr));
        checkBoxNorthPoleRA->setText(QCoreApplication::translate("manualImportWindow", "North pole right ascension:", nullptr));
        checkBoxNorthPoleDec->setText(QCoreApplication::translate("manualImportWindow", "North pole declination:", nullptr));
        checkBoxRotationalPeriod->setText(QCoreApplication::translate("manualImportWindow", "Period (hours):", nullptr));
        checkBoxPrecessionRate->setText(QCoreApplication::translate("manualImportWindow", "Precession rate:", nullptr));
        checkBoxRotationalEpoch->setText(QCoreApplication::translate("manualImportWindow", "Epoch:", nullptr));
        checkBoxOffset->setText(QCoreApplication::translate("manualImportWindow", "Offset:", nullptr));
        checkBoxEquatorAscendingNode->setText(QCoreApplication::translate("manualImportWindow", "Longitude of the ascending node of the ecliptic on the equator:", nullptr));
        pushButtonRotationalEpochJ2000->setText(QCoreApplication::translate("manualImportWindow", "J2000.0", nullptr));
        groupBoxRings->setTitle(QCoreApplication::translate("manualImportWindow", "Rings", nullptr));
        labelInnerSize->setText(QCoreApplication::translate("manualImportWindow", "Inner size:", nullptr));
        labelOuterSize->setText(QCoreApplication::translate("manualImportWindow", "Outer size:", nullptr));
        labelRingTexture->setText(QCoreApplication::translate("manualImportWindow", "Texture:", nullptr));
        pushButtonSelectRingTexture->setText(QCoreApplication::translate("manualImportWindow", "...", nullptr));
        labelErrorMessage->setText(QString());
        pushButtonSave->setText(QCoreApplication::translate("manualImportWindow", "Save", nullptr));
        (void)manualImportWindow;
    } // retranslateUi

};

namespace Ui {
    class manualImportWindow: public Ui_manualImportWindow {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_MANUALIMPORTWINDOW_H
