/********************************************************************************
** Form generated from reading UI file 'scmHideOrAbortMakerDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SCMHIDEORABORTMAKERDIALOG_H
#define UI_SCMHIDEORABORTMAKERDIALOG_H

#include <Dialog.hpp>
#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_scmHideOrAbortMakerDialog
{
public:
    QVBoxLayout *vboxLayout;
    TitleBar *titleBar;
    QFrame *Content;
    QGridLayout *gridLayout_2;
    QFrame *frame;
    QGridLayout *gridLayout_3;
    QVBoxLayout *verticalLayout;
    QLabel *questionLbl;
    QHBoxLayout *horizontalLayout;
    QPushButton *scmMakerHideButton;
    QPushButton *scmMakerAbortButton;
    QPushButton *scmMakerCancelButton;

    void setupUi(QWidget *scmHideOrAbortMakerDialog)
    {
        if (scmHideOrAbortMakerDialog->objectName().isEmpty())
            scmHideOrAbortMakerDialog->setObjectName("scmHideOrAbortMakerDialog");
        scmHideOrAbortMakerDialog->setStyleSheet(QString::fromUtf8(""));
        vboxLayout = new QVBoxLayout(scmHideOrAbortMakerDialog);
        vboxLayout->setSpacing(0);
        vboxLayout->setObjectName("vboxLayout");
        vboxLayout->setContentsMargins(0, 0, 0, 0);
        titleBar = new TitleBar(scmHideOrAbortMakerDialog);
        titleBar->setObjectName("titleBar");

        vboxLayout->addWidget(titleBar);

        Content = new QFrame(scmHideOrAbortMakerDialog);
        Content->setObjectName("Content");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Minimum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(Content->sizePolicy().hasHeightForWidth());
        Content->setSizePolicy(sizePolicy);
        Content->setFrameShape(QFrame::NoFrame);
        gridLayout_2 = new QGridLayout(Content);
        gridLayout_2->setSpacing(0);
        gridLayout_2->setObjectName("gridLayout_2");
        gridLayout_2->setContentsMargins(0, 0, 0, 0);
        frame = new QFrame(Content);
        frame->setObjectName("frame");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::MinimumExpanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(frame->sizePolicy().hasHeightForWidth());
        frame->setSizePolicy(sizePolicy1);
        frame->setFrameShape(QFrame::NoFrame);
        gridLayout_3 = new QGridLayout(frame);
        gridLayout_3->setSpacing(0);
        gridLayout_3->setObjectName("gridLayout_3");
        gridLayout_3->setContentsMargins(0, 0, 0, 0);
        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName("verticalLayout");
        questionLbl = new QLabel(frame);
        questionLbl->setObjectName("questionLbl");
        questionLbl->setAlignment(Qt::AlignCenter);
        questionLbl->setMargin(20);

        verticalLayout->addWidget(questionLbl);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(5);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(5, 5, 5, 5);
        scmMakerHideButton = new QPushButton(frame);
        scmMakerHideButton->setObjectName("scmMakerHideButton");

        horizontalLayout->addWidget(scmMakerHideButton);

        scmMakerAbortButton = new QPushButton(frame);
        scmMakerAbortButton->setObjectName("scmMakerAbortButton");

        horizontalLayout->addWidget(scmMakerAbortButton);

        scmMakerCancelButton = new QPushButton(frame);
        scmMakerCancelButton->setObjectName("scmMakerCancelButton");

        horizontalLayout->addWidget(scmMakerCancelButton);


        verticalLayout->addLayout(horizontalLayout);


        gridLayout_3->addLayout(verticalLayout, 0, 0, 1, 1);


        gridLayout_2->addWidget(frame, 0, 0, 1, 2);


        vboxLayout->addWidget(Content);


        retranslateUi(scmHideOrAbortMakerDialog);

        QMetaObject::connectSlotsByName(scmHideOrAbortMakerDialog);
    } // setupUi

    void retranslateUi(QWidget *scmHideOrAbortMakerDialog)
    {
        titleBar->setProperty("title", QVariant(QCoreApplication::translate("scmHideOrAbortMakerDialog", "Sky Culture Maker", nullptr)));
        questionLbl->setText(QCoreApplication::translate("scmHideOrAbortMakerDialog", "Hide the plugin to resume later or close it without saving the sky culture?", nullptr));
        scmMakerHideButton->setText(QCoreApplication::translate("scmHideOrAbortMakerDialog", "Hide editor", nullptr));
        scmMakerAbortButton->setText(QCoreApplication::translate("scmHideOrAbortMakerDialog", "Close without saving", nullptr));
        scmMakerCancelButton->setText(QCoreApplication::translate("scmHideOrAbortMakerDialog", "Don't close", nullptr));
        (void)scmHideOrAbortMakerDialog;
    } // retranslateUi

};

namespace Ui {
    class scmHideOrAbortMakerDialog: public Ui_scmHideOrAbortMakerDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SCMHIDEORABORTMAKERDIALOG_H
