/****************************************************************************
** Meta object code from reading C++ file 'StelPropertyMgr.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../stellarium/src/core/StelPropertyMgr.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'StelPropertyMgr.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN17StelPropertyProxyE_t {};
} // unnamed namespace

template <> constexpr inline auto StelPropertyProxy::qt_create_metaobjectdata<qt_meta_tag_ZN17StelPropertyProxyE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "StelPropertyProxy",
        "onPropertyChanged",
        "",
        "QVariant",
        "value"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'onPropertyChanged'
        QtMocHelpers::SlotData<void(const QVariant &)>(1, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<StelPropertyProxy, qt_meta_tag_ZN17StelPropertyProxyE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject StelPropertyProxy::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17StelPropertyProxyE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17StelPropertyProxyE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17StelPropertyProxyE_t>.metaTypes,
    nullptr
} };

void StelPropertyProxy::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<StelPropertyProxy *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->onPropertyChanged((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *StelPropertyProxy::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *StelPropertyProxy::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17StelPropertyProxyE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int StelPropertyProxy::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN20StelPropertyIntProxyE_t {};
} // unnamed namespace

template <> constexpr inline auto StelPropertyIntProxy::qt_create_metaobjectdata<qt_meta_tag_ZN20StelPropertyIntProxyE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "StelPropertyIntProxy",
        "propertyChanged",
        "",
        "value",
        "onPropertyChanged",
        "QVariant"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'propertyChanged'
        QtMocHelpers::SignalData<void(int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Slot 'onPropertyChanged'
        QtMocHelpers::SlotData<void(const QVariant &)>(4, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 5, 3 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<StelPropertyIntProxy, qt_meta_tag_ZN20StelPropertyIntProxyE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject StelPropertyIntProxy::staticMetaObject = { {
    QMetaObject::SuperData::link<StelPropertyProxy::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20StelPropertyIntProxyE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20StelPropertyIntProxyE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN20StelPropertyIntProxyE_t>.metaTypes,
    nullptr
} };

void StelPropertyIntProxy::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<StelPropertyIntProxy *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->propertyChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->onPropertyChanged((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (StelPropertyIntProxy::*)(int )>(_a, &StelPropertyIntProxy::propertyChanged, 0))
            return;
    }
}

const QMetaObject *StelPropertyIntProxy::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *StelPropertyIntProxy::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20StelPropertyIntProxyE_t>.strings))
        return static_cast<void*>(this);
    return StelPropertyProxy::qt_metacast(_clname);
}

int StelPropertyIntProxy::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StelPropertyProxy::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void StelPropertyIntProxy::propertyChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN21StelPropertyBoolProxyE_t {};
} // unnamed namespace

template <> constexpr inline auto StelPropertyBoolProxy::qt_create_metaobjectdata<qt_meta_tag_ZN21StelPropertyBoolProxyE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "StelPropertyBoolProxy",
        "propertyChanged",
        "",
        "value",
        "onPropertyChanged",
        "QVariant"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'propertyChanged'
        QtMocHelpers::SignalData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'onPropertyChanged'
        QtMocHelpers::SlotData<void(const QVariant &)>(4, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 5, 3 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<StelPropertyBoolProxy, qt_meta_tag_ZN21StelPropertyBoolProxyE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject StelPropertyBoolProxy::staticMetaObject = { {
    QMetaObject::SuperData::link<StelPropertyProxy::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21StelPropertyBoolProxyE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21StelPropertyBoolProxyE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN21StelPropertyBoolProxyE_t>.metaTypes,
    nullptr
} };

void StelPropertyBoolProxy::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<StelPropertyBoolProxy *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->propertyChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->onPropertyChanged((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (StelPropertyBoolProxy::*)(bool )>(_a, &StelPropertyBoolProxy::propertyChanged, 0))
            return;
    }
}

const QMetaObject *StelPropertyBoolProxy::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *StelPropertyBoolProxy::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21StelPropertyBoolProxyE_t>.strings))
        return static_cast<void*>(this);
    return StelPropertyProxy::qt_metacast(_clname);
}

int StelPropertyBoolProxy::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StelPropertyProxy::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void StelPropertyBoolProxy::propertyChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN23StelPropertyDoubleProxyE_t {};
} // unnamed namespace

template <> constexpr inline auto StelPropertyDoubleProxy::qt_create_metaobjectdata<qt_meta_tag_ZN23StelPropertyDoubleProxyE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "StelPropertyDoubleProxy",
        "propertyChanged",
        "",
        "value",
        "onPropertyChanged",
        "QVariant"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'propertyChanged'
        QtMocHelpers::SignalData<void(double)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'onPropertyChanged'
        QtMocHelpers::SlotData<void(const QVariant &)>(4, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 5, 3 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<StelPropertyDoubleProxy, qt_meta_tag_ZN23StelPropertyDoubleProxyE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject StelPropertyDoubleProxy::staticMetaObject = { {
    QMetaObject::SuperData::link<StelPropertyProxy::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23StelPropertyDoubleProxyE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23StelPropertyDoubleProxyE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN23StelPropertyDoubleProxyE_t>.metaTypes,
    nullptr
} };

void StelPropertyDoubleProxy::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<StelPropertyDoubleProxy *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->propertyChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 1: _t->onPropertyChanged((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (StelPropertyDoubleProxy::*)(double )>(_a, &StelPropertyDoubleProxy::propertyChanged, 0))
            return;
    }
}

const QMetaObject *StelPropertyDoubleProxy::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *StelPropertyDoubleProxy::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23StelPropertyDoubleProxyE_t>.strings))
        return static_cast<void*>(this);
    return StelPropertyProxy::qt_metacast(_clname);
}

int StelPropertyDoubleProxy::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StelPropertyProxy::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void StelPropertyDoubleProxy::propertyChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN23StelPropertyStringProxyE_t {};
} // unnamed namespace

template <> constexpr inline auto StelPropertyStringProxy::qt_create_metaobjectdata<qt_meta_tag_ZN23StelPropertyStringProxyE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "StelPropertyStringProxy",
        "propertyChanged",
        "",
        "value",
        "onPropertyChanged",
        "QVariant"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'propertyChanged'
        QtMocHelpers::SignalData<void(QString)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'onPropertyChanged'
        QtMocHelpers::SlotData<void(const QVariant &)>(4, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 5, 3 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<StelPropertyStringProxy, qt_meta_tag_ZN23StelPropertyStringProxyE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject StelPropertyStringProxy::staticMetaObject = { {
    QMetaObject::SuperData::link<StelPropertyProxy::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23StelPropertyStringProxyE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23StelPropertyStringProxyE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN23StelPropertyStringProxyE_t>.metaTypes,
    nullptr
} };

void StelPropertyStringProxy::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<StelPropertyStringProxy *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->propertyChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->onPropertyChanged((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (StelPropertyStringProxy::*)(QString )>(_a, &StelPropertyStringProxy::propertyChanged, 0))
            return;
    }
}

const QMetaObject *StelPropertyStringProxy::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *StelPropertyStringProxy::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23StelPropertyStringProxyE_t>.strings))
        return static_cast<void*>(this);
    return StelPropertyProxy::qt_metacast(_clname);
}

int StelPropertyStringProxy::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StelPropertyProxy::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void StelPropertyStringProxy::propertyChanged(QString _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN12StelPropertyE_t {};
} // unnamed namespace

template <> constexpr inline auto StelProperty::qt_create_metaobjectdata<qt_meta_tag_ZN12StelPropertyE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "StelProperty",
        "changed",
        "",
        "QVariant",
        "newValue",
        "getId",
        "getValue",
        "setValue",
        "value",
        "isReadOnly",
        "isSynchronizable",
        "canNotify",
        "getType",
        "QMetaType::Type",
        "getMetaProp",
        "QMetaProperty",
        "getTarget",
        "propertyChanged"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'changed'
        QtMocHelpers::SignalData<void(const QVariant &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'getId'
        QtMocHelpers::SlotData<QString() const>(5, 2, QMC::AccessPublic, QMetaType::QString),
        // Slot 'getValue'
        QtMocHelpers::SlotData<QVariant() const>(6, 2, QMC::AccessPublic, 0x80000000 | 3),
        // Slot 'setValue'
        QtMocHelpers::SlotData<bool(const QVariant &) const>(7, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 3, 8 },
        }}),
        // Slot 'isReadOnly'
        QtMocHelpers::SlotData<bool() const>(9, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'isSynchronizable'
        QtMocHelpers::SlotData<bool() const>(10, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'canNotify'
        QtMocHelpers::SlotData<bool() const>(11, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'getType'
        QtMocHelpers::SlotData<QMetaType::Type() const>(12, 2, QMC::AccessPublic, 0x80000000 | 13),
        // Slot 'getMetaProp'
        QtMocHelpers::SlotData<QMetaProperty() const>(14, 2, QMC::AccessPublic, 0x80000000 | 15),
        // Slot 'getTarget'
        QtMocHelpers::SlotData<QObject *() const>(16, 2, QMC::AccessPublic, QMetaType::QObjectStar),
        // Slot 'propertyChanged'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<StelProperty, qt_meta_tag_ZN12StelPropertyE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject StelProperty::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12StelPropertyE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12StelPropertyE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12StelPropertyE_t>.metaTypes,
    nullptr
} };

void StelProperty::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<StelProperty *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->changed((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1]))); break;
        case 1: { QString _r = _t->getId();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 2: { QVariant _r = _t->getValue();
            if (_a[0]) *reinterpret_cast<QVariant*>(_a[0]) = std::move(_r); }  break;
        case 3: { bool _r = _t->setValue((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 4: { bool _r = _t->isReadOnly();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 5: { bool _r = _t->isSynchronizable();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 6: { bool _r = _t->canNotify();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 7: { QMetaType::Type _r = _t->getType();
            if (_a[0]) *reinterpret_cast<QMetaType::Type*>(_a[0]) = std::move(_r); }  break;
        case 8: { QMetaProperty _r = _t->getMetaProp();
            if (_a[0]) *reinterpret_cast<QMetaProperty*>(_a[0]) = std::move(_r); }  break;
        case 9: { QObject* _r = _t->getTarget();
            if (_a[0]) *reinterpret_cast<QObject**>(_a[0]) = std::move(_r); }  break;
        case 10: _t->propertyChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (StelProperty::*)(const QVariant & )>(_a, &StelProperty::changed, 0))
            return;
    }
}

const QMetaObject *StelProperty::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *StelProperty::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12StelPropertyE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int StelProperty::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void StelProperty::changed(const QVariant & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN15StelPropertyMgrE_t {};
} // unnamed namespace

template <> constexpr inline auto StelPropertyMgr::qt_create_metaobjectdata<qt_meta_tag_ZN15StelPropertyMgrE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "StelPropertyMgr",
        "stelPropertyChanged",
        "",
        "StelProperty*",
        "prop",
        "QVariant",
        "value",
        "onStelPropChanged",
        "val"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'stelPropertyChanged'
        QtMocHelpers::SignalData<void(StelProperty *, const QVariant &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 5, 6 },
        }}),
        // Slot 'onStelPropChanged'
        QtMocHelpers::SlotData<void(const QVariant &)>(7, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 5, 8 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<StelPropertyMgr, qt_meta_tag_ZN15StelPropertyMgrE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject StelPropertyMgr::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15StelPropertyMgrE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15StelPropertyMgrE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15StelPropertyMgrE_t>.metaTypes,
    nullptr
} };

void StelPropertyMgr::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<StelPropertyMgr *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->stelPropertyChanged((*reinterpret_cast<std::add_pointer_t<StelProperty*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 1: _t->onStelPropChanged((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< StelProperty* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (StelPropertyMgr::*)(StelProperty * , const QVariant & )>(_a, &StelPropertyMgr::stelPropertyChanged, 0))
            return;
    }
}

const QMetaObject *StelPropertyMgr::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *StelPropertyMgr::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15StelPropertyMgrE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int StelPropertyMgr::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void StelPropertyMgr::stelPropertyChanged(StelProperty * _t1, const QVariant & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}
QT_WARNING_POP
