/****************************************************************************
** Meta object code from reading C++ file 'LabelMgr.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../stellarium/src/core/modules/LabelMgr.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'LabelMgr.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8LabelMgrE_t {};
} // unnamed namespace

template <> constexpr inline auto LabelMgr::qt_create_metaobjectdata<qt_meta_tag_ZN8LabelMgrE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "LabelMgr",
        "labelObject",
        "",
        "text",
        "objectName",
        "visible",
        "fontSize",
        "fontColor",
        "side",
        "labelDistance",
        "style",
        "autoDelete",
        "autoDeleteTimeoutMs",
        "Vec3f",
        "labelHorizon",
        "az",
        "alt",
        "labelEquatorial",
        "ra",
        "dec",
        "j2000epoch",
        "labelScreen",
        "x",
        "y",
        "getLabelShow",
        "id",
        "setLabelShow",
        "show",
        "setLabelText",
        "newText",
        "deleteLabel",
        "deleteAllLabels",
        "messageTimeout1",
        "messageTimeout2"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'labelObject'
        QtMocHelpers::SlotData<int(const QString &, const QString &, bool, float, const QString &, const QString &, double, const QString &, bool, int)>(1, 2, QMC::AccessPublic, QMetaType::Int, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 4 }, { QMetaType::Bool, 5 }, { QMetaType::Float, 6 },
            { QMetaType::QString, 7 }, { QMetaType::QString, 8 }, { QMetaType::Double, 9 }, { QMetaType::QString, 10 },
            { QMetaType::Bool, 11 }, { QMetaType::Int, 12 },
        }}),
        // Slot 'labelObject'
        QtMocHelpers::SlotData<int(const QString &, const QString &, bool, float, const QString &, const QString &, double, const QString &, bool)>(1, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 4 }, { QMetaType::Bool, 5 }, { QMetaType::Float, 6 },
            { QMetaType::QString, 7 }, { QMetaType::QString, 8 }, { QMetaType::Double, 9 }, { QMetaType::QString, 10 },
            { QMetaType::Bool, 11 },
        }}),
        // Slot 'labelObject'
        QtMocHelpers::SlotData<int(const QString &, const QString &, bool, float, const QString &, const QString &, double, const QString &)>(1, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 4 }, { QMetaType::Bool, 5 }, { QMetaType::Float, 6 },
            { QMetaType::QString, 7 }, { QMetaType::QString, 8 }, { QMetaType::Double, 9 }, { QMetaType::QString, 10 },
        }}),
        // Slot 'labelObject'
        QtMocHelpers::SlotData<int(const QString &, const QString &, bool, float, const QString &, const QString &, double)>(1, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 4 }, { QMetaType::Bool, 5 }, { QMetaType::Float, 6 },
            { QMetaType::QString, 7 }, { QMetaType::QString, 8 }, { QMetaType::Double, 9 },
        }}),
        // Slot 'labelObject'
        QtMocHelpers::SlotData<int(const QString &, const QString &, bool, float, const QString &, const QString &)>(1, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 4 }, { QMetaType::Bool, 5 }, { QMetaType::Float, 6 },
            { QMetaType::QString, 7 }, { QMetaType::QString, 8 },
        }}),
        // Slot 'labelObject'
        QtMocHelpers::SlotData<int(const QString &, const QString &, bool, float, const QString &)>(1, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 4 }, { QMetaType::Bool, 5 }, { QMetaType::Float, 6 },
            { QMetaType::QString, 7 },
        }}),
        // Slot 'labelObject'
        QtMocHelpers::SlotData<int(const QString &, const QString &, bool, float)>(1, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 4 }, { QMetaType::Bool, 5 }, { QMetaType::Float, 6 },
        }}),
        // Slot 'labelObject'
        QtMocHelpers::SlotData<int(const QString &, const QString &, bool)>(1, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 4 }, { QMetaType::Bool, 5 },
        }}),
        // Slot 'labelObject'
        QtMocHelpers::SlotData<int(const QString &, const QString &)>(1, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 4 },
        }}),
        // Slot 'labelObject'
        QtMocHelpers::SlotData<int(const QString &, const QString &, bool, float, const Vec3f &, const QString &, double, const QString &, bool, int)>(1, 2, QMC::AccessPublic, QMetaType::Int, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 4 }, { QMetaType::Bool, 5 }, { QMetaType::Float, 6 },
            { 0x80000000 | 13, 7 }, { QMetaType::QString, 8 }, { QMetaType::Double, 9 }, { QMetaType::QString, 10 },
            { QMetaType::Bool, 11 }, { QMetaType::Int, 12 },
        }}),
        // Slot 'labelObject'
        QtMocHelpers::SlotData<int(const QString &, const QString &, bool, float, const Vec3f &, const QString &, double, const QString &, bool)>(1, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 4 }, { QMetaType::Bool, 5 }, { QMetaType::Float, 6 },
            { 0x80000000 | 13, 7 }, { QMetaType::QString, 8 }, { QMetaType::Double, 9 }, { QMetaType::QString, 10 },
            { QMetaType::Bool, 11 },
        }}),
        // Slot 'labelObject'
        QtMocHelpers::SlotData<int(const QString &, const QString &, bool, float, const Vec3f &, const QString &, double, const QString &)>(1, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 4 }, { QMetaType::Bool, 5 }, { QMetaType::Float, 6 },
            { 0x80000000 | 13, 7 }, { QMetaType::QString, 8 }, { QMetaType::Double, 9 }, { QMetaType::QString, 10 },
        }}),
        // Slot 'labelObject'
        QtMocHelpers::SlotData<int(const QString &, const QString &, bool, float, const Vec3f &, const QString &, double)>(1, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 4 }, { QMetaType::Bool, 5 }, { QMetaType::Float, 6 },
            { 0x80000000 | 13, 7 }, { QMetaType::QString, 8 }, { QMetaType::Double, 9 },
        }}),
        // Slot 'labelObject'
        QtMocHelpers::SlotData<int(const QString &, const QString &, bool, float, const Vec3f &, const QString &)>(1, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 4 }, { QMetaType::Bool, 5 }, { QMetaType::Float, 6 },
            { 0x80000000 | 13, 7 }, { QMetaType::QString, 8 },
        }}),
        // Slot 'labelObject'
        QtMocHelpers::SlotData<int(const QString &, const QString &, bool, float, const Vec3f &)>(1, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 4 }, { QMetaType::Bool, 5 }, { QMetaType::Float, 6 },
            { 0x80000000 | 13, 7 },
        }}),
        // Slot 'labelHorizon'
        QtMocHelpers::SlotData<int(const QString &, float, float, bool, float, const QString &, bool, int)>(14, 2, QMC::AccessPublic, QMetaType::Int, {{
            { QMetaType::QString, 3 }, { QMetaType::Float, 15 }, { QMetaType::Float, 16 }, { QMetaType::Bool, 5 },
            { QMetaType::Float, 6 }, { QMetaType::QString, 7 }, { QMetaType::Bool, 11 }, { QMetaType::Int, 12 },
        }}),
        // Slot 'labelHorizon'
        QtMocHelpers::SlotData<int(const QString &, float, float, bool, float, const QString &, bool)>(14, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { QMetaType::QString, 3 }, { QMetaType::Float, 15 }, { QMetaType::Float, 16 }, { QMetaType::Bool, 5 },
            { QMetaType::Float, 6 }, { QMetaType::QString, 7 }, { QMetaType::Bool, 11 },
        }}),
        // Slot 'labelHorizon'
        QtMocHelpers::SlotData<int(const QString &, float, float, bool, float, const QString &)>(14, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { QMetaType::QString, 3 }, { QMetaType::Float, 15 }, { QMetaType::Float, 16 }, { QMetaType::Bool, 5 },
            { QMetaType::Float, 6 }, { QMetaType::QString, 7 },
        }}),
        // Slot 'labelHorizon'
        QtMocHelpers::SlotData<int(const QString &, float, float, bool, float)>(14, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { QMetaType::QString, 3 }, { QMetaType::Float, 15 }, { QMetaType::Float, 16 }, { QMetaType::Bool, 5 },
            { QMetaType::Float, 6 },
        }}),
        // Slot 'labelHorizon'
        QtMocHelpers::SlotData<int(const QString &, float, float, bool)>(14, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { QMetaType::QString, 3 }, { QMetaType::Float, 15 }, { QMetaType::Float, 16 }, { QMetaType::Bool, 5 },
        }}),
        // Slot 'labelHorizon'
        QtMocHelpers::SlotData<int(const QString &, float, float)>(14, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { QMetaType::QString, 3 }, { QMetaType::Float, 15 }, { QMetaType::Float, 16 },
        }}),
        // Slot 'labelHorizon'
        QtMocHelpers::SlotData<int(const QString &, float, float, bool, float, const Vec3f &, bool, int)>(14, 2, QMC::AccessPublic, QMetaType::Int, {{
            { QMetaType::QString, 3 }, { QMetaType::Float, 15 }, { QMetaType::Float, 16 }, { QMetaType::Bool, 5 },
            { QMetaType::Float, 6 }, { 0x80000000 | 13, 7 }, { QMetaType::Bool, 11 }, { QMetaType::Int, 12 },
        }}),
        // Slot 'labelHorizon'
        QtMocHelpers::SlotData<int(const QString &, float, float, bool, float, const Vec3f &, bool)>(14, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { QMetaType::QString, 3 }, { QMetaType::Float, 15 }, { QMetaType::Float, 16 }, { QMetaType::Bool, 5 },
            { QMetaType::Float, 6 }, { 0x80000000 | 13, 7 }, { QMetaType::Bool, 11 },
        }}),
        // Slot 'labelHorizon'
        QtMocHelpers::SlotData<int(const QString &, float, float, bool, float, const Vec3f &)>(14, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { QMetaType::QString, 3 }, { QMetaType::Float, 15 }, { QMetaType::Float, 16 }, { QMetaType::Bool, 5 },
            { QMetaType::Float, 6 }, { 0x80000000 | 13, 7 },
        }}),
        // Slot 'labelEquatorial'
        QtMocHelpers::SlotData<int(const QString &, const QString &, const QString &, bool, float, const QString &, const QString &, double, bool, int, bool)>(17, 2, QMC::AccessPublic, QMetaType::Int, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 18 }, { QMetaType::QString, 19 }, { QMetaType::Bool, 5 },
            { QMetaType::Float, 6 }, { QMetaType::QString, 7 }, { QMetaType::QString, 8 }, { QMetaType::Double, 9 },
            { QMetaType::Bool, 11 }, { QMetaType::Int, 12 }, { QMetaType::Bool, 20 },
        }}),
        // Slot 'labelEquatorial'
        QtMocHelpers::SlotData<int(const QString &, const QString &, const QString &, bool, float, const QString &, const QString &, double, bool, int)>(17, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 18 }, { QMetaType::QString, 19 }, { QMetaType::Bool, 5 },
            { QMetaType::Float, 6 }, { QMetaType::QString, 7 }, { QMetaType::QString, 8 }, { QMetaType::Double, 9 },
            { QMetaType::Bool, 11 }, { QMetaType::Int, 12 },
        }}),
        // Slot 'labelEquatorial'
        QtMocHelpers::SlotData<int(const QString &, const QString &, const QString &, bool, float, const QString &, const QString &, double, bool)>(17, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 18 }, { QMetaType::QString, 19 }, { QMetaType::Bool, 5 },
            { QMetaType::Float, 6 }, { QMetaType::QString, 7 }, { QMetaType::QString, 8 }, { QMetaType::Double, 9 },
            { QMetaType::Bool, 11 },
        }}),
        // Slot 'labelEquatorial'
        QtMocHelpers::SlotData<int(const QString &, const QString &, const QString &, bool, float, const QString &, const QString &, double)>(17, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 18 }, { QMetaType::QString, 19 }, { QMetaType::Bool, 5 },
            { QMetaType::Float, 6 }, { QMetaType::QString, 7 }, { QMetaType::QString, 8 }, { QMetaType::Double, 9 },
        }}),
        // Slot 'labelEquatorial'
        QtMocHelpers::SlotData<int(const QString &, const QString &, const QString &, bool, float, const QString &, const QString &)>(17, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 18 }, { QMetaType::QString, 19 }, { QMetaType::Bool, 5 },
            { QMetaType::Float, 6 }, { QMetaType::QString, 7 }, { QMetaType::QString, 8 },
        }}),
        // Slot 'labelEquatorial'
        QtMocHelpers::SlotData<int(const QString &, const QString &, const QString &, bool, float, const QString &)>(17, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 18 }, { QMetaType::QString, 19 }, { QMetaType::Bool, 5 },
            { QMetaType::Float, 6 }, { QMetaType::QString, 7 },
        }}),
        // Slot 'labelEquatorial'
        QtMocHelpers::SlotData<int(const QString &, const QString &, const QString &, bool, float)>(17, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 18 }, { QMetaType::QString, 19 }, { QMetaType::Bool, 5 },
            { QMetaType::Float, 6 },
        }}),
        // Slot 'labelEquatorial'
        QtMocHelpers::SlotData<int(const QString &, const QString &, const QString &, bool)>(17, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 18 }, { QMetaType::QString, 19 }, { QMetaType::Bool, 5 },
        }}),
        // Slot 'labelEquatorial'
        QtMocHelpers::SlotData<int(const QString &, const QString &, const QString &)>(17, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 18 }, { QMetaType::QString, 19 },
        }}),
        // Slot 'labelEquatorial'
        QtMocHelpers::SlotData<int(const QString &, const QString &, const QString &, bool, float, const Vec3f &, const QString &, double, bool, int, bool)>(17, 2, QMC::AccessPublic, QMetaType::Int, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 18 }, { QMetaType::QString, 19 }, { QMetaType::Bool, 5 },
            { QMetaType::Float, 6 }, { 0x80000000 | 13, 7 }, { QMetaType::QString, 8 }, { QMetaType::Double, 9 },
            { QMetaType::Bool, 11 }, { QMetaType::Int, 12 }, { QMetaType::Bool, 20 },
        }}),
        // Slot 'labelEquatorial'
        QtMocHelpers::SlotData<int(const QString &, const QString &, const QString &, bool, float, const Vec3f &, const QString &, double, bool, int)>(17, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 18 }, { QMetaType::QString, 19 }, { QMetaType::Bool, 5 },
            { QMetaType::Float, 6 }, { 0x80000000 | 13, 7 }, { QMetaType::QString, 8 }, { QMetaType::Double, 9 },
            { QMetaType::Bool, 11 }, { QMetaType::Int, 12 },
        }}),
        // Slot 'labelEquatorial'
        QtMocHelpers::SlotData<int(const QString &, const QString &, const QString &, bool, float, const Vec3f &, const QString &, double, bool)>(17, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 18 }, { QMetaType::QString, 19 }, { QMetaType::Bool, 5 },
            { QMetaType::Float, 6 }, { 0x80000000 | 13, 7 }, { QMetaType::QString, 8 }, { QMetaType::Double, 9 },
            { QMetaType::Bool, 11 },
        }}),
        // Slot 'labelEquatorial'
        QtMocHelpers::SlotData<int(const QString &, const QString &, const QString &, bool, float, const Vec3f &, const QString &, double)>(17, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 18 }, { QMetaType::QString, 19 }, { QMetaType::Bool, 5 },
            { QMetaType::Float, 6 }, { 0x80000000 | 13, 7 }, { QMetaType::QString, 8 }, { QMetaType::Double, 9 },
        }}),
        // Slot 'labelEquatorial'
        QtMocHelpers::SlotData<int(const QString &, const QString &, const QString &, bool, float, const Vec3f &, const QString &)>(17, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 18 }, { QMetaType::QString, 19 }, { QMetaType::Bool, 5 },
            { QMetaType::Float, 6 }, { 0x80000000 | 13, 7 }, { QMetaType::QString, 8 },
        }}),
        // Slot 'labelEquatorial'
        QtMocHelpers::SlotData<int(const QString &, const QString &, const QString &, bool, float, const Vec3f &)>(17, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 18 }, { QMetaType::QString, 19 }, { QMetaType::Bool, 5 },
            { QMetaType::Float, 6 }, { 0x80000000 | 13, 7 },
        }}),
        // Slot 'labelScreen'
        QtMocHelpers::SlotData<int(const QString &, int, int, bool, float, const QString &, bool, int)>(21, 2, QMC::AccessPublic, QMetaType::Int, {{
            { QMetaType::QString, 3 }, { QMetaType::Int, 22 }, { QMetaType::Int, 23 }, { QMetaType::Bool, 5 },
            { QMetaType::Float, 6 }, { QMetaType::QString, 7 }, { QMetaType::Bool, 11 }, { QMetaType::Int, 12 },
        }}),
        // Slot 'labelScreen'
        QtMocHelpers::SlotData<int(const QString &, int, int, bool, float, const QString &, bool)>(21, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { QMetaType::QString, 3 }, { QMetaType::Int, 22 }, { QMetaType::Int, 23 }, { QMetaType::Bool, 5 },
            { QMetaType::Float, 6 }, { QMetaType::QString, 7 }, { QMetaType::Bool, 11 },
        }}),
        // Slot 'labelScreen'
        QtMocHelpers::SlotData<int(const QString &, int, int, bool, float, const QString &)>(21, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { QMetaType::QString, 3 }, { QMetaType::Int, 22 }, { QMetaType::Int, 23 }, { QMetaType::Bool, 5 },
            { QMetaType::Float, 6 }, { QMetaType::QString, 7 },
        }}),
        // Slot 'labelScreen'
        QtMocHelpers::SlotData<int(const QString &, int, int, bool, float)>(21, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { QMetaType::QString, 3 }, { QMetaType::Int, 22 }, { QMetaType::Int, 23 }, { QMetaType::Bool, 5 },
            { QMetaType::Float, 6 },
        }}),
        // Slot 'labelScreen'
        QtMocHelpers::SlotData<int(const QString &, int, int, bool)>(21, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { QMetaType::QString, 3 }, { QMetaType::Int, 22 }, { QMetaType::Int, 23 }, { QMetaType::Bool, 5 },
        }}),
        // Slot 'labelScreen'
        QtMocHelpers::SlotData<int(const QString &, int, int)>(21, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { QMetaType::QString, 3 }, { QMetaType::Int, 22 }, { QMetaType::Int, 23 },
        }}),
        // Slot 'labelScreen'
        QtMocHelpers::SlotData<int(const QString &, int, int, bool, float, const Vec3f &, bool, int)>(21, 2, QMC::AccessPublic, QMetaType::Int, {{
            { QMetaType::QString, 3 }, { QMetaType::Int, 22 }, { QMetaType::Int, 23 }, { QMetaType::Bool, 5 },
            { QMetaType::Float, 6 }, { 0x80000000 | 13, 7 }, { QMetaType::Bool, 11 }, { QMetaType::Int, 12 },
        }}),
        // Slot 'labelScreen'
        QtMocHelpers::SlotData<int(const QString &, int, int, bool, float, const Vec3f &, bool)>(21, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { QMetaType::QString, 3 }, { QMetaType::Int, 22 }, { QMetaType::Int, 23 }, { QMetaType::Bool, 5 },
            { QMetaType::Float, 6 }, { 0x80000000 | 13, 7 }, { QMetaType::Bool, 11 },
        }}),
        // Slot 'labelScreen'
        QtMocHelpers::SlotData<int(const QString &, int, int, bool, float, const Vec3f &)>(21, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { QMetaType::QString, 3 }, { QMetaType::Int, 22 }, { QMetaType::Int, 23 }, { QMetaType::Bool, 5 },
            { QMetaType::Float, 6 }, { 0x80000000 | 13, 7 },
        }}),
        // Slot 'getLabelShow'
        QtMocHelpers::SlotData<bool(int) const>(24, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::Int, 25 },
        }}),
        // Slot 'setLabelShow'
        QtMocHelpers::SlotData<void(int, bool)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 25 }, { QMetaType::Bool, 27 },
        }}),
        // Slot 'setLabelText'
        QtMocHelpers::SlotData<void(int, const QString &)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 25 }, { QMetaType::QString, 29 },
        }}),
        // Slot 'deleteLabel'
        QtMocHelpers::SlotData<void(int)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 25 },
        }}),
        // Slot 'deleteAllLabels'
        QtMocHelpers::SlotData<int()>(31, 2, QMC::AccessPublic, QMetaType::Int),
        // Slot 'messageTimeout1'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'messageTimeout2'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<LabelMgr, qt_meta_tag_ZN8LabelMgrE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject LabelMgr::staticMetaObject = { {
    QMetaObject::SuperData::link<StelModule::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8LabelMgrE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8LabelMgrE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8LabelMgrE_t>.metaTypes,
    nullptr
} };

void LabelMgr::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<LabelMgr *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { int _r = _t->labelObject((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[8])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[9])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[10])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 1: { int _r = _t->labelObject((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[8])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[9])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 2: { int _r = _t->labelObject((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[8])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 3: { int _r = _t->labelObject((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[7])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 4: { int _r = _t->labelObject((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[6])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 5: { int _r = _t->labelObject((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 6: { int _r = _t->labelObject((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[4])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 7: { int _r = _t->labelObject((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 8: { int _r = _t->labelObject((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 9: { int _r = _t->labelObject((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[8])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[9])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[10])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 10: { int _r = _t->labelObject((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[8])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[9])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 11: { int _r = _t->labelObject((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[8])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 12: { int _r = _t->labelObject((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[7])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 13: { int _r = _t->labelObject((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[6])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 14: { int _r = _t->labelObject((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[5])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 15: { int _r = _t->labelHorizon((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[8])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 16: { int _r = _t->labelHorizon((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[7])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 17: { int _r = _t->labelHorizon((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[6])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 18: { int _r = _t->labelHorizon((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[5])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 19: { int _r = _t->labelHorizon((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 20: { int _r = _t->labelHorizon((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[3])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 21: { int _r = _t->labelHorizon((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[8])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 22: { int _r = _t->labelHorizon((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[7])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 23: { int _r = _t->labelHorizon((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[6])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 24: { int _r = _t->labelEquatorial((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[8])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[9])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[10])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[11])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 25: { int _r = _t->labelEquatorial((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[8])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[9])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[10])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 26: { int _r = _t->labelEquatorial((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[8])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[9])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 27: { int _r = _t->labelEquatorial((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[8])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 28: { int _r = _t->labelEquatorial((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[7])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 29: { int _r = _t->labelEquatorial((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[6])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 30: { int _r = _t->labelEquatorial((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[5])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 31: { int _r = _t->labelEquatorial((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 32: { int _r = _t->labelEquatorial((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 33: { int _r = _t->labelEquatorial((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[8])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[9])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[10])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[11])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 34: { int _r = _t->labelEquatorial((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[8])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[9])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[10])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 35: { int _r = _t->labelEquatorial((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[8])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[9])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 36: { int _r = _t->labelEquatorial((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[8])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 37: { int _r = _t->labelEquatorial((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[7])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 38: { int _r = _t->labelEquatorial((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[6])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 39: { int _r = _t->labelScreen((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[8])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 40: { int _r = _t->labelScreen((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[7])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 41: { int _r = _t->labelScreen((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[6])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 42: { int _r = _t->labelScreen((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[5])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 43: { int _r = _t->labelScreen((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 44: { int _r = _t->labelScreen((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 45: { int _r = _t->labelScreen((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[8])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 46: { int _r = _t->labelScreen((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[7])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 47: { int _r = _t->labelScreen((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<Vec3f>>(_a[6])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 48: { bool _r = _t->getLabelShow((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 49: _t->setLabelShow((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 50: _t->setLabelText((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 51: _t->deleteLabel((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 52: { int _r = _t->deleteAllLabels();
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 53: _t->messageTimeout1(); break;
        case 54: _t->messageTimeout2(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 4:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 4:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 4:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 4:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 13:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 4:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 14:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 4:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 21:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 5:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 22:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 5:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 23:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 5:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 33:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 5:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 34:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 5:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 35:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 5:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 36:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 5:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 37:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 5:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 38:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 5:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 45:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 5:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 46:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 5:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        case 47:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 5:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec3f >(); break;
            }
            break;
        }
    }
}

const QMetaObject *LabelMgr::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *LabelMgr::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8LabelMgrE_t>.strings))
        return static_cast<void*>(this);
    return StelModule::qt_metacast(_clname);
}

int LabelMgr::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StelModule::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 55)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 55;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 55)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 55;
    }
    return _id;
}
QT_WARNING_POP
