/********************************************************************************
** Form generated from reading UI file 'astroCalcExtraEphemerisDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_ASTROCALCEXTRAEPHEMERISDIALOG_H
#define UI_ASTROCALCEXTRAEPHEMERISDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "Dialog.hpp"
#include "gui/StelCloseButton.hpp"

QT_BEGIN_NAMESPACE

class Ui_astroCalcExtraEphemerisDialogForm
{
public:
    QVBoxLayout *verticalLayout;
    TitleBar *titleBar;
    QGroupBox *extraEphemerisGroupBox;
    QVBoxLayout *verticalLayout_3;
    QHBoxLayout *horizontalLayout;
    QCheckBox *skipDataCheckBox;
    QSpacerItem *horizontalSpacer;
    QSpinBox *dataStepSpinBox;
    QHBoxLayout *horizontalLayout_3;
    QSpacerItem *horizontalSpacer_3;
    QCheckBox *skipMarkersCheckBox;
    QCheckBox *smartDatesCheckBox;
    QCheckBox *scaleMarkersCheckBox;
    QCheckBox *currentLocationCheckBox;
    QCheckBox *alwaysOnCheckBox;
    QHBoxLayout *horizontalLayout_2;
    QLabel *label;
    QSpacerItem *horizontalSpacer_2;
    QSpinBox *lineThicknessSpinBox;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *astroCalcExtraEphemerisDialogForm)
    {
        if (astroCalcExtraEphemerisDialogForm->objectName().isEmpty())
            astroCalcExtraEphemerisDialogForm->setObjectName("astroCalcExtraEphemerisDialogForm");
        astroCalcExtraEphemerisDialogForm->resize(305, 209);
        verticalLayout = new QVBoxLayout(astroCalcExtraEphemerisDialogForm);
        verticalLayout->setSpacing(0);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        titleBar = new TitleBar(astroCalcExtraEphemerisDialogForm);
        titleBar->setObjectName("titleBar");

        verticalLayout->addWidget(titleBar);

        extraEphemerisGroupBox = new QGroupBox(astroCalcExtraEphemerisDialogForm);
        extraEphemerisGroupBox->setObjectName("extraEphemerisGroupBox");
        extraEphemerisGroupBox->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);
        verticalLayout_3 = new QVBoxLayout(extraEphemerisGroupBox);
        verticalLayout_3->setObjectName("verticalLayout_3");
        verticalLayout_3->setContentsMargins(0, 0, 0, 0);
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        skipDataCheckBox = new QCheckBox(extraEphemerisGroupBox);
        skipDataCheckBox->setObjectName("skipDataCheckBox");

        horizontalLayout->addWidget(skipDataCheckBox);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        dataStepSpinBox = new QSpinBox(extraEphemerisGroupBox);
        dataStepSpinBox->setObjectName("dataStepSpinBox");
        dataStepSpinBox->setMaximumSize(QSize(65, 16777215));
        dataStepSpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        dataStepSpinBox->setMinimum(1);

        horizontalLayout->addWidget(dataStepSpinBox);


        verticalLayout_3->addLayout(horizontalLayout);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        horizontalSpacer_3 = new QSpacerItem(20, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer_3);

        skipMarkersCheckBox = new QCheckBox(extraEphemerisGroupBox);
        skipMarkersCheckBox->setObjectName("skipMarkersCheckBox");

        horizontalLayout_3->addWidget(skipMarkersCheckBox);


        verticalLayout_3->addLayout(horizontalLayout_3);

        smartDatesCheckBox = new QCheckBox(extraEphemerisGroupBox);
        smartDatesCheckBox->setObjectName("smartDatesCheckBox");

        verticalLayout_3->addWidget(smartDatesCheckBox);

        scaleMarkersCheckBox = new QCheckBox(extraEphemerisGroupBox);
        scaleMarkersCheckBox->setObjectName("scaleMarkersCheckBox");

        verticalLayout_3->addWidget(scaleMarkersCheckBox);

        currentLocationCheckBox = new QCheckBox(extraEphemerisGroupBox);
        currentLocationCheckBox->setObjectName("currentLocationCheckBox");

        verticalLayout_3->addWidget(currentLocationCheckBox);

        alwaysOnCheckBox = new QCheckBox(extraEphemerisGroupBox);
        alwaysOnCheckBox->setObjectName("alwaysOnCheckBox");

        verticalLayout_3->addWidget(alwaysOnCheckBox);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        label = new QLabel(extraEphemerisGroupBox);
        label->setObjectName("label");

        horizontalLayout_2->addWidget(label);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_2);

        lineThicknessSpinBox = new QSpinBox(extraEphemerisGroupBox);
        lineThicknessSpinBox->setObjectName("lineThicknessSpinBox");
        lineThicknessSpinBox->setMaximumSize(QSize(65, 16777215));
        lineThicknessSpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        lineThicknessSpinBox->setMinimum(1);
        lineThicknessSpinBox->setMaximum(5);

        horizontalLayout_2->addWidget(lineThicknessSpinBox);


        verticalLayout_3->addLayout(horizontalLayout_2);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_3->addItem(verticalSpacer);


        verticalLayout->addWidget(extraEphemerisGroupBox);


        retranslateUi(astroCalcExtraEphemerisDialogForm);

        QMetaObject::connectSlotsByName(astroCalcExtraEphemerisDialogForm);
    } // setupUi

    void retranslateUi(QWidget *astroCalcExtraEphemerisDialogForm)
    {
        titleBar->setTitle(QCoreApplication::translate("astroCalcExtraEphemerisDialogForm", "Extra options", nullptr));
        extraEphemerisGroupBox->setTitle(QCoreApplication::translate("astroCalcExtraEphemerisDialogForm", "Extra options for ephemeris visualization", nullptr));
        skipDataCheckBox->setText(QCoreApplication::translate("astroCalcExtraEphemerisDialogForm", "Label each N steps, where N is", nullptr));
        skipMarkersCheckBox->setText(QCoreApplication::translate("astroCalcExtraEphemerisDialogForm", "Skip draw the unlabelled markers also", nullptr));
        smartDatesCheckBox->setText(QCoreApplication::translate("astroCalcExtraEphemerisDialogForm", "Use smart format for dates", nullptr));
        scaleMarkersCheckBox->setText(QCoreApplication::translate("astroCalcExtraEphemerisDialogForm", "Use small markers, when line is enabled", nullptr));
        currentLocationCheckBox->setText(QCoreApplication::translate("astroCalcExtraEphemerisDialogForm", "Show marker for current location", nullptr));
        alwaysOnCheckBox->setText(QCoreApplication::translate("astroCalcExtraEphemerisDialogForm", "Lines and markers always on", nullptr));
        label->setText(QCoreApplication::translate("astroCalcExtraEphemerisDialogForm", "Ephemeris line thickness:", nullptr));
        lineThicknessSpinBox->setSuffix(QCoreApplication::translate("astroCalcExtraEphemerisDialogForm", "px", "pixels"));
        (void)astroCalcExtraEphemerisDialogForm;
    } // retranslateUi

};

namespace Ui {
    class astroCalcExtraEphemerisDialogForm: public Ui_astroCalcExtraEphemerisDialogForm {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_ASTROCALCEXTRAEPHEMERISDIALOG_H
