/********************************************************************************
** Form generated from reading UI file 'tonemappingDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TONEMAPPINGDIALOG_H
#define UI_TONEMAPPINGDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "Dialog.hpp"
#include "gui/StelCloseButton.hpp"

QT_BEGIN_NAMESPACE

class Ui_TonemappingDialogForm
{
public:
    QVBoxLayout *verticalLayout;
    TitleBar *titleBar;
    QGroupBox *tonemappingGroupBox;
    QGridLayout *gridLayout;
    QLabel *dmLabel;
    QPushButton *resetPushButton;
    QSpacerItem *verticalSpacer;
    QDoubleSpinBox *dalSpinBox;
    QDoubleSpinBox *gammaSpinBox;
    QLabel *dalLabel;
    QDoubleSpinBox *dmSpinBox;
    QCheckBox *checkBox_extraGamma;
    QLabel *gammaLabel;
    QCheckBox *checkBox_sRGB;

    void setupUi(QWidget *TonemappingDialogForm)
    {
        if (TonemappingDialogForm->objectName().isEmpty())
            TonemappingDialogForm->setObjectName("TonemappingDialogForm");
        TonemappingDialogForm->resize(238, 188);
        verticalLayout = new QVBoxLayout(TonemappingDialogForm);
        verticalLayout->setSpacing(0);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        titleBar = new TitleBar(TonemappingDialogForm);
        titleBar->setObjectName("titleBar");

        verticalLayout->addWidget(titleBar);

        tonemappingGroupBox = new QGroupBox(TonemappingDialogForm);
        tonemappingGroupBox->setObjectName("tonemappingGroupBox");
        tonemappingGroupBox->setEnabled(true);
        gridLayout = new QGridLayout(tonemappingGroupBox);
        gridLayout->setObjectName("gridLayout");
        dmLabel = new QLabel(tonemappingGroupBox);
        dmLabel->setObjectName("dmLabel");

        gridLayout->addWidget(dmLabel, 0, 0, 1, 1);

        resetPushButton = new QPushButton(tonemappingGroupBox);
        resetPushButton->setObjectName("resetPushButton");
        resetPushButton->setFlat(false);

        gridLayout->addWidget(resetPushButton, 4, 0, 1, 2);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(verticalSpacer, 5, 0, 1, 1);

        dalSpinBox = new QDoubleSpinBox(tonemappingGroupBox);
        dalSpinBox->setObjectName("dalSpinBox");
        dalSpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        dalSpinBox->setDecimals(1);
        dalSpinBox->setMinimum(0.000000000000000);
        dalSpinBox->setMaximum(2000.000000000000000);
        dalSpinBox->setSingleStep(1.000000000000000);
        dalSpinBox->setValue(50.000000000000000);

        gridLayout->addWidget(dalSpinBox, 1, 1, 1, 1);

        gammaSpinBox = new QDoubleSpinBox(tonemappingGroupBox);
        gammaSpinBox->setObjectName("gammaSpinBox");
        gammaSpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        gammaSpinBox->setDecimals(4);
        gammaSpinBox->setMaximum(5.000000000000000);
        gammaSpinBox->setSingleStep(0.001000000000000);
        gammaSpinBox->setValue(2.222200000000000);

        gridLayout->addWidget(gammaSpinBox, 2, 1, 1, 1);

        dalLabel = new QLabel(tonemappingGroupBox);
        dalLabel->setObjectName("dalLabel");
        dalLabel->setInputMethodHints(Qt::ImhNone);
        dalLabel->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);

        gridLayout->addWidget(dalLabel, 1, 0, 1, 1);

        dmSpinBox = new QDoubleSpinBox(tonemappingGroupBox);
        dmSpinBox->setObjectName("dmSpinBox");
        dmSpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        dmSpinBox->setDecimals(1);
        dmSpinBox->setMaximum(2500.000000000000000);
        dmSpinBox->setValue(100.000000000000000);

        gridLayout->addWidget(dmSpinBox, 0, 1, 1, 1);

        checkBox_extraGamma = new QCheckBox(tonemappingGroupBox);
        checkBox_extraGamma->setObjectName("checkBox_extraGamma");

        gridLayout->addWidget(checkBox_extraGamma, 3, 0, 1, 1);

        gammaLabel = new QLabel(tonemappingGroupBox);
        gammaLabel->setObjectName("gammaLabel");

        gridLayout->addWidget(gammaLabel, 2, 0, 1, 1);

        checkBox_sRGB = new QCheckBox(tonemappingGroupBox);
        checkBox_sRGB->setObjectName("checkBox_sRGB");

        gridLayout->addWidget(checkBox_sRGB, 3, 1, 1, 1);


        verticalLayout->addWidget(tonemappingGroupBox);

        QWidget::setTabOrder(dalSpinBox, resetPushButton);

        retranslateUi(TonemappingDialogForm);

        QMetaObject::connectSlotsByName(TonemappingDialogForm);
    } // setupUi

    void retranslateUi(QWidget *TonemappingDialogForm)
    {
        titleBar->setTitle(QCoreApplication::translate("TonemappingDialogForm", "Tone Mapping Details", nullptr));
        tonemappingGroupBox->setTitle(QCoreApplication::translate("TonemappingDialogForm", "Custom settings for Tone Mapping", nullptr));
#if QT_CONFIG(tooltip)
        dmLabel->setToolTip(QCoreApplication::translate("TonemappingDialogForm", "Maximum luminance of the display (see display data sheet). Default 100cd/m\302\262", nullptr));
#endif // QT_CONFIG(tooltip)
        dmLabel->setText(QCoreApplication::translate("TonemappingDialogForm", "Display max luminance:", nullptr));
#if QT_CONFIG(tooltip)
        resetPushButton->setToolTip(QCoreApplication::translate("TonemappingDialogForm", "Reset to useful defaults", nullptr));
#endif // QT_CONFIG(tooltip)
        resetPushButton->setText(QCoreApplication::translate("TonemappingDialogForm", "Reset", nullptr));
#if QT_CONFIG(tooltip)
        dalLabel->setToolTip(QCoreApplication::translate("TonemappingDialogForm", "Relates to brightness surrounding the display. Default 50cd/m\302\262", nullptr));
#endif // QT_CONFIG(tooltip)
        dalLabel->setText(QCoreApplication::translate("TonemappingDialogForm", "Display adaptation luminance:", nullptr));
#if QT_CONFIG(tooltip)
        dmSpinBox->setToolTip(QCoreApplication::translate("TonemappingDialogForm", "Display property. Default 100cd/m\302\262 for a CRT.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        checkBox_extraGamma->setToolTip(QCoreApplication::translate("TonemappingDialogForm", "Stellarium's original formulation may include a gamma term too much, but it looks better than without.", nullptr));
#endif // QT_CONFIG(tooltip)
        checkBox_extraGamma->setText(QCoreApplication::translate("TonemappingDialogForm", "Use extra Gamma term", nullptr));
#if QT_CONFIG(tooltip)
        gammaLabel->setToolTip(QCoreApplication::translate("TonemappingDialogForm", "Assumed nonlinearity between input signal and output brightness. Default 2.2222.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        gammaLabel->setWhatsThis(QCoreApplication::translate("TonemappingDialogForm", "Extinction is the loss of star brightness due to Earth's atmosphere. It is given in mag/airmass, where airmass is number of atmospheres light has to pass. (zenith: 1; horizon: about 40)", nullptr));
#endif // QT_CONFIG(whatsthis)
        gammaLabel->setText(QCoreApplication::translate("TonemappingDialogForm", "Display Gamma:", nullptr));
#if QT_CONFIG(tooltip)
        checkBox_sRGB->setToolTip(QCoreApplication::translate("TonemappingDialogForm", "Use an XYZ->sRGB color transform matrix, not XYZ->Adobe RGB (1998).", nullptr));
#endif // QT_CONFIG(tooltip)
        checkBox_sRGB->setText(QCoreApplication::translate("TonemappingDialogForm", "Use sRGB", nullptr));
        (void)TonemappingDialogForm;
    } // retranslateUi

};

namespace Ui {
    class TonemappingDialogForm: public Ui_TonemappingDialogForm {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_TONEMAPPINGDIALOG_H
