/****************************************************************************
** Meta object code from reading C++ file 'OcularDialog.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../stellarium/plugins/Oculars/src/gui/OcularDialog.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'OcularDialog.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12OcularDialogE_t {};
} // unnamed namespace

template <> constexpr inline auto OcularDialog::qt_create_metaobjectdata<qt_meta_tag_ZN12OcularDialogE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OcularDialog",
        "closeWindow",
        "",
        "deleteSelectedCCD",
        "deleteSelectedOcular",
        "deleteSelectedTelescope",
        "deleteSelectedLens",
        "insertNewCCD",
        "insertNewOcular",
        "insertNewTelescope",
        "insertNewLens",
        "moveUpSelectedSensor",
        "moveUpSelectedOcular",
        "moveUpSelectedTelescope",
        "moveUpSelectedLens",
        "moveDownSelectedSensor",
        "moveDownSelectedOcular",
        "moveDownSelectedTelescope",
        "moveDownSelectedLens",
        "retranslate",
        "updateOcular",
        "selectOcular",
        "QModelIndex",
        "updateLens",
        "selectLens",
        "updateCCD",
        "selectCCD",
        "updateTelescope",
        "selectTelescope",
        "initAboutText",
        "setLabelsDescriptionText",
        "state",
        "updateTelradCustomFOV",
        "setupTelradFOVspins",
        "Vec4f",
        "fov",
        "updateGuiOptions",
        "updateCCDRotationAngles",
        "updateSuffixes"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'closeWindow'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'deleteSelectedCCD'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'deleteSelectedOcular'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'deleteSelectedTelescope'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'deleteSelectedLens'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'insertNewCCD'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'insertNewOcular'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'insertNewTelescope'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'insertNewLens'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'moveUpSelectedSensor'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'moveUpSelectedOcular'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'moveUpSelectedTelescope'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'moveUpSelectedLens'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'moveDownSelectedSensor'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'moveDownSelectedOcular'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'moveDownSelectedTelescope'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'moveDownSelectedLens'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'retranslate'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'updateOcular'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'selectOcular'
        QtMocHelpers::SlotData<void(const QModelIndex)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 22, 2 },
        }}),
        // Slot 'updateLens'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'selectLens'
        QtMocHelpers::SlotData<void(const QModelIndex)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 22, 2 },
        }}),
        // Slot 'updateCCD'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'selectCCD'
        QtMocHelpers::SlotData<void(const QModelIndex)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 22, 2 },
        }}),
        // Slot 'updateTelescope'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'selectTelescope'
        QtMocHelpers::SlotData<void(const QModelIndex)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 22, 2 },
        }}),
        // Slot 'initAboutText'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setLabelsDescriptionText'
        QtMocHelpers::SlotData<void(bool)>(30, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 31 },
        }}),
        // Slot 'updateTelradCustomFOV'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setupTelradFOVspins'
        QtMocHelpers::SlotData<void(Vec4f)>(33, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 34, 35 },
        }}),
        // Slot 'updateGuiOptions'
        QtMocHelpers::SlotData<void()>(36, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateCCDRotationAngles'
        QtMocHelpers::SlotData<void()>(37, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateSuffixes'
        QtMocHelpers::SlotData<void()>(38, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<OcularDialog, qt_meta_tag_ZN12OcularDialogE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OcularDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<StelDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12OcularDialogE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12OcularDialogE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12OcularDialogE_t>.metaTypes,
    nullptr
} };

void OcularDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<OcularDialog *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->closeWindow(); break;
        case 1: _t->deleteSelectedCCD(); break;
        case 2: _t->deleteSelectedOcular(); break;
        case 3: _t->deleteSelectedTelescope(); break;
        case 4: _t->deleteSelectedLens(); break;
        case 5: _t->insertNewCCD(); break;
        case 6: _t->insertNewOcular(); break;
        case 7: _t->insertNewTelescope(); break;
        case 8: _t->insertNewLens(); break;
        case 9: _t->moveUpSelectedSensor(); break;
        case 10: _t->moveUpSelectedOcular(); break;
        case 11: _t->moveUpSelectedTelescope(); break;
        case 12: _t->moveUpSelectedLens(); break;
        case 13: _t->moveDownSelectedSensor(); break;
        case 14: _t->moveDownSelectedOcular(); break;
        case 15: _t->moveDownSelectedTelescope(); break;
        case 16: _t->moveDownSelectedLens(); break;
        case 17: _t->retranslate(); break;
        case 18: _t->updateOcular(); break;
        case 19: _t->selectOcular((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 20: _t->updateLens(); break;
        case 21: _t->selectLens((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 22: _t->updateCCD(); break;
        case 23: _t->selectCCD((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 24: _t->updateTelescope(); break;
        case 25: _t->selectTelescope((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 26: _t->initAboutText(); break;
        case 27: _t->setLabelsDescriptionText((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 28: _t->updateTelradCustomFOV(); break;
        case 29: _t->setupTelradFOVspins((*reinterpret_cast<std::add_pointer_t<Vec4f>>(_a[1]))); break;
        case 30: _t->updateGuiOptions(); break;
        case 31: _t->updateCCDRotationAngles(); break;
        case 32: _t->updateSuffixes(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 29:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Vec4f >(); break;
            }
            break;
        }
    }
}

const QMetaObject *OcularDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OcularDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12OcularDialogE_t>.strings))
        return static_cast<void*>(this);
    return StelDialog::qt_metacast(_clname);
}

int OcularDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StelDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 33)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 33;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 33)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 33;
    }
    return _id;
}
QT_WARNING_POP
