/****************************************************************************
** Meta object code from reading C++ file 'ViewDialog.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../stellarium/src/gui/ViewDialog.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ViewDialog.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10ViewDialogE_t {};
} // unnamed namespace

template <> constexpr inline auto ViewDialog::qt_create_metaobjectdata<qt_meta_tag_ZN10ViewDialogE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ViewDialog",
        "styleChanged",
        "",
        "style",
        "retranslate",
        "populateLists",
        "configureSkyCultureCheckboxes",
        "updateSkyCultureInfoStyleFromCheckboxes",
        "updateSkyCultureScreenStyleFromCheckboxes",
        "populateCulturalCombo",
        "QComboBox*",
        "combo",
        "StelObject::CulturalDisplayStyle",
        "setZodiacLabelStyle",
        "index",
        "setLunarSystemLabelStyle",
        "populateToolTips",
        "skyCultureChanged",
        "changeProjection",
        "projectionNameI18n",
        "projectionChanged",
        "landscapeChanged",
        "id",
        "name",
        "updateZhrDescription",
        "zhr",
        "setCurrentLandscapeAsDefault",
        "setCurrentCultureAsDefault",
        "updateDefaultSkyCulture",
        "updateDefaultLandscape",
        "updateSkyCultureTimeValue",
        "year",
        "changeMinTime",
        "minYear",
        "changeMaxTime",
        "maxYear",
        "updateSkyCultureTimeRange",
        "filterSkyCultures",
        "initiateSkyCultureMapRotation",
        "updateSkyCultureGUI",
        "gridSpacingChanged",
        "updateGridSpacingComboBox",
        "showAddRemoveLandscapesDialog",
        "showAtmosphereDialog",
        "showSkylightDialog",
        "showTonemappingDialog",
        "showGreatRedSpotDialog",
        "showConfigureDSOColorsDialog",
        "showConfigureOrbitColorsDialog",
        "populatePlanetMagnitudeAlgorithmsList",
        "populatePlanetMagnitudeAlgorithmDescription",
        "setPlanetMagnitudeAlgorithm",
        "algorithmID",
        "setSelectedCardinalCheckBoxes",
        "setSelectedCatalogsFromCheckBoxes",
        "setSelectedTypesFromCheckBoxes",
        "changePage",
        "QListWidgetItem*",
        "current",
        "previous",
        "updateSelectedCatalogsCheckBoxes",
        "updateSelectedTypesCheckBoxes",
        "clearHips",
        "updateHips",
        "updateHipsText",
        "filterSurveys",
        "hipsListItemChanged",
        "QTreeWidgetItem*",
        "item",
        "populateHipsGroups",
        "toggleHipsDialog",
        "populateOrbitsControls",
        "flag",
        "populateTrailsControls",
        "populateNomenclatureControls",
        "setDisplayFormatForSpins",
        "flagDecimalDegrees"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'styleChanged'
        QtMocHelpers::SlotData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'retranslate'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'populateLists'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'configureSkyCultureCheckboxes'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateSkyCultureInfoStyleFromCheckboxes'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateSkyCultureScreenStyleFromCheckboxes'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'populateCulturalCombo'
        QtMocHelpers::SlotData<void(QComboBox *, StelObject::CulturalDisplayStyle)>(9, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 10, 11 }, { 0x80000000 | 12, 3 },
        }}),
        // Slot 'setZodiacLabelStyle'
        QtMocHelpers::SlotData<void(int)>(13, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 14 },
        }}),
        // Slot 'setLunarSystemLabelStyle'
        QtMocHelpers::SlotData<void(int)>(15, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 14 },
        }}),
        // Slot 'populateToolTips'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'skyCultureChanged'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'changeProjection'
        QtMocHelpers::SlotData<void(const QString &)>(18, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 19 },
        }}),
        // Slot 'projectionChanged'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'landscapeChanged'
        QtMocHelpers::SlotData<void(QString, QString)>(21, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 22 }, { QMetaType::QString, 23 },
        }}),
        // Slot 'updateZhrDescription'
        QtMocHelpers::SlotData<void(int)>(24, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 25 },
        }}),
        // Slot 'setCurrentLandscapeAsDefault'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setCurrentCultureAsDefault'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateDefaultSkyCulture'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateDefaultLandscape'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateSkyCultureTimeValue'
        QtMocHelpers::SlotData<void(int)>(30, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 31 },
        }}),
        // Slot 'changeMinTime'
        QtMocHelpers::SlotData<void(int)>(32, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 33 },
        }}),
        // Slot 'changeMaxTime'
        QtMocHelpers::SlotData<void(int)>(34, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 35 },
        }}),
        // Slot 'updateSkyCultureTimeRange'
        QtMocHelpers::SlotData<void(int, int)>(36, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 33 }, { QMetaType::Int, 35 },
        }}),
        // Slot 'filterSkyCultures'
        QtMocHelpers::SlotData<void()>(37, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'initiateSkyCultureMapRotation'
        QtMocHelpers::SlotData<void()>(38, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateSkyCultureGUI'
        QtMocHelpers::SlotData<void()>(39, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'gridSpacingChanged'
        QtMocHelpers::SlotData<void(int)>(40, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 14 },
        }}),
        // Slot 'updateGridSpacingComboBox'
        QtMocHelpers::SlotData<void()>(41, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'showAddRemoveLandscapesDialog'
        QtMocHelpers::SlotData<void()>(42, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'showAtmosphereDialog'
        QtMocHelpers::SlotData<void()>(43, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showSkylightDialog'
        QtMocHelpers::SlotData<void()>(44, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showTonemappingDialog'
        QtMocHelpers::SlotData<void()>(45, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showGreatRedSpotDialog'
        QtMocHelpers::SlotData<void()>(46, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showConfigureDSOColorsDialog'
        QtMocHelpers::SlotData<void()>(47, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showConfigureOrbitColorsDialog'
        QtMocHelpers::SlotData<void()>(48, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'populatePlanetMagnitudeAlgorithmsList'
        QtMocHelpers::SlotData<void()>(49, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'populatePlanetMagnitudeAlgorithmDescription'
        QtMocHelpers::SlotData<void()>(50, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setPlanetMagnitudeAlgorithm'
        QtMocHelpers::SlotData<void(int)>(51, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 52 },
        }}),
        // Slot 'setSelectedCardinalCheckBoxes'
        QtMocHelpers::SlotData<void()>(53, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setSelectedCatalogsFromCheckBoxes'
        QtMocHelpers::SlotData<void()>(54, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setSelectedTypesFromCheckBoxes'
        QtMocHelpers::SlotData<void()>(55, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'changePage'
        QtMocHelpers::SlotData<void(QListWidgetItem *, QListWidgetItem *)>(56, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 57, 58 }, { 0x80000000 | 57, 59 },
        }}),
        // Slot 'updateSelectedCatalogsCheckBoxes'
        QtMocHelpers::SlotData<void()>(60, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateSelectedTypesCheckBoxes'
        QtMocHelpers::SlotData<void()>(61, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'clearHips'
        QtMocHelpers::SlotData<void()>(62, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateHips'
        QtMocHelpers::SlotData<void()>(63, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateHipsText'
        QtMocHelpers::SlotData<void()>(64, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'filterSurveys'
        QtMocHelpers::SlotData<void()>(65, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'hipsListItemChanged'
        QtMocHelpers::SlotData<void(QTreeWidgetItem *)>(66, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 67, 68 },
        }}),
        // Slot 'populateHipsGroups'
        QtMocHelpers::SlotData<void()>(69, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'toggleHipsDialog'
        QtMocHelpers::SlotData<void()>(70, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'populateOrbitsControls'
        QtMocHelpers::SlotData<void(bool)>(71, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 72 },
        }}),
        // Slot 'populateTrailsControls'
        QtMocHelpers::SlotData<void(bool)>(73, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 72 },
        }}),
        // Slot 'populateNomenclatureControls'
        QtMocHelpers::SlotData<void(bool)>(74, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 72 },
        }}),
        // Slot 'setDisplayFormatForSpins'
        QtMocHelpers::SlotData<void(bool)>(75, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 76 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ViewDialog, qt_meta_tag_ZN10ViewDialogE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ViewDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<StelDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10ViewDialogE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10ViewDialogE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10ViewDialogE_t>.metaTypes,
    nullptr
} };

void ViewDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ViewDialog *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->styleChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->retranslate(); break;
        case 2: _t->populateLists(); break;
        case 3: _t->configureSkyCultureCheckboxes(); break;
        case 4: _t->updateSkyCultureInfoStyleFromCheckboxes(); break;
        case 5: _t->updateSkyCultureScreenStyleFromCheckboxes(); break;
        case 6: _t->populateCulturalCombo((*reinterpret_cast<std::add_pointer_t<QComboBox*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<StelObject::CulturalDisplayStyle>>(_a[2]))); break;
        case 7: _t->setZodiacLabelStyle((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 8: _t->setLunarSystemLabelStyle((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 9: _t->populateToolTips(); break;
        case 10: _t->skyCultureChanged(); break;
        case 11: _t->changeProjection((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 12: _t->projectionChanged(); break;
        case 13: _t->landscapeChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 14: _t->updateZhrDescription((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 15: _t->setCurrentLandscapeAsDefault(); break;
        case 16: _t->setCurrentCultureAsDefault(); break;
        case 17: _t->updateDefaultSkyCulture(); break;
        case 18: _t->updateDefaultLandscape(); break;
        case 19: _t->updateSkyCultureTimeValue((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 20: _t->changeMinTime((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 21: _t->changeMaxTime((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 22: _t->updateSkyCultureTimeRange((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 23: _t->filterSkyCultures(); break;
        case 24: _t->initiateSkyCultureMapRotation(); break;
        case 25: _t->updateSkyCultureGUI(); break;
        case 26: _t->gridSpacingChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 27: _t->updateGridSpacingComboBox(); break;
        case 28: _t->showAddRemoveLandscapesDialog(); break;
        case 29: _t->showAtmosphereDialog(); break;
        case 30: _t->showSkylightDialog(); break;
        case 31: _t->showTonemappingDialog(); break;
        case 32: _t->showGreatRedSpotDialog(); break;
        case 33: _t->showConfigureDSOColorsDialog(); break;
        case 34: _t->showConfigureOrbitColorsDialog(); break;
        case 35: _t->populatePlanetMagnitudeAlgorithmsList(); break;
        case 36: _t->populatePlanetMagnitudeAlgorithmDescription(); break;
        case 37: _t->setPlanetMagnitudeAlgorithm((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 38: _t->setSelectedCardinalCheckBoxes(); break;
        case 39: _t->setSelectedCatalogsFromCheckBoxes(); break;
        case 40: _t->setSelectedTypesFromCheckBoxes(); break;
        case 41: _t->changePage((*reinterpret_cast<std::add_pointer_t<QListWidgetItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QListWidgetItem*>>(_a[2]))); break;
        case 42: _t->updateSelectedCatalogsCheckBoxes(); break;
        case 43: _t->updateSelectedTypesCheckBoxes(); break;
        case 44: _t->clearHips(); break;
        case 45: _t->updateHips(); break;
        case 46: _t->updateHipsText(); break;
        case 47: _t->filterSurveys(); break;
        case 48: _t->hipsListItemChanged((*reinterpret_cast<std::add_pointer_t<QTreeWidgetItem*>>(_a[1]))); break;
        case 49: _t->populateHipsGroups(); break;
        case 50: _t->toggleHipsDialog(); break;
        case 51: _t->populateOrbitsControls((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 52: _t->populateTrailsControls((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 53: _t->populateNomenclatureControls((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 54: _t->setDisplayFormatForSpins((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *ViewDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ViewDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10ViewDialogE_t>.strings))
        return static_cast<void*>(this);
    return StelDialog::qt_metacast(_clname);
}

int ViewDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StelDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 55)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 55;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 55)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 55;
    }
    return _id;
}
QT_WARNING_POP
