/********************************************************************************
** Form generated from reading UI file 'scmConstellationDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SCMCONSTELLATIONDIALOG_H
#define UI_SCMCONSTELLATIONDIALOG_H

#include <Dialog.hpp>
#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGraphicsView>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTextBrowser>
#include <QtWidgets/QTextEdit>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_scmConstellationDialog
{
public:
    QVBoxLayout *verticalLayout;
    TitleBar *titleBar;
    QTabWidget *tabs;
    QWidget *drawingTab;
    QVBoxLayout *verticalLayout_5;
    QHBoxLayout *horizontalLayout_2;
    QPushButton *penBtn;
    QPushButton *undoBtn;
    QPushButton *eraserBtn;
    QTextBrowser *drawInfoBox;
    QWidget *constellationNameTab;
    QVBoxLayout *verticalLayout_2;
    QLabel *labelsTitle;
    QLabel *enNameLbl;
    QLineEdit *enNameLE;
    QLabel *idLbl;
    QLineEdit *idLE;
    QLabel *natNameLbl;
    QLineEdit *natNameLE;
    QLabel *pronounceLbl;
    QLineEdit *pronounceLE;
    QLabel *translitLbl;
    QLineEdit *translitLE;
    QLabel *ipaLbl;
    QLineEdit *ipaLE;
    QLabel *label;
    QTextEdit *description;
    QSpacerItem *constTabVertSpacer;
    QWidget *artworkTab;
    QGridLayout *artwortGridLayout;
    QGraphicsView *artwork_image;
    QToolButton *tooltipBtn;
    QPushButton *tooltipLabel;
    QHBoxLayout *horizontalLayout;
    QPushButton *bind_star;
    QPushButton *upload_image;
    QPushButton *remove_image;
    QFrame *Content;
    QVBoxLayout *verticalLayout_3;
    QHBoxLayout *bottomBtnLayout;
    QPushButton *saveBtn;
    QPushButton *cancelBtn;

    void setupUi(QWidget *scmConstellationDialog)
    {
        if (scmConstellationDialog->objectName().isEmpty())
            scmConstellationDialog->setObjectName("scmConstellationDialog");
        scmConstellationDialog->resize(554, 627);
        scmConstellationDialog->setStyleSheet(QString::fromUtf8(""));
        verticalLayout = new QVBoxLayout(scmConstellationDialog);
        verticalLayout->setSpacing(0);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        titleBar = new TitleBar(scmConstellationDialog);
        titleBar->setObjectName("titleBar");

        verticalLayout->addWidget(titleBar);

        tabs = new QTabWidget(scmConstellationDialog);
        tabs->setObjectName("tabs");
        tabs->setAutoFillBackground(false);
        tabs->setIconSize(QSize(32, 32));
        drawingTab = new QWidget();
        drawingTab->setObjectName("drawingTab");
        verticalLayout_5 = new QVBoxLayout(drawingTab);
        verticalLayout_5->setObjectName("verticalLayout_5");
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        penBtn = new QPushButton(drawingTab);
        penBtn->setObjectName("penBtn");
        penBtn->setCheckable(true);
        penBtn->setChecked(false);

        horizontalLayout_2->addWidget(penBtn);

        undoBtn = new QPushButton(drawingTab);
        undoBtn->setObjectName("undoBtn");

        horizontalLayout_2->addWidget(undoBtn);

        eraserBtn = new QPushButton(drawingTab);
        eraserBtn->setObjectName("eraserBtn");
        eraserBtn->setCheckable(true);

        horizontalLayout_2->addWidget(eraserBtn);


        verticalLayout_5->addLayout(horizontalLayout_2);

        drawInfoBox = new QTextBrowser(drawingTab);
        drawInfoBox->setObjectName("drawInfoBox");

        verticalLayout_5->addWidget(drawInfoBox);

        tabs->addTab(drawingTab, QString());
        constellationNameTab = new QWidget();
        constellationNameTab->setObjectName("constellationNameTab");
        verticalLayout_2 = new QVBoxLayout(constellationNameTab);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(15, 10, 15, 10);
        labelsTitle = new QLabel(constellationNameTab);
        labelsTitle->setObjectName("labelsTitle");

        verticalLayout_2->addWidget(labelsTitle);

        enNameLbl = new QLabel(constellationNameTab);
        enNameLbl->setObjectName("enNameLbl");

        verticalLayout_2->addWidget(enNameLbl);

        enNameLE = new QLineEdit(constellationNameTab);
        enNameLE->setObjectName("enNameLE");

        verticalLayout_2->addWidget(enNameLE);

        idLbl = new QLabel(constellationNameTab);
        idLbl->setObjectName("idLbl");

        verticalLayout_2->addWidget(idLbl);

        idLE = new QLineEdit(constellationNameTab);
        idLE->setObjectName("idLE");

        verticalLayout_2->addWidget(idLE);

        natNameLbl = new QLabel(constellationNameTab);
        natNameLbl->setObjectName("natNameLbl");

        verticalLayout_2->addWidget(natNameLbl);

        natNameLE = new QLineEdit(constellationNameTab);
        natNameLE->setObjectName("natNameLE");

        verticalLayout_2->addWidget(natNameLE);

        pronounceLbl = new QLabel(constellationNameTab);
        pronounceLbl->setObjectName("pronounceLbl");

        verticalLayout_2->addWidget(pronounceLbl);

        pronounceLE = new QLineEdit(constellationNameTab);
        pronounceLE->setObjectName("pronounceLE");

        verticalLayout_2->addWidget(pronounceLE);

        translitLbl = new QLabel(constellationNameTab);
        translitLbl->setObjectName("translitLbl");

        verticalLayout_2->addWidget(translitLbl);

        translitLE = new QLineEdit(constellationNameTab);
        translitLE->setObjectName("translitLE");

        verticalLayout_2->addWidget(translitLE);

        ipaLbl = new QLabel(constellationNameTab);
        ipaLbl->setObjectName("ipaLbl");

        verticalLayout_2->addWidget(ipaLbl);

        ipaLE = new QLineEdit(constellationNameTab);
        ipaLE->setObjectName("ipaLE");

        verticalLayout_2->addWidget(ipaLE);

        label = new QLabel(constellationNameTab);
        label->setObjectName("label");

        verticalLayout_2->addWidget(label);

        description = new QTextEdit(constellationNameTab);
        description->setObjectName("description");
        description->setAcceptRichText(false);

        verticalLayout_2->addWidget(description);

        constTabVertSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_2->addItem(constTabVertSpacer);

        tabs->addTab(constellationNameTab, QString());
        artworkTab = new QWidget();
        artworkTab->setObjectName("artworkTab");
        artwortGridLayout = new QGridLayout(artworkTab);
        artwortGridLayout->setSpacing(5);
        artwortGridLayout->setObjectName("artwortGridLayout");
        artwortGridLayout->setContentsMargins(5, 5, 5, 5);
        artwork_image = new QGraphicsView(artworkTab);
        artwork_image->setObjectName("artwork_image");
        artwork_image->viewport()->setProperty("cursor", QVariant(QCursor(Qt::CursorShape::CrossCursor)));
        artwork_image->setMouseTracking(true);
        artwork_image->setFrameShape(QFrame::NoFrame);
        artwork_image->setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded);
        artwork_image->setHorizontalScrollBarPolicy(Qt::ScrollBarAsNeeded);
        artwork_image->setSizeAdjustPolicy(QAbstractScrollArea::AdjustIgnored);

        artwortGridLayout->addWidget(artwork_image, 0, 0, 3, 3);

        tooltipBtn = new QToolButton(artworkTab);
        tooltipBtn->setObjectName("tooltipBtn");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(tooltipBtn->sizePolicy().hasHeightForWidth());
        tooltipBtn->setSizePolicy(sizePolicy);
        tooltipBtn->setCursor(QCursor(Qt::CursorShape::WhatsThisCursor));
        tooltipBtn->setContextMenuPolicy(Qt::NoContextMenu);
        tooltipBtn->setFocusPolicy(Qt::NoFocus);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/graphicGui/tabHelp-selected.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        icon.addFile(QString::fromUtf8(":/graphicGui/tabHelp-selected.png"), QSize(), QIcon::Mode::Disabled, QIcon::State::Off);
        tooltipBtn->setIcon(icon);
        tooltipBtn->setStyleSheet(QString::fromUtf8("\n"
"            background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 rgb(113, 114, 115), stop: 1 rgb(71, 73, 74));\n"
"            margin: 5px;\n"
"          "));

        artwortGridLayout->addWidget(tooltipBtn, 0, 2, 1, 1, Qt::AlignRight|Qt::AlignTop);

        tooltipLabel = new QPushButton(artworkTab);
        tooltipLabel->setObjectName("tooltipLabel");
        tooltipLabel->setStyleSheet(QString::fromUtf8("\n"
"            border: 1px solid black;\n"
"            margin: 2px;\n"
"            padding: 10px;\n"
"            background-color: rgb(143, 143, 143);\n"
"            text-align: left;\n"
"          "));

        artwortGridLayout->addWidget(tooltipLabel, 0, 2, 1, 1, Qt::AlignRight|Qt::AlignTop);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setSizeConstraint(QLayout::SetDefaultConstraint);
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        bind_star = new QPushButton(artworkTab);
        bind_star->setObjectName("bind_star");
        bind_star->setEnabled(false);
        bind_star->setCheckable(false);
        bind_star->setChecked(false);
        bind_star->setAutoDefault(false);
        bind_star->setFlat(false);

        horizontalLayout->addWidget(bind_star);

        upload_image = new QPushButton(artworkTab);
        upload_image->setObjectName("upload_image");

        horizontalLayout->addWidget(upload_image);

        remove_image = new QPushButton(artworkTab);
        remove_image->setObjectName("remove_image");

        horizontalLayout->addWidget(remove_image);


        artwortGridLayout->addLayout(horizontalLayout, 3, 0, 1, 3);

        tabs->addTab(artworkTab, QString());

        verticalLayout->addWidget(tabs);

        Content = new QFrame(scmConstellationDialog);
        Content->setObjectName("Content");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(Content->sizePolicy().hasHeightForWidth());
        Content->setSizePolicy(sizePolicy1);
        Content->setFrameShape(QFrame::NoFrame);
        verticalLayout_3 = new QVBoxLayout(Content);
        verticalLayout_3->setSpacing(0);
        verticalLayout_3->setObjectName("verticalLayout_3");
        verticalLayout_3->setContentsMargins(0, 0, 0, 0);
        bottomBtnLayout = new QHBoxLayout();
        bottomBtnLayout->setSpacing(10);
        bottomBtnLayout->setObjectName("bottomBtnLayout");
        bottomBtnLayout->setContentsMargins(20, 10, 20, 10);
        saveBtn = new QPushButton(Content);
        saveBtn->setObjectName("saveBtn");

        bottomBtnLayout->addWidget(saveBtn);

        cancelBtn = new QPushButton(Content);
        cancelBtn->setObjectName("cancelBtn");

        bottomBtnLayout->addWidget(cancelBtn);


        verticalLayout_3->addLayout(bottomBtnLayout);


        verticalLayout->addWidget(Content);

        titleBar->raise();
        Content->raise();
        tabs->raise();

        retranslateUi(scmConstellationDialog);

        tabs->setCurrentIndex(0);
        bind_star->setDefault(false);


        QMetaObject::connectSlotsByName(scmConstellationDialog);
    } // setupUi

    void retranslateUi(QWidget *scmConstellationDialog)
    {
        titleBar->setProperty("title", QVariant(QCoreApplication::translate("scmConstellationDialog", "SCM: Constellation Editor", nullptr)));
        penBtn->setText(QCoreApplication::translate("scmConstellationDialog", "Pen", nullptr));
#if QT_CONFIG(tooltip)
        undoBtn->setToolTip(QCoreApplication::translate("scmConstellationDialog", "Revert the last drawn line.", nullptr));
#endif // QT_CONFIG(tooltip)
        undoBtn->setText(QCoreApplication::translate("scmConstellationDialog", "Undo", nullptr));
        eraserBtn->setText(QCoreApplication::translate("scmConstellationDialog", "Eraser", nullptr));
        tabs->setTabText(tabs->indexOf(drawingTab), QCoreApplication::translate("scmConstellationDialog", "Drawing", nullptr));
        enNameLbl->setText(QCoreApplication::translate("scmConstellationDialog", "English name", nullptr));
        idLbl->setText(QCoreApplication::translate("scmConstellationDialog", "ID (Do not include 'CON' or the sky culture name)", nullptr));
        natNameLbl->setText(QCoreApplication::translate("scmConstellationDialog", "Native name (optional)", nullptr));
        pronounceLbl->setText(QCoreApplication::translate("scmConstellationDialog", "Pronounciation (optional, European glyphs or Pinyin)", nullptr));
        translitLbl->setText(QCoreApplication::translate("scmConstellationDialog", "Transliteration (optional)", nullptr));
        ipaLbl->setText(QCoreApplication::translate("scmConstellationDialog", "IPA (optional)", nullptr));
        label->setText(QCoreApplication::translate("scmConstellationDialog", "Description (optional)", nullptr));
        tabs->setTabText(tabs->indexOf(constellationNameTab), QCoreApplication::translate("scmConstellationDialog", "Information", nullptr));
        bind_star->setText(QCoreApplication::translate("scmConstellationDialog", "Bind Star", nullptr));
        upload_image->setText(QCoreApplication::translate("scmConstellationDialog", "Upload Image", nullptr));
        remove_image->setText(QCoreApplication::translate("scmConstellationDialog", "Remove Image", nullptr));
        tabs->setTabText(tabs->indexOf(artworkTab), QCoreApplication::translate("scmConstellationDialog", "Artwork", nullptr));
        saveBtn->setText(QCoreApplication::translate("scmConstellationDialog", "Save", nullptr));
        cancelBtn->setText(QCoreApplication::translate("scmConstellationDialog", "Cancel", nullptr));
        (void)scmConstellationDialog;
    } // retranslateUi

};

namespace Ui {
    class scmConstellationDialog: public Ui_scmConstellationDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SCMCONSTELLATIONDIALOG_H
