/********************************************************************************
** Form generated from reading UI file 'greatRedSpotDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_GREATREDSPOTDIALOG_H
#define UI_GREATREDSPOTDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDateTimeEdit>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "Dialog.hpp"
#include "gui/StelCloseButton.hpp"

QT_BEGIN_NAMESPACE

class Ui_GreatRedSpotDialogForm
{
public:
    QVBoxLayout *verticalLayout;
    TitleBar *titleBar;
    QGroupBox *greatRedSpotGroupBox;
    QGridLayout *gridLayout;
    QSpacerItem *verticalSpacer;
    QDoubleSpinBox *driftDoubleSpinBox;
    QDateTimeEdit *jdDateTimeEdit;
    QLabel *driftLabel;
    QLabel *longitudeLabel;
    QSpinBox *longitudeSpinBox;
    QLabel *jdLabel;
    QPushButton *recentGrsMeasurementPushButton;

    void setupUi(QWidget *GreatRedSpotDialogForm)
    {
        if (GreatRedSpotDialogForm->objectName().isEmpty())
            GreatRedSpotDialogForm->setObjectName("GreatRedSpotDialogForm");
        GreatRedSpotDialogForm->resize(327, 201);
        verticalLayout = new QVBoxLayout(GreatRedSpotDialogForm);
        verticalLayout->setSpacing(0);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        titleBar = new TitleBar(GreatRedSpotDialogForm);
        titleBar->setObjectName("titleBar");

        verticalLayout->addWidget(titleBar);

        greatRedSpotGroupBox = new QGroupBox(GreatRedSpotDialogForm);
        greatRedSpotGroupBox->setObjectName("greatRedSpotGroupBox");
        greatRedSpotGroupBox->setEnabled(true);
        gridLayout = new QGridLayout(greatRedSpotGroupBox);
        gridLayout->setObjectName("gridLayout");
        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(verticalSpacer, 4, 0, 1, 1);

        driftDoubleSpinBox = new QDoubleSpinBox(greatRedSpotGroupBox);
        driftDoubleSpinBox->setObjectName("driftDoubleSpinBox");
        driftDoubleSpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        driftDoubleSpinBox->setSuffix(QString::fromUtf8("\302\260"));
        driftDoubleSpinBox->setDecimals(5);
        driftDoubleSpinBox->setMinimum(-90.000000000000000);
        driftDoubleSpinBox->setMaximum(90.000000000000000);
        driftDoubleSpinBox->setSingleStep(0.000100000000000);
        driftDoubleSpinBox->setValue(15.218750000000000);

        gridLayout->addWidget(driftDoubleSpinBox, 1, 1, 1, 1);

        jdDateTimeEdit = new QDateTimeEdit(greatRedSpotGroupBox);
        jdDateTimeEdit->setObjectName("jdDateTimeEdit");
        jdDateTimeEdit->setInputMethodHints(Qt::ImhDate|Qt::ImhPreferNumbers|Qt::ImhTime);
        jdDateTimeEdit->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        jdDateTimeEdit->setCalendarPopup(true);

        gridLayout->addWidget(jdDateTimeEdit, 2, 1, 1, 1);

        driftLabel = new QLabel(greatRedSpotGroupBox);
        driftLabel->setObjectName("driftLabel");
        driftLabel->setInputMethodHints(Qt::ImhNone);
        driftLabel->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);

        gridLayout->addWidget(driftLabel, 1, 0, 1, 1);

        longitudeLabel = new QLabel(greatRedSpotGroupBox);
        longitudeLabel->setObjectName("longitudeLabel");

        gridLayout->addWidget(longitudeLabel, 0, 0, 1, 1);

        longitudeSpinBox = new QSpinBox(greatRedSpotGroupBox);
        longitudeSpinBox->setObjectName("longitudeSpinBox");
        longitudeSpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        longitudeSpinBox->setSuffix(QString::fromUtf8("\302\260"));
        longitudeSpinBox->setMaximum(359);
        longitudeSpinBox->setValue(216);

        gridLayout->addWidget(longitudeSpinBox, 0, 1, 1, 1);

        jdLabel = new QLabel(greatRedSpotGroupBox);
        jdLabel->setObjectName("jdLabel");

        gridLayout->addWidget(jdLabel, 2, 0, 1, 1);

        recentGrsMeasurementPushButton = new QPushButton(greatRedSpotGroupBox);
        recentGrsMeasurementPushButton->setObjectName("recentGrsMeasurementPushButton");
        recentGrsMeasurementPushButton->setFlat(false);

        gridLayout->addWidget(recentGrsMeasurementPushButton, 3, 0, 1, 2);


        verticalLayout->addWidget(greatRedSpotGroupBox);

        QWidget::setTabOrder(longitudeSpinBox, driftDoubleSpinBox);
        QWidget::setTabOrder(driftDoubleSpinBox, jdDateTimeEdit);
        QWidget::setTabOrder(jdDateTimeEdit, recentGrsMeasurementPushButton);

        retranslateUi(GreatRedSpotDialogForm);

        QMetaObject::connectSlotsByName(GreatRedSpotDialogForm);
    } // setupUi

    void retranslateUi(QWidget *GreatRedSpotDialogForm)
    {
        titleBar->setTitle(QCoreApplication::translate("GreatRedSpotDialogForm", "Great Red Spot Details", nullptr));
        greatRedSpotGroupBox->setTitle(QCoreApplication::translate("GreatRedSpotDialogForm", "Custom settings for position of GRS", nullptr));
        jdDateTimeEdit->setDisplayFormat(QCoreApplication::translate("GreatRedSpotDialogForm", "yyyy.MM.dd hh:mm", nullptr));
        driftLabel->setText(QCoreApplication::translate("GreatRedSpotDialogForm", "Annual drift:", nullptr));
#if QT_CONFIG(tooltip)
        longitudeLabel->setToolTip(QCoreApplication::translate("GreatRedSpotDialogForm", "Jovigraphic longitude of Great Red Spot in System II", nullptr));
#endif // QT_CONFIG(tooltip)
        longitudeLabel->setText(QCoreApplication::translate("GreatRedSpotDialogForm", "Longitude of GRS:", nullptr));
#if QT_CONFIG(tooltip)
        jdLabel->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        jdLabel->setWhatsThis(QCoreApplication::translate("GreatRedSpotDialogForm", "Extinction is the loss of star brightness due to Earth's atmosphere. It is given in mag/airmass, where airmass is number of atmospheres light has to pass. (zenith: 1; horizon: about 40)", nullptr));
#endif // QT_CONFIG(whatsthis)
        jdLabel->setText(QCoreApplication::translate("GreatRedSpotDialogForm", "Date and Time (UTC):", nullptr));
#if QT_CONFIG(tooltip)
        recentGrsMeasurementPushButton->setToolTip(QCoreApplication::translate("GreatRedSpotDialogForm", "Open JUPOS website", nullptr));
#endif // QT_CONFIG(tooltip)
        recentGrsMeasurementPushButton->setText(QCoreApplication::translate("GreatRedSpotDialogForm", "View recent GRS measurements", nullptr));
        (void)GreatRedSpotDialogForm;
    } // retranslateUi

};

namespace Ui {
    class GreatRedSpotDialogForm: public Ui_GreatRedSpotDialogForm {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_GREATREDSPOTDIALOG_H
