/****************************************************************************
** Resource object code
**
** Created by: The Resource Compiler for Qt version 6.11.0
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#ifdef _MSC_VER
// disable informational message "function ... selected for automatic inline expansion"
#pragma warning (disable: 4711)
#endif

static const unsigned char qt_resource_data[] = {
  // bt_network_on.png
  0x0,0x0,0x0,0x74,
  0x89,
  0x50,0x4e,0x47,0xd,0xa,0x1a,0xa,0x0,0x0,0x0,0xd,0x49,0x48,0x44,0x52,0x0,
  0x0,0x0,0x20,0x0,0x0,0x0,0x20,0x8,0x4,0x0,0x0,0x0,0xd9,0x73,0xb2,0x7f,
  0x0,0x0,0x0,0x3b,0x49,0x44,0x41,0x54,0x78,0xda,0x63,0xf8,0x4f,0x21,0x1c,0x46,
  0x6,0x8c,0x1a,0x30,0x6a,0x0,0x1c,0x90,0xca,0x86,0x1b,0x0,0xc4,0x60,0x1a,0xf,
  0x1b,0xa,0xd0,0xc5,0x47,0xd,0xa0,0x52,0x2c,0x20,0x4c,0x23,0x99,0x3d,0xcc,0xc,
  0x20,0x19,0x10,0x9f,0x99,0xfe,0x63,0x17,0x1f,0x35,0x60,0x58,0x95,0x89,0x0,0xaf,
  0xd0,0xa0,0x98,0x5f,0x98,0xfc,0x53,0x0,0x0,0x0,0x0,0x49,0x45,0x4e,0x44,0xae,
  0x42,0x60,0x82,
    // bt_network_off.png
  0x0,0x0,0x0,0x75,
  0x89,
  0x50,0x4e,0x47,0xd,0xa,0x1a,0xa,0x0,0x0,0x0,0xd,0x49,0x48,0x44,0x52,0x0,
  0x0,0x0,0x20,0x0,0x0,0x0,0x20,0x8,0x4,0x0,0x0,0x0,0xd9,0x73,0xb2,0x7f,
  0x0,0x0,0x0,0x3c,0x49,0x44,0x41,0x54,0x78,0xda,0x63,0xf8,0x4f,0x21,0x1c,0x46,
  0x6,0x8c,0x1a,0x30,0x6a,0x40,0x3,0xc,0x92,0xca,0x86,0x1b,0x0,0xc4,0x60,0x1a,
  0x1f,0x1b,0x2,0xd1,0xc5,0x47,0xd,0xa0,0x52,0x2c,0x20,0x4c,0x23,0x99,0x3d,0xdc,
  0xc,0x20,0x15,0x12,0x9f,0x99,0x1a,0xb0,0x8b,0x8f,0x1a,0x30,0xac,0xca,0x44,0x0,
  0xdf,0x93,0x4e,0xbd,0x57,0x3f,0xe7,0x93,0x0,0x0,0x0,0x0,0x49,0x45,0x4e,0x44,
  0xae,0x42,0x60,0x82,
  
};

static const unsigned char qt_resource_name[] = {
  // RemoteControl
  0x0,0xd,
  0xb,0xed,0x6d,0x1c,
  0x0,0x52,
  0x0,0x65,0x0,0x6d,0x0,0x6f,0x0,0x74,0x0,0x65,0x0,0x43,0x0,0x6f,0x0,0x6e,0x0,0x74,0x0,0x72,0x0,0x6f,0x0,0x6c,
    // resources
  0x0,0x9,
  0xa,0x6c,0x78,0x43,
  0x0,0x72,
  0x0,0x65,0x0,0x73,0x0,0x6f,0x0,0x75,0x0,0x72,0x0,0x63,0x0,0x65,0x0,0x73,
    // bt_network_on.png
  0x0,0x11,
  0xe,0xc3,0x1c,0x47,
  0x0,0x62,
  0x0,0x74,0x0,0x5f,0x0,0x6e,0x0,0x65,0x0,0x74,0x0,0x77,0x0,0x6f,0x0,0x72,0x0,0x6b,0x0,0x5f,0x0,0x6f,0x0,0x6e,0x0,0x2e,0x0,0x70,0x0,0x6e,0x0,0x67,
  
    // bt_network_off.png
  0x0,0x12,
  0xb,0xed,0xe6,0x87,
  0x0,0x62,
  0x0,0x74,0x0,0x5f,0x0,0x6e,0x0,0x65,0x0,0x74,0x0,0x77,0x0,0x6f,0x0,0x72,0x0,0x6b,0x0,0x5f,0x0,0x6f,0x0,0x66,0x0,0x66,0x0,0x2e,0x0,0x70,0x0,0x6e,
  0x0,0x67,
  
};

static const unsigned char qt_resource_struct[] = {
  // :
  0x0,0x0,0x0,0x0,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x1,
0x0,0x0,0x1,0x9d,0x62,0xdd,0xc3,0xd0,
  // :/RemoteControl
  0x0,0x0,0x0,0x0,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x2,
0x0,0x0,0x1,0x9d,0x62,0xdd,0xc3,0xd0,
  // :/RemoteControl/resources
  0x0,0x0,0x0,0x20,0x0,0x2,0x0,0x0,0x0,0x2,0x0,0x0,0x0,0x3,
0x0,0x0,0x1,0x9d,0x62,0xdd,0xc3,0xd0,
  // :/RemoteControl/resources/bt_network_off.png
  0x0,0x0,0x0,0x60,0x0,0x0,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x78,
0x0,0x0,0x1,0x9d,0x62,0xdd,0xc3,0xd0,
  // :/RemoteControl/resources/bt_network_on.png
  0x0,0x0,0x0,0x38,0x0,0x0,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x0,
0x0,0x0,0x1,0x9d,0x62,0xdd,0xc3,0xd0,

};

#ifdef QT_NAMESPACE
#  define QT_RCC_PREPEND_NAMESPACE(name) ::QT_NAMESPACE::name
#  define QT_RCC_MANGLE_NAMESPACE0(x) x
#  define QT_RCC_MANGLE_NAMESPACE1(a, b) a##_##b
#  define QT_RCC_MANGLE_NAMESPACE2(a, b) QT_RCC_MANGLE_NAMESPACE1(a,b)
#  define QT_RCC_MANGLE_NAMESPACE(name) QT_RCC_MANGLE_NAMESPACE2( \
        QT_RCC_MANGLE_NAMESPACE0(name), QT_RCC_MANGLE_NAMESPACE0(QT_NAMESPACE))
#else
#   define QT_RCC_PREPEND_NAMESPACE(name) name
#   define QT_RCC_MANGLE_NAMESPACE(name) name
#endif

#if defined(QT_INLINE_NAMESPACE)
inline namespace QT_NAMESPACE {
#elif defined(QT_NAMESPACE)
namespace QT_NAMESPACE {
#endif

bool qRegisterResourceData(int, const unsigned char *, const unsigned char *, const unsigned char *);
bool qUnregisterResourceData(int, const unsigned char *, const unsigned char *, const unsigned char *);

#ifdef QT_NAMESPACE
}
#endif

int QT_RCC_MANGLE_NAMESPACE(qInitResources_RemoteSync)();
int QT_RCC_MANGLE_NAMESPACE(qInitResources_RemoteSync)()
{
    int version = 3;
    QT_RCC_PREPEND_NAMESPACE(qRegisterResourceData)
        (version, qt_resource_struct, qt_resource_name, qt_resource_data);
    return 1;
}

int QT_RCC_MANGLE_NAMESPACE(qCleanupResources_RemoteSync)();
int QT_RCC_MANGLE_NAMESPACE(qCleanupResources_RemoteSync)()
{
    int version = 3;
    QT_RCC_PREPEND_NAMESPACE(qUnregisterResourceData)
       (version, qt_resource_struct, qt_resource_name, qt_resource_data);
    return 1;
}

#ifdef __clang__
#   pragma clang diagnostic push
#   pragma clang diagnostic ignored "-Wexit-time-destructors"
#endif

namespace {
   struct initializer {
       initializer() { QT_RCC_MANGLE_NAMESPACE(qInitResources_RemoteSync)(); }
       ~initializer() { QT_RCC_MANGLE_NAMESPACE(qCleanupResources_RemoteSync)(); }
   } dummy;
}

#ifdef __clang__
#   pragma clang diagnostic pop
#endif
