/****************************************************************************
** Meta object code from reading C++ file 'StelToneReproducer.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../stellarium/src/core/StelToneReproducer.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'StelToneReproducer.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN18StelToneReproducerE_t {};
} // unnamed namespace

template <> constexpr inline auto StelToneReproducer::qt_create_metaobjectdata<qt_meta_tag_ZN18StelToneReproducerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "StelToneReproducer",
        "worldAdaptationLuminanceChanged",
        "",
        "displayAdaptationLuminanceChanged",
        "maxDisplayLuminanceChanged",
        "inputScaleChanged",
        "displayGammaChanged",
        "flagUseTmGammaChanged",
        "flagSRGBChanged",
        "val",
        "setDisplayAdaptationLuminance",
        "Lda",
        "getDisplayAdaptationLuminance",
        "setWorldAdaptationLuminance",
        "worldAdaptationLuminance",
        "getWorldAdaptationLuminance",
        "setInputScale",
        "scale",
        "getInputScale",
        "setMaxDisplayLuminance",
        "maxdL",
        "getMaxDisplayLuminance",
        "getDisplayGamma",
        "setDisplayGamma",
        "gamma",
        "getFlagUseTmGamma",
        "setFlagUseTmGamma",
        "b",
        "setFlagSRGB",
        "getFlagSRGB",
        "displayAdaptationLuminance",
        "maxDisplayLuminance",
        "inputScale",
        "displayGamma",
        "flagUseTmGamma",
        "flagSRGB"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'worldAdaptationLuminanceChanged'
        QtMocHelpers::SignalData<void(double)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Signal 'displayAdaptationLuminanceChanged'
        QtMocHelpers::SignalData<void(double)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Signal 'maxDisplayLuminanceChanged'
        QtMocHelpers::SignalData<void(double)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Signal 'inputScaleChanged'
        QtMocHelpers::SignalData<void(double)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Signal 'displayGammaChanged'
        QtMocHelpers::SignalData<void(double)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Signal 'flagUseTmGammaChanged'
        QtMocHelpers::SignalData<void(bool)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'flagSRGBChanged'
        QtMocHelpers::SignalData<void(bool)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 9 },
        }}),
        // Slot 'setDisplayAdaptationLuminance'
        QtMocHelpers::SlotData<void(float)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 11 },
        }}),
        // Slot 'getDisplayAdaptationLuminance'
        QtMocHelpers::SlotData<float() const>(12, 2, QMC::AccessPublic, QMetaType::Float),
        // Slot 'setWorldAdaptationLuminance'
        QtMocHelpers::SlotData<void(float)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 14 },
        }}),
        // Slot 'getWorldAdaptationLuminance'
        QtMocHelpers::SlotData<float() const>(15, 2, QMC::AccessPublic, QMetaType::Float),
        // Slot 'setInputScale'
        QtMocHelpers::SlotData<void(float)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 17 },
        }}),
        // Slot 'setInputScale'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'getInputScale'
        QtMocHelpers::SlotData<float() const>(18, 2, QMC::AccessPublic, QMetaType::Float),
        // Slot 'setMaxDisplayLuminance'
        QtMocHelpers::SlotData<void(float)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 20 },
        }}),
        // Slot 'getMaxDisplayLuminance'
        QtMocHelpers::SlotData<float() const>(21, 2, QMC::AccessPublic, QMetaType::Float),
        // Slot 'getDisplayGamma'
        QtMocHelpers::SlotData<float() const>(22, 2, QMC::AccessPublic, QMetaType::Float),
        // Slot 'setDisplayGamma'
        QtMocHelpers::SlotData<void(float)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 24 },
        }}),
        // Slot 'getFlagUseTmGamma'
        QtMocHelpers::SlotData<bool() const>(25, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFlagUseTmGamma'
        QtMocHelpers::SlotData<void(bool)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 27 },
        }}),
        // Slot 'setFlagSRGB'
        QtMocHelpers::SlotData<void(bool)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 9 },
        }}),
        // Slot 'getFlagSRGB'
        QtMocHelpers::SlotData<bool() const>(29, 2, QMC::AccessPublic, QMetaType::Bool),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'worldAdaptationLuminance'
        QtMocHelpers::PropertyData<double>(14, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'displayAdaptationLuminance'
        QtMocHelpers::PropertyData<double>(30, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'maxDisplayLuminance'
        QtMocHelpers::PropertyData<double>(31, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'inputScale'
        QtMocHelpers::PropertyData<double>(32, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'displayGamma'
        QtMocHelpers::PropertyData<double>(33, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'flagUseTmGamma'
        QtMocHelpers::PropertyData<bool>(34, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
        // property 'flagSRGB'
        QtMocHelpers::PropertyData<bool>(35, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 6),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<StelToneReproducer, qt_meta_tag_ZN18StelToneReproducerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject StelToneReproducer::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18StelToneReproducerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18StelToneReproducerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18StelToneReproducerE_t>.metaTypes,
    nullptr
} };

void StelToneReproducer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<StelToneReproducer *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->worldAdaptationLuminanceChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 1: _t->displayAdaptationLuminanceChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 2: _t->maxDisplayLuminanceChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 3: _t->inputScaleChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 4: _t->displayGammaChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 5: _t->flagUseTmGammaChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->flagSRGBChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->setDisplayAdaptationLuminance((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 8: { float _r = _t->getDisplayAdaptationLuminance();
            if (_a[0]) *reinterpret_cast<float*>(_a[0]) = std::move(_r); }  break;
        case 9: _t->setWorldAdaptationLuminance((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 10: { float _r = _t->getWorldAdaptationLuminance();
            if (_a[0]) *reinterpret_cast<float*>(_a[0]) = std::move(_r); }  break;
        case 11: _t->setInputScale((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 12: _t->setInputScale(); break;
        case 13: { float _r = _t->getInputScale();
            if (_a[0]) *reinterpret_cast<float*>(_a[0]) = std::move(_r); }  break;
        case 14: _t->setMaxDisplayLuminance((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 15: { float _r = _t->getMaxDisplayLuminance();
            if (_a[0]) *reinterpret_cast<float*>(_a[0]) = std::move(_r); }  break;
        case 16: { float _r = _t->getDisplayGamma();
            if (_a[0]) *reinterpret_cast<float*>(_a[0]) = std::move(_r); }  break;
        case 17: _t->setDisplayGamma((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 18: { bool _r = _t->getFlagUseTmGamma();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 19: _t->setFlagUseTmGamma((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 20: _t->setFlagSRGB((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 21: { bool _r = _t->getFlagSRGB();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (StelToneReproducer::*)(double )>(_a, &StelToneReproducer::worldAdaptationLuminanceChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelToneReproducer::*)(double )>(_a, &StelToneReproducer::displayAdaptationLuminanceChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelToneReproducer::*)(double )>(_a, &StelToneReproducer::maxDisplayLuminanceChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelToneReproducer::*)(double )>(_a, &StelToneReproducer::inputScaleChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelToneReproducer::*)(double )>(_a, &StelToneReproducer::displayGammaChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelToneReproducer::*)(bool )>(_a, &StelToneReproducer::flagUseTmGammaChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelToneReproducer::*)(bool )>(_a, &StelToneReproducer::flagSRGBChanged, 6))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<double*>(_v) = _t->getWorldAdaptationLuminance(); break;
        case 1: *reinterpret_cast<double*>(_v) = _t->getDisplayAdaptationLuminance(); break;
        case 2: *reinterpret_cast<double*>(_v) = _t->getMaxDisplayLuminance(); break;
        case 3: *reinterpret_cast<double*>(_v) = _t->getInputScale(); break;
        case 4: *reinterpret_cast<double*>(_v) = _t->getDisplayGamma(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->getFlagUseTmGamma(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->getFlagSRGB(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setWorldAdaptationLuminance(*reinterpret_cast<double*>(_v)); break;
        case 1: _t->setDisplayAdaptationLuminance(*reinterpret_cast<double*>(_v)); break;
        case 2: _t->setMaxDisplayLuminance(*reinterpret_cast<double*>(_v)); break;
        case 3: _t->setInputScale(*reinterpret_cast<double*>(_v)); break;
        case 4: _t->setDisplayGamma(*reinterpret_cast<double*>(_v)); break;
        case 5: _t->setFlagUseTmGamma(*reinterpret_cast<bool*>(_v)); break;
        case 6: _t->setFlagSRGB(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *StelToneReproducer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *StelToneReproducer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18StelToneReproducerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int StelToneReproducer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 22)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 22)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 22;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void StelToneReproducer::worldAdaptationLuminanceChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void StelToneReproducer::displayAdaptationLuminanceChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void StelToneReproducer::maxDisplayLuminanceChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void StelToneReproducer::inputScaleChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void StelToneReproducer::displayGammaChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void StelToneReproducer::flagUseTmGammaChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void StelToneReproducer::flagSRGBChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}
QT_WARNING_POP
