/****************************************************************************
** Meta object code from reading C++ file 'StelSkyDrawer.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../stellarium/src/core/StelSkyDrawer.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'StelSkyDrawer.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13StelSkyDrawerE_t {};
} // unnamed namespace

template <> constexpr inline auto StelSkyDrawer::qt_create_metaobjectdata<qt_meta_tag_ZN13StelSkyDrawerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "StelSkyDrawer",
        "relativeStarScaleChanged",
        "",
        "b",
        "absoluteStarScaleChanged",
        "twinkleAmountChanged",
        "flagTwinkleChanged",
        "flagForcedTwinkleChanged",
        "lightPollutionLuminanceChanged",
        "luminance",
        "flagDrawBigStarHaloChanged",
        "flagStarSpikyChanged",
        "flagStarMagnitudeLimitChanged",
        "flagNebulaMagnitudeLimitChanged",
        "flagPlanetMagnitudeLimitChanged",
        "customStarMagLimitChanged",
        "limit",
        "customNebulaMagLimitChanged",
        "customPlanetMagLimitChanged",
        "flagLuminanceAdaptationChanged",
        "daylightLabelThresholdChanged",
        "t",
        "extinctionCoefficientChanged",
        "coeff",
        "atmosphereTemperatureChanged",
        "celsius",
        "atmospherePressureChanged",
        "mbar",
        "flagTfromKChanged",
        "flagDrawSunAfterAtmosphereChanged",
        "flagEarlySunHaloChanged",
        "turbidityChanged",
        "setRelativeStarScale",
        "getRelativeStarScale",
        "setAbsoluteStarScale",
        "getAbsoluteStarScale",
        "setTwinkleAmount",
        "getTwinkleAmount",
        "setFlagTwinkle",
        "getFlagTwinkle",
        "setFlagForcedTwinkle",
        "getFlagForcedTwinkle",
        "setLightPollutionLuminance",
        "getLightPollutionLuminance",
        "getBortleScaleIndex",
        "setFlagDrawBigStarHalo",
        "getFlagDrawBigStarHalo",
        "setFlagStarSpiky",
        "getFlagStarSpiky",
        "getLimitMagnitude",
        "setFlagStarMagnitudeLimit",
        "getFlagStarMagnitudeLimit",
        "setFlagNebulaMagnitudeLimit",
        "getFlagNebulaMagnitudeLimit",
        "setFlagPlanetMagnitudeLimit",
        "getFlagPlanetMagnitudeLimit",
        "getCustomStarMagnitudeLimit",
        "setCustomStarMagnitudeLimit",
        "getCustomNebulaMagnitudeLimit",
        "setCustomNebulaMagnitudeLimit",
        "getCustomPlanetMagnitudeLimit",
        "setCustomPlanetMagnitudeLimit",
        "getLimitLuminance",
        "setFlagLuminanceAdaptation",
        "getFlagLuminanceAdaptation",
        "setDaylightLabelThreshold",
        "getDaylightLabelThreshold",
        "getWorldAdaptationLuminance",
        "setFlagHasAtmosphere",
        "getFlagHasAtmosphere",
        "setExtinctionCoefficient",
        "extCoeff",
        "getExtinctionCoefficient",
        "setAtmosphereTemperature",
        "getAtmosphereTemperature",
        "setAtmospherePressure",
        "getAtmospherePressure",
        "getExtinction",
        "getRefraction",
        "getBig3dModelHaloRadius",
        "setBig3dModelHaloRadius",
        "r",
        "setFlagDrawSunAfterAtmosphere",
        "val",
        "getFlagDrawSunAfterAtmosphere",
        "setFlagEarlySunHalo",
        "getFlagEarlySunHalo",
        "getFlagTfromK",
        "setFlagTfromK",
        "getT",
        "setT",
        "newT",
        "relativeStarScale",
        "absoluteStarScale",
        "twinkleAmount",
        "flagStarTwinkle",
        "flagForcedTwinkle",
        "lightPollutionLuminance",
        "flagDrawBigStarHalo",
        "flagStarSpiky",
        "flagStarMagnitudeLimit",
        "flagNebulaMagnitudeLimit",
        "flagPlanetMagnitudeLimit",
        "customStarMagLimit",
        "customNebulaMagLimit",
        "customPlanetMagLimit",
        "flagLuminanceAdaptation",
        "daylightLabelThreshold",
        "extinctionCoefficient",
        "atmosphereTemperature",
        "atmospherePressure",
        "flagDrawSunAfterAtmosphere",
        "flagEarlySunHalo",
        "flagTfromK",
        "turbidity"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'relativeStarScaleChanged'
        QtMocHelpers::SignalData<void(double)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Signal 'absoluteStarScaleChanged'
        QtMocHelpers::SignalData<void(double)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Signal 'twinkleAmountChanged'
        QtMocHelpers::SignalData<void(double)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Signal 'flagTwinkleChanged'
        QtMocHelpers::SignalData<void(bool)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagForcedTwinkleChanged'
        QtMocHelpers::SignalData<void(bool)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'lightPollutionLuminanceChanged'
        QtMocHelpers::SignalData<void(double)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 9 },
        }}),
        // Signal 'flagDrawBigStarHaloChanged'
        QtMocHelpers::SignalData<void(bool)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagStarSpikyChanged'
        QtMocHelpers::SignalData<void(bool)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagStarMagnitudeLimitChanged'
        QtMocHelpers::SignalData<void(bool)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagNebulaMagnitudeLimitChanged'
        QtMocHelpers::SignalData<void(bool)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagPlanetMagnitudeLimitChanged'
        QtMocHelpers::SignalData<void(bool)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'customStarMagLimitChanged'
        QtMocHelpers::SignalData<void(double)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 16 },
        }}),
        // Signal 'customNebulaMagLimitChanged'
        QtMocHelpers::SignalData<void(double)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 16 },
        }}),
        // Signal 'customPlanetMagLimitChanged'
        QtMocHelpers::SignalData<void(double)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 16 },
        }}),
        // Signal 'flagLuminanceAdaptationChanged'
        QtMocHelpers::SignalData<void(bool)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'daylightLabelThresholdChanged'
        QtMocHelpers::SignalData<void(double)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 21 },
        }}),
        // Signal 'extinctionCoefficientChanged'
        QtMocHelpers::SignalData<void(double)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 23 },
        }}),
        // Signal 'atmosphereTemperatureChanged'
        QtMocHelpers::SignalData<void(double)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 25 },
        }}),
        // Signal 'atmospherePressureChanged'
        QtMocHelpers::SignalData<void(double)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 27 },
        }}),
        // Signal 'flagTfromKChanged'
        QtMocHelpers::SignalData<void(bool)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagDrawSunAfterAtmosphereChanged'
        QtMocHelpers::SignalData<void(bool)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'flagEarlySunHaloChanged'
        QtMocHelpers::SignalData<void(bool)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'turbidityChanged'
        QtMocHelpers::SignalData<void(double)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 21 },
        }}),
        // Slot 'setRelativeStarScale'
        QtMocHelpers::SlotData<void(double)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setRelativeStarScale'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'getRelativeStarScale'
        QtMocHelpers::SlotData<double() const>(33, 2, QMC::AccessPublic, QMetaType::Double),
        // Slot 'setAbsoluteStarScale'
        QtMocHelpers::SlotData<void(double)>(34, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'setAbsoluteStarScale'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'getAbsoluteStarScale'
        QtMocHelpers::SlotData<double() const>(35, 2, QMC::AccessPublic, QMetaType::Double),
        // Slot 'setTwinkleAmount'
        QtMocHelpers::SlotData<void(double)>(36, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'getTwinkleAmount'
        QtMocHelpers::SlotData<double() const>(37, 2, QMC::AccessPublic, QMetaType::Double),
        // Slot 'setFlagTwinkle'
        QtMocHelpers::SlotData<void(bool)>(38, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'getFlagTwinkle'
        QtMocHelpers::SlotData<bool() const>(39, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFlagForcedTwinkle'
        QtMocHelpers::SlotData<void(bool)>(40, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'getFlagForcedTwinkle'
        QtMocHelpers::SlotData<bool() const>(41, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setLightPollutionLuminance'
        QtMocHelpers::SlotData<void(double)>(42, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 9 },
        }}),
        // Slot 'getLightPollutionLuminance'
        QtMocHelpers::SlotData<double() const>(43, 2, QMC::AccessPublic, QMetaType::Double),
        // Slot 'getBortleScaleIndex'
        QtMocHelpers::SlotData<int() const>(44, 2, QMC::AccessPublic, QMetaType::Int),
        // Slot 'setFlagDrawBigStarHalo'
        QtMocHelpers::SlotData<void(bool)>(45, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'getFlagDrawBigStarHalo'
        QtMocHelpers::SlotData<bool() const>(46, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFlagStarSpiky'
        QtMocHelpers::SlotData<void(bool)>(47, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'getFlagStarSpiky'
        QtMocHelpers::SlotData<bool() const>(48, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'getLimitMagnitude'
        QtMocHelpers::SlotData<float() const>(49, 2, QMC::AccessPublic, QMetaType::Float),
        // Slot 'setFlagStarMagnitudeLimit'
        QtMocHelpers::SlotData<void(bool)>(50, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'getFlagStarMagnitudeLimit'
        QtMocHelpers::SlotData<bool() const>(51, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFlagNebulaMagnitudeLimit'
        QtMocHelpers::SlotData<void(bool)>(52, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'getFlagNebulaMagnitudeLimit'
        QtMocHelpers::SlotData<bool() const>(53, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFlagPlanetMagnitudeLimit'
        QtMocHelpers::SlotData<void(bool)>(54, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'getFlagPlanetMagnitudeLimit'
        QtMocHelpers::SlotData<bool() const>(55, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'getCustomStarMagnitudeLimit'
        QtMocHelpers::SlotData<double() const>(56, 2, QMC::AccessPublic, QMetaType::Double),
        // Slot 'setCustomStarMagnitudeLimit'
        QtMocHelpers::SlotData<void(double)>(57, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 16 },
        }}),
        // Slot 'getCustomNebulaMagnitudeLimit'
        QtMocHelpers::SlotData<double() const>(58, 2, QMC::AccessPublic, QMetaType::Double),
        // Slot 'setCustomNebulaMagnitudeLimit'
        QtMocHelpers::SlotData<void(double)>(59, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 16 },
        }}),
        // Slot 'getCustomPlanetMagnitudeLimit'
        QtMocHelpers::SlotData<double() const>(60, 2, QMC::AccessPublic, QMetaType::Double),
        // Slot 'setCustomPlanetMagnitudeLimit'
        QtMocHelpers::SlotData<void(double)>(61, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 16 },
        }}),
        // Slot 'getLimitLuminance'
        QtMocHelpers::SlotData<float() const>(62, 2, QMC::AccessPublic, QMetaType::Float),
        // Slot 'setFlagLuminanceAdaptation'
        QtMocHelpers::SlotData<void(bool)>(63, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'getFlagLuminanceAdaptation'
        QtMocHelpers::SlotData<bool() const>(64, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setDaylightLabelThreshold'
        QtMocHelpers::SlotData<void(double)>(65, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 21 },
        }}),
        // Slot 'getDaylightLabelThreshold'
        QtMocHelpers::SlotData<double() const>(66, 2, QMC::AccessPublic, QMetaType::Double),
        // Slot 'getWorldAdaptationLuminance'
        QtMocHelpers::SlotData<double() const>(67, 2, QMC::AccessPublic, QMetaType::Double),
        // Slot 'setFlagHasAtmosphere'
        QtMocHelpers::SlotData<void(bool)>(68, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'getFlagHasAtmosphere'
        QtMocHelpers::SlotData<bool() const>(69, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setExtinctionCoefficient'
        QtMocHelpers::SlotData<void(double)>(70, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 71 },
        }}),
        // Slot 'getExtinctionCoefficient'
        QtMocHelpers::SlotData<double() const>(72, 2, QMC::AccessPublic, QMetaType::Double),
        // Slot 'setAtmosphereTemperature'
        QtMocHelpers::SlotData<void(double)>(73, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 25 },
        }}),
        // Slot 'getAtmosphereTemperature'
        QtMocHelpers::SlotData<double() const>(74, 2, QMC::AccessPublic, QMetaType::Double),
        // Slot 'setAtmospherePressure'
        QtMocHelpers::SlotData<void(double)>(75, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 27 },
        }}),
        // Slot 'getAtmospherePressure'
        QtMocHelpers::SlotData<double() const>(76, 2, QMC::AccessPublic, QMetaType::Double),
        // Slot 'getExtinction'
        QtMocHelpers::SlotData<void() const>(77, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'getRefraction'
        QtMocHelpers::SlotData<void() const>(78, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'getBig3dModelHaloRadius'
        QtMocHelpers::SlotData<float() const>(79, 2, QMC::AccessPublic, QMetaType::Float),
        // Slot 'setBig3dModelHaloRadius'
        QtMocHelpers::SlotData<void(float)>(80, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 81 },
        }}),
        // Slot 'setFlagDrawSunAfterAtmosphere'
        QtMocHelpers::SlotData<void(bool)>(82, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 83 },
        }}),
        // Slot 'getFlagDrawSunAfterAtmosphere'
        QtMocHelpers::SlotData<bool() const>(84, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFlagEarlySunHalo'
        QtMocHelpers::SlotData<void(bool)>(85, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 83 },
        }}),
        // Slot 'getFlagEarlySunHalo'
        QtMocHelpers::SlotData<bool() const>(86, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'getFlagTfromK'
        QtMocHelpers::SlotData<bool() const>(87, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFlagTfromK'
        QtMocHelpers::SlotData<void(bool)>(88, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 83 },
        }}),
        // Slot 'getT'
        QtMocHelpers::SlotData<double() const>(89, 2, QMC::AccessPublic, QMetaType::Double),
        // Slot 'setT'
        QtMocHelpers::SlotData<void(double)>(90, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 91 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'relativeStarScale'
        QtMocHelpers::PropertyData<double>(92, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'absoluteStarScale'
        QtMocHelpers::PropertyData<double>(93, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'twinkleAmount'
        QtMocHelpers::PropertyData<double>(94, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'flagStarTwinkle'
        QtMocHelpers::PropertyData<bool>(95, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 3),
        // property 'flagForcedTwinkle'
        QtMocHelpers::PropertyData<bool>(96, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'lightPollutionLuminance'
        QtMocHelpers::PropertyData<double>(97, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
        // property 'flagDrawBigStarHalo'
        QtMocHelpers::PropertyData<bool>(98, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 6),
        // property 'flagStarSpiky'
        QtMocHelpers::PropertyData<bool>(99, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 7),
        // property 'flagStarMagnitudeLimit'
        QtMocHelpers::PropertyData<bool>(100, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 8),
        // property 'flagNebulaMagnitudeLimit'
        QtMocHelpers::PropertyData<bool>(101, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 9),
        // property 'flagPlanetMagnitudeLimit'
        QtMocHelpers::PropertyData<bool>(102, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 10),
        // property 'customStarMagLimit'
        QtMocHelpers::PropertyData<double>(103, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable, 11),
        // property 'customNebulaMagLimit'
        QtMocHelpers::PropertyData<double>(104, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable, 12),
        // property 'customPlanetMagLimit'
        QtMocHelpers::PropertyData<double>(105, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable, 13),
        // property 'flagLuminanceAdaptation'
        QtMocHelpers::PropertyData<bool>(106, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 14),
        // property 'daylightLabelThreshold'
        QtMocHelpers::PropertyData<double>(107, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 15),
        // property 'extinctionCoefficient'
        QtMocHelpers::PropertyData<double>(108, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 16),
        // property 'atmosphereTemperature'
        QtMocHelpers::PropertyData<double>(109, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 17),
        // property 'atmospherePressure'
        QtMocHelpers::PropertyData<double>(110, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 18),
        // property 'flagDrawSunAfterAtmosphere'
        QtMocHelpers::PropertyData<bool>(111, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 20),
        // property 'flagEarlySunHalo'
        QtMocHelpers::PropertyData<bool>(112, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 21),
        // property 'flagTfromK'
        QtMocHelpers::PropertyData<bool>(113, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 19),
        // property 'turbidity'
        QtMocHelpers::PropertyData<double>(114, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable, 22),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<StelSkyDrawer, qt_meta_tag_ZN13StelSkyDrawerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject StelSkyDrawer::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13StelSkyDrawerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13StelSkyDrawerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13StelSkyDrawerE_t>.metaTypes,
    nullptr
} };

void StelSkyDrawer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<StelSkyDrawer *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->relativeStarScaleChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 1: _t->absoluteStarScaleChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 2: _t->twinkleAmountChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 3: _t->flagTwinkleChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->flagForcedTwinkleChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->lightPollutionLuminanceChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 6: _t->flagDrawBigStarHaloChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->flagStarSpikyChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->flagStarMagnitudeLimitChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 9: _t->flagNebulaMagnitudeLimitChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 10: _t->flagPlanetMagnitudeLimitChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 11: _t->customStarMagLimitChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 12: _t->customNebulaMagLimitChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 13: _t->customPlanetMagLimitChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 14: _t->flagLuminanceAdaptationChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 15: _t->daylightLabelThresholdChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 16: _t->extinctionCoefficientChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 17: _t->atmosphereTemperatureChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 18: _t->atmospherePressureChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 19: _t->flagTfromKChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 20: _t->flagDrawSunAfterAtmosphereChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 21: _t->flagEarlySunHaloChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 22: _t->turbidityChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 23: _t->setRelativeStarScale((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 24: _t->setRelativeStarScale(); break;
        case 25: { double _r = _t->getRelativeStarScale();
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 26: _t->setAbsoluteStarScale((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 27: _t->setAbsoluteStarScale(); break;
        case 28: { double _r = _t->getAbsoluteStarScale();
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 29: _t->setTwinkleAmount((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 30: { double _r = _t->getTwinkleAmount();
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 31: _t->setFlagTwinkle((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 32: { bool _r = _t->getFlagTwinkle();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 33: _t->setFlagForcedTwinkle((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 34: { bool _r = _t->getFlagForcedTwinkle();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 35: _t->setLightPollutionLuminance((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 36: { double _r = _t->getLightPollutionLuminance();
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 37: { int _r = _t->getBortleScaleIndex();
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 38: _t->setFlagDrawBigStarHalo((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 39: { bool _r = _t->getFlagDrawBigStarHalo();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 40: _t->setFlagStarSpiky((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 41: { bool _r = _t->getFlagStarSpiky();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 42: { float _r = _t->getLimitMagnitude();
            if (_a[0]) *reinterpret_cast<float*>(_a[0]) = std::move(_r); }  break;
        case 43: _t->setFlagStarMagnitudeLimit((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 44: { bool _r = _t->getFlagStarMagnitudeLimit();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 45: _t->setFlagNebulaMagnitudeLimit((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 46: { bool _r = _t->getFlagNebulaMagnitudeLimit();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 47: _t->setFlagPlanetMagnitudeLimit((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 48: { bool _r = _t->getFlagPlanetMagnitudeLimit();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 49: { double _r = _t->getCustomStarMagnitudeLimit();
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 50: _t->setCustomStarMagnitudeLimit((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 51: { double _r = _t->getCustomNebulaMagnitudeLimit();
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 52: _t->setCustomNebulaMagnitudeLimit((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 53: { double _r = _t->getCustomPlanetMagnitudeLimit();
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 54: _t->setCustomPlanetMagnitudeLimit((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 55: { float _r = _t->getLimitLuminance();
            if (_a[0]) *reinterpret_cast<float*>(_a[0]) = std::move(_r); }  break;
        case 56: _t->setFlagLuminanceAdaptation((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 57: { bool _r = _t->getFlagLuminanceAdaptation();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 58: _t->setDaylightLabelThreshold((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 59: { double _r = _t->getDaylightLabelThreshold();
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 60: { double _r = _t->getWorldAdaptationLuminance();
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 61: _t->setFlagHasAtmosphere((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 62: { bool _r = _t->getFlagHasAtmosphere();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 63: _t->setExtinctionCoefficient((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 64: { double _r = _t->getExtinctionCoefficient();
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 65: _t->setAtmosphereTemperature((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 66: { double _r = _t->getAtmosphereTemperature();
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 67: _t->setAtmospherePressure((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 68: { double _r = _t->getAtmospherePressure();
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 69: _t->getExtinction(); break;
        case 70: _t->getRefraction(); break;
        case 71: { float _r = _t->getBig3dModelHaloRadius();
            if (_a[0]) *reinterpret_cast<float*>(_a[0]) = std::move(_r); }  break;
        case 72: _t->setBig3dModelHaloRadius((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 73: _t->setFlagDrawSunAfterAtmosphere((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 74: { bool _r = _t->getFlagDrawSunAfterAtmosphere();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 75: _t->setFlagEarlySunHalo((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 76: { bool _r = _t->getFlagEarlySunHalo();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 77: { bool _r = _t->getFlagTfromK();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 78: _t->setFlagTfromK((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 79: { double _r = _t->getT();
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 80: _t->setT((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (StelSkyDrawer::*)(double )>(_a, &StelSkyDrawer::relativeStarScaleChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelSkyDrawer::*)(double )>(_a, &StelSkyDrawer::absoluteStarScaleChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelSkyDrawer::*)(double )>(_a, &StelSkyDrawer::twinkleAmountChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelSkyDrawer::*)(bool )>(_a, &StelSkyDrawer::flagTwinkleChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelSkyDrawer::*)(bool )>(_a, &StelSkyDrawer::flagForcedTwinkleChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelSkyDrawer::*)(double )>(_a, &StelSkyDrawer::lightPollutionLuminanceChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelSkyDrawer::*)(bool )>(_a, &StelSkyDrawer::flagDrawBigStarHaloChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelSkyDrawer::*)(bool )>(_a, &StelSkyDrawer::flagStarSpikyChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelSkyDrawer::*)(bool )>(_a, &StelSkyDrawer::flagStarMagnitudeLimitChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelSkyDrawer::*)(bool )>(_a, &StelSkyDrawer::flagNebulaMagnitudeLimitChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelSkyDrawer::*)(bool )>(_a, &StelSkyDrawer::flagPlanetMagnitudeLimitChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelSkyDrawer::*)(double )>(_a, &StelSkyDrawer::customStarMagLimitChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelSkyDrawer::*)(double )>(_a, &StelSkyDrawer::customNebulaMagLimitChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelSkyDrawer::*)(double )>(_a, &StelSkyDrawer::customPlanetMagLimitChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelSkyDrawer::*)(bool )>(_a, &StelSkyDrawer::flagLuminanceAdaptationChanged, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelSkyDrawer::*)(double )>(_a, &StelSkyDrawer::daylightLabelThresholdChanged, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelSkyDrawer::*)(double )>(_a, &StelSkyDrawer::extinctionCoefficientChanged, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelSkyDrawer::*)(double )>(_a, &StelSkyDrawer::atmosphereTemperatureChanged, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelSkyDrawer::*)(double )>(_a, &StelSkyDrawer::atmospherePressureChanged, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelSkyDrawer::*)(bool )>(_a, &StelSkyDrawer::flagTfromKChanged, 19))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelSkyDrawer::*)(bool )>(_a, &StelSkyDrawer::flagDrawSunAfterAtmosphereChanged, 20))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelSkyDrawer::*)(bool )>(_a, &StelSkyDrawer::flagEarlySunHaloChanged, 21))
            return;
        if (QtMocHelpers::indexOfMethod<void (StelSkyDrawer::*)(double )>(_a, &StelSkyDrawer::turbidityChanged, 22))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<double*>(_v) = _t->getRelativeStarScale(); break;
        case 1: *reinterpret_cast<double*>(_v) = _t->getAbsoluteStarScale(); break;
        case 2: *reinterpret_cast<double*>(_v) = _t->getTwinkleAmount(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->getFlagTwinkle(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->getFlagForcedTwinkle(); break;
        case 5: *reinterpret_cast<double*>(_v) = _t->getLightPollutionLuminance(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->getFlagDrawBigStarHalo(); break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->getFlagStarSpiky(); break;
        case 8: *reinterpret_cast<bool*>(_v) = _t->getFlagStarMagnitudeLimit(); break;
        case 9: *reinterpret_cast<bool*>(_v) = _t->getFlagNebulaMagnitudeLimit(); break;
        case 10: *reinterpret_cast<bool*>(_v) = _t->getFlagPlanetMagnitudeLimit(); break;
        case 11: *reinterpret_cast<double*>(_v) = _t->getCustomStarMagnitudeLimit(); break;
        case 12: *reinterpret_cast<double*>(_v) = _t->getCustomNebulaMagnitudeLimit(); break;
        case 13: *reinterpret_cast<double*>(_v) = _t->getCustomPlanetMagnitudeLimit(); break;
        case 14: *reinterpret_cast<bool*>(_v) = _t->getFlagLuminanceAdaptation(); break;
        case 15: *reinterpret_cast<double*>(_v) = _t->getDaylightLabelThreshold(); break;
        case 16: *reinterpret_cast<double*>(_v) = _t->getExtinctionCoefficient(); break;
        case 17: *reinterpret_cast<double*>(_v) = _t->getAtmosphereTemperature(); break;
        case 18: *reinterpret_cast<double*>(_v) = _t->getAtmospherePressure(); break;
        case 19: *reinterpret_cast<bool*>(_v) = _t->getFlagDrawSunAfterAtmosphere(); break;
        case 20: *reinterpret_cast<bool*>(_v) = _t->getFlagEarlySunHalo(); break;
        case 21: *reinterpret_cast<bool*>(_v) = _t->getFlagTfromK(); break;
        case 22: *reinterpret_cast<double*>(_v) = _t->getT(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setRelativeStarScale(*reinterpret_cast<double*>(_v)); break;
        case 1: _t->setAbsoluteStarScale(*reinterpret_cast<double*>(_v)); break;
        case 2: _t->setTwinkleAmount(*reinterpret_cast<double*>(_v)); break;
        case 3: _t->setFlagTwinkle(*reinterpret_cast<bool*>(_v)); break;
        case 4: _t->setFlagForcedTwinkle(*reinterpret_cast<bool*>(_v)); break;
        case 5: _t->setLightPollutionLuminance(*reinterpret_cast<double*>(_v)); break;
        case 6: _t->setFlagDrawBigStarHalo(*reinterpret_cast<bool*>(_v)); break;
        case 7: _t->setFlagStarSpiky(*reinterpret_cast<bool*>(_v)); break;
        case 8: _t->setFlagStarMagnitudeLimit(*reinterpret_cast<bool*>(_v)); break;
        case 9: _t->setFlagNebulaMagnitudeLimit(*reinterpret_cast<bool*>(_v)); break;
        case 10: _t->setFlagPlanetMagnitudeLimit(*reinterpret_cast<bool*>(_v)); break;
        case 11: _t->setCustomStarMagnitudeLimit(*reinterpret_cast<double*>(_v)); break;
        case 12: _t->setCustomNebulaMagnitudeLimit(*reinterpret_cast<double*>(_v)); break;
        case 13: _t->setCustomPlanetMagnitudeLimit(*reinterpret_cast<double*>(_v)); break;
        case 14: _t->setFlagLuminanceAdaptation(*reinterpret_cast<bool*>(_v)); break;
        case 15: _t->setDaylightLabelThreshold(*reinterpret_cast<double*>(_v)); break;
        case 16: _t->setExtinctionCoefficient(*reinterpret_cast<double*>(_v)); break;
        case 17: _t->setAtmosphereTemperature(*reinterpret_cast<double*>(_v)); break;
        case 18: _t->setAtmospherePressure(*reinterpret_cast<double*>(_v)); break;
        case 19: _t->setFlagDrawSunAfterAtmosphere(*reinterpret_cast<bool*>(_v)); break;
        case 20: _t->setFlagEarlySunHalo(*reinterpret_cast<bool*>(_v)); break;
        case 21: _t->setFlagTfromK(*reinterpret_cast<bool*>(_v)); break;
        case 22: _t->setT(*reinterpret_cast<double*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *StelSkyDrawer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *StelSkyDrawer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13StelSkyDrawerE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QOpenGLFunctions"))
        return static_cast< QOpenGLFunctions*>(this);
    return QObject::qt_metacast(_clname);
}

int StelSkyDrawer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 81)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 81;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 81)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 81;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 23;
    }
    return _id;
}

// SIGNAL 0
void StelSkyDrawer::relativeStarScaleChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void StelSkyDrawer::absoluteStarScaleChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void StelSkyDrawer::twinkleAmountChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void StelSkyDrawer::flagTwinkleChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void StelSkyDrawer::flagForcedTwinkleChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void StelSkyDrawer::lightPollutionLuminanceChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void StelSkyDrawer::flagDrawBigStarHaloChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void StelSkyDrawer::flagStarSpikyChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void StelSkyDrawer::flagStarMagnitudeLimitChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void StelSkyDrawer::flagNebulaMagnitudeLimitChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void StelSkyDrawer::flagPlanetMagnitudeLimitChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void StelSkyDrawer::customStarMagLimitChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void StelSkyDrawer::customNebulaMagLimitChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void StelSkyDrawer::customPlanetMagLimitChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1);
}

// SIGNAL 14
void StelSkyDrawer::flagLuminanceAdaptationChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1);
}

// SIGNAL 15
void StelSkyDrawer::daylightLabelThresholdChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 15, nullptr, _t1);
}

// SIGNAL 16
void StelSkyDrawer::extinctionCoefficientChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 16, nullptr, _t1);
}

// SIGNAL 17
void StelSkyDrawer::atmosphereTemperatureChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 17, nullptr, _t1);
}

// SIGNAL 18
void StelSkyDrawer::atmospherePressureChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 18, nullptr, _t1);
}

// SIGNAL 19
void StelSkyDrawer::flagTfromKChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 19, nullptr, _t1);
}

// SIGNAL 20
void StelSkyDrawer::flagDrawSunAfterAtmosphereChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 20, nullptr, _t1);
}

// SIGNAL 21
void StelSkyDrawer::flagEarlySunHaloChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 21, nullptr, _t1);
}

// SIGNAL 22
void StelSkyDrawer::turbidityChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 22, nullptr, _t1);
}
QT_WARNING_POP
