/****************************************************************************
** Meta object code from reading C++ file 'SearchDialog.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../stellarium/src/gui/SearchDialog.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'SearchDialog.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN19CompletionListModelE_t {};
} // unnamed namespace

template <> constexpr inline auto CompletionListModel::qt_create_metaobjectdata<qt_meta_tag_ZN19CompletionListModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "CompletionListModel",
        "selectNext",
        "",
        "selectPrevious",
        "selectFirst"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'selectNext'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'selectPrevious'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'selectFirst'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CompletionListModel, qt_meta_tag_ZN19CompletionListModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject CompletionListModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QStringListModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19CompletionListModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19CompletionListModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN19CompletionListModelE_t>.metaTypes,
    nullptr
} };

void CompletionListModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CompletionListModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->selectNext(); break;
        case 1: _t->selectPrevious(); break;
        case 2: _t->selectFirst(); break;
        default: ;
        }
    }
    (void)_a;
}

const QMetaObject *CompletionListModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CompletionListModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19CompletionListModelE_t>.strings))
        return static_cast<void*>(this);
    return QStringListModel::qt_metacast(_clname);
}

int CompletionListModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QStringListModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 3;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN12SearchDialogE_t {};
} // unnamed namespace

template <> constexpr inline auto SearchDialog::qt_create_metaobjectdata<qt_meta_tag_ZN12SearchDialogE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "SearchDialog",
        "simbadUseChanged",
        "",
        "use",
        "simbadQueryDistChanged",
        "dist",
        "simbadQueryCountChanged",
        "count",
        "simbadGetsIdsChanged",
        "b",
        "simbadGetsSpecChanged",
        "simbadGetsDistChanged",
        "simbadGetsMorphoChanged",
        "simbadGetsTypesChanged",
        "simbadGetsDimsChanged",
        "retranslate",
        "setVisible",
        "v",
        "setSimpleStyle",
        "setCurrentCoordinateSystem",
        "SearchDialog::CoordinateSystem",
        "cs",
        "setCurrentCoordinateSystemKey",
        "key",
        "setRecentSearchSize",
        "maxSize",
        "setCoordinateSystem",
        "csID",
        "populateCoordinateSystemsList",
        "populateCoordinateAxis",
        "populateCoordinateData",
        "populateRecentSearch",
        "greekLetterClicked",
        "lookupCoordinates",
        "clearSimbadText",
        "StelModule::StelModuleSelectAction",
        "onSimbadStatusChanged",
        "onSearchTextChanged",
        "text",
        "gotoObject",
        "ObjectFound",
        "obj",
        "gotoObjectFromSearchResults",
        "QModelIndex",
        "modelIndex",
        "gotoObjectFromObjectsList",
        "searchListClear",
        "refreshFocus",
        "state",
        "manualPositionChanged",
        "enableSimbadSearch",
        "enable",
        "enableStartOfWordsAutofill",
        "enableSortingByLength",
        "enableLockPosition",
        "enableAutoClosing",
        "enableFOVCenterMarker",
        "setHasSelectedFlag",
        "selectSimbadServer",
        "index",
        "updateListView",
        "updateListTab",
        "showContextMenu",
        "QPoint",
        "pt",
        "pasteAndGo",
        "changeTab",
        "setSimbadQueryDist",
        "setSimbadQueryCount",
        "setSimbadGetsIds",
        "setSimbadGetsSpec",
        "setSimbadGetsMorpho",
        "setSimbadGetsTypes",
        "setSimbadGetsDims",
        "recentSearchSizeEditingFinished",
        "recentSearchClearDataClicked",
        "setCenterOfScreenCoordinates",
        "useSimbad",
        "simbadDist",
        "simbadCount",
        "simbadGetIds",
        "simbadGetSpec",
        "simbadGetMorpho",
        "simbadGetTypes",
        "simbadGetDims",
        "CoordinateSystem",
        "equatorialJ2000",
        "equatorial",
        "horizontal",
        "galactic",
        "supergalactic",
        "ecliptic",
        "eclipticJ2000"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'simbadUseChanged'
        QtMocHelpers::SignalData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'simbadQueryDistChanged'
        QtMocHelpers::SignalData<void(int)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 5 },
        }}),
        // Signal 'simbadQueryCountChanged'
        QtMocHelpers::SignalData<void(int)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 7 },
        }}),
        // Signal 'simbadGetsIdsChanged'
        QtMocHelpers::SignalData<void(bool)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 9 },
        }}),
        // Signal 'simbadGetsSpecChanged'
        QtMocHelpers::SignalData<void(bool)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 9 },
        }}),
        // Signal 'simbadGetsDistChanged'
        QtMocHelpers::SignalData<void(bool)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 9 },
        }}),
        // Signal 'simbadGetsMorphoChanged'
        QtMocHelpers::SignalData<void(bool)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 9 },
        }}),
        // Signal 'simbadGetsTypesChanged'
        QtMocHelpers::SignalData<void(bool)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 9 },
        }}),
        // Signal 'simbadGetsDimsChanged'
        QtMocHelpers::SignalData<void(bool)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 9 },
        }}),
        // Slot 'retranslate'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setVisible'
        QtMocHelpers::SlotData<void(bool)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 17 },
        }}),
        // Slot 'setSimpleStyle'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setCurrentCoordinateSystem'
        QtMocHelpers::SlotData<void(SearchDialog::CoordinateSystem)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 20, 21 },
        }}),
        // Slot 'setCurrentCoordinateSystemKey'
        QtMocHelpers::SlotData<void(QString)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 23 },
        }}),
        // Slot 'setRecentSearchSize'
        QtMocHelpers::SlotData<void(int)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 25 },
        }}),
        // Slot 'setCoordinateSystem'
        QtMocHelpers::SlotData<void(int)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 27 },
        }}),
        // Slot 'populateCoordinateSystemsList'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'populateCoordinateAxis'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'populateCoordinateData'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'populateRecentSearch'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'greekLetterClicked'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'lookupCoordinates'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'clearSimbadText'
        QtMocHelpers::SlotData<void(StelModule::StelModuleSelectAction)>(34, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 35, 2 },
        }}),
        // Slot 'onSimbadStatusChanged'
        QtMocHelpers::SlotData<void()>(36, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onSearchTextChanged'
        QtMocHelpers::SlotData<void(const QString &)>(37, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 38 },
        }}),
        // Slot 'gotoObject'
        QtMocHelpers::SlotData<void()>(39, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'gotoObject'
        QtMocHelpers::SlotData<void(const ObjectFound &)>(39, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 40, 41 },
        }}),
        // Slot 'gotoObjectFromSearchResults'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(42, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 43, 44 },
        }}),
        // Slot 'gotoObjectFromObjectsList'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(45, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 43, 44 },
        }}),
        // Slot 'searchListClear'
        QtMocHelpers::SlotData<void()>(46, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'refreshFocus'
        QtMocHelpers::SlotData<void(bool)>(47, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 48 },
        }}),
        // Slot 'manualPositionChanged'
        QtMocHelpers::SlotData<void()>(49, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'enableSimbadSearch'
        QtMocHelpers::SlotData<void(bool)>(50, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 51 },
        }}),
        // Slot 'enableStartOfWordsAutofill'
        QtMocHelpers::SlotData<void(bool)>(52, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 51 },
        }}),
        // Slot 'enableSortingByLength'
        QtMocHelpers::SlotData<void(bool)>(53, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 51 },
        }}),
        // Slot 'enableLockPosition'
        QtMocHelpers::SlotData<void(bool)>(54, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 51 },
        }}),
        // Slot 'enableAutoClosing'
        QtMocHelpers::SlotData<void(bool)>(55, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 51 },
        }}),
        // Slot 'enableFOVCenterMarker'
        QtMocHelpers::SlotData<void(bool)>(56, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 51 },
        }}),
        // Slot 'setHasSelectedFlag'
        QtMocHelpers::SlotData<void()>(57, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'selectSimbadServer'
        QtMocHelpers::SlotData<void(int)>(58, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 59 },
        }}),
        // Slot 'updateListView'
        QtMocHelpers::SlotData<void(int)>(60, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 59 },
        }}),
        // Slot 'updateListTab'
        QtMocHelpers::SlotData<void()>(61, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'showContextMenu'
        QtMocHelpers::SlotData<void(const QPoint &)>(62, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 63, 64 },
        }}),
        // Slot 'pasteAndGo'
        QtMocHelpers::SlotData<void()>(65, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'changeTab'
        QtMocHelpers::SlotData<void(int)>(66, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 59 },
        }}),
        // Slot 'setSimbadQueryDist'
        QtMocHelpers::SlotData<void(int)>(67, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 5 },
        }}),
        // Slot 'setSimbadQueryCount'
        QtMocHelpers::SlotData<void(int)>(68, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 7 },
        }}),
        // Slot 'setSimbadGetsIds'
        QtMocHelpers::SlotData<void(bool)>(69, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 9 },
        }}),
        // Slot 'setSimbadGetsSpec'
        QtMocHelpers::SlotData<void(bool)>(70, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 9 },
        }}),
        // Slot 'setSimbadGetsMorpho'
        QtMocHelpers::SlotData<void(bool)>(71, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 9 },
        }}),
        // Slot 'setSimbadGetsTypes'
        QtMocHelpers::SlotData<void(bool)>(72, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 9 },
        }}),
        // Slot 'setSimbadGetsDims'
        QtMocHelpers::SlotData<void(bool)>(73, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 9 },
        }}),
        // Slot 'recentSearchSizeEditingFinished'
        QtMocHelpers::SlotData<void()>(74, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'recentSearchClearDataClicked'
        QtMocHelpers::SlotData<void()>(75, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setCenterOfScreenCoordinates'
        QtMocHelpers::SlotData<void()>(76, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'useSimbad'
        QtMocHelpers::PropertyData<bool>(77, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 0),
        // property 'simbadDist'
        QtMocHelpers::PropertyData<int>(78, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable, 1),
        // property 'simbadCount'
        QtMocHelpers::PropertyData<int>(79, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable, 2),
        // property 'simbadGetIds'
        QtMocHelpers::PropertyData<bool>(80, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 3),
        // property 'simbadGetSpec'
        QtMocHelpers::PropertyData<bool>(81, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 4),
        // property 'simbadGetMorpho'
        QtMocHelpers::PropertyData<bool>(82, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 6),
        // property 'simbadGetTypes'
        QtMocHelpers::PropertyData<bool>(83, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 7),
        // property 'simbadGetDims'
        QtMocHelpers::PropertyData<bool>(84, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 8),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'CoordinateSystem'
        QtMocHelpers::EnumData<enum CoordinateSystem>(85, 85, QMC::EnumFlags{}).add({
            {   86, CoordinateSystem::equatorialJ2000 },
            {   87, CoordinateSystem::equatorial },
            {   88, CoordinateSystem::horizontal },
            {   89, CoordinateSystem::galactic },
            {   90, CoordinateSystem::supergalactic },
            {   91, CoordinateSystem::ecliptic },
            {   92, CoordinateSystem::eclipticJ2000 },
        }),
    };
    return QtMocHelpers::metaObjectData<SearchDialog, qt_meta_tag_ZN12SearchDialogE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject SearchDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<StelDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12SearchDialogE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12SearchDialogE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12SearchDialogE_t>.metaTypes,
    nullptr
} };

void SearchDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SearchDialog *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->simbadUseChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->simbadQueryDistChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->simbadQueryCountChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 3: _t->simbadGetsIdsChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->simbadGetsSpecChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->simbadGetsDistChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->simbadGetsMorphoChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->simbadGetsTypesChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->simbadGetsDimsChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 9: _t->retranslate(); break;
        case 10: _t->setVisible((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 11: _t->setSimpleStyle(); break;
        case 12: _t->setCurrentCoordinateSystem((*reinterpret_cast<std::add_pointer_t<SearchDialog::CoordinateSystem>>(_a[1]))); break;
        case 13: _t->setCurrentCoordinateSystemKey((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 14: _t->setRecentSearchSize((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 15: _t->setCoordinateSystem((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 16: _t->populateCoordinateSystemsList(); break;
        case 17: _t->populateCoordinateAxis(); break;
        case 18: _t->populateCoordinateData(); break;
        case 19: _t->populateRecentSearch(); break;
        case 20: _t->greekLetterClicked(); break;
        case 21: _t->lookupCoordinates(); break;
        case 22: _t->clearSimbadText((*reinterpret_cast<std::add_pointer_t<StelModule::StelModuleSelectAction>>(_a[1]))); break;
        case 23: _t->onSimbadStatusChanged(); break;
        case 24: _t->onSearchTextChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 25: _t->gotoObject(); break;
        case 26: _t->gotoObject((*reinterpret_cast<std::add_pointer_t<ObjectFound>>(_a[1]))); break;
        case 27: _t->gotoObjectFromSearchResults((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 28: _t->gotoObjectFromObjectsList((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 29: _t->searchListClear(); break;
        case 30: _t->refreshFocus((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 31: _t->manualPositionChanged(); break;
        case 32: _t->enableSimbadSearch((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 33: _t->enableStartOfWordsAutofill((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 34: _t->enableSortingByLength((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 35: _t->enableLockPosition((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 36: _t->enableAutoClosing((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 37: _t->enableFOVCenterMarker((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 38: _t->setHasSelectedFlag(); break;
        case 39: _t->selectSimbadServer((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 40: _t->updateListView((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 41: _t->updateListTab(); break;
        case 42: _t->showContextMenu((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 43: _t->pasteAndGo(); break;
        case 44: _t->changeTab((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 45: _t->setSimbadQueryDist((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 46: _t->setSimbadQueryCount((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 47: _t->setSimbadGetsIds((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 48: _t->setSimbadGetsSpec((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 49: _t->setSimbadGetsMorpho((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 50: _t->setSimbadGetsTypes((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 51: _t->setSimbadGetsDims((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 52: _t->recentSearchSizeEditingFinished(); break;
        case 53: _t->recentSearchClearDataClicked(); break;
        case 54: _t->setCenterOfScreenCoordinates(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 22:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< StelModule::StelModuleSelectAction >(); break;
            }
            break;
        case 26:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< ObjectFound >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (SearchDialog::*)(bool )>(_a, &SearchDialog::simbadUseChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (SearchDialog::*)(int )>(_a, &SearchDialog::simbadQueryDistChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (SearchDialog::*)(int )>(_a, &SearchDialog::simbadQueryCountChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (SearchDialog::*)(bool )>(_a, &SearchDialog::simbadGetsIdsChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (SearchDialog::*)(bool )>(_a, &SearchDialog::simbadGetsSpecChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (SearchDialog::*)(bool )>(_a, &SearchDialog::simbadGetsDistChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (SearchDialog::*)(bool )>(_a, &SearchDialog::simbadGetsMorphoChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (SearchDialog::*)(bool )>(_a, &SearchDialog::simbadGetsTypesChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (SearchDialog::*)(bool )>(_a, &SearchDialog::simbadGetsDimsChanged, 8))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->simbadSearchEnabled(); break;
        case 1: *reinterpret_cast<int*>(_v) = _t->getSimbadQueryDist(); break;
        case 2: *reinterpret_cast<int*>(_v) = _t->getSimbadQueryCount(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->getSimbadGetsIds(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->getSimbadGetsSpec(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->getSimbadGetsMorpho(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->getSimbadGetsTypes(); break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->getSimbadGetsDims(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->enableSimbadSearch(*reinterpret_cast<bool*>(_v)); break;
        case 1: _t->setSimbadQueryDist(*reinterpret_cast<int*>(_v)); break;
        case 2: _t->setSimbadQueryCount(*reinterpret_cast<int*>(_v)); break;
        case 3: _t->setSimbadGetsIds(*reinterpret_cast<bool*>(_v)); break;
        case 4: _t->setSimbadGetsSpec(*reinterpret_cast<bool*>(_v)); break;
        case 5: _t->setSimbadGetsMorpho(*reinterpret_cast<bool*>(_v)); break;
        case 6: _t->setSimbadGetsTypes(*reinterpret_cast<bool*>(_v)); break;
        case 7: _t->setSimbadGetsDims(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *SearchDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SearchDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12SearchDialogE_t>.strings))
        return static_cast<void*>(this);
    return StelDialog::qt_metacast(_clname);
}

int SearchDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StelDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 55)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 55;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 55)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 55;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void SearchDialog::simbadUseChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void SearchDialog::simbadQueryDistChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void SearchDialog::simbadQueryCountChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void SearchDialog::simbadGetsIdsChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void SearchDialog::simbadGetsSpecChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void SearchDialog::simbadGetsDistChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void SearchDialog::simbadGetsMorphoChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void SearchDialog::simbadGetsTypesChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void SearchDialog::simbadGetsDimsChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}
QT_WARNING_POP
