/********************************************************************************
** Form generated from reading UI file 'storedPointsDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_STOREDPOINTSDIALOG_H
#define UI_STOREDPOINTSDIALOG_H

#include <AngleSpinBox.hpp>
#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QTreeView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "Dialog.hpp"
#include "gui/StelCloseButton.hpp"

QT_BEGIN_NAMESPACE

class Ui_StoredPoints
{
public:
    QVBoxLayout *verticalLayout_4;
    TitleBar *titleBar;
    QTreeView *treeViewStoredPoints;
    QHBoxLayout *horizontalLayout_2;
    QPushButton *pushButtonCurrent;
    QPushButton *pushButtonCenter;
    QFrame *frame;
    QVBoxLayout *verticalLayout_3;
    QHBoxLayout *horizontalLayout_3;
    QLabel *labelPointName;
    QLineEdit *lineEditStoredPointName;
    QHBoxLayout *horizontalLayout;
    QVBoxLayout *verticalLayout;
    QLabel *labelRA;
    QLabel *labelDec;
    QVBoxLayout *verticalLayout_2;
    AngleSpinBox *spinBoxRA;
    AngleSpinBox *spinBoxDec;
    QHBoxLayout *horizontalLayout_6;
    QPushButton *pushButtonAddPoint;
    QPushButton *pushButtonRemovePoint;
    QPushButton *pushButtonClearList;

    void setupUi(QWidget *StoredPoints)
    {
        if (StoredPoints->objectName().isEmpty())
            StoredPoints->setObjectName("StoredPoints");
        StoredPoints->resize(540, 449);
        verticalLayout_4 = new QVBoxLayout(StoredPoints);
        verticalLayout_4->setSpacing(0);
        verticalLayout_4->setObjectName("verticalLayout_4");
        verticalLayout_4->setContentsMargins(0, 0, 0, 0);
        titleBar = new TitleBar(StoredPoints);
        titleBar->setObjectName("titleBar");

        verticalLayout_4->addWidget(titleBar);

        treeViewStoredPoints = new QTreeView(StoredPoints);
        treeViewStoredPoints->setObjectName("treeViewStoredPoints");

        verticalLayout_4->addWidget(treeViewStoredPoints);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        pushButtonCurrent = new QPushButton(StoredPoints);
        pushButtonCurrent->setObjectName("pushButtonCurrent");

        horizontalLayout_2->addWidget(pushButtonCurrent);

        pushButtonCenter = new QPushButton(StoredPoints);
        pushButtonCenter->setObjectName("pushButtonCenter");

        horizontalLayout_2->addWidget(pushButtonCenter);


        verticalLayout_4->addLayout(horizontalLayout_2);

        frame = new QFrame(StoredPoints);
        frame->setObjectName("frame");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(frame->sizePolicy().hasHeightForWidth());
        frame->setSizePolicy(sizePolicy);
        frame->setFrameShape(QFrame::StyledPanel);
        frame->setFrameShadow(QFrame::Raised);
        verticalLayout_3 = new QVBoxLayout(frame);
        verticalLayout_3->setObjectName("verticalLayout_3");
        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        labelPointName = new QLabel(frame);
        labelPointName->setObjectName("labelPointName");

        horizontalLayout_3->addWidget(labelPointName);

        lineEditStoredPointName = new QLineEdit(frame);
        lineEditStoredPointName->setObjectName("lineEditStoredPointName");

        horizontalLayout_3->addWidget(lineEditStoredPointName);


        verticalLayout_3->addLayout(horizontalLayout_3);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName("verticalLayout");
        labelRA = new QLabel(frame);
        labelRA->setObjectName("labelRA");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(labelRA->sizePolicy().hasHeightForWidth());
        labelRA->setSizePolicy(sizePolicy1);

        verticalLayout->addWidget(labelRA);

        labelDec = new QLabel(frame);
        labelDec->setObjectName("labelDec");
        sizePolicy1.setHeightForWidth(labelDec->sizePolicy().hasHeightForWidth());
        labelDec->setSizePolicy(sizePolicy1);

        verticalLayout->addWidget(labelDec);


        horizontalLayout->addLayout(verticalLayout);

        verticalLayout_2 = new QVBoxLayout();
        verticalLayout_2->setObjectName("verticalLayout_2");
        spinBoxRA = new AngleSpinBox(frame);
        spinBoxRA->setObjectName("spinBoxRA");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(spinBoxRA->sizePolicy().hasHeightForWidth());
        spinBoxRA->setSizePolicy(sizePolicy2);

        verticalLayout_2->addWidget(spinBoxRA);

        spinBoxDec = new AngleSpinBox(frame);
        spinBoxDec->setObjectName("spinBoxDec");
        sizePolicy2.setHeightForWidth(spinBoxDec->sizePolicy().hasHeightForWidth());
        spinBoxDec->setSizePolicy(sizePolicy2);

        verticalLayout_2->addWidget(spinBoxDec);


        horizontalLayout->addLayout(verticalLayout_2);


        verticalLayout_3->addLayout(horizontalLayout);

        horizontalLayout_6 = new QHBoxLayout();
        horizontalLayout_6->setObjectName("horizontalLayout_6");
        pushButtonAddPoint = new QPushButton(frame);
        pushButtonAddPoint->setObjectName("pushButtonAddPoint");

        horizontalLayout_6->addWidget(pushButtonAddPoint);

        pushButtonRemovePoint = new QPushButton(frame);
        pushButtonRemovePoint->setObjectName("pushButtonRemovePoint");

        horizontalLayout_6->addWidget(pushButtonRemovePoint);

        pushButtonClearList = new QPushButton(frame);
        pushButtonClearList->setObjectName("pushButtonClearList");

        horizontalLayout_6->addWidget(pushButtonClearList);


        verticalLayout_3->addLayout(horizontalLayout_6);


        verticalLayout_4->addWidget(frame);

#if QT_CONFIG(shortcut)
        labelRA->setBuddy(spinBoxRA);
        labelDec->setBuddy(spinBoxDec);
#endif // QT_CONFIG(shortcut)

        retranslateUi(StoredPoints);

        QMetaObject::connectSlotsByName(StoredPoints);
    } // setupUi

    void retranslateUi(QWidget *StoredPoints)
    {
        StoredPoints->setWindowTitle(QString());
        titleBar->setTitle(QCoreApplication::translate("StoredPoints", "Points", nullptr));
        pushButtonCurrent->setText(QCoreApplication::translate("StoredPoints", "&Current object", nullptr));
        pushButtonCenter->setText(QCoreApplication::translate("StoredPoints", "Center of the screen", nullptr));
        labelPointName->setText(QCoreApplication::translate("StoredPoints", "Name:", nullptr));
        labelRA->setText(QCoreApplication::translate("StoredPoints", "&Right Ascension (J2000):", nullptr));
        labelDec->setText(QCoreApplication::translate("StoredPoints", "De&clination (J2000):", nullptr));
        pushButtonAddPoint->setText(QCoreApplication::translate("StoredPoints", "Add point", nullptr));
        pushButtonRemovePoint->setText(QCoreApplication::translate("StoredPoints", "Remove selected", nullptr));
        pushButtonClearList->setText(QCoreApplication::translate("StoredPoints", "Clear list", nullptr));
    } // retranslateUi

};

namespace Ui {
    class StoredPoints: public Ui_StoredPoints {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_STOREDPOINTSDIALOG_H
