/********************************************************************************
** Form generated from reading UI file 'shortcutsDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SHORTCUTSDIALOG_H
#define UI_SHORTCUTSDIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QTreeView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "Dialog.hpp"
#include "ShortcutsDialog.hpp"
#include "gui/StelCloseButton.hpp"

QT_BEGIN_NAMESPACE

class Ui_shortcutsDialogForm
{
public:
    QVBoxLayout *verticalLayout;
    TitleBar *titleBar;
    QFrame *mainFrame;
    QVBoxLayout *verticalLayout_2;
    QTreeView *shortcutsTreeView;
    QLineEdit *lineEditSearch;
    QFrame *editFrame;
    QGridLayout *gridLayout;
    QLabel *primaryLabel;
    QLabel *altLabel;
    QPushButton *primaryBackspaceButton;
    QPushButton *restoreDefaultsButton;
    QPushButton *applyButton;
    QPushButton *altBackspaceButton;
    ShortcutLineEdit *primaryShortcutEdit;
    ShortcutLineEdit *altShortcutEdit;
    QPushButton *restoreAllDefaultsButton;

    void setupUi(QWidget *shortcutsDialogForm)
    {
        if (shortcutsDialogForm->objectName().isEmpty())
            shortcutsDialogForm->setObjectName("shortcutsDialogForm");
        shortcutsDialogForm->resize(654, 436);
        shortcutsDialogForm->setWindowTitle(QString::fromUtf8(""));
        verticalLayout = new QVBoxLayout(shortcutsDialogForm);
        verticalLayout->setSpacing(0);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        titleBar = new TitleBar(shortcutsDialogForm);
        titleBar->setObjectName("titleBar");

        verticalLayout->addWidget(titleBar);

        mainFrame = new QFrame(shortcutsDialogForm);
        mainFrame->setObjectName("mainFrame");
        mainFrame->setFrameShape(QFrame::StyledPanel);
        mainFrame->setFrameShadow(QFrame::Raised);
        verticalLayout_2 = new QVBoxLayout(mainFrame);
        verticalLayout_2->setSpacing(0);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        shortcutsTreeView = new QTreeView(mainFrame);
        shortcutsTreeView->setObjectName("shortcutsTreeView");
        shortcutsTreeView->setUniformRowHeights(true);
        shortcutsTreeView->setSortingEnabled(true);
        shortcutsTreeView->setAllColumnsShowFocus(true);
        shortcutsTreeView->header()->setMinimumSectionSize(100);
        shortcutsTreeView->header()->setDefaultSectionSize(200);
        shortcutsTreeView->header()->setProperty("showSortIndicator", QVariant(true));

        verticalLayout_2->addWidget(shortcutsTreeView);

        lineEditSearch = new QLineEdit(mainFrame);
        lineEditSearch->setObjectName("lineEditSearch");

        verticalLayout_2->addWidget(lineEditSearch);

        editFrame = new QFrame(mainFrame);
        editFrame->setObjectName("editFrame");
        editFrame->setFrameShape(QFrame::StyledPanel);
        gridLayout = new QGridLayout(editFrame);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setSizeConstraint(QLayout::SetMaximumSize);
        gridLayout->setContentsMargins(9, -1, 9, 9);
        primaryLabel = new QLabel(editFrame);
        primaryLabel->setObjectName("primaryLabel");

        gridLayout->addWidget(primaryLabel, 1, 0, 1, 1);

        altLabel = new QLabel(editFrame);
        altLabel->setObjectName("altLabel");

        gridLayout->addWidget(altLabel, 2, 0, 1, 1);

        primaryBackspaceButton = new QPushButton(editFrame);
        primaryBackspaceButton->setObjectName("primaryBackspaceButton");
        primaryBackspaceButton->setEnabled(false);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/graphicGui/uibtBackspace.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        icon.addFile(QString::fromUtf8(":/graphicGui/uibtBackspace-disabled.png"), QSize(), QIcon::Mode::Disabled, QIcon::State::Off);
        primaryBackspaceButton->setIcon(icon);

        gridLayout->addWidget(primaryBackspaceButton, 1, 2, 1, 1);

        restoreDefaultsButton = new QPushButton(editFrame);
        restoreDefaultsButton->setObjectName("restoreDefaultsButton");
        restoreDefaultsButton->setEnabled(false);

        gridLayout->addWidget(restoreDefaultsButton, 2, 3, 1, 1);

        applyButton = new QPushButton(editFrame);
        applyButton->setObjectName("applyButton");
        applyButton->setEnabled(false);

        gridLayout->addWidget(applyButton, 1, 3, 1, 1);

        altBackspaceButton = new QPushButton(editFrame);
        altBackspaceButton->setObjectName("altBackspaceButton");
        altBackspaceButton->setEnabled(false);
        altBackspaceButton->setIcon(icon);

        gridLayout->addWidget(altBackspaceButton, 2, 2, 1, 1);

        primaryShortcutEdit = new ShortcutLineEdit(editFrame);
        primaryShortcutEdit->setObjectName("primaryShortcutEdit");
        primaryShortcutEdit->setEnabled(false);

        gridLayout->addWidget(primaryShortcutEdit, 1, 1, 1, 1);

        altShortcutEdit = new ShortcutLineEdit(editFrame);
        altShortcutEdit->setObjectName("altShortcutEdit");
        altShortcutEdit->setEnabled(false);

        gridLayout->addWidget(altShortcutEdit, 2, 1, 1, 1);

        restoreAllDefaultsButton = new QPushButton(editFrame);
        restoreAllDefaultsButton->setObjectName("restoreAllDefaultsButton");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::MinimumExpanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(restoreAllDefaultsButton->sizePolicy().hasHeightForWidth());
        restoreAllDefaultsButton->setSizePolicy(sizePolicy);

        gridLayout->addWidget(restoreAllDefaultsButton, 1, 4, 2, 1);


        verticalLayout_2->addWidget(editFrame);


        verticalLayout->addWidget(mainFrame);


        retranslateUi(shortcutsDialogForm);
        QObject::connect(titleBar, &TitleBar::closeClicked, shortcutsDialogForm, qOverload<>(&QWidget::close));
        QObject::connect(primaryBackspaceButton, &QPushButton::clicked, primaryShortcutEdit, qOverload<>(&ShortcutLineEdit::backspace));
        QObject::connect(altBackspaceButton, &QPushButton::clicked, altShortcutEdit, qOverload<>(&ShortcutLineEdit::backspace));
        QObject::connect(primaryBackspaceButton, &QPushButton::clicked, primaryShortcutEdit, qOverload<>(&ShortcutLineEdit::setFocus));
        QObject::connect(altBackspaceButton, &QPushButton::clicked, altShortcutEdit, qOverload<>(&ShortcutLineEdit::setFocus));

        QMetaObject::connectSlotsByName(shortcutsDialogForm);
    } // setupUi

    void retranslateUi(QWidget *shortcutsDialogForm)
    {
        titleBar->setTitle(QCoreApplication::translate("shortcutsDialogForm", "Keyboard Shortcuts", nullptr));
        lineEditSearch->setPlaceholderText(QCoreApplication::translate("shortcutsDialogForm", "Search the action...", "placeholder for input line"));
        primaryLabel->setText(QCoreApplication::translate("shortcutsDialogForm", "Primary shortcut", nullptr));
        altLabel->setText(QCoreApplication::translate("shortcutsDialogForm", "Alternative shortcut", nullptr));
        restoreDefaultsButton->setText(QCoreApplication::translate("shortcutsDialogForm", "Restore Defaults", nullptr));
        applyButton->setText(QCoreApplication::translate("shortcutsDialogForm", "Apply", nullptr));
        restoreAllDefaultsButton->setText(QCoreApplication::translate("shortcutsDialogForm", "Restore All Defaults", nullptr));
        (void)shortcutsDialogForm;
    } // retranslateUi

};

namespace Ui {
    class shortcutsDialogForm: public Ui_shortcutsDialogForm {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SHORTCUTSDIALOG_H
